/* Seamless3d

Copyright (C) 2001-2010 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "gui.h"
#include "scene_tree.h"

Ptr<ToolTipWin> g_tt = new ToolTipWin;

//#define activeColor 0xe0e0ff

String Button::str;
const float Button::tipTimeOut = 0.75f;
int Button::repeatCounter = 0;


//ButtonTimer g_bt={false,0,0};
extern double g_time;

Button* DownButton::button=0;
double DownButton::lastTime=0;
DownButton* DownButton::downButton=new DownButton;


bool g_hoverEnabled=true;
bool g_toolTipShowing=false;


double Button::beginHoverTime=0;
bool Button::open=false;
bool outOfSyncWithRobot();



void ToolTipWin::paint(){
	int* dstPix=pixBuf.getArray();
	paintBlock2( 0xffffe0,	1, 1, size.x-1, size.y-1, size.x, size.y, dstPix);
	drawHorizontalLine2(0,	0,		0, size.x, size.x, size.y, dstPix);
	drawHorizontalLine2(0,	size.y-1,0, size.x, size.x, size.y, dstPix);
	drawVerticalLine2(0,	0,		1, size.y-1, size.x, size.y, dstPix);
	drawVerticalLine2(0,	size.x-1, 1, size.y-1, size.x, size.y, dstPix);
	drawText2(tip, 0, Vec2i(3, 11), size.x, size.y, dstPix);

	if(g_settings->invertWindowColor){
		invert();
	}
	loadSurfaceFromMemory();
}
void ToolTipWin::autoSizeAndStuff(String i_tip){
	tip=i_tip;
	int width=tip.length*8+6;
	Window::size=Vec2i(width, 17);
	paint();
	topWindow=this;
}

bool Button::initHover(const Vec2i ptCursor){
	if(ptCursor >= dstPos && ptCursor < dstPos + bSize){
		if(dynamic_cast<AnimSelect*>(this)){
			if(AnimControlPanel* animCP = dynamic_cast<AnimControlPanel*>(&win)){
				if(ptCursor >= animCP->animSlider->dstPos && ptCursor < animCP->animSlider->dstPos + animCP->animSlider->bSize){
					return false;
				}
			}
		}
		
		Window::removeToolTip();
		removeTypeFromList < Button>(g_callBackList);
		open = false;
		beginHoverTime = g_time;
		g_callBackList += this;
		workOutSubIndex(ptCursor-dstPos);
		win.paint();
		return true;
	}
	return false;
}
void Button::doCallBack(){ //while hover
	if(open || tip == 0){
		return;
	}
	if(g_time > beginHoverTime + tipTimeOut){
		open = true;
		g_tt->autoSizeAndStuff(tip);

		Rect rect;
		getClientRect(rect);

		Vec2i pos = win.pos + dstPos + Vec2i(bSize.x/2 - g_tt->size.x/2, -18);
		int subtract = rect.right - (pos.x + g_tt->size.x);
		if(pos.x<0){
			pos.x=0;
		}
		else if(subtract < 0){
			pos.x+=subtract;
		}
		if(pos.y<0){
			pos.y+=18 + 40;
		}
		g_tt->pos=pos;
	}
	return;
}

bool Button::onHoverMove(const Vec2i ptCursor){
	if( ptCursor >= dstPos && ptCursor < dstPos + bSize){
		if(dynamic_cast<AnimSelect*>(this)){
			if(AnimControlPanel* animCP = dynamic_cast<AnimControlPanel*>(&win)){
				if(ptCursor >= animCP->animSlider->dstPos && ptCursor < animCP->animSlider->dstPos + animCP->animSlider->bSize){
					open = false;
					Window::removeToolTip();
					removeTypeFromList < Button>(g_callBackList);
					win.paint();
					return false;
				}
			}
		}
		workOutSubIndex(ptCursor-dstPos);
		win.paint();
		return true;
	}
	else{
		open = false;
		Window::removeToolTip();
		removeTypeFromList < Button>(g_callBackList);
		win.paint();
		return false;
	}
}

template<const int n> void upDownButtonPaint(const Button* button, const int command, const int baseCommand, const Vec2i size, const Vec2i dstPos, const Vec2i srcPos, const Vec2i iconSize, Window& win){
	const int subIndex=command-baseCommand;

	const int subDx=size.x/n;
	const int subDy=size.y/2;
	const int subIconDx=iconSize.x/n;
	const int subIconDy=iconSize.y/2;
	int subXpos=-subDx;
	int subXIconPos=-subIconDx;
	for(int i=0; i<n*2; i++){
		int subYpos=0;
		int subYIconPos=0;
		if((i&1)==0){
			subYpos+=size.y/2;
			subXpos+=subDx;
			subYIconPos+=iconSize.y/2;
			subXIconPos+=subIconDx;
		}
		int offset=0;
		if(subIndex==i){
			int xLn=dstPos.x+subXpos;
			int yLn=dstPos.y+subYpos;

			if(button==DownButton::getDownButton()){
				offset++;
				pasteRectangle2( activeColor, dstPos.x+subXpos+offset, dstPos.y+subYpos+offset, win.size.x, win.size.y, subDx, subDy, win.pixBuf.getArray());
				drawHorizontalLine2( 0x808080, yLn, xLn, xLn+subDx+1, win.size.x, win.size.y, win.pixBuf.getArray());
				drawVerticalLine2( 0x808080, xLn, yLn+1, yLn+subDy, win.size.x, win.size.y, win.pixBuf.getArray());
			}
			else if(button==Button::getHoverButton()){
				yLn--;
				pasteRectangle2( activeColor, dstPos.x+subXpos+offset, dstPos.y+subYpos+offset, win.size.x, win.size.y, subDx, subDy, win.pixBuf.getArray());
				drawHorizontalLine2( 0x808080, yLn+subDy, xLn+1, xLn+subDx, win.size.x, win.size.y, win.pixBuf.getArray());
				drawVerticalLine2( 0x808080, xLn+subDx, yLn+1, yLn+subDy+1, win.size.x, win.size.y, win.pixBuf.getArray());
			}
		}
		pasteSprite2( dstPos.x+((size.x-iconSize.x)>>1)+subXIconPos+offset, dstPos.y+((size.y-iconSize.y)>>1)+subYIconPos+offset, srcPos.x+subXIconPos, srcPos.y+subYIconPos, win.size.x, win.size.y, iconSrcBufCX, iconSrcBufCY, subIconDx, subIconDy, win.pixBuf.getArray());
	}
}
template void upDownButtonPaint<1>(const Button* button, const int command, const int baseCommand, const Vec2i size, const Vec2i dstPos, const Vec2i srcPos, const Vec2i iconSize, Window& win);
template void upDownButtonPaint<2>(const Button* button, const int command, const int baseCommand, const Vec2i size, const Vec2i dstPos, const Vec2i srcPos, const Vec2i iconSize, Window& win);

void SixUpDownButton::paint(){
	upDownButtonPaint<3>(this, command, baseCommand, bSize, dstPos, srcPos, iconSize, win);
}
void TextButton::paint(){
	const int subDx=bSize.x;
	const int subDy=bSize.y;
	const int yLn=dstPos.y;

	
	int offset=0;
	int xLn=dstPos.x;

	if(this==getHoverButton()){
		pasteRectangle2( activeColor, dstPos.x, yLn, win.size.x, win.size.y, subDx, subDy, win.pixBuf.getArray());
		drawHorizontalLine2( 0x808080, yLn+subDy-1, xLn+1, xLn+subDx, win.size.x, win.size.y, win.pixBuf.getArray());
		drawVerticalLine2( 0x808080, xLn+subDx, yLn+1, yLn+subDy, win.size.x, win.size.y, win.pixBuf.getArray());
	}
	if(this==DownButton::getDownButton()){
		pasteRectangle2( activeColor, dstPos.x, yLn, win.size.x, win.size.y, subDx, subDy, win.pixBuf.getArray());
		offset++;
		drawHorizontalLine2( 0x808080, yLn, xLn, xLn+subDx, win.size.x, win.size.y, win.pixBuf.getArray());
		drawVerticalLine2( 0x808080, xLn, yLn+1, yLn+subDy-2, win.size.x, win.size.y, win.pixBuf.getArray());
	}
	pasteSprite2( dstPos.x+2+offset, dstPos.y+5+offset, 172, 30, win.size.x, win.size.y, iconSrcBufCX, iconSrcBufCY, 8, 13, win.pixBuf.getArray());
	pasteSprite2( dstPos.x+bSize.x-8-2+offset, dstPos.y+5+offset, 172+8, 30, win.size.x, win.size.y, iconSrcBufCX, iconSrcBufCY, 8, 13, win.pixBuf.getArray());
	win.blitHorizontalColorSweep(Vec2i(dstPos.x+2+8+offset, dstPos.y+5+offset), Vec2i(172+7, 30), Vec2i(172+8, 30), Vec2i(bSize.x-16-4, 13));
	drawText2(str=text, 0, dstPos+Vec2i(6,14), win.size.x, win.size.y, win.pixBuf.getArray());
}


bool blockModeButtons(SingleButton* button);

void SingleButton::paint(){
	const int subDx=bSize.x;
	const int subDy=bSize.y;
	const int yLn=dstPos.y;

	
	int offset=0;
	int xLn = dstPos.x;

	//dLog("SingleButton::paint() 1");
	if(this == getHoverButton()){
		//dLog("SingleButton::paint() 2");
		if(blockModeButtons(this)){
			pasteRectangle2( activeColor, dstPos.x, yLn, win.size.x, win.size.y, subDx, subDy, win.pixBuf.getArray());
			drawHorizontalLine2( 0x808080, yLn+subDy-1, xLn+1, xLn+subDx, win.size.x, win.size.y, win.pixBuf.getArray());
			drawVerticalLine2( 0x808080, xLn+subDx, yLn+1, yLn+subDy, win.size.x, win.size.y, win.pixBuf.getArray());
		}
	}
	if(this==DownButton::getDownButton()){
		pasteRectangle2( activeColor, dstPos.x, yLn, win.size.x, win.size.y, subDx, subDy, win.pixBuf.getArray());
		offset++;
		drawHorizontalLine2( 0x808080, yLn, xLn, xLn+subDx, win.size.x, win.size.y, win.pixBuf.getArray());
		drawVerticalLine2( 0x808080, xLn, yLn+1, yLn+subDy, win.size.x, win.size.y, win.pixBuf.getArray());
	}
//	if(iconSize.x){
	//dLog("SingleButton::paint() 7");
	pasteSprite2(offSetIcon + dstPos.x + ((bSize.x - iconSize.x) >> 1) + offset, dstPos.y+((bSize.y-iconSize.y)>>1)+offset, srcPos.x, srcPos.y, win.size.x, win.size.y, iconSrcBufCX, iconSrcBufCY, iconSize.x, iconSize.y, win.pixBuf.getArray());
//	}
//	else{
//		pasteSprite2( dstPos.x+offset, dstPos.y+offset, srcPos.x, srcPos.y, win.size.x, win.size.y, iconSrcBufCX, iconSrcBufCY, subDx, subDy, win.pixBuf.getArray());
//	}
}
void Settings::ShowToolBar::operator=(const bool& par){
	refValue()=par;
	//if(par){
	//	Window::activeWindow=g_tb.p;
	//}
	//else{
	//	Window::closeWindow(g_tb.p);
	//}
}

void Settings::ExtendNodeBar::operator=(const bool& par){
	refValue()=par;
	g_stw->paint();
}

void ResizeableWindow::loadCursor(TugBorder* i_tugBorder, long type){
	if(tugBorder != i_tugBorder){
		tugBorder=i_tugBorder;
	}
	if(false == g_panViewMode){
		changeCursor(type);
	}
}
bool ResizeableWindow::hover(Vec2i ptCursor){
	if(
		ptCursor.x>=pos.x &&
		ptCursor.x< pos.x+size.x &&
		ptCursor.y>=pos.y &&
		ptCursor.y< pos.y+size.y
		)
	{

		const Vec2i localCursor=ptCursor-pos;

		if(localCursor.x < 4){
			if(localCursor.y < 4){
				loadCursor(&tB_TopLeft, g_sizeNWSECursor);
			}
			else if(localCursor.y >= size.y - 4){
				loadCursor(&tB_BottomLeft, g_sizeNESWCursor);
			}
			else{
				loadCursor(&tB_Left, g_sizeWECursor);
			}
		}
		else if(localCursor.x >= size.x - 4){
			if(localCursor.y < 4){
				loadCursor(&tB_TopRight, g_sizeNESWCursor);
			}
			else if(localCursor.y >= size.y - 4){
				loadCursor(&tB_BottomRight, g_sizeNWSECursor);
			}
			else{
				loadCursor(&tB_Right, g_sizeWECursor);
			}
		}
		else if(localCursor.y < 4){
			loadCursor(&tB_Top, g_sizeNSCursor);
		}
		else if(localCursor.y >= size.y - 4){
			loadCursor(&tB_Bottom, g_sizeNSCursor);
		}
		else{ 
			tugBorder=0;
			changeCursor(g_arrowCursor);
			buttonHover(ptCursor - pos);
		}
		return true;
	}
	return false;
}
bool ControlPanel::hover(Vec2i ptCursor){
	if(
		ptCursor.x>=pos.x &&
		ptCursor.x< pos.x+size.x &&
		ptCursor.y>=pos.y &&
		ptCursor.y< pos.y+size.y
		)
	{

		Vec2i localCursor=ptCursor-pos;

		if(localCursor.x < 4){
			loadCursor(&tB_Left, g_sizeWECursor);
		}
		else if(localCursor.x >= size.x - 4){
			loadCursor(&tB_Right, g_sizeWECursor);
		}
		else{ 
			tugBorder=0;
			changeCursor(g_arrowCursor);
			buttonHover(ptCursor - pos);
		}
		return true;
	}
	return false;
}
TextButton::TextButton(const char* i_text, const char* i_tip, void (*i_function)(), Window& i_win):
	Button(Vec2i(strlen(i_text)*8 + 14, 24),Vec2i(),i_tip, 0, i_win),
	function(i_function),
	text(i_text)
	{
}
void TextButton::action(){
	if(outOfSyncWithRobot()){
		return;
	}
	win.paint();
	function();
}
void SingleButton::action(){
	if(outOfSyncWithRobot()){
		return;
	}
	win.paint();
	function();
}
void TwoUpDownButton::action(){
	if(outOfSyncWithRobot()){
		return;
	}
	win.paint();
	switch(command){
		case 0:{
			function0();
			break;
		}
		case 1:{
			function1();
			break;
		}
	}
}
void SixUpDownButton::action(){
	if(outOfSyncWithRobot()){
		return;
	}
	win.paint();
	switch(command){
		case 0:{
			function0();
			break;
		}
		case 1:{
			function1();
			break;
		}
		case 2:{
			function2();
			break;
		}
		case 3:{
			function3();
			break;
		}
		case 4:{
			function4();
			break;
		}
		case 5:{
			function5();
			break;
		}
	}
}
bool Menu::click(Vec2i ptCursor){
	if(ptCursor >= pos && ptCursor < pos+size){
		activeWindow=this;
		if(Button::getHoverButton()){
			Button::getHoverButton()->action();
		}
		return true;
	}
	return false;
}
extern unsigned g_srcRouteFieldColor;
extern unsigned g_dstRouteNodeColor;
extern unsigned g_dstRouteFieldColor;
extern unsigned g_fieldColor;
extern unsigned g_lightBlue;
extern unsigned g_white;
extern SettingsPtr g_settings;


String findMatchingIsName(Field* field);
void colorBlockColorRgn(MField& mfield, M_Properties& prop, Rect rc, int dstWidth, int dstHeight, int* dstPix);

//Ptr<ControlPanel> g_pw=new ControlPanel;
M_<PropWinLine> g_propWinLine;

void paintBlock(int color, int top, int bottom, int dstWidth, int dstHeight, int* dstPix){
	int rg=0xff;
	int i=0;
	for(; rg>0x0e8; i+=4){
		color=rg;
		color<<=8;
		color|=rg;
		color<<=8;
		color|=0x0ff;
		rg--;
		paintBlock2(color, 1+i, top, 1+i+4, bottom, dstWidth, dstHeight, dstPix);
	}
	paintBlock2(color, 1+i, top, dstWidth-1, bottom, dstWidth, dstHeight, dstPix);
}
void paintBlock2(const unsigned fromColor, const unsigned toColor, int tx, int ty, int bx, int by, int dstWidth, int dstHeight, int* dstPix){
	/*
	int fr=0xff;
	int fg=0xe8;
	int fb=0xe8;

	int tr=0xff;
	int tg=0xff;
	int tb=0xe0;
*/
	/*
	int fr=0xff;
	int fg=0xd8;
	int fb=0xd8;

	int tr=0xff;
	int tg=0xff;
	int tb=0xc0;
	*/
/*
	int fr=0xe4;
	int fg=0xe4;
	int fb=0xfd;

	int tr=0xe4;
	int tg=0xe4;
	int tb=0xfd;
	*/
	int changeInc=(bx-tx)>>5;

	int fr=fromColor;
	int fg=fr;
	int fb=fr;
	fr=(fr&0xff0000)>>16;
	fg=(fg&0x00ff00)>>8;
	fb&=0x0000ff;

	int tr=toColor;
	int tg=tr;
	int tb=tr;
	tr=(tr&0xff0000)>>16;
	tg=(tg&0x00ff00)>>8;
	tb&=0x0000ff;

//	int i=dstWidth-1;
	int i=0;
	unsigned color=0;
	for(; fr!=tr || fg!=tg || fb!=tb; i+=changeInc){
		color=fr;
		color<<=8;
		color|=fg;
		color<<=8;
		color|=fb;
		if(fr>tr){
			fr--;
		}
		else if(fr<tr){
			fr++;
		}

		if(fg>tg){
			fg--;
		}
		else if(fg<tg){
			fg++;
		}

		if(fb>tb){
			fb--;
		}
		else if(fb<tb){
			fb++;
		}
		paintBlock2(color, tx+i, ty, tx+i+changeInc, by, dstWidth, dstHeight, dstPix);
	}
	paintBlock2(color, tx+i, ty, bx, by, dstWidth, dstHeight, dstPix);
}
void paintColorSweepForCurvedBar(const unsigned* fromColor, const unsigned* toColor, int tx, int ty, Vec2i size, int dstWidth, int dstHeight, int* dstPix){
	for(int i=0; i<size.y; i++){
		paintBlock2(fromColor[-i*480], toColor[-i*480], tx, ty+i, tx+size.x+1+1, ty+i+1, dstWidth, dstHeight, dstPix);
		//big for parameter:
		//tx+size.x+1+1
		//should be just:
		//tx+size.x+1
	}
}
void paintSideEdge(const unsigned* color, int width, int tx, int ty, int bottom, int dstWidth, int dstHeight, int* dstPix){
	for(int i=0; i<width; i++){
		drawVerticalLine2( color[i], tx+i, ty, bottom, dstWidth, dstHeight, dstPix);
	}
}
void CaretWindow::highLightBlock(){
	int highlightPos=this->highlightPos-firstVisChar;
	int caretPos = this->caretPos-firstVisChar;
	if(highlightPos<0){
		highlightPos=0;
	}
	if(caretPos<0){
		caretPos=0;
	}
	if(caretPos > highlightPos){
		invertBlock( cellPos.x + highlightPos * charWidth + 1, cellPos.y, cellPos.x + caretPos * charWidth, cellPos.y + 16, win.size.x, win.size.y, win.pixBuf.getArray());
	}
	else if(caretPos < highlightPos){
		invertBlock( cellPos.x + caretPos * charWidth + 1, cellPos.y, cellPos.x + highlightPos * charWidth, cellPos.y + 16, win.size.x, win.size.y, win.pixBuf.getArray());
	}
}
#ifdef PROG_ROB
void commonRobProgButtonEnd(String& str){
	if(Button::repeatCounter>1){
		str+=',';
		str+=Button::repeatCounter;
	}
	str+="),";
}
#endif

void Button::onButtonUp(){
#ifdef PROG_ROB
	String str;
	Button* downButton=DownButton::getDownButton();
	if(SingleButton* singleButton=dynamic_cast<SingleButton*>(downButton)){
		dLog("new ClickButton(" + String(singleButton->name) + "),");
	}
	else if(IndexButton* indexButton=dynamic_cast<IndexButton*>(downButton)){
		dLog("new IncIndex(\"" + indexButton->sprop.name + "\"),");
	}
	else if(LengthButton* lengthButton=dynamic_cast<LengthButton*>(downButton)){
		dLog("new IncFieldGroup(\"" + lengthButton->sprop.name + "\"),");
	}
	else if(FieldButton* fieldButton=dynamic_cast<FieldButton*>(downButton)){
		const Properties& sprop = fieldButton->sprop;
		if(dynamic_cast<SFBoolButton*>(downButton)){
			str+="new ToggleField(\"";
			str += sprop.name;
			str += '\"';
			commonRobProgButtonEnd(str);
		}
		else{
			switch(command){
				case 0:
					str+="new DecFieldX(\"";
					break;
				case 1:
					str+="new IncFieldX(\"";
					break;
				case 2:
					str+="new DecFieldY(\"";
					break;
				case 3:
					str+="new IncFieldY(\"";
					break;
				case 4:
					str+="new DecFieldZ(\"";
					break;
				case 5:
					str+="new IncFieldZ(\"";
					break;
			}
			str += sprop.name;
			str += '\"';
			commonRobProgButtonEnd(str);
		}
	}
	if(downButton->getID()==reinterpret_cast<int>(decCoordX)){
		if(controlKeyState()){
			switch(command){
				case 0:
					str="new DecButtonCtrlX(";
					break;
				case 1:
					str="new IncButtonCtrlX(";
					break;
				case 2:
					str="new DecButtonCtrlY(";
					break;
				case 3:
					str="new IncButtonCtrlY(";
					break;
				case 4:
					str="new DecButtonCtrlZ(";
					break;
				case 5:
					str="new IncButtonCtrlZ(";
					break;
			}
		}
		else{
			switch(command){
				case 0:
					str="new DecButtonX(";
					break;
				case 1:
					str="new IncButtonX(";
					break;
				case 2:
					str="new DecButtonY(";
					break;
				case 3:
					str="new IncButtonY(";
					break;
				case 4:
					str="new DecButtonZ(";
					break;
				case 5:
					str="new IncButtonZ(";
					break;
			}
		}
		str+="decCoordX";
		commonRobProgButtonEnd(str);
	}
	if(downButton->getID()==reinterpret_cast<int>(viewOrientation)){
		switch(command){
			case 0:
				str="new DecButtonX(viewOrientation";
				break;
			case 1:
				str="new IncButtonX(viewOrientation";
				break;
			case 2:
				str="new DecButtonY(viewOrientation";
				break;
			case 3:
				str="new IncButtonY(viewOrientation";
				break;
			case 4:
				str="new DecButtonZ(viewOrientation";
				break;
			case 5:
				str="new IncButtonZ(viewOrientation";
				break;
		}
		commonRobProgButtonEnd(str);
	}
	if(downButton->getID()==reinterpret_cast<int>(viewOrientation90)){
		switch(command){
			case 0:
				str="new DecButtonX(viewOrientation90";
				break;
			case 1:
				str="new IncButtonX(viewOrientation90";
				break;
			case 2:
				str="new DecButtonY(viewOrientation90";
				break;
			case 3:
				str="new IncButtonY(viewOrientation90";
				break;
			case 4:
				str="new DecButtonZ(viewOrientation90";
				break;
			case 5:
				str="new IncButtonZ(viewOrientation90";
				break;
		}
		commonRobProgButtonEnd(str);
	}
	if(downButton->getID()==reinterpret_cast<int>(decRed)){
		switch(command){
			case 0:
				str="new DecButtonX(decRed";
				break;
			case 1:
				str="new IncButtonX(decRed";
				break;
			case 2:
				str="new DecButtonY(decRed";
				break;
			case 3:
				str="new IncButtonY(decRed";
				break;
			case 4:
				str="new DecButtonZ(decRed";
				break;
			case 5:
				str="new IncButtonZ(decRed";
				break;
		}
		commonRobProgButtonEnd(str);
	}
	if(downButton->getID()==reinterpret_cast<int>(decTugRadius)){
		switch(command){
			case 0:
				str="new DecButtonX(decTugRadius";
				break;
			case 1:
				str="new IncButtonX(decTugRadius";
				break;
		}
		commonRobProgButtonEnd(str);
	}
	if(downButton->getID()==reinterpret_cast<int>(zoom)){
		str="new MouseZoom(";
		str+=g_scene->scale.x;
		str+="),";
	}
	dLog(str);
#endif
//	g_bt.active=false;
	open=true; //prevent tool tip opening
	g_callBackList+=this;
	removeTypeFromList < DownButton>(g_callBackList);
	win.paint();
}
Button* Button::getHoverButton(){
	//dLog("Button::getHoverButton()");
	//int num = g_callBackList.length;
	//dLog(num);
	for(int i = g_callBackList.length; --i>=0;){
		if(Button* button=dynamic_cast<Button*>(g_callBackList[i])){
			return button;
		}
	}
	return 0;
}
Button* DownButton::getDownButton(){
	for(int i = g_callBackList.length; --i>=0;){
		if(DownButton* downButton=dynamic_cast<DownButton*>(g_callBackList[i])){
			return downButton->button;
		}
	}
	return 0;
}
void DownButton::doCallBack(){
	if(g_time-lastTime > .05){
		lastTime+=.05;
		button->win.paint();
		button->action();
	}
}
void DownButton::setDownButton(Button* i_button){
	button=i_button;
	lastTime=g_time+.5;
	removeTypeFromList<Button>(g_callBackList);
	removeTypeFromList<DownButton>(g_callBackList);
	g_callBackList+=downButton;
}
