/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include <windows.h>
#include <direct.h>

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "seamless3d_gui_render.h"
#include "gui.h"
#include "scene_tree.h"
#include "robot_aid.h"

bool SurfacePoint(
				  float u, float v, int uOrder, int vOrder, int uDimension, int vDimension,
				  const float *uKnot, const float * vKnot,
				  const Vec3f *P,const float *W,
				  Vec3f *dest // result
				  );

inline bool checkPoint(const bool perspective, const Vec3f sr, int& idx, int i);//used by NurbsPatch, bezier3f and BezierPatch
inline void guessPolyMapTug(NurbsPatch& patch, float& xCrd, float& yCrd, const Vec3f& pt, const bool& northPole, const bool& southPole);
void commonPostTug();
void fillPolyLineIndex(M_Vec3f& point, M_Int& coordIndex);
Vec2f convertPointToScreen2d(Vec3f point);
Seamless* getSeamless(int& selLine);
void updateNurbsCurveKnotMid0(int dim, M_Float& knot);
void resetCPNodeList();
Seamless* getSeamless();
void toggleHairRegionMode();

extern SeamlessPtr g_vcNurbsCtrlPoly;
extern ArtPtr g_vcNurbsCtrlPolyPart;
extern Group* g_vcSelectRectGroup;
extern ShapePtr g_vcSelectRectShape;
extern ShapePtr g_vcEllipticalPolyline2D;
extern TransformPtr g_vcBezierBase;
extern bool tugFlag;
extern Ptr<Region> g_region;
extern Ptr<Extrude> g_extrude;
extern ShapePtr g_transformLeverClue;

Ptr<RingletGenerator> g_ringletGenerator = new RingletGenerator;
Ptr<InsertLock> g_insertLock = new InsertLock;
Ptr<Region> g_region = new Region;
Ptr<Seed> g_seed = new Seed;


void evolveVertex(NurbsPatch& nurbsPatch, Vec2f& flatCoord, Vec3f& guessCoord, const Vec3f& originalCoord, const int spreadRes){
	const float spreadResFloat = spreadRes;
	float* stemUKnotPtr = nurbsPatch.uKnot.refValue().getArray();
	float* stemVKnotPtr = nurbsPatch.vKnot.refValue().getArray();
	Vec3f* stemPointPtr = nurbsPatch.point.refValue().getArray();
	float* stemWeightPtr = nurbsPatch.weight.refValue().getArray();
	int uOrder = nurbsPatch.order.x;
	int vOrder = nurbsPatch.order.y;
	int uDim = nurbsPatch.dim.x;
	int vDim = nurbsPatch.dim.y;

	float maxU = nurbsPatch.maxU();
	float maxV = nurbsPatch.maxV();
	float minU = nurbsPatch.minU();
	float minV = nurbsPatch.minV();

	for(int uPt = 0; uPt < spreadRes; uPt++){
		for(int vPt = 0; vPt < spreadRes; vPt++){
			Vec2f pt;
			pt.x = uPt / spreadResFloat;
			pt.y = vPt / spreadResFloat;

			float u = (maxU-minU)*pt.x + minU;
			float v = (maxV-minV)*pt.y + minV;

			Vec3f curvedEdgePoint;

			SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, &curvedEdgePoint);

			Vec3f diffA = guessCoord - originalCoord;
			Vec3f diffB = curvedEdgePoint - originalCoord;
			float fdA = fabs(sqrt(diffA.x*diffA.x + diffA.y*diffA.y + diffA.z*diffA.z));
			float fdB = fabs(sqrt(diffB.x*diffB.x + diffB.y*diffB.y + diffB.z*diffB.z));
			if(fdB < fdA){
				flatCoord = pt;
				guessCoord = curvedEdgePoint;
			}
		}
	}
}

void Hair::bindToSeperator(Strand* sd){
	for(int k = 0; k < lock.length; k++){
		NurbsPatch* patch = dynamic_cast<NurbsPatch*>(lock[k].p);
		if(patch->cage){
			int beginIdx = beginIndex[k];
			int numHoles = numOfHoles[k];
			//int pointIdx = scpi[longIdx];
			const int bendyIdx = sd->index[0];
			scpi.refValue().insertElement(beginIdx, bendyIdx);
			hole.refValue().insertElement(beginIdx, Vec2f());
			scpiLathe.insertElement(beginIdx, sd);
			numOfHoles.refValue()[k]++;

			while(++k < lock.length){
				beginIndex.refValue()[k]++;
			}

			Vec2f& hl = hole.refValue()[beginIdx];
			Vec3f guessCoord = patch->dpoint[0];
			evolveVertex(*patch, hl, guessCoord, sd->point.refValue()[bendyIdx], 200);

			sd->smls->doBuild();
			houseKeep();
			sd->smls->ctrf = true;
		}
	}
}


void Hair::initTug(){
	const bool perspective = g_perspective;

	Matrix matrix = g_viewMatrix;

	for(int k = 0; k < lockList.length; k++){
		selLock = lockList[k];
		if(selLock->cage){
			M_Int& scpi = selLock->scpi.refValue();
			M_<Strand*>& scpiLathe = selLock->scpiLathe;
			M_Vec2f& hole = selLock->hole.refValue();
			int numHoles = hole.length;
			for(int h = 0; h < numHoles; h++){
				Strand* bendy = scpiLathe[h];
				int pointIdx = scpi[h];
				Vec3f point = bendy->point[pointIdx];
				if(checkPoint(perspective,  matrix * point, selHoleIndex, h)){
					//processFields();
					g_stw->cp->paint();
					fastFullBuildForAutoIn(false);
					return;
				}
			}
		}
	}
	selLock = 0;
}

void Hair::drag(Vec3f pt){
	Vec3f newPt = g_scene->getIViewMatrix() * pt;

	M_Vec2f& hole = selLock->hole.refValue();
	M_Int& scpi = selLock->scpi.refValue();
	M_<Strand*>& scpiLathe = selLock->scpiLathe;

	Vec2f& hl = hole[selHoleIndex];
	Strand* bendy = scpiLathe[selHoleIndex];
	Seamless* smls = bendy->smls;
	const int idx = scpi[selHoleIndex];

	guessPolyMapTug(*selLock, hl.x, hl.y, newPt, false, false);
	
	
	g_pointer.translation = pt = g_scene->getViewMatrix() * selLock->generate(hl);
	if(g_selNode.getRaw() == this){

		Vec3f svec = selLock->generate(hl);
		g_transformCP->setFilteredDimension(bendy->point.refValue(), smls->cp.refValue(), bendy->cpIndex.refValue(), idx, svec * bendy->weight[idx]);
		bendy->dpoint.refValue()[idx] = bendy->point[idx] / bendy->weight[idx];
		bendy->mirrorSingleCP(idx);


		//processFields();
		//smls->doBuild();

		smls->calcAllPolyMidPoints();
		commonPostTug();
		smls->updateCage();
		houseKeep();
	}
	else{
		//g_pointer.translation = pt = g_scene->getViewMatrix() * bendy->dpoint[idx];

		Matrix pivotPointMatrix;
		g_transformCP->getPivotPointMatrix(pivotPointMatrix, smls);

		Vec3f pt2 = pt - (pivotPointMatrix * MultiSelect::beginDragPos[MultiSelect::clickedIndex] - pivotPointMatrix * g_transformCP->pivotPoint);

		newPt = g_scene->getIViewMatrix() * g_catchIMatrix * pt2;

		//g_transformCP->rotCtrlPnt = g_transformCP->remRotCtrlPnt + g_pointer.translation - g_pointer.remTranslation;

		g_transformCP->translation = newPt - g_transformCP->pivotPoint;
	}
	g_dragActuallyHappened = true;
}



void Hair::centerViewToControlPoint(){
	if(selLock){
		Strand* bendy = selLock->scpiLathe[selHoleIndex];
		const int idx = selLock->scpi[selHoleIndex];

		Vec3f vec = bendy->dpoint[idx];

		g_scene->translation = -vec;
	}
}

void Hair::locateBendyLatheNode(){
	const int h = index[0];
	Strand* bendy = scpiLathe[h];
	int line = getNodeLine(bendy);

	g_stw->textCell.cellActive = false;
	g_stw->selLine = line;
									
	houseKeepVisualClue();
	g_stw->paint();
}

void Lock::freePointFromSeperator(int scpiIdx){
	hole.refValue().deleteElement(scpiIdx);
	scpi.refValue().deleteElement(scpiIdx);
	scpiLathe.deleteElement(scpiIdx);
}

Strand* Hair::clone(Strand* oldStrand){
	Strand* newStrand = new Strand;
	newStrand->dpoint.refValue() = oldStrand->dpoint;
	newStrand->point.refValue() = oldStrand->point;
	newStrand->weight.refValue() = oldStrand->weight;
	newStrand->uKnot.refValue() = oldStrand->uKnot;
	newStrand->vKnot.refValue() = oldStrand->vKnot;
	newStrand->dim.refValue() = oldStrand->dim;
	newStrand->uJoinEnds.refValue() = false;
	strand.refValue() += newStrand;

	newStrand->processFields();

	processToFileFormat();
	scpiLathe.length = 0;
	postLoadProcess2 = true;
	processFields();

	return newStrand;

	//Strand* strand = new Strand(patch, prevStrand->hairFrom + prevStrand->ctrlLine->point.length);

	//m_strand += strand;
	//processFields();
}

bool Hair::isBoundToLock(Strand* ctrlLine){
	selLock = 0;
	int speratorIdx = 0;
	int scpiIdx = 0;
	Lock* testLock = 0;
	if(isBoundToLock(ctrlLine, ctrlLine->index[0], testLock, scpiIdx)){
		selHoleIndex = scpiIdx;
		selLock = testLock;
		//processFields();
		g_stw->cp->paint();
		fastFullBuildForAutoIn(false);
		return true;
	}
	return false;
}

bool Hair::isBoundToLock(Strand* ctrlLine, int idx, Lock*& lock, int& scpiIdx){
	
	for(int seperatorIdx = 0; seperatorIdx < lockList.length; seperatorIdx++){
		lock = lockList[seperatorIdx];
		M_Int& scpi = lock->scpi.refValue();
		M_<Strand*>& scpiLathe = lock->scpiLathe;
		int numHoles = scpi.length;
		for(scpiIdx = 0; scpiIdx < numHoles; scpiIdx++){
			if(scpiLathe[scpiIdx] == ctrlLine && scpi[scpiIdx] == idx){
				return true;
			}
		}
	}
	return false;
}

void Hair::cloneSeperator(Strand* oldSd, Strand* newSd){
	int selSeperator = 0;
	
	for(int pointIdx = oldSd->point.length; --pointIdx>=0;){
		Lock* lock = 0;
		int scpiIdx = 0;
		if(isBoundToLock(oldSd, pointIdx, lock, scpiIdx)){
			bindToLock(lock, newSd, pointIdx, lock->hole[scpiIdx]);
		}
	}
}

void Hair::bindToLock(Lock* lock, Strand* sd, const int pointIdx, Vec2f hl){
	M_Vec2f& hole = lock->hole.refValue();
	M_Int& scpi = lock->scpi.refValue();
	M_<Strand*>& scpiLathe = lock->scpiLathe;

	int numHoles = hole.length;
	scpi += pointIdx;
	hole += hl;
	scpiLathe += sd;
}

Vec2f getAverage(M_Vec2f& regionHole){
	Vec2f center;
	for(int s = regionHole.length; --s>=0;){
		center += regionHole[s];
	}
	center = center / regionHole.length;
	return center;
}
/*
void Hair::generateMultiples(){
	float satRad = .006;
	int numberOfSatellites = 7;
	int orgLen = m_strand.length;
	for(int sat = 0; sat < numberOfSatellites; sat++){
		sat = 6;
		Seamless* smls = new Seamless;
		ColorEffect* newColorEffect = new ColorEffect;
		smls->effect.refValue() = newColorEffect;
		if(part.p){
			if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part.refValue()->workEffect)){
				newColorEffect->ambient.refValue() = colorEffect->ambient;
				newColorEffect->phong.refValue() = colorEffect->phong;
				newColorEffect->transparency.refValue() = colorEffect->transparency;
			}
		}

		Art* part = new Part;
		part->name = "mul";
		part->name += sat;
		smls->skeleton.refValue() += part;
		Hair* newHair = new Hair;
		newHair->radius.refValue() = radius;
		if(textureMapper.length){
			if(ColorSweep* colorSweep = dynamic_cast<ColorSweep*>(this->textureMapper[0].p)){
				ColorSweep* newColorSweep = new ColorSweep;
				newColorSweep->copyFields(colorSweep);
				newHair->textureMapper.refValue() += colorSweep;
			}
		}
		newHair->part.refValue() = part;
		smls->build.refValue() += newHair;
		g_scene->children.refValue() += smls;
		for(int rgn = 0; rgn < orgLen; rgn++){
			NurbsPatch* ctrlLine = m_strand[rgn]->ctrlLine;
			int markStrand = m_strand.length;
			const M_Vec3f& oldPoint = ctrlLine->point;
			Strand* sd = newHair->clone(ctrlLine);
			M_Vec3f& newPoint = sd->ctrlLine->point.refValue();
			Rotation prevRot;
			for(int pointIdx = 0; pointIdx < oldPoint.length; pointIdx++){
				Rotation rot;
				if(pointIdx == 0){
					rot = Rotation(prevRot*Vec3f(0, 1, 0), Vec3f(oldPoint[pointIdx + 1] - oldPoint[pointIdx]).normalize());
				}
				else if(pointIdx == oldPoint.length - 1){
					rot = Rotation(prevRot*Vec3f(0, 1, 0), Vec3f(oldPoint[pointIdx] - oldPoint[pointIdx - 1]).normalize());
				}
				else{
					Rotation rotA = Rotation(prevRot*Vec3f(0, 1, 0), Vec3f(oldPoint[pointIdx] - oldPoint[pointIdx - 1]).normalize());
					Rotation rotB = Rotation(prevRot*Vec3f(0, 1, 0), Vec3f(oldPoint[pointIdx + 1] - oldPoint[pointIdx]).normalize());

					rot = rotA.slerp(rotB, .5);
				}
				prevRot = prevRot * rot;

				float angle = sat * PI * 2 / numberOfSatellites;
				//float angle = 1.0 * PI * 2.0 / 7.0;
				Vec3f localPos = Rotation(0, 1, 0, angle) * Vec3f(0, 0, satRad);
				localPos = prevRot * localPos;
				newPoint[pointIdx] = oldPoint[pointIdx] + localPos;
			}
			sd->ctrlLine->processFields();
		}
		smls->process();
		break;
	}

	g_scene->genNodeList2();
	g_stw->paint();
	int selLine = g_stw->selLine;
	if(Seamless* parentSmls = getSeamless(selLine)){
		parentSmls->populateMirrorNodes();
	}
	houseKeepVisualClue();
}

*/

// old poly algorithm
void Hair::generateMultiples(){
	M_Vec2f regionHole;
	regionHole.length = 5;
	//for(int s = 0; s < m_strand.length; s++){
	M_Strand& m_strand = this->strand.refValue();
	int orgLen = m_strand.length;
	for(int rgn = 0; rgn < orgLen; rgn += 4){
		Strand* ctrlLine = m_strand[rgn];
		int markStrand = m_strand.length;
		for(int pointIdx = 0; pointIdx < ctrlLine->point.length; pointIdx++){
			int mrk = markStrand;
			int edges = 4;
//			const int numOfRings = 14;
//			int numOfSteps = 22;
			const int numOfRings = 10;
			int numOfSteps = 10;

			Lock* lock = 0;
			int s = 0;

			for(s = 0; s < edges; s++){
				int scpiIdx = 0;
				if(isBoundToLock(m_strand[rgn + s], pointIdx, lock, scpiIdx)){
					regionHole[s] = hole[scpiIdx];
				}
			}
			
			regionHole[edges] = regionHole[0];

			if(lock->uJoinEnds){
				regionHole[edges].x = 1;
			}


			Vec2f center = getAverage(regionHole);
			//for(int r = 0; r < numOfRings - 1; r++){
			for(int r = 0; r < 2; r++){
				for(s = 0; s < edges; s++){
					Vec2f hlA = regionHole[s];
					Vec2f hlB = regionHole[s + 1];

					Vec2f diffCenterA;
					Vec2f diffCenterB;

					if(lock->uJoinEnds){
						diffCenterA = hlA;
						diffCenterB = hlB;
						diffCenterA.y = 1;
						diffCenterB.y = 1;
					}
					else{
						diffCenterA = (center - hlA);
						diffCenterB = (center - hlB);
					}

					hlA = hlA + (diffCenterA / (numOfRings - 1)) * r;
					hlB = hlB + (diffCenterB / (numOfRings - 1)) * r;

					Vec2f diff = (hlB - hlA);
					for(int k = 0; k < numOfSteps; k++){
						Strand* sd = 0;
						if(pointIdx == 0){
							sd = clone(ctrlLine);
						}
						else{
							sd = m_strand[mrk++];
						}
						Vec2f hl = hlA + (diff / (numOfSteps - 1)) * k;
						bindToLock(lock, sd, pointIdx, hl);
					}
				}
				numOfSteps -= 1;
			}
		}
		break;
	}
	g_scene->genNodeList2();
	processFields();
	updateBuild3dWindow();
	g_stw->paint();
}

void Lock::setFlag(M_Bool& flag, bool state){
	for(int k = scpi.length; --k >= 0;){
		flag[scpiLathe[k]->slidx] = state;
	}
}

void Lock::selectFreeEnds(M_Bool& flag){
	int numHoles = hole.length;

	for(int k = 0; k < numHoles; k++){
		Strand* sd = scpiLathe[k];
		if(flag[sd->slidx] == false){
			int v = scpi[k] + 1;
			if(v < sd->point.length){
				sd->selectRings(v, sd->dim.x, sd->dim.y);
			}
		}
	}
}

void Lock::selectRings(M_Bool& flag){
	NurbsPatch::selectRings(0, dim.x, dim.y);
	setFlag(flag, false);

	M_Node& m_lock = lock.refValue();

	for(int i = 0; i < m_lock.length; i++){
		Lock* childLock = dynamic_cast<Lock*>(m_lock[i].p);
		childLock->setFlag(flag, true);
	}

	selectFreeEnds(flag);

	for(int i = 0; i < m_lock.length; i++){
		Lock* childLock = dynamic_cast<Lock*>(m_lock[i].p);
		childLock->selectRings(flag);
	}
}

void Hair::selectRings(Lock* lock){
	static M_Bool flag;
	flag.length = strand.length;
	lock->selectRings(flag);
}
/*
void Hair::insertStrandControlPoints(Seamless* smls, Lock* lock, float ratio){
	M_Vec2f& hole = lock->hole.refValue();
	M_Int& scpi = lock->scpi.refValue();
	M_<Strand*>& scpiLathe = lock->scpiLathe;

	int numHoles = hole.length;

	for(int k = 0; k < numHoles; k++){
		int pointIdx = scpi[k];
		Strand* sd = scpiLathe[k];
		

		M_Vec3f& dpoint = sd->dpoint.refValue();
		M_Float& weight = sd->weight.refValue();
		M_Float& vKnot = sd->vKnot.refValue();
		int uDim = sd->dim.x;
		int vDim = sd->dim.y;

		M_Vec3f oldPoints;
		M_Float oldWeights;
		M_Int lookup;

		sd->insertRow(uDim, vDim, dpoint, weight, oldPoints, oldWeights, lookup, pointIdx, ratio);
		updateNurbsCurveKnotMid0(vDim, vKnot);
		sd->dpointsToPoints();
		dynamic_cast<NurbsPatch*>(sd)->processFields();

		insertStrandPoint(sd, pointIdx);
	}
	processFields();
	smls->populateMirrorNodes();
	MultiSelect::pointList.length = 0;


	fastFullBuildForAutoIn(true, smls);
	smls->doBuild();
	smls->ctrf = true;
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}
*/
void Strand::houseKeep(){
	NurbsPatch::houseKeep();
}

void Lock::splitLockMap(Lock* lockB, float ratio, bool doU){
	lockB->isCtrlPolyRef = false;
	lockB->lock.refValue().length = 0;
	float iRatio = 1.0f - ratio;

	M_Int& scpiA = this->scpi.refValue();
	M_<Strand*>& scpiLatheA = this->scpiLathe;
	M_Vec2f& holeA = this->hole.refValue();
	int numHolesA = holeA.length;

	M_Int& scpiB = lockB->scpi.refValue();
	M_<Strand*>& scpiLatheB = lockB->scpiLathe;
	M_Vec2f& holeB = lockB->hole.refValue();

	scpiB.length = 0;
	scpiLatheB.length = 0;
	holeB.length = 0;

	float f = 0;
	for(int k = 0; k < numHolesA;){
		Vec2f hl = hole[k];

		if(doU){
			f = hl.x;
		}
		else{
			f = hl.y;
		}
		if(f < ratio){
			Strand* sd = scpiLatheA[k]; 
			int pointIdx = scpiA[k];

			holeA.deleteElement(k);
			scpiA.deleteElement(k);
			scpiLatheA.deleteElement(k);

			f = f * (1.0 / ratio);
			if(doU){
				hl.x = f;
			}
			else{
				hl.y = f;
			}
			holeB += hl;
			scpiB += pointIdx;
			scpiLatheB += sd;

			numHolesA--;
		}
		else{
			f = f - ratio;
			f = f * (1.0 / iRatio);
			if(f > 1.0f){
				f = 1;
			}
			else if(f < 0){
				f = 0;
			}
			if(doU){
				holeA[k].x = f;
			}
			else{
				holeA[k].y = f;
			}
			
			k++;
		}
	}
	getHair()->processFields();
}


void relateHitPointToMapTri(Vec2f sa, Vec2f sb, Vec2f sc, Vec2f& hit, Vec2f ta, Vec2f tb, Vec2f tc){
	sb -= sa;
	sc -= sa;
	hit -= sa;

	double sangle = sb.angle();
	sb = sb.rotate(-sangle);
	sc = sc.rotate(-sangle);
	hit = hit.rotate(-sangle);

	tb -= ta;
	tc -= ta;
	double tangle = tb.angle();
	tb = tb.rotate(-tangle);
	tc = tc.rotate(-tangle);

	float xScale = tb.x / sb.x;
	sb *= xScale;
	sc *= xScale;
	hit *= xScale;

	float scy = (hit.y / sc.y);

	float yScale = tc.y / sc.y;
	hit.y *= yScale;

	float diff = tc.x - sc.x;
	xScale = diff * scy;
	hit.x += xScale;

	hit = hit.rotate(tangle);
	hit += ta;
}

bool Lock::addedAllready(int& brdrOffstt, M_Vec3f& coord){
	for(int k = 0; k < relevantLockList.length; k++){
		if(relevantLockList[k] == this){
			brdrOffstt = borderOffset[k];
			return true;
		}
	}
	brdrOffstt = coord.length;
	return false;
}


void Seamless::genRegionPolygons(Art* part, const M_Vec3f& point, int brdrOffstt, int numOfCorners, int s, const M_Int& region, M_Color3f& color, int regionSelectedIdx){
	M_Vec3f& coord = part->coord.refValue();
	M_Triangle& m_triangle = part->triangle;

	int coordOffest = coord.length;
	coord.length += numOfCorners + 1;
	color.length += numOfCorners + 1;
	int centerIdx = coord.length - 1;

	Color3f clr = (regionSelectedIdx < Lock::regionSelected.length && Lock::regionSelected[regionSelectedIdx]) ?  Color3f(1, 0, 0) : Color3f(0, 1, 1);

	Vec3f center = point[region[s] + brdrOffstt];

	coord[coordOffest] = center;
	color[coordOffest] = clr;

	for(int k = 1; k < numOfCorners; k++){
		int i = region[s + k] + brdrOffstt;
		Vec3f v = point[i];
		coord[coordOffest + k] = v;
		color[coordOffest + k] = clr;
		center += v;
		int crdIdx = coordOffest + k;
		int prevCrdIdx = crdIdx - 1;
		m_triangle += S_Triangle(part, centerIdx, centerIdx, part, prevCrdIdx, prevCrdIdx, part, crdIdx, crdIdx);
	}
	
	m_triangle += S_Triangle(part, centerIdx, centerIdx, part, centerIdx - 1, centerIdx - 1, part, coordOffest, coordOffest);

	center = center / numOfCorners;
	coord[centerIdx] = center;
	color[centerIdx] = clr;

}


void Lock::houseKeepRegions(M_Vec3f& coord, M_Node& children, const float inverseSize, int& numOfCorners, M_Int& coordIndex, int& s, M_Int& region, M_Color3f& color, int regionSelectedIdx){
	if(cage){
		int numBorderPoints = border.length;
		int brdrOffstt = 0;
		bool flag = addedAllready(brdrOffstt, coord);
		borderOffset += brdrOffstt;

		relevantLockList += this;

		if(flag == false){
			coord.length += numBorderPoints;
			for(int h = 0; h < numBorderPoints; h++){

				Vec3f v = generate(border[h]);
				coord[brdrOffstt + h] = v;

				Transform* transform = new Transform;
				if(this == selBorderLock && h == selBorder){
					transform->children.refValue() += g_redPoint.p;
				}
				else{
					transform->children.refValue() += g_nextPoint.p;
				}
				children += transform;

				transform->translation = v;
				transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
			}
		}
		if(s < region.length){
			int cornerCntr = numOfCorners;
			int remS = s;

			for(int i = 0; i < numOfCorners; i++){
				int ridx = region[s++];
				if(ridx < 0){
					i = -1;
					numOfCorners = -ridx;
					cornerCntr = numOfCorners;
					remS = s;
					continue;
				}

				coordIndex += ridx + brdrOffstt;
			
				if(--cornerCntr == 0){
					coordIndex += region[s - numOfCorners] + brdrOffstt;
					coordIndex += -1;
				}
			}
			g_vcNurbsCtrlPoly->genRegionPolygons(g_vcNurbsCtrlPolyPart, coord, brdrOffstt, numOfCorners, remS, region, color, regionSelectedIdx);
		}
	}
	else{
		if(s < region.length){
			int cornerCntr = numOfCorners;

			for(int i = 0; i < numOfCorners; i++){
				int ridx = region[s++];
				if(ridx < 0){
					i = -1;
					numOfCorners = -ridx;
					continue;
				}
			}
		}
	}
}

//void Lock::houseKeepRegions(M_Vec3f& coord){
//
//		coord.length += border.length;
//
//	/////////////
//	}
//
//	int from = numBorderPoints;
//	numBorderPoints = border.length;
//	for(int h = 0; h < numBorderPoints; h++){
//
//		Vec3f v = generate(border[h]);
//		coord[from + h] = v;
//
//		Transform* transform = new Transform;
//		if(this == selBorderLock && h == selBorder){
//			transform->children.refValue() += g_redPoint.p;
//		}
//		else{
//			transform->children.refValue() += g_nextPoint.p;
//		}
//		children += transform;
//
//		transform->translation = v;
//		transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
//	}
//
//	polyCntr = polyCount;
//	relevantLockList += this;
//	if(s < region.length){
//		for(int i = 0; i < polyCount; i++){
//			int ridx = region[s++];
//			if(ridx < 0){
//				i = -1;
//				polyCount = -ridx;
//				polyCntr = polyCount;
//				continue;
//			}
//			coordIndex += ridx + from;
//			if(--polyCntr == 0){
//				coordIndex += region[s - polyCount] + from;
//				coordIndex += -1;
//			}
//		}
//	}
//}

void Lock::houseKeep(){
	
	if((g_scene->mode & HAIR_REGION_MODE)){
		if(g_visualClue == g_vcSelectRectGroup){
			IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcSelectRectShape->geometry.p);
			M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

			coord.length = 0;
			coord += g_scene->getIViewMatrix() * Vec3f(MultiSelect::topLeft.x, MultiSelect::bottomRight.y, MultiSelect::topLeft.z);
			coord += g_scene->getIViewMatrix() * MultiSelect::bottomRight;
			coord += g_scene->getIViewMatrix() * Vec3f(MultiSelect::bottomRight.x, MultiSelect::topLeft.y, MultiSelect::topLeft.z);
			coord += g_scene->getIViewMatrix() * MultiSelect::topLeft;
			fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
			polyLine->coordIndex.refValue() += 0;
			polyLine->convertToRenderFormat(g_vcSelectRectShape->render);
		}



		if(g_vcNurbsCtrlPolyPart->workEffect==0)return;
		g_vcNurbsCtrlPolyPart->deleteGeometry();

		M_Triangle& m_triangle = g_vcNurbsCtrlPolyPart->triangle;
		//m_triangle.length = 0;

		/////

		setMathHiPresision();
		M_Int& region = this->region;
		borderOffset.length = 0;
		relevantLockList.length = 0;

		IndexedLineSet* polyline = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
		M_Vec3f& coord = dynamic_cast<Coordinate*>(polyline->coord.p)->point;
		coord.length = 0;
		Effect* effect = g_vcNurbsCtrlPolyPart->workEffect;
		int effectLength=effect->getEffectLength();
		M_Color3f& color = dynamic_cast<ColorEffect*>(effect)->color.refValue();
		color.length = 0;
		//regionSelected.length = 4;
		//regionSelected[3] = true;

		M_Int& coordIndex = polyline->coordIndex.refValue();
		coordIndex.length = 0;
		int s = 0;

		M_Node& children = g_vcBezierBase->children;
		children.length = 1;
		coord.length += border.length;

		const float inverseSize = (1 / g_scene->getZoom());

		int numOfCorners = 4;

		if(Lock* src = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
			int polyCntr = 0;
			for(int k = 0; ; k++){
				src->houseKeepRegions(coord, children, inverseSize, numOfCorners, coordIndex, s, region, color, polyCntr++);
				houseKeepRegions(coord, children, inverseSize, numOfCorners, coordIndex, s, region, color, polyCntr++);

				if(s == region.length && k >= unify.length){
					break;
				}
				if(k < unify.length){
					src = dynamic_cast<Lock*>(unify[k].p);
				}
			}
		}
		g_vcNurbsCtrlPolyPart->relevantPart = M_Art(g_vcNurbsCtrlPolyPart);
		g_vcNurbsCtrlPoly->process();
		
		children += g_vcNurbsCtrlPoly.p;

		///////////
		polyline->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
		g_vcBezierBase->rotation = Rotation();

		if(g_visualClue == g_vcSelectRectGroup){
			g_vcSelectRectGroup->children.refValue().length = 0;
			g_vcSelectRectGroup->children.refValue() += g_vcBezierBase.p;
			g_vcSelectRectGroup->children.refValue() += g_vcSelectRectShape.p;
			g_visualClue = g_vcSelectRectGroup;
		}
		else{
			g_visualClue = g_vcBezierBase;
		}
		restoreMathPresision();
	}
	else{
		NurbsPatch::houseKeep();
	}
}


void Lock::initTugRegion(const bool perspective){
	IndexedLineSet* polyline = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyline->coord.p)->point;
	Matrix matrix = g_viewMatrix;

	for(int k = 0; k < relevantLockList.length; k++){
		Lock* lock = relevantLockList[k];
		int brdrOffstt = borderOffset[k];
		int borderLen = lock->border.length;
		for(int b = 0; b < borderLen; b++){
			Vec3f crd = matrix * coord[brdrOffstt + b];
			selBorderLock = lock;
			if(checkPoint(perspective, crd, selBorder, b)){
				if(breakBorderMode){
					breakBorder();
				}
				g_stw->cp->paint();
				tugFlag = true;
				g_region->borderPoint.refValue() = selBorderLock->border[selBorder];
				return;
			}
		}
	}
	selBorderLock = 0;
	houseKeep();
}

void Lock::drag(Vec3f pt){
	if((g_scene->mode & HAIR_REGION_MODE)){
		Vec3f newPt = g_scene->getIViewMatrix() * pt;

		M_Vec2f& border = selBorderLock->border.refValue();
		Vec2f& brdr = border[selBorder];

		guessPolyMapTug(*selBorderLock, brdr.x, brdr.y, newPt, false, false);
	
	
		g_pointer.translation = pt = g_scene->getViewMatrix() * selBorderLock->generate(brdr);

		//commonPostTug();
		houseKeep();
	}
	else{
		NurbsPatch::drag(pt);
	}
}

void Lock::insertBorderPoint(){
	if(selBorderLock == 0){
		if(border.length == 0){
			M_Vec2f& border = this->border.refValue();
			border+= Vec2f(0, 0);
			border+= Vec2f(1, 0);
			border+= Vec2f(0, 1);
			border+= Vec2f(1, 1);
		}
	}
	else{
		M_Vec2f& border = selBorderLock->border.refValue();
		if(border.length == 0){
			border+= Vec2f(0, 0);
			border+= Vec2f(1, 0);
			border+= Vec2f(0, 1);
			border+= Vec2f(1, 1);
		}
		else{
			Vec2f brdr;
			if(selBorder < border.length){
				brdr = border[selBorder];
			}
			border += brdr;
		}
	}
	houseKeep();
}

bool triIntersectsRect(Vec2f a, Vec2f b, Vec2f c);
bool isRectCornerInTriangle(const Vec2f* tri);

void Lock::selectRegions(){
	MultiSelect::selectRectMode = false;
	if(region.length == 0){
		return;
	}

	int numOfCorners = 4;
	int s = 0;
	if(region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}
	int edgeCntr = numOfCorners;

	const M_Vec3f& coord = g_vcNurbsCtrlPolyPart->coord;
	const M_Triangle& triangle = g_vcNurbsCtrlPolyPart->triangle;
	int polyCntr = 0;
	regionSelected.length = 0;
	int triIdx = 0;

	for(int k = 0; ; k++){
		regionSelected += false;

		while(--edgeCntr >= 0){
			const S_Triangle& tri = triangle[triIdx++];

			Vec2f va = convertPointToScreen2d(g_vcNurbsCtrlPolyPart->matrix * coord[tri.ia]);
			Vec2f vb = convertPointToScreen2d(g_vcNurbsCtrlPolyPart->matrix * coord[tri.ib]);
			Vec2f vc = convertPointToScreen2d(g_vcNurbsCtrlPolyPart->matrix * coord[tri.ic]);

			Vec2f tri2d[3] = {va, vb, vc};

			if(MultiSelect::isInBox(va) || MultiSelect::isInBox(vb) || MultiSelect::isInBox(vc) || triIntersectsRect(va, vb, vc) || isRectCornerInTriangle(tri2d)){
				regionSelected[polyCntr] = true;
			}
		}

		s += numOfCorners;
		if(s < region.length){
			if(region[s] < 0){
				numOfCorners = -region[s++];
			}
		}
		else{
			break;
		}

		edgeCntr = numOfCorners;

		polyCntr++;
	}
	houseKeep();
}

void Lock::deleteSelectedRegions(){
	MultiSelect::selectRectMode = false;
	if(region.length == 0){
		return;
	}

	int numOfCorners = 4;
	int s = 0;
	int d = 0;
	if(region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}
	int edgeCntr = numOfCorners;

	int polyCntr = 0;
	M_Int& region = this->region.refValue();

	for(int k = 0; ; k++){
		if(regionSelected[polyCntr]){
			s += numOfCorners;
		}
		else{
			if(s > 0 && region[s - 1] < 0){
				region[d++] = region[s - 1];
			}
			while(--edgeCntr >= 0){
				region[d++] = region[s++];
			}

		}

		
		if(s < region.length){
			if(region[s] < 0){
				numOfCorners = -region[s++];
			}
		}
		else{
			break;
		}

		edgeCntr = numOfCorners;

		polyCntr++;
	}
	region.length = d;
	regionSelected.length = 0;
	houseKeep();
}

void Region::BorderPoint::operator=(const Vec2f& par){

	if(Lock::selBorderLock){
		refValue() = par;
		Lock::selBorderLock->border.refValue()[Lock::selBorder] = par;
	}
}

void Hair::houseKeep(){

	setMathHiPresision();
	const float inverseSize = ( 1 / g_scene->getZoom());

	IndexedLineSet* polyline = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyline->coord.p)->point;
	coord.length = 0;
	polyline->coordIndex.refValue().length = 0;

/////////////
	M_Node& children = g_vcBezierBase->children;
	children.length = 1;

	for(int k = 0; k < lockList.length; k++){
		Lock* lock = lockList[k];
		if(lock->cage){
			M_Vec2f& hole = lock->hole.refValue();
			M_Int& scpi = lock->scpi.refValue();
			M_<Strand*>& scpiLathe = lock->scpiLathe;
			int numHoles = hole.length;
			for(int h = 0; h < numHoles; h++){
				Strand* bendy = scpiLathe[h];
				int pointIdx = scpi[h];
				Vec3f point = bendy->point[pointIdx];

				Transform* transform = new Transform;
				if(lock == selLock && h == selHoleIndex){
					transform->children.refValue() += g_redPoint.p;
				}
				else{
					transform->children.refValue() += g_nextPoint.p;
				}
				children += transform;

				transform->translation = point;
				transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
			}
		}
	}


///////////
	g_vcBezierBase->rotation = Rotation();

	g_visualClue = g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
}

void clone(Hair* hair, Strand* oldSd, Strand* newSd){
	hair->cloneSeperator(oldSd, newSd);
	g_scene->genNodeList2();
	g_stw->paint();
	int selLine = g_stw->selLine;
	if(Seamless* parentSmls = getSeamless(selLine)){
		parentSmls->populateMirrorNodes();
	}
	houseKeepVisualClue();
}

void Lock::deleteSeperatorMap(){
	hole.refValue().length = 0;
	scpi.refValue().length = 0;
	scpiLathe.length = 0;
}

/*
void Hair::copySeperatorMapForRepete(int fromIdx, int toIdx){ //assumes seperator map len == 0
	int beginIdx = beginIndex[fromIdx];
	int numHoles = numOfHoles[fromIdx];
	int diff = toIdx - fromIdx;

	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	M_Int scpiStore;
	M_<Strand*> scpiLatheStore;
	M_Vec2f holeStore;
	for(int k = 0; k < numHoles; k++){
		Strand* std = scpiLathe[beginIdx + k];
		int pointIdx = scpi[beginIdx + k];
		if(pointIdx + diff < std->point.length){
			scpiStore += pointIdx + diff;
			scpiLatheStore += std;
			holeStore += hole[beginIdx + k];
		}
	}
	int bginToIdx =  beginIndex[toIdx];
	insertElements(bginToIdx, 0, scpi, scpiStore);
	insertElements(bginToIdx, 0, scpiLathe, scpiLatheStore);
	insertElements(bginToIdx, 0, hole, holeStore);

	numHoles = scpiStore.length;
	numOfHoles.refValue()[toIdx] += numHoles;

	for(int k = toIdx + 1; k < beginIndex.length; k++){
		beginIndex.refValue()[k] += numHoles;
	}
}

void Hair::copySeperatorMap(int fromIdx, int toIdx){ //assumes seperator map len == 0
	int beginIdx = beginIndex[fromIdx];
	int numHoles = numOfHoles[fromIdx];

	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	M_Int scpiStore;
	M_<Strand*> scpiLatheStore;
	M_Vec2f holeStore;
	for(int k = 0; k < numHoles; k++){
		scpiStore += scpi[beginIdx + k];
		scpiLatheStore += scpiLathe[beginIdx + k];
		holeStore += hole[beginIdx + k];
	}
	int bginToIdx =  beginIndex[toIdx];
	insertElements(bginToIdx, 0, scpi, scpiStore);
	insertElements(bginToIdx, 0, scpiLathe, scpiLatheStore);
	insertElements(bginToIdx, 0, hole, holeStore);

	numHoles = scpiStore.length;
	numOfHoles.refValue()[toIdx] += numHoles;

	for(int k = toIdx + 1; k < beginIndex.length; k++){
		beginIndex.refValue()[k] += numHoles;
	}
}

void Hair::moveSeperatorMapBelow(int fromIdx, int toIdx){
	if(fromIdx > toIdx){
		fromIdx++;
	}
	beginIndex.refValue().insertElement(toIdx + 1, beginIndex[toIdx] + numOfHoles[toIdx]);
	numOfHoles.refValue().insertElement(toIdx + 1, 0);
	
	copySeperatorMap(fromIdx, toIdx + 1);

	deleteSeperatorMap(fromIdx);
	beginIndex.refValue().deleteElement(fromIdx);
	numOfHoles.refValue().deleteElement(fromIdx);
	processFields();
}
*/

Vec2f getPolyCenter(int numOfCorners, int beginIdx, const M_Vec2f& border, const M_Int& region){
	Vec2f center;

	for(int polyCntr = 0; polyCntr < numOfCorners; ){
		int ridx = region[beginIdx + polyCntr];
		center += border[ridx];
		if(++polyCntr == numOfCorners){
			center = center / numOfCorners;
			break;
		}
	}
	return center;
}

int Lock::numOfPolys(){
	int polys = 0;
	int numOfCorners = 4;
	int polyCntr = numOfCorners;
	int s = 0;
	for(int r = region.length; --r>=0;){
		int ridx = region[s++];
		if(ridx < 0){
			numOfCorners = -ridx;
			polyCntr = numOfCorners;
			continue;
		}
		if(--polyCntr == 0){
			polys++;
		}
	}
	return polys;
}

void Lock::copyMapRepete(Lock* src){
	//if(cage == false){
	//	return;
	//}
	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	scpi.length = 0;
	scpiLathe.length = 0;
	hole.length = 0;

	if(region.length){
		int numOfCorners = 4;
		Vec2f center;
		int s = 0;
		int remS = s;
		if(region.length && region[s] < 0){
			numOfCorners = -region[s++];
		}
		int polyCntr = numOfCorners;


		if(region.length == 4){

			Vec2f center = getPolyCenter(numOfCorners, s, src->border, region);

			Vec2f center2 = Vec2f(.5, .5);
			Vec2f square[4] = {Vec2f(0, 0), Vec2f(1, 0), Vec2f(1, 1), Vec2f(0, 1)};

			Vec2f v[3];
			Vec2f d[3];
			v[0] = center;
			d[0] = center2;

			for(int h = 0; h < numOfCorners; h++){
				for(int k = 0; k < src->scpi.length; k++){
					int pointIdx = src->scpi[k] + 1;
					Strand* sd = src->scpiLathe[k];
					if(pointIdx < sd->point.length){
						v[1] = src->border[region[s + h]];
						d[1] = square[h];
						int lIdx = s;
						int lIdx2 = 0;
						if(h != numOfCorners - 1){
							lIdx += h + 1;
							lIdx2 += h + 1;
						}
						v[2] = src->border[region[lIdx]];
						d[2] = square[lIdx2];
							
						Vec2f hl = src->hole[k];
						if(isTriangleHit(hl, v)){
							relateHitPointToMapTri(v[0], v[1], v[2], hl, d[0], d[1], d[2]);

							scpi += pointIdx;
							scpiLathe += sd;
							hole += hl;
						}
					}
				}
			//	i += numOfCorners;
				//this->scaleMap();
			}
		}
		else{
			for(int k = 0; ; k++){

				Vec2f center = getPolyCenter(numOfCorners, s, src->border, region);

				int s2 = s + numOfCorners;
				if(s2 >= region.length){
					throw;
				}
				if(region[s2] < 0){
					if(region[s2] != -numOfCorners){
						throw;
					}
					s2++;
				}
				Vec2f center2 = getPolyCenter(numOfCorners, s2, border, region);

				Vec2f v[3];
				Vec2f d[3];
				v[0] = center;
				d[0] = center2;

				for(int h = 0; h < numOfCorners; h++){
					for(int k = 0; k < src->scpi.length; k++){
						int pointIdx = src->scpi[k] + 1;
						Strand* sd = src->scpiLathe[k];
						if(pointIdx < sd->point.length){
							v[1] = src->border[region[s + h]];
							d[1] = border[region[s2 + h]];
							int lIdx = s;
							int lIdx2 = s2;
							if(h != numOfCorners - 1){
								lIdx += h + 1;
								lIdx2 += h + 1;
							}
							v[2] = src->border[region[lIdx]];
							d[2] = border[region[lIdx2]];
							
							Vec2f hl = src->hole[k];
							if(isTriangleHit(hl, v)){
								relateHitPointToMapTri(v[0], v[1], v[2], hl, d[0], d[1], d[2]);
								if(hl.x < 0){
									hl.x = 0;
								}
								if(hl.y < 0){
									hl.y = 0;
								}

								scpi += pointIdx;
								scpiLathe += sd;
								hole += hl;
							}
						}
					}
				}
				s = s2 + numOfCorners;
				if(s < region.length){
					if(region[s] < 0){
						numOfCorners = -region[s++];
					}
				}
				else{
					break;
				}

				polyCntr = numOfCorners;

				if(k < unify.length){
					src = dynamic_cast<Lock*>(unify[k].p);
				}
			}
		}
	}
	//else{
	//	for(int k = 0; k < src->scpi.length; k++){
	//		int pointIdx = src->scpi[k] + 1;
	//		Strand* sd = src->scpiLathe[k];
	//		if(pointIdx < sd->point.length){
	//			scpi += pointIdx;
	//			scpiLathe += sd;
	//			hole += src->hole[k];
	//		}
	//	}
	//}
	M_Node& m_lock = this->lock.refValue();
	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->copyMapRepete(this);
	}
}


void Lock::copyMapRepete(Lock* src, const M_Bool& flag){//same as above except checks strand is flaged and does not check lock is showing

	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	scpi.length = 0;
	scpiLathe.length = 0;
	hole.length = 0;

	if(region.length){
		int numOfCorners = 4;
		Vec2f center;
		int s = 0;
		int remS = s;
		if(region.length && region[s] < 0){
			numOfCorners = -region[s++];
		}
		int polyCntr = numOfCorners;


		if(region.length == 4){

			Vec2f center = getPolyCenter(numOfCorners, s, src->border, region);

			Vec2f center2 = Vec2f(.5, .5);
			Vec2f square[4] = {Vec2f(0, 0), Vec2f(1, 0), Vec2f(1, 1), Vec2f(0, 1)};

			Vec2f v[3];
			Vec2f d[3];
			v[0] = center;
			d[0] = center2;

			for(int h = 0; h < numOfCorners; h++){
				for(int k = 0; k < src->scpi.length; k++){
					int pointIdx = src->scpi[k] + 1;
					Strand* sd = src->scpiLathe[k];
					if(flag[sd->slidx]){
						if(pointIdx < sd->point.length){
							v[1] = src->border[region[s + h]];
							d[1] = square[h];
							int lIdx = s;
							int lIdx2 = 0;
							if(h != numOfCorners - 1){
								lIdx += h + 1;
								lIdx2 += h + 1;
							}
							v[2] = src->border[region[lIdx]];
							d[2] = square[lIdx2];
							
							Vec2f hl = src->hole[k];
							if(isTriangleHit(hl, v)){
								relateHitPointToMapTri(v[0], v[1], v[2], hl, d[0], d[1], d[2]);

								scpi += pointIdx;
								scpiLathe += sd;
								hole += hl;
							}
						}
					}
				}
			//	i += numOfCorners;
				//this->scaleMap();
			}
		}
		else{
			for(int k = 0; ; k++){

				Vec2f center = getPolyCenter(numOfCorners, s, src->border, region);

				int s2 = s + numOfCorners;
				if(region[s2] < 0){
					s2++;
				}
				Vec2f center2 = getPolyCenter(numOfCorners, s2, border, region);

				Vec2f v[3];
				Vec2f d[3];
				v[0] = center;
				d[0] = center2;

				for(int h = 0; h < numOfCorners; h++){
					for(int k = 0; k < src->scpi.length; k++){
						int pointIdx = src->scpi[k] + 1;
						Strand* sd = src->scpiLathe[k];
						if(flag[sd->slidx]){
							if(pointIdx < sd->point.length){
								v[1] = src->border[region[s + h]];
								d[1] = border[region[s2 + h]];
								int lIdx = s;
								int lIdx2 = s2;
								if(h != numOfCorners - 1){
									lIdx += h + 1;
									lIdx2 += h + 1;
								}
								v[2] = src->border[region[lIdx]];
								d[2] = border[region[lIdx2]];
							
								Vec2f hl = src->hole[k];
								if(isTriangleHit(hl, v)){
									relateHitPointToMapTri(v[0], v[1], v[2], hl, d[0], d[1], d[2]);

									scpi += pointIdx;
									scpiLathe += sd;
									hole += hl;
								}
							}
						}
					}
				}
				s = s2 + numOfCorners;
				if(s < region.length){
					if(region[s] < 0){
						numOfCorners = -region[s++];
					}
				}
				else{
					break;
				}

				polyCntr = numOfCorners;

				if(k < unify.length){
					src = dynamic_cast<Lock*>(unify[k].p);
				}
			}
		}
	}
	else{
		for(int k = 0; k < src->scpi.length; k++){
			int pointIdx = src->scpi[k] + 1;
			Strand* sd = src->scpiLathe[k];
			if(flag[sd->slidx]){
				if(pointIdx < sd->point.length){
					scpi += pointIdx;
					scpiLathe += sd;
					hole += src->hole[k];
				}
			}
		}
	}
	M_Node& m_lock = this->lock.refValue();
	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->copyMapRepete(this, flag);
	}
}



void Lock::selectLocksFromRegions(Lock* src){
	cage.refValue() = true;

	M_Node& m_lock = this->lock.refValue();
	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->selectLocksFromRegions(this);
	}
}


void repeatSeperatorMap(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){

			//if(Lock* parentLock = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
			//	lock->copyMapRepete(parentLock);
			//}

			M_Node& m_lock = lock->lock.refValue();
			for(int k = 0; k < m_lock.length; k++){
				dynamic_cast<Lock*>(m_lock[k].p)->copyMapRepete(lock);
			}

/*		
			int selIdx = hair->getSelectedSeperatorIndex();
		
			M_Node& m_seperator = hair->lock.refValue();
			for(int k = selIdx + 1; k < m_seperator.length; k++){
				NurbsPatch* sep = dynamic_cast<NurbsPatch*>(m_seperator[k].p);
				if(sep->cage == false){
					break;
				}
				hair->deleteSeperatorMap(k);
				hair->copySeperatorMapForRepete(selIdx, k);
			}
*/
			hair->part->smls->doBuild();
			hair->processFields();
			hair->part->smls->doBuild();
			houseKeepVisualClue();
			hair->part->smls->ctrf = true;
		}
	}
}

void Lock::autoLengthStrands(Lock* src){

	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	scpi.length = 0;
	scpiLathe.length = 0;
	hole.length = 0;

	if(region.length){
		int numOfCorners = 4;
		Vec2f center;
		int s = 0;
		int remS = s;
		if(region.length && region[s] < 0){
			numOfCorners = -region[s++];
		}
		int polyCntr = numOfCorners;


		if(region.length == 4){

			Vec2f center = getPolyCenter(numOfCorners, s, src->border, region);

			Vec2f center2 = Vec2f(.5, .5);
			Vec2f square[4] = {Vec2f(0, 0), Vec2f(1, 0), Vec2f(1, 1), Vec2f(0, 1)};

			Vec2f v[3];
			Vec2f d[3];
			v[0] = center;
			d[0] = center2;

			for(int h = 0; h < numOfCorners; h++){
				for(int k = 0; k < src->scpi.length; k++){
					int pointIdx = src->scpi[k] + 1;
					Strand* sd = src->scpiLathe[k];

					int vDim = pointIdx + 1;
					sd->dim.refValue().y = vDim;
					sd->point.refValue().length = vDim;
					sd->dpoint.refValue().length = vDim;
					sd->weight.refValue().length = vDim;
					updateNurbsCurveKnotMid0(vDim, sd->vKnot.refValue());

					if(pointIdx < sd->point.length){
						v[1] = src->border[region[s + h]];
						d[1] = square[h];
						int lIdx = s;
						int lIdx2 = 0;
						if(h != numOfCorners - 1){
							lIdx += h + 1;
							lIdx2 += h + 1;
						}
						v[2] = src->border[region[lIdx]];
						d[2] = square[lIdx2];
							
						Vec2f hl = src->hole[k];
						if(isTriangleHit(hl, v)){
							relateHitPointToMapTri(v[0], v[1], v[2], hl, d[0], d[1], d[2]);

							scpi += pointIdx;
							scpiLathe += sd;
							hole += hl;
						}
					}
				}
			}
		}
		else{
			for(int k = 0; ; k++){

				Vec2f center = getPolyCenter(numOfCorners, s, src->border, region);

				int s2 = s + numOfCorners;
				if(region[s2] < 0){
					s2++;
				}
				Vec2f center2 = getPolyCenter(numOfCorners, s2, border, region);

				Vec2f v[3];
				Vec2f d[3];
				v[0] = center;
				d[0] = center2;

				for(int h = 0; h < numOfCorners; h++){
					for(int k = 0; k < src->scpi.length; k++){
						int pointIdx = src->scpi[k] + 1;
						Strand* sd = src->scpiLathe[k];


						v[1] = src->border[region[s + h]];
						d[1] = border[region[s2 + h]];
						int lIdx = s;
						int lIdx2 = s2;
						if(h != numOfCorners - 1){
							lIdx += h + 1;
							lIdx2 += h + 1;
						}
						v[2] = src->border[region[lIdx]];
						d[2] = border[region[lIdx2]];
							
						Vec2f hl = src->hole[k];
						if(isTriangleHit(hl, v)){
							relateHitPointToMapTri(v[0], v[1], v[2], hl, d[0], d[1], d[2]);
							if(hl.x < 0){
								hl.x = 0;
							}
							if(hl.y < 0){
								hl.y = 0;
							}

							scpi += pointIdx;
							scpiLathe += sd;
							hole += hl;

							int vDim = pointIdx + 1; //this is the only bit thats different to repete, not quite any more
							sd->dim.refValue().y = vDim;
							sd->point.refValue().length = vDim;
							sd->dpoint.refValue().length = vDim;
							sd->weight.refValue().length = vDim;
							updateNurbsCurveKnotMid0(vDim, sd->vKnot.refValue());

						}
					}
				}
				s = s2 + numOfCorners;
				if(s < region.length){
					if(region[s] < 0){
						numOfCorners = -region[s++];
					}
				}
				else{
					break;
				}

				polyCntr = numOfCorners;

				if(k < unify.length){
					src = dynamic_cast<Lock*>(unify[k].p);
				}
			}
		}
	}
	M_Node& m_lock = this->lock.refValue();
	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->autoLengthStrands(this);
	}
}

void autoLengthStrands(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){

			M_Node& m_lock = lock->lock.refValue();
			for(int k = 0; k < m_lock.length; k++){
				dynamic_cast<Lock*>(m_lock[k].p)->autoLengthStrands(lock);
			}

			//hair->part->smls->doBuild();
			hair->processFields();
			hair->part->smls->doBuild();
			houseKeepVisualClue();
			hair->part->smls->ctrf = true;
		}
	}
}



Lock* getOtherNonSelectedLock(M_<Lock*>& lockList, const Lock* selLock){
	for(int k = 0; k < lockList.length; k++){
		Lock* lock = lockList[k];
		if(lock->cage && selLock != lockList[k]){
			return lock;
		}
	}
	return 0;
}

void Lock::copyMap(Lock* src){
	if(cage == false){
		return;
	}

	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	scpi.length = 0;
	scpiLathe.length = 0;
	hole.length = 0;

	for(int k = 0; k < src->scpi.length; k++){
		int pointIdx = src->scpi[k] + 1;
		Strand* sd = src->scpiLathe[k];
		if(sd->cage && pointIdx < sd->point.length){
			scpi += pointIdx;
			scpiLathe += sd;
			hole += src->hole[k];
		}
	}
}

void copyLockMap(){
	if(Lock* selLock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){
			if(Lock* srcLock = getOtherNonSelectedLock(hair->lockList, selLock)){

				selLock->copyMap(srcLock);
				//selLock->scaleMap();

				hair->part->smls->doBuild();
				hair->processFields();
				hair->part->smls->doBuild();
				houseKeepVisualClue();
				hair->part->smls->ctrf = true;
			}
		}
	}
}


void addBorders(int uDim, int vDim, M_Vec2f& border){
	border.length = uDim * vDim;
	for(int v = 0; v < vDim; v++){
		for(int u = 0; u < uDim; u++){
			border[v * uDim + u] = Vec2f(u / static_cast<float>(uDim - 1), v / static_cast<float>(vDim - 1));
		}
	}
}

void addRegions(int uDim, int vDim, M_Int& region){
	region.length = 0;
	for(int v = 1; v < vDim; v++){
		for(int u = 1; u < uDim; u++){
			region += -4;
			region += (v - 1) * uDim + u - 1;
			region += (v - 1) * uDim + u;
			region += v * uDim + u;
			region += v * uDim + u - 1;

			region += -4;
			region += (v - 1) * uDim + u - 1;
			region += (v - 1) * uDim + u;
			region += v * uDim + u;
			region += v * uDim + u - 1;
		}
	}
}

void addRegions(){
	if(Lock* selLock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Lock* srcLock = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
			if(selLock->dim == srcLock->dim){
				int uDim = selLock->dim.x / 2 + 1;
				int vDim = selLock->dim.y / 2 + 1;

				addBorders(uDim, vDim, selLock->border.refValue());
				addBorders(uDim, vDim, srcLock->border.refValue());
				addRegions(uDim, vDim, selLock->region.refValue());

				houseKeepVisualClue();
			}
		}
	}
}

void pasteBorderRegion(){
	if(Lock* selLock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Lock* copyLock = dynamic_cast<Lock*>(g_copyNode.getRaw())){
			selLock->region.refValue() = copyLock->region;
			selLock->border.refValue() = copyLock->border;
			houseKeepVisualClue();
		}
	}
}

void Lock::scaleMap(){
	M_Vec2f& hole = this->hole.refValue();
	float west = 1;
	float east = 0;
	float south = 1;
	float north = 0;

	for(int i = hole.length; --i >= 0;){
		Vec2f hl = hole[i];
		if(hl.x < west){
			west = hl.x;
		}
		else if(hl.x > east){
			east = hl.x;
		}
		if(hl.y < south){
			south = hl.y;
		}
		else if(hl.y > north){
			north = hl.y;
		}
	}
	Vec2f scale = Vec2f(1.0 / (east - west), 1.0 / (north - south));
	Vec2f offset = Vec2f(west, south);

	for(int i = hole.length; --i >= 0;){
		hole[i] = (hole[i] - offset) * scale;
	}
}

void scaleLockMap(){
	if(Lock* selLock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		selLock->scaleMap();
		updateBuild3dWindow();
	}
}

void Lock::showHideBoundStrands(bool state){
	for(int i = scpiLathe.length; --i>=0;){
		scpiLathe[i]->cage = state;
	}
}

void Hair::showHideBoundStrands(bool state){
	M_Strand& m_strand = this->strand.refValue();
	bool istate = !state;
	for(int k = m_strand.length; --k>=0;){
		m_strand[k]->cage = istate;
	}
	for(int k = 0; k < lockList.length; k++){
		Lock* lock = lockList[k];
		if(lock->cage){
			lock->showHideBoundStrands(state);
		}
	}
	g_stw->paint();
	updateBuild3dWindow();
}

void hideBoundStrands(){
	if(Hair* hair = getHair()){
		hair->showHideBoundStrands(false);
	}
}

void showBoundStrands(){
	if(Hair* hair = getHair()){
		hair->showHideBoundStrands(true);
	}
}

void cutLock(){
	if(Lock* selLock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		
		selLock->cutLock();

		repeatSeperatorMap();
		deleteNode();

		//if(Hair* hair = getHair()){
		//	hair->refreshHairOffset();
		//}

		//updateBuild3dWindow();
	}
}

/*
void decrementSeperatorMap(){
	if(Lock* selLock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		
		selLock->decrementMap();

		updateBuild3dWindow();
	}
}
void Lock::decrementMap(){

	M_Int& scpi = this->scpi.refValue();

	for(int k = scpi.length; --k >= 0;){
		int pointIdx = scpi[k];
		if(pointIdx > 1){
			scpi[k] = pointIdx - 1;
		}
	}
}
*/
void interpolateLock(Lock* newLock, Lock* childLock, float ratio){
	M_Vec3f& dpoint = newLock->dpoint.refValue();
	M_Vec3f& childDpoint = childLock->dpoint.refValue();
	if(dpoint.length != childDpoint.length){
		return;
	}

	for(int k = dpoint.length; --k>=0;){
		dpoint[k] = dpoint[k] * (1 - ratio) + childDpoint[k] * ratio;
	}
	newLock->dpointsToPoints();
}


void Lock::flagStrandsInLock(M_Bool& flag){
	for(int k = scpiLathe.length; --k>=0;){
		flag[scpiLathe[k]->slidx] = true;
	}
}

Lock* insertLock(Lock* lock, Lock* childLock){
	Lock* newLock = new Lock;

	newLock->dpoint.refValue() = lock->dpoint;
	newLock->point.refValue() = lock->point;
	newLock->weight.refValue() = lock->weight;
	newLock->uKnot.refValue() = lock->uKnot;
	newLock->vKnot.refValue() = lock->vKnot;
	newLock->dim.refValue() = lock->dim;
	newLock->uJoinEnds.refValue() = lock->uJoinEnds;
				
	newLock->border.refValue() = childLock->border;
	newLock->region.refValue() = childLock->region;

	return newLock;
}

void insertLock(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		g_stw->cp->setupNodeWindow(g_insertLock);
		Window::activeWindow = g_stw->cp.p;
	}
}

void ringletGenerator(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		g_stw->cp->setupNodeWindow(g_ringletGenerator);
		Window::activeWindow = g_stw->cp.p;
	}
}

void InsertLock::Ok::operator=(bool const&){
	int insertions = g_insertLock->insertions;
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){
			if(lock->lock.length){
				Lock* childLock = dynamic_cast<Lock*>(lock->lock.refValue()[0].p);
				int selIdx = 0;//hair->getSelectedSeperatorIndex();
				if(selIdx >= 0){
					Lock* newLock = 0;
					M_Node holdChildren = lock->lock;
					lock->lock.refValue().length = 0;
					Lock* prevLock = lock;

					M_Bool flag;
					flag.length = hair->strand.length;
					lock->flagStrandsInLock(flag);
					hair->insertStrandControlPoints(lock, flag);

					for(int k = 0; k < insertions; k++){
						newLock = insertLock(lock, childLock);

						prevLock->lock.refValue() += newLock;

						interpolateLock(newLock, childLock, 1.0 / (insertions + 1) * (k + 1));
						hair->insertStrandControlPoints(lock, flag);

						prevLock = newLock;
					}
					newLock->lock.refValue() = holdChildren;

					dynamic_cast<Lock*>(lock->lock.refValue()[0].p)->copyMapRepete(lock, flag);

					hair->processFields();
					g_stw->genNodeList();
					resetCPNodeList();

					//g_stw->lesserPostInsertNodeStuff(false);


					Window::closeWindow(Window::activeWindow.getActiveWindow());
					g_stw->cp->setupNodeWindow(g_selNode.getRaw());
					Window::activeWindow = g_stw->cp.p;



					g_stw->paint();
					updateBuild3dWindow();

				}
			}
		}
	}
}


#include <time.h>
int density[8][6] = {
	10, 40, 40, 40, 40, 30,
	80, 70, 70, 70, 70, 60,
	110, 110, 110, 110, 110, 100,
	140, 140, 140, 140, 140, 130,
	140, 140, 140, 140, 140, 130,
	110, 110, 110, 110, 110, 100,
	80, 70, 70, 70, 70, 60,
	110, 40, 40, 40, 40, 30,
};
void seed(){
	//int numOfStrands = 4200;
	int numOfStrands = 5;
	srand( (unsigned)time( NULL ) );
	int rows = 1;//8;
	int cols = 1;//6;
	if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		hair->strand.refValue().length = 0;
		Lock* lock = dynamic_cast<Lock*>(hair->lock.refValue()[0].p);
		M_Vec2f& hole = lock->hole.refValue();
		M_Int& scpi = lock->scpi.refValue();
		M_<Strand*>& scpiLathe = lock->scpiLathe;

		hole.length = 0;
		scpi.length = 0;
		scpiLathe.length = 0;


		for(int r = 0; r < rows; r++){
			for(int c = 0; c < cols; c++){
				numOfStrands = density[r][c];
				for(int s = 0; s < numOfStrands; s++){
		
					int uDim = 1;
					int vDim = 3;

					Strand* ctrlLine = new Strand;
					ctrlLine->dim.refValue() = Vec2f(uDim, vDim);
					ctrlLine->point.refValue().length = vDim;
					ctrlLine->weight.refValue().length = vDim;
					ctrlLine->uJoinEnds.refValue() = false;


					for(int k = 0; k < vDim; k++){
						ctrlLine->point.refValue()[k] = Vec3f(0, .1 * k, 0);
						ctrlLine->weight.refValue()[k] = 1;
					}

					ctrlLine->uKnot.refValue() = M_Float(0, 0, 0, .25, .25, .5, .5, .75, .75, 1, 1, 1);
					updateNurbsCurveKnotMid0(vDim, ctrlLine->vKnot.refValue());
					

					ctrlLine->processFields();
					hair->strand.refValue() += ctrlLine;
				}
			}
		}
		hair->refreshHairOffset();
		hair->processFields();
		int si = 0;
		for(int r = 0; r < rows; r++){
			for(int c = 0; c < cols; c++){
				numOfStrands = density[r][c];
				M_Strand& m_strand = hair->strand.refValue();
				for(int s = 0; s < numOfStrands; s++){
					Strand* sd = m_strand[si++];

					double range_minf = 0;
					double range_maxf = 1;

					float x = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
					float y = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);

					x = x / rows;
					y = y / cols;
					x += r * (1.0 / rows);
					y += c * (1.0 / cols);
					Vec2f hl = Vec2f(x, y);

					hole.insertElement(0, hl);
					scpi.insertElement(0, 0);
					scpiLathe.insertElement(0, sd);
				}
			}
		}

		g_stw->commonPostInsertNodeStuff(false);
	}
}

void Lock::deleteStrandsInSelRegions(M_Bool& flag){
	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	int d = 0;

	for(int s = 0; s < hole.length; s++){
		if(flag[scpiLathe[s]->slidx] == false){
			scpi[d] = scpi[s];
			scpiLathe[d] = scpiLathe[s];
			hole[d] = hole[s];
			d++;
		}
	}

	scpi.length = d;
	scpiLathe.length = d;
	hole.length = d;

	M_Node& m_lock = this->lock.refValue();

	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->deleteStrandsInSelRegions(flag);
	}
}


void Hair::deleteStrandsInSelRegions(M_Bool& flag){
	M_Node& m_lock = this->lock.refValue();

	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->deleteStrandsInSelRegions(flag);
	}

	M_Strand& m_strand = this->strand.refValue();

	int d = 0;

	for(int s = 0; s < m_strand.length; s++){
		if(flag[s] == false){
			m_strand[d] = m_strand[s];
			d++;
		}
	}
	int diff = m_strand.length - d;
	m_strand.length = d;
	
	g_stw->selLine -= diff;

	processFields();
}

void Lock::selectLocks(M_Bool& flag){
	cage.refValue() = false;
	for(int i = scpiLathe.length; --i >= 0;){
		if(flag[scpiLathe[i]->slidx]){
			cage.refValue() = true;
			break;
		}
	}

	M_Node& m_lock = this->lock.refValue();

	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->selectLocks(flag);
	}
}

void Hair::selectLocks(M_Bool& flag){
	for(int k = flag.length; --k >= 0;){
		strand.refValue()[k]->cage.refValue() = flag[k];
	}

	M_Node& m_lock = this->lock.refValue();

	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->selectLocks(flag);
	}
}

void Lock::flagStrandsInLock(M_Bool& flag, int& numOfCorners, int& s, const M_Int& region, int polyCntr){
	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();

	if(s < region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}
	int cornerCntr = numOfCorners;

	if(regionSelected[polyCntr]){
		Vec2f v[3];

		v[0] = getPolyCenter(numOfCorners, s, border, region);

		for(int h = 0; h < numOfCorners; h++){
			for(int k = 0; k < scpi.length; k++){
				v[1] = border[region[s + h]];
				int lIdx = s;
				if(h != numOfCorners - 1){
					lIdx += h + 1;
				}
				v[2] = border[region[lIdx]];
							
				Vec2f hl = hole[k];
				if(isTriangleHit(hl, v)){
					flag[scpiLathe[k]->slidx] = true;
				}
			}
		}
	}
	s = s + numOfCorners;
}


void Hair::flagStrandsInLock(Lock* lock, M_Bool& flag){
	int numOfCorners = 4;
	flag.length = strand.length;
	M_Node& unify = lock->unify.refValue();

	if(Lock* src = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
		int s = 0;
		int polyCntr = 0;
		M_Int& region = lock->region.refValue();
		for(int k = 0; ; k++){
			src->flagStrandsInLock(flag, numOfCorners, s, region, polyCntr++);
			lock->flagStrandsInLock(flag, numOfCorners, s, region, polyCntr++);

			if(s == region.length && k >= unify.length){
				break;
			}
			if(k < unify.length){
				src = dynamic_cast<Lock*>(unify[k].p);
			}
		}
	}
}

void selectLocksFromRegions(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){
			M_Bool flag;
			hair->flagStrandsInLock(lock, flag);

			hair->selectLocks(flag);
			g_stw->paint();
			getSeamless()->ctrf = true;
			hair->doBuild();
			//updateBuild3dWindow();
		}
	}
}

void deleteStrandsInSelRegions(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){
			M_Bool flag;
			hair->flagStrandsInLock(lock, flag);

			hair->deleteStrandsInSelRegions(flag);
			g_stw->commonPostInsertNodeStuff();
			g_stw->paint();
			updateBuild3dWindow();
		}
	}
}


bool Lock::isInsideRegion(int& numOfCorners, int& s, const M_Int& region, const Vec2f hl){
	M_Int& scpi = this->scpi.refValue();
	M_Vec2f& hole = this->hole.refValue();
	bool flg = false;

	if(s < region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}
	int cornerCntr = numOfCorners;

	Vec2f v[3];

	v[0] = getPolyCenter(numOfCorners, s, border, region);

	for(int h = 0; h < numOfCorners; h++){
		v[1] = border[region[s + h]];
		int lIdx = s;
		if(h != numOfCorners - 1){
			lIdx += h + 1;
		}
		v[2] = border[region[lIdx]];
							
		if(isTriangleHit(hl, v)){
			flg = true;
			break;
		}
	}
	return flg;
}

void nextRegion(int& numOfCorners, int& s, const M_Int& region){

	if(s < region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}
	int cornerCntr = numOfCorners;

	s = s + numOfCorners;
}

void nextRegion(int& numOfCorners, int& s, const M_Int& region, int& polyCntr){

	nextRegion(numOfCorners, s, region);
	polyCntr++;
}


void Hair::addNewBoundStrand(Lock* lock, const Vec2f hl){
	int uDim = 1;
	int vDim = 3;

	Strand* ctrlLine = new Strand;
	ctrlLine->dim.refValue() = Vec2f(uDim, vDim);
	ctrlLine->point.refValue().length = vDim;
	ctrlLine->weight.refValue().length = vDim;
	ctrlLine->uJoinEnds.refValue() = false;


	for(int k = 0; k < vDim; k++){
		ctrlLine->point.refValue()[k] = Vec3f(0, .1 * k, 0);
		ctrlLine->weight.refValue()[k] = 1;
	}

	ctrlLine->uKnot.refValue() = M_Float(0, 0, 0, .25, .25, .5, .5, .75, .75, 1, 1, 1);
	updateNurbsCurveKnotMid0(vDim, ctrlLine->vKnot.refValue());
					

	ctrlLine->processFields();

	lock->hole.refValue() += hl;
	lock->scpi.refValue() += 0;
	lock->scpiLathe += ctrlLine;
	strand.refValue() += ctrlLine;
}


void Hair::seedStrandsInSelRegions(Lock* lock, int numOfSeeds){
	
	int numOfCorners = 4;
	M_Node& unify = lock->unify.refValue();

	if(Lock* src = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
		int s = 0;
		int polyCntr = 0;
		M_Int& region = lock->region.refValue();
		for(int k = 0; ; k++){

			if(Lock::regionSelected[polyCntr]){

				for(int r = 0; r < numOfSeeds;){
					double range_minf = 0;
					double range_maxf = 1;

					float x = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
					float y = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);

					Vec2f hl = Vec2f(x, y);

					if(src->isInsideRegion(numOfCorners, s, region, hl)){
						r++;

						addNewBoundStrand(src, hl);
					}
				}
			}
			nextRegion(numOfCorners, s, region, polyCntr);

			nextRegion(numOfCorners, s, region, polyCntr);

			if(s == region.length && k >= unify.length){
				break;
			}
			if(k < unify.length){
				src = dynamic_cast<Lock*>(unify[k].p);
			}
		}
	}
}

void seedStrandsInSelRegions(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		g_stw->cp->setupNodeWindow(g_seed);
		Window::activeWindow = g_stw->cp.p;
	}
}

void Seed::Ok::operator=(bool const&){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){//should not need to check if selected node is Lock
		if(Hair* hair = getHair()){

			int numOfseeds = home.number_of_strands;
			hair->seedStrandsInSelRegions(lock, numOfseeds);

			g_stw->genNodeList();
			g_stw->selLine += numOfseeds;

			Window::closeWindow(Window::activeWindow.getActiveWindow());
			g_stw->cp->setupNodeWindow(g_selNode.getRaw());
			Window::activeWindow = g_stw->cp.p;

			g_stw->paint();
			updateBuild3dWindow();
		}
	}
}


void Lock::connectLockToRegion(M_Int& region, int s, const M_Vec2f& border, int numOfCorners, Lock* copyLock, M_Int& redirect){
	if(s < region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}

	copyLock->region.refValue() += -numOfCorners;

	int s2 = s;
	for(int cornerCntr = numOfCorners; --cornerCntr >= 0;){
		copyLock->region.refValue() += region[s2++];
	}

	copyLock->region.refValue() += -numOfCorners;

	for(int cornerCntr = numOfCorners; --cornerCntr >= 0;){
		int borderIdx = region[s++];
		if(redirect[borderIdx] < 0){
			redirect[borderIdx] = copyLock->border.length;
			copyLock->border.refValue() += border[borderIdx];
		}
		copyLock->region.refValue() += redirect[borderIdx];
	}
}

void Lock::scaleBorder(int borderOffset){
	M_Vec2f& border = this->border.refValue();
	float west = 1;
	float east = 0;
	float south = 1;
	float north = 0;

	for(int i = border.length; --i >= borderOffset;){
		Vec2f hl = border[i];
		if(hl.x < west){
			west = hl.x;
		}
		else if(hl.x > east){
			east = hl.x;
		}
		if(hl.y < south){
			south = hl.y;
		}
		else if(hl.y > north){
			north = hl.y;
		}
	}
	Vec2f scale = Vec2f(1.0 / (east - west), 1.0 / (north - south));
	Vec2f offset = Vec2f(west, south);

	for(int i = border.length; --i >= borderOffset;){
		border[i] = (border[i] - offset) * scale;
		//border[i] = border[i] - offset;
	}
}

void Lock::connectLockToSelRegions(Lock* src, Lock* copyLock, M_Int& redirect){
	int numOfCorners = 4;
	int s = 0;
	int polyCntr = 0;
	for(int k = 0; ; k++){

		nextRegion(numOfCorners, s, region, polyCntr);

		if(Lock::regionSelected[polyCntr]){
			connectLockToRegion(region, s, border, numOfCorners, copyLock, redirect);
		}

		nextRegion(numOfCorners, s, region, polyCntr);

		if(s == region.length && k >= unify.length){
			break;
		}
		if(k < unify.length){
			src = dynamic_cast<Lock*>(unify[k].p);
		}
	}
}

void connectLockToSelRegions(){
	if(Lock* copyLock = dynamic_cast<Lock*>(g_copyNode.getRaw())){
		if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
			if(Lock* src = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
				M_Int redirect;
				redirect.length = lock->region.length;
				for(int k = redirect.length; --k>=0;){
					redirect[k] = -1;
				}
				int borderOffset = copyLock->border.length;
				lock->connectLockToSelRegions(src, copyLock, redirect);
				copyLock->scaleBorder(borderOffset);
			}
		}
	}
}

Lock* Lock::createLockFromRegion(int s, int numOfCorners, int brdrOffstt){
	if(s < region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}

	M_Vec3f corners;
	corners.length = numOfCorners;

	IndexedLineSet* polyline = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyline->coord.p)->point;

	for(int b = 0; b < numOfCorners; b++){
		corners[b] = coord[region[s + b] + brdrOffstt];
	}


	Lock* newLock = new Lock;
	int len = 9;
	newLock->point.refValue().length = len;
	newLock->weight.refValue().length = len;
	for(int i = len; --i>= 0;){
		newLock->weight.refValue()[i] = 1;
	}

	newLock->point.refValue()[0] = corners[0];
	newLock->point.refValue()[2] = corners[1];
	newLock->point.refValue()[8] = corners[2];
	newLock->point.refValue()[6] = corners[3];

	newLock->point.refValue()[1] = (corners[0] + corners[1]) / 2.0;
	newLock->point.refValue()[3] = (corners[0] + corners[3]) / 2.0;
	newLock->point.refValue()[4] = (corners[0] + corners[1] + corners[2] + corners[3]) / 4.0;
	newLock->point.refValue()[5] = (corners[1] + corners[2]) / 2.0;
	newLock->point.refValue()[7] = (corners[3] + corners[2]) / 2.0;


	newLock->dim.refValue() = Vec2f(3, 3);

	newLock->updateKnot(newLock->dim.refValue().x, newLock->uKnot.refValue(), newLock->order.refValue().x);
	newLock->updateKnot(newLock->dim.refValue().y, newLock->vKnot.refValue(), newLock->order.refValue().y);

	newLock->uJoinEnds.refValue() = false;
	newLock->processFields();
				
	return newLock;
}

Lock* Lock::extrudeRegion(M_Int& redirect){
	int numOfCorners = 4;
	int s = 0;
	int polyCntr = 0;
	for(int k = 0; ; k++){

		nextRegion(numOfCorners, s, region, polyCntr);

		if(Lock::regionSelected[polyCntr]){
			Lock* newLock = createLockFromRegion(s, numOfCorners, borderOffset[polyCntr]);
			lock.refValue() += newLock;

			connectLockToRegion(region, s, border, numOfCorners, newLock, redirect);
			//newLock->scaleBorder(0);

			return newLock;
		}

		nextRegion(numOfCorners, s, region, polyCntr);

		if(s == region.length && k >= unify.length){
			break;
		}
	}
	return 0;
}


void Lock::splitLock2(M_Int& region, int sA, int sB, const M_Vec2f& border, int numOfCorners, Lock* newLock, M_Int& redirect){
	if(sA < region.length && region[sA] < 0){
		numOfCorners = -region[sA++];
	}
	if(sB < region.length && region[sB] < 0){
		numOfCorners = -region[sB++];
	}
	newLock->region.refValue() += -numOfCorners;

	for(int cornerCntr = numOfCorners; --cornerCntr >= 0;){
		newLock->region.refValue() += region[sA++];
	}


	newLock->region.refValue() += -numOfCorners;

	for(int cornerCntr = numOfCorners; --cornerCntr >= 0;){
		int borderIdx = region[sB++];
		if(redirect[borderIdx] < 0){
			redirect[borderIdx] = newLock->border.length;
			newLock->border.refValue() += border[borderIdx];
		}
		newLock->region.refValue() += redirect[borderIdx];
	}

}

bool compareRegionIndex(int offset, M_Int& regionA, int beginSA, M_Int& regionB, int sB, int cornerCntr){
	int sA = beginSA + offset;
	for(; --cornerCntr >= 0;){
		if(sA == regionA.length || regionA[sA] < 0){
			sA = beginSA;
		}
		if(regionA[sA++] != regionB[sB++]){
			return false;
		}
	}
	return true;
}

bool compareRegionIndex(M_Int& regionA, int beginSA, M_Int& regionB, int sB, int  numOfCorners){
	for(int offset = 0; offset <  numOfCorners; offset++){
		if(compareRegionIndex(offset, regionA, beginSA, regionB, sB, numOfCorners)){
			return true;
		}
	}
	return false;
}

int Lock::findMatchingChildBase(M_Int& compareRegion, int compareS, int numOfCorners){
	int numOfCorners2 = numOfCorners;
	if(compareS < compareRegion.length && compareRegion[compareS] < 0){
		numOfCorners2 = -compareRegion[compareS++];
	}
	int sBase = 0;
	if(sBase < region.length && region[sBase] < 0){
		numOfCorners = -region[sBase++];
	}

	int cs = compareS;
	int s = sBase;

	while(true){
		if(compareRegionIndex(region, sBase, compareRegion, compareS, numOfCorners)){
			return sBase;
		}

		do{
			sBase += numOfCorners2;
			if(sBase < region.length && region[sBase] < 0){
				numOfCorners2 = -region[sBase++];
			}
			sBase += numOfCorners2;
			if(sBase < region.length && region[sBase] < 0){
				numOfCorners2 = -region[sBase++];
			}

			if(sBase >= region.length){
				return -1;
			}
		}while(numOfCorners2 != numOfCorners);
	}
	return -1;
}

void Lock::redirectChildBase(int sDst, M_Int& fromRegion, int numOfCorners){
	int sSrc = 0;
	if(sSrc < region.length && region[sSrc] < 0){
		sSrc++;
	}
	sSrc += numOfCorners;
	if(sSrc < region.length && region[sSrc] < 0){
		sSrc++;
	}

	for(int cornerCntr = numOfCorners; --cornerCntr >= 0;){
		region.refValue()[sDst++] = fromRegion[sSrc++];
	}
}

Lock* Lock::splitLock(Lock* src, M_Int& redirect){
	int numOfCorners = 4;
	int s = 0;
	int polyCntr = 0;
	for(int k = 0; ; k++){
		int sA = s;

		nextRegion(numOfCorners, s, region, polyCntr);

		if(Lock::regionSelected[polyCntr]){
			Lock* newLock = createLockFromRegion(s, numOfCorners, borderOffset[polyCntr]);
			src->lock.refValue() += newLock;

			splitLock2(region, sA, s, border, numOfCorners, newLock, redirect);

			M_Node& m_lock = this->lock.refValue();
			int dk = 0;
			for(int sk = 0; sk < m_lock.length; sk++){
				Lock* childLock = dynamic_cast<Lock*>(m_lock[sk].p);
				m_lock[dk] = childLock;
				
				int sBase = childLock->findMatchingChildBase(region, s, numOfCorners);
					
				if(sBase >= 0){
					childLock->redirectChildBase(sBase, childLock->region.refValue(), numOfCorners);
					newLock->lock.refValue() += childLock;
				}
				else{
					dk++;
				}
			}
			m_lock.length = dk;
			//newLock->scaleBorder(0);

			return newLock;
		}

		nextRegion(numOfCorners, s, region, polyCntr);

		if(s == region.length && k >= unify.length){
			break;
		}
	}
	return 0;
}


Vec3f calcRingPivotPointForHair(NurbsPatch* patch);

void duplicteLock(Lock* lock, int insertions){
	int polyCntr = 0; //not used but needed
	for(int i = 0; i < insertions; i++){
		Lock* newLock = new Lock;

		newLock->dpoint.refValue() = lock->dpoint;
		newLock->point.refValue() = lock->point;
		newLock->weight.refValue() = lock->weight;
		newLock->uKnot.refValue() = lock->uKnot;
		newLock->vKnot.refValue() = lock->vKnot;
		newLock->dim.refValue() = lock->dim;
		newLock->uJoinEnds.refValue() = lock->uJoinEnds;
		newLock->border.refValue() = lock->border;
		int s = 0;
		int numOfCorners = 4;
		nextRegion(numOfCorners, s, lock->region.refValue(), polyCntr);

		if(s < lock->region.length && lock->region[s] < 0){
			numOfCorners = -lock->region[s++];
		}

		newLock->region.refValue() += -numOfCorners;

		for(int k = numOfCorners; --k>=0;){
			newLock->region.refValue() += lock->region[s++];
		}

		newLock->region.refValue() += -numOfCorners;
		s -= numOfCorners;

		for(int k = numOfCorners; --k>=0;){
			newLock->region.refValue() += lock->region[s++];
		}

		lock->lock.refValue() += newLock;
		lock = newLock;
	}
}

void selectRingsRecursion(Lock* lock, int insertions){
	lock->NurbsPatch::selectRings(0, lock->dim.x, lock->dim.y);
	if(--insertions){
		selectRingsRecursion(dynamic_cast<Lock*>(lock->lock.refValue()[0].p), insertions);
	}
}

void extrudeRegion(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Lock* src = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
			if(Hair* hair = getHair()){
				M_Int redirect;
				redirect.length = lock->region.length;
				for(int k = redirect.length; --k>=0;){
					redirect[k] = -1;
				}

				Lock* newLock = lock->extrudeRegion(redirect);
				duplicteLock(newLock, g_extrude->insertions - 1);
				g_stw->genNodeList();


				M_Bool flag;
				hair->flagStrandsInLock(lock, flag);
				for(int i = g_extrude->insertions; --i>=0;){
					hair->insertStrandControlPoints(lock, flag);
				}
				lock->copyMapRepete(src, flag);


				g_stw->paint();

		
				hair->processFields();
				resetCPNodeList();

				//newLock->NurbsPatch::selectRings(0, newLock->dim.x, newLock->dim.y);
				selectRingsRecursion(newLock, g_extrude->insertions);

				if(g_scene->mode & HAIR_REGION_MODE){

					
					Vec3f pivotPoint = calcRingPivotPointForHair(newLock);

					g_transformCP->scale.refValue() = Vec3f(1, 1, 1);
					g_transformCP->scaleOrientation.refValue() = Rotation();
					g_transformCP->pivotPoint.refValue() = pivotPoint;
					g_transformCP->rotation.refValue() = Rotation();
					g_transformCP->translation.refValue() = Vec3f();

					g_transformCP->houseKeep();

					toggleHairRegionMode();

					//g_stw->setResultsForSelectingPoint(newLock);
					//newLock->selectSelectedPoint(g_perspective);
					//g_extrude->distance.refValue() = 0;

					g_stw->cp->setupNodeWindow(g_extrude);
				
					Vec3f normal;

					calcNormal(
						newLock->generate(Vec2f(.5 - .01, .5)),
						newLock->generate(Vec2f(.5, .5)),
						newLock->generate(Vec2f(.5, .5 + .01)),
						normal
					);
					normal = -normal.normalize();

					g_extrude->normal = normal;
					Window::activeWindow = g_stw->cp.p;
				}

				g_extrude->processFields();
			}
		}
	}
}

void splitLock(){
	if(g_scene->mode & HAIR_REGION_MODE){
		if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
			if(Lock* src = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
				if(Hair* hair = getHair()){
					M_Int redirect;
					redirect.length = lock->region.length;
					for(int k = redirect.length; --k>=0;){
						redirect[k] = -1;
					}

					Lock* newLock = lock->splitLock(src, redirect);
					g_stw->genNodeList();


					M_Bool flag;
					hair->flagStrandsInLock(lock, flag);
					//hair->insertStrandControlPoints(lock, flag);
					lock->copyMapRepete(src, flag);


					g_stw->paint();

		
					hair->processFields();
					resetCPNodeList();

					newLock->NurbsPatch::selectRings(0, newLock->dim.x, newLock->dim.y);
					Vec3f pivotPoint = calcRingPivotPointForHair(newLock);

					g_transformCP->scale.refValue() = Vec3f(1, 1, 1);
					g_transformCP->scaleOrientation.refValue() = Rotation();
					g_transformCP->pivotPoint.refValue() = pivotPoint;
					g_transformCP->rotation.refValue() = Rotation();
					g_transformCP->translation.refValue() = Vec3f();

					IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_transformLeverClue->geometry.p);
					M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
					g_transformCP->rotCtrlPnt[g_transformCP->leverIdx] = g_scene->getViewMatrix() * g_transformCP->pivotPoint + Vec3f(.5, 0, 0);
					g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];

					g_transformCP->houseKeep();

					toggleHairRegionMode();

					g_stw->setResultsForSelectingPoint(newLock);
					//newLock->selectSelectedPoint(g_perspective);
					g_extrude->distance.refValue() = 0;

					g_stw->cp->setupNodeWindow(g_extrude);
				
					Vec3f normal;

					calcNormal(
						newLock->generate(Vec2f(.5 - .01, .5)),
						newLock->generate(Vec2f(.5, .5)),
						newLock->generate(Vec2f(.5, .5 + .01)),
						normal
					);
					normal = -normal.normalize();

					g_extrude->normal = normal;
					g_extrude->r_cap = lock;
					Window::activeWindow = g_stw->cp.p;
				}
			}
		}
	}
}

void deleteUnboundStrands(){
	if(Hair* hair = getHair()){
		M_Bool flag;
		flag.length = hair->strand.length;

		for(int i = flag.length; --i>=0;){
			flag[i] = true;
		}

		M_<Lock*>& lockList = hair->lockList;
		for(int k = lockList.length; --k>=0;){
			Lock* lock = lockList[k];
			M_<Strand*>& scpiLathe = lock->scpiLathe;
			for(int i = scpiLathe.length; --i>=0;){
				flag[scpiLathe[i]->slidx] = false;
			}
		}

		M_Strand& m_strand = hair->strand.refValue();

		int d = 0;

		for(int s = 0; s < m_strand.length; s++){
			if(flag[s] == false){
				m_strand[d] = m_strand[s];
				d++;
			}
		}
		m_strand.length = d;
		hair->processFields();

		g_stw->commonPostInsertNodeStuff();
		g_stw->paint();
		updateBuild3dWindow();
	}
}

bool pointIntersectsEdge(Vec2f vA, Vec2f vB, Vec2f point, float& ratio){
	Vec2f v = vB - vA;
	float angle = v.getAngle2();
	v = v.rotate(-angle);
	point = point - vA;
	point = point.rotate(-angle);
	if(point.y < .01 && point.y > -.01){
		ratio = point.x / v.x;
		return true;
	}
	return false;
}

bool Lock::splitLineIntersectsRegion(const int s, int numOfCorners, int brdrOffstt, const M_Vec3f& coord, Vec2f point, float& ratio, int& roffset0, int& roffset1){
	Vec2f vA = convertPointToScreen2d(coord[region[s] + brdrOffstt]);
	roffset0 = 0;
	for(int b = 1; b < numOfCorners + 1; b++){
		roffset1 = 0;
		if(b < numOfCorners){
			roffset1 += b;
		}
		Vec2f vB = convertPointToScreen2d(coord[region[s + roffset1] + brdrOffstt]);
		if(pointIntersectsEdge(vA, vB, point, ratio)){
			return true;
		}
		roffset0 = roffset1;
		vA = vB;
	}
	return false;
}

void copyRegion(M_Int& region, int s, int numOfCorners, int b, int bEnd){
	while(b != bEnd){
		region += region[s + b];
		b++;
		if(b == numOfCorners){
			b = 0;
		}
	}
}

bool Lock::splitLineIntersectsRegion(int s, Lock* src, int sSrc, int& numOfCorners, int brdrOffstt, int brdrOffsttSrc){
	if(s < region.length){
		if(region[s] < 0){
			numOfCorners = -region[s++];
		}
		if(region[sSrc] < 0){
			sSrc++;
		}
	}

	IndexedLineSet* polyline = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyline->coord.p)->point;

	Vec2f vA = convertPointToScreen2d(coord[region[s] + brdrOffstt]);

	float ratioA = 0;
	float ratioB = 0;
	int roffset0A = 0;
	int roffset1A = 0;
	int roffset0B = 0;
	int roffset1B = 0;

	if(splitLineIntersectsRegion(s, numOfCorners, brdrOffstt, coord, splitRegionPoints[0], ratioA, roffset0A, roffset1A)){
		if(splitLineIntersectsRegion(s, numOfCorners, brdrOffstt, coord, splitRegionPoints[1], ratioB, roffset0B, roffset1B)){

			Vec2f v0A = src->border[region[sSrc + roffset0A]] * (1.0 - ratioA);
			Vec2f v1A = src->border[region[sSrc + roffset1A]] * ratioA;

			Vec2f v0B = src->border[region[sSrc + roffset0B]] * (1.0 - ratioB);
			Vec2f v1B = src->border[region[sSrc + roffset1B]] * ratioB;

			int newBorderIdxASrc = src->border.length;
			src->border.refValue() += v0A + v1A;
			int newBorderIdxBSrc = src->border.length;
			src->border.refValue() += v0B + v1B;
			


			///////////
			v0A = border[region[s + roffset0A]] * (1.0 - ratioA);
			v1A = border[region[s + roffset1A]] * ratioA;

			v0B = border[region[s + roffset0B]] * (1.0 - ratioB);
			v1B = border[region[s + roffset1B]] * ratioB;

			
			int newBorderIdxA = border.length;
			border.refValue() += v0A + v1A;
			int newBorderIdxB = border.length;
			border.refValue() += v0B + v1B;


			//////////////
			region += -numOfCorners;
			region += newBorderIdxASrc;

			copyRegion(region, sSrc, numOfCorners, roffset1A, roffset1B);

			region += newBorderIdxBSrc;


			//
			region += -numOfCorners;
			region += newBorderIdxA;

			copyRegion(region, s, numOfCorners, roffset1A, roffset1B);

			region += newBorderIdxB;


			///////////////
			region += -numOfCorners;
			region += newBorderIdxBSrc;

			copyRegion(region, sSrc, numOfCorners, roffset1B, roffset1A);

			region += newBorderIdxASrc;


			////
			region += -numOfCorners;
			region += newBorderIdxB;

			copyRegion(region, s, numOfCorners, roffset1B, roffset1A);

			region += newBorderIdxA;
			return true;
		}
	}
	return false;
}

void Lock::splitRegion(){
	if(Lock* src = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
		int numOfCorners = 4;
		int s = 0;
		int polyCntr = 0;
		for(int k = 0; ; k++){

			int sSrc = 0;
			int brdrOffsttSrc = borderOffset[polyCntr];
			nextRegion(numOfCorners, s, region, polyCntr);

			if(splitLineIntersectsRegion(s, src, sSrc, numOfCorners, borderOffset[polyCntr], brdrOffsttSrc)){
				int len = numOfCorners;
				if(region[s] < 0){
					len++;
				}
				region.refValue().deleteElement(s, len);

				len = numOfCorners;
				if(region[sSrc] < 0){
					len++;
				}
				region.refValue().deleteElement(sSrc, len);


				houseKeepVisualClue();
				return;
			}

			nextRegion(numOfCorners, s, region, polyCntr);

			if(s == region.length && k >= unify.length){
				break;
			}
		}
	}
}

void Lock::cutLock(){
	M_Vec2f& hole = this->hole.refValue();
	M_Int& scpi = this->scpi.refValue();
	M_<Strand*>& scpiLathe = this->scpiLathe;

	for(int i = hole.length; --i>=0;){
		Strand* strand = scpiLathe[i];
		int vDim = scpi[i];
		strand->index[0] = 0;
		strand->dim.refValue().y = vDim;
		strand->point.refValue().length = vDim;
		strand->dpoint.refValue().length = vDim;
		strand->weight.refValue().length = vDim;
		updateNurbsCurveKnotMid0(vDim, strand->vKnot.refValue());
	}
}

void Hair::convertToPatches(){
	M_Strand& m_strand = this->strand.refValue();
	Seamless* smls = part->smls;

	for(int k = 0; k < strand.length; k++){
		Strand* strand = m_strand[k];
		smls->build.refValue() += strand->geometry;
	}
	g_stw->commonPostInsertNodeStuff();
}
