/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/


#include <windows.h>
#include <direct.h>

#include <sys/stat.h>
#include <share.h>
#include <fcntl.h>
#include <io.h>

#include <algorithm>    // std::max
using namespace std;

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "seamless3d_gui_render.h"
#include "gui.h"
#include "scene_tree.h"
#include "robot_aid.h"

extern IndexedLineSet* g_nurbsIls;
extern Coordinate* g_nurbsIlsCoord;
extern Color* g_nurbsIlsColor;
extern ShapePtr g_nurbsIlsShape;

extern Ptr<CutComplete> g_cutComplete;

void set_m_nodeRef(NurbsPatch* patch, M_Node*& m_node);
void updateNurbsCurveKnotMid0(int dim, M_Float& knot);
void updateNurbsCurveKnotMid1(int dim, M_Float& knot);
int convertToLineFormat1(int idx, int idx2, int uDim, bool doU);

int evenEdge(int par, NurbsPatch* patch, int k){
	if(k == 0 && patch->mirror && patch->mirrorPatch == 0){
		int uDim = patch->dim.x;
		if(par == uDim / 2){
			return par;
		}
	}
	if(par & 1){
		par++;
	}
	return par;
}

int evenEdge(int par, const Insert& insert, bool crossedMiddle){
	if(par & 1){
		par++;
	}
	return par;
}

inline int evenEdge(int par){
	if(par & 1){
		par++;
	}
	return par;
}

Vec2i evenEdge(Vec2i otherUV){
	if(otherUV.x & 1){
		otherUV.x++;
	}
	if(otherUV.y & 1){
		otherUV.y++;
	}
	return otherUV;
}

int convertToActualFormat(Vec2i otherUV, int uDim){
	otherUV = evenEdge(otherUV);
	return otherUV.y * uDim + otherUV.x;
}

int convertToActualFormat(int uOrV, int cut, int uDim, bool doU){
	uOrV = evenEdge(uOrV);
	cut = evenEdge(cut);
	if(doU){
		return cut * uDim + uOrV;
	}
	else{
		return uOrV * uDim + cut;
	}
}

void addPatchToList(M_MirrorPatch& m_mirrorPatch, NurbsPatch* patch, bool leaveNorthOrEast){
	for(int k = m_mirrorPatch.length; --k>=0;){
		if(m_mirrorPatch[k].patch == patch){
			return;
		}
	}
	m_mirrorPatch += MirrorPatch(patch, leaveNorthOrEast);
}

void adjustInsert(M_Insert& m_insert, const NurbsPatch* patch, const int uOrVIdx, bool doU){
	int uDim = patch->dim.x;
	for(int k = 0; k < m_insert.length; k++){
		Insert& insert = m_insert[k];
		if(insert.p == patch){
			if(insert.doU == doU){
				if(insert.uOrVIdx > uOrVIdx){
					//if(m_insert[k].uOrVIdx > uOrVIdx || (uOrVIdx & 1) == 0){
					//if(patch->mirror && patch->mirrorPatch == 0 && uOrVIdx >= (uDim >> 1)){
					//if(patch->mirror && patch->mirrorPatch == 0){
					//	m_insert[k].uOrVIdx += 2;
					//}
					//else{
						insert.uOrVIdx += 2;
					//}
				}
			}
			else{
				if(insert.cutLine.x >= uOrVIdx){
					insert.cutLine.x += 2;
				}
				if(insert.cutLine.y >= uOrVIdx){
					insert.cutLine.y += 2;
				}
			}
		}
		if(insert.otherPatch == patch){
			if(doU){
				if(insert.otherUV.x >= uOrVIdx){
					insert.otherUV.x += 2;
				}
			}
			else{
				if(insert.otherUV.y >= uOrVIdx){
					insert.otherUV.y += 2;
				}
			}
		}
	}
}

void adjustForInsideOut(M_Insert& m_insert, const NurbsPatch* patch){
	Insert newInsert;
	bool addNewInsert = false;

	for(int k = 0; k < m_insert.length; k++){
		Insert& insert = m_insert[k];

		if(patch == insert.p){
			if(insert.doU){
				insert.uOrVIdx = patch->getMirrorTwinlessIdx(insert.uOrVIdx);
				insert.uOrVIdx--;
				insert.ratio = 1.0 - insert.ratio;
			}
			else{
				insert.cutLine.x = patch->getMirrorTwinlessIdx(insert.cutLine.x);
				insert.cutLine.x--;

				insert.cutLine.y = patch->getMirrorTwinlessIdx(insert.cutLine.y);
				insert.cutLine.y--;
				insert.leaveNorthOrEast = !insert.leaveNorthOrEast;
			}
		}
		if(patch == insert.otherPatch){
			insert.otherUV.x = patch->getMirrorTwinlessIdx(insert.otherUV.x);
			insert.otherUV.x--;
		}
	}
}


void uAdjustForCut(M_Insert& m_insert, const NurbsPatch* patch, const int oldUDim, NurbsPatch* swapPatch, int u, const int uOrVIdx){
	Insert newInsert;
	bool addNewInsert = false;

	for(int k = 0; k < m_insert.length; k++){
		Insert& insert = m_insert[k];

		int _uOrVIdx = insert.uOrVIdx;

		if(patch == insert.p){
			if(insert.doU){
				if(_uOrVIdx < uOrVIdx || (_uOrVIdx == uOrVIdx && uOrVIdx & 1)){
					insert.p = swapPatch;
				}
				else{
					insert.uOrVIdx -= u;					
				}
			}
			else{
				if(insert.leaveNorthOrEast){
					if(insert.cutLine.x < u){
						if(insert.cutLine.y <= u){
						}
						else{
							addNewInsert = true;
							newInsert = insert;
							newInsert.diversion = false;
							newInsert.cutLine.x = 0;
							newInsert.cutLine.y -= u;
							insert.cutLine.y = u;
						}
						insert.p = swapPatch;
					}
					else{
						insert.cutLine.x -= u;
						insert.cutLine.y -= u;
					}
				}
				else{
					if(evenEdge(insert.cutLine.y) < u && insert.cutLine.x > u){
						addNewInsert = true;
						newInsert = insert;
						newInsert.diversion = false;
						newInsert.cutLine.x = u;
						insert.cutLine.x -= u;
						insert.cutLine.y = 0;

						newInsert.p = swapPatch;
					}
					else if(insert.cutLine.x <= u){
						insert.p = swapPatch;
					}
					else{
						insert.cutLine.x -= u;
						insert.cutLine.y -= u;
					}
				}
			}
		}
		if(patch == insert.otherPatch){
			if(insert.otherUV.x < u){
				insert.otherPatch = swapPatch;
			}
			else{
				insert.otherUV.x -= u;
			}
		}
		if(addNewInsert){//we have to insert the new element at the end so as not to mess up the reference to the m_insert element
			k++;
			m_insert.insertElement(k, newInsert);
			addNewInsert = false;
		}
	}
}


void vAdjustForCut(M_Insert& m_insert, const NurbsPatch* patch, const int oldUDim, NurbsPatch* swapPatch, int v, const int uOrVIdx){
	Insert newInsert;
	bool addNewInsert = false;

	for(int k = 0; k < m_insert.length; k++){
		Insert& insert = m_insert[k];

		int _uOrVIdx = insert.uOrVIdx;

		if(patch == insert.p){
			if(insert.doU == false){
				if(_uOrVIdx < uOrVIdx || (_uOrVIdx == uOrVIdx && uOrVIdx & 1)){
					insert.p = swapPatch;
				}
				else{
					insert.uOrVIdx -= v;
				}
			}
			else{
				//if(insert.leaveNorthOrEast){
				//	if(insert.cutLine.y > v){
				//		insert.cutLine.x -= v;
				//		insert.cutLine.y -= v;
				//	}
				//	else{
				//		insert.p = swapPatch;
				//	}
				//}
				//else{
				//	if(insert.cutLine.y < v){
				//		insert.p = swapPatch;
				//	}
				//	else{
				//		insert.cutLine.x -= v;
				//		insert.cutLine.y -= v;
				//	}
				//}
				
				if(insert.leaveNorthOrEast){
					if(insert.cutLine.x < v){
						if(insert.cutLine.y <= v){
						}
						else{
							addNewInsert = true;
							newInsert = insert;
							newInsert.diversion = false;
							newInsert.cutLine.x = 0;
							newInsert.cutLine.y -= v;
							insert.cutLine.y = v;
						}
						insert.p = swapPatch;
					}
					else{
						insert.cutLine.x -= v;
						insert.cutLine.y -= v;
					}
				}
				else{
					if(evenEdge(insert.cutLine.y) < v && insert.cutLine.x > v){
						addNewInsert = true;
						newInsert = insert;
						newInsert.diversion = false;
						newInsert.cutLine.x = v;
						insert.cutLine.x -= v;
						insert.cutLine.y = 0;

						newInsert.p = swapPatch;
					}
					else if(insert.cutLine.x <= v){
						insert.p = swapPatch;
					}
					else{
						insert.cutLine.x -= v;
						insert.cutLine.y -= v;
					}
				}
			}
		}
		if(patch == insert.otherPatch){
			if(insert.otherUV.y < v){
				insert.otherPatch = swapPatch;
			}
			else{
				insert.otherUV.y -= v;
			}
		}
		if(addNewInsert){//we have to insert the new element at the end so as not to mess up the reference to the m_insert element
			k++;
			m_insert.insertElement(k, newInsert);
			addNewInsert = false;
		}
	}
}


void cutUpTwinMirorPatch(NurbsPatch* patch, NurbsPatch* r_newPatch, int uOrV, bool doU, M_MirrorPatch& m_mirrorPatch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	NurbsPatch* l_newPatch = 0;
	if(doU){
		l_newPatch = patch->uSplitPatch(uOrV);
	}
	else{
		l_newPatch = patch->vSplitPatch(uOrV);

	}
	addPatchToList(m_mirrorPatch, patch, true);
	addPatchToList(m_mirrorPatch, l_newPatch, true);
	r_newPatch->name = "r_side";
	r_newPatch->makeExistingNameUniqueForMirror(l_newPatch);

	g_stw->lesserPostInsertNodeStuff();
	//patch->smls->populateMirrorNodes();

	return;
}

void uCutUpTwinlessMirrorPatchCenterLine(NurbsPatch* patch, int u, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch, int oldUDim, int uOrVIdx, bool leaveNorthOrEast){
	int mirror_uOrVIdx = patch->getMirrorTwinlessIdx(uOrVIdx);
	int mirrorU = patch->getMirrorTwinlessIdx(u);

	bool swapped = false;

	if(mirror_uOrVIdx < uOrVIdx){
		swap(mirror_uOrVIdx, uOrVIdx);
		swap(mirrorU, u);
		swapped = true;
	}
	NurbsPatch* r_patch = patch->uSplitPatch(u);
	g_stw->lesserPostInsertNodeStuff();
	uAdjustForCut(m_insert, patch, oldUDim, r_patch, u, uOrVIdx);
	r_patch->name = "r_side";
	r_patch->makeExistingNameUniqueForMirror(patch);
	patch->setInsideOut();
	patch->setInsideOut2();
	adjustForInsideOut(m_insert, patch);

	addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
	addPatchToList(m_mirrorPatch, r_patch, leaveNorthOrEast);

	g_stw->lesserPostInsertNodeStuff();
}

void uCutUpTwinlessMirrorPatch(NurbsPatch* patch, int u, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch, int oldUDim, int uOrVIdx, bool leaveNorthOrEast){
//	Insert& insert = m_insert[0];

	int mirror_uOrVIdx = patch->getMirrorTwinlessIdx(uOrVIdx);
	int mirrorU = patch->getMirrorTwinlessIdx(u);

	bool swapped = false;

	if(mirror_uOrVIdx < uOrVIdx){
		swap(mirror_uOrVIdx, uOrVIdx);
		swap(mirrorU, u);
		swapped = true;
	}
	NurbsPatch* midPatch = patch->uSplitPatch(mirrorU);
	g_stw->lesserPostInsertNodeStuff();
	uAdjustForCut(m_insert, patch, oldUDim, midPatch, mirrorU, mirror_uOrVIdx);

	oldUDim = midPatch->dim.x;
	NurbsPatch* r_patch = midPatch->uSplitPatch(u);
	g_stw->lesserPostInsertNodeStuff();
	uAdjustForCut(m_insert, midPatch, oldUDim, r_patch, u, uOrVIdx);
	r_patch->name = "r_side";
	r_patch->makeExistingNameUniqueForMirror(patch);
	patch->setInsideOut();
	patch->setInsideOut2();
	adjustForInsideOut(m_insert, patch);

	addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
	addPatchToList(m_mirrorPatch, midPatch, leaveNorthOrEast);
	addPatchToList(m_mirrorPatch, r_patch, leaveNorthOrEast);

	g_stw->lesserPostInsertNodeStuff();
	patch->smls->generateRawStitchPolys();
	r_patch->mirrorTwinPatch(patch);
}

void vCutUpTwinlessMirrorPatch(NurbsPatch* patch, int v, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch, const int oldUDim, const int uOrVIdx, bool leaveNorthOrEast){
//	int mirrorIdx = patch->getMirrorTwinlessIdx(idx);
	//NurbsPatch* midPatch = patch->uSplitPatch(idx);
	//g_stw->lesserPostInsertNodeStuff();
	NurbsPatch* r_patch = patch->vSplitPatch(v);
	g_stw->lesserPostInsertNodeStuff();
	vAdjustForCut(m_insert, patch, oldUDim, r_patch, v, uOrVIdx);
	//r_patch->name = "r_side";
	//patch->name = "l_side";
	//makeExistingNameUnique(r_patch);
	//makeExistingNameUnique(patch);
	//patch->setInsideOut();

	addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
	//addPatchToList(m_nurbsPatch, midPatch);
	addPatchToList(m_mirrorPatch, r_patch, leaveNorthOrEast);

	patch->smls->generateRawStitchPolys();
	//r_patch->mirrorTwinPatch(patch);
}

//void reEnableLines(M_Insert& m_insert){
//	for(int k = 0; k < m_insert.length; k++){
//		int cutLineA = m_insert[k].cutLineA;
//		if(cutLineA < 0){
//			m_insert[k].cutLineA = cutLineA
//		}
//	}
//}

void insertColumnNoInvert(Vec3f pointA, Vec3f pointB, Vec3f pointC, M_Vec3f& dpoint, int& uDim){
	dpoint.insertElement(uDim * 2, pointC);
	dpoint.insertElement(uDim * 1, pointB);
	dpoint.insertElement(0, pointA);
	uDim++;
}


void insertColumn(Vec3f pointA, Vec3f pointB, Vec3f pointC, M_Vec3f& dpoint, int& uDim){
	pointA.x = -pointA.x;
	pointB.x = -pointB.x;
	pointC.x = -pointC.x;

	dpoint.insertElement(uDim * 2, pointC);
	dpoint.insertElement(uDim * 1, pointB);
	dpoint.insertElement(0, pointA);
	uDim++;
}

void deleteLastColumn(M_Vec3f& dpoint, int& uDim){
	dpoint.deleteElement(uDim * 3 - 1);
	dpoint.deleteElement(uDim * 2 - 1);
	dpoint.deleteElement(uDim * 1 - 1);
	uDim--;
}


void copyLastCell(const M_Vec3f& newPoint, const int& newUdim, M_Vec3f& newPoint2, int& newUdim2){
	insertColumn(newPoint[newUdim * 1 - 2], newPoint[newUdim * 2 - 2], newPoint[newUdim * 3 - 2], newPoint2, newUdim2);
	insertColumnNoInvert(newPoint[newUdim * 1 - 1], newPoint[newUdim * 2 - 1], newPoint[newUdim * 3 - 1], newPoint2, newUdim2);
	insertColumnNoInvert(newPoint[newUdim * 1 - 2], newPoint[newUdim * 2 - 2], newPoint[newUdim * 3 - 2], newPoint2, newUdim2);
}


void adjustForCut(NurbsPatch* originalPatch, NurbsPatch* newPatch, int targetInsertIdx, M_Insert& m_insert){
	int uOrVIdx = m_insert[targetInsertIdx].uOrVIdx;
	for(int k = 0; k < m_insert.length; k++){
		if(targetInsertIdx == k){
			continue;
		}
		Insert& insert = m_insert[k];
	}
}
void singleMirrorBeginStrip(M_Vec3f& newPoint, int& newUdim){
	int end = newUdim;
	for(int k = 1; k < end; k++){
		insertColumn(newPoint[k], newPoint[k + newUdim], newPoint[k + newUdim * 2], newPoint, newUdim);
		//insertColumn(Vec3f(), Vec3f(), Vec3f(), newPoint, newUdim);
		k++;
		end++;
	}
}

void addExtraPatchForLoopStrip(NurbsPatchPtr& newPatch2, M_MirrorPatch& m_mirrorPatch, M_Node& r_m_node, Seamless* smls, M_Vec3f& newPoint, const int& newUdim){
	newPatch2 = new NurbsPatch;
	addPatchToList(m_mirrorPatch, newPatch2, true);
	r_m_node += newPatch2.p;
	newPatch2->sync.refValue() = true;
	newPatch2->mirror.refValue() = true;
	//r_shaft->showAll.refValue() = true;

	newPatch2->smls = smls;
	newPatch2->uJoinEnds.refValue() = false;
	int newUdim2 = 0;
	int newVdim2 = 3;

	M_Vec3f& newPoint2 = newPatch2->point.refValue();
	M_Float& newWeight2 = newPatch2->weight.refValue();
	M_Float& newUknot2 = newPatch2->uKnot.refValue();
	M_Float& newVknot2 = newPatch2->vKnot.refValue();

	copyLastCell(newPoint, newUdim, newPoint2, newUdim2);


	newPatch2->dim.refValue() = Vec2f(newUdim2, newVdim2);

	int newPointLen2 = newUdim2 * newVdim2;
	newWeight2.length = newPointLen2;
	for(int i = newPointLen2; --i >= 0;){
		newWeight2[i] = 1;
	}

	updateNurbsCurveKnotMid1(newUdim2, newUknot2);
	updateNurbsCurveKnotMid1(newVdim2, newVknot2);
}

void mirrorPatchForInsert(NurbsPatch* patch, int uOrVIdx, bool doU){
	if(patch->mirror){
		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			mirrorPatch->dpointsToPoints();
			mirrorPatch->processFields();
			patch->mirrorTwinPatch(mirrorPatch);
			//mirrorPatch->synchronizeCellsAndBuild();

		}
		else if(doU){
			int uDim = patch->dim.x;
			if(uOrVIdx > uDim / 2){
				patch->mirrorModifyWest();
			}
			else{
				patch->mirrorModifyEast();
			}
			patch->processFields();
		}
	}
}

void uSetMidPosition(M_Vec3f& dpoint, int u, int idx, int uDim){
	if(u & 1){
		dpoint[idx + 1] = (dpoint[idx] + dpoint[idx + 2]) * .5;
		//if(u < uDim - 4){
		//	dpoint[idx + 3] = (dpoint[idx + 2] + dpoint[idx + 4]) * .5;
		//}
	}
	else{
		dpoint[idx + 2] = (dpoint[idx + 1] + dpoint[idx + 3]) * .5;
		//if(u > 0){
		//	dpoint[idx] = (dpoint[idx - 1] + dpoint[idx + 1]) * .5;
		//}
	}
}

void vSetMidPosition(M_Vec3f& dpoint, int v, int idx, int uDim, int vDim){
	if(v & 1){
		dpoint[idx + uDim] = (dpoint[idx] + dpoint[idx + 2 * uDim]) * .5;
		//if(v < vDim - 4){
		//	dpoint[idx + 3 * uDim] = (dpoint[idx + 2 * uDim] + dpoint[idx + 4 * uDim]) * .5;
		//}
	}
	else{
		dpoint[idx + 2 * uDim] = (dpoint[idx + uDim] + dpoint[idx + 3 * uDim]) * .5;
		//if(v > 0){
		//	dpoint[idx] = (dpoint[idx - uDim] + dpoint[idx + uDim]) * .5;
		//}
	}
}

void splitPatchForInsertLine(NurbsPatch* patch){
	M_Float& uKnot = patch->uKnot;
	M_Float& vKnot = patch->vKnot;
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	updateNurbsCurveKnotMid1(uDim, uKnot);
	updateNurbsCurveKnotMid1(vDim, vKnot);
	patch->dpointsToPoints();
	patch->smls->generateRawStitchPolys();
	patch->processFields();
	patch->smls->populateMirrorNodes();
}

void uSetMidPosition(NurbsPatch* patch, int u, NurbsPatch*& newPatch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	M_Vec3f& dpoint = patch->dpoint.refValue();
	int idx = u;

	//if(u == uDim){
	//	for(int v = 0; v < vDim; v++){
	//		int offsetA = v * uDim;
	//		int offsetB = (v + 1) * uDim - 1;
	//		dpoint[offsetA] = dpoint[offsetB] = (dpoint[offsetA] + dpoint[offsetA + 1]) * .5;
	//	}
	//	splitPatchForInsertLine(patch);
	//	newPatch = patch->uSplitPatch(uDim - 3);
	//	return;
	//}
	if(u == -1){
		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			M_Vec3f& mdpoint = mirrorPatch->dpoint.refValue();
			int muDim = mirrorPatch->dim.x;
			for(int v = 0; v < vDim; v++){
				int offsetA = v * uDim;
				int offsetB = v * muDim;
				dpoint[offsetA] = mdpoint[offsetB] = (mdpoint[offsetB] + mdpoint[offsetB + 1]) * .5;
			}
		}
		else{
			for(int v = 0; v < vDim; v++){
				int offsetA = v * uDim;
				int offsetB = (v + 1) * uDim - 1;
				dpoint[offsetA] = dpoint[offsetB] = (dpoint[offsetB] + dpoint[offsetB - 1]) * .5;
			}
		}
		idx = 0;
		uSetMidPosition(dpoint, 0, idx, uDim);
		idx += uDim * (vDim - 1);
		uSetMidPosition(dpoint, 0, idx, uDim);

		splitPatchForInsertLine(patch);
		newPatch = patch->uSplitPatch(2);
		return;
	}

	uSetMidPosition(dpoint, u, idx, uDim);
	idx += uDim * (vDim - 1);
	uSetMidPosition(dpoint, u, idx, uDim);
}

void vSetMidPosition(NurbsPatch* patch, int v){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	M_Vec3f& dpoint = patch->dpoint.refValue();
	int idx = v * uDim;

	vSetMidPosition(dpoint, v, idx, uDim, vDim);
	idx += uDim - 1;
	vSetMidPosition(dpoint, v, idx, uDim, vDim);
}
/*
void bubbleSortInsertData(M_Insert& insert){
	bool flag;
	do{
		flag = false;
		for(int i = 0; i < insert.length - 1; i++){

			if(insert[i].uOrVIdx > insert[i + 1].uOrVIdx){
				Insert swap = insert[i + 1];
				insert[i+1] = insert[i];
				insert[i] = swap;

				flag = true;
			}
		}
	}while(flag);
}
*/

bool uIsCentre(const Insert& insert){
	const NurbsPatch* patch = insert.p;
	int uDim = patch->dim.x;
	int u = insert.uOrVIdx;
	if(patch->mirror){
		if(patch->mirrorPatch){
			return true;
		}
		else if(u == 0 || u == uDim - 2){
			return true;
		}

	}
	return false;
}

bool beginsFromCenterLineFunc(int k, const Insert& insert){
	NurbsPatch* patch = insert.p;
	int uDim = patch->dim.x;
	if(k == 0 && patch->mirror && patch->mirrorPatch == 0 && ((uDim / 2) & 1) && insert.doU == false && insert.cutLine.x == (uDim / 2)){
		return true;
	}
	return false;
}

bool endsOnCenterLineFunc(int k, int length, const Insert& insert){
	NurbsPatch* patch = insert.p;
	int uDim = patch->dim.x;
	if(k == length - 1 && patch->mirror && patch->mirrorPatch == 0 && ((uDim / 2) & 1) && insert.doU == false && insert.cutLine.y == (uDim / 2)){
		return true;
	}
	return false;
}


bool lastEndsAtMirrorCenter(const M_Insert& m_insert){
	int k = m_insert.length - 1;
	const Insert& insert = m_insert[k];
	NurbsPatch* patch = insert.p;
	int uDim = patch->dim.x;
	if(patch->mirror && patch->mirrorPatch == 0 && insert.cutLine.y == (uDim / 2)){
		if(k == 0 && ((uDim / 2) & 1) && insert.cutLine.x == (uDim / 2)){
			return false;
		}
		return true;
	}
	return false;
}


void addInsertVisualClue2(const M_Insert& m_insert, M_Int& iFScoordIndex, M_Vec3f& iFSpoint, M_Color3f& iFScolor){
	Color3f oldColor = Color3f(.8, .5, .0);
	Color3f newColor = Color3f(1, .5, .5);
	
	int begin = 0;
	int endPlus = m_insert.length;
	if(endPlus > 1 && g_insertPatch.cutMode && g_cutComplete->_continue == false){
		g_insertPatch.verifyFirst();
		begin = g_insertPatch.init_element;
		endPlus = begin + 1;
	}
	for(int k = endPlus; --k >= begin;){
		const Insert& insert = m_insert[k];
		if(insert.inLoop == false){
			continue;
		}
		Color3f color = oldColor;
		if(g_insertPatch.diversionHover && k >= g_insertPatch.beginInsertIdx){
			color = newColor;
		}
		NurbsPatch* patch = insert.p;
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		M_Vec3f& dpoint = patch->dpoint.refValue();

		M_Float& uKnot = patch->uKnot;
		M_Float& vKnot = patch->vKnot;
		const int uMid = checkNurbsType(uDim, uKnot);
		const int vMid = checkNurbsType(vDim, vKnot);

		float ratio = insert.ratio;

		int uOrVIdx = insert.uOrVIdx;
		if(insert.doU){
			if(NurbsPatch::MID_1 == uMid){
				int v = 0;
				int lastV = vDim - 1;
				if(insert.leaveNorthOrEast){
					if(insert.cutLine.x > 0){
						v = evenEdge(insert.cutLine.x, patch, k);
					}
					if(insert.cutLine.y > 0){
						lastV = evenEdge(insert.cutLine.y);
					}
				}
				else{
					if(insert.cutLine.x > 0){
						lastV = evenEdge(insert.cutLine.x);
					}
					if(insert.cutLine.y > 0){
						v = evenEdge(insert.cutLine.y);
					}
				}

				for(; v != lastV; v++){
					iFScoordIndex += iFSpoint.length;
					Vec3f vA = dpoint[v * uDim + uOrVIdx];
					Vec3f vB = dpoint[v * uDim + uOrVIdx + 1];

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += color;

					iFScoordIndex += iFSpoint.length;
					int uIdx = uOrVIdx + uDim;
					vA = dpoint[v * uDim + uIdx];
					vB = dpoint[v * uDim + uIdx + 1];

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += color;

					iFScoordIndex += -1;
				}
			}
		}
		else{
			if(NurbsPatch::MID_1 == vMid && uDim > 1){
				int u = insert.cutLine.x;
				int lastU = insert.cutLine.y;
				if(false == beginsFromCenterLineFunc(k, insert)){
					u = evenEdge(u);
				}
				if(false == endsOnCenterLineFunc(k, endPlus, insert)){
					lastU = evenEdge(lastU);
				}

				if(u > lastU){
					swap(u, lastU);
				}

					
				for(; u != lastU; u++){
					iFScoordIndex += iFSpoint.length;
					Vec3f vA = dpoint[uOrVIdx * uDim + u];
					Vec3f vB = dpoint[(uOrVIdx + 1) * uDim + u];

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += color;

					iFScoordIndex += iFSpoint.length;
					vA = dpoint[uOrVIdx * uDim + u + 1];
					vB = dpoint[(uOrVIdx + 1) * uDim + u + 1];

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += color;

					iFScoordIndex += -1;
				}
			}
		}
	}
}

void addInsertVisualClueMirror(const M_Insert& m_insert, M_Int& iFScoordIndex, M_Vec3f& iFSpoint, M_Color3f& iFScolor){//exactly the same function above except we convert the index and patch to mirror values
	int begin = 0;
	int endPlus = m_insert.length;
	if(endPlus > 1 && g_insertPatch.cutMode && g_cutComplete->_continue == false){
		g_insertPatch.verifyFirst();
		begin = g_insertPatch.init_element;
		endPlus = begin + 1;
	}
	for(int k = endPlus; --k >= begin;){
		const Insert& insert = m_insert[k];
		
		if(insert.inLoop == false){
			continue;
		}

		NurbsPatch* patch = insert.p;
		if(patch->mirror == false){
			continue;
		}
		if(patch->mirrorPatch){
			patch = patch->mirrorPatch;
		}
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		M_Vec3f& dpoint = patch->dpoint.refValue();

		M_Float& uKnot = patch->uKnot;
		M_Float& vKnot = patch->vKnot;
		const int uMid = checkNurbsType(uDim, uKnot);
		const int vMid = checkNurbsType(vDim, vKnot);

		float ratio = insert.ratio;

		int uOrVIdx = insert.uOrVIdx;
		if(insert.doU){
			if(patch->mirrorPatch == 0){
				uOrVIdx = patch->getMirrorTwinlessIdx(uOrVIdx);
			}
			if(NurbsPatch::MID_1 == uMid){
				int v = 0;
				int lastV = vDim - 1;
				if(insert.leaveNorthOrEast){
					if(insert.cutLine.x > 0){
						v = evenEdge(insert.cutLine.x, patch, k);
					}
					if(insert.cutLine.y > 0){
						lastV = evenEdge(insert.cutLine.y);
					}
				}
				else{
					if(insert.cutLine.x > 0){
						lastV = evenEdge(insert.cutLine.x);
					}
					if(insert.cutLine.y > 0){
						v = evenEdge(insert.cutLine.y);
					}
				}

				for(; v != lastV; v++){
					iFScoordIndex += iFSpoint.length;

					Vec3f vA = dpoint[v * uDim + uOrVIdx];
					Vec3f vB;
					if(patch->mirrorPatch == 0){
						vB = dpoint[v * uDim + uOrVIdx - 1];
					}
					else{
						vB = dpoint[v * uDim + uOrVIdx + 1];
					}

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += Color3f(1, .5, .5);

					iFScoordIndex += iFSpoint.length;
					int uIdx = uOrVIdx + uDim;
					vA = dpoint[v * uDim + uIdx];
					if(patch->mirrorPatch == 0){
						vB = dpoint[v * uDim + uIdx - 1];
					}
					else{
						vB = dpoint[v * uDim + uIdx + 1];
					}

					

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += Color3f(1, .5, .5);

					iFScoordIndex += -1;
				}
			}
		}
		else{
			if(NurbsPatch::MID_1 == vMid && uDim > 1){
				int u = insert.cutLine.x;
				int lastU = insert.cutLine.y;
				if(false == beginsFromCenterLineFunc(k, insert)){
					u = evenEdge(u);
				}
				if(false == endsOnCenterLineFunc(k, endPlus, insert)){
					lastU = evenEdge(lastU);
				}

				if(patch->mirrorPatch == 0){
					u = patch->getMirrorTwinlessIdx(u);
					lastU = patch->getMirrorTwinlessIdx(lastU);
				}

				if(u > lastU){
					swap(u, lastU);
				}
					
				for(; u != lastU; u++){
					iFScoordIndex += iFSpoint.length;
					Vec3f vA = dpoint[uOrVIdx * uDim + u];
					Vec3f vB = dpoint[(uOrVIdx + 1) * uDim + u];

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += Color3f(1, .5, .5);

					iFScoordIndex += iFSpoint.length;
					vA = dpoint[uOrVIdx * uDim + u + 1];
					vB = dpoint[(uOrVIdx + 1) * uDim + u + 1];

					iFSpoint += vA + (vB - vA) * ratio;
					iFScolor += Color3f(1, .5, .5);

					iFScoordIndex += -1;
				}
			}
		}
	}
}

void InsertPatch::showDirection(M_Int& iFScoordIndex, M_Vec3f& iFSpoint, M_Color3f& iFScolor)const{
	int uDim = init_patch->dim.x;
	int vDim = init_patch->dim.y;
	int u = init_idx % uDim;
	int v = init_idx / uDim;

	Vec3f arrorA;
	Vec3f arrorB;
	Vec3f arrorC;

	Vec3f basePoint = init_patch->dpoint[init_idx];

	Vec3f basePointB;
	Vec3f directionPoint;
	Vec3f directionPointB;
			
	if(g_insertPatch.init_doU){
		if(g_insertPatch.init_leaveNorthOrEast){
			directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + uDim];
			directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + uDim + 1];
		}
		else{
			directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - uDim];
			directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - uDim + 1];
		}
	}
	else{
		if(g_insertPatch.init_leaveNorthOrEast){
			directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + 1];
			directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + 1 + uDim];
		}
		else{
			directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - 1];
			directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - 1 + uDim];
		}
	}



	if(init_doU){
		basePointB = init_patch->dpoint[init_idx + 1];
	}
	else{
		basePointB = init_patch->dpoint[init_idx + uDim];
	}
			
	if((init_doU == true && u & 1) || (init_doU == false && v & 1)){
		basePoint = basePoint * init_ratio + basePointB * (1.0 - init_ratio);
		directionPoint = directionPoint * init_ratio + directionPointB * (1.0 - init_ratio);
	}
	else{
		basePoint = basePoint * (1.0 - init_ratio) + basePointB * init_ratio;
		directionPoint = directionPoint * (1.0 - init_ratio) + directionPointB * init_ratio;
	}
//////

	arrorB = basePoint;
	arrorC = directionPoint;


	g_viewMatrix = g_scene->getViewMatrix();
	Matrix iViewMatrix = g_scene->getIViewMatrix();

	basePoint = g_viewMatrix * basePoint;
	directionPoint = g_viewMatrix * directionPoint;

	Vec2f diff = Vec2f(directionPoint.x - basePoint.x, directionPoint.y - basePoint.y);


	if(abs(diff.x) > .0001 || abs(diff.y) > .0001){
		float angle = diff.getAngle2();
		Vec2f headBaseA = Vec2f(.05, 0).rotate(angle + PI * .75);
		Vec2f headBaseB = Vec2f(.05, 0).rotate(angle);
		Vec2f headBaseC = Vec2f(.05, 0).rotate(angle - PI * .75);


		Vec3f headBaseA3f = Vec3f(headBaseA.x, headBaseA.y, 0) + basePoint;
		Vec3f headBaseB3f = Vec3f(headBaseB.x, headBaseB.y, 0) + basePoint;
		Vec3f headBaseC3f = Vec3f(headBaseC.x, headBaseC.y, 0) + basePoint;

		headBaseA3f = iViewMatrix * headBaseA3f;
		headBaseB3f = iViewMatrix * headBaseB3f;
		headBaseC3f = iViewMatrix * headBaseC3f;

		float senseLen = g_mouseSense * 5;

		iFScoordIndex += iFSpoint.length;
		iFSpoint += headBaseB3f;
		iFScolor += Color3f(1, 0, 0);
		
		iFScoordIndex += iFSpoint.length;
		iFSpoint += headBaseA3f;
		iFScolor += Color3f(1, 0, 0);
		iFScoordIndex += -1;

		iFScoordIndex += iFSpoint.length;
		iFSpoint += headBaseB3f;
		iFScolor += Color3f(1, 0, 0);

		iFScoordIndex += iFSpoint.length;
		iFSpoint += headBaseC3f;
		iFScolor += Color3f(1, 0, 0);
	}
}

void InsertPatch::addInsertVisualClue(){

	M_Int& iFScoordIndex = g_nurbsIls->coordIndex.refValue();
	M_Vec3f& iFSpoint = g_nurbsIlsCoord->point.refValue();
	M_Color3f& iFScolor = g_nurbsIlsColor->color.refValue();

	iFScoordIndex.length = beginCoordIndexVisualClue;
	iFSpoint.length = beginCoordVisualClue;
	iFScolor.length = beginCoordVisualClue;

	addInsertVisualClue2(m_insert, iFScoordIndex, iFSpoint, iFScolor);
	if(false == (g_insertPatch.cutMode && g_cutComplete->_continue == false)){
		addInsertVisualClue2(m_insertOD, iFScoordIndex, iFSpoint, iFScolor);
	}

	addInsertVisualClueMirror(m_insert, iFScoordIndex, iFSpoint, iFScolor);
	if(false == (g_insertPatch.cutMode && g_cutComplete->_continue == false)){
		addInsertVisualClueMirror(m_insertOD, iFScoordIndex, iFSpoint, iFScolor);
	}

	if((imousePhase == DIVERT_STANDBY || imousePhase == MONITOR_DIRECTION) && m_insert.length){
		showDirection(iFScoordIndex, iFSpoint, iFScolor);
	}
	

	g_nurbsIls->convertToRenderFormat(g_nurbsIlsShape->render);
}

void insertLine(Insert& insert, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch){

	NurbsPatch* patch = insert.p;
	Seamless* smls = patch->smls;

	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	M_Float& uKnot = patch->uKnot;
	M_Float& vKnot = patch->vKnot;
	M_Vec3f oldPoints;
	M_Float oldWeights;
	M_Int oldCpIndex;
	NurbsPatch* newPatch = 0;

	int uOrVIdx = insert.uOrVIdx;
	bool doU = insert.doU;
	adjustInsert(m_insert, patch, uOrVIdx, doU);

	bool joinEnds = false;
	bool joinBeginning = false;

	if(doU){
		if(g_insertPatch.centerMirrorLine){
			if(uOrVIdx == (uDim >> 1) - 1 || uOrVIdx == (uDim >> 1)){
				patch->insertCol(oldPoints, oldWeights, oldCpIndex, (uDim >> 1) - 1, .5, uDim, g_insertPatch.centerMirrorLine);
				patch->insertCol(oldPoints, oldWeights, oldCpIndex, (uDim >> 1), .5, uDim, g_insertPatch.centerMirrorLine);
			}
			else{
				patch->insertCol(oldPoints, oldWeights, oldCpIndex, 0, 0, uDim, g_insertPatch.centerMirrorLine);
				patch->insertCol(oldPoints, oldWeights, oldCpIndex, 0, 0, uDim, g_insertPatch.centerMirrorLine);

				uSetMidPosition(patch, -1, newPatch);
			}
		}
		else{
			float ratio = insert.ratio;
			if(patch->mirror && g_insertPatch.centerMirrorLine == false && patch->mirrorPatch == 0){ //make sure we have west side
				if(uOrVIdx >= uDim >> 1){
					uOrVIdx = patch->getMirrorTwinlessIdx(uOrVIdx) - 1;
					ratio = 1.0 - ratio;
					adjustInsert(m_insert, patch, uOrVIdx, doU);
				}
			}
			patch->insertCol(oldPoints, oldWeights, oldCpIndex, uOrVIdx, ratio, uDim, g_insertPatch.centerMirrorLine);
			patch->insertCol(oldPoints, oldWeights, oldCpIndex, uOrVIdx + 1, 0, uDim, g_insertPatch.centerMirrorLine);
			uSetMidPosition(patch, uOrVIdx, newPatch);
			if(patch->mirrorPatch){
				uSetMidPosition(patch->mirrorPatch, uOrVIdx, newPatch);
			}
		}
		patch->uUpdateKnotMid1();
		if(patch->mirrorPatch){
			patch->mirrorPatch->uUpdateKnotMid1();
		}

		uOrVIdx = insert.uOrVIdx;

	}
	else{
		int beginIdx = convertToActualFormat(uOrVIdx, insert.cutLine.x, uDim, doU);
		int endIdx = convertToActualFormat(uOrVIdx, insert.cutLine.y, uDim, doU);

		if(patch->mirrorPatch){
			M_Int& r_cpIndex = patch->cpIndex.refValue();
			M_Int& l_cpIndex = patch->mirrorPatch->cpIndex.refValue();
			if(r_cpIndex[beginIdx] == l_cpIndex[beginIdx]){
				joinBeginning = true;
			}
			if(r_cpIndex[endIdx] == l_cpIndex[endIdx]){
				joinEnds = true;
			}
		}

		patch->insertRow(oldPoints, oldWeights, oldCpIndex, uOrVIdx, insert.ratio, vDim);
		patch->insertRow(oldPoints, oldWeights, oldCpIndex, uOrVIdx + 1, 0, vDim);

		if(joinEnds){
			M_Int& r_cpIndex = patch->cpIndex.refValue();
			M_Int& l_cpIndex = patch->mirrorPatch->cpIndex.refValue();

			r_cpIndex[endIdx + uDim * 0] = l_cpIndex[endIdx + uDim * 0];
			r_cpIndex[endIdx + uDim * 1] = l_cpIndex[endIdx + uDim * 1];
			r_cpIndex[endIdx + uDim * 2] = l_cpIndex[endIdx + uDim * 2];
		}
		if(joinBeginning){
			M_Int& r_cpIndex = patch->cpIndex.refValue();
			M_Int& l_cpIndex = patch->mirrorPatch->cpIndex.refValue();

			r_cpIndex[beginIdx + uDim * 0] = l_cpIndex[beginIdx + uDim * 0];
			r_cpIndex[beginIdx + uDim * 1] = l_cpIndex[beginIdx + uDim * 1];
			r_cpIndex[beginIdx + uDim * 2] = l_cpIndex[beginIdx + uDim * 2];
		}


		vSetMidPosition(patch, uOrVIdx);
		patch->vUpdateKnotMid1();

		uOrVIdx = insert.uOrVIdx;
	}
	patch->dpointsToPoints();
	if(patch->mirrorPatch){
		patch->mirrorPatch->dpointsToPoints();
	}
	if(patch->isCtrlPolyRef){
		smls->generateRawStitchPolys();
	}

	patch->processFields();
	//patch->synchronizeCellsAndBuild();

	if(insert.doU){
		mirrorPatchForInsert(patch, 0, insert.doU);
	}
	else{
		mirrorPatchForInsert(patch, uOrVIdx, insert.doU);
	}
	patch->pointsToCpws();
	if(patch->mirrorPatch){
		patch->mirrorPatch->pointsToCpws();
	}
	if(newPatch){
		//newPatch->synchronizeAllStemCells();
		newPatch->dpointsToPoints();
		if(newPatch->isCtrlPolyRef){
			smls->generateRawStitchPolys();
		}
		//newPatch->synchronizeAllStemCells();
		newPatch->processFields();
		newPatch->synchronizeCellsAndBuild();
		//patch->synchronizeAllStemCells();
		//bnspeBuildPolies(newPatch);
	}


}

//bool isFirstInLoop(const int k, M_Insert& m_insert){
//	if(g_insertPatch.loop && m_insert[k].inLoop){
//		if(k == 0){
//			return true;
//		}
//		if(m_insert[k - 1].inLoop == false){
//			return true;
//		}
//	}
//	return false;
//}

bool isLastInLoop(const int k, M_Insert& m_insert){
	if(g_insertPatch.loop){
		if(m_insert[k].inLoop){
			if(k == m_insert.length - 1){
				return true;
			}
			if(k + 1 < m_insert.length && m_insert[k + 1].inLoop == false){
				return true;
			}
		}
	}
	return false;
}
/*
bool uCutPatch(int u, M_Insert& m_insert, const int k, M_MirrorPatch& m_mirrorPatch, bool& cutWasMade){

	Insert& insert = m_insert[k];
	NurbsPatch* patch = insert.p;
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	const int uOrVIdx = insert.uOrVIdx;
	const int oldUDim = patch->dim.x;
	bool leaveNorthOrEast = insert.leaveNorthOrEast;

	if(u != 0 && u != uDim -1){
		cutWasMade = true;
		NurbsPatch* swapPatch = patch->uSplitPatch(u);
		g_stw->lesserPostInsertNodeStuff();
		addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
		addPatchToList(m_mirrorPatch, swapPatch, leaveNorthOrEast);
		uAdjustForCut(m_insert, patch, oldUDim, swapPatch, u, uOrVIdx);
	}
	return cutWasMade;
}

bool vCutPatch(int v, M_Insert& m_insert, const int k, M_MirrorPatch& m_mirrorPatch, bool& cutWasMade){

	Insert& insert = m_insert[k];
	NurbsPatch* patch = insert.p;
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	const int uOrVIdx = insert.uOrVIdx;
	const int oldUDim = patch->dim.x;
	bool leaveNorthOrEast = insert.leaveNorthOrEast;

	if(v != 0 && v != vDim -1){
		cutWasMade = true;
		NurbsPatch* swapPatch = patch->vSplitPatch(v);
		addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
		addPatchToList(m_mirrorPatch, swapPatch, leaveNorthOrEast);
		vAdjustForCut(m_insert, patch, oldUDim, swapPatch, v, uOrVIdx);
	}
	return cutWasMade;
}
*/
void cutPatches(Seamless* smls, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch){
	g_visualClue = 0;
	bool cutWasMade = false;

	for(int k = 0; k < m_insert.length; k++){
		Insert& insert = m_insert[k];
		NurbsPatch* patch = insert.p;
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		const int oldUDim = uDim;

		const int uOrVIdx = insert.uOrVIdx;
		bool leaveNorthOrEast = insert.leaveNorthOrEast;

		if(insert.doU){
			int u = uOrVIdx;
			if(uOrVIdx & 1){
				u++;
			}
			if(patch->mirror){
				if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
					if(u != 0 && u != uDim -1){
						cutWasMade = true;
						NurbsPatch* r_swapPatch = patch->uSplitPatch(u);
						
						cutUpTwinMirorPatch(mirrorPatch, r_swapPatch, u, insert.doU, m_mirrorPatch);
						uAdjustForCut(m_insert, patch, oldUDim, r_swapPatch, u, uOrVIdx);

					}
				}
				else{
					if(u != 0 && u != uDim -1){
						cutWasMade = true;
						if(u == uDim / 2){
							uCutUpTwinlessMirrorPatchCenterLine(patch, u, m_insert, m_mirrorPatch, oldUDim, uOrVIdx, leaveNorthOrEast);
						}
						else{
							uCutUpTwinlessMirrorPatch(patch, u, m_insert, m_mirrorPatch, oldUDim, uOrVIdx, leaveNorthOrEast);
						}
					}
				}
			}
			else{
				if(u != 0 && u != uDim -1){
					cutWasMade = true;
					NurbsPatch* swapPatch = patch->uSplitPatch(u);
					g_stw->lesserPostInsertNodeStuff();
					uAdjustForCut(m_insert, patch, oldUDim, swapPatch, u, uOrVIdx);
				}
			}
		}
		else{
			int v = uOrVIdx;
			if(uOrVIdx & 1){
				v++;
			}
			if(patch->mirror){
				if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
					if(v != 0 && v != vDim -1){
						cutWasMade = true;
						NurbsPatch* r_swapPatch = patch->vSplitPatch(v);
						
						cutUpTwinMirorPatch(mirrorPatch, r_swapPatch, v, insert.doU, m_mirrorPatch);
						vAdjustForCut(m_insert, patch, oldUDim, r_swapPatch, v, uOrVIdx);
					}

				}
				else{
					if(v != 0 && v != vDim -1){
						cutWasMade = true;
						vCutUpTwinlessMirrorPatch(patch, v, m_insert, m_mirrorPatch, oldUDim, uOrVIdx, leaveNorthOrEast);
					}
				}
			}
			else{
				if(v != 0 && v != vDim -1){
					cutWasMade = true;
					NurbsPatch* swapPatch = patch->vSplitPatch(v);
					vAdjustForCut(m_insert, patch, oldUDim, swapPatch, v, uOrVIdx);
				}
			}
		}
	}
	if(cutWasMade || 1){
		MultiSelect::reset();

		g_stw->lesserPostInsertNodeStuff();

		smls->populateMirrorNodes();
		smls->generateRawStitchPolys();


		smls->doBuild();

		for(int k = m_mirrorPatch.length; --k >= 0;){
			NurbsPatch* p = m_mirrorPatch[k].patch;
			p->synchronizeCellsAndBuild();
		}
		g_stw->paint();


/////////////

		g_stw->lesserPostInsertNodeStuff();

		smls->populateMirrorNodes();
		MultiSelect::pointList.length = 0;


		fastFullBuildForAutoIn(true, smls);
		smls->doBuild();

		smls->stitchAllPolys();

		for(int k = m_insert.length; --k >= 0;){
			NurbsPatch* patch = m_insert[k].p;
			patch->bnspeBuildPolies();
		}

		smls->ctrf = true;
		BackUp::monostable.beginCycle();

	}
}

bool cutUpPatches(Seamless* smls, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch){
	//return false;

	bool cutWasMade = false;

	g_visualClue = 0;
	for(int k = 0; k < m_insert.length; k++){
		Insert& insert = m_insert[k];
		NurbsPatch* patch = insert.p;
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		int cutLineA = insert.cutLine.x;
		int cutLineB = insert.cutLine.y;
		const int uOrVIdx = insert.uOrVIdx;
		const int oldUDim = patch->dim.x;
		bool leaveNorthOrEast = insert.leaveNorthOrEast;


		if(g_insertPatch.centerMirrorLine && (uOrVIdx == uDim - 2 || uOrVIdx == 0)){
			return false;
		}
		if(insert.inLoop == false){
			continue;
		}

		if(insert.diversion || (k < m_insert.length - 1 && m_insert[k + 1].diversion) || g_insertPatch.loop){
			if(insert.doU){
				int u = uOrVIdx;
				if(uOrVIdx & 1){
					u++;
				}
				if(patch->mirror){
					if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
						if(u != 0 && u != uDim -1){
							cutWasMade = true;
							NurbsPatch* r_swapPatch = patch->uSplitPatch(u);
							addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
							addPatchToList(m_mirrorPatch, r_swapPatch, leaveNorthOrEast);
						
							cutUpTwinMirorPatch(mirrorPatch, r_swapPatch, u, insert.doU, m_mirrorPatch);
							uAdjustForCut(m_insert, patch, oldUDim, r_swapPatch, u, uOrVIdx);

						}
					}
					else{
						if(u != 0 && u != uDim -1){
							cutWasMade = true;
							uCutUpTwinlessMirrorPatch(patch, u, m_insert, m_mirrorPatch, oldUDim, uOrVIdx, leaveNorthOrEast);
						}
					}
				}
				else{
					if(u != 0 && u != uDim -1){
						cutWasMade = true;
						NurbsPatch* swapPatch = patch->uSplitPatch(u);
						g_stw->lesserPostInsertNodeStuff();
						addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
						addPatchToList(m_mirrorPatch, swapPatch, leaveNorthOrEast);
						uAdjustForCut(m_insert, patch, oldUDim, swapPatch, u, uOrVIdx);
					}
				}
			}
			else{
				int v = uOrVIdx;
				if(uOrVIdx & 1){
					v++;
				}
				if(patch->mirror){
					if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
						if(v != 0 && v != vDim -1){
							cutWasMade = true;
							NurbsPatch* r_swapPatch = patch->vSplitPatch(v);
							addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
							addPatchToList(m_mirrorPatch, r_swapPatch, leaveNorthOrEast);
						
							cutUpTwinMirorPatch(mirrorPatch, r_swapPatch, v, insert.doU, m_mirrorPatch);
							vAdjustForCut(m_insert, patch, oldUDim, r_swapPatch, v, uOrVIdx);
						}

					}
					else{
						if(v != 0 && v != vDim -1){
							cutWasMade = true;
							vCutUpTwinlessMirrorPatch(patch, v, m_insert, m_mirrorPatch, oldUDim, uOrVIdx, leaveNorthOrEast);
						}
					}
				}
				else{
					if(v != 0 && v != vDim -1){
						cutWasMade = true;
						NurbsPatch* swapPatch = patch->vSplitPatch(v);
						addPatchToList(m_mirrorPatch, patch, leaveNorthOrEast);
						addPatchToList(m_mirrorPatch, swapPatch, leaveNorthOrEast);
						vAdjustForCut(m_insert, patch, oldUDim, swapPatch, v, uOrVIdx);
					}
				}
			}
		}
		else{
			//insertLine(insert, m_insert, m_mirrorPatch);
		}
	}


	if(cutWasMade || 1){
//		g_visualClue = 0;
		MultiSelect::reset();
//		g_insertPatch.imousePhase = InsertPatch::NEUTRAL;
//		g_insertPatch.m_insert.length = 0;
//		g_insertPatch.m_insert.deleteElement(1);

		g_stw->lesserPostInsertNodeStuff();

		smls->populateMirrorNodes();
		smls->generateRawStitchPolys();


		//fastFullBuildForAutoIn(true, smls);
		smls->doBuild();
		//smls->stitchAllPolys();
		//smls->populateMirrorNodes();

		for(int k = m_mirrorPatch.length; --k >= 0;){
			NurbsPatch* p = m_mirrorPatch[k].patch;
			p->synchronizeCellsAndBuild();
		}
		g_stw->paint();

		smls->ctrf = true;
		//houseKeepVisualClue();
		BackUp::monostable.beginCycle();
	}

	for(int k = 0; k < m_insert.length; k++){ 
		Insert& insert = m_insert[k];
		int a = 0; //great point for checking diversion data elements
	}
	return cutWasMade;
}




int InsertPatch::numberOfPointsInLine(const M_Insert& m_insert)const{
	int count = 0;
	for(int k = m_insert.length; --k>=0;){
		const Insert& isrt = m_insert[k];
		count += abs(isrt.cutLine.y - isrt.cutLine.x);
	}
	return count;
}

void InsertPatch::reverseDirection(M_Insert& m_insert){
	int half = m_insert.length / 2;
	for(int k = 0; k < half; k++){
		int ok = m_insert.length - 1 - k;
		Insert store = m_insert[k];
		m_insert[k] = m_insert[ok];
		m_insert[ok] = store;
	}
	for(int k = m_insert.length; --k>=0;){
		Insert& isrt = m_insert[k];
		isrt.leaveNorthOrEast = ! isrt.leaveNorthOrEast;
		isrt.cutLine = Vec2i(isrt.cutLine.y, isrt.cutLine.x);
	}
}
void InsertPatch::changeInitElement(){
	if(m_insert.length){
		init_element = m_insert.length - init_element - 1;
	}
	verifyFirst();
}



void addColumn(Vec3f pointA, Vec3f pointB, Vec3f pointC, M_Vec3f& dpoint, int& uDim){
	dpoint += pointC;
	dpoint.insertElement(uDim * 2, pointB);
	dpoint.insertElement(uDim, pointA);
	uDim++;
}

void setColumn(int& idx, Vec3f pointA, Vec3f pointB, Vec3f pointC, M_Vec3f& dpoint, const int uDim){
	dpoint[idx + uDim * 0] = pointA;
	dpoint[idx + uDim * 1] = pointB;
	dpoint[idx + uDim * 2] = pointC;
	idx++;
}

void addRow(Vec3f pointA, Vec3f pointB, Vec3f pointC, M_Vec3f& dpoint, int& vDim){
	dpoint += pointA;
	dpoint += pointB;
	dpoint += pointC;
	vDim++;
}



void getLastDiversion(M_Insert& m_insert, int k, int& lastDiversionIndex, int& uDim){

	if(k < m_insert.length){
		Vec2i cutLine = m_insert[k].cutLine;
		uDim += abs(evenEdge(cutLine.y) - evenEdge(cutLine.x));
	}

	k++;
	lastDiversionIndex = k;

	while(k < m_insert.length && m_insert[k].diversion){
		Vec2i cutLine = m_insert[k].cutLine;
		uDim += abs(evenEdge(cutLine.y) - evenEdge(cutLine.x));
		if(k == m_insert.length - 1){
			return;
		}
		if(k < m_insert.length - 1 && m_insert[k + 1].diversion == false){
			if(k < m_insert.length - 2 && m_insert[k + 2].diversion){
				k++;
			}
			else{
				if(isLastInLoop(k + 1, m_insert)){
					lastDiversionIndex++;
				}
				return;
			}
		}
		k++;
		lastDiversionIndex = k;
	}
}

void getLastDiversionForLoop(M_Insert& m_insert, int k, int& lastDiversionIndex){

	k++;
	lastDiversionIndex = k;

	while(k < m_insert.length && m_insert[k].inLoop){
		if(k == m_insert.length - 1){
			return;
		}
		if(k < m_insert.length - 1 && m_insert[k + 1].inLoop == false){
			if(k < m_insert.length - 2 && m_insert[k + 2].inLoop){
				k++;
			}
			else{
				if(isLastInLoop(k + 1, m_insert)){
					lastDiversionIndex++;
				}
				return;
			}
		}
		k++;
		lastDiversionIndex = k;
	}
}

void getLastDiversionForCentreControlLine(M_Insert& m_insert, int k, int& lastDiversionIndex){

	while(k < m_insert.length && uIsCentre(m_insert[k])){
		lastDiversionIndex = k;
		k++;
	}
}


void updateDimWeightsKnots2(NurbsPatch* shaft, int newUdim, int newVdim, int& newPointLen){
	shaft->dim.refValue() = Vec2f(newUdim, newVdim);
	M_Float& weight = shaft->weight.refValue();

	newPointLen = newUdim * newVdim;
	weight.length = newPointLen;
	for(int i = newPointLen; --i >= 0;){
		weight[i] = 1;
	}

	updateNurbsCurveKnotMid1(newUdim, shaft->uKnot.refValue());
	updateNurbsCurveKnotMid1(newVdim, shaft->vKnot.refValue());

	//g_stw->lesserPostInsertNodeStuff();

	shaft->isCtrlPolyRef = true;
	//shaft->processFields();
	shaft->pointsToDpoints();
}

void updateDimWeightsKnots(NurbsPatch* r_shaft, NurbsPatch* l_shaft, int newUdim, int newVdim, int& newPointLen){
	updateDimWeightsKnots2(r_shaft, newUdim, newVdim, newPointLen);
	r_shaft->patchToCentralCPs();
	if(l_shaft){
		updateDimWeightsKnots2(l_shaft, newUdim, newVdim, newPointLen);
		l_shaft->patchToCentralCPs();
	}
	r_shaft->smls->generateRawStitchPolys();
}

void setColumn(int& idx, Vec3f pointA, Vec3f pointB, Vec3f pointC, const int uDim, NurbsPatch* patch, NurbsPatch* r_shaft, NurbsPatch* l_patch, NurbsPatch* l_shaft){
	M_Vec3f& newPoint = r_shaft->point.refValue();

	newPoint[idx + uDim * 0] = pointA;
	newPoint[idx + uDim * 1] = pointB;
	newPoint[idx + uDim * 2] = pointC;

	if(l_patch){
		M_Vec3f& l_newPoint = l_shaft->point.refValue();

		pointA.x = -pointA.x;
		pointB.x = -pointB.x;
		pointC.x = -pointC.x;

		l_newPoint[idx + uDim * 0] = pointA;
		l_newPoint[idx + uDim * 1] = pointB;
		l_newPoint[idx + uDim * 2] = pointC;
	}

	idx++;
}

void setRow(int& idx, Vec3f point, const int uDim, NurbsPatch* patch, NurbsPatch* r_shaft){
	M_Vec3f& newPoint = r_shaft->point.refValue();
	newPoint[idx * uDim + 1] = point;
	idx++;
}

void setCpIndex2(NurbsPatch* patchDst, int idxDst, const NurbsPatch* patchSrc, int idxSrc){
	M_Int& cpIndexDst = patchDst->cpIndex.refValue();
	const M_Int& cpIndexSrc = patchSrc->cpIndex.refValue();

	cpIndexDst[idxDst] = cpIndexSrc[idxSrc];

	bool mirror = false;
	if(patchDst->mirror){
		mirror = true;
		if(patchDst->mirrorPatch){
			patchDst = patchDst->mirrorPatch;
		}
		else{
			idxDst = patchDst->getMirrorTwinlessIdx(idxDst);
		}
	}
	if(patchSrc->mirror){
		mirror = true;
		if(patchSrc->mirrorPatch){
			patchSrc = patchSrc->mirrorPatch;
		}
		else{
			idxSrc = patchSrc->getMirrorTwinlessIdx(idxSrc);
		}
	}
	if(mirror){
		M_Int& cpIndexDst = patchDst->cpIndex.refValue();
		const M_Int& cpIndexSrc = patchSrc->cpIndex.refValue();

		cpIndexDst[idxDst] = cpIndexSrc[idxSrc];
	}
}


int getStripLen(int k, const bool beginsFromCenterLine, const M_Insert& m_insert, const int lastDiversionIndex){
	int newUDim = 0;
	for(; k < m_insert.length; k++){
		const Insert& insert = m_insert[k];

		int uOrVIdx = insert.uOrVIdx;
		bool doU = insert.doU;
		bool leaveNorthOrEast = insert.leaveNorthOrEast;
		Vec2i cutLine = insert.cutLine;

		int incAmount = 1;

		if(doU){
			int v = evenEdge(cutLine.x);
			int endV = evenEdge(cutLine.y);

			if(leaveNorthOrEast){
				endV++;
			}
			else{
				endV--;
				incAmount = -1;
			}

			if((uOrVIdx & 1) == 0){
				while(v != endV){
					v += incAmount;
					if(v == endV && k < m_insert.length - 1){
						if(k < lastDiversionIndex){
							break;
						}
					}
					newUDim++;
				}
			}
			else{
				while(v != endV){

					v += incAmount;

					if(v == endV && k < m_insert.length - 1){
						if(k < lastDiversionIndex){
							break;
						}
					}
					newUDim++;
				}
			}
		}
		else{
			int u = cutLine.x;
			if(beginsFromCenterLine && k == 0){
			}
			else{
				u = evenEdge(u);
			}

			int endU = cutLine.y;
			if(endsOnCenterLineFunc(k, m_insert.length, insert)){
			}
			else{
				endU = evenEdge(cutLine.y);
			}

			if(leaveNorthOrEast){
				endU++;
			}
			else{
				endU--;
				incAmount = -1;
			}

			if((uOrVIdx & 1) == 0){
				while(u != endU){
					u += incAmount;
					if(u == endU && k < m_insert.length - 1){
						if(k < lastDiversionIndex){
							break;
						}
					}
					newUDim++;
				}
			}
			else{
				while(u != endU){
					u += incAmount;
					if(u == endU && k < m_insert.length - 1){
						if(k < lastDiversionIndex){
							break;
						}
					}
					newUDim++;
				}
			}
		}

		if(k == lastDiversionIndex){
			break;
		}
	}
	return newUDim;
}

void getEndEdgeIndices(const Insert& insert, int& cpIdxA, int& cpIdxB){
	const int uOrVIdx = insert.uOrVIdx;
	const int uDim = insert.p->dim.x;
	const int vDim = insert.p->dim.y;

	if(insert.doU){
		if(insert.uOrVIdx & 1){
			cpIdxA = uOrVIdx + 2;
			cpIdxB = uOrVIdx + 1;
		}
		else{
			cpIdxA = uOrVIdx + 1;
			cpIdxB = uOrVIdx + 2;
		}
		if(insert.leaveNorthOrEast){
			int forOtherSide = (vDim - 1) * uDim;
			cpIdxA += forOtherSide;
			cpIdxB += forOtherSide;
		}
	}
	else{
		if(insert.uOrVIdx & 1){
			cpIdxA = (uOrVIdx + 2) * uDim;
			cpIdxB = (uOrVIdx + 1) * uDim;
		}
		else{
			cpIdxA = (uOrVIdx + 1) * uDim;
			cpIdxB = (uOrVIdx + 2) * uDim;
		}
		if(insert.leaveNorthOrEast){
			cpIdxA += uDim - 1;
			cpIdxB += uDim - 1;
		}
	}
}

void joinStripIndices(const Insert& prevInsert, NurbsPatch* r_shaft, const int newUdim, const int prevCpIdxA, int const prevCpIdxB){
	NurbsPatch* prevPatch = prevInsert.p;
	M_Int& prevCpIndex = prevPatch->cpIndex.refValue(); 
	M_Int& newCpIndex = r_shaft->cpIndex.refValue();
	
	int newIdxA = newUdim;
	int newIdxB = 0;
	
	if(prevPatch->insideOut){
		if(prevInsert.doU){
			if(prevInsert.uOrVIdx & 1){
				if(prevInsert.leaveNorthOrEast == false){
					newIdxB = newUdim * 2; //untested
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 2; //untested
				}
			}
		}
		else{
			if(prevInsert.uOrVIdx & 1){
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 2; //untested
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast == false){
					newIdxB = newUdim * 2; //tested
				}
			}
		}
	}
	else{
		if(prevInsert.doU){
			if(prevInsert.uOrVIdx & 1){
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 2;
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast == false){
					newIdxB = newUdim * 2;
				}
			}
		}
		else{
			if(prevInsert.uOrVIdx & 1){
				if(prevInsert.leaveNorthOrEast == false){
					newIdxB = newUdim * 2;
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 2;
				}
			}
		}
	}

	//newCpIndex[newIdxA] = prevCpIndex[prevCpIdxA];
	//newCpIndex[newIdxB] = prevCpIndex[prevCpIdxB];
	setCpIndex2(r_shaft, newIdxA, prevPatch, prevCpIdxA);
	setCpIndex2(r_shaft, newIdxB, prevPatch, prevCpIdxB);
}

void joinRowStripIndices(const Insert& prevInsert, NurbsPatch* r_shaft, const int newVdim, const int prevCpIdxA, int const prevCpIdxB){
	NurbsPatch* prevPatch = prevInsert.p;
	M_Int& prevCpIndex = prevPatch->cpIndex.refValue(); 
	M_Int& newCpIndex = r_shaft->cpIndex.refValue();
	
	int newIdxA = 1;
	int newIdxB = 2;

	if(prevInsert.doU){
		if(prevInsert.uOrVIdx & 1){
			if(prevInsert.leaveNorthOrEast){
				newIdxB = 0;
			}
		}
		else{
			if(prevInsert.leaveNorthOrEast == false){
				newIdxB = 0;
			}
		}
	}
	else{
		if(prevInsert.uOrVIdx & 1){
			if(prevInsert.leaveNorthOrEast == false){
				newIdxB = 0;
			}
		}
		else{
			if(prevInsert.leaveNorthOrEast){
				newIdxB = 0;
			}
		}
	}

	//newCpIndex[newIdxA] = prevCpIndex[prevCpIdxA];
	//newCpIndex[newIdxB] = prevCpIndex[prevCpIdxB];
	setCpIndex2(r_shaft, newIdxA, prevPatch, prevCpIdxA);
	setCpIndex2(r_shaft, newIdxB, prevPatch, prevCpIdxB);
}

void uPinchCentralStrip(NurbsPatch* patch, NurbsPatch* r_shaft, bool leaveNorthOrEast){
	M_Int& cpIndex = patch->cpIndex.refValue();


	int newUdim = r_shaft->dim.x;
	int newVdim = r_shaft->dim.y;
	int offset = 0;

	if(leaveNorthOrEast == false){
		offset = (newVdim - 1)  * newUdim;
	}

	if(cpIndex[0] == cpIndex[1]){
		setCpIndex2(r_shaft, 0 + offset, patch, 0);
		setCpIndex2(r_shaft, 1 + offset, patch, 0);
		setCpIndex2(r_shaft, 2 + offset, patch, 0);
	}
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int idxA = (vDim - 1) * uDim;
	int idxB = idxA + 1;

	offset = 0;

	if(leaveNorthOrEast){
		offset = (newVdim - 1)  * newUdim;
	}

	if(cpIndex[idxA] == cpIndex[idxB]){
		setCpIndex2(r_shaft, 0 + offset, patch, idxA);
		setCpIndex2(r_shaft, 1 + offset, patch, idxA);
		setCpIndex2(r_shaft, 2 + offset, patch, idxA);
	}
}

void addCentreSeam(NurbsPatch* patch, Seamless* smls, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch, int k, M_Node& r_m_node, NurbsPatchPtr& r_shaft, int& newUdim, int& newVdim, int& markCp, const int firstInLoop, int& lastDiversionIndex, int& rowIdx){
	NurbsPatch* l_patch = 0;

	Insert& insert = m_insert[k];


	int uOrVIdx = insert.uOrVIdx;
	patch = insert.p;

	//if(patch->mirrorPatch){
	//	patch = patch->mirrorPatch;
	//}


	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	bool f = true;
	if(uOrVIdx == 0){
		f = false;
	}
	
	int prev_uDim = -1;
	int prev_uOrVIdx = -1;
	if(k > 0){
		prev_uDim = m_insert[k - 1].p->dim.x;
		prev_uOrVIdx = m_insert[k-1].uOrVIdx;
	}

	if(r_shaft == 0){

		getLastDiversionForCentreControlLine(m_insert, k, lastDiversionIndex);

		r_shaft = new NurbsPatch;
		//r_shaft->name = "shaft";


		bool f2 = f;
		if(insert.leaveNorthOrEast == false){
			f2 = !f2;
		}


		addPatchToList(m_mirrorPatch, r_shaft, f2);
		r_m_node += r_shaft.p;
		r_shaft->sync.refValue() = true;
		r_shaft->smls = smls;
		r_shaft->uJoinEnds.refValue() = false;
		r_shaft->mirror.refValue() = true;

		newUdim = 3;
		rowIdx = 0;

		newVdim = getStripLen(k, false, m_insert, lastDiversionIndex);
		markCp = smls->addCpsForStrip(r_shaft, newUdim, newVdim);

		if(k > 0 && (false == (g_insertPatch.loop && k == firstInLoop))){
			int cpIdxA = 0;
			int cpIdxB = 0;
			getEndEdgeIndices(m_insert[k - 1], cpIdxA, cpIdxB);
			joinRowStripIndices(m_insert[k - 1], r_shaft, newUdim, cpIdxA, cpIdxB);
		}
	}
//////
	M_Vec3f& newPoint = r_shaft->point.refValue();
	M_Float& newWeight = r_shaft->weight.refValue();
	M_Int& newCpIndex = r_shaft->cpIndex.refValue();
	M_Float& newUknot = r_shaft->uKnot.refValue();
	M_Float& newVknot = r_shaft->vKnot.refValue();


	M_Vec3f& dpoint = patch->dpoint.refValue();
	M_Float& weight = patch->weight.refValue();

	M_Float& uKnot = patch->uKnot;
	M_Float& vKnot = patch->vKnot;

	
	bool doU = insert.doU;
	bool leaveNorthOrEast = insert.leaveNorthOrEast;
	Vec2i cutLine = insert.cutLine;

	int incAmount = 1;

	bool mirrorEast = !leaveNorthOrEast;
 	addPatchToList(m_mirrorPatch, patch, !f);

	uPinchCentralStrip(patch, r_shaft, leaveNorthOrEast);

	if(doU){
		int v = cutLine.x;
		int endV = cutLine.y;
		if(leaveNorthOrEast){
			v = 0;
			endV = vDim;
		}
		else{
			v = vDim - 1;
			endV = -1;
			incAmount = -1;
		}

		if((uOrVIdx & 1) == 0){
			while(v != endV){
				int idxA = v * uDim + uOrVIdx;
				Vec3f vA = dpoint[idxA];

				int srcIdx = rowIdx * newUdim;

				if(patch->isInsideOut()){
					if(leaveNorthOrEast == false){
						srcIdx += 2;
					}
				}
				else{
					if(leaveNorthOrEast){
						srcIdx += 2;
					}
				}
				setCpIndex2(patch, idxA, r_shaft, srcIdx);

				v += incAmount;
				if(v == endV && k < m_insert.length - 1){
					if(insert.diversion == false){
						break;
					}
				}
				setRow(rowIdx, vA, newUdim, patch, r_shaft);
			}
		}
		else{
			while(v != endV){
				int idxA = v * uDim + uOrVIdx + 1;

				Vec3f vA = dpoint[idxA];

				int srcIdx = rowIdx * newUdim;
				if(patch->isInsideOut()){
					if(leaveNorthOrEast){
						srcIdx += 2;
					}
				}
				else{
					if(leaveNorthOrEast == false){
						srcIdx += 2;
					}
				}
				setCpIndex2(patch, idxA, r_shaft, srcIdx);

				v += incAmount;
				if(v == endV && k < m_insert.length - 1){
					if(insert.diversion == false){
						break;
					}
				}
				setRow(rowIdx, vA, newUdim, patch, r_shaft);
			}
		}
	}
	///////////////
	patch->dpointsToPoints();
	if(patch){
		bool twinStrips = false;

		if(patch->mirror){
			r_shaft->mirror.refValue() = true;
			if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
				twinStrips = true;
			}
			else{
				//if(k == m_insert.length - 1 && m_insert[k].crossedMiddle){
				if(endsOnCenterLineFunc(k, m_insert.length, insert)){
					int len = newPoint.length / newVdim;
					int uDim = patch->dim.x;
					//len -= uDim;
					len--;
					for(int i = 0; i < len; i++){
						addColumn(Vec3f(), Vec3f(), Vec3f(), newPoint, newUdim);
					}
				}
				else{
					twinStrips = true;
				}
			}
		}



		//r_shaft->dpointsToPoints();
		int newPointLen = 0;
		updateDimWeightsKnots(r_shaft, 0, newUdim, newVdim, newPointLen);

		g_stw->lesserPostInsertNodeStuff();
		r_shaft->isCtrlPolyRef = true;
		smls->generateRawStitchPolys();
		smls->populateMirrorNodes();
	}
	smls->generateRawStitchPolys();
	smls->stitchAllPolys();
	smls->populateMirrorNodes();
}

bool getFirstInLoop(int& k, M_Insert& m_insert){
	for(k = 0; k < m_insert.length; k++){
		if(m_insert[k].inLoop == true){
			return true;
		}
	}
	return false;
}

NurbsPatch* getOtherEndOfLoop(int& otherIdx, M_Insert& m_insert, int& insrtIdx){
	NurbsPatch* otherPatch = 0;
	for(insrtIdx = m_insert.length; --insrtIdx>=0;){
		if(m_insert[insrtIdx].inLoop == true){
			if(insrtIdx == m_insert.length - 1){
				Insert& insert = m_insert[insrtIdx];
				otherPatch = insert.p;
				Vec2i otherUV;

				if(insert.doU){
					otherUV.x = insert.uOrVIdx;
					otherUV.y = insert.cutLine.y;
				}
				else{
					otherUV.y = insert.uOrVIdx;
					otherUV.x = insert.cutLine.y;
				}
				otherIdx = convertToActualFormat(otherUV, otherPatch->dim.x);
				return otherPatch;
			}
			insrtIdx++;
			Insert& insert = m_insert[insrtIdx];
			otherPatch = insert.p;
			Vec2i otherUV;

			if(insert.doU){
				otherUV.x = insert.uOrVIdx;
				otherUV.y = insert.cutLine.x;
			}
			else{
				otherUV.y = insert.uOrVIdx;
				otherUV.x = insert.cutLine.x;
			}
			otherIdx = convertToActualFormat(otherUV, otherPatch->dim.x);
			break;
		}
	}
	return otherPatch;
}

void beelineCorner(NurbsPatch* patch, int idx, Vec3f& altVec){
	int uDim = patch->dim.x;
	int u = idx % uDim;
	int v = idx / uDim;
	int uA = u;
	int vA = v;
	int uB = u;
	int vB = v;

	if(u == 0){
		uA++;
		if(v == 0){
			vB++;
		}
		else{
			vB--;
		}
	}
	else{
		uA--;
		if(v == 0){
			vB++;
		}
		else{
			vB--;
		}
	}

	altVec = (patch->dpoint[vA * uDim + uA] + patch->dpoint[vB * uDim + uB]) * .5;
}


void mergeStrips(NurbsPatch* patchA, NurbsPatch* patchB){
	patchA->rotateIndexOrder270();
	patchB->rotateIndexOrder90();
	patchB->setInsideOut2();

	patchB->removeSouthEdge(patchA);
	patchA->point.refValue() += patchB->point;
	patchA->weight.refValue() += patchB->weight;
	patchA->cpIndex.refValue() += patchB->cpIndex;

	patchA->rotateIndexOrder90();

	//patchA->updateKnot(patchA->dim.refValue().x, patchA->uKnot.refValue(), patchA->order.refValue().x);
	//patchA->updateKnot(patchA->dim.refValue().y, patchA->vKnot.refValue(), patchA->order.refValue().y);

	//patchA->pointsToDpoints();

	patchA->processFields();
	//patchA->smls->centralCPsToPatches();
}

void mergeStrips3(NurbsPatch* patchA, NurbsPatch* patchB){
	patchA->rotateIndexOrder90();
	patchB->setInsideOut2();
	patchB->rotateIndexOrder90();

	patchB->removeSouthEdge(patchA);
	patchA->point.refValue() += patchB->point;
	patchA->weight.refValue() += patchB->weight;
	patchA->cpIndex.refValue() += patchB->cpIndex;

	patchA->rotateIndexOrder270();

	patchA->processFields();
	//patchB->processFields();
}

void removellExceptLastTwoNorthernRowsThenInsertRow(NurbsPatch* patch, NurbsPatch* fromPatch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int from_uDim = fromPatch->dim.x;
	int from_vDim = fromPatch->dim.y;

	M_Vec3f& point = patch->point.refValue();
	M_Float& weight = patch->weight.refValue();
	M_Int& cpIndex = patch->cpIndex.refValue();

	M_Vec3f& fromPoint = fromPatch->point.refValue();
	M_Float& fromWeight = fromPatch->weight.refValue();
	M_Int& fromCpIndex = fromPatch->cpIndex.refValue();

	int numOf = (vDim - 2) * uDim;

	point.deleteElement(0, numOf);
	weight.deleteElement(0, numOf);
	cpIndex.deleteElement(0, numOf);

	int mark = point.length;

	point.length += 3;
	weight.length += 3;
	cpIndex.length += 3;

	for(int i = 0; i < 3; i++){
		point[mark + i] = fromPoint[from_uDim + i];
		weight[mark + i] = fromWeight[from_uDim + i];
		cpIndex[mark + i] = fromCpIndex[from_uDim + i];
	}

}

void mergeStrips2(NurbsPatch* patchA, NurbsPatch* patchB, bool stripEndsLineOfSymmetry, bool endsOnCenterLine){
	patchA->name = "";
	patchA->mirrorPatch = 0;
	patchA->rotateIndexOrder90();
	patchB->rotateIndexOrder270();
	patchB->setInsideOut2();


	patchB->removeSouthEdge(patchA);
	patchA->point.refValue() += patchB->point;
	patchA->weight.refValue() += patchB->weight;
	patchA->cpIndex.refValue() += patchB->cpIndex;

	if(stripEndsLineOfSymmetry){
		patchB->name = "";
		patchB->mirrorPatch = 0;

		patchB->insideOut.refValue() = false;
		removellExceptLastTwoNorthernRowsThenInsertRow(patchB, patchA);
		patchB->dim.refValue() = Vec2f(3, 3);
		patchB->rotateIndexOrder270();
		patchB->updateKnot(patchB->dim.refValue().x, patchB->uKnot.refValue(), patchB->order.refValue().x);
		patchB->updateKnot(patchB->dim.refValue().y, patchB->vKnot.refValue(), patchB->order.refValue().y);
		patchB->processFields();

		if(endsOnCenterLine){
			patchA->removeNorthEdge(0); //why did I need to comment out these 2 lines? must have been needed for something?
			patchA->removeSouthEdge(0);
		}
	}


	patchA->rotateIndexOrder270();

	patchA->processFields();
}


void fixExtras(M_Extra& m_extra, const NurbsPatch* patch, int idx, M_MirrorPatch& m_mirrorPatch){
	for(int k = m_extra.length; --k>=0;){
		NurbsPatch* extraPatch = m_extra[k].p;
		int extraIdx = convertToActualFormat(m_extra[k].uv, extraPatch->dim.x);
		//extraPatch->cpIndex.refValue()[extraIdx] = altVecCpIdx;
		setCpIndex2(extraPatch, extraIdx, patch, idx);
		//extraPatch->dpointsToPoints();
		addPatchToList(m_mirrorPatch, extraPatch, extraPatch->isWestSide(extraIdx));
	}
}




void fixDiversion(M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch, const int k, const int prevK, NurbsPatch* shaft, M_Vec3f& newPoint, M_Int& newCpIndex, const int newUdim, const int remColIdx, NurbsPatch* otherPatch, int otherIdx){
	//return;
	Insert& insert = m_insert[k];
	NurbsPatch* patch = insert.p;

	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int uOrVIdx = insert.uOrVIdx;
	bool doU = insert.doU;

	bool leaveNorthOrEast = insert.leaveNorthOrEast;
	Vec2i cutLine = insert.cutLine;

	//patch is altPatch
	int idx = convertToActualFormat(uOrVIdx, cutLine.x, uDim, doU);

	NurbsPatch* prevPatch = m_insert[prevK].p;
	int prev_uDim = prevPatch->dim.x;
	bool prevLeaveNorthOrEast = m_insert[prevK].leaveNorthOrEast;
	int prevCenterIdx = convertToActualFormat(m_insert[prevK].uOrVIdx, m_insert[prevK].cutLine.y, prev_uDim, m_insert[prevK].doU);

	Vec3f altVec = patch->dpoint[idx];
	int altVecCpIdx = patch->cpIndex[idx];

	//String str = "patch ";
	//str += patch->plidx;
	//dLog(str);

	//str = "idx ";
	//str += idx;
	//dLog(str);


	if(patch == prevPatch){
		beelineCorner(patch, idx, altVec);

		patch->dpoint.refValue()[idx] = altVec;

		if(doU){
			if(	patch->isInsideOut() == false && ((prevLeaveNorthOrEast == true && leaveNorthOrEast == true) || (prevLeaveNorthOrEast == false && leaveNorthOrEast == false)) ||
				patch->isInsideOut() == true && ((prevLeaveNorthOrEast == false && leaveNorthOrEast == true) || (prevLeaveNorthOrEast == true && leaveNorthOrEast == false))){

				//newPoint[remColIdx + newUdim * 2] = altVec;
				//newCpIndex[remColIdx] = otherPatch->cpIndex[otherIdx];
				setCpIndex2(shaft, remColIdx, otherPatch, otherIdx);
			}
			else{
				//newPoint[remColIdx] = altVec;
				//newCpIndex[remColIdx + newUdim * 2] = otherPatch->cpIndex[otherIdx];
				setCpIndex2(shaft, remColIdx + newUdim * 2, otherPatch, otherIdx);
			}
		}
		else{
			if(	patch->isInsideOut() == true && ((prevLeaveNorthOrEast == true && leaveNorthOrEast == true) || (prevLeaveNorthOrEast == false && leaveNorthOrEast == false)) ||
				patch->isInsideOut() == false && ((prevLeaveNorthOrEast == false && leaveNorthOrEast == true) || (prevLeaveNorthOrEast == true && leaveNorthOrEast == false))){

//				newPoint[remColIdx + newUdim * 2] = altVec;
				//newCpIndex[remColIdx] = otherPatch->cpIndex[otherIdx];
				setCpIndex2(shaft, remColIdx,  otherPatch, otherIdx);
			}
			else{
				//newPoint[remColIdx] = altVec;
				//newCpIndex[remColIdx + newUdim * 2] = otherPatch->cpIndex[otherIdx];
				setCpIndex2(shaft, remColIdx + newUdim * 2, otherPatch, otherIdx);
			}
		}
	}
	else{
		//prevPatch->dpoint.refValue()[prevCenterIdx] = altVec;
		//otherPatch->dpoint.refValue()[otherIdx] = altVec;
		//otherPatch->cpIndex.refValue()[otherIdx] = prevPatch->cpIndex[prevCenterIdx];
		setCpIndex2(otherPatch, otherIdx, prevPatch, prevCenterIdx);
		//otherPatch->cpIndex.refValue()[otherIdx] = newCpIndex[remColIdx + newUdim * 2];
		//otherPatch->cpIndex.refValue()[otherIdx] = newCpIndex[remColIdx];
		//fixExtras(insert.m_extra, altVec, m_mirrorPatch);
		//altVecCpIdx
		fixExtras(insert.m_extra, patch, idx, m_mirrorPatch);

		//String str = "prevCenterIdx ";
		//str += prevCenterIdx;
		//dLog(str);

		//str = "otherPatch ";
		//str += otherPatch->plidx;
		//dLog(str);

		//str = "otherIdx ";
		//str += otherIdx;
		//dLog(str);

		//NurbsPatch* p0 = smls->cpNodeList[0];
		//p0->dpoint.refValue()[0] = altVec;
	}

	//if(prevPatch->mirror && prevPatch->mirrorPatch == 0 && ((prev_uDim / 2) & 1)){
//					if(beginsFromCenterLine){
		//singleMirrorBeginStrip(newPoint, newUdim);
//					}
	//if(beginsFromCenterLine && k == m_insert.length - 1){
		//singleMirrorBeginStrip(newPoint, newUdim);
	//}

	prevPatch->dpointsToPoints();
	otherPatch->dpointsToPoints();
	patch->dpointsToPoints();

	addPatchToList(m_mirrorPatch, prevPatch, prevLeaveNorthOrEast);
	addPatchToList(m_mirrorPatch, otherPatch, otherPatch->isWestSide(otherIdx));
}

void outOfLoop(const int prevK, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch, bool mirrorEast, NurbsPatch* r_shaft, M_Vec3f& newPoint, M_Int& newCpIndex, const int newUdim, NurbsPatch* firstStrip, const bool beginsWithStrip){
	int k = 0;
	if(getFirstInLoop(k, m_insert) == false){
		return;
	}


	int otherIdx = 0;
	int insrtIdx = 0;
	NurbsPatch* otherPatch = getOtherEndOfLoop(otherIdx, m_insert, insrtIdx);

	

	if(beginsWithStrip){
		int remNewUdim = 0;
		fixDiversion(m_insert, m_mirrorPatch, insrtIdx, prevK, r_shaft, newPoint, newCpIndex, newUdim, remNewUdim, otherPatch, otherIdx);

		int strip_uDim = firstStrip->dim.x;
		int strip_vDim = firstStrip->dim.y;


		//newCpIndex[newUdim - 1] = firstStrip->cpIndex[0];
		//newCpIndex[newUdim * 2 - 1] = firstStrip->cpIndex[strip_uDim];
		//newCpIndex[newUdim * 3 - 1] = firstStrip->cpIndex[strip_uDim * 2];

		setCpIndex2(r_shaft, newUdim - 1, firstStrip, 0);
		setCpIndex2(r_shaft, newUdim * 2 - 1, firstStrip, strip_uDim);
		setCpIndex2(r_shaft, newUdim * 3 - 1, firstStrip, strip_uDim * 2);

		//firstStrip->cpIndex.refValue()[0] = newCpIndex[newUdim - 1];
		//firstStrip->cpIndex.refValue()[strip_uDim] = newCpIndex[newUdim * 2 - 1];
		//firstStrip->cpIndex.refValue()[strip_uDim * 2] = newCpIndex[newUdim * 3 - 1];

		//Seamless* smls = firstStrip->smls;
		//smls->redirectSingleCpIndex(firstStrip->cpIndex[0], newCpIndex[newUdim - 1]);
		//smls->redirectSingleCpIndex(firstStrip->cpIndex[strip_uDim], newCpIndex[newUdim * 2 - 1]);
		//smls->redirectSingleCpIndex(firstStrip->cpIndex[strip_uDim * 2], newCpIndex[newUdim * 3 - 1]);

		//smls->redirectSingleCpIndex(newCpIndex[newUdim - 1], firstStrip->cpIndex[0]);
		//smls->redirectSingleCpIndex(newCpIndex[newUdim * 2 - 1], firstStrip->cpIndex[strip_uDim]);
		//smls->redirectSingleCpIndex(newCpIndex[newUdim * 3 - 1], firstStrip->cpIndex[strip_uDim * 2]);

	}
	else{
		Insert& insert = m_insert[k];
		if(insert.doU){
			int a = 0;
		}
		else{
			newPoint[newUdim * 3 - 1] = otherPatch->dpoint[otherIdx];

			int uDim = insert.p->dim.x;
			int idx = insert.uOrVIdx * uDim;
			if(insert.leaveNorthOrEast == false){
				idx += uDim - 1;
			}
			insert.p->dpoint.refValue()[idx] = newPoint[newUdim * 3 - 1];
			insert.p->dpoint.refValue()[idx + uDim] = newPoint[newUdim * 2 - 1];
			insert.p->dpoint.refValue()[idx + uDim * 2] = newPoint[newUdim * 1 - 1];

			//insert.p->dpoint.refValue()[idx + uDim * 2] = newPoint[newUdim * 3 - 1];
			//insert.p->dpoint.refValue()[idx + uDim] = newPoint[newUdim * 2 - 1];
			//insert.p->dpoint.refValue()[idx] = newPoint[newUdim * 1 - 1];
		}
		insert.p->dpointsToPoints();
	}
}

void Seamless::redirectSingleCpIndex(int toCpIdx, int fromCpIdx){
	dLog(toCpIdx);
	dLog(fromCpIdx);
	for(int k = cpNodeList.length; --k>=0;){
		NurbsPatch* patch = cpNodeList[k];
		M_Int& cpIndex = patch->cpIndex.refValue();
		for(int i = cpIndex.length; --i>=0;){
			if(cpIndex[i] == fromCpIdx){
				cpIndex[i] = toCpIdx;
			}
		}
	}
}

void redirectToStripEdge(const M_ControlPoint& redirectEdge, NurbsPatch* r_shaft){
	int uDim = r_shaft->dim.x;

	for(int k = uDim; --k>=0;){
		NurbsPatch* patch = redirectEdge[k].p;
		int idx = redirectEdge[k].i;
		int cpIdx = patch->cpIndex[idx];
		patch->cpIndex.refValue()[idx] = r_shaft->cpIndex[k + uDim * 2];
		patch->smls->redirectSingleCpIndex(r_shaft->cpIndex[k], cpIdx);
	}
}

int Seamless::addCpsForStrip(NurbsPatch* shaft, int newUdim, int newVdim){
	shaft->dim.refValue() = Vec2f(newUdim, newVdim);
	M_Int& cpIndex = shaft->cpIndex.refValue();

	shaft->point.refValue().length = shaft->weight.refValue().length = shaft->cpIndex.refValue().length = newUdim * newVdim;

	int markCp = cp.length;

	cp.refValue().length += newUdim * newVdim;
	cw.refValue().length += newUdim * newVdim;

	for(int k = newUdim * newVdim; --k>=0;){
		cpIndex[k] = markCp + k;
		shaft->weight.refValue()[k] = 1;
		cw.refValue()[markCp + k] = 1;
	}

	return markCp;
}

void getBeginEdgeIndices(const Insert& insert, const int uOrVIdx, int& cpIdxA, int& cpIdxB){
	const int uDim = insert.p->dim.x;
	const int vDim = insert.p->dim.y;

	if(insert.doU){
		if(insert.uOrVIdx & 1){
			cpIdxA = uOrVIdx + 2; //good
			cpIdxB = uOrVIdx + 1;
		}
		else{
			cpIdxA = uOrVIdx + 1; //good
			cpIdxB = uOrVIdx + 2;
		}
		if(insert.leaveNorthOrEast == false){
			int forOtherSide = (vDim - 1) * uDim;
			cpIdxA += forOtherSide;
			cpIdxB += forOtherSide;
		}
	}
	else{
		if(insert.uOrVIdx & 1){
			cpIdxA = (uOrVIdx + 2) * uDim; //good
			cpIdxB = (uOrVIdx + 1) * uDim;
		}
		else{
			cpIdxA = (uOrVIdx + 1) * uDim; //good
			cpIdxB = (uOrVIdx + 2) * uDim;
		}
		if(insert.leaveNorthOrEast == false){
			cpIdxA += uDim - 1; //good
			cpIdxB += uDim - 1;
		}
	}
}

void getEndEdgeIndices(const Insert& insert, const int uOrVIdx, int& cpIdxA, int& cpIdxB){
	const int uDim = insert.p->dim.x;
	const int vDim = insert.p->dim.y;

	if(insert.doU){
		if(insert.uOrVIdx & 1){
			cpIdxA = uOrVIdx + 2;
			cpIdxB = uOrVIdx + 1;
		}
		else{
			cpIdxA = uOrVIdx + 1;
			cpIdxB = uOrVIdx + 2;
		}
		if(insert.leaveNorthOrEast){
			int forOtherSide = (vDim - 1) * uDim;
			cpIdxA += forOtherSide;
			cpIdxB += forOtherSide;
		}
	}
	else{
		if(insert.uOrVIdx & 1){
			cpIdxA = (uOrVIdx + 2) * uDim;
			cpIdxB = (uOrVIdx + 1) * uDim;
		}
		else{
			cpIdxA = (uOrVIdx + 1) * uDim;
			cpIdxB = (uOrVIdx + 2) * uDim;
		}
		if(insert.leaveNorthOrEast){
			cpIdxA += uDim - 1;
			cpIdxB += uDim - 1;
		}
	}
}


void joinPrevIndices(const Insert& insert, NurbsPatch* nextPatch, const int nextCpIdxA, const int nextCpIdxB){
	NurbsPatch* patch = insert.p;

	int cpIdxA = 0;
	int cpIdxB = 0;

	getEndEdgeIndices(insert, insert.uOrVIdx, cpIdxA, cpIdxB);

	setCpIndex2(patch, cpIdxA, nextPatch, nextCpIdxA);
	setCpIndex2(patch, cpIdxB, nextPatch, nextCpIdxB);
}


void joinLastIndices(const Insert& insert, NurbsPatch* lastPatch, const int lastCpIdxA, const int lastCpIdxB){
	NurbsPatch* patch = insert.p;

	int cpIdxA = 0;
	int cpIdxB = 0;
	getBeginEdgeIndices(insert, insert.uOrVIdx, cpIdxA, cpIdxB);

	setCpIndex2(patch, cpIdxA, lastPatch, lastCpIdxA);
	setCpIndex2(patch, cpIdxB, lastPatch, lastCpIdxB);
}

void joinPrevStripIndices(const Insert& prevInsert, NurbsPatch* patch, NurbsPatch* r_shaft, const int newUdim, const int cpIdxA, int const cpIdxB){
	
	bool prevIsInsideOut = prevInsert.p->isInsideOut();

	int newIdxA = newUdim * 2 - 1;
	int newIdxB = 0;
	if(prevInsert.doU){
		if(prevInsert.uOrVIdx & 1){
			if(prevIsInsideOut){
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 1 - 1;
				}
				else{
					newIdxB = newUdim * 3 - 1;
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 3 - 1;
				}
				else{
					newIdxB = newUdim * 1 - 1;
				}
			}
		}
		else{
			if(prevIsInsideOut){
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 3 - 1;
				}
				else{
					newIdxB = newUdim * 1 - 1;
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 1 - 1;
				}
				else{
					newIdxB = newUdim * 3 - 1;
				}
			}
		}
	}
	else{
		if(prevInsert.uOrVIdx & 1){
			if(prevIsInsideOut){
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 3 - 1;
				}
				else{
					newIdxB = newUdim * 1 - 1;
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 1 - 1;
				}
				else{
					newIdxB = newUdim * 3 - 1;
				}
			}
		}
		else{
			if(prevIsInsideOut){
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 1 - 1;
				}
				else{
					newIdxB = newUdim * 3 - 1;
				}
			}
			else{
				if(prevInsert.leaveNorthOrEast){
					newIdxB = newUdim * 3 - 1;
				}
				else{
					newIdxB = newUdim * 1 - 1;
				}
			}
		}
	}
	//cpIndex[cpIdxA] = newCpIndex[newIdxA];
	//cpIndex[cpIdxB] = newCpIndex[newIdxB];

	setCpIndex2(patch, cpIdxA, r_shaft, newIdxA);
	setCpIndex2(patch, cpIdxB, r_shaft, newIdxB);
}

void joinPrevRowStripIndices(const Insert& prevInsert, NurbsPatch* patch, NurbsPatch* r_shaft, const int newVdim, const int cpIdxA, int const cpIdxB){
	
	int newIdxA = newVdim * 3 - 2;
	int newIdxB = newIdxA;
	if(prevInsert.doU){
		if(prevInsert.uOrVIdx & 1){
			if(prevInsert.leaveNorthOrEast){
				newIdxB--;
			}
			else{
				newIdxB++;
			}
		}
		else{
			if(prevInsert.leaveNorthOrEast){
				newIdxB++;
			}
			else{
				newIdxB--;
			}
		}
	}
	else{
		if(prevInsert.uOrVIdx & 1){
			if(prevInsert.leaveNorthOrEast){
				newIdxB = newVdim * 1 - 1;
			}
			else{
				newIdxB = newVdim * 3 - 1;
			}
		}
		else{
			if(prevInsert.leaveNorthOrEast){
				newIdxB = newVdim * 3 - 1;
			}
			else{
				newIdxB = newVdim * 1 - 1;
			}
		}
	}
	//cpIndex[cpIdxA] = newCpIndex[newIdxA];
	//cpIndex[cpIdxB] = newCpIndex[newIdxB];

	setCpIndex2(patch, cpIdxA, r_shaft, newIdxA);
	setCpIndex2(patch, cpIdxB, r_shaft, newIdxB);
}


void scrunchStrip(int idxA, int idxB, int colIdx, M_Int& cpIndex, NurbsPatch* patch, NurbsPatch* r_shaft, int newUdim){
	if(cpIndex[idxA] == cpIndex[idxB]){
		setCpIndex2(r_shaft, newUdim * 0 + colIdx, patch, idxA);
		setCpIndex2(r_shaft, newUdim * 1 + colIdx, patch, idxA);
		setCpIndex2(r_shaft, newUdim * 2 + colIdx, patch, idxA);
	}
}
/*
void uScrunch(NurbsPatch* patch, int u){
	M_Int& cpIndex = patch->cpIndex.refValue();

	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int idxA = u;
	int idxB = idxA + 3;
	if(cpIndex[idxA] == cpIndex[idxB]){
		setCpIndex2(patch, idxA + 1, patch, idxA);
		setCpIndex2(patch, idxA + 2, patch, idxA);
	}
	idxA = (vDim - 1) * uDim + u;
	idxB = idxA + 3;

	if(cpIndex[idxA] == cpIndex[idxB]){
		setCpIndex2(patch, idxA + 1, patch, idxA);
		setCpIndex2(patch, idxA + 2, patch, idxA);
	}

}

void vScrunch(NurbsPatch* patch, int v){
	M_Int& cpIndex = patch->cpIndex.refValue();

	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int idxA = v * uDim;
	int idxB = idxA + 3 * uDim;
	if(cpIndex[idxA] == cpIndex[idxB]){
		setCpIndex2(patch, idxA + 1 * uDim, patch, idxA);
		setCpIndex2(patch, idxA + 2 * uDim, patch, idxA);
	}
	idxA += uDim - 1;
	idxB += uDim - 1;

	if(cpIndex[idxA] == cpIndex[idxB]){
		setCpIndex2(patch, idxA + 1 * uDim, patch, idxA);
		setCpIndex2(patch, idxA + 2 * uDim, patch, idxA);
	}

}
*/
/*
void scrunchPatch(NurbsPatch* patch, bool doU, int uOrVIdx){
	if(doU){
		uScrunch(patch, uOrVIdx);
	}
	else{
		vScrunch(patch, uOrVIdx);
	}
}
*/

inline void joinEastAndWestEnds(Insert& insert, int uOrVIdx, bool doU, NurbsPatch* patch, int uDim, M_Int& r_cpIndex, M_Int& l_cpIndex){
	int idxA = convertToActualFormat(uOrVIdx, insert.cutLine.y, uDim, doU);


	if(doU){

	}
	else{
		if(r_cpIndex[idxA - uDim] == l_cpIndex[idxA - uDim]){
			r_cpIndex[idxA] = l_cpIndex[idxA];
			r_cpIndex[idxA + uDim] = l_cpIndex[idxA + uDim];
		}

		//if(r_cpIndex[idxA] == l_cpIndex[idxA]){
		//	r_cpIndex[idxA + uDim] = l_cpIndex[idxA + uDim];
		//	r_cpIndex[idxA + uDim * 2] = l_cpIndex[idxA + uDim * 2];
		//}
	}
}

void commonInsertStrip(Seamless* smls, M_Insert& m_insert, M_MirrorPatch& m_mirrorPatch){
	m_mirrorPatch.length = 0;
	bool withStem = false;
	M_Node* r_m_nodePtr = 0;
	M_Node* l_m_nodePtr = 0;

	int markCp = 0;

	if(NurbsPatchPtr patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		set_m_nodeRef(patch, r_m_nodePtr);

		l_m_nodePtr = r_m_nodePtr;
		if(patch->mirrorPatch){
			set_m_nodeRef(patch->mirrorPatch, l_m_nodePtr);
		}

	}
	else{
		return;
	}


	M_Node& r_m_node = *r_m_nodePtr;
	M_Node& l_m_node = *l_m_nodePtr;



	NurbsPatch* patch = 0;
	NurbsPatch* l_patch = 0;
	NurbsPatchPtr r_shaft = 0;
	NurbsPatchPtr l_shaft = 0;
	NurbsPatchPtr newPatch2 = 0;
	int newUdim = 0;
	int newVdim = 3;
	int newPointLen = 0;
	bool beginsFromCenterLine = false;
	int lastDiversionIndex = -1;
	NurbsPatch* firstStrip = 0;
	bool beginsWithStrip = true;
	int colIdx = 0;

	NurbsPatch* centre = m_insert[0].p;
	bool prevWasCentreStrip = false;

	for(int k = 0; k < m_insert.length; k++){
		Insert& insert = m_insert[k];
		patch = insert.p;
		l_patch = 0;
		if(patch->mirror){
			l_patch = patch;
			if(patch->mirrorPatch){
				l_patch = patch->mirrorPatch;
			}
		}

		M_Vec3f& dpoint = patch->dpoint.refValue();
		M_Float& weight = patch->weight.refValue();
		M_Int& cpIndex = patch->cpIndex.refValue();

		M_Float& uKnot = patch->uKnot;
		M_Float& vKnot = patch->vKnot;


		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		int uOrVIdx = insert.uOrVIdx;
		bool doU = insert.doU;

		if(insert.inLoop == false){
			continue;
		}

		int firstInLoop = 0;
		getFirstInLoop(firstInLoop, m_insert);


		if(g_insertPatch.centerMirrorLine && (uOrVIdx == uDim - 2 || uOrVIdx == 0)){
			addCentreSeam(patch, smls, m_insert, m_mirrorPatch, k, r_m_node, r_shaft, newUdim, newVdim, markCp, firstInLoop, lastDiversionIndex, colIdx);
			prevWasCentreStrip = true;
			beginsFromCenterLine = false;

		}
		else if(insert.diversion || (k < m_insert.length - 1 && m_insert[k + 1].diversion) || g_insertPatch.loop){
			if(k > lastDiversionIndex){
			//if((k < m_insert.length - 1 && m_insert[k + 1].diversion && insert.diversion == false) || (isLastInLoop(k, m_insert) && m_insert[k - 1].diversion == false)){
				prevWasCentreStrip = false;
				r_shaft = new NurbsPatch;
				markCp = smls->cp.length;

				if(firstStrip == 0){
					firstStrip = r_shaft;
					beginsWithStrip = true;
				}


				addPatchToList(m_mirrorPatch, r_shaft, true);
				r_m_node += r_shaft.p;
				r_shaft->sync.refValue() = true;
				//r_shaft->showAll.refValue() = true;
				//r_shaft->name = "strip";

				r_shaft->smls = smls;
				r_shaft->uJoinEnds.refValue() = false;

				newUdim = 0;
				newVdim = 3;
				colIdx = 0;

				beginsFromCenterLine = beginsFromCenterLineFunc(k, insert);

				if(g_insertPatch.loop){
					getLastDiversionForLoop(m_insert, k, lastDiversionIndex);
				}
				else{
					getLastDiversion(m_insert, k, lastDiversionIndex, newUdim);
				}
				newUdim = getStripLen(k, beginsFromCenterLine, m_insert, lastDiversionIndex);


				markCp = smls->addCpsForStrip(r_shaft, newUdim, newVdim);

				l_shaft = 0;
				if(patch->mirror){
					l_shaft = new NurbsPatch;
					//r_m_node += l_shaft.p;
					r_shaft->mirror.refValue() = true;
					l_shaft->mirror.refValue() = true;
					r_shaft->mirrorPatch = l_shaft;
					l_shaft->mirrorPatch = r_shaft;
					//r_shaft->name = "r_shaft";
					//l_shaft->name = "l_shaft";

					l_shaft->insideOut.refValue() = true;
					l_shaft->sync.refValue() = true;
					l_shaft->smls = smls;
					l_shaft->uJoinEnds.refValue() = false;
					smls->addCpsForStrip(l_shaft, newUdim, newVdim);
				}

				if(k > 0 && (false == (g_insertPatch.loop && k == firstInLoop))){
					int cpIdxA = 0;
					int cpIdxB = 0;
					getEndEdgeIndices(m_insert[k - 1], cpIdxA, cpIdxB);
					joinStripIndices(m_insert[k - 1], r_shaft, newUdim, cpIdxA, cpIdxB);
				}
			}

			M_Vec3f& newPoint = r_shaft->point.refValue();
			M_Float& newWeight = r_shaft->weight.refValue();
			M_Int& newCpIndex = r_shaft->cpIndex.refValue();
			M_Float& newUknot = r_shaft->uKnot.refValue();
			M_Float& newVknot = r_shaft->vKnot.refValue();


			//Insert& insert = m_insert[k];
			
			
			bool leaveNorthOrEast = insert.leaveNorthOrEast;
			Vec2i cutLine = insert.cutLine;

			int incAmount = 1;
			int remColIdx = colIdx;

			bool mirrorEast = leaveNorthOrEast;

			if(patch->mirror && patch->mirrorPatch == 0){
				if(doU){
					if(insert.uOrVIdx < (uDim >> 1)){
						mirrorEast = true;
					}
					else{
						mirrorEast = false;
					}
				}
				else{
					if((cutLine.x < (uDim >> 1)) || (cutLine.y < (uDim >> 1))){
						mirrorEast = true;
					}
					else{
						mirrorEast = false;
					}
				}
			}

 			addPatchToList(m_mirrorPatch, patch, mirrorEast);

			if(doU){
				int v = evenEdge(cutLine.x);
				int endV = evenEdge(cutLine.y);

				int _k = k;
				if(leaveNorthOrEast){
					//if(false == (beginsWithStrip && isLastInLoop(k, m_insert) && getFirstInLoop(_k, m_insert))){
						endV++;
					//}
				}
				else{
					//if(false == (beginsWithStrip && isLastInLoop(k, m_insert) && getFirstInLoop(_k, m_insert))){
						endV--;
					//}
					incAmount = -1;
				}


				if(k == 0){
					scrunchStrip(v * uDim + uOrVIdx, v * uDim + uOrVIdx + 1, 0, cpIndex, patch, r_shaft, newUdim);
				}
				if(k == m_insert.length - 1){
					scrunchStrip((endV - incAmount) * uDim + uOrVIdx, (endV - incAmount) * uDim + uOrVIdx + 1, newUdim - 1, cpIndex, patch, r_shaft, newUdim);
				}

				if((uOrVIdx & 1) == 0){
					while(v != endV){
						int idxA = v * uDim + uOrVIdx;
						int idxB = v * uDim + uOrVIdx + 1;
						Vec3f vA = dpoint[idxA];
						Vec3f vB = dpoint[idxB];
						Vec3f vec = vB + (vA - vB) * (1.0 - insert.ratio);
						Vec3f v2 = vec - (vec - vB) * .5;
						
						//dpoint[idxA] = v2;

						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx + newUdim * 2);
							}
							else{
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx);
							}
						}
						else{
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx);
							}
							else{
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx + newUdim * 2);
							}
						}

						v += incAmount;
						if(v == endV && k < m_insert.length - 1){
							if(k < lastDiversionIndex){
								break;
							}
							else if(isLastInLoop(k, m_insert)){//good
								int _k = 0;
								if(getFirstInLoop(_k, m_insert)){
									if(beginsWithStrip){
										int strip_uDim = firstStrip->dim.x;
										int strip_vDim = firstStrip->dim.y;

										if(leaveNorthOrEast){
											setCpIndex2(patch, idxA, firstStrip, 0);
										}
										else{
											setCpIndex2(patch, idxA, firstStrip, strip_uDim * 2);
										}
									}
								}
								break;
							}
						}


						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								setColumn(colIdx, vA, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
							else{
								setColumn(colIdx, v2, vec, vA, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
						else{
							if(leaveNorthOrEast){
								setColumn(colIdx, v2, vec, vA, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
							else{
								setColumn(colIdx, vA, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
					}
				}
				else{
					while(v != endV){
						int idxA = v * uDim + uOrVIdx;
						int idxB = v * uDim + uOrVIdx + 1;

						Vec3f vA = dpoint[idxA];
						Vec3f vB = dpoint[idxB];
						Vec3f vec = vA + (vB - vA) * insert.ratio;
						Vec3f v2 = vec - (vec - vA) * .5;


						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx);
							}
							else{
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx + newUdim * 2);
							}
						}
						else{
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx + newUdim * 2);
							}
							else{
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx);
							}
						}

						v += incAmount;

						if(v == endV && k < m_insert.length - 1){
							if(k < lastDiversionIndex){
								break;
							}
							else if(isLastInLoop(k, m_insert)){//good
								int _k = 0;
								if(getFirstInLoop(_k, m_insert)){
									if(beginsWithStrip){
										int strip_uDim = firstStrip->dim.x;
										int strip_vDim = firstStrip->dim.y;

										if(leaveNorthOrEast){
											//cpIndex[idxB] = firstStrip->cpIndex[strip_uDim * 2];
											setCpIndex2(patch, idxB, firstStrip, strip_uDim * 2);
										}
										else{
											//cpIndex[idxB] = firstStrip->cpIndex[0];
											setCpIndex2(patch, idxB, firstStrip, 0);
										}
									}
								}
								break;
							}
						}
						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								setColumn(colIdx, v2, vec, vB, newUdim, patch, r_shaft, l_patch, l_shaft);

							}
							else{
								setColumn(colIdx, vB, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
						else{
							if(leaveNorthOrEast){
								setColumn(colIdx, vB, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
							else{
								setColumn(colIdx, v2, vec, vB, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
					}
				}
				updateDimWeightsKnots(r_shaft, l_shaft, newUdim, newVdim, newPointLen);
				if(insert.diversion){
					NurbsPatch* otherPatch = insert.otherPatch;
					int otherIdx = convertToActualFormat(insert.otherUV, otherPatch->dim.x);

					fixDiversion(m_insert, m_mirrorPatch, k, k - 1, r_shaft, newPoint, newCpIndex, newUdim, remColIdx, otherPatch, otherIdx);
				}
				if(isLastInLoop(k, m_insert)){//a
					outOfLoop(k, m_insert, m_mirrorPatch, mirrorEast, r_shaft, newPoint, newCpIndex, newUdim, firstStrip, beginsWithStrip);
				}
			}
			else{
				int u = cutLine.x;
				if(beginsFromCenterLine && k == 0){
				}
				else{
					u = evenEdge(u);
				}

				int endU = cutLine.y;
				if(endsOnCenterLineFunc(k, m_insert.length, insert)){
				}
				else{
					endU = evenEdge(cutLine.y);
				}

				int _k = k;
				if(leaveNorthOrEast){
					//if(false == (beginsWithStrip && isLastInLoop(k, m_insert) && getFirstInLoop(_k, m_insert))){
						endU++;
					//}
				}
				else{
					//if(false == (beginsWithStrip && isLastInLoop(k, m_insert) && getFirstInLoop(_k, m_insert))){
						endU--;
					//}
					incAmount = -1;
				}

				if(k == 0){
					scrunchStrip(uOrVIdx * uDim + u, (uOrVIdx + 1) * uDim + u, 0, cpIndex, patch, r_shaft, newUdim);
				}

				if((uOrVIdx & 1) == 0){
					while(u != endU){
						int idxA = uOrVIdx * uDim + u;
						int idxB = (uOrVIdx + 1) * uDim + u;
						Vec3f vA = dpoint[idxA];
						Vec3f vB = dpoint[idxB];
						Vec3f vec = vB + (vA - vB) * (1.0 - insert.ratio);
						Vec3f v2 = vec - (vec - vB) * .5;

						//dpoint[idxA] = v2;
						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx);
							}
							else{
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx + newUdim * 2);
							}
						}
						else{
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx + newUdim * 2);
							}
							else{
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxA);
								}
								setCpIndex2(patch, idxA, r_shaft, colIdx);
							}
						}

						u += incAmount;
						if(u == endU && k < m_insert.length - 1){
							if(k < lastDiversionIndex){
								break;
							}
							else if(isLastInLoop(k, m_insert)){//good
								int _k = 0;
								if(getFirstInLoop(_k, m_insert)){
									if(beginsWithStrip){
										int strip_uDim = firstStrip->dim.x;
										int strip_vDim = firstStrip->dim.y;

										if(leaveNorthOrEast){
											setCpIndex2(patch, idxA, firstStrip, strip_uDim * 2);
										}
										else{
											setCpIndex2(patch, idxA, firstStrip, 0);
										}
									}
								}
								break;
							}
						}
						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								setColumn(colIdx, v2, vec, vA, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
							else{
								setColumn(colIdx, vA, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
						else{
							if(leaveNorthOrEast){
								setColumn(colIdx, vA, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
							else{
								setColumn(colIdx, v2, vec, vA, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
					}
				}
				else{
					while(u != endU){
						int idxA = uOrVIdx * uDim + u;
						int idxB = (uOrVIdx + 1) * uDim + u;

						Vec3f vA = dpoint[idxA];
						Vec3f vB = dpoint[idxB];
						Vec3f vec = vA + (vB - vA) * insert.ratio;
						Vec3f v2 = vec - (vec - vA) * .5;

						//dpoint[idxB] = v2;

						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx + newUdim * 2);
							}
							else{
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx);
							}
						}
						else{
							if(leaveNorthOrEast){
								if(newCpIndex[colIdx + newUdim * 2] >= markCp){
									setCpIndex2(r_shaft, colIdx + newUdim * 2, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx);
							}
							else{
								if(newCpIndex[colIdx] >= markCp){
									setCpIndex2(r_shaft, colIdx, patch, idxB);
								}
								setCpIndex2(patch, idxB, r_shaft, colIdx + newUdim * 2);
							}
						}
						u += incAmount;
						if(u == endU && k < m_insert.length - 1){
							if(k < lastDiversionIndex){
								break;
							}
							else if(isLastInLoop(k, m_insert)){//good
								int _k = 0;
								if(getFirstInLoop(_k, m_insert)){
									if(beginsWithStrip){
										int strip_uDim = firstStrip->dim.x;
										int strip_vDim = firstStrip->dim.y;

										if(leaveNorthOrEast){
											//cpIndex[idxB] = firstStrip->cpIndex[0];
											setCpIndex2(patch, idxB, firstStrip, 0);
										}
										else{
											//cpIndex[idxB] = firstStrip->cpIndex[strip_uDim * 2];
											setCpIndex2(patch, idxB, firstStrip, strip_uDim * 2);
										}
									}
								}
								break;
							}
						}
						if(patch->isInsideOut()){
							if(leaveNorthOrEast){
								//setColumn(colIdx, vB, vec, v2, newPoint, newUdim);
								setColumn(colIdx, vB, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
							else{
								//setColumn(colIdx, v2, vec, vB, newPoint, newUdim);
								setColumn(colIdx, v2, vec, vB, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
						else{
							if(leaveNorthOrEast){
								//setColumn(colIdx, v2, vec, vB, newPoint, newUdim);
								setColumn(colIdx, v2, vec, vB, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
							else{
								//setColumn(colIdx, vB, vec, v2, newPoint, newUdim);
								setColumn(colIdx, vB, vec, v2, newUdim, patch, r_shaft, l_patch, l_shaft);
							}
						}
					}
				}
				updateDimWeightsKnots(r_shaft, l_shaft, newUdim, newVdim, newPointLen);
				if(insert.diversion){
					NurbsPatch* otherPatch = insert.otherPatch;
					int otherIdx = convertToActualFormat(insert.otherUV, otherPatch->dim.x);

					fixDiversion(m_insert, m_mirrorPatch, k, k - 1, r_shaft, newPoint, newCpIndex, newUdim, remColIdx, otherPatch, otherIdx);
				}
				if(isLastInLoop(k, m_insert)){
					outOfLoop(k, m_insert, m_mirrorPatch, mirrorEast, r_shaft, newPoint, newCpIndex, newUdim, firstStrip, beginsWithStrip);
				}
			}
			patch->dpointsToPoints();
			if(k == lastDiversionIndex){
				if(patch){
					if(patch->mirror){
						r_shaft->mirror.refValue() = true;

						NurbsPatch* prevPatch = m_insert[k - 1].p;
						int prev_uDim = prevPatch->dim.x;

						//if(k == 1 && prevPatch->mirror && prevPatch->mirrorPatch == 0 && ((prev_uDim / 2) & 1)){
						//if(beginsFromCenterLine && k == lastDiversionIndex){
						//	if(endsOnCenterLineFunc(k, m_insert.length, insert)){
						//		addExtraPatchForLoopStrip(newPatch2, m_mirrorPatch, r_m_node, smls, newPoint, newUdim);

						//		deleteLastColumn(newPoint, newUdim);
						//		singleMirrorBeginStrip(newPoint, newUdim);
						//		beginsFromCenterLine = false;
						//	}
						//	else{
						//		singleMirrorBeginStrip(newPoint, newUdim);
						//		beginsFromCenterLine = false;
						//	}
						//}
						//else{
							//if(false){ //must be answered
							//	bool twinStrips = false;//was back earlier
							//	if(patch->mirrorPatch){
							//		if(false == (endsOnCenterLineFunc(lastDiversionIndex, m_insert.length, insert))){
							//			twinStrips = true;
							//		}
							//	}
							//	else{
							//		if(endsOnCenterLineFunc(k, m_insert.length, insert)){
							//			int len = newPoint.length / newVdim;
							//			len--;
							//			for(int i = 0; i < len; i++){
							//				setColumn(colIdx, Vec3f(), Vec3f(), Vec3f(), newPoint, newUdim);
							//			}
							//		}
							//		else{
							//			twinStrips = true;
							//		}
							//	}
							//}
						//}
					}
					//r_shaft->dpointsToPoints();
					updateDimWeightsKnots(r_shaft, l_shaft, newUdim, newVdim, newPointLen); // do we need this here still?




					g_stw->lesserPostInsertNodeStuff();

					r_shaft->isCtrlPolyRef = true;
					//r_shaft->processFields();
					r_shaft->pointsToDpoints();
					//smls->addCps(r_shaft, smls->cp.length);
					r_shaft->pointsToCpws();
					//smls->addCps(r_shaft, smls->cp.length);
					//redirectToStripEdge(redirectEdge, r_shaft);



					if(newPatch2){
						newPatch2->isCtrlPolyRef = true;
						newPatch2->processFields();
						newPatch2->pointsToDpoints();
						newPatch2 = 0;
					}


//						if(twinStrips){
						//NurbsPatch* l_shaft = r_shaft->mirrorPatch;
					if(l_shaft){
						//l_shaft = r_shaft->createCopyOf();

						int strip_uDim = l_shaft->dim.x;

						bool stripBeginsLineOfSymmetry = false;
						if(firstStrip == r_shaft){
							if(
								l_shaft->cpIndex[0] == r_shaft->cpIndex[0] ||
								l_shaft->cpIndex[strip_uDim * 2] == r_shaft->cpIndex[strip_uDim * 2]
								){

								stripBeginsLineOfSymmetry = true;
							}
						}



						bool stripEndsLineOfSymmetry = false;
						bool endsOnVisibleCenterLine = false;
						if(
							(
								l_shaft->cpIndex[strip_uDim - 1] == r_shaft->cpIndex[strip_uDim - 1] ||
								l_shaft->cpIndex[strip_uDim * 3 - 1] == r_shaft->cpIndex[strip_uDim * 3 - 1]
							)

							&& l_shaft->cpIndex[strip_uDim - 1] != l_shaft->cpIndex[strip_uDim * 3 - 1] //does not apply if ends with spider web center
							){

							stripEndsLineOfSymmetry = true;

							endsOnVisibleCenterLine = endsOnCenterLineFunc(lastDiversionIndex, m_insert.length, insert);
						}


						if(beginsFromCenterLine && firstStrip == r_shaft){
							//l_shaft->cpIndex.refValue()[strip_uDim] = r_shaft->cpIndex[strip_uDim];

							if(stripEndsLineOfSymmetry && endsOnVisibleCenterLine == false){
								smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim * 1 - 1], r_shaft->cpIndex[strip_uDim * 1 - 1]);
								smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim * 2 - 1], r_shaft->cpIndex[strip_uDim * 2 - 1]);
								smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim * 3 - 1], r_shaft->cpIndex[strip_uDim * 3 - 1]);
							}

							smls->redirectSingleCpIndex(l_shaft->cpIndex[0], r_shaft->cpIndex[0]);
							smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim * 2], r_shaft->cpIndex[strip_uDim * 2]);

							mergeStrips2(r_shaft, l_shaft, stripEndsLineOfSymmetry, endsOnVisibleCenterLine);
							newUdim = r_shaft->dim.x;
							if(endsOnVisibleCenterLine){
								r_m_node += l_shaft.p;
							}

						}
						else if(stripEndsLineOfSymmetry){
							l_shaft->cpIndex.refValue()[strip_uDim * 2 - 1] = r_shaft->cpIndex[strip_uDim * 2 - 1];
							smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim - 1], r_shaft->cpIndex[strip_uDim - 1]);
							smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim * 3 - 1], r_shaft->cpIndex[strip_uDim * 3 - 1]);

							if(stripBeginsLineOfSymmetry){
								l_shaft->cpIndex.refValue()[strip_uDim] = r_shaft->cpIndex[strip_uDim];
								smls->redirectSingleCpIndex(l_shaft->cpIndex[0], r_shaft->cpIndex[0]);
								smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim * 2], r_shaft->cpIndex[strip_uDim * 2]);
							}

							mergeStrips(r_shaft, l_shaft);
						}
						else{
							if(
								l_shaft->cpIndex[0] == r_shaft->cpIndex[0] ||
								l_shaft->cpIndex[strip_uDim * 2] == r_shaft->cpIndex[strip_uDim * 2]
								){

								l_shaft->cpIndex.refValue()[strip_uDim] = r_shaft->cpIndex[strip_uDim];
								smls->redirectSingleCpIndex(l_shaft->cpIndex[0], r_shaft->cpIndex[0]);
								smls->redirectSingleCpIndex(l_shaft->cpIndex[strip_uDim * 2], r_shaft->cpIndex[strip_uDim * 2]);
								mergeStrips3(r_shaft, l_shaft);
								newUdim = r_shaft->dim.x;
								//r_shaft->rotateIndexOrder90();
								//smls->centralCPsToPatches();
								//patch->processFields();
								//smls->generateRawStitchPolys();
							}
							else{

								l_m_node += l_shaft.p;
								//l_shaft->setInsideOut();
								r_shaft->name = "r_shaft";
								r_shaft->makeExistingNameUniqueForMirror(l_shaft);
							}

						}
						smls->centralCPsToPatches();
						g_stw->lesserPostInsertNodeStuff();
						l_shaft->isCtrlPolyRef = true;
					}
					//else{
					//	//l_shaft->point.refValue().length = newPointLen;
					//	//l_shaft->dpoint.refValue().length = newPointLen;
					//	//l_shaft->weight.refValue() = newWeight;
					//	l_shaft->uKnot.refValue() = r_shaft->uKnot;
					//	l_shaft->vKnot.refValue() = r_shaft->vKnot;
					//	l_shaft->dim.refValue() = r_shaft->dim;
					//}

					//r_shaft->mirrorTwinPatch(l_shaft);

					r_shaft->isCtrlPolyRef = true;
					//}


					//smls->generateRawStitchPolys();

					//for(int k = m_mirrorPatch.length; --k>=0;){
					//	patch = m_mirrorPatch[k].patch;
					//	if(patch->mirror){
					//		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
					//			patch->mirrorTwinPatch(mirrorPatch);
					//		}
					//		else{
					//			if(m_mirrorPatch[k].modifiedWest){
					//				patch->mirrorModifyEast();
					//			}
					//			else{
					//				patch->mirrorModifyWest();
					//			}
				
					//		}
					//		patch->pointsToDpoints();
					//	}
					//}
					smls->generateRawStitchPolys();
					//smls->stitchAllPolys();
					smls->populateMirrorNodes();
				}
			}
		}
		else{
			insertLine(insert, m_insert, m_mirrorPatch);

			uOrVIdx = insert.uOrVIdx;

			//scrunchPatch(patch, doU, uOrVIdx);



			int cpIdxA = 0;
			int cpIdxB = 0;
			getBeginEdgeIndices(insert, uOrVIdx, cpIdxA, cpIdxB);

			if(k == m_insert.length - 1 && patch->mirrorPatch){
				//joinEastAndWestEnds(insert, uOrVIdx, doU, patch, uDim, cpIndex, patch->mirrorPatch->cpIndex.refValue());
			}

			if(k > 0){
				if(prevWasCentreStrip){
					joinPrevRowStripIndices(m_insert[k - 1], patch, r_shaft, newVdim, cpIdxA, cpIdxB);
				}
				else{
					if(r_shaft){
						joinPrevStripIndices(m_insert[k - 1], patch, r_shaft, newUdim, cpIdxA, cpIdxB);
					}
					else{
						joinPrevIndices(m_insert[k - 1], patch, cpIdxA, cpIdxB);
					}
				}
			}
			if(k == m_insert.length - 1 && g_insertPatch.foundFirstStitchPoly){

				getEndEdgeIndices(insert, uOrVIdx, cpIdxA, cpIdxB);

				if(prevWasCentreStrip){
				}
				else{
					if(r_shaft){
					}
					else{
						joinLastIndices(m_insert[0], patch, cpIdxA, cpIdxB);
					}
				}
			}
			//else if(m_insert.length == 1 && g_insertPatch.foundFirstStitchPoly){
			//	joinPrevIndices(m_insert[0], patch, cpIdxA, cpIdxB);
			//}
			r_shaft = 0;

			if(firstStrip == 0){
				firstStrip = insert.p;
				beginsWithStrip = false;
				beginsFromCenterLine = false;
			}
			prevWasCentreStrip = false;
		}
	}


	g_stw->lesserPostInsertNodeStuff();

	smls->populateMirrorNodes();
	MultiSelect::pointList.length = 0;


	fastFullBuildForAutoIn(true, smls);
//	smls->doBuild();

	smls->stitchAllPolys();

//	for(int k = m_insert.length; --k >= 0;){
//		NurbsPatch* patch = m_insert[k].p;
//		bnspeBuildPolies(patch);
//	}

	smls->ctrf = true;
	BackUp::monostable.beginCycle();

}

void changeInitElement();

void InsertPatch::sortDirectionsOut(int leaveNorthOrEast, int uOrV, int uOrVDim){

	if(beginsFromCenterLineFunc(0, m_insert[0])){
		reverseDirection(m_insert);
		m_insert.length--;
		m_insertOD[0].first = true;
		m_insert += m_insertOD;
		verifyFirst();
		return;
	}

	int lineCountA = numberOfPointsInLine(m_insert);
	int lineCountB = numberOfPointsInLine(m_insertOD);
	if(leaveNorthOrEast){
		lineCountA -= uOrV;
		lineCountB -= uOrVDim - uOrV;
	}
	else{
		lineCountA -= uOrVDim - uOrV;
		lineCountB -= uOrV;
	}
	if(lineCountA > lineCountB){
		reverseDirection(m_insertOD);
		if(m_insert.length && m_insertOD.length){
			m_insert[0].cutLine.x = m_insertOD[m_insertOD.length - 1].cutLine.x;
		}
		m_insertOD.length--;
		init_element = m_insertOD.length;
		m_insertOD += m_insert;
		m_insert = m_insertOD;
		verifyFirst();
		
	}
	else{
		verifyFirst();
		reverseDirection(m_insert);
		changeInitElement();
		if(m_insert.length && m_insertOD.length){
			m_insertOD[0].cutLine.x = m_insert[m_insert.length - 1].cutLine.x;
			m_insertOD[0].first = true;
			m_insert.length--;
			m_insert += m_insertOD;
		}
		init_leaveNorthOrEast = !init_leaveNorthOrEast;
		verifyFirst();
	}
}

void swapEastWest(M_Insert& m_insert){
	for(int k = m_insert.length; --k >= 0;){
		Insert& insert = m_insert[k];
		NurbsPatch* p = insert.p;
		if(p->mirror){
			bool doU = insert.doU;
			if(NurbsPatch* mirrorPatch = p->mirrorPatch){
				insert.p = mirrorPatch;
			}
			else{
				int uDim = p->dim.x;
				//insert.alternativeCentreIdx = p->getMirrorTwinlessLineFormat1(insert.alternativeCentreIdx, uDim, doU);
				//insert.alternativeEdgeIdx = p->getMirrorTwinlessLineFormat1(insert.alternativeEdgeIdx, uDim, doU);
				insert.alternativeCentreIdx = p->getMirrorTwinlessIdx(insert.alternativeCentreIdx);
				insert.uOrVIdx = p->getMirrorTwinlessLineFormat1(insert.uOrVIdx, uDim, doU);
				if(insert.doU == false){
					insert.leaveNorthOrEast = !insert.leaveNorthOrEast;
					//insert.cutLine.x = p->getMirrorTwinlessIdx(insert.cutLine.x);
					insert.cutLine = Vec2i(p->getMirrorTwinlessIdx(insert.cutLine.x), p->getMirrorTwinlessIdx(insert.cutLine.y));
					if(false == beginsFromCenterLineFunc(k, insert)){
						insert.cutLine.x--;
					}
					//if(false == insert.crossedMiddle){
					if(false == endsOnCenterLineFunc(k, m_insert.length, insert)){
						insert.cutLine.y--;
					}
					
				}
				else{
					insert.ratio = 1.0 - insert.ratio;
					//insert.invertRatio = !insert.invertRatio;
					//insert.prevUorVCenterIdx
				}
			}
			if(NurbsPatch* otherPatch = insert.otherPatch){
				if(NurbsPatch* mirrorPatch = otherPatch->mirrorPatch){
					insert.otherPatch = mirrorPatch;
				}
				else{
					int x = otherPatch->getMirrorTwinlessIdx(insert.otherUV.x) - 1;
					//if(x < insert.otherUV.x){
					//	x++;
					//}
					//else{
					//	x--;
					//}
					insert.otherUV.x = x;
				}
			}
		}
	}
}

bool isIntersecting(const int j_uOrVIdx, const Vec2i j_cutLine, const bool j_leaveNorthOrEast, const int k_uOrVIdx, const Vec2i k_cutLine, const bool k_leaveNorthOrEast){
	if(	(k_leaveNorthOrEast == true  && j_uOrVIdx >= k_cutLine.x && j_uOrVIdx <= k_cutLine.y) ||
		(k_leaveNorthOrEast == false && j_uOrVIdx <= k_cutLine.x && j_uOrVIdx >= k_cutLine.y))
	{
		if(	(j_leaveNorthOrEast == true  && k_uOrVIdx >= j_cutLine.x && k_uOrVIdx <= j_cutLine.y) ||
			(j_leaveNorthOrEast == false && k_uOrVIdx <= j_cutLine.x && k_uOrVIdx >= j_cutLine.y))
		{
			return true;
		}
	}
	return false;
}

bool uIsOnEdge(Insert& insert, int cut){
	int vDim = insert.p->dim.y;
	if(cut == 0){
		return true;
	}
	if(evenEdge(cut) == vDim - 1){
		return true;
	}
	return false;
}

bool vIsOnEdge(Insert& insert, int cut){
	int uDim = insert.p->dim.x;
	if(cut == 0){
		return true;
	}
	if(evenEdge(cut) == uDim - 1){
		return true;
	}
	return false;
}


bool checkForLoop(M_Insert& m_insert, Insert& k_insert, const int k){
	/*
	look for where j element intersect k element
	if interesction is found:
	if j element ends on the edge of a patch discard it
	if k element begins on the edge of a patch discard it
	discard any elements before j element
	discard any elements after k element
	*/
	//m_insert.deleteElement(0);
	//m_insert.length--;
	//return true;
	
	g_insertPatch.loop = false;

//	return false;


	for(int j = 0; j < k - 1; j++){
		Insert& j_insert = m_insert[j];
		if(j_insert.p == k_insert.p){
			int uDim = j_insert.p->dim.x;
			int vDim = j_insert.p->dim.y;

			if(k_insert.doU && j_insert.doU == false){
				if(isIntersecting(j_insert.uOrVIdx, j_insert.cutLine, j_insert.leaveNorthOrEast, k_insert.uOrVIdx, k_insert.cutLine, k_insert.leaveNorthOrEast)){

					if(j_insert.leaveNorthOrEast){
						if(evenEdge(j_insert.cutLine.y) == uDim - 1 && evenEdge(k_insert.uOrVIdx) == uDim - 1){
							j_insert.inLoop = false;
						}
					}
					else{
						if(j_insert.cutLine.y == 0 && k_insert.uOrVIdx == 0){
							j_insert.inLoop = false;
						}
					}
					if(k_insert.leaveNorthOrEast){
						if(k_insert.cutLine.x == 0 && j_insert.uOrVIdx == 0){
							k_insert.inLoop = false;
						}
					}
					else{
						if(evenEdge(k_insert.cutLine.x) == vDim - 1 && evenEdge(j_insert.uOrVIdx) == vDim - 1){
							k_insert.inLoop = false;
						}
					}
					for(int j2 = 0; j2 < j; j2++){
						m_insert[j2].inLoop = false;
					}
					for(int k2 = k + 1; k2 < m_insert.length; k2++){
						m_insert[k2].inLoop = false;
					}

					if(m_insert[k].inLoop){
						//k_insert.cutLine.y = j_insert.uOrVIdx;
						//if(vCutPatch(m_insert[j].uOrVIdx, m_insert, k, m_mirrorPatch, cutWasMade)){

						int j_uOrVIdx0 = m_insert[j].uOrVIdx;
						int j_uOrVIdx1 = j_uOrVIdx0;

						if(false == uIsOnEdge(m_insert[k], evenEdge(j_uOrVIdx0))){
							if(m_insert[k].leaveNorthOrEast){
								if(j_uOrVIdx0 & 1){
									j_uOrVIdx1++;
								}
								else{
									j_uOrVIdx1--;
								}
							}
							else{
								if(j_uOrVIdx0 & 1){
									j_uOrVIdx0++;
								}
								else{
									j_uOrVIdx1--;
								}
							}
						
							m_insert.insertElement(k, m_insert[k]);
							m_insert[k + 1].inLoop = false;
							m_insert[k + 1].diversion = false;
							m_insert[k].cutLine.y = j_uOrVIdx0;
							m_insert[k + 1].cutLine.x = j_uOrVIdx1;
						}
					}
					if(m_insert[j].inLoop){
						//j_insert.cutLine.x = k_insert.uOrVIdx;
						//if(uCutPatch(m_insert[k].uOrVIdx, m_insert, j, m_mirrorPatch, cutWasMade)){

						int k_uOrVIdx0 = m_insert[k].uOrVIdx;
						int k_uOrVIdx1 = k_uOrVIdx0;

						if(false == vIsOnEdge(m_insert[j], evenEdge(k_uOrVIdx0))){
							if(m_insert[j].leaveNorthOrEast){
								if(k_uOrVIdx0 & 1){
									k_uOrVIdx1++;
								}
								else{
									k_uOrVIdx0--;
								}
							}
							else{
								if(k_uOrVIdx0 & 1){
									k_uOrVIdx0--;
								}
								else{
									k_uOrVIdx0--;
								}
							}
						
							m_insert.insertElement(j, m_insert[j]);
							m_insert[j].inLoop = false;
							m_insert[j].diversion = false;

							m_insert[j].cutLine.y = k_uOrVIdx0;
							m_insert[j + 1].cutLine.x = k_uOrVIdx1;
						}
					}

					g_insertPatch.loop = true;
					return true;
				}
			}
			else if(k_insert.doU == false && j_insert.doU){
				if(isIntersecting(j_insert.uOrVIdx, j_insert.cutLine, j_insert.leaveNorthOrEast, k_insert.uOrVIdx, k_insert.cutLine, k_insert.leaveNorthOrEast)){

					if(j_insert.leaveNorthOrEast){
						if(evenEdge(j_insert.cutLine.y) == vDim - 1 && evenEdge(k_insert.uOrVIdx) == vDim - 1){
							j_insert.inLoop = false;
						}
					}
					else{
						if(j_insert.cutLine.y == 0 && k_insert.uOrVIdx == 0){
							j_insert.inLoop = false;
						}
					}
					if(k_insert.leaveNorthOrEast){
						if(k_insert.cutLine.x == 0 && j_insert.uOrVIdx == 0){
							k_insert.inLoop = false;
						}
					}
					else{
						if(evenEdge(k_insert.cutLine.x) == uDim - 1 && evenEdge(j_insert.uOrVIdx) == uDim - 1){
							k_insert.inLoop = false;
						}
					}
					for(int j2 = 0; j2 < j; j2++){
						m_insert[j2].inLoop = false;
					}
					for(int k2 = k + 1; k2 < m_insert.length; k2++){
						m_insert[k2].inLoop = false;
					}
					if(k_insert.inLoop){
						//k_insert.cutLine.y = evenEdge(j_insert.uOrVIdx);

						int j_uOrVIdx0 = m_insert[j].uOrVIdx;
						int j_uOrVIdx1 = j_uOrVIdx0;

						if(false == vIsOnEdge(m_insert[k], evenEdge(j_uOrVIdx0))){
							if(j_uOrVIdx0 & 1){
								j_uOrVIdx1++;
							}
							else{
								if(j_uOrVIdx0 & 1){
									j_uOrVIdx1--;
								}
								else{
									j_uOrVIdx0--;
								}
							}
						
							m_insert.insertElement(k, m_insert[k]);
							m_insert[k + 1].inLoop = false;
							m_insert[k + 1].diversion = false;
							m_insert[k].cutLine.y = j_uOrVIdx0;
							m_insert[k + 1].cutLine.x = j_uOrVIdx1;
						}
					}

					if(j_insert.inLoop){
						//j_insert.cutLine.x = evenEdge(k_insert.uOrVIdx);

						int k_uOrVIdx0 = m_insert[k].uOrVIdx;
						int k_uOrVIdx1 = k_uOrVIdx0;

						if(false == uIsOnEdge(m_insert[j], evenEdge(k_uOrVIdx0))){
							if(m_insert[k].leaveNorthOrEast){
								if(k_uOrVIdx0 & 1){
									k_uOrVIdx1++;
								}
								else{
									k_uOrVIdx0--;
								}
							}
							else{
								if(k_uOrVIdx0 & 1){
									k_uOrVIdx1++;
								}
								else{
									k_uOrVIdx0--;
								}
							}
						
							m_insert.insertElement(j, m_insert[j]);
							m_insert[j].inLoop = false;
							m_insert[j].diversion = false;
							m_insert[j].cutLine.y = k_uOrVIdx0;
							m_insert[j + 1].cutLine.x = k_uOrVIdx1;
						}
					}

					g_insertPatch.loop = true;
					return true;
				}

			}
		}
	}
	return false;
}

void printOutInsertions(const M_Insert& m_insert){
	String str;
	for(int k = 0; k < m_insert.length; k++){ 
		const Insert& insert = m_insert[k];
		str = "k ";
		str += k;
		dLog(str);

		str = "plidx ";
		str += insert.p->plidx;
		dLog(str);

		str = "doU ";
		str += insert.doU;
		dLog(str);

		str = "uOrVIdx ";
		str += insert.uOrVIdx;
		dLog(str);

		str = "cutLine ";
		str += insert.cutLine;
		dLog(str);

		str = "diversion ";
		str += insert.diversion;
		dLog(str);

		str = "inLoop ";
		str += insert.inLoop;
		dLog(str);

		str = "otherPatch ";
		if(insert.otherPatch){
			str += insert.otherPatch->plidx;
		}
		else{
			str += "NULL";
		}
		dLog(str);

		str = "otherUV ";
		str += insert.otherUV;
		dLog(str);

		dLog("");
	}
}

void checkForLoop(){
	M_Insert& m_insert = g_insertPatch.m_insert;

	for(int i = m_insert.length;--i>=0;){
		m_insert[i].inLoop = true;
	}
	int k = m_insert.length;
	while(true){
		Insert& k_insert = m_insert[--k];

		if(checkForLoop(m_insert, k_insert, k)){
			break;
		}

		if(k_insert.diversion == true || k == 0){
			break;
		}
	};
//	printOutInsertions(m_insert);
}


void InsertPatch::doLastPhase(){
//	reverseDirection(m_insert);
	if(m_insert.length == 0){
		return;
	}
	if(g_insertPatch.cutMode && g_cutComplete->_continue == false){
		verifyFirst();
		Insert insert = m_insert[init_element];
		m_insert = insert;
		m_insertOD.length = 0;
		/*
		m_insert.length = 0;
		Insert insert;
		insert.p = init_patch;
		insert.doU = init_doU;
		int uDim = init_patch->dim.x;
		if(init_doU){
			insert.uOrVIdx = convertToLineFormat1(init_idx, init_idx + uDim, uDim, init_doU);
		}
		else{
			insert.uOrVIdx = convertToLineFormat1(init_idx, init_idx + 1, uDim, init_doU);
		}
		insert.leaveNorthOrEast = init_leaveNorthOrEast;
		insert.ratio = init_ratio;
		m_insert += insert;
		m_insertOD.length = 0;
		*/
	}


	//if(m_insertOD.length > 1){
	//	sortDirectionsOut(m_insert[0].leaveNorthOrEast, m_insert[0].uOrVIdx, m_insert[0].p->dim.y);
	//}

	NurbsPatchPtr l_shaft = 0;

	//if(m_insertOD.length > 1){
	//	m_insertOD.deleteElement(0);
	//	m_insert[0].leaveNorthOrEast = !m_insert[0].leaveNorthOrEast;
	//	m_insert += m_insertOD;
	//}


	if(remLeftSide == false){//do we need this still?
		swapEastWest(m_insert);
	}

	M_Int lookup;
	M_MirrorPatch m_mirrorPatch;
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		if(g_insertPatch.imousePhase){ //why do we ask this if we got here?
			imousePhase = NEUTRAL; //prevent updating line visual clue after patches have been cut up
			//checkForLoop();
			//printOutInsertions(m_insert);
			smls->convertToCpFormat();

			if(cutMode){
				cutPatches(smls, m_insert, m_mirrorPatch);
				if(g_cutComplete->unstitch){
					g_cutComplete->unstitchPatches(m_insert);
					smls->generateRawStitchPolys();
					smls->stitchAllPolys();
				}
			}
			else{
				cutUpPatches(smls, m_insert, m_mirrorPatch);

				commonInsertStrip(smls, m_insert, m_mirrorPatch);
			}

			//l_shaft = dynamic_cast<NurbsPatch*>(smls->build.refValue()[smls->build.length - 1].p);

			smls->centralCPsToPatches();
			smls->calcAllPolyMidPoints();

			m_insert.length = 0;
			m_insertOD.length = 0;
			imousePhase = NEUTRAL;


			smls->deleteAllSyncParts();
			smls->generateAllSyncPatchesBnspe();
			if(smls->autoJoin && false == (cutMode && g_cutComplete->unstitch)){
				smls->joinAllParts();
			}

		}
		smls->doBuild();
	//smls->generateRawStitchPolys();/////why??
	//smls->stitchAllPolys();/////why??

		patch->houseKeep();
	}
}

void InsertPatch::verifyFirst(){
	for(int k = 0; k < m_insert.length; k++){
		if(m_insert[k].first){
			if(k != init_element){
				init_element = k;
				if(m_insert[init_element].p != init_patch){
					throw;
				}
			}
			break;
		}
	}
}

void setCpIndexIndependence(NurbsPatch* patchDst, int idxDst, M_Vec3f& cp, M_Float& cw){
	M_Int& cpIndexDst = patchDst->cpIndex.refValue();

	cpIndexDst[idxDst] = cp.length;
	cp += patchDst->point[idxDst];
	cw += patchDst->weight[idxDst];

	if(patchDst->mirror){
		if(patchDst->mirrorPatch){
			patchDst = patchDst->mirrorPatch;
		}
		else{
			idxDst = patchDst->getMirrorTwinlessIdx(idxDst);
		}

		M_Int& cpIndexDst = patchDst->cpIndex.refValue();

		cpIndexDst[idxDst] = cp.length;
		cp += patchDst->point[idxDst];
		cw += patchDst->weight[idxDst];
	}
}

void NurbsPatch::unstitchEdge(Insert& insert, M_Insert m_insert, const int isrtIdx, NurbsPatch*& prevPatch, int& prevIndex, const int offset, const int incAmount, const bool doLoop, const NurbsPatch* loopPatch, const int loopIndex){
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Vec3f& cp = smls->cp;
	M_Float& cw = smls->cw;
	int uDim = dim.x;
	int vDim = dim.y;


	int first = insert.cutLine.x;
	if(false == beginsFromCenterLineFunc(isrtIdx, insert)){
		first = evenEdge(first);
	}

	int k = first;

	int last = insert.cutLine.y;
	if(false == endsOnCenterLineFunc(isrtIdx, m_insert.length, insert)){
		last = evenEdge(last);
	}

	int count = last - first;
	int inc = 1;
	if(insert.leaveNorthOrEast == false){
		count = -count;
		inc = -1;
	}
	count++;

	for(int i = 0; i < count; i++){
		
		if(prevPatch && k == first){
			setCpIndex2(this, offset + k * incAmount, prevPatch, prevIndex);

			if(insert.otherPatch){
				int otherIdx = convertToActualFormat(insert.otherUV, insert.otherPatch->dim.x);
				setCpIndex2(insert.otherPatch, otherIdx, prevPatch, prevIndex);

				for(int j = insert.m_extra.length; --j >= 0;){
					otherIdx = convertToActualFormat(insert.m_extra[j].uv, insert.m_extra[j].p->dim.x);
					setCpIndex2(insert.m_extra[j].p, otherIdx, prevPatch, prevIndex);
				}
			}
		}
		else if(doLoop && i == count - 1){
			setCpIndex2(this, offset + k * incAmount, loopPatch, loopIndex);

			if(insert.otherPatch){
				int k = 0;
				if(getFirstInLoop(k, m_insert) == false){
					return;
				}


				int otherIdx = 0;
				int insrtIdx = 0;
				NurbsPatch* otherPatch = getOtherEndOfLoop(otherIdx, m_insert, insrtIdx);

				setCpIndex2(otherPatch, otherIdx, loopPatch, loopIndex);

				Insert& otherInsert = m_insert[insrtIdx];

				for(int j = otherInsert.m_extra.length; --j >= 0;){
					otherIdx = convertToActualFormat(otherInsert.m_extra[j].uv, otherInsert.m_extra[j].p->dim.x);
					setCpIndex2(otherInsert.m_extra[j].p, otherIdx, loopPatch, loopIndex);
				}
			}
		}
		else{
			prevIndex = offset + k * incAmount;
			prevPatch = this;
			setCpIndexIndependence(this, prevIndex, cp, cw);

			//if(i == count - 1 && g_insertPatch.foundFirstStitchPoly && isrtIdx == m_insert.length - 1){
			if(i == count - 1 && isrtIdx == m_insert.length - 1){
				if(mirrorPatch){
					M_Int& r_cpIndex = cpIndex;
					M_Int& l_cpIndex = mirrorPatch->cpIndex;
					
					setCpIndex2(this, prevIndex, mirrorPatch, prevIndex);
				}
				else if(g_insertPatch.foundFirstStitchPoly && m_insert.length == 1){
					setCpIndex2(this, offset + (uDim - 1) * incAmount, this, offset);
				}

				/*
				if(joinEnds){
					M_Int& r_cpIndex = cpIndex;
					M_Int& l_cpIndex = mirrorPatch->cpIndex.refValue();

					r_cpIndex[endIdx + uDim * 0] = l_cpIndex[endIdx + uDim * 0];
					r_cpIndex[endIdx + uDim * 1] = l_cpIndex[endIdx + uDim * 1];
					r_cpIndex[endIdx + uDim * 2] = l_cpIndex[endIdx + uDim * 2];
				}
				*/

			}
		}
		k += inc;
	}
}


void CutComplete::unstitchPatches(M_Insert& m_insert){
	bool usePrevIndex = false;
	NurbsPatch* prevPatch = 0;
	int prevIndex = 0;
	
	Seamless* smls = getSeamless2();
	int loopIndex = 0;
	NurbsPatch* loopPatch = 0;
	int lastLoopIndex = 0;

	if(g_insertPatch.loop){
		for(; lastLoopIndex < m_insert.length; lastLoopIndex++){
			if(m_insert[lastLoopIndex].inLoop){
				break;
			}
		}
		Insert& loopInsert = m_insert[lastLoopIndex];
		loopPatch = loopInsert.p;
		loopIndex = convertToActualFormat(loopInsert.uOrVIdx, loopInsert.cutLine.x, loopPatch->dim.x, loopInsert.doU);

		for(; lastLoopIndex < m_insert.length; lastLoopIndex++){
			if(m_insert[lastLoopIndex].inLoop == false){
				lastLoopIndex--;
				break;
			}
		}
		if(lastLoopIndex == m_insert.length){
			lastLoopIndex--;
		}
	}


	for(int isrtIdx = 0; isrtIdx < m_insert.length; isrtIdx++){
		Insert& insert = m_insert[isrtIdx];
		if(insert.inLoop == false){
			continue;
		}
		NurbsPatch* patch = insert.p;
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		if(insert.m_extra.length){
			int a = 0;
		}

		bool doLoop = (loopPatch && isrtIdx == lastLoopIndex);

		if(insert.doU){
			if(insert.uOrVIdx == 0){
				patch->unstitchEdge(insert, m_insert, isrtIdx, prevPatch, prevIndex, 0, uDim, doLoop, loopPatch, loopIndex);//west
			}
			else if(insert.uOrVIdx == uDim - 2){
				patch->unstitchEdge(insert, m_insert, isrtIdx, prevPatch, prevIndex, uDim - 1, uDim, doLoop, loopPatch, loopIndex);//east
			}
		}
		else{
			if(insert.uOrVIdx == 0){
				patch->unstitchEdge(insert, m_insert, isrtIdx, prevPatch, prevIndex, 0, 1, doLoop, loopPatch, loopIndex);//south
			}
			else if(insert.uOrVIdx == vDim - 2){
				patch->unstitchEdge(insert, m_insert, isrtIdx, prevPatch, prevIndex, uDim * (vDim - 1), 1, doLoop, loopPatch, loopIndex);//north
			}
		}
		usePrevIndex = true;
	}
}
