/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/
#include "seamless3d.h"
extern bool g_perspective;

Vec2f convertPointToScreen2d(Vec3f point);
Vec2f convertPointToScreen2dNoMatrix(Vec3f sr);
extern ShapePtr g_vcSelectRectShape;

bool MultiSelect::selectRectMode = false;
Vec3f MultiSelect::topLeft;
Vec3f MultiSelect::bottomRight;
int MultiSelect::uRingDim = 0;
int MultiSelect::vRingDim = 0;

Vec2f MultiSelect::topLeft2d;
Vec2f MultiSelect::bottomRight2d;
M_Vec3f MultiSelect::beginDragPos;
Vec3f MultiSelect::autoPivotPoint;
int MultiSelect::clickedIndex = 0;
bool MultiSelect::withinSights = false;
float MultiSelect::distance = 1000000;
bool MultiSelect::withinSightsIsSelected = false;
int MultiSelect::withinSightsIndex = 0;
NodePtr MultiSelect::withinSightsNode;

M_<S_VertexIndex> MultiSelect::triList;
M_<S_VertexIndex> MultiSelect::coordList;
M_ControlPoint MultiSelect::pointList;
M_Node MultiSelect::objectList;
M_Int MultiSelect::ctrlPolySelList;
M_ControlPoint MultiSelect::boxPolySelList;

int MultiSelect::selectedPoint = 0;
bool MultiSelect::accumulate = false;


void MultiSelect::resetDistance(){
	withinSights = false;
	distance = 1000000;
}

void MultiSelect::reset(){
	clickedIndex = 0;
	selectRectMode=false;

	if(MultiSelect::accumulate == false){
		if(g_paletteMode == false){
			triList.length = 0;
		}
		coordList.length = 0;
		pointList.length = 0;
		ctrlPolySelList.length = 0;
	}
	//g_vcArtTriBase->transpSmls->skeleton.refValue()[0]->deleteGeometry();
	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcSelectRectShape->geometry.p);
	dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue().length=0;
	polyLine->coordIndex.refValue().length=0;
	beginDragPos.length = 0;
}

void MultiSelect::checkToReset(Art* part){
	Seamless* smls = part->smls;
	int len = coordList.length;
	for(int i = 0; i<len; i++){
		Art* prt = coordList[i].p;
		int idx = coordList[i].i;
		if(part == prt && smls->ActCtrlPnt == idx){
			return;
		}
	}
	reset();
}

void MultiSelect::processRectangle(){
	if(topLeft.x > bottomRight.x){
		float store = topLeft.x;
		topLeft.x = bottomRight.x;
		bottomRight.x = store;
	}
	if(topLeft.y < bottomRight.y){
		float store = topLeft.y;
		topLeft.y = bottomRight.y;
		bottomRight.y = store;
	}
	if(g_perspective){

		Vec2f vec = convertPointToScreen2dNoMatrix(MultiSelect::topLeft);
		MultiSelect::topLeft.x = vec.x;
		MultiSelect::topLeft.y = vec.y;
		vec = convertPointToScreen2dNoMatrix(MultiSelect::bottomRight);
		MultiSelect::bottomRight.x = vec.x;
		MultiSelect::bottomRight.y = vec.y;
	}
}

bool MultiSelect::isInBox(Vec3f point){
	Vec2f v = convertPointToScreen2d(point);
	if(v.x > topLeft.x && v.x < bottomRight.x && v.y < topLeft.y && v.y > bottomRight.y){
		return true;
	}
	return false;
}
bool MultiSelect::isInBox(Vec2f v){
	if(v.x > topLeft.x && v.x < bottomRight.x && v.y < topLeft.y && v.y > bottomRight.y){
		return true;
	}
	return false;
}
