/* Seamless3d

Copyright (C) 2001-2006 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/


// Our custom FVF, which describes our custom vertex structure
#define D3DFVF_VERTEX (D3DFVF_XYZ)
// A structure for our custom vertex type
struct VERTEX{
    D3DXVECTOR3 position; // The 3D position for the vertex
};

// Our custom FVF, which describes our custom vertex structure
#define D3DFVF_SIMPLEVERTEX (D3DFVF_XYZ | D3DFVF_NORMAL )
// A structure for our custom vertex type. We added a normal, and omitted the
// color (which is provided by the material)
struct SIMPLEVERTEX{
    D3DXVECTOR3 position; // The 3D position for the vertex
    D3DXVECTOR3 normal;   // The surface normal for the vertex
};


#define D3DFVF_COLOR_VERTEX_NO_NORMAL (D3DFVF_XYZ | D3DFVF_DIFFUSE )

struct COLOR_VERTEX_NO_NORMAL{
    D3DXVECTOR3 position; // The 3D position for the vertex
	DWORD color;
};

// Our custom FVF, which describes our custom vertex structure
#define D3DFVF_COLOR_VERTEX (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_DIFFUSE )

struct COLOR_VERTEX{
    D3DXVECTOR3 position; // The 3D position for the vertex
    D3DXVECTOR3 normal;   // The surface normal for the vertex
	DWORD color;
};

struct TEXTUREVERTEX{
    D3DXVECTOR3 position; // The position
    D3DXVECTOR3 normal;   // The surface normal for the vertex
    FLOAT       tu, tv;   // The texture coordinates
};

struct TEXTUREVERTEX_0WEIGHT{
    D3DXVECTOR3 position; // The position
	DWORD       matrixIndices; // Referenced as v2.xyzw in the vertex shader
    D3DXVECTOR3 normal;   // The surface normal for the vertex
    FLOAT       tu, tv;   // The texture coordinates
};

struct COLORVERTEX_0WEIGHT{
    D3DXVECTOR3 position; // The position
	DWORD       matrixIndices; // Referenced as v2.xyzw in the vertex shader
    D3DXVECTOR3 normal;   // The surface normal for the vertex
	DWORD color;
};

struct COLORVERTEX_1WEIGHT{
    D3DXVECTOR3 position; // The 3D position for the vertex

    FLOAT       blend0;  // Referenced as v1.x in the vertex shader
    DWORD       matrixIndices; // Referenced as v2.xyzw in the vertex shader 	DWORD matrixIndices;
    D3DXVECTOR3 normal;   // The surface normal for the vertex
	DWORD color;
};

struct TEXTUREVERTEX_1WEIGHT{
    D3DXVECTOR3 position; // The 3D position for the vertex

    FLOAT       blend0;  // Referenced as v1.x in the vertex shader
    DWORD       matrixIndices; // Referenced as v2.xyzw in the vertex shader 	DWORD matrixIndices;
    D3DXVECTOR3 normal;   // The surface normal for the vertex
    FLOAT       tu, tv;   // The texture coordinates
};



// Our custom FVF, which describes our custom vertex structure
#define TEXTURE_CUSTOMVERTEX (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_TEX1)

#define TEXTURE_CUSTOMVERTEX_0WEIGHT (D3DFVF_XYZB1 | D3DFVF_LASTBETA_UBYTE4|D3DFVF_NORMAL|D3DFVF_TEX1)
#define TEXTURE_CUSTOMVERTEX_1WEIGHT (D3DFVF_XYZB2 | D3DFVF_LASTBETA_UBYTE4|D3DFVF_NORMAL|D3DFVF_TEX1)
#define COLOR_CUSTOMVERTEX_0WEIGHT (D3DFVF_XYZB1 | D3DFVF_LASTBETA_UBYTE4|D3DFVF_NORMAL|D3DFVF_DIFFUSE)
#define COLOR_CUSTOMVERTEX_1WEIGHT (D3DFVF_XYZB2 | D3DFVF_LASTBETA_UBYTE4|D3DFVF_NORMAL|D3DFVF_DIFFUSE)

class Render {
public:
	int type;
	int indexCount;
	int vertexCount;
#ifdef DX8
	LPDIRECT3DVERTEXBUFFER8 pVB; // Buffer to hold vertices
	LPDIRECT3DINDEXBUFFER8  pIB;
#endif
#ifdef DX9
	LPDIRECT3DVERTEXBUFFER9 pVB; // Buffer to hold vertices
	LPDIRECT3DINDEXBUFFER9  pIB;
#endif
	Render():
		pVB(0),
		pIB(0),
		vertexCount(0),
		indexCount(0),
		type(0)
	{
	}
void convertFromVrml(Shape& shape);
};
void renderParts(SFNode* node ,D3DXMATRIX parentMatrix, Vec3f parentColScale);
bool InitRender();

void Cleanup();
void cleanupForRefreshSize();
