/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include <windows.h>
#include <time.h>
#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "gui.h"
#include "ms-gui.h"
#include "scene_tree.h"

//#define STRETCH_PATCH_HARMONICS
//#define RANDOM_HAMONIC_AMPLITUDE

extern const BIProto* builtInProtoList[];
extern const char* msg_Can_not_create_output_file_name;
extern SettingsPtr g_settings;
extern ScenePtr g_scene;
void postLoadProcessNodes(M_<SceneTreeListItem>& list);
void setTitleName();
extern bool g_forceSaveAs;
void insertRecentFile(const char* fullFileName);
bool outOfSyncWithRobot();

Ptr<WaveToPolyControlPanel> g_wtpcp;

const float HarmonicPatch::initHarmonicPatch_point[48]={0, 1.834999, -2, .973333, 1.269999, -2, 1.686666, 1.639999, -2, 2, 0, -2.4, 0, 1.072512, -2, .582312, .912682, -2, 1.17276, 1.390645, -1.780084, 1.395, 0, -1.82, 0, .730221, -.767018, .392107, .646949, -.766165, .761666, .943333, -.765, .745, 0, -.735, 0, .015, 0, .048333, .14, 0, .107436, .117438, 0, .165001, 0, 0};
const float HarmonicPatch::initHarmonicPatch_vCoord[41]={0, .2, .36, .488, .5904, .67232, .737856, .790285, .832228, .865782, .892626, .914101, .931281, .945024, .95602, .964816, .971853, .977482, .981986, .985588, .988471, .990777, .992621, .994097, .995278, .996222, .996978, .997582, .998066, .998453, .998762, .99901, .999208, .999366, .999493, .999594, .999676, .99974, .999792, .999834, .999867};
const float HarmonicPatch::initHarmonicPatch_multiply[41]={1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41};


unsigned char formatM16[0x2C] = {0X52,0X49,0X46,0X46,0X00,0X00,0X00,0X00,0X57,0X41,0X56,0X45,0X66,0X6D,0X74,0X20,0X10,0X00,0X00,0X00,0X01,0X00,0X01,0X00,0X44,0XAC,0X00,0X00,0X88,0X58,0X01,0X00,0X02,0X00,0X10,0X00,0X64,0X61,0X74,0X61,0X00,0X00,0X00,0X00};
#define WAVE_HEADER_SIZE 44

Vec2f curvePoint2d(int dim, int degree, const float *knot, const Vec2f *pnt, const float *w, const float t, const float tc);
Vec2f curvePoint2d(int dim, int degree, const float *knot, const Vec2f *pnt, const float *w, const float t);

void saveToWav(short int* iBuf, int workBufLen, int numOfChannels, int sampleRate, const String& filename);

void WaveToPolyControlPanel::setWinSize(Vec2i& raw){
	pixBuf.length = raw.x * raw.y;
	if(false == initWindow()){
		throw;
	}
}

/*
void WaveToPoly::fadeInOutAtMarkers(){

	M_Int& cycle = this->cycle.refValue();



	for(int k = 0; k < cycle.length - 1; k++){
		
		int begin = cycle[k];
		int end = cycle[k + 1];
		int fadeLength = end - begin;

		for(int i = 0; i < fadeLength; i++){
			double e = PI / double(fadeLength) * double(i);
			double env = 1.0 - ((cos(e) + 1.0) * .5);
			srcBuf[begin + i] = srcBuf[begin + i] * env;
			srcBuf[end - i] = srcBuf[end - i] * env;
		}
	}
}
*/

void fadeInFadeOutWindow(M_Float& window){
	int windowLen = window.length;
	int fadeLength = window.length;

	int i = 0;

	for(; i < fadeLength; i++){
		double e = PI * 2 / double(fadeLength) * double(i);
		double env = 1.0 - ((cos(e) + 1.0) * .5);
		window[i] = window[i] * env;
	}
	//pad any samples after envelope
	for(; i < windowLen; i++){
		window[i] = 0;
	}
}

void WaveToPoly::fadeInOutAtMarkers(){

	M_Int& cycle = this->cycle.refValue();

	int fadeLength = 2048 * 2;


	for(int k = 0; k < cycle.length - 1; k++){
		
		int begin = cycle[k];
		int end = cycle[k + 1];
		int cycleLength = end - begin;
		int i = 0;

		for(; i < fadeLength; i++){
			double e = PI * 2 / double(fadeLength) * double(i);
			double env = 1.0 - ((cos(e) + 1.0) * .5);
			srcBuf[begin + i] = srcBuf[begin + i] * env;
		}
		for(; i < cycleLength; i++){
			srcBuf[begin + i] = 0;
		}
	}
}

void WaveToPoly::divideMarkers(){
	M_Int& cycle = this->cycle.refValue();

	int len = cycle.length;

	for(int k = 0; k + 1 < len; k++){
		cycle += cycle[k] + ((cycle[k] - cycle[k + 1]) >> 1);
	}
	bubbleSort(cycle);
	g_wtpcp->paint();
}

void WaveToPoly::addWindowMarkers(){

	int halfWindowSize = windowSize >> 1;

	M_Int& cycle = this->cycle.refValue();
	cycle.length = 0;

	int m = 0;

	while(m < srcBufLen){
		cycle += m;
		m += halfWindowSize;
	};

	//fadeInOutAtMarkers();

	g_wtpcp->paint();
}

void WaveToPoly::keepCyclesInRangeOfBuffer(){
	M_Int& cycle = this->cycle.refValue();
	int& cMark = this->cMark.refValue();

	for(int k = cycle.length; --k >= 0;){
		if(cycle[k] >= srcBufLen){
			cycle.deleteElement(k);
		}
	}
	if(cMark >= cycle.length){
		cMark = cycle.length - 1;
	}
}

int loadWavFile(const String& srcFilename, short int*& buf, WavHeader& wavHeader, int sampleSize, bool convertToFloat); //return bufLen

void WaveToPoly::processFields(){

	bubbleSort(cycle.refValue());

	WavHeader wavHeader;

	short int* shortSrcPtr = 0;

	if(srcBufLen = loadWavFile(file, shortSrcPtr, wavHeader, 2, true)){

		srcBuf = reinterpret_cast<float*>(shortSrcPtr);
		
		sampleRate = wavHeader.sampleRate;

		keepCyclesInRangeOfBuffer();
	}
}

void WaveToPolyControlPanel::paint(){
	if(node==0)return;

	int dstWidth=size.x;
	int dstHeight=size.y;
	int* dstPix=pixBuf.getArray();
	const M_Properties& properties=node->getProperties();


	//colorBlockColorRgn(dstPix);
	blitHorizontalColorSweep(Vec2i(10, 0), Vec2i(470, 0), Vec2i(471, 0), Vec2i(dstWidth-9-10, title_bar_height));
	pasteSprite2( 0, 0, 461, 0, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 10, 25, dstPix);
	pasteSprite2( dstWidth-9, 0, 471, 0, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 9, 25, dstPix);


	static String str;
	int i = 0;
	int j = 0;
	int currentGroup=-1;

	int g=str.length * 8 + 15;
	int type=node->getType();
	int iconNodeType = type;
	if(type>=g_userProtoBegin){
		iconNodeType = PROTOINSTANCE;
	}
	int iconExtraX = (builtInProtoList[iconNodeType]->iconSize.x-16)>>1;
	//pasteSprite2( 5-iconExtraX, 5-((nodeIconLookUp[iconNodeType].size.y-16)>>1), nodeIconLookUp[iconNodeType].srcPos.x, nodeIconLookUp[iconNodeType].srcPos.y, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, nodeIconLookUp[iconNodeType].size.x, nodeIconLookUp[iconNodeType].size.y, dstPix);


	str = filenameOnly;
	str+=".wav";
	drawText2(str, 0x0, Vec2i(600, 16), dstWidth, dstHeight, dstPix);

	exitWindow.dstPos=Vec2i(size.x - 22, 4);
	exitWindow.paint();

	for(int i=buttonList.length; --i >= 0;){
		FieldButton* button = buttonList[i];

		if(i == 1){
			button->paintFieldName();
		}
		
		button->paint();
		Vec2i valPos=button->valPos;
		valPos.y+=16;
		if(i == 1){
			if(button->editMode){
				drawText2(textCell.cellStr, textCell.firstVisChar, textCell.cellStr.length, 0, valPos, size.x, size.y, dstPix);
				textCell.highLightBlock();
			}
			else{
				button->toString(Button::str="");
				drawText2(Button::str, 0, valPos, size.x, size.y, dstPix);
			}
		}
	}
	paintBlock2(0xffffff, 0, 25, dstWidth, dstHeight, dstWidth, dstHeight, dstPix);

	int viewFrom = waveToPoly->viewFrom;
	float* srcBuf = waveToPoly->srcBuf;
	int zoom = waveToPoly->zoom;
	int r = 1;

	drawHorizontalLine2(0xc0c0c0, 125, 0, dstWidth, dstWidth, dstHeight, dstPix);
	//line(0xff0000, 0, 2, 2, 0, dstWidth, dstHeight, dstPix);
	if(srcBuf){
		for(int x = 1; x < dstWidth; r++){
			int y = 125;
			//plotPixel2(0x808080, x, y, dstWidth, dstHeight, dstPix);
			line(0, x - zoom, y - srcBuf[r - 1 + viewFrom] * 100, x, y - srcBuf[r + viewFrom] * 100, dstWidth, dstHeight, dstPix);
			drawVerticalLine2(0xff0000, x, y - srcBuf[r + viewFrom] * 100 - 2, y - srcBuf[r + viewFrom] * 100 + 3, dstWidth, dstHeight, dstPix);
			//plotPixel2(0x00ff00, x - 1, y - srcBuf[r + viewFrom] * 100, dstWidth, dstHeight, dstPix);
			x += zoom;
		}
	}

	M_Int& cycle = waveToPoly->cycle.refValue();

	for(int i = 0; i < cycle.length; i++){
		int c = (cycle[i] - viewFrom) * zoom;
		if(c > 0 && c < dstWidth){

			int color = 0x0000ff0;
			if(i == waveToPoly->cMark){
				color = 0xff0000;
			}

			drawVerticalLine2(color, c, 25, dstHeight, dstWidth, dstHeight, dstPix);
		}
	}

	if(g_settings->invertWindowColor){
		invert();
	}
	loadSurfaceFromMemory();
	if(textCell.cellActive==true){
		textCell.doNoThinkWink();
	}
}
void WaveToPolyControlPanel::CMark::action(){
	if(outOfSyncWithRobot()){
		return;
	}
	M_Int& cycle = waveToPoly.cycle.refValue();

	int v = cycle[bField];
	switch(command){
		case 0:
			v--;
			break;
		case 1:
			v++;
			break;
	}
	if(v >= 0){
		if(altKeyState()){
			int diff = v - cycle[bField];
			for(int i = cycle.length; --i >= 0;){
				cycle[i] += diff;
			}
		}
		else{
			cycle[bField] = v;
		}
	}
}

int WaveToPoly::getNextCMark(){
	int v = viewFrom;
	for(int i = 0; i < cycle.length; i++){
		if(cycle[i] > viewFrom){
			return cycle[i];
		}
	}
	return v;
}
int WaveToPoly::getPrevCMark(){
	int v = viewFrom;
	for(int i = cycle.length; --i >= 0;){
		if(cycle[i] < viewFrom){
			return cycle[i];
		}
	}
	return v;
}

void WaveToPolyControlPanel::NextCycle::action(){
	if(outOfSyncWithRobot()){
		return;
	}
	switch(command){
		case 0:
			waveToPoly.viewFrom = waveToPoly.getPrevCMark();
			break;
		case 1:
			waveToPoly.viewFrom = waveToPoly.getNextCMark();
			break;
	}
}


void WaveToPolyControlPanel::setupNodeWindow(Node* node){
	this->node=node;
	waveToPoly = dynamic_cast<WaveToPoly*>(node);

	processFilename();

	const M_Properties& properties = node->getProperties();

	g_controlPanelOpen = true;
	g_propWinLine.length = 0;
	buttonList.length = 0;
	int yPos = title_bar_height + 1;
	int lastYPos = yPos;

	size = Vec2i(800, 200 + 25);
	pos = Vec2i(0, 100);

	openButton=dynamic_cast<SFBoolButton*>(waveToPoly->open.setUpControl(*node, waveToPoly->open, properties[0], 0, 340+48, 0, *this));
	openButton->bSize=Vec2i(24, 24);
	openButton->dstPos=Vec2i(230, 1);
	openButton->srcPos=Vec2i(openSmls_x,openSmls_y);
	openButton->iconSize=Vec2i(24, 24);
	buttonList+=openButton;
	g_propWinLine+=PropWinLine(yPos, openButton);

	FieldButton* button = waveToPoly->viewFrom.setUpControl(*node, waveToPoly->viewFrom, properties[2], 0, size.y - 30, 1, *this);
	button->bSize=Vec2i(24, 24);
	//button->srcPos=Vec2i(268, 191);
	button->iconSize=Vec2i(16, 24);
	button->dstPos.x = 350;
	button->valPos.x = button->dstPos.x + 24 + 4;
	button->valSize.x = 8*7;
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);

	button = waveToPoly->zoom.setUpControl(*node, waveToPoly->zoom, properties[3], 3, size.y - 30, 1, *this);
	button->valSize.x = 0;
	button->bSize = Vec2i(24, 24);
	button->srcPos = Vec2i(24*3,24);
	button->iconSize = Vec2i(24, 24);
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);

	button = waveToPoly->convert.setUpControl(*node, waveToPoly->convert, properties[4], 4, 500, 0, *this);
	button->dstPos.x = 500;
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);


	button = new CMark(Vec2i(24, 24), Vec2i(20, 1), 0, this, this, waveToPoly, waveToPoly->cMark, properties[5]);
	button->bSize = Vec2i(24, 24);
	button->srcPos = Vec2i(24*3,24);
	button->iconSize = Vec2i(24, 24);
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);

	button = new NextCycle(Vec2i(24, 24), Vec2i(80, 1), 0, this, this, waveToPoly, waveToPoly->nextCycle, properties[6]);
	button->bSize = Vec2i(24, 24);
	button->srcPos = Vec2i(24*3,24);
	button->iconSize = Vec2i(24, 24);
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);

	button=dynamic_cast<SFBoolButton*>(waveToPoly->addMark.setUpControl(*node, waveToPoly->addMark, properties[7], 0, 340+48, 0, *this));
	button->bSize=Vec2i(24, 24);
	button->dstPos=Vec2i(150-25, 1);
	button->srcPos=Vec2i(openSmls_x,openSmls_y);
	button->iconSize=Vec2i(24, 24);
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);

	button=dynamic_cast<SFBoolButton*>(waveToPoly->deleteMark.setUpControl(*node, waveToPoly->deleteMark, properties[8], 0, 340+48, 0, *this));
	button->bSize=Vec2i(24, 24);
	button->dstPos=Vec2i(150, 1);
	button->srcPos=Vec2i(openSmls_x,openSmls_y);
	button->iconSize=Vec2i(24, 24);
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);

	button=dynamic_cast<SFBoolButton*>(waveToPoly->sawTooth.setUpControl(*node, waveToPoly->sawTooth, properties[10], 0, 173, 0, *this));
	button->bSize=Vec2i(24, 24);
	button->dstPos=Vec2i(173, 1);
	buttonList+=button;
	g_propWinLine+=PropWinLine(yPos, button);

	houseKeepVisualClue();
	paint();
}

void WaveToPolyControlPanel::processFilename(){
	String ext;
	String drive;
	String dir;

	splitPath(waveToPoly->file, drive, dir, filenameOnly, ext);
}

bool WaveToPolyControlPanel::rightClick(const Vec2i ptCursor){
	if(ptCursor >= pos && ptCursor < pos + size){
		waveToPoly->viewFrom = waveToPoly->getNextCMark();
		paint();
		return true;
	}
	return false;
}

bool WaveToPolyControlPanel::click(const Vec2i ptCursor){
	if(ptCursor >= pos && ptCursor < pos+size){
		activeWindow=this;
		if(tugBorder){
			tugBorder->window = this;
			setDrag(tugBorder);
			return true;
		}

		const Vec2i localCursor = ptCursor-pos;

		{
			int newCaretPos = 0;
			for(int k = buttonList.length; --k >= 0;){
				FieldButton* button=buttonList[k];
				if(button->checkClickTextCell(localCursor, newCaretPos)){
					if(button->editMode == false){
						textCell.cellActive = false;
						textCell.selectField(*button);
					}
					textCell.highlightPos = textCell.caretPos = newCaretPos + textCell.firstVisChar;
					textCell.updateCaret();

					textCell.checkForDoubleLeftClick();

					setDrag(&textCell);
					paint();
					return true;
				}
			}
		}//above is same as ControlPanel::click except for the OpenNetFile & TextureMapper conditions
		if(localCursor.y < title_bar_height){
			setDrag(this);
			return true;
		}
		if(r_controlKeyState()){
			clickInsertCycle(localCursor);
		}
		else if(deleteKeyState()){
			clickDeleteCycle(localCursor);
		}
		else{
			clickSelectCycle(localCursor);
		}
		return true;
	}
	else{
		return false;
	}
}

void WaveToPolyControlPanel::clickInsertCycle(const Vec2i ptCursor){
	int mark = (ptCursor.x + (waveToPoly->zoom >> 1)) / waveToPoly->zoom + waveToPoly->viewFrom;
	int i = mark - 20;
	int highestIdx = i;
	float highest = waveToPoly->srcBuf[highestIdx];
	for(; i <= mark + 20; i++){
		if(waveToPoly->srcBuf[i] > highest){
			highestIdx = i;
			highest = waveToPoly->srcBuf[highestIdx];
		}
	}

	waveToPoly->cycle.refValue() += highestIdx;

	bubbleSort(waveToPoly->cycle.refValue());
}

void WaveToPolyControlPanel::clickDeleteCycle(const Vec2i ptCursor){
	int i = (ptCursor.x + (waveToPoly->zoom >> 1)) / waveToPoly->zoom + waveToPoly->viewFrom;
	M_Int& cycle = waveToPoly->cycle.refValue();
	for(int k = cycle.length; --k >= 0;){
		if(cycle[k] >= i - 5 && cycle[k] <= i + 5){
			cycle.deleteElement(k);
		}
	}
}

void WaveToPolyControlPanel::clickSelectCycle(const Vec2i ptCursor){

	int i = (ptCursor.x + (waveToPoly->zoom >> 1)) / waveToPoly->zoom + waveToPoly->viewFrom;
	M_Int& cycle = waveToPoly->cycle.refValue();
	for(int k = cycle.length; --k >= 0;){
		if(cycle[k] >= i - 10 && cycle[k] <= i + 10){
			waveToPoly->cMark = k;
			return;
		}
	}

	//add cycle if no hit
	int& cMark =	waveToPoly->cMark.refValue();

	cycle += i;

	bubbleSort(cycle);

	for(int j = cycle.length; --j >= 0;){
		if(cycle[j] == i){
			cMark = j;
		}
	}
	waveToPoly->findSignal();


	paint();
}

void WaveToPoly::AddMark::operator=(const bool& par){
	//home.divideMarkers();
	//return;
	home.cycle.refValue().length = 0;
	home.addWindowMarkers();
	home.cMark = 0;
	return;

	if(home.cycle.refValue().length == 0){
		//home.cycle.refValue() += 0;
		//home.cycle.refValue() += 512; //1920;
		//home.cycle.refValue() += 2048; //1920;
		home.addWindowMarkers();
		return;
	}
	if(home.cycle.refValue().length == 1){
		home.cycle.refValue() += 128;
		return;
	}
	int mark = home.cycle[home.cMark] + home.cycle[home.cMark] - home.cycle[home.cMark - 1];
	//int i = mark - 20;
	//int highestIdx = i;
	//float highest = home.srcBuf[highestIdx];
	//for(; i <= mark + 20; i++){
	//	if(home.srcBuf[i] > highest){
	//		highestIdx = i;
	//		highest = home.srcBuf[highestIdx];
	//	}
	//}
	home.cycle.refValue() += mark;
	bubbleSort(home.cycle.refValue());
	home.viewFrom = home.cycle[home.cMark];
	//home.findSignal();
	home.cMark = home.cMark + 1;
	home.findSignal();
	//home.findSignal();
}

void WaveToPoly::DeleteMark::operator=(const bool& par){
	M_Int& cycle = home.cycle.refValue();
	int& cMark = home.cMark.refValue();

	if(cycle.length <= 1){
		return;
	}

	cycle.deleteElement(cMark);
	if(cMark >= cycle.length){
		cMark = cycle.length - 1;
	}

	bubbleSort(cycle);
	home.viewFrom = cycle[cMark];
}

bool SurfacePoint(
				  float u, float v, int uOrder, int vOrder, int uDimension, int vDimension,
				  const float *uKnot, const float * vKnot,
				  const Vec3f *P,const float *W,
				  Vec3f *dest // result
				  );
void CurvePoint(int numOf, int degree, const float *knot, const Vec3f* point, const float *weight, const float t, Vec3f& outPoint);
void updateNurbsCurveKnotMid0(int dim, M_Float& knot);

/*
void Track::Output::operator=(const bool& par){
	int bufLen=44100*9.6;
	float* fbuf = new float[bufLen];
	for(int i=bufLen; --i>=0;){
		fbuf[i]=0;
	}
	
	M_Node& sound = home.sound.refValue();

	for(int k = 0; k < sound.length; k++){
		dynamic_cast<Oscillator*>(sound[k].p)->genSound(fbuf, bufLen);
	}
	short int* ibuf = reinterpret_cast<short int*>(fbuf);
	float volume = home.volume * 0x7fff;
	for(int i=0; i<bufLen; i++){
		ibuf[i]=fbuf[i]*volume;
	}

	FILE* stream = fopen("sound_track.wav", "w+b");

	unsigned long TOTAL_FILE_SIZE = bufLen*2 + 0x2c;

	((long*) formatM16)[1] = (TOTAL_FILE_SIZE - 8);
	((long*) formatM16)[(0x28/4)] = (TOTAL_FILE_SIZE - 0x2c);
	fwrite(formatM16, 1, 0x2c, stream);
	fwrite(ibuf, sizeof(short int), bufLen, stream);

	fclose(stream);

	delete fbuf;
}
*/
bool modifyWavFilename(String& filename){
	if(filename=="")return false;
	for(int i=filename.length; --i>=1;){
		if(filename[i]=='.'){
			filename=filename.replace(i, 0x7fffffff, ".wav");
			return true;
		}
		if(i==1||filename[i]=='\\'||filename[i]=='/'){
			filename+=".wav";
			return true;
		}
	}
	return false;
}

void Composition::Output::operator=(const bool& par){
	String filename = g_currentFilename;

	if(modifyWavFilename(filename)==false){
		msg(msg_Can_not_create_output_file_name,MB_OK | MB_ICONEXCLAMATION);
		return;
	};

	int& bufLen = home.bufLen;
	float*& fbuf = home.fbuf;
	short int*& ibuf = home.ibuf;

	if(bufLen != 44100.0 * home.duration){
		delete[] fbuf;
		delete[] ibuf;
		bufLen = 44100.0 * home.duration;
		fbuf = new float[bufLen];
		ibuf = new short int[bufLen];
	}
	for(int i=bufLen; --i>=0;){
		fbuf[i]=0;
	}
	
	M_Node& m_track = home.track.refValue();

	for(int k = 0; k < m_track.length; k++){
		dynamic_cast<Track*>(m_track[k].p)->genSound(home.volume, home.delay, fbuf, bufLen);
	}
	const float scaleToInt = 0x7fff;
	for(int i=0; i<bufLen; i++){
		ibuf[i] = fbuf[i] * scaleToInt;
	}

	if(FILE* stream = fopen(filename.c_str(), "w+b")){

		unsigned long TOTAL_FILE_SIZE = bufLen*2 + 0x2c;

		((long*) formatM16)[1] = (TOTAL_FILE_SIZE - 8);
		((long*) formatM16)[(0x28/4)] = (TOTAL_FILE_SIZE - 0x2c);
		fwrite(formatM16, 1, 0x2c, stream);
		fwrite(ibuf, sizeof(short int), bufLen, stream);

		fclose(stream);
	}
	//if(Anim* anim = Window::animMode()){
	//	anim->setUpSoundPlay();
	//}

	//delete[] fbuf;
}

void Composition::updateAndPlay(Anim* anim, Oscillator* osc){
	M_Node& m_filter = osc->filter.refValue();
	for(int k = 0; k < m_filter.length; k++){
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(m_filter[k].p)){
			patch->updateVCoord(*osc);

			output = true;
			anim->play = true;
			return;
		}
	}
}

void copyNurbsCurveToOsc(Oscillator* osc, Chords* chords, M_Float& multiple, M_Float& amplitude, M_Float& phase, double increment, double interval, double baseMultiple, double level, bool sawTooth, bool square){

	const M_Vec3f& coord = chords->coord;
	int numOfHarmonics = chords->harmonics;

	double freq = pow((pow(2.0, 1.0 / 12.0)), interval) * baseMultiple;
	double m = 1;
	if(sawTooth){
		for(int i = 0; i < numOfHarmonics; i++){
			
#ifdef STRETCH_PATCH_HARMONICS
			amplitude += level * (1.0 / m);//for sawtooth patch harmonics
#else
			amplitude += level * (1.0 / m * coord[i].y);
#endif
			multiple += freq * m;
			m += increment;
		}
	}
	else{
		for(int i = 0; i < numOfHarmonics; i++){
			amplitude += level * coord[i].y;
			multiple += freq * m;
			m += increment;
		}
	}
	if(numOfHarmonics != phase.length){
		phase.length = numOfHarmonics;
		if(sawTooth){
			for(int i = 0; i < numOfHarmonics; i++){
				if(i & 1){
					phase[i] = .5;
				}
				else{
					phase[i] = 0;
				}
			}
		}
		else{
			double range_minf = 0;
			double range_maxf = 1;
			for(int i = 0; i < numOfHarmonics; i++){
				phase[i] = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
			}
		}
	}

	//float range_maxf = 90;
	//float range_minf = 0;
	//for(int i = 0; i < numOfHarmonics; i++){
	//	float r = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
	//	multiple[i] = r + 1;
	//	amplitude[i] = 1;
	//}
	
}
/*
void bubbleSortHarmonics(M_Float& multiple, M_Float& amplitude){
	bool flag;
	do{
		flag = false;
		for(int i = 0; i < multiple.length - 1; i++){

			if(multiple[i] > multiple[i + 1]){

				float swap = multiple[i + 1];
				multiple[i + 1] = multiple[i];
				multiple[i] = swap;

				swap = amplitude[i + 1];
				amplitude[i + 1] = amplitude[i];
				amplitude[i] = swap;

				flag = true;
			}
		}
	}while(flag);
}
*/

void Composition::updateAndPlay(Anim* anim, NurbsPatch* patch){
	M_Node& m_track = this->track.refValue();

	for(int k = 0; k < m_track.length; k++){
		if(Track* track = dynamic_cast<Track*>(m_track[k].p)){
			M_Node& m_sound = track->sound.refValue();
			for(int k = 0; k < m_sound.length; k++){
				if(Oscillator* osc = dynamic_cast<Oscillator*>(m_sound[k].p)){
					M_Node& m_timbre = osc->filter.refValue();
					for(int k = 0; k < m_timbre.length; k++){
						if(patch == dynamic_cast<NurbsPatch*>(m_timbre[k].p)){

							patch->updateVCoord(*osc);


							//searchDoBuild(g_scene->children.refValue());
							//makePointerEqualIndexLocation();
							//Window::paintAllControlPanels();
							patch->houseKeep();
							output = true;
							anim->play = true;
							return;
						}
					}
				}
			}
		}
	}
}

void Composition::updateAndPlay(Anim* anim, Chords* selChords){
	M_Node& m_track = this->track.refValue();

	for(int k = 0; k < m_track.length; k++){
		if(Track* track = dynamic_cast<Track*>(m_track[k].p)){
			M_Node& m_sound = track->sound.refValue();
			for(int k = 0; k < m_sound.length; k++){
				if(Oscillator* osc = dynamic_cast<Oscillator*>(m_sound[k].p)){
					M_Node& m_filter = osc->filter.refValue();
					M_Int& limit = osc->limit.refValue();
					if(Larynx* larynx = dynamic_cast<Larynx*>(osc->larynx.p)){
						M_Node& m_chords = larynx->chords.refValue();
						bool found = false;
						for(int k = 0; k < m_chords.length && found == false; k++){
							Chords* chords = dynamic_cast<Chords*>(m_chords[k].p);
							if(chords == selChords){
								found = true;
								break;
							}
						}
						if(found){
							for(int k = 0; k < m_chords.length; k++){

								Chords* chords = dynamic_cast<Chords*>(m_chords[k].p);
								M_Float& multiple = chords->multiple.refValue();
								M_Float& amplitude = chords->amplitude.refValue();
								M_Float& phase = chords->phase.refValue();
								multiple.length = 0;
								amplitude.length = 0;
								//phase.length = 0;
								if(chords->enable){
									copyNurbsCurveToOsc(osc, chords, multiple, amplitude, phase, chords->increment, chords->interval, chords->baseMultiple, chords->level, chords->sawTooth, chords->square);
								}
								//phase.length = multiple.length;
							}
							//bubbleSortHarmonics(multiple, amplitude);
							for(int k = 0; k < m_chords.length; k++){
								Chords* chords = dynamic_cast<Chords*>(m_chords[k].p);
								if(k >= limit.length){
									limit += chords->multiple.length;
								}
							}

							for(int k = m_sound.length; --k>=0;){
								if(Oscillator* osc = dynamic_cast<Oscillator*>(m_sound[k].p)){
									if(larynx == dynamic_cast<Larynx*>(osc->larynx.p)){
										if(osc->filter.length){
											if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(osc->filter.refValue()[0].p)){
												patch->updateVCoord(*osc);
											}
										}
									}
								}
							}							
							output = true;
							anim->play = true;
							return;
						}
					}
				}
			}
		}
	}
}

/*
void Harmonic::play(float* fbuf, int bufLen, float frequency){
	int e=1;
	for(int i=0; i<bufLen; i++){
		float freq = i*.05;
		float f = i*.00003;
		if(f > envLookUp[e].x){
			e++;
			if(e >= envLookUp.length){
				return;
			}
		}
		Vec2f from = envLookUp[e-1];
		Vec2f to = envLookUp[e];
		Vec2f diff = to - from;
		float offset = f - from.x;
		float amplitude = from.y + (offset/diff.x) * diff.y;

		fbuf[i] += amplitude * sin(freq*multiply);
	}
}
*/
void HarmonicPatch::Dim::operator=(const Vec2f& par){
	M_Vec3f& dpoint = home.dpoint.refValue();
	M_Float& weight = home.weight.refValue();

	if(dpoint.length==0){
		refValue()=par;
		home.processFields();
		return;
	}

	Vec2f& dim=home.dim.refValue();

	int numOfU = dim.x;
	int numOfV = dim.y;
	int newU = numOfU;
	int newV = numOfV;
	bool changedU =false;
	bool changedV =false;

	M_Vec3f oldPoints=dpoint;
	M_Float oldWeights=weight;

	int idx=home.index[0];
	int col=idx%numOfU;
	int row=idx/numOfU;

	

	if(par.x > dim.x){//insert colum
		newU++;
		int newLen=newU*newV;
		weight.length = dpoint.length = newLen;
		
		for(int v = 0; v < numOfV; v++){//row
			for(int u = 0, nu=0; u<numOfU; u++, nu++){//collum
				dpoint[v*(numOfU+1)+nu] = oldPoints[v*numOfU+u];
				weight[v*(numOfU+1)+nu] = oldWeights[v*numOfU+u];
				if(u==col){
					if(u!=numOfU-1){
						Vec3f va = oldPoints[v*numOfU+u];
						Vec3f vb = oldPoints[v*numOfU+u+1];

						float wa = oldWeights[v*numOfU+u];
						float wb = oldWeights[v*numOfU+u+1];

						nu++;
						dpoint[v*(numOfU+1)+nu] = va + (vb-va)*.5;
						weight[v*(numOfU+1)+nu] = wa + (wb-wa)*.5;

					}
					else{
						Vec3f va = oldPoints[v*numOfU+u-1];
						Vec3f vb = oldPoints[v*numOfU+u];

						float wa = oldWeights[v*numOfU+u-1];
						float wb = oldWeights[v*numOfU+u];

						dpoint[v*(numOfU+1)+nu+1]=dpoint[v*(numOfU+1)+nu];
						dpoint[v*(numOfU+1)+nu] = va + (vb-va)*.5;

						weight[v*(numOfU+1)+nu+1]=weight[v*(numOfU+1)+nu];
						weight[v*(numOfU+1)+nu] = wa + (wb-wa)*.5;

					}
				}
			}
		}
	}
	else if(par.x < dim.x){
		newU--;
		int newLen=newU*newV;
		dpoint.length=newLen;
		for(int v = 0; v < numOfV; v++){//row
			for(int u = 0, nu=0; u<numOfU; u++){//collum
				if(u==col){
					continue;
				}
				dpoint[v*(numOfU-1)+nu]=oldPoints[v*numOfU+u];
				nu++;
			}
		}
	}
	else if(par.y > dim.y){
		newV++;
		int newLen=newU*newV;
		weight.length = dpoint.length = newLen;
		for(int v = 0, nv=0; v < numOfV; v++, nv++){//row
			if(v!=row){
				for(int u = 0; u<numOfU; u++){//collum
					dpoint[nv*numOfU+u]=oldPoints[v*numOfU+u];
					weight[nv*numOfU+u]=oldWeights[v*numOfU+u];
				}
			}
			else{
				if(v < numOfV-1){
					nv++;
					for(int u = 0; u<numOfU; u++){//collum
						Vec3f va = oldPoints[v*numOfU+u];
						Vec3f vb = oldPoints[(v+1)*numOfU+u];

						float wa = oldWeights[v*numOfU+u];
						float wb = oldWeights[(v+1)*numOfU+u];

						dpoint[nv*numOfU+u] = va + (vb-va)*.5;
						weight[nv*numOfU+u] = wa + (wb-wa)*.5;
					}
				}
				else{
					for(int u = 0; u<numOfU; u++){//collum
						dpoint[(nv+1)*numOfU+u] = dpoint[nv*numOfU+u];
						weight[(nv+1)*numOfU+u] = weight[nv*numOfU+u];

						Vec3f vb = oldPoints[v*numOfU+u];
						Vec3f va = oldPoints[(v-1)*numOfU+u];

						float wb = oldWeights[v*numOfU+u];
						float wa = oldWeights[(v-1)*numOfU+u];

						dpoint[nv*numOfU+u] = va + (vb-va)*.5;
						weight[nv*numOfU+u] = wa + (wb-wa)*.5;
					}
				}
			}
		}
	}
	else if(par.y < dim.y){
		int newLen=newU*newV;
		newV--;
		dpoint.length=newLen;
		for(int v = 0, nv=0; v < numOfV; v++){//row
			if(v==row){
				continue;
			}
			for(int u = 0; u<numOfU; u++){//collum
				dpoint[nv*numOfU+u]=oldPoints[v*numOfU+u];
				weight[nv*numOfU+u]=oldWeights[v*numOfU+u];
			}
			nv++;
		}
	}

	refValue()=Vec2f(newU, newV);
	if(col>=numOfU){
		col--;
	}
	home.index[0]=row*newU+col;

	M_Float& uKnot = home.uKnot.refValue();
	M_Float& vKnot = home.vKnot.refValue();

	uKnot.length = newU+3;
	vKnot.length = newV+3;

	int n = 0;

	for(; n < 3; n++){
		uKnot[n] = 0;
		vKnot[n] = 0;
	}
	/*
	for(; n < uKnot.length-4; n++){
		uKnot[n] = n-3;
	}
	int last = n-3;
	for(; n < uKnot.length; n++){
		uKnot[n] = last;
	}
*/
	n = 3;
	for(; n < vKnot.length-3; n++){
		vKnot[n] = n-2;
	}
	int last = n-2;
	for(; n < vKnot.length; n++){
		vKnot[n] = last;
	}
	home.dpointsToPoints();

//	home.uKnot.refValue() = M_Double(0,0,0,0, 1, 2,2,2,2);
}
void HarmonicPatch::dpointsToPoints(){
	M_Vec3f& point = this->point.refValue();
	M_Vec3f& dpoint = this->dpoint.refValue();

	point.length = dpoint.length;

	for(int i=dpoint.length; --i>=0;){
		point[i] = dpoint[i] * weight[i];
	}
}
void HarmonicPatch::processFields(){
	uDegree = order.x-1;
	vDegree = order.y-1;

	vCoord.refValue().length = coord.refValue().length = harmonics * (staves + 1);

	M_Vec3f& point = this->point.refValue();
	M_Vec3f& dpoint = this->dpoint.refValue();

	dpoint.length=point.length;

	for(int i=dpoint.length; --i>=0;){
		dpoint[i] = point[i] / weight[i];
	}

	int uDim=dim.x;
	int vDim=dim.y;
	int shouldBeLen=uDim*vDim;

	if(shouldBeLen!=point.length){
		point.length=shouldBeLen;
		for(int v = 0; v < vDim; v++){
			for(int u = 0; u < uDim; u++){
				point[v*uDim+u]=Vec3f(1.0/(uDim-1) * u - .5, 1.0/(vDim-1) * v, 0);
			}
		}
	}
	r_point.length=vDim;
	r_weight.length=vDim;
	for(int i=0; i<vDim; i++){
		r_point[i] = point[i*uDim+uDim-1];
		r_weight[i] = weight[i*uDim+uDim-1];
	}
}
void Oscillator::DPoint::operator=(const M_Vec2f& m_par){
	int idx = home.index[0];
	float w = 1;
	if(home.fWeight.length){
		w = home.fWeight[idx];
	}
	home.note.refValue()[idx] = m_par[idx]*w;
	home.processFields();
	home.houseKeep();
}
void Harmonic::DPoint::operator=(const M_Vec2f& m_par){
	int index = home.index[0];
	home.point.refValue()[index]=m_par[index]*home.weight[index];
	home.processFields();
}
void HarmonicPatch::DPoint::operator=(const M_Vec3f& m_par){
	int index = home.index[0];
	home.point.refValue()[index]=m_par[index]*home.weight[index];
	home.processFields();
}
void Oscillator::Weight::operator=(const M_Float& m_par){
	refValue() = m_par;
	int index = home.index[0];
	home.note.refValue()[index]=home.fdpoint[index] * home.fWeight[index];
}
void Harmonic::Weight::operator=(const M_Float& m_par){
	refValue() = m_par;
	int index = home.index[0];
	home.point.refValue()[index]=home.dpoint[index] * home.weight[index];
}
void HarmonicPatch::Weight::operator=(const M_Float& m_par){
	refValue() = m_par;
	int index = home.index[0];
	home.point.refValue()[index]=home.dpoint[index] * home.weight[index];
}

void Oscillator::copyToMelody(NurbsLine* melody){
	M_Vec2f& note = this->note.refValue();
	M_Vec3f& point = melody->point.refValue();

	point.length = note.length;

	for(int k = note.length; --k >= 0;){
		point[k] = Vec3f(note[k].x, note[k].y * .1, 0);
	}

	melody->weight.refValue() = fWeight;
	melody->uKnot.refValue() = fKnot;
	melody->order.refValue().x = fOrder; //do we need this?

	melody->processFields();
}

void Oscillator::copyFromMelody(NurbsLine* melody){
	M_Vec2f& note = this->note.refValue();
	M_Vec3f& point = melody->point.refValue();

	note.length = point.length;

	for(int k = note.length; --k >= 0;){
		note[k] = Vec2f(point[k].x, point[k].y * 10.0);
	}

	fWeight.refValue() = melody->weight;
	fKnot.refValue() = melody->uKnot;
	fOrder.refValue() = melody->order.x; //do we need this?

	processFields();
}

void Oscillator::buildSound(float masterVolume, float masterDelay, const int voices,  const float detune, float* fbuf, int bufLen){
	if(NurbsLine* mel = dynamic_cast<NurbsLine*>(this->melody.p)){
		if(mel->point.length == 2){
			copyToMelody(mel);
		}
		else{
			copyFromMelody(mel);
		}
	}
	for(int k = 0; k < filter.length; k++){
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(this->filter[k].p)){
			patch->genSound(masterVolume*volume, masterDelay+delay, voices,  detune, *this, fbuf, bufLen);
		}
		else if(Timbre* timbre = dynamic_cast<Timbre*>(this->filter[k].p)){
			timbre->genSound(masterVolume*volume, masterDelay+delay, voices,  detune, *this, fbuf, bufLen);
		}
	}
}
void Phrase::buildSound(float masterVolume, float masterDelay, const int voices,  const float detune, float* fbuf, int bufLen){
	M_Node& sound = this->sound;
	for(int k=0; k<sound.length; k++){
		if(SoundBuild* soundBuild = dynamic_cast<SoundBuild*>(sound[k].p)){
			soundBuild->buildSound(masterVolume*volume, masterDelay+delay, voices,  detune, fbuf, bufLen);
		}
	}
}
void Track::genSound(float masterVolume, float delay, float* fbuf, int bufLen){
	M_Node& sound = this->sound;
	for(int k = 0; k < sound.length; k++){
		if(SoundBuild* soundBuild = dynamic_cast<SoundBuild*>(sound[k].p)){
			soundBuild->buildSound(masterVolume*(volume/voices), delay, voices,  detune, fbuf, bufLen);
		}
	}
	int echoDelayInt = echoDelay * 44100;
	for(int s = delay; s < bufLen - echoDelayInt; s++){
		float sample = fbuf[s];
		int d = s + echoDelayInt;
		for(int e = echos; --e>=0 && d < bufLen;){
			sample = sample * decay;
			fbuf[d] += sample;
			d += echoDelayInt;
		}
	}
}

void Harmonic::genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen){
	float step=1.0/1000.0;

	const float* uKnotPtr = this->knot.refValue().getArray();
	Vec2f* pointPtr = this->point.refValue().getArray();
	float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order;
	int vDim = multiply.length;
	const int uDim = this->point.refValue().length / vDim;
	const float maxU=this->maxT();
	const float minU=this->minT();

	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const Vec2f* fPointPtr = osc.note.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();
	const int fOrder = osc.fOrder;
	const int dim = osc.note.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int fDegree = osc.fDegree;

	int idelay = delay * 44100.0;

	float detuneStep=detune/voices;
	float detuneAdd =-detune * .5;

	for(int d=0; d<voices; d++){

		for(int h = 0; h < vDim; h++){//h=harmonic
			double a = PI * 2 * this->phase[h];
			double multiply = this->multiply[h];
			//float freq = multiply*(.05+d*.00025);
			float uc = 0;
			//vc-= r;
			//r*=.4;


			float u = (maxU-minU)*uc + minU;

			Vec2f from = curvePoint2d(uDim, this->degree, uKnotPtr, pointPtr, weightPtr, u, uc);

			uc += step;
			u = (maxU-minU)*uc + minU;
			Vec2f to = curvePoint2d(uDim, this->degree, uKnotPtr, pointPtr, weightPtr, u, uc);

			//////////frequency
			float tc = 0;
			float t = (maxT-minT)*tc + minT;

			Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

			tc += step;
			t = (maxT-minT)*tc + minT;
			Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
			/////////

			for(int s=0, j = idelay; j<bufLen; s++, j++){//s=sample
			
				float duration = s * (1.0/44100.0);
				if(duration > to.x){
					uc += step;
					if(uc >= 1.0){
						break;
					}
					from=to;
					u = (maxU-minU)*uc + minU;
					to = curvePoint2d(uDim, this->degree, uKnotPtr, pointPtr, weightPtr, u, uc);
				}
				Vec2f diff = to - from;
				float offset = duration - from.x;
				float amplitude = from.y + (offset/diff.x) * diff.y;

				//frequency
				if(duration > fTo.x){
					tc += step;
					if(tc >= 1.0){
						break;
					}
					fFrom = fTo;
					t = (maxT-minT)*tc + minT;
					fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
				}
				Vec2f fDiff = fTo - fFrom;
				float fOffset = duration - fFrom.x;
				double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
				//cent+=6; //change key
				cent*=100;
				cent+=detuneAdd;
				//
				double freq = 220; //A4 = 440 hertz so why and octive lower?
				freq = freq * pow((pow(2.0,1.0/1200.0)),cent);

				a += (freq / 44100.0) * PI * 2 * multiply;

				fbuf[j] += amplitude * volume * sin(a);
			}
			pointPtr += uDim;
			weightPtr += uDim;
		}
		detuneAdd+=detuneStep;
	}
}

void HarmonicPatch::genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen){
	float step=1.0 / this->staves;

	const float* uKnotPtr = this->uKnot.refValue().getArray();
	const float* vKnotPtr = this->vKnot.refValue().getArray();
	const Vec3f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order.x;
	const int vOrder = this->order.y;
	const int uDim = this->dim.x;
	const int vDim = this->dim.y;
	const M_Float& vCoord = this->vCoord;
	const float maxU=this->maxU();
	const float maxV=this->maxV();
	const float minU=this->minU();
	const float minV=this->minV();

	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const Vec2f* fPointPtr = osc.note.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();
	const int fOrder = osc.fOrder;
	const int dim = osc.note.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int fDegree = osc.fDegree;

	int idelay = delay * 44100.0;

	float detuneStep=detune/voices;
	float detuneAdd=-detune*.5;

	for(int d=0; d < voices; d++){

		for(int h = 0; h < harmonics; h++){//h=harmonic
			double a = 0;
			if(h & 1){
				a = PI;
			}

			double multiply = h + 1;

			//////////frequency
			float tc = 0;
			float t = (maxT-minT)*tc + minT;
			int i2 = h * (staves + 1);
			Vec3f from = coord[i2++];
			Vec3f to = coord[i2++];


			Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

			tc += step;
			t = (maxT-minT)*tc + minT;
			Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
			/////////

			for(int s=0, j = idelay; j<bufLen; s++, j++){//s=sample
			
				float duration = s * (1.0/44100.0);
				if(duration > to.x){
					from = to;
					to = coord[i2++];
				}
				Vec3f diff = to - from;
				float offset = duration - from.x;
				float amplitude = from.y + (offset/diff.x) * diff.y;

				//frequency
				if(duration > fTo.x){
					tc += step;
					if(tc > 1.0){
						break;
					}
					fFrom=fTo;
					t = (maxT-minT)*tc + minT;
					fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
				}
				Vec2f fDiff = fTo - fFrom;
				float fOffset = duration - fFrom.x;
				double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
				cent*=100;
				cent+=detuneAdd;
				//
				double freq=220; //A4 = 440 hertz so why and octive lower?
				freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);

				a += (freq / 44100.0) * PI * 2 * multiply;

				fbuf[j] += amplitude * volume * sin(a) / multiply * 10;
			}
		}
		detuneAdd+=detuneStep;
	}
}

/*
void NurbsPatch::genSound(const float volume, const float delay, const int voices,  const float detune, const Oscillator& osc, float* fbuf, int bufLen){
	if(this->stem.length == 0){
		return;
	}
	Stem* stem = dynamic_cast<Stem*>(this->stem.refValue()[0].p);
	if(stem->part.length == 0){
		return;
	}
	Art* part = stem->part.refValue()[0].p;
	if(stem->bands.length == 0){
		return;
	}
	M_Vec3f& coord = part->coord.refValue();
	int harmonics = stem->bands[0] + 1;
	int staves = stem->staves;

	float step = 1.0 / stem->staves;

	const float* uKnotPtr = this->uKnot.refValue().getArray();
	const float* vKnotPtr = this->vKnot.refValue().getArray();
	const Vec3f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order.x;
	const int vOrder = this->order.y;
	const int uDim = this->dim.x;
	const int vDim = this->dim.y;
	static M_Float vCoord;
	if(stem->source.length){
		Art* source = stem->source.refValue()[0].p;
		const M_Vec3f& partCoord = source->coord;
		vCoord.length = partCoord.length;
		for(int i = partCoord.length; --i>=0;){
			vCoord[i] = partCoord[i].y;
		}
	}
	else{
		const M_Vec2f& stemCoord = stem->coord;
		vCoord.length = vCoord.length;
		for(int i = stemCoord.length; --i>=0;){
			vCoord[i] = stemCoord[i].y;
		}
	}
	const float maxU=this->maxU();
	const float maxV=this->maxV();
	const float minU=this->minU();
	const float minV=this->minV();

	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const Vec2f* fPointPtr = osc.fPoint.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();
	const int fOrder = osc.fOrder;
	const int dim = osc.fPoint.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int fDegree = osc.fDegree;

	int idelay = delay * 44100.0;

	float detuneStep=detune/voices;
	float detuneAdd=-detune*.5;

	for(int d=0; d < voices; d++){

		for(int h = 0; h < harmonics; h++){//h=harmonic
			double a = 0;
			if(h & 1){
				a = PI;
			}

			double multiply = h + 1;

			//////////frequency
			float tc = 0;
			float t = (maxT-minT)*tc + minT;
			int i2 = h * (staves + 1);
			Vec3f from = coord[i2++];
			Vec3f to = coord[i2++];


			Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

			tc += step;
			t = (maxT-minT)*tc + minT;
			Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
			/////////

			for(int s=0, j = idelay; j<bufLen; s++, j++){//s=sample
			
				float duration = s * (1.0/44100.0);
				if(duration > to.x){
					from = to;
					to = coord[i2++];
				}
				Vec3f diff = to - from;
				float offset = duration - from.x;
				float amplitude = from.y + (offset/diff.x) * diff.y;

				//frequency
				if(duration > fTo.x){
					tc += step;
					if(tc > 1.0){
						break;
					}
					fFrom=fTo;
					t = (maxT-minT)*tc + minT;
					fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
				}
				Vec2f fDiff = fTo - fFrom;
				float fOffset = duration - fFrom.x;
				double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
				cent*=100;
				cent+=detuneAdd;
				//
				double freq=220; //A4 = 440 hertz so why and octive lower?
				freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);

				a += (freq / 44100.0) * PI * 2 * multiply;

				fbuf[j] += amplitude * volume * sin(a) / multiply * 10;
			}
		}
		detuneAdd+=detuneStep;
	}
}
*/

inline void getToFreqToPatchStart(int& s, int& j, const Vec3f* pointPtr, const int bufLen, Vec2f& fFrom, Vec2f& fTo, float& t, float& tc, const float& step, const float& minT, const float& maxT, const float& detuneAdd, double& a, const Vec2f* fPointPtr, const float* fWeightPtr, const float* fKnotPtr, const int& fDegree, const int& dim, const double& multiply){
	if(pointPtr->x == 0){
		return;
	}
	
	int beginSample = pointPtr->x * 44100.0;
	beginSample += j;

	for(; j < bufLen && j < beginSample; s++, j++){//s=sample
			
		float duration = s * (1.0/44100.0);

		//frequency
		if(duration > fTo.x){
			tc += step;
			if(tc > 1.0){
				break;
			}
			fFrom=fTo;
			t = (maxT-minT)*tc + minT;
			fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
		}
		Vec2f fDiff = fTo - fFrom;
		float fOffset = duration - fFrom.x;
		double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
		cent*=100;
		cent+=detuneAdd;
		//
		double freq=220; //A4 = 440 hertz so why and octive lower?
		freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);

		a += (freq / 44100.0) * PI * 2 * multiply;
	}
}

void Envelope::doEnvelope(float* chordsBuf, int chordsBufLen){
	if(coord.length < 2){
		return;
	}
	Vec2f from = coord[0];
	Vec2f to = coord[1];
	int ci = 2;

	float fullDuration = chordsBufLen * (1.0/44100.0);

	for(int s = 0; s < chordsBufLen; s++){

		float duration = s * (1.0/44100.0);


		//frequency
		if(duration > to.x){
			if(ci == coord.length){
				break;
			}
			from = to;
			to = coord[ci++];
		}
		Vec2f diff = to - from;
		float offset = duration - from.x;
		float pan = from.y + (offset/diff.x) * diff.y;
		//float ipan = 1.0 - pan;

		chordsBuf[s] = chordsBuf[s] * pan;
	}
}

void chordsGenSound(const float volume, int voices, float detuneStep, float detuneAdd, int staves, const int limit, float*& chordsBufRef, int& chordsBufLenRef, const M_Float& phase, int numOfHarmonics, const M_Float& amplitude, int& chordsCoordLen, const M_Vec3f& coord, int& coordOffset, const Vec2f* fPointPtr, const float* fWeightPtr, const float* fKnotPtr, 	const int dim, const float maxT, const float minT, const int fDegree, const Vec3f* pointPtr, float soundPeriod, const int idelay, float* fbuf, const int bufLen, Envelope* envelope, bool doPatch){
	
	float* chordsBuf = chordsBufRef;
	int chordsBufLen = chordsBufLenRef;

	if(chordsBufLen != soundPeriod * 44100){
		chordsBufLen = soundPeriod * 44100;
		delete[] chordsBuf;
		chordsBuf = new float[chordsBufLen];
		chordsBufRef = chordsBuf;
		chordsBufLenRef = chordsBufLen;
	}

	for(int e = chordsBufLen; --e>=0;){
		chordsBuf[e] = 0;
	}

	float step = 1.0 / staves;
	if(numOfHarmonics > limit){
		numOfHarmonics = limit;
	}
	chordsCoordLen = (staves + 1) * numOfHarmonics;

	for(int d = 0; d < voices; d++){

		for(int h = 0; h < numOfHarmonics; h++){//h=harmonic
			double a = PI * 2 * phase[h];

			//double multiply = m_multiple[h];
			double harmonicAmplitude = amplitude[h];

			//////////frequency
			float tc = 0;
			float t = (maxT-minT)*tc + minT;
			int i2 = coordOffset + h * (staves + 1);
			Vec3f from = coord[i2++];
			Vec3f to = coord[i2++];


			Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

			tc += step;
			t = (maxT-minT)*tc + minT;
			Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
			/////////
			int s = pointPtr->x * 44100.0;;
			//int j = idelay + s;
			//int s = 0;


			int randomAmpSegLen = 2500;
			int randomAmpSegCntr = randomAmpSegLen;
			double randomAmpFrom = 0;
			double randomAmpTo = 0;
			double randomAmpDiff = 0;


			int j = 0;
	
			//getToFreqToPatchStart(s, j, pointPtr, bufLen, fFrom, fTo, t, tc, step, minT, maxT, detuneAdd, a, fPointPtr, fWeightPtr, fKnotPtr, fDegree, dim, multiply);

			for(int i = 2; j < chordsBufLen; s++, j++){//s=sample
			
				float duration = s * (1.0 / 44100.0);
				if(duration > to.x){
					from = to;
					if(i > staves){
						break;
					}
					to = coord[i2++];
					i++;
				}
				Vec3f diff = to - from;
				float offset = duration - from.x;
				double amplitude = from.y + (offset/diff.x) * diff.y;
				double freq = -(from.z + (offset/diff.x) * diff.z) * 10000;

#ifdef RANDOM_HAMONIC_AMPLITUDE
				if(randomAmpSegCntr == randomAmpSegLen){
					randomAmpSegCntr = 0;
					randomAmpFrom = randomAmpTo;


					double range_maxf = harmonicAmplitude * 1;
					double range_minf = -range_maxf;

					randomAmpTo = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
					randomAmpDiff = randomAmpTo - randomAmpFrom;
				}
				double randAmp = randomAmpFrom + (static_cast<double>(randomAmpSegCntr) / static_cast<double>(randomAmpSegLen)) * randomAmpDiff;

				randomAmpSegCntr++;
				
#endif

				//frequency
				//if(duration > fTo.x){
				//	tc += step;
				//	if(tc > 1.0){
				//		break;
				//	}
				//	fFrom=fTo;
				//	t = (maxT-minT)*tc + minT;
				//	fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
				//}
				//Vec2f fDiff = fTo - fFrom;
				//float fOffset = duration - fFrom.x;
				//double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
				//cent*=100;
				//cent+=detuneAdd;
				////
				//double freq = 220; //A4 = 440 hertz so why and octive lower?
				//freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);

				//a += (freq / 44100.0) * PI * 2 * multiply;

				//freq = freq * pow((pow(2.0, 1.0/1200.0)), double(detuneAdd));
				a += (freq / 44100.0) * PI * 2;

				//chordsBuf[j] += amplitude * volume * sin(a) * harmonicAmplitude * 10;

				if(doPatch){
					//chordsBuf[j] += amplitude * volume * sin(a);
					chordsBuf[j] += amplitude * volume * sin(a) * harmonicAmplitude * 10;//for sawtooth
				}
				else{
#ifdef RANDOM_HAMONIC_AMPLITUDE
					double harAmp = (harmonicAmplitude + randAmp);
					if(harAmp < 0){
						harAmp = 0;
					}
					chordsBuf[j] += amplitude * volume * sin(a) * harAmp * 10;
#else

					chordsBuf[j] += amplitude * volume * sin(a) * harmonicAmplitude * 10;
#endif
				}
			}
		}
		detuneAdd += detuneStep;
	}
	//if(AmplitudeModulator* amplitudeModulator = dynamic_cast<AmplitudeModulator*>(chords->modulator.p)){
	//	if(Oscillator* carrierOsc = dynamic_cast<Oscillator*>(amplitudeModulator->oscillator.p)){
	//		if(Larynx* larynx = dynamic_cast<Larynx*>(carrierOsc->larynx.p)){
	//			if(Chords* carrierChords = dynamic_cast<Chords*>(larynx->chords[0].p)){
	//						

	//				amplitudeModulator->genCarrierSignal(carrierChords->level, soundPeriod, *carrierOsc);
	//				amplitudeModulator->modulate(fbuf, bufLen);
	//			}
	//		}
	//	}
	//}
	if(envelope){
		envelope->doEnvelope(chordsBuf, chordsBufLen);
	}
	coordOffset += chordsCoordLen;
	int d = pointPtr->x * 44100.0;
	d += idelay;
	int len = chordsBufLen;
	if(d + len >= bufLen){
		len = bufLen - d;
	}
	for(int s = 0; s < len; s++, d++){
		fbuf[d] += chordsBuf[s];
	}
}
/*
void chordsGenSound(const float volume, int voices, float detuneStep, float detuneAdd, int staves, const int limit, float*& chordsBufRef, int& chordsBufLenRef, const M_Float& phase, int numOfHarmonics, const M_Float& amplitude, int& chordsCoordLen, const M_Vec3f& coord, int& coordOffset, const Vec2f* fPointPtr, const float* fWeightPtr, const float* fKnotPtr, 	const int dim, const float maxT, const float minT, const int fDegree, const Vec3f* pointPtr, float soundPeriod, const int idelay, float* fbuf, const int bufLen, Envelope* envelope, bool doPatch){
	
	float* chordsBuf = chordsBufRef;
	int chordsBufLen = chordsBufLenRef;

	if(chordsBufLen != soundPeriod * 44100){
		chordsBufLen = soundPeriod * 44100;
		delete[] chordsBuf;
		chordsBuf = new float[chordsBufLen];
		chordsBufRef = chordsBuf;
		chordsBufLenRef = chordsBufLen;
	}

	for(int e = chordsBufLen; --e>=0;){
		chordsBuf[e] = 0;
	}

	float step = 1.0 / staves;
	if(numOfHarmonics > limit){
		numOfHarmonics = limit;
	}
	chordsCoordLen = (staves + 1) * numOfHarmonics;

	for(int d = 0; d < voices; d++){
		for(int h = 0; h < numOfHarmonics; h++){//h=harmonic
			double a = PI * 2 * phase[h];

			//double multiply = m_multiple[h];
			double harmonicAmplitude = amplitude[h];

			//////////frequency
			float tc = 0;
			float t = (maxT-minT)*tc + minT;
			int i2 = coordOffset + h * (staves + 1);
			Vec3f from = coord[i2++];
			Vec3f to = coord[i2++];


			Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

			tc += step;
			t = (maxT-minT)*tc + minT;
			Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
			/////////
			int s = pointPtr->x * 44100.0;;
			//int j = idelay + s;
			//int s = 0;
			int j = 0;
	
			//getToFreqToPatchStart(s, j, pointPtr, bufLen, fFrom, fTo, t, tc, step, minT, maxT, detuneAdd, a, fPointPtr, fWeightPtr, fKnotPtr, fDegree, dim, multiply);

			for(int i = 2; j < chordsBufLen; s++, j++){//s=sample
				float duration = s * (1.0 / 44100.0);
				if(duration > to.x){
					from = to;
					if(i > staves){
						break;
					}
					to = coord[i2++];
					i++;
				}
				Vec3f diff = to - from;
				float offset = duration - from.x;
				//harmonicAmplitude = from.y + (offset/diff.x) * diff.y;
				//double freq = -(from.z + (offset/diff.x) * diff.z) * 10000;


				float amplitude = from.y + (offset/diff.x) * diff.y;
				double freq = -(from.z + (offset/diff.x) * diff.z) * 10000;

				//frequency
				//if(duration > fTo.x){
				//	tc += step;
				//	if(tc > 1.0){
				//		break;
				//	}
				//	fFrom=fTo;
				//	t = (maxT-minT)*tc + minT;
				//	fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
				//}
				//Vec2f fDiff = fTo - fFrom;
				//float fOffset = duration - fFrom.x;
				//double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
				//cent*=100;
				//cent+=detuneAdd;
				////
				//double freq = 220; //A4 = 440 hertz so why and octive lower?
				//freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);

				//a += (freq / 44100.0) * PI * 2 * multiply;

				//freq = freq * pow((pow(2.0, 1.0/1200.0)), double(detuneAdd));
				a += (freq / 44100.0) * PI * 2;

				chordsBuf[j] += amplitude * volume * sin(a) * harmonicAmplitude * 10;
			}
		}
		detuneAdd+=detuneStep;
	}
	//if(AmplitudeModulator* amplitudeModulator = dynamic_cast<AmplitudeModulator*>(chords->modulator.p)){
	//	if(Oscillator* carrierOsc = dynamic_cast<Oscillator*>(amplitudeModulator->oscillator.p)){
	//		if(Larynx* larynx = dynamic_cast<Larynx*>(carrierOsc->larynx.p)){
	//			if(Chords* carrierChords = dynamic_cast<Chords*>(larynx->chords[0].p)){
	//						

	//				amplitudeModulator->genCarrierSignal(carrierChords->level, soundPeriod, *carrierOsc);
	//				amplitudeModulator->modulate(fbuf, bufLen);
	//			}
	//		}
	//	}
	//}
	if(envelope){
		envelope->doEnvelope(chordsBuf, chordsBufLen);
	}
	coordOffset += chordsCoordLen;
	int d = pointPtr->x * 44100.0;
	d += idelay;
	int len = chordsBufLen;
	if(d + len >= bufLen){
		len = bufLen - d;
	}
	for(int s = 0; s < len; s++, d++){
		fbuf[d] += chordsBuf[s];
	}
}
*/

void NurbsPatch::genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen){
	Larynx* larynx = dynamic_cast<Larynx*>(osc.larynx.p);
	if(larynx == 0){
		return;
	}
	int staves = osc.staves;
	const M_Vec3f& coord = osc.coord.refValue();
	const M_Vec2f& vCoord = osc.vCoord;


	const Vec3f* pointPtr = this->point.refValue().getArray();
	const int uDim = this->dim.x;

	const Vec2f* fPointPtr = osc.note.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();
	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const int fOrder = osc.fOrder;
	const int dim = osc.note.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int fDegree = osc.fDegree;

	int idelay = delay * 44100.0;

	float detuneStep=detune/voices;
	float detuneAdd=-detune*.5;


	M_Node& m_chords = larynx->chords.refValue();
	int coordOffset = 0;
	int limitIdx = 0;
	float soundPeriod = pointPtr[uDim - 1].x - pointPtr->x;
	bool doPatch = false;

	for(int k = 0; k < m_chords.length; k++){
		int chordsCoordLen = 0;

		if(Chords* chords = dynamic_cast<Chords*>(m_chords[k].p)){
			doPatch = false;
			if(chords->patials.p){
				doPatch = true;
			}
			chordsGenSound(volume * chords->level, voices, detuneStep, detuneAdd, staves, osc.limit[limitIdx], osc.chordsBuf[limitIdx], osc.chordsBufLen[limitIdx], chords->phase, chords->multiple.length, chords->amplitude, chordsCoordLen, coord, coordOffset, fPointPtr, fWeightPtr, fKnotPtr, 	dim, maxT, minT, fDegree, pointPtr, soundPeriod, idelay, fbuf, bufLen, 0, doPatch);
			limitIdx++;
		}
		if(Envelope* envelope = dynamic_cast<Envelope*>(m_chords[k].p)){
			M_Node& m_chords = envelope->chords.refValue();
			for(int k = 0; k < m_chords.length; k++){
				if(Chords* chords = dynamic_cast<Chords*>(m_chords[k].p)){
					doPatch = false;
					if(chords->patials.p){
						doPatch = true;
					}
					chordsGenSound(volume * chords->level, voices, detuneStep, detuneAdd, staves, osc.limit[limitIdx], osc.chordsBuf[limitIdx], osc.chordsBufLen[limitIdx], chords->phase, chords->multiple.length, chords->amplitude, chordsCoordLen, coord, coordOffset, fPointPtr, fWeightPtr, fKnotPtr, 	dim, maxT, minT, fDegree, pointPtr, soundPeriod, idelay, fbuf, bufLen, envelope, doPatch);
					limitIdx++;
				}
			}
		}
	}
}

/*
void NurbsPatch::genSound(const float volume, const float delay, const int voices,  const float detune, const Oscillator& osc, float* fbuf, int bufLen){
	if(this->stem.length == 0){
		return;
	}
	Stem* stem = dynamic_cast<Stem*>(this->stem.refValue()[0].p);
	if(stem->part.length == 0){
		return;
	}
	Art* part = stem->part.refValue()[0].p;
	if(stem->bands.length == 0){
		return;
	}
	Larynx* larynx = dynamic_cast<Larynx*>(osc.larynx.p);
	if(larynx == 0){
		return;
	}

	M_Float& phase = larynx->phase;
	M_Float& m_multiple = larynx->multiple;
	M_Float& amplitude = larynx->amplitude;

	int harmonics = stem->bands[0] + 1;
	if(harmonics > m_multiple.length){
		harmonics = m_multiple.length;
	}

	int staves = stem->staves;
	M_Vec3f& coord = part->coord.refValue();

	float step = 1.0 / stem->staves;


	const float* uKnotPtr = this->uKnot.refValue().getArray();
	const float* vKnotPtr = this->vKnot.refValue().getArray();
	const Vec3f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order.x;
	const int vOrder = this->order.y;
	const int uDim = this->dim.x;
	const int vDim = this->dim.y;
	const float maxU=this->maxU();
	const float maxV=this->maxV();
	const float minU=this->minU();
	const float minV=this->minV();


	float* carrierBuf = 0;
	if(Chords* chords = dynamic_cast<Chords*>(larynx->chords[0].p)){
		if(FrequencyModulator* frequencyModulator = dynamic_cast<FrequencyModulator*>(chords->modulator.p)){
			if(Oscillator* carrierOsc = dynamic_cast<Oscillator*>(frequencyModulator->oscillator.p)){
				if(Larynx* larynx = dynamic_cast<Larynx*>(carrierOsc->larynx.p)){
					if(Chords* chords = dynamic_cast<Chords*>(larynx->chords[0].p)){
						float soundPeriod = pointPtr[uDim - 1].x - pointPtr->x;

						frequencyModulator->genCarrierSignal(chords->level, soundPeriod, *carrierOsc);
						carrierBuf = frequencyModulator->fbuf;
					}
				}
			}
		}
	}
	if(carrierBuf == 0){
		return;
	}


	const Vec2f* fPointPtr = osc.note.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();
	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const int fOrder = osc.fOrder;
	const int dim = osc.note.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int fDegree = osc.fDegree;

	int idelay = delay * 44100.0;

	float detuneStep=detune/voices;
	float detuneAdd=-detune*.5;

	for(int d=0; d < voices; d++){

		for(int h = 0; h < harmonics; h++){//h=harmonic
			double a = PI * 2 * phase[h];

			double multiply = m_multiple[h];
			double harmonicAmplitude = amplitude[h];

			//////////frequency
			float tc = 0;
			float t = (maxT-minT)*tc + minT;
			int i2 = h * (staves + 1);
			Vec3f from = coord[i2++];
			Vec3f to = coord[i2++];


			Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

			tc += step;
			t = (maxT-minT)*tc + minT;
			Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
			/////////
			int s = pointPtr->x * 44100.0;;
			int j = idelay + s;
	
			//getToFreqToPatchStart(s, j, pointPtr, bufLen, fFrom, fTo, t, tc, step, minT, maxT, detuneAdd, a, fPointPtr, fWeightPtr, fKnotPtr, fDegree, dim, multiply);

			for(int i = 2; j < bufLen; s++, j++){//s=sample
			
				float duration = s * (1.0/44100.0);
				if(duration > to.x){
					from = to;
					if(i > staves){
						break;
					}
					to = coord[i2++];
					i++;
				}
				Vec3f diff = to - from;
				float offset = duration - from.x;
				float amplitude = from.y + (offset/diff.x) * diff.y;
				double freq = -(from.z + (offset/diff.x) * diff.z) * 10000;

				//frequency
				//if(duration > fTo.x){
				//	tc += step;
				//	if(tc > 1.0){
				//		break;
				//	}
				//	fFrom=fTo;
				//	t = (maxT-minT)*tc + minT;
				//	fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
				//}
				//Vec2f fDiff = fTo - fFrom;
				//float fOffset = duration - fFrom.x;
				//double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
				//cent*=100;
				//cent+=detuneAdd;
				////
				//double freq = 220; //A4 = 440 hertz so why and octive lower?
				//freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);

				//a += (freq / 44100.0) * PI * 2 * multiply;

				freq = freq * pow((pow(2.0, 1.0 / 12.0)), double(carrierBuf[j]));
				a += (freq / 44100.0) * PI * 2;

				fbuf[j] += amplitude * volume * sin(a) * harmonicAmplitude * 10;
			}
		}
		detuneAdd+=detuneStep;
	}
}
*/

void AmplitudeModulator::modulate(float* dbuf, int dbufLen){
	if(coord.length < 2){
		return;
	}
	Vec2f from = coord[0];
	Vec2f to = coord[1];
	int ci = 2;

	for(int s = 0; s < dbufLen; s++){

		float duration = s * (1.0/44100.0);


		//frequency
		if(duration > to.x){
			from = to;
			to = coord[ci++];
		}
		Vec2f diff = to - from;
		float offset = duration - from.x;
		float pan = from.y + (offset/diff.x) * diff.y;
		float ipan = 1.0 - pan;

		dbuf[s] = fbuf[s] * dbuf[s] * pan + dbuf[s] * ipan;
		//dbuf[i] = fbuf[i] * dbuf[i];
	}
}

void FrequencyModulator::modulate(float* dbuf, int dbufLen){
	for(int i = 0; i < dbufLen; i++){
		//dbuf[i] = dbuf[i] + fbuf[i] * dbuf[i];
		//dbuf[i] = sin() * fbuf[i] * dbuf[i];
	}
}

void AmplitudeModulator::genCarrierSignal(const float volume, float soundPeriod, const Oscillator& osc){
	Larynx* larynx = dynamic_cast<Larynx*>(osc.larynx.p);
	if(larynx == 0){
		return;
	}
	bufLen = 44100 / soundPeriod;
	fbuf = new float[bufLen];
	for(int i = bufLen; --i>=0;){
		fbuf[i] = 0;
	}
		
	M_Node& m_chords = larynx->chords;
	for(int k = m_chords.length; --k >= 0;){
		if(Chords* chords = dynamic_cast<Chords*>(m_chords[k].p)){
			M_Float& phase = chords->phase;
			M_Float& m_multiple = chords->multiple;
			M_Float& amplitude = chords->amplitude;


			int harmonics = m_multiple.length;

			const Vec2f* fPointPtr = osc.note.refValue().getArray();
			const float* fWeightPtr = osc.fWeight.refValue().getArray();
			const float* fKnotPtr = osc.fKnot.refValue().getArray();
			const int fOrder = osc.fOrder;
			const int dim = osc.note.refValue().length;
			const float maxT = osc.maxT();
			const float minT = osc.minT();
			const int fDegree = osc.fDegree;

			int idelay = 0;

			float step = 1.0 / 200;

			for(int h = 0; h < harmonics; h++){//h=harmonic
				double a = PI * 2 * phase[h];

				double multiply = m_multiple[h];
				double harmonicAmplitude = amplitude[h];

				//////////frequency
				float tc = 0;
				float t = (maxT-minT)*tc + minT;


				Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

				tc += step;
				t = (maxT-minT)*tc + minT;
				Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
				/////////
				int s = 0;
				int j = idelay + s;
	
				//getToFreqToPatchStart(s, j, pointPtr, bufLen, fFrom, fTo, t, tc, step, minT, maxT, detuneAdd, a, fPointPtr, fWeightPtr, fKnotPtr, fDegree, dim, multiply);

				for(int i = 2; j < bufLen; s++, j++){//s=sample
			
					float duration = s * (1.0/44100.0);


					//frequency
					if(duration > fTo.x){
						tc += step;
						if(tc > 1.0){
							break;
						}
						fFrom=fTo;
						t = (maxT-minT)*tc + minT;
						fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
					}
					Vec2f fDiff = fTo - fFrom;
					float fOffset = duration - fFrom.x;
					double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
					cent*=100;
					//
					double freq = 220; //A4 = 440 hertz so why an octive lower?
					freq = freq * pow((pow(2.0, 1.0/1200.0)),cent) * multiply;

					a += (freq / 44100.0) * PI * 2;

					fbuf[j] += volume * sin(a) * harmonicAmplitude * 1;
					//fbuf[j] += volume * (sin(a) * .5 + .5) * harmonicAmplitude; //monopolar
				}
			}
		}
	}



	short int* dstBuf = new short int[bufLen];

	const float scaleToInt = 0x7fff;
	for(int i = bufLen; --i>=0;){
		dstBuf[i] = static_cast<short int>(fbuf[i] * 0x7fff);
	}

	FILE* writeStream = fopen("am.wav", "w+b");
	((int*)formatM16)[1] = (bufLen * 2 + WAVE_HEADER_SIZE - 8);
	((int*)formatM16)[10] = bufLen * 2;
	int sampleRate = ((int*)formatM16)[24];
	fwrite(formatM16, 1, WAVE_HEADER_SIZE, writeStream);
	fwrite(dstBuf, 2, bufLen, writeStream);
	fclose(writeStream);

}

void FrequencyModulator::genCarrierSignal(const float volume, float soundPeriod, const Oscillator& osc){
	Larynx* larynx = dynamic_cast<Larynx*>(osc.larynx.p);
	if(larynx == 0){
		return;
	}
	bufLen = 44100 / soundPeriod;
	fbuf = new float[bufLen];
	for(int i = bufLen; --i>=0;){
		fbuf[i] = 0;
	}
		

	M_Node& m_chords = larynx->chords;
	for(int k = m_chords.length; --k >= 0;){
		if(Chords* chords = dynamic_cast<Chords*>(m_chords[k].p)){
			M_Float& phase = chords->phase;
			M_Float& m_multiple = chords->multiple;
			M_Float& amplitude = chords->amplitude;

			int harmonics = m_multiple.length;

			const Vec2f* fPointPtr = osc.note.refValue().getArray();
			const float* fWeightPtr = osc.fWeight.refValue().getArray();
			const float* fKnotPtr = osc.fKnot.refValue().getArray();
			const int fOrder = osc.fOrder;
			const int dim = osc.note.refValue().length;
			const float maxT = osc.maxT();
			const float minT = osc.minT();
			const int fDegree = osc.fDegree;

			int idelay = 0;

			float step = 1.0 / 200;

			for(int h = 0; h < harmonics; h++){//h=harmonic
				double a = PI * 2 * phase[h];

				double multiply = m_multiple[h];
				double harmonicAmplitude = amplitude[h];

				//////////frequency
				float tc = 0;
				float t = (maxT-minT)*tc + minT;


				Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

				tc += step;
				t = (maxT-minT)*tc + minT;
				Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
				/////////
				int s = 0;
				int j = idelay + s;
	
				//getToFreqToPatchStart(s, j, pointPtr, bufLen, fFrom, fTo, t, tc, step, minT, maxT, detuneAdd, a, fPointPtr, fWeightPtr, fKnotPtr, fDegree, dim, multiply);

				for(int i = 2; j < bufLen; s++, j++){//s=sample
			
					float duration = s * (1.0/44100.0);


					//frequency
					if(duration > fTo.x){
						tc += step;
						if(tc > 1.0){
							break;
						}
						fFrom=fTo;
						t = (maxT-minT)*tc + minT;
						fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
					}
					Vec2f fDiff = fTo - fFrom;
					float fOffset = duration - fFrom.x;
					double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
					cent*=100;
					//
					double freq = 220; //A4 = 440 hertz so why and octive lower?
					freq = freq * pow((pow(2.0, 1.0/1200.0)),cent) * multiply;

					a += (freq / 44100.0) * PI * 2;

					fbuf[j] += volume * sin(a) * harmonicAmplitude * 1;
					//fbuf[j] += volume * (sin(a) * .5 + .5) * harmonicAmplitude; //monopolar
				}
			}
		}
	}


	short int* dstBuf = new short int[bufLen];

	const float scaleToInt = 0x7fff;
	for(int i = bufLen; --i>=0;){
		dstBuf[i] = static_cast<short int>(fbuf[i] * 0x7fff);
	}

	FILE* writeStream = fopen("fm.wav", "w+b");
	((int*)formatM16)[1] = (bufLen * 2 + WAVE_HEADER_SIZE - 8);
	((int*)formatM16)[10] = bufLen * 2;
	int sampleRate = ((int*)formatM16)[24];
	fwrite(formatM16, 1, WAVE_HEADER_SIZE, writeStream);
	fwrite(dstBuf, 2, bufLen, writeStream);
	fclose(writeStream);

}

void Art::randomizeHarmonicFreq(){
	double range_minf = -.01;
	double range_maxf =  .01;
	M_Vec3f& coord = this->coord.refValue();

	for(int i = coord.length; --i>=0;){
		coord[i].z += ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
	}
	smls->ctrf = true;
}

void randomizeHarmonicFreq(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		part->randomizeHarmonicFreq();
	}
}
/*
void randomizeHarmonicAmplitude(Oscillator* osc){
	double range_minf = -.1;
	double range_maxf =  .1;
	M_Float& amplitude = osc->amplitude.refValue();

	for(int i = amplitude.length; --i>=0;){
		amplitude[i] += amplitude[i] * ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
	}
}
*/

int getFileSize(FILE* stream);


void loadSDIF(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		String& str=g_loadStr;
		str="";
		str = readFile("C:\\Users\\thyme\\Documents\\Audio\\anita\\sentence\\very-nice-simplification-slow-eye.txt");
		int i = 0;
		str.stnl(i);
		str.stnl(i);
		str.stwo_stb(i);
		int numOfHarmonics = getInt(str, i);
		str.stnl(i);

		Seamless* smls = part->smls;
		M_Vec3f& coord = part->coord.refValue();
		Effect* effect=part->workEffect;
		int effectLength = effect->getEffectLength();
		part->deleteGeometry();
		M_Triangle& triangle = part->triangle;
		

		for(int h = 0; h < numOfHarmonics; h++){
			str.stwo_stb(i);
			getInt2(str, i);
			int steps = getInt2(str, i);

			getFloat2(str, i);
			getFloat2(str, i);

			for(int k = 0; k < steps; k++){
				float x = getFloat2(str, i);
				float z = -getFloat2(str, i) / 10000;
				float y = getFloat2(str, i);
				int a = coord.length;
				int b = a + 1;
				int c = b + 1;
				coord += Vec3f(x - .008, y, z);
				coord += Vec3f(x + .004, y, z + .008 * 0.866025404);
				coord += Vec3f(x + .004, y, z - .008 * 0.866025404);

				triangle += S_Triangle(
					part, a, effectLength + a,
					part, b, effectLength + b,
					part, c, effectLength + c
				);
			}
		}

		dynamic_cast<ColorEffect*>(effect)->color.refValue().length += coord.length;


		smls->resetRelevantParts();
		smls->process();
		smls->verifyIntegrity();
		smls->ctrf = true;

	}
}

void HarmonicPatch::updateVCoord(const Oscillator& osc){

	float step = 1.0 / this->staves;

	const float* uKnotPtr = this->uKnot.refValue().getArray();
	const float* vKnotPtr = this->vKnot.refValue().getArray();
	const Vec3f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order.x;
	const int vOrder = this->order.y;
	const int uDim = this->dim.x;
	const int vDim = this->dim.y;
	M_Float& vCoord = this->vCoord;
	const float maxU=this->maxU();
	const float maxV=this->maxV();
	const float minU=this->minU();
	const float minV=this->minV();

	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const Vec2f* fPointPtr = osc.note.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();

	const Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
	const float* topWeightPtr = weightPtr+uDim*vDim-uDim;

	Vec3f* r_pointPtr = r_point.getArray();
	float* r_weightPtr = r_weight.getArray();


	const int fOrder = osc.fOrder;
	const int dim = osc.note.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int fDegree = osc.fDegree;

	M_Vec3f& coord = this->coord.refValue();

	for(int h = 0; h < harmonics; h++){//h=harmonic
		double a = 0;
		double multiply = h + 1;

		//////////frequency
		float tc = 0;
		float t = (maxT - minT) * tc + minT;

		Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

		tc += step;
		t = (maxT-minT)*tc + minT;
		Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
		/////////

		for(int s = 0; s < staves + 1; s++){
			int i2 = h * (staves + 1) + s;
			Vec3f crd = coord[i2];
			float vc = vCoord[i2];
			if(vc > 1){
				vc = .9999;
			}

			for(int z = 10; --z >=0 && crd.x > fTo.x;){
				fFrom = fTo;
				tc += step;
				if(tc >= 1.0){
					tc = 1;
					t = (maxT - minT) * tc + minT;
					fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
					break;
				}
				t = (maxT - minT) * tc + minT;
				fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
			}
			Vec2f fDiff = fTo - fFrom;
			float fOffset = crd.x - fFrom.x;
			double cent = fFrom.y + (fOffset / fDiff.x) * fDiff.y;
			cent *= 100;
			//
			double freq = 220; //A4 = 440 hertz so why and octive lower?
			freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);
			freq = freq * multiply * -0.0001; //invert and scale for patch

			if(abs(freq) > abs(crd.z)){
				vc += .0001;
			}
			else{
				vc -= .0001;
			}
			if(vc < 0){
				vc = 0;
			}

			float uc = s * (1.0 / staves);

			float u = (maxU - minU) * uc + minU;
			float v = (maxV - minV) * vc + minV;
			Vec3f testCrd;

			if(uc==1){
				CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, testCrd);
			}
			else if(vc==1){
				CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, testCrd);
			}
			else{
				SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &testCrd);
			}


			//SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &testCrd);
			if(abs(freq - testCrd.z) < abs(freq - crd.z)){
				coord[i2] = testCrd;
				vCoord[i2] = vc;
			}
		}
	}
}

/*
void NurbsPatch::updateVCoord(const Oscillator& osc){
	if(this->stem.length == 0){
		return;
	}
	Stem* stem = dynamic_cast<Stem*>(this->stem.refValue()[0].p);
	if(stem->part.length == 0){
		return;
	}
	Art* part = stem->part.refValue()[0].p;
	if(stem->bands.length == 0){
		return;
	}
	Larynx* larynx = dynamic_cast<Larynx*>(osc.larynx.p);
	if(larynx == 0){
		return;
	}
	M_Float& m_multiple = larynx->multiple.refValue();


	int harmonics = stem->bands[0] + 1;
	if(harmonics > m_multiple.length){
		harmonics = m_multiple.length;
	}
	int staves = stem->staves;

	float step = 1.0 / stem->staves;

	const float* uKnotPtr = this->uKnot.refValue().getArray();
	const float* vKnotPtr = this->vKnot.refValue().getArray();
	const Vec3f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order.x;
	const int vOrder = this->order.y;
	const int uDim = this->dim.x;
	const int vDim = this->dim.y;
	M_Vec2f& vCoord = stem->coord;
	const float maxU=this->maxU();
	const float maxV=this->maxV();
	const float minU=this->minU();
	const float minV=this->minV();

	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const Vec2f* fPointPtr = osc.note.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();

	const Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
	const float* topWeightPtr = weightPtr+uDim*vDim-uDim;

	Vec3f* r_pointPtr = r_point.getArray();
	float* r_weightPtr = r_weight.getArray();

	double increment = 1;
	double multiple = 2;
	for(int h = 1; h < harmonics; h++){//h=harmonic
		double a = 0;
		

		//////////frequency


		/////////

		for(int s = 0; s < staves + 1; s++){
			int i2 = h * (staves + 1) + s;
//			Vec3f crd = coord[i2];
			float remVc = 1.0 / (harmonics - 1) * h;
			float vc = remVc;
			if(vc > 1){
				vc = .9999;
			}
			float uc = s * (1.0 / staves);

			float u = (maxU - minU) * uc + minU;
			float v = (maxV - minV) * vc + minV;

			Vec3f crd;
			CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, crd);


			double freq = 220 * multiple * -0.0001; //invert and scale for patch

			float amount = .01;
			int z = 1000;
			while(true){
				if(--z <= 0){
					return;
				}
				if(abs(freq) > abs(crd.z)){
					vc += amount;
				}
				else{
					vc -= amount;
				}
				if(vc < 0){
					vc = 0;
				}

				uc = s * (1.0 / staves);

				u = (maxU - minU) * uc + minU;
				v = (maxV - minV) * vc + minV;

				Vec3f testCrd;
				CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, testCrd);


				float diff = abs(freq - testCrd.z);
				if(diff < abs(freq - crd.z)){
					crd = testCrd;
					remVc = vc;
				}
				else{
					vc = remVc;
					amount = amount * .5;
				}
				if(amount < .00001){
					vCoord[i2].y = vc;
					break;
				}
			}
		}
		multiple += increment;
	}
}
*/
Art* storeGeneratedPolyMap(Stem* stem);

int g_oscPartCoordLen = 0;

int stretchPartials(M_Vec3f*& oscCoord, Chords* osc){
	Seamless* smls = dynamic_cast<Seamless*>(g_scene->children[0].p);
	GroupBuild* group = dynamic_cast<GroupBuild*>(smls->build[0].p);
	NurbsPatch* partials = dynamic_cast<NurbsPatch*>(group->build[0].p);
	Stem* stem = dynamic_cast<Stem*>(partials->stem[0].p);
	Art* oscPart = dynamic_cast<Part*>(stem->part[0].p);

	g_oscPartCoordLen = oscPart->coord.length;

	int staves = stem->staves[0];
	int bands = stem->bands[0];
	int staveSteps = staves + 1;
	int bandSteps = bands + 1;
	int total = staveSteps * bandSteps;

	osc->addCoordsPatch();
	oscCoord = &oscPart->coord.refValue();

	for(int k = osc->uvCoord.length; --k>= 0;){
		oscPart->uvCoord.refValue()[k] = osc->uvCoord[k];
	}


	return staveSteps;
}

void Chords::addCoordsPatch(){
	M_Vec3f& coord = this->coord;
	M_Vec2f& uvCoord = this->uvCoord;

	float increment = .009;

	Seamless* smls = dynamic_cast<Seamless*>(g_scene->children[0].p);
	GroupBuild* group = dynamic_cast<GroupBuild*>(smls->build[0].p);
	NurbsPatch* partials = dynamic_cast<NurbsPatch*>(group->build[0].p);
	Stem* stem = dynamic_cast<Stem*>(partials->stem[0].p);

	int staves = stem->staves[0];
	int bands = stem->bands[0];

	int staveSteps = staves + 1;
	int bandSteps = bands + 1;
	int total = staveSteps * bandSteps;

	coord.length = total;
	uvCoord.length = total;

	float v = 0;
	for(int k = 0; k < bandSteps; k++){
		for(int i = 0; i < staveSteps; i++){
			Vec2f vec;
			vec.x = 1.0f / staves * i;
			vec.y = v;
			uvCoord[k * staveSteps + i] = vec;
		}
		v+=increment;
	}


}

void NurbsPatch::updateVCoordChords(int& limit, int staves, const M_Float& phase, const M_Float& m_multiple, float randomize, M_Vec3f& coord, M_Vec2f& vCoord, Oscillator* synth, bool doPatch, M_Vec3f& oscCoord, int oscStaveSteps){


	int numOfHarmonics = m_multiple.length;
	if(numOfHarmonics > limit){
		numOfHarmonics = limit;
	}

	int chordsCoordLen = (staves + 1) * numOfHarmonics;

	int coordOffset = coord.length;
	coord.length += chordsCoordLen;
	vCoord.length += chordsCoordLen;

	float step = 1.0 / staves;

	const float* uKnotPtr = this->uKnot.refValue().getArray();
	const float* vKnotPtr = this->vKnot.refValue().getArray();
	const Vec3f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order.x;
	const int vOrder = this->order.y;
	const int uDim = this->dim.x;
	const int vDim = this->dim.y;
	const float maxU=this->maxU();
	const float maxV=this->maxV();
	const float minU=this->minU();
	const float minV=this->minV();

	const float* fKnotPtr = synth->fKnot.refValue().getArray();
	const Vec2f* fPointPtr = synth->note.refValue().getArray();
	const float* fWeightPtr = synth->fWeight.refValue().getArray();

	const Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
	const float* topWeightPtr = weightPtr+uDim*vDim-uDim;

	Vec3f* r_pointPtr = r_point.getArray();
	float* r_weightPtr = r_weight.getArray();


	const int fOrder = synth->fOrder;
	const int dim = synth->note.refValue().length;
	const float maxT = synth->maxT();
	const float minT = synth->minT();
	const int fDegree = synth->fDegree;

	float stretch = 0.0;
	Vec3f oFrom;
	Vec3f oTo;


	for(int h = 0; h < numOfHarmonics; h++){//h=harmonic
		int oscStepIdx = h * oscStaveSteps;
		if(doPatch){
			oFrom = oscCoord[oscStepIdx++];
			oTo = oscCoord[oscStepIdx++];
		}

		double multiple = m_multiple[h];

		//////////frequency
		float tc = 0;
		float t = (maxT - minT) * tc + minT;

		Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

		tc += step;
		t = (maxT-minT)*tc + minT;
		Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
		/////////
		float vc = 1.0 / (numOfHarmonics + 1) * h;

		for(int s = 0; s < staves + 1; s++){
			int i2 = coordOffset + h * (staves + 1) + s;


			Vec3f crd;
			float uc = s * (1.0 / staves);
			vCoord[i2] = Vec2f(uc, vc);

			float u = (maxU - minU) * uc + minU;
			float v = (maxV - minV) * vc + minV;

			if(uc==1){
				CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, crd);
			}
			else if(vc==1){
				CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, crd);
			}
			else{
				SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &crd);
			}

			double amp = 1;
			if(doPatch){
				while(crd.x > oTo.x){
					if(oscStepIdx >= (h + 1) * oscStaveSteps){
						break;
					}
					oFrom = oTo;
					oTo = oscCoord[oscStepIdx++];
				}
				Vec3f oDiff = oTo - oFrom;
				double oOffset = crd.x - oFrom.x;
				amp = (oFrom.y + (oOffset / oDiff.x) * oDiff.y) * 10;
				double partialFreq = oFrom.z + (oOffset / oDiff.x) * oDiff.z;
				double increment = .009 * 2.0;
				double scaleMultiple = -(1.0 /  increment);
				multiple = partialFreq * scaleMultiple + 1.0;
			}


			while(crd.x > fTo.x){
				fFrom = fTo;
				tc += step;
				if(tc >= 1.0){
					tc = 1;
					t = (maxT - minT) * tc + minT;
					fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
					break;
				}
				t = (maxT - minT) * tc + minT;
				fTo = curvePoint2d(dim - 1, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t);
			}
			Vec2f fDiff = fTo - fFrom;
			float fOffset = crd.x - fFrom.x;
			double cent = fFrom.y + (fOffset / fDiff.x) * fDiff.y;
			//if(randomize != 0){
			//	double range_minf = -randomize;
			//	double range_maxf =  randomize;

			//	cent += ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
			//}

			//double gf = crd.x * 8;
			//if(gf > 1.0){
			//	gf = 1;
			//}
			//mult = multiple * (1.0 + stretch * (1.0 - gf));
			//if(oscStepIdx > 7){
			//	mult = multiple;
			//}

			cent *= 100;
			//
			double freq = 220; //A4 = 440 hertz so why an octive lower?
			freq = freq * pow((pow(2.0, 1.0 / 1200.0)), cent);
			freq = freq * multiple * -0.0001; //invert and scale for patch





			float amount = .01;
			int z = 1000;

			while(true){
				if(--z <= 0){
					int sub = chordsCoordLen - (staves + 1) * h;
					coord.length -= sub;
					vCoord.length -= sub;
					limit = h;
					goto here;
				}
				if(abs(freq) > abs(crd.z)){
					vc += amount;
				}
				else{
					vc -= amount;
				}
				if(vc < 0){
					vc = 0;
				}
				if(vc > 1){
					vc = .9999;
				}

				float uc = s * (1.0 / staves);

				float u = (maxU - minU) * uc + minU;
				float v = (maxV - minV) * vc + minV;
				Vec3f testCrd;

				if(uc==1){
					CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, testCrd);
				}
				else if(vc==1){
					CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, testCrd);
				}
				else{
					SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &testCrd);
				}


				//SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &testCrd);
				float diff = abs(freq - testCrd.z);
				if(diff < abs(freq - crd.z)){
					crd = testCrd;
					vCoord[i2].y = vc;
				}
				else{
					vc = vCoord[i2].y;
					amount = amount * .5;
				}
				if(amount < .00001){
					crd.y *= amp;
					coord[i2] = crd;
					break;
				}
			}
		}
		//stretch += .05;
	}
here:
	if(randomize != 0){
		double range_minf = -randomize;
		double range_maxf =  randomize;

		for(int i = coord.length; --i>=0;){
			coord[i].z += ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
		}
	}

	if(doPatch){
		Seamless* smls = dynamic_cast<Seamless*>(g_scene->children[0].p);
		GroupBuild* group = dynamic_cast<GroupBuild*>(smls->build[0].p);
		NurbsPatch* patchFilter = dynamic_cast<NurbsPatch*>(group->build[1].p);
		Stem* stem = dynamic_cast<Stem*>(patchFilter->stem[0].p);
		Art* filter = dynamic_cast<Part*>(stem->part[0].p);

		for(int k = vCoord.length; --k>=0;){
			filter->uvCoord.refValue()[k] = vCoord[k];
		}
	}
}

void NurbsPatch::updateVCoord(Oscillator& synth){
	Larynx* larynx = dynamic_cast<Larynx*>(synth.larynx.p);
	if(larynx == 0){
		return;
	}
	M_Vec3f& coord = synth.coord.refValue();
	M_Vec2f& vCoord = synth.vCoord;

	coord.length = 0;
	vCoord.length = 0;

	M_Node& m_chords = larynx->chords;
	M_Int& limit = synth.limit.refValue();
	int limitIdx = 0;

	M_Vec3f* oscCoord = 0;
	int oscStaveSteps = 0;
	bool doPatch = false;

	for(int k = 0; k < m_chords.length; k++){
		if(Chords* chords = dynamic_cast<Chords*>(m_chords[k].p)){
			doPatch = false;
			if(chords->patials.p){
				oscStaveSteps = stretchPartials(oscCoord, chords);
				doPatch = true;
			}
			updateVCoordChords(limit[limitIdx], synth.staves, chords->phase, chords->multiple, chords->randomize, synth.coord, synth.vCoord, &synth, doPatch, *oscCoord, oscStaveSteps);
			limitIdx++;
		}
		else if(Envelope* envelope = dynamic_cast<Envelope*>(m_chords[k].p)){
			M_Node& m_chords = envelope->chords.refValue();
			for(int k = 0; k < m_chords.length; k++){
				if(Chords* chords = dynamic_cast<Chords*>(m_chords[k].p)){
					doPatch = false;
					if(chords->patials.p){
						oscStaveSteps = stretchPartials(oscCoord, chords);
						doPatch = true;
					}
					updateVCoordChords(limit[limitIdx], synth.staves, chords->phase, chords->multiple, chords->randomize, synth.coord, synth.vCoord, &synth, doPatch, *oscCoord, oscStaveSteps);
					limitIdx++;
				}
			}
		}
	}

	/*
	if(this->stem.length){
		Stem* stem = dynamic_cast<Stem*>(this->stem.refValue()[0].p);

		if(Larynx* larynx = dynamic_cast<Larynx*>(osc.larynx.p)){
			int harmonics = larynx->multiple.length;
			if(harmonics > osc.limit){
				harmonics = osc.limit;
			}
			if(stem->part.length){
				if(Art* part = stem->part.refValue()[0].p){
					stem->staves.refValue() = osc.staves;
					stem->bands.refValue().length = 1;
					stem->bands.refValue()[0] = harmonics - 1;
					stem->bands.refValue().length = 1;
					stem->bands.refValue()[0] = harmonics - 1;
					stem->yDistance.refValue().length = 1;
					stem->yDistance.refValue()[0] = 1;
					stem->height.refValue().length = 2;
					stem->height.refValue()[0] = 0;
					stem->height.refValue()[1] = 1;
			
					stem->add.refValue() = false;

					if(stem->coord.length != osc.vCoord.length){
						stem->store.refValue() = false;
						stem->processFields();
						part->smls->doBuild();
						storeGeneratedPolyMap(stem);
					}


					stem->coord.refValue() = osc.vCoord;
					stem->color.refValue().length = stem->coord.length;
					stem->store.refValue() = true;
					stem->processFields();
					part->smls->doBuild();
					//storeGeneratedPolyMap(stem);

					part->smls->ctrf = true;
				}
			}
		}
		*/
}

void Chords::updateSpectrumPatch(){
}

void Chords::updateSpectrum(){

	float step = 1.0 / (harmonics - 1);

	const float* knotPtr = this->knot.getArray();
	const Vec2f* pointPtr = this->spectrum.refValue().getArray();
	const float* weightPtr = this->weight.getArray();
	const int dim = spectrum.length;
	const float maxT=this->maxT();
	const float minT=this->minT();

	Vec2f s0 = spectrum[0];
	coord[0] = Vec3f(s0.x, s0.y, 0);

	double multiple = 1.0 + increment;
	for(int h = 1; h < harmonics; h++){//h=harmonic
		double a = 0;
		

		//////////frequency


		/////////

		float remVc = 1.0 / (harmonics - 1) * h;
		float vc = remVc;
		if(vc >= 1.0f){
			vc = .9999;
		}

		float v = (maxT - minT) * vc + minT;

		Vec2f crd = curvePoint2d(dim, degree, knotPtr, pointPtr, weightPtr, v, vc);


		double freq = spectrum[0].x * multiple; //scale for nurbs curve

		float amount = .01;
		int z = 1000;
		while(true){
			if(--z <= 0){
				return;
			}
			if(freq > crd.x){
				vc += amount;
			}
			else{
				vc -= amount;
			}
			if(vc < 0){
				vc = 0;
			}

			v = (maxT - minT) * vc + minT;

			Vec2f testCrd = curvePoint2d(dim, degree, knotPtr, pointPtr, weightPtr, v, vc);


			float diff = abs(freq - testCrd.x);
			if(diff < abs(freq - crd.x)){
				crd = testCrd;
				remVc = vc;
			}
			else{
				vc = remVc;
				amount = amount * .5;
			}
			if(amount < .00001){
				coord[h] = Vec3f(crd.x, crd.y, 0);
				break;
			}
		}
		multiple += increment;
	}
}


void Envelope::updateCoord(){

	float step = 1.0 / (resolution - 1);

	const float* knotPtr = this->knot.getArray();
	const Vec2f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.getArray();
	const int dim = point.length;
	const float maxT=this->maxT();
	const float minT=this->minT();

	coord[0] = point[0];

	for(int h = 1; h < resolution; h++){//h=harmonic
		double a = 0;
		

		//////////frequency


		/////////

		float remVc = 1.0 / (resolution - 1) * h;
		float vc = remVc;
		if(vc >= 1.0f){
			vc = .9999;
		}

		float v = (maxT - minT) * vc + minT;

		coord[h] = curvePoint2d(dim, degree, knotPtr, pointPtr, weightPtr, v, vc);
	}
}

void AmplitudeModulator::updateWetCoord(){

	float step = 1.0 / (resolution - 1);

	const float* knotPtr = this->knot.getArray();
	const Vec2f* pointPtr = this->wet.refValue().getArray();
	const float* weightPtr = this->weight.getArray();
	const int dim = wet.length;
	const float maxT=this->maxT();
	const float minT=this->minT();

	coord[0] = wet[0];

	for(int h = 1; h < resolution; h++){//h=harmonic
		double a = 0;
		

		//////////frequency


		/////////

		float remVc = 1.0 / (resolution - 1) * h;
		float vc = remVc;
		if(vc >= 1.0f){
			vc = .9999;
		}

		float v = (maxT - minT) * vc + minT;

		coord[h] = curvePoint2d(dim, degree, knotPtr, pointPtr, weightPtr, v, vc);
	}
}


void FrequencyModulator::updateWetCoord(){
}

void Sample::genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen){
	float step=1.0/1000.0;

	const float* fKnotPtr = osc.fKnot.refValue().getArray();
	const Vec2f* fPointPtr = osc.note.refValue().getArray();
	const float* fWeightPtr = osc.fWeight.refValue().getArray();
	const int fOrder = osc.fOrder;
	const int dim = osc.note.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int fDegree = osc.fDegree;

	int idelay = delay * 44100.0;

	float detuneStep=detune/voices;
	float detuneAdd=-detune*.5;

	for(int d=0; d<voices; d++){
		double a = 0;

		//////////frequency
		float tc = 0;
		float t = (maxT-minT)*tc + minT;

		Vec2f fFrom = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);

		tc += step;
		t = (maxT-minT)*tc + minT;
		Vec2f fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
		/////////
		for(int s = 0, j = idelay; j<bufLen && s < srcBufLen; s++, j++){

			float duration = s * (1.0/44100.0);
			//frequency
			if(duration > fTo.x){
				tc += step;
				if(tc >= 1.0){
					break;
				}
				fFrom=fTo;
				t = (maxT-minT)*tc + minT;
				fTo = curvePoint2d(dim, fDegree, fKnotPtr, fPointPtr, fWeightPtr, t, tc);
			}
			Vec2f fDiff = fTo - fFrom;
			float fOffset = duration - fFrom.x;
			double cent = fFrom.y + (fOffset/fDiff.x) * fDiff.y;
			cent*=100;
			cent += detuneAdd;
			//
			double freq=220;
			freq = freq * pow((pow(2.0,1.0/1200.0)),cent);

			a+=(freq/44100.0)*220;
			int fs=a;
			if(fs >= srcBufLen){
				break;
			}
			fbuf[j] += srcBuf[fs] * (1.0 / 0x7fff) * volume;
		}
		detuneAdd+=detuneStep;
	}
}
void Sample::processFields(){
	char header[0x2c];
	String& url = this->url.refValue();
	if(url==""){ 
		return;
	}
	if(FILE* stream = fopen(url.c_str(), "rb")){
		fread(header, 1, sizeof(header), stream);

		srcBufLen = ((long*) header)[0x28/4];
		srcBufLen>>=1;
		srcBuf = new short int[srcBufLen];
		int numOf = fread(srcBuf, sizeof(short int), srcBufLen, stream);

		fclose(stream);
	}
}
void Oscillator::removeControlPoint(int& newLen){
	M_Vec2f& fPoint = this->note;
	M_Float& fWeight = this->fWeight;
	M_Float& fKnot = this->fKnot;

	if(fPoint.length==2){
		return;
	}
	int idx=index[0];

	if(idx==0){
		return;
	}
	else if(idx == fPoint.length-1){
		fPoint.deleteElement(idx);
		fWeight.deleteElement(idx);
		idx--;
		index[0] = idx;
	}
	else {
		fPoint.deleteElement(idx);
		fWeight.deleteElement(idx);
	}
	updateNurbsCurveKnotMid0(fPoint.length, fKnot);
	newLen = fPoint.length;
}

void Chords::removeControlPoint(int& newLen){
	M_Vec2f& fPoint = this->spectrum;

	if(fPoint.length==2){
		return;
	}
	int idx=index[0];

	if(idx==0){
		return;
	}
	else if(idx == fPoint.length-1){
		fPoint.deleteElement(idx);
		weight.deleteElement(idx);
		idx--;
		index[0] = idx;
	}
	else {
		fPoint.deleteElement(idx);
		weight.deleteElement(idx);
	}
	updateNurbsCurveKnotMid0(fPoint.length, knot);
	newLen = fPoint.length;
}

void Harmonic::removeControlPoint(int& newLen){
	M_Vec2f& point = this->point;
	M_Float& weight = this->weight;
	M_Float& knot = this->knot;

	if(point.length==2){
		return;
	}
	int idx=index[0];

	if(idx==0){
		return;
	}
	else if(idx == point.length-1){
		point.deleteElement(idx);
		weight.deleteElement(idx);
		idx--;
		index[0] = idx;
	}
	else {
		point.deleteElement(idx);
		weight.deleteElement(idx);
	}
	updateNurbsCurveKnotMid0(point.length, knot);
	newLen = point.length;
}

void Oscillator::addControlPoint(int& newLen){
	M_Vec2f& fPoint = this->note;
	M_Float& fWeight = this->fWeight;
	M_Float& fKnot = this->fKnot;

	if(fPoint.length==2){
		fWeight=M_Float(1,1);
	}
	int idx=index[0];
	if(idx < fPoint.length-1){
		idx++;
	}
	fPoint.insertElement(idx, (fPoint[idx] + fPoint[idx-1]) * Vec2f(.5,.5));
	fWeight.insertElement(idx, 1);

	updateNurbsCurveKnotMid0(fPoint.length, fKnot);

	index[0] = idx;
	newLen = fPoint.length;
}

void Chords::addControlPoint(int& newLen){
	M_Vec2f& fPoint = this->spectrum;

	if(fPoint.length==2){
		weight=M_Float(1,1);
	}
	int idx=index[0];
	if(idx < fPoint.length-1){
		idx++;
	}
	fPoint.insertElement(idx, (fPoint[idx] + fPoint[idx-1]) * Vec2f(.5,.5));
	weight.insertElement(idx, 1);

	updateNurbsCurveKnotMid0(fPoint.length, knot);

	index[0] = idx;
	newLen = fPoint.length;
}

void Harmonic::addControlPoint(int& newLen){
	M_Vec2f& point = this->point;
	M_Float& weight = this->weight;
	M_Float& knot = this->knot;

	if(point.length==2){
		weight=M_Float(1,1);
	}
	int idx=index[0];
	if(idx < point.length-1){
		idx++;
	}
	point.insertElement(idx, (point[idx] + point[idx-1]) * Vec2f(.5,.5));
	weight.insertElement(idx, 1);

	updateNurbsCurveKnotMid0(point.length, knot);

	index[0] = idx;
	newLen = point.length;
}

int genNote(){
	return 0;
}
unsigned short getMidiWord(unsigned char *ptr, int i){
	return ((ptr[i]) << 8)+ ptr[i+1];
}
unsigned getMidiInt24(unsigned char *ptr, unsigned& i){
	unsigned a = ptr[i++];
	a <<= 8;
	a += ptr[i++];
	a <<= 8;
	a += ptr[i++];
	return a;
}
unsigned getMidiDWord(unsigned char *ptr, int i){
	return ((((((ptr[i]) << 8) + ptr[i+1]) << 8) + ptr[i+2]) << 8) + ptr[i+3];
}
unsigned getMidiInt(unsigned char *ptr, unsigned& i){
	unsigned n = 0;
	unsigned u = 0;

	do{
		n = n << 7;
		u = ptr[i++];
		n += u & 0x7f;
	}while((u & 0x80) == 0x80);

	return n;
}
int getIndexForList(unsigned char noteNumber, M_<MidiNoteOnItem>& noteOnList){
	for(int k=noteOnList.length; --k>=0;){
		if(noteNumber == noteOnList[k].noteNumber){
			return k;
		}
	}
	return -1;
}
void noteOnOff(M_<MidiNoteOnItem>& noteOnList, const unsigned char* trackData, unsigned& on, const double& tempo, unsigned& delay, M_Node& sound, HarmonicPatch* harmonicPatch, const short& timeDivision){
	unsigned char noteNumber = trackData[on++];
	unsigned char velocity = trackData[on++];
		
	int k = getIndexForList(noteNumber, noteOnList);

	if(k>=0){
		Oscillator* osc = new Oscillator;
		double noteOnDelay = noteOnList[k].delay*(60.0/tempo/timeDivision);
		float pitch = noteNumber-57;
		osc->delay=noteOnDelay;
		osc->note.refValue()[0]=Vec2f(0, pitch);
		osc->note.refValue()[1]=Vec2f(delay*(60.0/tempo/timeDivision)-noteOnDelay, pitch);
		osc->volume=noteOnList[k].velocity/127.0;
		osc->filter.refValue()+=harmonicPatch;
		sound+=osc;
		noteOnList.deleteElement(k);
	}
	else{
		noteOnList += MidiNoteOnItem(delay, noteNumber, velocity);
	}
}
void openMidi(const String& filename){
	if(FILE* mhandle = fopen(filename.c_str(), "rb")){
		g_forceSaveAs = true;
		insertRecentFile(filename.c_str());
		unsigned char *midi_bufer = new unsigned char[0x4000];
		int fileLen=fread(midi_bufer, sizeof(unsigned char), 0x4000, mhandle);
		fclose(mhandle);

		M_Node& children = g_scene->children.refValue();
		children.length=0;

		Composition* composition = new Composition;
		children+=composition;

		M_<MidiNoteOnItem> noteOnList;
		double tempo = 120;
		int numerator = 4;
		int denominator = 4;
		int metronome = 24;
		int number_of_32nd_notes_per_quarter_note = 8;

		if(fileLen>22 && strcmp(reinterpret_cast<char*>(midi_bufer), "MThd")==0){

			M_Node& m_track = composition->track;

			unsigned short numOfTracks = getMidiWord(midi_bufer, 10);
			unsigned short timeDivision = getMidiWord(midi_bufer, 12);
			unsigned char* trackHeader = midi_bufer + 14;

			for(int trackNum=0; trackNum < numOfTracks; trackNum++){
				noteOnList.length=0;


				if(strcmp(reinterpret_cast<char*>(trackHeader), "MTrk")==0){
					unsigned trackDataLen = getMidiDWord(trackHeader, 4);

					unsigned char* trackData = trackHeader + 8;

					Track* track = new Track;
					Ptr<HarmonicPatch> harmonicPatch = new HarmonicPatch;
					harmonicPatch->name="harmonicPatch"+String(trackNum);
					m_track+=track;
					M_Node& sound=track->sound;

					int patch = 0;

					unsigned on = 0;
					unsigned off = 0;

					unsigned char midiCtrl = 0;
					unsigned delay = 0;

					while(on < trackDataLen){

						delay += getMidiInt(trackData, on);

						midiCtrl = trackData[on];
						if(midiCtrl<0x80){
							noteOnOff(noteOnList, trackData, on, tempo, delay, sound, harmonicPatch, timeDivision);
							continue;
						}
						on++;

						switch(midiCtrl & 0xf0){
							case 0x90://Note On Event
							case 0x80:{//Note Off Event
								noteOnOff(noteOnList, trackData, on, tempo, delay, sound, harmonicPatch, timeDivision);
								break;
							}
							case 0xf0:{
								if(midiCtrl == 0xff){
									switch(trackData[on++]){
										case 0x00:{//Sequence Number
											on+=2;
											break;
										}
										case 0x01:{//Text Event
											throw;
											break;
										}
										case 0x02:{//Copyright Notice
											int strLen=trackData[on++];
											on+=strLen;
											on++;
											break;
										}
										case 0x03:{//Sequence/Track Name
											int strLen = trackData[on++];
											String& name = track->name;
											for(int i = 0; i < strLen; i++){
												char chr = trackData[on++];
												if(chr <= ' '){
													chr = '_';
												}
												name += chr;
											}
											break;
										}
										case 0x2f:{//End Of Track
											on++;
											break;
										}
										case 0x51:{//Set Tempo
											on++;
											tempo = 60000000 / getMidiInt24(trackData, on);
											break;
										}
										case 0x58:{//Time Signature
											on++;
											numerator = trackData[on++];
											denominator = trackData[on++];
											metronome = trackData[on++];
											number_of_32nd_notes_per_quarter_note = trackData[on++];
											break;
										}
										case 0x59:{//Key Signature
											on++;
											int key = trackData[on++];
											int scale = trackData[on++];
											break;
										} 
										default:{//unknown
											on+=trackData[on++];
										}
									}
								}
								else throw;
								break;
							}
							case 0xb0:{//Controller Event
								int type=trackData[on++];
								int value=trackData[on++];
								switch(type){
									case 0x07:{//Main Volume
										track->volume=value/127.0;
										break;
									}
								}
								break;
							}
							case 0xc0:{//patch
								patch = trackData[on++];
								break;
							}
							default:{
								throw;
							}
						}
					}
					trackHeader = trackData + on;
				}
			}
		}
		g_scene->wOpen.refValue().length=0;
		g_scene->wPos.refValue().length=0;
		g_scene->wSize.refValue().length=0;

		g_scene->genNodeList2();
		postLoadProcessNodes(g_sceneTreeList);

		resetLooseEnds();
		Window::postLoadProcessWin(*g_scene);

		BackUp::deleteBackupFiles();
		BackUp::createUniqueBackupName();
		Window::activeWindow = g_stw.p;
		Window::activeWindow = g_stw->cp.p;

		g_stw->paint();
		Window::paintAllControlPanels();
		setSmlsSaved();
		setTitleName();
	}
}

Vec2f pitchArray[] = {Vec2f(0, -.0001), Vec2f(0, .0001), Vec2f(-.001, 0), Vec2f(.001, 0)};
//float phaseArray[] = {-.00001, .00001};

double compareWaveform(float* fbuf, int bufLen, float* sampleFBuf){
	double error = 0;

	for(int i = 0; i < bufLen; i++){
		error += abs(fbuf[i] - sampleFBuf[i]);
	}
	return error;
}

double compareWaveform(float catVolume, Track* track, Oscillator* osc, float* harmonicBuf, float* fbuf, int bufLen, float* sampleFBuf){
	for(int i = bufLen; --i>=0;){
		harmonicBuf[i]=0;
	}

	osc->buildSound(catVolume, osc->delay, track->voices,  track->detune, harmonicBuf, bufLen);
	double error = 0;

	for(int i = 0; i < bufLen; i++){
		error += abs(harmonicBuf [i] + fbuf[i] - sampleFBuf[i]);
	}
	return error;
}
double compareWaveform(float catVolume, Track* track, Oscillator* osc0, Oscillator* osc1, float* harmonicBuf0, float* harmonicBuf1, float* fbuf, int bufLen, float* sampleFBuf){
	for(int i = bufLen; --i>=0;){
		harmonicBuf0[i]=0;
		harmonicBuf1[i]=0;
	}

	osc0->buildSound(catVolume, osc0->delay, track->voices,  track->detune, harmonicBuf0, bufLen);
	osc1->buildSound(catVolume, osc1->delay, track->voices,  track->detune, harmonicBuf1, bufLen);
	double error = 0;

	for(int i = 0; i < bufLen; i++){
		error += abs(harmonicBuf0[i] + harmonicBuf1[i] + fbuf[i] - sampleFBuf[i]);
	}
	return error;
}

//double tugVertex(Oscillator* osc, int i, Vec3f pt, M_Vec3f& dpoint, M_Vec3f& point){
//	Vec3f remPoint = point[i];
//
//	point[i] = pt;
//
//	osc->processFields();
//	
//	double error = compareWaveform(float catVolume, Track* track, Oscillator* osc, 	float* fbuf, int bufLen, float* sampleFBuf, int srcBufLen);
//
//	if(testSumDiff<sumDiff){
//		sumDiff = testSumDiff;
//		averageDiff = testSumDiff/guessCoord.length;
//	}
//	else{//undo
//		point[i] = remPoint;
//		if(mirrorIdx>=0){
//			point[mirrorIdx] = remMirrorPoint;
//		}
//		patch->processFields();
//	}
//}


void clearBuffer(float* fbuf, int bufLen){
	for(int i = bufLen; --i>=0;){
		fbuf[i] = 0;
	}
}

void mixHarmonics(M_<float*>& m_harmonicBuf, float* fbuf, int bufLen, int omit0, int omit1){
	clearBuffer(fbuf, bufLen);

	for(int o = m_harmonicBuf.length; --o>=0;){
		if(omit0 == o || omit1 == o){
			continue;
		}
		float* harmonicBuf = m_harmonicBuf[o];
		for(int i = bufLen; --i>=0;){
			fbuf[i] += harmonicBuf[i];
		}
	}
}
inline void swapPointers(float*& a, float*& b){
	float* c = a;
	a = b;
	b = c;
}


void buildBuffers(M_Node& m_osc, int numOfHarmonics, M_<float*>& m_harmonicBuf, float catVolume, Track* track, float* fbuf, int bufLen, float* sampleFBuf){
	for(int o = 0; o < numOfHarmonics; o++){
		if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
			for(int k = 0; k < osc->filter.length; k++){
				if(Harmonic* harmonic = dynamic_cast<Harmonic*>(osc->filter[k].p)){
					//if(o & 1){
					//	harmonic->phase.refValue()[0] = .5;
					//}
					//else{
					//	harmonic->phase.refValue()[0] = 0;
					//}

					M_Vec2f& fPoint = osc->note.refValue();
					int vDim = harmonic->multiply.length;
					int uDim = harmonic->point.length / vDim;
					clearBuffer( m_harmonicBuf[o], bufLen);
					osc->buildSound(catVolume, osc->delay, track->voices,  track->detune, m_harmonicBuf[o], bufLen);
				}
			}
		}
	}
}

void findPhase(String& str, double& remError, M_Node& m_osc, int numOfHarmonics, M_<float*>& m_harmonicBuf, float catVolume, Track* track, float*& spareHarmBuf0, float* fbuf, int bufLen, float* sampleFBuf){
	for(int o = 0; o < numOfHarmonics; o++){
		if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
			for(int k = 0; k < osc->filter.length; k++){
				if(Harmonic* harmonic = dynamic_cast<Harmonic*>(osc->filter[k].p)){
					for(int g = 0; g < 10; g++){
						float remPhase = harmonic->phase[0];
						harmonic->phase.refValue()[0] = g / 10.0;

						buildBuffers(m_osc, numOfHarmonics, m_harmonicBuf, catVolume, track, fbuf, bufLen, sampleFBuf);
						mixHarmonics(m_harmonicBuf, fbuf, bufLen, -1, -1);
						double error = compareWaveform(fbuf, bufLen, sampleFBuf);

						if(error < remError){
							remError = error;
							//str = "phase wave error = ";
							//str += remError;
							//dLog(str);
						}
						else{
							harmonic->phase.refValue()[0] = remPhase;
						}
					}
				}
			}
		}
	}
}

void setAllAmpSame(M_Node& m_osc, int numOfHarmonics, Vec2f newValue, M_Vec2f& point){
	for(int o = 0; o < numOfHarmonics; o++){
		if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
			for(int k = 0; k < osc->filter.length; k++){
				if(Harmonic* harmonic = dynamic_cast<Harmonic*>(osc->filter[k].p)){
					for(int i = 0; i < point.length; i++){
						point[i] = newValue;
					}
				}
			}
		}
	}
}

void setAllPitchSame(M_Node& m_osc, int numOfHarmonics, float newValue){
	for(int o = 0; o < numOfHarmonics; o++){
		if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
			M_Vec2f& fPoint = osc->note.refValue();
			for(int i = 0; i < fPoint.length; i++){
				fPoint[i].y = newValue;
			}
		}
	}
}

void evolvePitchOrderly(String& str, double& remError, M_<float*>& m_harmonicBuf, M_Node& m_osc, int numOfHarmonics, float catVolume, Track* track, float*& spareHarmBuf0, float* fbuf, int bufLen, float* sampleFBuf){
	float pitchChange0 = .0001;
	for(int k = 0; k < 1; k++){
		if(Oscillator* osc0 = dynamic_cast<Oscillator*>(m_osc[0].p)){
			M_Vec2f& oscPoint0 = osc0->note.refValue();
			float remOscPoint0 = oscPoint0[0].y;
			for(int i = 0; i < 2; i++){
				if(i){
					setAllPitchSame(m_osc, numOfHarmonics, remOscPoint0 + pitchChange0);
				}
				else{
					setAllPitchSame(m_osc, numOfHarmonics, remOscPoint0 - pitchChange0);
				}

				buildBuffers(m_osc, numOfHarmonics, m_harmonicBuf, catVolume, track, fbuf, bufLen, sampleFBuf);
				mixHarmonics(m_harmonicBuf, fbuf, bufLen, -1, -1);
				double error = compareWaveform(fbuf, bufLen, sampleFBuf);

				if(error < remError){
					remError = error;
					//str = "pitch wave error = ";
					//str += remError;
					//dLog(str);
					break;
				}
				else{
					setAllPitchSame(m_osc, numOfHarmonics, remOscPoint0);
				}
			}
		}
	}
}

void evolveAmpSingleCycle(String& str, double& remError, int oscIdx0, int oscIdx1, Oscillator* osc0, Oscillator* osc1, Harmonic* harmonic0, Harmonic* harmonic1, M_<float*>& m_harmonicBuf, float catVolume, Track* track, float*& spareHarmBuf0, float*& spareHarmBuf1, float* fbuf, int bufLen, float* sampleFBuf){
	M_Vec2f& point0 = harmonic0->point;
	M_Vec2f& point1 = harmonic1->point;
	double range_minf = -.0001;
	double range_maxf =  .0001;

	for(int i = 0; i < 1; i++){

		float pointChange0 = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
		float pointChange1 = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);

		//int r = offset;
		//r++;
		//if(r>=randomLen){
		//	r = 0;
		//}
		//offset = r;

		float remPoint0 = point0[i].y;
		float remPoint1 = point1[i].y;


		float pt0 = remPoint0;
		float pt1 = remPoint1;


		pt0 += pointChange0;
		pt1 += pointChange1;

		if(pt0 < 0){
			pt0 = 0;
		}
		if(pt1 < 0){
			pt1 = 0;
		}

		for(int k = 0; k < point0.length; k++){
			point0[k].y = pt0;
			point1[k].y = pt1;
		}


		double error = compareWaveform(catVolume, track, osc0, osc1, spareHarmBuf0, spareHarmBuf1, fbuf, bufLen, sampleFBuf);
		if(error < remError){
			swapPointers(spareHarmBuf0, m_harmonicBuf[oscIdx0]);
			swapPointers(spareHarmBuf1, m_harmonicBuf[oscIdx1]);

			remError = error;
			//str = "wave error = ";
			//str += remError;
			//dLog(str);
		}
		else{
			for(int k = 0; k < point0.length; k++){
				point0[k].y = remPoint0;
				point1[k].y = remPoint1;
			}

		}
	}
}

void findPitch(int oscIdx0, int uDim, String& str, double& remError, M_<float*>& m_harmonicBuf, float catVolume, Track* track, Oscillator* osc0, float* spareHarmBuf0, float* fbuf, int bufLen, float* sampleFBuf){
	for(int u = 0; u < uDim; u++){
		for(int g = 0; g < 2000; g++){
			float remPitch0 = osc0->note[u].y;
			osc0->note.refValue()[u].y = -20 + 10 * (g / 2000.0);

			double error = compareWaveform(catVolume, track, osc0, spareHarmBuf0, fbuf, bufLen, sampleFBuf);
			if(error < remError){
				swapPointers(spareHarmBuf0, m_harmonicBuf[oscIdx0]);
				remError = error;
				//str = "pitch wave error = ";
				//str += remError;
				//dLog(str);
			}
			else{
				osc0->note.refValue()[u].y = remPitch0;
			}
		}
	}
}
void setAllPitchToFundamental(M_Node& m_osc, int numOfHarmonics){
	for(int o = 0; o < numOfHarmonics; o++){
		if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
			M_Vec2f& fPoint = osc->note.refValue();
			fPoint = dynamic_cast<Oscillator*>(m_osc[0].p)->note;
		}
	}
}

void changeFundamentalOnly(double& remError, M_Node& m_osc, int numOfHarmonics, M_<float*>& m_harmonicBuf, float catVolume, Track* track, float* fbuf, int bufLen, float* sampleFBuf){
	double range_minf = -.01;
	double range_maxf =  .01;
	Oscillator* osc0 = dynamic_cast<Oscillator*>(m_osc[0].p);
	Harmonic* harmonic0 = dynamic_cast<Harmonic*>(osc0->filter[0].p);
	M_Vec2f& oscPoint0 = osc0->note.refValue();
	int vDim = harmonic0->multiply.length;
	int uDim = harmonic0->point.length / vDim;
	int osc_uDim = osc0->note.length;

	for(int j = 0; j < osc_uDim - 1; j++){
		float pitchChange0 = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
		float pitchChange0b = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
		float remOscPoint0 = oscPoint0[j].y;
		float remOscPoint0b = oscPoint0[j+1].y;

		oscPoint0[j].y += pitchChange0;
		oscPoint0[j+1].y += pitchChange0b;

		oscPoint0[0].x = 0;
		
		setAllPitchToFundamental(m_osc, numOfHarmonics);
		buildBuffers(m_osc, numOfHarmonics, m_harmonicBuf, catVolume, track, fbuf, bufLen, sampleFBuf);
		mixHarmonics(m_harmonicBuf, fbuf, bufLen, -1, -1);
		double error = compareWaveform(fbuf, bufLen, sampleFBuf);
		if(error < remError){
			remError = error;
		}
		else{

			oscPoint0[j].y =  remOscPoint0;
			oscPoint0[j+1].y =  remOscPoint0b;
		}
	}
}

void Composition::evolveSound(){
	const int randomLen = 2;
	static int offset = 0;
	String filename = g_currentFilename;

	if(modifyWavFilename(filename)==false){
		msg(msg_Can_not_create_output_file_name,MB_OK | MB_ICONEXCLAMATION);
		return;
	};
	String str;

	duration = duration + .0125;

	int bufLen = 44100.0 * duration;
	//int bufLen=2;
	float* fbuf = new float[bufLen];
	M_Node& m_track = this->track.refValue();
	if(m_track.length == 1){
		if(Track* track = dynamic_cast<Track*>(m_track[0].p)){
			M_Node& m_osc = track->sound.refValue();
			int numOfHarmonics = m_osc.length - 1;
			if(numOfHarmonics > 0){

				float* spareHarmBuf0 = new float[bufLen];
				float* remSpareHarmBuf0 = spareHarmBuf0;
				float* spareHarmBuf1 = new float[bufLen];
				float* remSpareHarmBuf1 = spareHarmBuf1;

				float* harmonicBuf = new float[bufLen * numOfHarmonics];
				M_<float*> m_harmonicBuf;
				m_harmonicBuf.length = numOfHarmonics;
				for(int o = 0; o < numOfHarmonics; o++){
					m_harmonicBuf[o] = &harmonicBuf[bufLen * o];
				}

				for(int o = 0; o < numOfHarmonics; o++){
					if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
						M_Vec2f& fPoint = osc->note.refValue();
						M_Float& fWeight = osc->fWeight.refValue();
						fWeight += 1;
						Vec2f v = fPoint[fPoint.length - 1];
						v.x += .0125;
						fPoint += v;
						updateNurbsCurveKnotMid0(fPoint.length, osc->fKnot.refValue());

						osc->processFields();
						if(Harmonic* harmonic = dynamic_cast<Harmonic*>(osc->filter[0].p)){
							M_Vec2f& point = harmonic->point.refValue();
							M_Float& weight = harmonic->weight.refValue();
							weight += 1.0;
							Vec2f v = point[point.length - 1];
							v.x += .0125;
							point += v;
							updateNurbsCurveKnotMid0(point.length, harmonic->knot.refValue());

							harmonic->processFields();
						}
					}
				}
				//return;


				if(Oscillator* sampleOsc = dynamic_cast<Oscillator*>(m_osc[m_osc.length-1].p)){
					if(Sample* sample = dynamic_cast<Sample*>(sampleOsc->filter[0].p)){
						short int* srcBuf = sample->srcBuf;
						int srcBufLen = sample->srcBufLen;
						float* sampleFBuf = new float[srcBufLen];
						for(int i = 0; i < srcBufLen; i++){
							sampleFBuf[i] = srcBuf[i] * (1.0 / 0x7fff);
						}
						float catVolume = volume * track->volume;

						buildBuffers(m_osc, numOfHarmonics, m_harmonicBuf, catVolume, track, fbuf, bufLen, sampleFBuf);

						mixHarmonics(m_harmonicBuf, fbuf, bufLen, -1, -1);
						double remError = compareWaveform(fbuf, bufLen, sampleFBuf);

						//findPhase(str, remError, m_osc, numOfHarmonics, m_harmonicBuf, catVolume, track, spareHarmBuf0, fbuf, bufLen, sampleFBuf);
						//mixHarmonics(m_harmonicBuf, fbuf, bufLen, -1, -1);
						//remError = compareWaveform(fbuf, bufLen, sampleFBuf);//does this help

						
						srand( (unsigned)time( NULL ) );
						for(int q = 0; q < 150; q++){

							int range_min = 0;
							int range_max = numOfHarmonics;

							int oscIdx0 = (double)rand() / (RAND_MAX + 1) * (range_max - range_min) + range_min;
							//int oscIdx1 = (double)rand() / (RAND_MAX + 1) * (range_max - range_min) + range_min;
							int oscIdx1 = numOfHarmonics - 1;
							if(oscIdx0 == oscIdx1){
								continue;
							}

//							oscIdx0 = 1;
//							oscIdx1 = 1;

							double range_minf = -.1;
							double range_maxf =  .1;

							double phase_minf = -.01;
							double phase_maxf =  .01;


							//pointChange0 = .000;
							//pointChange1 = .001;


							str = "begin wave error = ";
							str += remError;
							dLog(str);
							dLog(oscIdx0);
							dLog(oscIdx1);
							//for(int o = 0; o < numOfHarmonics; o++){
							mixHarmonics(m_harmonicBuf, fbuf, bufLen, oscIdx0, oscIdx1);

							if(Oscillator* osc0 = dynamic_cast<Oscillator*>(m_osc[oscIdx0].p)){
								if(Harmonic* harmonic0 = dynamic_cast<Harmonic*>(osc0->filter[0].p)){
									Oscillator* osc1 = dynamic_cast<Oscillator*>(m_osc[oscIdx1].p);
									Harmonic* harmonic1 = dynamic_cast<Harmonic*>(osc1->filter[0].p);

									M_Vec2f& oscPoint0 = osc0->note.refValue();
									M_Vec2f& oscPoint1 = osc1->note.refValue();
									int vDim = harmonic0->multiply.length;
									int uDim = harmonic0->point.length / vDim;

									////findPitch(0, numOfHarmonics, str, remError, m_harmonicBuf, catVolume, track, osc0, spareHarmBuf0, fbuf, bufLen, sampleFBuf);

									//float phaseChange0 = ((double)rand() / (RAND_MAX + 1) * (phase_maxf - phase_minf) + phase_minf);
									//float phaseChange1 = ((double)rand() / (RAND_MAX + 1) * (phase_maxf - phase_minf) + phase_minf);

									//for(int k = 0; k < vDim; k++){
									//	float remPhase0 = harmonic0->phase[k];
									//	float remPhase1 = harmonic1->phase[k];
									//	harmonic0->phase.refValue()[k] += phaseChange0;
									//	harmonic1->phase.refValue()[k] += phaseChange1;

									//	double error = compareWaveform(catVolume, track, osc0, osc1, spareHarmBuf0, spareHarmBuf1, fbuf, bufLen, sampleFBuf);
									//	if(error < remError){
									//		swapPointers(spareHarmBuf0, m_harmonicBuf[oscIdx0]);
									//		swapPointers(spareHarmBuf1, m_harmonicBuf[oscIdx1]);
									//		remError = error;
									//		str = "phase wave error = ";
									//		str += remError;
									//		dLog(str);
									//	}
									//	else{
									//		harmonic0->phase.refValue()[k] = remPhase0;
									//		harmonic1->phase.refValue()[k] = remPhase1;
									//	}
									//}
									////changeFundamentalOnly(remError, m_osc, numOfHarmonics, m_harmonicBuf, catVolume, track, fbuf, bufLen, sampleFBuf);
									////setAllPitchToFundamental(m_osc, numOfHarmonics);
									////buildBuffers(m_osc, numOfHarmonics, m_harmonicBuf, catVolume, track, fbuf, bufLen, sampleFBuf);
									////mixHarmonics(m_harmonicBuf, fbuf, bufLen, oscIdx0, oscIdx1);

									int osc_uDim = osc0->note.length;
									for(int z = 0; z < 100; z++){
									for(int j =  osc_uDim - 4; j < osc_uDim - 1; j++){
										float pitchChange0 = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
										float pitchChange1 = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
										float pitchChange0b = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
										float pitchChange1b = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
										Vec2f remOscPoint0 = oscPoint0[j];
										Vec2f remOscPoint1 = oscPoint1[j];
										Vec2f remOscPoint0b = oscPoint0[j+1];
										Vec2f remOscPoint1b = oscPoint1[j+1];

										oscPoint0[j].y += pitchChange0;
										oscPoint1[j].y += pitchChange1;
										oscPoint0[j+1].y += pitchChange0b;
										oscPoint1[j+1].y += pitchChange1b;

										oscPoint0[0].x = 0;
										oscPoint1[0].x = 0;

										//oscPoint0[j+1].y = oscPoint0[j].y;
										//oscPoint0[j+2].y = oscPoint0[j].y;
										//oscPoint1[j+1].y = oscPoint1[j].y;
										//oscPoint1[j+2].y = oscPoint1[j].y;

										double error = compareWaveform(catVolume, track, osc0, osc1, spareHarmBuf0, spareHarmBuf1, fbuf, bufLen, sampleFBuf);
										if(error < remError){

											swapPointers(spareHarmBuf0, m_harmonicBuf[oscIdx0]);
											swapPointers(spareHarmBuf1, m_harmonicBuf[oscIdx1]);
											remError = error;
											str = "pitch wave error = ";
											str += remError;
											dLog(str);
										}
										else{
											oscPoint0[j] = remOscPoint0;
											oscPoint1[j] = remOscPoint1;
											oscPoint0[j+1] = remOscPoint0b;
											oscPoint1[j+1] = remOscPoint1b;

											//oscPoint0[j+1].y = remOscPoint0.y;
											//oscPoint0[j+2].y = remOscPoint0.y;
											//oscPoint1[j+1].y = remOscPoint1.y;
											//oscPoint1[j+2].y = remOscPoint1.y;

										}
									}

//									evolveAmpSingleCycle(str, remError, oscIdx0, oscIdx1, osc0, osc1, harmonic0, harmonic1, m_harmonicBuf, catVolume, track, spareHarmBuf0, spareHarmBuf1, fbuf, bufLen, sampleFBuf);



									M_Vec2f& point0 = harmonic0->point;
									M_Vec2f& point1 = harmonic1->point;


									for(int v = 0; v < vDim; v++){
										for(int u = uDim - 4; u < uDim - 1; u++){
										//for(int u = 0; u < 3 - 1; u++){
											int i = v * uDim + u;

											float pointChange0 = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
											float pointChange1 = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
											float pointChange0b = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
											float pointChange1b = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);

											//int r = offset;
											//r++;
											//if(r>=randomLen){
											//	r = 0;
											//}
											//offset = r;

											Vec2f remPoint0 = point0[i];
											Vec2f remPoint1 = point1[i];
											Vec2f remPoint0b = point0[i + 1];
											Vec2f remPoint1b = point1[i + 1];


											Vec2f pt0 = remPoint0;
											Vec2f pt1 = remPoint1;
											Vec2f pt0b = remPoint0b;
											Vec2f pt1b = remPoint1b;
											//int bits = rand();
											//bits&=0x7;

											//dLog(bits);

											//if(1 & bits){
											//	pt.x-=pointChange;
											//}
											//if(2 & bits){
											//	pt.x+=pointChange;
											//}
											pt0.y += pointChange0;
											pt1.y += pointChange1;
											pt0b.y += pointChange0b;
											pt1b.y += pointChange1b;
											//if(16 & bits){
											//	pt.z-=pointChange;
											//}
											//if(32 & bits){
											//	pt.z+=pointChange;
											//}
											if(pt0.y < 0){
												pt0.y = 0;
											}
											if(pt1.y < 0){
												pt1.y = 0;
											}
											if(pt0b.y < 0){
												pt0b.y = 0;
											}
											if(pt1b.y < 0){
												pt1b.y = 0;
											}

											point0[i] = pt0;
											point1[i] = pt1;
											point0[i+1] = pt0b;
											point1[i+1] = pt1b;


											double error = compareWaveform(catVolume, track, osc0, osc1, spareHarmBuf0, spareHarmBuf1, fbuf, bufLen, sampleFBuf);
											if(error < remError){
												swapPointers(spareHarmBuf0, m_harmonicBuf[oscIdx0]);
												swapPointers(spareHarmBuf1, m_harmonicBuf[oscIdx1]);

												remError = error;
												str = "wave error = ";
												str += u;
												str += ' ';
												str += v;
												str += ' ';
												str += remError;
												dLog(str);
											}
											else{
												point0[i] = remPoint0;
												point1[i] = remPoint1;
												point0[i+1] = remPoint0b;
												point1[i+1] = remPoint1b;

											}
										}
									}
									}
								}


							}
							//dLog(q);
							//dLog("end");
						}
						str = "remError = ";
						str += remError;
						dLog(str);
						for(int o = 0; o < numOfHarmonics; o++){
							if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
								osc->processFields();
								if(Harmonic* harmonic = dynamic_cast<Harmonic*>(osc->filter[0].p)){
									harmonic->processFields();
								}
							}
						}
						delete[] sampleFBuf;
						delete[] remSpareHarmBuf0;
						delete[] remSpareHarmBuf1;
						delete[] harmonicBuf;
					}
				}
			}
		}
	}
	delete[] fbuf;
}



/*
void Harmonic::toMesh(const Oscillator& osc, M_Vec3f& coord, int staves){
	float step = 1.0 / staves;
	float ampStep = step;

	const float* uKnotPtr = this->knot.refValue().getArray();
	const Vec2f* pointPtr = this->point.refValue().getArray();
	const float* weightPtr = this->weight.refValue().getArray();
	const int uOrder = this->order;
	const int uDim = this->point.length;

	const float maxU=this->maxT();
	const float minU=this->minT();
	const int uDegree = this->degree;

	const float* knotPtr = osc.knot.refValue().getArray();
	const Vec2f* fPointPtr = osc.point.refValue().getArray();
	const float* fWeightPtr = osc.weight.refValue().getArray();

	const int order = osc.order;
	const int dim = osc.point.refValue().length;
	const float maxT = osc.maxT();
	const float minT = osc.minT();
	const int degree = osc.degree;

	double a = 0;
	double multiply = this->multiply[0];
	float ampWadeIn = 0;

	//////////frequency
	float tc = 0;

	for(int s = 0; s < staves + 1; s++){
		if(s > 0){
			tc += step;
		};
		if(tc >= 1.0){
			tc = 1;
		}
		float t = (maxT - minT) * tc + minT;
		Vec2f f = curvePoint2d(dim - 1, degree, knotPtr, fPointPtr, fWeightPtr, t);
		double cent = f.y * 100;
		double freq = 220; //A4 = 440 hertz so why and octive lower?
		freq = freq * pow((pow(2.0, 1.0/1200.0)),cent);
		freq = freq * multiply * -0.0001; //invert and scale for patch

		while(1){
			float uc = 0;
			if(s > 0){
				uc = ampWadeIn + ampStep;
			}

			float u = (maxU - minU) * uc + minU;

			Vec2f amplitude = curvePoint2d(uDim - 1, uDegree, uKnotPtr, pointPtr, weightPtr, u);

			float xDiff = f.x - amplitude.x;
			if(abs(xDiff) < .000001){
				coord += Vec3f(f.x, .1 * (amplitude.y / (1 / multiply)), freq);
				break;
			}
			if(xDiff > 0){
				ampStep += .000001;
			}
			else{
				ampStep -= .000001;
			}
		}
		ampWadeIn += ampStep;
	}
}

void Composition::harmonicsToMesh(){
	int staves = 100;
	Seamless* smls = dynamic_cast<Seamless*>(g_scene->children.refValue()[0].p);
	Art* part = smls->partList[0];
	M_Vec3f& coord = part->coord.refValue();
	Effect* effect=part->workEffect;
	int effectLength=effect->getEffectLength();
	part->deleteGeometry();
	M_Triangle& triangle = part->triangle;
	M_Node& m_track = this->track.refValue();
	if(m_track.length == 1){
		if(Track* track = dynamic_cast<Track*>(m_track[0].p)){
			M_Node& m_osc = track->sound.refValue();
			int numOfHarmonics = m_osc.length - 1;
			if(numOfHarmonics > 0){
				for(int o = 0; o < numOfHarmonics; o++){
					if(Oscillator* osc = dynamic_cast<Oscillator*>(m_osc[o].p)){
						osc->processFields();
						if(Harmonic* harmonic = dynamic_cast<Harmonic*>(osc->timbre.p)){
							harmonic->toMesh(*osc, coord, staves);
						}
					}
				}
			}
			int tri = 0;
			dynamic_cast<ColorEffect*>(effect)->color.refValue().length += coord.length;

			for(int h = 0; h < numOfHarmonics - 1; h++){
				for(int s = 0; s < staves; s++){
					int a = (staves + 1) * h + s + 1;
					int b = (staves + 1) * (h + 1) + s;
					int c = (staves + 1) * h + s;

					triangle += S_Triangle(
						part, a, effectLength + a,
						part, b, effectLength + b,
						part, c, effectLength + c
					);

					a = (staves + 1) * h + s + 1;
					b = (staves + 1) * (h + 1) + s + 1;
					c = (staves + 1) * (h + 1) + s;

					triangle += S_Triangle(
						part, a, effectLength + a,
						part, b, effectLength + b,
						part, c, effectLength + c
					);
				}
			}
		}
	}
	smls->resetRelevantParts();
	smls->process();
	smls->verifyIntegrity();
	smls->ctrf = true;
}

*/

void toMesh(const Oscillator& osc, M_Vec3f& coord, int staves, int harmonic, const M_Int& cycle, const int sampleRate, M_<M_<double> >& mm_amp, bool sawTooth, const bool overlap){
	const int numOfCycles = cycle.length;

	double multiply = harmonic + 1;
	float step = 1.0 / staves;
	float ampStep = step;

	const M_Vec2f& fPoint = osc.note;
	int samplesOffset = cycle[0];

	int cycleInc = 1;
	if(overlap){
		cycleInc = 2;
	}

	for(int k = 0; k + cycleInc < numOfCycles; k++){
		int sampleOffset = cycle[k];
		int numOfSamples = cycle[k + 1] - sampleOffset;
		int windowSize = cycle[k + cycleInc] - sampleOffset;
		
		const float x = samplesOffset / static_cast<double>(sampleRate);
		double baseFreq = static_cast<double>(sampleRate) / windowSize; //convert window to frequency
		double freq = baseFreq * double(harmonic + 1);
		freq = freq * -0.0001; //invert and scale for patch

		double amplitude = mm_amp[k][harmonic] * 2.0;

		if(sawTooth){
			amplitude = .1 * (amplitude / (1.0 / multiply));
		}

		coord += Vec3f(x, amplitude, freq);

		samplesOffset += numOfSamples;
	}
}

void toMesh(const Oscillator& osc, M_Vec3f& coord, int staves, int harmonic, M_<M_<double> >& mm_amp, bool sawTooth){
	double multiply = harmonic + 1;
	float step = 1.0 / staves;
	float ampStep = step;

	const M_Vec2f& fPoint = osc.note;


	if(sawTooth){
		for(int k = 0; k < staves + 1; k++){
			Vec2f f = fPoint[k];
			double cent = f.y * 100;
			double freq = 220; //A4 = 440 hertz so why an octive lower?
			freq = freq * pow((pow(2.0, 1.0 / 1200.0)), cent);
			freq = freq * multiply * -0.0001; //invert and scale for patch

			double amplitude = mm_amp[k][harmonic] * 2.0;
			coord += Vec3f(f.x, .1 * (amplitude / (1 / multiply)), freq);
		}
	}
	else{
		for(int k = 0; k < staves + 1; k++){
			Vec2f f = fPoint[k];
			double cent = f.y * 100;
			double freq = 220; //A4 = 440 hertz so why an octive lower?
			freq = freq * pow((pow(2.0, 1.0 / 1200.0)), cent);
			freq = freq * multiply * -0.0001; //invert and scale for patch

			double amplitude = mm_amp[k][harmonic] * 2.0;
			coord += Vec3f(f.x, amplitude, freq);
		}
	}
}


void Oscillator::alignWithPatch(){
	Seamless* smls = 0;
	if(note.length){
		for(int k = 0; k < filter.length; k++){
			if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(filter[k].p)){
				if(patch->cage){
					M_Vec3f& point = patch->point.refValue();
					if(point.length){
						float diff = note[0].x - point[0].x;
						for(int i = point.length; --i>=0;){
							point[i].x += diff;
						}
						patch->processFields();
						smls = patch->smls;
					}
				}
			}
		}
		if(smls){
			smls->doBuild();
			smls->ctrf = true;
		}
	}
}


void WaveToPoly::harmonicsToMesh(M_<M_<double> >& mm_amp, int numOfHarmonics){
	const M_Int& cycle = this->cycle;
	int staves = 0;
	Art* part = this->part;
	Oscillator* osc = dynamic_cast<Oscillator*>(this->osc.p);
	if(part == 0 || osc == 0){
		return;
	}

	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord.refValue();
	Effect* effect=part->workEffect;
	int effectLength = effect->getEffectLength();
	part->deleteGeometry();
	M_Triangle& triangle = part->triangle;
	staves = osc->note.length - 1;
	NurbsPatch* patch = 0;
	if(osc->filter.length){
		if(patch = dynamic_cast<NurbsPatch*>(osc->filter[0].p)){
			if(patch->stem.length){
				if(Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p)){
					stem->staves = staves;
					if(stem->bands.length){
						stem->bands.refValue()[0] = numOfHarmonics - 1;
						//stem->store.refValue() = true;
					}
				}
			}
		}
	}
	else{
		patch = new NurbsPatch;
		createUniqueName(patch);
		patch->dim.refValue() = Vec2f(4, 4);
		patch->uJoinEnds.refValue() = false;
		patch->showAll.refValue() = true;

		Stem* stem = new Stem;
		stem->bands.refValue() += numOfHarmonics - 1;
		stem->staves.refValue() = osc->note.length - 1;
		stem->height.refValue() += 0;
		stem->height.refValue() += 1;
		stem->xDistance.refValue() = 1;
		stem->xCentred.refValue() = false;
		stem->uWrap.refValue() = false;
		stem->add.refValue() = false;
		//stem->store.refValue() = true;
		stem->processFields();

		Part* whitePart = new Part;
		stem->part.refValue() += whitePart;
		createUniqueName(whitePart);
		part->smls->skeleton.refValue() += whitePart;
		storeGeneratedPolyMap(stem);

		float xBegin = osc->note[0].x;
		float xEnd = osc->note[osc->note.refValue().length - 1].x;
		float xDiff = xEnd - xBegin;
		M_Vec3f& point = patch->point.refValue();
		M_Float& weight = patch->weight.refValue();
		point.length = 16;
		weight.length = 16;
		for(int u = 0; u < 4; u++){
			for(int v = 0; v < 4; v++){
				float x = xBegin + xDiff / 3 * u;
				if(u != 0 && u != 3){
					x = xBegin + xDiff / 2 * u - xDiff / 4;
				}
				point[v * 4 + u] = Vec3f(x, 0, -2.0 / 3.0 * v);
				weight[v * 4 + u] = 1;
			}
		}
		updateNurbsCurveKnotMid0(4, patch->uKnot.refValue());
		updateNurbsCurveKnotMid0(4, patch->vKnot.refValue());

		patch->stem.refValue() += stem;
		patch->processFields();
		osc->filter += patch;
		part->smls->build.refValue() += patch;
	}
	for(int h = 0; h < numOfHarmonics; h++){
		toMesh(*osc, coord, staves, h, cycle, sampleRate, mm_amp, sawTooth, overlap);
		//toMesh(*osc, coord, staves, h, mm_amp, sawTooth);
	}
	int tri = 0;
	dynamic_cast<ColorEffect*>(effect)->color.refValue().length += coord.length;

	for(int h = 0; h < numOfHarmonics - 1; h++){
		for(int s = 0; s < staves; s++){
			int a = (staves + 1) * h + s + 1;
			int b = (staves + 1) * (h + 1) + s;
			int c = (staves + 1) * h + s;

			triangle += S_Triangle(
				part, a, effectLength + a,
				part, b, effectLength + b,
				part, c, effectLength + c
			);

			a = (staves + 1) * h + s + 1;
			b = (staves + 1) * (h + 1) + s + 1;
			c = (staves + 1) * (h + 1) + s;

			triangle += S_Triangle(
				part, a, effectLength + a,
				part, b, effectLength + b,
				part, c, effectLength + c
			);
		}
	}
	bool makePointsEqCoords = true;
	Seamless* patchSmls = patch->smls;
	if(makePointsEqCoords){ // make patch points == coords
		patchSmls->ctrlPolyPoint.length = 0;
		patch->isCtrlPolyRef = false;

		//int divideBy = 20;
		int divideBy = 1;

		//int vDim = numOfHarmonics / divideBy + 1;
		int vDim = numOfHarmonics / divideBy;
		int uDim = staves + 1;

		patch->point.refValue().length = uDim * vDim;

		for(int v = 0; v < vDim; v++){
			for(int u = 0; u < uDim; u++){
				patch->point.refValue()[v * uDim + u] = coord[v * uDim * divideBy + u];
			}
		}



		patch->dim.refValue() = Vec2f(uDim, vDim);

		updateNurbsCurveKnotMid0(uDim, patch->uKnot.refValue());
		updateNurbsCurveKnotMid0(vDim, patch->vKnot.refValue());


		patch->weight.refValue().length = coord.length;
		for(int i = uDim * vDim; --i>=0;){
			patch->weight.refValue()[i] = 100;
		}
		patch->dpoint.refValue() = patch->point;
		patch->dpointsToPoints();

		patch->processFields();
		patchSmls->process();

		patchSmls->populateMirrorNodes();
		patchSmls->addCps(patch, smls->cp.length);
	}

	smls->resetRelevantParts();
	smls->process();

	smls->doBuild();
	smls->verifyIntegrity();
	smls->ctrf = true;
	g_stw->commonPostInsertNodeStuff();
}

void WaveToPolyControlPanel::openWaveFile(const char* filename){
	waveToPoly->file = filename;
	processFilename();
	waveToPoly->processFields();
	paint();
}

void Audio::openWaveFile(const char* filename){
	file = filename;
	g_stw->cp->paint();
}


#include <complex>
using namespace std;
// M number of harmonics

short int DFS(const float* sampleBuf, short int numOfSamples, M_<double>& m_amp, short int numOfHarmonics)
// calculate numOfHarmonics harmonics of sampleBuf[numOfSamples] and saves them in c[numOfHarmonics]
{
	complex <double> *X;

	// calc complex DFS
	if ((X = (complex <double> *)calloc(numOfHarmonics, sizeof(complex <double>))) == NULL) return 1;
	int L = (numOfHarmonics > numOfSamples / 2) ? numOfSamples / 2 : numOfHarmonics; //limit to max DFS coeffs.
	for(int i = 0; i < L; i++) {             // do each harmonic
		X[i] = 0;
		//Wk = 2 * PI * i / numOfSamples;  					// frequency of the harmonic
		double arg = 2.0 * PI * double(i + 1) / double(numOfSamples);
		for(int k = 0; k < numOfSamples; k++) {
			//double cosarg = cos(k * arg);
			//double sinarg = sin(k * arg);
			//X[i] += (sampleBuf[k] * cosarg - k * sinarg);
			X[i] += polar(double(sampleBuf[k]), -arg * k);
			//double _Rho = sampleBuf[k];
			//double _Theta = -k * arg;
			//complex <double> cplx = (complex <double>(_Rho * cos(_Theta), _Rho * _CTR(double)::sin(_Theta)));
			//complex <double> cplx = complex <double>(_Rho * cos(_Theta), _Rho * sin(_Theta));
			//complex <double> cplx = complex <double>(_Rho * cosarg, _Rho * sinarg);


			//X[i] += cplx;
		}
	}

	// store amplitude spectrum
	for(int k = 0; k < numOfHarmonics; k++){
		m_amp[k] = 2 * abs(X[k]) / numOfSamples;
		//c[k] = (short int) 2 * arg(X[k]) / numOfSamples;
	}

	return 0;
}


/*
void DFS(const float* sampleBuf, int numOfSamples, M_<double>& m_amp, int numOfHarmonics)
// calculate numOfHarmonics of x[numOfSamples] and saves them in m_amp[numOfHarmonics]
{

	// calc complex DFS
	complex <double> *X = new complex <double>[numOfHarmonics];

	//numOfHarmonics = (numOfHarmonics > numOfSamples / 2) ? numOfSamples / 2 : numOfHarmonics; //limit to max DFS coeffs.

	for (int h = 0; h < numOfHarmonics; h++) {             // do each harmonic
		X[h] = 0;
		double Wk =  2 * PI * double(h) / double(numOfSamples);  					// frequency of the harmonic
		for (int s = 0; s < numOfSamples; s++){
			X[h] += polar(double(sampleBuf[s]), -Wk * s);
		}

		m_amp[h] = 2.0 * abs(X[h]) / numOfSamples;
	}


	delete[] X;
}
*/
// x = sample buffer
// N = number of samples in sample buffer
/*

void DFS(const float* sampleBuf, int numOfSamples, M_<double>& m_amp, int numOfHarmonics)
// calculate M harmonics of x[N] and saves them in c[M]
{
	//double Wk;
	complex <double> *X;

	// calc complex DFS
	X = new complex <double>[numOfHarmonics];

	int L = (numOfHarmonics > numOfSamples / 2) ? numOfSamples / 2 : numOfHarmonics; //limit to max DFS coeffs.

	for(int i = 0; i < L; i++) {             // do each harmonic
		X[i] = 0;
		//Wk = 2 * PI * i / N;  					// frequency of the harmonic
		double arg = 2.0 * PI * (double)(i) / (double)numOfSamples;
		//double arg = 2.0 * PI * (double)(i + 1) / (double)N;
		for(int k = 0; k < numOfSamples; k++) {
			//double cosarg = cos(k * arg);
			//double sinarg = sin(k * arg);
			X[i] += polar((double)sampleBuf[k], -arg*k);
			//double _Rho = x[k];
			//double _Theta = -k * arg;
			//X[i] += (complex <double>(_Rho * cos(_Theta), _Rho * _CTR(double)::sin(_Theta)));
			//X[i] += complex <double>(_Rho * cos(_Theta), _Rho * sin(_Theta));
			//X[i] += complex <double>(_Rho * cosarg, _Rho * sinarg);
		}
	}

	// store amplitude spectrum
	for(int k = 0; k < numOfHarmonics; k++){
		m_amp[k] = 2 * abs(X[k]) / numOfSamples;
	}
	delete[] X;
}
*/

/*
void DFS(const float* sampleBuf, int numOfSamples, M_<double>& m_amp, int numOfHarmonics){//does not work
    // time and frequency domain data arrays
    int n = 0;
    
	double* Xre = new double[numOfHarmonics];
	double* Xim = new double[numOfHarmonics];
     
    // Calculate DFT of x using brute force
    for (int h = 0 ; h < numOfHarmonics; h++){
        // Real part of X[k]
        Xre[h] = 0;
		Xim[h] = 0;
        for (n = 0 ; n < numOfSamples ; n++){
			Xre[h] += sampleBuf[n] * cos(double(n) * double(h) * (PI * 2) / double(numOfSamples));
			Xim[h] -= sampleBuf[n] * sin(double(n) * double(h) * (PI * 2) / double(numOfSamples));
         
		}
		// Power at kth frequency bin
		m_amp[h] = (Xre[h] * Xre[h] + Xim[h] * Xim[h]);
    }

	delete[] Xre;
	delete[] Xim;
}
*/
void synthTimbre(char* filename, M_<M_<double> >& mm_amp, double* ddstBuf, int dstLen, int numOfHarmonics, const M_<int>& cycle, const int overlap, int numOfPeriods, int sampleRate){
	short int* dstBuf = new short int[dstLen];

	for(int i = dstLen; --i>=0;){
		ddstBuf[i] = 0;
	}

	//stop click by randomising phase
	M_<double> m_phase;
	m_phase.length = numOfHarmonics;

	double range_minf = 0;
	double range_maxf = 1;
	//double inc = 0;
	for(int g = 0; g < numOfHarmonics; g++){
		m_phase[g] = ((double)rand() / (RAND_MAX + 1) * (range_maxf - range_minf) + range_minf);
	}

	int cycleInc = 2; //1 to offset 2 to miss offsets

	for(int k = 0; k + 2 < numOfPeriods; k += cycleInc){
		int sampleOffset = cycle[k];
		int numOfSamples = cycle[k + cycleInc] - sampleOffset;
		int sampleSize = cycle[k + 2] - sampleOffset;
		double fundamentalFreq = 2 * PI / sampleSize;



		for(int h = 0; h < numOfHarmonics; h++){
			double phase = PI * 2.0 * m_phase[h];

			double multiply = h + 1;
			double freq = multiply * fundamentalFreq;
			double prev_a = 0;
			if(k > 0){
				prev_a = mm_amp[k - cycleInc][h];
			}
			double a = mm_amp[k][h];
			double diff = a - prev_a;
			double fnumOfSamples = numOfSamples;

			for(int i = 0; i < numOfSamples; i++){
				double a2 = diff / fnumOfSamples * (i + 1);
				a2 = a2 + prev_a;


				ddstBuf[sampleOffset + i] += a2 * sin(phase + i * freq);
			}
		}
	}
	const float scaleToInt = 0x7fff;
	for(int i = dstLen; --i>=0;){
		dstBuf[i] = static_cast<short int>(ddstBuf[i] * 0x7fff);
	}

	saveToWav(dstBuf, dstLen, 1, sampleRate, filename);
}


void synthTimbre(char* filename, M_<M_<double> >& mm_amp, double* ddstBuf, int dstLen, int numOfHarmonics, const M_<int>& cycle, int numOfPeriods, int sampleRate){
	short int* dstBuf = new short int[dstLen];

	for(int i = dstLen; --i>=0;){
		ddstBuf[i] = 0;
	}

	for(int k = 0; k + 1 < numOfPeriods; k++){
		int sampleOffset = cycle[k];
		int numOfSamples = cycle[k + 1] - sampleOffset;
		double fundamentalFreq = 2 * PI / numOfSamples;

		for(int h = 0; h < numOfHarmonics; h++){

			double multiply = h + 1;
			double freq = multiply * fundamentalFreq;
			double a = mm_amp[k][h];
			for(int i = 0; i < numOfSamples; i++){
				ddstBuf[sampleOffset + i] += a * sin(i * freq);
			}
		}
	}
	const float scaleToInt = 0x7fff;
	for(int i = dstLen; --i>=0;){
		dstBuf[i] = static_cast<short int>(ddstBuf[i] * 0x7fff);
	}

	saveToWav(dstBuf, dstLen, 1, sampleRate, filename);
}

void setOsc(Oscillator* osc, const M_<int>& cycle, const int numOfCycles, const int sampleRate, const M_<M_<double> > mm_amp, const int numOfHarmonics, double pitchLimit){
	M_Vec2f& point = osc->note.refValue();
	M_Float& weight = osc->fWeight.refValue();
	M_Float& knot = osc->fKnot.refValue();

	point.length = 0;
	weight.length = 0;


	for(int k = 0; k + 2 < numOfCycles; k++){
		
		const int sampleOffset = cycle[k];
		const int windowSize = cycle[k + 2] - sampleOffset;
		const float x = sampleOffset / static_cast<double>(sampleRate);
		double baseFreq = static_cast<double>(sampleRate) / windowSize; //convert window to frequency

		double bestAmp = 0;
		double bestFreq = 0;

		for(int h = 0; h < numOfHarmonics; h++){

			double freq = baseFreq * double(h + 1);
			
			if(freq > pitchLimit){//read as decimal point 4 to the right
				break;
			}

			double amp = mm_amp[k][h];
			if(amp > bestAmp){
				bestAmp = amp;
				bestFreq = freq;
			}

		}
		double cents = 12.0 * log(bestFreq / 220.0) / log(2.0); //convert frequency to pitch

		point += Vec2f(x, cents);
		weight += 1;
	}

	updateNurbsCurveKnotMid0(point.length, knot);

	osc->processFields();
}

void setOsc(Oscillator* osc, const M_<int>& cycle, int numOfCycles, int sampleRate){
	M_Vec2f& point = osc->note.refValue();
	M_Float& weight = osc->fWeight.refValue();
	M_Float& knot = osc->fKnot.refValue();

	point.length = 0;
	weight.length = 0;

	int samplesOffset = cycle[0];
	for(int k = 0; k + 1 < numOfCycles; k++){
		int numOfSamples = cycle[k + 1] - cycle[k];
		float x = samplesOffset / static_cast<double>(sampleRate);
		double freq = static_cast<double>(sampleRate) / numOfSamples;
		double cents = 12.0 * log(freq / 220.0) / log(2.0);
		point += Vec2f(x, cents);
		weight += 1;
		samplesOffset += numOfSamples;
	}

	updateNurbsCurveKnotMid0(point.length, knot);

	osc->processFields();
}

void WaveToPoly::Convert::operator=(const bool& par){
	Oscillator* osc = dynamic_cast<Oscillator*>(home.osc.p);
	if(osc == 0){
		return;
	}
	setMathHiPresision();
	const M_Int& cycle = home.cycle;
	const float* srcBuf = home.srcBuf;
	const int sampleRate = home.sampleRate;

	int numOfHarmonics = home.partials;

	if(osc->filter.length){
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(osc->filter[0].p)){
			if(patch->stem.length){
				if(Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p)){
					if(stem->bands.length){
						numOfHarmonics = stem->bands[0] + 1;
					}
				}
			}
		}
	}


	double* ddstBuf = new double[home.srcBufLen];

	//int dstLen = numOfPeriods * numOfSamples;
	M_Float window;


	M_<M_<double> > mm_amp;

	int numOfPeriods = cycle.length;

	mm_amp.length = numOfPeriods;

	if(home.overlap){
		for(int k = 0; k + 2 < numOfPeriods; k++){
			const int sampleOffset = cycle[k];
			const int windowSize = cycle[k + 2] - sampleOffset;

			mm_amp[k].length = numOfHarmonics;
			window.length = windowSize;

			for(int j = 0; j < windowSize; j++){
				window[j] = srcBuf[sampleOffset + j];
			}

			fadeInFadeOutWindow(window);

			DFS(window.getArray(), windowSize, mm_amp[k], numOfHarmonics);
		}
		synthTimbre("test-fourier.wav", mm_amp, ddstBuf, home.srcBufLen, numOfHarmonics,  cycle, home.overlap, numOfPeriods, sampleRate);
	
		setOsc(osc, cycle, numOfPeriods, sampleRate, mm_amp, numOfHarmonics, home.pitchLimit);
	}
	else{
		for(int k = 0; k + 1 < numOfPeriods; k++){
			mm_amp[k].length = numOfHarmonics;
			const int windowSize = cycle[k + 1] - cycle[k];
			DFS(srcBuf + cycle[k], windowSize, mm_amp[k], numOfHarmonics);
		}
		synthTimbre("test-fourier.wav", mm_amp, ddstBuf, home.srcBufLen, numOfHarmonics,  cycle, numOfPeriods, sampleRate);

		setOsc(osc, cycle, numOfPeriods, sampleRate);

	}



	home.harmonicsToMesh(mm_amp, numOfHarmonics);
}

/*
void WaveToPoly::findSignal(){
	setMathHiPresision();
	M_Int& cycle = this->cycle.refValue();
	const float* srcBuf = this->srcBuf;

	double* ddstBuf = new double[srcBufLen];

	int markFrom = cycle[cMark - 1];
	int markTo = cycle[cMark];

	int numOfHarmonics = 1;
	M_<double> m_amp;

	m_amp.length = numOfHarmonics;
	int numOfSamples = markTo - markFrom;
	DFS(srcBuf + markFrom, numOfSamples, m_amp, numOfHarmonics);

	double highest = m_amp[0];

	do{
		DFS(srcBuf + markFrom, numOfSamples + 1, m_amp, numOfHarmonics);
		if(highest > m_amp[0]){
			break;
		}
		highest = m_amp[0];
		numOfSamples++;
	}while(true);

	do{
		DFS(srcBuf + markFrom, numOfSamples - 1, m_amp, numOfHarmonics);
		if(highest > m_amp[0]){
			break;
		}
		highest = m_amp[0];
		numOfSamples--;
	}while(true);

	cycle[cMark] = markFrom + numOfSamples;
	g_wtpcp->paint();
}

void WaveToPoly::findSignal(){
	if(srcBuf[cycle[cMark]] > 0){
		for(int i = cycle[cMark]; i > 0; --i){
			if(srcBuf[i] <= 0){
				cycle.refValue()[cMark] = i;
				g_wtpcp->paint();
				break;
			}
		}
	}
	else{
		for(int i = cycle[cMark]; i < srcBufLen; i++){
			if(srcBuf[i] >= 0){
				cycle.refValue()[cMark] = i;
				g_wtpcp->paint();
				break;
			}
		}
	}
}
*/
/*
void WaveToPoly::findSignal(){//highest or lowest (good)
	if(srcBuf[cycle[cMark]] < 0){
		while(true){
			if(srcBuf[cycle[cMark]] > srcBuf[cycle[cMark] + 1]){
				cycle.refValue()[cMark]++;
				continue;
			}
			if(srcBuf[cycle[cMark]] > srcBuf[cycle[cMark] - 1]){
				cycle.refValue()[cMark]--;
				continue;
			}
			break;
		}
	}
	else{
		while(true){
			if(srcBuf[cycle[cMark]] < srcBuf[cycle[cMark] + 1]){
				cycle.refValue()[cMark]++;
				continue;
			}
			if(srcBuf[cycle[cMark]] < srcBuf[cycle[cMark] - 1]){
				cycle.refValue()[cMark]--;
				continue;
			}
			break;
		}
	}
	g_wtpcp->paint();
}
*/
void WaveToPoly::findSignal(){//find cross over
	while(true){
		if(abs(srcBuf[cycle[cMark]]) > abs(srcBuf[cycle[cMark] + 1])){
			cycle.refValue()[cMark]++;
			continue;
		}
		if(abs(srcBuf[cycle[cMark]]) > abs(srcBuf[cycle[cMark] - 1])){
			cycle.refValue()[cMark]--;
			continue;
		}
		break;
	}
	g_wtpcp->paint();
}
