/* Seamless3d

Copyright (C) 2001-2013 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include <windows.h>

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "gui.h"
#include "resource.h"

extern const BIProto* builtInProtoList[];
Ptr<ToolBarWin> g_tb = new ToolBarWin;
Ptr<ToolBarWin> g_tb2 = new ToolBarWin;
extern Ptr<AnimControlPanel> g_acp;

extern const char* tip_toggleWireframe;
extern const char* tip_resetViewPosition;
extern const char* tip_resetViewOrientation;
extern const char* tip_centerViewToControlPoint;
extern const char* tip_centerViewToPivotPoint;
extern const char* tip_findThinTriangle;//"find thin triangle (Alt F)"
extern const char* tip_join_vertex_keep_both_colors;
extern const char* tip_join_vertex_keep_dest_color;
extern const char* tip_joinCloseVertices;
extern const char* tip_joinCloseVertecesForAllShownParts;

extern const char* tip_zeroToX;

extern const char* tip_build_sphere_for_selected_part;
extern const char* tip_build_cylinder_for_selected_part;
extern const char* tip_build_cone_for_selected_part;
const char* tip_build_rings_for_selected_part="build rings for selected part";
const char* tip_build_nurbs_lathe_for_selected_part="create nurbs lathe (click in 3d window to specify location, oriantation and control points)";
const char* tip_build_bendy_lathe_for_selected_part="build bendy lathe for selected part";
const char* tip_build_subdivision_for_selected_part="build_subdivision_for_selected_part";
extern const char* tip_build_torus_for_selected_part;
extern const char* tip_build_bezier_lathe_for_selected_part;
extern const char* tip_build_box_for_selected_part;
extern const char* tip_build_rectangle_for_selected_part;

extern const char* tip_undo;
extern const char* tip_redo;
extern const char* tip_reset_rotations;

extern const char* tip_new_smls_file;
extern const char* tip_open_smls_file;
extern const char* tip_open_smls_file_from_the_web;
extern const char* tip_save_smls_file;
//extern const char* tip_import_triangles_from_file_to_part;
//extern const char* tip_export_triangles_from_part_to_file;
extern const char* tip_scene_tree_window;
extern const char* tip_copy_vertex_part;
extern const char* tip_vertex_coord;
extern const char* tip_vertex_color;
extern const char* tip_zoom;
extern const char* tip_view_orientation;
extern const char* tip_reset_view_orientation;
extern const char* tip_view_orientation_at_right_angle;
extern const char* tip_view_position;
extern const char* tip_move_pivot_point;
extern const char* tip_compile_Part_and_its_decendents_scale_and_rotation_fields;
extern const char* tip_toggle_perspective;
extern const char* tip_help;
extern const char* tip_break_coord_join;
extern const char* tip_delete_selected_triangle;
extern const char* tip_delete_triangles_down_from_selected_vertex;
extern const char* tip_delete_triangles_left_from_selected_vertex;
extern const char* tip_join_colors_in_vertices_that_equal_the_same_color_and_whose_coords_are_joined;
extern const char* tip_paste_copied_color_to_selected_vertex;
extern const char* tip_paste_copied_color_to_part;
extern const char* tip_paste_ownership_from_copied_part_into_the_selected_vertex;
extern const char* tip_paste_triangles_from_the_copied_part_in_X_mirrored_form_into_the_selected_part;
extern const char* tip_add_triangles_from_the_copied_part_in_X_mirrored_form_to_the_selected_part;
const char* tip_copyAndPasteMirroredPart="mirror part";
extern const char* tip_swap_triangle_divide_between_the_copied_and_selected_triangles;
extern const char* tip_mirror_triangle;
extern const char* tip_split_triangle_between_the_vertices_marked_by_the_red_and_green_dots;
extern const char* tip_add_a_ColorSweep_for_selected_parts;
extern const char* tip_copy_upper_selected_parts_to_lower_selected_parts;
extern const char* tip_place_selected_parts;
extern const char* tip_swap_vertexs_index_order_with_previous_next;
extern const char* tip_select_previous_next_vertex_in_index_order;
extern const char* tip_select_previous_next_triangle_that_shares_the_selected_vertex;
extern const char* tip_select_previous_next_corner_in_triangle;
extern const char* tip_toggle_background_color;
const char* tip_quickTriangle="generate triangle from selected edge and copied vertex";
const char* tip_build_model_morpher_for_selected_seamless_node = "build model morpher for selected seamless node";
const char* tip_build_av_segment_for_selected_part = "build av segment for selected part";
const char* tip_do_build_cycle = "do build cycle";
const char* tip_toggle_backface_ignore = "toggle backface ignore (Alt Space bar)";

extern bool g_dragWithDescendants;

extern const char* menu_rotate_uv_90;
void setShowDimFilterVisual();
void rectangleSelect();
void rectangleUnselect();
void rectangleSelectStitchPoly();
void toggleRadialTug();
void toggleTransformFallOff();

///////////////
const char** tip_list[]={
&tip_toggleWireframe,
&tip_resetViewPosition,
&tip_resetViewOrientation,
&tip_centerViewToControlPoint,
&tip_centerViewToPivotPoint,
&tip_findThinTriangle,
&tip_join_vertex_keep_both_colors,
&tip_join_vertex_keep_dest_color,
&tip_joinCloseVertices,
&tip_joinCloseVertecesForAllShownParts,

&tip_zeroToX,

&tip_build_sphere_for_selected_part,
&tip_build_cylinder_for_selected_part,
&tip_build_cone_for_selected_part,
&tip_build_rings_for_selected_part,
&tip_build_nurbs_lathe_for_selected_part,
&tip_build_bendy_lathe_for_selected_part,
&tip_build_torus_for_selected_part,
&tip_build_bezier_lathe_for_selected_part,
&tip_build_box_for_selected_part,
&tip_build_rectangle_for_selected_part,

&tip_undo,
&tip_redo,
&tip_reset_rotations,

&tip_new_smls_file,
&tip_open_smls_file,
&tip_open_smls_file_from_the_web,
&tip_save_smls_file,
//&tip_import_triangles_from_file_to_part,
//&tip_export_triangles_from_part_to_file,
&tip_scene_tree_window,
&tip_copy_vertex_part,
&tip_vertex_coord,
&tip_vertex_color,
&tip_zoom,
&tip_view_orientation,
&tip_reset_view_orientation,
&tip_view_orientation_at_right_angle,
&tip_view_position,
&tip_move_pivot_point,
&tip_compile_Part_and_its_decendents_scale_and_rotation_fields,
&tip_toggle_perspective,
&tip_help,
&tip_break_coord_join,
&tip_delete_selected_triangle,
&tip_delete_triangles_down_from_selected_vertex,
&tip_delete_triangles_left_from_selected_vertex,
&tip_join_colors_in_vertices_that_equal_the_same_color_and_whose_coords_are_joined,
&tip_paste_copied_color_to_selected_vertex,
&tip_paste_copied_color_to_part,
&tip_paste_ownership_from_copied_part_into_the_selected_vertex,
&tip_paste_triangles_from_the_copied_part_in_X_mirrored_form_into_the_selected_part,
&tip_add_triangles_from_the_copied_part_in_X_mirrored_form_to_the_selected_part,
&tip_copyAndPasteMirroredPart,
&tip_swap_triangle_divide_between_the_copied_and_selected_triangles,
&tip_mirror_triangle,
&tip_split_triangle_between_the_vertices_marked_by_the_red_and_green_dots,
&tip_add_a_ColorSweep_for_selected_parts,
&tip_copy_upper_selected_parts_to_lower_selected_parts,
&tip_place_selected_parts,
&tip_swap_vertexs_index_order_with_previous_next,
&tip_select_previous_next_vertex_in_index_order,
&tip_select_previous_next_triangle_that_shares_the_selected_vertex,
&tip_select_previous_next_corner_in_triangle,
&tip_toggle_background_color,
&tip_quickTriangle,
&tip_build_model_morpher_for_selected_seamless_node,
&tip_build_av_segment_for_selected_part,
&tip_do_build_cycle
};


String g_language;

void doBuild();

void dummyFunction(){}
void openPalette();
void duplicate();
void duplicateMirror();
extern bool g_curvedCageMode;

SingleButton* g_toggleZBufferCageButton = 0;
SingleButton* g_toggleCurvedCageButton = 0;
SingleButton* g_animToggleZBufferCageButton = 0;
SingleButton* g_toggleSingleActiveButton = 0;
SingleButton* g_animToggleSingleActiveButton = 0;
SingleButton* g_createMirrorObjectButton = 0;
SingleButton* g_createMirrorBoneButton = 0;
SingleButton* g_dragWithDescendantsButton = 0;
SingleButton* g_toggleBackfaceSelectButton = 0;
SingleButton* g_animToggleBackfaceSelectButton = 0;
SingleButton* g_paintToggleBackfaceSelectButton = 0;

extern SingleButton* g_toggleHidePolysInHiddenPatchesButton;
extern SingleButton* g_toggleAccumulateSelectButton;
extern SingleButton* g_togglePolyMorphCaptureButton;
extern SingleButton* g_toggleSelectObjectButton;
extern SingleButton* g_togglePerspectiveButton;

SingleButton* g_openTransformCPButton = 0;

SingleButton* g_poseButton = 0;
SingleButton* g_selectObjectButton = 0;
SingleButton* g_polyEditButton = 0;
SingleButton* g_modellingButton = 0;
SingleButton* g_paintButton = 0;
SingleButton* g_animationButton = 0;
SingleButton* g_xRayEditButton = 0;
SingleButton* g_ownershipButton = 0;
SingleButton* g_paintNurbsColorCode = 0;

SingleButton* g_stitchPolyModeButton = 0;
SingleButton* g_skeletonEditStitchPolyModeButton = 0;
SingleButton* g_animStitchPolyModeButton = 0;


extern SingleButton* g_selectRingButton;
extern SingleButton* g_selectRingsButton;

extern SingleButton* g_boxPatchModellingButton;

void togglePolyMorphCapture();
void toggleSelectObject();
void selectWholePatch();
void openTransformCP();

M_Button g_m_add_geometry_button;
M_Button g_m_paint_button;
M_Button g_m_modeling_button;
M_Button g_m_poly_edit_button;
M_Button g_m_hair_button;
M_Button g_m_select_object_button;
M_Button g_m_xRayEdit_button;

void ToolBarWin::doDrag(Vec2i ptCursor){
	commonDoDrag(ptCursor, pos, dragPos);
}

void paintRectangleOutline(Vec2i rect, int dstWidth, int dstHeight, int* dstPix){
	drawHorizontalLine2(0, rect.y, rect.x, rect.x + 23, dstWidth, dstHeight, dstPix);
	drawHorizontalLine2(0, rect.y + 23, rect.x, rect.x + 23, dstWidth, dstHeight, dstPix);
	drawVerticalLine2(0, rect.x, rect.y, rect.y + 23, dstWidth, dstHeight, dstPix);
	drawVerticalLine2(0, rect.x + 23, rect.y, rect.y + 23, dstWidth, dstHeight, dstPix);
}

bool blockModeButtons(SingleButton* button){
	if(&button->win != g_tb.p){
		return true;
	}
	switch(g_scene->mode & MODE_MASK){
		case HOME_MODE:
			if(button == g_poseButton){
				return false;
			}
			break;
		case SELECT_OBJECT_MODE:
			if(button == g_selectObjectButton){
				return false;
			}
			break;
		case ANIMATION_MODE:
			if(button == g_animationButton){
				return false;
			}
			break;
		case MODELLING_MODE:
			if(button == g_modellingButton){
				return false;
			}
			break;
		case PAINT_MODE:
			if(button == g_paintButton){
				return false;
			}
			break;
		case POLY_EDIT_MODE:
			if(button == g_polyEditButton){
				return false;
			}
			break;
		case XRAY_EDIT_MODE:
			if(button == g_xRayEditButton){
				return false;
			}
			break;
		case OWNERSHIP_MODE:
			if(button == g_ownershipButton){
				return false;
			}
			break;
		case NURBS_OWNERSHIP_MODE:
			if(button == g_paintNurbsColorCode){
				return false;
			}
			break;
	}
	return true;
}

void ToolBarWin::paint(){
	int* pixBuf = this->pixBuf.getArray();
	M_Button& m_button = *(this->m_button);

	int indent = 0, line = 0;

	int* pEnd = pixBuf + size.x * size.y;

	{
		paintColorSweepForCurvedBar(&iconSrcBuf[(iconSrcBufCY-1)*480+470], &iconSrcBuf[(iconSrcBufCY-1)*480+471], 10, 0, Vec2i(size.x-9-10, title_bar_height), size.x, size.y, pixBuf);
		pasteSprite2( 0, 0, 461, 0, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 10, title_bar_height, pixBuf);
		pasteSprite2( this->size.x-9, 0, 471, 0, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 9, title_bar_height, pixBuf);

		paintColorSweepForCurvedBar(&iconSrcBuf[(iconSrcBufCY-1)*480+470], &iconSrcBuf[(iconSrcBufCY-1)*480+471], 10, title_bar_height, Vec2i(size.x-9-10, title_bar_height), size.x, size.y, pixBuf);
		pasteSprite2( 0, title_bar_height, 461, 0, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 10, title_bar_height, pixBuf);
		pasteSprite2( this->size.x-9, title_bar_height, 471, 0, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 9, title_bar_height, pixBuf);
	}

	if(this == g_tb){
		switch(g_scene->mode & MODE_MASK){
			case HOME_MODE:
				//paintRectangleOutline(g_poseButton->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_poseButton->dstPos.x - 3, g_poseButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case SELECT_OBJECT_MODE:
				pasteSprite2(g_selectObjectButton->dstPos.x - 3, g_selectObjectButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case ANIMATION_MODE:
				//paintRectangleOutline(g_animationButton->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_animationButton->dstPos.x - 3, g_animationButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case MODELLING_MODE:
				//paintRectangleOutline(g_modellingButton->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_modellingButton->dstPos.x - 3, g_modellingButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case PAINT_MODE:
				//paintRectangleOutline(g_paintButton->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_paintButton->dstPos.x - 3, g_paintButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case POLY_EDIT_MODE:
				//paintRectangleOutline(g_polyEditButton->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_polyEditButton->dstPos.x - 3, g_polyEditButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case XRAY_EDIT_MODE:
				//paintRectangleOutline(g_xRayEditButton->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_xRayEditButton->dstPos.x - 3, g_xRayEditButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case OWNERSHIP_MODE:
				//paintRectangleOutline(g_ownershipButton->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_ownershipButton->dstPos.x - 3, g_ownershipButton->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
			case NURBS_OWNERSHIP_MODE:
				//paintRectangleOutline(g_paintNurbsColorCode->dstPos, size.x, size.y, pixBuf);
				pasteSprite2(g_paintNurbsColorCode->dstPos.x - 3, g_paintNurbsColorCode->dstPos.y, 144, 281, this->size.x, this->size.y, iconSrcBufCX, iconSrcBufCY, 30, 24, pixBuf);
				break;
		}
	}

	for(int k = m_button.length; --k>=0;){
		Button* button = m_button[k];
		button->paint();
	}


	//{
	//	for(int bar = 0, barY = BORDER_DIST + SINGLE_BAR_CY; bar < this->mm_button->length; bar++, barY += SINGLE_BAR_CY){
	//		MM_Button& mm_button = *(this->mm_button);
	//		M_Button& m_button = mm_button[bar];
	//		for(int k = 0, h = TOOLBAR_BUTTON_OFFSET_X; k < m_button.length; k++){

	//			Button* button = m_button[k];
	//			button->paint();
	//			h+=button->bSize.x;
	//		}
	//	}
	//}
	if(g_settings->invertWindowColor){
		invert();
	}
	loadSurfaceFromMemory();
}
void ToolBarWin::buttonHover(Vec2i ptCursor){
	M_Button& m_button = *(this->m_button);

	for(int k = m_button.length; --k>=0;){
		Button* button = m_button[k];
		button->initHover(ptCursor);
	}
}

Button* getButtonForRobot(int id, M_Button& m_button){

	for(int k = m_button.length; --k>=0;){
		Button* button = m_button[k];
		if(button->getID()==id){
			return button;
		}
	}
	return 0;
}

Button* getButtonForRobot(int id, Window*& win){
	win = g_tb;
	Button* button = getButtonForRobot(id, *(g_tb->m_button));
	if(button == 0){
		if(g_acp->animMode()){
			win = g_acp;
			button = getButtonForRobot(id, *(g_acp->m_button));
		}
		else{
			win = g_tb2;
			button = getButtonForRobot(id, *(g_tb2->m_button));
		}
	}
	return button;
}


void ToolBarWin::onLoseFocus(){
	if(this==0)return;
	removeToolTip();
}

void loadLanguage(String& str){
	str = readFile((g_appDataDir + "language.txt").c_str());

	if(str.length){
		str = str.substring(str.indexOf("*/"));
		int langSize = str.length;
		int i = str.indexOf("Version:") + 8;
		float langVersion = getFloat(str, i);
		str.stb(i);

		for(int k=0; k < sizeof(tip_list)/sizeof(char*); k++){
			*tip_list[k] = str.c_str() + i;
			str.stnl(i);
			str.stb(i);
		}
		unsigned char* langData = reinterpret_cast<unsigned char*>(const_cast<char*>(str.c_str()));
		for(int i = 0; i < langSize; i++){
			if(langData[i] < ' '){
				langData[i] = 0;
			}
		}
	}
}

//ButtonPtr g_placeSelParts = new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(placeIcon_x, placeIcon_y),tip_place_selected_parts, 0, placeForSelParts, *g_tb);
ButtonPtr g_tugRadButton = new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(336,144), "radiul tug", "", toggleRadialTug, *g_tb2);
//ButtonPtr g_transformFallOffButton = new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(336,144), "radical patch", "", toggleTransformFallOff, *g_tb2);
ButtonPtr g_deleteTriButton = new SingleButton(Vec2i(24,24), Vec2i(deleteTri_x,deleteTri_y), tip_delete_selected_triangle, "deleteTriangle", deleteTriangle, *g_tb2);


void ToolBarWin::initToolbar(){
	loadLanguage(g_language);

	g_tb2->m_button = &g_m_add_geometry_button;
	M_Button& m_mode_button = *(g_tb->m_button);

	//m_button0+=new Button(24,24,24*15,24*3, "open the F10 menu (F10)", SELECT_PREV_VERTEX);
	//m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(newSmls_x,newSmls_y), tip_new_smls_file, "newSmls", newSmls, *g_tb);
	//m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(openSmls_x,openSmls_y), tip_open_smls_file, "openSmls", openSmls, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(10 * 24, 281), "file menu", "fileMenu", fileMenu, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(saveSmls_x,saveSmls_y), tip_save_smls_file, "saveSmls", saveSmls, *g_tb);
	//m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(loadPartFromFile_x,loadPartFromFile_y), tip_import_triangles_from_file_to_part, "importPart", importPart, *g_tb);
	//m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(savePartAs_x,savePartAs_y), tip_export_triangles_from_part_to_file, "exportPart", exportPart, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(undo_x, undo_y),	tip_undo, "unDo", unDo, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(redo_x, redo_y),tip_redo, "reDo", reDo, *g_tb);
	//m_mode_button += new SingleButton(Vec2i(24,24),Vec2i(16,16), Vec2i(remember_x, remember_y), tip_copy_vertex_part, "remember", remember, *g_tb);

	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 281), "open node control panel", "openControlPanel", openControlPanel, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(openSceneTree_x,openSceneTree_y), tip_scene_tree_window, "openSceneTree", openSceneTree, *g_tb);
	//m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24, 305 - 24), "pan view", "togglePanViewMode", togglePanViewMode, *g_tb);
	
	g_poseButton = new SingleButton(Vec2i(24,24), Vec2i(24*15, 209), "home/add geometry mode", "setHomeMode", setHomeMode, *g_tb);
	m_mode_button += g_poseButton;

	g_selectObjectButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 7, 257), "select object mode", "setSelectObjectMode", setSelectObjectMode, *g_tb);
	m_mode_button += g_selectObjectButton;
	

	g_animationButton = new SingleButton(Vec2i(24,24), Vec2i(24*13, 209), "animation mode", "setAnimationMode", setAnimationMode, *g_tb);
	m_mode_button+= g_animationButton;

	g_modellingButton = new SingleButton(Vec2i(24,24), Vec2i(24*17, 209), "modelling mode", "setModellingMode", setModellingMode, *g_tb);
	m_mode_button += g_modellingButton;

	g_paintButton = new SingleButton(Vec2i(24,24), Vec2i(48 + 24, 305 - 24), "paint mode", "setPaintMode", setPaintMode, *g_tb);
	m_mode_button += g_paintButton;

	g_polyEditButton = new SingleButton(Vec2i(24,24), Vec2i(24*11, 209), "poly edit mode", "setPolyEditMode", setPolyEditMode, *g_tb);
	m_mode_button += g_polyEditButton;

	g_ownershipButton = new SingleButton(Vec2i(24,24), Vec2i(360,48), "paint colour coded vertices mode", "openOwnership", openOwnership, *g_tb);
	m_mode_button += g_ownershipButton;

	g_paintNurbsColorCode = new SingleButton(Vec2i(24,24), Vec2i(48 + 24, 305 - 48), "paint colour coded control points mode", "setPaintColorCodedControlPoints", setPaintColorCodedControlPoints, *g_tb);
	m_mode_button += g_paintNurbsColorCode;

	g_xRayEditButton = new SingleButton(Vec2i(24,24), Vec2i(24, 233), "skeleton edit mode", "setXRayEditMode", setXRayEditMode, *g_tb);
	m_mode_button += g_xRayEditButton;
	//m_button3 += g_xRayEditButton;

	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(0, 209), "toggle x-ray mode", "toggleXRayMode", toggleXRayMode, *g_tb);


	m_mode_button += new TwoUpDownButton(Vec2i(24,24), Vec2i(24*3,24), tip_zoom, zoom, zoomIn, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24*0,24*1), "set front view orientation", "setFrontViewOrientation", setFrontViewOrientation, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24*1,24*1), "set back view orientation", "setBackViewOrientation", setBackViewOrientation, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24*2,24*1), "set left view orientation", "setLeftViewOrientation", setLeftViewOrientation, *g_tb);

	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24*15,24*3), "set right view orientation", "setRightViewOrientation", setRightViewOrientation, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24*16,24*3), "set bird's eye view orientation", "setBirdsEyeViewOrientation", setBirdsEyeViewOrientation, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(24*17,24*3), "set worm's eye view orientation", "setWormsEyeViewOrientation", setWormsEyeViewOrientation, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(resetViewOrientation_x, resetViewOrientation_y), "Snap view orientation (V)", "snap view orientation (V)", quantiseViewOrientation, *g_tb);

	m_mode_button += new SixUpDownButton(Vec2i(24*3,24), Vec2i(24*7,0), tip_view_orientation_at_right_angle, decViewOrientation90X, incViewOrientation90X, decViewOrientation90Y, incViewOrientation90Y, decViewOrientation90Z, incViewOrientation90Z, *g_tb);

	//m_button0 += new SingleButton(Vec2i(24,24), Vec2i(resetViewPosition_x,resetViewPosition_y), tip_resetViewPosition, "resetViewPosition", resetViewPosition, *g_tb);
	//m_button2 += new SingleButton(Vec2i(24,24), Vec2i(resetViewPosition_x,resetViewPosition_y), tip_resetViewPosition, "resetViewPosition", resetViewPosition, *g_tb);
	
	
	
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(centerViewToControlPoint_x,centerViewToControlPoint_y), tip_centerViewToControlPoint, "centerViewToControlPoint", centerViewToControlPoint, *g_tb);
	//m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(centerViewToPivotPoint_x,centerViewToPivotPoint_y), tip_centerViewToPivotPoint, "centerViewToPivotPoint", centerViewToPivotPoint, *g_tb);


	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(invertBackgroundColor_x,invertBackgroundColor_y), tip_toggle_background_color, "invertBackgroundColor", invertBackgroundColor, *g_tb);
	g_togglePerspectiveButton = new SingleButton(Vec2i(24,24), Vec2i(0, 233), tip_toggle_perspective, "togglePerspective", togglePerspective, *g_tb);
	m_mode_button += g_togglePerspectiveButton;
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(floorBuild_x,floorBuild_y), "open grid control panel", "grid", openGridControl, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(toggleWireframeMode_x,toggleWireframeMode_y), tip_toggleWireframe, "toggleWireFrame", toggleWireFrame, *g_tb);
	m_mode_button += new SingleButton(Vec2i(24,24), Vec2i(help_x,help_y), tip_help, "help", helpMenu, *g_tb);







	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(openNetFile_x,openNetFile_y), tip_open_smls_file_from_the_web, "openNetFile", openNetFile, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24),  Vec2i(24 * 9, 281), "settings", "openSettings", openSettings, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(96+24, 24), "open palette window", "openPalette", openPalette, *g_tb2);

	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(sphereBuildIcon_x, sphereBuildIcon_y),tip_build_sphere_for_selected_part, "buildSphereForSelPart", buildSphereForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(cylinderBuildIcon_x, cylinderBuildIcon_y),tip_build_cylinder_for_selected_part, "buildCylinderForSelPart", buildCylinderForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(coneBuildIcon_x,coneBuildIcon_y),tip_build_cone_for_selected_part, "buildConeForSelPart", buildConeForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(torusBuildIcon_x,torusBuildIcon_y),tip_build_torus_for_selected_part, "buildTorusForSelPart", buildTorusForSelPart, *g_tb2);
//	m_button1+=new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(bezierLatheBuildIcon_x,bezierLatheBuildIcon_y),tip_build_bezier_lathe_for_selected_part, "buildBezierLatheForSelPart", buildBezierLatheForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(boxBuildIcon_x,boxBuildIcon_y),tip_build_box_for_selected_part, "buildBoxForSelPart", buildBoxForSelPart, *g_tb2);
	//m_button1+=new SingleButton(Vec2i(24,24), nodeIconLookUp[RINGBUILD].size, nodeIconLookUp[RINGBUILD].srcPos,tip_build_rings_for_selected_part, "buildRingsForSelPart", buildRingsForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(rectangleBuildIcon_x, rectangleBuildIcon_y),tip_build_rectangle_for_selected_part, "newRectangle", newRectangle, *g_tb2);
	//g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(160+16*3,192-16),tip_build_av_segment_for_selected_part, "buildAvSegmentForSelPart", buildAvSegmentForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), builtInProtoList[NURBSLATHE]->iconSize, builtInProtoList[NURBSLATHE]->iconSrcPos, tip_build_nurbs_lathe_for_selected_part, "buildNurbsLatheForSelPart", buildNurbsLatheForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), builtInProtoList[BENDYLATHE]->iconSize, builtInProtoList[BENDYLATHE]->iconSrcPos,tip_build_bendy_lathe_for_selected_part, "buildBendyLatheForSelPart", buildBendyLatheForSelPart, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(24,24), Vec2i(48, 305 - 24), "add hair node for selected part", "addHairNodeForSelPart", addHairNodeForSelPart, *g_tb2);
	//g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 12, 233), "add nurbs line (click 2 times in 3d window to add first 2 ponts)", "addNurbsLine", addNurbsLine, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 19, 281), "create nurbs line (click 2 times in 3d window to add first 2 ponts)", "addNurbsLine", addNurbsLine, *g_tb2);
	g_m_add_geometry_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 17, 257), "draw nurbs line (drag to draw line)", "drawNurbsLine", drawNurbsLine, *g_tb2);
	g_createMirrorObjectButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 15, 281), "toggle create mirror object mode", "toggleCreateMirrorObjectMode", toggleCreateMirrorObjectMode, *g_tb2);
	g_m_add_geometry_button += g_createMirrorObjectButton;
	



//	m_button2 += new SingleButton(Vec2i(24,24),Vec2i(16,16), Vec2i(remember_x, remember_y), tip_copy_vertex_part, "remember", remember, *g_tb2);


	//m_button2+=new SixUpDownButton(Vec2i(54,24), Vec2i(0,0),tip_vertex_coord, decCoordX, incCoordX, decCoordY, incCoordY, decCoordZ, incCoordZ, *g_tb2);

	g_toggleBackfaceSelectButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 5, 0), tip_toggle_backface_ignore, "toggleBackfaceSelect", toggleBackfaceSelect, *g_tb2);
	g_toggleAccumulateSelectButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 7, 24 * 3), "toggle accumulate selection", "toggleAccumulateSelect", toggleAccumulateSelect, *g_tb2);
	g_toggleHidePolysInHiddenPatchesButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 11, 233 + 24), "toggle hide polys in hidden patches", "toggleHidePolysInHiddenPatches", toggleHidePolysInHiddenPatches, *g_tb2);
	
	//g_m_poly_edit_button += g_toggleBackfaceSelectButton;
	//g_m_poly_edit_button += g_toggleAccumulateSelectButton;

//	m_button0+=new SingleButton(Vec2i(24,24), Vec2i(24 * 5, 0), "toggle backface selection", "toggleBackfaceSelect", toggleBackfaceSelect, *g_tb2);
//	m_button2+=new SingleButton(Vec2i(24,24), Vec2i(24 * 5, 0), "toggle backface selection", "toggleBackfaceSelect", toggleBackfaceSelect, *g_tb2);
//	m_button0+=new SingleButton(Vec2i(24,24), Vec2i(24 * 11, 24), "toggle accumulate selection", "toggleAccumulateSelect", toggleAccumulateSelect, *g_tb2);
//	m_button2+=new SingleButton(Vec2i(24,24), Vec2i(24 * 11, 24), "toggle accumulate selection", "toggleAccumulateSelect", toggleAccumulateSelect, *g_tb2);

	//m_button0+=new SixUpDownButton(Vec2i(54,24), Vec2i(54,0), tip_vertex_color, decRed, incRed, decGreen, incGreen, decBlue, incBlue, *g_tb2);
	//m_button2+=new SixUpDownButton(Vec2i(54,24), Vec2i(54,0), tip_vertex_color, decRed, incRed, decGreen, incGreen, decBlue, incBlue, *g_tb2);

//	m_button0+=new SixUpDownButton(Vec2i(24*3,24), Vec2i(24*15,24*3), tip_view_orientation, decViewOrientationX, incViewOrientationX, decViewOrientationY, incViewOrientationY, decViewOrientationZ, incViewOrientationZ, *g_tb2);
//	m_button2+=new SixUpDownButton(Vec2i(24*3,24), Vec2i(24*15,24*3), tip_view_orientation, decViewOrientationX, incViewOrientationX, decViewOrientationY, incViewOrientationY, decViewOrientationZ, incViewOrientationZ, *g_tb2);

//	m_button0+=new SixUpDownButton(Vec2i(54,24), Vec2i(54*2,0), tip_view_position, decViewPositionX, incViewPositionX, decViewPositionY, incViewPositionY, decViewPositionZ, incViewPositionZ, *g_tb2);
//	m_button2+=new SixUpDownButton(Vec2i(54,24), Vec2i(54*2,0), tip_view_position, decViewPositionX, incViewPositionX, decViewPositionY, incViewPositionY, decViewPositionZ, incViewPositionZ, *g_tb2);


	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(breakVertexJoin_x,breakVertexJoin_y),tip_break_coord_join, "breakVertexJoin", breakVertexJoin, *g_tb2);
	////m_button3+=new SingleButton(Vec2i(24,24), Vec2i(breakColorJoin_x,breakColorJoin_y),"break color join (Alt B)", breakColorJoin, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(deleteTri_x,deleteTri_y), tip_delete_selected_triangle, "deleteTriangle", deleteTriangle, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(cutDownFromTheCursor_x, cutDownFromTheCursor_y), tip_delete_triangles_down_from_selected_vertex, "cutDownFromVertex", cutDownFromVertex, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(cutUpFromTheCursor_x,cutUpFromTheCursor_y), "delete triangles up from selected vertex (Shif U)", "cutUpFromVertex", cutUpFromVertex, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinVertexKeepBothColors_x,joinVertexKeepBothColors_y), tip_join_vertex_keep_both_colors, "joinVertexKeepBothColors", joinVertexKeepBothColors, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinVertexKeepDestColor_x,joinVertexKeepDestColor_y), tip_join_vertex_keep_dest_color, "joinVertexKeepDestColor", joinVertexKeepDestColor, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(transferCloseVerticesJoin_x,transferCloseVerticesJoin_y), tip_joinCloseVertices, "transferCloseVerticesJoin", transferCloseVerticesJoin, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinCloseVerticesMacro_x,joinCloseVerticesMacro_y), tip_joinCloseVertecesForAllShownParts, "joinCloseVerticesMacro", joinCloseVerticesMacro, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinMatchingColors_x,joinMatchingColors_y),tip_join_colors_in_vertices_that_equal_the_same_color_and_whose_coords_are_joined, "joinMatchingColors", joinMatchingColors, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(colorVertex_x,colorVertex_y),tip_paste_copied_color_to_selected_vertex, "colorVertex", colorVertex, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(pasteColorToTriangle_x, pasteColorToTriangle_y),"paste copied color to selected triangle (K))", "pasteColorToTriangle", pasteColorToTriangle, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(colorPart_x,colorPart_y),tip_paste_copied_color_to_part, "colorPart", colorPart, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(replaceColorPart_x,replaceColorPart_y),"paste copied color into all vertices that are colored the same as the selected vertex for only the selected part", "replaceColorPart", replaceColorPart, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(replaceColorSkeleton_x,replaceColorSkeleton_y),"paste copied color into all vertices that are colored the same as the selected vertex for all parts", "replaceColorSkeleton", replaceColorSkeleton, *g_tb2);
//	m_button3+=new SingleButton(Vec2i(24,24), Vec2i(compileScaleRotation_x,compileScaleRotation_y), tip_compile_Part_and_its_decendents_scale_and_rotation_fields, compileScaleRotation, *g_tb2);
	//m_button3+=g_tugRadButton;

//	m_button3+=new SingleButton(Vec2i(24,24), Vec2i(transferSingleVertex_x,transferSingleVertex_y),tip_paste_ownership_from_copied_part_into_the_selected_vertex, "transferSingleVertex", transferSingleVertex, *g_tb2);
	//m_button3 += g_togglePolyMorphCaptureButton;
//	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(zeroVetexX_x,zeroVetexX_y),tip_zeroToX, "zeroVetexX", zeroVetexX, *g_tb2);
	
////	m_button3+=new SingleButton(Vec2i(24,24), Vec2i(replaceWithCopyOfRememberedPartMirrorX_x,replaceWithCopyOfRememberedPartMirrorX_y),tip_paste_triangles_from_the_copied_part_in_X_mirrored_form_into_the_selected_part, replaceWithCopyOfRememberedPartMirrorX, *g_tb2);
////	m_button3+=new SingleButton(Vec2i(24,24), Vec2i(pasteAddCopyOfMirrorX_x,pasteAddCopyOfMirrorX_y),tip_add_triangles_from_the_copied_part_in_X_mirrored_form_to_the_selected_part, pasteAddCopyOfMirrorX, *g_tb2);
//	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(swapDivide_x,swapDivide_y), tip_swap_triangle_divide_between_the_copied_and_selected_triangles, "swapDivide", swapDivide, *g_tb2);
//	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(quickTriangle_x, quickTriangle_y), tip_quickTriangle, "quickTriangle", quickTriangle, *g_tb2);
//	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(mirrorTriangleImmediate_x,mirrorTriangleImmediate_y),tip_mirror_triangle, "mirrorTriangleImmediate", mirrorTriangleImmediate, *g_tb2);
//	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(projectSingleVertexOntoPoly_x,projectSingleVertexOntoPoly_y),"project selected vertex onto a polygon surface", "projectSingleVertexOntoPoly", projectSingleVertexOntoPoly, *g_tb2);
//	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(splitTriangleImmediate_x,splitTriangleImmediate_y),tip_split_triangle_between_the_vertices_marked_by_the_red_and_green_dots, "splitTriangleImmediate", splitTriangleImmediate, *g_tb2);

//	m_button1+=new SingleButton(Vec2i(24,24), nodeIconLookUp[SUBDIVISION].size, nodeIconLookUp[SUBDIVISION].srcPos,tip_build_subdivision_for_selected_part, "buildSubdivisionForSelPart", buildSubdivisionForSelPart, *g_tb2);
	//m_button1+=new SingleButton(Vec2i(24,24), nodeIconLookUp[MODELMORPHER].size, nodeIconLookUp[MODELMORPHER].srcPos,tip_build_model_morpher_for_selected_seamless_node, "buildModelMorpherForSelSmls", buildModelMorpherForSelSmls, *g_tb2);
	//m_button1+=new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(colorSweepIcon_x, colorSweepIcon_y),tip_add_a_ColorSweep_for_selected_parts, "colorSweepForSelParts", colorSweepForSelParts, *g_tb2);
	//m_button1+=new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(copyPartIcon_x, copyPartIcon_y),tip_copy_upper_selected_parts_to_lower_selected_parts, "copyPartForSelParts", copyPartForSelParts, *g_tb2);
	//m_button1+=g_placeSelParts;




	g_toggleSelectObjectButton = new SingleButton(Vec2i(24,24), Vec2i(24*5, 209), "toggle select whole patches mode", "selectWholePatch", selectWholePatch, *g_tb2);;
	g_openTransformCPButton = new SingleButton(Vec2i(24,24), Vec2i(24*7, 209), "open selected control points transform panel (Ctrl T)", "openTransformCP", openTransformCP, *g_tb2);

	//m_button1 += g_togglePolyMorphCaptureButton;

	g_m_poly_edit_button += g_deleteTriButton;
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinVertexKeepBothColors_x,joinVertexKeepBothColors_y), tip_join_vertex_keep_both_colors, "joinVertexKeepBothColors", joinVertexKeepBothColors, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinVertexKeepDestColor_x,joinVertexKeepDestColor_y), tip_join_vertex_keep_dest_color, "joinVertexKeepDestColor", joinVertexKeepDestColor, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(transferCloseVerticesJoin_x,transferCloseVerticesJoin_y), tip_joinCloseVertices, "transferCloseVerticesJoin", transferCloseVerticesJoin, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinCloseVerticesMacro_x,joinCloseVerticesMacro_y), tip_joinCloseVertecesForAllShownParts, "joinCloseVerticesMacro", joinCloseVerticesMacro, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(copyAndPasteMirroredPart_x,copyAndPasteMirroredPart_y),tip_copyAndPasteMirroredPart, "copyAndPasteMirroredPart", copyAndPasteMirroredPart, *g_tb2);

	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(swapDivide_x,swapDivide_y), tip_swap_triangle_divide_between_the_copied_and_selected_triangles, "swapDivide", swapDivide, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(quickTriangle_x, quickTriangle_y), tip_quickTriangle, "quickTriangle", quickTriangle, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(mirrorTriangleImmediate_x,mirrorTriangleImmediate_y),tip_mirror_triangle, "mirrorTriangleImmediate", mirrorTriangleImmediate, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(splitTriangleImmediate_x,splitTriangleImmediate_y),tip_split_triangle_between_the_vertices_marked_by_the_red_and_green_dots, "splitTriangleImmediate", splitTriangleImmediate, *g_tb2);
	g_m_poly_edit_button += new TwoUpDownButton(Vec2i(24,24), Vec2i(24*12,24), tip_swap_vertexs_index_order_with_previous_next, swapVertexOrderDec, swapVertexOrderInc, *g_tb2);
	g_m_poly_edit_button += new TwoUpDownButton(Vec2i(24,24), Vec2i(24*10,24), tip_select_previous_next_triangle_that_shares_the_selected_vertex, decToNextTriThatSharesActivePoint, incToNextTriThatSharesActivePoint, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(findThinTriangle_x,findThinTriangle_y), tip_findThinTriangle, "findThinTriangle", findThinTriangle, *g_tb2);


	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(96+24, 24), "open palette window", "openPalette", openPalette, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(13 * 24, 281), "inside out part", "insideOutPart", insideOutPart, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(24*15, 209), "paint fill", "openFill", openFill, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(colorVertex_x,colorVertex_y),tip_paste_copied_color_to_selected_vertex, "colorVertex", colorVertex, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(pasteColorToTriangle_x, pasteColorToTriangle_y),"paste copied color to selected triangle (K))", "pasteColorToTriangle", pasteColorToTriangle, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(joinMatchingColors_x,joinMatchingColors_y),tip_join_colors_in_vertices_that_equal_the_same_color_and_whose_coords_are_joined, "joinMatchingColors", joinMatchingColors, *g_tb2);

	g_m_poly_edit_button += new SingleButton(Vec2i(24,24),Vec2i(16,16), Vec2i(remember_x, remember_y), tip_copy_vertex_part, "remember", remember, *g_tb2);
	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(224,192), tip_do_build_cycle, "doBuild", doBuild, *g_tb2);
	//g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(toggleWireframeMode_x,toggleWireframeMode_y), tip_toggleWireframe, "toggleWireFrame", toggleWireFrame, *g_tb2);

	g_m_poly_edit_button += new SingleButton(Vec2i(24,24), Vec2i(14 * 24, 281),"more", "buildF10Menu", buildF10Menu, *g_tb2);

//	m_button3+=g_tugRadButton;

	//m_button3 += new TwoUpDownButton(Vec2i(24,24), Vec2i(24*12,24), tip_swap_vertexs_index_order_with_previous_next, swapVertexOrderDec, swapVertexOrderInc, *g_tb2);


	//m_button1+=new TwoUpDownButton(Vec2i(24,24), Vec2i(24*14,24*3), tip_select_previous_next_vertex_in_index_order, selectPreviousVertex, selectNextVertex, *g_tb2);
	//m_button3+=new TwoUpDownButton(Vec2i(24,24), Vec2i(24*14,24*3), tip_select_previous_next_vertex_in_index_order, selectPreviousVertex, selectNextVertex, *g_tb2);

	//m_button3 += new TwoUpDownButton(Vec2i(24,24), Vec2i(24*10,24), tip_select_previous_next_triangle_that_shares_the_selected_vertex, decToNextTriThatSharesActivePoint, incToNextTriThatSharesActivePoint, *g_tb2);

//	m_button1+=new TwoUpDownButton(Vec2i(24,24), Vec2i(24*11,24), tip_select_previous_next_corner_in_triangle, selectPreviousCornerInTri, selectNextCornerInTri, *g_tb2);
//	m_button3+=new TwoUpDownButton(Vec2i(24,24), Vec2i(24*11,24), tip_select_previous_next_corner_in_triangle, selectPreviousCornerInTri, selectNextCornerInTri, *g_tb2);

	//m_button3 += new SingleButton(Vec2i(24,24), Vec2i(findThinTriangle_x,findThinTriangle_y), tip_findThinTriangle, "findThinTriangle", findThinTriangle, *g_tb2);


	//m_button1 += new SingleButton(Vec2i(24,24), Vec2i(resetRotations_x,resetRotations_y), tip_reset_rotations, "resetRotations", resetRotations, *g_tb2);
	//m_button3 += new SingleButton(Vec2i(24,24), Vec2i(resetRotations_x,resetRotations_y), tip_reset_rotations, "resetRotations", resetRotations, *g_tb2);

	

//	m_button1 += new SingleButton(Vec2i(24*8,24), Vec2i(1,1), Vec2i(1,1), 0, dummyFunction, *g_tb2);


//	for(int bar = 0, barY = BORDER_DIST; bar < g_mm_build_button.length; bar++, barY += SINGLE_BAR_CY){
//		M_Button& m_button = g_mm_build_button[bar];
//		for(int k=0, h = TOOLBAR_BUTTON_OFFSET_X; k < m_button.length; k++){
//			if(bar == 0){
//				if(k == 4){
//					h += 24;
//				}
////				else if(k == 12){
////					h += 24 + 12;
////				}
//			}
//			else if(bar == 1){
//				if(k == 10){
//					h += 24 * 1;
//				}
//				if(k == 15){
//					h += 24 * 1;
//				}
//			}
//			Button* button = m_button[k];
//			button->dstPos = Vec2i(h, barY);
//			h += button->bSize.x;
//		}
//	}
//	for(int bar = 0, barY = BORDER_DIST; bar < g_mm_immediate_button.length; bar++, barY += SINGLE_BAR_CY){
//		M_Button& m_button = g_mm_immediate_button[bar];
//		for(int k=0, h = TOOLBAR_BUTTON_OFFSET_X; k < m_button.length; k++){
//			if(bar == 0){
//				//if(k == 13){
//				//	h += 24 + 12;
//				//}
//			}
//			else if(bar == 1){
//				if(k == 15){
//					//h += 1 * 24;
//				}
//			}
//			Button* button = m_button[k];
//			button->dstPos = Vec2i(h, barY);
//			h += button->bSize.x;
//		}
//	}
//
//
//	m_button4 = m_button0;
//
	g_boxPatchModellingButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 3, 233), "toggle box modelling mode", "boxMmodellingMode", toggleBoxPatchModellingMode, *g_tb2);
//
	g_selectRingButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 0, 257), "select ring (R)", "selectRing", selectRing, *g_tb2);
	//g_selectRingsButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 1, 257), "select rings", "toggleSelectRings", toggleSelectRings, *g_tb2);
//
//
	g_m_paint_button += new SingleButton(Vec2i(24,24), Vec2i(96+24, 24), "open palette window", "openPalette", openPalette, *g_tb2);
	g_m_paint_button += new SingleButton(Vec2i(24,24), Vec2i(5 * 24, 281), "brush", "openBrush", openBrush, *g_tb2);
	g_m_paint_button += new SingleButton(Vec2i(24,24), Vec2i(10 * 24, 209), "fill", "openFill", openFill, *g_tb2);
	g_m_paint_button += new SingleButton(Vec2i(24,24), Vec2i(4 * 24, 281), "color picker", "openColorPicker", openColorPicker, *g_tb2);
	g_paintToggleBackfaceSelectButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 5, 0), tip_toggle_backface_ignore, "toggleBackfaceSelect", toggleBackfaceSelect, *g_tb2);
	g_m_paint_button += g_paintToggleBackfaceSelectButton;

//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 233), "rectangle select (drag to select)", "rectangleSelect", rectangleSelect, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 5, 233), "convert to patch", "convertToPatch", convertToPatch, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 4, 233), "extrude", "extrudePatch", extrudePatch, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 7, 233), "insert control line (Ctrl I)", "insertControlPoint", insertControlPoint, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 233), "delete control line", "deleteControlPoint", deleteControlPoint, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 6, 233), "cut patches", "kutPatches", cutPatches, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 10, 233), "merge patches", "mergePatches", mergePatches, *g_tb2);
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 7, 233), "rotate control staves", "rotateCtrlStaves", rotateCtrlStaves, *g_tb2);
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 233), menu_rotate_uv_90, "rotate_uv_90", rotate_uv_90, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 12, 233), "flip polarity", "flipPolarity", flipPolarity, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 13, 233), "inside out", "insideOutPatch", insideOutPatch, *g_tb2);

g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 15, 233), "mirror control points", " mirrorCtrlPoints",  mirrorCtrlPoints, *g_tb2);

//
//
//
//
//
//
//
//
//
//
//
//
//
//
g_animStitchPolyModeButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 9, 233 - 24), "toggle stitch poly mode", "toggleStitchPolyMode", toggleStitchPolyMode, *g_acp);

g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 233 - 24), "show patch borders mode", "toggleShowPatchBordersMode", toggleShowPatchBordersMode, *g_tb2);

//g_stitchPolyModeButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 9, 233 - 24), "toggle stitch poly mode", "toggleStitchPolyMode", toggleStitchPolyMode, *g_tb2);
g_stitchPolyModeButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 9, 233 - 24), "toggle stitch poly mode", "toggleBoxPatchModellingMode", toggleBoxPatchModellingMode, *g_tb2);

g_m_modeling_button += g_stitchPolyModeButton;
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 16, 233), "generate raw stitch polys", "generateRawStitchPolys", generateRawStitchPolys, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 17, 233), "stitch selected points (J)", "joinVertexKeepDestColor", joinVertexKeepDestColor, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 11, 233), "unstitch selected points (B)", "breakVertexJoin", breakVertexJoin, *g_tb2);

//	
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 18, 281), "paste control points", "pasteCtrlPoints", pasteCtrlPoints, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 17, 281), "move selected control point to nearest other (M)", "moveVertex", moveVertex, *g_tb2);

g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(16,16), Vec2i(256, 160), "delete selected patches", "deleteSelectedPatches", deleteSelectedPatches, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 18, 281), "duplicate selected patches", "duplicate", duplicate, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 9, 233), "duplicate mirror selected patches", "duplicateMirror", duplicateMirror, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 4, 257), "show all cages (Alt A)", "showAllCages", showAllCages, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 5, 257), "show selected cages (Alt S)", "showSingleCage", showSingleCage, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 6, 257), "hide selected cages (Alt H)", "hideSelectedCages", hideSelectedCages, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 15, 257), "show selected groups (Alt S)", "showSingleCage", showSelectedGroups, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 14, 257), "hide selected groups", "hideSelectedCages", hideSelectedGroups, *g_tb2);

g_m_modeling_button += g_toggleHidePolysInHiddenPatchesButton;
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 7, 257), "align selecteded points (Shift A)", "alignMidSelectededPoints", alignMidSelectededPoints, *g_tb2);

//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 257), "inside out selected stitch polies", "invertSelCtrlPoly", invertSelCtrlPoly, *g_tb2);
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 257), "hide/show mesh", "hideShowMesh", hideShowMesh, *g_tb2);
//g_toggleCurvedCageButton = new SingleButton(Vec2i(24,24), Vec2i(24, 305 - 24), "toggle curved cage mode", "toggleCurvedCageMode", toggleCurvedCageMode, *g_tb2);
//g_m_modeling_button += g_toggleCurvedCageButton;
g_toggleZBufferCageButton = new SingleButton(Vec2i(24,24), Vec2i(24, 305 - 24), "toggle zbuffer control cage mode (space)", "toggleZBufferCageMode", toggleZBufferCageMode, *g_tb2);
g_m_modeling_button += g_toggleZBufferCageButton;

//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 18, 233), "paste stave", "pasteColumn", pasteColumn, *g_tb2);
g_toggleSingleActiveButton = new SingleButton(Vec2i(24,24), Vec2i(3 * 24, 233), "unactivate/activate selected points (A)", "setSingleActiveMode", setSingleActiveMode, *g_tb2);
g_m_modeling_button += g_toggleSingleActiveButton;

//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 18, 233), "delete polies owned by selected patch", "deletePoliesOwnedByPatch", deletePoliesOwnedByPatch, *g_tb2);
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(resetViewPosition_x,resetViewPosition_y), "delete selected node", "deleteNode", deleteNode, *g_tb2);

//
g_m_modeling_button += g_openTransformCPButton;
g_m_modeling_button += g_toggleSelectObjectButton;

g_m_modeling_button += g_selectRingButton;
//g_m_modeling_button += g_selectRingsButton;
g_m_modeling_button += g_tugRadButton;
//g_m_modeling_button += g_transformFallOffButton;
//
//g_m_modeling_button += g_boxPatchModellingButton;



	//g_m_modeling_button += g_openTransformCPButton;
	//g_m_modeling_button += g_toggleSelectObjectButton;
	//g_m_modeling_button += g_tugRadButton;

//
//


g_m_modeling_button += g_toggleBackfaceSelectButton;
g_m_modeling_button += g_toggleAccumulateSelectButton;
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 13, 257), "dimension tug filter (Alt Space)", "dimFilter", setShowDimFilterVisual, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(11 * 24, 281), "control point rectangle select", "rectangleSelect", rectangleSelect, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 16, 257), "control point rectangle unselect", "rectangleUnselect", rectangleUnselect, *g_tb2);
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(12 * 24, 281), "stitch poly rectangle select", "rectangleSelectStitchPoly", rectangleSelectStitchPoly, *g_tb2);
g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(14 * 24, 281), "more", "moreModellingMenu", moreModellingMenu, *g_tb2);
//g_m_modeling_button += new SingleButton(Vec2i(24,24), Vec2i(toggleWireframeMode_x,toggleWireframeMode_y), tip_toggleWireframe, "toggleWireFrame", toggleWireFrame, *g_tb2);

	g_skeletonEditStitchPolyModeButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 9, 233 - 24), "hide/show mesh", "toggleStitchPolyMode", toggleStitchPolyMode, *g_tb2);
	g_m_xRayEdit_button += g_skeletonEditStitchPolyModeButton;

	g_m_xRayEdit_button += new SingleButton(Vec2i(24,24), Vec2i(24 * 8, 257), "add bone mode", "setAddBoneMode", setAddBoneMode, *g_tb2);
	//g_m_xRayEdit_button += new SingleButton(Vec2i(24,24),Vec2i(16,16), Vec2i(remember_x, remember_y), "copy (Ctrl C)", "remember", remember, *g_tb2);
	//g_m_xRayEdit_button += new SingleButton(Vec2i(24,24),Vec2i(16,16), Vec2i(16, 192), "paste (Ctrl V)", "pasteBone", pasteBone, *g_tb2);
	g_m_xRayEdit_button += new SingleButton(Vec2i(24,24),Vec2i(16,16), Vec2i(delete_x, delete_y), "delete bone (Delete)", "deleteNode", deleteNode, *g_tb2);

	g_dragWithDescendantsButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 9, 257), "toggle drag with descendants mode", "toggleDragWithDescendantsMode", toggleDragWithDescendantsMode, *g_tb2);
	g_m_xRayEdit_button += g_dragWithDescendantsButton;

	g_createMirrorBoneButton = new SingleButton(Vec2i(24,24), Vec2i(24 * 15, 281), "toggle create mirror object mode", "toggleCreateMirrorObjectMode", toggleCreateMirrorObjectMode, *g_tb2);
	g_m_xRayEdit_button += g_createMirrorBoneButton;

	int barY = BORDER_DIST;
	for(int k = 0, h = TOOLBAR_BUTTON_OFFSET_X; k < m_mode_button.length; k++){
		if(k == 6){
			h += 24 * 1;
		}
		if(k == 15){
			h += 24 * 1;
		}
		Button* button = m_mode_button[k];
		button->dstPos = Vec2i(h, barY);
		h += button->bSize.x;
	}

	//barY = BORDER_DIST + SINGLE_BAR_CY;
	for(int k = 0, h = TOOLBAR_BUTTON_OFFSET_X; k < g_m_add_geometry_button.length; k++){
		if(k == 1){
			h += 24 * 2;
		}
		if(k == 2){
			h += 24 * 3;
		}
		if(k == 3){
			h += 24 * 1;
		}
		//if(k == 13){
		//	h += 24 * 12;
		//}
		Button* button = g_m_add_geometry_button[k];
		button->dstPos = Vec2i(h, barY);
		h += button->bSize.x;
	}

	for(int k = 0, h = TOOLBAR_BUTTON_OFFSET_X; k < g_m_modeling_button.length; k++){
		//if(k == 4){
		//	h += 24 * 2;
		//}
		//if(k == 9){
		//	h += 24 * 1;
		//}

		//if(k == 17){
		//	h += 24 * 1;
		//}
		//if(k == 22){
			//h += 24 * 1;
		//}
		Button* button = g_m_modeling_button[k];
		button->dstPos = Vec2i(h, barY);
		h += button->bSize.x;
	}
	for(int k = 0, h = TOOLBAR_BUTTON_OFFSET_X; k < g_m_paint_button.length; k++){
		if(k == 0){
			h += 24 * 7;
		}
		if(k == 1){
			h += 24 * 1;
		}
		if(k == 4){
			h += 24 * 1;
		}
		Button* button = g_m_paint_button[k];
		button->dstPos = Vec2i(h, barY);
		h += button->bSize.x;
	}
	for(int k = 0, h = TOOLBAR_BUTTON_OFFSET_X; k < g_m_poly_edit_button.length; k++){
		if(k == 6){
			h += 24 * 3;
		}
		if(k == 12){
			h += 24 * 3;
		}
		if(k == 18){
			h += 24 * 4;
		}
		Button* button = g_m_poly_edit_button[k];
		button->dstPos = Vec2i(h, barY);
		h += button->bSize.x;
	}
	for(int k = 0, h = TOOLBAR_BUTTON_OFFSET_X; k < g_m_xRayEdit_button.length; k++){
		if(k == 1){
			h += 24 * 3;
		}
		if(k == 3){
			h += 24 * 3;
		}
		Button* button = g_m_xRayEdit_button[k];
		button->dstPos = Vec2i(h, barY);
		h += button->bSize.x;
	}


	String language;

	g_tt->size = Vec2i(100, 100);
	g_tb->size = Vec2i(UPPER_TOOLBAR_CX,TOOLBAR_CY + BORDER_DIST * 2);

	g_tb2->size = Vec2i(LOWER_TOOLBAR_CX,TOOLBAR_CY + BORDER_DIST * 2);

	g_settings->showToolBar = g_settings->showToolBar.refValue();
}




void updateFuseSurfaceClue();

void ToolBarWin::updateToolBarContext(){
	//paint();
	updateFuseSurfaceClue();
	M_Button*& m_context_bar = g_tb2->m_button;

	if(dynamic_cast<Lock*>(g_selNode.getRaw()) && (g_scene->mode & HAIR_REGION_MODE)){
		m_context_bar = &g_m_hair_button;
	}
	else if(g_scene->mode & MODELLING_MODE){
		m_context_bar = &g_m_modeling_button;
	}
	else if(g_scene->mode & PAINT_MODE){
		m_context_bar = &g_m_paint_button;
	}
	else if(g_scene->mode & POLY_EDIT_MODE){
		m_context_bar = &g_m_poly_edit_button;
	}
	else if(g_scene->mode & SELECT_OBJECT_MODE){
		m_context_bar = &g_m_select_object_button;
	}
	else if(g_scene->mode & HOME_MODE){
		m_context_bar = &g_m_add_geometry_button;
	}
	else if(g_scene->mode & XRAY_EDIT_MODE){
		m_context_bar = &g_m_xRayEdit_button;
	}
	//else{
	//	switch(g_selNode->getType()){
	//		case IMMEDIATEPART:{
	//			mm_button = &g_mm_immediate_button;
	//			break;
	//		}
	//		case NURBSPATCH:{
	//			mm_button = &g_mm_patch_button;
	//			break;
	//		}
	//		default:{
	//			mm_button = &g_mm_build_button;
	//		}
	//	}
	//}
	//if(type==PART){
	//	if(g_vcArtAnimBase == g_visualClue){
	//		g_mm_build_button[1][15] = g_togglePolyMorphCaptureButton;
	//		paintFlag = true;
	//	}
	//	else{
	//		g_mm_build_button[1][15] = g_deleteTriButton;
	//		paintFlag = true;
	//	}
	//}
	//if(dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
	//	if(g_mm_build_button[1][14]->baseCommand!=reinterpret_cast<int>(decTugRadius)){
	//		g_mm_build_button[1][14] = g_tugRadButton;
	//		paintFlag = true;
	//	}
	//}
	//else{
	//	if(g_mm_build_button[1][15]->baseCommand!=reinterpret_cast<int>(placeForSelParts)){
	//		g_mm_build_button[1][15] = g_placeSelParts;
	//		paintFlag = true;
	//	}
	//}
	g_tb->paint();
	g_tb2->paint();
}

void updateIconPointers(){

	if(g_scene->mode & BACKFACE_IGNORE_MODE){
		g_toggleBackfaceSelectButton->srcPos = Vec2i(24 * 5, 0);
		g_animToggleBackfaceSelectButton->srcPos = Vec2i(24 * 5, 0);
		g_paintToggleBackfaceSelectButton->srcPos = Vec2i(24 * 5, 0);
	}
	else{
		g_toggleBackfaceSelectButton->srcPos = Vec2i(24 * 6, 0);
		g_animToggleBackfaceSelectButton->srcPos = Vec2i(24 * 6, 0);
		g_paintToggleBackfaceSelectButton->srcPos = Vec2i(24 * 6, 0);
	}

	if(g_scene->mode & STITCH_POLY_MODE){
		g_stitchPolyModeButton->srcPos = Vec2i(24 * 8, 233 - 24);
		g_skeletonEditStitchPolyModeButton->srcPos = Vec2i(24 * 8, 233 - 24);
		g_animStitchPolyModeButton->srcPos = Vec2i(24 * 8, 233 - 24);
	}
	else{
		g_stitchPolyModeButton->srcPos = Vec2i(24 * 9, 233 - 24);
		g_skeletonEditStitchPolyModeButton->srcPos = Vec2i(24 * 9, 233 - 24);
		g_animStitchPolyModeButton->srcPos = Vec2i(24 * 9, 233 - 24);
	}


	if(g_scene->mode & SINGLE_ACTIVE_MODE){
		g_toggleSingleActiveButton->srcPos = Vec2i(3 * 24, 233);
		g_animToggleSingleActiveButton->srcPos = Vec2i(3 * 24, 233);

	}
	else{
		g_toggleSingleActiveButton->srcPos = Vec2i(2 * 24, 233);
		g_animToggleSingleActiveButton->srcPos = Vec2i(2 * 24, 233);
	}

	//if(g_curvedCageMode){
	//	g_toggleCurvedCageButton->srcPos = Vec2i(0, 305 - 24);
	//}
	//else{
	//	g_toggleCurvedCageButton->srcPos = Vec2i(24, 305 - 24);
	//}
	if(g_scene->mode & ZBUFFER_CAGE_MODE){
		g_toggleZBufferCageButton->srcPos = Vec2i(0, 305 - 24);
		g_animToggleZBufferCageButton->srcPos = Vec2i(0, 305 - 24);
	}
	else{
		g_toggleZBufferCageButton->srcPos = Vec2i(24, 305 - 24);
		g_animToggleZBufferCageButton->srcPos = Vec2i(24, 305 - 24);
	}

	if(g_scene->mode.refValue() & ENABLE_MORPH_CAPTURE){
		g_togglePolyMorphCaptureButton->srcPos = Vec2i(24 * 2, 208);
	}
	else{
		g_togglePolyMorphCaptureButton->srcPos = Vec2i(24 * 1, 208);
	}

	if(g_scene->mode.refValue() & CREATE_MIRROR_OBJECT){
		g_createMirrorObjectButton->srcPos = Vec2i(24 * 15, 281);
		g_createMirrorBoneButton->srcPos = Vec2i(24 * 15, 281);
	}
	else{
		g_createMirrorObjectButton->srcPos = Vec2i(24 * 16, 281);
		g_createMirrorBoneButton->srcPos = Vec2i(24 * 16, 281);
	}

	if(g_dragWithDescendants){
		g_dragWithDescendantsButton->srcPos = Vec2i(Vec2i(24 * 9, 257));
	}
	else{
		g_dragWithDescendantsButton->srcPos = Vec2i(Vec2i(24 * 10, 257));
	}

	if(g_perspective){
		g_togglePerspectiveButton->srcPos = Vec2i(togglePerspective_x, togglePerspective_y);
	}
	else{
		g_togglePerspectiveButton->srcPos = Vec2i(0, 233);
	}
}
