/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include <windows.h>
#include <direct.h>

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "seamless3d_gui_render.h"
#include "gui.h"
#include "ms-gui.h"
#include "scene_tree.h"
#include "robot_aid.h"

extern bool g_useWinTab;

extern const char* msg_needPartOrCenterJoint;
extern const char* msg_needPartOrBezier;
extern const char* msg_centerJointHasNoRef;
extern const char* msg_needPart;
extern const char* msg_needWhitePart;
extern const char* msg_needBluePart;
extern const char* msg_needPartOrSmls;
extern const char* msg_needPartCopied;
extern const char* msg_needPartOrMorpher;
extern const char* msg_reverse_operation;
extern const char* msg_vertex_belongs_to_only_one;
extern const char* msg_divideMessage;
extern const char* msg_one_tri_must_exist;
extern const char* msg_both_tri_must_not_be_same;
extern const char* msg_both_tri_can_not_shar_3_points;
extern const char* msg_both_tri_must_share_2_points;
extern const char* msg_num_of_sel_must_be_more_than_one;
extern const char* msg_num_of_sel_must_be_even;

extern const char* name_front_view;
extern const char* name_left_view;
extern const char* name_right_view;
extern const char* name_back_view;
extern const char* name_birds_eye_view;
extern const char* name_worms_eye_view;
extern const char* name_showcase_view;
extern const char* name_floor;
extern const char* name_ruller;

extern ShapePtr g_vcSelectRectShape;
extern M_Matrix m_matrix;
extern M_Int m_smi;
extern M_Vec3f g_m_coord;
extern M_Bool g_pointIsVisible;
extern IndexedLineSet* g_nurbsIls;
extern Ptr<Region> g_region;
extern M_Int g_pointLookup;
extern Coordinate* g_nurbsIlsCoord;
extern bool g_curvedCageMode;
extern bool g_viewPosDragMode;
Ptr<Thinner> g_thinner = new Thinner;
extern int g_fps;

bool Viewpoint::neutral = false;
bool Anim::autoCapture = true;
bool g_dragWithDescendants = true;

ImmediatePart* convertPartToImmediate(Art* part);
Part* convertImmediateToPart(Art* immediate);
int getSeamlessParent(int line);
Seamless* getSeamless(int& selLine);
void buildModelMorpherForSelSmls(Seamless* smls);
StretchCurvePtr g_stretchCurve = new StretchCurve;
Ptr<TransformCP> g_transformCP = new TransformCP;
Ptr<AnimLever> g_vcArtAnimBase = new AnimLever;
Ptr<TransformObj> g_transformObj = new TransformObj;
Ptr<PasteOptions> g_pasteOptions = new PasteOptions;
Ptr<DeleteOptions> g_deleteOptions = new DeleteOptions;
Ptr<Extrude> g_extrude = new Extrude;
Ptr<InsertComplete> g_insertComplete = new InsertComplete;
Ptr<CutComplete> g_cutComplete = new CutComplete;
ConvertToPatchPtr g_convertToPatch=new ConvertToPatch;
Ptr<ReduceCapturedPoses> g_reduceCapturedPoses = new ReduceCapturedPoses;
void updateNurbsCurveKnotMid0(int dim, M_Float& knot);
void updateNurbsCurveKnotMid1(int dim, M_Float& knot);
void openWithHtmlBrowser(String url, const String htmlBrowser);
void replaceNodeRefInSceneTree(NodePtr from, Node* to);
void commonInitInsertNodeStuff(Node* node, Field*& field, Node*& newNode, unsigned fieldIdx, unsigned nodeType);
void checkForNameClash(M_Node& importDefList, M_Node& existingDefList);
void resetDefList();
void snapRotationLever();
void duplicate();
bool isPointInTriangleHit(Vec3f& clickPos, Vec3f a, Vec3f b, Vec3f c, const bool perspective);
void get2dMousePosFrom3d();
void pivotPointToClipBoard();
void deletePatch(NurbsPatch* patch);
void addIfNotInList(NurbsPatch* patch, M_<NurbsPatchPtr>& m_delPatchList);
void deletePatches(M_<NurbsPatchPtr>& m_delPatchList);
void toggleHomeMode();

#ifdef PROG_ROB
	extern bool g_altDown;
#endif
bool g_forceSaveAs = true;
extern bool g_browseMode;
extern bool tugFlag;
extern bool g_dontBackup;
extern bool g_showPatchBorderMode;

float g_joinRange = .001;
CenterJointPtr g_immediateCenterJoint=new CenterJoint;
extern StemPtr g_stem = new Stem;//used for stemLess bnspe;
extern Ptr<PartClue> g_vcArtTriBase;
extern ShapePtr g_vcEllipticalPolyline2D;
extern ShapePtr g_transformLeverClue;
extern bool g_optimized;


void searchDoBuild(M_Node& children);
void keyboardRadialTug(Vec3f coord, Vec3f pt);
void removeRouteFromProto(Node* windowNode);
ProtoDeclare* getSceneProtoDeclare(int selLine);
void insertElement(bool move, bool replace, int after);
void createUniqueName(Node* node);
void quantizeInc(float& v);
void quantizeDec(float& v);
void recurseShow(Art* part, bool show);
bool backFace(const Vec3f& v0, const Vec3f& v1, const Vec3f& v2);
bool backFace(const Vec2f& v0, const Vec2f& v1, const Vec2f& v2);
bool makeNameLegalForCpp(String& str);
void extractFileName(String& filename, const String& path);
void getFFmpegPath(String& commandLine);
float getHighestPoint(M_Vec2f& point);
float getLowestPoint(M_Vec2f& point);
bool getPolyMap(NurbsPatch*& patch, Stem*& stem, Polymap*& polymap, Art* part, int selTriIndex, ImmediatePart*& sourcePart);
bool isTriangleHit(const Vec2f& pt, const Vec2f* v);
void saveRelWinMousePos();
void emptyPacketQue();
void closeRecentFileWindow();
void scrollUpCP();
void scrollDownCP();
void scrollPageUpCP();
void scrollPageDownCP();
extern Ptr<Palette> g_palette;
void applyOwnership(Art* smls);
void uvApplyOwnership(Art* smls);
void applyColor(Seamless* smls);
PaletteCP* paletteCPAllreadyOpen();
void captureFrame();
void showAllCages();
void hideSelectedCages();
void showSingleCage();
void incorporate();
void extrudePatch();
void extrudeRegion();
void toggleRecordSoundMovie();
void calcAllPolyMidPoints();
void fillEdgesWithCtrlPolys();
void mergeCtrlPolys();
void stitchFlower();
void mergeTo5CtrlPoly();
void mergeTo6CtrlPoly();
void linkPointToEdge();
void openTransformCP();
void snapPivotPointToControlPoint();
void invertSelCtrlPoly();
void pasteColumn();
void deleteLink();
void compileMorph();
void randomizeHarmonicFreq();
void loadSDIF();
void makeTrianglesTiny();
void setAllIns(M_Build& m_build, const bool inState);
void hairOnSurface(M_Build& m_build);
void openBrush();
void openFill();
void locateBuildNode(Art* part, int selTriIndex);
int getSeamless2Line();
void genHairFromPatch();
void bnspeBuildPolies();
void setUpMidi();
void joinBorderVertices();
bool cutUpPatches(Seamless* smls, M_Insert& m_insert, M_MirrorPatch& m_nurbsPatch);
void updateIconPointers();
void shortcutToggleWireFrame();
void renderRayTrace();
void toggleShowCameras();
void deleteAllStitchPolys();
void clone(Hair* hair, Strand* oldSd, Strand* newSd);
Seamless* getSeamless2();
void setCpIndex2(NurbsPatch* patchDst, int idxDst, const NurbsPatch* patchSrc, int idxSrc);

int g_animBarListIndex = 0;
ArtPtr g_lastSelectedPart;
BuildPtr g_lastSelectedPatch;
extern BrushPtr g_brush;
extern FillPtr g_fill;
extern Ptr<Grid> g_grid;
Ptr<Duplicate> g_duplicate = new Duplicate;
Ptr<InsideOutPart> g_insideOutPart = new InsideOutPart;
extern Ptr<ExportMovie> g_exportMovie;
extern Ptr<ExportVrmlX3d> g_exportVrmlX3d;
extern Ptr<ColorPicker> g_colorPicker;
extern Node* g_parent;
extern bool g_showCamerasInAnimMode;
void setShowDimFilterVisual();
void updateColorFromColorMappers();

bool g_runAnim = true;

//SingleButton* g_m_mode_button = 0;
//SingleButton* g_polyEditButton = 0;
//SingleButton* g_modellingButton = 0;
//SingleButton* g_paintButton = 0;
//SingleButton* g_animationButton = 0;
//SingleButton* g_xRayEditButton = 0;
//SingleButton* g_ownershipButton = 0;
//SingleButton* g_paintNurbsColorCode = 0;

SingleButton* g_boxPatchModellingButton = 0;
SingleButton* g_selectRingButton = 0;
SingleButton* g_selectRingsButton = 0;

SingleButton* g_toggleHidePolysInHiddenPatchesButton = 0;
SingleButton* g_animToggleHidePolysInHiddenPatchesButton = 0;
SingleButton* g_toggleAccumulateSelectButton = 0;
SingleButton* g_animToggleAccumulateSelectButton = 0;
SingleButton* g_togglePolyMorphCaptureButton = 0;
SingleButton* g_toggleSelectObjectButton = 0;
SingleButton* g_togglePerspectiveButton = 0;

Build* isNurbsCageNode(Node* node){
	if(Build* build = dynamic_cast<Build*>(node)){
		if(build->getPatch()){
			return build;
		}
	}
	return false;
}

Build* getLastSelectedPatch(int& selLine){
	if(isNurbsCageNode(g_lastSelectedPatch.p)){
		selLine = getNodeLine(g_lastSelectedPatch);
		return g_lastSelectedPatch;
	}
	return 0;
}

Seamless* getSeamless3(int& selLine);

bool nodeExistsInScene(Node* node, int& selLine){
	if(g_lastSelectedPatch){
		for(int k = 0; k < g_sceneTreeList.length; k++){
			if(node == isNurbsCageNode(g_sceneTreeList[k].node.p)){
				selLine = k;
				return true;
			}
		}
	}
	return false;
}

void selectLastSelectedPatch(){
	if(Build* build = isNurbsCageNode(g_selNode.getRaw())){
		g_lastSelectedPatch = build;
		return;
	}
	
	int smlsLine = getSeamless2Line();
	Seamless* selSmls = 0;
	if(smlsLine > 0){
		selSmls = dynamic_cast<Seamless*>(g_sceneTreeList[smlsLine].node.p);
	}

	int line = 0;
	if(Build* build = getLastSelectedPatch(line)){
		int smlsLine2 = line;
		if(selSmls && selSmls == getSeamless3(smlsLine2)){
			if(smlsLine2 >= 0){
				g_lastSelectedPatch = build;
				g_stw->selLine = line;
				g_stw->setResultsForSelectingPoint(g_lastSelectedPatch);
				return;
			}
		}
	}

	if(smlsLine > 0){
		int indent = g_sceneTreeList[smlsLine].indent;
		for(int k = smlsLine + 1; k < g_sceneTreeList.length && indent < g_sceneTreeList[k].indent; k++){
			if(Build* bld = isNurbsCageNode(g_sceneTreeList[k].node.p)){
				g_lastSelectedPatch = bld;
				g_stw->selLine = k;
				g_stw->setResultsForSelectingPoint(g_lastSelectedPatch);
				return;
			}
		}
	}

	int selLine = 0;
	if(nodeExistsInScene(g_lastSelectedPatch, selLine)){
		g_stw->selLine = selLine;
		g_stw->setResultsForSelectingPoint(g_lastSelectedPatch);
		return;
	}

	for(int k = 0; k < g_sceneTreeList.length; k++){
		if(Build* bld = isNurbsCageNode(g_sceneTreeList[k].node.p)){
			g_lastSelectedPatch = bld;
			g_stw->selLine = k;
			g_stw->setResultsForSelectingPoint(g_lastSelectedPatch);
			return;
		}
	}

	g_lastSelectedPatch = 0;
}

void selectLastSelectedPart(){
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		g_lastSelectedPart = part;
	}
	if(dynamic_cast<Art*>(g_lastSelectedPart.p)){
		int selLine = getNodeLine(g_lastSelectedPart);
		if(selLine >= 0){
			g_stw->selLine = selLine;
			//g_stw->cp->setupNodeWindow(g_lastSelectedPart);
			g_stw->setResultsForSelectingVertex(g_lastSelectedPart);
			return;
		}
		else{
			g_lastSelectedPart = 0;
		}
	}
	if(dynamic_cast<Art*>(g_selNode.getRaw()) == 0){
		for(int k = 0; k < g_sceneTreeList.length; k++){
			if(g_lastSelectedPart == g_sceneTreeList[k].node){
				g_stw->selLine = k;
				g_stw->setResultsForSelectingVertex(g_lastSelectedPart);
				//g_stw->cp->setupNodeWindow(g_lastSelectedPart);
				return;
			}
		}
	}
	if(Seamless* smls = getSeamless2()){
		if(smls->partList.length){
			g_lastSelectedPart = smls->partList[0];
			for(int k = g_sceneTreeList.length; --k>=0;){
				if(g_lastSelectedPart == g_sceneTreeList[k].node){
					g_stw->selLine = k;
					g_stw->setResultsForSelectingVertex(g_lastSelectedPart);
					//g_stw->cp->setupNodeWindow(g_lastSelectedPart);
					return;
				}
			}
		}
	}

	int selLine = 0;
	if(nodeExistsInScene(g_lastSelectedPatch, selLine)){
		g_stw->selLine = selLine;
		g_stw->setResultsForSelectingPoint(g_lastSelectedPatch);
		return;
	}

	for(int k = 0; k < g_sceneTreeList.length; k++){
		if(Art* part = dynamic_cast<Art*>(g_sceneTreeList[k].node.p)){
			g_lastSelectedPart = part;
			g_stw->selLine = k;
			g_stw->setResultsForSelectingVertex(part);
			//g_stw->cp->setupNodeWindow(g_lastSelectedPart);
			return;
		}
	}

	g_lastSelectedPart = 0;
}

void fastFullBuildGlobal(bool inState);
void storePoints(M_<NurbsPatch*>& cpNodeList);

void setWorkAnimCPsInScene(){
	for(int k = 0; k < g_sceneTreeList.length; k++){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			if(smls->animCP && smls->workAnimCP == false){
				M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
				setAllIns(smls->build.refValue(), false);
				smls->workAnimCP = true;
				storePoints(cpNodeList);
				smls->doBuild();
				smls->ctrf = true;
			}
		}
	}
}

void clearWorkAnimCPModeInScene(){
	for(int k = 0; k < g_sceneTreeList.length; k++){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->setModellingPatchMode();
		}
	}
}

void setHomeMode(){
	if(g_addNurbsLine->mousePhase != AddNurbsLine::NEUTRAL){
		g_addNurbsLine->mousePhase = AddNurbsLine::INITIAL_POINTS;
	}
	else if(g_drawNurbsLine->mousePhase != DrawNurbsLine::NEUTRAL){
		g_drawNurbsLine->mousePhase = DrawNurbsLine::DRAW;
	}
	else{
		resetTemporaryModes();
	}

	if(g_scene->mode & POLY_EDIT_MODE){
		g_joinAndTransferActive = true;
		fastFullBuildGlobal(true);
		fastFullBuildGlobal(false);
	}
	MultiSelect::reset();
	g_visualClue = nullPtr;
	if(Window* win = Window::animCPMode()){
		Window::activeWindow=(g_tb2.p);
		Window::closeWindow(win);
	}
	if(g_tb2->isVisible() == false){
		Window::activeWindow = g_tb2.p;
	}

	g_scene->mode = (g_scene->mode & IMODE_MASK) | HOME_MODE;

	g_scene->mode.refValue() = g_scene->mode & ~HIDE_MESH_MODE;
	
	g_runAnim = true;
	g_joinAndTransferActive = true;

//	if(Window::animMode()){
//	}
	setWorkAnimCPsInScene();

	g_tb->updateToolBarContext();
	g_visualClue = 0;

	if(g_addNurbsLine->mousePhase != AddNurbsLine::NEUTRAL){
		g_addNurbsLine->newLine();
	}
	else if(g_drawNurbsLine->mousePhase != DrawNurbsLine::NEUTRAL){
		g_drawNurbsLine->newLine();
	}
	else{
		g_modellingCursor = g_crossCursor;
	}
}

void setSelectObjectMode(){
	resetTemporaryModes();
	if(g_scene->mode & POLY_EDIT_MODE){
		g_joinAndTransferActive = true;
		fastFullBuildGlobal(true);
		fastFullBuildGlobal(false);
	}
	Viewpoint* view = dynamic_cast<Viewpoint*>(g_selNode.getRaw());
	Seamless* smls = getSeamless2();
	if(view || smls){
		if(smls && smls != dynamic_cast<Seamless*>(g_selNode.getRaw())){
			g_stw->setResultsForSelectingNode(smls);
			MultiSelect::objectList.length = 0;
			MultiSelect::objectList += smls;
			g_transformObj->calcPivotPoint();
		}
		MultiSelect::reset();
		g_visualClue = nullPtr;
		if(Window* win = Window::animCPMode()){
			Window::activeWindow=(g_tb2.p);
			Window::closeWindow(win);
		}
		if(g_tb2->isVisible() == false){
			Window::activeWindow = g_tb2.p;
		}

		g_scene->mode = (g_scene->mode & IMODE_MASK) | SELECT_OBJECT_MODE;

		g_scene->mode.refValue() = g_scene->mode & ~HIDE_MESH_MODE;

		g_tb->updateToolBarContext();
		g_visualClue = 0;
		g_modellingCursor = g_crossCursor;

		g_viewMatrix = g_scene->getViewMatrix();
		//g_scene->senseTug(Matrix(),Matrix(), 0); //we only call this to set the part matrices

		houseKeepVisualClue();
	}
}

extern Ptr<Menu> g_selAnimMenu;

void selectAnimBarFromList(){
	g_animBarListIndex = g_selAnimMenu->blisterItem;
	Anim* anim = g_animBarList[g_animBarListIndex];
	g_acp->anim = anim;
	g_acp->setupNodeWindow(anim);
	Window::insertUnderActive(g_acp.p);
	if(anim->play == false){
		anim->update = true;
	}
	selectLastSelectedPart();
	g_stw->commonPostInsertNodeStuff(false);
	g_vcArtAnimBase->resetRotationLever();
	g_selAnimMenu = 0;
}

void setAnimationMode(){
	resetTemporaryModes();

	if(g_scene->mode & POLY_EDIT_MODE){
		g_joinAndTransferActive = true;
		fastFullBuildGlobal(true);
		fastFullBuildGlobal(false);
	}
	g_scene->mode = (g_scene->mode & IMODE_MASK) | ANIMATION_MODE;

	g_runAnim = true;

	setWorkAnimCPsInScene();


	if(Window::animMode() == 0){
		if(g_animBarListIndex < g_animBarList.length){
			Anim* anim = g_animBarList[g_animBarListIndex];
			g_acp->anim = anim;
			g_acp->setupNodeWindow(anim);
			Window::insertUnderActive(g_acp.p);
			if(anim->play == false){
				anim->update = true;
			}
			if(0 == dynamic_cast<Viewpoint*>(g_selNode.getRaw())){
				selectLastSelectedPart();
				g_dontBackup = true;
				g_stw->commonPostInsertNodeStuff(false);
			}
		}
		else{
			Anim* anim = new Anim;
			g_scene->children.refValue() += anim;
			g_stw->genNodeList();
			g_acp->setupNodeWindow(anim);
			Window::insertUnderActive(g_acp.p);
			if(0 == dynamic_cast<Viewpoint*>(g_selNode.getRaw())){
				selectLastSelectedPart();
				g_dontBackup = true;
				g_stw->commonPostInsertNodeStuff(false);
			}
		}
	}
	g_vcArtAnimBase->resetRotationLever();
}

void setPaintMode(){
	resetTemporaryModes();
	g_runAnim = false;
	g_joinAndTransferActive = false;
	fastFullBuildGlobal(true);
	if(g_scene->mode & PAINT_MODE){
		return;
	}
	
	removeTypeFromList < Button>(g_callBackList); //prevents crash from initialising from recent files

	if(Window* win = Window::animCPMode()){
		Window::activeWindow=(g_tb2.p);
		Window::closeWindow(win);
	}
	if(g_tb2->isVisible() == false){
		Window::activeWindow = g_tb2.p;
	}
	g_scene->mode = (g_scene->mode & IMODE_MASK) | PAINT_MODE;



	selectLastSelectedPart();
	if(dynamic_cast<Art*>(g_selNode.getRaw()) == 0){
		if(Seamless* smls = getSeamless2()){
			if(smls->skeleton.length){
				g_stw->setResultsForSelectingVertex(dynamic_cast<Art*>(smls->skeleton[0].p));
			}
		}
	}

	if(g_lastSelectedPart){
		g_stw->setResultsForSelectingVertex(g_lastSelectedPart);
		g_vcArtTriBase->smls = g_lastSelectedPart->smls;
	}

	g_tb->updateToolBarContext();

	if(g_scene->mode & BRUSH_MODE){
		g_scene->mode.refValue() ^= BRUSH_MODE;
		openBrush();
	}
	else if(g_scene->mode & FILL_MODE){
		g_scene->mode.refValue() ^= FILL_MODE;
		openFill();
	}
	else if(g_scene->mode & COLOR_PICKER_MODE){
		g_scene->mode.refValue() ^= COLOR_PICKER_MODE;
		openColorPicker();
	}

}

void togglePanViewMode(){
	g_panViewMode = !g_panViewMode;
	if(g_panViewMode){
		changeCursor(g_sizeAllCursor);
	}
	else{
	}
}

Art* getPart();

void openOwnership(){
	resetTemporaryModes();
	if(g_scene->mode & OWNERSHIP_MODE){
		g_joinAndTransferActive = true;
		fastFullBuildGlobal(true);
		fastFullBuildGlobal(false);
	}

	if(Window* win = Window::animCPMode()){
		Window::activeWindow=(g_tb2.p);
		Window::closeWindow(win);
	}
	if(g_tb2->isVisible() == false){
		Window::activeWindow = g_tb2.p;
	}

	g_scene->mode = (g_scene->mode & IMODE_MASK) | OWNERSHIP_MODE;
	
	if(paletteCPAllreadyOpen()){
		return;
	}
	if(getPart() == 0){
		msg("you need a Part (or a TransferOwnership or TransferBCyl referencing a Part) selected to open color ownership mode");
		return;
	}

	g_paletteMode = false;
	g_colorOwnership = true;
	for(int k = g_sceneTreeList.length; --k>=0;){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->ctrf = true;
		}
	}
	if(Art* part = getPart()){
		if(part->smls->workAnimCP){
			g_lookupProtoInterface[PALETTE]->properties[2].name = "points";
		}
		else{
			g_lookupProtoInterface[PALETTE]->properties[2].name = "coords";
		}
	}
	PaletteCP* paletteCP = new PaletteCP;
	paletteCP->setupNodeWindow(g_palette);
	Window::activeWindow = paletteCP;
	houseKeepVisualClue();
}

void setPaintColorCodedControlPoints(){
	resetTemporaryModes();
	if(g_scene->mode & (OWNERSHIP_MODE | NURBS_OWNERSHIP_MODE)){
		g_joinAndTransferActive = true;
		fastFullBuildGlobal(true);
		fastFullBuildGlobal(false);
	}

	if(Window* win = Window::animCPMode()){
		Window::activeWindow=(g_tb2.p);
		Window::closeWindow(win);
	}
	if(g_tb2->isVisible() == false){
		Window::activeWindow = g_tb2.p;
	}

	g_scene->mode = (g_scene->mode & IMODE_MASK) | NURBS_OWNERSHIP_MODE;
	
	if(paletteCPAllreadyOpen()){
		return;
	}
	selectLastSelectedPart();
	if(dynamic_cast<Art*>(g_selNode.getRaw()) == 0){
		if(Seamless* smls = getSeamless2()){
			if(smls->skeleton.length){
				g_stw->setResultsForSelectingVertex(dynamic_cast<Art*>(smls->skeleton[0].p));
			}
		}
	}

	if(g_lastSelectedPart){
		g_stw->setResultsForSelectingVertex(g_lastSelectedPart);
		g_vcArtTriBase->smls = g_lastSelectedPart->smls;
		g_visualClue = g_vcArtTriBase;
	}


	g_paletteMode = false;
	g_colorOwnership = true;
	for(int k = g_sceneTreeList.length; --k>=0;){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->ctrf = true;
		}
	}
	if(Art* part = getPart()){
		if(part->smls->workAnimCP){
			g_lookupProtoInterface[PALETTE]->properties[2].name = "points";
		}
		else{
			g_lookupProtoInterface[PALETTE]->properties[2].name = "coords";
		}
	}
	PaletteCP* paletteCP = new PaletteCP;
	paletteCP->setupNodeWindow(g_palette);
	Window::activeWindow = paletteCP;
	houseKeepVisualClue();
}

bool notSelectedFilter(Anim* anim){
	M_Node& soundTrack = anim->soundTrack.refValue();
	for(int k = soundTrack.length; --k>=0;){
		if(Composition* composition = dynamic_cast<Composition*>(anim->soundTrack[k].p)){
			M_Node& m_track = composition->track.refValue();
			if(m_track.length){
				return false;
			}
		}
	}
	return true;
}

void setModellingMode2(){
	if(g_scene->mode & POLY_EDIT_MODE){
		g_joinAndTransferActive = true;
		fastFullBuildGlobal(true);
		fastFullBuildGlobal(false);
	}

	if(AnimControlPanel* animCP = Window::animCPMode()){
		if(notSelectedFilter(animCP->anim)){
			Window::activeWindow = g_tb2.p;
			Window::closeWindow(animCP);
		}
	}
	if(g_tb2->isVisible() == false){
		Window::activeWindow = g_tb2.p;
	}
	g_scene->mode = (g_scene->mode & IMODE_MASK) | MODELLING_MODE;
	g_nurbsIls->noZbuf = false;

	g_runAnim = false;
	g_joinAndTransferActive = true;

	clearWorkAnimCPModeInScene();

	selectLastSelectedPatch();

	//dLog("setModellingMode3");
	//num = g_callBackList.length;
	//dLog(num);

	g_dontBackup = true;
	g_stw->commonPostInsertNodeStuff(false);
	g_tb->updateToolBarContext();

	if(g_drawNurbsLine->mousePhase == DrawNurbsLine::NEUTRAL){
		houseKeepVisualClue();
	}
}

void setModellingMode(){
	resetTemporaryModes();
	
	setModellingMode2();
}

void setPolyEditMode(){
	resetTemporaryModes();

	g_runAnim = false;
	g_joinAndTransferActive = false;
	fastFullBuildGlobal(true);
	if(g_scene->mode & POLY_EDIT_MODE){
		return;
	}
	if(Window* win = Window::animCPMode()){
		Window::activeWindow=(g_tb2.p);
		Window::closeWindow(win);
	}
	if(g_tb2->isVisible() == false){
		Window::activeWindow = g_tb2.p;
	}

	g_scene->mode = (g_scene->mode & IMODE_MASK) | POLY_EDIT_MODE;
	clearWorkAnimCPModeInScene();


	selectLastSelectedPart();
	if(dynamic_cast<Art*>(g_selNode.getRaw()) == 0){
		if(Seamless* smls = getSeamless2()){
			if(smls->skeleton.length){
				g_stw->setResultsForSelectingVertex(dynamic_cast<Art*>(smls->skeleton[0].p));
			}
		}
	}

	if(g_lastSelectedPart){
		g_stw->setResultsForSelectingVertex(g_lastSelectedPart);
		g_vcArtTriBase->smls = g_lastSelectedPart->smls;
		g_visualClue = g_vcArtTriBase;
	}
	g_tb->updateToolBarContext();

/*
 
	else{
		g_polyEditMode = false;
		g_polyEditButton->srcPos = Vec2i(24 * 8, 208);
		fastFullBuildForAutoIn(true);
		MultiSelect::coordList.length = 0;
		if(Seamless* smls = getSeamless2()){
			smls->doBuild();
			smls->ctrf = true;
			smls->actTriPart = 0;
		}
	}

	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		MultiSelect::coordList.length = 0;
		part->smls->actTriPart = 0;
		part->houseKeep();
	}
	*/
}

void setXRayEditMode(){
	resetTemporaryModes();

	if(Window* win = Window::animCPMode()){
		Window::activeWindow=(g_tb2.p);
		Window::closeWindow(win);
	}
	if(g_tb2->isVisible() == false){
		Window::activeWindow = g_tb2.p;
	}

	g_scene->mode = (g_scene->mode & IMODE_MASK) | XRAY_EDIT_MODE;

	clearWorkAnimCPModeInScene();

	selectLastSelectedPart();
	if(g_lastSelectedPart){
		g_lastSelectedPart->smls->xRay = true;
		g_stw->commonPostInsertNodeStuff(false);
	}

	g_tb->updateToolBarContext();


	MultiSelect::reset();
	houseKeepVisualClue();
}

void hideShowMesh(){
	g_scene->mode.refValue() = g_scene->mode ^ HIDE_MESH_MODE;
}
void toggleCurvedCageMode(){
	g_curvedCageMode = !g_curvedCageMode;
	//g_nurbsIls->noZbuf = (g_scene->mode.refValue() & ZBUFFER_CAGE_MODE) != 0;
	houseKeepVisualClue();
	updateIconPointers();

}
void toggleZBufferCageMode(){
	g_scene->mode.refValue() ^= ZBUFFER_CAGE_MODE;
	g_nurbsIls->noZbuf = (g_scene->mode.refValue() & ZBUFFER_CAGE_MODE) == 0;
	updateIconPointers();
	houseKeepVisualClue();
}

void toggleDragWithDescendantsMode(){
	g_dragWithDescendants = !g_dragWithDescendants;
	updateIconPointers();
	g_tb2->paint();
}


void getColorWithEyeDropRadius(Art* selPart, Seamless* smls);

void openBrush(){
	if(g_scene->mode & BRUSH_MODE){
		return;
	}
	g_scene->mode = (g_scene->mode & IPAINT_TOOL_MASK) | BRUSH_MODE;
	
	g_stw->cp->setupNodeWindow(g_brush);
	Window::activeWindow = g_stw->cp.p;
	houseKeepVisualClue();
}

void openFill(){
	if(g_scene->mode & FILL_MODE){
		return;
	}
	g_scene->mode = (g_scene->mode & IPAINT_TOOL_MASK) | FILL_MODE;

	g_stw->cp->setupNodeWindow(g_fill);
	Window::activeWindow = g_stw->cp.p;
}

void openColorPicker(){
	if(g_scene->mode & COLOR_PICKER_MODE){
		return;
	}
	g_scene->mode = (g_scene->mode & IPAINT_TOOL_MASK) | COLOR_PICKER_MODE;
	g_stw->cp->setupNodeWindow(g_colorPicker);
	Window::activeWindow = g_stw->cp.p;
}

void toggleCreateMirrorObjectMode(){
	g_scene->mode.refValue() ^= CREATE_MIRROR_OBJECT;
	updateIconPointers();

	if(g_addLathe->mousePhase == AddLathe::CREATE_SPINDLE){
		g_modellingCursor = g_addGeometryCursor;
		if(g_scene->mode.refValue() & CREATE_MIRROR_OBJECT){
			g_modellingCursor = g_addGeometryMirrorCursor;
		}
		changeCursor(g_modellingCursor);
	}
}

void toggleBoxPatchModellingMode(){
	g_showPatchBorderMode = false;
	g_scene->mode.refValue() ^= BOX_PATCH_MODELLING_MODE;
	if(g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE){
		g_boxPatchModellingButton->srcPos = Vec2i(24 * 2, 233);
	}
	else{
		g_boxPatchModellingButton->srcPos = Vec2i(24 * 3, 233);
	}
	houseKeepVisualClue();
}

void toggleStitchPolyMode(){



	g_scene->mode.refValue() ^= STITCH_POLY_MODE;
	if(g_scene->mode & STITCH_POLY_MODE){
		g_scene->mode.refValue() = g_scene->mode | HIDE_MESH_MODE;
	}
	else{
		MultiSelect::ctrlPolySelList.length = 0;
		g_scene->mode.refValue() = g_scene->mode & ~HIDE_MESH_MODE;
	}
	updateIconPointers();
	g_tb2->paint(); // needed for short cut
	houseKeepVisualClue();
}


void Seamless::selectWholePatches(){
	M_<NurbsPatch*> contigList;
	createSelContigList(contigList);
	M_Bool m_flag;
	m_flag.length = cpNodeList.length;

	for(int k = MultiSelect::pointList.length; --k>=0;){
		NurbsPatch* patch = MultiSelect::pointList[k].p;
		m_flag[patch->plidx] = true;
	}

	MultiSelect::pointList.length = 0;
	for(int pi = 0; pi < m_flag.length; pi++){
		if(m_flag[pi]){
			NurbsPatch* patch = cpNodeList[pi];
			patch->selectAllPoints(contigList);
		}
	}
	
	if(g_showPatchBorderMode == false){
		for(int i = 0; i < ctrlPolyPoint.length;){
			int ctrlPoly = i;
			int len = ctrlPolyPoint[ctrlPoly].len;
			int midPoly = ctrlPoly + len;
		
			for(int k = len + 1; --k >= 0;){
				int midLen = ctrlPolyPoint[midPoly].len;
			
				if(midLen == 0){
					midPoly++;
				}
				for(int m = midLen; --m >= 0;){
					NurbsPatch* patch = ctrlPolyPoint[midPoly].p;
					int idx = ctrlPolyPoint[midPoly].i;
					midPoly++;
					contigList[patch->fromPoint + idx] = 0;
				}
			}
			i = getNextCtrlPoly(ctrlPoly);
		}
	}

	MultiSelect::pointList.length = 0;
	for(int k = 0; k < pointTotal; k++){
		if(NurbsPatch* patch = contigList[k]){
			MultiSelect::pointList += ControlPoint(patch, k - patch->fromPoint);
		}
	}


	g_transformCP->calcPivotPoint();
	g_transformCP->resetRotationLever();
}

void selectWholePatch(){
	//g_transformCP->selectWholePatches = !g_transformCP->selectWholePatches;
	//if(g_transformCP->selectWholePatches){
		//g_transformCP->selectRing.refValue() = false;
		//g_transformCP->selectRings.refValue() = false;
		//g_toggleSelectObjectButton->srcPos = Vec2i(24 * 6, 209);


		Seamless* smls = getSeamless2();
		smls->selectWholePatches();
	//}
	//else{
	//	g_toggleSelectObjectButton->srcPos = Vec2i(24 * 5, 209);
	//}
	houseKeepVisualClue();
}

void toggleSelectObject(){
	//if(Seamless* smls = getSeamless2()){
	//	smls->clearObjects();
	//}
	//MultiSelect::reset();
	g_transformCP->selectObject = !g_transformCP->selectObject;
	if(g_transformCP->selectObject){
		//g_transformCP->selectRing.refValue() = false;
		//g_transformCP->selectRings.refValue() = false;
		g_toggleSelectObjectButton->srcPos = Vec2i(24 * 6, 209);
	}
	else{
		g_toggleSelectObjectButton->srcPos = Vec2i(24 * 5, 209);
	}
	houseKeepVisualClue();
}


//void toggleSelectRing(){
//	g_transformCP->selectRing.refValue() ^= true;
//	if(g_transformCP->selectRing){
//		g_transformCP->selectRings.refValue() = false;
//		g_transformCP->selectObject.refValue() = false;
//		g_selectRingButton->srcPos = Vec2i(24 * 0, 257);
//		g_selectRingsButton->srcPos = Vec2i(24 * 3, 257);
//		g_toggleSelectObjectButton->srcPos = Vec2i(24 * 5, 209);
//	}
//	else{
//		g_selectRingButton->srcPos = Vec2i(24 * 1, 257);
//	}
//	houseKeepVisualClue();
//}

void disabledMessage(){
	msG("Sorry this function has been disabled because it is not ready yet. Please check back for updates");
}

void toggleSelectRings(){
	disabledMessage();
	return;
	resetTemporaryModes();
	g_transformCP->selectRings ^= true;
	if(g_transformCP->selectRings){
		g_transformCP->selectRing = false;
		g_transformCP->selectObject.refValue() = false;
		g_selectRingsButton->srcPos = Vec2i(24 * 2, 257);
		g_selectRingButton->srcPos = Vec2i(24 * 1, 257);
		g_toggleSelectObjectButton->srcPos = Vec2i(24 * 5, 209);
	}
	else{
		g_selectRingsButton->srcPos = Vec2i(24 * 3, 257);
	}
	houseKeepVisualClue();
}


void toggleStrandCtrlLineMode(){
	g_scene->mode.refValue() ^= SHOW_STRAND_CTRL_LINE_MODE;
	updateBuild3dWindow();
	//houseKeepVisualClue();
}

void toggleHairRegionMode(){
	g_scene->mode.refValue() ^= HAIR_REGION_MODE;

	if((g_scene->mode & HAIR_REGION_MODE)){
		g_stw->cp->setupNodeWindow(g_region);
		Window::activeWindow = g_stw->cp.p;
	}
	else{
		g_stw->cp->setupNodeWindow(g_selNode.getRaw());
		Window::activeWindow = g_stw->cp.p;
	}

	houseKeepVisualClue();
}

void togglePolyMorphCapture(){

	g_scene->mode.refValue() ^= ENABLE_MORPH_CAPTURE;
	updateIconPointers();
}


Ptr<FloorHeight> g_floorHeight = new FloorHeight;

#ifdef PROG_ROB
	String g_upStr;
#endif

//void matchEndsForPatch();
void alignMidSelectededPoints();
//void alignAllMidPoints();

void makeStitchPointsPlanar();
void updateNurbsCurveKnotBezier(int dim, M_Float& knot);

M_<ShortCutKey> globalShortCut = M_<ShortCutKey>(
	ShortCutKey(VK_F1, help),
	ShortCutKey(VK_F3, selectPreviousVertex),
	ShortCutKey(VK_F4, selectNextVertex),
	ShortCutKey(VK_F5, prevTriangle),
	ShortCutKey(VK_F6, nextTriangle),
	ShortCutKey(VK_F7, selectPreviousCornerInTri),
	ShortCutKey(VK_F8, selectNextCornerInTri),
	ShortCutKey(VK_F9, doBuild),
	ShortCutKey(VK_F11, toggleRecordSoundMovie),
	//ShortCutKey(VK_F11, outputScreenShotSingle),
	ShortCutKey(VK_F12, outputScreenShotSingle)
//    ShortCutKey(VK_F11, spaghetti)
);

M_<ShortCutKey> globalShortCutAlt = M_<ShortCutKey>(
	ShortCutKey('R',	resetRotations),
	ShortCutKey('X',	zeroVetexX),
//	ShortCutKey('M',	matchEndsForPatch),
	ShortCutKey('J',	joinVertexKeepSourceColor),
	ShortCutKey('F',	captureFrame),
	ShortCutKey('I',	incorporate),
	//ShortCutKey('W',	quantiseViewOrientation),

//	ShortCutKey('Q', centerViewToControlPoint),
	//ShortCutKey(VK_NUMPAD1, centerViewToControlPoint),
	ShortCutKey(VK_NUMPAD0, centerViewToPivotPoint),
//	ShortCutKey(VK_NUMPAD4, decPivotX),
//	ShortCutKey(VK_NUMPAD6, incPivotX),
//	ShortCutKey(VK_NUMPAD2, decPivotY),
//	ShortCutKey(VK_NUMPAD8, incPivotY),
//	ShortCutKey(VK_NUMPAD9, decPivotZ),
//	ShortCutKey(VK_NUMPAD3, incPivotZ)
	ShortCutKey(VK_NUMPAD4, decCoordX),
	ShortCutKey(VK_NUMPAD6, incCoordX),
	ShortCutKey(VK_NUMPAD2, decCoordY),
	ShortCutKey(VK_NUMPAD8, incCoordY),
	ShortCutKey(VK_NUMPAD9, decCoordZ),
	ShortCutKey(VK_NUMPAD3, incCoordZ)

);
M_<ShortCutKey> globalShortCutCtrl = M_<ShortCutKey>(
	ShortCutKey(VK_F2, openNodeWindow),
	ShortCutKey('O', openSmls),
	ShortCutKey('S', saveSmls),
	ShortCutKey('A', saveSmlsAs),
	ShortCutKey('N', newSmls),
	ShortCutKey('I', reverseIndices),
	ShortCutKey('B', invertBackgroundColor),
	ShortCutKey('C', remember),
	ShortCutKey('X', cut),
        //ShortCutKey(VK_F11, openNodeWindow),
//ShortCutKey(VK_F11, openNodeWindow),

	ShortCutKey(219, decRed),
	ShortCutKey(221, incRed),
	ShortCutKey(186, decGreen),
	ShortCutKey(VK_TAB,	setShowDimFilterVisual)
);
M_<ShortCutKey> globalShortCutShift = M_<ShortCutKey>();
void initializeShortCuts(){
	globalShortCut+=ShortCutKey(VK_F2, toggleHomeMode);

	globalShortCutCtrl+=ShortCutKey('Z', unDo);
	globalShortCutCtrl+=ShortCutKey('Y', reDo);
	globalShortCutCtrl+=ShortCutKey('J',	joinVertexKeepBothColors);
	globalShortCutAlt+=ShortCutKey('P',		makeStitchPointsPlanar);
	globalShortCutCtrl+=ShortCutKey('P',	openPalette);
	globalShortCutCtrl+=ShortCutKey('N',	calcAllPolyMidPoints);
	globalShortCutCtrl+=ShortCutKey('T',	openTransformCP);
	globalShortCutCtrl+=ShortCutKey('F',	fillEdgesWithCtrlPolys);
	globalShortCutCtrl+=ShortCutKey('E',	extrudePatch);
	globalShortCutCtrl+=ShortCutKey('M', mergeCtrlPolys);
	globalShortCutCtrl+=ShortCutKey('5', mergeTo5CtrlPoly);
	globalShortCutCtrl+=ShortCutKey('6', mergeTo6CtrlPoly);
	globalShortCutCtrl+=ShortCutKey('G', toggleStitchPolyMode);
	globalShortCutCtrl+=ShortCutKey('I', insertControlPoint);
	//globalShortCutCtrl+=ShortCutKey('L', invertSelCtrlPoly);
	globalShortCutCtrl+=ShortCutKey('L', linkPointToEdge);
	//globalShortCutCtrl+=ShortCutKey('F', stitchFlower);
	globalShortCutCtrl+=ShortCutKey('H', replaceColorPart);
	globalShortCutCtrl+=ShortCutKey('W', snapPivotPointToControlPoint);
	globalShortCutCtrl+=ShortCutKey('Q', pasteColumn);
	//globalShortCutCtrl+=ShortCutKey('I', alignOscToPatch);
	globalShortCutCtrl+=ShortCutKey('D', deleteLink);
	globalShortCutCtrl+=ShortCutKey('K', togglePolyMorphCapture);
	globalShortCutCtrl+=ShortCutKey('R', compileMorph);

	globalShortCutCtrl+=ShortCutKey('U', genHairFromPatch);

	globalShortCutCtrl+=ShortCutKey(VK_F7, saveRelWinMousePos);
	//globalShortCutCtrl+=ShortCutKey(VK_F8, emptyPacketQue);

	globalShortCutCtrl+=ShortCutKey(222, incGreen);
	globalShortCutCtrl+=ShortCutKey(190, decBlue);
	globalShortCutCtrl+=ShortCutKey(191, incBlue);
	globalShortCutCtrl+=ShortCutKey(187, lightenVertex);
	globalShortCutCtrl+=ShortCutKey(189, darkenVertex);

	globalShortCutCtrl+=ShortCutKey(VK_NUMPAD4, decCoordX);
	globalShortCutCtrl+=ShortCutKey(VK_NUMPAD6, incCoordX);
	globalShortCutCtrl+=ShortCutKey(VK_NUMPAD2, decCoordY);
	globalShortCutCtrl+=ShortCutKey(VK_NUMPAD8, incCoordY);
	globalShortCutCtrl+=ShortCutKey(VK_NUMPAD9, decCoordZ);
	globalShortCutCtrl+=ShortCutKey(VK_NUMPAD3, incCoordZ);

	//globalShortCutCtrl+=ShortCutKey(VK_F3, setUpMidi);

	globalShortCutCtrl+=ShortCutKey(VK_F4, bnspeBuildPolies);
	globalShortCutCtrl+=ShortCutKey(VK_F5, joinBorderVertices);
	globalShortCutCtrl+=ShortCutKey(VK_F6, insertControlPoint);
	globalShortCutCtrl+=ShortCutKey(VK_F7, duplicate);
	globalShortCutCtrl+=ShortCutKey(VK_F8, updateColorFromColorMappers);
	globalShortCutCtrl+=ShortCutKey(VK_F9, toggleCurvedCageMode);

	globalShortCutCtrl+=ShortCutKey(VK_SUBTRACT, decTugRadius);
	globalShortCutCtrl+=ShortCutKey(VK_ADD, incTugRadius);



	_3DWin::shortCut+=ShortCutKey(VK_ESCAPE, reset3dScene);
	_3DWin::shortCut+=ShortCutKey(0xdb,	decViewOrientationX);//[
	_3DWin::shortCut+=ShortCutKey(0xdd,	incViewOrientationX);//]
	_3DWin::shortCut+=ShortCutKey(0xba,	decViewOrientationY);//;
	_3DWin::shortCut+=ShortCutKey(0xde,	incViewOrientationY);//'
	_3DWin::shortCut+=ShortCutKey(0xbe,	decViewOrientationZ);//.
	_3DWin::shortCut+=ShortCutKey(0xbf,	incViewOrientationZ);///
	_3DWin::shortCut+=ShortCutKey(0xbc, resetViewOrientation);

	_3DWin::shortCut+=ShortCutKey(0xbb, zoomIn);
	_3DWin::shortCut+=ShortCutKey(0xbd, zoom);

	_3DWin::shortCut+=ShortCutKey(VK_PRIOR, Viewpoint::gotoPrevViewpoint);
	_3DWin::shortCut+=ShortCutKey(VK_NEXT, Viewpoint::gotoNextViewpoint);
	_3DWin::shortCut+=ShortCutKey(VK_NEXT, Viewpoint::gotoNextViewpoint);
	_3DWin::shortCut+=ShortCutKey(VK_NUMPAD2, decViewPositionY);
	_3DWin::shortCut+=ShortCutKey(VK_NUMPAD3, incViewPositionZ);
	_3DWin::shortCut+=ShortCutKey(VK_NUMPAD4, decViewPositionX);
	_3DWin::shortCut+=ShortCutKey(VK_NUMPAD5, resetViewPosition);
	_3DWin::shortCut+=ShortCutKey(VK_NUMPAD6, incViewPositionX);
	_3DWin::shortCut+=ShortCutKey(VK_NUMPAD8, incViewPositionY);
	_3DWin::shortCut+=ShortCutKey(VK_NUMPAD9, decViewPositionZ);
	_3DWin::shortCut+=ShortCutKey('1', decViewOrientation90X);
	_3DWin::shortCut+=ShortCutKey('2', incViewOrientation90X);
	_3DWin::shortCut+=ShortCutKey('3', decViewOrientation90Y);
	_3DWin::shortCut+=ShortCutKey('4', incViewOrientation90Y);
	_3DWin::shortCut+=ShortCutKey('5', decViewOrientation90Z);
	_3DWin::shortCut+=ShortCutKey('6', incViewOrientation90Z);

	_3DWin::shortCut+=ShortCutKey(' ',	shortcutToggleWireFrame);
	_3DWin::shortCut+=ShortCutKey('B',	breakVertexJoin);
	_3DWin::shortCut+=ShortCutKey('C',	colorVertex);
	_3DWin::shortCut+=ShortCutKey('D',	swapDivide);
	_3DWin::shortCut+=ShortCutKey('E',	incToNextTriThatSharesActivePoint);
	_3DWin::shortCut+=ShortCutKey('F',	forceToMirrorX);
	_3DWin::shortCut+=ShortCutKey('I',	swapVertexOrderInc);
	_3DWin::shortCut+=ShortCutKey('J',	joinVertexKeepDestColor); //moveNurbsCP //joinVertexImmediate(part,false)??????????????
	_3DWin::shortCut+=ShortCutKey('K',	pasteColorToTriangle);
	_3DWin::shortCut+=ShortCutKey('M',	mirrorTriangleImmediate);
	_3DWin::shortCut+=ShortCutKey('P',	projectSingleVertexOntoPoly);
	_3DWin::shortCut+=ShortCutKey('Q',	quickTriangle);
	_3DWin::shortCut+=ShortCutKey('R',	selectRing);
	_3DWin::shortCut+=ShortCutKey('V',	quantiseViewOrientation);
	_3DWin::shortCut+=ShortCutKey('S',	splitTriangleImmediate);
	_3DWin::shortCut+=ShortCutKey('T',	openSceneTree);
	_3DWin::shortCut+=ShortCutKey('U',	swapVertexOrderDec);
	_3DWin::shortCut+=ShortCutKey('W',	decToNextTriThatSharesActivePoint);
	//_3DWin::shortCut+=ShortCutKey('Z',	setZDragNode);
	_3DWin::shortCut+=ShortCutKey('A',	setSingleActiveMode);
	_3DWin::shortCut+=ShortCutKey('Y',	unMultiSelTri);
	_3DWin::shortCut+=ShortCutKey('O',	centerViewToControlPoint);

	globalShortCutAlt += ShortCutKey('T',	randomizeHarmonicFreq);
	globalShortCutAlt += ShortCutKey('W',	loadSDIF);
	globalShortCutAlt += ShortCutKey('Y',	makeTrianglesTiny);
	globalShortCutAlt += ShortCutKey('E',	extrudePatch);
	//globalShortCutAlt += ShortCutKey(' ',	toggleBackfaceSelect);
	globalShortCutAlt += ShortCutKey('Q',	renderRayTrace);
	globalShortCutAlt += ShortCutKey('C',	toggleShowCameras);
	globalShortCutAlt += ShortCutKey('G',	deleteAllStitchPolys);

	globalShortCutAlt += ShortCutKey(VK_F13,	decViewOrientationZ);//.
	globalShortCutAlt += ShortCutKey(VK_F14,	incViewOrientationZ);///


}
M_<ShortCutKey> _3DWin::shortCut = M_<ShortCutKey>(
	ShortCutKey(VK_SUBTRACT,	zoom),
	ShortCutKey(VK_ADD,			zoomIn),
	ShortCutKey(VK_DELETE, 		deleteTriangle)
);
M_<ShortCutKey> _3DWin::shortCutAlt = M_<ShortCutKey>(
	ShortCutKey('B',	breakColorJoin),
	ShortCutKey('C',	colorPart),
	ShortCutKey('F',	findThinTriangle),
	ShortCutKey('L',	findLayeredTriangle),
	ShortCutKey('M',	moveVertex),
	//ShortCutKey('O',	transferSingleVertex),
	ShortCutKey('A',	showAllCages),
	ShortCutKey('S',	showSingleCage),
	ShortCutKey('H',	hideSelectedCages)
);
M_<ShortCutKey> _3DWin::shortCutCtrl = M_<ShortCutKey>(
);
M_<ShortCutKey> _3DWin::shortCutShift = M_<ShortCutKey>(
	ShortCutKey('R',	cutRightFromVertex),
	ShortCutKey('J',	transferCloseVerticesJoin),
	ShortCutKey('L',	cutLeftFromVertex),
	ShortCutKey('U',	cutUpFromVertex),
	ShortCutKey('D',	cutDownFromVertex),

	ShortCutKey('X',	pasteAddCopyOfMirrorX),
	ShortCutKey('Y',	pasteAddCopyOfMirrorY),
	//ShortCutKey('Z',	pasteAddCopyOfMirrorZ),
	ShortCutKey('0',	openSettings),
	ShortCutKey('1',	openBrush),
	ShortCutKey('2',	openFill),
	ShortCutKey('A',	alignMidSelectededPoints),

	ShortCutKey(0xbb,	zoomIn)
);
M_<ShortCutKey> SceneTree::shortCut = M_<ShortCutKey>(
	ShortCutKey(VK_RIGHT,	incCaret),
	ShortCutKey(VK_LEFT,	decCaret),
	ShortCutKey(VK_HOME,	homeCaret),
	ShortCutKey(VK_END,		endCaret),
	ShortCutKey(VK_DELETE,	deleteChar),
	ShortCutKey(VK_ESCAPE,	SceneTree::escape)
);
M_<ShortCutKey> ChatWin::shortCut = M_<ShortCutKey>(
	ShortCutKey(VK_RIGHT,	incCaret),
	ShortCutKey(VK_LEFT,	decCaret),
	ShortCutKey(VK_HOME,	homeCaret),
	ShortCutKey(VK_END,		endCaret),
	ShortCutKey(VK_DELETE,	deleteChar),
	ShortCutKey(VK_ESCAPE,	SceneTree::escape)
);
M_<ShortCutKey> ChatWin::shortCutCtrl = M_<ShortCutKey>(
	ShortCutKey('V',	pasteFromClipboard)
);
M_<ShortCutKey> ChatWin::shortCutShift = M_<ShortCutKey>(
	ShortCutKey(VK_RIGHT,	incCaretShift),
	ShortCutKey(VK_LEFT,	decCaretShift),
	ShortCutKey(VK_HOME,	homeCaretShift),
	ShortCutKey(VK_END,		endCaretShift),
	ShortCutKey(VK_DELETE,	deleteCharShift)
);
M_<ShortCutKey> SceneTree::shortCutAlt = M_<ShortCutKey>();
M_<ShortCutKey> SceneTree::shortCutCtrl = M_<ShortCutKey>(
	ShortCutKey('V',	pasteFromClipboard)
);
M_<ShortCutKey> SceneTree::shortCutShift = M_<ShortCutKey>(
	ShortCutKey(VK_RIGHT,	incCaretShift),
	ShortCutKey(VK_LEFT,	decCaretShift),
	ShortCutKey(VK_HOME,	homeCaretShift),
	ShortCutKey(VK_END,		endCaretShift),
	ShortCutKey(VK_DELETE,	deleteCharShift),
	ShortCutKey(VK_UP,		SceneTree::decMultipleNodeSelection),
	ShortCutKey(VK_DOWN,	SceneTree::incMultipleNodeSelection)
);
M_<ShortCutKey> ControlPanel::shortCut = M_<ShortCutKey>(
	ShortCutKey(VK_RIGHT,	incCaret),
	ShortCutKey(VK_LEFT,	decCaret),
	ShortCutKey(VK_HOME,	homeCaret),
	ShortCutKey(VK_END,		endCaret),
	ShortCutKey(VK_DELETE,	deleteChar),
	ShortCutKey(VK_UP,		scrollUpCP),
	ShortCutKey(VK_DOWN,	scrollDownCP),
	ShortCutKey(VK_PRIOR,	scrollPageUpCP),
	ShortCutKey(VK_NEXT,	scrollPageDownCP)
);

M_<ShortCutKey> ControlPanel::shortCutAlt = M_<ShortCutKey>();
M_<ShortCutKey> ControlPanel::shortCutCtrl = M_<ShortCutKey>(
	ShortCutKey('V',	pasteFromClipboard)
);
M_<ShortCutKey> ControlPanel::shortCutShift = M_<ShortCutKey>(
	ShortCutKey(VK_RIGHT,	incCaretShift),
	ShortCutKey(VK_LEFT,	decCaretShift),
	ShortCutKey(VK_HOME,	homeCaretShift),
	ShortCutKey(VK_END,		endCaretShift),
	ShortCutKey(VK_DELETE,	deleteCharShift)
);
void houseKeepForNurbsViewChange(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		if(g_radial_Tug->active){
			patch->houseKeep();
		}
	}
}

bool needNSPE_enabledMsg(){
	if((g_scene->mode.refValue() & POLY_EDIT_MODE) == false){
		msG("poly editing must be enabled for this command");
		return false;
	}
	return true;
}
void needPartOrCenterJointMsg(){
	msG(msg_needPartOrCenterJoint);
}
void needPartOrBezierMsg(){
	msG(msg_needPartOrBezier);
}

void centerJointHasNoRefMsg(){
	msG(msg_centerJointHasNoRef);
}

void needWhitePartMsg(){
	msG(msg_needWhitePart);
}
void needBluePartMsg(){
	msG(msg_needBluePart);
}
void needPartMsg(){
	msG(msg_needPart);
}
void needPartOrSmlsMsg(){
	msG(msg_needPartOrSmls);
}
void needPartCopiedMsg(){
	msG(msg_needPartCopied);
}
void commonEndPivotPointChange(){
	g_stw->cp->paint();
	BackUp::monostable.beginCycle();
}
void commonEndVertexChange(){
	Seamless* smls = dynamic_cast<Art*>(g_selNode.getRaw())->smls;
	smls->doBuild();
	smls->updateAllVertexInBuffers();
	g_stw->cp->paint();
	BackUp::monostable.beginCycle();
}

void commonEndVertexBuild(){
	Seamless* smls = dynamic_cast<Art*>(g_selNode.getRaw())->smls;
	smls->actTriPart = 0;
	MultiSelect::reset();
	smls->doBuild();
	//makePointerEqualIndexLocation();
	smls->ctrf = true;
	Window::paintAllControlPanels();
}

void replaceColorInPart(ArtPtr part, Color3f oldColor, Color3f newColor){
	ColorEffect* colorEffect=dynamic_cast<ColorEffect*>(part->workEffect);
	if(colorEffect==0)return;
	M_Color3f& color=colorEffect->color.refValue();
	M_Art& relevantPart=part->relevantPart;
	for(int k=relevantPart.length; --k>=0; ){
		M_Triangle& triangle=relevantPart[k]->triangle;
		for(int i=triangle.length*3; --i>=0; ){
			if(triangle.refPart(i)==part){
				if(color[triangle.refTexIndex(i)]==oldColor){
					color[triangle.refTexIndex(i)]=newColor;
				}
			}
		}
	}
	part->smls->updateAllVertexInBuffers();
	BackUp::monostable.beginCycle();
}
void transferCloseVertices(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Art* copied=dynamic_cast<Art*>(g_copyNode.getRaw())){
			copied->transferCloseVertices(part, g_joinRange);
			part->smls->convertToRenderFormat();
			makePointerEqualIndexLocation();
			BackUp::monostable.beginCycle();
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}
void pasteCloseVerticesJoin(Art* source, Art* copied, bool dontShareColor, float rangeFrom = .001){

	SeamlessPtr smls=copied->smls;

	JoinCloseVertex jcv;
	jcv.joinRange.refValue() = rangeFrom;
	jcv.part+=copied;
	jcv.source+=source;

	if(copied->plidx<source->plidx){
		int question=msG(msg_reverse_operation ,MB_OKCANCEL|MB_ICONQUESTION);
		if (question==IDOK){
			jcv.part+=source;
			jcv.source+=copied;
		}
		else{
			return;
		}

	}


	jcv.srcColor=dontShareColor;
	jcv.doBuild();

}
inline bool checkForMatchingVertex(M_Triangle& triangle, int i,int j){
	for(int b=3; --b>=0;j++){
		if(triangle.refPart(i)==triangle.refPart(j)&&triangle.refIndex(i)==triangle.refIndex(j)){
			return true;;
		}
	}
	return false;
}
inline bool layeredTri(M_Triangle& triangle, int i,int j){
	for(int a=3; --a>=0;i++){
		if(checkForMatchingVertex(triangle, i,j)==false){
			return false;
		}
	}
	return true;
}
void findLayeredTriangle(){
	Seamless* smls=dynamic_cast<Seamless*>(g_selNode.getRaw());
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		smls=part->smls;
	}
	if(smls){
		DelPoly dp;
		dp.poly.resize(1);
		int delCount=0;
		M_Art& partList = smls->partList;
		for(int k=partList.length; --k>=0;){
			ArtPtr part=partList[k];
			M_Triangle& triangle=part->triangle;
			for(int i=triangle.length*3; --i>=0;){
				for(int j=triangle.length*3; --j>=0;){
					if(triangle.refPart(i)==triangle.refPart(j)&&triangle.refIndex(i)==triangle.refIndex(j)){
						int ib=i/3;
						int jb=j/3;
						if(ib==jb)continue; // if comapiring same triangle forget it
						if(layeredTri(triangle ,ib*3,jb*3)){
							delCount++;
							dp.part=part;
							dp.poly.refValue()[0]=jb;
							dp.doBuild();
							i=triangle.length*3;
							j=0;// get us back to i loop start
						}
					}
				}
			}
		}
		dLog(String("deleted ")+String(delCount)+" layered triangles");
		smls->convertToRenderFormat();
		makePointerEqualIndexLocation();
		g_stw->genNodeList();
		g_stw->paint();
	}
	else{
		needPartOrSmlsMsg();
	}
}
void help(){
	openWithHtmlBrowser("http://www.seamless3d.com/tut", g_settings->htmlBrowser.refValue());
}
void about(){
			msG("Seamless3d Copyright  2001-2013 Graham John Perrett\n\n\
Seamless3d is FREE Open Source 3d modelling software under the MIT license\n\
The Seamless3d modelling language is an open standard designed for the real world, verifiable by Seamless3d Software\n\
Seamless3d is typically very stable when used with experience but I must stress it is still a program being developed so use with care. Backup any files you are working on regularly!\n*Important*:\nWhen the output button is pressed Seamless3d will generate a file with the same filename as the .smls file that is open with the appropriate filename extension. (.x3d;.x3dz;.x3dv;.x3dvz;.wrl;.wrz) Any file that has this name or extension will get overwritten so take care where you leave files with matching names if you don't want them overwritten.\n\
Please send bug reports or questions or files that won't open any more to the Seamless3d forum: www.seamless3d.com or my email thyme@seamless3d.com\n\
\nSeamless3d developed and written by:\n\n\tGraham John Perrett (thyme) thyme@seamless3d.com\n\twww.seamless3d.com\n\ngzip C code (zlib 1.2.3 general purpose data compression library) written by:\n\n\tJean-loup Gailly gzip@prep.ai.mit.edu\n\tMark Adler madler@alumni.caltech.edu\n\nSome quaternion/rotation maths C code that has been adapted was originally written by:\n\n\tGavin Bell");
}

void openSceneTree(){//IDD_SCENE_TREE_WINDOW
	closeRecentFileWindow();
	Window::activeWindow = g_stw.p;
}

void openControlPanel(){
	closeRecentFileWindow();
	
	if(g_stw->cp->node != g_selNode.getRaw()){
		g_stw->cp->setupNodeWindow(g_selNode.getRaw());
	}
	Window::activeWindow = g_stw->cp.p;
}
/*
void spaghetti(){//SPAGHETTI
	g_spaghetti=(bool)(g_spaghetti^true);
	houseKeepVisualClue();
}
*/
void resetViewPosition(){//RESET_POSITION
	g_scene->translation=Vec3f(0,0,0);
}
void resetViewOrientation(){//RESET_ORIENTATION
	g_scene->rotation = Rotation();
	houseKeepForNurbsViewChange();
}

bool resetTemporaryModes(){
	g_drag = 0;
	MultiSelect::withinSightsNode = 0;

	g_viewPosDragMode = false;

	g_insertPatch.m_insert.length = 0;
	g_insertPatch.m_insertOD.length = 0;
	g_deleteControlLine.m_delCtrlLn.length = 0;
	g_deleteControlLine.mirror_delCtrlLn.length = 0;
	g_rectangleSelectPointReady = false;
	g_rectangleUnselectPointReady = false;
	g_rectangleSelectPolyReady = false;


	if(	g_insertPatch.imousePhase != g_insertPatch.NEUTRAL ||
		g_deleteControlLine.mousePhase != g_deleteControlLine.NEUTRAL ||
		g_mergePatches.mousePhase != g_mergePatches.NEUTRAL ||
		g_addNurbsLine->mousePhase != AddNurbsLine::NEUTRAL ||
		g_drawNurbsLine->mousePhase != DrawNurbsLine::NEUTRAL ||
		g_addLathe->mousePhase != AddLathe::NEUTRAL ||
		g_addBone->mousePhase != AddBone::NEUTRAL
		
		){

		g_insertPatch.imousePhase = InsertPatch::NEUTRAL;

		g_deleteControlLine.mousePhase = DeleteControlLine::NEUTRAL;

		g_mergePatches.mousePhase = g_mergePatches.NEUTRAL;

		g_addNurbsLine->mousePhase = AddNurbsLine::NEUTRAL;
		g_drawNurbsLine->mousePhase = DrawNurbsLine::NEUTRAL;
		g_addLathe->mousePhase = AddLathe::NEUTRAL;

		g_addBone->mousePhase = AddBone::NEUTRAL;
		g_addBone->rightKeyDown = false;


		return true;
	}


	return false;
}

void reset3dScene(){//RESET_ALL	VK_ESCAPE:
	if(g_panViewMode){
		g_panViewMode = false;
		changeCursor(g_arrowCursor);
	}
	InputEvent::resumeSequence();

	resetTemporaryModes();

	g_pointer.translation = Vec3f();
//	g_pointer.rotation=Rotation();
//	g_pointer.scale=Vec3f(1,1,1);

	g_scene->translation = Vec3f();
	g_scene->rotation = Rotation();
	g_scene->scale = Vec3f(1,1,1);

	Viewpoint::gotoFirstViewpoint();
//	BackUp::monostable.beginCycle();

	houseKeepVisualClueViewChange();
}

void commonCenterViewToPivotPoint(Art* part){
//	g_scene->rotation=Rotation();
	part->smls->catIMatrix();
	part->smls->catMatrix();

	g_scene->translation=-(part->matrix * part->absolutePivotPoint);

	g_pointer.translation = Vec3f();
//	g_pointer.rotation=Rotation();
}
void Art::centerViewToControlPoint(){
	Vec3f crd;
	if(Window::animMode()){
		if(smls->animCP == false && MultiSelect::coordList.length == 1){
			smls->genDynamicMatricesAndPopulate();
			int idx = index[0];
			if(idx < coord.length){
				crd = m_matrix[m_smi[fromCoord + idx]] * (g_m_coord[fromCoord + idx] - absolutePivotPoint);
			}
			g_scene->translation = -crd;
			g_pointer.translation = Vec3f();
			return;
		}
		if(controlPoint.length == 0){
			return;
		}
		if(MultiSelect::selectedPoint >= controlPoint.length){
			MultiSelect::selectedPoint = 0;
		}
		ControlPoint& cp = controlPoint[MultiSelect::selectedPoint];
		NurbsPatch* patch = cp.p;
		patch->index[0] = cp.i;

		crd = patch->dpoint[cp.i];
	}
	else{
		int idx = index[0];
		if(idx<coord.length){
			crd = coord[idx];
		}
	}
	g_scene->translation = -crd;
	g_pointer.translation = Vec3f();
	if(g_scene->mode & ANIMATION_MODE){
		g_vcArtAnimBase->calcRotCtrlPnts();
		//g_vcArtAnimBase->resetRotationLever();
		g_vcArtAnimBase->houseKeep();
	}
}
void BezierLathe::centerViewToControlPoint(){
	int idx=index[0];
	Vec3f vec=Vec3f(point[idx].x, point[idx].y, 0);
	g_scene->translation=-vec;

	g_pointer.translation = Vec3f();
}
void BezierLatheBuild::centerViewToControlPoint(){
	int idx=index[0];
	Vec3f vec = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation) * Vec3f(point[idx].x, point[idx].y, 0);
	g_scene->translation=-vec;

	g_pointer.translation = Vec3f();
}

void NurbsLathe::centerViewToControlPoint(){
	int idx = index[0];
	Vec3f vec = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation) * Vec3f(dpoint[idx].x, dpoint[idx].y, 0);

	g_scene->translation = -vec;
}

void Lock::centerViewToControlPoint(){
	if(g_scene->mode & HAIR_REGION_MODE){
		if(Lock::selBorderLock){
			Vec3f v = Lock::selBorderLock->generate(Lock::selBorderLock->border[Lock::selBorder]);
			Vec3f vec = Vec3f(v.x, v.y, 0);
			g_scene->translation = -v;
		}
	}
	else{
		NurbsPatch::centerViewToControlPoint();
	}
}

void NurbsTrack::centerViewToControlPoint(){
	int idx = index[1];
	Vec3f vec = keyValue[idx];
	g_scene->translation = -vec;
	g_pointer.translation = Vec3f();
}
void BendyLathe::centerViewToControlPoint(){
	Vec3f vec;
	if(showRingControlPoints){
		vec = ringBase[index[2]];
	}
	else{
		vec = dpoint[index[0]];
	}
	vec = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation) * vec;
	g_scene->translation=-vec;
}

void Oscillator::centerViewToControlPoint(){
	int idx = index[0];
	Vec3f vec = Vec3f(fdpoint[idx].x, fdpoint[idx].y / 10, 0);;
	g_scene->translation = -vec;

	g_pointer.translation = Vec3f();
}

void Envelope::centerViewToControlPoint(){
	int idx = index[0];
	Vec3f vec = Vec3f(point[idx].x, point[idx].y / 10, 0);;
	g_scene->translation = -vec;

	g_pointer.translation = Vec3f();
}

void AmplitudeModulator::centerViewToControlPoint(){
	int idx = index[0];
	Vec3f vec = Vec3f(wet[idx].x, wet[idx].y / 10, 0);;
	g_scene->translation = -vec;

	g_pointer.translation = Vec3f();
}

void FrequencyModulator::centerViewToControlPoint(){
	int idx = index[0];
	Vec3f vec = Vec3f(wet[idx].x, wet[idx].y / 10, 0);;
	g_scene->translation = -vec;

	g_pointer.translation = Vec3f();
}

void Chords::centerViewToControlPoint(){
	int idx = index[0];
	Vec3f vec = Vec3f(spectrum[idx].x, spectrum[idx].y / 10, 0);;
	g_scene->translation = -vec;

	g_pointer.translation = Vec3f();
}

void NurbsPatch::centerViewToControlPoint(){
	int idx = index[0];

	Vec3f vec = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation) * dpoint[idx];

	g_scene->translation = -vec;

	g_pointer.translation = Vec3f();
}

void Harmonic::centerViewToControlPoint(){
}

void HarmonicPatch::centerViewToControlPoint(){
	int idx=index[0];
	Vec3f vec=dpoint[idx];
	g_scene->translation=-vec;

	g_pointer.translation = Vec3f();
}
void RingBuild::centerViewToControlPoint(){
	Vec3f vec = translationMatrix(location.refValue()[index[0]]) * 
		scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation) * Vec3f();

	g_scene->translation=-vec;

	g_pointer.translation = Vec3f();
}
void CCLathe::centerViewToControlPoint(){
	int idx=index[0];
	Vec2f point = seg[idx].Pf;
	Vec3f vec=Vec3f(point.x, point.y, 0);

	g_scene->translation = -vec;
	g_pointer.translation = Vec3f();
}
void VisualAid::centerViewToControlPoint(){
	needPartOrBezierMsg();
}
void centerViewToControlPoint(){
	if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		Vec3f crd;
		int idx=subdivision->index[0];
		if(idx<subdivision->point.length){
			crd=subdivision->point[idx];
		}
		g_scene->translation = -crd;
		g_pointer.translation = Vec3f();
		
	}
	else if(VisualAid* visualAid = dynamic_cast<VisualAid*>(g_selNode.getRaw())){
		visualAid->centerViewToControlPoint();
	}
	else{
		needPartOrBezierMsg();
	}
}
void centerViewToPivotPoint(){//CENTRE_PART_VIEW
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		commonCenterViewToPivotPoint(part);
	}
	else if(CenterJoint* cj=dynamic_cast<CenterJoint*>(g_selNode.getRaw())){
		if(Art* part=dynamic_cast<Art*>(cj->part.p)){
			commonCenterViewToPivotPoint(part);
		}
		else{
			centerJointHasNoRefMsg();
		}
	}
	else if(RingBuild* ringBuild=dynamic_cast<RingBuild*>(g_selNode.getRaw())){
		Vec3f vec = translationMatrix(ringBuild->location.refValue()[ringBuild->index[0]]) * 
			scaleMatrix(ringBuild->scale.refValue()) * rotationMatrix(ringBuild->rotation.refValue()) * translationMatrix(ringBuild->translation.refValue()) * Vec3f();

		g_scene->translation=-vec;

		g_pointer.translation = Vec3f();
	}
	else{
		needPartOrCenterJointMsg();
	}
}
void findThinTriangle(){//FIND_THIN_TRIANGLE
	if(Seamless* smls=dynamic_cast<Seamless*>(g_selNode.getRaw())){
		findThinTriangle(smls);
	}
	else if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		findThinTriangle(part->smls);
	}
}
void selectPreviousVertex(){//SELECT_PREV_VERTEX
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		if(smls->ActCtrlPnt>0){
			smls->ActCtrlPnt--;
		}
		else if(part->plidx>1){
			part=smls->partList[part->plidx-1];;
			g_stw->setSelNodeLine(part);
			smls->ActCtrlPnt=part->coord.length-1;
			if(smls->ActCtrlPnt<0){
				smls->ActCtrlPnt=0;
			}
		}
		makePointerEqualIndexLocation();
	}
	else{
		needPartMsg();
	}
}
void selectNextVertex(){//SELECT_NEXT_VERTEX
#ifdef PROG_ROB
	dLog("new IncButtonY(selectPrevNextVertex),");
#endif
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		if(smls->ActCtrlPnt<part->coord.length-1){
			smls->ActCtrlPnt++;
		}
		else if(part->plidx<smls->partList.length-1){
			g_stw->setSelNodeLine(smls->partList[part->plidx+1]);
			smls->ActCtrlPnt=0;
		}
		makePointerEqualIndexLocation();
	}
	else{
		needPartMsg();
	}
}
void nextTriangle(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		if(smls->animCP){
			g_transformCP->rotAxisX = true;
			return;
		}
		part=smls->actTriPart;
		if(part==0)return;
		int k=part->plidx;
		M_Art& partList=smls->partList;
		int actTri=smls->ActTri+1;
		while(1){
			if(actTri>=part->triangle.length){
				if(++k>=partList.length){
					return;
				}
				part=partList[k];
				actTri=0;
				continue;
			}
			break;
		}
		smls->ActTri=actTri;
		smls->actTriPart=part;
		g_stw->setSelNodeLine(part);
		S_Triangle tri=part->triangle[smls->ActTri];
		while(tri.refPart(smls->ActTriCorner)!=part){
			smls->ActTriCorner=(smls->ActTriCorner+1)%3;
		}
		smls->ActCtrlPnt=tri.refIndex(smls->ActTriCorner);;
		makePointerEqualIndexLocation();
	}
	else if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		g_transformCP->rotAxisX = true;

		//nurbsPatch->smls->getNextCtrlPoly();
		//nurbsPatch->houseKeep();
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->selNextPoly();
	}
	else{
		needPartMsg();
	}
}
void prevTriangle(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		part=smls->actTriPart;
		if(part==0)return;
		int k=part->plidx;
		M_Art& partList=smls->partList;
		int actTri=smls->ActTri-1;
		while(1){
			if(actTri<0){
				if(--k<0){
					return;
				}
				part=partList[k];
				actTri=part->triangle.length-1;
				continue;
			}
			break;
		}
		if(actTri<0){
			actTri=0;
		}
		smls->ActTri=actTri;
		smls->actTriPart=part;
		g_stw->setSelNodeLine(part);
		S_Triangle tri=part->triangle[smls->ActTri];
		while(tri.refPart(smls->ActTriCorner)!=part){
			smls->ActTriCorner=(smls->ActTriCorner+5)%3;
		}
		smls->ActCtrlPnt=tri.refIndex(smls->ActTriCorner);;
		makePointerEqualIndexLocation();
	}
	else if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		nurbsPatch->smls->getPrevCtrlPoly();
		nurbsPatch->houseKeep();
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->selPrevPoly();
	}
	else{
		needPartMsg();
	}
}

void selectCornerInTri(Seamless* smls){
	g_stw->setSelNodeLine(smls->actTriPart->triangle[smls->ActTri].refPart(smls->ActTriCorner));
	smls->ActCtrlPnt=smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
	makePointerEqualIndexLocation();
}
void selectPreviousCornerInTri(){//SELECT_PREV_CORNER_IN_TRI
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		if(smls->animCP){
			g_transformCP->rotAxisY = true;
			return;
		}
		if(smls->actTriPart!=0){
			smls->ActTriCorner=(smls->ActTriCorner+5)%3;
			selectCornerInTri(smls);
		}
	}
	if(dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		g_transformCP->rotAxisY = true;
	}
	else{
		needPartMsg();
	}
}
void selectNextCornerInTri(){//SELECT_NEXT_CORNER_IN_TRI
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		if(smls->animCP){
			g_transformCP->rotAxisZ = true;
			return;
		}
		if(smls->actTriPart!=0){
			smls->ActTriCorner=( smls->ActTriCorner+1)%3;
			selectCornerInTri(smls);
		}
	}
	if(dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		g_transformCP->rotAxisZ = true;
	}
	else{
		needPartMsg();
	}
}

void joinVertexImmediate(Art* copyPart, Art* part, Art* remActTriPart, Art* actTriPart, const int remActCtrlPnt, const int ActCtrlPnt, const int remActTri, const int ActTri, const int remActTriCorner, const int ActTriCorner, bool srcColor, bool dstColor, bool move, bool endWithBuild = true){
#ifdef PROG_ROB
	if(move){
		dLog("new MoveToF10(),");
		dLog("new Move_MenuItem(menu_join_move),");
		dLog("new Move_MenuItem(menu_join_vertex_keep_both_colors),");
		dLog("new Move_MenuItem(menu_move_vertex),");
		dLog("new Click,");
	}
	else if(srcColor && dstColor){
		dLog("new ClickButton(joinVertexKeepBothColors),");
	}
	else if(dstColor){
		dLog("new ClickButton(joinVertexKeepDestColor),");
	}
	else if(srcColor){
		dLog("new MoveToF10(),");
		dLog("new Move_MenuItem(menu_join_move),");
		dLog("new Move_MenuItem(menu_join_vertex_keep_source_color),");
		dLog("new Click,");
	}
#endif

	Seamless* smls=part->smls;
	BackUp::monostable.beginCycle();

	fastFullBuildForAutoIn(true);

	if(actTriPart==remActTriPart && actTriPart->triangle.refIndex(ActTri*3+ActTriCorner)==remActTriPart->triangle.refIndex(remActTri*3+remActTriCorner) && ActTri == remActTri){
		actTriPart->triangle.refTexIndex(ActTri*3+ActTriCorner)=remActTriPart->triangle.refTexIndex(remActTri*3+remActTriCorner);
	}
	else {
		//if(move){
		//	M_Vec3f& coord=part->coord.refValue();
		//	Vec3f pt = copyPart->matrix*copyPart->coord.refValue()[remActCtrlPnt];

		//	Vec3f newPt=pt-part->matrix*Vec3f();
		//	part->smls->senseMirrorVertex(part, ActCtrlPnt, newPt);
		//	coord[ActCtrlPnt]=newPt;

		//}
		//else{
		JoinVertex joinVertex;
		joinVertex.move = move;
		joinVertex.srcCoord=getIndexRelativeToPart( part, actTriPart, ActTri*3+ActTriCorner);
		joinVertex.dstCoord=getIndexRelativeToPart( copyPart, remActTriPart, remActTri*3+remActTriCorner);
		joinVertex.part=copyPart;
		joinVertex.source=part;
		joinVertex.srcColor=srcColor;
		joinVertex.dstColor=dstColor;
		//g_joinAndTransferActive = true;
		joinVertex.doBuild();
		//g_joinAndTransferActive = false;
	}
	if(endWithBuild){
		smls->doBuild();
		smls->ctrf = true;
		if(move == false){
			//g_stw->selLine=getNodeLine(copyPart);
			smls->ActTri=0;
			smls->ActTriCorner=0;
			smls->actTriPart=0;
			MultiSelect::triList.length = 0;
			MultiSelect::coordList.length = 0;
			copyPart->index[0] = 0;
			g_stw->setResultsForSelectingVertex(copyPart);

	//		if(copyPart == part && smls->ActCtrlPnt < remActCtrlPnt){
	//			smls->ActCtrlPnt=remActCtrlPnt-1;
	//		}
	//		else{
	//			smls->ActCtrlPnt=remActCtrlPnt;
	//		}
	//		makePointerEqualIndexLocation();
			g_stw->paint();
		}
		Window::paintAllControlPanels();
	}
}
inline Vec2f get2dCoord(const Art* part, const int& i, const int& perspective){
	Vec3f crd3f = part->matrix * g_scene->getViewMatrix() * part->coord.refValue()[i];
	Vec2f crd;
	if(perspective){
		crd=Vec2f(
			crd3f.x / (-crd3f.z + g_viewDist),
			crd3f.y / (-crd3f.z + g_viewDist)
		);
	}
	else{
		crd=Vec2f(crd3f.x, crd3f.y);
	}
	return crd;
}
int getClosestCorner(const float* dist){
	float minDist = dist[0];
	int minCorner = 0;
	for(int i = 3; --i>=1;){
		if(dist[i] < minDist){
			minDist = dist[i];
			minCorner = i;
		}
	}
	return minCorner;
}
Art* getClosestVertex(Art* selPart, const int selIdx, Art*& dstTriPart, int& dstIndex, int& dstTriangle, int& dstCorner, const int perspective){
	if(selIdx >= selPart->coord.length){
		return 0;
	}
	Vec2f selCoord = get2dCoord(selPart, selIdx, perspective);

	Seamless* smls = selPart->smls;
	M_Art& partList = smls->partList;

	float minDist = 10000000;
	Art* dstPart = 0;

	for(int k = partList.length; --k>=0;){
		Art* testTriPart = partList[k];
		const M_Triangle& triangle = testTriPart->triangle;
		for(int i = triangle.length; --i>=0;){
			S_Triangle tri = triangle[i];
			const Vec2f v[3] = {
				get2dCoord(tri.pa, tri.ia, perspective),
				get2dCoord(tri.pb, tri.ib, perspective),
				get2dCoord(tri.pc, tri.ic, perspective)
			};
			if(false == backFace(v[0], v[1], v[2])){
				continue;
			}
			const float dist[3] = {
				(v[0] - selCoord).length(),
				(v[1] - selCoord).length(),
				(v[2] - selCoord).length()
			};
			if(isTriangleHit(selCoord, v)){
				if((
					(tri.pa == selPart && selIdx == tri.ia) || 
					(tri.pb == selPart && selIdx == tri.ib) ||
					(tri.pc == selPart && selIdx == tri.ic)) == false){

					dstCorner = getClosestCorner(dist);
					if(tri.refPart(dstCorner)->show){
						switch(dstCorner){
							case 0:
								dstPart = tri.pa;
								dstIndex = tri.ia;
								break;
							case 1:
								dstPart = tri.pb;
								dstIndex = tri.ib;
								break;
							case 2:
								dstPart = tri.pc;
								dstIndex = tri.ic;
								break;
						}
						dstTriPart = testTriPart;
						dstTriangle = i;
						return dstPart;
					}
				}
			}
			if(dist[0] < minDist && (tri.pa == selPart && selIdx == tri.ia) == false && tri.pa->show){
				dstCorner = 0;
				minDist = dist[0];
				dstPart = tri.pa;
				dstIndex = tri.ia;
				dstTriPart = testTriPart;
				dstTriangle = i;
			}
			if(dist[1] < minDist && (tri.pb == selPart && selIdx == tri.ib) == false && tri.pb->show){
				dstCorner = 1;
				minDist = dist[1];
				dstPart = tri.pb;
				dstIndex = tri.ib;
				dstTriPart = testTriPart;
				dstTriangle = i;
			}
			if(dist[2] < minDist && (tri.pc == selPart && selIdx == tri.ic) == false && tri.pc->show){
				dstCorner = 2;
				minDist = dist[2];
				dstPart = tri.pc;
				dstIndex = tri.ic;
				dstTriPart = testTriPart;
				dstTriangle = i;
			}
		}
	}
	return dstPart;
}


void removeIfFound(NurbsPatch* patch, int idx){
	for(int i = MultiSelect::pointList.length; --i >= 0;){
		if(MultiSelect::pointList[i].p == patch && MultiSelect::pointList[i].i == idx){
			MultiSelect::pointList.deleteElement(i);
		}
	}
}

void getNeighbouringEdgePoints(int idx, int& idxA, int& idxB, int& idxAm, int& idxBm, int uDim, int vDim){
	if(NurbsPatch::isSouthEastCorner(idx, uDim)){
		idxA = idx - 2;
		idxB = idx + uDim * 2;

		idxAm = idx - 1;
		idxBm = idx + uDim;
	}
	else if(NurbsPatch::isSouthWestCorner(idx)){
		idxA = idx + 2;
		idxB = idx + uDim * 2;

		idxAm = idx + 1;
		idxBm = idx + uDim;
	}
	else if(NurbsPatch::isNorthEastCorner(idx, uDim, vDim)){
		idxA = idx - 2;
		idxB = idx - uDim * 2;

		idxAm = idx - 1;
		idxBm = idx - uDim;
	}
	else if(NurbsPatch::isNorthWestCorner(idx, uDim, vDim)){
		idxA = idx + 2;
		idxB = idx - uDim * 2;

		idxAm = idx + 1;
		idxBm = idx - uDim;
	}
	else if(NurbsPatch::isEastEdge(idx, uDim) || NurbsPatch::isWestEdge(idx, uDim)){
		idxA = idx - uDim * 2;
		idxB = idx + uDim * 2;

		idxAm = idx - uDim;
		idxBm = idx + uDim;

		int v = idx / uDim;
		if(v == 1){
			idxA = -1;
		}
		if(v == vDim - 2){
			idxB = -1;
		}
	}
	else if(NurbsPatch::isNorthEdge(idx, uDim, vDim) || NurbsPatch::isSouthEdge(idx, uDim)){
		idxA = idx - 2;
		idxB = idx + 2;

		idxAm = idx - 1;
		idxBm = idx + 1;

		int u = idx % uDim;
		if(u == 1){
			idxA = -1;
		}
		if(u == uDim - 2){
			idxB = -1;
		}
	}
}

void NurbsPatch::joinBetweenPoints(const int idx1, NurbsPatch* patch2, const int idx2){
	int uDim1 = dim.x;
	int vDim1 = dim.y;
	int uDim2 = patch2->dim.x;
	int vDim2 = patch2->dim.y;

	int idx1A = 0;
	int idx1B = 0;

	int idx2A = 0;
	int idx2B = 0;

	int idx1Am = 0;
	int idx1Bm = 0;

	int idx2Am = 0;
	int idx2Bm = 0;

	getNeighbouringEdgePoints(idx1, idx1A, idx1B, idx1Am, idx1Bm, uDim1, vDim1);
	getNeighbouringEdgePoints(idx2, idx2A, idx2B, idx2Am, idx2Bm, uDim2, vDim2);

	if(idx1A >= 0 && idx2A >= 0 && cpIndex[idx1A] == patch2->cpIndex[idx2A]){
		smls->redirectSingleCpIndex(patch2->cpIndex[idx2Am], cpIndex[idx1Am]);
		//if(this == patch2 && abs(idx1A - idx2A) == 2){
		//	smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx2Am]);
		//}
	}
	else if(idx1A >= 0 && idx2B >= 0 && cpIndex[idx1A] == patch2->cpIndex[idx2B]){
		smls->redirectSingleCpIndex(patch2->cpIndex[idx2Bm], cpIndex[idx1Am]);
		//if(this == patch2 && abs(idx1A - idx2B) == 2){
		//	smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx2Bm]);
		//}
	}
	else if(idx1B >= 0 && idx2A >= 0 && cpIndex[idx1B] == patch2->cpIndex[idx2A]){
		smls->redirectSingleCpIndex(patch2->cpIndex[idx2Am], cpIndex[idx1Bm]);
		//if(this == patch2 && abs(idx1B - idx2A) == 2){
		//	smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx2Am]);
		//}
	}

	if(idx1B >= 0 && idx2B >= 0 && cpIndex[idx1B] == patch2->cpIndex[idx2B]){
		smls->redirectSingleCpIndex(patch2->cpIndex[idx2Bm], cpIndex[idx1Bm]);
		//if(this == patch2 && abs(idx1B - idx2B) == 2){
		//	smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx2Bm]);
		//}
	}
	else if(idx1B >= 0 && idx2A >= 0 && cpIndex[idx1B] == patch2->cpIndex[idx2A]){
		smls->redirectSingleCpIndex(patch2->cpIndex[idx2Am], cpIndex[idx1Bm]);
		//if(this == patch2 && abs(idx1B - idx2A) == 2){
		//	smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx2Am]);
		//}
	}
	else if(idx1A >= 0 && idx2B >= 0 && cpIndex[idx1A] == patch2->cpIndex[idx2B]){
		smls->redirectSingleCpIndex(patch2->cpIndex[idx2Bm], cpIndex[idx1Am]);
		//if(this == patch2 && abs(idx1A - idx2B) == 2){
		//	smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx2Bm]);
		//}
	}

	if(this == patch2){
		int u = idx1 % uDim1;
		if(isNorthEdge(idx1, uDim1, vDim1) || isSouthEdge(idx1, uDim1)){
			if(idx1 - idx2 == -2){
				if(u < uDim1 - 1){
					smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx1 + 1]);
				}
			}
			else if(idx1 - idx2 == 2){
				if(u > 1){
					smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx1 - 1]);
				}
			}
		}
		else{
			int v = idx1 / uDim1;
			if(idx1 - idx2 == - uDim1 * 2){
				if(v < vDim1 - 1){
					smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx1 + uDim1]);
				}
			}
			else if(idx1 - idx2 == uDim1 * 2){
				if(v > 1){
					smls->redirectSingleCpIndex(cpIndex[idx1], cpIndex[idx1 - uDim1]);
				}
			}
		}
	}
}

void NurbsPatch::collectSharedPoints(int cIdx, M_NurbsPatch& m_patch, M_Int& m_idx){
	for(int k = cpIndex.length; --k >= 0;){
		if(cpIndex[k] == cIdx){
			m_idx += k;
			m_patch += this;
		}
	}
}

void Seamless::joinBetweenPoints(NurbsPatch* patch1, const int idx1, NurbsPatch* patch2, const int idx2){
	M_NurbsPatch m_patch1;
	M_NurbsPatch m_patch2;
	M_Int m_idx1;
	M_Int m_idx2;

	int cIdx1 = patch1->cpIndex[idx1];
	int cIdx2 = patch2->cpIndex[idx2];


	for(int k = cpNodeList.length; --k >= 0;){
		NurbsPatch* patch = cpNodeList[k];
		patch->collectSharedPoints(cIdx1, m_patch1, m_idx1);
		patch->collectSharedPoints(cIdx2, m_patch2, m_idx2);
	}

	for(int k1 = m_patch1.length; --k1 >= 0;){
		NurbsPatch* patch1 = m_patch1[k1];
		int idx1 = m_idx1[k1];

		for(int k2 = m_patch2.length; --k2 >= 0;){
			NurbsPatch* patch2 = m_patch2[k2];
			int idx2 = m_idx2[k2];

			patch1->joinBetweenPoints(idx1, patch2, idx2);
			

			removeIfFound(patch1, idx1);
			removeIfFound(patch2, idx2);
		}
	}

	redirectSingleCpIndex(patch2->cpIndex[idx2], patch1->cpIndex[idx1]);
}

/*
void Seamless::redirectSingleCpIndexExtra(NurbsPatch* selPatch, const int selIdx, NurbsPatch* shortestPatch, const int shortestIdx){
	if(shortestPatch->mirror && selPatch->mirror){
		NurbsPatch* mirrorSelPatch = selPatch->mirrorPatch;
		if(mirrorSelPatch == 0){
			mirrorSelPatch = selPatch;
		}
		int mirrorSelIdx = mirrorSelPatch->getMirrorIdx(selIdx);

		NurbsPatch* mirrorShortestPatch = shortestPatch->mirrorPatch;
		if(mirrorShortestPatch == 0){
			mirrorShortestPatch = shortestPatch;
		}

		int mirrorShortestIdx = mirrorShortestPatch->getMirrorIdx(shortestIdx);

		mirrorSelPatch->smls->redirectSingleCpIndex(mirrorShortestPatch->cpIndex[mirrorShortestIdx], mirrorSelPatch->cpIndex[mirrorSelIdx]);
		if(shortestPatch->showAll == false){
		}

		removeIfFound(mirrorShortestPatch, mirrorShortestIdx);
		removeIfFound(mirrorSelPatch, mirrorSelIdx);
	}

	selPatch->smls->redirectSingleCpIndex(shortestPatch->cpIndex[shortestIdx], selPatch->cpIndex[selIdx]);
	if(shortestPatch->showAll == false){
	}

	removeIfFound(shortestPatch, shortestIdx);
	removeIfFound(selPatch, selIdx);
}
*/

NurbsPatch* getMirrorPatches(NurbsPatch* selPatch, const int selIdx, NurbsPatch* shortestPatch, const int shortestIdx, int& mirrorSelIdx, NurbsPatch*& mirrorShortestPatch, int& mirrorShortestIdx){
	if(shortestPatch->mirror && selPatch->mirror){
		NurbsPatch* mirrorSelPatch = selPatch->mirrorPatch;
		if(mirrorSelPatch == 0){
			mirrorSelPatch = selPatch;
		}
		mirrorSelIdx = mirrorSelPatch->getMirrorIdx(selIdx);

		mirrorShortestPatch = shortestPatch->mirrorPatch;
		if(mirrorShortestPatch == 0){
			mirrorShortestPatch = shortestPatch;
		}

		mirrorShortestIdx = mirrorShortestPatch->getMirrorIdx(shortestIdx);

		return mirrorSelPatch;
	}
	return 0;
}

void Seamless::moveNurbsCP(bool join){
	genVisibleMask();

	if(MultiSelect::pointList.length == 0){
		return;
	}
	const bool perspective = g_perspective;
	
	while(MultiSelect::pointList.length){
		NurbsPatch* selPatch = MultiSelect::pointList[0].p;
		int selIdx = MultiSelect::pointList[0].i;

		Vec2f selPoint = get2dCoord(selPatch->dpoint[selIdx], perspective);

		float shortest = 100000000;
		int shortestIdx = 0;
		int g = 0;
		NurbsPatch* shortestPatch = 0;

		for(int k = cpNodeList.length; --k >= 0;){
			NurbsPatch* patch = cpNodeList[k];
			if(patch->cage){
				M_Vec3f& dpoint = patch->dpoint.refValue();
				for(g = dpoint.length; --g >= 0;){
					//if(patch != selPatch || selIdx != g){
					if(patch->isEdge(g) && patch->pointIsVisible(patch->fromPoint + g)){
						if(selPatch->cpIndex[selIdx] != patch->cpIndex[g]){
							Vec2f vec = get2dCoord(patch->dpoint[g], perspective);
							float distance = (selPoint - vec).length();
							if(distance < shortest){
								shortest = distance;
								shortestPatch = patch;
								shortestIdx = g;
							}
						}
					}
				}
			}
		}
		Vec3f vec = shortestPatch->dpoint[shortestIdx] * selPatch->weight[selIdx];
		selPatch->point.refValue()[selIdx] = vec;
		cp.refValue()[selPatch->cpIndex[selIdx]] = vec;
		selPatch->mirrorSingleCP(selIdx);

		int mirrorSelIdx = 0;
		int mirrorShortestIdx = 0;
		NurbsPatch*  mirrorShortestPatch = 0;
		NurbsPatch* mirrorSelPatch = getMirrorPatches(selPatch, selIdx, shortestPatch, shortestIdx, mirrorSelIdx, mirrorShortestPatch, mirrorShortestIdx);


		if(join){
			//selPatch->cpIndex.refValue()[selIdx] = shortestPatch->cpIndex[shortestIdx];
			

			if(g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE){
				//selPatch->joinBetweenPoints(selIdx, shortestPatch, shortestIdx);
				joinBetweenPoints(selPatch, selIdx, shortestPatch, shortestIdx);


				if(mirrorSelPatch){
					joinBetweenPoints(mirrorSelPatch, mirrorSelIdx, mirrorShortestPatch, mirrorShortestIdx);
				}


			}
			else{
				redirectSingleCpIndex(shortestPatch->cpIndex[shortestIdx], selPatch->cpIndex[selIdx]);
				if(shortestPatch->showAll == false){
					shareMidPointsTooInStitchPolys(shortestPatch, shortestIdx);
				}
				//redirectSingleCpIndexExtra(selPatch, selIdx, shortestPatch, shortestIdx);



				//do same for mirror points

				if(mirrorSelPatch){

					mirrorSelPatch->smls->redirectSingleCpIndex(mirrorShortestPatch->cpIndex[mirrorShortestIdx], mirrorSelPatch->cpIndex[mirrorSelIdx]);

					if(shortestPatch->showAll == false){
						shareMidPointsTooInStitchPolys(mirrorShortestPatch, mirrorShortestIdx);
					}

					removeIfFound(mirrorShortestPatch, mirrorShortestIdx);
					removeIfFound(mirrorSelPatch, mirrorSelIdx);

				}
				removeIfFound(shortestPatch, shortestIdx);
				removeIfFound(selPatch, selIdx);
			}
		}
		else{
			if(mirrorSelPatch){
				removeIfFound(mirrorShortestPatch, mirrorShortestIdx);
				removeIfFound(mirrorSelPatch, mirrorSelIdx);
			}
			removeIfFound(shortestPatch, shortestIdx);
			removeIfFound(selPatch, selIdx);
		}

		//selPatch->weight.refValue()[selIdx] = shortestPatch->weight[shortestIdx];
		selPatch->processFields();


	}
	//generateRawStitchPolys();
	//stitchAllPolys();
	//joinAllParts();


	//houseKeepVisualClue();


	MultiSelect::reset();
	cleanCps();
	BackUp::monostable.beginCycle();
	generateRawStitchPolys();
	doBuild(); //must this be done before stitchAllPolys to prevent cage going crazy?
	stitchAllPolys();
	//doBuild();
	houseKeepVisualClue();
}

extern ImmediatePart* g_workPolyMapColor;
extern ImmediatePart* g_workPolyMapTexture;

ImmediatePart* selectPolyMapPart(Art* part){
	if(dynamic_cast<ColorEffect*>(part->workEffect)){
		return g_workPolyMapColor;
	}
	else{
		return g_workPolyMapTexture;
	}
}

void TextureEffect::clear(){
	texCoord.refValue().length = 0;
}

void ColorEffect::clear(){
	color.refValue().length = 0;
}
void MonoColorEffect::clear(){
	color.refValue().length = 0;
}

void Stem::copyPolyMapToPart(Art* part){
	part->coord.refValue().length = 0;
	part->normal.refValue().length = 0;
	part->workEffect->clear();
	part->triangle.length = 0;
	part->workEffect->stemCopyLocalTriNonMirror(this, part, part->coord.refValue(), 0, 0);
	part->smls->resetRelevantParts();
}

void moveVertex(){//MOVE_VERTEX
	int dstIndex=0;
	int dstTri=0;
	int dstCorner=0;
	Art* dstTriPart=0;
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			Seamless* smls = whitePart->smls;
			Art* dstTriPart=0;
			if(Art* dstPart=getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
				joinVertexImmediate(dstPart, whitePart, dstTriPart, smls->actTriPart, dstIndex, smls->ActCtrlPnt, dstTri, smls->ActTri, dstCorner, smls->ActTriCorner, true, true, true);
			}
			return;
		}
		//toggleNSPE();
		//g_polyEdit = false; //enable joinVertex node
		if(Art* dstPart=getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
			if(dstPart == whitePart){
				NurbsPatch* patch = 0;
				Stem* srcStem = 0;
				Stem* dstStem = 0;
				Polymap* polymap = 0;
				ImmediatePart* bluePart = 0;
				if(getPolyMap(patch, srcStem, polymap, whitePart, smls->ActTri, bluePart)){
					if(getPolyMap(patch, dstStem, polymap, whitePart, dstTri, bluePart)){
						if(bluePart){
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex-dstStem->beginCoordOffset, smls->ActCtrlPnt-srcStem->beginCoordOffset, dstTri-dstStem->beginTriOffset, smls->ActTri-srcStem->beginTriOffset, dstCorner, smls->ActTriCorner, true, true, true);
						}
						else{
							bluePart = selectPolyMapPart(whitePart);
							dstStem->copyPolyMapToPart(bluePart);
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex-dstStem->beginCoordOffset, smls->ActCtrlPnt-srcStem->beginCoordOffset, dstTri-dstStem->beginTriOffset, smls->ActTri-srcStem->beginTriOffset, dstCorner, smls->ActTriCorner, true, true, true, false);
							dstStem->copyPartToPolyMap(bluePart);
							commonEndVertexBuild();
						}
					}
				}
			}
		}
		//toggleNSPE();
		//g_polyEdit = true;
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->moveNurbsCP();
	}
	else{
		needPartMsg();
	}
}

int getRelIndex(Stem*& stem, int& ActTri, int& ActTriCorner){//returns coord ActCtrlPnt
	ActTri = (ActTri - stem->beginTriOffset) * 3;
	if(stem->mirror){
		if(stem->mirrorStem == 0 && ActTri >= stem->coordIndex.length){
			//ActCtrlPnt -= stem->coord.length;
			ActTri -= stem->coordIndex.length;
			ActTriCorner = abs(3 - ActTriCorner) % 3;
		}
		if(stem->mirrorStem && stem->coordIndex.length == 0 && stem->mirrorStem->coordIndex.length){
			stem = stem->mirrorStem;
			ActTriCorner = abs(3 - ActTriCorner) % 3;
		}
	}
	if(ActTri + ActTriCorner >= stem->coordIndex.length){
		return -1;
	}
	return stem->coordIndex[ActTri + ActTriCorner];
}

void joinVertexKeepDestColor(){//JOIN_VERTEX //JOIN_COLOR //join/stitch control point
	int dstIndex=0;
	int dstTri=0;
	int dstCorner=0;
	Art* dstTriPart=0;
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			if(Art* dstPart=getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
				joinVertexImmediate(dstPart, whitePart, dstTriPart, smls->actTriPart, dstIndex, smls->ActCtrlPnt, dstTri, smls->ActTri, dstCorner, smls->ActTriCorner, false, true, false);
			}
			return;
		}
		if(Art* dstPart=getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
			if(dstPart == whitePart){
				NurbsPatch* patch = 0;
				Stem* srcStem = 0;
				Stem* dstStem = 0;
				Polymap* polymap = 0;
				ImmediatePart* bluePart = 0;
				if(getPolyMap(patch, srcStem, polymap, whitePart, smls->ActTri, bluePart)){
					if(getPolyMap(patch, dstStem, polymap, whitePart, dstTri, bluePart)){
						if(bluePart){
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex-dstStem->beginCoordOffset, smls->ActCtrlPnt-srcStem->beginCoordOffset, dstTri-dstStem->beginTriOffset, smls->ActTri-srcStem->beginTriOffset, dstCorner, smls->ActTriCorner, false, true, false);
						}
						else{
							int ActTri = smls->ActTri;
							int ActTriCorner = smls->ActTriCorner;
							if(0 > getRelIndex(srcStem, ActTri, ActTriCorner)){
								return;
							}
							int ActCtrlPnt = srcStem->rcCoord(ActTri + ActTriCorner);
				

							if(0 > getRelIndex(dstStem, dstTri, dstCorner)){
								return;
							}
							int dstIndex = dstStem->rcCoord(dstTri + dstCorner);

							bluePart = selectPolyMapPart(whitePart);
							dstStem->copyPolyMapToPart(bluePart);
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex, ActCtrlPnt, dstTri / 3, ActTri / 3, dstCorner, ActTriCorner, false, true, false, false);
							dstStem->copyPartToPolyMap(bluePart);
							commonEndVertexBuild();
						}
					}
				}
			}
		}
		//g_joinActive = false;
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->joinVertex();
	}
	else if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		Hair* hair = getHair();
		if((g_scene->mode & HAIR_REGION_MODE)){
			if(Lock::selBorderLock){
				int closestBorderIdx = Lock::selBorderLock->getClosestBorderPoint(Lock::selBorder);
				M_Node& m_hairLock = hair->lock.refValue();
				for(int k = 0; k < m_hairLock.length; k++){
					Lock* srcLock = dynamic_cast<Lock*>(m_hairLock[k].p);
					M_Node& m_lock = srcLock->lock.refValue();
					for(int k = 0; k < m_lock.length; k++){
						Lock* lock = dynamic_cast<Lock*>(m_lock[k].p);
						lock->joinBorderPoints(srcLock, closestBorderIdx);
						//Lock::selBorderLock = 0;
					}
				}
				houseKeepVisualClue();
			}
		//if(Lock::selBorderLock){
		//	if(Lock* parentLock = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
		//		int closestBorderIdx = Lock::selBorderLock->getClosestBorderPoint(Lock::selBorder);
		//		lock->joinBorderPoints(parentLock, closestBorderIdx);
		//		houseKeepVisualClue();
		//	}
		}
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->moveNurbsCP(true);
	}
	else if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		MultiSelect::selectRectMode=true;
	}
	else{
		needPartMsg();
	}
}

void addMidPoint(M_CtrlPolyPoint& ctrlPolyPoint, int ctrlPolyIdx, M_ControlPoint& pointList);

void joinVertexKeepSourceColor(){
	int dstIndex=0;
	int dstTri=0;
	int dstCorner=0;
	Art* dstTriPart=0;
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;

		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			Art* dstTriPart = 0;
			if(Art* dstPart=getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
				joinVertexImmediate(dstPart, whitePart, dstTriPart, smls->actTriPart, dstIndex, smls->ActCtrlPnt, dstTri, smls->ActTri, dstCorner, smls->ActTriCorner, true, false, false);
			}
			return;
		}
		if(smls->workAnimCP && MultiSelect::pointList.length){
			smls->addMidPoint(MultiSelect::pointList);
			return;
		}
		if(Art* dstPart=getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
			if(dstPart == whitePart){
				NurbsPatch* patch = 0;
				Stem* srcStem = 0;
				Stem* dstStem = 0;
				Polymap* polymap = 0;
				ImmediatePart* bluePart = 0;
				if(getPolyMap(patch, srcStem, polymap, whitePart, smls->ActTri, bluePart)){
					if(getPolyMap(patch, dstStem, polymap, whitePart, dstTri, bluePart)){
						if(bluePart){
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex-dstStem->beginCoordOffset, smls->ActCtrlPnt-srcStem->beginCoordOffset, dstTri-dstStem->beginTriOffset, smls->ActTri-srcStem->beginTriOffset, dstCorner, smls->ActTriCorner, true, false, false);
						}
						else{
							bluePart = selectPolyMapPart(whitePart);
							dstStem->copyPolyMapToPart(bluePart);
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex-dstStem->beginCoordOffset, smls->ActCtrlPnt-srcStem->beginCoordOffset, dstTri-dstStem->beginTriOffset, smls->ActTri-srcStem->beginTriOffset, dstCorner, smls->ActTriCorner, true, false, false, false);
							dstStem->copyPartToPolyMap(bluePart);
							commonEndVertexBuild();
						}
					}
				}
			}
		}
	}
	else if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = nurbsPatch->smls;
		if(MultiSelect::pointList.length){
			smls->addMidPoint(MultiSelect::pointList);
			return;
		}
	}
	else if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		MultiSelect::selectRectMode=true;
	}
	else{
		needPartMsg();
	}
}
void joinVertexKeepBothColors(){
	int dstIndex=0;
	int dstTri=0;
	int dstCorner=0;
	Art* dstTriPart=0;
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		if(MultiSelect::pointList.length){
			smls->createControlPolygon(MultiSelect::pointList);
			return;
		}

		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			Art* dstTriPart = 0;
			if(Art* dstPart=getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
				joinVertexImmediate(dstPart, whitePart, dstTriPart, smls->actTriPart, dstIndex, smls->ActCtrlPnt, dstTri, smls->ActTri, dstCorner, smls->ActTriCorner, true, true, false);
			}
			return;
		}


		if(Art* dstPart = getClosestVertex(whitePart, smls->ActCtrlPnt, dstTriPart, dstIndex, dstTri, dstCorner, g_perspective)){
			if(dstPart == whitePart){
				NurbsPatch* patch = 0;
				Stem* srcStem = 0;
				Stem* dstStem = 0;
				Polymap* polymap = 0;
				ImmediatePart* bluePart = 0;
				if(getPolyMap(patch, srcStem, polymap, whitePart, smls->ActTri, bluePart)){
					if(getPolyMap(patch, dstStem, polymap, whitePart, dstTri, bluePart)){
						if(bluePart){
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex-dstStem->beginCoordOffset, smls->ActCtrlPnt-srcStem->beginCoordOffset, dstTri-dstStem->beginTriOffset, smls->ActTri-srcStem->beginTriOffset, dstCorner, smls->ActTriCorner, true, true, false);
						}
						else{
							bluePart = selectPolyMapPart(whitePart);
							dstStem->copyPolyMapToPart(bluePart);
							joinVertexImmediate(bluePart, bluePart, bluePart, bluePart, dstIndex-dstStem->beginCoordOffset, smls->ActCtrlPnt-srcStem->beginCoordOffset, dstTri-dstStem->beginTriOffset, smls->ActTri-srcStem->beginTriOffset, dstCorner, smls->ActTriCorner, true, true, false, false);
							dstStem->copyPartToPolyMap(bluePart);
							commonEndVertexBuild();
						}
					}
				}
			}
		}
	}
	else if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = nurbsPatch->smls;
		if(MultiSelect::pointList.length){
			smls->createControlPolygon(MultiSelect::pointList);
			return;
		}
	}
	else if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		MultiSelect::selectRectMode=true;
	}
	else{
		needPartMsg();
	}
}
void joinMatchingColors(){//JOIN_MATCHING_COLORS
	Seamless* smls=dynamic_cast<Seamless*>(g_selNode.getRaw());
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		smls=part->smls;
	}
	if(smls){
		dynamic_cast<Effect*>(smls->effect.refValue().p)->optimize(smls->partList);
		smls->process();
		BackUp::monostable.beginCycle();
	}
	else{
		needPartOrSmlsMsg();
	}
}
void colorPart(){//COLOR_PART
	if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
			part->colorize(g_palette->color);
			commonEndVertexChange();
		}
	}
	else{
		needBluePartMsg();
	}
}


void replaceColorPart(){//REPLACE_COLOR_PART
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<ColorEffect*>(whitePart->workEffect)==0)return;
		Seamless* smls = whitePart->smls;
		
		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			Color3f oldColor;
			if(getOldColor(oldColor)){
				replaceColorInPart(whitePart, oldColor, g_palette->color);
			}
			return;
		}

		Color3f oldColor;
		if(getOldColor(oldColor)){
			NurbsPatch* patch = 0;
			Stem* stem = 0;
			Polymap* polymap = 0;
			ImmediatePart* bluePart = 0;
			if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
				if(bluePart){
				}
				else{
					int ActTri = smls->ActTri;
					int ActTriCorner = smls->ActTriCorner;
					if(0 > getRelIndex(stem, ActTri, ActTriCorner)){
						return;
					}

					M_Color3f& color = stem->color.refValue();
					M_Int& coordIndex = stem->coordIndex.refValue();
					for(int i = coordIndex.length; --i >= 0;){
						int ci = coordIndex[i];
						if(color[ci] == oldColor){
							color[ci] = g_palette->color;
						}
					}
					commonEndVertexBuild();
				}
			}
		}
	}
	else if(GroupBuild* group = dynamic_cast<GroupBuild*>(g_selNode.getRaw())){
		hairOnSurface(group->build.refValue());
	}
	else if(Strand* strand = dynamic_cast<Strand*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){
			hair->bindToSeperator(strand);
		}
	}
	else{
		needBluePartMsg();
	}
	BackUp::monostable.beginCycle();
}
void replaceColorSkeleton(){//REPLACE_COLOR_SKELETON
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Art* copied=dynamic_cast<Art*>(g_copyNode.getRaw())){
			Color3f oldColor;
			if(getOldColor(oldColor)){
				M_Art& partList=part->smls->partList;
				for(int k=partList.length; --k>=0;){
					replaceColorInPart(partList[k], oldColor, g_palette->color);
				}
			}
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needBluePartMsg();
	}
}
void transferCloseVerticesJoin(){//	 TRANSFERCLOSEVERTICES_JOIN
	if(Art* source=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Art* copied=dynamic_cast<Art*>(g_copyNode.getRaw())){
			fastFullBuildForAutoIn(true);
			pasteCloseVerticesJoin(source, copied, true);
			Seamless* smls=source->smls;
			smls->verifyIntegrity();

			smls->convertToRenderFormat();
			makePointerEqualIndexLocation();
		//			g_stw->setSelNodeLine();
			g_stw->paint();
			g_stw->cp->paint();
			BackUp::monostable.beginCycle();
		}
		else{
			needPartCopiedMsg();
		}
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->joinCloseVertices();
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->joinAllSelectedStitchPolys();
	}
	else{
		needPartMsg();
	}
}
inline bool checkVertexIsShared(M_Art& partList, Art* targetVertexPart, int& targetVertexIndex, Art* targetTrianglePart, int targetTriangleIndex){
	for(int k=partList.length; --k>=0; ){
		Art* part=partList[k];
		M_Triangle& triangle=part->triangle;
		for(int i=triangle.length*3, t=triangle.length; --t>=0; ){
			if(targetTriangleIndex==t && targetTrianglePart==part){ //first check its not the same triangle before asking any more questions
				i-=3;
			}
			else {
				--i;
				if(triangle.refPart(i)==targetVertexPart&&triangle.refIndex(i)==targetVertexIndex){
					return true;
				}
				--i;
				if(triangle.refPart(i)==targetVertexPart&&triangle.refIndex(i)==targetVertexIndex){
					return true;
				}
				--i;
				if(triangle.refPart(i)==targetVertexPart&&triangle.refIndex(i)==targetVertexIndex){
					return true;
				}
			}
		}
	}
	return false;
}

void joinBorderVertices(){
	if(NurbsPatch* patch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		smls->joinBorderVertices(patch);
		smls->verifyIntegrity();

		smls->convertToRenderFormat();
		makePointerEqualIndexLocation();
		g_stw->paint();
		g_stw->cp->paint();
		BackUp::monostable.beginCycle();
	}
}

void Seamless::joinBorderVertices(){
	for(int k = cpNodeList.length; --k>=0;){
		joinBorderVertices(cpNodeList[k]);
	}
	verifyIntegrity();
	convertToRenderFormat();
	makePointerEqualIndexLocation();
	g_stw->paint();
	g_stw->cp->paint();
	BackUp::monostable.beginCycle();
}

void Seamless::joinAllParts(){
	if(bnspe){
		joinBorderVertices();
	}

	for(int k = partList.length; --k>=0;){
		Art* partK=partList[k];
		if(partK->show==false){
			continue;
		}
		pasteCloseVerticesJoin(partK, partK, true, .00001);
	}
	for(int k = partList.length; --k>=0;){
		Art* partK=partList[k];
		for(int j=partList.length; --j>=0;){
			Art* partJ = partList[j];
			if(partJ->show == false){
				continue;
			}
			if(j==k){
				continue;
			}
			if(j>k){
				pasteCloseVerticesJoin(partK, partJ, true, .00001);	
			}
			else{
				pasteCloseVerticesJoin(partJ, partK, true, .00001);	
			}
		}
	}
	verifyIntegrity();

	convertToRenderFormat();
	makePointerEqualIndexLocation();
//			g_stw->setSelNodeLine();
	g_stw->paint();
	g_stw->cp->paint();
}

void joinCloseVerticesMacro(){//	 TRANSFERCLOSEVERTICES_JOIN
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){

		part->smls->joinAllParts();
		BackUp::monostable.beginCycle();
	}
	else{
		needPartMsg();
	}
}

void multiBreak(Seamless* smls);

void breakVertexJoin(){
#ifdef PROG_ROB
	dLog("new MoveToF10(),");
	dLog("new Move_MenuItem(menu_break_join),");
	dLog("new Move_MenuItem(menu_break_coord),");
	dLog("new Click,");
#endif

	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		fastFullBuildForAutoIn(true);

		Seamless* smls = part->smls;

		if(MultiSelect::triList.length){
			multiBreak(smls);
			return;
		}

		M_Vec3f& coord = part->coord.refValue();
		M_Vec2f& uvCoord = part->uvCoord.refValue();
		M_NurbsPatch& patchOwner = part->patchOwner;
		M_Vec3f& normal = part->normal.refValue();

		if(coord.length==0){
			return;
		}
		if(checkVertexIsShared(smls->partList, part, smls->ActCtrlPnt, smls->actTriPart, smls->ActTri)==false){
			msG(msg_vertex_belongs_to_only_one);
			return;
		}
		BackUp::monostable.beginCycle();

		int i=smls->ActTri*3+smls->ActTriCorner;

		M_Triangle& triangle=smls->actTriPart->triangle;

		int ia = triangle.refIndex(i);
		int cli = triangle.refTexIndex(i);

		triangle.refIndex(i)=coord.length;

		smls->ActCtrlPnt=coord.length;

		coord += coord[ia];
		if(smls->bnspe){
			uvCoord += uvCoord[ia];
			patchOwner += patchOwner[ia];
		}


		if(normal.length == coord.length){
			normal+=normal[smls->ActCtrlPnt];
		}

		if(part->workEffect!=0){
			if(part->workEffect->getType()==COLOREFFECT){
				M_Color3f& color = dynamic_cast<ColorEffect*>(part->workEffect)->color.refValue();
				triangle.refTexIndex(i) = color.length;
				color += color[cli];
			}
			else if(part->workEffect->getType()==TEXTUREEFFECT){
				M_Vec2f& texCoord = dynamic_cast<TextureEffect*>(part->workEffect)->texCoord.refValue();
				triangle.refTexIndex(i) = texCoord.length;
				texCoord += texCoord[cli];
			}
		}
		MultiSelect::coordList.length = 0;
		MultiSelect::beginDragPos.length = 0;

		smls->doBuild();
		smls->convertToRenderFormat();
		makePointerEqualIndexLocation();
	}
	else if(dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(g_scene->mode & HAIR_REGION_MODE){
			Lock::breakBorderMode = true;
		}
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		if(MultiSelect::pointList.length){
			patch->breakControlPointJoin(MultiSelect::pointList[0].i);
		}
	}
	else{
		needPartMsg();
	}

}

void breakColorJoin(){//BREAK_COLOR_JOIN
	if(g_scene->mode & MODELLING_MODE){
		toggleBoxPatchModellingMode();
		return;
	}
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
#ifdef PROG_ROB
		dLog("new MoveToF10(),");
		dLog("new Move_MenuItem(menu_break_join),");
		dLog("new Move_MenuItem(menu_break_color),");
		dLog("new Click,");
#endif
		part->workEffect->breakColorJoin(*(part->smls));
	}
	else{
		needPartMsg();
	}
}
void cutWholePart(){//CUT_WHOLE_PART
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		fastFullBuildForAutoIn(true);
		part->deleteGeometry();
		part->smls->convertToRenderFormat();
		makePointerEqualIndexLocation();
		BackUp::monostable.beginCycle();
	}
	else{
		needPartMsg();
	}
}
Vec2f g_storeClickPos=Vec2f(.195313, .728516);
void Art::tripleSplitTriangle(int triIdx){
	int newCoordIndex=coord.length;
	Vec3f newCoord;

	int a = triIdx;
	int b = triIdx+1;
	int c = triIdx+2;

	int ia=triangle.refIndex(a);
	int ib=triangle.refIndex(b);
	int ic=triangle.refIndex(c);

	Art* pa=triangle.refPart(a);
	Art* pb=triangle.refPart(b);
	Art* pc=triangle.refPart(c);

	Vec3f vecA = pa->matrix * g_viewMatrix * pa->coord.refValue()[ia];
	Vec3f vecB = pb->matrix * g_viewMatrix * pb->coord.refValue()[ib];
	Vec3f vecC = pc->matrix * g_viewMatrix * pc->coord.refValue()[ic];

//	newCoord = (vecA + vecB + vecC) / 3.0f;
//	newCoord = g_scene->getIViewMatrix() * imatrix * newCoord;

	Vec3f v[3]={vecA, vecB, vecC};

	Vec3f clickPos;
	if(getTriangleHitPos(clickPos, g_storeClickPos, v)){

		newCoord = g_scene->getIViewMatrix() * imatrix * clickPos;

		int newColIndex=0;

		if(workEffect->getType()==COLOREFFECT){
			ColorEffect* effect=dynamic_cast<ColorEffect*>(workEffect);
			M_Color3f& color=effect->color.refValue();
			newColIndex=color.length;

			Color3f colA=color[triangle.refTexIndex(a)];
			Color3f colB=color[triangle.refTexIndex(b)];
			Color3f colC=color[triangle.refTexIndex(b)];

			Color3f newColor;
			newColor.r = colA.r - (colA.r - colB.r)/2.;
			newColor.g = colA.g - (colA.g - colB.g)/2.;
			newColor.b = colA.b - (colA.b - colB.b)/2.;
			color+=newColor;
		}
		else if(workEffect->getType()==TEXTUREEFFECT){
			TextureEffect* effect=dynamic_cast<TextureEffect*>(workEffect);
			M_Vec2f& texCoord=effect->texCoord.refValue();
			newColIndex=texCoord.length;
			Vec2f texA=texCoord[triangle.refTexIndex(a)];;
			Vec2f texB=texCoord[triangle.refTexIndex(b)];;

			Vec2f newTexCoord;
			newTexCoord.x = texA.x - (texA.x - texB.x)/2.;
			newTexCoord.y = texA.y - (texA.y - texB.y)/2.;
			texCoord+=newTexCoord;
		}
		else throw this;

		coord.refValue()+=newCoord;

		if(normal.length){
			normal.refValue()+=Vec3f();
		}

		pc=triangle.refPart(c);
		ic=triangle.refIndex(c);
		int tc=triangle.refTexIndex(c);
		
		triangle.refPart(c)=this;
		triangle.refIndex(c)=newCoordIndex;
		triangle.refTexIndex(c)=newColIndex;

		triangle+=S_Triangle(
			pb, ib, triangle.refTexIndex(b),
			pc, ic, tc,
			this, newCoordIndex,newColIndex
		);
		triangle+=S_Triangle(
			pc, ic, tc,
			pa, ia, triangle.refTexIndex(a),
			this, newCoordIndex,newColIndex
		);
	}
}

/*
void  projectSingleVertexOntoPoly(){
#ifdef PROG_ROB
	dLog("new ClickButton(projectSingleVertexOntoPoly),");
#endif

	if(Art* selPart=dynamic_cast<Art*>(g_selNode.getRaw())){

		const bool perspective=g_perspective;
		Seamless* smls=selPart->smls;
		smls->catMatrix();
		smls->catIMatrix();
		const int selTri=smls->ActTri*3;
		const int ActCtrlPnt=smls->ActCtrlPnt;

		g_viewMatrix=g_scene->getViewMatrix();
		g_pointer.translation=selPart->matrix * g_scene->getViewMatrix() * selPart->coord[smls->ActCtrlPnt];

		M_Art &partList=smls->partList;
		for(int k=0;k<partList.length;k++){

			Art* part=partList[k];
			M_Triangle& triangle = part->triangle;

			if(part->show){
				for(int i=triangle.getTriangleLength(); (i-=3)>=0; ){
					if(part->checkPointInTriangleForProjectVertex(perspective, i)){
						
						int a = i;
						int b = i+1;
						int c = i+2;

						int ia=triangle.refIndex(a);
						int ib=triangle.refIndex(b);
						int ic=triangle.refIndex(c);

						Art* pa=triangle.refPart(a);
						Art* pb=triangle.refPart(b);
						Art* pc=triangle.refPart(c);
						if(selPart == part){
							if(selTri == i ||
								pa==selPart && ia == ActCtrlPnt ||
								pb==selPart && ib == ActCtrlPnt ||
								pc==selPart && ic == ActCtrlPnt
								){
								continue;
							}
						}

						Vec3f vecA = pa->matrix * g_scene->getViewMatrix() * pa->coord.refValue()[ia];
						Vec3f vecB = pb->matrix * g_scene->getViewMatrix() * pb->coord.refValue()[ib];
						Vec3f vecC = pc->matrix * g_scene->getViewMatrix() * pc->coord.refValue()[ic];


						Vec3f v[3]={vecA, vecB, vecC};

						Vec3f clickPos;
						if(getTriangleHitPos(clickPos, g_storeClickPos, v)){
							ImmediatePart* part=dynamic_cast<ImmediatePart*>(selPart);

							Vec3f newPt = g_scene->getIViewMatrix() * part->imatrix * clickPos;

							M_Vec3f& coord=part->coord.refValue();

							smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
							coord[smls->ActCtrlPnt]=newPt;

							makePointerEqualIndexLocation();
							smls->convertToRenderFormat();

							BackUp::monostable.beginCycle();
							return;
						}

					}
				}
			}
		}

	}
	else{
		needPartMsg();
	}
};
*/

void  projectSingleVertexOntoPoly(){

	if(Art* selPart=dynamic_cast<Art*>(g_selNode.getRaw())){

		const bool perspective=g_perspective;
		Seamless* smls=selPart->smls;
		smls->catMatrix();
		smls->catIMatrix();
		const int selTri = smls->ActTri * 3;
		const int ActCtrlPnt=smls->ActCtrlPnt;
		Vec3f clickPos;

		g_viewMatrix=g_scene->getViewMatrix();
		g_pointer.translation = selPart->matrix * g_scene->getViewMatrix() * selPart->coord[smls->ActCtrlPnt];

		M_Art &partList=smls->partList;
		for(int k=0;k<partList.length;k++){

			Art* part=partList[k];
			M_Triangle& triangle = part->triangle;

			if(part->show){
				for(int i = triangle.getTriangleLength(); (i-=3)>=0; ){
					if(false == (selPart == part && i == selTri)){
						if(part->checkPointInTriangleForProjectVertex(clickPos, perspective, i)){
							ImmediatePart* part = dynamic_cast<ImmediatePart*>(selPart);

							Vec3f newPt = g_scene->getIViewMatrix() * part->imatrix * clickPos;

							M_Vec3f& coord = part->coord.refValue();

							smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
							coord[smls->ActCtrlPnt]=newPt;

							makePointerEqualIndexLocation();
							smls->convertToRenderFormat();

							BackUp::monostable.beginCycle();
							return;
						}
					}
				}
			}
		}

	}
	else{
		needPartMsg();
	}
};


void  tripleSplitTriangleImmediate(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		part=smls->actTriPart;

     	part->tripleSplitTriangle(smls->ActTri*3);

		makePointerEqualIndexLocation();
		smls->convertToRenderFormat();

		BackUp::monostable.beginCycle();
	}
	else{
		needPartMsg();
	}
}


Art* findTriangleJoinedBy2Vertices(const Art* triPart, int triIndex, int& otherTriIndex, const Art* selPartA, const int selIdxA, const Art* selPartB, const int selIdxB, int& notConected, M_<S_VertexIndex>& triList){
	for(int k = triList.length; --k>=0; ){
		Art* part = triList[k].p;
		int tri = triList[k].i;
		if(triPart == part && triIndex == tri){
			continue;
		}
		S_Triangle& triangle = part->triangle[tri];
		if(	triangle.refPart(0) == selPartA && triangle.refIndex(0) == selIdxA){
			if(	triangle.refPart(1) == selPartB && triangle.refIndex(1) == selIdxB){
				otherTriIndex = tri;
				notConected = 2;
				return part;
			}
			if(	triangle.refPart(2) == selPartB && triangle.refIndex(2) == selIdxB){
				otherTriIndex = tri;
				notConected = 1;
				return part;
			}
		}
		if(	triangle.refPart(1) == selPartA && triangle.refIndex(1) == selIdxA){
			if(	triangle.refPart(0) == selPartB && triangle.refIndex(0) == selIdxB){
				otherTriIndex = tri;
				notConected = 2;
				return part;
			}
			if( triangle.refPart(2) == selPartB && triangle.refIndex(2) == selIdxB){
				otherTriIndex = tri;
				notConected = 0;
				return part;
			}
		}
		if(	triangle.refPart(2) == selPartA && triangle.refIndex(2) == selIdxA){
			if(	triangle.refPart(0) == selPartB && triangle.refIndex(0) == selIdxB){
				otherTriIndex = tri;
				notConected = 1;
				return part;
			}
			if(	triangle.refPart(1) == selPartB && triangle.refIndex(1) == selIdxB){
				otherTriIndex = tri;
				notConected = 0;
				return part;
			}
		}
	}
	return 0;
}

Art* findTriangleJoinedBy2Vertices(const Art* triPart, int triIndex, int& otherTriIndex, const Art* selPartA, const int selIdxA, const Art* selPartB, const int selIdxB, int& notConected){
	M_Art& partList = selPartA->smls->partList;
	for(int k = partList.length; --k>=0; ){
		Art* part = partList[k];
		M_Triangle& triangle = part->triangle;
		for(int i = triangle.getTriangleLength(); (i-=3)>=0;){
			if(triPart == part && triIndex == i){
				continue;
			}
			if(	triangle.refPart(i+0) == selPartA && triangle.refIndex(i+0) == selIdxA){
				if(	triangle.refPart(i+1) == selPartB && triangle.refIndex(i+1) == selIdxB){
					otherTriIndex = i / 3;
					notConected = 2;
					return part;
				}
				if(	triangle.refPart(i+2) == selPartB && triangle.refIndex(i+2) == selIdxB){
					otherTriIndex = i / 3;
					notConected = 1;
					return part;
				}
			}
			if(	triangle.refPart(i+1) == selPartA && triangle.refIndex(i+1) == selIdxA){
				if(	triangle.refPart(i+0) == selPartB && triangle.refIndex(i+0) == selIdxB){
					otherTriIndex = i / 3;
					notConected = 2;
					return part;
				}
				if( triangle.refPart(i+2) == selPartB && triangle.refIndex(i+2) == selIdxB){
					otherTriIndex = i / 3;
					notConected = 0;
					return part;
				}
			}
			if(	triangle.refPart(i+2) == selPartA && triangle.refIndex(i+2) == selIdxA){
				if(	triangle.refPart(i+0) == selPartB && triangle.refIndex(i+0) == selIdxB){
					otherTriIndex = i / 3;
					notConected = 1;
					return part;
				}
				if(	triangle.refPart(i+1) == selPartB && triangle.refIndex(i+1) == selIdxB){
					otherTriIndex = i / 3;
					notConected = 0;
					return part;
				}
			}
		}
	}
	return 0;
}
Art* findTriangleJoinedBy2Vertices(const Art* selPartA, const int selIdxA, const Art* selPartB, const int selIdxB, int& triIndex, int& triCorner){
	M_Art& partList = selPartA->smls->partList;
	for(int k = partList.length; --k>=0; ){
		Art* part = partList[k];
		for(int tri = part->triangle.length; --tri>=0;){
			S_Triangle& triangle = part->triangle[tri];
			if(	triangle.refPart(0) == selPartA && triangle.refIndex(0) == selIdxA){
				if(	triangle.refPart(1) == selPartB && triangle.refIndex(1) == selIdxB){
					triIndex = tri;
					triCorner = 0;
					return part;
				}
			}
			if(	triangle.refPart(1) == selPartA && triangle.refIndex(1) == selIdxA){
				if(	triangle.refPart(2) == selPartB && triangle.refIndex(2) == selIdxB){
					triIndex = tri;
					triCorner = 1;
					return part;
				}
			}
			if(	triangle.refPart(2) == selPartA && triangle.refIndex(2) == selIdxA){
				if(	triangle.refPart(0) == selPartB && triangle.refIndex(0) == selIdxB){
					triIndex = tri;
					triCorner = 2;
					return part;
				}
			}
		}
	}
	return 0;
}

void genTriangles(Art* part, M_<S_VertexIndex>& triList){
	if(part->workEffect==0)return;
	part->deleteGeometry();
	Seamless* smls = part->smls;
	Effect* effect=part->workEffect;
	int effectLength=effect->getEffectLength();
	int triLen = triList.length;
	M_Color3f& color = dynamic_cast<ColorEffect*>(effect)->color.refValue();
	color.length=triLen*3;
	for(int i = color.length; --i>=0;){
		color[i] = Color3f(0,1,1);
	}

	M_Vec3f& coord = part->coord.refValue();
	coord.length = triLen * 3;
	M_Triangle& m_triangle = part->triangle;
	m_triangle.length = triLen;

	for(int i = 0; i < triLen; i++){
		int t = triList[i].i;
		S_Triangle tri = triList[i].p->triangle[t];
		coord[i*3+0] = tri.pa->coord[tri.ia];
		coord[i*3+1] = tri.pb->coord[tri.ib];
		coord[i*3+2] = tri.pc->coord[tri.ic];

		m_triangle[i]=S_Triangle(part, i*3+0, i*3+0, part, i*3+1, i*3+1, part, i*3+2, i*3+2);
	}

	part->relevantPart=M_Art(part);
	smls->process();
}
bool triIntersectsRect(Vec2f a, Vec2f b, Vec2f c);
Vec2f convertPointToScreen2d(Vec3f point);
bool isRectCornerInTriangle(const Vec2f* tri);

void colorVertex(Art* part, int crdIdx){
	Seamless* smls = part->smls;
	M_Art& partList = smls->partList;
	for(int k = partList.length; --k>=0;){
		Art* p = partList[k];
		M_Triangle& triangle = p->triangle;

		for(int i = triangle.getTriangleLength(); --i >= 0;){
			int crdIdxB = triangle.refIndex(i);
			Art* partB = triangle.refPart(i);
			if(partB == part && crdIdxB == crdIdx){
				if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(partB->workEffect)){
					M_Color3f& color = colorEffect->color;
					color[triangle.refTexIndex(i)] = g_palette->color;
				}
			}
		}
	}
}

void applyTriangleColor(Seamless* smls);
void applyCleanEdgeColor(Seamless* smls);
void selectMultipleTriangles(Seamless* smls);
void applyCPOwnership(Art* part);


void NurbsPatch::selectControlPointsBackfaceIgnore(M_<NurbsPatch*>& contigList, const M_Bool& m_pointIsVisible){
	int uDim = this->dim.x;
	int vDim = this->dim.y;

	if(cage == false){
		return;
	}

	if(showAll){
		M_Vec3f& dpoint = this->dpoint.refValue();
		int len = dpoint.length;
		for(int i = 0; i < len; i++){
			if(m_pointIsVisible[fromPoint + i]){
				if(MultiSelect::isInBox(dpoint[i])){
					contigList[fromPoint + i] = this;
				}
			}
		}
	}
	else{
		if(uJoinEnds){
			if(isCtrlPolyRef){
				for(int u = 1; u < uDim - 1; u += 2){
					for(int v = 1; v < vDim - 1; v++){
						int i = v * uDim + u;
						if(m_pointIsVisible[fromPoint + i]){
							if(MultiSelect::isInBox(dpoint[i])){
								contigList[fromPoint + i] = this;
							}
						}
					}
				}
			}
			else{
				if(vDim == 1){
					for(int u = 1; u < uDim - 1; u++){
						if(m_pointIsVisible[fromPoint + u]){
							if(MultiSelect::isInBox(dpoint[u])){
								contigList[fromPoint + u] = this;
							}
						}
					}
				}
				else{
					for(int u = 1; u < uDim; u += 2){
						for(int v = 0; v < vDim; v++){
							int i = v * uDim + u;
							if(m_pointIsVisible[fromPoint + i]){
								if(MultiSelect::isInBox(dpoint[i])){
									contigList[fromPoint + i] = this;
								}
							}
						}
					}
				}
			}
		}
		else{
			if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
				if(isCtrlPolyRef){
					for(int v = 1; v < vDim - 1; v += 2){
						for(int u = 1; u < uDim - 1; u += 2){
							int i = v * uDim + u;
							if(m_pointIsVisible[fromPoint + i]){
								if(MultiSelect::isInBox(dpoint[i])){
									contigList[fromPoint + i] = this;
								}
							}
						}
					}
				}
				else{
					if(vDim == 1){
						for(int u = 0; u < uDim; u++){
							if(m_pointIsVisible[fromPoint + u]){
								if(MultiSelect::isInBox(dpoint[u])){
									contigList[fromPoint + u] = this;
								}
							}
						}
					}
					else{
						for(int v = 0; v < vDim; v += 2){
							for(int u = 0; u < uDim; u += 2){
								int i = v * uDim + u;
								if(m_pointIsVisible[fromPoint + i]){
									if(MultiSelect::isInBox(dpoint[i])){
										contigList[fromPoint + i] = this;
									}
								}
								if(u == 0 || u == uDim - 2){
									u--;
								}
							}
							if(v == 0 || v == vDim - 2){
								v--;
							}
						}
					}
				}
			}
			else{
				if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
					if(isCtrlPolyRef){
						for(int v = 1; v < vDim - 1; v++){
							for(int u = 1; u < uDim - 1; u += 2){
								int i = v * uDim + u;
								if(m_pointIsVisible[fromPoint + i]){
									if(MultiSelect::isInBox(dpoint[i])){
										contigList[fromPoint + i] = this;
									}
								}
							}
						}
					}
					else{
						if(uDim == 1){
							for(int v = 0; v < vDim; v++){
								if(m_pointIsVisible[fromPoint + v]){
									if(MultiSelect::isInBox(dpoint[v])){
										contigList[fromPoint + v] = this;
									}
								}
							}
						}
						else{
							for(int v = 0; v < vDim; v++){
								for(int u = 1; u < uDim; u +=2){
									int i = v * uDim + u;
									if(m_pointIsVisible[fromPoint + i]){
										if(MultiSelect::isInBox(dpoint[i])){
											contigList[fromPoint + i] = this;
										}
									}
								}
							}
						}
					}
				}
				else{
					if(isCtrlPolyRef){
						for(int v = 1; v < vDim - 1; v++){
							for(int u = 1; u < uDim - 1; u++){
								int i = v * uDim + u;
								if(m_pointIsVisible[fromPoint + i]){
									if(MultiSelect::isInBox(dpoint[i])){
										contigList[fromPoint + i] = this;
									}
								}
							}
						}
					}
					else {
						for(int v = 0; v < vDim; v++){
							for(int u = 0; u < uDim; u++){
								int i = v * uDim + u;
								if(m_pointIsVisible[fromPoint + i]){
									if(MultiSelect::isInBox(dpoint[i])){
										contigList[fromPoint + i] = this;
									}
								}
							}
						}
					}
				}
			}
		}
	}
}

bool NurbsPatch::objectIsInBox() const{
	int uDim = this->dim.x;
	int vDim = this->dim.y;

	if(cage == false){
		return false;
	}

	if(showAll){
		const M_Vec3f& dpoint = this->dpoint;
		int len = dpoint.length;
		for(int i = 0; i < len; i++){
			if(MultiSelect::isInBox(dpoint[i])){
				return true;
			}
		}
	}
	else{
		if(uJoinEnds){
			if(isCtrlPolyRef){
				for(int u = 1; u < uDim - 1; u += 2){
					for(int v = 1; v < vDim - 1; v++){
						int i = v * uDim + u;
						if(MultiSelect::isInBox(dpoint[i])){
							return true;
						}
					}
				}
			}
			else{
				for(int u = 1; u < uDim; u += 2){
					for(int v=0; v < vDim; v++){
						int i = v * uDim + u;
						if(MultiSelect::isInBox(dpoint[i])){
							return true;
						}
					}
				}
			}
		}
		else{
			if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
				if(isCtrlPolyRef){
					for(int v = 1; v < vDim - 1; v += 2){
						for(int u = 1; u < uDim - 1; u += 2){
							int i = v * uDim + u;
							if(MultiSelect::isInBox(dpoint[i])){
								return true;
							}
						}
					}
				}
				else{
					for(int v = 0; v < vDim; v += 2){
						for(int u = 0; u < uDim; u += 2){
							int i = v * uDim + u;
							if(MultiSelect::isInBox(dpoint[i])){
								return true;
							}
							if(u == 0 || u == uDim - 2){
								u--;
							}
						}
						if(v == 0 || v == vDim - 2){
							v--;
						}
					}
				}
			}
			else{
				if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
					if(isCtrlPolyRef){
						for(int v = 1; v < vDim - 1; v++){
							for(int u = 1; u < uDim - 1; u += 2){
								int i = v * uDim+u;
								if(MultiSelect::isInBox(dpoint[i])){
									return true;
								}
							}
						}
					}
					else{
						for(int v = 0; v < vDim; v++){
							for(int u = 1; u < uDim; u +=2){
								int i = v * uDim + u;
								if(MultiSelect::isInBox(dpoint[i])){
									return true;
								}
							}
						}
					}
				}
				else{
					if(isCtrlPolyRef){
						for(int v = 1; v < vDim - 1; v++){
							for(int u = 1; u < uDim - 1; u++){
								int i = v * uDim + u;
								if(MultiSelect::isInBox(dpoint[i])){
									return true;
								}
							}
						}
					}
					else {
						for(int v = 0; v < vDim; v++){
							for(int u = 0; u < uDim; u++){
								int i = v * uDim + u;
								if(MultiSelect::isInBox(dpoint[i])){
									return true;
								}
							}
						}
					}
				}
			}
		}
	}
	return false;
}

void Seamless::unselectCtrlPolys(){
	g_viewMatrix = g_scene->getViewMatrix();
	int d = 0;
	for(int s = 0; s < MultiSelect::ctrlPolySelList.length; s++){
		const int i = MultiSelect::ctrlPolySelList[s];
		Vec3f va3f = ctrlPolyPoint[i].p->dpoint[ctrlPolyPoint[i].i];
		int len = ctrlPolyPoint[i].len;
		bool selPoly = false;
		for(int j = 1; j < len - 1; j++){
			Vec3f vb3f = ctrlPolyPoint[i + j].p->dpoint[ctrlPolyPoint[i + j].i];
			Vec3f vc3f = ctrlPolyPoint[i + j + 1].p->dpoint[ctrlPolyPoint[i + j + 1].i];
		
			Vec2f va = convertPointToScreen2d(va3f);
			Vec2f vb = convertPointToScreen2d(vb3f);
			Vec2f vc = convertPointToScreen2d(vc3f);

			if(g_scene->mode & BACKFACE_IGNORE_MODE){
				if(!backFace(va, vb, vc)){
					break;
				}
			}

			Vec2f tri2d[3] = {va, vb, vc};

			if(MultiSelect::isInBox(va) || MultiSelect::isInBox(vb) || MultiSelect::isInBox(vc) || triIntersectsRect(va, vb, vc) || isRectCornerInTriangle(tri2d)){
				selPoly = true;
				break;
			}
		}
		if(selPoly == false){
			MultiSelect::ctrlPolySelList[d++] = i;
		}
	}
	MultiSelect::ctrlPolySelList.length = d;
}

void Seamless::selectControlPoints(){
	MultiSelect::processRectangle();
	const bool perspective = g_perspective;

	if(g_rectangleSelectPolyReady){
		g_rectangleSelectPointReady = false;
		g_rectangleUnselectPointReady = false;
		g_rectangleSelectPolyReady = false;
		changeModellingCursor(g_crossCursor);
		if((r_controlKeyState()) || InputEvent::r_ctrlDown){
			unselectCtrlPolys();
		}
		else{
			if(g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE){
				selectBoxModelingPatchPolys();
			}
			else{
				selectCtrlPolys();
			}
		}
		return;
	}
	g_rectangleSelectPointReady = false;
	g_rectangleSelectPolyReady = false;
	changeModellingCursor(g_crossCursor);
	M_<NurbsPatch*> contigList;  //make sure we dont add points twice by making contig list
	createSelContigList(contigList);

	if((controlKeyState()) || InputEvent::l_ctrlDown || g_rectangleUnselectPointReady){
		g_rectangleUnselectPointReady = false;
		int d = 0;
		for(int s = 0; s < MultiSelect::pointList.length; s++){
			NurbsPatch* patch = MultiSelect::pointList[s].p;
			int idx = MultiSelect::pointList[s].i;
			if(MultiSelect::isInBox(patch->dpoint[idx])){
				contigList[patch->fromPoint + idx] = 0;
			}
		}
	}
	else{
		if(g_transformCP->selectObject){
			for(int k = 0; k < cpNodeList.length; ){
				Build* bld = cpNodeList[k]->container;
				int remK = k;
				for(int i = 0; i < bld->numOfPatches(); i++){
					NurbsPatch* patch = cpNodeList[k++];
					if(patch->objectIsInBox()){
						k = remK;
						bld->remTransform();
						for(int i = 0; i < bld->numOfPatches(); i++){
							NurbsPatch* patch = cpNodeList[k++];
							patch->selectControlPointsNoBoxTest(contigList);
						}
						break;
					}
				}
			}
		}
		else{
			for(int k = 0; k < cpNodeList.length; k++){
				NurbsPatch* patch = cpNodeList[k];
				if(dynamic_cast<Strand*>(patch)){
					if((g_scene->mode & SHOW_STRAND_CTRL_LINE_MODE) == false){
						continue;
					}
				}
				{
					if(g_curvedCageMode){
						patch->selectControlPoints(contigList, perspective);
						
						M_Vec3f& point = dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;

						for(int i = 0; i < ctrlPolyPoint.length;){
							int ctrlPoly = i;
							int len = ctrlPolyPoint[ctrlPoly].len;
							for(int k = len; --k >= 0;){
								NurbsPatch* patch = ctrlPolyPoint[i].p;
								if(patch->cage){
									int actIdx = ctrlPolyPoint[i].i;
									int visIdx = patch->convertActualToVisual(actIdx, patch->dim.x, patch->dim.y);
									int lookUpIndex = g_pointLookup[patch->plidx];
									Vec3f pnt = point[lookUpIndex + visIdx];
									if(patch->cage && MultiSelect::isInBox(pnt)){
										contigList[patch->fromPoint + actIdx] = patch;
									}
									i++;
								}
							}
							i = getNextCtrlPoly(ctrlPoly);
						}
					}
					else{
						if(g_scene->mode & BACKFACE_IGNORE_MODE){
							patch->selectControlPointsBackfaceIgnore(contigList, g_pointIsVisible);

							for(int i = 0; i < ctrlPolyPoint.length;){
								int ctrlPoly = i;
								int len = ctrlPolyPoint[ctrlPoly].len;
								for(int k = len; --k >= 0;){
									NurbsPatch* patch = ctrlPolyPoint[i].p;
									int idx = ctrlPolyPoint[i].i;
									if(g_pointIsVisible[patch->fromPoint + idx]){
										if(patch->cage && MultiSelect::isInBox(patch->dpoint[idx])){
											contigList[patch->fromPoint + idx] = patch;
										}
									}
									i++;
								}
								i = getNextCtrlPoly(ctrlPoly);
							}
						}
						else{
							patch->selectControlPointsStraightCage(contigList, perspective);

							for(int i = 0; i < ctrlPolyPoint.length;){
								int ctrlPoly = i;
								int len = ctrlPolyPoint[ctrlPoly].len;
								for(int k = len; --k >= 0;){
									NurbsPatch* patch = ctrlPolyPoint[i].p;
									int idx = ctrlPolyPoint[i].i;
									if(patch->cage && MultiSelect::isInBox(patch->dpoint[idx])){
										contigList[patch->fromPoint + idx] = patch;
									}
									i++;
								}
								i = getNextCtrlPoly(ctrlPoly);
							}
						}
					}
				}
			}
		}
	}
	MultiSelect::pointList.length = 0;
	for(int k = 0; k < pointTotal; k++){
		if(NurbsPatch* patch = contigList[k]){
			MultiSelect::pointList += ControlPoint(patch, k - patch->fromPoint);
		}
	}
	g_transformCP->calcPivotPoint();
	g_transformCP->resetRotationLever();
}

void MultiSelect::selectTriangles(Art* part){
	Seamless* smls = part->smls;
	M_Art& partList = smls->partList;

	//if(MultiSelect::accumulate == false){
	//	if(g_paletteMode == false){
	//		Art::triList.length = 0;
	//	}
	//	Art::coordList.length = 0;
	//}

	g_viewMatrix = g_scene->getViewMatrix();

	selectRectMode = false;
	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcSelectRectShape->geometry.p);
	dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue().length=0;
	polyLine->coordIndex.refValue().length=0;

	processRectangle();

	if((r_shiftKeyState()) || InputEvent::r_shiftDown){
		if(g_paletteMode){
			applyTriangleColor(smls);
			return;
		}
		selectMultipleTriangles(smls);
	}
	else if((shiftKeyState()) || InputEvent::l_shiftDown){
		if(smls->workAnimCP){
			if(g_colorOwnership){
				applyCPOwnership(part);
				part->houseKeep();
				return;
			}
			else{
				smls->selectControlPoints();
				part->houseKeep();
				return;
			}
		}
		if(g_paletteMode){
			if(triList.length){
				applyCleanEdgeColor(smls);
				g_visualClue = g_vcArtTriBase;
			}
			else{
				applyColor(smls);
				g_visualClue = g_vcArtTriBase;
			}
		}
		else{
			if(g_scene->mode & BACKFACE_IGNORE_MODE){
				//selectFrontFaceVertices(smls);
				selectFrontFaceRenderedVertices(smls);
			}
			else{
				//selectFrontAndBackFaceVertices(smls);
				selectFrontAndBackFaceTransformedVertices(smls);
			}
			if(g_colorOwnership){
				if(part->smls->bnspe){
					uvApplyOwnership(part);
				}
				else{
					applyOwnership(part);
				}
			}
		}
	}
	else{
		return;
	}
	if(smls->animCP == false && Window::animMode()){
		g_visualClue = g_vcArtAnimBase;
	}
	else{
		g_visualClue = g_vcArtTriBase;
	}
}

void unMultiSelTri(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		if(MultiSelect::triList.length){
			Seamless* smls = part->smls;
			for(int k = MultiSelect::triList.length; --k>=0;){
				if(MultiSelect::triList[k].i == smls->ActTri && smls->actTriPart == MultiSelect::triList[k].p){
					MultiSelect::triList.deleteElement(k);
					genTriangles(g_vcArtTriBase->transpSmls->skeleton.refValue()[0], MultiSelect::triList);
					return;
				}
			}
		}
		else{
			msG("You must have multiple triangles selected for this command");
		}
	}
	else{
		needBluePartMsg();
	}
}


void splitTriangleImmediate(Art* part, Art* actTriPart, const int& ActTri, bool complete=false){
	fastFullBuildForAutoIn(true);

	Seamless* smls=part->smls;

	const int ActTriCorner = smls->ActTriCorner;
	
	int idxRed = ActTri * 3 + ActTriCorner;
	int idxGreen = ActTri * 3 + (ActTriCorner + 1) % 3;
	int notConectedA = (ActTriCorner + 2) % 3;
	const int cRed = actTriPart->triangle.refIndex(idxRed);
	const int cGreen = actTriPart->triangle.refIndex(idxGreen);
	Art* red = actTriPart->triangle.refPart(idxRed);
	Art* green = actTriPart->triangle.refPart(idxGreen);
	int otherTriIndex = 0;
	int notConectedB = 0;
	
	Art* otherTriPart = findTriangleJoinedBy2Vertices(actTriPart, ActTri*3, otherTriIndex, red, cRed, green, cGreen, notConectedB);
	if(smls->bnspe){
		if(otherTriPart){
			actTriPart->uvSplit2Triangles(ActTri, ActTriCorner, otherTriPart, otherTriIndex, (notConectedB+2)%3);
		}
		else{
			actTriPart->uvSplitTriangle(ActTri, smls->ActTriCorner);
		}
	}
	else{
		smls->catMatrix();
		smls->catIMatrix();
		if(otherTriPart){
			actTriPart->split2Triangles(ActTri, ActTriCorner, otherTriPart, otherTriIndex, (notConectedB+2)%3);
		}
		else{
			actTriPart->splitTriangle(ActTri, smls->ActTriCorner);
		}
	}
	if(complete){

	//	if(smls->build.length && red == green){
		if(red == green){
			int to = cGreen;
			if(cRed > cGreen){
				to = cRed;
				smls->ActCtrlPnt++;
			}
			to++;
			for(int i = red->coord.length; --i>=to;){
				if(smls->bnspe){
					red->uvSwapVertex(i, i - 1);
				}
				else{
					red->swapVertex(i, i - 1);
				}
			}
		}
	}

	//smls->doBuild();
	makePointerEqualIndexLocation();
	smls->convertToRenderFormat();
	BackUp::monostable.beginCycle();
}

void swapDivide(Art* part, Art* actTriPart, const int ActTri, const int ActCorner);
class Edge{
public:
	Edge():
		part(0),
		x(0),
		y(0)
	{}
	Edge(Art* part, int x, int y):
		part(part),
		x(x),
		y(y)
	{
	}
	Art* part;
	int x;
	int y;
};

class TwoPoint{
public:
	TwoPoint():
		partA(0),
		idxA(0),
		partB(0),
		idxB(0)
	{}
	TwoPoint(Art* partA, int idxA, Art* partB, int idxB):
		partA(partA),
		idxA(idxA),
		partB(partB),
		idxB(idxB)
	{
	}
	Art* partA;
	int idxA;
	Art* partB;
	int idxB;
};

inline bool splitFan(Art* part, Art* actTriPart, M_<Edge>& edges, M_<S_VertexIndex>& triList){
	Seamless* smls = part->smls;

	for(int i = triList.length; --i>=0;){
		Art* triPart = triList[i].p;

		Art* vertPartA = triPart->triangle[triList[i].i].pa;
		int crdIdxA = triPart->triangle[triList[i].i].ia;

		Art* vertPartB = triPart->triangle[triList[i].i].pb;
		int crdIdxB = triPart->triangle[triList[i].i].ib;

		Art* vertPartC = triPart->triangle[triList[i].i].pc;
		int crdIdxC = triPart->triangle[triList[i].i].ic;

		if( vertPartA == part && crdIdxA == smls->ActCtrlPnt){
			continue;
		}
		if( vertPartB == part && crdIdxB == smls->ActCtrlPnt){
			continue;
		}
		if( vertPartC == part && crdIdxC == smls->ActCtrlPnt){
			continue;
		}
		return false;
	}
	int lastSplitCoordA = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
	int lastSplitCoordB = smls->actTriPart->triangle[smls->ActTri].refIndex((smls->ActTriCorner + 1) % 3);

	int ActTri = smls->ActTri;
	int ActTriCorner = smls->ActTriCorner;
	Art* remActTriPart=actTriPart;


	while(1){
		edges += Edge(smls->actTriPart, smls->ActTri, smls->ActTriCorner);

		int ActTri = smls->ActTri; //triArray[nextTriIdx++];
		const int ActTriCorner = (smls->ActTriCorner + 2) % 3;
		
		int idxRed = ActTri * 3 + ActTriCorner;
		int idxGreen = ActTri * 3 + (ActTriCorner + 1) % 3;
		int notConectedA = (ActTriCorner + 2) % 3;
		const int cRed = actTriPart->triangle.refIndex(idxRed);
		const int cGreen = actTriPart->triangle.refIndex(idxGreen);
		Art* red = actTriPart->triangle.refPart(idxRed);
		Art* green = actTriPart->triangle.refPart(idxGreen);
		int otherTriIndex=0;
		int notConectedB = 0;

		Art* otherTriPart = findTriangleJoinedBy2Vertices(actTriPart, ActTri, otherTriIndex, red, cRed, green, cGreen, notConectedB, MultiSelect::triList);
		if(edges[0].x==otherTriIndex){
			break;
		}
		if(otherTriPart){
			smls->actTriPart = otherTriPart;
			smls->ActTri = otherTriIndex;
			smls->ActTriCorner = (notConectedB + 1) % 3;
			smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);

			lastSplitCoordA = smls->ActCtrlPnt;
			lastSplitCoordB = smls->actTriPart->triangle[smls->ActTri].refIndex((smls->ActTriCorner + 1) % 3);
		}
		else{
			//take care of the end edge
			for(int i = 3; --i>=0;){
				int cornerA = (smls->ActTriCorner + i) % 3;
				int cornerB = (cornerA + 1) % 3;
				int a = smls->actTriPart->triangle[smls->ActTri].refIndex(cornerA);
				int b = smls->actTriPart->triangle[smls->ActTri].refIndex(cornerB);

				if(a != lastSplitCoordA && b != lastSplitCoordA && b != lastSplitCoordB){
					edges += Edge(smls->actTriPart, smls->ActTri, cornerA);
					break;
				}
				else if(b != lastSplitCoordA && a != lastSplitCoordA && a != lastSplitCoordB){
					edges += Edge(smls->actTriPart, smls->ActTri, cornerB);
					break;
				}
			}
			break;
		}
	}
	int divTri = 0;
	for(int i = 0; i < edges.length; i++){
		smls->ActTri = edges[i].x;
		smls->ActTriCorner = edges[i].y;
		smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
		splitTriangleImmediate(part, actTriPart, smls->ActTri);

		if(i == 0){
			divTri = part->triangle.length-1;
		}
	}
	smls->actTriPart = remActTriPart;
	smls->ActTri = ActTri;
	smls->ActTriCorner = ActTriCorner;
	smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);


	smls->actTriPart = remActTriPart;
	smls->ActTri = divTri;
	smls->ActTriCorner = 0;
	smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);

	swapDivide(part, smls->actTriPart, smls->ActTri, smls->ActTriCorner);
	return true;


	int idxRed = smls->ActTri * 3 + ActTriCorner;
	int idxGreen = smls->ActTri * 3 + (ActTriCorner + 1) % 3;
	int notConectedA = (ActTriCorner + 2) % 3;
	int cRed = remActTriPart->triangle.refIndex(idxRed);
	int cGreen = remActTriPart->triangle.refIndex(idxGreen);
	Art* red = remActTriPart->triangle.refPart(idxRed);
	Art* green = remActTriPart->triangle.refPart(idxGreen);
	int otherTriIndex=0;
	int notConectedB = 0;

	Art* otherTriPart = findTriangleJoinedBy2Vertices(remActTriPart, smls->ActTri, otherTriIndex, red, cRed, green, cGreen, notConectedB);

	smls->actTriPart = otherTriPart;
	smls->ActTri = otherTriIndex;
	smls->ActTriCorner = (notConectedB + 1 + 2) % 3;
	smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
////////////////
	ActTriCorner = smls->ActTriCorner;

	idxRed = smls->ActTri * 3 + ActTriCorner;
	idxGreen = smls->ActTri * 3 + (ActTriCorner + 1) % 3;
	notConectedA = (ActTriCorner + 2) % 3;
	cRed = remActTriPart->triangle.refIndex(idxRed);
	cGreen = remActTriPart->triangle.refIndex(idxGreen);
	red = remActTriPart->triangle.refPart(idxRed);
	green = remActTriPart->triangle.refPart(idxGreen);
	otherTriIndex=0;
	notConectedB = 0;

	otherTriPart = findTriangleJoinedBy2Vertices(remActTriPart, smls->ActTri, otherTriIndex, red, cRed, green, cGreen, notConectedB);

	smls->actTriPart = otherTriPart;
	smls->ActTri = otherTriIndex;
	smls->ActTriCorner = (notConectedB + 1 + 2) % 3;
	smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);


	swapDivide(part, smls->actTriPart, smls->ActTri, smls->ActTriCorner);

	return true;	
}
void splitBand(Art* part, Art* _actTriPart){
	//M_Int triArray = M_Int(6, 15, 14, 23, 7, 16, 22, 17, 25, 24, 26, 27);
	//M_Art partArray = M_Art(actTriPart, actTriPart, actTriPart, actTriPart, actTriPart, actTriPart, actTriPart, actTriPart, actTriPart, actTriPart, actTriPart, actTriPart);

	M_<Edge> edges;
	M_<TwoPoint> divIdx;
	M_Art divPartA;
	M_Art divPartB;


	fastFullBuildForAutoIn(true);

	Seamless* smls = part->smls;
	smls->actTriPart = _actTriPart;

	if(splitFan(part, smls->actTriPart, edges, MultiSelect::triList)){
		goto here;
	}


	int lastSplitCoordA = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
	int lastSplitCoordB = smls->actTriPart->triangle[smls->ActTri].refIndex((smls->ActTriCorner + 1) % 3);

	//if(nextTriIdx>=triArray.length){
	//	nextTriIdx=0;
	//}
	//S_Triangle& tri=part->triangle[smls->ActTri];
	//int nsel = 0;
	////for(int i = 0; i < 3; i++){
	////	if(tri.refPart(i) == part && tri.refIndex(i) == cRed){
	////		nsel = (i + 1) % 3;;
	////	}
	////}
	//smls->ActTriCorner=nsel;
	//smls->ActCtrlPnt=tri.refIndex(nsel);

	bool splitCycle = true;
	while(1){
		if(splitCycle){
			edges += Edge(smls->actTriPart, smls->ActTri, smls->ActTriCorner);
			//dLog(String("Tri ") + smls->ActTri + String(" ") + String(smls->ActTriCorner));
		}
		else{
			int next = (smls->ActTriCorner + 1) % 3;
			divIdx += TwoPoint(
				smls->actTriPart->triangle[smls->ActTri].refPart(smls->ActTriCorner),
				smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner),
				smls->actTriPart->triangle[smls->ActTri].refPart(next),
				smls->actTriPart->triangle[smls->ActTri].refIndex(next)
			);
		}
		splitCycle = !splitCycle;

		int ActTri = smls->ActTri; //triArray[nextTriIdx++];
		const int ActTriCorner = (smls->ActTriCorner + 1) % 3;
		
		int idxRed = ActTri * 3 + ActTriCorner;
		int idxGreen = ActTri * 3 + (ActTriCorner + 1) % 3;
		int notConectedA = (ActTriCorner + 2) % 3;
		const int cRed = smls->actTriPart->triangle.refIndex(idxRed);
		const int cGreen = smls->actTriPart->triangle.refIndex(idxGreen);
		Art* red = smls->actTriPart->triangle.refPart(idxRed);
		Art* green = smls->actTriPart->triangle.refPart(idxGreen);
		int otherTriIndex=0;
		int notConectedB = 0;

		Art* otherTriPart = findTriangleJoinedBy2Vertices(smls->actTriPart, ActTri, otherTriIndex, red, cRed, green, cGreen, notConectedB, MultiSelect::triList);

		//smls->catMatrix();
		//smls->catIMatrix();
		if(edges[0].x==otherTriIndex){
			break;
		}
		if(otherTriPart){
	//		actTriPart->split2Triangles(ActTri, ActTriCorner, otherTriPart, otherTriIndex, (notConectedB+2)%3);

			smls->actTriPart = otherTriPart;
			smls->ActTri = otherTriIndex;
			smls->ActTriCorner = (notConectedB + 1) % 3;
			smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);

		}
		else{
			const int ActTriCorner = (smls->ActTriCorner + 2) % 3;
			
			int idxRed = ActTri * 3 + ActTriCorner;
			int idxGreen = ActTri * 3 + (ActTriCorner + 1) % 3;
			int notConectedA = (ActTriCorner + 2) % 3;
			const int cRed = smls->actTriPart->triangle.refIndex(idxRed);
			const int cGreen = smls->actTriPart->triangle.refIndex(idxGreen);
			Art* red = smls->actTriPart->triangle.refPart(idxRed);
			Art* green = smls->actTriPart->triangle.refPart(idxGreen);
			int otherTriIndex=0;
			int notConectedB = 0;

			Art* otherTriPart = findTriangleJoinedBy2Vertices(smls->actTriPart, ActTri, otherTriIndex, red, cRed, green, cGreen, notConectedB, MultiSelect::triList);
			if(edges[0].x==otherTriIndex){
				break;
			}
			if(otherTriPart){
				smls->actTriPart = otherTriPart;
				smls->ActTri = otherTriIndex;
				smls->ActTriCorner = (notConectedB + 1) % 3;
				smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);

				lastSplitCoordA = smls->ActCtrlPnt;
				lastSplitCoordB = smls->actTriPart->triangle[smls->ActTri].refIndex((smls->ActTriCorner + 1) % 3);
			}
			else{
				//take care of the end edge
				for(int i = 3; --i>=0;){
					int cornerA = (smls->ActTriCorner + i) % 3;
					int cornerB = (cornerA + 1) % 3;
					int a = smls->actTriPart->triangle[smls->ActTri].refIndex(cornerA);
					int b = smls->actTriPart->triangle[smls->ActTri].refIndex(cornerB);

					if(a != lastSplitCoordA && b != lastSplitCoordA && b != lastSplitCoordB){
						edges += Edge(smls->actTriPart, smls->ActTri, cornerA);
						break;
					}
					else if(b != lastSplitCoordA && a != lastSplitCoordA && a != lastSplitCoordB){
						edges += Edge(smls->actTriPart, smls->ActTri, cornerB);
						break;
					}
				}
				break;
			}
		}
	}
	for(int i = 0; i < edges.length; i++){
		smls->actTriPart = edges[i].part;
		smls->ActTri = edges[i].x;
		smls->ActTriCorner = edges[i].y;
		smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
		splitTriangleImmediate(dynamic_cast<ImmediatePart*>(smls->actTriPart), smls->actTriPart, smls->ActTri);

		//smls->ActTri = part->triangle.length-1;
		//smls->ActTriCorner = 2;
		//smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
		//swapDivide(part, actTriPart, smls->ActTri);
	}
	for(int i = 0; i < divIdx.length; i++){
	//	smls->ActTri = edges[i].x;
	//	smls->ActTriCorner = edges[i].y;
	//	smls->ActCtrlPnt = smls->actTriPart->triangle[smls->ActTri].refIndex(smls->ActTriCorner);
		int triIndex = 0, triCorner = 0;
		Art* otherPart = findTriangleJoinedBy2Vertices(divIdx[i].partA, divIdx[i].idxA, divIdx[i].partB, divIdx[i].idxB, triIndex, triCorner);
		swapDivide(otherPart, otherPart, triIndex, triCorner);

		//dLog(String("div ") + triIndex + String(" ") + triCorner);
	////	//swapDivide(part, actTriPart, smls->ActTri);
	////	//splitTriangleImmediate(part, actTriPart, smls->ActTri);
	}
here:
	MultiSelect::reset();
	smls->doBuild();
	makePointerEqualIndexLocation();
	smls->convertToRenderFormat();
	BackUp::monostable.beginCycle();
}
void splitBand(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				splitBand(bluePart, bluePart);
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		splitBand(bluePart, smls->actTriPart);
	}
	else{
		needPartMsg();
	}
}
void splitTriangleImmediate(){
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;

		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){

			if(MultiSelect::triList.length){
				splitBand(whitePart, smls->actTriPart);
			}
			else{
				splitTriangleImmediate(whitePart, smls->actTriPart, smls->ActTri, true);
			}
			return;
		}

		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				if(MultiSelect::triList.length){
					splitBand(bluePart, smls->actTriPart);
				}
				else{
					splitTriangleImmediate(bluePart, bluePart, smls->ActTri-stem->beginTriOffset, true);
				}
			}
			else{
				int ActTri = smls->ActTri;
				int ActTriCorner = smls->ActTriCorner;
				if(stem->mirror){
					if(	(stem->mirrorStem && stem->insideOut) ||
						(stem->mirrorStem == 0 && ActTri >= stem->coordIndex.length / 3)
						){
						ActTriCorner = abs(ActTriCorner + 1) % 3;
					}
				}
				if(0 > getRelIndex(stem, ActTri, ActTriCorner)){
					return;
				}

				bluePart = selectPolyMapPart(whitePart);
				stem->copyPolyMapToPart(bluePart);
				bluePart->smls->ActTriCorner = ActTriCorner;
				splitTriangleImmediate(bluePart, bluePart, ActTri / 3);
				stem->copyPartToPolyMap(bluePart);
				BackUp::monostable.beginCycle();
				commonEndVertexBuild();
			}
		}
	}
	else if(Subdivision* subdivision = dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->extrudePoly();
	}
	else if(dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(g_scene->mode & HAIR_REGION_MODE){
			Lock::splitRegionMode = true;
			Lock::splitRegionPoints.length = 0;

		}
	}
	else{
		needPartMsg();
	}
}
void lightenVertex(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
			M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
			Vec3f vec=Vec3f(color[smls->ActCtrlPnt].r,color[smls->ActCtrlPnt].g,color[smls->ActCtrlPnt].b);
			vec=vec+vec*.005;
			if(vec.x>1)vec.x=1;
			if(vec.y>1)vec.y=1;
			if(vec.z>1)vec.z=1;
			color[smls->ActCtrlPnt]=Color3f(vec.x,vec.y,vec.z);
			commonEndVertexChange();
		}
	}
	else{
		needPartMsg();
	}
}
void darkenVertex(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
			M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
			Vec3f vec=Vec3f(color[smls->ActCtrlPnt].r,color[smls->ActCtrlPnt].g,color[smls->ActCtrlPnt].b);
			vec=vec-vec*.005;
			if(vec.x<0)vec.x=0;
			if(vec.y<0)vec.y=0;
			if(vec.z<0)vec.z=0;
			color[smls->ActCtrlPnt]=Color3f(vec.x,vec.y,vec.z);
			commonEndVertexChange();
		}
	}
	else{
		needPartMsg();
	}
}
void reverseIndices(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		part->insideOut();
		makePointerEqualIndexLocation();
		part->smls->convertToRenderFormat();
		BackUp::monostable.beginCycle();
	}
	else{
		needPartMsg();
	}
}

void doBuild(){
	if(Oscillator* osc = dynamic_cast<Oscillator*>(g_selNode.getRaw())){
		for(int k = 0; k < osc->filter.length; k++){
			if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(osc->filter[k].p)){
				patch->updateVCoord(*osc);
				patch->processFields();
				//patch->doBuild();
				//patch->smls->ctrf = true;
			}
		}
	}
	if(HarmonicPatch* patch = dynamic_cast<HarmonicPatch*>(g_selNode.getRaw())){
		int selLine = g_stw->selLine;
		int parentLine = getParent(selLine);
		if(Oscillator* osc = dynamic_cast<Oscillator*>(g_sceneTreeList[parentLine].node.p)){
			patch->updateVCoord(*osc);
			patch->processFields();
			patch->houseKeep();
		}
	}
	else{
		searchDoBuild(g_scene->children.refValue());
		makePointerEqualIndexLocation();
		Window::paintAllControlPanels();
		if(NurbsPatch* nurbsPatch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
			nurbsPatch->houseKeep();
		}
	}
}

void resetRotations(){
	if(Window::chatMode()){
		return;
	}
	else if(dynamic_cast<Lock*>(g_selNode.getRaw())){
		toggleHairRegionMode();
	}
	else{
		g_scene->resetRotations();
		Window::paintAllControlPanels();
	}
}


void zeroVetexX(){
	if(Window::chatMode())return;
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
			}
			else{
				stem->coord.refValue()[smls->ActCtrlPnt-stem->beginCoordOffset].x=.5;
				commonEndVertexBuild();
			}
		}
	}
	else if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls=part->smls;

		part->coord.refValue()[smls->ActCtrlPnt].x=0;
		smls->convertToRenderFormat();
		makePointerEqualIndexLocation();
		BackUp::monostable.beginCycle();
	}
	else{
		needPartMsg();
	}
}
void BackUp::unDo(){
	if(Window::chatMode())return;
	if(monostable.state())return;


	if(resetTemporaryModes()){
		houseKeepVisualClue();
		return;
	}

	bool accumulate = MultiSelect::accumulate;
	MultiSelect::accumulate = false;
	MultiSelect::reset();
	MultiSelect::accumulate = accumulate;


	static String filename;

	int storeMode; float storeZoom; Vec3f storeTranslation; Rotation storeRotation; g_scene->storeModeZoomRotTran(storeMode, storeZoom, storeTranslation, storeRotation);

	if(backedUpFileCntr < 1)return;
	filename = g_undoDir;
	filename += ourBackUpName;
	filename += --backedUpFileCntr;
	filename += ".smls";

	initCompile();
	Browser::createSceneFromFile(filename);
	g_scene->restoreModeZoomRotTran(storeMode, storeZoom, storeTranslation, storeRotation);

	commonUndoRedo();
}

void BackUp::reDo(){
	resetTemporaryModes();
	if(Window::chatMode())return;
	if(monostable.state())return;
	static String filename;

	int storeMode; float storeZoom; Vec3f storeTranslation; Rotation storeRotation; g_scene->storeModeZoomRotTran(storeMode, storeZoom, storeTranslation, storeRotation);

	if(backedUpFileCntr>=backedUpFileCntrEnd)return;
	filename=g_undoDir;
	filename+=ourBackUpName;
	filename+=++backedUpFileCntr;
	filename+=".smls";

	initCompile();
	Browser::createSceneFromFile(filename);

	g_scene->restoreModeZoomRotTran(storeMode, storeZoom, storeTranslation, storeRotation);

	commonUndoRedo();
}

void unDo(){
	BackUp::unDo();
}
void reDo(){
	BackUp::reDo();
}
void cut(){
	Window::activeWindow->copyTextCellToClipboard(true);
}



void remember(){
	Window::activeWindow->copyTextCellToClipboard();

	g_stw->copyLine = g_stw->selLine;
	g_copySibCount=g_selCount;
	g_copyAllCount=g_selCount.all;
	M_Int* indexPtr=g_selNode->getIndexPtr();
	if(indexPtr!=nullPtr){
		g_copyIndex=*indexPtr;
	}
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		remPJ = part;
		Seamless* smls = part->smls;
		smls->remActCtrlPnt = smls->ActCtrlPnt;
		smls->remActTriPart = smls->actTriPart;
		smls->remActTri = smls->ActTri;
		smls->remActTriCorner = smls->ActTriCorner;
		if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
			int triIdx = smls->ActTri * 3 + smls->ActTriCorner;
			if(triIdx < part->triangle.getTriangleLength()){
				int colorIdx = part->triangle.refTexIndex(triIdx);
				if(colorIdx < colorEffect->color.length){
					g_palette->color = colorEffect->color[colorIdx];
					if(PaletteCP* paletteCP = paletteCPAllreadyOpen()){
						paletteCP->paint();
					}
				}
			}
		}
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		if(dynamic_cast<Lock*>(patch)){
			if(g_scene->mode & HAIR_REGION_MODE){
				if(Lock::selBorderLock){
					Lock::copyBorderList += Lock::selBorderLock->selBorder;
				}
			}
		}
		g_rememberIndex = patch->index[0];
	}
//		triVertexRem.refPart(1)=triVertexRem.refPart(2);
///		triVertexRem.refPart(2)=smls->actTriPart;
//		triVertexRem.refIndex(1)=triVertexRem.refIndex(2);
//		triVertexRem.refIndex(2)=smls->ActTri*3+smls->ActTriCorner;

}
void swapVertexOrderDec(ImmediatePart* part, int ActCtrlPnt){
	Seamless* smls=part->smls;
	if(ActCtrlPnt>0){
		fastFullBuildForAutoIn(true);
		part->swapVertex(ActCtrlPnt,ActCtrlPnt-1);
		smls->ActCtrlPnt--;
		smls->doBuild();
		smls->ctrf=true;
		makePointerEqualIndexLocation();
		BackUp::monostable.beginCycle();
	}
}
void swapVertexOrderInc(ImmediatePart* part, int ActCtrlPnt){
	Seamless* smls=part->smls;
	M_Vec3f& coord = part->coord.refValue();
	if( ActCtrlPnt<coord.length-1){
		fastFullBuildForAutoIn(true);
		part->swapVertex(ActCtrlPnt,ActCtrlPnt+1);
		smls->ActCtrlPnt++;
		smls->doBuild();
		smls->ctrf=true;
		makePointerEqualIndexLocation();
		BackUp::monostable.beginCycle();
	}
}
ImmediatePart* swapVertexOrderSub(Part* whitePart, int& ActCtrlPnt, Stem*& stem, Seamless* smls){
	smls->atThisStage=0;
	smls->doBuild();
	if(TransferOwnership* transferOwnership = dynamic_cast<TransferOwnership*>(smls->atThisStage)){
		if(Part* source = dynamic_cast<Part*>(transferOwnership->source.p)){
			M_Int& qty = transferOwnership->qty.refValue();
			M_Int& numOf = transferOwnership->numOf.refValue();
			M_SArt& m_dest = transferOwnership->dest.refValue();
			for(int k = 0; k < qty.length && k < m_dest.length && whitePart != m_dest[k]; k++){
				ActCtrlPnt += qty[k] * numOf[k];
			}
			if(transferOwnership->beginSP && ActCtrlPnt > 0){
				ActCtrlPnt-=qty[0]-1;
			}
			ActCtrlPnt+=smls->ActCtrlPnt;
			NurbsPatch* patch = 0;
			ImmediatePart* bluePart = 0;
			Polymap* polymap = 0;
			if(getPolyMap(patch, stem, polymap, source, smls->ActTri, bluePart)){
				return bluePart;	
			}
		}
	}
	return 0;
}
void swapVertexOrderSub(const Vec3f& selCoordPos, Seamless* smls){
	M_Art& partList = smls->partList;
	for(int k = partList.length; --k>=0;){
		Art* part = partList[k];
		const M_Vec3f& coord = part->coord;
		for(int i = coord.length; --i>=0;){
			if(coord[i] == selCoordPos){
				smls->ActCtrlPnt=i;
				g_stw->setSelNodeLine(partList[part->plidx]);
			}
		}
	}
	makePointerEqualIndexLocation();
}
void swapVertexOrderDec(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		if(g_colorOwnership){
			Seamless* smls = whitePart->smls;
			M_Vec3f& coord = whitePart->coord;

			NurbsPatch* patch = 0;
			Stem* stem = 0;
			Polymap* polymap = 0;
			ImmediatePart* bluePart = 0;
			if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
				if(bluePart){
					swapVertexOrderDec(bluePart, smls->ActCtrlPnt-stem->beginCoordOffset);
				}
			}
			else if(smls->ActCtrlPnt < coord.length){
				smls->selCoordPos = coord[smls->ActCtrlPnt];
				int ActCtrlPnt = 0;
				if(ImmediatePart* bluePart = swapVertexOrderSub(whitePart, ActCtrlPnt, stem, smls)){
					Vec3f selCoordPos = smls->selCoordPos;
					swapVertexOrderDec(bluePart, ActCtrlPnt-stem->beginCoordOffset);
					swapVertexOrderSub(selCoordPos, smls);
				}
				smls->atThisStage=smls;
			}
		}
	}
	else if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		swapVertexOrderDec(part, part->smls->ActCtrlPnt);
	}
	else{
		needPartMsg();
	}
}
void swapVertexOrderInc(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		if(g_colorOwnership){
			Seamless* smls = whitePart->smls;
			M_Vec3f& coord = whitePart->coord;

			NurbsPatch* patch = 0;
			Stem* stem = 0;
			Polymap* polymap = 0;
			ImmediatePart* bluePart = 0;
			if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
				if(bluePart){
					swapVertexOrderInc(bluePart, smls->ActCtrlPnt-stem->beginCoordOffset);
				}
			}
			else if(smls->ActCtrlPnt < coord.length){
				smls->selCoordPos = coord[smls->ActCtrlPnt];
				int ActCtrlPnt = 0;
				if(ImmediatePart* bluePart = swapVertexOrderSub(whitePart, ActCtrlPnt, stem, smls)){
					Vec3f selCoordPos = smls->selCoordPos;
					swapVertexOrderInc(bluePart, ActCtrlPnt-stem->beginCoordOffset);
					swapVertexOrderSub(selCoordPos, smls);
				}
				smls->atThisStage=smls;
			}
		}
	}
	else if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		swapVertexOrderInc(part, part->smls->ActCtrlPnt);
	}
	else{
		needPartMsg();
	}
}
void mirrorTriangleImmediate(ImmediatePart* part, Art* actTriPart, const int ActTri){//DEC_RED:
	fastFullBuildForAutoIn(true);
	Seamless* smls=part->smls;
	BackUp::monostable.beginCycle();
	smls->catMatrix();
	smls->catIMatrix();
	actTriPart->mirrorTriangle( ActTri, smls->ActTriCorner);
}
void mirrorTriangleImmediate(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				mirrorTriangleImmediate(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
				commonEndVertexBuild();
			}
			else{
				bluePart = selectPolyMapPart(whitePart);
				stem->copyPolyMapToPart(bluePart);
				bluePart->smls->ActTriCorner = smls->ActTriCorner;
				mirrorTriangleImmediate(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
				stem->copyPartToPolyMap(bluePart);
				commonEndVertexBuild();
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		mirrorTriangleImmediate(bluePart, smls->actTriPart, smls->ActTri);
		commonEndVertexBuild();
	}
	else if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		bezierPatch->mirrorPatch();
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->mirrorPoly();
	}
	else{
		needPartMsg();
	}
}

void genTriangle(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		
		if(part->workEffect==0)return;
		part->deleteGeometry();
		Seamless* smls = part->smls;
		BackUp::monostable.beginCycle();
		Effect* effect=part->workEffect;
		int effectLength=effect->getEffectLength();
		if(effect->getType()==COLOREFFECT){
			dynamic_cast<ColorEffect*>(effect)->color.refValue().length+=3;
		}
		else if(effect->getType()==TEXTUREEFFECT){
			dynamic_cast<TextureEffect*>(effect)->texCoord.refValue().length+=3;
		}
		else if(effect->getType()==MONOCOLOREFFECT){
		}
		else throw;

		part->coord=M_Vec3f(
			Vec3f(-1, 0, 0),
			Vec3f(1, 0, 0),
			Vec3f(0, 1, 0)
			);
		part->triangle+=S_Triangle(part, 0, effectLength, part, 1, effectLength+1, part, 2, effectLength+2);
		part->relevantPart=M_Art(part);
		smls->process();

		smls->convertToRenderFormat();
		makePointerEqualIndexLocation();
	}
	else{
		needBluePartMsg();
	}
}
void transferSingleVertex(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			fastFullBuildForAutoIn(true);
			BackUp::monostable.beginCycle();
			Seamless* smls=part->smls;
			smls->catMatrix();
			smls->catIMatrix();

			transferOneVertex(part, remPJ, smls->ActCtrlPnt);

			smls->process();
			
			smls->catMatrix();
			smls->catIMatrix();
			smls->convertToRenderFormat();

			g_stw->selLine=g_stw->copyLine;
			smls->ActCtrlPnt=remPJ->coord.length;
			g_selNode->getSceneDisplayString(g_stw->textCell.cellStr="");
			g_stw->textCell.updateCaret();
			makePointerEqualIndexLocation();
			setupSelectedNodeWindow();
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}
void transferPartVertex(){
	if(remPJ==NULL)return;
	BackUp::monostable.beginCycle();
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	Seamless* &smls=part->smls;
	smls->ActCtrlPnt=remPJ->coord.length+smls->ActCtrlPnt;
	for (int i=part->coord.length; --i>=0; ){
		transferOneVertex(part, remPJ, 0);
	}


	smls->process();
	
	smls->catMatrix();
	smls->catIMatrix();
	smls->convertToRenderFormat();

	g_stw->selLine=g_stw->copyLine;
	smls->ActCtrlPnt=remPJ->coord.length;
	makePointerEqualIndexLocation();
	g_stw->paint();
	Window::paintAllControlPanels();
}
void openSettings(){
	ControlPanel* settings = new ControlPanel;
	settings->setupNodeWindow(g_settings);
	Window::activeWindow = settings;
}

void floorBuild(float floorHeight){
	removeTypeFromList <DownButton>(g_callBackList);
	g_stw->cp->setupNodeWindow(g_selNode.getRaw());

	float yPos = .92 + floorHeight;
	//front_view->position=Vec3f(0,-.83,3);
	//front_view->position=Vec3f(0,.92,3);

	NoOutputGroup* noOutputGroup = new NoOutputGroup;
	Viewpoint* front_view = new Viewpoint;
	front_view->name=name_front_view;
	front_view->perspective=false;
	front_view->orientation=Rotation(0,1,0,0);
	//front_view->position=Vec3f(0,-.83,3);
	front_view->position=Vec3f(0,yPos,3);
	noOutputGroup->children+=front_view;
	Viewpoint* left_view = new Viewpoint;
	left_view->name=name_left_view;
	left_view->perspective=false;
	left_view->orientation=Rotation(0,1,0,1.570796);
	left_view->position=Vec3f(3,yPos,0);
	noOutputGroup->children+=left_view;
	Viewpoint* right_view = new Viewpoint;
	right_view->name=name_right_view;
	right_view->perspective=false;
	right_view->orientation=Rotation(0,-1,0,1.570796);
	right_view->position=Vec3f(-3,yPos,0);
	noOutputGroup->children+=right_view;
	Viewpoint* back_view = new Viewpoint;
	back_view->name=name_back_view;
	back_view->perspective=false;
	back_view->orientation=Rotation(0,1,0,3.141593);
	back_view->position=Vec3f(0,yPos,-3);
	noOutputGroup->children+=back_view;
	Viewpoint* birds_eye_view = new Viewpoint;

	birds_eye_view->name=name_birds_eye_view;
	birds_eye_view->perspective=false;
	birds_eye_view->orientation=Rotation(-1,0,0,1.570796);
	birds_eye_view->position=Vec3f(0,(2.17 + 1.75) + floorHeight, 0);
	noOutputGroup->children+=birds_eye_view;
	Viewpoint* worms_eye_view = new Viewpoint;
	worms_eye_view->name=name_worms_eye_view;
	worms_eye_view->perspective=false;
	worms_eye_view->orientation=Rotation(1,0,0,1.570796);
	worms_eye_view->position=Vec3f(0,(-3.83 + 1.75) + floorHeight, 0);
	noOutputGroup->children+=worms_eye_view;
	Viewpoint* showcase_view = new Viewpoint;
	showcase_view->name=name_showcase_view;
	showcase_view->perspective=false;
	showcase_view->orientation=Rotation(-.483572, .867491, .116694,.675209);
	showcase_view->position=Vec3f(1.589557, (.143424 + 1.75) + floorHeight, 2.350692);
	noOutputGroup->children+=showcase_view;
	Seamless* smls = new Seamless;
	smls->translation.y = floorHeight;
	smls->collapsed=true;
	ColorEffect* n01136290 = new ColorEffect;
	smls->effect=n01136290;
	Part* floor = new Part;
	floor->name=name_floor;
	Part* ruller = new Part;
	ruller->name=name_ruller;
	floor->part+=ruller;
	smls->skeleton+=floor;
	GroupBuild* groupBuild = new GroupBuild;
	groupBuild->collapsed=true;
	Stem* stem = new Stem;
	stem->staves=1;
	stem->xDistance=.25;
	stem->bands=M_Int(1);
	stem->yDistance=M_Float(.25);
	stem->uWrap=false;
	stem->part+=ruller;
	groupBuild->build+=stem;
	ColorSweep* n01138f40 = new ColorSweep;
	n01138f40->location=Vec3f(0,-.001,0);
	n01138f40->color=M_Color3f(Color3f(1,1,.5),Color3f(0,0,1));
	n01138f40->distance=M_Float(.252);
	n01138f40->opacity=M_Float(1,1);
	n01138f40->straight=true;
	n01138f40->part+=ruller;
	groupBuild->build+=n01138f40;
	CopyPart* n011395f8 = new CopyPart;
	n011395f8->add=true;
	n011395f8->insideOut=true;
	n011395f8->source+=ruller;
	n011395f8->dest+=ruller;
	groupBuild->build+=n011395f8;
	CopyPart* n01139a40 = new CopyPart;
	n01139a40->source+=ruller;
	n01139a40->dest+=floor;
	groupBuild->build+=n01139a40;
	Place* n01139e78 = new Place;
	n01139e78->translation=Vec3f(0,.25,0);
	n01139e78->descendants=false;
	n01139e78->part+=floor;
	groupBuild->build+=n01139e78;
	TransferVertex* n0113a290 = new TransferVertex;
	n0113a290->source=floor;
	n0113a290->dest=ruller;
	groupBuild->build+=n0113a290;
	CopyPart* n0113a620 = new CopyPart;
	n0113a620->source+=ruller;
	n0113a620->dest+=floor;
	groupBuild->build+=n0113a620;
	Place* n0113ab18 = new Place;
	n0113ab18->translation=Vec3f(0,.5,0);
	n0113ab18->descendants=false;
	n0113ab18->part+=floor;
	groupBuild->build+=n0113ab18;
	TransferVertex* n0113aee8 = new TransferVertex;
	n0113aee8->source=floor;
	n0113aee8->dest=ruller;
	groupBuild->build+=n0113aee8;
	CopyPart* n0113b278 = new CopyPart;
	n0113b278->source+=ruller;
	n0113b278->dest+=floor;
	groupBuild->build+=n0113b278;
	ColorSweep* n0113b6f8 = new ColorSweep;
	n0113b6f8->color=M_Color3f(Color3f(1,.5,.5),Color3f(1,.5,.5));
	n0113b6f8->distance=M_Float(1);
	n0113b6f8->opacity=M_Float(.5,.5);
	n0113b6f8->straight=true;
	n0113b6f8->part+=ruller;
	groupBuild->build+=n0113b6f8;
	Place* n0113bdb0 = new Place;
	n0113bdb0->translation=Vec3f(0,1,0);
	n0113bdb0->descendants=false;
	n0113bdb0->part+=floor;
	groupBuild->build+=n0113bdb0;
	TransferVertex* n0113c180 = new TransferVertex;
	n0113c180->source=floor;
	n0113c180->dest=ruller;
	groupBuild->build+=n0113c180;
	Place* n0113c4c8 = new Place;
	n0113c4c8->translation=Vec3f(.5,0,0);
	n0113c4c8->part+=ruller;
	groupBuild->build+=n0113c4c8;
	Stem* stem2 = new Stem;
	stem2->add=false;
	stem2->slide=Vec2f(0,-1);
	stem2->orientation=Rotation(1,0,0,-1.57096);
	stem2->staves=1;
	stem2->xDistance=2;
	stem2->bands=M_Int(1);
	stem2->yDistance=M_Float(2);
	stem2->uWrap=false;
	stem2->part+=floor;
	groupBuild->build+=stem2;
	CopyPart* n0113d0d0 = new CopyPart;
	n0113d0d0->add=true;
	n0113d0d0->insideOut=true;
	n0113d0d0->source+=floor;
	n0113d0d0->dest+=floor;
	groupBuild->build+=n0113d0d0;
	smls->build+=groupBuild;
	stem->processFields();
	stem2->processFields();

	//Place* n0113d4c0 = new Place;
	//n0113d4c0->translation=Vec3f(0,-1.75,0);
	//n0113d4c0->part+=floor;
	//smls->build+=n0113d4c0;
	noOutputGroup->children+=smls;

	M_Node importDefList;
	importDefList=M_Node(
		front_view,
		left_view,
		right_view,
		back_view,
		birds_eye_view,
		worms_eye_view,
		showcase_view,
		floor,
		ruller
	);

	resetDefList();
	checkForNameClash(importDefList, g_defLookUp);

	g_scene->children+=noOutputGroup;

	smls->process();
	smls->doBuild();
	g_stw->genNodeList();
	g_stw->selLine=g_sceneTreeList.length-1;
	g_stw->textCell.cellStr = g_selNode->name;
	g_stw->paint();
	updateBuild3dWindow();
	Viewpoint::gotoFirstViewpoint();

/*
	String str = g_scene->children.refValue()[g_scene->children.refValue().length-1]->toCPPString();
	str.write("seamless_1212121_floor.cpp");
*/
}
void FloorHeight::ZeroHeight::operator = (const bool& par){
	floorBuild(0);
}
void FloorHeight::BlaxxunHeight::operator = (const bool& par){
	floorBuild(-1.75);
}
void floorBuild(){
	g_stw->cp->setupNodeWindow(g_floorHeight);
	Window::activeWindow=g_stw->cp.p;
}
void togglePerspective(){

	g_perspective = !g_perspective;
	//if(g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE){
	updateIconPointers();
	
}
void decCoordX(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		float res=getIncDecRes();
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0){
			if(g_radial_Tug->active){
				keyboardRadialTug(coord[smls->ActCtrlPnt], Vec3f(-res,0,0));
			}
			else{
				Vec3f pt = coord[smls->ActCtrlPnt];
				Vec3f newPt = pt;
				quantizeDec(newPt.x);
				smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
				coord[smls->ActCtrlPnt]=newPt;
			}
			commonEndVertexChange();
		}
	}
	else{
		needBluePartMsg();
	}
}
void incCoordX(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		float res=getIncDecRes();
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0){
			if(g_radial_Tug->active){
				keyboardRadialTug(coord[smls->ActCtrlPnt], Vec3f(res,0,0));
			}
			else{
				Vec3f pt = coord[smls->ActCtrlPnt];
				Vec3f newPt = pt;
				quantizeInc(newPt.x);
				smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
				coord[smls->ActCtrlPnt] = newPt;
			}
			commonEndVertexChange();
		}
	}
	else{
		needBluePartMsg();
	}
}
void decCoordY(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		float res=getIncDecRes();
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0){
			if(g_radial_Tug->active){
				keyboardRadialTug(coord[smls->ActCtrlPnt], Vec3f(0,-res,0));
			}
			else{
				Vec3f pt = coord[smls->ActCtrlPnt];
				Vec3f newPt = pt;
				quantizeDec(newPt.y);
				smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
				coord[smls->ActCtrlPnt] = newPt;
			}
			commonEndVertexChange();
		}
	}
	else{
		needBluePartMsg();
	}
}
void incCoordY(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		float res=getIncDecRes();
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0){
			if(g_radial_Tug->active){
				keyboardRadialTug(coord[smls->ActCtrlPnt], Vec3f(0,res,0));
			}
			else{
				Vec3f pt = coord[smls->ActCtrlPnt];
				Vec3f newPt = pt;
				quantizeInc(newPt.y);
				smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
				coord[smls->ActCtrlPnt] = newPt;
			}
			commonEndVertexChange();
		}
	}
	else{
		needBluePartMsg();
	}
}
void decCoordZ(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		float res=getIncDecRes();
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0){
			if(g_radial_Tug->active){
				keyboardRadialTug(coord[smls->ActCtrlPnt], Vec3f(0,0,-res));
			}
			else{
				Vec3f pt = coord[smls->ActCtrlPnt];
				Vec3f newPt = pt;
				quantizeDec(newPt.z);
				smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
				coord[smls->ActCtrlPnt] = newPt;
			}
			commonEndVertexChange();
		}
	}
	else{
		needBluePartMsg();
	}
}
void incCoordZ(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		float res=getIncDecRes();
		Seamless* smls=part->smls;
		M_Vec3f& coord=part->coord;
		if(coord.length>0){
			if(g_radial_Tug->active){
				keyboardRadialTug(coord[smls->ActCtrlPnt], Vec3f(0,0,res));
			}
			else{
				Vec3f pt = coord[smls->ActCtrlPnt];
				Vec3f newPt = pt;
				quantizeInc(newPt.z);
				smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
				coord[smls->ActCtrlPnt] = newPt;
			}
			commonEndVertexChange();
		}
	}
	else{
		needBluePartMsg();
	}
}
void quantizeIncColor(float& v);
void quantizeDecColor(float& v);

void commonIncDecColorChange(Seamless* smls, Art* part, M_Color3f& color, const int colorIdx){
	int returnCoordIdxIdx=0;
	if(Art* mirrorPart=smls->getMirrorTriangle(part, returnCoordIdxIdx)){
		color[mirrorPart->triangle.refTexIndex(returnCoordIdxIdx)]=color[colorIdx];
	}
	commonEndVertexChange();
}
void decRed(ImmediatePart* part, Art* actTriPart, const int ActTri){//DEC_RED:
	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord;
	if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
		M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
		int colorIdx=actTriPart->triangle.refTexIndex(ActTri*3+smls->ActTriCorner);
		quantizeDecColor(color[colorIdx].r);

		commonIncDecColorChange(smls, part, color, colorIdx);
	}
}
void decRed(){//DEC_RED:
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				decRed(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		decRed(bluePart, smls->actTriPart, smls->ActTri);
	}
	else{
		needPartMsg();
	}
}
void incRed(ImmediatePart* part, Art* actTriPart, const int ActTri){//DEC_RED:
	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord;
	if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
		M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
		int colorIdx=actTriPart->triangle.refTexIndex(ActTri*3+smls->ActTriCorner);
		quantizeIncColor(color[colorIdx].r);

		commonIncDecColorChange(smls, part, color, colorIdx);
	}
}
void incRed(){//INC_RED:
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				incRed(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		incRed(bluePart, smls->actTriPart, smls->ActTri);
	}
	else{
		needPartMsg();
	}
}
void decGreen(ImmediatePart* part, Art* actTriPart, const int ActTri){//DEC_RED:
	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord;
	if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
		M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
		int colorIdx=actTriPart->triangle.refTexIndex(ActTri*3+smls->ActTriCorner);
		quantizeDecColor(color[colorIdx].g);

		commonIncDecColorChange(smls, part, color, colorIdx);
	}
}
void decGreen(){//DEC_GREEN:
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				decGreen(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		decGreen(bluePart, smls->actTriPart, smls->ActTri);
	}
	else{
		needPartMsg();
	}
}
void incGreen(ImmediatePart* part, Art* actTriPart, const int ActTri){//DEC_RED:
	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord;
	if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
		M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
		int colorIdx=actTriPart->triangle.refTexIndex(ActTri*3+smls->ActTriCorner);
		quantizeIncColor(color[colorIdx].g);

		commonIncDecColorChange(smls, part, color, colorIdx);
	}
}
void incGreen(){//INC_GREEN:
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				incGreen(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		incGreen(bluePart, smls->actTriPart, smls->ActTri);
	}
	else{
		needPartMsg();
	}
}
void decBlue(ImmediatePart* part, Art* actTriPart, const int ActTri){//DEC_RED:
	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord;
	if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
		M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
		int colorIdx=actTriPart->triangle.refTexIndex(ActTri*3+smls->ActTriCorner);
		quantizeDecColor(color[colorIdx].b);

		commonIncDecColorChange(smls, part, color, colorIdx);
	}
}
void decBlue(){//DEC_BLUE:
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				decBlue(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		decBlue(bluePart, smls->actTriPart, smls->ActTri);
	}
	else{
		needPartMsg();
	}
}
void incBlue(ImmediatePart* part, Art* actTriPart, const int ActTri){//DEC_RED:
	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord;
	if(coord.length>0 &&part->workEffect!=0&&part->workEffect->getType()==COLOREFFECT){
		M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color;
		int colorIdx=actTriPart->triangle.refTexIndex(ActTri*3+smls->ActTriCorner);
		quantizeIncColor(color[colorIdx].b);

		commonIncDecColorChange(smls, part, color, colorIdx);
	}
}
void incBlue(){//INC_BLUE:
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				incBlue(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		incBlue(bluePart, smls->actTriPart, smls->ActTri);
	}
	else{
		needPartMsg();
	}
}
void zoom(){
	g_scene->setZoom(g_scene->getZoom()/1.01F);
	houseKeepVisualClue();
}
void zoomIn(){
	g_scene->setZoom(g_scene->getZoom()*1.01F);
	houseKeepVisualClue();
}
void quantiseRotation(Rotation&);

void setFrontViewOrientation(){
	g_scene->rotation = Rotation(0, 0, 1, 0);
	houseKeepVisualClueViewChange();
//	else if(MultiSelect::pointList.length >= g_transformCP->selNumberToShow){
//		g_transformCP->calcRotCtrlPnts();
//		g_transformCP->houseKeep();
//	}
}

void setBackViewOrientation(){
	g_scene->rotation=Rotation(0, 1, 0, PI);
	houseKeepVisualClueViewChange();
}
void setLeftViewOrientation(){
	g_scene->rotation = Rotation(0, 1, 0, -PI*.5);
	houseKeepVisualClueViewChange();
}
void setRightViewOrientation(){
	g_scene->rotation = Rotation(0, 1, 0, PI*.5);
	houseKeepVisualClueViewChange();
}
void setBirdsEyeViewOrientation(){
	g_scene->rotation = Rotation(1, 0, 0, PI*.5);
	houseKeepVisualClueViewChange();
}
void setWormsEyeViewOrientation(){
	g_scene->rotation = Rotation(1, 0, 0, -PI*.5);
	houseKeepVisualClueViewChange();
}

enum initialOrientation{
	FRONT = 1,
	BACK = 2,
	LEFT = 3,
	RIGHT = 4,
	BIRDS = 5,
	WORMS = 6,
};

void quantiseViewOrientation(){
	//quantiseRotation(g_scene->rotation);
	Rotation rot = g_scene->rotation;
	Vec3f v = rot * Vec3f(0, 0, 1);
	float closest = 10;
	Rotation newRot;
	int i = 0;

	float f = 1.0 - v.z;
	if(f < closest){
		closest = f;
		i = FRONT;
	}
	f = 1.0 + v.z;
	if(f < closest){
		closest = f;
		i = BACK;
	}

	v = rot * Vec3f(1, 0, 0);
	f = 1.0 - v.z;
	if(f < closest){
		closest = f;
		i = LEFT;
	}
	f = 1.0 + v.z;
	if(f < closest){
		closest = f;
		i = RIGHT;
	}

	v = rot * Vec3f(0, 1, 0);
	f = 1.0 - v.z;
	if(f < closest){
		closest = f;
		i = BIRDS;
	}
	f = 1.0 + v.z;
	if(f < closest){
		closest = f;
		i = WORMS;
	}
/*
	if(i == WORMS || i == BIRDS){
		v = rot * Vec3f(0, 1, 0);
		if(v.z < 0){
			i = WORMS;
		}
		else{
			i = BIRDS;
		}
	}
*/
	closest = 10;

	switch(i){
	case FRONT:
		v = rot * Vec3f(1, 0, 0);
		f = 1.0 - v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 0, 1, 0);
		}
		f = 1.0 + v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 0, 1, PI);
		}
		f = 1.0 - v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 0, 1, PI * .5);
		}
		f = v.y + 1.0;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 0, 1, -PI * .5);
		}
		break;
	case BACK:
		v = rot * Vec3f(1, 0, 0);
		f = 1.0 + v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI);
		}
		f = 1.0 - v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI) * Rotation(0, 0, 1, PI);
		}
		f = 1.0 + v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI) * Rotation(0, 0, 1, PI * .5);
		}
		f = 1.0 - v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI) * Rotation(0, 0, 1, -PI * .5);
		}
		break;
	case LEFT:
		v = rot * Vec3f(0, 0, 1);
		f = 1.0 + v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, -PI*.5);
		}
		f = 1.0 - v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, -PI*.5) * Rotation(0, 0, 1, PI);
		}
		f = 1.0 + v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, -PI*.5) * Rotation(0, 0, 1, PI * .5);
		}
		f = 1.0 - v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, -PI*.5) * Rotation(0, 0, 1, -PI * .5);
		}	
		break;
	case RIGHT:
		v = rot * Vec3f(0, 0, 1);
		f = 1.0 - v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI*.5);
		}
		f = 1.0 + v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI*.5) * Rotation(0, 0, 1, PI);
		}
		f = 1.0 - v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI*.5) * Rotation(0, 0, 1, PI * .5);
		}
		f = 1.0 + v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(0, 1, 0, PI*.5) * Rotation(0, 0, 1, -PI * .5);
		}	
		break;
	case WORMS:
		v = rot * Vec3f(1, 0, 0);
		f = 1.0 - v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, -PI*.5);
		}
		f = 1.0 + v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, -PI*.5) * Rotation(0, 0, 1, PI);
		}
		f = 1.0 - v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, -PI*.5) * Rotation(0, 0, 1, PI * .5);
		}
		f = v.y + 1.0;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, -PI*.5) * Rotation(0, 0, 1, -PI * .5);
		}
		break;
	case BIRDS:
		v = rot * Vec3f(1, 0, 0);
		f = 1.0 - v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, PI*.5);
		}
		f = 1.0 + v.x;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, PI*.5) * Rotation(0, 0, 1, PI);
		}
		f = 1.0 - v.y;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, PI*.5) * Rotation(0, 0, 1, PI * .5);
		}
		f = v.y + 1.0;
		if(f < closest){
			closest = f;
			newRot = Rotation(1, 0, 0, PI*.5) * Rotation(0, 0, 1, -PI * .5);
		}
		break;
	};

	g_scene->rotation = newRot;
	houseKeepVisualClueViewChange();
}
void commonDecIncViewOrientation90(){
	if(controlKeyState()){
		quantiseRotation(g_scene->rotation);
	}
	houseKeepVisualClueViewChange();
}
void decViewOrientation90X(){//DEC_VIEW_ORIENTATION_90_X
	g_scene->rotation=g_scene->rotation*Rotation(1,0,0,-PI*.25);
	commonDecIncViewOrientation90();
}
void incViewOrientation90X(){//INC_VIEW_ORIENTATION_90_X
	g_scene->rotation=g_scene->rotation*Rotation(1,0,0,PI*.25);
	commonDecIncViewOrientation90();
}
void decViewOrientation90Y(){//DEC_VIEW_ORIENTATION_90_Y
	g_scene->rotation=g_scene->rotation*Rotation(0,1,0,-PI*.25);
	commonDecIncViewOrientation90();
}
void incViewOrientation90Y(){//INC_VIEW_ORIENTATION_90_Y
	g_scene->rotation=g_scene->rotation*Rotation(0,1,0,PI*.25);
	commonDecIncViewOrientation90();
}
void decViewOrientation90Z(){//DEC_VIEW_ORIENTATION_90_Z
	g_scene->rotation=g_scene->rotation*Rotation(0,0,1,-PI*.25);
	commonDecIncViewOrientation90();
}
void incViewOrientation90Z(){//INC_VIEW_ORIENTATION_90_Z
	g_scene->rotation=g_scene->rotation*Rotation(0,0,1,PI*.25);
	commonDecIncViewOrientation90();
}
void decViewOrientationX(){//DEC_VIEW_ORIENTATION_X	//[
	if(controlKeyState()){
		decViewOrientation90X();
	}
	else{
		g_scene->rotation=g_scene->rotation*Rotation(1,0,0,-PI*.01);
	}
	houseKeepVisualClueViewChange();
}
void incViewOrientationX(){//INC_VIEW_ORIENTATION_X	//]
	if(controlKeyState()){
		incViewOrientation90X();
	}
	else{
		g_scene->rotation=g_scene->rotation*Rotation(1,0,0,PI*.01);
	}
	houseKeepVisualClueViewChange();
}
void decViewOrientationY(){//DEC_VIEW_ORIENTATION_Y	//;
	if(controlKeyState()){
		decViewOrientation90Y();
	}
	else{
		g_scene->rotation=g_scene->rotation*Rotation(0,1,0,-PI*.01);
	}
	houseKeepVisualClueViewChange();
}
void incViewOrientationY(){//INC_VIEW_ORIENTATION_Y	//'
	if(controlKeyState()){
		incViewOrientation90Y();
	}
	else{
		g_scene->rotation=g_scene->rotation*Rotation(0,1,0,PI*.01);
	}
	houseKeepVisualClueViewChange();
}
void decViewOrientationZ(){//DEC_VIEW_ORIENTATION_Z	//.
	if(controlKeyState()){
		decViewOrientation90Z();
	}
	else{
		g_scene->rotation=g_scene->rotation*Rotation(0,0,1,-PI*.01);
	}
	houseKeepVisualClueViewChange();
}
void incViewOrientationZ(){//INC_VIEW_ORIENTATION_Z	///
	if(controlKeyState()){
		incViewOrientation90Z();
	}
	else{
		g_scene->rotation=g_scene->rotation*Rotation(0,0,1,PI*.01);
	}
	houseKeepVisualClueViewChange();
}
float tranP(){
	return 1/g_scene->getZoom()*.01F;
}
void decViewPositionX(){//DEC_VIEW_POSITION_X
	g_scene->translation+=-g_scene->rotation*Vec3f(-tranP(),0,0);
}
void incViewPositionX(){//INC_VIEW_POSITION_X
	g_scene->translation+=-g_scene->rotation*Vec3f(tranP(),0,0);
}
void decViewPositionY(){//DEC_VIEW_POSITION_Y
	g_scene->translation+=-g_scene->rotation*Vec3f(0,-tranP(),0);
}
void incViewPositionY(){//INC_VIEW_POSITION_Y
	g_scene->translation+=-g_scene->rotation*Vec3f(0,tranP(),0);
}
void decViewPositionZ(){//DEC_VIEW_POSITION_Z
	g_scene->translation+=-g_scene->rotation*Vec3f(0,0,tranP());
}
void incViewPositionZ(){//INC_VIEW_POSITION_Z
	g_scene->translation+=-g_scene->rotation*Vec3f(0,0,-tranP());
}

//////////////
/*
void decViewOrientationX(){//DEC_VIEW_ORIENTATION_X	//[
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(1,0,0,PI*.01);
}
void incViewOrientationX(){//INC_VIEW_ORIENTATION_X	//]
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(1,0,0,-PI*.01);
}
void decViewOrientationY(){//DEC_VIEW_ORIENTATION_Y	//;
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,1,0,PI*.01);
}
void incViewOrientationY(){//INC_VIEW_ORIENTATION_Y	//'
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,1,0,-PI*.01);
}
void decViewOrientationZ(){//DEC_VIEW_ORIENTATION_Z	//.
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,0,1,PI*.01);
}
void incViewOrientationZ(){//INC_VIEW_ORIENTATION_Z	///
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,0,1,-PI*.01);
}
void decViewOrientation90X(){//DEC_VIEW_ORIENTATION_90_X
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(1,0,0,PI*.5);
}
void incViewOrientation90X(){//INC_VIEW_ORIENTATION_90_X
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(1,0,0,-PI*.5);
}
void decViewOrientation90Y(){//DEC_VIEW_ORIENTATION_90_Y
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,1,0,PI*.5);
}
void incViewOrientation90Y(){//INC_VIEW_ORIENTATION_90_Y
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,1,0,-PI*.5);
}
void decViewOrientation90Z(){//DEC_VIEW_ORIENTATION_90_Z
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,0,1,PI*.5);
}
void incViewOrientation90Z(){//INC_VIEW_ORIENTATION_90_Z
	g_scene->orientation=g_scene->orientation.refValue()*Rotation(0,0,1,-PI*.5);
}
float tranP(){
	return 1/g_scene->zoom*.01;
}
void decViewPositionX(){//DEC_VIEW_POSITION_X
	g_scene->position=g_scene->position+g_scene->orientation.refValue()*Vec3f(tranP(),0,0);
}
void incViewPositionX(){//INC_VIEW_POSITION_X
	g_scene->position=g_scene->position+g_scene->orientation.refValue()*Vec3f(-tranP(),0,0);
}
void decViewPositionY(){//DEC_VIEW_POSITION_Y
	g_scene->position=g_scene->position+g_scene->orientation.refValue()*Vec3f(0,tranP(),0);
}
void incViewPositionY(){//INC_VIEW_POSITION_Y
	g_scene->position=g_scene->position+g_scene->orientation.refValue()*Vec3f(0,-tranP(),0);
}
void decViewPositionZ(){//DEC_VIEW_POSITION_Z
	g_scene->position=g_scene->position+g_scene->orientation.refValue()*Vec3f(0,0,-tranP());
}
void incViewPositionZ(){//INC_VIEW_POSITION_Z
	g_scene->position=g_scene->position+g_scene->orientation.refValue()*Vec3f(0,0,tranP());
}
*/
Seamless* getSeamless(){
	return getSeamless2();
	/*
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Seamless* smls=dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			return smls;
		}
	}
	Seamless* smls = new Seamless;
	ColorEffect* colorEffect=new ColorEffect;
	smls->effect=colorEffect;
	g_scene->children+=smls;
	return smls;
	*/
}
int getNumOfDescendents(const int line){
	int indent = g_sceneTreeList[line].indent;

	int k = line;
	while(++k < g_sceneTreeList.length && indent < g_sceneTreeList[k].indent);
	
	return k-line-1;
}
int getFirstBuildNodeLine(const int line){//line = seamless
	int indent = g_sceneTreeList[line].indent;

	int k = line+1;
	for(; k < g_sceneTreeList.length && indent < g_sceneTreeList[k].indent; k++){
		
		Node* node = g_sceneTreeList[k].node.p;

//		if(dynamic_cast<Build*>(node)){
//			break;
//		}
		if(
			dynamic_cast<Effect*>(node)==0 && //if not a part or effect its the line we want
			dynamic_cast<Art*>(node)==0
			){
			break;
		}
	}
	return k;
}
int getLastBuildNodeLine(const int line){ //line == first build node line
	int indent=g_sceneTreeList[line].indent;
	int k = line+1;
	while(true){
		if(k >= g_sceneTreeList.length || indent > g_sceneTreeList[k].indent){
			break;
		}
		if(indent==g_sceneTreeList[k].indent){
			if(dynamic_cast<Build*>(g_sceneTreeList[k].node.p)==0){
				break;
			}
		}
		k++;
	}
	
	return k-1;
}
bool searchForFirstExitBuildLine(int& line, const int lastLine){
	int indent = g_sceneTreeList[line].indent;

	for(int k=line;k<=lastLine; k++){
		if(ExitBuild* exitBuild = dynamic_cast<ExitBuild*>(g_sceneTreeList[k].node.p)){
			if(exitBuild->enabled){
				line = k;
				return true;
			}
		}
	}
	return false;
}
int getDeepestOpenGroup(Seamless* smls, const int smlsIndex, const int firstLine, const int lastLine){
	int deepest=firstLine;
	for(int k=firstLine; k<=lastLine; k++){
		if(GroupBuild* groupBuild=dynamic_cast<GroupBuild*>(g_sceneTreeList[k].node.p)){
			if(groupBuild->collapsed==false){
				if(g_sceneTreeList[k].indent>=g_sceneTreeList[deepest].indent){
					if(groupBuild->collapsed==false){
						int dummy=0;
						if(getCollapsedLine(k, dummy)){
							deepest=k;
						}
					}
				}
			}
		}
	}
	return deepest;
}
void insertNodeAtExitOrEnd(MFNode& mfbuild, Build* buildNode, int buildLine){
	MFNode* m=&mfbuild;
	M_Node* m_build=&mfbuild.refValue();
	insertNodeAbove(buildLine, *m_build, buildNode);
	g_stw->selLine=buildLine;
}

void initBnspePolies(Build* build, Art* part){
	if(NurbsPatch* patch = build->getPatch()){
		int plidx = patch->plidx;
		int numOfPatches = build->numOfPatches();
		if(numOfPatches == 1){
			patch->mirror.refValue() = part->mirror;
		}
		M_<NurbsPatch*>& cpNodeList = patch->smls->cpNodeList;
		for(int k = 0; k < numOfPatches; k++){
			NurbsPatch* patch = cpNodeList[plidx + k];
			if(patch->smls->bnspe){
				patch->bnspeBuildPolies();
			}
		}
	}
}

void createNewBuildNode(Build* buildNode, Seamless* smls, Art* part){
	if(part && part->name==""){
		createUniqueName(part);
	}

	{
		int smlsLine = getNodeLine(smls);
		int firstLine = getFirstBuildNodeLine(smlsLine);
		if(smls->build.refValue().length){
			int lastLine = getLastBuildNodeLine(firstLine);
			int numOf = lastLine-firstLine + 1;

			int buildLine = firstLine;

			if(searchForFirstExitBuildLine(buildLine, lastLine)){
				int parentLine=getParent(buildLine);
				Node* parentNode= g_sceneTreeList[parentLine].node.p;
				int fieldIdx=0, element=0;

				getFNodeElementForParent(parentLine, buildLine, g_sceneTreeList, fieldIdx, element);

				Field* field=parentNode->getField(fieldIdx);
				MFNode& mfnode=*dynamic_cast<MFNode*>(field);
				M_Node& m_node=mfnode.refValue();
				insertNodeAbove(element, m_node, buildNode);
				g_stw->selLine=buildLine;
			}
			else{
				buildLine=getDeepestOpenGroup(smls, smlsLine, firstLine, lastLine);
				GroupBuild* group=dynamic_cast<GroupBuild*>(g_sceneTreeList[buildLine].node.p);
				if(group && group->collapsed == false){
					group->build.refValue()+=buildNode;
					int numOfDescendents = getNumOfDescendents(buildLine);
					g_stw->selLine=buildLine+1+numOfDescendents;
				}
				else{
					smls->build.refValue()+=buildNode;
					g_stw->selLine=lastLine+1;
				}
			}
		}
		else{
			smls->build.refValue()+=buildNode;
			g_stw->selLine=firstLine;
		}
	}
	g_stw->genNodeList();
	//setModellingMode();
	buildNode->processFields();
	smls->process();
	//smls->cpNodeListAdd();

	g_stw->textCell.cellStr = "";
	g_stw->textCell.cellActive = false;
	if(dynamic_cast<NurbsPatch*>(buildNode) == 0){
		g_stw->cp->setupNodeWindow(buildNode);
	}
	g_stw->paint();
	g_stw->commonPostInsertNodeStuff();
	setModellingMode2();

	if(part){
		initBnspePolies(buildNode, part);
	}

	//updateBuild3dWindow();
	//smls->cpNodeListAdd();
}
void checkAllAreNamedAndFixIfNot(){
	for(int i=0; i<g_selCount.all; i++){
		Node* part = g_sceneTreeList[g_stw->selLine+i].node.p;
		if(part->name==""){
			createUniqueName(part);
		}
	}
}
void copyPartForSelParts(){
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		if(g_selCount.all==1){
			msG(msg_num_of_sel_must_be_more_than_one);
			return;
		}
		if(g_selCount.all % 2){
			msG(msg_num_of_sel_must_be_even);
			return;
		}
		g_stw->textCell.cellActive=false;

		checkAllAreNamedAndFixIfNot();


		CopyPart* cp=new CopyPart;
		for(int i=0; i<g_selCount.all/2; i++){
			cp->source.refValue()+=dynamic_cast<Art*>(g_sceneTreeList[g_stw->selLine+i].node.p);
			cp->dest.refValue()+=dynamic_cast<Art*>(g_sceneTreeList[g_selCount.all/2+g_stw->selLine+i].node.p);
		}
		g_selCount.reset();

		createNewBuildNode(cp, part->smls, part);
	}
	else{
		needWhitePartMsg();
	}
}
void colorSweepForSelParts(){
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		g_stw->textCell.cellActive=false;

		checkAllAreNamedAndFixIfNot();


		ColorSweep* cs=new ColorSweep;
		for(int i=0; i<g_selCount.all; i++){
			cs->part.refValue()+=dynamic_cast<Art*>(g_sceneTreeList[g_stw->selLine+i].node.p);
		}
		g_selCount.reset();

		createNewBuildNode(cs, part->smls, part);

	}
	else{
		needWhitePartMsg();
	}
}
void placeForSelParts(){
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		g_stw->textCell.cellActive=false;

		checkAllAreNamedAndFixIfNot();


		Place* place=new Place;
		for(int i=0; i<g_selCount.all; i++){
			place->part.refValue()+=dynamic_cast<Art*>(g_sceneTreeList[g_stw->selLine+i].node.p);
		}
		g_selCount.reset();

		createNewBuildNode(place, part->smls, part);
	}
	else{
		needWhitePartMsg();
	}
}
void averagePivotPoint(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		M_Vec3f& coord = part->coord.refValue();
		if(coord.length){
			Vec3f v;
			if(MultiSelect::coordList.length){
				for(int i = MultiSelect::coordList.length; --i>=0;){
					v+=coord[MultiSelect::coordList[i].i];
				}
				v/=MultiSelect::coordList.length;
			}
			else{
				for(int i=coord.length; --i>=0;){
					v+=coord[i];
				}
				v/=coord.length;
			}
			if(part->mirror && part->mirrorPart == 0){
				v.x = 0;
			}
			Vec3f parentAbsolutePivotPoint;
			int selLine = g_stw->selLine;
			int parentLine=getParent(selLine);
			Node* parentNode= g_sceneTreeList[parentLine].node.p;
			if(Art* parentPart = dynamic_cast<Art*>(parentNode)){
				parentAbsolutePivotPoint = parentPart->absolutePivotPoint;
			}

			part->pivotPoint=v-parentAbsolutePivotPoint;
			commonEndPivotPointChange();
		}
	}
}

void Art::PivotPoint::operator=(const Vec3f& par){
	Vec3f _par = par;
	Vec3f store = refValue();
	int numOfParts = 0;
	if(home.mirror){
		if(Art* mirrorPart = home.mirrorPart){
			Vec3f mirrorPar = par;
			mirrorPar.x = -mirrorPar.x;
			mirrorPart->pivotPoint.refValue() = mirrorPar;
		}
		else if(fabs(home.absolutePivotPoint.x) < .000001){
			if(home.isLeftRightName() == false){
				_par.x = 0;
			}
		}
	}
	refValue() = _par;

	if(altKeyState()){
		Vec3f diff = store - refValue();
		M_SArt& m_part = home.part.refValue();
		for(int k = m_part.length; --k>=0;){
			m_part[k]->pivotPoint.refValue()+=diff;
		}
		if(Art* mirrorPart = home.mirrorPart){
			M_SArt& m_part = mirrorPart->part.refValue();
			for(int k = m_part.length; --k>=0;){
				m_part[k]->pivotPoint.refValue()-=diff;
			}
		}
	}

	M_SArt& skeleton=home.smls->skeleton.refValue();

	for(int k=0; k<skeleton.length; k++){
		Art* part=skeleton[k];
		part->calcAbsolutePivotPoint(Vec3f());
	}
	home.smls->ctrf=true;
}
Art* getPartForPivotPoint(){
	Node* node = g_selNode.getRaw();
	if(Art* part = dynamic_cast<Art*>(node)){
		return part;
	}
	else if(TransferBCyl* transferBCyl = dynamic_cast<TransferBCyl*>(node)){
		return transferBCyl->dest;
	}
	else if(TransferOwnership* transferOwnership = dynamic_cast<TransferOwnership*>(node)){
		return transferOwnership->getSelectedPart();
	}
	else if(CenterJoint* centerJoint = dynamic_cast<CenterJoint*>(node)){
		return centerJoint->part;
	}
	needPartOrCenterJointMsg();
	return 0;
}
void decPivotX(){//DEC_PIVOT_X:
	if(Art* part = getPartForPivotPoint()){
		Vec3f pivotPoint = part->pivotPoint;

		quantizeDec(pivotPoint.x);

		part->pivotPoint = pivotPoint;
		commonEndPivotPointChange();
	}
}
void incPivotX(){//INC_PIVOT_X:
	if(Art* part = getPartForPivotPoint()){
		Vec3f pivotPoint = part->pivotPoint;

		quantizeInc(pivotPoint.x);

		part->pivotPoint = pivotPoint;
		commonEndPivotPointChange();
	}
}
void decPivotY(){//DEC_PIVOT_Y:
	if(Art* part = getPartForPivotPoint()){
		Vec3f pivotPoint = part->pivotPoint;

		quantizeDec(pivotPoint.y);

		part->pivotPoint = pivotPoint;
		commonEndPivotPointChange();
	}
}
void incPivotY(){//INC_PIVOT_Y:
	if(Art* part = getPartForPivotPoint()){
		Vec3f pivotPoint = part->pivotPoint;

		quantizeInc(pivotPoint.y);

		part->pivotPoint = pivotPoint;
		commonEndPivotPointChange();
	}
}
void decPivotZ(){//DEC_PIVOT_Z:
	if(Art* part = getPartForPivotPoint()){
		Vec3f pivotPoint = part->pivotPoint;

		quantizeDec(pivotPoint.z);

		part->pivotPoint = pivotPoint;
		commonEndPivotPointChange();
	}
}
void incPivotZ(){//INC_PIVOT_Z:
	if(Art* part = getPartForPivotPoint()){
		Vec3f pivotPoint = part->pivotPoint;

		quantizeInc(pivotPoint.z);

		part->pivotPoint = pivotPoint;
		commonEndPivotPointChange();
	}
}
void swapDivide(Art* part, Art* actTriPart, const int ActTri){
		fastFullBuildForAutoIn(true);


	Seamless* smls=part->smls;
	
	int idxRed = ActTri * 3 + smls->ActTriCorner;
	int idxGreen = ActTri * 3 + (smls->ActTriCorner + 1) % 3;
	int notConectedA = (smls->ActTriCorner + 2) % 3;
	const int cRed = actTriPart->triangle.refIndex(idxRed);
	const int cGreen = actTriPart->triangle.refIndex(idxGreen);
	Art* red = actTriPart->triangle.refPart(idxRed);
	Art* green = actTriPart->triangle.refPart(idxGreen);
	int otherTriIndex=0;
	int notConectedB = 0;
	

	if(Art* otherTriPart = findTriangleJoinedBy2Vertices(actTriPart, ActTri*3, otherTriIndex, red, cRed, green, cGreen, notConectedB)){
		Seamless* smls=part->smls;

		BackUp::monostable.beginCycle();
		SwapTriDivide swapTriDivide;
		swapTriDivide.corner.refValue().length=0;
		swapTriDivide.part=actTriPart;
		M_Int& corner=swapTriDivide.corner.refValue();
		corner+=getIndexRelativeToPart(actTriPart, otherTriPart, otherTriIndex*3+notConectedB);
		corner+=ActTri*3+notConectedA;
		swapTriDivide.doBuild();
	}
}
void swapDivide(Art* part, Art* actTriPart, const int ActTri, const int ActTriCorner){
		fastFullBuildForAutoIn(true);


	Seamless* smls=part->smls;
	
	int idxRed = ActTri * 3 + ActTriCorner;
	int idxGreen = ActTri * 3 + (ActTriCorner + 1) % 3;
	int notConectedA = (ActTriCorner + 2) % 3;
	const int cRed = actTriPart->triangle.refIndex(idxRed);
	const int cGreen = actTriPart->triangle.refIndex(idxGreen);
	Art* red = actTriPart->triangle.refPart(idxRed);
	Art* green = actTriPart->triangle.refPart(idxGreen);
	int otherTriIndex=0;
	int notConectedB = 0;
	

	if(Art* otherTriPart = findTriangleJoinedBy2Vertices(actTriPart, ActTri*3, otherTriIndex, red, cRed, green, cGreen, notConectedB)){
		Seamless* smls=part->smls;

		BackUp::monostable.beginCycle();
		SwapTriDivide swapTriDivide;
		swapTriDivide.corner.refValue().length=0;
		swapTriDivide.part=actTriPart;
		M_Int& corner=swapTriDivide.corner.refValue();
		corner+=getIndexRelativeToPart(actTriPart, otherTriPart, otherTriIndex*3+notConectedB);
		corner+=ActTri*3+notConectedA;
		swapTriDivide.doBuild();
	}
}

void swapDivide(){
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			Seamless* smls = whitePart->smls;
			swapDivide(whitePart, smls->actTriPart, smls->ActTri);
			commonEndVertexChange();
			return;
		}
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				swapDivide(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
				BackUp::monostable.beginCycle();
				commonEndVertexBuild();
			}
			else{
				int ActTri = smls->ActTri;
				int ActTriCorner = smls->ActTriCorner;
				if(stem->mirror){
					if(	(stem->mirrorStem && stem->insideOut) ||
						(stem->mirrorStem == 0 && ActTri >= stem->coordIndex.length / 3)
						){
						ActTriCorner = abs(ActTriCorner + 1) % 3;
					}
				}
				if(0 > getRelIndex(stem, ActTri, ActTriCorner)){
					return;
				}

				bluePart = selectPolyMapPart(whitePart);
				stem->copyPolyMapToPart(bluePart);
				bluePart->smls->ActTriCorner = ActTriCorner;
				swapDivide(bluePart, bluePart, ActTri / 3);
				stem->copyPartToPolyMap(bluePart);

				BackUp::monostable.beginCycle();
				commonEndVertexBuild();
			}
		}
	}
	else if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if((g_scene->mode & HAIR_REGION_MODE)){
			if(Lock::selBorderLock){
				if(Lock* parentLock = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
					lock->deleteCorner(parentLock);
					houseKeepVisualClue();
				}
			}
		}
	}
	else{
		needPartMsg();
	}
}
void forceToMirrorX(){
	if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		if(dynamic_cast<ImmediatePart*>(g_copyNode.getRaw())){
			Seamless* smls=part->smls;
			BackUp::monostable.beginCycle();
			M_Vec3f& coord=part->coord.refValue();

			Vec3f svec = remPJ->coord.refValue()[smls->remActCtrlPnt];
			svec.x=-svec.x;
			coord[smls->ActCtrlPnt]=svec;

			makePointerEqualIndexLocation();
			smls->convertToRenderFormat();
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needBluePartMsg();
	}
}
void openNodeWindow(){ //OPEN_NODE_WINDOW
	throw;
}
void setZDragNode(){
	g_zDragMode=true;
};
void quickTriangle(Art* copyPart, ImmediatePart* part, Art* remActTriPart, Art* actTriPart, int remActTri, int ActTri){
	SeamlessPtr smls=part->smls;
	const int remActTriCorner = smls->remActTriCorner;
	const int ActTriCorner = smls->ActTriCorner;

	fastFullBuildForAutoIn(true);


//		triVertexRem.refPart(0)=triVertexRem.refPart(1);
//		triVertexRem.refPart(1)=triVertexRem.refPart(2);
	triVertexRem.refPart(0)=remActTriPart;
	triVertexRem.refPart(1)=actTriPart;
	triVertexRem.refPart(2)=actTriPart;
//		triVertexRem.refIndex(0)=triVertexRem.refIndex(1);
//		triVertexRem.refIndex(1)=triVertexRem.refIndex(2);
	triVertexRem.refIndex(0)=remActTri*3+remActTriCorner;
	triVertexRem.refIndex(1)=ActTri*3+ActTriCorner;
	triVertexRem.refIndex(2)=ActTri*3+(ActTriCorner+1)%3;

	//find if anti-clockwise
	{
		int idxA = triVertexRem.refIndex(0);
		int idxB = triVertexRem.refIndex(1);
		int idxC = triVertexRem.refIndex(2);
		Art* partA = triVertexRem.refPart(0)->triangle.refPart(idxA);
		Art* partB = triVertexRem.refPart(1)->triangle.refPart(idxB);
		Art* partC = triVertexRem.refPart(2)->triangle.refPart(idxC);

		Vec3f v[3]={
			partA->matrix * g_viewMatrix * partA->coord[triVertexRem.refPart(0)->triangle.refIndex(idxA)],
			partB->matrix * g_viewMatrix * partB->coord[triVertexRem.refPart(1)->triangle.refIndex(idxB)],
			partC->matrix * g_viewMatrix * partC->coord[triVertexRem.refPart(2)->triangle.refIndex(idxC)]
		};
		if(backFace(v[0], v[1], v[2]) == false){
			Art* storeTriPart = triVertexRem.refPart(2);
			triVertexRem.refPart(2)=triVertexRem.refPart(1);
			triVertexRem.refPart(1)=storeTriPart;

			int storeIndex = triVertexRem.refIndex(2);
			triVertexRem.refIndex(2)=triVertexRem.refIndex(1);
			triVertexRem.refIndex(1)=storeIndex;
		}
	}


	BackUp::monostable.beginCycle();

	if(smls->build.length&&g_settings->record==true){
		M_Node* build;
		int bidx=smls->getIndexForLastBuildNode(build);
		if(bidx==-1)return;

		GenPoly* genPoly = dynamic_cast<GenPoly*>((*build)[bidx].p);

		if(genPoly==0||genPoly->part!=0&&(genPoly->part.refValue()!=triVertexRem.pa||genPoly->part.refValue()!=triVertexRem.pb||genPoly->part.refValue()!=triVertexRem.pc)){
			genPoly=new GenPoly;
			genPoly->part=part;
			insertNodeBelow(bidx, *build, genPoly);
			g_stw->genNodeList();
		}
		else if(genPoly->part==0){
			genPoly->vertex.refValue().length=0;
			genPoly->part=part;
			g_stw->genNodeList();
		}
		Art* part=genPoly->part.refValue();

		genPoly->vertex.refValue()+=getIndexRelativeToPart(part, triVertexRem.pa, triVertexRem.ia);
		genPoly->vertex.refValue()+=getIndexRelativeToPart(part, triVertexRem.pb, triVertexRem.ib);
		genPoly->vertex.refValue()+=getIndexRelativeToPart(part, triVertexRem.pc, triVertexRem.ic);

	}
	else {
		GenPoly gp;
		gp.independentColor = true;
		gp.part=part;

		gp.vertex.refValue()+=getIndexRelativeToPart(part, triVertexRem.pa, triVertexRem.ia);
		gp.vertex.refValue()+=getIndexRelativeToPart(part, triVertexRem.pb, triVertexRem.ib);
		gp.vertex.refValue()+=getIndexRelativeToPart(part, triVertexRem.pc, triVertexRem.ic);

		gp.doBuild();
	}
}
void quickTriangle(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		openSceneTree();
		if(g_copyNode.getRaw()==whitePart){
			Seamless* smls = whitePart->smls;
			NurbsPatch* patch = 0;
			Stem* selStem = 0;
			Stem* copyStem = 0;
			Polymap* polymap = 0;
			ImmediatePart* bluePart = 0;
			if(getPolyMap(patch, selStem, polymap, whitePart, smls->ActTri, bluePart)){
				if(getPolyMap(patch, copyStem, polymap, whitePart, smls->remActTri, bluePart)){
					if(bluePart){
						quickTriangle(bluePart, bluePart, bluePart, bluePart, smls->remActTri-copyStem->beginTriOffset, smls->ActTri-selStem->beginTriOffset);
						commonEndVertexBuild();
					}
					else{
						bluePart = selectPolyMapPart(whitePart);
						selStem->copyPolyMapToPart(bluePart);
						bluePart->smls->remActTriCorner = smls->remActTriCorner;
						bluePart->smls->ActTriCorner = smls->ActTriCorner;
						quickTriangle(bluePart, bluePart, bluePart, bluePart, smls->remActTri-copyStem->beginTriOffset, smls->ActTri-selStem->beginTriOffset);
						selStem->copyPartToPolyMap(bluePart);
						commonEndVertexBuild();
					}
				}
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		if(Art* copyPart=dynamic_cast<Art*>(g_copyNode.getRaw())){
			Seamless* smls = bluePart->smls;
			quickTriangle(copyPart, bluePart, smls->remActTriPart , smls->actTriPart, smls->remActTri, smls->ActTri);
			commonEndVertexBuild();
		}
		else{
			needPartCopiedMsg();
		}
	}
	else if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if((g_scene->mode & HAIR_REGION_MODE)){
			if(Lock::selBorderLock){
				M_Int& region = lock->region.refValue();
				M_Int& border = Lock::selBorderLock->region.refValue();
				region += -Lock::copyBorderList.length;
				for(int i = 0; i < Lock::copyBorderList.length; i++){
					region += Lock::copyBorderList[i];
				}
				//region += -1;
				Lock::copyBorderList.length = 0;
				lock->houseKeep();
			}
		}
	}
	else{
		needPartMsg();
	}
}

Art* getPartFromBuildNode(Node* node){
	if(SphereBuild* sphere = dynamic_cast<SphereBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(sphere->part.p)){
			return part;
		}
	}
	else if(CylinderBuild* cylinder = dynamic_cast<CylinderBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(cylinder->part.p)){
			return part;
		}
	}
	else if(ConeBuild* cone = dynamic_cast<ConeBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(cone->part.p)){
			return part;
		}
	}
	else if(TorusBuild* torus = dynamic_cast<TorusBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(torus->part.p)){
			return part;
		}
	}
	else if(BoxBuild* box = dynamic_cast<BoxBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(box->part.p)){
			return part;
		}
	}
	else if(BoxBuild* box = dynamic_cast<BoxBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(box->part.p)){
			return part;
		}
	}
	else if(RectangleBuild* rectangle = dynamic_cast<RectangleBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(rectangle->part.p)){
			return part;
		}
	}
	else if(NurbsLathe* lathe = dynamic_cast<NurbsLathe*>(node)){
		if(Art* part = dynamic_cast<Art*>(lathe->part.p)){
			return part;
		}
	}
	else if(BezierLatheBuild* bezierLathe = dynamic_cast<BezierLatheBuild*>(node)){
		if(Art* part = dynamic_cast<Art*>(bezierLathe->part.p)){
			return part;
		}
	}
	else if(BendyLathe* bendyLathe = dynamic_cast<BendyLathe*>(node)){
		if(Art* part = dynamic_cast<Art*>(bendyLathe->part.p)){
			return part;
		}
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(node)){
		if(patch->stem.length){
			Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
			if(stem->part.length){
				if(Art* part = dynamic_cast<Art*>(stem->part[0].p)){
					return part;
				}
			}
		}
	}
	return 0;
}

/*
void buildSphereForSelPart(Art* part){
	openSceneTree();

	Vec3f from = part->absolutePivotPoint;
	Vec3f to;
	if(part->part.length){
		to = part->part.refValue()[0]->pivotPoint;
	}
	if(to.length() < .0001){
		to = Vec3f(0, .25, 0);
	}
	if(SphereBuild* sphereBuild = part->sphereFromTwoPoints(from, to)){
		if(dynamic_cast<TextureEffect*>(part->workEffect)){
			sphereBuild->textureMapper = new TextureMapper;
		}
		createNewBuildNode(sphereBuild, part->smls, part);
		if(Art* mirrorPart = part->mirrorPart){
			from.x = -from.x;
			to.x = -to.x;
			if(SphereBuild* sphereBuild = mirrorPart->sphereFromTwoPoints(from, to)){
				if(dynamic_cast<TextureEffect*>(mirrorPart->workEffect)){
					sphereBuild->textureMapper = new TextureMapper;
				}
				createNewBuildNode(sphereBuild, part->smls, mirrorPart);
			}
		}
	}
}
*/

void buildSphereForSelPart(Art* part){
	closeRecentFileWindow();
	g_addSphere.mousePhase = AddSphere::READY_FOR_LOCATION;
	SphereBuild* sphere = new SphereBuild;
	sphere->radius.refValue() = .1;
	sphere->part.refValue() = part;
	g_addSphere.sphere = sphere;

	long cursor = g_addGeometryCursor;
	if(g_scene->mode.refValue() & CREATE_MIRROR_OBJECT){
		cursor = g_addGeometryMirrorCursor;
	}
	changeModellingCursor(cursor);


	g_addSphere.houseKeep();
	
}

void buildSphereForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	if(Part* part = dynamic_cast<Part*>(g_selNode.getRaw())){
		buildSphereForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		buildSphereForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
}

void newRectangle(Art* part){
	openSceneTree();
	RectangleBuild* rectangleBuild = new RectangleBuild;
	rectangleBuild->part = part;
	if(dynamic_cast<TextureEffect*>(part->workEffect)){
		rectangleBuild->textureMapper = new TextureMapper;
	}
	createNewBuildNode(rectangleBuild, part->smls, part);
}

void newRectangle(){
	resetTemporaryModes();
	selectLastSelectedPart();
//	Part* part=new Part;
	if(Part* part = dynamic_cast<Part*>(g_selNode.getRaw())){
		newRectangle(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		newRectangle(part);
	}
	else{
		needWhitePartMsg();
	}
}

void buildConeForSelPart(Art* part){
	openSceneTree();
	ConeBuild* coneBuild= new ConeBuild;
	coneBuild->part=part;
	if(dynamic_cast<TextureEffect*>(part->workEffect)){
		coneBuild->textureMapper = new TextureMapper;
	}
	createNewBuildNode(coneBuild, part->smls, part);
}

void buildConeForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	//if(Place* place=dynamic_cast<Place*>(g_selNode.getRaw())){
	//	quantiseRotation(place->rotation);
	//	updateBuild3dWindow();
	//}
	//if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
	//	Vec3f axis = (-g_scene->rotation * -Rotation(1, 0, 0, .698132) * Vec3f(0, 0, 1)).normalize();
	//	part->rotation= part->rotation.refValue() * Rotation(axis, PI*.01);
	//	part->smls->convertToRenderFormat();
	//	return;
	//}
	if(Part* part = dynamic_cast<Part*>(g_selNode.getRaw())){
		buildConeForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		buildConeForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
}

void buildTorusForSelPart(Art* part){
	openSceneTree();
	TorusBuild* torusBuild = new TorusBuild;
	torusBuild->part=part;
	if(dynamic_cast<TextureEffect*>(part->workEffect)){
		torusBuild->textureMapper = new TextureMapper;
	}
	createNewBuildNode(torusBuild, part->smls, part);
}

void buildTorusForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		buildTorusForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		buildTorusForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
}
void buildBezierLatheForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		openSceneTree();
		BezierLatheBuild* bezierLatheBuild= new BezierLatheBuild;
		bezierLatheBuild->point.refValue()+=Vec2f(.25, 0);
		bezierLatheBuild->point.refValue()+=Vec2f(.5, .1);
		bezierLatheBuild->point.refValue()+=Vec2f(.5, .5);
		bezierLatheBuild->point.refValue()+=Vec2f(0, .3);
		bezierLatheBuild->point.refValue()+=Vec2f(0, .75);
		bezierLatheBuild->point.refValue()+=Vec2f(.15, .7);
		bezierLatheBuild->point.refValue()+=Vec2f(.34, .8);
		bezierLatheBuild->point.refValue()+=Vec2f(.25, 1);
		bezierLatheBuild->fix.refValue()+=true;
		bezierLatheBuild->fix.refValue()+=false;
		bezierLatheBuild->fix.refValue()+=false;
		bezierLatheBuild->fix.refValue()+=false;
		bezierLatheBuild->fix.refValue()+=false;
		bezierLatheBuild->fix.refValue()+=false;
		bezierLatheBuild->fix.refValue()+=false;
		bezierLatheBuild->fix.refValue()+=true;
		bezierLatheBuild->part=part;
		if(dynamic_cast<TextureEffect*>(part->workEffect)){
			bezierLatheBuild->textureMapper = new TextureMapper;
		}
		createNewBuildNode(bezierLatheBuild, part->smls, part);
	}
	else{
		needWhitePartMsg();
	}
}

void initMultiCageTug2(NurbsPatch* patch);

void NurbsLine::completeLoop(bool insert){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& point = this->point.refValue();
	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Float& weight = this->weight.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Color3f& color = this->color.refValue();
	M_Float& uKnot = this->uKnot.refValue();

	Vec3f newPoint = (point[0] + point[point.length - 1]) * .5;
	if(insert){
		point.insertElement(0, newPoint);
		dpoint.insertElement(0, newPoint);
		weight.insertElement(0, 1);
		cpIndex.insertElement(0, cp.length);
		color.insertElement(0, g_palette->color);
		cp += newPoint;
		cw += 1;
	}

	point += newPoint;
	dpoint += newPoint;
	weight += 1;
	cpIndex += cp.length;
	color += g_palette->color;
	cp += newPoint;
	cw += 1;

	updateNurbsCurveKnotMid0(point.length, uKnot);
	uJoinEnds.refValue() = true;

	if(insert){
		NurbsPatch::index[0] = index[0] = 1;
	}
	else{
		NurbsPatch::index[0] = index[0] = point.length - 2;
	}
	MultiSelect::pointList = ControlPoint(this, index[0]);



	processFields();
}
void setPreviousZPos(Vec3f& newPoint){
	Vec3f viewAxis = g_scene->rotation * Vec3f(0, 0, 1);

	if(abs(viewAxis.x) < .000001 && abs(viewAxis.y) < .000001){
		newPoint.z = g_transformCP->remCtrlPnt.z;
	}
	else if(abs(viewAxis.x) < .000001 && abs(viewAxis.z) < .000001){
		newPoint.y = g_transformCP->remCtrlPnt.y;
	}
	else if(abs(viewAxis.y) < .000001 && abs(viewAxis.z) < .000001){
		newPoint.x = g_transformCP->remCtrlPnt.x;
	}
}

void AddNurbsLine::addPoint(){
	g_viewMatrix = g_scene->getViewMatrix();

	Vec3f newPoint = g_scene->getIViewMatrix() * g_pointer.translation;
	setPreviousZPos(newPoint);


	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& point = line->point.refValue();
	M_Vec3f& dpoint = line->dpoint.refValue();
	M_Float& weight = line->weight.refValue();
	M_Int& cpIndex = line->cpIndex.refValue();
	M_Color3f& color = line->color.refValue();
	M_Float& uKnot = line->uKnot.refValue();

	if(MultiSelect::withinSightsNode){
		if(point.length > 2 && MultiSelect::withinSightsIndex == 0){
			line->completeLoop(true);
			if(NurbsLine* mirrorLine = dynamic_cast<NurbsLine*>(line->mirrorPatch)){
				mirrorLine->completeLoop(true);
			}

			smls->populateMirrorNodes();
			smls->genVisibleMask();
			smls->doBuild();


			tugFlag = true;
			showCursor(false);
			g_pointer.translation = g_viewMatrix * newPoint;
			g_cursorOverPointIlsBase->translation = g_pointer.translation;
			MultiSelect::withinSightsNode = line;
			MultiSelect::withinSightsIndex++;
			MultiSelect::beginDragPos = newPoint;
			MultiSelect::pointList = ControlPoint(line, MultiSelect::withinSightsIndex);
			MultiSelect::clickedIndex = 0;
			
			//done.refValue() = true;

			g_transformCP->translation.refValue() = Vec3f();
			g_transformCP->remTranslation = Vec3f();
			g_transformCP->clickPoint = MultiSelect::beginDragPos[MultiSelect::clickedIndex];
			g_transformCP->calcPivotPoint();

			g_stw->cp->paint();


			//addNurbsLine();
			//mousePhase = AddNurbsLine::INITIAL_POINTS;
			//setHomeMode();
			//g_stw->cp->setupNodeWindow(line);
			//Window::activeWindow = g_stw->cp.p;

			houseKeep();

			return;
		}

		tugFlag = true;
		showCursor(false);
		g_pointer.translation = g_viewMatrix * newPoint;
		g_cursorOverPointIlsBase->translation = g_pointer.translation;
		MultiSelect::withinSightsNode = this;
		line->index[0] = line->NurbsPatch::index[0] = MultiSelect::withinSightsIndex;
		MultiSelect::beginDragPos = newPoint;
		MultiSelect::pointList = ControlPoint(line, MultiSelect::withinSightsIndex);
		MultiSelect::clickedIndex = 0;

		//initMultiCageTug2(line);


		houseKeep();
		return;
	}


	if(g_addNurbsLine->line->uJoinEnds){
	//	addNurbsLine();
	//	addPoint();
		return;
	}



	int idx = line->index[0] = point.length;

	point += newPoint;
	dpoint += newPoint;
	cpIndex += cp.length;
	cp += newPoint;
	cw += 1;
	color += g_palette->color;
	weight += 1;

//changeModellingCursor(g_addPointCursor);
	tugFlag = true;
	showCursor(false);
	changeModellingCursor(g_lockedOnPointCursor);

	g_pointer.translation = g_viewMatrix * newPoint;
	g_cursorOverPointIlsBase->translation = g_pointer.translation;

	if(point.length == 2){
		uKnot = M_Float(0, 0, 0, 1, 1, 1);

		line->processFields();
		createNewBuildNode(line, smls, 0);

		if(g_scene->mode & CREATE_MIRROR_OBJECT){
			line->mirror.refValue() = true;
			int remselLine = g_stw->selLine;
			if(line->dpoint[0].x != 0){
				NurbsLine* mirrorLine = new NurbsLine;
				mirrorLine->smls = smls;
				mirrorLine->copyFields(line);
				mirrorLine->point.refValue() = line->point;
				mirrorLine->dpoint.refValue() = line->dpoint;
				mirrorLine->weight.refValue() = line->weight;
				mirrorLine->uKnot.refValue() = uKnot;
				mirrorLine->mirrorPoints();
				mirrorLine->insideOut.refValue() = true;
				mirrorLine->translation.refValue().x = -mirrorLine->translation.x;
				mirrorLine->rotation.refValue().y = -mirrorLine->rotation.y;
				mirrorLine->rotation.refValue().z = -mirrorLine->rotation.z;
				line->mirrorPatch = mirrorLine;
				mirrorLine->mirrorPatch = line;


				createNewBuildNode(mirrorLine, smls, 0);

				line->name = "r_side";
				line->makeExistingNameUniqueForMirror(mirrorLine);

				mirrorLine->createCpFromPatchPoints();


				g_stw->textCell.cellStr = mirrorLine->name;

				g_stw->selLine = remselLine;
				g_stw->textCell.cellStr = line->name;
				g_stw->paint();

				smls->populateMirrorNodes();
				line->processFields();
			}
		}




	}
	if(point.length > 2){
		updateNurbsCurveKnotMid0(point.length, uKnot);
		line->processFields();

		if(NurbsLine* mirrorLine = dynamic_cast<NurbsLine*>(line->mirrorPatch)){
			Vec3f mirrorNewPoint = newPoint;
			mirrorNewPoint.x = -mirrorNewPoint.x;
			M_Vec3f& point = mirrorLine->point.refValue();
			M_Vec3f& dpoint = mirrorLine->dpoint.refValue();
			M_Int& cpIndex = mirrorLine->cpIndex.refValue();
			M_Color3f& color = mirrorLine->color.refValue();
			mirrorLine->index[0] = point.length;
			point += mirrorNewPoint;
			dpoint += mirrorNewPoint;
			cpIndex += cp.length;
			cp += mirrorNewPoint;
			cw += 1;
			color += g_palette->color;
			mirrorLine->weight.refValue() += 1;


			updateNurbsCurveKnotMid0(mirrorLine->point.length, mirrorLine->uKnot.refValue());
			mirrorLine->processFields();
		}

	}
	if(point.length > 1){
		smls->populateMirrorNodes();
		smls->genVisibleMask();
		smls->doBuild();
	}


	MultiSelect::withinSightsNode = this;
	MultiSelect::withinSightsIndex = idx;
	MultiSelect::beginDragPos = newPoint;
	MultiSelect::pointList = ControlPoint(line, idx);
	MultiSelect::clickedIndex = 0;

	houseKeep();
}

void DrawNurbsLine::addPoint(){
	g_viewMatrix = g_scene->getViewMatrix();

	Vec3f newPoint = g_scene->getIViewMatrix() * g_pointer.translation;
	setPreviousZPos(newPoint);

	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& point = line->point.refValue();
	M_Vec3f& dpoint = line->dpoint.refValue();
	M_Float& weight = line->weight.refValue();
	M_Int& cpIndex = line->cpIndex.refValue();
	M_Color3f& color = line->color.refValue();
	M_Float& uKnot = line->uKnot.refValue();

	if(MultiSelect::withinSightsNode){
		if(point.length > 2 && MultiSelect::withinSightsIndex == 0){
			line->completeLoop(false);
			if(NurbsLine* mirrorLine = dynamic_cast<NurbsLine*>(line->mirrorPatch)){
				mirrorLine->completeLoop(false);
			}

			smls->populateMirrorNodes();
			smls->genVisibleMask();
			smls->doBuild();


			tugFlag = true;
			showCursor(false);
			g_pointer.translation = g_viewMatrix * newPoint;
			g_cursorOverPointIlsBase->translation = g_pointer.translation;
			MultiSelect::withinSightsNode = line;
			MultiSelect::withinSightsIndex++;
			MultiSelect::beginDragPos = newPoint;
			MultiSelect::pointList = ControlPoint(line, MultiSelect::withinSightsIndex);
			MultiSelect::clickedIndex = 0;
			
			done = true;

			g_transformCP->translation.refValue() = Vec3f();
			g_transformCP->remTranslation = Vec3f();
			g_transformCP->clickPoint = MultiSelect::beginDragPos[MultiSelect::clickedIndex];
			g_transformCP->calcPivotPoint();

			g_stw->cp->paint();


			g_stw->cp->setupNodeWindow(line);
			//Window::activeWindow = g_stw->cp.p;

			return;
		}
		tugFlag = true;
		showCursor(false);
		g_pointer.translation = g_viewMatrix * newPoint;
		g_cursorOverPointIlsBase->translation = g_pointer.translation;
		MultiSelect::withinSightsNode = this;
		line->index[0] = MultiSelect::withinSightsIndex;
		MultiSelect::beginDragPos = newPoint;
		MultiSelect::pointList = ControlPoint(line, MultiSelect::withinSightsIndex);
		MultiSelect::clickedIndex = 0;

		houseKeep();
		enablePen();
		return;
	}

	int idx = line->index[0] = point.length;

	point += newPoint;
	dpoint += newPoint;
	cpIndex += cp.length;
	cp += newPoint;
	cw += 1;
	color += g_palette->color;
	weight += 1;

//changeModellingCursor(g_addPointCursor);
	tugFlag = true;
	showCursor(false);
	changeModellingCursor(g_lockedOnPointCursor);

	g_pointer.translation = g_viewMatrix * newPoint;
	g_cursorOverPointIlsBase->translation = g_pointer.translation;

	//if(point.length > 1){
	//	smls->populateMirrorNodes();
	//	smls->genVisibleMask();
	//	smls->doBuild();
	//}


	MultiSelect::withinSightsNode = this;
	MultiSelect::withinSightsIndex = idx;
	MultiSelect::beginDragPos = newPoint;
	MultiSelect::pointList = ControlPoint(line, idx);
	MultiSelect::clickedIndex = 0;

	houseKeep();
	enablePen();
}

void Grid::snapIt(Vec3f& par){
	if(snap){
		float a = 1.0 / subSquares * scale;
		par = Vec3f(floor(par.x / a + .5), floor(par.y / a + .5), floor(par.z / a + .5)) * a;
	}
}


void Grid::snapIt(Vec2f& par){
	if(snap){
		float a = 1.0 / subSquares * scale;
		par = Vec2f(floor(par.x / a + .5), floor(par.y / a + .5)) * a;
	}
}

void gridPlanePointsA(Vec3f& cornerA, Vec3f& cornerB, Vec3f& cornerC, Vec3f& cornerD){
	float width = 10000;//g_grid->scale * g_grid->squares;
	float halfWidth = width / 2.0;
	if(g_grid->xzPlane){
		cornerA = Vec3f(-halfWidth, 0, halfWidth);
		cornerB = Vec3f(halfWidth, 0, halfWidth);
		cornerC = Vec3f(halfWidth, 0, -halfWidth);
		cornerD = Vec3f(-halfWidth, 0, -halfWidth);
	}
	else if(g_grid->xyPlane){
		cornerA = Vec3f(-halfWidth, -halfWidth, 0);
		cornerB = Vec3f(halfWidth, -halfWidth, 0);
		cornerC = Vec3f(halfWidth, halfWidth, 0);
		cornerD = Vec3f(-halfWidth, halfWidth, 0);
	}
	else{
		cornerA = Vec3f(0, -halfWidth, -halfWidth);
		cornerB = Vec3f(0, halfWidth, -halfWidth);
		cornerC = Vec3f(0, halfWidth, halfWidth);
		cornerD = Vec3f(0, -halfWidth, halfWidth);
	}
}


void gridPlanePointsC(Vec3f& cornerA, Vec3f& cornerB, Vec3f& cornerC, Vec3f& cornerD, const Matrix& matrix, const Vec3f& clickPosB){
	float width = 10000;//g_grid->scale * g_grid->squares;
	float halfWidth = width / 2.0;
	if(g_grid->xzPlane){
		cornerA = matrix * Vec3f(-halfWidth, -halfWidth, clickPosB.z);
		cornerB = matrix * Vec3f(halfWidth, -halfWidth, clickPosB.z);
		cornerC = matrix * Vec3f(halfWidth, halfWidth, clickPosB.z);
		cornerD = matrix * Vec3f(-halfWidth, halfWidth, clickPosB.z);
	}
	else if(g_grid->xyPlane){
		cornerA = matrix * Vec3f(-halfWidth, clickPosB.y, halfWidth);
		cornerB = matrix * Vec3f( halfWidth, clickPosB.y, halfWidth);
		cornerC = matrix * Vec3f( halfWidth, clickPosB.y, -halfWidth);
		cornerD = matrix * Vec3f(-halfWidth, clickPosB.y, -halfWidth);
	}
	else{
		cornerA = matrix * Vec3f(-halfWidth, -halfWidth, clickPosB.z);
		cornerB = matrix * Vec3f(halfWidth, -halfWidth, clickPosB.z);
		cornerC = matrix * Vec3f(halfWidth, halfWidth, clickPosB.z);
		cornerD = matrix * Vec3f(-halfWidth, halfWidth, clickPosB.z);
	}
}

void gridPlanePointsForLathe(Vec3f& cornerA, Vec3f& cornerB, Vec3f& cornerC, Vec3f& cornerD, const Matrix& matrix){
	float width = 10000;//g_grid->scale * g_grid->squares;
	float halfWidth = width / 2.0;
	if(g_grid->xzPlane){
		cornerA = matrix * Vec3f(-halfWidth, -halfWidth, 0);
		cornerB = matrix * Vec3f(halfWidth, -halfWidth, 0);
		cornerC = matrix * Vec3f(halfWidth, halfWidth, 0);
		cornerD = matrix * Vec3f(-halfWidth, halfWidth, 0);
	}
	else if(g_grid->xyPlane){
		//cornerA = matrix * Vec3f(-halfWidth, 0, halfWidth);
		//cornerB = matrix * Vec3f( halfWidth, 0, halfWidth);
		//cornerC = matrix * Vec3f( halfWidth, 0, -halfWidth);
		//cornerD = matrix * Vec3f(-halfWidth, 0, -halfWidth);
		cornerA = matrix * Vec3f(-halfWidth, -halfWidth, 0);
		cornerB = matrix * Vec3f(halfWidth, -halfWidth, 0);
		cornerC = matrix * Vec3f(halfWidth, halfWidth, 0);
		cornerD = matrix * Vec3f(-halfWidth, halfWidth, 0);
	}
	else{
		cornerA = matrix * Vec3f(-halfWidth, -halfWidth, 0);
		cornerB = matrix * Vec3f(halfWidth, -halfWidth, 0);
		cornerC = matrix * Vec3f(halfWidth, halfWidth, 0);
		cornerD = matrix * Vec3f(-halfWidth, halfWidth, 0);
	}
}


bool isGridHit(Vec3f& clickPos, Vec3f cornerA, Vec3f cornerB, Vec3f cornerC, Vec3f cornerD){
	if(	isPointInTriangleHit(clickPos, cornerA, cornerB, cornerC, g_perspective) ||
		isPointInTriangleHit(clickPos, cornerC, cornerD, cornerA, g_perspective) ||
		isPointInTriangleHit(clickPos, cornerA, cornerC, cornerB, g_perspective) ||
		isPointInTriangleHit(clickPos, cornerC, cornerA, cornerD, g_perspective)
		){
		return true;
	}
	return false;
}


void AddBox::updateDimensions(){
	g_viewMatrix = g_scene->getViewMatrix();
	Vec3f cornerA;
	Vec3f cornerB;
	Vec3f cornerC;
	Vec3f cornerD;

	Matrix iMatrix = translationMatrix(-box->translation) * rotationMatrix(-box->rotation);
	Matrix matrix = rotationMatrix(box->rotation) * translationMatrix(box->translation);
	//Vec3f vec = g_scene->getIViewMatrix() * imatrix * g_pointer.translation;

	float middleSnapError = g_grid->scale / g_grid->subSquares;

	if(mousePhase == READY_FOR_B){
		gridPlanePointsA(cornerA, cornerB, cornerC, cornerD);

		if(isGridHit(clickPosB, cornerA, cornerB, cornerC, cornerD)){
			if(g_grid->xzPlane){
				box->scale.x = clickPosB.x - box->translation.x;
				box->scale.z = clickPosB.z - box->translation.z;
			}
			else if(g_grid->xyPlane){
				box->scale.x = clickPosB.x - box->translation.x;
				box->scale.y = -clickPosB.y + box->translation.y;
			}
			else{
				box->scale.y = clickPosB.y - box->translation.y;
				box->scale.z = clickPosB.z - box->translation.z;
			}


			processFieldsPlus(*box);
		}
		Art* part = dynamic_cast<Art*>(box->part.p);
		Seamless* smls = part->smls;

		clickPosB = iMatrix * clickPosB;
	}
	else if(mousePhase == READY_FOR_C){

		gridPlanePointsC(cornerA, cornerB, cornerC, cornerD, matrix, clickPosB);
		
		if(isGridHit(clickPosC, cornerA, cornerB, cornerC, cornerD)){
			clickPosC = iMatrix * clickPosC;
			if(g_grid->xzPlane){
				box->scale.y = clickPosC.y * .5;
			}
			else if(g_grid->xyPlane){
				box->scale.z = clickPosC.z * .5;
			}
			else{
				if(box->xCentred){
					box->scale.x = clickPosC.x;
				}
				else{
					box->scale.x = clickPosC.x * .5;
				}
			}
			
			processFieldsPlus(*box);
		}
	}
	if(BoxBuild* mirrorContainer = box->mirrorContainer){
		mirrorContainer->scale.refValue() = box->scale;
		processFieldsPlus(*mirrorContainer);
	}
}

Vec3f gridBoxStartOffset(){
	if(g_grid->xzPlane){
		return Vec3f(.1, .001, .1);
	}
	else if(g_grid->xyPlane){
		return Vec3f(.1, -.1, .001);
	}
	else{
		return Vec3f(.001, .1, .1);
	}
}

void AddBox::addPoint(){
	g_viewMatrix = g_scene->getViewMatrix();

	Matrix iMatrix = translationMatrix(-box->translation) * rotationMatrix(-box->rotation);
	Matrix matrix = rotationMatrix(box->rotation) * translationMatrix(box->translation);
	//Vec3f vec = g_scene->getIViewMatrix() * imatrix * g_pointer.translation;

	float middleSnapError = g_grid->scale / g_grid->subSquares;


	if(mousePhase == READY_FOR_A){
		mousePhase = READY_FOR_B;
		changeModellingCursor(g_addPointCursor);

		Vec3f cornerA;
		Vec3f cornerB;
		Vec3f cornerC;
		Vec3f cornerD;
		gridPlanePointsA(cornerA, cornerB, cornerC, cornerD);

		if(isGridHit(clickPosA, cornerA, cornerB, cornerC, cornerD)){
			if((g_scene->mode & CREATE_MIRROR_OBJECT) && abs(clickPosA.x) < middleSnapError){
				clickPosA.x = 0;
			}
		}
		Art* part = dynamic_cast<Art*>(box->part.p);
		Seamless* smls = part->smls;

		g_grid->snapIt(clickPosA);
		box->translation.refValue() = clickPosA;
		g_pointer.translation = g_scene->getViewMatrix() * (box->translation + gridBoxStartOffset());

		get2dMousePosFrom3d();

		///////////

		if(dynamic_cast<TextureEffect*>(part->workEffect)){
			box->textureMapper = new TextureMapper;
		}
		


		if(g_scene->mode & CREATE_MIRROR_OBJECT){
			box->mirror.refValue() = true;
			BoxBuild* mirrorContainer = 0;
			if(box->translation.x != 0){
				mirrorContainer = new BoxBuild;
				mirrorContainer->copyFields2(box);
				box->mirrorContainer = mirrorContainer;
			}
			createNewBuildNode(box, part->smls, part);
			int remselLine = g_stw->selLine;
			if(box->translation.x != 0){
				
				//mirrorContainer->mirrorPoints();
				mirrorContainer->insideOut.refValue() = true;
				mirrorContainer->translation.refValue().x = -mirrorContainer->translation.x;
				mirrorContainer->rotation.refValue().y = -mirrorContainer->rotation.y;
				mirrorContainer->rotation.refValue().z = -mirrorContainer->rotation.z;
				mirrorContainer->mirrorContainer = box;


				Art* mirrorPart = part->mirrorPart;
				if(mirrorPart == 0){
					mirrorPart = part;
				}
				if(dynamic_cast<TextureEffect*>(mirrorPart->workEffect)){
					mirrorContainer->textureMapper = new TextureMapper;
				}
					
				createNewBuildNode(mirrorContainer, part->smls, mirrorPart);

				box->name = "r_side";
				makeExistingNameUnique(box);
				mirrorContainer->name = "l";
				mirrorContainer->name += box->name.substr(1);
				makeExistingNameUnique(mirrorContainer);
				g_stw->textCell.cellStr = mirrorContainer->name;

				g_stw->selLine = remselLine;
				g_stw->textCell.cellStr = box->name;
				g_stw->paint();

				smls->populateMirrorNodes();
				box->processFields();
			}
		}
		else{
			createNewBuildNode(box, part->smls, part);
		}
	}
	else if(mousePhase == READY_FOR_B){
		updateDimensions();
		mousePhase = READY_FOR_C;
	}
	else if(mousePhase == READY_FOR_C){
		updateDimensions();
		mousePhase = NEUTRAL;
		changeModellingCursor(g_crossCursor);
	}
}


void AddSphere::updateRadius(){
	g_viewMatrix = g_scene->getViewMatrix();

	Matrix iMatrix = translationMatrix(-sphere->translation) * rotationMatrix(-sphere->rotation);
	Matrix matrix = rotationMatrix(sphere->rotation) * translationMatrix(sphere->translation);
	//Vec3f vec = g_scene->getIViewMatrix() * imatrix * g_pointer.translation;

	Vec3f clickPos;

	Vec3f cornerA;
	Vec3f cornerB;
	Vec3f cornerC;
	Vec3f cornerD;

	gridPlanePointsA(cornerA, cornerB, cornerC, cornerD);

	if(isGridHit(clickPos, cornerA, cornerB, cornerC, cornerD)){
		sphere->radius = (clickPos - sphere->translation ).length();
		processFieldsPlus(*sphere);
		if(SphereBuild* mirrorContainer = sphere->mirrorContainer){
			mirrorContainer->radius = sphere->radius;
			processFieldsPlus(*mirrorContainer);
		}
	}
}

Vec3f gridSphereOffset(){
	if(g_grid->xzPlane){
		return Vec3f(.1, 0, 0);
	}
	else if(g_grid->xyPlane){
		return Vec3f(.1, 0, 0);
	}
	else{
		return Vec3f(0, 0, -.1);
	}
}

void AddSphere::addPoint(){
	g_viewMatrix = g_scene->getViewMatrix();

	Matrix iMatrix = translationMatrix(-sphere->translation) * rotationMatrix(-sphere->rotation);
	Matrix matrix = rotationMatrix(sphere->rotation) * translationMatrix(sphere->translation);
	//Vec3f vec = g_scene->getIViewMatrix() * imatrix * g_pointer.translation;

	Vec3f clickPos;
	float middleSnapError = g_grid->scale / g_grid->subSquares;

	if(mousePhase == READY_FOR_LOCATION){
		mousePhase = READY_FOR_RADIUS;
		changeModellingCursor(g_addPointCursor);

		Vec3f cornerA;
		Vec3f cornerB;
		Vec3f cornerC;
		Vec3f cornerD;
		gridPlanePointsA(cornerA, cornerB, cornerC, cornerD);

		if(isGridHit(clickPos, cornerA, cornerB, cornerC, cornerD)){
			if((g_scene->mode & CREATE_MIRROR_OBJECT) && abs(clickPos.x) < middleSnapError){
				clickPos.x = 0;
			}
		}
		Art* part = dynamic_cast<Art*>(sphere->part.p);
		Seamless* smls = part->smls;

		g_grid->snapIt(clickPos);
		sphere->translation.refValue() = clickPos;
		///////////
		g_pointer.translation = g_scene->getViewMatrix() * (sphere->translation + gridSphereOffset());
		get2dMousePosFrom3d();

		if(dynamic_cast<TextureEffect*>(part->workEffect)){
			sphere->textureMapper = new TextureMapper;
		}
		createNewBuildNode(sphere, part->smls, part);


		if(g_scene->mode & CREATE_MIRROR_OBJECT){
			sphere->mirror.refValue() = true;
			int remselLine = g_stw->selLine;
			if(sphere->translation.x != 0){
				SphereBuild* mirrorContainer = new SphereBuild;
				mirrorContainer->copyFields(sphere);
				//mirrorContainer->mirrorPoints();
				mirrorContainer->insideOut.refValue() = true;
				mirrorContainer->translation.refValue().x = -mirrorContainer->translation.x;
				mirrorContainer->rotation.refValue().y = -mirrorContainer->rotation.y;
				mirrorContainer->rotation.refValue().z = -mirrorContainer->rotation.z;
				sphere->mirrorContainer = mirrorContainer;
				mirrorContainer->mirrorContainer = sphere;


				Art* mirrorPart = part->mirrorPart;
				if(mirrorPart == 0){
					mirrorPart = part;
				}
				mirrorContainer->part.refValue() = mirrorPart;

				if(dynamic_cast<TextureEffect*>(mirrorPart->workEffect)){
					mirrorContainer->textureMapper = new TextureMapper;
				}
					
				createNewBuildNode(mirrorContainer, part->smls, mirrorPart);

				sphere->name = "r_side";
				makeExistingNameUnique(sphere);
				mirrorContainer->name = "l";
				mirrorContainer->name += sphere->name.substr(1);
				makeExistingNameUnique(mirrorContainer);
				g_stw->textCell.cellStr = mirrorContainer->name;

				g_stw->selLine = remselLine;
				g_stw->textCell.cellStr = sphere->name;
				g_stw->paint();

				smls->populateMirrorNodes();
				sphere->processFields();
			}
		}
	}
	else if(mousePhase == READY_FOR_RADIUS){
		mousePhase = NEUTRAL;
		changeModellingCursor(g_crossCursor);
	}
}



void AddLathe::addPoint(){
	//Vec3f newAbsolutePivotPoint = g_scene->getIViewMatrix() * g_catchIMatrix * part->imatrix * g_pointer.translation;
	g_viewMatrix = g_scene->getViewMatrix();

	//Vec3f vec = g_scene->getIViewMatrix() * imatrix * g_pointer.translation;
	if(MultiSelect::withinSightsNode == this && mousePhase == INITIAL_POINTS){
		if(false == controlKeyState()){
			tugFlag = true;
			showCursor(false);
			g_addLathe->lathe->index[0] = MultiSelect::clickedIndex;
			g_pointer.translation = g_cursorOverPointIlsBase->translation;
			return;
		}
	}

	Vec3f clickPos;
	float middleSnapError = g_grid->scale / g_grid->subSquares;

	if(mousePhase == CREATE_SPINDLE){
		MultiSelect::withinSightsNode = 0;
		mousePhase = SET_SPINDLE_ORIENTATION;

		changeModellingCursor(g_addPointCursor);

		//lathe->rotation.refValue() = -g_scene->rotation;
		//vec.y = 0;

		Vec3f cornerA;
		Vec3f cornerB;
		Vec3f cornerC;
		Vec3f cornerD;
		gridPlanePointsA(cornerA, cornerB, cornerC, cornerD);


		if(isGridHit(clickPos, cornerA, cornerB, cornerC, cornerD)){
			if((g_scene->mode & CREATE_MIRROR_OBJECT) && abs(clickPos.x) < middleSnapError){
				clickPos.x = 0;
			}
			g_grid->snapIt(clickPos);
			lathe->translation.refValue() = clickPos;
		}
	}
	else{
		if(mousePhase == SET_SPINDLE_ORIENTATION){
			mousePhase = INITIAL_POINTS;
			if(MultiSelect::withinSights){
				MultiSelect::withinSightsNode = 0;
				lathe->rotation.refValue() = orientationArray[MultiSelect::withinSightsIndex];
//				if(g_grid->xyPlane){
					//lathe->rotation.refValue() = Rotation(1, 0, 0, PI * .5) * lathe->rotation;
//				}
//				else if(g_grid->yzPlane){
					//lathe->rotation.refValue() = Rotation(0, 1, 0, -PI * .5) * lathe->rotation;
//				}


				MultiSelect::resetDistance();
				MultiSelect::distance = 1000000;
				MultiSelect::withinSightsNode = 0;
				changeModellingCursor(g_addPointCursor);

				houseKeep();
				return;
			}
		}
		Matrix iMatrix = translationMatrix(-lathe->translation) * rotationMatrix(-lathe->rotation);
		Matrix matrix = rotationMatrix(lathe->rotation) * translationMatrix(lathe->translation);
		lathe->matrix = matrix;
		lathe->imatrix = iMatrix;

		Vec3f cornerA;
		Vec3f cornerB;
		Vec3f cornerC;
		Vec3f cornerD;
		gridPlanePointsForLathe(cornerA, cornerB, cornerC, cornerD, matrix);


		if(isGridHit(clickPos, cornerA, cornerB, cornerC, cornerD)){
			clickPos = iMatrix * clickPos;
			g_grid->snapIt(clickPos);

			Vec2f newPoint;

			if(g_grid->xzPlane){
				newPoint = Vec2f(clickPos.x, clickPos.y);
			}
			else if(g_grid->xyPlane){
				newPoint = Vec2f(clickPos.x, clickPos.y);
			}
			else{
				newPoint = Vec2f(clickPos.x, clickPos.y);
			}


			M_Vec2f& point = lathe->point;
			M_Vec2f& dpoint = lathe->dpoint;
			M_Float& weight = lathe->weight;

			int last = point.length - 1;
			if(point.length > 2){
				point += point[last];
				dpoint += point[last];
				weight += 1;
			}
				
			point += newPoint;
			dpoint += newPoint;
			weight += 1;
			lathe->index[0] = point.length - 1;

			Art* part = dynamic_cast<Art*>(lathe->part.p);
			Seamless* smls = part->smls;

			lathe->bands.refValue() = lathe->dpoint.length / 2 * smls->tesselate;

			tugFlag = true;
			showCursor(false);

			g_pointer.translation = matrix * g_scene->getViewMatrix() * clickPos;
			g_cursorOverPointIlsBase->translation = g_pointer.translation;

			MultiSelect::withinSightsNode = this;

			if(point.length == 3){
				lathe->knot.refValue() = M_Float(0, 0, 0, 1, 1, 1);

				if(dynamic_cast<TextureEffect*>(part->workEffect)){
					lathe->textureMapper = new TextureMapper;
				}
				createNewBuildNode(lathe, part->smls, part);


				if(g_scene->mode & CREATE_MIRROR_OBJECT){
					lathe->mirror.refValue() = true;
					int remselLine = g_stw->selLine;
					if(lathe->translation.x != 0){
						NurbsLathe* mirrorContainer = new NurbsLathe;
						mirrorContainer->copyFields(lathe);
						mirrorContainer->point.refValue() = lathe->point;
						mirrorContainer->dpoint.refValue() = lathe->dpoint;
						mirrorContainer->weight.refValue() = lathe->weight;
						mirrorContainer->knot.refValue() = lathe->knot;
						mirrorContainer->mirrorPoints();
						mirrorContainer->insideOut.refValue() = true;
						mirrorContainer->translation.refValue().x = -mirrorContainer->translation.x;
						mirrorContainer->rotation.refValue().y = -mirrorContainer->rotation.y;
						mirrorContainer->rotation.refValue().z = -mirrorContainer->rotation.z;
						lathe->mirrorContainer = mirrorContainer;
						mirrorContainer->mirrorContainer = lathe;


						Art* mirrorPart = part->mirrorPart;
						if(mirrorPart == 0){
							mirrorPart = part;
						}
						mirrorContainer->part.refValue() = mirrorPart;


						if(dynamic_cast<TextureEffect*>(mirrorPart->workEffect)){
							mirrorContainer->textureMapper = new TextureMapper;
						}
					
						createNewBuildNode(mirrorContainer, part->smls, mirrorPart);

						lathe->name = "r_side";
						makeExistingNameUnique(lathe);
						mirrorContainer->name = "l";
						mirrorContainer->name += lathe->name.substr(1);
						makeExistingNameUnique(mirrorContainer);
						g_stw->textCell.cellStr = mirrorContainer->name;

						g_stw->selLine = remselLine;
						g_stw->textCell.cellStr = lathe->name;
						g_stw->paint();

						smls->populateMirrorNodes();
						lathe->processFields();
					}
				}


				g_stw->cp->setupNodeWindow(g_addLathe);
				Window::activeWindow = g_stw->cp.p;
			}
			if(point.length > 3){
		
				updateNurbsCurveKnotMid1(lathe->point.length, lathe->knot.refValue());
				lathe->processFields();

				if(NurbsLathe* mirrorContainer = lathe->mirrorContainer){
					M_Vec2f& point = mirrorContainer->point;
					M_Vec2f& dpoint = mirrorContainer->dpoint;
					M_Float& weight = mirrorContainer->weight;

					newPoint.x = -newPoint.x;

					point += point[last];
					dpoint += mirrorContainer->point[last];
					weight += 1;

					point += newPoint;
					dpoint += newPoint;
					weight += 1;

					mirrorContainer->bands.refValue() = lathe->bands;
					mirrorContainer->knot.refValue() = lathe->knot;
					mirrorContainer->weight.refValue() += 1;
					mirrorContainer->processFields();
				}


				smls->populateMirrorNodes();
				//smls->cpNodeListAdd();

				initBnspePolies(lathe, part);
				if(NurbsLathe* mirrorContainer = lathe->mirrorContainer){
					initBnspePolies(lathe->mirrorContainer, part);
				}
			}
		}
	}
	if(mousePhase == SET_SPINDLE_ORIENTATION || mousePhase == INITIAL_POINTS){
		houseKeep();
	}
}
void AddNurbsLine::newLine(){
	resetTemporaryModes();

	long cursor = g_scene->mode.refValue() & CREATE_MIRROR_OBJECT ? g_addGeometryMirrorCursor : g_addGeometryCursor;
	changeModellingCursor(cursor);

	g_addNurbsLine->smls = smls;
	g_addNurbsLine->mousePhase = AddNurbsLine::INITIAL_POINTS;
	line = new NurbsLine();
	line->smls = smls;

	line->point.refValue().length = 0;
	line->dpoint.refValue().length = 0;
	houseKeep();
}


void addNurbsLine(){
	resetTemporaryModes();
	if(Seamless* smls = getSeamless()){
		g_addNurbsLine->smls = smls;
		g_addNurbsLine->newLine();
		g_stw->cp->setupNodeWindow(g_addNurbsLine);
		Window::activeWindow = g_stw->cp.p;
	}
}
void DrawNurbsLine::newLine(){
	resetTemporaryModes();

	long cursor = g_scene->mode.refValue() & CREATE_MIRROR_OBJECT ? g_addGeometryMirrorCursor : g_addGeometryCursor;
	changeModellingCursor(cursor);

	pixelSketch.length = 0;
	m_pressure.length = 0;
	
	mousePhase = DrawNurbsLine::DRAW;
	line = new NurbsLine();
	patch = 0;
	line->smls = smls;

	line->point.refValue().length = 0;
	line->dpoint.refValue().length = 0;
	houseKeep();
}

void drawNurbsLine(){
	resetTemporaryModes();
	if(Seamless* smls = getSeamless()){
		g_drawNurbsLine->smls = smls;
		g_drawNurbsLine->newLine();

		g_stw->cp->setupNodeWindow(g_drawNurbsLine);
		Window::activeWindow = g_stw->cp.p;

	}
}

void buildNurbsLatheForSelPart(Art* part){
	closeRecentFileWindow();
	g_addLathe->mousePhase = AddLathe::CREATE_SPINDLE;
	NurbsLathe* lathe = new NurbsLathe;
	lathe->part.refValue() = part;
	lathe->point.refValue().length = 0;
	lathe->dpoint.refValue().length = 0;
	lathe->beginSP.refValue() = false;
	lathe->endSP.refValue() = false;

	//lathe->rotation.refValue() = -g_scene->rotation;
	g_addLathe->lathe = lathe;

	long cursor = g_scene->mode.refValue() & CREATE_MIRROR_OBJECT ? g_addGeometryMirrorCursor : g_addGeometryCursor;
	changeModellingCursor(cursor);

	//openSceneTree2();
	//g_addLathe->houseKeep();


	//NurbsLathe* nurbsLathe= new NurbsLathe;
	//nurbsLathe->beginSP.refValue() = false;
	//nurbsLathe->endSP.refValue() = false;
	//nurbsLathe->bands.refValue() = 25;
	//if(part->isRightMirror()){
	//	nurbsLathe->mirrorPoints();
	//}
	//nurbsLathe->part = part;

	//if(dynamic_cast<TextureEffect*>(part->workEffect)){
	//	nurbsLathe->textureMapper = new TextureMapper;
	//}
	//createNewBuildNode(nurbsLathe, part->smls, part);
}

void buildNurbsLatheForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	Node* selNode = g_selNode.getRaw();
	if(Part* part = dynamic_cast<Part*>(selNode)){
		buildNurbsLatheForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(selNode)){
		buildNurbsLatheForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
}



void buildAvSegmentForSelPart(Art* part){
	openSceneTree();

	Vec3f from = part->absolutePivotPoint;
	Vec3f to;
	if(part->part.length){
		to = part->part.refValue()[0]->pivotPoint;
	}


	NurbsLathe* nurbsLathe = part->latheFromTwoPoints(from, to);
	if(nurbsLathe == 0){
		nurbsLathe = new NurbsLathe;
		nurbsLathe->part.refValue() = part;
		nurbsLathe->point.refValue() = M_Vec2f(Vec2f(0, 0), Vec2f(.05, 0), Vec2f(.05, .15), Vec2f(.05, .3), Vec2f(0, .3));
		nurbsLathe->weight.refValue() = M_Float(1, 1, 1, 1, 1);
		nurbsLathe->knot.refValue() = M_Float(0, 0, 0, 1, 2, 3, 3, 3);
		nurbsLathe->translation.refValue() = from;
		if(part->isRightMirror()){
			nurbsLathe->mirrorPoints();
		}
		nurbsLathe->processFields();
	}

	if(dynamic_cast<TextureEffect*>(part->workEffect)){
		nurbsLathe->textureMapper = new TextureMapper;
	}
	createNewBuildNode(nurbsLathe, part->smls, part);
}

void buildAvSegmentForSelPart(){
	if(Part* part = dynamic_cast<Part*>(g_selNode.getRaw())){
		buildAvSegmentForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		buildAvSegmentForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
}

void buildModelMorpherForSelSmls(){
	openSceneTree();
	if(Seamless* smls=dynamic_cast<Seamless*>(g_selNode.getRaw())){
		buildModelMorpherForSelSmls(smls);
	}
	else if(dynamic_cast<Art*>(g_selNode.getRaw())||dynamic_cast<Effect*>(g_selNode.getRaw())||dynamic_cast<Build*>(g_selNode.getRaw())||dynamic_cast<Generator*>(g_selNode.getRaw())){
		int selLine = g_stw->selLine;
		if(Seamless* smls=getSeamless(selLine)){
			g_stw->selLine=selLine;
			buildModelMorpherForSelSmls(smls);
		}
	}
	else{
		needPartOrSmlsMsg();
	}
}

void buildBendyLatheForSelPart(Art* part){
	openSceneTree();
	BendyLathe* bendyLathe= new BendyLathe;
	bendyLathe->bands.refValue()=25;
	bendyLathe->part=part;
	if(dynamic_cast<TextureEffect*>(part->workEffect)){
		bendyLathe->textureMapper = new TextureMapper;
	}
	createNewBuildNode(bendyLathe, part->smls, part);
}

void buildBendyLatheForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		buildBendyLatheForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		buildBendyLatheForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
}



void addHairNodeForSelPart(){
	disabledMessage();
	return;
	resetTemporaryModes();
	selectLastSelectedPart();

	if(Part* part = dynamic_cast<Part*>(g_selNode.getRaw())){
		openSceneTree();
		Hair* hair = new Hair;
		hair->part=part;
		createNewBuildNode(hair, part->smls, part);
	}
}


void buildSubdivisionForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	openSceneTree();
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		Subdivision* subdivision = new Subdivision;
		subdivision->part=part;
		createNewBuildNode(subdivision, part->smls, part);
	}
	else{
		needWhitePartMsg();
	}
}
void buildRingsForSelPart(){
	
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		openSceneTree();
		RingBuild* ringBuild = new RingBuild;
		ringBuild->scaleX=M_Vec2f(Vec2f(.25, .25), Vec2f(.25, .25));
		ringBuild->scaleZ=M_Vec2f(Vec2f(.25, .25), Vec2f(.25, .25));
		ringBuild->orientation=M_Rotation(Rotation(0,0,1,0), Rotation(0,0,1,0));
		ringBuild->location=M_Vec3f(Vec3f(), Vec3f(0, .5, 0));
		ringBuild->part.refValue()+=part;
		createNewBuildNode(ringBuild, part->smls, part);
	}
	else{
		needWhitePartMsg();
	}
}

///////back to here
void buildBoxForSelPart(Art* part){
	closeRecentFileWindow();
	g_addBox.mousePhase = AddBox::READY_FOR_A;
	BoxBuild* box = new BoxBuild;

	if(g_grid->xzPlane){
		box->scale.refValue() = Vec3f(.1, .001, .1);
	}
	else if(g_grid->xyPlane){
		box->xCentred.refValue() = true;
		box->yCentred.refValue() = true;
		box->zCentred.refValue() = false;
		box->scale.refValue() = Vec3f(.1, .1, .001);
	}
	else{
		if(g_scene->mode.refValue() & CREATE_MIRROR_OBJECT){
			box->xCentred.refValue() = true;
		}
		else{
			box->xCentred.refValue() = false;
		}
		box->yCentred.refValue() = true;
		box->zCentred.refValue() = true;
		box->scale.refValue() = Vec3f(.001, .1, .1);
	}

	

	box->part.refValue() = part;
	//lathe->rotation.refValue() = -g_scene->rotation;
	g_addBox.box = box;
	//openSceneTree2();

	long cursor = g_addGeometryCursor;
	if(g_scene->mode.refValue() & CREATE_MIRROR_OBJECT){
		cursor = g_addGeometryMirrorCursor;
	}
	changeModellingCursor(cursor);


	g_addBox.houseKeep();
	//if(dynamic_cast<TextureEffect*>(part->workEffect)){
	//	for(int k=6; --k>=0;){
	//		boxBuild->textureMapper.refValue() += new TextureMapper;
	//	}
	//}
	
}

void buildBoxForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		buildBoxForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		buildBoxForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
}

void buildCylinderForSelPart(Art* part){
	resetTemporaryModes();
	openSceneTree();
	CylinderBuild* cylinderBuild = new CylinderBuild;
	cylinderBuild->part=part;
	if(dynamic_cast<TextureEffect*>(part->workEffect)){
		cylinderBuild->textureMapper = new TextureMapper;
	}
	createNewBuildNode(cylinderBuild, part->smls, part);
}

void buildCylinderForSelPart(){
	resetTemporaryModes();
	selectLastSelectedPart();
	//Part* part=new Part;
	if(Part* part=dynamic_cast<Part*>(g_selNode.getRaw())){
		buildCylinderForSelPart(part);
	}
	else if(Art* part = getPartFromBuildNode(g_selNode.getRaw())){
		buildCylinderForSelPart(part);
	}
	else{
		needWhitePartMsg();
	}
/*
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		buildCylinderForSelPart(part);
		BackUp::monostable.beginCycle();
		g_pw->setupNodeWindow(g_immediateCylinder);
		Window::activeWindow = g_pw;
	}
	else{
		needPartMsg();
	}
	*/
}
void SceneTree::decMultipleNodeSelection(){
	if(g_selCount>1){
		int collapsedLine;
		if(getCollapsedLine(g_stw->selLine, collapsedLine)){
			g_selCount--;
			if(g_selCount.all>g_stw->numOfVisLines){
				g_scene->scroll=collapsedLine+g_selCount.all-1;
			}
			g_stw->setScrolPos(0);
		}
	}
}
void SceneTree::incMultipleNodeSelection(){
	int collapsedLine;
	if(getCollapsedLine(g_stw->selLine, collapsedLine)){
		g_stw->textCell.cellActive=false;

		g_selCount++;
		int s = (collapsedLine+g_selCount.all-g_scene->scroll)*pixSymbolCY+beginYOffset+10;
		int amount=0;
		if( s > g_stw->size.y){
			amount=(s - g_stw->size.y) / pixSymbolCY+1;
		}
		g_stw->setScrolPos(amount);
	}
}
void hide(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	part->show=!part->show;
	part->smls->convertToRenderFormat();
	BackUp::monostable.beginCycle();
	g_stw->paint();
}
void hideAndDescendants(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	part->show = !part->show;
	recurseShow(part, part->show);
	part->smls->convertToRenderFormat();
	BackUp::monostable.beginCycle();
	g_stw->paint();
}

void recurseTrack(Art* part, bool track){
	part->track=track;
	M_SArt& m_art = part->part.refValue();
	for(int k=m_art.length; --k>=0;){
		recurseTrack(m_art[k], track);
	}
}

void setTrackAndDescendants(){
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		part->track = !part->track;
		recurseTrack(part, part->track);
		g_stw->cp->paint();
	}
	else if(Seamless* smls = dynamic_cast<Seamless*>(g_selNode.getRaw())){
		smls->track = !smls->track;
		bool track = smls->track;
		M_Art& partList = smls->partList;
		for(int k = partList.length; --k>=0;){
			partList[k]->track = track;
		}
	}
	g_stw->paint();
}

void toggleTrack(){
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		part->track = !part->track;
		g_stw->cp->paint();
	}
	else if(Seamless* smls = dynamic_cast<Seamless*>(g_selNode.getRaw())){
		smls->track = !smls->track;
	}
	else if(Viewpoint* view = dynamic_cast<Viewpoint*>(g_selNode.getRaw())){
		view->track = !view->track;
	}
	g_stw->paint();
}

void unselectAllTracks(){
	for(int k = g_sceneTreeList.length; --k >= 0;){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->track = false;
		}
		else if(Viewpoint* view = dynamic_cast<Viewpoint*>(g_sceneTreeList[k].node.p)){
			view->track = false;
		}
	}
}

void selectTrackExclusive(){
	unselectAllTracks();
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		part->smls->track = true;
		M_Art& partList = part->smls->partList;
		for(int i = partList.length; --i >= 0;){
			partList[i]->track = false;
		}
		part->track = true;
	}
	else if(Seamless* smls = dynamic_cast<Seamless*>(g_selNode.getRaw())){
		smls->track = true;
	}
	else if(Viewpoint* view = dynamic_cast<Viewpoint*>(g_selNode.getRaw())){
		view->track = true;
	}
	g_stw->paint();
}
void selectModelTracks(){
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	part->smls->track = true;
	M_Art& partList = part->smls->partList;
	for(int i = partList.length; --i >= 0;){
		partList[i]->track = true;
	}
	g_stw->paint();
}

void selectModelTracksExclusive(){
	unselectAllTracks();
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	part->smls->track = true;
	M_Art& partList = part->smls->partList;
	for(int i = partList.length; --i >= 0;){
		partList[i]->track = true;
	}
	g_stw->paint();
}

void selectSceneTracks(){
	for(int k = g_sceneTreeList.length; --k >= 0;){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->track = true;
		}
		else if(Art* part = dynamic_cast<Art*>(g_sceneTreeList[k].node.p)){
			part->track = true;
		}
		else if(Viewpoint* view = dynamic_cast<Viewpoint*>(g_sceneTreeList[k].node.p)){
			view->track = true;
		}
	}
	g_stw->paint();
}

//////
void undoIs(){
	removeRouteFromProto(g_selNode.getRaw());
	updateBuild3dWindow();
}
void deleteRoute(int fieldIndex, int routeIndex){
	g_selNode->getField(fieldIndex)->refRoute().deleteElement(routeIndex);
	Window::reSetupAllControlPanels();
	updateBuild3dWindow();
}

void addIsField(int nodeFieldIndex, int protoFieldIndex){
	removeRouteFromProto(g_selNode.getRaw());
	Field* field=g_selNode->getField(nodeFieldIndex);
	ProtoInterface* protoInterface=getSceneProtoDeclare(g_stw->selLine)->protoInterface;
	field->connect(g_selNode.getRaw(), protoInterface, protoInterface->getField(protoFieldIndex));
	g_stw->cp->paint();
}
void elementBeforeCopyOf(){//ELEMENT_BEFORE_COPYOF
	insertElement(false, false, 0);
}
void elementBeforeMove(){//ELEMENT_BEFORE_MOVE:
	insertElement(true, false, 0);
}
void elementReplaceCopyOf(){//ELEMENT_REPLACE_COPYOF
	insertElement(false, true, 0);
}
void elementReplaceMove(){//ELEMENT_REPLACE_MOVE
	insertElement(true, true, 0);
}
void elementAfterCopyOf(){//ELEMENT_AFTER_COLoadCursorPYOF
	insertElement(false, false, 1);
}
void elementAfterMove(){//ELEMENT_AFTER_MOVE
	insertElement(true, false, 1);
}
void addRouteToField(int dstFieldIndex, int srcFieldIndex){

	S_Route( g_selNode.getRaw(), srcFieldIndex, g_copyNode.getRaw(), dstFieldIndex);
	bool paintSceneTree=false;
	if(g_selNode->name==""){
		createUniqueName(g_selNode.getRaw());
		paintSceneTree=true;
	}
	if(g_copyNode->name==""){
		createUniqueName(g_copyNode.getRaw());
		paintSceneTree=true;
	}
	if(paintSceneTree){
		g_stw->genNodeList();
		g_stw->paint();
	}
	setupSelectedNodeWindow();
}
void deleteElement(){//DELETE_ELEMENT
	const M_Properties& m_properties=g_selNode->getProperties();
	M_Int& mdstIndex=*(g_selNode->getIndexPtr());
	int dstNewSelElement=mdstIndex[g_menuGroupIndex];

	for(int fidx=0; fidx<m_properties.length; fidx++){
		const Properties& s_prop=m_properties[fidx];
		if(s_prop.groupIndex==g_menuGroupIndex){
			
			Field* dst=g_selNode->getField(fidx);
			int dstIndex=mdstIndex[g_menuGroupIndex]-s_prop.startAtIndex;

			switch(dst->getType()){
			case MFBOOL:{	MFBool& _dst=	*dynamic_cast<MFBool*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFCOLOR:{	MFColor& _dst=	*dynamic_cast<MFColor*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFFLOAT:{	MFFloat& _dst=	*dynamic_cast<MFFloat*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFIMAGE:{	MFImage& _dst=	*dynamic_cast<MFImage*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFINT32:{	MFInt32& _dst=	*dynamic_cast<MFInt32*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFROTATION:{ MFRotation& _dst=*dynamic_cast<MFRotation*>(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFSTRING:{	MFString& _dst=	*dynamic_cast<MFString*>(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFDOUBLE:{	MFTime& _dst=	*dynamic_cast<MFTime*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFVEC2F:{	MFVec2f& _dst=	*dynamic_cast<MFVec2f*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;
			case MFVEC3F:{	MFVec3f& _dst=	*dynamic_cast<MFVec3f*>	(dst); _dst.refValue().deleteElement(dstIndex);} break;

			}
			if(dstNewSelElement>=dst->size()+s_prop.startAtIndex){
				dstNewSelElement=dst->size()+s_prop.startAtIndex-1;
				if(dstNewSelElement<0){
					dstNewSelElement=0;
				}
			}
		}
	}
	mdstIndex[g_menuGroupIndex]=dstNewSelElement;
	Window::paintAllControlPanels();
	updateBuild3dWindow();
}
void beamTo(){
	Transform& transform=*dynamic_cast<Transform*>(g_selNode.getRaw());
	Viewpoint viewpoint;
	viewpoint.position=transform.translation + Vec3f(0, 0, g_viewDist);
	g_scene->setSceneFromViewpoint(viewpoint);
}
void optimize(){
	do{
		g_optimized=false;
		optimize(dynamic_cast<AGroup*>(g_selNode.getRaw()));
		g_stw->genNodeList();
	}while(g_optimized);
	g_stw->paint();
}
void newPanel(){
	ControlPanel* controlPanel = new ControlPanel;
	controlPanel->setupNodeWindow(g_selNode.getRaw());
	controlPanel->pos=g_stw->cp->pos+Vec2i(50,50);
	Window::activeWindow=controlPanel;
}
void convertToWhitePart(Ptr<ImmediatePart> immediate, M_Art& accumOthers){
	Ptr<Part> whitePart = convertImmediateToPart(immediate);
	replaceNodeRefInSceneTree(immediate.p, whitePart.p);
	Seamless* smls=immediate->smls;
	smls->process();
	smls->resetRelevantParts();
//	smls->doBuild();
}
void convertToWhitePart(){
#ifdef PROG_ROB
	dLog("new MoveToNode(" + String(g_stw->selLine) + "),");
	dLog("new RightClick,");
	dLog("new Move_MenuItem(menu_convert),");
	dLog("new Click,");
#endif
	M_Art accumOthers;
	Ptr<ImmediatePart> immediate=dynamic_cast<ImmediatePart*>(g_selNode.getRaw());
	Seamless* smls = immediate->smls;
	Ptr<Part> part = convertImmediateToPart(immediate);
	convertToWhitePart(immediate, accumOthers);
	smls->doBuild();
	g_stw->genNodeList();
	g_stw->paint();
	g_stw->cp->setupNodeWindow(g_selNode.getRaw());

	g_stw->cp->setupNodeWindow(part);
	g_tb->updateToolBarContext();
	part->smls->doBuild();
}

void deleteNode(int nodeLine, int selCount = 1);

void convertToImmediatePart(Ptr<Part> part, M_Art& accumOthers){
#ifdef PROG_ROB
	dLog("new MoveToNode(" + String(g_stw->selLine) + "),");
	dLog("new RightClick,");
	dLog("new Move_MenuItem(menu_convert),");
	dLog("new Click,");
#endif
	MultiSelect::coordList.length = 0;
	MultiSelect::beginDragPos.length = 0;
	Ptr<ImmediatePart> immediate = convertPartToImmediate(part);
	replaceNodeRefInSceneTree(part.p, immediate.p);
	Seamless* smls=part->smls;
	smls->process();
	smls->resetRelevantParts();
	g_stw->genNodeList();
	g_stw->paint();
	g_stw->cp->setupNodeWindow(immediate);

	for(int k=g_sceneTreeList.length; --k>=0;){
		if(Build* build=dynamic_cast<Build*>(g_sceneTreeList[k].node.p)){
			if(build->illegalImmediateRef(immediate, accumOthers)){
				deleteNode(k);
			}
		}
	}
}

void convertToImmediatePart(){
	M_Art accumOthers;
	Ptr<Part> part=dynamic_cast<Part*>(g_selNode.getRaw());
	accumOthers+=part;
	while(accumOthers.length){
		if(Ptr<Part> part = dynamic_cast<Part*>(accumOthers[0])){
			convertToImmediatePart(part, accumOthers);
			accumOthers.deleteElement(0);
		}
	}
	g_stw->commonPostInsertNodeStuff();
	houseKeepVisualClue();
	g_tb->updateToolBarContext();
}
void convertAllToImmediateParts(){
	M_Art accumOthers;

	Ptr<Seamless> smls=dynamic_cast<Seamless*>(g_selNode.getRaw());

	M_Art& partList = smls->partList;

	smls->build.refValue().length=0;

	for(int k = partList.length; --k>=0;){
		if(Ptr<Part> part = dynamic_cast<Part*>(partList[k])){
			convertToImmediatePart(part, accumOthers);
		}
	}
	g_stw->commonPostInsertNodeStuff();
	houseKeepVisualClue();
	g_tb->updateToolBarContext();
}
void convertAllToWhiteParts(){
	M_Art accumOthers;

	Ptr<Seamless> smls=dynamic_cast<Seamless*>(g_selNode.getRaw());

	M_Art& partList = smls->partList;

	for(int k = partList.length; --k>=0;){
		if(Ptr<ImmediatePart> immediate = dynamic_cast<ImmediatePart*>(partList[k])){
			convertToWhitePart(immediate, accumOthers);
		}
	}
	smls->doBuild();
	smls->ctrf=true;
	g_stw->commonPostInsertNodeStuff();
	houseKeepVisualClue();
	g_tb->updateToolBarContext();
}

void fastFullBuildGlobal(bool inState){

	for(int k = g_sceneTreeList.length; --k>=0;){
		Node* node = g_sceneTreeList[k].node.p;
		if(JoinVertex* joinVertex = dynamic_cast<JoinVertex*>(node)){
			joinVertex->in = inState;
		}
		else if(TransferBCyl* transferBCyl = dynamic_cast<TransferBCyl*>(node)){
			transferBCyl->in = inState;
		}
	}
	if(inState){
		for(int k = g_sceneTreeList.length; --k>=0;){
			if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
				smls->doBuild();
				smls->ctrf = true;
			}
		}
	}

	houseKeepVisualClue();
	g_stw->paint();
	g_stw->cp->paint();
}


void fastFullBuild(bool inState){
	int smlsLine=getSeamlessParent(g_stw->selLine);
	int smlsIndent=g_sceneTreeList[smlsLine].indent;

	for(int k=g_stw->selLine; k<g_sceneTreeList.length && g_sceneTreeList[k].indent > smlsIndent; k++){
		Node* node=g_sceneTreeList[k].node.p;
		if(JoinVertex* joinVertex=dynamic_cast<JoinVertex*>(node)){
			joinVertex->in=inState;
		}
		else if(TransferBCyl* transferBCyl=dynamic_cast<TransferBCyl*>(node)){
			transferBCyl->in=inState;
		}
		else if(FuseSurface* fuseSurface=dynamic_cast<FuseSurface*>(node)){
			fuseSurface->in=inState;
		}
	}
	dynamic_cast<Seamless*>(g_sceneTreeList[smlsLine].node.p)->doBuild();

	houseKeepVisualClue();
	g_stw->paint();
	g_stw->cp->paint();
}

void fastFullBuildForAutoIn(bool inState, Seamless* smls){
	if(smls->workAnimCP){
		return;
	}
	if(smls->autoIn && smls->masterIn != inState){
		smls->masterIn.refValue() = inState;
		setAllIns(smls->build.refValue(), inState);
	}
}
void fastFullBuildForAutoIn(bool inState){
	int selLine = g_stw->selLine;
	if(Seamless* smls = getSeamless(selLine)){
		if(smls->workAnimCP == false){
			fastFullBuildForAutoIn(inState, smls);
		}
	}
}
void updateAutoInForClick(){
	int selLine = g_stw->selLine;
	if(Seamless* smls = getSeamless(selLine)){
		Node* node = g_selNode.getRaw();
		if(
			dynamic_cast<Generator*>(node)||
			dynamic_cast<ColorSweep*>(node)||
			dynamic_cast<TextureMapper*>(node)||
			dynamic_cast<GroupBuild*>(node)||
			dynamic_cast<Effect*>(node)
			){
			fastFullBuildForAutoIn(false, smls);
		}
		else{
			 Node* node = g_selNode.getRaw();
			smls->masterIn.refValue()=false; //force all to get set true
			fastFullBuildForAutoIn(true, smls);
		}
	}
}

void fastBuild(){
	fastFullBuild(false);
}
void fullBuild(){
	fastFullBuild(true);
}
void colorVertex(Art* part, Art* actTriPart, const int ActTri,const int ActTriCorner, bool endWithBuild = true){
	Seamless* smls = part->smls;
	BackUp::monostable.beginCycle();
	ColorEffect* effect=dynamic_cast<ColorEffect*>(part->workEffect);
	if(effect==0)return;
	M_Color3f& color=effect->color.refValue();

	M_Triangle& dstTriangle=actTriPart->triangle;
	int dstIndex = ActTri * 3 + ActTriCorner;

	int dstVertex = dstTriangle.refTexIndex(dstIndex);
	color[dstVertex] = g_palette->color;

	int returnCoordIdxIdx = 0;
	if(Art* mirrorPart = smls->getMirrorTriangle(part, returnCoordIdxIdx)){
		color[mirrorPart->triangle.refTexIndex(returnCoordIdxIdx)] = g_palette->color;
	}
	if(endWithBuild){
		smls->doBuild();
		smls->updateAllVertexInBuffers();
	}
}

void colorVertex(){
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<ColorEffect*>(whitePart->workEffect)==0)return;

		Seamless* smls = whitePart->smls;

		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			colorVertex(whitePart, smls->actTriPart, smls->ActTri, smls->ActTriCorner);
			return;
		}

		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				colorVertex(bluePart, bluePart, smls->ActTri-stem->beginTriOffset, smls->ActTriCorner);
			}
			else{
				int ActTri = smls->ActTri;
				int ActTriCorner = smls->ActTriCorner;
				if(0 > getRelIndex(stem, ActTri, ActTriCorner)){
					return;
				}
				
				int dstIndex = ActTri + ActTriCorner;

				int dstVertex = stem->coordIndex[dstIndex];
				stem->color.refValue()[dstVertex] = g_palette->color;

				smls->doBuild();
				makePointerEqualIndexLocation();
				smls->ctrf=true;
				Window::paintAllControlPanels();
			}
		}
	}
	else if(Strand* oldSd = dynamic_cast<Strand*>(g_selNode.getRaw())){
		int selLine = g_stw->selLine;
		int parentLine = getParent(selLine);
		if(Hair* hair = dynamic_cast<Hair*>(g_sceneTreeList[parentLine].node.p)){
			Strand* newSd = hair->clone(oldSd);
			clone(hair, oldSd, newSd);
		}
	}
	else if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		Strand* oldSd = hair->scpiLathe[hair->index[0]];
		Strand* newSd = hair->clone(oldSd);
		clone(hair, oldSd, newSd);
	}
	else if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(g_scene->mode & HAIR_REGION_MODE){
			lock->insertBorderPoint();
		}
	}
	else{
		needPartMsg();
	}
}
void pasteColorToTriangle(Art* part, Art* triPart, const int ActTri){
	fastFullBuildForAutoIn(true);
	Seamless* smls=part->smls;
	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
		M_Color3f& color = colorEffect->color.refValue();

		Color3f newColor = g_palette->color;

		color+=newColor;
		color+=newColor;
		color+=newColor;
		triPart->triangle.refTexIndex(ActTri*3+0)=color.length-3;
		triPart->triangle.refTexIndex(ActTri*3+1)=color.length-2;
		triPart->triangle.refTexIndex(ActTri*3+2)=color.length-1;

		int returnCoordIdxIdx=0;
		if(part->mirror){
			if(Art* mirrorPart=smls->getMirrorTriangle(part, returnCoordIdxIdx)){
				color+=newColor;
				color+=newColor;
				color+=newColor;
				returnCoordIdxIdx = returnCoordIdxIdx / 3 * 3;
				mirrorPart->triangle.refTexIndex(returnCoordIdxIdx+0)=color.length-3;
				mirrorPart->triangle.refTexIndex(returnCoordIdxIdx+1)=color.length-2;
				mirrorPart->triangle.refTexIndex(returnCoordIdxIdx+2)=color.length-1;
			}
		}
	}
}

void pasteColorToTriangle(){
	if(Art* whitePart = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		if(dynamic_cast<ImmediatePart*>(whitePart) || smls->bnspe){
			pasteColorToTriangle(whitePart, smls->actTriPart, smls->ActTri);
			commonEndVertexBuild();
			return;
		}
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				pasteColorToTriangle(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
				commonEndVertexBuild();
			}
			else{
				int ActTri = smls->ActTri;
				int ActTriCorner = smls->ActTriCorner;
				if(0 > getRelIndex(stem, ActTri, ActTriCorner)){
					return;
				}
				bluePart = selectPolyMapPart(whitePart);
				stem->copyPolyMapToPart(bluePart);
				pasteColorToTriangle(bluePart, bluePart, ActTri/3);
				stem->copyPartToPolyMap(bluePart);
				commonEndVertexBuild();
			}
		}
	}
	else{
		needPartMsg();
	}
}

void commonEnd(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	part->smls->ctrf=true;
	part->smls->doBuild();
	makePointerEqualIndexLocation();
	BackUp::monostable.beginCycle();
	MultiSelect::reset();
}
void toggleMultiSelectTriVert(){
	//MultiSelect::backfaceIgnore = !MultiSelect::backfaceIgnore;
}
void cutLeftFromVertex(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		fastFullBuildForAutoIn(true);
		part->cutLeft((part->matrix*part->coord.refValue()[part->smls->ActCtrlPnt]).x+.0001);
		commonEnd();
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->cutLeft();
	}
}
void cutRightFromVertex(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	fastFullBuildForAutoIn(true);
	part->cutRight((part->matrix*part->coord.refValue()[part->smls->ActCtrlPnt]).x-.0001);
	commonEnd();
}
void cutUpFromVertex(){
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		fastFullBuildForAutoIn(true);
		part->cutUp((part->matrix*part->coord.refValue()[part->smls->ActCtrlPnt]).y -.0001);
		commonEnd();
	}
}

void cutDownFromVertex(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	fastFullBuildForAutoIn(true);
	part->cutDown((part->matrix*part->coord.refValue()[part->smls->ActCtrlPnt]).y+.0001);
	commonEnd();
}
void commonPasteAddCopyOfMirror(CopyPart& cp, Art* mirrorPart){
	fastFullBuildForAutoIn(true);
	ArtPtr temp = new Part;
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	part->part+=temp;

	Mesh geomSource;
	mirrorPart->extractGeometryGS(geomSource, g_tmpIndex);
	temp->workEffect->convertGeomSourceToPart(temp.getRawPtr(), geomSource);

	cp.source=temp;
	cp.dest=temp;
	cp.doBuild();
	TransferVertex tv;
	tv.source=temp;
	tv.dest=part;
	tv.doBuild();
	part->part.resize(part->part.refValue().length-1);
	part->smls->process();
	commonEnd();
}
void mirrorXPolyMapBnspe(Art* part){
	M_Vec2f& uvCoord = part->uvCoord.refValue();
	M_NurbsPatch& m_patchOwner = part->patchOwner;
	for(int i = uvCoord.length; --i>=0;){
		NurbsPatch* patch = m_patchOwner[i];
		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			m_patchOwner[i] = mirrorPatch;
		}
		else{
			float x = uvCoord[i].x - .5;
			x = -x;
			uvCoord[i].x = x + .5;
		}
	}
}

void commonPasteAddCopyOfMirrorBnspe(CopyPart& cp){
	ArtPtr temp = new Part;
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	part->part += temp;

	Mesh geomSource;
	M_Vec2f gsUvCoord;
	M_<NurbsPatch*> gsPatchOwner;

	part->extractGeometryGS(geomSource, gsUvCoord, gsPatchOwner, g_tmpIndex);
	temp->workEffect->convertGeomSourceToPart(temp.getRawPtr(), geomSource, gsUvCoord, gsPatchOwner);
	temp->insideOut();
	mirrorXPolyMapBnspe(temp);

	cp.source=temp;
	cp.dest=temp;
	cp.doBuild();
	TransferVertex tv;
	tv.source=temp;
	tv.dest=part;
	tv.doBuild();
	part->part.resize(part->part.refValue().length-1);
	part->smls->process();
	commonEnd();
}

void InsideOutPart::doInsideOutBnspe(CopyPart& cp, Art* dest){
	ArtPtr temp = new Part;
	dest->part += temp;

	Mesh geomSource;
	M_Vec2f gsUvCoord;
	M_<NurbsPatch*> gsPatchOwner;

	source->extractGeometryGS(geomSource, gsUvCoord, gsPatchOwner, g_tmpIndex);
	temp->workEffect->convertGeomSourceToPart(temp.getRawPtr(), geomSource, gsUvCoord, gsPatchOwner);
	temp->insideOut();
	//mirrorXPolyMapBnspe(temp);

	cp.source=temp;
	cp.dest=temp;
	cp.doBuild();
	TransferVertex tv;
	tv.source=temp;
	tv.dest=dest;

	if(add == false){
		dest->deleteGeometry();
	}


	tv.doBuild();
	dest->part.resize(dest->part.refValue().length-1);
	dest->smls->process();
	commonEnd();
}

void pasteAddCopyOf(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			CopyPart cp;
			commonPasteAddCopyOfMirror(cp, remPJ);
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}
void pasteAddCopyOfMirrorX(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			CopyPart cp;
			if(part->smls->bnspe){
				commonPasteAddCopyOfMirrorBnspe(cp);
			}
			else{
				cp.mirrorX = true;
				commonPasteAddCopyOfMirror(cp, remPJ);
			}
		}
		else{
			needPartCopiedMsg();
		}
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->mirrorControlPoints();
	}
	else{
		needPartMsg();
	}
}
void pasteAddCopyOfMirrorY(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			CopyPart cp;
			cp.mirrorY=true;
			commonPasteAddCopyOfMirror(cp, remPJ);
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}
void pasteAddCopyOfMirrorZ(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			CopyPart cp;
			cp.mirrorZ=true;
			commonPasteAddCopyOfMirror(cp, remPJ);
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}

void addRelevantPart(M_Art& relevant, Art*& part);

void replaceWithCopyOfRememberedPart(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			fastFullBuildForAutoIn(true);
			part->deleteGeometry();
			Mesh geomSource;
			remPJ->extractGeometryGS(geomSource, g_tmpIndex);
			part->workEffect->convertGeomSourceToPart(part, geomSource);
			commonEnd();
		}
		else if(Stem* stem = dynamic_cast<Stem*>(g_copyNode.getRaw())){
			part->deleteGeometry();
			part->workEffect->stemCopyLocalTri(stem, part, part->coord.refValue(), 0, 0);
//			part->workEffect->stemCopyLocalTri(stem, part, part->coord.refValue(), part->coord.refValue().length, part->triangle.length);
			//stem->copyPolyMapToPart(part);
			addRelevantPart(part->relevantPart, part);
			commonEnd();
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}
/*
void commonPasteAddCopyOfMirrorBnspe(CopyPart& cp){
	ArtPtr temp = new Part;
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	part->part += temp;

	Mesh geomSource;
	M_Vec2f gsUvCoord;
	M_<NurbsPatch*> gsPatchOwner;

	part->extractGeometryGS(geomSource, gsUvCoord, gsPatchOwner, g_tmpIndex);
	temp->workEffect->convertGeomSourceToPart(temp.getRawPtr(), geomSource, gsUvCoord, gsPatchOwner);
	temp->insideOut();
	mirrorXPolyMapBnspe(temp);

	cp.source = temp;
	cp.dest = temp;
	cp.doBuild();
	TransferVertex tv;
	tv.source=temp;
	tv.dest=part;
	tv.doBuild();
	part->part.resize(part->part.refValue().length-1);
	part->smls->process();
	commonEnd();
}
*/
void commonReplaceWithCopyOfRememberedPartBnspe(CopyPart& cp, Art* mirrorPart){
	ArtPtr temp = new Part;
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	part->part += temp;

	Mesh geomSource;
	M_Vec2f gsUvCoord;
	M_<NurbsPatch*> gsPatchOwner;

	part->extractGeometryGS(geomSource, gsUvCoord, gsPatchOwner, g_tmpIndex);
	temp->workEffect->convertGeomSourceToPart(temp.getRawPtr(), geomSource, gsUvCoord, gsPatchOwner);
	//temp->insideOut();
	mirrorXPolyMapBnspe(temp);

	cp.source = temp;
	cp.dest = temp;
	cp.doBuild();
	TransferVertex tv;

	mirrorPart->deleteGeometry();

	tv.source = temp;
	tv.dest = mirrorPart;
	tv.doBuild();
	part->part.resize(part->part.refValue().length-1);
	part->smls->process();
	commonEnd();
	//setupNodeWindow(g_selNode);
	Window::paintAllControlPanels();
}

void commonReplaceWithCopyOfRememberedPart(CopyPart& cp, Art* mirrorPart){
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	fastFullBuildForAutoIn(true);

	//seems to make this work ok but why did I comment the folowing 5 lines once before? does this get used in a build cycle and will it slow things down if it is?
	Mesh geomSource;
	mirrorPart->extractGeometryGS(geomSource, g_tmpIndex);
	part->deleteGeometry();
	part->workEffect->convertGeomSourceToPart(part, geomSource);
	part->translation.refValue()=mirrorPart->translation;

	cp.source.refValue().length=1;
	cp.dest.refValue().length=1;

	cp.source.refValue()[0] = mirrorPart;
	cp.dest.refValue()[0] = part;
	cp.doBuild();
	part->coordGroup.refValue() = mirrorPart->coordGroup;
	part->weight.refValue() = mirrorPart->weight;
	commonEnd();
	//setupNodeWindow(g_selNode);
	Window::paintAllControlPanels();
}

void replaceWithCopyOfPolyMapMirrorX(){
	if(remPJ==0) return;
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		CopyPart cp;
		cp.insideOut.refValue() = true;
		commonReplaceWithCopyOfRememberedPart(cp, remPJ);
		M_Vec3f& coord = part->coord.refValue();
		for(int i = coord.length; --i>=0;){
			float x = coord[i].x - .5;
			x = -x;
			coord[i].x = x + .5;
		}
		part->smls->process();
		commonEndVertexChange();
	}
}

void replaceWithCopyOfRememberedPartMirrorX(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			CopyPart cp;
			cp.mirrorX=true;
			commonReplaceWithCopyOfRememberedPart(cp, remPJ);
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}

void replaceWithCopyOfRememberedPartMirrorY(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			CopyPart cp;
			cp.mirrorY=true;
			commonReplaceWithCopyOfRememberedPart(cp, remPJ);
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}
void replaceWithCopyOfRememberedPartMirrorZ(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(dynamic_cast<Art*>(g_copyNode.getRaw())){
			CopyPart cp;
			cp.mirrorZ=true;
			commonReplaceWithCopyOfRememberedPart(cp, remPJ);
		}
		else{
			needPartCopiedMsg();
		}
	}
	else{
		needPartMsg();
	}
}

void copyAndPasteMirroredPart(){


	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(part->mirror){
			CopyPart cp;
			cp.mirrorX = true;
			cp.copyTransform = true;
			if(Art* mirrorPart = part->mirrorPart){
				if(part->smls->bnspe){
					commonReplaceWithCopyOfRememberedPartBnspe(cp, mirrorPart);
				}
				else{
					commonReplaceWithCopyOfRememberedPart(cp, mirrorPart);
				}
			}
			else{
				Seamless* smls = part->smls;
				M_Vec3f& coord = part->coord.refValue();

				if(smls->ActCtrlPnt < coord.length){
					if(coord[smls->ActCtrlPnt].x > 0){
						part->cutLeft(.0001);
					}
					else{
						part->cutRight(-.0001);
					}
				}
				else{
					return;
				}

				CopyPart cp;
				if(part->smls->bnspe){
					commonPasteAddCopyOfMirrorBnspe(cp);
				}
				else{
					cp.mirrorX = true;
					commonPasteAddCopyOfMirror(cp, remPJ);
				}
			}
		}
	}
	else{
		needPartMsg();
	}
}
void invertBackgroundColor(){
	Color3f& backgroundColor = g_scene->backgroundColor.refValue();
	backgroundColor = hexToColor(backgroundColor.colorToHex() ^ 0xffffff);

	Color3f& scene3DFontColor = g_scene->scene3DFontColor.refValue();
	scene3DFontColor = hexToColor(scene3DFontColor.colorToHex() ^ 0xffffff);
	houseKeepVisualClue();


}
inline void deleteVertex(Art* vertexPart, int vertexIdx){
	int storeCoordlen=vertexPart->coord.refValue().length;
	
	M_Art& relevantPart=vertexPart->relevantPart;
	for(int k=relevantPart.length; --k>=0;){
		Art* part=relevantPart[k];
		M_Triangle& triangle=part->triangle;
		for(int i=triangle.length; --i>=0; ){
			if(	(triangle[i].pa==vertexPart && triangle[i].ia==vertexIdx) ||
				(triangle[i].pb==vertexPart && triangle[i].ib==vertexIdx) ||
				(triangle[i].pc==vertexPart && triangle[i].ic==vertexIdx))
			{
				vertexPart->deleteTriangle(i);
				if(storeCoordlen!=vertexPart->coord.refValue().length)return;
			}
		}
	}
}
void immediateDeleteVertex(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = part->smls;
		fastFullBuildForAutoIn(true);

		deleteVertex(part, smls->ActCtrlPnt);

		makePointerEqualIndexLocation();
		smls->convertToRenderFormat();
		BackUp::monostable.beginCycle();
	}
	else{
		needPartMsg();
	}
}

void bubbleSortPartsInTriList(M_<S_VertexIndex>& triList){
	bool flag;
	do{
		flag = false;
		for(int i=0; i < triList.length - 1; i++){

			if(triList[i].p->plidx > triList [i + 1].p->plidx){

				S_VertexIndex swap = triList[i + 1];
				triList[i + 1] = triList[i];
				triList[i] = swap;

				flag = true;
			}
		}
	}while(flag);
}

void deleteTriangle(Art* part, Art* actTriPart, const int ActTri, bool endWithBuild = true){
	if((actTriPart == 0 || actTriPart->triangle.length == 0) && MultiSelect::triList.length == 0){
		return;
	}
	fastFullBuildForAutoIn(true);
	Seamless* smls=part->smls;

	BackUp::monostable.beginCycle();

	DelPoly delPoly;
	M_Int& poly = delPoly.poly.refValue();

	if(MultiSelect::triList.length){
		bubbleSortPartsInTriList(MultiSelect::triList);

		poly += MultiSelect::triList[0].i;
		delPoly.part = MultiSelect::triList[0].p;
		for(int k = 1; k < MultiSelect::triList.length; k++){
			if(MultiSelect::triList[k].p != delPoly.part){
				bubbleSort(poly);
				delPoly.doBuild();
				poly.length = 0;
			}
			poly += MultiSelect::triList[k].i;
			delPoly.part = MultiSelect::triList[k].p;
		}
		bubbleSort(poly);
		delPoly.doBuild();
		MultiSelect::triList.length = 0;
		MultiSelect::coordList.length = 0;
	}
	else{
		delPoly.part = actTriPart;
		poly += ActTri;
		delPoly.doBuild();
	}
	if(endWithBuild == false){
		return;
	}
	smls->doBuild();
	makePointerEqualIndexLocation();
	smls->convertToRenderFormat();
	g_stw->paint();
}

void resetCPNodeList();

void deleteTriangle(){
	if(Part* whitePart = dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		if(smls->bnspe){
			deleteTriangle(whitePart, smls->actTriPart, smls->ActTri);
			return;
		}

		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
				deleteTriangle(bluePart, bluePart, smls->ActTri-stem->beginTriOffset);
			}
			else{
				int ActTri = smls->ActTri;
				int ActTriCorner = smls->ActTriCorner;
				if(0 > getRelIndex(stem, ActTri, ActTriCorner)){
					return;
				}

				bluePart = selectPolyMapPart(whitePart);
				stem->copyPolyMapToPart(bluePart);
				deleteTriangle(bluePart, bluePart, ActTri / 3, false);
				stem->copyPartToPolyMap(bluePart);
				commonEndVertexBuild();
			}
		}
	}
	else if(ImmediatePart* bluePart = dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Seamless* smls = bluePart->smls;
		deleteTriangle(bluePart, smls->actTriPart, smls->ActTri);
	}
	else if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		Hair* hair = getHair();
		if((g_scene->mode & HAIR_REGION_MODE)){
			if(Lock::selBorderLock){
				Lock::selBorderLock->border.refValue().deleteElement(Lock::selBorder);
				M_Node& m_hairLock = hair->lock.refValue();
				for(int k = 0; k < m_hairLock.length; k++){
					Lock* srcLock = dynamic_cast<Lock*>(m_hairLock[k].p);
					M_Node& m_lock = srcLock->lock.refValue();
					for(int k = 0; k < m_lock.length; k++){
						Lock* lock = dynamic_cast<Lock*>(m_lock[k].p);
						lock->fixBorderIndexForDelete(srcLock);
						Lock::selBorderLock = 0;
					}
				}
				houseKeepVisualClue();
			}
			else{
				lock->deleteSelectedRegions();
			}
		}
		else{
			hair->deleteBoundControlPoints(lock);
		}
	}
	else if(NurbsLine* line = dynamic_cast<NurbsLine*>(g_selNode.getRaw())){
		line->deleteControlPoint();
	}
	else if(Strand* strand = dynamic_cast<Strand*>(g_selNode.getRaw())){
		//if(g_transformCP->selectRing || g_transformCP->selectRings){
		//	if(Hair* hair = getHair()){
		//		Lock* lock = 0;
		//		int scpiIdx = 0;
		//		if(hair->isBoundToLock(strand, strand->index[0], lock, scpiIdx)){
		//			hair->deleteBoundControlPoints(lock);
		//		}
		//	}
		//}
		//else{
		strand->deleteControlPoint();
		//}
	}
	else if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = nurbsPatch->smls;
		
		smls->deleteCtrlPoly(MultiSelect::ctrlPolySelList);

		MultiSelect::ctrlPolySelList.length = 0;
		resetCPNodeList();
		nurbsPatch->houseKeep();
	}
	else if(BezierPatch* bezierPatch = dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		bezierPatch->deletePatch();
	}
	else if(CCLathe* ccLathe = dynamic_cast<CCLathe*>(g_selNode.getRaw())){
		ccLathe->deleteControlPoint();
	}
	else if(Subdivision* subdivision = dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->delPoly();
	}
	else if(NurbsLathe* nurbsLathe = dynamic_cast<NurbsLathe*>(g_selNode.getRaw())){
		int newLen=0;
		int idx = nurbsLathe->index[0];
		nurbsLathe->removeControlPoint(idx);
		if(NurbsLathe* mirrorContainer = nurbsLathe->mirrorContainer){
			mirrorContainer->removeControlPoint(idx);
			mirrorContainer->processFields();
			mirrorContainer->getPatch()->synchronizeCellsAndBuild();
		}
		nurbsLathe->processFields();
		nurbsLathe->getPatch()->synchronizeCellsAndBuild();
		houseKeepVisualClue();
	}
	else if(NurbsTrack* nurbsTrack = dynamic_cast<NurbsTrack*>(g_selNode.getRaw())){
		nurbsTrack->removeControlPoint(nurbsTrack->index[1]);
	}
	else{
		needPartMsg();
	}
}

void Hair::deleteBoundControlPoints(Lock* lock){
	M_Vec2f& hole = lock->hole.refValue();
	M_Int& scpi = lock->scpi.refValue();
	M_<Strand*>& scpiLathe = lock->scpiLathe;

	for(int i = hole.length; --i>=0;){
		Strand* strand = scpiLathe[i];
		int pointidx = scpi[i];
		strand->index[0] = scpi[i];
		int vDim = strand->dim.y;
		strand->dim = Vec2f(1, vDim - 1);
		strand->index[0] = strand->NurbsPatch::index[0];

		deleteStrandPoint(strand, pointidx);
	}
	refreshHairOffset();

	updateBuild3dWindow();
}

void Hair::insertStrandControlPoints(Lock* lock, const M_Bool& flag){
	M_Vec2f& hole = lock->hole.refValue();
	M_Int& scpi = lock->scpi.refValue();
	M_<Strand*>& scpiLathe = lock->scpiLathe;

	for(int i = hole.length; --i>=0;){
		Strand* strand = scpiLathe[i];
		if(flag[strand->slidx]){
			int pointidx = scpi[i];
			strand->index[0] = scpi[i];
			int vDim = strand->dim.y;
			strand->dim = Vec2f(1, vDim + 1);
			strand->index[0] = strand->NurbsPatch::index[0];

			updateNurbsCurveKnotMid0(vDim + 1, strand->vKnot.refValue());

			insertStrandPoint(strand, pointidx);

		}
	}
	refreshHairOffset();
}

void Hair::insertStrandControlPoints(Lock* lock){
	M_Vec2f& hole = lock->hole.refValue();
	M_Int& scpi = lock->scpi.refValue();
	M_<Strand*>& scpiLathe = lock->scpiLathe;

	for(int i = hole.length; --i>=0;){
		Strand* strand = scpiLathe[i];
		int pointidx = scpi[i];
		strand->index[0] = scpi[i];
		int vDim = strand->dim.y;
		strand->dim.refValue() = Vec2f(1, vDim + 1);
		strand->index[0] = strand->NurbsPatch::index[0];

		insertStrandPoint(strand, pointidx);
	}
	refreshHairOffset();

	updateBuild3dWindow();
}

void insertStrandControlPoints(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Hair* hair = getHair()){
			hair->insertStrandControlPoints(lock);
		}
	}
}

void Strand::deleteControlPoint(){
	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Float& weight = this->weight.refValue();
	M_Float& knot = this->vKnot.refValue();

	int vDim = dim.y;

	dim.refValue() = Vec2f(1, vDim - 1);
	int idx = index[0] = NurbsPatch::index[0];

	dpoint.deleteElement(idx);
	point.deleteElement(idx);
	cpIndex.deleteElement(idx);
	weight.deleteElement(idx);
	MultiSelect::pointList.length = 0;

	if(idx >= dpoint.length){
		idx = dpoint.length - 1;
	}
	if(idx < 0){
		idx = 0;
	}
	index[0] = NurbsPatch::index[0] = idx;

	updateNurbsCurveKnotMid0(point.length, knot);


/////
	Hair* hair = 0;
	Art* part = 0;
	if(hair = getHair(getNodeLine(this))){
		part = dynamic_cast<Art*>(hair->part.p);
		hair->deleteStrandPoint(this, index[0]);
		hair->refreshHairOffset();
	}
	index[0] = NurbsPatch::index[0];


	smls->populateMirrorNodes();

	resetCPNodeList();


	part->deleteGeometry();
	part->smls->convertToRenderFormat();
	makePointerEqualIndexLocation();


	smls->ctrf = true;
	hair->doBuild();
	houseKeepVisualClue();
}

void NurbsLine::deleteControlPoint(){
	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Float& weight = this->weight.refValue();
	M_Color3f& color = this->color.refValue();
	M_Float& knot = this->uKnot.refValue();

	int vDim = dim.y;

	dim.refValue() = Vec2f(1, vDim - 1);
	int idx = index[0] = NurbsPatch::index[0];

	dpoint.deleteElement(idx);
	point.deleteElement(idx);
	cpIndex.deleteElement(idx);
	weight.deleteElement(idx);
	color.deleteElement(idx);
	MultiSelect::pointList.length = 0;

	if(idx >= dpoint.length){
		idx = dpoint.length - 1;
	}
	if(idx < 0){
		idx = 0;
	}
	index[0] = NurbsPatch::index[0] = idx;

	updateNurbsCurveKnotMid0(point.length, knot);

	processFields();
	smls->populateMirrorNodes();

	smls->doBuild();
	houseKeepVisualClue();
}

void convertToCNames(){
	for(int k=0; k<g_sceneTreeList.length; k++){
		Node* node=g_sceneTreeList[k].node.p;
		if(makeNameLegalForCpp(node->name)){
			 makeExistingNameUnique(node);
		}
	}
	BackUp::monostable.beginCycle();
}

void NurbsPatch::mirrorModifyWest(){
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	const Vec2f& dim = this->dim;
	int uDim=dim.x;
	int vDim=dim.y;
	int middleIdx = (uDim >> 1);
	for(int v=vDim; --v>=0;){

		for(int u = 0; u < uDim>>1; u++){
			Vec3f p = point[v*uDim + uDim - 1 - u];
			p.x = -p.x;
			point[v*uDim + u] = p;

			weight[v*uDim + u] = weight[v*uDim + uDim - 1 - u];;
		}
		if(uDim && 1){
			point[v*uDim + middleIdx].x = 0;
		}
	}
}
void NurbsPatch::mirrorModifyEast(){
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	const Vec2f& dim = this->dim;
	int uDim=dim.x;
	int vDim=dim.y;
	int middleIdx = (uDim >> 1);
	for(int v=vDim; --v>=0;){

		for(int u = 0; u < uDim>>1; u++){
			Vec3f p = point[v*uDim + u];
			p.x = -p.x;
			point[v*uDim + uDim - 1 - u] = p;

			weight[v*uDim + uDim - 1 - u] = weight[v*uDim + u];
		}
		if(uDim && 1){
			point[v*uDim + middleIdx].x = 0;
		}
	}
}
/*
void matchEndsForPatch(){
	if(Window::chatMode())return;
	if(NurbsPatch* nurbsPatch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		M_Vec3f& point = nurbsPatch->point.refValue();
		M_Float& weight = nurbsPatch->weight.refValue();
		const Vec2f& dim = nurbsPatch->dim;
		int uDim=dim.x;
		int vDim=dim.y;
		for(int v=vDim; --v>=0;){

			point[v * uDim + uDim - 1] = point[v * uDim];

			point[v * uDim + uDim - 2].y = point[v * uDim].y;
			point[v * uDim + uDim - 2].z = point[v * uDim].z;

			point[v * uDim + 1].y = point[v * uDim].y;
			point[v * uDim + 1].z = point[v * uDim].z;

		}
		nurbsPatch->processFields();
	}
	else if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		M_Vec3f& point = bezierPatch->point.refValue();
		const M_Vec2f& dim = bezierPatch->dim;
		int uDim=dim[0].x;
		int vDim=dim[0].y;
		for(int v=vDim; --v>=0;){

			point[v * uDim + uDim - 1] = point[v * uDim];

			point[v * uDim + uDim - 2].y = point[v * uDim].y;
			point[v * uDim + uDim - 2].z = point[v * uDim].z;

			point[v * uDim + 1].y = point[v * uDim].y;
			point[v * uDim + 1].z = point[v * uDim].z;

		}
	}
	else{
		return;
	}
	updateBuild3dWindow();
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}
*/
void NurbsPatch::flipPolarity(){//same as 2 * rotate uv 90
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Int _cpIndex = cpIndex;
	int uDim = dim.x;
	int vDim = dim.y;

	int len = uDim * vDim;
	for(int i = 0; i < len; i++){
		cpIndex[i] = _cpIndex[len - 1 - i];
	}
}
/*
void NurbsPatch::flipPolarity(){
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	const Vec2f& dim = this->dim;
	int uDim=dim.x;
	int vDim=dim.y;
	for(int v = vDim; --v>=(vDim>>1);){
		for(int u = uDim; --u>= 0;){
			Vec3f h = point[(vDim - v - 1) * uDim + u];
			float w = weight[(vDim - v - 1) * uDim + u];
			point[(vDim - v - 1) * uDim + u] = point[v * uDim + u];
			weight[(vDim - v - 1) * uDim + u] = weight[v * uDim + u];
			point[v * uDim + u] = h;
			weight[v * uDim + u] = w;
		}
	}
	for(int v = vDim; --v>=0;){
		for(int u = uDim; --u>=(uDim>>1);){
			Vec3f h = point[v * uDim + (uDim - u - 1)];
			float w = weight[v * uDim + (uDim - u - 1)];
			point[v * uDim + (uDim - u - 1)] = point[v * uDim + u];
			weight[v * uDim + (uDim - u - 1)] = weight[v * uDim + u];
			point[v * uDim + u] = h;
			weight[v * uDim + u] = w;
		}
	}
	processFields();
}
*/
void flipPolarity(){
	if(NurbsPatch* patch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		smls->convertToCpFormat();
		patch->flipPolarity();
		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			mirrorPatch->flipPolarity();
		}
		smls->centralCPsToPatches();
		patch->processFields();
		patch->synchronizeAllStemCells();
		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			mirrorPatch->processFields();
			mirrorPatch->synchronizeAllStemCells();
		}

		smls->generateRawStitchPolys();
		smls->generateAllPatchesBnspe();

		smls->stitchAllPolys();

		if(smls->autoJoin){
			smls->joinAllParts();
		}



		//patch->processFields();
		houseKeepVisualClue();
		updateBuild3dWindow();

	}
	else if(WaveToPoly* waveToPoly = dynamic_cast<WaveToPoly*>(g_selNode.getRaw())){
		waveToPoly->findSignal();
	}
	else{
		disabledMessage();
		return;
	}
	if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		M_Vec3f& point = bezierPatch->point.refValue();
		const M_Vec2f& dim = bezierPatch->dim;
		int uDim=dim[0].x;
		int vDim=dim[0].y;
		for(int v = vDim; --v>=(vDim>>1);){
			for(int u = uDim; --u>= 0;){
				Vec3f h = point[(vDim - v - 1) * uDim + u];
				point[(vDim - v - 1) * uDim + u] = point[v * uDim + u];
				point[v * uDim + u] = h;
			}
		}
		for(int v = vDim; --v>=0;){
			for(int u = uDim; --u>=(uDim>>1);){
				Vec3f h = point[v * uDim + (uDim - u - 1)];
				point[v * uDim + (uDim - u - 1)] = point[v * uDim + u];
				point[v * uDim + u] = h;
			}
		}
	}
	else if(NurbsLathe* lathe = dynamic_cast<NurbsLathe*>(g_selNode.getRaw())){
		M_Vec2f& point = lathe->point.refValue();
		int len = point.length;
		int to = len >> 1;
		for(int k = 0; k < to; k++){
			Vec2f store = point[k];
			point[k] = point[len - 1 - k];
			point[len - 1 - k] = store;
		}
		lathe->processFields();
		updateBuild3dWindow();
	}
	else{
		return;
	}
	updateBuild3dWindow();
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}

void insideOutPatch(){
	//disabledMessage();
	//return;

	if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		nurbsPatch->setInsideOut2();
		nurbsPatch->processFields();
		updateBuild3dWindow();
		houseKeepVisualClue();
		BackUp::monostable.beginCycle();
	}
}

extern TransformPtr g_nurbsIlsBase;
void fillPolyLineIndex(M_Vec3f& point, M_Int& coordIndex);

/*
void alignAllMidPoints(){
	if(NurbsPatch* nurbsPatch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		M_Vec3f& dpoint = nurbsPatch->dpoint.refValue();
		M_Vec3f& point = nurbsPatch->point.refValue();
		const M_Float& weight = nurbsPatch->weight;
		int uDim = nurbsPatch->dim.x;
		int vDim = nurbsPatch->dim.y;

		for(int v=0; v<vDim; v++){
			for(int u=0; u<uDim; u+=2){
				int i = v*uDim + u;
				if(u == 0){
					point[i+uDim-1]=point[i]=(dpoint[i+1]+(dpoint[i+uDim-2]-dpoint[i+1])*.5)/weight[i];
				}
				else if(u == uDim-1){
					point[i-uDim+1]=point[i]=(dpoint[i-1]+(dpoint[i-uDim+2]-dpoint[i-1])*.5)/weight[i];
				}
				else{
					point[i]=(dpoint[i-1]+(dpoint[i+1]-dpoint[i-1])*.5)/weight[i];
				}
			}
		}

		nurbsPatch->processFields();
	}
	else{
		return;
	}
	updateBuild3dWindow();
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}
*/

inline void bubbleControlPointLeftToRight(Matrix& viewMatrix, M_ControlPoint& pointList){
	bool flag;
	do{
		flag = false;
		for(int i=0; i < pointList.length - 1; i++){

			if((viewMatrix * pointList[i].p->dpoint[pointList[i].i]).x > (viewMatrix * pointList[i + 1].p->dpoint[pointList[i + 1].i]).x){

				ControlPoint swap = pointList[i + 1];
				pointList[i + 1] = pointList[i];
				pointList[i] = swap;

				flag = true;
			}
		}
	}while(flag);
}

inline void bubbleControlPointBottomToTop(Matrix& viewMatrix, M_ControlPoint& pointList){
	bool flag;
	do{
		flag = false;
		for(int i=0; i < pointList.length - 1; i++){

			if((viewMatrix * pointList[i].p->dpoint[pointList[i].i]).y > (viewMatrix * pointList[i + 1].p->dpoint[pointList[i + 1].i]).y){

				ControlPoint swap = pointList[i + 1];
				pointList[i + 1] = pointList[i];
				pointList[i] = swap;

				flag = true;
			}
		}
	}while(flag);
}

void alignMidSelectededPoints(){
	if(dynamic_cast<NurbsPatch*>(g_selNode.getRaw()) == 0){
		return;
	}
	float mostLeft = 100000;
	float mostRight = -100000;
	float mostBottom = 100000;
	float mostTop = -100000;
	Matrix viewMatrix = g_scene->getViewMatrix();
	for(int i = MultiSelect::pointList.length; --i>=0;){
		int idx = MultiSelect::pointList[i].i;
		Vec3f v = viewMatrix * MultiSelect::pointList[i].p->dpoint[idx];
		if(v.x < mostLeft){
			mostLeft = v.x;
		}
		if(v.x > mostRight){
			mostRight = v.x;
		}
		if(v.y < mostBottom){
			mostBottom = v.y;
		}
		if(v.y > mostTop){
			mostTop = v.y;
		}
	}

	if(abs(mostTop - mostBottom) > abs(mostLeft - mostRight)){
		bubbleControlPointBottomToTop(viewMatrix, MultiSelect::pointList);
	}
	else{
		bubbleControlPointLeftToRight(viewMatrix, MultiSelect::pointList);
	}
	int len = MultiSelect::pointList.length;
	int last = len - 1;
	Vec3f firstV = MultiSelect::pointList[0].p->dpoint[MultiSelect::pointList[0].i];
	Vec3f lastV = MultiSelect::pointList[last].p->dpoint[MultiSelect::pointList[last].i];
	Vec3f diff = lastV - firstV;


	for(int i = 1; i < last; i++){
		NurbsPatch* patch = MultiSelect::pointList[i].p;
		int idx = MultiSelect::pointList[i].i;

		float w = patch->weight[idx];
		Vec3f pnt = (firstV + diff / last * i) * w;
		patch->point.refValue()[idx] = pnt;
		patch->smls->cp.refValue()[patch->cpIndex[idx]] = pnt;
		patch->dpoint.refValue()[idx] = pnt / w;
	}
	MultiSelect::pointList.length = 0;
	updateBuild3dWindow();
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}


/*
void oldAlignMidPoint(){
	if(NurbsPatch* nurbsPatch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		M_Vec3f& dpoint = nurbsPatch->dpoint.refValue();
		M_Vec3f& point = nurbsPatch->point.refValue();
		const M_Float& weight = nurbsPatch->weight;
		int uDim = nurbsPatch->dim.x;

		int i = nurbsPatch->index[0];
		if(i%uDim == 0){
			point[i+uDim-1]=point[i]=(dpoint[i+1]+(dpoint[i+uDim-2]-dpoint[i+1])*.5)/weight[i];
		}
		else if(i%uDim == uDim-1){
			point[i-uDim+1]=point[i]=(dpoint[i-1]+(dpoint[i-uDim+2]-dpoint[i-1])*.5)/weight[i];
		}
		else{
			point[i]=(dpoint[i-1]+(dpoint[i+1]-dpoint[i-1])*.5)/weight[i];
		}

		nurbsPatch->processFields();
	}
	else{
		return;
	}
	updateBuild3dWindow();
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}
*/
void outputPovRayMovie(){
	Anim* anim = Window::animMode();
	g_exportMovie->exportScene(anim);
}

const char audio[3][5]={
	".wav",
	".mp3",
	".au",
};

void addAudioFileName(const String& fname, String& ffmpegLine){
	for(int i=0; i<3; i++){
		String audioFile=fname;
		audioFile+=audio[i];
		FILE* file=fopen(audioFile.c_str(), "rb");
		if(file){
			fclose(file);
//			if(g_scene->movieAVI){
//				ffmpegLine+=" -acodec mp3"; //caused crash in ffmpeg auto installer
//			}
			ffmpegLine+=" -i ";
			ffmpegLine+=audioFile;
			break;
		}
	}
}

void addMovieFileExtension(String& fname){
	ExportMovie* exportMovie = dynamic_cast<ExportMovie*>(g_scene->exportMovie.p);

	if(exportMovie->mpg){
		fname+=".mpg";
	}
	else if(exportMovie->mp4){
		fname+=".mp4";
	}
	else if(exportMovie->avi){
		fname+=".avi";
	}
	else if(exportMovie->flv){
		fname+=".flv";
	}
}

void createFFmpegCommand(String& ffmpegLine, String& fname, bool batchFile){
	ExportMovie* exportMovie = dynamic_cast<ExportMovie*>(g_scene->exportMovie.p);

	Anim* anim = Window::animMode();
	int numOfFrames = anim->period*anim->stepsPerSecond+1;

	String filename = numOfFrames;
	int numOfDigits = filename.length;

	filename = g_currentFilename;

	String drive;
	String dir;
	String ext;
	String iniFileName;
	splitPath(filename, drive, dir, fname, ext);




	//getFFmpegPath(ffmpegLine);
	ffmpegLine+=" -r ";
	ffmpegLine+=anim->stepsPerSecond;
	ffmpegLine+=" -f image2 -i ";
	ffmpegLine+=fname;
	if(batchFile){
		ffmpegLine+="%";
	}
	ffmpegLine+="%0";
	ffmpegLine+=numOfDigits;
	ffmpegLine+="d.png";
	if(exportMovie->avi){
		ffmpegLine+=" -vcodec msmpeg4v2";
	}

	addAudioFileName(fname, ffmpegLine);

	ffmpegLine+=" -sameq ";
	
	//fname+="-movie";
}

void renderScene();
extern Pix24* g_frameBuffer;
void createPNG24(const char* filename, int width, int height, Pix24* pixels);
void createBatchFile(String& batchFileName);
bool g_screenshotEarly = false;
bool g_screenshotLate = false;
extern bool g_recordSoundMovie;
extern bool g_editVideoFrames;
void openWithFFmpeg(String url);
void outMovieFrames(bool outputSoundTrack, bool outputFrames);

void outputScreenShotMovie(){
	Anim* anim = Window::animMode();

	if(g_editVideoFrames){
		if(Video* video = dynamic_cast<Video*>(anim->video.p)){
			outMovieFrames(true, true);
			return;
		}
	}

	g_recordSoundMovie = false;
	g_screenshotLate = false;
	g_screenshotEarly = false;
	if(g_settings->screenshotWindows){
		g_screenshotLate = true;
	}
	else{
		g_screenshotEarly = true;
	}


	int width = g_3d->size.x;
	int height = g_3d->size.y;

	g_frameBuffer = new Pix24[width * height];
	String filename = g_currentFilename;
	String drive;
	String dir;
	String fname;
	String ext;
	String iniFileName;
	splitPath(filename, drive, dir, fname, ext);


	float time = 0;	///g_freezeTime->time;//needs to be fixed
	float multiply = 1;
	float period = 1;
	float step = 0;

	if(anim){
		multiply = anim->speed;
		period = anim->period;
		step = 1.0 / (anim->stepsPerSecond * period);
		anim->slider = 0;
	}
	

	int numOfFrames = 1;


	numOfFrames += period * anim->stepsPerSecond;

	String str;
	String cntrStr;

	str = numOfFrames;
	int numOfDigits = str.length;

	//////////// start inc files
	static M_<Effect*> effectList;
	effectList.length=0;
	float remTime=0; //freezeTime->time;
	for(int i = 0; i < numOfFrames; i++){
		anim->slider = step * i * multiply + time;
		//SendMessage(hWndMain, WM_PAINT, 0, 0);
		renderScene();

		str = drive;
		str += dir;
		str += fname;
		//outPovRay(str, effectList, Matrix(), Matrix());
		cntrStr = i;
		while(cntrStr.length < numOfDigits){
			cntrStr = '0' + cntrStr;
		}
		str += cntrStr;

		str += ".png";
		
		createPNG24(str.c_str(), width, height, g_frameBuffer);
	}


	g_screenshotLate = false;
	g_screenshotEarly = false;

	delete[] g_frameBuffer;
	g_frameBuffer = 0;

	//createBatchFile(str);
	String ffmpegLine;
	createFFmpegCommand(ffmpegLine, fname, false);
	addMovieFileExtension(fname);
	DeleteFile(fname.c_str());
	ffmpegLine += fname;
//	ffmpegLine+=fname;

	
	openWithFFmpeg(ffmpegLine);
}

void outputScreenShotSingle(){
	g_recordSoundMovie = false;
	g_screenshotLate = false;
	g_screenshotEarly = false;
	if(g_settings->screenshotWindows){
		g_screenshotLate = true;
	}
	else{
		g_screenshotEarly = true;
	}

	int width = g_3d->size.x;
	int height = g_3d->size.y;
	g_frameBuffer = new Pix24[width * height];

	renderScene();
	
	SYSTEMTIME ToDaySystemTime;
	GetLocalTime(&ToDaySystemTime);
	char tmpChar[0x200];
	ToDaySystemTime.wYear -= 2000;
	sprintf(tmpChar,"%.2i%.2i%.2i_%.2i%.2i%.2i.png",ToDaySystemTime.wYear, ToDaySystemTime.wMonth, ToDaySystemTime.wDay, ToDaySystemTime.wHour,ToDaySystemTime.wMinute, ToDaySystemTime.wSecond);
	
	String str = g_screenshotDir;
	str += tmpChar;
	createPNG24(str.c_str(), width, height, g_frameBuffer);

	
	g_screenshotLate = false;
	g_screenshotEarly = false;

	delete[] g_frameBuffer;
	g_frameBuffer = 0;

}
bool modifyVrmlX3dFilename(String& filename, bool writeX3DXML, bool writeX3DVRMLClassic);

void writeOutputHeader(String& str,  const Properties* switchProp, const Properties* lodProp);
void changeFloatFormat(int numOfDigits);
void listRoutes(String&);
extern const char* msg_Can_not_create_output_file_name;

void outputVrmlMorph2(Seamless* smls, Anim* anim){
	g_vrmlSquareOpenBracket[0]='[';
	g_vrmlSquareCloseBracket[0]=']';
	g_vrmlQuote[0]='\"';

	String filename = g_currentFilename;

	ExportVrmlX3d* exportVrmlX3d = dynamic_cast<ExportVrmlX3d*>(g_scene->exportVrmlX3d.p);
	
	if(false == modifyVrmlX3dFilename(filename, exportVrmlX3d->x3dXml, exportVrmlX3d->x3dVrmlClassic)){
		msg(msg_Can_not_create_output_file_name,MB_OK | MB_ICONEXCLAMATION);
		return;
	};
	if(exportVrmlX3d->gZip && (exportVrmlX3d->x3dXml || exportVrmlX3d->x3dVrmlClassic)){
		filename+='z';
	}
	bool remGenNormals = g_genNormals;

	g_formatCoords = exportVrmlX3d->roundCoords;
	g_outputMode = true;
	g_genNormals = false;
	g_genIdxTriSetNodes=false;
	g_writeX3DXML = false;
	g_writeX3DVRMLClassic = false;
	g_genHAnimNodes=false;
	g_genContactSBVT = exportVrmlX3d->genContactSBVT;
	g_genColorIndex = true;

	M_Node temp;
	float multiply = anim->speed;
	float period = anim->period;
	float step = 1.0 / (anim->stepsPerSecond * period);
	anim->slider = 0;
	int numOfFrames = 1;
	numOfFrames += period * anim->stepsPerSecond;
	ScenePtr newScene = new Scene;

	CoordinateInterpolator* interpolator = new CoordinateInterpolator;
	interpolator->name = "interpolator";

	TimeSensor* timeSensor = new TimeSensor;
	timeSensor->cycleInterval = anim->period;
	timeSensor->loop = true;
	timeSensor->name = "timeSensor";
	String str;

	for(int i = 0; i < numOfFrames; i++){
		anim->slider = step * i * multiply;
		//SendMessage(hWndMain, WM_PAINT, 0, 0);
		renderScene();

		str="";
		temp.length = 0;
		g_defLookUp.length=0;
		smls->outputToString(str, false);
		g_defLookUp.length=0;
		createSmlsFromString(temp, str);
		ShapePtr shape = dynamic_cast<Shape*>(temp[temp.length-1].p);
		IndexedFaceSet* ifs = dynamic_cast<IndexedFaceSet*>(shape->geometry.p);
		Coordinate* coord = dynamic_cast<Coordinate*>(ifs->coord.p);
		interpolator->key.refValue()+=anim->slider;
		interpolator->keyValue.refValue()+=coord->point;
		if(i == 0){
			newScene->children.refValue() += temp;
			coord->name = "coord";
			S_Route(timeSensor, timeSensor->getFieldIndex(&timeSensor->fraction_changed), interpolator, interpolator->getFieldIndex(&interpolator->set_fraction));
			S_Route(interpolator, interpolator->getFieldIndex(&interpolator->value_changed), coord, coord->getFieldIndex(&coord->point));
		}
	}
	newScene->children.refValue() += interpolator;
	newScene->children.refValue() += timeSensor;

	g_writeX3DXML = exportVrmlX3d->x3dXml;
	g_writeX3DVRMLClassic = exportVrmlX3d->x3dVrmlClassic;

	_chdir(g_seamlessDir.c_str());
	writeOutputHeader(str,  0, 0);

	g_defLookUp.length=0;
	changeFloatFormat(g_formatCoords=g_settings->roundCoordsSave.getValue());
	if(g_writeX3DXML){
		g_vrmlSquareOpenBracket[0]=0;
		g_vrmlSquareCloseBracket[0]=0;
		g_vrmlQuote[0]=0;
		str+='\n';
		newScene->children.toXMLString(str, false, "children");
		g_vrmlSquareOpenBracket[0]='[';
		g_vrmlSquareCloseBracket[0]=']';
		g_vrmlQuote[0]='\"';
	}
	else{
		M_Node& children = newScene->children;
		for(int i=0;i<children.length;i++){
			str+='\n';
			children[i]->toString(str, false);
		}
	}
	changeFloatFormat(g_formatCoords=g_settings->roundCoordsSave.getValue());

	listRoutes(str);
	if(g_writeX3DXML){
		str+="\n</Scene>\n</X3D>";
	}


	//if(g_writeX3DXML){
	//	g_vrmlSquareOpenBracket[0]=0;
	//	g_vrmlSquareCloseBracket[0]=0;
	//	g_vrmlQuote[0]=0;
	//	str+='\n';
	//	scene->toXMLString(str, true, "children");
	//	str+="\n<ROUTE fromNode='interpolator' fromField='value_changed' toNode='coord' toField='point'/>\n<ROUTE fromNode='timeSensor' fromField='fraction_changed' toNode='interpolator' toField='set_fraction'/>\n</Scene>\n</X3D>";
	//	g_vrmlSquareOpenBracket[0]='[';
	//	g_vrmlSquareCloseBracket[0]=']';
	//	g_vrmlQuote[0]='\"';
	//}
	//else{
	//	scene->toString(str);
	//	//str+="\nROUTE timeSensor.fraction_changed TO interpolator.set_fraction\nROUTE interpolator.value_changed TO coord.point\n";
	//}
	g_writeGzipped = exportVrmlX3d->gZip;
	str.write(filename);
	g_writeGzipped=g_settings->saveGZipped;

	g_outputMode=false;
	g_writeX3DXML=false;
	g_writeX3DVRMLClassic=false;
	g_genContactSBVT=true;
	g_genNormals=true;
	g_genColorIndex=false;

	g_genNormals = remGenNormals;
}


#ifdef DEMO_TO_MOVIE

int g_frame = 0;
int g_clipCounter = 0;

void outputScreenShotForDemo1(){
	g_recordSoundMovie = false;
	g_screenshotLate = false;
	g_screenshotEarly = false;
	if(g_settings->screenshotWindows){
		g_screenshotLate = true;
	}
	else{
		g_screenshotEarly = true;
	}

	int width = g_3d->size.x;
	int height = g_3d->size.y;
	g_frameBuffer = new Pix24[width * height];
}

void outputScreenShotForDemo2(){
	int width = g_3d->size.x;
	int height = g_3d->size.y;

	String filename = g_screenshotDir;

	filename+= "\\clip";

	String cntrStr = g_clipCounter;
	while(cntrStr.length < 4){
		cntrStr = '0' + cntrStr;
	}
	filename += cntrStr;
	filename += "\\frame";

	cntrStr = g_frame;
	while(cntrStr.length < 5){
		cntrStr = '0' + cntrStr;
	}
	filename += cntrStr;

	filename += ".png";

	createPNG24(filename.c_str(), width, height, g_frameBuffer);
	
	g_screenshotLate = false;
	g_screenshotEarly = false;

	delete[] g_frameBuffer;
	g_frameBuffer = 0;
	g_frame++;

}
#endif

void createBatchFile(String& batchFileName){
	ExportVrmlX3d* exportVrmlX3d = dynamic_cast<ExportVrmlX3d*>(g_scene->exportVrmlX3d.p);
	String ffmpegLine;
	String fname;
	getFFmpegPath(ffmpegLine);
	createFFmpegCommand(ffmpegLine, fname, true);

	batchFileName=fname;
	batchFileName+=".bat";
	addMovieFileExtension(fname);

	ffmpegLine+=fname;
	ffmpegLine+="\r\npause";

	g_writeGzipped = false;
	ffmpegLine.write(batchFileName);
	g_writeGzipped = exportVrmlX3d->gZip;
}

void outputBatchFile(){
	String batchFileName;
	String outputMsg;
	createBatchFile(batchFileName);
	extractFileName(outputMsg, batchFileName);
	outputMsg+="\n\nhas been created";
	msG(outputMsg);
}


void endInitConvertToPatch(NurbsPatch* patch, M_Node& stemStore){
	Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
	patch->sync.refValue() = true;
	stem->sync.refValue() = true;
	stemStore = patch->stem;
	if(g_convertToPatch->stem == false){
		patch->stem.refValue().length = 0;
	}
}

void endInitConvertToPatch(M_<NurbsPatchPtr>& m_patch, M_Node& stemStore){
	stemStore.length = m_patch.length;
	for(int k = m_patch.length; --k>=0;){
		NurbsPatch* patch = m_patch[k];
		Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
		patch->sync.refValue() = true;
		stem->sync.refValue() = true;
		stemStore[k] = stem;
		if(g_convertToPatch->stem == false){
			patch->stem.refValue().length = 0;
		}
	}
}

void convertToPatch2(NurbsPatch* patch, Art* part, int selLine, M_Node& stemStore){
	patch->container = patch;
	int parentLine = getParent(selLine);
	Node* parentNode = g_sceneTreeList[parentLine].node.p;
	int fieldIdx = 0, element = 0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field = parentNode->getField(fieldIdx);
	patch->container = patch;
	patch->processFields();
	GroupBuild* group = new GroupBuild;
	group->part = part;
	group->build.refValue() += patch;

	if(field->getType() == MFNODE){
		MFNode *mfnode = dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element] = group;
		*mfnode = mfnode->refValue();
	}

	g_scene->genNodeList2();
	if(dynamic_cast<GroupBuild*>(g_selNode.getRaw())){
		g_stw->selLine++;
		if(group->part.p){
			g_stw->selLine++;
		}
	}
	

	endInitConvertToPatch(patch, stemStore);
}


void NurbsLathe::convertToPatch(){
	g_convertToPatch->ctrlStaves.refValue() = ctrlStaves;
	patch->showAll.refValue() = false;

	convertToPatch2(patch, part.refValue(), g_stw->selLine, g_convertToPatch->stemStore);

	if(mirrorContainer){
		patch->name = name;
		NurbsPatch* mirrorPatch = mirrorContainer->patch;
		mirrorPatch->name = mirrorContainer->name;
		convertToPatch2(mirrorPatch, mirrorContainer->part.refValue(), getNodeLine(mirrorContainer), g_convertToPatch->mirrorStemStore);
	}
	g_stw->commonPostInsertNodeStuff();
}

void BendyLathe::convertToPatch(){
	patch->container = patch;
	g_convertToPatch->ctrlStaves.refValue() = ctrlStaves;

	int selLine = g_stw->selLine;
	int parentLine=getParent(selLine);
	Node* parentNode= g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	patch->processFields();

	if(field->getType()==MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element]=patch;
		*mfnode=mfnode->refValue();
	}
	if(getSeamless2()->animCP){
		patch->tpoint = patch->point;
	}
	g_stw->commonPostInsertNodeStuff();
}
/*
void NurbsLathe::convertToBendy(){
	Ptr<BendyLathe> bendy = new BendyLathe;
	M_Float& radius = bendy->radius.refValue();
	M_Float& at = bendy->at.refValue();
	M_Vec3f& bendyPoint = bendy->point.refValue();
	M_Float& bendyWeight = bendy->weight.refValue();
	M_Float& bendyKnot = bendy->knot.refValue();
	float minT = bendy->minT();
	float maxT = bendy->maxT();

	int selLine = g_stw->selLine;
	int parentLine=getParent(selLine);
	Node* parentNode= g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	bendy->part.refValue() = part.refValue();
	bendy->patch = patch;
	bendy->textureMapper.refValue() = textureMapper;
	bendy->color.refValue() = color;
	bendy->scale.refValue() = scale;
	bendy->rotation.refValue() = rotation;
	bendy->translation.refValue() = translation;
	bendy->staves.refValue() = staves;
	bendy->bands.refValue() = bands;
//	bendy->point.length = point.length;
	float lowestPoint = getLowestPoint(point.refValue());
	float highestPoint = getHighestPoint(point.refValue());
	float diff = highestPoint - lowestPoint;
	int numOfRings = point.length;
	int numOfBendyPonts = bendy->point.length;
	radius.length = numOfRings;
	at.length = numOfRings;

	for(int i = 0; i < numOfBendyPonts; i++){
		bendyPoint[i].y = lowestPoint + diff/(numOfBendyPonts-1) * i;
	}
	for(int i = 0; i < numOfRings; i++){
		//at[i] = 1.0/(numOfRings-1) * i;
		at[i] = bendy->zeroinOnVerticalPoint(bendyPoint, bendyWeight, bendyKnot, minT, maxT, point[i].y);
		radius[i] = point[i].x;
	}

	bendy->processFields();

	if(field->getType()==MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element]=bendy;
		*mfnode=mfnode->refValue();
	}
	g_stw->commonPostInsertNodeStuff();
}
*/

void NurbsLathe::convertToBendy(){
	Ptr<BendyLathe> bendy = new BendyLathe;
	M_Float& radius = bendy->radius.refValue();
	M_Float& at = bendy->at.refValue();
	M_Vec3f& bendyPoint = bendy->point.refValue();
	M_Float& bendyWeight = bendy->weight.refValue();
	M_Float& bendyKnot = bendy->knot.refValue();
	float minT = bendy->minT();
	float maxT = bendy->maxT();

	int selLine = g_stw->selLine;
	int parentLine=getParent(selLine);
	Node* parentNode= g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	bendy->part.refValue() = part.refValue();
	bendy->stem = stem;
	bendy->patch = patch;
	bendy->color.refValue() = color;
	bendy->scale.refValue() = scale;
	bendy->rotation.refValue() = rotation;
	bendy->translation.refValue() = translation;
	bendy->staves.refValue() = staves;
	bendy->bands.refValue() = bands;
//	bendy->point.length = point.length;
	float lowestPoint = getLowestPoint(point.refValue());
	float highestPoint = getHighestPoint(point.refValue());
	float diff = highestPoint - lowestPoint;
	int numOfRings = point.length;
	int numOfBendyPonts = bendy->point.length;
	radius.length = numOfRings;
	at.length = numOfRings;

	for(int i = 0; i < numOfBendyPonts; i++){
		bendyPoint[i].y = lowestPoint + diff/(numOfBendyPonts-1) * i;
	}
	for(int i = 0; i < numOfRings; i++){
		//at[i] = 1.0/(numOfRings-1) * i;
		at[i] = bendy->zeroinOnVerticalPoint(bendyPoint, bendyWeight, bendyKnot, minT, maxT, point[i].y);
		radius[i] = point[i].x;
	}

	bendy->processFields();

	if(field->getType()==MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element]=bendy;
		*mfnode=mfnode->refValue();
	}
	g_stw->commonPostInsertNodeStuff();
}

void BezierLatheBuild::convertToPatch(){
	g_convertToPatch->ctrlStaves.refValue()=4;
	g_convertToPatch->ctrlBands.refValue()=3;
	
	int selLine = g_stw->selLine;
	int parentLine=getParent(selLine);
	Node* parentNode= g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	if(nopfs.length > 1){
		createUniqueName(colorSweep);
		if(textureMapper.length < nopfs.length){
			for(int k = textureMapper.length; --k>=0;){
				createUniqueName(textureMapper[k]);
			}
		}
	}

	if(field->getType()==MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element] = m_patch[0];
		for(int k = 1; k < m_patch.length; k++){
			mfnode->refValue().insertElement(++element, m_patch[k].p);
		}
		if(m_patch.length > 1){
			mfnode->refValue().insertElement(++element, joinVertex.p);
		}
		*mfnode=mfnode->refValue();
	}
	if(getSeamless2()->animCP){
		for(int k = m_patch.length; --k >= 0;){
			m_patch[k]->tpoint = m_patch[k]->point;
		}
	}
	g_stw->commonPostInsertNodeStuff();
}

void SphereBuild::convertToPatch(){
	patch->container = patch;
	g_convertToPatch->ctrlStaves.refValue() = 4;
	g_convertToPatch->ctrlBands.refValue() = 3;
	

	convertToPatch2(patch, part.refValue(), g_stw->selLine, g_convertToPatch->stemStore);

	if(mirrorContainer){
		patch->name = name;
		NurbsPatch* mirrorPatch = mirrorContainer->patch;
		mirrorPatch->name = mirrorContainer->name;
		convertToPatch2(mirrorPatch, mirrorContainer->part.refValue(), getNodeLine(mirrorContainer), g_convertToPatch->mirrorStemStore);
	}


	//if(getSeamless2()->animCP){
	//	patch->tpoint = patch->point;
	//}
	g_stw->commonPostInsertNodeStuff();
}
void TorusBuild::convertToPatch(){
	patch->container = patch;
	g_convertToPatch->ctrlStaves.refValue()=6;
	g_convertToPatch->ctrlBands.refValue()=4;
	
	int selLine = g_stw->selLine;
	int parentLine=getParent(selLine);
	Node* parentNode= g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	if(field->getType()==MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element] = patch;
		*mfnode=mfnode->refValue();
	}
	//if(getSeamless2()->animCP){
	//	patch->tpoint = patch->point;
	//}
	g_stw->commonPostInsertNodeStuff();
}

void RectangleBuild::convertToPatch(){
	patch->container = patch;
	int selLine = g_stw->selLine;
	int parentLine = getParent(selLine);
	Node* parentNode = g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	if(field->getType() == MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element] = patch;
		*mfnode = mfnode->refValue();
	}
	//if(getSeamless2()->animCP){
	//	patch->tpoint = patch->point;
	//}
	endInitConvertToPatch(patch, g_convertToPatch->stemStore);
	g_stw->commonPostInsertNodeStuff();
}

bool convertNurbsLinesToStrands(Seamless* smls){
	if(smls->partList.length){
		Art* part = smls->partList[0];
		Hair* hair = new Hair;
		hair->part.refValue() = part;

		for(int k = 0; k < smls->cpNodeList.length; k++){
		

			if(NurbsLine* line = dynamic_cast<NurbsLine*>(smls->cpNodeList[k])){
				Strand* strand = new Strand;

				strand->point.refValue() = line->point;
				strand->dpoint.refValue() = line->dpoint;
				strand->weight.refValue() = line->weight;
				strand->smls = smls;

				hair->strand.refValue() += strand;
				strand->processFields();
			}
		}


		smls->build.refValue() += hair;
		hair->processFields();
		g_stw->commonPostInsertNodeStuff();
		return true;
	}
	return false;

}

void convertNurbsLinesToStrands(){
	if(Seamless* smls = getSeamless2()){
		convertNurbsLinesToStrands(smls);
	}
}

bool convertNurbsLineToBendyLathe(NurbsLine* line){
	resetTemporaryModes();
	Seamless* smls = line->smls;
	if(smls->partList.length){
		Art* part = smls->partList[0];

		openSceneTree();
		BendyLathe* bendyLathe= new BendyLathe;
		bendyLathe->bands.refValue() = 25;
		bendyLathe->part = part;

		int lineLen = line->point.length;;

		bendyLathe->point.refValue() = line->point;
		bendyLathe->dpoint.refValue() = line->dpoint;

		bendyLathe->knot.refValue() = line->uKnot;

		bendyLathe->at.refValue().length = lineLen;
		bendyLathe->radius.refValue().length = lineLen;

		for(int i = 0; i < lineLen; i++){
			bendyLathe->radius.refValue()[i] = .1;
			float a = 1.0f / (lineLen - 2) * i;
			if(i == lineLen - 1){
				a = 1;
			}
			else if(i > 0){
				a -= 1.0f / ((lineLen - 2) * 2.0f);
			}
			bendyLathe->at.refValue()[i] = a;
		}


		if(dynamic_cast<TextureEffect*>(part->workEffect)){
			bendyLathe->textureMapper = new TextureMapper;
		}
		createNewBuildNode(bendyLathe, part->smls, part);

		return true;
	}
	return false;
}


void addIfNotInList(NurbsLine* line, M_<NurbsLinePtr>& lineList){
	for(int k = lineList.length; --k>=0;){
		if(line == lineList[k].p){
			return;
		}
	}
	lineList += line;
}


bool getListOfSelectedLines(M_<NurbsLinePtr>& lineList){
	for(int k = 0; k < MultiSelect::pointList.length; k++){
		if(NurbsLine* line = dynamic_cast<NurbsLine*>(MultiSelect::pointList[k].p)){
			addIfNotInList(line, lineList);
		}
	}
	return lineList.length > 0;
}

bool NurbsPatch::centerBackface(){
	int uDim = dim.x;
	int vDim = dim.y;

	int u = uDim >> 1;
	int v = vDim >> 1;

	int idxA = v * uDim + u;
	int idxB = idxA + 1;
	int idxC = idxA + uDim;

	g_viewMatrix = g_scene->getViewMatrix();

	Vec3f vecA = g_viewMatrix * dpoint[idxA];
	Vec3f vecB = g_viewMatrix * dpoint[idxB];
	Vec3f vecC = g_viewMatrix * dpoint[idxC];
	
	return backFace(vecA, vecB, vecC);
}

Art* getLastSelectedPart(bool select){
	if(select){
		for(int k = 0; k < g_sceneTreeList.length; k++){
			if(g_lastSelectedPart == g_sceneTreeList[k].node){
				g_stw->selLine = k;
				g_stw->setResultsForSelectingVertex(g_lastSelectedPart);
				//g_stw->cp->setupNodeWindow(g_lastSelectedPart);
				break;
			}
		}
	}
	return g_lastSelectedPart;
}

void NurbsLine::convertToThickLine(NurbsLine* lineB, NurbsPatch* patch, bool insideOut){
	patch->name = name;
	patch->mirror.refValue() = mirror;
	patch->insideOut.refValue() = insideOut;
	patch->smls = smls;
	int uDim = dim.x * 2 - 3;
	int vDim = 3;
	patch->point.refValue().length = uDim * vDim;
	patch->dpoint.refValue().length = uDim * vDim;
	patch->cpIndex.refValue().length = uDim * vDim;
	patch->weight.refValue().length = uDim * vDim;
	patch->dim.refValue() = Vec2f(uDim, vDim);


	int mark = smls->cp.length;

	for(int i = 0; i < uDim * vDim; i++){
		patch->weight.refValue()[i] = 1;
		patch->cpIndex.refValue()[i] = mark + i;
	}

	if(lineB == 0){
		Vec3f centerPoint;
		for(int k = 0; k < point.length - 1; k++){
			centerPoint += point[k];
		}
		centerPoint = centerPoint / (point.length - 1);

		for(int u = 0, i = 0; i < point.length; i++){
			patch->point.refValue()[u] = point[i];
			patch->point.refValue()[u + uDim] = (point[i] + centerPoint) / 2.0f;
			patch->point.refValue()[u + uDim * 2] = centerPoint;
			u++;
			if(u > 1 && u < uDim - 1){
				u++;
			}
		}
		patch->joinEastToWest();
		//patch->scrunchSouth();
		patch->scrunchNorth();
		//patch->fillEdgesWithCtrlPolys();
	}
	else{
		const M_Vec3f& pointB = lineB->point;

		for(int u = 0, i = 0; i < point.length; i++){
			patch->point.refValue()[u] = point[i];
			patch->point.refValue()[u + uDim] = (point[i] + pointB[i]) / 2.0f;
			patch->point.refValue()[u + uDim * 2] = pointB[i];
			u++;
			if(u > 1 && u < uDim - 1){
				u++;
			}
		}
		if(uJoinEnds){
			patch->joinEastToWest();
		}
	}




	smls->cp.refValue() += patch->point.refValue();
	smls->cw.refValue() += patch->weight.refValue();


	patch->pointsToDpoints();

	updateNurbsCurveKnotMid1(uDim, patch->uKnot.refValue());
	updateNurbsCurveKnotMid1(vDim, patch->vKnot.refValue());

	patch->container = patch;

/*
	int selLine = getNodeLine(this);
	int parentLine = getParent(selLine);
	Node* parentNode = g_sceneTreeList[parentLine].node.p;
	int fieldIdx = 0, element = 0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field = parentNode->getField(fieldIdx);

	if(field->getType() == MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element] = patch;
		*mfnode = mfnode->refValue();
	}

	endInitConvertToPatch(patch, g_convertToPatch->stemStore);
	g_stw->commonPostInsertNodeStuff();
*/
	patch->processFields();
	patch->isCtrlPolyRef = true;

}

NurbsPatch* NurbsLine::convertToPatch2(NurbsLine* lineB, bool insideOut){
	NurbsPatch* patch = new NurbsPatch;
	patch->name = name;
	patch->mirror.refValue() = mirror;
	patch->insideOut.refValue() = insideOut;
	patch->smls = smls;
	Stem* stem = new Stem;
	stem->part.refValue() += smls->partList[0];
	stem->insideOut.refValue() = insideOut;
	ColorSweep* colorSweep = new ColorSweep;
	colorSweep->straight.refValue() = true;
	colorSweep->color.refValue() += color[1];
	colorSweep->color.refValue() += color[1];
	colorSweep->distance.refValue() += 1;
	stem->textureMapper.refValue() += colorSweep;
	patch->stem.refValue() += stem;
	g_convertToPatch->stem.refValue() = true;
	patch->uJoinEnds.refValue() = false;
	int uDim = dim.x * 2 - 3;
	int vDim = 3;
	patch->point.refValue().length = uDim * vDim;
	patch->dpoint.refValue().length = uDim * vDim;
	patch->cpIndex.refValue().length = uDim * vDim;
	patch->weight.refValue().length = uDim * vDim;
	patch->dim.refValue() = Vec2f(uDim, vDim);


	int mark = smls->cp.length;

	for(int i = 0; i < uDim * vDim; i++){
		patch->weight.refValue()[i] = 1;
		patch->cpIndex.refValue()[i] = mark + i;
	}

	if(lineB == 0){
		Vec3f centerPoint;
		for(int k = 0; k < point.length - 1; k++){
			centerPoint += point[k];
		}
		centerPoint = centerPoint / (point.length - 1);

		for(int u = 0, i = 0; i < point.length; i++){
			patch->point.refValue()[u] = point[i];
			patch->point.refValue()[u + uDim] = (point[i] + centerPoint) / 2.0f;
			patch->point.refValue()[u + uDim * 2] = centerPoint;
			u++;
			if(u > 1 && u < uDim - 1){
				u++;
			}
		}
		patch->joinEastToWest();
		//patch->scrunchSouth();
		patch->scrunchNorth();
		//patch->fillEdgesWithCtrlPolys();
	}
	else{
		const M_Vec3f& pointB = lineB->point;

		for(int u = 0, i = 0; i < point.length; i++){
			patch->point.refValue()[u] = point[i];
			patch->point.refValue()[u + uDim] = (point[i] + pointB[i]) / 2.0f;
			patch->point.refValue()[u + uDim * 2] = pointB[i];
			u++;
			if(u > 1 && u < uDim - 1){
				u++;
			}
		}
		if(uJoinEnds){
			patch->joinEastToWest();
		}
	}




	smls->cp.refValue() += patch->point.refValue();
	smls->cw.refValue() += patch->weight.refValue();


	patch->pointsToDpoints();

	updateNurbsCurveKnotMid1(uDim, patch->uKnot.refValue());
	updateNurbsCurveKnotMid1(vDim, patch->vKnot.refValue());

	patch->container = patch;
	int selLine = getNodeLine(this);
	int parentLine = getParent(selLine);
	Node* parentNode = g_sceneTreeList[parentLine].node.p;
	int fieldIdx = 0, element = 0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field = parentNode->getField(fieldIdx);

	if(field->getType() == MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element] = patch;
		*mfnode = mfnode->refValue();
	}
	//if(getSeamless2()->animCP){
	//	patch->tpoint = patch->point;
	//}

	endInitConvertToPatch(patch, g_convertToPatch->stemStore);
	g_stw->commonPostInsertNodeStuff();

	patch->processFields();
	patch->isCtrlPolyRef = true;


	return patch;
}

void createThickLine(NurbsLine* line, M_Vec3f& point, const int uDim, const int vDim){
	int lineDim = line->point.length;
	float section = lineDim - 2;
	section = 1.0 / section;

	double t = 0;
	int u = 0;

	for(int i = 0; i < lineDim; i++){
		Rotation rot = line->getOrientation(t);
		float thick = g_drawNurbsLine->thickness;
		if(g_useWinTab && g_drawNurbsLine->thickPressure){
			thick *= g_drawNurbsLine->m_processedPressure[i];
		}

		Vec3f l = rot * Vec3f(thick, 0, 0);
		Vec3f r = rot * Vec3f(-thick, 0, 0);

		point[u] += l;
		point[uDim * 2 + u] += r;

		u++;

		if(i == 0){
			t += section * .5;
		}
		else if(i == lineDim - 2){
			t = 1;
		}
		else{
			t += section;
			u++;
		}
	}
}


bool NurbsLine::convertToThickLine(NurbsPatch* patch){
	M_<NurbsPatchPtr> m_delPatchList;

	convertToThickLine(this, patch, false);

	createThickLine(this, patch->point.refValue(), patch->dim.x, patch->dim.y);
	patch->processFields();
	patch->pointsToCps();

	/*
	if(false == patch->centerBackface()){
		patch->setInsideOut2();
		patch->processFields();

		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			mirrorPatch->setInsideOut2();
			mirrorPatch->processFields();
		}


	}
	*/

	patch->fillEdgesWithCtrlPolys();
	smls->stitchSinglePatchPolys(patch, 1);
	smls->calcAllPolyMidPointsAndpatchToCentralCPs(patch, 1);
	patch->synchronizeCellsAndBuild();

	if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
		mirrorPatch->fillEdgesWithCtrlPolys();

		mirrorPatch->fillEdgesWithCtrlPolys();
		smls->stitchSinglePatchPolys(mirrorPatch, 1);
		smls->calcAllPolyMidPointsAndpatchToCentralCPs(mirrorPatch, 1);
		mirrorPatch->synchronizeCellsAndBuild();
	}
	
	deletePatches(m_delPatchList);

	g_stw->commonPostInsertNodeStuff();



	

	Art* part = getLastSelectedPart(false);
	if(part == 0 || part->smls != smls){
		part = smls->partList[0];
		g_lastSelectedPart = part;
	}
	
	if(part && part->name==""){
		createUniqueName(part);
		g_stw->paint();
	}

	//smls->genVisibleMask();
	patch->houseKeep();

	return true;
}



bool NurbsLine::convertToPatch(NurbsLine* lineB){
	M_<NurbsPatchPtr> m_delPatchList;
	if(lineB == 0 && uJoinEnds == false){
		return false;
	}

	NurbsLinePtr mirrorLine = 0;

	if(lineB){
		m_delPatchList += lineB;
		if(NurbsPatch* mirrorLineB = dynamic_cast<NurbsLine*>(lineB->mirrorPatch)){
			m_delPatchList += mirrorLineB;
		}
	}

	if(mirrorLine = dynamic_cast<NurbsLine*>(mirrorPatch)){
		NurbsLine* mirrorLineB = 0;
		if(lineB){
			mirrorLineB = dynamic_cast<NurbsLine*>(lineB->mirrorPatch);
		}
		mirrorLine->convertToPatch2(mirrorLineB, true);
	}
	NurbsPatch* patch = convertToPatch2(lineB, false);

	if(false == patch->centerBackface()){
		patch->setInsideOut2();
		patch->processFields();

		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			mirrorPatch->setInsideOut2();
			mirrorPatch->processFields();
		}


	}

	patch->fillEdgesWithCtrlPolys();
	smls->stitchSinglePatchPolys(patch, 1);
	smls->calcAllPolyMidPointsAndpatchToCentralCPs(patch, 1);
	patch->synchronizeCellsAndBuild();

	if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
		mirrorPatch->fillEdgesWithCtrlPolys();

		mirrorPatch->fillEdgesWithCtrlPolys();
		smls->stitchSinglePatchPolys(mirrorPatch, 1);
		smls->calcAllPolyMidPointsAndpatchToCentralCPs(mirrorPatch, 1);
		mirrorPatch->synchronizeCellsAndBuild();
	}
	
	deletePatches(m_delPatchList);

	g_stw->commonPostInsertNodeStuff();



	

	Art* part = getLastSelectedPart(false);
	if(part == 0 || part->smls != smls){
		part = smls->partList[0];
		g_lastSelectedPart = part;
	}
	
	if(part && part->name==""){
		createUniqueName(part);
		g_stw->paint();
	}


	return true;
}

void convertToPatch3(M_<NurbsPatchPtr>& m_patch, Art* part, int selLine, M_Node& stemStore, M_Node& textureMapper){
	int parentLine = getParent(selLine);
	Node* parentNode = g_sceneTreeList[parentLine].node.p;
	int fieldIdx = 0, element=0;

	for(int k = 0; k < m_patch.length; k++){
		NurbsPatch* patch = m_patch[k];
		patch->container = patch;
		Stem* stem = dynamic_cast<Stem*>(patch->stem.refValue()[0].p);
		stem->bands.refValue()[0] = 16;
		stem->staves.refValue() = 16;
	}

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	if(textureMapper.length == 0 || dynamic_cast<Stem*>(textureMapper[0].p) == 0){
		Stem* stem = dynamic_cast<Stem*>(m_patch[0]->stem[0].p);
		ColorSweep* colorSweep = dynamic_cast<ColorSweep*>(stem->textureMapper[0].p);
		createUniqueName(colorSweep);
	}

	if(textureMapper.length < 6){
		for(int k = textureMapper.length; --k>=0;){
			createUniqueName(textureMapper[k]);
		}
	}

	GroupBuild* group = new GroupBuild;
	group->part = part;

	if(field->getType() == MFNODE){
		MFNode *mfnode = dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element] = group; 
		for(int k = 0; k < m_patch.length; k++){
			NurbsPatch* patch = m_patch[k];
			patch->container = patch;
			group->build.refValue() += patch;
		}
		if(textureMapper.length){
			g_stw->selLine++;
		}
		*mfnode = mfnode->refValue();
	}
	g_scene->genNodeList2();
	if(dynamic_cast<GroupBuild*>(g_selNode.getRaw())){
		g_stw->selLine++;
		if(group->part.p){
			g_stw->selLine++;
		}
	}

	//if(getSeamless2()->animCP){
	//	for(int k = m_patch.length; --k >= 0;){
	//		m_patch[k]->tpoint = m_patch[k]->point;
	//	}
	//}
	endInitConvertToPatch(m_patch, stemStore);
}

void BoxBuild::convertToPatch(int nodeLine, M_Node& stemStore, M_Node& mirrorStemStore){
	convertToPatch3(m_patch, part.refValue(), nodeLine, stemStore, textureMapper.refValue());

	if(mirrorContainer){
		for(int k = m_patch.length; --k>=0;){
			NurbsPatch* patch = m_patch[k];
			patch->name = name + k;
			NurbsPatch* mirrorPatch = mirrorContainer->m_patch[k];
			mirrorPatch->name = mirrorContainer->name + k;

			patch->makeExistingNameUniqueForMirror(mirrorPatch);
		}
		convertToPatch3(mirrorContainer->m_patch, mirrorContainer->part.refValue(), getNodeLine(mirrorContainer), mirrorStemStore, mirrorContainer->textureMapper.refValue());
	}
}

void BoxBuild::convertToPatch(){
	if(mirrorContainer && getPatch()->plidx < mirrorContainer->getPatch()->plidx){
		mirrorContainer->convertToPatch(getNodeLine(mirrorContainer), g_convertToPatch->mirrorStemStore, g_convertToPatch->stemStore);
	}
	else{
		convertToPatch(g_stw->selLine, g_convertToPatch->stemStore, g_convertToPatch->mirrorStemStore);
		g_stw->genNodeList();
		g_stw->selLine = getNodeLine(getPatch());
	}
	if(mirror && mirrorContainer == 0){
		m_patch[1]->name = "r_side";
		m_patch[1]->makeExistingNameUniqueForMirror(m_patch[2]);
	}

	g_stw->commonPostInsertNodeStuff();
}

void CylinderBuild::convertToPatch(int nodeLine, M_Node& stemStore, M_Node& mirrorStemStore){
	convertToPatch3(m_patch, part.refValue(), nodeLine, stemStore, textureMapper.refValue());

	if(mirrorContainer){
		for(int k = m_patch.length; --k>=0;){
			NurbsPatch* patch = m_patch[k];
			patch->name = name + k;
			NurbsPatch* mirrorPatch = mirrorContainer->m_patch[k];
			mirrorPatch->name = mirrorContainer->name + k;
		}
		convertToPatch3(mirrorContainer->m_patch, mirrorContainer->part.refValue(), getNodeLine(mirrorContainer), mirrorStemStore, mirrorContainer->textureMapper.refValue());
	}
}

void CylinderBuild::convertToPatch(){
	if(mirrorContainer && getPatch()->plidx < mirrorContainer->getPatch()->plidx){
		mirrorContainer->convertToPatch(getNodeLine(mirrorContainer), g_convertToPatch->mirrorStemStore, g_convertToPatch->stemStore);
	}
	else{
		convertToPatch(g_stw->selLine, g_convertToPatch->stemStore, g_convertToPatch->mirrorStemStore);
		g_stw->genNodeList();
		g_stw->selLine = getNodeLine(getPatch());
	}

	g_stw->commonPostInsertNodeStuff();
}

void ConeBuild::convertToPatch(int nodeLine, M_Node& stemStore, M_Node& mirrorStemStore){
	convertToPatch3(m_patch, part.refValue(), nodeLine, stemStore, textureMapper.refValue());

	if(mirrorContainer){
		for(int k = m_patch.length; --k>=0;){
			NurbsPatch* patch = m_patch[k];
			patch->name = name + k;
			NurbsPatch* mirrorPatch = mirrorContainer->m_patch[k];
			mirrorPatch->name = mirrorContainer->name + k;
		}
		convertToPatch3(mirrorContainer->m_patch, mirrorContainer->part.refValue(), getNodeLine(mirrorContainer), mirrorStemStore, mirrorContainer->textureMapper.refValue());
	}
}

void ConeBuild::convertToPatch(){
	if(mirrorContainer && getPatch()->plidx < mirrorContainer->getPatch()->plidx){
		mirrorContainer->convertToPatch(getNodeLine(mirrorContainer), g_convertToPatch->mirrorStemStore, g_convertToPatch->stemStore);
	}
	else{
		convertToPatch(g_stw->selLine, g_convertToPatch->stemStore, g_convertToPatch->mirrorStemStore);
		g_stw->genNodeList();
		g_stw->selLine = getNodeLine(getPatch());
	}

	g_stw->commonPostInsertNodeStuff();
}

void Seamless::addCps(Build* build, int mark){
	M_Vec3f& cp = this->cp.refValue();
	M_Float& cw = this->cw.refValue();
	cp.length = mark;
	cw.length = mark;
	int numOfPatches = build->numOfPatches();

	int plidx = build->getPatch()->plidx;

	for(int k = 0; k < numOfPatches; k++){

		NurbsPatch* patch = cpNodeList[plidx + k];
		int pointLen = patch->point.length;
		M_Int& cpIndex = patch->cpIndex;
		cpIndex.length = pointLen;
		cp += patch->point;
		cw += patch->weight;

		for(int i = 0; i < pointLen; i++){
			cpIndex[i] = mark + i;
		}

		mark += pointLen;
	}
}

void ConvertToPatch::convertToPatch(){
	M_<NurbsLinePtr> linesList;

	mirrorConvertBuild = 0;

	M_Properties& properties = g_lookupProtoInterface[CONVERTTOPATCH]->properties;

	properties[1].set_disable(true); //ctrlBands
	properties[2].set_disable(true); //ctrlZStaves
	set_all_weights_to_1.refValue() = false;

	if(Build* build = dynamic_cast<Build*>(g_selNode.getRaw())){
		convertBuild = build;
		mirrorConvertBuild = build->getMirrorBuild();

		if(Ptr<NurbsLathe> nurbsLathe = dynamic_cast<NurbsLathe*>(build)){
			ctrlStaves.refValue() = nurbsLathe->ctrlStaves;
			set_all_weights_to_1.refValue() = true;
			nurbsLathe->convertToPatch();
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		
		}
		if(Ptr<BezierLatheBuild> bezierLathe = dynamic_cast<BezierLatheBuild*>(build)){
			bezierLathe->convertToPatch();
			ctrlStaves.refValue() = bezierLathe->ctrlStaves;
			convertBuild=bezierLathe;
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		else if(Ptr<BendyLathe> bendyLathe = dynamic_cast<BendyLathe*>(build)){
			convertBuild = bendyLathe;
			ctrlStaves.refValue() = bendyLathe->ctrlStaves;
			bendyLathe->convertToPatch();
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		else if(Ptr<SphereBuild> sphereBuild = dynamic_cast<SphereBuild*>(build)){
			properties[1].set_disable(false); //ctrlBands
			ctrlStaves.refValue() = sphereBuild->ctrlStaves;
			ctrlBands.refValue() = sphereBuild->ctrlBands;
			convertBuild = sphereBuild;
			sphereBuild->convertToPatch();
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		else if(Ptr<CylinderBuild> cylinderBuild = dynamic_cast<CylinderBuild*>(build)){
			properties[1].set_disable(false); //ctrlBands
			ctrlStaves.refValue() = cylinderBuild->ctrlStaves;
			ctrlBands.refValue() = cylinderBuild->ctrlBands;
			cylinderBuild->convertToPatch();
			convertBuild=cylinderBuild;
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		else if(Ptr<ConeBuild> coneBuild = dynamic_cast<ConeBuild*>(build)){
			properties[1].set_disable(false); //ctrlBands
			ctrlStaves.refValue() = coneBuild->ctrlStaves;
			ctrlBands.refValue() = coneBuild->ctrlBands;
			coneBuild->convertToPatch();
			convertBuild=coneBuild;
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		else if(Ptr<TorusBuild> torusBuild = dynamic_cast<TorusBuild*>(build)){
			properties[1].set_disable(false); //ctrlBands
			ctrlStaves.refValue() = torusBuild->ctrlStaves;
			ctrlBands.refValue() = torusBuild->ctrlBands;

			torusBuild->convertToPatch();
			convertBuild=torusBuild;
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		else if(Ptr<RectangleBuild> rectangleBuild = dynamic_cast<RectangleBuild*>(build)){
			properties[1].set_disable(false); //ctrlBands
			ctrlStaves.refValue() = rectangleBuild->ctrlStaves;
			ctrlBands.refValue() = rectangleBuild->ctrlBands;

			rectangleBuild->convertToPatch();
			convertBuild=rectangleBuild;
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		else if(getListOfSelectedLines(linesList)){
			//properties[1].set_disable(false); //ctrlBands
			//ctrlStaves.refValue() = 1;
			//ctrlBands.refValue() = nurbsLine->dim.y;

			if(linesList.length == 1){
				linesList[0]->convertToPatch(0);
			}
			if(linesList.length == 2){
				linesList[0]->convertToPatch(linesList[1].p);
			}
			return;
			
			//convertBuild = nurbsLine;
			//g_stw->cp->setupNodeWindow(this);
			//Window::activeWindow=g_stw->cp.p;
		}
		else if(Ptr<BoxBuild> boxBuild = dynamic_cast<BoxBuild*>(build)){
			properties[1].set_disable(false); //ctrlBands
			properties[2].set_disable(false); //ctrlZStaves
			ctrlStaves.refValue() = boxBuild->ctrlStaves;
			ctrlBands.refValue() = boxBuild->ctrlBands;
			ctrlZStaves.refValue() = boxBuild->ctrlZStaves;

			boxBuild->convertToPatch();
			convertBuild = boxBuild;
			g_stw->cp->setupNodeWindow(this);
			Window::activeWindow=g_stw->cp.p;
		}
		g_stw->cp->paint();
		processFields();
	}
}

void convertToPatch(){
	if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		hair->convertToPatches();
		return;
	}

	Seamless* smls = getSeamless();
	smls->convertToCpFormat();
	g_convertToPatch->markCp = smls->cp.length;
	resetTemporaryModes();
	g_convertToPatch->convertToPatch();
}

void convertToBendy(){
	if(NurbsLathe* nurbsLathe = dynamic_cast<NurbsLathe*>(g_selNode.getRaw())){
		nurbsLathe->convertToBendy();
	}
}

void commonPostTug();
inline Matrix dxWindMatrix(Matrix& parentMatrix, Vec3f& scale, Rotation& rotation, Vec3f& translation);


void SphereBuild::doCPTransform(){
	if(this->patch->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}

void CylinderBuild::doCPTransform(){
	if(this->patch->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}

void ConeBuild::doCPTransform(){
	if(this->patch->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}

void TorusBuild::doCPTransform(){
	if(this->patch->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}

void BezierLatheBuild::doCPTransform(){
	if(this->m_patch[0]->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		translation.refValue() = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
		processFields();
	}
}

void BoxBuild::doCPTransform(){
	if(this->m_patch[0]->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}

void RectangleBuild::doCPTransform(){
	if(this->patch->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}

void NurbsLathe::doCPTransform(){
	if(this->patch->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		//translation.refValue() = g_transformCP->scale * (g_transformCP->rotation.refValue() * g_transformCP->translation - g_transformCP->pivotPoint) + g_transformCP->pivotPoint;
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}

void Seamless::doObjTransform(){
	rotation.refValue() = remRotation * g_transformObj->rotation;
	translation = g_transformObj->rotation.refValue() * (g_transformObj->scaleOrientation.refValue() * ((-g_transformObj->scaleOrientation.refValue() * (remTranslation - g_transformObj->pivotPoint) * g_transformObj->scale))) + g_transformObj->translation + g_transformObj->pivotPoint;
}

void BendyLathe::doCPTransform(){
	if(this->patch->object){
		rotation.refValue() = remRotation * g_transformCP->rotation;
		if(mirrorContainer){
			Rotation mirrorRot = rotation;
			mirrorRot.y = -mirrorRot.y;
			mirrorRot.z = -mirrorRot.z;
			mirrorContainer->rotation.refValue() = mirrorRot;
		}
		translation = g_transformCP->rotation.refValue() * (g_transformCP->scaleOrientation.refValue() * ((-g_transformCP->scaleOrientation.refValue() * (remTranslation - g_transformCP->pivotPoint) * g_transformCP->scale))) + g_transformCP->translation + g_transformCP->pivotPoint;
	}
}


void TransformObj::houseKeep(){
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_transformLeverClue->geometry.p);
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	coord.length = 0;
	for(int k = 0; k < rotCtrlPnt.length; k++){
		coordIndex += coord.length;
		coord += pp;
		coordIndex += coord.length;
		coord += rotCtrlPnt[k];
		coordIndex += -1;
	}

	polyLine->convertToRenderFormat(g_transformLeverClue->render);
}

void AnimLever::houseKeep(Art* part, Anim* anim){
	if(part->niki == g_stw->cp->node){
		return;
	}
	g_vcArtAnimBase->anim = anim;

	//IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	//M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	//coord.length = 0;
	//coord += Vec3f();
	//coord += Vec3f(1, 0, 0);
	//fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
	//g_vcArtAnimBase->rotCtrlPnt = part->pivotPointMatrix * Vec3f(0, 0, 0) + Vec3f(.5, 0, 0);
	//polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);


///////////////


	//Vec3f pivotPoint = part->absolutePivotPoint;
	Vec3f pivotPoint  = part->pivotPointMatrix * Vec3f(0,0,0);

	//Vec3f pp = g_scene->getViewMatrix() * pivotPoint;

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();
	coord.length = 0;
	coordIndex.length = 0;

	for(int k = 0; k < rotCtrlPnt.length; k++){
		coordIndex += coord.length;
		coord += pivotPoint;
		coordIndex += coord.length;
		coord += rotCtrlPnt[k];
		coordIndex += -1;
	}

	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void AnimLever::houseKeep(){
	if(g_scene->mode & ANIMATION_MODE){
		if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
			g_vcArtAnimBase->houseKeep(part, Window::getAnimBar());
		}
	}
}


void TransformObj::processFields(){
 	if(MultiSelect::beginDragPos.length != MultiSelect::objectList.length){
		refreshRemPoints();
	}
	

	Matrix matrix = translationMatrix(-pivotPoint) * rotationMatrix(-scaleOrientation) * scaleMatrix(scale) * rotationMatrix(scaleOrientation) * rotationMatrix(rotation) * translationMatrix(pivotPoint) * translationMatrix(translation);

	if(MultiSelect::beginDragPos.length){
		M_Node& objectList = MultiSelect::objectList;
		for(int k = 0; k < objectList.length; k++){
			Node* node = objectList[k];
			if(Seamless* smls = dynamic_cast<Seamless*>(node)){
				smls->doObjTransform();
				smls->houseKeep();
			}
		}
	}
	houseKeep();

	commonPostTug();
}

void processAllPatches(M_<NurbsPatch*>& cpNodeList);

void TransformCP::processFieldsForAnim(Art* part){
	Seamless* smls = part->smls;
	if(MultiSelect::pointList.length){
		g_dragActuallyHappened = true;
		//Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * pt;
		//Vec3f diff = newPt - MultiSelect::beginDragPos[MultiSelect::clickedIndex];
		Matrix matrix = translationMatrix(-pivotPoint) * rotationMatrix(-scaleOrientation) * scaleMatrix(scale) * rotationMatrix(scaleOrientation) * rotationMatrix(rotation) * translationMatrix(pivotPoint) * translationMatrix(translation);

		//for(int i = MultiSelect::pointList.length; --i>=0;){
		//	NurbsPatch* patch = MultiSelect::pointList[i].p;
		//	if(dynamic_cast<NurbsPatch*>(patch->container)){
		//		int idx = MultiSelect::pointList[i].i;
		//		Vec3f svec = matrix * MultiSelect::beginDragPos[i];
		//		Vec3f pnt;
		//		float w = patch->weight[idx];
		//		setFilteredDimension(patch->point.refValue(), smls->cp.refValue(), patch->cpIndex.refValue(), idx, svec * patch->weight[idx]);
		//		patch->dpoint.refValue()[idx] = patch->point[idx] / patch->weight[idx];
		//	}
		//}


		for(int i = MultiSelect::pointList.length; --i>=0;){
			int idx = MultiSelect::pointList[i].i;
			NurbsPatch* patch = MultiSelect::pointList[i].p;
			Vec3f svec = matrix * MultiSelect::beginDragPos[i];
			g_transformCP->setFilteredDimension(patch->point.refValue(), smls->cp.refValue(), patch->cpIndex.refValue(), idx, svec * patch->weight[idx]);

			patch->smls->dragCtrlPoly(patch, idx);
		}
		processAllPatches(smls->cpNodeList);
		//commonPostTug();
		smls->updateCage();

		if(AnimLever* animLever = dynamic_cast<AnimLever*>(g_visualClue)){
			animLever->anim->captureMorph(smls);
		}
		Anim* anim = Window::getAnimBar();
		anim->update = true;
	}
}



void snapPivotPointToControlPoint(){
	if(MultiSelect::clickedIndex < MultiSelect::beginDragPos.length){
		Vec3f v = g_scene->getViewMatrix() * MultiSelect::beginDragPos[MultiSelect::clickedIndex];
		Vec3f pp = g_scene->getViewMatrix() * g_transformCP->pivotPoint;
		pp.y = v.y;

		g_transformCP->refreshRemPoints();
		g_transformCP->rotCtrlPnt[g_transformCP->leverIdx] = pp + Vec3f(.5, 0, 0);
		g_transformCP->pivotPoint = g_scene->getIViewMatrix() * pp;
	}
}

void openTransformCP(){
	if(	(g_scene->mode & MODELLING_MODE && dynamic_cast<Build*>(g_selNode.getRaw())) ||
		(g_scene->mode & ANIMATION_MODE && dynamic_cast<Art*>(g_selNode.getRaw()))
		){
		MultiSelect::beginDragPos.length = MultiSelect::pointList.length;

		g_transformCP->dragPivotPointMode = false;
		g_transformCP->dragRotPointMode = false;


		g_stw->cp->setupNodeWindow(g_transformCP);
		Window::activeWindow=g_stw->cp.p;
	}
	else if(g_scene->mode & SELECT_OBJECT_MODE){
		g_transformObj->mode = TransformObj::NEUTRAL;

		g_stw->cp->setupNodeWindow(g_transformObj);
		Window::activeWindow = g_stw->cp.p;
	}
	//else if(Oscillator* osc = dynamic_cast<Oscillator*>(g_selNode.getRaw())){
	//	M_Vec2f& fPoint = osc->note.refValue();
	//	for(int i = fPoint.length; --i>=0;){
	//		fPoint[i].y = fPoint[i].y + osc->transpose;
	//	}
	//	osc->processFields();
	//}
}

void stretchCurve(){//STRETCH_CURVE
	g_stw->cp->setupNodeWindow(g_stretchCurve);
	Window::activeWindow=g_stw->cp.p;
}

void ConvertToPatch::commonStemless(NurbsPatch* patch, M_Node& stemStore){
	if(stem != (patch->stem.length > 0)){
		if(stem){
			patch->stem.refValue() = stemStore;
		}
		else{
			patch->stem.refValue().length = 0;
		}
		g_scene->genNodeList2();
		g_stw->paint();
	}
}

void ConvertToPatch::commonStemless(M_<NurbsPatchPtr>& m_patch, M_Node& stemStore){
	for(int k = m_patch.length; --k>=0;){
		NurbsPatch* patch = m_patch[k];
		if(stem != (patch->stem.length > 0)){
			if(stem){
				patch->stem.refValue() = stemStore[k];
			}
			else{
				patch->stem.refValue().length = 0;
			}
		}
	}
	g_scene->genNodeList2();
	g_stw->paint();
}

void stitchPatches(M_<NurbsPatchPtr>& m_patch, int numOfPatches){
	NurbsPatch* patch = m_patch[0];
	Seamless* smls = patch->smls;
	int plidx = patch->plidx;
	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
	for(int k = 0; k < numOfPatches; k++){
		NurbsPatch* patch = cpNodeList[plidx + k];
		if(patch->stem.length){
			Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
			stem->sync.refValue() = true;
		}
		patch->synchronizeAllStemCells();
		patch->bnspeBuildPolies();
		patch->deleteStitchPolys();
		patch->fillEdgesWithCtrlPolys();
	}

}

void CylinderBuild::convertToPatch(ConvertToPatch* convertToPatch, M_Node& stemStore){
	ctrlStaves = convertToPatch->ctrlStaves;
	ctrlBands = convertToPatch->ctrlBands;

	for(int k = m_patch.length; --k>=0;){
		m_patch[k]->deleteStitchPolys();
	}
	convertToPatch->commonStemless(m_patch, stemStore);
}

void ConeBuild::convertToPatch(ConvertToPatch* convertToPatch, M_Node& stemStore){
	ctrlStaves = convertToPatch->ctrlStaves;
	ctrlBands = convertToPatch->ctrlBands;

	for(int k = m_patch.length; --k>=0;){
		m_patch[k]->deleteStitchPolys();
	}
	convertToPatch->commonStemless(m_patch, stemStore);
}

void BoxBuild::convertToPatch(ConvertToPatch* convertToPatch, M_Node& stemStore){
	ctrlStaves = convertToPatch->ctrlStaves;
	ctrlBands = convertToPatch->ctrlBands;
	ctrlZStaves = convertToPatch->ctrlZStaves;

	for(int k = m_patch.length; --k>=0;){
		m_patch[k]->deleteStitchPolys();
	}
	convertToPatch->commonStemless(m_patch, stemStore);
}

void NurbsPatch::joinEastToEast(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int v = vDimA; --v>=0;){
		cpIndexA[v * uDimA + uDimA - 1] = cpIndexB[v * uDimB + uDimB - 1];
	}
}

void NurbsPatch::joinWestToWest(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int v = vDimA; --v>=0;){
		cpIndexA[v * uDimA] = cpIndexB[v * uDimB];
	}
}

void NurbsPatch::joinEastToWest(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int v = vDimA; --v>=0;){
		cpIndexA[v * uDimA + uDimA - 1] = cpIndexB[v * uDimB];
	}
}

void NurbsPatch::joinWestToEast(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int v = vDimA; --v>=0;){
		cpIndexA[v * uDimA] = cpIndexB[v * uDimB + uDimB - 1];
	}
}

void NurbsPatch::joinNorthToSouth(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[u] = cpIndexB[vDimB * uDimB - uDimB + u];
	}
}

void NurbsPatch::joinEastToSouth(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[u] = cpIndexB[u * uDimB + uDimB - 1];
	}
}

void NurbsPatch::joinEastToSouthReverse(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[uDimA - 1 - u] = cpIndexB[u * uDimB + uDimB - 1];
	}
}

void NurbsPatch::joinWestToSouth(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[u] = cpIndexB[u * uDimB];
	}
}

void NurbsPatch::joinSouthToSouthReverse(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[u] = cpIndexB[uDimB - 1 - u];
	}
}


void NurbsPatch::joinNorthToNorthReverse(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[vDimA * uDimA - uDimA + u] = cpIndexB[vDimB * uDimB - 1 - u];
	}
}

void NurbsPatch::joinEastToNorth(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[vDimA * uDimA - uDimA + u] = cpIndexB[u * uDimB + uDimB - 1];
	}
}

void NurbsPatch::joinWestToNorth(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[vDimA * uDimA - uDimA + u] = cpIndexB[u * uDimB];
	}
}

void NurbsPatch::joinWestToNorthReverse(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[vDimA * uDimA - uDimA + u] = cpIndexB[(uDimA - 1 - u) * uDimB];
	}
}

void NurbsPatch::joinSouthToNorth(NurbsPatch* patchB){
	M_Int& cpIndexA = this->cpIndex.refValue();
	M_Int& cpIndexB = patchB->cpIndex.refValue();

	int uDimA = dim.x;
	int vDimA = dim.y;

	int uDimB = patchB->dim.x;
	int vDimB = patchB->dim.y;

	for(int u = uDimA; --u>=0;){
		cpIndexA[vDimA * uDimA - uDimA + u] = cpIndexB[u];
	}
}


void BoxBuild::joinStitchedEdges(){
	if(this->mirrorContainer){
		m_patch[0]->joinEastToWest(m_patch[2]);
		m_patch[2]->joinEastToWest(m_patch[3]);
		m_patch[2]->joinEastToSouthReverse(m_patch[4]);
		m_patch[2]->joinWestToNorthReverse(m_patch[5]);
	}
	else{
		m_patch[0]->joinEastToEast(m_patch[2]);
		m_patch[2]->joinWestToWest(m_patch[3]);
		m_patch[2]->joinEastToSouth(m_patch[4]);
		m_patch[2]->joinWestToNorth(m_patch[5]);
	}
	m_patch[3]->joinEastToWest(m_patch[1]);
	m_patch[0]->joinWestToEast(m_patch[1]);
	m_patch[0]->joinNorthToSouth(m_patch[4]);
	m_patch[1]->joinWestToSouth(m_patch[4]);
	m_patch[3]->joinSouthToSouthReverse(m_patch[4]);

	m_patch[0]->joinNorthToNorthReverse(m_patch[5]);
	m_patch[1]->joinEastToNorth(m_patch[5]);
	m_patch[3]->joinSouthToNorth(m_patch[5]);

	stitchPatches(m_patch, numOfPatches());
}

void NurbsLathe::joinStitchedEdges(){
	NurbsPatch* patch = getPatch();
	patch->beginSP.refValue() = false;
	patch->endSP.refValue() = false;
	patch->uJoinEnds.refValue() = false;

	patch->deleteStitchPolys();

	patch->joinEastToWest();
	if(beginSP){
		patch->scrunchSouth();
	}
	if(endSP){
		patch->scrunchNorth();
	}
	patch->fillEdgesWithCtrlPolys();
}

void SphereBuild::joinStitchedEdges(){
	NurbsPatch* patch = getPatch();

	patch->beginSP.refValue() = false;
	patch->endSP.refValue() = false;
	patch->uJoinEnds.refValue() = false;

	patch->deleteStitchPolys();

	patch->joinEastToWest();
	patch->scrunchSouth();
	patch->scrunchNorth();
	patch->fillEdgesWithCtrlPolys();
}

void CylinderBuild::joinStitchedEdges(){
	patch->joinEastToWest();
	if(topCap){
		topPatch->joinEastToWest();
		topPatch->scrunchNorth();
	}
	if(baseCap){
		basePatch->joinEastToWest();
		basePatch->scrunchSouth();
	}
	stitchPatches(m_patch, numOfPatches());
}

void ConeBuild::joinStitchedEdges(){
	patch->joinEastToWest();
	if(topCap){
		topPatch->joinEastToWest();
		topPatch->scrunchNorth();
	}
	if(baseCap){
		basePatch->joinEastToWest();
		basePatch->scrunchSouth();
	}
	stitchPatches(m_patch, numOfPatches());
}

void TorusBuild::joinStitchedEdges(){
	patch->deleteStitchPolys();
	patch->joinEastToWest();
	patch->joinNorthToSouth(patch);
	patch->fillEdgesWithCtrlPolys();
}

void RectangleBuild::joinStitchedEdges(){
	patch->deleteStitchPolys();
	patch->fillEdgesWithCtrlPolys();
}


void ConvertToPatch::processFields(Build* convertBuild, M_Node& stemStore, const int mark){
	if(NurbsLathe* nurbsLathe = dynamic_cast<NurbsLathe*>(convertBuild)){
		NurbsPatch* patch = nurbsLathe->getPatch();
		patch->deleteStitchPolys();
		nurbsLathe->ctrlStaves = ctrlStaves;
		commonStemless(patch, stemStore);
	}
	else if(BezierLatheBuild* bezierLathe = dynamic_cast<BezierLatheBuild*>(convertBuild)){
		bezierLathe->ctrlStaves = ctrlStaves;
	}
	else if(BendyLathe* bendyLathe = dynamic_cast<BendyLathe*>(convertBuild)){
		bendyLathe->ctrlStaves=ctrlStaves;
	}
	else if(SphereBuild* sphere = dynamic_cast<SphereBuild*>(convertBuild)){
		NurbsPatch* patch = sphere->getPatch();
		patch->deleteStitchPolys();
		sphere->ctrlStaves = ctrlStaves;
		sphere->ctrlBands = ctrlBands;
		commonStemless(patch, stemStore);
	}
	else if(CylinderBuild* cylinderBuild = dynamic_cast<CylinderBuild*>(convertBuild)){
		cylinderBuild->convertToPatch(this, stemStore);
	}
	else if(ConeBuild* coneBuild = dynamic_cast<ConeBuild*>(convertBuild)){
		coneBuild->convertToPatch(this, stemStore);
	}
	else if(TorusBuild* torusBuild = dynamic_cast<TorusBuild*>(convertBuild)){
		torusBuild->ctrlStaves=ctrlStaves;
		torusBuild->ctrlBands=ctrlBands;
	}
	else if(RectangleBuild* rectangleBuild = dynamic_cast<RectangleBuild*>(convertBuild)){
		commonStemless(rectangleBuild->getPatch(), stemStore);
		rectangleBuild->ctrlStaves=ctrlStaves;
		rectangleBuild->ctrlBands=ctrlBands;
	}
	else if(BoxBuild* boxBuild = dynamic_cast<BoxBuild*>(convertBuild)){
		boxBuild->convertToPatch(this, stemStore);
	}
	convertBuild->processFields();
	Seamless* smls = getSeamless();
	smls->addCps(convertBuild, mark);

	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->populateMirrorNodes();
		patch->smls->ctrf = true;
	}

	if(stitched){
		convertBuild->joinStitchedEdges();
	}
	if(set_all_weights_to_1){
		int plidx = convertBuild->getPatch()->plidx;
		for(int k = convertBuild->numOfPatches(); --k>=0;){
			smls->cpNodeList[plidx + k]->setAllWeightsToOne();
		}
	}


	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->populateMirrorNodes();
		patch->smls->ctrf = true;
	}

}

void ConvertToPatch::processFields(){
	Node* selPatch = g_selNode.getRaw();
	Seamless* smls = getSeamless();
	smls->cp.refValue().length = markCp;
	smls->cw.refValue().length = markCp;
	processFields(convertBuild, stemStore, markCp);

	if(Build* mirrorBuild = convertBuild->getMirrorBuild()){
		processFields(mirrorBuild, mirrorStemStore, smls->cp.length);
		g_stw->genNodeList();
		g_stw->selLine = getNodeLine(selPatch);
		g_stw->paint();
	}
	NurbsPatch* patch = convertBuild->getPatch();
	int numOfPatches = smls->cpNodeList.length - patch->plidx;

	smls->stitchSinglePatchPolys(patch, numOfPatches);

	if(dynamic_cast<SphereBuild*>(convertBuild.p)){
		smls->calcAllPolyMidPointsAndpatchToCentralCPs(patch, numOfPatches);
	}


	//calcAllPolyMidPoints();

	//int plidx = patch->plidx;
	//for(int k = 0; k < numOfPatches; k++){
	//	NurbsPatch* patch = smls->cpNodeList[plidx + k];
	//	patch->patchToCentralCPs();
	//}


	//smls->calcAllPolyMidPoints();

	//Seamless* smls = getSeamless();
	//smls->ctrf = true;
	//updateBuild3dWindow(true);
	doBuild();
	//searchDoBuild(g_scene->children.refValue());
	houseKeepVisualClue();
}
void NurbsPatch::rotateIndexOrder90(){
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Vec3f _point = point;
	M_Int _cpIndex = cpIndex;
	int uDim = dim.x;
	int vDim = dim.y;
	int u2 = 0;
	int v2 = uDim - 1;

	for(int v = 0; v < vDim; v++){
		for(int u = 0; u < uDim; u++){
			cpIndex[v * uDim + u] = _cpIndex[u2 * uDim + v2];

			int idx = cpIndex[v * uDim + u];
			Vec3f v = smls->cp[idx];

			u2++;
			if(u2 == vDim){
				u2 = 0;
				v2--;
			}
		}
	}
	dim.refValue() = Vec2f(vDim, uDim);
	M_Float knotStore = vKnot;
	vKnot.refValue() = uKnot;
	uKnot.refValue() = knotStore;
}
/*
void NurbsPatch::rotateIndexOrder90(){
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Vec3f _point = point;
	M_Float _weight = weight;
	M_Int _cpIndex = cpIndex;
	int uDim = dim.x;
	int vDim = dim.y;

	int i = 0;

	for(int u = uDim; --u >= 0;){
		for(int v = 0; v < vDim; v++){
			point[i] = _point[v * uDim + u];
			weight[i] = _weight[v * uDim + u];
			cpIndex[i] = _cpIndex[v * uDim + u];
			i++;
		}
	}
	dim.refValue() = Vec2f(vDim, uDim);
	M_Float knotStore = vKnot;
	vKnot.refValue() = uKnot;
	uKnot.refValue() = knotStore;
}
*/
void NurbsPatch::rotateIndexOrder180(){
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Vec3f _point = point;
	M_Float _weight = weight;
	M_Int _cpIndex = cpIndex;
	for(int i = _point.length; --i>=0;){
		point[i] = _point[point.length - 1 - i];
		weight[i] = _weight[point.length - 1 - i];
		cpIndex[i] = _cpIndex[point.length - 1 - i];
	}
}

void NurbsPatch::rotateIndexOrder270(){
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Int _cpIndex = cpIndex;
	int uDim = dim.x;
	int vDim = dim.y;
	int u2 = vDim - 1;
	int v2 = 0;

	for(int v = 0; v < vDim; v++){
		for(int u = 0; u < uDim; u++){
			cpIndex[v * uDim + u] = _cpIndex[u2 * uDim + v2];

			int idx = cpIndex[v * uDim + u];
			Vec3f v = smls->cp[idx];

			u2--;
			if(u2 == -1){
				u2 = vDim - 1;
				v2++;
			}
		}
	}
	dim.refValue() = Vec2f(vDim, uDim);
	M_Float knotStore = vKnot;
	vKnot.refValue() = uKnot;
	uKnot.refValue() = knotStore;
}
/*
void NurbsPatch::rotateIndexOrder270(){
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Vec3f _point = point;
	M_Float _weight = weight;
	M_Int _cpIndex = cpIndex;
	int uDim = dim.x;
	int vDim = dim.y;

	int i = 0;

	for(int u = 0; u < uDim; u++){
		for(int v = vDim; --v >= 0;){
			point[i] = _point[v * uDim + u];
			weight[i] = _weight[v * uDim + u];
			cpIndex[i] = _cpIndex[v * uDim + u];
			i++;
		}
	}
	dim.refValue() = Vec2f(vDim, uDim);
	M_Float knotStore = vKnot;
	vKnot.refValue() = uKnot;
	uKnot.refValue() = knotStore;
}
*/

void rotate_uv_90(){
	NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw());
	//M_Vec3f& point=patch->point.refValue();
	//M_Float& weight=patch->weight.refValue();
	//M_Int& cpIndex=patch->cpIndex.refValue();

	//M_Vec3f c_point=point;
	//M_Float c_weight=weight;
	//M_Int c_cpIndex=cpIndex;
	//int uDim = patch->dim.x;
	//int vDim = patch->dim.y;

	//for(int i = c_point.length; --i>=0;){
	//	point[i]=c_point[c_point.length-1-i];
	//	weight[i]=c_weight[c_point.length-1-i];
	//}
	//for(int r = 0; r < vDim; r++){
	//	for(int c = 0; c < uDim; c++){
	//		int rev = vDim-r-1;
	//		point[r*vDim + c] = c_point[rev * vDim + c];
	//		weight[r*vDim + c] = c_weight[rev * vDim + c];
	//	}
	//}

	Seamless* smls = patch->smls;
	smls->convertToCpFormat();
	patch->rotateIndexOrder90();
	if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
		mirrorPatch->rotateIndexOrder90();
	}
	//patch->setInsideOut2();
	smls->centralCPsToPatches();
	patch->processFields();
	patch->synchronizeAllStemCells();
	if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
		mirrorPatch->processFields();
		mirrorPatch->synchronizeAllStemCells();
	}

	smls->generateRawStitchPolys();
	smls->generateAllPatchesBnspe();

	smls->stitchAllPolys();

	if(smls->autoJoin){
		smls->joinAllParts();
	}



	//patch->processFields();
	houseKeepVisualClue();
	updateBuild3dWindow();
}

void reduceCapturedPoses(){
	g_stw->cp->setupNodeWindow(g_reduceCapturedPoses);
	Window::activeWindow = g_stw->cp.p;
}

void ReduceCapturedPoses::processFields(){
	Anim* anim = Window::animMode();
	if(Video* video = dynamic_cast<Video*>(anim->video.p)){
		int frameIndex = anim->selectFrom * anim->period / (1.0 / anim->stepsPerSecond) + .5;
		int frameTo =  anim->selectTo * anim->period / (1.0 / anim->stepsPerSecond) + .5;

		video->reduceFrames(frameIndex, frameTo, keepCaptureForEvery);
		return;
	}
	M_Node& interpolator = anim->children;
	int keepCaptureForEvery = g_reduceCapturedPoses->keepCaptureForEvery;

	for(int k = interpolator.length; --k>=0;){
		if(OrientationInterpolator* oi = dynamic_cast<OrientationInterpolator*>(interpolator[k].p)){
			M_Float& key = oi->key;
			M_Rotation& keyValue = oi->keyValue;

			int i = 1;
			int e = keepCaptureForEvery;

			for(; e<key.length; i++, e+=keepCaptureForEvery){
				key[i] = key[e];
				keyValue[i] = keyValue[e];
			}
			key[i] = key[key.length-1];
			keyValue[i] = keyValue[keyValue.length-1];
			key.length = i + 1;
			keyValue.length = i + 1;
		}
		else if(PositionInterpolator* pi = dynamic_cast<PositionInterpolator*>(interpolator[k].p)){
			M_Float& key = pi->key;
			M_Vec3f& keyValue = pi->keyValue;

			int i = 1;
			int e = keepCaptureForEvery;

			for(; e<key.length; i++, e+=keepCaptureForEvery){
				key[i] = key[e];
				keyValue[i] = keyValue[e];
			}
			key[i] = key[key.length-1];
			keyValue[i] = keyValue[keyValue.length-1];
			key.length = i + 1;
			keyValue.length = i + 1;
		}
	}
}

void convertPivotPointsToTranslations(Art* part, Vec3f accumTran){
	accumTran+=part->pivotPoint;
	part->translation.refValue() += part->pivotPoint;
	M_Vec3f& coord = part->coord;
	coord.translate(-accumTran);
	part->pivotPoint.refValue()=Vec3f();
	part->absolutePivotPoint=Vec3f();
	M_SArt& m_part = part->part.refValue();
	for(int k = 0; k < m_part.length; k++){
		convertPivotPointsToTranslations(m_part[k], accumTran);
	}
}
void convertPivotPointsToTranslations(Seamless* smls){
	M_SArt& skeleton = smls->skeleton.refValue();
	for(int k = 0; k < skeleton.length; k++){
		convertPivotPointsToTranslations(skeleton[k], Vec3f());
	}
}
void convertPivotPointsToTranslations(){
	Seamless* smls = dynamic_cast<Seamless*>(g_selNode.getRaw());
	convertPivotPointsToTranslations(smls);
	smls->convertToRenderFormat();
}
/////////////
void convertTranslationsToPivotPoints(Art* part, Vec3f accumTran){
	accumTran+=part->translation;
	part->pivotPoint.refValue() += part->translation;
	M_Vec3f& coord = part->coord;
	coord.translate(accumTran);
	part->translation.refValue()=Vec3f();
	part->absolutePivotPoint=accumTran;
	M_SArt& m_part = part->part.refValue();
	for(int k = 0; k < m_part.length; k++){
		convertTranslationsToPivotPoints(m_part[k], accumTran);
	}
}
void convertTranslationsToPivotPoints(Seamless* smls){
	M_SArt& skeleton = smls->skeleton.refValue();
	for(int k = 0; k < skeleton.length; k++){
		convertTranslationsToPivotPoints(skeleton[k], Vec3f());
	}
}
void convertTranslationsToPivotPoints(){
	Seamless* smls = dynamic_cast<Seamless*>(g_selNode.getRaw());
	convertTranslationsToPivotPoints(smls);
	smls->convertToRenderFormat();
}
void convertMercatorToNorthPole(M_Vec3f& coord);
void convertMercatorToSouthPole(M_Vec3f& coord);
void convertNorthPoleToMercator(M_Vec3f& coord);
void convertSouthPoleToMercator(M_Vec3f& coord);
void convertPoleToPole(M_Vec3f& coord);

void mercatorToNorthPole(){
	if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		convertMercatorToNorthPole(part->coord.refValue());
		Seamless* smls = part->smls;
		smls->ctrf=true;
		smls->doBuild();
		smls->convertToRenderFormat();
		BackUp::monostable.beginCycle();
	}
}
void mercatorToSouthPole(){
	if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		convertMercatorToSouthPole(part->coord.refValue());
		Seamless* smls = part->smls;
		smls->ctrf=true;
		smls->doBuild();
		smls->convertToRenderFormat();
		BackUp::monostable.beginCycle();
	}
}
void northPoleToMercator(){
	if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		convertNorthPoleToMercator(part->coord.refValue());
		Seamless* smls = part->smls;
		smls->ctrf=true;
		smls->doBuild();
		smls->convertToRenderFormat();
		BackUp::monostable.beginCycle();
	}
}
void southPoleToMercator(){
	if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		convertSouthPoleToMercator(part->coord.refValue());
		Seamless* smls = part->smls;
		smls->ctrf=true;
		smls->doBuild();
		smls->convertToRenderFormat();
		BackUp::monostable.beginCycle();
	}
}
void poleToPole(){
	if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		convertPoleToPole(part->coord.refValue());
		Seamless* smls = part->smls;
		smls->ctrf=true;
		smls->doBuild();
		smls->convertToRenderFormat();
		BackUp::monostable.beginCycle();
	}
}
void copyColorToPolyMap(){
	if(Stem* stem = dynamic_cast<Stem*>(g_selNode.getRaw())){
		if(stem->source.length){
			if(ImmediatePart* bluePart = dynamic_cast<ImmediatePart*>(stem->source.refValue()[0].p)){
				if(stem->part.length){
					if(Art* whitePart = dynamic_cast<Art*>(stem->part.refValue()[0].p)){
						if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(whitePart->workEffect)){
							M_Triangle& whiteTri = whitePart->triangle;
							M_Triangle& blueTri = bluePart->triangle;
							M_Vec3f& whiteCoord = whitePart->coord.refValue();
							M_Vec3f& blueCoord = bluePart->coord.refValue();
							M_Color3f& m_color = colorEffect->color.refValue();
							if(whiteTri.length == blueTri.length && whiteCoord.length == blueCoord.length){
								for(int i = whiteTri.getTriangleLength(); --i>=0;){
									m_color[blueTri.refTexIndex(i)] = m_color[whiteTri.refTexIndex(i)];
								}
							}
							Seamless* smls = bluePart->smls;
							smls->doBuild();
							smls->ctrf=true;
							BackUp::monostable.beginCycle();
							return;
						}
					}
				}
			}
		}
		else if(Art* whitePart = dynamic_cast<Art*>(stem->part.refValue()[0].p)){
			if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(whitePart->workEffect)){

				M_Triangle& whiteTri = whitePart->triangle;
				M_Int& coordIndex = stem->coordIndex;
				M_Color3f& dstColor = stem->color.refValue();
				M_Color3f& m_color = colorEffect->color.refValue();
				int beginTriOffset = stem->beginTriOffset * 3;
				for(int i = 0; i < coordIndex.length; i++){
					dstColor[coordIndex[i]] = m_color[whiteTri.refTexIndex(beginTriOffset + i)];
				}
				Seamless* smls = whitePart->smls;
				smls->doBuild();
				smls->ctrf=true;
				BackUp::monostable.beginCycle();
				return;
			}
		}
	}
	msG("Stem must have source and destination parts with equal triangle and coord length");
}


Art* storeGeneratedPolyMap(Stem* stem){
	if(stem->part.length){
		Art* whitePart = stem->part[0].p;
		ImmediatePart* bluePart = selectPolyMapPart(whitePart);
		stem->copyPolyMapToPart(bluePart);

		Stem tmpStem;
		tmpStem.copyFields(stem);
		tmpStem.source.refValue().length=0;
		tmpStem.part.refValue().length=0;
		tmpStem.part.refValue()+=bluePart;
		tmpStem.mirror.refValue()=false;
		tmpStem.add.refValue()=false;
		tmpStem.mould.refValue()=false;
		tmpStem.store.refValue()=false;
		tmpStem.doBuild();
		stem->copyPartToPolyMap(bluePart);
		return whitePart;
	}
	return 0;
}


void storeGeneratedPolyMap(){
	if(Stem* stem = dynamic_cast<Stem*>(g_selNode.getRaw())){
		if(stem->source.length){
			if(ImmediatePart* bluePart = dynamic_cast<ImmediatePart*>(stem->source.refValue()[0].p)){
#ifdef PROG_ROB
	dLog("new MoveToNode(" + String(g_stw->selLine) + "),");
	dLog("new RightClick,");
	dLog("new Move_MenuItem(menu_copy_poly_map_to_source),");
	dLog("new Click,");
#endif

				bluePart->deleteGeometry();
				Stem tmpStem;
				tmpStem.copyFields(stem);
				tmpStem.source.refValue().length=0;
				tmpStem.part.refValue().length=0;
				tmpStem.part.refValue()+=bluePart;
				tmpStem.mould.refValue()=false;
				tmpStem.add.refValue()=false;
				tmpStem.doBuild();

				bluePart->smls->doBuild();
				makePointerEqualIndexLocation();
				bluePart->smls->ctrf=true;
				Window::paintAllControlPanels();

			}
			else{
				msG("a blue part must be referenced for the source");
			}
		}
		else{
			if(Art* whitePart = storeGeneratedPolyMap(stem)){
				whitePart->smls->doBuild();
				makePointerEqualIndexLocation();
				whitePart->smls->ctrf=true;
				Window::paintAllControlPanels();
			}
		}
	}
}
void incToNextTriThatSharesActivePoint(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = part->smls;
		smls->ActTri=findNextTriangleFromVertex(part, smls->actTriPart, smls->ActCtrlPnt, smls->ActTriCorner,  smls->ActTri);
		makePointerEqualIndexLocation();
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->incToNextPolyThatSharesActivePoint();
	}
}
void decToNextTriThatSharesActivePoint(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = part->smls;
		smls->ActTri=findPrevTriangleFromVertex(part, smls->actTriPart, smls->ActCtrlPnt, smls->ActTriCorner,  smls->ActTri);
		makePointerEqualIndexLocation();
	}
	else if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		subdivision->decToNextPolyThatSharesActivePoint();
	}
}

void convertToSubdiv(NurbsPatch* patch, M_Vec3f& point, M_Int& coordIndex, int& offSet);

void convertToSubdiv(FuseSurface* fuseSurface, M_Vec3f& point, M_Int& coordIndex, int& offSet){
	if(NurbsPatch* patch=dynamic_cast<NurbsPatch*>(fuseSurface->patch.refValue().p)){
		convertToSubdiv(patch, point, coordIndex, offSet);
	}
}
void convertToSubdiv(Stem* stem, M_Vec3f& point, M_Int& coordIndex, int& offSet){
	M_Node& fuse = stem->fuse.refValue();
	for(int k = fuse.length; --k>=0;){
		convertToSubdiv(dynamic_cast<FuseSurface*>(fuse[k].p), point, coordIndex, offSet);
	}
}
void convertToSubdiv(NurbsPatch* patch, M_Vec3f& point, M_Int& coordIndex, int& offSet){
	point += patch->dpoint;
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	for(int v = 0; v < vDim-1; v++){
		for(int u = 0; u < uDim - 1; u+=2){
			coordIndex += offSet + v * uDim + u + 0;
			coordIndex += offSet + v * uDim + u + 2;
			coordIndex += offSet + v * uDim + u + 2 + uDim;
			coordIndex += offSet + v * uDim + u + 0 + uDim;
		}
	}
	offSet += patch->dpoint.length;

	M_Node& m_stem = patch->stem.refValue();
	for(int k = m_stem.length; --k>=0;){
		convertToSubdiv(dynamic_cast<Stem*>(m_stem[k].p), point, coordIndex, offSet);
	}
}
void convertToSubdiv(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Subdivision* subdiv = new Subdivision;
		M_Vec3f& point = subdiv->point.refValue();
		M_Int& coordIndex = subdiv->coordIndex.refValue();
		coordIndex.length = 0;
		point.length = 0;

		int offSet = 0;
		convertToSubdiv(patch, point, coordIndex, offSet);
		subdiv->processFields();



		int selLine = g_stw->selLine;
		int parentLine=getParent(selLine);
		Node* parentNode= g_sceneTreeList[parentLine].node.p;
		int fieldIdx=0, element=0;

		getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

		Field* field=parentNode->getField(fieldIdx);


		if(field->getType()==MFNODE){
			MFNode *mfnode=dynamic_cast<MFNode*>(field);
			mfnode->refValue()[element]=subdiv;
			*mfnode=mfnode->refValue();
		}
		g_stw->commonPostInsertNodeStuff();
	}
}
void compileBuildTransform(){
	int k = g_stw->selLine;
	int indent = g_sceneTreeList[k].indent;
	Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p);
	Matrix matrix = scaleMatrix(smls->scale) * rotationMatrix(smls->rotation) * translationMatrix(smls->translation);
	
	M_SArt& skeleton = smls->skeleton.refValue();
	for(int k = skeleton.length; --k>=0;){
		Art* part = skeleton[k];
		part->pivotPoint.refValue() += smls->translation;
	}
	while(++k < g_sceneTreeList.length && g_sceneTreeList[k].indent > indent){
		Node* node = g_sceneTreeList[k].node;
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(node)){
			if(patch->translation==Vec3f()){

				M_Vec3f& point = patch->point.refValue();
				M_Vec3f& dpoint = patch->dpoint.refValue();
				M_Float& weight = patch->weight.refValue();

				int uDim = patch->dim.x;
				int vDim = patch->dim.y;

				for(int i = uDim*vDim; --i>=0;){
					point[i] = matrix * dpoint[i] * weight[i];
				}
				patch->scale.refValue() = Vec3f(1,1,1);
				patch->rotation.refValue() = Rotation();
				patch->translation.refValue() = Vec3f();
			}
			else{
				patch->translation.refValue() += smls->translation;
			}
			patch->processFields();
		}
		else if(NurbsLathe* lathe = dynamic_cast<NurbsLathe*>(node)){
			lathe->translation.refValue() += smls->translation;
			lathe->processFields();
		}
		else if(BendyLathe* lathe = dynamic_cast<BendyLathe*>(node)){
			lathe->translation.refValue() += smls->translation;
			lathe->processFields();
		}
		else if(BezierLatheBuild* lathe = dynamic_cast<BezierLatheBuild*>(node)){
			lathe->translation.refValue() += smls->translation;
			lathe->processFields();
		}
		else if(TorusBuild* torus = dynamic_cast<TorusBuild*>(node)){
			torus->translation.refValue() += smls->translation;
			torus->processFields();
		}
		else if(SphereBuild* sphereBuild = dynamic_cast<SphereBuild*>(node)){
			sphereBuild->translation.refValue() += smls->translation;
			sphereBuild->processFields();
		}
		else if(CylinderBuild* cylinderBuild = dynamic_cast<CylinderBuild*>(node)){
			cylinderBuild->translation.refValue() += smls->translation;
			cylinderBuild->processFields();
		}
		else if(ConeBuild* coneBuild = dynamic_cast<ConeBuild*>(node)){
			coneBuild->translation.refValue() += smls->translation;
			coneBuild->processFields();
		}
		else if(ColorSweep* colorSweep = dynamic_cast<ColorSweep*>(node)){
			if(dynamic_cast<Stem*>(g_sceneTreeList[getParent(k)].node.p) == 0){
				colorSweep->location.refValue() += smls->translation;
			}
		}
	}
	smls->scale.refValue() = Vec3f(1,1,1);
	smls->rotation.refValue() = Rotation();
	smls->translation.refValue() = Vec3f();
	smls->process();
	smls->doBuild();
	smls->ctrf=true;
	BackUp::monostable.beginCycle();
}
void saveRelWinMousePos(){
	Vec2i pos = g_mousePos - Window::activeWindow->pos; 
	dLog("new MoveRelWindow(" + String(pos.x) + ", " + String(pos.y) + "),");
}

extern Coordinate* g_vcSkeletonLinesCoordinate;
extern Material* g_stickManMaterial;

void StickMan::addLineForBone(M_SArt& m_part, int parentLineIndex, M_Vec3f& point, M_Int& coordIndex, bool& beginNewLine){
	for(int k = 0; k < m_part.length; k++){
		Art* part = m_part[k];
		if(beginNewLine){
			if(coordIndex.length > 0){
				coordIndex += -1;
			}
			coordIndex += parentLineIndex;
			beginNewLine = false;
		}
		int last = point.length;
		coordIndex += last;
		partList += part;
		point += part->pivotPointMatrix * Vec3f(0,0,0);

		if(part->part.length){
			addLineForBone(part->part.refValue(), last, point, coordIndex, beginNewLine);
		}
		beginNewLine = true;
	}
	beginNewLine = true;
}


void StickMan::process(Seamless* smls){
	smls->stickMan = this;

	M_Vec3f& point = coord->point.refValue();
	M_Int& coordIndex = ils->coordIndex.refValue();

	point.length = 0;
	coordIndex.length = 0;
	partList.length = 0;

	bool beginNewLine = true;

	M_SArt& skeleton = smls->skeleton.refValue();
	for(int k = 0; k < skeleton.length; k++){
		Art* part = skeleton[k];
		if(part->part.length){
			partList += part;
			point += part->pivotPointMatrix * Vec3f(0,0,0);
			addLineForBone(part->part.refValue(), point.length-1, point, coordIndex, beginNewLine);
		}
	}
	if(partList.length == 0){
		if(smls->partList.length == 1){
			partList+= smls->partList[0];
		}
	}
	//Color* color = new Color;
	//color->color.refValue().length = point.length;
	//ils->color = color;
	//for(int i = color->color.refValue().length; --i >= 0;){
	//	color->color.refValue()[i] = Color3f(1,1,0);
	//}

	if(partList.length >= 2 && g_bReady){
		ils->convertToRenderFormat(shape->render);
	}
}

void Seamless::XRay::operator=(const bool& par){
	refValue() = par;
	if(par){
		home.stickMan = new StickMan;
		home.stickMan->process(&home);
	}
	else{
		delete home.stickMan;
		home.stickMan = 0;
	}
}

StickMan::StickMan():
	ils(new IndexedLineSet),
	coord(new Coordinate),
	shape(new Shape)
{
	Appearance* appearance = new Appearance;
	appearance->material = g_stickManMaterial;
	shape->appearance = appearance;

	ils->coord = coord;
	shape->geometry = ils;
}

void toggleXRayMode(){
	if(Seamless* smls = getSeamless2()){
		smls->xRay = !smls->xRay;
	}
}

void toggleBackfaceSelect(){

	g_scene->mode.refValue() ^= BACKFACE_IGNORE_MODE;
	updateIconPointers();
	g_tb2->paint();
	houseKeepVisualClue();
}

void reRenderPolys(){
	for(int k = g_sceneTreeList.length; --k>=0;){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->ctrf = true;
		}
	}
}

void reRenderPolysIfInHidePolyMode(){
	if(g_hidePolysInHiddenPatches){
		reRenderPolys();
	}
}

void toggleHidePolysInHiddenPatches(){
	g_hidePolysInHiddenPatches = !g_hidePolysInHiddenPatches;
	
	if(g_hidePolysInHiddenPatches){
		g_toggleHidePolysInHiddenPatchesButton->srcPos = Vec2i(24 * 12, 233 + 24);
		g_animToggleHidePolysInHiddenPatchesButton->srcPos = Vec2i(24 * 12, 233 + 24);
	}
	else{
		g_toggleHidePolysInHiddenPatchesButton->srcPos = Vec2i(24 * 11, 233 + 24);
		g_animToggleHidePolysInHiddenPatchesButton->srcPos = Vec2i(24 * 11, 233 + 24);
	}

	reRenderPolys();

}

void toggleAccumulateSelect(){
	MultiSelect::accumulate = !MultiSelect::accumulate;
	if(MultiSelect::accumulate){
		g_toggleAccumulateSelectButton->srcPos = Vec2i(24 * 11, 24);
		g_animToggleAccumulateSelectButton->srcPos = Vec2i(24 * 11, 24);
	}
	else{
		g_toggleAccumulateSelectButton->srcPos = Vec2i(24 * 7, 24 * 3);
		g_animToggleAccumulateSelectButton->srcPos = Vec2i(24 * 7, 24 * 3);
		MultiSelect::reset();
		houseKeepVisualClue();
	}
}

void manualCaptureIKPoints(Anim* anim, Art* part, Rotation accumRot){
	if(accumRot.angle != 0 || part->rotation.refValue().angle != 0){
		accumRot = part->rotation.refValue() * accumRot;
	}
	if(part->ik){
		part->ikPos = part->matrix * part->absolutePivotPoint;
		anim->captureSinglePartPose(part, &part->ikPos);
		if(part->pin){
			part->ikRot = part->rotation;
			anim->captureSinglePartPose(part, &part->ikRot);
		}
	}
	M_SArt& m_part = part->part.refValue();
	for(int k = m_part.length; --k>=0;){
		manualCaptureIKPoints(anim, m_part[k].p, part->rotation);
	}
}

void manualCaptureIKPoints(){
	if(Anim* anim = Window::animMode()){
		if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
			Seamless* smls = part->smls;
			M_SArt& skeleton = smls->skeleton.refValue();
			for(int k = skeleton.length; --k>=0;){
				manualCaptureIKPoints(anim, skeleton[k].p, smls->rotation);
			}
		}
	}
}

void manualCaptureIKPoint(){
	if(Anim* anim = Window::animMode()){
		if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
			if(part->ik){
				part->ikPos = part->matrix * part->absolutePivotPoint;
				anim->captureSinglePartPose(part, &part->ikPos);
			}
		}
	}
}

void captureFrame(){
	if(Anim* anim = Window::animMode()){
		if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
			Seamless* smls = part->smls;
			M_Art& partList = smls->partList;
			for(int k = partList.length; --k>=0;){
				part = partList[k];
				if(part->track){
					anim->captureMorph(part->smls);
					anim->captureSinglePartPose(part, &part->rotation);
				}
			}
		}
		else{
			needPartMsg();
		}
	}
}

void addOscToOsc(Oscillator* fromOsc, Oscillator* toOsc){
	toOsc->note.refValue() += fromOsc->note.refValue();
	toOsc->fWeight.refValue() += fromOsc->fWeight.refValue();
	updateNurbsCurveKnotMid0(toOsc->note.length, toOsc->fKnot.refValue());
	toOsc->processFields();
	toOsc->houseKeep();
}

void scaleOsc(Oscillator* osc){
	M_Vec2f& fPoint = osc->note;
	Vec2f pivotPoint;
	Vec2f scale;
	Vec2f translate;
	pivotPoint.x = g_transformCP->pivotPoint.x;
	pivotPoint.y = g_transformCP->pivotPoint.y;
	scale.x = g_transformCP->scale.x;
	scale.y = g_transformCP->scale.y;
	translate.x = g_transformCP->translation.x;
	translate.y = g_transformCP->translation.y;
	for(int i = fPoint.length; --i>=0;){
		fPoint[i] = (fPoint[i] - pivotPoint) * scale + pivotPoint + translate;
	}
	if(osc->delay != 0){
		osc->delay.refValue() = (osc->delay.refValue() - pivotPoint.x) * scale.x + pivotPoint.x + translate.x;
	}
	osc->processFields();
}

void showSingleCage(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int k = cpNodeList.length; --k >= 0;){
			cpNodeList[k]->cage = false;
		}
		for(int k = MultiSelect::pointList.length; --k >= 0;){
			MultiSelect::pointList[k].p->cage = true;
		}
		g_stw->paint();
		reRenderPolysIfInHidePolyMode();
		houseKeepVisualClue();
	}
	else if(Oscillator* osc = dynamic_cast<Oscillator*>(g_selNode.getRaw())){
		scaleOsc(osc);
		houseKeepVisualClue();
	}
	else if(Track* track = dynamic_cast<Track*>(g_selNode.getRaw())){
		M_Node& m_node = track->sound.refValue();
		for(int k = m_node.length; --k>=0;){
			if(Oscillator* osc = dynamic_cast<Oscillator*>(m_node[k].p)){
				scaleOsc(osc);
			}
		}
		houseKeepVisualClue();
	}
}
/*
void generateAllStrands(Hair* hair){
	M_Strand& strand = hair->strand.refValue();
	for(int k = strand.length; --k >= 0;){
		strand[k]->generate = true;
	}
}
*/
/*
void generateAllStrands(){
	if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		generateAllStrands(hair);
	}
	else if(dynamic_cast<Lock*>(g_selNode.getRaw())){
		generateAllStrands(getHair());
	}
}

void generateOnlySelecteStrands(Hair* hair){
	int selLine = g_stw->selLine;
	Seamless* smls = getSeamless(selLine);
	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
	M_Bool flag;
	flag.length = cpNodeList.length;
	for(int k = MultiSelect::pointList.length; --k >= 0;){
		flag[MultiSelect::pointList[k].p->plidx] = true;
	}

	M_Strand& m_strand = hair->strand.refValue();

	for(int k = m_strand.length; --k >= 0;){
		m_strand[k]->generate = flag[m_strand[k]->plidx];
		//m_strand[k]->cage.refValue() = false;

	}
	updateBuild3dWindow();
}

void generateOnlySelecteStrands(){
	if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		generateOnlySelecteStrands(hair);
	}
	else if(dynamic_cast<Lock*>(g_selNode.getRaw())){
		generateOnlySelecteStrands(getHair());
	}
}
*/
void showAllCages(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int k = 0; k < cpNodeList.length; k++){
			NurbsPatch* patch = cpNodeList[k];
			patch->cage = true;
			if(dynamic_cast<Strand*>(patch)){
				k++;
			}
			
		}
		//for(int k = MultiSelect::pointList.length; --k >= 0;){
		//	if(MultiSelect::pointList[k].p != patch){
		//		MultiSelect::pointList[k].p->cage = false;
		//	}
		//}
		//if(MultiSelect::pointList.length > 1){
		//	MultiSelect::pointList.length = 0;
		//}
		g_stw->paint();
		reRenderPolysIfInHidePolyMode();
		houseKeepVisualClue();
	}
	else if(Oscillator* toOsc = dynamic_cast<Oscillator*>(g_selNode.getRaw())){
		if(Oscillator* fromOsc = dynamic_cast<Oscillator*>(g_copyNode.getRaw())){
			addOscToOsc(fromOsc, toOsc);
		}
	}
}


void hideSelectedCages(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int k = MultiSelect::pointList.length; --k >= 0;){
			MultiSelect::pointList[k].p->cage = false;
		}
		MultiSelect::pointList.length = 0;
		g_stw->paint();
		reRenderPolysIfInHidePolyMode();
		houseKeepVisualClue();
	}

}

void showCagesForStem(M_Node& m_stem, bool state){
	for(int k = 0; k < m_stem.length; k++){
		Stem* stem = dynamic_cast<Stem*>(m_stem[k].p);
		M_Node& m_fuse = stem->fuse.refValue();
		for(int k = 0; k < m_fuse.length; k++){
			if(FuseSurface* fuse = dynamic_cast<FuseSurface*>(m_fuse[k].p)){
				if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(fuse->patch.p)){
					patch->cage = state;
					showCagesForStem(patch->stem.refValue(), state);
				}
			}
		}
	}
}

void showCages(M_Build& m_build, bool state){
	for(int k = m_build.length; --k >= 0;){
		Build* build = m_build[k];

		if(build->getPatch()){
			build->setCage(state);
			if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(build)){
				showCagesForStem(patch->stem.refValue(), state);
			}
		}
		else if(GroupBuild* groupBuild = dynamic_cast<GroupBuild*>(build)){
			showCages(groupBuild->build.refValue(), state);
		}
	}
	reRenderPolysIfInHidePolyMode();
}

void showCages(Hair* hair, bool state){
	M_Strand& m_strand = hair->strand.refValue();
	for(int k = 0; k < m_strand.length; k++){
		m_strand[k]->setCage(state);
	}
}

void showCages(){
	if(GroupBuild* groupBuild = dynamic_cast<GroupBuild*>(g_sceneTreeList[g_stw->selLine].node.p)){
		showCages(groupBuild->build.refValue(), true);
	}
	else if(Hair* hair = dynamic_cast<Hair*>(g_sceneTreeList[g_stw->selLine].node.p)){
		showCages(hair, true);
	}
	g_stw->paint();
}

void hideCages(){
	if(GroupBuild* groupBuild = dynamic_cast<GroupBuild*>(g_sceneTreeList[g_stw->selLine].node.p)){
		showCages(groupBuild->build.refValue(), false);
	}
	else if(Hair* hair = dynamic_cast<Hair*>(g_sceneTreeList[g_stw->selLine].node.p)){
		showCages(hair, false);
	}
	g_stw->paint();
}

void mirrorPatchesForStem(M_Node& m_stem, bool state){
	for(int k = 0; k < m_stem.length; k++){
		Stem* stem = dynamic_cast<Stem*>(m_stem[k].p);
		M_Node& m_fuse = stem->fuse.refValue();
		for(int k = 0; k < m_fuse.length; k++){
			if(FuseSurface* fuse = dynamic_cast<FuseSurface*>(m_fuse[k].p)){
				if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(fuse->patch.p)){
					patch->mirror = state;
					mirrorPatchesForStem(patch->stem.refValue(), state);
				}
			}
		}
	}
}

void mirrorPatches(M_Build& m_build, bool state){
	for(int k = m_build.length; --k >= 0;){
		Build* build = m_build[k];

		if(build->getPatch()){
			build->setMirror(state);
			if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(build)){
				mirrorPatchesForStem(patch->stem.refValue(), state);
			}
		}
		else if(GroupBuild* groupBuild = dynamic_cast<GroupBuild*>(build)){
			mirrorPatches(groupBuild->build.refValue(), state);
		}
	}
}

void mirrorPatches(){
	if(GroupBuild* groupBuild = dynamic_cast<GroupBuild*>(g_sceneTreeList[g_stw->selLine].node.p)){
		mirrorPatches(groupBuild->build.refValue(), true);
	}
}
void unmirrorPatches(){
	if(GroupBuild* groupBuild = dynamic_cast<GroupBuild*>(g_sceneTreeList[g_stw->selLine].node.p)){
		mirrorPatches(groupBuild->build.refValue(), false);
	}
}

void NurbsPatch::removeSouthEdge(NurbsPatch* patchA){
	int uDim = dim.x;
	int vDim = dim.y;
	point.refValue().deleteElement(0, uDim);
	cpIndex.refValue().deleteElement(0, uDim);
	weight.refValue().deleteElement(0, uDim);
	dim.refValue().y--;
	//updateKnot(vDim, dim.refValue().y, vKnot.refValue(), order.refValue().y);
	updateNurbsCurveKnotMid1(dim.refValue().y, vKnot.refValue());
	//processFields();

	if(patchA){
		int vDimA = patchA->dim.y;
		patchA->dim.refValue().y += dim.y;
		//patchA->updateKnot(vDimA, patchA->dim.refValue().y, patchA->vKnot.refValue(), patchA->order.refValue().y);
		updateNurbsCurveKnotMid1(patchA->dim.refValue().y, patchA->vKnot.refValue());
	}
}

void NurbsPatch::removeNorthEdge(NurbsPatch* patchA){
	int uDimB = dim.x;
	int vDimB = dim.y;
	int idx = (vDimB + 1) * uDimB;
	point.refValue().deleteElement(idx, uDimB);
	cpIndex.refValue().deleteElement(idx, uDimB);
	weight.refValue().deleteElement(idx, uDimB);
	dim.refValue().y--;
	//updateKnot(dim.refValue().y, vKnot.refValue(), order.refValue().y);
	//processFields();
	if(patchA){
		patchA->dim.refValue().y += dim.y;
		//patchA->updateKnot(patchA->dim.refValue().y, patchA->vKnot.refValue(), patchA->order.refValue().y);
		updateNurbsCurveKnotMid1(patchA->dim.refValue().y, patchA->vKnot.refValue());
	}
}

void NurbsPatch::removeWestEdge(NurbsPatch* patchA){//never tested and no cpIndex taken care of !
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	int uDim = dim.x;
	int vDim = dim.y;

	int i = 0;

	for(int v = 0; v < vDim; v++){
		for(int u = 0; u < uDim - 1; u++){
			point[i] = point[v * uDim + u];
			weight[i] = weight[v * uDim + u];
			i++;
		}
	}
	point.length = i;
	weight.length = i;
	dim.refValue().x--;

	if(patchA){
		//patchA->updateKnot(patchA->dim.refValue().x, patchA->uKnot.refValue(), patchA->order.refValue().x);
		updateNurbsCurveKnotMid1(patchA->dim.refValue().x, patchA->uKnot.refValue());

		patchA->dim.refValue().x += dim.x;
		//patchA->updateKnot(patchA->dim.refValue().x, patchA->uKnot.refValue(), patchA->order.refValue().x);
		updateNurbsCurveKnotMid1(patchA->dim.refValue().x, patchA->uKnot.refValue());
	}
}

void NurbsPatch::removeEastEdge(NurbsPatch* patchA){//never tested and no cpIndex taken care of !
	M_Vec3f& pointB = point.refValue();
	M_Float& weightB = weight.refValue();
	int uDimB = dim.x;
	int vDimB = dim.y;

	int i = 0;

	for(int v = 0; v < vDimB; v++){
		for(int u = 1; u < uDimB; u++){
			pointB[i] = pointB[v * uDimB + u];
			weightB[i] = weightB[v * uDimB + u];
			i++;
		}
	}
	pointB.length = i;
	weightB.length = i;
	dim.refValue().x--;

	if(patchA){
		updateKnot(dim.refValue().x, uKnot.refValue(), order.refValue().x);

		patchA->dim.refValue().x += dim.x;
		patchA->updateKnot(patchA->dim.refValue().x, patchA->uKnot.refValue(), patchA->order.refValue().x);
	}
}

void kurPatches(){
	disabledMessage();
	return;
}


void incorporate(NurbsPatch* patchA, NurbsPatch* patchB, int edgeRotTypeA, int edgeRotTypeB, M_<NurbsPatchPtr>& m_delPatchList){

	addIfNotInList(patchB, m_delPatchList);

	if(NurbsLine* lineA = dynamic_cast<NurbsLine*>(patchA)){
		if(NurbsLine* lineB = dynamic_cast<NurbsLine*>(patchB)){

			M_Vec3f& pointA = lineA->point.refValue();
			M_Vec3f& dpointA = lineA->dpoint.refValue();
			M_Int& cpIndexA = lineA->cpIndex.refValue();
			M_Float& weightA = lineA->weight.refValue();
			M_Color3f& colorA = lineA->color.refValue();
			M_Float& knot = lineA->uKnot.refValue();

			M_Vec3f& pointB = lineB->point.refValue();
			M_Vec3f& dpointB = lineB->dpoint.refValue();
			M_Int& cpIndexB = lineB->cpIndex.refValue();
			M_Float& weightB = lineB->weight.refValue();
			M_Color3f& colorB = lineB->color.refValue();

			if(edgeRotTypeA == NurbsPatch::EAST){
				if(edgeRotTypeB == NurbsPatch::EAST){
					lineB->setInsideOut2();
				}
			}
			else if(edgeRotTypeA == NurbsPatch::WEST){
				lineA->setInsideOut2();
				if(edgeRotTypeB == NurbsPatch::EAST){
					lineB->setInsideOut2();
				}
			}
			pointB.deleteElement(0, 1);
			pointA += pointB;
			dpointB.deleteElement(0, 1);
			dpointA += dpointB;
			cpIndexB.deleteElement(0, 1);
			cpIndexA += cpIndexB;
			weightB.deleteElement(0, 1);
			weightA += weightB;
			colorB.deleteElement(0, 1);
			colorA += colorB;
			updateNurbsCurveKnotMid0(pointA.length, knot);
			patchA->processFields();
		}

		return;
	}


	if(edgeRotTypeA == NurbsPatch::EAST){
		patchA->rotateIndexOrder270();
	}
	else if(edgeRotTypeA == NurbsPatch::SOUTH){
		patchA->rotateIndexOrder180();
	}
	else if(edgeRotTypeA == NurbsPatch::WEST){
		patchA->rotateIndexOrder90();
	}

	if(edgeRotTypeB == NurbsPatch::NORTH){
		patchB->rotateIndexOrder180();
	}
	else if(edgeRotTypeB == NurbsPatch::WEST){
		patchB->rotateIndexOrder270();
	}
	else if(edgeRotTypeB == NurbsPatch::EAST){
		patchB->rotateIndexOrder90();
	}

	if(patchA->insideOut != patchB->insideOut){
		patchB->setInsideOut2();
	}
	patchB->removeSouthEdge(patchA);
	patchA->point.refValue() += patchB->point;
	patchA->weight.refValue() += patchB->weight;
	patchA->cpIndex.refValue() += patchB->cpIndex;

	if(edgeRotTypeA == NurbsPatch::EAST){
		patchA->rotateIndexOrder90();
	}
	else if(edgeRotTypeA == NurbsPatch::SOUTH){
		patchA->rotateIndexOrder180();
	}
	else if(edgeRotTypeA == NurbsPatch::WEST){
		patchA->rotateIndexOrder270();
	}
}


float edgeDistance(const NurbsPatch* patchA, int offsetA, const int incA, const NurbsPatch* patchB, int offsetB, const int incB, const int count){
	const M_Vec3f& m_dpointA = patchA->dpoint;
	const M_Vec3f& m_dpointB = patchB->dpoint;

	int iA = offsetA;
	int iB = offsetB;

	float distance = 0;

	for(int k = 0; k < count; k++){
		Vec3f pointA = m_dpointA[iA];
		Vec3f pointB = m_dpointB[iB];

		Vec3f diff = pointA - pointB;

		distance += abs(diff.length());//do we need abs?

		iA += incA;
		iB += incB;
	}
	return distance;
}


NurbsPatch* NurbsPatch::getNearestMatchingEdge(const int edgeA, int& nearestEdge)const{	
	float nearestDistance = 1000000;
	NurbsPatch* nearestPatch = 0;

	int uDimA = dim.x;
	int vDimA = dim.y;
	float distance = 0;
	
	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;

	for(int k = cpNodeList.length; --k >= 0;){
		NurbsPatch* patchB = cpNodeList[k];
		if(patchB == this){
			continue;
		}
		int uDimB = patchB->dim.x;
		int vDimB = patchB->dim.y;

		if(edgeA == SOUTH){

			if(uDimA == uDimB){
				distance = edgeDistance(this, 0, 1, patchB, 0, 1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, 0, 1, patchB, uDimB - 1, -1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, 0, 1, patchB, (vDimB - 1) * uDimB, 1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
				distance = edgeDistance(this, 0, 1, patchB, vDimB * uDimB - 1, -1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
			}
			if(uDimA == vDimB){
				distance = edgeDistance(this, 0, 1, patchB, 0, uDimB, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = WEST;
				}
				distance = edgeDistance(this, 0, 1, patchB, (vDimB - 1) * uDimB, -uDimB, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = WEST;
				}
				distance = edgeDistance(this, 0, 1, patchB, uDimB - 1, uDimB, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = EAST;
				}
				distance = edgeDistance(this, 0, 1, patchB, vDimB * uDimB - 1, -uDimB, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = EAST;
				}
			}
		}
		else if(edgeA == EAST){
			if(vDimA == vDimB){
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, 0, uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = WEST;
				}
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, (vDimB - 1) * uDimB, -uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = WEST;
				}
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, uDimB - 1, uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = EAST;
				}
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, vDimB * uDimB - 1, -uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = EAST;
				}
			}
			if(vDimA == uDimB){
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, 0, 1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, uDimB - 1, -1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, (vDimB - 1) * uDimB, 1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
				distance = edgeDistance(this, uDimA - 1, uDimA, patchB, vDimB * uDimB - 1, -1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
			}
		}
		if(edgeA == NORTH){
			int northOffsetA = (vDimA - 1) * uDimA;

			if(uDimA == uDimB){
				distance = edgeDistance(this, northOffsetA, 1, patchB, 0, 1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, northOffsetA, 1, patchB, uDimB - 1, -1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, northOffsetA, 1, patchB, (vDimB - 1) * uDimB, 1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
				distance = edgeDistance(this, northOffsetA, 1, patchB, vDimB * uDimB - 1, -1, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
			}
			if(uDimA == vDimB){
				distance = edgeDistance(this, northOffsetA, 1, patchB, 0, uDimB, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = WEST;
				}
				distance = edgeDistance(this, northOffsetA, 1, patchB, uDimB - 1, uDimB, uDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = EAST;
				}
			}
		}
		else if(edgeA == WEST){
			if(vDimA == vDimB){
				distance = edgeDistance(this, 0, uDimA, patchB, uDimB - 1, uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = EAST;
				}
				distance = edgeDistance(this, 0, uDimA, patchB, vDimB * uDimB - 1, -uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = EAST;
				}
				distance = edgeDistance(this, 0, uDimA, patchB, 0, uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = WEST;
				}
				distance = edgeDistance(this, 0, uDimA, patchB, (vDimB - 1) * uDimB, -uDimB, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = WEST;
				}
			}
			if(vDimA == uDimB){
				distance = edgeDistance(this, 0, uDimA, patchB, 0, 1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, 0, uDimA, patchB, uDimB - 1, -1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = SOUTH;
				}
				distance = edgeDistance(this, 0, uDimA, patchB, (vDimB - 1) * uDimB, 1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
				distance = edgeDistance(this, 0, uDimA, patchB, vDimB * uDimB - 1, -1, vDimA);
				if(distance < nearestDistance){
					nearestDistance = distance;
					nearestPatch = patchB;
					nearestEdge = NORTH;
				}
			}
		}
	}
	return nearestPatch;
}

void toggleShowPatchBordersMode(){
	g_showPatchBorderMode = !g_showPatchBorderMode;
	g_scene->mode.refValue() &= ~BOX_PATCH_MODELLING_MODE;
	houseKeepVisualClue();
}

void mergePatches(){
	resetTemporaryModes();

	if(g_showPatchBorderMode == false){
		toggleShowPatchBordersMode();
	}

	g_mergePatches.patchA = 0;
	g_mergePatches.patchB = 0;

	g_mergePatches.mousePhase = MergePatches::STANDBY;
	g_mergePatches.smls = getSeamless2();
	changeModellingCursor(g_mergePatchesCursor);
	MultiSelect::reset();
	houseKeepVisualClue();
}

void MergePatches::doLastPhase(){
	changeModellingCursor(g_crossCursor);

	smls->deleteAllSyncParts();

	NurbsPatch* mirror_patchA = patchA->mirrorPatch;
	NurbsPatch* mirror_patchB = patchB->mirrorPatch;
	M_<NurbsPatchPtr> m_delPatchList;


	if(patchA->mirror && patchB->mirror){
		if(mirror_patchA && mirror_patchB){
			if(mirror_patchA == patchB){
				incorporate(patchA, patchB, edgeA, edgeB, m_delPatchList);
				patchA->name = "";
			}
			else{
				incorporate(patchA, patchB, edgeA, edgeB, m_delPatchList);
				incorporate(mirror_patchA, mirror_patchB, edgeA, edgeB, m_delPatchList);
				mirror_patchA->cpwsToPoints();
			}
		}
		else if(mirror_patchA == 0 && mirror_patchB){
			incorporate(patchA, patchB, edgeA, edgeB, m_delPatchList);

			if(edgeA == NurbsPatch::EAST){
				edgeA = NurbsPatch::WEST;
			}
			else if(edgeA == NurbsPatch::WEST){
				edgeA = NurbsPatch::EAST;
			}

			incorporate(patchA, mirror_patchB, edgeA, edgeB, m_delPatchList);
		}
		else if(mirror_patchA && mirror_patchB == 0){
			incorporate(patchB, patchA, edgeB, edgeA, m_delPatchList);

			if(edgeB == NurbsPatch::EAST){
				edgeB = NurbsPatch::WEST;
			}
			else if(edgeB == NurbsPatch::WEST){
				edgeB = NurbsPatch::EAST;
			}

			incorporate(patchB, mirror_patchA, edgeB, edgeA, m_delPatchList);

			patchA = patchB;
		}
		else{
			incorporate(patchA, patchB, edgeA, edgeB, m_delPatchList);
		}
	}
	else{
		incorporate(patchA, patchB, edgeA, edgeB, m_delPatchList);
	}

	MultiSelect::pointList.length = 0;

	deletePatches(m_delPatchList);


	patchA->processFields();
	patchA->synchronizeAllStemCells();
	patchA->cpwsToPoints();

	smls->populateMirrorNodes();
	smls->generateRawStitchPolys();
	smls->stitchAllPolys();

	g_stw->selLine = getNodeLine(patchA);
	mousePhase = NEUTRAL;
	g_stw->lesserPostInsertNodeStuff();


	smls->generateAllPatchesBnspe();
	if(smls->autoJoin){
		smls->joinAllParts();
	}

	smls->doBuild();
	houseKeepVisualClue();
}

void incorporate(){
	//disabledMessage();
	//return;
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Lock* parentLock = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
			lock->mergeRegions(parentLock);
		}
	}
}


void convert_to_0_v_mid_points(NurbsPatch* patch){
	M_Vec3f& dpoint = patch->dpoint.refValue();
	M_Vec3f& point = patch->point.refValue();
	M_Float& weight = patch->weight.refValue();
	M_Vec3f _point = point;
	M_Float _weight = weight;
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;


	int i = 0;

	for(int u = 0; u < uDim; u++){
		point[i] = _point[u];
		weight[i] = _weight[u];
		i++;
	}
	for(int v = 1; v < vDim - 1; v++){
		for(int u = 0; u < uDim; u++){
			point[i] = _point[v * uDim + u];
			weight[i] = _weight[v * uDim + u];
			i++;
		}
		v++;
	}
	for(int v = vDim - 1, u = 0; u < uDim; u++){
		point[i] = _point[v * uDim + u];
		weight[i] = _weight[v * uDim + u];
		i++;
	}

	vDim -= (vDim - 3) / 2;
	patch->dim.refValue().y = vDim;
	point.length = weight.length = vDim * uDim;
	updateNurbsCurveKnotMid0(vDim, patch->vKnot.refValue());
	patch->processFields();
}
void NurbsPatch::convert_to_1_v_mid_points(){//why must save and reopen after this operation?
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Int _cpIndex = cpIndex;

	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	M_Vec3f& cp = smls->cp;
	M_Float& cw = smls->cw;
	int uDim = this->dim.x;
	int vDim = this->dim.y;

	int cpMark = cp.length;
	int extra = (vDim - 3) * uDim;
	point.length += extra;
	weight.length += extra;
	cpIndex.length += extra;
	cp.length += extra;
	cw.length += extra;



	int i = 0;

	for(int u = 0; u < uDim; u++){
		cpIndex[i] = _cpIndex[u];
		i++;
	}
	for(int v = 1; v < vDim; v++){
		for(int u = 0; u < uDim; u++){
			cpIndex[i] = _cpIndex[v * uDim + u];
			i++;
		}
		if(v < vDim - 2){
			for(int u = 0; u < uDim; u++){
	//			weight[i] = (_weight[v * uDim + u] + _weight[(v + 1) * uDim + u]) / 2;
	//			point[i] = (dpoint[v * uDim + u] + dpoint[(v + 1) * uDim + u]) / 2 * weight[i];
				cpIndex[i] = cpMark;
				cw[cpMark] = 1;
				weight[i] = 1;

				i++;
				cpMark++;
			}
		}
	}


	vDim += vDim - 3;
	this->dim.refValue().y = vDim;
	updateNurbsCurveKnotMid1(vDim, vKnot.refValue());


	processFields();
	this->pointsToDpoints();
	smls->populateMirrorNodes();
	smls->calcAllPolyMidPoints();
	commonPostTug();
	houseKeepVisualClue();

}

void convert_to_0_v_mid_points(){
	NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw());
	convert_to_0_v_mid_points(patch);
	patch->smls->populateMirrorNodes();
	updateBuild3dWindow();
}

void convert_to_1_v_mid_points(){
	NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw());
	patch->convert_to_1_v_mid_points();
	patch->smls->populateMirrorNodes();
	updateBuild3dWindow();
}

void convert_to_0_u_mid_points(){
	NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw());
	patch->rotateIndexOrder90();
	patch->processFields();
	convert_to_0_v_mid_points(patch);
	patch->rotateIndexOrder270();
	patch->processFields();
	patch->smls->populateMirrorNodes();
	updateBuild3dWindow();
}

void NurbsLine::convert_to_1_u_mid_points(){
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Int _cpIndex = cpIndex;

	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	M_Color3f& color = this->color.refValue();
	M_Color3f _color = color;

	M_Vec3f& cp = smls->cp;
	M_Float& cw = smls->cw;
	
	int uDim = this->dim.x;
	int vDim = this->dim.y;

	int cpMark = cp.length;
	int extra = (uDim - 3) * vDim;
	dpoint.length += extra;
	point.length += extra;
	weight.length += extra;
	cpIndex.length += extra;
	color.length += extra;
	cp.length += extra;
	cw.length += extra;



	int i = 0;

	color[0] = _color[0];

	cpIndex[i++] = _cpIndex[0];

	for(int u = 1; u < uDim; u++){
		color[i] = _color[u];
		cpIndex[i] = _cpIndex[u];
		i++;
		if(u < uDim - 2){
			cpIndex[i] = cpMark;
			weight[i] = 1;
			color[i] = Color3f(1, 0, 0);
			cw[cpMark] = 1;

			i++;
			cpMark++;
		}
	}



	uDim += uDim - 3;
	this->dim.refValue().x = uDim;
	updateNurbsCurveKnotMid1(uDim, uKnot.refValue());
	

	smls->populateMirrorNodes();
	smls->calcAllPolyMidPoints();
	commonPostTug();
	houseKeepVisualClue();
}

void NurbsPatch::convert_to_1_u_mid_points(){
	if(NurbsLine* nurbsLine = dynamic_cast<NurbsLine*>(this)){
		nurbsLine->convert_to_1_u_mid_points();
	}
	else{
		rotateIndexOrder90();
		processFields();
		convert_to_1_v_mid_points();
		rotateIndexOrder270();
		processFields();
	}
}

void setAllWeightsTo1(NurbsPatch* patch, int length){
	M_Float& weight = patch->weight.refValue();
	weight.length = length;
	for(int k = length; --k>=0;){
		weight[k] = 1;
	}
}

void subdivPatch(NurbsPatch* patch){
	M_Vec3f& dpoint = patch->dpoint.refValue();
	M_Vec3f newDpoint;
	
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int newUdim = (uDim - 1) * 2 + 1;
	int newVdim = (vDim - 1) * 2 + 1;

	newDpoint.length = newVdim * newUdim;
	int destIdx = 0;

	for(int v = 0; v < vDim; v++){
		Vec3f dB;
		Vec3f dA;
		for(int u = 0; u < uDim - 1; u++){
			dA = dpoint[v * uDim + u];
			dB = dpoint[v * uDim + u + 1];
			newDpoint[destIdx++] = dA;
			newDpoint[destIdx++] = (dA + dB) / 2.0;
		}
		newDpoint[destIdx++] = dB;

		if(v < vDim - 1){
			for(int u = 0; u < uDim - 1; u++){
				dA = (dpoint[v * uDim + u] + dpoint[v * uDim + u + uDim]) / 2.0;
				dB = (dpoint[v * uDim + u + 1] + dpoint[v * uDim + u + 1 + uDim]) / 2.0;
				newDpoint[destIdx++] = dA;
				newDpoint[destIdx++] = (dA + dB) / 2.0;
			}
			newDpoint[destIdx++] = dB;
		}
	}



	dpoint = newDpoint;
	setAllWeightsTo1(patch, newDpoint.length);
	patch->dim.refValue() = Vec2f(newUdim, newVdim);
	updateNurbsCurveKnotMid1(newUdim, patch->uKnot.refValue());
	updateNurbsCurveKnotMid1(newVdim, patch->vKnot.refValue());
	patch->dpointsToPoints();
	patch->processFields();
}

void legalizeKnots(NurbsPatch* patch){
	updateNurbsCurveKnotMid1(patch->dim.x, patch->uKnot.refValue());
	updateNurbsCurveKnotMid1(patch->dim.y, patch->vKnot.refValue());
	patch->processFields();
}

void subdivPatches(){//failed
	if(Seamless* smls = getSeamless2()){
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int j = 0; j < cpNodeList.length; j++){
			NurbsPatch* patch = cpNodeList[j];
			//subdivPatch(patch);
			legalizeKnots(patch);
		}
		smls->populateMirrorNodes();
		smls->generateRawStitchPolys();
		smls->stitchAllPolys();
		smls->doBuild();
		smls->ctrf = true;
		houseKeepVisualClue();
	}

}

void convert_to_1_u_mid_points(){
	NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw());
	patch->convert_to_1_u_mid_points();
	patch->smls->populateMirrorNodes();
	updateBuild3dWindow();
}

void calcAllPolyMidPoints(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->calcAllPolyMidPoints();
	}
	updateBuild3dWindow();
}

void fillEdgesWithCtrlPolys(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->fillEdgesWithCtrlPolys();
		resetCPNodeList();
		houseKeepVisualClue();
	}
	else if(Composition* composition = dynamic_cast<Composition*>(g_selNode.getRaw())){
		composition->evolveSound();
	}
}

void mixLayers();

void mergeCtrlPolys(){
	if(g_scene->mode & ANIMATION_MODE){
		if(Anim* anim = Window::animMode()){
			if(Video* video = dynamic_cast<Video*>(anim->video.p)){
				video->mixLayers();
			}
		}
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->stitchControlPolygons();
	}
	else if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		hair->generateMultiples();
	}
}

void stitchFlower(){
	if(NurbsPatch* patch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->stitchFlower();
	}
	else if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		Hair::selLock->freePointFromSeperator(Hair::selHoleIndex);
		houseKeepVisualClue();
		g_stw->paint();
	}
}

void mergeTo5CtrlPoly(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->mergeTo5CtrlPoly();
	}
}

void mergeTo6CtrlPoly(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->smls->mergeTo6CtrlPoly();
	}
}

void linkPointToEdge(){
	Node* selNode = g_selNode.getRaw();
	if(dynamic_cast<Hair*>(selNode) || dynamic_cast<Lock*>(selNode) || dynamic_cast<Strand*>(selNode)){
		toggleStrandCtrlLineMode();
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(selNode)){
		if(MultiSelect::pointList.length){
			if(MultiSelect::ctrlPolySelList.length){
				patch->smls->linkPointToEdge(MultiSelect::ctrlPolySelList[0]);
			}
		}
	}
	else if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		locateBuildNode(part, part->smls->ActTri);
	}
	else if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		hair->locateBendyLatheNode();
	}
}

void Stem::copyPartToPolyMap(Art* part){
	Mesh geomSource;
	part->extractGeometryGS(geomSource, g_tmpIndex);

	M_Vec3f& srcCoord = geomSource.coord.refValue();
	M_Vec2f& dstCoord = this->coord.refValue();
	dstCoord.length = srcCoord.length;
	for(int i = srcCoord.length; --i>=0;){
		Vec3f vec3f = srcCoord[i];
		dstCoord[i] = Vec2f(vec3f.x, vec3f.y);
	}

	color.refValue() = geomSource.color;
	texCoord.refValue() = geomSource.texCoord;
	coordIndex.refValue() = geomSource.coordIndex;
	rc.refValue() = geomSource.rc;
}

void mirrorAllCtrlPoints(){
	if(NurbsPatch* nurbsPatch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		M_Vec3f& point = nurbsPatch->point.refValue();
		M_Float& weight = nurbsPatch->weight.refValue();

		for(int i = point.length; --i >= 0;){
			point[i].x = -point[i].x;
		}

		nurbsPatch->processFields();
		updateBuild3dWindow();
		houseKeepVisualClue();
		BackUp::monostable.beginCycle();
	}
}


int uInvertIdx(int uDim, int idx){
	int u = idx % uDim;
	int v = idx / uDim;

	u = uDim - u - 1;
	return v * uDim + u;
}

int vInvertIdx(int uDim, int vDim, int idx){
	int u = idx % uDim;
	int v = idx / uDim;

	v = vDim - v - 1;
	return v * uDim + u;
}

void mirrorStitchPolys(M_CtrlPolyPoint& ctrlPolyPoint, const NurbsPatch* checkPatch){
	int poly = 0;
	int uDim = checkPatch->dim.x;
	int vDim = checkPatch->dim.y;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		for(int k = len; --k >= 0;){
			if(ctrlPolyPoint[poly].p == checkPatch){
				//ctrlPolyPoint[poly].i = vInvertIdx(uDim, vDim, ctrlPolyPoint[poly].i);
				ctrlPolyPoint[poly].i = uInvertIdx(uDim, ctrlPolyPoint[poly].i);
			}
			poly++;
		}
		for(int k = len + 1; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				poly++;
			}
			else{
				for(int k = midLen; --k >= 0;){
					if(ctrlPolyPoint[poly].p == checkPatch){
						//ctrlPolyPoint[poly].i = vInvertIdx(uDim, vDim, ctrlPolyPoint[poly].i);
						ctrlPolyPoint[poly].i = uInvertIdx(uDim, ctrlPolyPoint[poly].i);
					}
					poly++;
				}
			}
		}
	}
}
void NurbsPatch::mirrorTwinlessPatch(int idx){
	int uDim = dim.x;
	int vDim = dim.y;
	int u = idx % uDim;
	int v = idx / uDim;
	if(u > uDim / 2){
		mirrorModifyWest();
	}
	else{
		mirrorModifyEast();
	}
	processFields();
}

void NurbsPatch::mirrorTwinPatch(NurbsPatch* mirrorPatch){
	M_Vec3f& point = this->point.refValue();

	for(int i = point.length; --i >= 0;){
		Vec3f vec = point[i];
		vec.x = -vec.x;
		//mirrorPatch->point.refValue()[i] = vec;
		mirrorPatch->setPointAndCp(i, vec);
	}
	//mirrorStitchPolys(mirrorPatch->smls->ctrlPolyPoint, mirrorPatch);
	mirrorPatch->processFields();
}

void NurbsPatch::mirrorTwinPatch2(NurbsPatch* mirrorPatch){
	M_Vec3f& point = this->point.refValue();

	for(int i = point.length; --i >= 0;){
		Vec3f vec = point[i];
		vec.x = -vec.x;
		mirrorPatch->point.refValue()[i] = vec;
	}
	//mirrorStitchPolys(mirrorPatch->smls->ctrlPolyPoint, mirrorPatch);
	mirrorPatch->processFields();
}
void copySourceCP(M_<NurbsPatch*>& cpNodeList);

void Art::mirrorMorphedCtrlPoints(bool left){
	
	for(int k = controlPoint.length; --k>=0;){
		NurbsPatch* patch = controlPoint[k].p;
		int idx = controlPoint[k].i;
		Vec3f pnt = patch->point[idx];
		pnt.x = -pnt.x;
		if((pnt.x < 0) == left){
			if(patch->mirror){
				if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
					mirrorPatch->point.refValue()[idx] = pnt;
					mirrorPatch->dpoint.refValue()[idx] = pnt * mirrorPatch->weight[idx];
				}
				else{
					M_Vec3f& point = patch->point.refValue();
					M_Vec3f& dpoint = patch->dpoint.refValue();
					M_Float& weight = patch->weight.refValue();

					int midx = patch->getMirrorTwinlessIdx(idx);

					weight[midx] = weight[idx];

					point[midx] = pnt;
					dpoint[midx] = pnt * weight[idx];
				}
			}
		}
	}
}

void Seamless::mirrorMorphedCtrlPoints(Art* targetPart, Anim* anim, bool left){

	setMathHiPresision();

	//if(nurbsLineShape){
	//	if(IndexedLineSet* ils = dynamic_cast<IndexedLineSet*>(nurbsLineShape->geometry.p)){
	//		dynamic_cast<Coordinate*>(ils->coord.p)->point.refValue().length = 0;
	//		dynamic_cast<Color*>(ils->color.p)->color.refValue().length = 0;
	//		ils->coordIndex.refValue().length = 0;
	//	}
	//}

	//M_Build& build=this->build.refValue();

	if(workAnimCP){
		catMatrix();
		ctrf = true;
		if(bnspe == false){
			for(int k = partList.length; --k >= 0;){
				Art* part = partList[k];
				part->doClearCycleAnimCP();
			}
		}
		//cpNodeList.length = 0;
		//cpNodeListAdd(build, cpNodeList);

		copySourceCP(cpNodeList);


		anim->morphAllParts();

		calcAllPolyMidPoints();


		targetPart->mirrorMorphedCtrlPoints(left);

		for(int k = skeleton.length; --k >= 0;){
			Art* part = skeleton.refValue()[k].p;
			part->transformPoints(Matrix());
		}
		calcAllPolyMidPoints();
	}
}

void mirrorMorphedCtrlPoints(){
	if(Anim* anim = Window::animMode()){
		if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
			//if(MultiSelect::pointList.length){
			if(MultiSelect::selectedPoint < part->controlPoint.length){
				ControlPoint& cp = part->controlPoint[MultiSelect::selectedPoint];
				NurbsPatch* patch = cp.p;

				//if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p)){
					//int idx = MultiSelect::pointList[MultiSelect::withinSightsIndex].i;

					bool left = patch->tpoint[cp.i].x > 0;

					anim->slider = anim->slider;
					part->smls->mirrorMorphedCtrlPoints(part, anim, left);
					anim->captureMorph(part->smls);
				
			}
		}
	}
}

void mirrorCtrlPoints(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		for(int k = MultiSelect::pointList.length; --k >= 0;){
			NurbsPatch* patch = MultiSelect::pointList[k].p;
			if(patch->mirror){
				int idx = MultiSelect::pointList[k].i;
				patch->mirrorSingleCP(idx);
			}
		}
		updateBuild3dWindow();
		houseKeepVisualClue();
		BackUp::monostable.beginCycle();
	}
	else if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		M_Vec3f& point = bezierPatch->point.refValue();
		const M_Vec2f& dim = bezierPatch->dim;
		int uDim=dim[0].x;
		int vDim=dim[0].y;
		for(int v=vDim; --v>=0;){

			point[v * uDim + uDim - 2].y = point[v * uDim + uDim - 1].y;
			point[v * uDim + uDim - 2].z = point[v * uDim + uDim - 1].z;

			for(int u = 0; u < uDim>>1; u++){
				Vec3f p = point[v*uDim + uDim - 1 - u];
				p.x = -p.x;
				point[v*uDim + u] = p;
			}
		}
	}
	else{
		return;
	}
	updateBuild3dWindow();
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}

void copyStemMapToStemMap(Stem* selStem, Stem* copyStem, Seamless* smls){
	selStem->coord.refValue() = copyStem->coord;
	selStem->color.refValue() = copyStem->color;
	selStem->texCoord.refValue() = copyStem->texCoord;
	selStem->coordIndex.refValue() = copyStem->coordIndex;
	selStem->rc.refValue() = copyStem->rc;
	selStem->processFields();
	smls->doBuild();
	smls->ctrf=true;
	BackUp::monostable.beginCycle();
}

void pasteToPolyMap(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		openSceneTree();
		if(g_copyNode.getRaw()==whitePart){
			Seamless* smls = whitePart->smls;
			NurbsPatch* patch = 0;
			Stem* selStem = 0;
			Stem* copyStem = 0;
			Polymap* polymap = 0;
			ImmediatePart* bluePart = 0;
			if(getPolyMap(patch, selStem, polymap, whitePart, smls->ActTri, bluePart)){
				if(getPolyMap(patch, copyStem, polymap, whitePart, smls->remActTri, bluePart)){
					if(bluePart){
						selStem->copyPartToPolyMap(bluePart);
						updateBuild3dWindow();
						houseKeepVisualClue();
						BackUp::monostable.beginCycle();
					}
					else{
						copyStemMapToStemMap(selStem, copyStem, whitePart->smls);
						makePointerEqualIndexLocation();
					}
				}
			}
		}
	}
	else if(ImmediatePart* bluePart=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
		Stem* stem = dynamic_cast<Stem*>(g_selNode.getRaw());
		stem->copyPartToPolyMap(bluePart);
		updateBuild3dWindow();
		houseKeepVisualClue();
		BackUp::monostable.beginCycle();
	}
	if(Stem* selStem = dynamic_cast<Stem*>(g_selNode.getRaw())){
		if(Stem* copyStem=dynamic_cast<Stem*>(g_copyNode.getRaw())){
			copyStemMapToStemMap(selStem, copyStem, getSeamless());
		}
		else if(Art* part=dynamic_cast<Art*>(g_copyNode.getRaw())){
			selStem->copyPartToPolyMap(part);
			updateBuild3dWindow();
			houseKeepVisualClue();
			BackUp::monostable.beginCycle();
		}
	}
}

void pasteCtrlPoints(NurbsPatch* patch, NurbsPatch* copyPatch){
	MultiSelect::pointList.length = 0;
	MultiSelect::ctrlPolySelList.length = 0;
	patch->point.refValue()	=	copyPatch->point;
	patch->weight.refValue()=	copyPatch->weight;
	patch->uKnot.refValue()	=	copyPatch->uKnot;
	patch->vKnot.refValue()	=	copyPatch->vKnot;
	patch->dim.refValue()	=	copyPatch->dim;
	patch->order.refValue()	=	copyPatch->order;
	patch->uJoinEnds.refValue()	=	copyPatch->uJoinEnds;
	patch->processFields();
}


void transferSelectedPoints(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		MultiSelect::ctrlPolySelList.length = 0;

		for(int k = MultiSelect::pointList.length; --k >= 0;){
			NurbsPatch* p = MultiSelect::pointList[k].p;
			int i = MultiSelect::pointList[k].i;
			patch->point.refValue()[i] = p->point[i];
			MultiSelect::pointList[k].p = patch;
		}
		//Seamless* smls = patch->smls;
		//patch->processFields();
		//smls->doBuild();
		//smls->ctrf = true;

		g_transformCP->processFields();
		BackUp::monostable.beginCycle();
	}
}

void pasteCtrlPoints(){
	if(NurbsPatch* copyPatch=dynamic_cast<NurbsPatch*>(g_copyNode.getRaw())){
		NurbsPatch* patch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw());
		pasteCtrlPoints(patch, copyPatch);
		updateBuild3dWindow();
		houseKeepVisualClue();
		BackUp::monostable.beginCycle();
	}
}
void pasteCtrlPointsMirror(){
	if(NurbsPatch* copyPatch=dynamic_cast<NurbsPatch*>(g_copyNode.getRaw())){
		NurbsPatch* patch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw());
		pasteCtrlPoints(patch, copyPatch);
		mirrorAllCtrlPoints();
	}
}

void mirrorHalfPolyMap(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		if(dynamic_cast<ColorEffect*>(whitePart->workEffect)==0)return;

		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
			}
			else{
				int selIdx = smls->ActCtrlPnt-stem->beginCoordOffset;
				M_Vec2f& coord = stem->coord.refValue();
				const M_Int& rc = stem->rc;
				const M_Int& coordIndex = stem->coordIndex;
				int half = coord.length / 2;
				if(selIdx < half){
					for(int ci = 0; ci < coordIndex.length; ci++){
						int i = coordIndex[ci];
						int g = i - coord.length;
						if(g >= 0){
							i = rc[g];
						}
						if(i < half){
							Vec2f crd = coord[i];
							crd.x = -(crd.x - .5) + .5;
							coord[i + half] = crd;
						}
					}
				}
				else{
					for(int ci = 0; ci < coordIndex.length; ci++){
						int i = coordIndex[ci];
						int g = i - coord.length;
						if(g >= 0){
							i = rc[g];
						}
						if(i > half){
							Vec2f crd = coord[i];
							crd.x = -(crd.x - .5) + .5;
							coord[i - half] = crd;
						}
					}
				}
				smls->doBuild();
				makePointerEqualIndexLocation();
				smls->ctrf=true;
				Window::paintAllControlPanels();
			}
		}
	}
}

void addMirrorHalfPolyMap(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
			}
			else{
				M_Vec2f& coord = stem->coord.refValue();
				for( int i=coord.length; --i>=0; ){
					coord[i].x -= .5;
				}

				bluePart = selectPolyMapPart(whitePart);
				stem->copyPolyMapToPart(bluePart);

				CopyPart cp;
				cp.mirrorX=true;
				cp.add=true;
				cp.dest+=bluePart;
				cp.doBuild();			
				stem->copyPartToPolyMap(bluePart);
				for( int i=coord.length; --i>=0; ){
					coord[i].x += .5;
				}
				commonEndVertexBuild();
			}
		}
	}
}

void deletePolyMap(){
	if(Stem* stem = dynamic_cast<Stem*>(g_selNode.getRaw())){
		stem->coord.refValue().length = 0;
		stem->color.refValue().length = 0;
		stem->texCoord.refValue().length = 0;
		stem->coordIndex.refValue().length = 0;
		stem->rc.refValue().length = 0;
		stem->processFields();
		BackUp::monostable.beginCycle();
		updateBuild3dWindow();
	}
}

void deleteHalfPolyMap(){
	if(Part* whitePart=dynamic_cast<Part*>(g_selNode.getRaw())){
		Seamless* smls = whitePart->smls;
		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* bluePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, bluePart)){
			if(bluePart){
			}
			else{
				if(Stem* mirrorStem = stem->mirrorStem){
					mirrorStem->coord.refValue().length = 0;
					mirrorStem->color.refValue().length = 0;
					mirrorStem->texCoord.refValue().length = 0;
					mirrorStem->coordIndex.refValue().length = 0;
					mirrorStem->rc.refValue().length = 0;
					mirrorStem->processFields();
				}
				else{
					bluePart = selectPolyMapPart(whitePart);
					stem->copyPolyMapToPart(bluePart);
					int selIdx = smls->ActCtrlPnt-stem->beginCoordOffset;
					Vec2f selVec = stem->coord[selIdx];
					if(selVec.x >= .5){
						bluePart->cutLeft(.5+.0001);
					}
					else{
						bluePart->cutRight(.5-.0001);
					}
					stem->copyPartToPolyMap(bluePart);
				}
				commonEndVertexBuild();
			}
		}
	}
}

void NurbsPatch::setAllWeightsToOne(){
	M_Float& weight = this->weight.refValue();
	const M_Vec3f& dpoint = this->dpoint;
	M_Vec3f& point = this->point.refValue();
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();
	const M_Int& cpIndex = this->cpIndex;
	for(int i = point.length; --i>=0;){
		cw[cpIndex[i]] = weight[i] = 1;
		cp[cpIndex[i]] = point[i] = dpoint[i];
	}
}


void setAllWeightsToOne(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		patch->setAllWeightsToOne();
		patch->processFields();
		updateBuild3dWindow();
	}
}

inline void addPartToList(M_Art& relevant, Art*& part){
	for(int r=relevant.length; --r>=0;){
		if(relevant[r]==part)return;
	}
	relevant+=part;
}

void Seamless::removeAllPatchesFromPartsNotInList(M_Art& m_part){
	for(int k = partList.length; --k>=0;){
		Art* part = partList[k];
		M_ControlPoint& controlPoint = part->controlPoint;
		int d = 0;
		for(int s = 0; s < controlPoint.length; s++){
			NurbsPatch* patch = controlPoint[s].p;
			if(patch->plidx >= cpNodeList.length || cpNodeList[patch->plidx] != patch){
				addPartToList(m_part, part);
				continue;				
			}
			controlPoint[d++] = controlPoint[s];
		}
		controlPoint.length = d;
	}
}

/*
void delNullVertices(M_Art& partList){
	static DelPoly dp;
	for(int k = partList.length; --k >= 0;){
		Art* part = partList[k];
		dp.part.refValue() = part;
		M_Triangle& m_triangle = part->triangle;
		M_Vec3f& coord = part->coord.refValue();
		M_Vec2f& uvCoord = part->uvCoord.refValue();
		M_NurbsPatch& patchOwner = part->patchOwner;
		for(int i = 0; i < m_triangle.length; i++){
			const S_Triangle tri = m_triangle[k];
			i(tri.pa->patchOwner[tri.ia] == 0){
			}
		}
	}
}
*/

void preDeleteNodeForSeamlessNode(M_<NodePtr>& m_hold, Seamless* smls){
	g_stw->textCell.cellActive = false;

	if(smls){
		smls->ctrf = true;
		smls->masterIn.refValue() = true;
		setAllIns(smls->build.refValue(), true);
		if(smls->workAnimCP){
			smls->setModellingPatchMode();
			smls->resetRotations();
		}
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int k = cpNodeList.length; --k>=0;){
			cpNodeList[k]->smls = 0; //so that we can tell if morpher is pointing to a node that exists
		}
	}


	for(int k = g_sceneTreeList.length; --k>=0;){
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			for(int k = 0; k < smls->cpNodeList.length; k++){
				m_hold += smls->cpNodeList[k];
			}
		}
	}
}

void postDeleteNodeForSeamlessNode(Seamless* smls){
	M_Art m_part;
	for(int k = g_sceneTreeList.length; --k>=0;){//must check all smls nodes in case for morphers
		if(Seamless* smls = dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->cpNodeListAdd();
			smls->deleteLinksToNonExistentPatches(); //not to be confused with non existing control polys
			smls->deletePolysWithNonExistingPoints();
			smls->removeAllPatchesFromPartsNotInList(m_part);
	
		}
	}

	if(smls){
		smls->populateMirrorNodes();
		for(int k = g_animBarList.length; --k>=0;){
			g_animBarList[k]->deleteNonExistingMorpherPoints(smls);
		}
		smls->deletePoliesOwnedByInvalidPatchs();
		smls->generateRawStitchPolys();
		smls->stitchAllPolys();
		smls->doBuild();
	}

	g_lastSelectedPatch = 0;
	g_lastSelectedPart = 0;
}


void deleteNode(){//DELETE_NODE



	Build* mirrorBuild = 0;
	Build* buildPatch = 0;
	Art* mirrorPart = 0;
	Art* part = 0;

	Seamless* smls = getSeamless2();

	int parentLine = -1;
	if(dynamic_cast<Art*>(g_selNode.getRaw())){
		int selLine = g_stw->selLine;
		parentLine = getParent(selLine);
		Node* parentNode = g_sceneTreeList[parentLine].node.p;
		if(dynamic_cast<Art*>(parentNode) == 0){
			parentLine = -1;
		}
	}

	//if(smls){
	//	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
	//	for(int k = cpNodeList.length; --k>=0;){
	//		cpNodeList[k].patch->smls = 0;
	//	}
	//}
	if(g_selCount == 1){
		Node* selNode = g_selNode.getRaw();
		if(Build* b = dynamic_cast<Build*>(selNode)){
			if(NurbsPatch* patch = b->getPatch()){
				buildPatch = b;
				if(mirrorBuild = buildPatch->getMirrorBuild()){
					NurbsPatch* mirrorPatch = mirrorBuild->getPatch();
					if(mirrorPatch->plidx > patch->plidx){
						Build* tmp = buildPatch;
						buildPatch = mirrorBuild;
						mirrorBuild = tmp;
						g_stw->setResultsForSelectingPoint(buildPatch);
					}
				}
				if(smls->bnspe){
					smls->deletePoliesOwnedByPatch(patch);
					if(mirrorBuild){
						smls->deletePoliesOwnedByPatch(mirrorBuild->getPatch());
					}
				}
			}
		}
		else if(part = dynamic_cast<Art*>(selNode)){
			if(mirrorPart = part->mirrorPart){
				if(mirrorPart->plidx > part->plidx){
					Art* tmp = part;
					part = mirrorPart;
					mirrorPart = tmp;
					g_stw->setResultsForSelectingVertex(part);
				}
			}
		}
	}
	M_<NodePtr> m_hold;

	preDeleteNodeForSeamlessNode( m_hold, smls);


	deleteNode(g_stw->selLine, g_selCount);
	if(mirrorBuild && smls->bnspe && g_selCount == 1){
		deleteNode(getNodeLine(mirrorBuild), g_selCount);
	}
	if(mirrorPart && g_selCount == 1){
		deleteNode(getNodeLine(mirrorPart), g_selCount);
	}
	
#ifdef PROG_ROB
	dLog("new MoveToNode(" + String(g_stw->selLine) + "),");
	dLog("new RightClick,");
	dLog("new Move_MenuItem(menu_delete),");
	dLog("new Click,");
#endif

	g_stw->genNodeList();
	postDeleteNodeForSeamlessNode(smls);

	if(parentLine > 0){
		g_stw->selLine = parentLine;
	}

	g_stw->commonPostInsertNodeStuff(true);
	updateAutoInForClick();


}

void deleteLink(){
	if((g_scene->mode & HAIR_REGION_MODE)){
		if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
			lock->region.refValue().length = 0;
			lock->houseKeep();
		}
	}
	else{
		if(MultiSelect::pointList.length == 1){
			NurbsPatch* patch = MultiSelect::pointList[0].p;
			int idx = MultiSelect::pointList[0].i;
			M_Link& link = patch->smls->link;
			for(int i = link.length; --i>=0;){
				if(link[i].patch == patch && link[i].point == idx){
					patch->smls->link.deleteElement(i);
					patch->processFields();
					updateBuild3dWindow();
					break;
				}
			}
		}
	}
}

void pasteColumn(){
	if(NurbsPatch* copyPatch = dynamic_cast<NurbsPatch*>(g_copyNode.getRaw())){
		int copy_uDim = copyPatch->dim.x;
		int copy_vDim = copyPatch->dim.y;
		int column = g_rememberIndex % copy_uDim;
		M_Vec3f& srcPoint = copyPatch->point.refValue();
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
			int uDim = patch->dim.x;
			int vDim = patch->dim.y;
			int index = patch->index[0] % uDim;
			M_Vec3f& dstPoint = patch->point.refValue();
			for(int v = 0; v < vDim; v++){
				Vec3f pnt = Vec3f(dstPoint[v * uDim + index].x, srcPoint[v * copy_uDim + column].y, srcPoint[v * copy_uDim + column].z);
				patch->setPointAndCp(v * uDim + index, pnt);
			}
			patch->processFields();
			updateBuild3dWindow();
		}
	}
}

void rotateCtrlStaves(NurbsPatch* patch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	M_Float& weight = patch->weight.refValue();
	M_Vec3f& point = patch->point.refValue();
	for(int i = 2; --i>=0;){
		for(int u = 0; u < (uDim - 1); u++){
			for(int v = 0; v < vDim; v++){
				point[v * uDim + u] = point[v * uDim + (u + 1)];
				weight[v * uDim + u] = weight[v * uDim + (u + 1)];
			}
		}
		for(int v = 0; v < vDim; v++){
			point[v * uDim + uDim - 1] = point[v * uDim];
			weight[v * uDim + uDim - 1] = weight[v * uDim];
		}
	}
	patch->patchToCentralCPs();
	patch->processFields();
}

void rotateCtrlStaves(){
	if(NurbsPatch* patch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		rotateCtrlStaves(patch);
		if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
			rotateCtrlStaves(mirrorPatch);
		}
		updateBuild3dWindow();
	}
}

void calcLatheWeights(){
	if(NurbsPatch* patch=dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		M_Vec3f& point = patch->point.refValue();
		M_Vec3f& dpoint = patch->dpoint.refValue();
		M_Float& weight = patch->weight.refValue();
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		int ctrlStaves = uDim / 2;
		double sinWeight = cos(PI / ctrlStaves);

		for(int r = 0; r < vDim; r++){
			double a = PI / ctrlStaves;
			for(int c = 0; c < uDim; ){
				double b = a * c;
				int i = r * uDim + c++;
				weight[i] = 1;
				point[i] = dpoint[i] * weight[i];

				if(c == uDim){
					break;
				}

				i = r * uDim + c++;
				weight[i] = sinWeight;
				point[i] = dpoint[i] * weight[i];
			}
		}
		patch->processFields();
		updateBuild3dWindow();
	}
}

void deleteMorpherChannels(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Anim* anim = Window::animMode()){
			anim->deleteMorpherChannel();
			BackUp::monostable.beginCycle();
		}
	}
	else{
		needPartMsg();
	}
}

void compileMorph(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Anim* anim = Window::animMode()){
			anim->compileMorph(part->smls);
		}
	}
	else{
		needPartMsg();
	}
}

void compileOrientationInterpolator(const Art* part, Rotation accumRot, M_<SceneTreeListItem>& sceneTreeList);

void untickIkFields(){
	if(Seamless* smls = dynamic_cast<Seamless*>(g_selNode.getRaw())){
		M_Art& partList = smls->partList;
		for(int k = partList.length; --k >= 0;){
			Art* part = partList[k];
			part->ik = false;
			part->pin = false;
			part->ikPos = Vec3f();
			part->ikRot = Rotation();
			part->fka = 0;
			part->fkb = 0;
		}
	}
}

void transformAnimation(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Anim* anim = Window::animMode()){
			anim->transformInterpolators(part);
		}
	}
	else{
		needPartMsg();
	}
}

void toggleAnimNeutral(){
	Anim::autoCapture = !Anim::autoCapture;
}

void take2(Vec2f slider, Vec3f pos, M_Int& index, M_Vec2f& moment, M_Vec3f& relPos);

void beginCompensation(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Anim* anim = Window::animMode()){
			if(anim->play == false){
				PosCompensator* interp = 0;
				if(interp = anim->getPosCompensator(part)){
				}
				else if(interp = anim->connectPartToPosCompensator(part)){
				}
				if(interp){
					take2(Vec2f(anim->slider, anim->slider + 1.0 / anim->stepsPerSecond * 2 / anim->period), part->ikPos, interp->index, interp->moment.refValue(), interp->relPos.refValue());
				}
			}
		}
	}
	else{
		needPartMsg();
	}
}



void removeCompensation(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		if(Anim* anim = Window::animMode()){
			if(anim->play == false){
				anim->removeCompensation(part);
			}
		}
	}
	else{
		needPartMsg();
	}
}

void PosCompensator::take(float slider, Node* node, Field* field){};
void PosCompensator::deleteSample(float slider){};

void highlightWholeTimeLine(){
	Anim* anim = Window::animMode();
	anim->selectFrom = 0;
	anim->selectTo = 1;
}


void makeTrianglesTiny(){
	if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){
		M_Vec3f& coord = part->coord.refValue();
		Seamless* smls = part->smls;
		for(int k = 0; k < coord.length; k+=3){
			Vec3f center = (coord[k] + coord[k + 1] + coord[k + 2]) / 3.0;
			coord[k + 0] = Vec3f(center.x - .0008, center.y, center.z);
			coord[k + 1] = Vec3f(center.x + .0004, center.y, center.z + .0008 * 0.866025404);
			coord[k + 2] = Vec3f(center.x + .0004, center.y, center.z - .0008 * 0.866025404);
		}
		smls->resetRelevantParts();
		smls->process();
		smls->verifyIntegrity();
		smls->ctrf = true;
	}
}

/*
void exposeStem(){
	Node* selNode = g_selNode.getRaw();
	if(SphereBuild* sphere = dynamic_cast<SphereBuild*>(selNode)){
		if(sphere->textureMapper.refValue().length == 0 || dynamic_cast<Stem*>(sphere->textureMapper[0].p) == 0){
			sphere->textureMapper = sphere->getPatch()->stem;
		}
	}
	else if(NurbsLathe* lathe = dynamic_cast<NurbsLathe*>(selNode)){
		if(lathe->textureMapper.refValue().length == 0 || dynamic_cast<Stem*>(lathe->textureMapper[0].p) == 0){
			lathe->textureMapper = lathe->getPatch()->stem;
		}
	}
	else if(BendyLathe* bendy = dynamic_cast<BendyLathe*>(selNode)){
		if(bendy->textureMapper.refValue().length == 0 || dynamic_cast<Stem*>(bendy->textureMapper[0].p) == 0){
			bendy->textureMapper = bendy->getPatch()->stem;
		}
	}
	else if(TorusBuild* torus = dynamic_cast<TorusBuild*>(selNode)){
		if(torus->textureMapper.refValue().length == 0 || dynamic_cast<Stem*>(torus->textureMapper[0].p) == 0){
			torus->textureMapper = torus->getPatch()->stem;
		}
	}
	else if(RectangleBuild* rectangle = dynamic_cast<RectangleBuild*>(selNode)){
		if(rectangle->textureMapper.refValue().length == 0 || dynamic_cast<Stem*>(rectangle->textureMapper[0].p) == 0){
			rectangle->textureMapper = rectangle->getPatch()->stem;
		}
	}
	else if(BoxBuild* box = dynamic_cast<BoxBuild*>(selNode)){
		if(box->textureMapper.refValue().length == 0 || dynamic_cast<Stem*>(box->textureMapper[0].p) == 0){
			if(box->textureMapper.refValue().length < 6){
				int idx = box->textureMapper.refValue().length - 1;
				if(idx < 0){
					idx = 0;
				}
				Stem* stem = dynamic_cast<Stem*>(box->m_patch[idx]->stem[0].p);
				if(Node* colorSweep = dynamic_cast<ColorSweep*>(stem->textureMapper[0].p)){
					createUniqueName(colorSweep);
				}
			}
			box->textureMapper.refValue().length = 0;
			for(int k = 0; k < 6; k++){
				box->textureMapper.refValue() += box->m_patch[k]->stem.refValue();
			}
			box->processFields();
		}
	}
	else if(CylinderBuild* cylinder = dynamic_cast<CylinderBuild*>(selNode)){
		NurbsPatch* patch = cylinder->patch;
		if(cylinder->textureMapper.refValue().length == 0 || dynamic_cast<Stem*>(cylinder->textureMapper[0].p) == 0){
			if(cylinder->textureMapper.refValue().length < 3){
				int idx = cylinder->textureMapper.refValue().length - 1;
				if(idx < 0){
					idx = 0;
				}
				Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
				if(Node* colorSweep = dynamic_cast<ColorSweep*>(stem->textureMapper[0].p)){
					createUniqueName(colorSweep);
				}
			}
			cylinder->textureMapper.refValue().length = 0;
			for(int k = 0; k < patch->stem.length; k++){
				cylinder->textureMapper.refValue() += patch->stem.refValue()[k];
			}
			cylinder->processFields();
		}
	}
	g_stw->commonPostInsertNodeStuff();
}
*/

void hairOnSurface(M_Build& m_build){
	if(NurbsPatch* copyPatch = dynamic_cast<NurbsPatch*>(g_copyNode.getRaw())){
		if(copyPatch->stem.length){
			if(Stem* stem = dynamic_cast<Stem*>(copyPatch->stem[0].p)){
				if(stem->store){
					M_Vec2f& coord = stem->coord.refValue();
					int coordIdx = 0;
					for(int k = 0; k < m_build.length; k++){
						if(NurbsPatch* hairPatch = dynamic_cast<NurbsPatch*>(m_build[k].p)){
							copyPatch->hairOnSurface(coordIdx, coord, hairPatch);
						}
					}
				}
			}
		}
		updateBuild3dWindow();
	}
}

void genHairFromPatch(){
	if(NurbsPatch* selPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		if(selPatch->stem.length){
			if(Stem* stem = dynamic_cast<Stem*>(selPatch->stem[0].p)){
				if(stem->part.length){
					Art* part = stem->part[0];
					M_Vec3f& coord = part->coord.refValue();
					int uDim = 1;
					int vDim = stem->bands[0] + 1;
					int numOfStrands = stem->staves[0] + 1;
					Hair* hair = new Hair;
					hair->radius.refValue() = .00035;
					int from = 0;

					for(int h = 0; h < numOfStrands; h++){
						Strand* ctrlLine = new Strand;
						ctrlLine->dim.refValue() = Vec2f(uDim, vDim);
						ctrlLine->point.refValue().length = vDim;
						ctrlLine->weight.refValue().length = vDim;
						ctrlLine->uJoinEnds.refValue() = false;

						for(int k = 0; k < vDim; k++){
							ctrlLine->point.refValue()[k] = coord[from + (k * numOfStrands)];
							ctrlLine->weight.refValue()[k] = 1;
						}
						from++;

						ctrlLine->uKnot.refValue() = M_Float(0, 0, 0, .25, .25, .5, .5, .75, .75, 1, 1, 1);
						updateNurbsCurveKnotMid0(vDim, ctrlLine->vKnot.refValue());
					

						ctrlLine->processFields();
						hair->strand.refValue() += ctrlLine;
					}
					hair->part.refValue() = part;

					selPatch->smls->build.refValue() += hair;
					g_scene->genNodeList2();
					hair->processFields();
					updateBuild3dWindow();
					g_stw->paint();
				}
			}
		}
	}
}


void synchronizeCells(NurbsPatch* patch, Seamless* smls, Stem* stem);

void synchronizeCells(){
	if(Stem* stem = dynamic_cast<Stem*>(g_selNode.getRaw())){
		int selLine = g_stw->selLine;
		int parentLine = getParent(selLine);
		Node* parentNode= g_sceneTreeList[parentLine].node.p;
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(parentNode)){
			Seamless* smls = patch->smls;
			synchronizeCells(patch, smls, stem);
			g_stw->cp->paint();
			BackUp::monostable.beginCycle();
			updateBuild3dWindow();
		}
	}
}

Vec3f getPatchEdgeNormal(NurbsPatch* patch, int idx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int u = idx % uDim;
	int v = idx / uDim;
	M_Vec3f dpoint = patch->dpoint.refValue();
	Vec3f normal = Vec3f(0,0,1);
	Vec3f normal2;
	Vec3f ap = dpoint[idx];;
	Vec3f bp;
	Vec3f cp;

	if(u == 0){
		bp = dpoint[idx + 1];
		if(v == vDim - 1){
			cp = dpoint[idx - uDim];
			return calcNormal(ap, bp, cp);
		}
		else{
			cp = dpoint[idx + uDim];
		}
		normal = -calcNormal(ap, bp, cp);
	}
	else if(u == uDim - 1){
		bp = dpoint[idx - 1];
		if(v == vDim - 1){
			cp = dpoint[idx - uDim];
			return -calcNormal(ap, bp, cp);
		}
		else{
			cp = dpoint[idx + uDim];
		}
		normal = calcNormal(ap, bp, cp);
	}
	else if(v == 0){
		cp = dpoint[idx + 1];
		bp = dpoint[idx + uDim];
		normal = calcNormal(ap, bp, cp);
	}
	else if(v == vDim - 1){
		bp = dpoint[idx + 1];
		cp = dpoint[idx - uDim];
		normal = calcNormal(ap, bp, cp);
	}
	return normal;
}

void elevatePoint(NurbsPatch* patch, int idx, float amount){
	patch->point.refValue()[idx] += getPatchEdgeNormal(patch, idx) * amount;
}
		//NurbsPatch* newPatch = patch->vSplitPatch(v - 1);
		//newPatch->name = "abc";
		//g_stw->commonPostInsertNodeStuff();
		//return;
/*
NurbsPatch* NurbsPatch::createCopyOf(){//why dont we do a deep copy first?
	NurbsPatch* newPatch = 0;
	if(NurbsLine* thisLine = dynamic_cast<NurbsLine*>(this)){
		NurbsLine* newLine = new NurbsLine;
		newLine->color.refValue() = thisLine->color;
		newPatch = newLine;
	}
	else if(Strand* thisStrand = dynamic_cast<Strand*>(this)){
		Strand* newStrand = new Strand;
		newPatch = newStrand;
	}
	else{
		newPatch = new NurbsPatch;
	}
	if(stem.length){
		Stem* stem = dynamic_cast<Stem*>(this->stem[0].p);
		Stem* newStem = new Stem;
		newStem->part.refValue() += stem->part;
		newStem->sync.refValue() = stem->sync;
		newStem->insideOut.refValue() = stem->insideOut;
		newPatch->stem.refValue() += newStem;
		if(stem->textureMapper.length){
			if(ColorSweep* colorSweep = dynamic_cast<ColorSweep*>(stem->textureMapper[0].p)){
				ColorSweep* newColorSweep = new ColorSweep;
				
				newColorSweep->orientation.refValue() = colorSweep->orientation;
				newColorSweep->location.refValue() = colorSweep->location;
				newColorSweep->color.refValue() = colorSweep->color;
				newColorSweep->distance.refValue() = colorSweep->distance;
				newColorSweep->opacity.refValue() = colorSweep->opacity;
				newColorSweep->spherical.refValue() = colorSweep->spherical;
				newColorSweep->mirror.refValue() = colorSweep->mirror;
				newColorSweep->cylindrical.refValue() = colorSweep->cylindrical;
				newColorSweep->straight.refValue() = colorSweep->straight;
				newColorSweep->part.refValue() = colorSweep->part;

				newStem->textureMapper.refValue() += newColorSweep;
			}
		}
	}
	newPatch->dim.refValue() = dim;
	newPatch->point.refValue() = point;
	newPatch->cpIndex.refValue() = cpIndex;
	newPatch->weight.refValue() = weight;
	newPatch->uKnot.refValue() = uKnot;
	newPatch->vKnot.refValue() = vKnot;
	newPatch->uJoinEnds.refValue() = uJoinEnds;
	newPatch->mirror.refValue() = mirror;
	newPatch->showAll.refValue() = showAll;
	newPatch->sync.refValue() = sync;
	newPatch->insideOut.refValue() = insideOut;
	newPatch->isCtrlPolyRef = isCtrlPolyRef;
	newPatch->smls = smls;

	int mark = smls->cp.length;
	smls->cp.refValue() += point;
	smls->cw.refValue() += weight;

	for(int k = 0; k < dim.x * dim.y; k++){
		newPatch->cpIndex.refValue()[k] = mark + k;
	}


	return newPatch;
}
*/

void NurbsPatch::copyData(const NurbsPatch* from){
	dim.refValue() = from->dim;
	point.refValue() = from->point;
	cpIndex.refValue() = from->cpIndex;
	weight.refValue() = from->weight;
	uKnot.refValue() = from->uKnot;
	vKnot.refValue() = from->vKnot;
	uJoinEnds.refValue() = from->uJoinEnds;
	mirror.refValue() = from->mirror;
	showAll.refValue() = from->showAll;
	sync.refValue() = from->sync;
	insideOut.refValue() = from->insideOut;
	isCtrlPolyRef = from->isCtrlPolyRef;
	smls = from->smls;

	if(ColorMapper* colorMapper = dynamic_cast<ColorMapper*>(from->mapper.p)){
		if(ColorPatch* colorPatch = dynamic_cast<ColorPatch*>(this)){
			colorPatch->color.refValue() = colorMapper->color;
		}
	}
}

NurbsPatch* NurbsPatch::createCopyOf(){
	NurbsPatch* newPatch = 0;

	if(NurbsLine* thisLine = dynamic_cast<NurbsLine*>(this)){
		NurbsLine* newLine = new NurbsLine;
		newLine->color.refValue() = thisLine->color;
		newPatch = newLine;
	}
	else{
		newPatch = new NurbsPatch;
	}
	if(stem.length){
		Stem* stem = dynamic_cast<Stem*>(this->stem[0].p);
		Stem* newStem = new Stem;
		newStem->part.refValue() += stem->part;
		newStem->sync.refValue() = stem->sync;
		newStem->insideOut.refValue() = stem->insideOut;
		newPatch->stem.refValue() += newStem;
	}

	newPatch->copyData(this);

	return newPatch;
}


NurbsPatch* NurbsPatch::createCopyOf(M_Int& redirect){//why dont we do a deep copy first?
	NurbsPatch* newPatch = 0;
	if(NurbsLine* thisLine = dynamic_cast<NurbsLine*>(this)){
		NurbsLine* newLine = new NurbsLine;
		newLine->color.refValue() = thisLine->color;
		newPatch = newLine;
	}
	else if(Strand* thisStrand = dynamic_cast<Strand*>(this)){
		Strand* newStrand = new Strand;
		newPatch = newStrand;
	}
	else{
		newPatch = new NurbsPatch;
	}
	if(stem.length){
		Stem* stem = dynamic_cast<Stem*>(this->stem[0].p);
		Stem* newStem = new Stem;
		newStem->part.refValue() += stem->part;
		newStem->sync.refValue() = stem->sync;
		newStem->insideOut.refValue() = stem->insideOut;
		newStem->xCentred.refValue() = stem->xCentred;
		newPatch->stem.refValue() += newStem;
		if(stem->textureMapper.length){
			if(ColorSweep* colorSweep = dynamic_cast<ColorSweep*>(stem->textureMapper[0].p)){
				ColorSweep* newColorSweep = new ColorSweep;
				
				newColorSweep->orientation.refValue() = colorSweep->orientation;
				newColorSweep->location.refValue() = colorSweep->location;
				newColorSweep->color.refValue() = colorSweep->color;
				newColorSweep->distance.refValue() = colorSweep->distance;
				newColorSweep->opacity.refValue() = colorSweep->opacity;
				newColorSweep->spherical.refValue() = colorSweep->spherical;
				newColorSweep->mirror.refValue() = colorSweep->mirror;
				newColorSweep->cylindrical.refValue() = colorSweep->cylindrical;
				newColorSweep->straight.refValue() = colorSweep->straight;
				newColorSweep->part.refValue() = colorSweep->part;

				newStem->textureMapper.refValue() += newColorSweep;
			}
		}
	}
	newPatch->dim.refValue() = dim;
	newPatch->point.refValue() = point;
	newPatch->cpIndex.refValue() = cpIndex;
	newPatch->weight.refValue() = weight;
	newPatch->uKnot.refValue() = uKnot;
	newPatch->vKnot.refValue() = vKnot;
	newPatch->uJoinEnds.refValue() = uJoinEnds;
	newPatch->mirror.refValue() = mirror;
	newPatch->showAll.refValue() = showAll;
	newPatch->sync.refValue() = sync;
	newPatch->insideOut.refValue() = insideOut;
	newPatch->isCtrlPolyRef = isCtrlPolyRef;
	newPatch->smls = smls;

	M_Int& cpIndex = newPatch->cpIndex;
	for(int i = cpIndex.length; --i>=0;){
		cpIndex[i] = redirect[cpIndex[i]];
	}

	return newPatch;
}

int getItemIndex(M_Node& m_node, Node* node){
	for(int i = m_node.length; --i>=0;){
		if(m_node[i].p == node){
			return i;
		}
	}
	return -1;
}

void SceneTree::selectNodeForExtrude(Node* node){
	g_stw->setSelNodeLine(node);
	g_stw->lastSelNodeLine = selLine;
	g_stw->paint();
	setTitleName();
}


void set_m_nodeRef(NurbsPatch* patch, M_Node*& m_node){
	int line = getNodeLine(patch);
	int parentLine = getParent(line);
	Node* parentNode = g_sceneTreeList[parentLine].node.p;
	int itemIdx = 0;
	int fieldIdx = 0;
	getFNodeElementForParent(parentLine, line, g_sceneTreeList, fieldIdx, itemIdx);
	MFNode* parentField = 0;

	if(Seamless* smls = dynamic_cast<Seamless*>(parentNode)){
		parentField = &smls->build;
	}
	else if(GroupBuild* group = dynamic_cast<GroupBuild*>(parentNode)){
		parentField = &group->build;
	}
	m_node = &parentField->refValue();
}

void Extrude::adjustRightItemIdx(){
	if(leftBeforeRight){
		r_itemIdx++;
	}
}


void extrudePatch(){
	resetTemporaryModes();

	NurbsPatchPtr patch;

	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(g_scene->mode & HAIR_REGION_MODE){
			g_extrude->r_cap = lock;
			extrudeRegion();
			return;
		}
	}
	else if(MultiSelect::pointList.length == 1){
		patch = dynamic_cast<NurbsPatch*>(MultiSelect::pointList[0].p);
		if(patch == 0){
			return;
		}



		g_extrude->smls = patch->smls;
		g_extrude->distance.refValue() = .01;
		g_extrude->insertions.refValue() = 1;
		g_extrude->l_cap = 0;
		g_extrude->leftBeforeRight = false;

		bool withStem = patch->stem.length > 0;

		g_stw->cp->setupNodeWindow(g_extrude);
				
		Vec3f normal;

		Window::activeWindow = g_stw->cp.p;
		//g_extrude->processFields();





		NurbsPatch* originalPatch = patch;
		NurbsPatch* r_patch0 = 0;
		NurbsPatch* l_patch0 = 0;
		NurbsPatch* r_patchA = 0;
		NurbsPatch* l_patchA = 0;
		NurbsPatch* r_patchB = 0;
		NurbsPatch* l_patchB = 0;

		NurbsPatchPtr r_cap;
		NurbsPatchPtr l_cap;
		NurbsPatchPtr r_patchTemp;
		NurbsPatchPtr l_patchTemp;


		NurbsPatch* r_patchC = 0;
		NurbsPatch* r_patchD = 0;
		NurbsPatch* r_patchE = 0;
		NurbsPatch* patchG = 0;

		Seamless* smls = patch->smls;

#ifndef CPINDEX
		smls->convertToCpFormat();
#endif

		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		int idx = MultiSelect::pointList[0].i;
		//int idx = patch->index[0];
		int u = idx % uDim;
		int v = idx / uDim;
		int mirrorIdx = patch->getMirrorTwinlessIdx(idx);
		int newU = mirrorIdx % uDim;
		g_extrude->centerMirror = false;

		MultiSelect::reset();

		smls->generateRawStitchPolys();


		set_m_nodeRef(patch, g_extrude->r_m_node);
		g_extrude->l_m_node = g_extrude->r_m_node;
		if(patch->mirrorPatch){
			set_m_nodeRef(patch->mirrorPatch, g_extrude->l_m_node);
		}
		M_Node& r_m_node = *g_extrude->r_m_node;
		M_Node& l_m_node = *g_extrude->l_m_node;


		g_extrude->_cap.refValue() = false;
		g_extrude->hole.refValue() = true;
		g_extrude->spiderWebCenter.refValue() = false;

		if(l_cap = patch->mirrorPatch){
			

			if(patch->isInsideOut()){
				NurbsPatch* store = patch;
				patch = l_cap;
				l_cap = store;
			}
			r_patchB = patch;
			if(v > 1){
				smls->generateRawStitchPolys();
				r_patchA = patch->vSplitPatch(v - 1);
			}
			if(vDim - v > 3){
				smls->generateRawStitchPolys();
				r_patchB = patch->vSplitPatch(2);
			}

			smls->generateRawStitchPolys();

			int r_itemIdx = getItemIndex(r_m_node, r_patchB);
			if(uDim - u > 2){
				r_itemIdx++;
				r_cap = r_patchB->uSplitPatch(u + 1, r_m_node, r_itemIdx, g_extrude->_cap);
			}
			else{
				r_cap = r_patchB;
				r_patchB = 0;
				if(g_extrude->_cap == false){
					r_m_node.deleteElement(r_itemIdx);
					smls->deletePoliesOwnedByPatch(r_cap);
				}
			}
			r_cap->name = "r_cap";
			g_extrude->r_itemIdx = r_itemIdx;
			if(g_extrude->_cap){
				r_itemIdx++;
			}
			g_extrude->r_cap = r_cap;
				
			g_stw->lesserPostInsertNodeStuff();

			smls->generateRawStitchPolys();
			if(u > 1){
				//r_patchTemp = patch->uSplitPatch(u - 1);
				r_patchTemp = r_cap->uSplitPatch(u - 1, r_m_node, r_itemIdx, true);
				r_patchTemp->name = "r_patchTemp";
				g_stw->lesserPostInsertNodeStuff();
				r_patchTemp->isCtrlPolyRef = true;
				smls->generateRawStitchPolys();
			}

			//do left patches

			if(getItemIndex(l_m_node, l_cap) < g_extrude->r_itemIdx && g_extrude->l_m_node == g_extrude->r_m_node){
				g_extrude->leftBeforeRight = true;
			}

			if(v > 1){
				smls->generateRawStitchPolys();
				l_patchA = l_cap->vSplitPatch(v - 1);
				g_extrude->adjustRightItemIdx();
			}
			l_patchB = l_cap;
			if(vDim - v > 3){
				smls->generateRawStitchPolys();
				l_patchB = l_cap->vSplitPatch(2);
				g_extrude->adjustRightItemIdx();
			}
			smls->generateRawStitchPolys();
			int l_itemIdx = getItemIndex(l_m_node, l_patchB);
			if(uDim - u > 2){
				l_itemIdx++;
				l_cap = l_patchB->uSplitPatch(u + 1, l_m_node, l_itemIdx, g_extrude->_cap);
				g_extrude->adjustRightItemIdx();
			}
			else{
				l_cap = l_patchB;
				l_patchB = 0;
				if(g_extrude->_cap == false){
					l_m_node.deleteElement(l_itemIdx);
					smls->deletePoliesOwnedByPatch(l_cap);
				}
			}
			l_cap->name = "l_cap";
			g_extrude->l_itemIdx = l_itemIdx;
			if(g_extrude->_cap){
				l_itemIdx++;
			}
			g_extrude->l_cap = l_cap;

			g_stw->lesserPostInsertNodeStuff();

			smls->generateRawStitchPolys();
			if(u > 1){
				l_patchTemp = l_cap->uSplitPatch(u - 1, l_m_node, l_itemIdx, true);
				g_extrude->adjustRightItemIdx();
				l_patchTemp->name = "l_patchTemp";
				g_stw->lesserPostInsertNodeStuff();
				l_patchTemp->isCtrlPolyRef = true;
				smls->generateRawStitchPolys();
			}

			
			patch = r_cap;
			r_cap->mirrorPatch = l_cap;
			l_cap->mirrorPatch = r_cap;


			
		//smls->populateMirrorNodes();
		//smls->commonEndSitchPolys();
		//g_stw->lesserPostInsertNodeStuff();
		//smls->generateRawStitchPolys();
		//smls->stitchAllPolys();
		//return;

		}
		else{
			if(patch->mirror && u == uDim >> 1){
				g_extrude->centerMirror = true;
			}
			if(vDim - v > 3){
				smls->generateRawStitchPolys();
				patch = patch->vSplitPatch(v + 1);
			}
			if(v > 1){
				smls->generateRawStitchPolys();
				r_patchD = patch;
				r_patchC = patch->vSplitPatch(v - 1);
			}

			if(patch->mirror && g_extrude->centerMirror == false){

				smls->generateRawStitchPolys();

				if(u >= newU){
					newU = u % uDim;
					u = patch->getMirrorTwinlessIdx(u);
				}

				g_extrude->leftBeforeRight = true;
				
				int l_itemIdx = getItemIndex(r_m_node, patch);
				if(u > 1){
					l_patchTemp = patch;
					l_itemIdx++;
					l_cap = patch->uSplitPatch(newU + 1, r_m_node, l_itemIdx, g_extrude->_cap);
				}
				else{
					r_patchD = 0;
					l_cap = patch;
					if(g_extrude->_cap == false){
						r_m_node.deleteElement(l_itemIdx);
						smls->deletePoliesOwnedByPatch(l_cap);
					}
				}
				g_extrude->l_itemIdx = l_itemIdx;
				if(g_extrude->_cap){
					l_itemIdx++;
				}
				g_extrude->l_cap = l_cap;


				l_cap->name = "l_cap";
				g_stw->lesserPostInsertNodeStuff();
				smls->generateRawStitchPolys();
				patch = l_cap->uSplitPatch(newU - 1, r_m_node, l_itemIdx, true);
				g_stw->lesserPostInsertNodeStuff();
				patch->isCtrlPolyRef = true;

				r_patchE = patch;
				l_cap->setInsideOut4();


				if(newU - 1 != u + 1){
					l_itemIdx++;
					smls->generateRawStitchPolys();
					patch = patch->uSplitPatch(u + 1, r_m_node, l_itemIdx, g_extrude->_cap);
					g_stw->lesserPostInsertNodeStuff();
				}
				else{
					r_cap = patch;
					if(g_extrude->_cap == false){
						r_m_node.deleteElement(l_itemIdx);
						smls->deletePoliesOwnedByPatch(r_cap);
					}
				}
				g_extrude->r_itemIdx = l_itemIdx;
				if(g_extrude->_cap){
					l_itemIdx++;
				}

				smls->generateRawStitchPolys();
				if(u > 1){
					r_patchTemp = patch->uSplitPatch(u - 1, r_m_node, l_itemIdx, true);
					g_stw->lesserPostInsertNodeStuff();
					r_patchTemp->isCtrlPolyRef = true;
					smls->generateRawStitchPolys();
				}
				else{
					r_cap = patch;
				}
				/*
				}
				else{
					if(uDim - u > 2){
						l_patchTemp = patch;
						l_itemIdx++;
						//l_cap = patch->uSplitPatch(u + 1);
						l_cap = patch->uSplitPatch(u + 1, r_m_node, l_itemIdx, true);
					}
					else{
						l_cap = patch;
					}

					l_cap->name = "l_cap";
					smls->generateRawStitchPolys();
					patch = l_cap->uSplitPatch(u - 1);
					r_patchE = patch;
					l_cap->setInsideOut();

					if(u - 1 != newU + 1){
						smls->generateRawStitchPolys();
						patch = patch->uSplitPatch(newU + 1);
						g_stw->lesserPostInsertNodeStuff();
					}

					smls->generateRawStitchPolys();
					if(uDim - u > 2){
						r_patchTemp = patch->uSplitPatch(newU - 1);
					}
					else{
						r_patchTemp = patch;
					}
				}
				*/
				r_cap = patch;

				r_cap->name = "r_cap";
				if(r_patchTemp){
					l_patchTemp->name = "l_patchTemp";
					l_patchTemp->setInsideOut4();
					r_patchTemp->name = "r_patchTemp";
					r_patchTemp->mirrorTwinPatch(l_patchTemp);
					r_patchTemp->mirrorTwinPatch(l_patchTemp);
				}

				patch = r_cap;
				patch->mirrorPatch = l_cap;
				l_cap->mirrorPatch = patch;
			}
			else{
				smls->generateRawStitchPolys();

				int r_itemIdx = getItemIndex(r_m_node, patch);
				if(uDim - u > 2){
					patchG = patch;
					r_itemIdx++;
					r_cap = patch->uSplitPatch(u + 1, r_m_node, r_itemIdx, g_extrude->_cap);
				}
				else{
					r_cap = patch;
					r_patchD = 0;
					if(g_extrude->_cap == false){
						r_m_node.deleteElement(r_itemIdx);
						smls->deletePoliesOwnedByPatch(r_cap);
					}
				}

				g_extrude->r_itemIdx = r_itemIdx;
				if(g_extrude->_cap){
					r_itemIdx++;
				}

				//patch->name = "cap";
				
				g_stw->lesserPostInsertNodeStuff();

				smls->generateRawStitchPolys();
				if(u > 1){
					//r_patchTemp = patch->uSplitPatch(u - 1);
					r_patchTemp = r_cap->uSplitPatch(u - 1, r_m_node, r_itemIdx, true);

					//g_stw->genNodeList();
					smls->populateMirrorNodes();
					r_patchTemp->isCtrlPolyRef = true;
				}
				smls->generateRawStitchPolys();

				if(g_extrude->centerMirror && uDim > 3){
					r_patchA = r_patchTemp;
					l_patchA = patchG;
					r_patchA->mirrorPatch = l_patchA;
					l_patchA->setInsideOut4();
					r_patchA->mirrorTwinPatch(l_patchA);
					smls->generateRawStitchPolys();
				}
				patch = r_cap;
				//patch->name = "cap";
			}
		}
		g_extrude->r_cap = patch;
		l_cap = patch->mirrorPatch;

		calcNormal(
			patch->generate(Vec2f(.5 - .01, .5)),
			patch->generate(Vec2f(.5, .5)),
			patch->generate(Vec2f(.5, .5 + .01)),
			normal
		);
		normal = -normal.normalize();
		if(g_extrude->centerMirror){
			normal.x = 0;
		}

		g_extrude->normal = normal;



		g_extrude->storePointList = patch->point;

		g_stw->lesserPostInsertNodeStuff();



		uDim = patch->dim.x;
		vDim = patch->dim.y;
		M_Vec3f& point = patch->point.refValue();

		NurbsPatchPtr r_shaft = new NurbsPatch;
		g_extrude->r_shaft = r_shaft;
		r_shaft->sync.refValue() = true;

		//r_newPatch->showAll.refValue() = true;
		r_shaft->isCtrlPolyRef = true;
		int newUdim = 9;
		int newVdim = g_extrude->insertions * 2 + 1;
		r_shaft->dim.refValue() = Vec2f(newUdim, newVdim);
		r_shaft->smls = smls;
		r_shaft->uJoinEnds.refValue() = false;
		M_Vec3f& newPoint = r_shaft->point.refValue();
		M_Float& newWeight = r_shaft->weight.refValue();
		M_Float& newUknot = r_shaft->uKnot.refValue();
		M_Float& newVknot = r_shaft->vKnot.refValue();
		
		int newPointLen = newUdim * newVdim;
		newPoint.length = newPointLen;
		newWeight.length = newPointLen;

		for(int i = newPointLen; --i >= 0;){
			newWeight[i] = 1;
		}

		g_extrude->markCp = smls->addCpsForStrip(r_shaft, newUdim, newVdim);

		updateNurbsCurveKnotMid1(newUdim, newUknot);
		updateNurbsCurveKnotMid1(newVdim, newVknot);
		if(withStem){
			StemPtr newStem = new Stem;
			newStem->sync.refValue() = true;

			if(patch->stem.length){
				Stem* oldStem = dynamic_cast<Stem*>(patch->stem.refValue()[0].p);
				if(oldStem->part.length){
					newStem->part.refValue() += oldStem->part.refValue()[0].p;
				}
			}
			r_shaft->stem.refValue() += newStem.p;
		}
		int topRowOffset = (newVdim - 1) * newUdim;

		g_extrude->extraCenterMirrorPatch = 0;

		r_shaft->mirror.refValue() = patch->mirror;

		NurbsPatchPtr l_shaft;

		if(l_cap){
			l_shaft = r_shaft->createCopyOf();
			r_shaft->mirrorPatch = l_shaft;
			l_shaft->mirrorPatch = r_shaft;
			l_shaft->setInsideOut4();
			smls->addCpsForStrip(l_shaft, newUdim, newVdim);
			r_shaft->name = "r_shaft";
			l_shaft->name = "l_shaft";
			g_extrude->l_storeCpIndex = l_cap->cpIndex;

			g_extrude->l_storeCpIndex = l_cap->cpIndex;
		}

		g_extrude->r_storeCpIndex = patch->cpIndex;


		g_extrude->doDistance();

//		return;
		if(l_cap){
			patch->mirrorTwinPatch(l_cap);
		}


		//if(newPatch->mirrorPatch){
		//	newPatch->mirrorTwinPatch(newPatch->mirrorPatch);
		//}

		smls->commonEndSitchPolys();



//		M_Int deleteCtrlPolyList;
//		smls->findStitchPolyForExtrude(deleteCtrlPolyList, patch, 0);
//		smls->findStitchPolyForExtrude(deleteCtrlPolyList, patch, uDim - 1);
//		smls->findStitchPolyForExtrude(deleteCtrlPolyList, patch, vDim * uDim - 1);
//		smls->findStitchPolyForExtrude(deleteCtrlPolyList, patch, (vDim - 1) * uDim);


//		bubbleSort(deleteCtrlPolyList);
//		smls->deleteCtrlPoly(deleteCtrlPolyList);
//		smls->commonEndSitchPolys();

		r_shaft->processFields();
		if(l_cap){
			l_shaft->processFields();
		}
		
		smls->generateRawStitchPolys();



		g_stw->lesserPostInsertNodeStuff();
		if(l_cap){
			l_cap->synchronizeAllStemCells();
		}


		if(l_cap){
			r_shaft->mirrorTwinPatch(l_shaft);
		}

		r_m_node += r_shaft.p;
		if(l_shaft){
			l_m_node += l_shaft.p;
		}

		g_stw->lesserPostInsertNodeStuff();
		r_shaft->isCtrlPolyRef = true;
		if(l_shaft){
			l_shaft->isCtrlPolyRef = true;
		}
		//r_newPatch->fillEdgesWithCtrlPolys();
		smls->generateRawStitchPolys();

		if(l_cap){
			if(r_cap && l_cap){
				r_cap->makeExistingNameUniqueForMirror(l_cap);
			}
			r_shaft->makeExistingNameUniqueForMirror(l_shaft);
			if(r_patchTemp && l_patchTemp){
				r_patchTemp->makeExistingNameUniqueForMirror(l_patchTemp);
			}

			if(r_patchA){
				r_patchA->name = "r_patchA";
				r_patchA->makeExistingNameUniqueForMirror(l_patchA);
			}
			if(r_patchB){
				r_patchB->name = "r_patchB";
				r_patchB->makeExistingNameUniqueForMirror(l_patchB);
			}
		}
		if(g_extrude->centerMirror && r_patchA){
			r_patchA->name = "r_patchA";
			r_patchA->makeExistingNameUniqueForMirror(l_patchA);
		}

		r_shaft->patchToCentralCPs();
		if(l_shaft){
			l_shaft->patchToCentralCPs();
		}


		//smls->synchronizeAllStemCells();

		g_stw->lesserPostInsertNodeStuff();
		smls->commonEndSitchPolys();
		smls->stitchAllPolys();

		if(g_extrude->centerMirror){
			g_extrude->r_shaft = r_shaft->uSplitPatch(2);
			g_extrude->extraCenterMirrorPatch = r_shaft;

			g_stw->lesserPostInsertNodeStuff();
			smls->generateRawStitchPolys();
			smls->stitchAllPolys();
			//smls->synchronizeAllStemCells();
		}
		

		//if(originalPatch && originalPatch != g_extrude->r_cap){
		//	originalPatch->synchronizeCellsAndBuild();
		//	if(originalPatch->mirrorPatch){
		//		//originalPatch->mirrorPatch->synchronizeCellsAndBuild();
		//	}
		//}

		smls->populateMirrorNodes();
		smls->deleteAllSyncParts();
		smls->generateAllSyncPatchesBnspe();
		if(smls->autoJoin){
			smls->joinAllParts();
		}
		smls->doBuild();
		houseKeepVisualClue();
	}
}
void Seamless::deleteAllParts(){
	deleteAllPartsNonBnspe();
/*
	for(int k = 0; k < partList.length; k++){
		partList[k]->deleteGeometry();
	}
*/
}

void Seamless::generateAllPatchesBnspe(){
	//deleteAllPartsNonBnspe();
	for(int k = 0; k < cpNodeList.length; k++){
		cpNodeList[k]->synchronizeCellsAndBuild();
	}
}

void Seamless::generateAllSyncPatchesBnspe(){//exactly same as above except we call synchronizeCellsAndBuildSync()
	//deleteAllPartsNonBnspe();
	for(int k = 0; k < cpNodeList.length; k++){
		cpNodeList[k]->synchronizeCellsAndBuildSync();
	}
}

void Extrude::refreshCap(){
	M_Vec3f& point = r_cap->point.refValue();
	Vec3f a = normal * distance;

	for(int i = storePointList.length; --i>=0;){
		point[i] = storePointList[i] + a;
	}
}


void joinCap(M_Int& cpIndex, M_Int& newCpIndex, int offsetIdx, int uDim){
	cpIndex[0] = newCpIndex[offsetIdx + 0];
	cpIndex[1] = newCpIndex[offsetIdx + 1];

	cpIndex[2] = newCpIndex[offsetIdx + 2];
	cpIndex[uDim + 2] = newCpIndex[offsetIdx + 3];

	cpIndex[uDim * 2 + 2] = newCpIndex[offsetIdx + 4];
	cpIndex[uDim * 2 + 1] = newCpIndex[offsetIdx + 5];

	cpIndex[uDim * 2 + 0] = newCpIndex[offsetIdx + 6];
	cpIndex[uDim] = newCpIndex[offsetIdx + 7];

	cpIndex[0] = newCpIndex[offsetIdx + 8];
}


void joinExtraCenterCap(M_Int& cpIndex, M_Int& newCpIndex, int offsetIdx, int uDim, M_Int& eCpIndex, int eoffsetIdx){
	cpIndex[0] = newCpIndex[offsetIdx + 0];
	cpIndex[1] = newCpIndex[offsetIdx + 1];

	cpIndex[2] = newCpIndex[offsetIdx + 2];
	cpIndex[2] = eCpIndex[eoffsetIdx + 0];

	cpIndex[uDim + 2] = eCpIndex[eoffsetIdx + 1];

	cpIndex[uDim * 2 + 2] = eCpIndex[eoffsetIdx + 2];
	cpIndex[uDim * 2 + 1] = eCpIndex[eoffsetIdx + 3];

	cpIndex[uDim * 2 + 0] = eCpIndex[eoffsetIdx + 4];
	cpIndex[uDim] = eCpIndex[eoffsetIdx + 5];

	cpIndex[0] = eCpIndex[eoffsetIdx + 6];
}


void Extrude::resetTopShaftPoints(){
	refreshCap();

	int uDim = r_cap->dim.x;

	int newUdim = r_shaft->dim.x;
	int newVdim = r_shaft->dim.y;
	int offsetIdx = newUdim * (newVdim - 1);
	M_Vec3f& newPoint = r_shaft->point.refValue();
	M_Int& newCpIndex = r_shaft->cpIndex.refValue();

	M_Vec3f& point = r_cap->point.refValue();
	M_Int& cpIndex = r_cap->cpIndex.refValue();

	if(extraCenterMirrorPatch){
		M_Vec3f& newPoint2 = extraCenterMirrorPatch->point.refValue();
		int newUdim2 = extraCenterMirrorPatch->dim.x;
		int newVdim2 = extraCenterMirrorPatch->dim.y;
		int eoffsetIdx = newUdim2 * (newVdim2 - 1);

		newPoint[offsetIdx + 0] = point[0];
		newPoint[offsetIdx + 1] = point[1];

		newPoint[offsetIdx + 2] = point[2];
		newPoint2[eoffsetIdx + 0] = point[2];

		newPoint2[eoffsetIdx + 1] = point[uDim + 2];

		newPoint2[eoffsetIdx + 2] = point[uDim * 2 + 2];
		newPoint2[eoffsetIdx + 3] = point[uDim * 2 + 1];

		newPoint2[eoffsetIdx + 4] = point[uDim * 2 + 0];
		newPoint2[eoffsetIdx + 5] = point[uDim];

		newPoint2[eoffsetIdx + 6] = point[0];

		if(_cap){
			joinExtraCenterCap(cpIndex, newCpIndex, offsetIdx, uDim, extraCenterMirrorPatch->cpIndex.refValue(), eoffsetIdx);
		}
	}
	else{
		newPoint[offsetIdx + 0] = point[0];
		newPoint[offsetIdx + 1] = point[1];

		newPoint[offsetIdx + 2] = point[2];
		newPoint[offsetIdx + 3] = point[uDim + 2];

		newPoint[offsetIdx + 4] = point[uDim * 2 + 2];
		newPoint[offsetIdx + 5] = point[uDim * 2 + 1];

		newPoint[offsetIdx + 6] = point[uDim * 2 + 0];
		newPoint[offsetIdx + 7] = point[uDim];

		newPoint[offsetIdx + 8] = point[0];

		if(_cap){
			joinCap(cpIndex, newCpIndex, offsetIdx, uDim);
			if(NurbsPatch* l_shaft = r_shaft->mirrorPatch){

				joinCap(l_cap->cpIndex.refValue(), l_shaft->cpIndex.refValue(), offsetIdx, uDim);
			}
		}
	}
}

int NurbsPatch::closeEndPointsForSpiderWeb(Vec3f center){
	int uDim = dim.x;
	int vDim = dim.y;
	int offsetIdx = uDim * (vDim - 1);
	M_Vec3f& point = this->point.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	int centerIdx = cpIndex[(vDim - 1) * uDim];

	for(int i = 0; i < uDim; i++){
		point[offsetIdx + i] = center;
		cpIndex[offsetIdx + i] = centerIdx;
	}

	return centerIdx;
}

void Extrude::closeEndPointsForSpiderWeb(){
	Vec3f center = r_cap->point[4];
	int centerIdx = r_shaft->closeEndPointsForSpiderWeb(center);

	if(NurbsPatch* l_shaft = r_shaft->mirrorPatch){
		Vec3f l_center = r_cap->mirrorPatch->point[4];
		l_shaft->closeEndPointsForSpiderWeb(l_center);
	}

	if(extraCenterMirrorPatch){
		M_Vec3f& newPoint2 = extraCenterMirrorPatch->point.refValue();
		M_Int& newCpIndex2 = extraCenterMirrorPatch->cpIndex.refValue();
		int newUdim2 = extraCenterMirrorPatch->dim.x;
		int newVdim2 = extraCenterMirrorPatch->dim.y;
		int offsetIdx2 = newUdim2 * (newVdim2 - 1);

		for(int i = 0; i < newUdim2; i++){
			newPoint2[offsetIdx2 + i] = center;
			newCpIndex2[offsetIdx2 + i] = centerIdx;
		}
	}
}

void Extrude::insertCaps(){
	if(l_cap.p && leftBeforeRight){
		l_m_node->insertElement(l_itemIdx, l_cap.p);
	}
	r_m_node->insertElement(g_extrude->r_itemIdx, g_extrude->r_cap.p);
	if(l_cap.p && leftBeforeRight == false){
		l_m_node->insertElement(l_itemIdx, l_cap.p);
	}

	smls->populateMirrorNodes();
	g_stw->lesserPostInsertNodeStuff();

}

void Extrude::deleteCaps(){
	smls->deletePoliesOwnedByPatch(r_cap);

	if(_cap){
		if(g_extrude->l_cap && leftBeforeRight == false){
			smls->deletePoliesOwnedByPatch(g_extrude->l_cap);
			g_extrude->l_m_node->deleteElement(g_extrude->l_itemIdx);
		}
		smls->deletePoliesOwnedByPatch(r_cap);
		g_extrude->r_m_node->deleteElement(r_itemIdx);
		if(g_extrude->l_cap && leftBeforeRight){
			smls->deletePoliesOwnedByPatch(l_cap);
			g_extrude->l_m_node->deleteElement(l_itemIdx);
		}
		smls->populateMirrorNodes();
		g_stw->lesserPostInsertNodeStuff();
	}
}

void Extrude::tidyUpCells(){
	//if(_cap){
		//cap->synchronizeAllStemCells();
		//bnspeBuildPolies(cap);
		//if(l_cap){
		//	l_cap->synchronizeCellsAndBuild();
		//}
	//}

	//shaft->synchronizeCellsAndBuild();

	//if(NurbsPatch* mirror_shaft = shaft->mirrorPatch){
	//	mirror_shaft->synchronizeCellsAndBuild();
	//}
	//if(extraCenterMirrorPatch){
	//	extraCenterMirrorPatch->synchronizeCellsAndBuild();
	//}

	smls->deleteAllSyncParts();
	smls->generateAllSyncPatchesBnspe();
	if(smls->autoJoin){
		smls->joinAllParts();
	}

}

void Extrude::Cap::operator=(const bool& par){
	if(home._cap == false){

		home.insertCaps();


		home._cap.refValue() = true;
		home.hole.refValue() = false;
		home.spiderWebCenter.refValue() = false;

		NurbsPatch* r_cap = home.r_cap;
		NurbsPatch* extraCenterMirrorPatch = home.extraCenterMirrorPatch;
		NurbsPatch* l_cap = home.l_cap;
		NurbsPatch* r_shaft = g_extrude->r_shaft;
		NurbsPatch* l_shaft = r_shaft->mirrorPatch;
		NurbsPatch* mirror_shaft = r_shaft->mirrorPatch;
		Seamless* smls = g_extrude->smls;
		r_cap->isCtrlPolyRef = true;
		smls->generateRawStitchPolys();

		home.resetTopShaftPoints();


		smls->cp.refValue().length = home.markCp;
		smls->addCpsForStrip(r_shaft, r_shaft->dim.x, r_shaft->dim.y);
		if(l_shaft){
			smls->addCpsForStrip(l_shaft, l_shaft->dim.x, l_shaft->dim.y);
		}
		if(extraCenterMirrorPatch){
			smls->addCpsForStrip(extraCenterMirrorPatch, extraCenterMirrorPatch->dim.x, extraCenterMirrorPatch->dim.y);
		}


		if(home.l_cap.p){
			home.r_cap->mirrorTwinPatch(home.l_cap);
			home.r_shaft->mirrorTwinPatch(home.r_shaft->mirrorPatch);
			home.l_cap->isCtrlPolyRef = true;
		}

		home.doDistance();

		r_shaft->patchToCentralCPs();
		if(l_shaft){
			l_shaft->patchToCentralCPs();
		}
		if(extraCenterMirrorPatch){
			extraCenterMirrorPatch->patchToCentralCPs();
		}


		smls->generateRawStitchPolys();
		g_stw->lesserPostInsertNodeStuff();
		smls->commonEndSitchPolys();
		smls->stitchAllPolys();

		home.tidyUpCells();
	}
}
void Extrude::Hole::operator=(const bool& par){
	if(home.hole == false){

		NurbsPatch* r_shaft = home.r_shaft;
		NurbsPatch* extraCenterMirrorPatch = home.extraCenterMirrorPatch;
		NurbsPatch* l_shaft = r_shaft->mirrorPatch;
		Seamless* smls = home.smls;

		home.deleteCaps();

		home._cap.refValue() = false;
		home.hole.refValue() = true;
		home.spiderWebCenter.refValue() = false;

		smls->generateRawStitchPolys();
		
		home.resetTopShaftPoints();
//
		smls->cp.refValue().length = home.markCp;
		smls->addCpsForStrip(r_shaft, r_shaft->dim.x, r_shaft->dim.y);
		if(extraCenterMirrorPatch){
			smls->addCpsForStrip(extraCenterMirrorPatch, extraCenterMirrorPatch->dim.x, extraCenterMirrorPatch->dim.y);
		}
		if(l_shaft){
			smls->addCpsForStrip(l_shaft, l_shaft->dim.x, l_shaft->dim.y);
		}
//
		if(home.l_cap.p){
			home.r_shaft->mirrorTwinPatch(home.r_shaft->mirrorPatch);
		}

		home.doDistance();

		smls->generateRawStitchPolys();
		g_stw->lesserPostInsertNodeStuff();
		smls->commonEndSitchPolys();
		smls->stitchAllPolys();

		r_shaft->patchToCentralCPs();
		if(l_shaft){
			l_shaft->patchToCentralCPs();
		}

		home.tidyUpCells();

	}
}
void Extrude::SpiderWebCenter::operator=(const bool& par){
	if(home.spiderWebCenter == false){

		NurbsPatch* patch = g_extrude->r_cap;
		NurbsPatch* mirrorPatch = patch->mirrorPatch;
		NurbsPatch* newPatch = g_extrude->r_shaft;
		Seamless* smls = g_extrude->smls;

		home.deleteCaps();

		home._cap.refValue() = false;
		home.hole.refValue() = false;
		home.spiderWebCenter.refValue() = true;


		smls->generateRawStitchPolys();
		

		int newUdim = newPatch->dim.x;
		int newVdim = newPatch->dim.y;
		int offsetIdx = newUdim * (newVdim - 1);

		home.closeEndPointsForSpiderWeb();
		if(home.l_cap.p){
			home.r_shaft->mirrorTwinPatch(home.r_shaft->mirrorPatch);
		}

		smls->generateRawStitchPolys();
		g_stw->lesserPostInsertNodeStuff();
		smls->commonEndSitchPolys();
		smls->stitchAllPolys();

		home.tidyUpCells();
	}
}

void generateRawStitchPolys(){
	Seamless* smls = dynamic_cast<Seamless*>(g_selNode.getRaw());
	if(smls == 0){
		smls = getSeamless();
	}
	if(smls){
		smls->generateRawStitchPolys();
		houseKeepVisualClue();
	}

}

void locateBuildNode(Art* part, int selTriIndex){
	for(int line = g_sceneTreeList.length; --line >= 0;){
		if(Build* bld = dynamic_cast<Build*>(g_sceneTreeList[line].node.p)){
			if(NurbsPatch* patch = bld->getPatch()){
				if(patch->stem.length){
					M_Node& m_stem = patch->stem.refValue();
					for(int j = m_stem.length; --j>=0;){
						if(Stem* stem = dynamic_cast<Stem*>(m_stem[j].p)){
							if(selTriIndex >= stem->beginTriOffset){
								if(stem->part.length == 1 && stem->part.refValue()[0] == part){
									//g_stw->selectNode(line, bld, 0, tmpInt);
									g_stw->textCell.cellActive = false;
									g_stw->selLine = line;

									
									houseKeepVisualClue();
									g_stw->paint();
									return;
								}
							}
						}
					}
				}
			}
		}
	}
}



void Extrude::processFields(){
	if(dynamic_cast<Lock*>(g_selNode.getRaw())){
		g_transformCP->translation.refValue() = Vec3f();

		int ins = 0;
		Vec3f translation;
		Matrix matrix;

		NurbsPatch* prevPatch = 0;

		for(int i = 0; i < MultiSelect::pointList.length; i++){
			int idx = MultiSelect::pointList[i].i;
			NurbsPatch* patch = MultiSelect::pointList[i].p;
			if(prevPatch != patch){
				prevPatch = patch;
				ins++;
				translation = normal * (distance / insertions * ins);
				matrix = translationMatrix(-g_transformCP->pivotPoint) * rotationMatrix(-g_transformCP->scaleOrientation) * scaleMatrix(g_transformCP->scale) * rotationMatrix(g_transformCP->scaleOrientation) * rotationMatrix(g_transformCP->rotation) * translationMatrix(g_transformCP->pivotPoint) * translationMatrix(translation);
			}
			Vec3f svec = matrix * MultiSelect::beginDragPos[i];
			g_transformCP->setFilteredDimension(patch->point.refValue(), smls->cp.refValue(), patch->cpIndex.refValue(), idx, svec * patch->weight[idx]);
			patch->dpoint.refValue()[idx] = patch->point[idx] / patch->weight[idx];
			patch->mirrorSingleCP(idx);
		}

		IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_transformLeverClue->geometry.p);
		M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
		coord.length = 0;
		coord += g_transformCP->pivotPoint;
		coord += g_transformCP->rotCtrlPnt;
		fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
		polyLine->convertToRenderFormat(g_transformLeverClue->render);

		Seamless* smls = getSeamless2();
		smls->calcAllPolyMidPoints();
		commonPostTug();
		smls->updateCage();
		houseKeepVisualClue();
	}
}


void extendNewPatch(NurbsPatch* newPatch){
	int newUdim = newPatch->dim.x;
	//int newVdim = newPatch->dim.y;
	int newVdim = g_extrude->insertions * 2 + 1;

	//int offsetIdx = newUdim * newVdim;

	//newVdim += 2;

	int newLen = newUdim * newVdim;
	newPatch->dim.refValue().y = newVdim;

	newPatch->smls->addCpsForStrip(newPatch, newUdim, newVdim);

	M_Vec3f& newPoint = newPatch->point.refValue();
	M_Float& newWeight = newPatch->weight.refValue();

	newPoint.length = newLen;
	newWeight.length = newLen;

	for(int k = 0; k < newLen; k++){
		newWeight[k]  = 1;
	}



	updateNurbsCurveKnotMid1(newVdim, newPatch->vKnot.refValue());

	newPatch->pointsToDpoints();

	newPatch->processFields();
}

void Seamless::findIllegalStitchPoly(M_Int& deleteCtrlPolyList, int poly){//indices out of range
	int len = ctrlPolyPoint[poly].len;
	int beginPoly = poly;

	for(int k = len; --k >= 0;){
		if(ctrlPolyPoint[poly + k].i >= ctrlPolyPoint[poly + k].p->point.length){
			deleteCtrlPolyList += beginPoly;
			return;
		}
	}


	poly += len;
	int remCtrlPoly = poly;
	for(int k = len; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		for(int i = 0; i < midLen; i++){
			if(ctrlPolyPoint[poly + i].i >= ctrlPolyPoint[poly + i].p->point.length){
				deleteCtrlPolyList += beginPoly;
				return;
			}
		}
		if(midLen == 0){
			midLen++;
		}
		poly += midLen;
	}
		
	int centerLen = ctrlPolyPoint[poly].len;

	for(int i = 0; i < centerLen; i++){
		if(ctrlPolyPoint[poly + i].i >= ctrlPolyPoint[poly + i].p->point.length){
			deleteCtrlPolyList += beginPoly;
			return;
		}
	}

	if(centerLen == 0){
		centerLen++;
	}
	poly += centerLen;
}

void Seamless::findIllegalStitchPoly(M_Int& deleteCtrlPolyList){//indices out of range
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		findIllegalStitchPoly(deleteCtrlPolyList, poly);
		poly = getNextCtrlPoly(poly);
	}
}

/*
void Extrude::Insertions::operator=(const int& par){
	refValue() = par;

	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(lock->lock.length){
			lock->lock.refValue().length--;

			MultiSelect::reset();

			g_stw->genNodeList();
			getHair()->processFields();
			resetCPNodeList();
			//g_stw->commonPostInsertNodeStuff();




			extrudeRegion();
		}
	}
	else{
		NurbsPatch* cap = home.cap;
		NurbsPatch* l_cap = home.l_cap;
		NurbsPatch* shaft = home.shaft;
		NurbsPatch* mirror_shaft = home.shaft->mirrorPatch;
		NurbsPatch* extraCenterMirrorPatch = home.extraCenterMirrorPatch;
		Seamless* smls = home.smls;
		M_Vec3f& newPoint = shaft->point.refValue();

		//newPatch->mirror.refValue() = false;
		//newPatch->mirrorPatch->mirror.refValue() = false;

		int newUdim = shaft->dim.x;
		int newVdim = shaft->dim.y;
		int offset = newUdim * (newVdim - 1);

		for(int v = 0; v < newVdim - 1; v++){
			for(int u = 0; u < newUdim; u++){
				newPoint[v * newUdim + u]  = newPoint[offset + u];
			}
		}
		extendNewPatch(shaft);

		if(extraCenterMirrorPatch){
			M_Vec3f& newPoint2 = extraCenterMirrorPatch->point.refValue();
			int newUdim2 = extraCenterMirrorPatch->dim.x;
			int newVdim2 = extraCenterMirrorPatch->dim.y;
			int offset2 = newUdim * (newVdim - 1);
			for(int v = 0; v < newVdim2 - 1; v++){
				for(int u = 0; u < newUdim2; u++){
					newPoint2[v * newUdim2 + u]  = newPoint2[offset2 + u];
				}
			}
			extendNewPatch(extraCenterMirrorPatch);
		}

		if(mirror_shaft){
			extendNewPatch(mirror_shaft);
		}
		smls->populateMirrorNodes();

		g_extrude->doDistance();




		//smls->generateRawStitchPolys();
		//newPatch->mirror.refValue() = false;
		//newPatch->uJoinEnds.refValue() = true;
		//newPatch->processFields();
		//calcAllPolyMidPoints();
		//newPatch->uJoinEnds.refValue() = false;
		//newPatch->synchronizeAllStemCells();
		//updateBuild3dWindow();




		if(mirror_shaft){
			shaft->mirrorTwinPatch(mirror_shaft);
			mirror_shaft->synchronizeAllStemCells();
		}


		//smls->ctrlPolyPoint.length = 0;
//		smls->commonEndSitchPolys();
		smls->stitchAllPolys();
		shaft->synchronizeAllStemCells();

		shaft->processFields();
		shaft->index[0] = 0;

		if(extraCenterMirrorPatch){
			extraCenterMirrorPatch->synchronizeAllStemCells();

			extraCenterMirrorPatch->processFields();
			extraCenterMirrorPatch->index[0] = 0;
		}

		if(l_cap){
			cap->mirrorTwinPatch(l_cap);
		}


		smls->generateRawStitchPolys();
		//g_stw->commonPostInsertNodeStuff();












		//newPatch->smls->generateRawStitchPolys();

		smls->commonEndSitchPolys();
		smls->stitchAllPolys();
		shaft->synchronizeAllStemCells();
		bnspeBuildPolies(shaft);

		if(extraCenterMirrorPatch){
			extraCenterMirrorPatch->synchronizeAllStemCells();
			bnspeBuildPolies(extraCenterMirrorPatch);
		}

		if(mirror_shaft){
			bnspeBuildPolies(mirror_shaft);
		}

		//newPatch->houseKeep();
	}

}
*/

void Extrude::Insertions::operator=(const int& par){
	refValue() = par;

	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(lock->lock.length){
			lock->lock.refValue().length--;

			MultiSelect::reset();

			g_stw->genNodeList();
			getHair()->processFields();
			resetCPNodeList();
			//g_stw->commonPostInsertNodeStuff();




			extrudeRegion();
		}
	}
	else{
		bool _cap = home._cap;
		NurbsPatch* r_cap = home.r_cap;
		NurbsPatch* l_cap = home.l_cap;
		NurbsPatch* r_shaft = home.r_shaft;
		NurbsPatch* l_shaft = home.r_shaft->mirrorPatch;
		NurbsPatch* extraCenterMirrorPatch = home.extraCenterMirrorPatch;
		Seamless* smls = home.smls;
		//M_Vec3f& newPoint = r_shaft->point.refValue();

		smls->cp.refValue().length = home.markCp;

		extendNewPatch(r_shaft);

		if(l_shaft){
			extendNewPatch(l_shaft);
		}

		if(extraCenterMirrorPatch){
			extendNewPatch(extraCenterMirrorPatch);
			extraCenterMirrorPatch->index[0] = 0;
		}

		smls->populateMirrorNodes();
		smls->generateRawStitchPolys();

		g_extrude->doDistance(1);
		smls->populateMirrorNodes();

		r_shaft->index[0] = 0;
		if(l_shaft){
			r_shaft->mirrorTwinPatch(l_shaft);
			if(_cap){
				r_cap->mirrorTwinPatch(l_cap);
			}
			l_shaft->index[0] = 0;
		}

		smls->commonEndSitchPolys();

		smls->stitchAllPolys();

		r_shaft->patchToCentralCPs();
		if(l_shaft){
			l_shaft->patchToCentralCPs();
		}

		home.tidyUpCells();

		smls->doBuild();
		houseKeepVisualClue();

		return;
	}
}




void joinShaftPatchIndices(NurbsPatch* shaft, M_Int& storeCpIndex, int uDim){
	M_Int& shaftCpIndex = shaft->cpIndex.refValue();
	shaftCpIndex[0] = storeCpIndex[0];
	shaftCpIndex[1] = storeCpIndex[1];

	shaftCpIndex[2] = storeCpIndex[2];
	shaftCpIndex[3] = storeCpIndex[uDim + 2];

	shaftCpIndex[4] = storeCpIndex[uDim * 2 + 2];
	shaftCpIndex[5] = storeCpIndex[uDim * 2 + 1];

	shaftCpIndex[6] = storeCpIndex[uDim * 2 + 0];
	shaftCpIndex[7] = storeCpIndex[uDim];

	shaftCpIndex[8] = storeCpIndex[0];
}

void Extrude::doDistance(int fromRow){
	NurbsPatch* l_shaft = r_shaft->mirrorPatch;

	resetTopShaftPoints();

	int uDim = r_cap->dim.x;

	M_Vec3f& newPoint = r_shaft->point.refValue();

	int newUdim = r_shaft->dim.x;
	int newVdim = r_shaft->dim.y;
	int offsetIdx = newUdim * (newVdim - 1);
	
	if(extraCenterMirrorPatch){

		r_shaft->joinExtraCenterShaftPoints(extraCenterMirrorPatch);

		M_Vec3f& newPoint2 = extraCenterMirrorPatch->point.refValue();
		M_Int& shaftCpIndex = r_shaft->cpIndex.refValue();
		M_Int& newCpIndex2 = extraCenterMirrorPatch->cpIndex.refValue();
		int newUdim2 = extraCenterMirrorPatch->dim.x;
		int newVdim2 = extraCenterMirrorPatch->dim.y;
		int offsetIdx2 = newUdim2 * (newVdim2 - 1);


		shaftCpIndex[0] = r_storeCpIndex[0];
		shaftCpIndex[1] = r_storeCpIndex[1];

		shaftCpIndex[2] = r_storeCpIndex[2];
		newCpIndex2[0] = r_storeCpIndex[2];

		newCpIndex2[1] = r_storeCpIndex[uDim + 2];

		newCpIndex2[2] = r_storeCpIndex[uDim * 2 + 2];
		newCpIndex2[3] = r_storeCpIndex[uDim * 2 + 1];

		newCpIndex2[4] = r_storeCpIndex[uDim * 2 + 0];
		newCpIndex2[5] = r_storeCpIndex[uDim];

		newCpIndex2[6] = r_storeCpIndex[0];


		for(int v = fromRow; v < newVdim - 1; v++){
			int d = v * newUdim;
			int d2 = v * newUdim2;
			float ratio = v * (1.0 / static_cast<float>(newVdim - 1));

			newPoint[d + 0] = storePointList[0] + (newPoint[offsetIdx + 0] - storePointList[0]) * ratio;
			newPoint[d + 1] = storePointList[1] + (newPoint[offsetIdx + 1] - storePointList[1]) * ratio;

			newPoint[d + 2] = storePointList[2] + (newPoint[offsetIdx + 2] - storePointList[2]) * ratio;
			newPoint2[d2 + 0] = storePointList[2] + (newPoint2[offsetIdx2 + 0] - storePointList[2]) * ratio;

			newPoint2[d2 + 1] = storePointList[uDim + 2] + (newPoint2[offsetIdx2 + 1] - storePointList[uDim + 2]) * ratio;

			newPoint2[d2 + 2] = storePointList[uDim * 2 + 2] + (newPoint2[offsetIdx2 + 2] - storePointList[uDim * 2 + 2]) * ratio;
			newPoint2[d2 + 3] = storePointList[uDim * 2 + 1] + (newPoint2[offsetIdx2 + 3] - storePointList[uDim * 2 + 1]) * ratio;

			newPoint2[d2 + 4] = storePointList[uDim * 2 + 0] + (newPoint2[offsetIdx2 + 4] - storePointList[uDim * 2 + 0]) * ratio;
			newPoint2[d2 + 5] = storePointList[uDim] + (newPoint2[offsetIdx2 + 5] - storePointList[uDim]) * ratio;

			newPoint2[d2 + 6] = storePointList[0] + (newPoint2[offsetIdx2 + 6] - storePointList[0]) * ratio;
		}

	}
	else{
		r_shaft->joinEastToWest();
		if(l_shaft){
			l_shaft->joinEastToWest();
		}


		joinShaftPatchIndices(r_shaft, r_storeCpIndex, uDim);
		if(l_shaft){
			joinShaftPatchIndices(l_shaft, l_storeCpIndex, uDim);
		}

		for(int v = fromRow; v < newVdim - 1; v++){
			int d = v * newUdim;
			float ratio = v * (1.0 / static_cast<float>(newVdim - 1));

			newPoint[d + 0] = storePointList[0] + (newPoint[offsetIdx + 0] - storePointList[0]) * ratio;
			newPoint[d + 1] = storePointList[1] + (newPoint[offsetIdx + 1] - storePointList[1]) * ratio;

			newPoint[d + 2] = storePointList[2] + (newPoint[offsetIdx + 2] - storePointList[2]) * ratio;
			newPoint[d + 3] = storePointList[uDim + 2] + (newPoint[offsetIdx + 3] - storePointList[uDim + 2]) * ratio;

			newPoint[d + 4] = storePointList[uDim * 2 + 2] + (newPoint[offsetIdx + 4] - storePointList[uDim * 2 + 2]) * ratio;
			newPoint[d + 5] = storePointList[uDim * 2 + 1] + (newPoint[offsetIdx + 5] - storePointList[uDim * 2 + 1]) * ratio;

			newPoint[d + 6] = storePointList[uDim * 2 + 0] + (newPoint[offsetIdx + 6] - storePointList[uDim * 2 + 0]) * ratio;
			newPoint[d + 7] = storePointList[uDim] + (newPoint[offsetIdx + 7] - storePointList[uDim]) * ratio;

			newPoint[d + 8] = storePointList[0] + (newPoint[offsetIdx + 8] - storePointList[0]) * ratio;
		}
	}
	if(spiderWebCenter){
		closeEndPointsForSpiderWeb();
	}
	if(_cap){
		r_cap->patchToCentralCPs();
		if(l_cap){
			r_cap->mirrorTwinPatch(l_cap);
			l_cap->patchToCentralCPs();
		}
	}
	if(extraCenterMirrorPatch){
		extraCenterMirrorPatch->patchToCentralCPs();
	}
}

void Extrude::doDistanceA(){
	doDistance();

	if(NurbsPatch* mirror_shaft = r_shaft->mirrorPatch){
		r_shaft->mirrorTwinPatch(mirror_shaft);
	}
	

	smls->commonEndSitchPolys();
	r_shaft->patchToCentralCPs();
	if(r_shaft->mirrorPatch){
		r_shaft->mirrorPatch->patchToCentralCPs();
	}

	smls->doBuild();
	smls->ctrf = true;
	houseKeepVisualClue();
}

void Extrude::Distance::operator=(const float& par){
	refValue() = par;
	if(dynamic_cast<Lock*>(home.r_cap.p)){
		home.processFields();
	}
	else{
		home.doDistanceA();
	}
}

void setLockSize(Lock* lock, Vec2f size, const Rotation& tilt, float angle, float radius, float height, Vec3f pivotPoint, Rotation orientation){
	M_Vec3f& point = lock->point.refValue();
	int uDim = lock->dim.x;
	int vDim = lock->dim.y;


	for(int v = 0; v < vDim; v++){
		for(int u = 0; u < uDim; u++){
			float fu = static_cast<float>(size.x) / (uDim - 1) * u - size.x * .5;
			float fv = static_cast<float>(size.y) / (vDim - 1) * v - size.y * .5;
			Vec3f pnt = tilt * Vec3f(fu, 0, -fv);
			pnt.x += radius;
			pnt.y += height;
			pnt = Rotation(0, 1, 0, angle) * pnt;
			pnt = orientation * pnt;
			point[v * uDim + u] = pnt + pivotPoint;
		}
	}
	lock->processFields();
}

void RingletGenerator::processFields(){

	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		Vec3f pivotPoint = lock->generate(Vec2f(.5, .5));
		Vec3f normal;


		calcNormal(
			lock->generate(Vec2f(.5 - .01, .5)),
			pivotPoint,
			lock->generate(Vec2f(.5, .5 + .01)),
			normal
		);
		normal = -normal.normalize();

		Rotation orientation = Rotation(Vec3f(0, 1, 0), normal);


		if(lock->lock.length){
			int llidx = dynamic_cast<Lock*>(lock->lock.refValue()[0].p)->llidx;
			Hair* hair = getHair();
			M_<Lock*>& lockList = hair->lockList;

			float e = PI * 2 / insertions;
			float h = distance / insertions;

			Vec3f pntA = Vec3f(radius, 0, 0);
			Vec3f pntB = Vec3f(radius, h, 0);
			pntA = Rotation(0, -1, 0, e * .5) * pntA;
			pntB = Rotation(0, 1, 0, e * .5) * pntB;
			pntB = pntB - pntA;
			Rotation tilt = Rotation(Vec3f(0, 1, 0), pntB.normalize());

			for(int k = 0; k < insertions * revolutions && lock->lock.length; k++){
				lock = lockList[llidx + k];
				setLockSize(lock, _size, tilt, e * k, radius, h * (k + 1), pivotPoint, orientation);
			}
		}
	}
}


void Thinner::Ok::operator=(bool const&){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){//should not need to check if selected node is Lock
		M_Vec2f& hole = lock->hole.refValue();
		M_Int& scpi = lock->scpi.refValue();
		M_<Strand*>& scpiLathe = lock->scpiLathe;

		for(int i = hole.length, c = home.number_of_strands; --c>=0;){
			--i;
			Strand* strand = scpiLathe[i];
			int vDim = scpi[i] + 1;
			strand->index[0] = 0;
			strand->dim.refValue().y = vDim;
			strand->point.refValue().length = vDim;
			strand->dpoint.refValue().length = vDim;
			strand->weight.refValue().length = vDim;
			updateNurbsCurveKnotMid0(vDim, strand->vKnot.refValue());
		}
		repeatSeperatorMap();

		updateBuild3dWindow();
	}
}

void thinner(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		g_stw->cp->setupNodeWindow(g_thinner);
		Window::activeWindow = g_stw->cp.p;
	}
}


void insideOutRegionPoly(M_Int& region, int s, int numOfCorners){
	if(s < region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}

	int s2 = s + numOfCorners - 1;

	for(int i = 0; i < numOfCorners >> 1; i++){
		int g = region[s];
		region[s] = region[s2];
		region[s2] = g;
		s++;
		s2--;
	}
}

void mirrorLock(Lock* lock){
	M_Vec3f& point = lock->point;
	for(int i = point.length; --i>=0;){
		point[i].x = -point[i].x;
	}
	lock->setInsideOut2();
	lock->flipPolarity();

	//int numOfCorners = 4;
	//int s = 0;
	//int polyCntr = 0;
	//M_Int& region = lock->region.refValue();

	//while(s < region.length){

	//	if(first == false){
	//		insideOutRegionPoly(region, s, numOfCorners);
	//		//M_Vec2f& border = lock->border;
	//		//for(int i = border.length; --i>=0;){
	//		//	float b = -(border[i].x - .5) + .5;
	//		//	if(b < 0){
	//		//		b = 0;
	//		//	}
	//		//	if(b > 1){
	//		//		b = 1;
	//		//	}
	//		//	border[i].x = b;
	//		//}
	//	}

	//	nextRegion(numOfCorners, s, region);

	//	insideOutRegionPoly(region, s, numOfCorners);

	//	nextRegion(numOfCorners, s, region);
	//}


//	first = false;

	lock->processFields();

	M_Node& m_lock = lock->lock.refValue();
	for(int k = m_lock.length; --k>=0;){
		Lock* childLock = dynamic_cast<Lock*>(m_lock[k].p);
		mirrorLock(childLock);
	}
}

void mirrorLock(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		mirrorLock(lock);
	}
	updateBuild3dWindow();
}


void rotateRegionPoly(M_Int& region, int s, int numOfCorners){
	if(s < region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}

	int store = region[s];

	for(int i = 0; i < numOfCorners - 1; i++){
		region[s] = region[s + 1];
		s++;
	}
	region[s] = store;
}

void Lock::rotateSelectedRegions(){
	int numOfCorners = 4;
	int s = 0;
	int remS = s;
	if(region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}
	int polyCntr = 0;

	for(int k = 0; ; k++){

		int s2 = s + numOfCorners;
		if(region[s2] < 0){
			s2++;
		}
		if(regionSelected[polyCntr++]){
			rotateRegionPoly(region, s, numOfCorners);
		}
		if(regionSelected[polyCntr++]){
			rotateRegionPoly(region, s2, numOfCorners);
		}
		s = s2 + numOfCorners;
		if(s < region.length){
			if(region[s] < 0){
				numOfCorners = -region[s++];
			}
		}
		else{
			break;
		}

	}
}

void rotateRegion(){
	if(Lock* lock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		lock->rotateSelectedRegions();

		if(Lock* parentLock = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
			lock->copyMapRepete(parentLock);
		}
		updateBuild3dWindow();
	}
	
}

void mergeLocks(){
	if(Lock* selLock = dynamic_cast<Lock*>(g_selNode.getRaw())){
		if(Lock* copyLock = dynamic_cast<Lock*>(g_copyNode.getRaw())){
			int borderOffset = selLock->border.length;
			int copyRegionLen = copyLock->region.length;
			selLock->border.refValue() += copyLock->border;
			bool srcFlag = false;

			for(int s = 0; s < copyRegionLen; s++){
				int i = copyLock->region[s];
				if(i >= 0){
					if(srcFlag == false){
						i += borderOffset;
					}
				}
				else{
					srcFlag = !srcFlag;
				}
				selLock->region.refValue() += i;
			}
			//selLock->unify.refValue() += copyLock;
			//g_stw->genNodeList();
			g_stw->paint();
			houseKeepVisualClue();
		}
	}
}

int Lock::getClosestBorderPoint(int compareBorderIdx){
	float shortestDistance = 2;
	int shortestIdx = 0;
	Vec2f compareBorder = border[compareBorderIdx];
	int i = border.length;

	while(--i >= 0){
		if(compareBorderIdx == i){
			continue;
		}
		float distance = (border[i] - compareBorder).length();
		if(distance <  shortestDistance){
			shortestDistance = distance;
			shortestIdx = i;
		}
	}
	return shortestIdx;
}

void Lock::joinBorderPoints(Lock* src, int closestBorderIdx){

	if(region.length){
		int numOfCorners = 4;
		int s = 0;
		int remS = s;
		if(region.length && region[s] < 0){
			numOfCorners = -region[s++];
		}

		for(int k = 0; ; k++){

			int s2 = s + numOfCorners;
			if(region[s2] < 0){
				s2++;
			}
			for(int h = 0; h < numOfCorners; h++){
				if(src == Lock::selBorderLock && region[s + h] == selBorder){
					region.refValue()[s + h] = closestBorderIdx;
				}
				if(this == Lock::selBorderLock && region[s2 + h] == selBorder){
					region.refValue()[s2 + h] = closestBorderIdx;
				}
			}
			s = s2 + numOfCorners;
			if(s < region.length){
				if(region[s] < 0){
					numOfCorners = -region[s++];
				}
			}
			else{
				break;
			}


			if(k < unify.length){
				src = dynamic_cast<Lock*>(unify[k].p);
			}
		}
	}

	M_Node& m_lock = this->lock.refValue();
	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->joinBorderPoints(this, closestBorderIdx);
	}
}

int Lock::getBaseRegionOffset(Lock* src, int blockRegion, Lock*& returnSrc, int& numOfCorners){
	numOfCorners = 4;
	int s = 0;
	int remS = s;
	if(region.length && region[s] < 0){
		numOfCorners = -region[s++];
	}
	int polyCntr = 0;

	for(int k = 0; ; k++){

		int s2 = s + numOfCorners;
		if(region[s2] < 0){
			s2++;
		}
		if(s != blockRegion && regionSelected[polyCntr]){
			returnSrc = src;
			return s;
		}
		polyCntr++;
		s = s2 + numOfCorners;
		if(s < region.length){
			if(region[s] < 0){
				numOfCorners = -region[s++];
			}
		}
		else{
			break;
		}


		if(k < unify.length){
			src = dynamic_cast<Lock*>(unify[k].p);
		}
	}
	return -1;
}

bool Lock::isPointInRegion(int s, int pointIdx, int numOfCorners, int& h){
	for(h = 0; h < numOfCorners; h++){
		if(region[s++] == pointIdx){
			return true;
		}
	}
	return false;
}


void Lock::mergeRegions(Lock* src){
	M_Int& region = this->region.refValue();
	Lock* returnSrcA = 0;
	Lock* returnSrcB = 0;
	int numOfCorners = 0;

	int sa = getBaseRegionOffset(src, -1, returnSrcA, numOfCorners);
	int sb = getBaseRegionOffset(src, sa, returnSrcB, numOfCorners);
	if(returnSrcA != returnSrcB){
		return;
	}


	M_Int newPoly2;
	int sa2 = sa + numOfCorners;
	if(region[sa2] < 0){
		sa2++;
	}
	int sb2 = sb + numOfCorners;
	if(region[sb2] < 0){
		sb2++;
	}
	int hb = 0;

	for(int h = 0; h < numOfCorners; h++){//1st phase
		int pointIdx = region[sa + h];
		if(false == isPointInRegion(sb, pointIdx, numOfCorners, hb)){
			while(true){//2nd phase
				if(++h >= numOfCorners){
					h = 0;
				}
				pointIdx = region[sa + h];
				if(isPointInRegion(sb, pointIdx, numOfCorners, hb)){
					h = hb;
					int mark = region.length;
					region += 0;
					while(true){//3rd phase (copy points from poly b)
						if(++h >= numOfCorners){
							h = 0;
						}
						pointIdx = region[sb + h];
						region += pointIdx;
						newPoly2 += region[sb2 + h];
						if(isPointInRegion(sa, pointIdx, numOfCorners, hb)){
							h = hb;
							while(true){//4th phase (copy points from poly a)
								if(++h >= numOfCorners){
									h = 0;
								}
								pointIdx = region[sa + h];
								region += pointIdx;
								newPoly2 += region[sa2 + h];
								if(isPointInRegion(sb, pointIdx, numOfCorners, hb)){
									int newNumOfCorners = newPoly2.length;
									region[mark] = -newNumOfCorners;
									region += -newNumOfCorners;
									region += newPoly2;
									houseKeepVisualClue();
									regionSelected.length += 2;
									deleteSelectedRegions();
									return;
								}
							}
						}
					}
				}
			}
		}
	}
}

void Lock::fixBorderIndexForDelete(Lock* src){
	M_Int& region = this->region.refValue();

	if(region.length){
		int numOfCorners = 4;
		int s = 0;
		if(region.length && region[s] < 0){
			numOfCorners = -region[s++];
		}
		for(int k = 0; ; k++){

			int s2 = s + numOfCorners;
			if(region[s2] < 0){
				s2++;
			}

			for(int h = 0; h < numOfCorners; h++){
				if(src == Lock::selBorderLock){
					if(region[s + h] > Lock::selBorder){
						region[s + h]--;
					}
				}
				if(this == Lock::selBorderLock){
					if(region[s2 + h] > Lock::selBorder){
						region[s2 + h]--;
					}
				}
			}
			s = s2 + numOfCorners;
			if(s < region.length){
				if(region[s] < 0){
					numOfCorners = -region[s++];
				}
			}
			else{
				break;
			}


			if(k < unify.length){
				src = dynamic_cast<Lock*>(unify[k].p);
			}
		}
	}
	M_Node& m_lock = this->lock.refValue();
	for(int k = 0; k < m_lock.length; k++){
		dynamic_cast<Lock*>(m_lock[k].p)->fixBorderIndexForDelete(this);
	}
}

void Lock::breakBorder(){
	breakBorderMode = false;

	

	if(Lock* parentLock = dynamic_cast<Lock*>(getParentNode(g_stw->selLine))){
		int newBorderIdx = selBorderLock->border.length;
		selBorderLock->border.refValue() += selBorderLock->border[selBorder];

		joinBorderPoints(parentLock, newBorderIdx);

		selBorder = newBorderIdx;
	}
}

void Lock::deleteCorner(Lock* src){
	M_Int& region = this->region.refValue();

	if(region.length){
		int numOfCorners = 4;
		int s = 0;
		if(region.length && region[s] < 0){
			numOfCorners = -region[s++];
		}

		for(int k = 0; ; k++){

			int s2 = s + numOfCorners;
			if(region[s2] < 0){
				s2++;
			}
			for(int h = 0; h < numOfCorners; h++){
				if(	src == Lock::selBorderLock && region[s + h] == selBorder ||
					this == Lock::selBorderLock && region[s2 + h] == selBorder){

					region.deleteElement(s + h);
					s2--;
					region.deleteElement(s2 + h);

					numOfCorners--;

					region[s - 1] = -numOfCorners;
					region[s2 - 1]= -numOfCorners;
				}
			}
			s = s2 + numOfCorners;
			if(s < region.length){
				if(region[s] < 0){
					numOfCorners = -region[s++];
				}
			}
			else{
				break;
			}


			if(k < unify.length){
				src = dynamic_cast<Lock*>(unify[k].p);
			}
		}
	}
}


void cutPatches(){
	resetTemporaryModes();
	g_insertPatch.cutMode = true;
	//if(g_showPatchBorderMode == false){
	//	toggleShowPatchBordersMode();
	//}

	g_insertPatch.imousePhase = InsertPatch::STANDBY;
	changeModellingCursor(g_knifeCursor);
	MultiSelect::reset();
	houseKeepVisualClue();
}

void insertControlPoint(){
	resetTemporaryModes();
	g_insertPatch.cutMode = false;
	g_insertPatch.imousePhase = InsertPatch::STANDBY;
	changeModellingCursor(g_insertLineCursor);
	MultiSelect::reset();
	houseKeepVisualClue();
}

void deleteControlPoint(){
	resetTemporaryModes();

	g_deleteControlLine.selectRing = false;

	g_deleteControlLine.mousePhase = DeleteControlLine::STANDBY;
	g_deleteControlLine.smls = getSeamless2();
	changeModellingCursor(g_deleteLineCursor);
	MultiSelect::reset();
	houseKeepVisualClue();
}

void selectRing(){
	resetTemporaryModes();

	g_deleteControlLine.selectRing = true;
	g_deleteControlLine.mousePhase = DeleteControlLine::STANDBY;
	g_deleteControlLine.smls = getSeamless();

	changeModellingCursor(g_ringSelectCursor);
}

Art* getGroupPart(NurbsPatch* patch);

void NurbsPatch::synchronizeCellsAndBuild(){
	M_Node& m_stem = this->stem.refValue();
	bool stemless = false;

	if(m_stem.length == 0){
		stemless = true;
		g_stem->sync.refValue() = sync;
		g_stem->insideOut.refValue() = insideOut;
		g_stem->part.refValue().length = 0;

		if(Art* groupPart = getGroupPart(this)){
			g_stem->part.refValue() += groupPart;
		}
		else if(smls->skeleton.length){
			g_stem->part.refValue() += dynamic_cast<Art*>(smls->skeleton[0].p);
		}

		m_stem += g_stem.p;
	}

	synchronizeAllStemCells();
	bnspeBuildPolies();

	if(stemless){
		m_stem.length = 0;
	}
}

void NurbsPatch::synchronizeCellsAndBuildSync(){//exactly same as above except we call bnspeBuildSyncPolies();
	M_Node& m_stem = this->stem.refValue();
	bool stemless = false;

	if(m_stem.length == 0){
		stemless = true;
		g_stem->sync.refValue() = sync;
		g_stem->insideOut.refValue() = insideOut;
		g_stem->part.refValue().length = 0;

		if(Art* groupPart = getGroupPart(this)){
			g_stem->part.refValue() += groupPart;
		}
		else if(smls->skeleton.length){
			g_stem->part.refValue() += dynamic_cast<Art*>(smls->skeleton[0].p);
		}

		m_stem += g_stem.p;
	}

	synchronizeAllStemCells();
	bnspeBuildSyncPolies();

	if(stemless){
		m_stem.length = 0;
	}
}

void deletePoliesOwnedByPatch(){
	if(Strand* strand = dynamic_cast<Strand*>(g_selNode.getRaw())){
		Seamless* smls = strand->smls;
		smls->fastDeletePoliesOwnedByPatch(strand->geometry);
		smls->ctrf = true;
	}
	else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		smls->deletePoliesOwnedByPatch(patch);
		smls->ctrf = true;
	}
}

void rectangleSelect(){
	g_rectangleSelectPointReady = true;
	changeModellingCursor(g_rectangleSelectPointCursor);
}

void rectangleUnselect(){
	g_rectangleUnselectPointReady = true;
	changeModellingCursor(g_rectangleUnselectCursor);
}


void rectangleSelectStitchPoly(){
	g_rectangleSelectPolyReady = true;
	changeModellingCursor(g_rectangleSelectPolyCursor);
}



void InsertComplete::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	g_insertPatch.doLastPhase();
	Window::closeWindow(g_stw->cp.p);
}

void CutComplete::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	g_insertPatch.doLastPhase();
	Window::closeWindow(g_stw->cp.p);
}

void CutComplete::Continue::operator=(bool const &par){
#ifdef PROG_ROB
	dLog("new ToggleField(\"continue\"),");
#endif
	refValue() = par;
	houseKeepVisualClue();
}

void AddLathe::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	g_addLathe->mousePhase = AddLathe::NEUTRAL;
	Window::closeWindow(g_stw->cp.p);
}

void AddNurbsLine::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	//setHomeMode();
	g_addNurbsLine->mousePhase = AddNurbsLine::NEUTRAL;
	Window::closeWindow(g_stw->cp.p);
	houseKeepVisualClue();
}

void DrawNurbsLine::Loop::operator=(bool const &){
	if(home.mousePhase == READY_TO_DRAG){
#ifdef PROG_ROB
	dLog("new ToggleField(\"loop\"),");
#endif

		NurbsLine* line = home.line;
		Seamless* smls = line->smls;

		//line->completeLoop(false);

		line->uJoinEnds.refValue() = true;


		if(line->index[0] == 0 || line->index[0] == line->point.length - 1){
			line->NurbsPatch::index[0] = line->index[0] = 1;

			MultiSelect::pointList = ControlPoint(line, line->index[0]);
		}

		line->processFields();


		smls->populateMirrorNodes();
		smls->genVisibleMask();
		smls->doBuild();
		houseKeepVisualClue();
	}
}

void DrawNurbsLine::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	home.mousePhase = NEUTRAL;
	//2;
	//g_drawNurbsLine->smls->doBuild();
	Window::closeWindow(g_stw->cp.p);
	//setHomeMode();
	houseKeepVisualClue();
	//g_visualClue = nullPtr;
	
	//setHomeMode();
}

void AddBone::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	g_addBone->mousePhase = AddBone::NEUTRAL;
	Window::closeWindow(g_stw->cp.p);
	changeModellingCursor(g_crossCursor);
}

void Extrude::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	Window::closeWindow(g_stw->cp.p);
}

void saveTextureFile(){
	if(TextureEffect* textureEffect = dynamic_cast<TextureEffect*>(g_selNode.getRaw())){
		textureEffect->copyPixelsToStore();
		//textureEffect->savePng(unsigned int* pBits, M_<Pix24>& pixels, unsigned int width, unsigned int height);
		createPNG24(textureEffect->url.refValue().c_str(), g_brush->pitch / 4, g_brush->pitch / 4, g_brush->storePixel.getArray());
	}
}

void exportMovie(){
	g_stw->cp->setupNodeWindow(g_exportMovie);
	Window::activeWindow = g_stw->cp.p;
}

void exportVrmlX3d(){
	g_stw->cp->setupNodeWindow(g_exportVrmlX3d);
	Window::activeWindow = g_stw->cp.p;
}

extern ShapePtr g_gridShape;

void openGridControl(){
	g_stw->cp->setupNodeWindow(g_grid);
	Window::activeWindow = g_stw->cp.p;
	g_grid->houseKeep();
	//g_gridShape->render = 0;
}

void snapRotationLever(){
#ifdef PROG_ROB
	dLog("new VirtKeyDown(VK_SPACE),");
	dLog("new VirtKeyUp(VK_SPACE),");
#endif
	g_transformCP->quantizeRotationLever();
}

void setSingleActiveMode(){
	g_scene->mode.refValue() = g_scene->mode ^ SINGLE_ACTIVE_MODE;

	if(g_scene->mode & SINGLE_ACTIVE_MODE){
		MultiSelect::selectedPoint = -1;
	}
	updateIconPointers();
	houseKeepVisualClue();
	g_tb2->paint();
}

void Grid::Scale::operator=(const float& par){
	refValue() = par;
	home.houseKeep();
}

void Grid::Squares::operator=(const int& par){
	refValue() = par;
	home.houseKeep();
}

void Grid::SubSquares::operator=(const int& par){
	refValue() = par;
	home.houseKeep();
}

void Grid::XyPlane::operator=(const bool& par){
	home.xyPlane.refValue() = true;
	home.xzPlane.refValue() = false;
	home.yzPlane.refValue() = false;
	home.houseKeep();
}

void Grid::XzPlane::operator=(const bool& par){
	home.xyPlane.refValue() = false;
	home.xzPlane.refValue() = true;
	home.yzPlane.refValue() = false;
	home.houseKeep();
}

void Grid::YzPlane::operator=(const bool& par){
	home.xyPlane.refValue() = false;
	home.xzPlane.refValue() = false;
	home.yzPlane.refValue() = true;
	home.houseKeep();
}


void duplicate(){
	//disabledMessage();
	//return;
	g_stw->cp->setupNodeWindow(g_duplicate);
	Window::activeWindow = g_stw->cp.p;
}

void makeListOfSelectedPatches(Seamless* smls, M_<NurbsPatchPtr>& m_selPatchList){
	m_selPatchList.length = 0;
	for(int k = 0; k < MultiSelect::pointList.length; k++){
		NurbsPatch* patch = MultiSelect::pointList[k].p;
		addIfNotInList(patch, m_selPatchList);
		if(patch->mirrorPatch){
			addIfNotInList(patch->mirrorPatch, m_selPatchList);
		}
	}
}

void addNewPointsAndMakeRedirectTable(Seamless* smls, const M_<NurbsPatchPtr>& m_selPatchList, M_Int& redirect){
	M_Vec3f& cp = smls->cp;
	M_Float& cw = smls->cw;
	redirect.length = 0;
	redirect.length = smls->cp.length;

	for(int j = m_selPatchList.length; --j>=0;){
		const NurbsPatch* patch = m_selPatchList[j];
		const M_Int& cpIndex = patch->cpIndex;

		for(int i = cpIndex.length; --i>=0;){
			redirect[cpIndex[i]] = 1;
		}
	}

	{
		int mark = cp.length;
		for(int r = 0, ix = 0; r < redirect.length; r++){
			if(redirect[r]){
				cp += cp[r];
				cw += cw[r];
				redirect[r] = mark + ix++;
			}
		}
	}
}

void Duplicate::Run::operator=(const bool& par){
	if(Seamless* smls = getSeamless2()){

		M_Int redirect;

		M_<NurbsPatchPtr> m_selPatchList;

		makeListOfSelectedPatches(smls, m_selPatchList);
		


		Rotation rot = home.rotation;
		Vec3f translation = home.translation;
		Vec3f scale = home.scale;

		NurbsPatch* previousNewPatch = 0;

		for(int k = 0; k < home.iterations; k++){
			
			addNewPointsAndMakeRedirectTable(smls, m_selPatchList, redirect);
			int twin = 0;

			for(int j = 0; j < m_selPatchList.length; j++){
				NurbsPatch* patch = m_selPatchList[j];


				NurbsPatch* newPatch = patch->createCopyOf(redirect);
				newPatch->name = patch->name;

				
				if(patch->mirrorPatch){
					twin++;

				}
				else{
					twin = 0;
				}

				if(Strand* strand = dynamic_cast<Strand*>(newPatch)){
					int selLine = g_stw->selLine;
					Hair* hair = dynamic_cast<Hair*>(getParentNode(selLine));
					hair->strand.refValue() += strand;
					hair->processFields();

					Art* part = dynamic_cast<Art*>(hair->part.p);

					part->deleteGeometry();
					part->smls->convertToRenderFormat();
					makePointerEqualIndexLocation();


					smls->ctrf = true;
					hair->doBuild();

				}
				else{
					smls->build.refValue() += newPatch;
				}

				Matrix matrix = scaleMatrix(scale) * rotationMatrix(rot) * translationMatrix(translation);
				M_Vec3f& point = newPatch->point.refValue();
				for(int i = point.length; --i>=0;){
					Vec3f pnt = matrix * point[i];
					newPatch->setPointAndCp(i, pnt);
				}

				
				newPatch->processFields();
				if(newPatch->isCtrlPolyRef){
					newPatch->fillEdgesWithCtrlPolys();
				}
				newPatch->synchronizeCellsAndBuild();
				/////

				if(twin == 2){
					previousNewPatch->makeExistingNameUniqueForMirror(patch);
					twin = 0;
				}
				else if(newPatch->name.length){
					makeExistingNameUnique(newPatch);
				}

				g_stw->genNodeList();

				previousNewPatch = newPatch;
			}

			scale = scale * home.scale;
			rot = home.rotation * rot;
			translation += home.translation;
		}


		smls->populateMirrorNodes();
		smls->stitchAllPolys();
		g_stw->commonPostInsertNodeStuff();
		houseKeepVisualClue();
	}
}

bool checkForRightName(Node* node){
	if(node->name.length < 2){
		return false;
	}
	if(node->name[0] == 'r'){
		if(node->name[1] == '_'){
			return true;
		}
		else if(node->name[1] >= 'A' && node->name[1] <= 'Z')
			return true;
	}
	return false;
}

bool checkForLeftName(Node* node){
	if(node->name.length < 2){
		return false;
	}
	if(node->name[0] == 'l'){
		if(node->name[1] == '_'){
			return true;
		}
		else if(node->name[1] >= 'A' && node->name[1] <= 'Z')
			return true;
	}
	return false;
}

void duplicateMirror(){
	//NurbsPatch* temp = 0;
//	disabledMessage();
	if(Seamless* smls = getSeamless2()){
		M_Int redirect;

		redirect.length = smls->cp.length;
		M_<NurbsPatchPtr> m_selPatchList;

		makeListOfSelectedPatches(smls, m_selPatchList);
		addNewPointsAndMakeRedirectTable(smls, m_selPatchList, redirect);

		for(int j = 0; j < m_selPatchList.length; j++){
			NurbsPatch* r_patch = m_selPatchList[j];

			NurbsPatch* l_patch = r_patch->createCopyOf(redirect);
			r_patch->smls = smls;

			smls->build.refValue() += l_patch;
			l_patch->mirror.refValue() = true;
			r_patch->mirror.refValue() = true;

			l_patch->sync.refValue() = true; //we should not need to sync anything
			r_patch->sync.refValue() = true;

			if(r_patch->name.length == 0){
				r_patch->name = "patch";
			}
			if(false == checkForRightName(r_patch) && false == checkForLeftName(r_patch)){
				r_patch->name = "r_" + r_patch->name;
			}
			r_patch->makeExistingNameUniqueForMirror(l_patch);

			l_patch->insideOut.refValue() = !r_patch->insideOut;
			if(r_patch->stem.length && l_patch->stem.length){
				Stem* r_stem = dynamic_cast<Stem*>(r_patch->stem[0].p);
				Stem* l_stem = dynamic_cast<Stem*>(l_patch->stem[0].p);
				l_stem->insideOut.refValue() = !r_stem->insideOut;
			}

			r_patch->mirrorTwinPatch(l_patch);

			l_patch->processFields();
			if(l_patch->isCtrlPolyRef){
				l_patch->fillEdgesWithCtrlPolys();
			}
			l_patch->synchronizeCellsAndBuild();

			g_stw->genNodeList();

			//r_patch->processFields();
			//r_patch->isCtrlPolyRef = true;
			//temp = r_patch;
		}
		smls->populateMirrorNodes();
		smls->stitchAllPolys();
		g_stw->commonPostInsertNodeStuff();
		houseKeepVisualClue();


		//g_stw->paint();

	}
}

void genDynamicTransforms(Seamless* smls);

void Scene::setZoom(const float& par){
	scale = Vec3f(par, par, par);
	g_viewMatrix = g_scene->getViewMatrix();
	g_transformCP->calcRotCtrlPnts();
	g_transformCP->houseKeep();
	g_vcArtAnimBase->calcRotCtrlPnts();
	g_vcArtAnimBase->houseKeep();
}

void shortcutToggleWireFrame(){
	if(g_scene->mode & MODELLING_MODE){
		toggleZBufferCageMode();
		g_tb2->paint();
	}
	else if(g_scene->mode & ANIMATION_MODE){
		toggleZBufferCageMode();
		g_acp->paint();
	}
	else{
		toggleWireFrame();
	}
}

void setAddBoneMode(){

	g_addBone->mousePhase = AddBone::ACTIVE;
	changeModellingCursor(g_addBoneCursor);
	g_stw->cp->setupNodeWindow(g_addBone);
	Window::activeWindow = g_stw->cp.p;
}

void addColumn(Vec3f pointA, Vec3f pointB, Vec3f pointC, M_Vec3f& dpoint, int& uDim);


void addColumn(NurbsPatch* patchA, int idxA, NurbsPatch* patchB, int idxB, M_Vec3f& newPoint, int& newUdim){
	Vec3f vecA = patchA->point[idxA];
	Vec3f vecB = patchB->point[idxB];
	Vec3f vecC = (vecA + vecB) * .5;
	addColumn(vecA, vecC, vecB, newPoint, newUdim);
}

bool pointIsFreeToOccupy(const Seamless* smls, const NurbsPatch* checkPatch, const int checkIdx){
	if(checkIdx < 0){
		return false;
	}
	
	int uCheckDim = checkPatch->dim.x;
	int vCheckDim = checkPatch->dim.y;

	if(checkIdx >= uCheckDim * vCheckDim){
		return false;
	}

	const Vec3f currentDpoint = checkPatch->dpoint[checkIdx];
	const M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;

	for(int k = cpNodeList.length; --k>=0;){
		const NurbsPatch* patch = cpNodeList[k];
		if(patch == checkPatch){
			continue;
		}
		const M_Vec3f& dpoint = patch->dpoint.refValue();

		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		for(int u = 0; u < uDim; u++){
			if(dpoint[u] == currentDpoint){//south
				return false;
			}
			if(dpoint[u + uDim * (vDim - 1)] == currentDpoint){//north
				return false;
			}
		}
		for(int v = 0; v < vDim; v++){
			int idx = v * uDim;
			if(dpoint[idx] == currentDpoint){//east
				return false;
			}
			idx += uDim - 1;
			if(dpoint[idx] == currentDpoint){//west
				return false;
			}
		}
	}
	return true;
}

void checkForNearPoint(Seamless* smls, int& currentEdge, bool& currentAscend, NurbsPatch*& currentPatch, int& currentIdx){
	const Vec3f currentDpoint = currentPatch->dpoint[currentIdx];
	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;

	for(int k = cpNodeList.length; --k>=0;){
		NurbsPatch* patch = cpNodeList[k];
		if(patch == currentPatch){
			continue;
		}
		M_Vec3f& dpoint = patch->dpoint.refValue();

		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		for(int u = 0; u < uDim; u++){
			int idx = u;
			if(dpoint[idx] == currentDpoint){//south
				currentPatch = patch;
				currentIdx = idx;
				if(NurbsPatch::isWestEdge(idx, uDim)){ //SW corner
					if(pointIsFreeToOccupy(smls, patch, idx + uDim)){
						currentEdge = NurbsPatch::WEST;
						currentAscend = true;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx + 1)){
						currentEdge = NurbsPatch::SOUTH;
						currentAscend = true;
						return;
					}
					return;
				}
				else if(NurbsPatch::isEastEdge(idx, uDim)){ //SE corner
					if(pointIsFreeToOccupy(smls, patch, idx + uDim)){
						currentEdge = NurbsPatch::EAST;
						currentAscend = true;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - 1)){
						currentEdge = NurbsPatch::SOUTH;
						currentAscend = false;
						return;
					}
					return;
				}
				else{
					currentEdge = NurbsPatch::SOUTH;
					if(pointIsFreeToOccupy(smls, patch, idx + 1)){
						currentAscend = true;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - 1)){
						currentAscend = false;
						return;
					}
				}
				continue;
			}
			idx += (vDim - 1) * uDim;
			if(dpoint[idx] == currentDpoint){//north
				currentPatch = patch;
				currentIdx = idx;
				if(NurbsPatch::isWestEdge(idx, uDim)){ //NW corner
					if(pointIsFreeToOccupy(smls, patch, idx - uDim)){
						currentEdge = NurbsPatch::WEST;
						currentAscend = false;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx + 1)){
						currentEdge = NurbsPatch::NORTH;
						currentAscend = true;
						return;
					}
					return;
				}
				else if(NurbsPatch::isEastEdge(idx, uDim)){ //NE corner
					if(pointIsFreeToOccupy(smls, patch, idx - uDim)){
						currentEdge = NurbsPatch::EAST;
						currentAscend = false;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - 1)){
						currentEdge = NurbsPatch::NORTH;
						currentAscend = false;
						return;
					}
					return;
				}
				else{
					currentEdge = NurbsPatch::NORTH;
					if(pointIsFreeToOccupy(smls, patch, idx + 1)){
						currentAscend = true;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - 1)){
						currentAscend = false;
						return;
					}
				}
			}
		}
		for(int v = 0; v < vDim; v++){
			int idx = v * uDim;
			if(dpoint[idx] == currentDpoint){//west
				currentPatch = patch;
				currentIdx = idx;

				if(NurbsPatch::isNorthEdge(idx, uDim, vDim)){ //NW corner
					if(pointIsFreeToOccupy(smls, patch, idx + uDim)){
						currentEdge = NurbsPatch::WEST;
						currentAscend = true;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx + 1)){
						currentEdge = NurbsPatch::NORTH;
						currentAscend = true;
						return;
					}
				}
				else if(NurbsPatch::isNorthEdge(idx, uDim, vDim)){ //SW corner
					if(pointIsFreeToOccupy(smls, patch, idx - uDim)){
						currentEdge = NurbsPatch::WEST;
						currentAscend = false;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx + 1)){
						currentEdge = NurbsPatch::NORTH;
						currentAscend = true;
						return;
					}
				}
				else{
					currentEdge = NurbsPatch::WEST;
					if(pointIsFreeToOccupy(smls, patch, idx + uDim)){
						currentAscend = true;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - uDim)){
						currentAscend = false;
						return;
					}
				}
				return;
			}
			idx += uDim - 1;
			if(dpoint[idx] == currentDpoint){//east
				currentPatch = patch;
				currentIdx = idx;

				if(NurbsPatch::isNorthEdge(idx, uDim, vDim)){ //NE corner
					if(pointIsFreeToOccupy(smls, patch, idx - uDim)){
						currentEdge = NurbsPatch::EAST;
						currentAscend = false;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - 1)){
						currentEdge = NurbsPatch::NORTH;
						currentAscend = false;
						return;
					}
				}
				else if(NurbsPatch::isSouthEdge(idx, uDim)){ //SE corner
					if(pointIsFreeToOccupy(smls, patch, idx - uDim)){
						currentEdge = NurbsPatch::EAST;
						currentAscend = false;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - 1)){
						currentEdge = NurbsPatch::SOUTH;
						currentAscend = false;
						return;
					}
				}
				else{
					currentEdge = NurbsPatch::EAST;
					if(pointIsFreeToOccupy(smls, patch, idx + uDim)){
						currentAscend = true;
						return;
					}
					else if(pointIsFreeToOccupy(smls, patch, idx - uDim)){
						currentAscend = false;
						return;
					}
				}
				return;
			}
		}
	}
}

bool nextEdgeIndex(int& edge, bool& ascend, NurbsPatch*& patch, int& idx){

	checkForNearPoint(patch->smls, edge, ascend, patch, idx);
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	if(ascend){
		switch(edge){
			case NurbsPatch::SOUTH:{
				idx++;
				if(idx == uDim - 1){
					edge = NurbsPatch::EAST;
				}
				break;
			}
			case NurbsPatch::EAST:{
				idx += uDim;
				if(idx == uDim * vDim - 1){
					edge = NurbsPatch::NORTH;
					ascend = false;
				}
				break;
			}
			case NurbsPatch::NORTH:{
				idx++;
				if(idx == uDim * vDim - 1){
					edge = NurbsPatch::EAST;
					ascend = false;
				}
				break;
			}
			case NurbsPatch::WEST:{
				idx += uDim;
				if(idx == uDim * (vDim - 1)){
					edge = NurbsPatch::NORTH;
				}
				break;
			}
		}
	}
	else{
		switch(edge){
			case NurbsPatch::SOUTH:{
				idx--;
				if(idx == 0){
					edge = NurbsPatch::WEST;
					ascend = true;
				}
				break;
			}
			case NurbsPatch::EAST:{
				idx-=uDim;
				if(idx == uDim - 1){
					edge = NurbsPatch::SOUTH;
				}
				break;
			}
			case NurbsPatch::NORTH:{
				idx--;
				if(idx == uDim * vDim - uDim){
					edge = NurbsPatch::WEST;
				}
				break;
			}
			case NurbsPatch::WEST:{
				idx-=uDim;
				if(idx == 0){
					edge = NurbsPatch::SOUTH;
					ascend = true;
				}
				break;
			}
		}
	}
	return false;
}

void fillet(){
	NurbsPatch* patchA = 0;
	NurbsPatch* patchB = 0;

	if(MultiSelect::pointList.length == 2){
		patchA = dynamic_cast<NurbsPatch*>(MultiSelect::pointList[0].p);
		patchB = dynamic_cast<NurbsPatch*>(MultiSelect::pointList[1].p);
		if(patchA == 0 || patchB == 0){
			return;
		}
		Seamless* smls = patchA->smls;
		M_Node* m_nodePtr;

		set_m_nodeRef(patchA, m_nodePtr);

		M_Node& r_m_node = *m_nodePtr;

		NurbsPatchPtr r_shaft = new NurbsPatch;

		
		r_m_node += r_shaft.p;
		r_shaft->sync.refValue() = true;
		r_shaft->smls = smls;
		r_shaft->uJoinEnds.refValue() = false;

		M_Vec3f& newPoint = r_shaft->point.refValue();
		M_Float& newWeight = r_shaft->weight.refValue();
		M_Float& newUknot = r_shaft->uKnot.refValue();
		M_Float& newVknot = r_shaft->vKnot.refValue();


		M_Vec3f& dpointA = patchA->dpoint.refValue();
		M_Float& weightA = patchA->weight.refValue();

		M_Float& uKnotA = patchA->uKnot;
		M_Float& vKnotA = patchA->vKnot;

		M_Vec3f& dpointB = patchB->dpoint.refValue();
		M_Float& weightB = patchB->weight.refValue();

		M_Float& uKnotB = patchB->uKnot;
		M_Float& vKnotB = patchB->vKnot;

		int edgeA = NurbsPatch::SOUTH;
		bool ascendA = false;
		int edgeB = NurbsPatch::SOUTH;
		bool ascendB = true;

		int newUdim = 0;
		int newVdim = 3;

		int idxA = MultiSelect::pointList[0].i;
		int idxB = MultiSelect::pointList[1].i;
		int remIdxA = idxA;
		NurbsPatch* remPatchA = patchA;
		
		//for(int i = 0; i < 201; i++){
		//for(int i = 0; i < 3; i++){
		for(int i = 0; i < 1001; i++){
			addColumn(patchA, idxA, patchB, idxB, newPoint, newUdim);
			if(i != 0 && remPatchA == patchA && idxA == remIdxA){
				break;
			}
			nextEdgeIndex(edgeA, ascendA, patchA, idxA);
			nextEdgeIndex(edgeB, ascendB, patchB, idxB);
		}

		r_shaft->dim.refValue() = Vec2f(newUdim, newVdim);

		int newPointLen = newUdim * newVdim;
		newWeight.length = newPointLen;
		for(int i = newPointLen; --i >= 0;){
			newWeight[i] = 1;
		}

		updateNurbsCurveKnotMid1(newUdim, newUknot);
		updateNurbsCurveKnotMid1(newVdim, newVknot);

		r_shaft->isCtrlPolyRef = true;

		//r_shaft->processFields();
		r_shaft->pointsToDpoints();


		g_stw->commonPostInsertNodeStuff();

		r_shaft->isCtrlPolyRef = true;

		r_shaft->processFields();
		//r_shaft->pointsToDpoints();

		smls->generateRawStitchPolys();

		//r_shaft->synchronizeAllStemCells();
		//bnspeBuildPolies(r_shaft);
		//smls->ctrf = true;
		//smls->doBuild();



		smls->populateMirrorNodes();
		smls->deleteAllParts();
		smls->generateAllPatchesBnspe();
		//smls->joinAllParts();

		houseKeepVisualClue();
	}
}


void setNearPoints(M_<NurbsPatch*>& cpNodeList, const int offsetPlidx, const int offsetPoint, M_Bool& flag, const Vec3f checkVec, const Vec3f newPnt){
	for(int k = offsetPlidx; --k >= 0;){
		NurbsPatch* patch = cpNodeList[k];
		NurbsPatch* newPatch = cpNodeList[k + offsetPlidx];
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		M_Vec3f& dpoint = newPatch->dpoint.refValue();

		for(int v = 0; v < vDim; v++){
			for(int u = 0; u < uDim; u++){
				int idx = v * uDim + u;
				if(false == flag[patch->fromPoint + idx]){
					if(dpoint[idx] == checkVec){
						dpoint[idx] = newPnt;
						flag[patch->fromPoint + idx] = true;
					}
				}
			}
		}
	}
}

void createDoubleSurface2(const NurbsPatch* patch, int u, int v, int uDim, M_<NurbsPatch*>& cpNodeList, M_Bool& flag, const int offsetPlidx, const int offsetPoint, Vec3f& normal, const float distance){
	NurbsPatch* newPatch = cpNodeList[offsetPlidx + patch->plidx];
	int idx = v * uDim + u;
	Vec3f newPnt = patch->dpoint.refValue()[idx];
	newPnt = newPnt + normal * distance;

	if(false == flag[patch->fromPoint + idx]){
		const Vec3f checkVec = patch->dpoint.refValue()[idx];
		flag[patch->fromPoint + idx] = true;
		newPatch->dpoint.refValue()[idx] = newPnt;

		setNearPoints(cpNodeList, offsetPlidx, offsetPoint, flag, checkVec, newPnt);
	}
}

void createDoubleSurface(){

	M_Bool flag;



	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		const int offsetPoint = smls->pointTotal;
		flag.length = offsetPoint;

		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		
		const int offsetPlidx = cpNodeList.length;

		for(int k = 0; k < offsetPlidx; k++){
			patch = cpNodeList[k]->createCopyOf();
			patch->name = cpNodeList[k]->name;
			if(patch->name.length){
				patch->name += 'B';
			}
			patch->smls = smls;
			smls->build.refValue() += patch;
			

			patch->processFields();
		}

		g_stw->genNodeList();
		smls->populateMirrorNodes();
		g_stw->commonPostInsertNodeStuff();


		for(int k = offsetPlidx; --k >= 0;){
			const NurbsPatch* patch = cpNodeList[k];
			int uDim = patch->dim.x;
			int vDim = patch->dim.y;

			for(int v = 1; v < vDim - 1; v++){
				for(int u = 1; u < uDim - 1; u++){
					float fu = u;
					float fv = v;
					Vec2f vec2f = Vec2f(fu / (uDim - 1), fv / (vDim - 1));
					Vec3f normal = patch->generateNormal(vec2f);
					if(!patch->insideOut){
						normal = -normal;
					}
					float distance = .008;
					createDoubleSurface2(patch, u, v, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
					//distance = .001;

					if(v == 1){
						if(u == 1){
							createDoubleSurface2(patch, 0, 0, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
						}
						if(u == uDim - 2){
							createDoubleSurface2(patch, uDim - 1, 0, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
						}
						createDoubleSurface2(patch, u, 0, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
					}
					if(v == vDim - 2){
						if(u == 1){
							createDoubleSurface2(patch, 0, vDim - 1, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
						}
						if(u == uDim - 2){
							createDoubleSurface2(patch, uDim - 1, vDim - 1, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
						}
						createDoubleSurface2(patch, u, vDim - 1, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
					}
					if(u == 1){
						createDoubleSurface2(patch, 0, v, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
					}
					if(u == uDim - 2){
						createDoubleSurface2(patch, uDim - 1, v, uDim, cpNodeList, flag, offsetPlidx, offsetPoint, normal, distance);
					}
				}
			}
		}
		for(int k = offsetPlidx; --k >= 0;){
			NurbsPatch* newPatch = cpNodeList[k + offsetPlidx];
			newPatch->dpointsToPoints();
			newPatch->setInsideOut2();
			newPatch->isCtrlPolyRef = true;
			newPatch->processFields();
		}

		smls->generateRawStitchPolys();
		houseKeepVisualClue();
	}
}

void InsideOutPart::Ok::operator=(const bool& par){
	if(Art* dest = dynamic_cast<Art*>(g_selNode.getRaw())){
		CopyPart cp;
		if(dest->smls->bnspe){
			home.doInsideOutBnspe(cp, dest);
		}
		else{
			//cp.mirrorX = true;
			//commonPasteAddCopyOfMirror(cp, remPJ);
		}
	}
	home.active = false;
	Window::closeWindow(g_stw->cp.p);
}

void insideOutPart(){
	if(g_insideOutPart->source = dynamic_cast<Art*>(g_selNode.getRaw())){
		g_insideOutPart->active = true;
		g_stw->cp->setupNodeWindow(g_insideOutPart);
		Window::activeWindow = g_stw->cp.p;
	}
}

void NurbsPatch::breakControlPointJoin(int selIdx){
	

	M_Int& cpIndex = this->cpIndex.refValue();
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();


	//if(showAll == false){
		//tugFlag = 0;
		//MultiSelect::pointList.length = 0;
		//smls->unShareMidPointsTooInStitchPolys(this, selIdx);
	//}

	cpIndex[selIdx] = cp.length;
	cp += point[selIdx];
	cw += weight[selIdx];

	smls->cleanCps();

	processFields();

	if(smls->autoStitch){
		generateRawStitchPolys();
		smls->stitchAllPolys();
		smls->joinAllParts();
	}
	smls->calcAllPolyMidPoints();
	commonPostTug();
	smls->updateCage();
	houseKeep();


	BackUp::monostable.beginCycle();
	doBuild();

}

void addAsLockIfNotContained(NurbsPatch* patch, M_Node& m_lock, M_<NurbsPatchPtr>& m_delPatchList){
	for(int k = m_lock.length; --k>=0;){
		if(patch == m_lock[k].p){
			return;
		}
	}
	for(int k = m_delPatchList.length; --k>=0;){
		if(patch == m_delPatchList[k].p){
			return;
		}
	}
	Lock* lock = new Lock;
	lock->point.refValue() = patch->point;
	lock->weight.refValue() = patch->weight;
	lock->uKnot.refValue() = patch->uKnot;
	lock->vKnot.refValue() = patch->vKnot;
	lock->order.refValue() = patch->order;
	lock->endSP.refValue() = patch->endSP;
	lock->uJoinEnds.refValue() = patch->uJoinEnds;
	lock->beginSP.refValue() = patch->beginSP;
	lock->dim.refValue() = patch->dim;

	m_lock += lock;


	m_delPatchList += patch;
}

void addToGroupIfNotContained(NurbsPatch* patch, M_Build& m_build, M_<NurbsPatchPtr>& m_delPatchList){
	for(int k = m_build.length; --k>=0;){
		if(patch == m_build[k].p){
			return;
		}
	}
	for(int k = m_delPatchList.length; --k>=0;){
		if(patch == m_delPatchList[k].p){
			return;
		}
	}
	m_delPatchList += patch;
}

void addIfNotInList(NurbsPatch* patch, M_<NurbsPatchPtr>& m_delPatchList){
	for(int k = m_delPatchList.length; --k>=0;){
		if(patch == m_delPatchList[k].p){
			return;
		}
	}
	m_delPatchList += patch;
}

void deletePatches(M_<NurbsPatchPtr>& a);

void addSelectedPatchesToGroup(){
	if(GroupBuild* group = dynamic_cast<GroupBuild*>(g_selNode.getRaw())){

		M_<NurbsPatchPtr> m_delPatchList;
		M_Build& m_build = group->build.refValue();

		for(int k = 0; k < MultiSelect::pointList.length; k++){
			NurbsPatch* patch = MultiSelect::pointList[k].p;
			addToGroupIfNotContained(patch, m_build, m_delPatchList);
		}

		deletePatches(m_delPatchList);


		for(int k = 0; k < m_delPatchList.length; k++){
			m_build += m_delPatchList[k].p;
		}

		g_stw->commonPostInsertNodeStuff(true);
	}
	if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		M_<NurbsPatchPtr> m_delPatchList;
		M_Node& m_lock = hair->lock.refValue();

		for(int k = 0; k < MultiSelect::pointList.length; k++){
			NurbsPatch* patch = MultiSelect::pointList[k].p;
			addAsLockIfNotContained(patch, m_lock, m_delPatchList);
		}

		deletePatches(m_delPatchList);



		g_stw->commonPostInsertNodeStuff(true);
		hair->processFields();
		resetCPNodeList();
	}
}


void deleteSelectedPatches(){
	M_<NurbsPatchPtr> m_delPatchList;
	M_<NodePtr> m_hold;

	Seamless* smls  = getSeamless();

	preDeleteNodeForSeamlessNode(m_hold, smls);

	for(int k = 0; k < MultiSelect::pointList.length; k++){
		NurbsPatch* patch = MultiSelect::pointList[k].p;
		addIfNotInList(patch, m_delPatchList);
		if(patch->mirrorPatch){
			addIfNotInList(patch->mirrorPatch, m_delPatchList);
		}
	}
	
	deletePatches(m_delPatchList);

	MultiSelect::reset();

	g_stw->genNodeList();
	if(smls != getSeamless()){
		if(smls->cpNodeList.length){
			g_scene->selLine = getNodeLine(smls->cpNodeList[0]);
		}
	}

	postDeleteNodeForSeamlessNode(smls);



	g_stw->commonPostInsertNodeStuff(true);
	updateAutoInForClick();
}

void calcLookAtFromOrientation(){
	if(Viewpoint* selView = dynamic_cast<Viewpoint*>(g_selNode.getRaw())){
		selView->calcLookAtFromOrientation();
		houseKeepVisualClue();
	}
}

void toggleShowCameras(){
	g_showCamerasInAnimMode = !g_showCamerasInAnimMode;
}

void showSelectedGroups(int line){
	int parentLine = getParent(line);
	int parentIndent = g_sceneTreeList[parentLine].indent;

	for(int k = parentLine + 1; k < g_sceneTreeList.length && parentIndent < g_sceneTreeList[k].indent; k++){

		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_sceneTreeList[k].node.p)){
			patch->cage = true;
		}
	}
	reRenderPolysIfInHidePolyMode();
}

void showSelectedGroups(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = patch->smls;
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		for(int k = cpNodeList.length; --k >= 0;){
			cpNodeList[k]->cage = false;
		}

		patch = 0;
		for(int k = MultiSelect::pointList.length; --k >= 0;){
			if(patch != MultiSelect::pointList[k].p){//this simple optimation could be improved
				patch = MultiSelect::pointList[k].p;
				int line = getNodeLine(patch);
				showSelectedGroups(line);
			}
		}

		MultiSelect::reset();
		houseKeepVisualClue();
		g_stw->paint();
	}
}

void hideSelectedGroups(int line){
	int parentLine = getParent(line);
	int parentIndent = g_sceneTreeList[parentLine].indent;

	for(int k = parentLine + 1; k < g_sceneTreeList.length && parentIndent < g_sceneTreeList[k].indent; k++){

		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_sceneTreeList[k].node.p)){
			patch->cage = false;
		}
	}
	reRenderPolysIfInHidePolyMode();
}

void hideSelectedGroups(){
	if(dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){

		NurbsPatch* patch = 0;
		for(int k = MultiSelect::pointList.length; --k >= 0;){
			if(patch != MultiSelect::pointList[k].p){//this simple optimation could be improved
				patch = MultiSelect::pointList[k].p;
				int line = getNodeLine(patch);
				hideSelectedGroups(line);
			}
		}

		MultiSelect::reset();
		houseKeepVisualClue();
		g_stw->paint();
	}
}

void deleteAllStitchPolys(){
	if(Seamless* smls = getSeamless2()){
		smls->generateRawStitchPolys();
		houseKeepVisualClue();
	}
}

void setShowDimFilterVisual(){
	if(g_transformCP->dimFilterClickEnable && g_transformCP->xTug && g_transformCP->yTug && g_transformCP->zTug){
		g_transformCP->showDimFilterVisual = false;
		g_transformCP->dimFilterClickEnable = false;
	}
	else{
		g_transformCP->xTug.refValue() = true;
		g_transformCP->yTug.refValue() = true;
		g_transformCP->zTug.refValue() = true;
		g_transformCP->showDimFilterVisual = true;
		g_transformCP->dimFilterClickEnable = true;
		//g_transformCP->calcPivotPoint();
	}
	g_transformCP->houseKeep();
	houseKeepVisualClue();
}

void decTugRadius(){ //DEC_RADIUS
	g_radial_Tug->radius = g_radial_Tug->radius - getIncDecRes();
}
void incTugRadius(){ //INC_RADIUS
	g_radial_Tug->radius = g_radial_Tug->radius + getIncDecRes();
}

void toggleRadialTug(){

	if(g_radial_Tug->active = !g_radial_Tug->active){
		g_stw->cp->setupNodeWindow(g_radial_Tug);
		Window::activeWindow = g_stw->cp.p;
	}
	else if(Window* window = Window::getControlPanel(g_radial_Tug)){
		Window::closeWindow(window);
	}
	houseKeepVisualClue();
}

bool SurfacePoint(
				  float u, float v, int uOrder, int vOrder, int uDimension, int vDimension,
				  const float *uKnot, const float * vKnot,
				  const Vec3f *P,const float *W,
				  Vec3f *dest // result
				  );

inline bool mapToPatchFineTune(NurbsPatch* selPatch, float& u, const float uInc, float& v, const float vInc, float& closest, const int uOrder, const int vOrder, const int uDim, const int vDim, const float* uKnotPtr, const float* vKnotPtr, const Vec3f* pointPtr, const float* weightPtr, const Vec2f& tartgetPoint2f, Vec3f& crd, const Matrix& iMatrix){
	SurfacePoint(u + uInc, v + vInc, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &crd);
	crd = iMatrix * crd;

	Vec2f diff = Vec2f(crd.x, crd.y) - tartgetPoint2f;

	float distance = abs(diff.length());
	if(distance < closest){
		closest = distance;
		u += uInc;
		v += vInc;
		return true;
	}

	return false;
}

inline bool mapToPatchFineTune(NurbsPatch* selPatch, float& u, float uInc, float& v, const float vInc, float& closest, const int uOrder, const int vOrder, const int uDim, const int vDim, const float* uKnotPtr, const float* vKnotPtr, const Vec3f* pointPtr, const float* weightPtr, const Vec3f& tartgetPoint, Vec3f& crd){
	bool negCrossOver = false;
	bool posCrossOver = false;
	float newU = u + uInc;
	if(newU < 0){
		negCrossOver = true;
		newU = 1 + newU;
	}
	else if(newU > 1){
		posCrossOver = true;
		newU = newU - 1;
	}
	SurfacePoint(newU, v + vInc, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &crd);

	Vec3f diff = crd - tartgetPoint;

	float distance = abs(diff.length());
	if(distance < closest){
		closest = distance;
		if(negCrossOver || posCrossOver){
			u = newU;
			uInc = -uInc;
		}
		else{
			u += uInc;
			v += vInc;
		}
		return true;
	}

	return false;
}

void RadicalPatch::mapToPatch(NurbsPatch* p, int i, NurbsPatch* selPatch){


	Vec3f targetPoint = p->dpoint[i];

	const float* uKnotPtr = selPatch->uKnot.refValue().getArray();
	const float* vKnotPtr = selPatch->vKnot.refValue().getArray();
	const Vec3f* pointPtr = selPatch->point.refValue().getArray();
	const float* weightPtr = selPatch->weight.refValue().getArray();
	const int uOrder = selPatch->order.x;
	const int vOrder = selPatch->order.y;
	const int uDim = selPatch->dim.x;
	const int vDim = selPatch->dim.y;
	const float maxU = selPatch->maxU();
	const float maxV = selPatch->maxV();
	const float minU = selPatch->minU();
	const float minV = selPatch->minV();


	float closest = 1000000000;
	Vec2f closestUV;
	int uLen = 25;
	int vLen = 25;
	Vec3f crd;
	Vec3f closestCrd;

	for(int vi = 0; vi < vLen - 1; vi++){
		float v = static_cast<float>(vi) / (vLen - 1);
		for(int ui = 0; ui < uLen - 1; ui++){
			float u = static_cast<float>(ui) / (uLen - 1);
			SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &crd);
			crd = crd;

			Vec3f tempDiff = crd - targetPoint;

			float distance = abs(tempDiff.length());
			if(distance < closest){
				closest = distance;
				closestUV = Vec2f(u, v);
				closestCrd = crd;
			}
		}
	}
	int numOfTries = 100;

	float uInc = .1;
	float vInc = .1;

	while(--numOfTries >= 0){
		if(closest <= .0001){
			break;
		}
		if(false == mapToPatchFineTune(selPatch, closestUV.x, uInc, closestUV.y, 0, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint, closestCrd)){
			uInc = -uInc;
			if(false == mapToPatchFineTune(selPatch, closestUV.x, uInc, closestUV.y, 0, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint, closestCrd)){
				uInc = -uInc * .5;
			}
		}
		if(false == mapToPatchFineTune(selPatch, closestUV.x, 0, closestUV.y, vInc, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint, closestCrd)){
			vInc = -vInc;
			if(false == mapToPatchFineTune(selPatch, closestUV.x, 0, closestUV.y, vInc, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint, closestCrd)){
				vInc = -vInc * .5;
			}
		}
	}

	mapVec += closestUV;
	diff += Vec3f();
	srcP += p;
	srcI += i;
	mapP += selPatch;
}

void RadicalPatch::mapToPatchProject(NurbsPatch* p, int i, NurbsPatch* selPatch, const Matrix& iMatrix, const Rotation rot){


	Vec3f targetPoint = iMatrix * p->dpoint[i];
	Vec2f targetPoint2f = Vec2f(targetPoint.x, targetPoint.y);

	const float* uKnotPtr = selPatch->uKnot.refValue().getArray();
	const float* vKnotPtr = selPatch->vKnot.refValue().getArray();
	const Vec3f* pointPtr = selPatch->point.refValue().getArray();
	const float* weightPtr = selPatch->weight.refValue().getArray();
	const int uOrder = selPatch->order.x;
	const int vOrder = selPatch->order.y;
	const int uDim = selPatch->dim.x;
	const int vDim = selPatch->dim.y;
	const float maxU = selPatch->maxU();
	const float maxV = selPatch->maxV();
	const float minU = selPatch->minU();
	const float minV = selPatch->minV();


	float closest = 1000000000;
	Vec2f closestUV;
	int uLen = 25;
	int vLen = 25;
	Vec3f crd;
	Vec3f closestCrd;

	for(int vi = 0; vi < vLen - 1; vi++){
		float v = static_cast<float>(vi) / (vLen - 1);
		for(int ui = 0; ui < uLen - 1; ui++){
			float u = static_cast<float>(ui) / (uLen - 1);
			SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &crd);
			crd = iMatrix * crd;

			Vec2f tempDiff = Vec2f(crd.x, crd.y) - targetPoint2f;

			float distance = abs(tempDiff.length());
			if(distance < closest){
				closest = distance;
				closestUV = Vec2f(u, v);
				closestCrd = crd;
			}
		}
	}
	int numOfTries = 100;

	float uInc = .1;
	float vInc = .1;

	while(--numOfTries >= 0 && closest > .0001){
		if(false == mapToPatchFineTune(selPatch, closestUV.x, uInc, closestUV.y, 0, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint2f, closestCrd, iMatrix)){
			uInc = -uInc;
			if(false == mapToPatchFineTune(selPatch, closestUV.x, uInc, closestUV.y, 0, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint2f, closestCrd, iMatrix)){
				uInc = -uInc * .5;
			}
		}
		if(false == mapToPatchFineTune(selPatch, closestUV.x, 0, closestUV.y, vInc, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint2f, closestCrd, iMatrix)){
			vInc = -vInc;
			if(false == mapToPatchFineTune(selPatch, closestUV.x, 0, closestUV.y, vInc, closest, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, targetPoint2f, closestCrd, iMatrix)){
				vInc = -vInc * .5;
			}
		}
	}

	if(closest <= .0001){
		mapVec += closestUV;
		Matrix matrix = rotationMatrix(rot);
		diff += matrix * closestCrd - p->dpoint[i];
		srcP += p;
		srcI += i;
		mapP += selPatch;
	}
}

void RadicalPatch::collectPoints(){
	if(NurbsPatch* selPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){


		//Art* part = selPatch->smls->partList[5];


		if(Seamless* smls = getSeamless()){
			M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;


			for(int i = MultiSelect::pointList.length; --i >= 0;){
				mapToPatch(MultiSelect::pointList[i].p, MultiSelect::pointList[i].i, selPatch);

			}

			//for(int j = 0; j < 3; j++){
			//	part->uvCoord.refValue()[j] = mapVec[2 - j];
			//}
			//smls->doBuild();
			//smls->ctrf = true;

			for(int k = 0; k < cpNodeList.length; k++){
				cpNodeList[k]->cage = false;
			}

			selPatch->cage = true;
			MultiSelect::reset();
		}
	}
}

void RadicalPatch::collectPointsProject(){
	if(NurbsPatch* selPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Rotation rot = g_scene->rotation;
		//Rotation rot;
		Matrix matrix = rotationMatrix(rot);
		rot = -rot;


		//Art* part = selPatch->smls->partList[5];


		if(Seamless* smls = getSeamless()){
			M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;


			for(int i = MultiSelect::pointList.length; --i >= 0;){
				mapToPatchProject(MultiSelect::pointList[i].p, MultiSelect::pointList[i].i, selPatch, matrix, rot);

			}

			//for(int j = 0; j < 3; j++){
			//	part->uvCoord.refValue()[j] = mapVec[2 - j];
			//}
			//smls->doBuild();
			//smls->ctrf = true;

			for(int k = 0; k < cpNodeList.length; k++){
				cpNodeList[k]->cage = false;
			}

			selPatch->cage = true;
			MultiSelect::reset();
		}
	}
}

void simplifyCurve(){
	if(g_radicalPatch->active = !g_radicalPatch->active){
		g_radicalPatch->diff.length = 0;
		g_radicalPatch->srcP.length = 0;
		g_radicalPatch->srcI.length = 0;
		g_radicalPatch->mapVec.length = 0;
		g_radicalPatch->mapP.length = 0;

		g_radicalPatch->collectPoints();

		nodeLookUpStr[g_radicalPatch->getType()] = "SmoothingPatch";

		g_stw->cp->setupNodeWindow(g_radicalPatch);
		Window::activeWindow = g_stw->cp.p;
	}
	else if(Window* window = Window::getControlPanel(g_radicalPatch)){
		Window::closeWindow(window);
	}
	houseKeepVisualClue();


	//if(g_transformFallOff->active = !g_transformFallOff->active){
	//	g_stw->cp->setupNodeWindow(g_transformFallOff);
	//	Window::activeWindow = g_stw->cp.p;
	//	g_transformFallOff->setWeights();
	//}
	//else if(Window* window = Window::getControlPanel(g_transformFallOff)){
	//	Window::closeWindow(window);
	//}
	//houseKeepVisualClue();
}


void toggleTransformFallOff(){
	if(g_radicalPatch->active = !g_radicalPatch->active){
		g_radicalPatch->diff.length = 0;
		g_radicalPatch->srcP.length = 0;
		g_radicalPatch->srcI.length = 0;
		g_radicalPatch->mapVec.length = 0;
		g_radicalPatch->mapP.length = 0;

		if(controlKeyState()){
			g_radicalPatch->collectPoints();
		}
		else{
			g_radicalPatch->collectPointsProject();
		}

		nodeLookUpStr[g_radicalPatch->getType()] = "RadicalPatch";

		g_stw->cp->setupNodeWindow(g_radicalPatch);
		Window::activeWindow = g_stw->cp.p;
	}
	else if(Window* window = Window::getControlPanel(g_radicalPatch)){
		Window::closeWindow(window);
	}
	houseKeepVisualClue();


	//if(g_transformFallOff->active = !g_transformFallOff->active){
	//	g_stw->cp->setupNodeWindow(g_transformFallOff);
	//	Window::activeWindow = g_stw->cp.p;
	//	g_transformFallOff->setWeights();
	//}
	//else if(Window* window = Window::getControlPanel(g_transformFallOff)){
	//	Window::closeWindow(window);
	//}
	//houseKeepVisualClue();
}

void Radial_Tug::Radius::operator=(const float& par){
	float p = par;
	if(p <= .001){
		p = .001;
	}
	refValue() = p;
	if(Window* window = Window::getControlPanel(&home)){
		window->paint();
	}
	houseKeepVisualClue();
}


void Radial_Tug::houseKeep(){
}

void outputNodeBind(){
	g_scene->outputNodeBind();
}


void Seamless::generatePoliesForSelPatches(){
	if(Seamless* smls = getSeamless2()){
		M_<NurbsPatch*> contigList;
		smls->createSelContigList(contigList);
		M_Bool m_flag;
		m_flag.length = smls->cpNodeList.length;

		for(int k = MultiSelect::pointList.length; --k>=0;){
			NurbsPatch* patch = MultiSelect::pointList[k].p;
			m_flag[patch->plidx] = true;
		}

		MultiSelect::pointList.length = 0;
		for(int pi = 0; pi < m_flag.length; pi++){
			if(m_flag[pi]){
				NurbsPatch* patch = smls->cpNodeList[pi];
				patch->synchronizeCellsAndBuild();
			}
		}
	}
}


void bnspeBuildPolies(){
	M_MirrorPatch m_mirrorPatch;
	if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		getSeamless()->ctrf = true;
		hair->doBuild();
	}
	if(MultiSelect::pointList.length){
		if(Seamless* smls = getSeamless2()){
			smls->generatePoliesForSelPatches();
		}
	}
	else if(Build* build = dynamic_cast<Build*>(g_selNode.getRaw())){
		if(NurbsPatch* patch = build->getPatch()){
			if(g_insertPatch.imousePhase){
				cutUpPatches(patch->smls, g_insertPatch.m_insert, m_mirrorPatch);
			}
			else{
				int numOfPatches = build->numOfPatches();
				M_<NurbsPatch*>& cpNodeList = patch->smls->cpNodeList;
				int plidx = patch->plidx;
				for(int k = 0; k < numOfPatches; k++){
					cpNodeList[k + plidx]->synchronizeCellsAndBuild();
				}
			}
		}
	}
}

unsigned g_key = 0;
bool checkKey(M_<ShortCutKey>& shortCut, unsigned key){
	for(int i=shortCut.length; --i>=0;){
		if(shortCut[i].keyCode==key){
			shortCut[i].function();
			return true;
		}
	}
	return false;
}

bool keyDown(unsigned key){
	g_key=key;
	if(checkKey(globalShortCut, key))return true;
	return Window::activeWindow->keyDown(key);
}
bool _3DWin::keyDown(unsigned key){
	if(key == VK_ESCAPE){
		reset3dScene();
		return true;
	}
	if(false == (g_scene->mode & (MODELLING_MODE | POLY_EDIT_MODE)))return false;
	return checkKey(shortCut, key);
}
bool keyDownAlt(unsigned key){
	if(checkKey(globalShortCutAlt, key))return true;

#ifdef PROG_ROB
	if(g_altDown == false){
		g_altDown = true;
		dLog("new VirtKeyDown(VK_MENU),");
	}
#endif

	return Window::activeWindow->keyDownAlt(key);
}
bool keyDownCtrl(unsigned key){
	if(checkKey(globalShortCutCtrl, key))return true;
	return Window::activeWindow->keyDownCtrl(key);
}
bool blockShortCutKeys(unsigned key){
	if(key>=0x21 && key <= 0x28){
		return false; //includes arrow keys
	}
	if(Window::chatMode())return true;
	return false;
}
bool keyDownShift(unsigned key){
	if(blockShortCutKeys(key))return false;
	if(checkKey(globalShortCutShift, key))return true;
	return Window::activeWindow->keyDownShift(key);
}
bool _3DWin::keyDownAlt(unsigned key){
	return checkKey(shortCutAlt, key);
}
bool _3DWin::keyDownCtrl(unsigned key){
	return checkKey(shortCutCtrl, key);
}
bool _3DWin::keyDownShift(unsigned key){
	if(Window::chatMode())return false;
	return checkKey(shortCutShift, key);
}
bool ToolBarWin::keyDown(unsigned key){
	if(Window::chatMode())return false;
	return checkKey(_3DWin::shortCut, key);
}
bool ToolBarWin::keyDownAlt(unsigned key){
	return checkKey(_3DWin::shortCutAlt, key);
}
bool ToolBarWin::keyDownCtrl(unsigned key){
	return checkKey(_3DWin::shortCutCtrl, key);
}
bool ToolBarWin::keyDownShift(unsigned key){
	if(Window::chatMode())return false;
	return checkKey(_3DWin::shortCutShift, key);
}
bool SceneTree::keyDown(unsigned key){
	return checkKey(shortCut, key);
}
bool SceneTree::keyDownAlt(unsigned key){
	return checkKey(shortCutAlt, key);
}
bool SceneTree::keyDownCtrl(unsigned key){
	return checkKey(shortCutCtrl, key);
}
bool SceneTree::keyDownShift(unsigned key){
	return checkKey(shortCutShift, key);
}

bool ControlPanel::keyDown(unsigned key){
	return checkKey(shortCut, key);
}
bool ControlPanel::keyDownAlt(unsigned key){
	return checkKey(shortCutAlt, key);
}
bool ControlPanel::keyDownCtrl(unsigned key){
	return checkKey(shortCutCtrl, key);
}
bool ControlPanel::keyDownShift(unsigned key){
	return checkKey(shortCutShift, key);
}
bool ChatWin::keyDown(unsigned key){
	return checkKey(shortCut, key);
}
bool ChatWin::keyDownCtrl(unsigned key){
	return checkKey(shortCutCtrl, key);
}
bool ChatWin::keyDownShift(unsigned key){
	return checkKey(shortCutShift, key);
}

void toggleHomeMode(){
	if(g_scene->mode & HOME_MODE){
		setModellingMode();
	}
	else{
		setHomeMode();
	}
}


void updateColorFromColorMappers(){

	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		part->smls->updateColorFromColorMappers(part);
	}
}

void Seamless::replacePatchRef(NurbsPatch* from, NurbsPatch* to){
	for(int k = ctrlPolyPoint.length; --k >= 0;){
		if(ctrlPolyPoint[k].p == from){
			ctrlPolyPoint[k].p = to;
		}
	}

	for(int k = partList.length; --k>= 0;){
		Art* part = partList[k];
		M_NurbsPatch& patchOwner = part->patchOwner;
		M_ControlPoint& controlPoint = part->controlPoint;

		for(int j = patchOwner.length; --j>=0;){
			if(patchOwner[j] == from){
				patchOwner[j] = to;
			}
		}

		for(int c = controlPoint.length; --c>=0;){
			if(controlPoint[c].p == from){
				controlPoint[c].p = to;
			}
		}
	}
}

void convertToColorPatches(){
	Seamless* smls = getSeamless2();
	int line = g_stw->selLine;
	int smlsIndent = g_sceneTreeList[line].indent;
	while(++line < g_sceneTreeList.length && g_sceneTreeList[line].indent > smlsIndent){
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_sceneTreeList[line].node.p)){
			if(dynamic_cast<NurbsLine*>(patch)){
				continue;
			}
			ColorPatchPtr colorPatch = new ColorPatch;
			colorPatch->name = patch->name;
			colorPatch->copyData(patch);
			colorPatch->processFields();

			replaceNodeRefInSceneTree(patch, colorPatch);
			smls->replacePatchRef(patch, colorPatch);
		}
	}
	g_stw->commonPostInsertNodeStuff(false);
}


