/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

//#define SEAMLESS3DGUI
#define DIRECT3D
//#define NOZLIB

//#define BIND_DEVELOPER_SCRIPT 1

//#define PROG_ROB 1
//#define DEMO_TO_MOVIE
#define RECORD_SOUNDMOVIE 1

#define CPINDEX 1

#ifndef NOZLIB
#include "zlib.h"
#endif

#define WACOM_FILES 1

#ifdef WIN32
#pragma warning (disable : 4244 4305 4100 4355 4996)
#endif
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#ifdef ANDROID
typedef int asmPointer;
#elif WIN32
typedef int asmPointer;
#else
typedef long int asmPointer;
#endif

#ifdef ANDROID
inline long double sinl(long double par){
    return sin(par);
}
inline long double cosl(long double par){
    return cos(par);
}
inline long double asinl(long double par){
    return asin(par);
}
inline long double acosl(long double par){
    return acos(par);
}
inline long double atan2l(long double parA, long parB){
    return atan2(parA, parB);
}
inline long double sqrtl(long double par){
    return sqrt(par);
}
inline long double powl(long double parA, long parB){
    return pow(parA, parB);
}
#endif

#define PI 3.14159265358979323846
#define TINY_AMOUNT 0.00000001
#define FIELDLOOKUPLEN 22
extern char floatFormat[];
extern char floatFormat2[];
extern char g_vrmlSquareOpenBracket[];
extern char g_vrmlSquareCloseBracket[];
extern char g_vrmlQuote[];
extern const char g_exposedField[];
extern const char g_eventIn[];
extern const char g_eventOut[];
extern const char g_field[];
extern const char g_inputOutput[];
extern const char g_inputOnly[];
extern const char g_outputOnly[];
extern const char g_initializeOnly[];
extern const char* g_exposedFieldPtr;
extern const char* g_eventInPtr;
extern const char* g_eventOutPtr;
extern const char* g_fieldPtr;


#define nullPtr 0
extern char g_exceptioninNode[];


enum NodeType{
    MESH,
    SCENE,
    SEAMLESS,
    COLOREFFECT,
    TEXTUREEFFECT,
    MONOCOLOREFFECT,
    PART,
    IMMEDIATEPART,
    GROUPBUILD,
    STEM,
    TEXTUREMAPPER,
    COLORSWEEP,
	COLORMAPPER,
    SURFACEGENERATOR,
    CCLATHE,
    NURBSPATCH,
	COLORPATCH,
//////seamless above
    SHAPE,
    APPEARANCE,
    MATERIAL,
    INDEXEDLINESET,
    COORDINATE,
    COLOR,

    PROTODECLARE,
    PROTOINTERFACE,
    PROTOBODY,
    TRANSFORM,


    NULLNODE,
    AVATARSOCKET,
    INDEXEDFACESET,
    IMAGETEXTURE,
    NAVIGATIONINFO,
    _GROUP,
    NOOUTPUTGROUP,
    GESTUREGROUP,
    MOBILE,
    CCBEND,
    STEM2,
    POLYMAP,
    BRANCHJOIN,
    GENERATOR,
    SURFACETRANSFORM,
    PLACE,
    DIRECTIONALLIGHT,
    CCSCALE,
    CCTRANSLATE,
    CCRINGSCALE,
    BEZIERLATHE,
    ECLATHE,
    ECSCALE,
    ECSHIFT,
    ECBEND,
    ECSYNCBEND,
    ECTWIST,
    BACKGROUND,
    SCGROUPBUILD,
    JOINCLOSEVERTEX,
    CENTERJOINT,
    DELPOLY,
    JOINVERTEX,
    COPYPART,
//	VIEWMODEL,
    VIEWPOINT,
    TRANSFERBCYL,
    TRANSFEROWNERSHIP,
    TRANSFERVERTEX,
    NORMAL,
    TEXTURECOORDINATE,
    SINETUG,
    SIMULSINETUG,
    GENPOLY,
    SCRIPT,
    BRANCHJOINNEGX,
    BRANCHJOINPOSX,
    BRANCHJOINNEGY,
    BRANCHJOINPOSY,
    BRANCHJOINNEGZ,
    BRANCHJOINPOSZ,
    PROJECTONTOTRI,
    EVOLVECOORD,
    FUNCTION,
    MEMBERFUNCTION,
    CODELINE,
    MORPHPART,
    MORPHER,
    TEXTURETRANSFORM,
    PROTOINSTANCE,
    MOVIE,
    VIDEO,
	AUDIO,
    ANIM,
    TIMESENSOR,
    ORIENTATIONINTERPOLATOR,
    POSITIONINTERPOLATOR,
    NURBSTRACK,
    SCALARINTERPOLATOR,
    POSCOMPENSATOR,
	NIKINTERPOLATOR,
    COLLISION,
    COLORINTERPOLATOR,
    COORDINATEINTERPOLATOR,
    NORMALINTERPOLATOR,
    MODELSEQUENCE,
    MODELMORPHER,
    WORLDINFO,
    TOUCHSENSOR,
    CCPATH,
    BSPTREE,
    INLINE,
    FOG,
    FOG2,
    ANCHOR,
    AUDIOCLIP,
    BILLBOARD,
    BOX,
    CONE,
    CYLINDER,
    CYLINDERSENSOR,
    ELEVATIONGRID,
    EXTRUSION,
    FONTSTYLE,
    NURBSSURFACE,
    LOD_TYPE,
    MOVIETEXTURE,
    PIXELTEXTURE,
    PLANESENSOR,
    POINTLIGHT,
    POINTSET,
    PROXIMITYSENSOR,
    SOUND,
    SPHERE,
    SPHERESENSOR,
    SPOTLIGHT,
    SWITCH,
    TEXT,
    VISIBILITYSENSOR,
    EXITBUILD,
    SPLITTRI,
    SWAPTRIDIVIDE,
    PLACETEXTURE,
    HANIMHUMANOID,
    HANIMJOINT,
    HANIMDISPLACER,
    HANIMSEGMENT,
    HANIMSITE,
    BOOLEANFILTER,
    CAMERASELECTOR,
    BOOLEANTOGGLE,
    BOOLEANTRIGGER,
    INTEGERSEQUENCER,
    INTEGERTRIGGER,
    TIMETRIGGER,
    STATICGROUP,
    FLOATACCELERATOR,
    ROTACCELERATOR,
    COPYTOMORPH,
    RELATEMORPHER,
    DELPLANE,
    DELSPHERE,
    RADIALTUG,
    TAPEMEASURE,
    POSACCELERATOR,
    ANIMBYANKLE,
    ANIMBYWRIST,
    ROTFILTER,
    ROTMULT,
    MULTIUSER,
    INDEXEDTRIANGLESET,
    POLYLINE2D,

    SPHEREBUILD,
    CYLINDERBUILD,
    CONEBUILD,
    TORUSBUILD,
    BOXBUILD,
    RECTANGLEBUILD,
    BEZIERLATHEBUILD,
    MIRROREDIT,
    MIRRORGROUP,
    POLYTONURBS,
    RINGBUILD,
    BEZIERSCALE,
    BEZIERCIRCUMSCALE,
    BEZIERPATCH,
    FEEDBACK,
    POVLIGHT,
    POVFOG,
    BEZIERSHIFT,
    BEZIERTWIST,
    BEZIERTAPE,
    BEZIERBEND,
    NURBSLINE,
    NURBSLATHE,
    BENDYLATHE,
    FUSESURFACE,
    SUBDIVISION,
    OSCILLATOR,
    ENVELOPE,
    AMPLITUDEMODULATOR,
    FREQUENCYMODULATOR,
    CHORDS,
    LARYNX,
    PHRASE,
    HARMONIC,
    HARMONICPATCH,
    SAMPLE,
    WAVETOPOLY,
	MIDIFORMANT,
    TRACK,
    COMPOSITION,
    CPMORPHER,
    HAIR,
    STRAND,
    LOCK,

    G_S_BOOL,
    G_S_COLOR,
    G_S_FLOAT,
    G_S_IMAGE,
    G_S_INT32,
    G_NODEPTR,
    G_S_ROTATION,
    G_S_STRING,
    G_S_TIME,
    G_S_VEC2F,
    G_S_VEC3F,
    G_M_BOOL,
    G_M_COLOR,
    G_M_FLOAT,
    G_M_IMAGE,
    G_M_INT32,
    G_M_NODE,
    G_M_ROTATION,
    G_M_STRING,
    G_M_TIME,
    G_M_VEC2F,
    G_M_VEC3F,

    FILE_NODE,
    FLOORHEIGHT,
    MESSAGE_1_BUTTON,
    QUESTION_MESSAGE_3_BUTTON,
    RECENT_FILES,
	RADICAL_PATCH,
	RADIAL_TUG,
    PALETTE,
    COMMON_SPEC,

    BRUSH,
    FILL,
    COLORPICKER,
    SETTINGS,
    STRETCHCURVE,
    OPENNETFILE,
    DEMO,
    CONVERTTOPATCH,
    REDUCECAPTUREDPOSES,
    TRANSFORMOBJ,
    TRANSFORMCP,
	ANIMLEVER,
    TIMELINEFILTER,
    PASTEOPTIONS,
    DELETEOPTIONS,
    REGION,
    SEED,
    THINNER,
    INSERTLOCK,
    EXTRUDE,
    INSERTCOMPLETE,
	CUTCOMPLETE,
    ADDLATHE,
    ADDNURBSLINE,
	DRAW_NURBSLINE,
    ADDBONE,
    GRID,
    DUPLICATE_PATCHES,
    INSIDEOUT_PART,
    EXPORT_MOVIE,
    EXPORT_VRMLX3D,
    RINGLETGENERATOR,
	ROBOTSAVE,
//	MATCHINGNAMEQUERY,
    ART //abstract keep last coz no node uses this for an index

};

enum FieldType{
    SFBOOL,
    SFCOLOR,
    SFFLOAT,
    SFIMAGE,
    SFINT32,
    SFNODE,
    SFROTATION,
    SFSTRING,
    SFTIME,
    SFVEC2F,
    SFVEC3F,
    MFBOOL,
    MFCOLOR,
    MFFLOAT,
    MFIMAGE,
    MFINT32,
    MFNODE,
    MFROTATION,
    MFSTRING,
    MFDOUBLE,
    MFVEC2F,
    MFVEC3F,

    MROUTE,
    MFPROPERTIES,
    MFTRIANGLE,
    MFPART
};

enum {
    HOME_MODE = 0x0001,
    ANIMATION_MODE = 0x0002,
    MODELLING_MODE = 0x0004,
    POLY_EDIT_MODE = 0x0008,
    XRAY_EDIT_MODE = 0x0010,
    OWNERSHIP_MODE = 0x0020,
    NURBS_OWNERSHIP_MODE = 0x0040,
    PAINT_MODE = 0x0080,
    SELECT_OBJECT_MODE = 0x00100,

    //PAN_VIEW_MODE =				0x00001000,
    CREATE_MIRROR_OBJECT	=	0x00002000,
    SINGLE_ACTIVE_MODE	=		0x00004000,
    ZBUFFER_CAGE_MODE	=		0x00008000,
    BACKFACE_IGNORE_MODE =		0x00010000,
    STITCH_POLY_MODE =			0x00020000,
    HAIR_REGION_MODE =			0x00040000,
    SHOW_STRAND_CTRL_LINE_MODE =0x00080000,
    ENABLE_MORPH_CAPTURE =		0x00100000,
    WIREFRAME_MODE =			0x00200000,
    HIDE_MESH_MODE =			0x00400000,
    BOX_PATCH_MODELLING_MODE =	0x00800000,

    BRUSH_MODE =				0x01000000,
    FILL_MODE =					0x02000000,
    COLOR_PICKER_MODE =			0x04000000,



    MODE_MASK = 0x01ff,
    IMODE_MASK = 0xfffffE00,

    PAINT_TOOL_MASK = 0xff000000,
    IPAINT_TOOL_MASK = 0x00FFFFFF
};

//template <typename T > class _S_Vec3f;
//template <typename T> class SF;

class Single{};
class Global{};

template <typename T > class M_;
template <typename T > class M_DEP_;
template <class T, class S = Single> class FD;
template <class T, class S = Single, class V = Global> class V_;

class Scene;
class Shape;
class Art;
class Build;
class Appearance;
class IndexedFaceSet;
class ImageTexture;
class Material;
class NavigationInfo;
class Group;
class NoOutputGroup;
class CCLathe;
class ECLathe;
class ECBend;
class ECSyncBend;
class CCBend;
class SurfaceGenerator;
class Stem;
class Stem2;
class BranchJoin;
class Generator;
class ALathe;
class Mesh;
class GeomRender;
class SurfaceTransform;
class Place;
class DirectionalLight;
class CCScale;
class CCTranslate;
class CCRingScale;
class BezierLathe;
class Background;
class Seamless;
class Polymap;
class JoinCloseVertex;
class CenterJoint;
class DelPoly;
class JoinVertex;
class CopyPart;
class Viewpoint;
class TransferBCyl;
class TransferOwnership;
class TransferVertex;
class Coordinate;
class Normal;
class Color;
class TextureCoordinate;
class SineTug;
class SimulSineTug;
class GenPoly;
class Script;
class BranchJoinNegX;
class BranchJoinPosX;
class BranchJoinNegY;
class BranchJoinPosY;
class BranchJoinNegZ;
class BranchJoinPosZ;
class ProjectOntoTri;
class ColorSweep;
class ColorMapper;
class EvolveCoord;
class MorphPart;
class Morpher;
class TextureTransform;
class ProtoDeclare;
class ProtoInterface;
class ProtoBody;
class ProtoInstance;
class Anim;
class OrientationInterpolator;
class PositionInterpolator;
class ScalarInterpolator;
class PosCompensator;
class Collision;
class Transform;
class WorldInfo;
class TouchSensor;
class CCPath;
class BspTree;
class Inline;
class Fog;
class Fog2;
class Anchor;
class AudioClip;
class Billboard;
class Box;
class Cone;
class Cylinder;
class CylinderSensor;
class ElevationGrid;
class Extrusion;
class FontStyle;
class IndexedLineSet;
class LOD;
class MovieTexture;
class PixelTexture;
class PlaneSensor;
class PointLight;
class PointSet;
class ProximitySensor;
class Sound;
class Sphere;
class SphereSensor;
class SpotLight;
class Switch;
class Text;
class VisibilitySensor;
class TextureMapper;
class TextureEffect;
class SwapTriDivide;
class PlaceTexture;
class HAnimHumanoid;
class HAnimJoint;
class HAnimDisplacer;
class BooleanFilter;
class CameraSelector;
class BooleanToggle;
class BooleanTrigger;
class IntegerSequencer;
class IntegerTrigger;
class TimeTrigger;
class StaticGroup;
class FloatAccelerator;
class PosAccelerator;
class CopyToMorph;
class RelateMorpher;
class DelPlane;
class DelSphere;
class RadialTug;
class Radial_Tug;
class ColorEffect;
class MonoColorEffect;
class ImmediatePart;
class TapeMeasure;
class RotAccelerator;
class AnimByAnkle;
class AnimByWrist;
class RotFilter;
class RotMult;
class VertexGeometry;
class IndexedTriangleSet;
class Polyline2D;
class Brush;
class Fill;
class Settings;
class RectangleBuild;
class StretchCurve;
class ConvertToPatch;
class OpenNetFile;
class Geometry;
class AGroup;
class SFPart;
class AOperand;
class Operation;
class Function;
class MemberFunction;
class CodeLine;
class MirrorEdit;
class BezierTape;
class NurbsPatch;
class ColorPatch;
class NurbsLine;
class FuseSurface;
class ModelMorpher;
class SceneTreeListItem;
class FileCP;
class CPMorpher;
class NurbsLathe;
class SphereBuild;
class Oscillator;
class Effect;
class NurbsTrack;
class Properties;
class S_Route;
class CPContigElement;
class Strand;
class Lock;
class NikInterpolator;

//class bool;
class Color3f;
//class float;
class Image;
//class int;
class Rotation;
class Rotation4d;
class String;
//class double;
template <typename T>class Vec2;
typedef Vec2<float> Vec2f;
typedef Vec2<int> Vec2i;
typedef Vec2<double> Vec2d;

template <typename T>class Vec3;
typedef Vec3<float> Vec3f;
typedef Vec3<int> Vec3i;
typedef Vec3<double> Vec3d;


class Matrix;
class Color3f;
class Node;
template <class T>class Ptr;
typedef Ptr <Node> NodePtr;
class G_Variable;
class L_Variable;
typedef M_ <Function*> M_Function;

class Button;
class FieldButton;
class Window;
class InputEvent;
/*
class InputOutput{
};
class InputOnly{
};
*/

    //Color,
typedef Image Image;
typedef Color3f Color3f;

class BIField;
class BIProto;




class AnimTimeLine;
class AnimSlider;
class AnimControlPanel;
class SamplePose;


class Multiple{};
class GlobalVar;
class LocalVar;
class Local{};

class Route;

#ifdef DIRECT3D
#ifdef DX8
struct IDirect3DTexture8;
typedef IDirect3DTexture8 IDirect3DTexture;
#endif
#ifdef DX9
struct IDirect3DTexture9;
typedef IDirect3DTexture9 IDirect3DTexture;
#endif
#endif
//typedef void Event(Node& node, void* value, const double& timeStamp);
class Event{public: virtual void doIt(Node&, void*, const double&){};};
















#define BEGIN_WITH_SINGLE_POINT 0x0001
#define END_WITH_SINGLE_POINT 0x0002
#define X_CENTRED	0x0010
#define Y_CENTRED	0x0020
#define X_NEG_SIDE	0x0040
#define Y_NEG_SIDE	0x0080
#define BEGIN_WITH_POINTS_DOUBLED 0x0100
#define BEGIN_WITH_POINTS_HALVED 0x0200
#define END_WITH_POINTS_DOUBLED 0x0400
#define END_WITH_POINTS_HALVED 0x0800
#define MIRROR_X 1
#define MIRROR_Y 2
#define MIRROR_Z 4


//extern bool g_autoMFLength;
extern bool g_writeGzipped;

float getFloat(const String& str, int& i);
float getFloat2(const String& str, int& i);
double getDouble(const String& str, int& i);
int getInt(const String& str, int& i);
int getInt2(const String& str, int& i);
bool getBool(const String& str, int& i);
bool getCppBool(const String& str, int& i);

void searchTillNextWord(char*& p);
void searchTillText(char*& p);
bool isLegalNameChar(const char& ch);
float atof2(const char* str);


extern const String g_appDataDir;
extern const String g_debugLogPath;
extern const String g_setupPath;

extern void dLog(const String str);

extern bool g_colorOwnership;
extern bool g_paletteMode;

Color3f hexToColor(unsigned hex);
void addNodeNameForScript(String& str, Node* node, int fieldIndex);
void makeNodeNameFromAddr(String& str, const Node* par);

class Pix24{
public:
    unsigned char r;
    unsigned char g;
    unsigned char b;
    Pix24():
        r(255),
        g(255),
        b(255)
    {}
    Pix24(unsigned char r, unsigned char g, unsigned char b):
        r(r),
        g(g),
        b(b)
    {}
};

Color3f pix24ToColor(Pix24 pix);
Pix24 intToPix24(unsigned int i);

class Pix32{
public:
    unsigned char r;
    unsigned char g;
    unsigned char b;
    unsigned char a;
    Pix32():
        r(255),
        g(255),
        b(255),
        a(255)
    {}
    Pix32(unsigned char r, unsigned char g, unsigned char b):
        r(r),
        g(g),
        b(b),
        a(255)
    {}
    Pix32(unsigned char r, unsigned char g, unsigned char b, unsigned char a):
        r(r),
        g(g),
        b(b),
        a(a)
    {}
};


class Image  {
public:
    void getField(const String&, int&){}
    void toString(String& str);
};
typedef bool S_Bool;
typedef int S_Int32;
typedef float S_Float;

class Color3f  {
public:
    float r;
    float g;
    float b;
    Color3f(){
        r=1;
        g=1;
        b=1;
    }
    Color3f(float nr,float ng,float nb){
        r=nr;
        g=ng;
        b=nb;
    }
    Color3f(char* str){
        char* p=str;
        searchTillText(p);
        r=atof2(p);
        searchTillNextWord(p);
        g=atof2(p);
        searchTillNextWord(p);
        b=atof2(p);
    }
    Color3f	operator /(const float v)const{
        return (Color3f(r / v, g / v, b / v));
    }
    Color3f	operator +(const Color3f v)const{
        return (Color3f(r + v.r, g + v.g, b + v.b));
    }
    Color3f	operator -(const Color3f v)const{
        return (Color3f(r - v.r, g - v.g, b - v.b));
    }
    Color3f	operator -()const{
        return (Color3f(-r, -g, -b));
    }
    Color3f	invert()const{
        return (Color3f(1 - r, 1 - g, 1 - b));
    }

    Color3f	operator *(const Color3f& par)const{
        return (Color3f(r * par.r, g * par.g, b * par.b));
    }
    Color3f	operator *(const float par)const{
        return (Color3f(r * par, g * par, b * par));
    }
    friend Color3f operator *(const float par0, const Color3f& par1){
        return (Color3f(par0 * par1.r, par0 * par1.g, par0 * par1.b ));
    }

    Color3f	operator/(const Color3f v)const{
        return (Color3f(r / v.r, g / v.g, b / v.b));
    }

    friend bool	operator !=(const Color3f &par1, const Color3f &par2){
        if(
            par1.r == par2.r &&
            par1.g == par2.g &&
            par1.b == par2.b
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Color3f &par1, const Color3f &par2){
        if(
            par1.r == par2.r &&
            par1.g == par2.g &&
            par1.b == par2.b
            )
            return true;
        else return false;
    }
    unsigned colorToHex()const;
    unsigned colorToHex(const float alpha)const;
    void toString(String& str)const;
    int fromString(const String& str, int i=0);
    void getField(const String& str, int& i);
    void toPOVRayString(String& str, const String& finish)const;
    void toPOVRayString(String& str, const ColorEffect* effect)const;
    void toPOVRayString(String& str)const;
    void toPOVRayString(String& str, float f)const;
    void toStrCpp(String& str)const;
};
//template <typename T>

template <typename T>class Vec3{
public:

    T x;
    T y;
    T z;
    Vec3(){
        x = 0;
        y = 0;
        z = 0;
    }

    Vec3(T nx, T ny, T nz){
        x = nx;
        y = ny;
        z = nz;
    }
/*
    Vec3(char* str){
        char* p=str;
        searchTillText(p);
        x=atof2(p);
        searchTillNextWord(p);
        y=atof2(p);
        searchTillNextWord(p);
        z=atof2(p);
    }
*/
//    Vec3&	operator =(SF<Vec3,	float,	SFVec3>& par);
//    Vec3&	operator -=(SF<Vec3,	float,	SFVec3>& par);
//    Vec3&	operator +=(SF<Vec3,	float,	SFVec3>& par);
//    Vec3&	operator /=(SF<Vec3,	float,	SFVec3>& par);
//    Vec3&	operator *=(SF<Vec3,	float,	SFVec3>& par);

    Vec3 operator +(const Vec3& v)const{
        return (Vec3(x+v.x, y+v.y, z+v.z));
    }
    Vec3 operator -(const Vec3& v)const{
        return (Vec3(x-v.x, y-v.y, z-v.z));
    }
    Vec3 operator -()const{
        return (Vec3(-x, -y, -z));
    }

    Vec3 operator *(const Vec3& par)const{
        return (Vec3(x * par.x, y * par.y, z * par.z));
    }
    Vec3 operator *(float par)const{
        return (Vec3(x * par, y * par, z * par));
    }
//	friend Vec3 operator *(const float& par0, const Vec3& par1){
//		return (Vec3(par0*par1.x, par0*par1.y, par0*par1.z ));
//	}

    Vec3 operator /(const Vec3& v)const{
        return (Vec3(x / v.x, y / v.y, z / v.z));
    }
    Vec3 operator /(const float par)const{
        return (Vec3(x / par, y / par, z / par));
    }

    Vec3& operator -=(const Vec3& par1){
        x -= par1.x;
        y -= par1.y;
        z -= par1.z;
        return *this;
    }
    Vec3& operator +=(const Vec3& par1){
        x += par1.x;
        y += par1.y;
        z += par1.z;
        return *this;
    }
    Vec3& operator /=(const Vec3& par1){
        x /= par1.x;
        y /= par1.y;
        z /= par1.z;
        return *this;
    }
    Vec3& operator *=(const Vec3& par1){
        x *= par1.x;
        y *= par1.y;
        z *= par1.z;
        return *this;
    }
    Vec3& operator /=(const float& par1){
        x /= par1;
        y /= par1;
        z /= par1;
        return *this;
    }
    Vec3& operator *=(const float& par1){
        x *= par1;
        y *= par1;
        z *= par1;
        return *this;
    }
    friend bool	operator !=(const Vec3& par1, const Vec3& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Vec3& par1, const Vec3& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return true;
        else return false;
    }
    float dot(const Vec3& v) const{
        return (x * v.x +	y * v.y +  z * v.z);
    }

    Vec3 mirrorX()const{
       return Vec3(-x, y, z);
    }

    Vec3 cross ( const Vec3& v)const{
       return	Vec3(
                    y * v.z - z * v.y,
                    z * v.x - x * v.z,
                    x * v.y - y * v.x
                );
    }
    void getField(const String& str, int& i);
    void toString(String& str) const;

    Vec3 operator *=(const Matrix matrix);
    Vec3 normalize()const;
    T length()const{
        return sqrt(x * x + y * y + z * z);
    }


    void toStrCpp(String& str)const;
    void toPOVRayString(String& str)const;


};

template <typename T>class Vec2{
public:
    T x;
    T y;
    Vec2():
        x(0),
        y(0)
        {}

    Vec2(T nx,T ny){
        x=nx;
        y=ny;
    }
    friend bool	operator ==(const Vec2 &par1, const Vec2 &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return true;
        else return false;
    }
    friend bool	operator !=(const Vec2 &par1, const Vec2 &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return false;
        else return true;
    }
//////////
    friend bool	operator <(const Vec2 &parA, const Vec2 &parB){
        if(
            parA.x<parB.x&&
            parA.y<parB.y
            )
            return true;
        else return false;
    }
    friend bool	operator >=(const Vec2 &parA, const Vec2 &parB){
        if(
            parA.x>=parB.x&&
            parA.y>=parB.y
            )
            return true;
        else return false;
    }


    Vec2& operator+=(const Vec2& par){
        x += par.x;
        y += par.y;
        return *this;
    }
    Vec2& operator-=(const Vec2& par){
        x -= par.x;
        y -= par.y;
        return *this;
    }
    Vec2& operator*=(const Vec2& par){
        x *= par.x;
        y *= par.y;
        return *this;
    }
    Vec2& operator/=(const Vec2& par){
        x /= par.x;
        y /= par.y;
        return *this;
    }
    Vec2& operator/=(const float par){
        x /= par;
        y /= par;
        return *this;
    }
    Vec2& operator*=(const float par){
        x *= par;
        y *= par;
        return *this;
    }
    Vec2 operator -()const{
        return (Vec2(-x, -y));
    }
    Vec2 operator -(const Vec2& v)const{
        return (Vec2(x-v.x, y-v.y));
    }
    Vec2	operator +(const Vec2& v)const{
        return (Vec2(x+v.x, y+v.y));
    }
    Vec2	operator *(const Vec2& v)const{
        return Vec2(x*v.x, y*v.y);
    }
    Vec2	operator /(const Vec2& v)const{
        return Vec2(x/v.x, y/v.y);
    }
    Vec2	operator /(const float& par)const{
        return Vec2(x/par, y/par);
    }
    Vec2	operator *(const double& v)const{
        return Vec2(x*v, y*v);
    }
    Vec2	operator >>(const int& par)const{
        return Vec2(x >> par, y >> par);
    }
    Vec2 rotate(double angle)const{
        double newX;
        double newY;
        double cosAngle=(double)cos(angle);
        double sinAngle=(double)sin(angle);
        newX=x*cosAngle-y*sinAngle;
        newY=x*sinAngle+y*cosAngle;
        return Vec2(newX,newY);
    }
    float getAngle(){
        float hype=sqrt(x*x + y*y);
        float angle = acos(x/hype);
        if(y < 0){
            angle = -angle;
        }
        return angle;
    }
    float getAngle2(){
        float hype=sqrt(x*x + y*y);
        float angle = acos(x/hype);
        if(y < 0){
            angle = PI*2 - angle;
        }
        return angle;
    }
    float getAngle3(){
        float hype=sqrt(x*x + y*y);
        float angle = asin(y/hype) + PI*.5;
        if(x > 0){
            angle = PI*2 - angle;
        }
        return angle;
    }

    // return the principal angle of v (-PI < angle <= PI)
    double angle() const {
        return atan2(y, x);
    }
    Vec2 normalize()const;

    T length()const{
        return sqrt(x * x + y * y);
    }
    Vec2 mirrorX()const{
       return Vec2(-x, y);
    }

    void getField(const String& str, int& i);
    void toString(String& str) const;
    void toStrCpp(String& str)const;
    void toPOVRayString(String& str)const;
    void toPOVRayString(String& str, const Node* node)const;

};


class Quaternion{
public:
    union {
        struct{
            double x, y, z, w;
        };
        float qArray[4];
    };

    operator double *() { return &x; }
    Quaternion(){
    }
    Quaternion(double x, double y, double z, double w):
        x(x),
        y(y),
        z(z),
        w(w)
    {}
    Rotation rotation();
    Rotation4d rotation4d();
    Vec3d eulerXYZ();
    Vec3d eulerYZX();

        /*
    double dot(const Quaternion& a)const{
        return x * a.x + y * a.y + z * a.z + w * a.w;
    }
        // Scalar multiplication
    Quaternion operator*(double s )const{
        return Quaternion(x * s, y * s, z * s, w * s );
    }

    Quaternion slerp(const Quaternion& a, const Quaternion& b, float t){//slerp function adapted from http://www.3dkingdoms.com/weekly/weekly.php?a=36
        double w1, w2;
        double cosTheta	=	a.dot(b);
        double theta	=	acos(cosTheta);
        double sinTheta	=	sin(theta);
        if( sinTheta > 0.001f ){
            w1 = sin( (1.0f-t)*theta ) / sinTheta;
            w2 = sin( t*theta) / sinTheta;
        }
        else{
            w1 = 1.0f - t;
            w2 = t;
        }
        a = a*w1 + b*w2;
    }
    */
};
class Rotation {
public:
    float x;
    float y;
    float z;
    float angle;
    Rotation (){
        x=0;
        y=0;
        z=1;
        angle=0;
    }
    Rotation (const float i_x, const float i_y, const float i_z, const float i_angle){
        x=i_x;
        y=i_y;
        z=i_z;
        angle=i_angle;
    }
    Rotation (const Vec3f vec, const float i_angle){
        x=vec.x;
        y=vec.y;
        z=vec.z;
        angle=i_angle;
    }
    Rotation (Vec3f fromVector, Vec3f toVector);

    Rotation(char* str){
        char* p=str;
        searchTillText(p);
        x=atof2(p);
        searchTillNextWord(p);
        y=atof2(p);
        searchTillNextWord(p);
        z=atof2(p);
        searchTillNextWord(p);
        angle=atof2(p);
    }
    friend bool	operator !=(const Rotation &par1, const Rotation &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Rotation &par1, const Rotation &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return true;
        else return false;
    }
    Rotation	operator *=(const Rotation& par1);
    Rotation	operator *(const Rotation& par1)const;
    /*
    void* getMultiplyAddress(){
        (Rotation (__thiscall * )(const Rotation &) const)* p;
        p=&multiply;
        return p;
    }
    */
    Rotation	multiply(const Rotation& par1)const;
    Vec3f operator *(const Vec3f& vec)const;

    Quaternion quaternion()const{
        return Quaternion(x*sin(angle / 2.0), y*sin(angle / 2.0), z*sin(angle / 2.0), cos(angle / 2.0));
    }

    void toStrCpp(String& str)const;
    void toString(String& str)const;
    Vec3f getAxis()const;
    Vec3f setAxis(const Vec3f& par){
        x=par.x;
        y=par.y;
        z=par.z;
        return par;
    };
    Rotation operator -()const;
    void getField(const String& str, int& i);
    Rotation slerp(const Rotation dest, const float amount)const;
};
inline Rotation Quaternion::rotation(){
    Rotation rot;
    if(w>1.0L){
        w = 1;
    }
    rot.angle = acos(w) * 2.0;
    if(rot.angle==0){
        rot=Rotation();
    }
    else{
        rot.x = x / sin(rot.angle / 2.0);
        rot.y = y / sin(rot.angle / 2.0);
        rot.z = z / sin(rot.angle / 2.0);
    }
    return rot;
}


class Rotation4d {
public:
    double x;
    double y;
    double z;
    double angle;
    Rotation4d (){
        x=0;
        y=0;
        z=1;
        angle=0;
    }
    Rotation4d (const double i_x, const double i_y, const double i_z, const double i_angle){
        x=i_x;
        y=i_y;
        z=i_z;
        angle=i_angle;
    }
    Rotation4d (const Vec3d vec, const double i_angle){
        x=vec.x;
        y=vec.y;
        z=vec.z;
        angle=i_angle;
    }
    Rotation4d (Vec3d fromVector, Vec3d toVector);

    Rotation4d(char* str){
        char* p=str;
        searchTillText(p);
        x=atof2(p);
        searchTillNextWord(p);
        y=atof2(p);
        searchTillNextWord(p);
        z=atof2(p);
        searchTillNextWord(p);
        angle=atof2(p);
    }
    friend bool	operator !=(const Rotation4d &par1, const Rotation4d &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Rotation4d &par1, const Rotation4d &par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z&&
            par1.angle==par2.angle
            )
            return true;
        else return false;
    }
    Rotation4d	operator *=(const Rotation4d& par1);
    Rotation4d	operator *(const Rotation4d& par1)const;
    /*
    void* getMultiplyAddress(){
        (Rotation4d (__thiscall * )(const Rotation4d &) const)* p;
        p=&multiply;
        return p;
    }
    */
    Rotation4d	multiply(const Rotation4d& par1)const;
    Vec3d operator *(const Vec3d& vec)const;

    Quaternion quaternion()const{
        return Quaternion(x*sin(angle / 2.0), y*sin(angle / 2.0), z*sin(angle / 2.0), cos(angle / 2.0));
    }

    void toStrCpp(String& str)const;
    void toString(String& str)const;
    Vec3d getAxis()const;
    Vec3d setAxis(const Vec3d& par){
        x=par.x;
        y=par.y;
        z=par.z;
        return par;
    };
    Rotation4d operator -()const;
    void getField(const String& str, int& i);
    Rotation4d slerp(const Rotation4d dest, const double amount)const;
};


typedef M_ <Vec3f> M_Vec3f;


#define STANDARDBUFLEN 20

class IntList2{
public:
    const int* p;
    const int len;
    IntList2(const int* i_p, const int& i_len):
        p(i_p),
        len(i_len)
    {}
};
class FloatList2{
public:
    const float* p;
    const int len;
    FloatList2(const float* p, const int& len):
        p(p),
        len(len)
    {}
};
class EventList{
public:
    InputEvent** p;
    const int len;
    EventList(InputEvent** p, const int& len):
        p(p),
        len(len)
    {}
};

#define BOOL_LIST(x) BoolList2(x, sizeof(x))
#define DOUBLE_LIST(x) DoubleList2(x, sizeof(x)>>3)
#define FLOAT_LIST(x) FloatList2(x, sizeof(x)>>2)
#define INT_LIST(x) IntList2(x, sizeof(x)>>2)
#define EVENT_LIST(x) EventList(x, sizeof(x)>>2)

template <class S> inline void iinsertElement(M_<S>& t, int targetIndex, S& value){
    int i=t.length;
    t.length++;
    if(t.length>1){
        do{
            --i;
            t[i+1]=t[i];
        }while(i>targetIndex);

    }
    t[targetIndex]=value;
};

template <typename T>class M_ {
class Length{
    friend class M_;
    friend M_DEP_<T>;
protected:
    unsigned bufLen;
    unsigned rawLen;
    T* pointer;
public:
    Length():
        rawLen(0), bufLen(0), pointer(0)
    {
    }
    Length(const T& f0):
        rawLen(1),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
    }
    Length(const T& f0,const T& f1):
        rawLen(2),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
        pointer[1] = f1;
    }
    Length(const T& f0,const T& f1,const T& f2):
        rawLen(3),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
        pointer[1] = f1;
        pointer[2] = f2;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3):
        rawLen(4),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
        pointer[1] = f1;
        pointer[2] = f2;
        pointer[3] = f3;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4):
        rawLen(5),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
        pointer[1] = f1;
        pointer[2] = f2;
        pointer[3] = f3;
        pointer[4] = f4;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5):
        rawLen(6),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
        pointer[1] = f1;
        pointer[2] = f2;
        pointer[3] = f3;
        pointer[4] = f4;
        pointer[5] = f5;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6):
        rawLen(7),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
        pointer[1] = f1;
        pointer[2] = f2;
        pointer[3] = f3;
        pointer[4] = f4;
        pointer[5] = f5;
        pointer[6] = f6;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7):
        rawLen(8),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0] = f0;
        pointer[1] = f1;
        pointer[2] = f2;
        pointer[3] = f3;
        pointer[4] = f4;
        pointer[5] = f5;
        pointer[6] = f6;
        pointer[7] = f7;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8):
        rawLen(9),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0]=f0;
        pointer[1]=f1;
        pointer[2]=f2;
        pointer[3]=f3;
        pointer[4]=f4;
        pointer[5]=f5;
        pointer[6]=f6;
        pointer[7]=f7;
        pointer[8]=f8;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9):
        rawLen(10),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0]=f0;
        pointer[1]=f1;
        pointer[2]=f2;
        pointer[3]=f3;
        pointer[4]=f4;
        pointer[5]=f5;
        pointer[6]=f6;
        pointer[7]=f7;
        pointer[8]=f8;
        pointer[9]=f9;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10):
        rawLen(11),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0]=f0;
        pointer[1]=f1;
        pointer[2]=f2;
        pointer[3]=f3;
        pointer[4]=f4;
        pointer[5]=f5;
        pointer[6]=f6;
        pointer[7]=f7;
        pointer[8]=f8;
        pointer[9]=f9;
        pointer[10]=f10;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11):
        rawLen(12),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0]=f0;
        pointer[1]=f1;
        pointer[2]=f2;
        pointer[3]=f3;
        pointer[4]=f4;
        pointer[5]=f5;
        pointer[6]=f6;
        pointer[7]=f7;
        pointer[8]=f8;
        pointer[9]=f9;
        pointer[10]=f10;
        pointer[11]=f11;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12):
        rawLen(13),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0]=f0;
        pointer[1]=f1;
        pointer[2]=f2;
        pointer[3]=f3;
        pointer[4]=f4;
        pointer[5]=f5;
        pointer[6]=f6;
        pointer[7]=f7;
        pointer[8]=f8;
        pointer[9]=f9;
        pointer[10]=f10;
        pointer[11]=f11;
        pointer[12]=f12;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13):
        rawLen(14),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0]=f0;
        pointer[1]=f1;
        pointer[2]=f2;
        pointer[3]=f3;
        pointer[4]=f4;
        pointer[5]=f5;
        pointer[6]=f6;
        pointer[7]=f7;
        pointer[8]=f8;
        pointer[9]=f9;
        pointer[10]=f10;
        pointer[11]=f11;
        pointer[12]=f12;
        pointer[13]=f13;
    }
    Length(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13, const T& f14, const T& f15):
        rawLen(16),
        bufLen(STANDARDBUFLEN),
        pointer(new T[STANDARDBUFLEN])
    {
        pointer[0]=f0;
        pointer[1]=f1;
        pointer[2]=f2;
        pointer[3]=f3;
        pointer[4]=f4;
        pointer[5]=f5;
        pointer[6]=f6;
        pointer[7]=f7;
        pointer[8]=f8;
        pointer[9]=f9;
        pointer[10]=f10;
        pointer[11]=f11;
        pointer[12]=f12;
        pointer[13]=f13;
        pointer[14]=f14;
        pointer[15]=f15;
    }
    Length(const IntList2& par):
        rawLen(par.len*sizeof(int)/sizeof(T))
    {
        if(par.len % rawLen){
            throw this;
        }
        bufLen=rawLen;
        pointer=new T[bufLen];
        for(int i=0; i<par.len; i++){
            reinterpret_cast<int*>(pointer)[i]=par.p[i];
        }
    }
    Length(const FloatList2& par):
        rawLen(par.len*sizeof(float)/sizeof(T))
    {
        if(par.len % rawLen){
            throw this;
        }
        bufLen=rawLen;
        pointer=new T[bufLen];
        for(int i=0; i<par.len; i++){
            reinterpret_cast<float*>(pointer)[i]=par.p[i];
        }
    }
    Length(const EventList& par):
        rawLen(par.len*sizeof(InputEvent*)/sizeof(InputEvent*))
    {
        bufLen=rawLen;
        pointer=new InputEvent*[bufLen];
        for(int i=0; i<par.len; i++){
            pointer[i]=par.p[i];
        }
    }
    Length(const M_& param):
        rawLen(param.length.rawLen),
        bufLen(param.length.bufLen),
        pointer(new T[bufLen])
    {
        for(unsigned i=0;i<rawLen;i++)pointer[i]=param.length.pointer[i];
    }
    ~Length(){
        delete[] pointer;
    }
    int operator =(const Length& par){
        unsigned newLength=par.rawLen;
        unsigned i;
        if(bufLen<newLength){
            bufLen=newLength*2;
            T* newPointer=new T[bufLen];
            for(i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;
        }
        for(i=rawLen;i<newLength;i++){
            pointer[i]=T();
        }
        rawLen=newLength;
        return rawLen;
    }
    inline int operator =(const int& newLength){
        unsigned i;
        if(bufLen<(unsigned)newLength){
            bufLen=newLength*2;
            T* newPointer=new T[bufLen];
            for(i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;
        }
        for(i=rawLen;i<(unsigned)newLength;i++){
            pointer[i]=T();
        }
        rawLen=newLength;
        return rawLen;
    }
    int operator ++ (int){
        unsigned newLength=rawLen+1;

        if(bufLen<newLength){
            bufLen=newLength*2;
            T* newPointer=new T[bufLen];
            for(unsigned i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;
        }
        pointer[rawLen]=T();
        rawLen=newLength;
        return rawLen-1;
    }
    int operator -- (int){
        rawLen--;
        if((int)rawLen<0){
			throw this;
		}
        return rawLen;
    }
    int operator -= (int par){
        rawLen-=par;
        if((int)rawLen<0){
			throw this;
		}
        return rawLen;
    }
    int operator += (int par){
        unsigned newLength=par+rawLen;

        if(bufLen<newLength){
            bufLen=newLength*2;
            T* newPointer=new T[bufLen];
            for(unsigned i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;
        }
        for(unsigned i=rawLen;i<newLength;i++){
            pointer[i]=T();
        }
        rawLen=newLength;
        return newLength;
    }
    operator int ()const{
        return rawLen;
    }
};

public:
    M_():
        length()
    {}
    M_(const T& f0):
        length(f0)
    {}
    M_(const T& f0,const T& f1):
        length(f0,f1)
    {}
    M_(const T& f0,const T& f1,const T& f2):
        length(f0,f1,f2)
    {}
    M_(const T& f0,const T& f1,const T& f2,const T& f3):
        length(f0,f1,f2,f3)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4):
        length(f0,f1,f2,f3,f4)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5):
        length(f0,f1,f2,f3,f4,f5)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6):
        length(f0,f1,f2,f3,f4,f5,f6)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7):
        length(f0,f1,f2,f3,f4,f5,f6,f7)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8):
        length(f0,f1,f2,f3,f4,f5,f6,f7,f8)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9):
        length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10):
        length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11):
        length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12):
        length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13):
        length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13)
    {}
    M_(const T& f0,const T& f1,const T& f2, const T& f3, const T& f4, const T& f5, const T& f6, const T& f7, const T& f8, const T& f9, const T& f10, const T& f11, const T& f12, const T& f13, const T& f14, const T& f15):
        length(f0,f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13,f14,f15)
    {}
    M_(const IntList2& par):
        length(par)
    {}
    M_(const FloatList2& par):
        length(par)
    {}
    M_(const EventList& par):
        length(par)
    {}
    M_(const M_& param):
        length(param)
    {}
    ~M_(){
    }
    M_& operator= (const M_& par){
        if(par.length.pointer==0){
            length.rawLen=0;
        }
        else if(par.length.pointer!=length.pointer){
            if(length.bufLen<par.length.bufLen){
                delete[] length.pointer;
                length.rawLen=par.length.rawLen;
                length.bufLen=length.rawLen*2;
                length.pointer=new T[length.bufLen];
            }
            else{
                length.rawLen=par.length.rawLen;
            }
            for(unsigned i=0;i<length.rawLen;i++)length.pointer[i]=par.length.pointer[i];
        }
        return *this;
    }
    //const M_& operator =(const SF<M_,	T>& par);

    M_& operator +=(T par){
        unsigned newLength=length.rawLen+1;

        if(length.bufLen<newLength){
            length.bufLen=newLength*2;
            T* newPointer=new T[length.bufLen];
            for(unsigned i=0;i<length.rawLen;i++){
                newPointer[i]=length.pointer[i];
            }
            delete[] length.pointer;
            length.pointer=newPointer;
        }
        length.pointer[length.rawLen]=par;
        length.rawLen=newLength;
        return *this;
    }
    M_& operator +=(const M_& par1){
        unsigned i;
        unsigned newLength=par1.length.rawLen+length.rawLen;
        if(length.bufLen<newLength){
            length.bufLen=newLength*2;
            T* newPointer=new T[length.bufLen];
            for(unsigned i=0;i<length.rawLen;i++){
                newPointer[i]=length.pointer[i];
            }
            delete[] length.pointer;
            length.pointer=newPointer;
        }
        for(i=0;i<par1.length.rawLen;i++){
            length.pointer[i+length.rawLen]=par1.length.pointer[i];
        }
        length.rawLen=newLength;
        return *this;
    }
    M_ operator +(const T& par)const{
        M_ m=*this;
        m+=par;
        return m;
    }
    M_ operator +(const M_& par)const{
        M_ m=*this;
        m+=par;
        return m;
    }
    const char *showReason() const {
        return "Exception in M_ class.";
    }
    int size()const{
        return length.rawLen;
    }
    void setElement(const int index, const T& val){
        length.pointer[index]=val;
    }
    T& operator [](const int& index){
        if((unsigned)index>=length.rawLen){
            throw this;
            /*
            unsigned newLength=index+1;
            unsigned i;
            if(length.bufLen<newLength){
                length.bufLen=newLength*2;
                T* newPointer=new T[length.bufLen];
                for(i=0;i<length.rawLen;i++){
                    newPointer[i]=length.pointer[i];
                }
                delete[] length.pointer;
                length.pointer=newPointer;
            }
            for(i=length.rawLen;i<newLength;i++){
                length.pointer[i]=T();
            }
            length.rawLen=newLength;
            */
        }
        return length.pointer[index];
    }
    const T& operator [](const int& index)const{
        if((unsigned)index>=length.rawLen){
            throw this;
        }
        return length.pointer[index];
    }
    const T* getArray()const{
        return length.pointer;
    }
    T* getArray(){
        return length.pointer;
    }
    bool operator ==(const M_& par)const{
        if(M_::rawLen!=par->rawLen){
            return false;
        }
        for(int i=length; --i>=0;){
            if(length.pointer[i]!=par[i]){
                return false;
            }
        }
        return true;
    }
    bool operator !=(const M_& par)const{
        if(length!=par.length){
            return true;
        }
        for(int i=length; --i>=0;){
            if(length.pointer[i]!=par[i]){
                return true;
            }
        }
        return false;
    }
    void toString(String& str)const;
    void freeMem(){
        length.rawLen=0;
        length.bufLen=0;
        delete[] length.pointer;
        length.pointer=0;
    }
    void getField(const String& str,int& i);

    //'class M_<int> &(class SF<class M_<int>,int,69> &)'
//inline    M_<T, type> &	operator =(SF <M_<T,type>,T,type>& par);
//inline    M_<T, type> &	operator +=(const T par);
    void toMNodeString(String& str, bool newList=true);
    void removeNode(T par1);
    void removeNode(M_& par1);
    Art*& refPart(int i)const;
    int& refIndex(int i)const;
    int& refTexIndex(int i)const;
    int getTriangleLength()const{
        return length*3;
    };
    bool write(String fileName)const;

    void rotate(Rotation rot,int offset=0);
    void rotate( float x, float y, float z, float angle, int offset=0);
    void translate(T tran,int offset=0);
    void translate( float x, float y, float z, int offset=0);
    void scale(float xsc,float ysc,float zsc);
    void translateToZero(int offset,int numOf, Vec3f& center);
    void insertElement(int targetIndex, T value);
    void deleteElement(int targetIndex);
    void deleteElement(int from, int numOf);

    void toCppString(String& str);


    void translateToZero(int offset,int numOf,Rotation& center);
    void multMatrixLeft(Matrix& matrix);
    void setUpRoute( int& yPos);
    int getIndex(String str)const;
    Length length;
};

template <class T, class L> void removeTypeFromList(M_<Ptr<L> >& list){
    int d=0;
    int s=0;
    while(s<list.length){
        if(dynamic_cast<T*>(list[s].p)){
            list[s]=0;
            s++;
            continue;
        }
        list[d++]=list[s++];
    }
    list.length=d;
};
template <class T, class L> void removeTypeFromList(M_<L*>& list){
    int d=0;
    int s=0;
    while(s<list.length){
        if(dynamic_cast<T*>(list[s])){
            s++;
            continue;
        }
        list[d++]=list[s++];
    }
    list.length=d;
};
template <class T> void removeFromList(M_<T*>& list, T* ptr){
    int d=0;
    int s=0;
    while(s<list.length){
        if(ptr==list[s]){
            list[s]=0;
            s++;
            continue;
        }
        list[d++]=list[s++];
    }
    list.length=d;
};
template <class T> void removeFromList(M_<Ptr<T> >& list, T* ptr){
    int d=0;
    int s=0;
    while(s<list.length){
        if(ptr==list[s]){
            list[s]=0;
            s++;
            continue;
        }
        list[d++]=list[s++];
    }
    list.length=d;
};
template <class T> bool isInList(const M_<T*>& list, const T* ptr){
    for(int k=list.length; --k>=0;){
        if(ptr==list[k]){
            return true;
        }
    }
    return false;
};
template <class T> bool isInList(const M_<Ptr<T> >& list, const T* ptr){
    for(int k=list.length; --k>=0;){
        if(ptr==list[k]){
            return true;
        }
    }
    return false;
};


/*
class S_RedirectCoord {
public:
    Art* p;
    int i;
    S_RedirectCoord():
        p(0),
        i(0)
    {
    }
    S_RedirectCoord(Art*& i_p, int& i_i):
        p(i_p),
        i(i_i)
    {
    }
};
*/

class Extra{
public:
    NurbsPatch* p;
    Vec2i uv;
    Extra():
        p(0)
    {}
    Extra(NurbsPatch* p, Vec2i uv):
        p(p),
        uv(uv)
    {}
};


typedef M_ <bool> M_Bool;
typedef M_ <Color3f> M_Color3f;
typedef M_ <float> M_Float;
typedef M_ <Image> M_Image;
typedef M_ <int> M_Int;
//typedef M_ <NodePtr> M_Node;
typedef M_ <Rotation> M_Rotation;
typedef M_ <Rotation4d> M_Rotation4d;
typedef M_ <String> M_String;
typedef M_ <double> M_Double;
typedef M_ <Vec2i> M_Vec2i;
typedef M_ <Vec2f> M_Vec2f;
typedef M_ <Vec2d> M_Vec2d;
typedef M_ <Vec3f> M_Vec3f;
typedef M_ <NodePtr> M_Node;
typedef M_ <S_Route> M_Route;
typedef M_ <Properties> M_Properties;
//typedef M_ <S_RedirectCoord> M_RedirectCoord;
//typedef M_ <S_Instruc> M_Instruc;
typedef M_ <Pix24> M_Pix24;
typedef M_ <Extra> M_Extra;
typedef M_ <long double> M_LongDouble;

typedef M_ <M_Color3f> MM_Color;
typedef M_ <M_Int> MM_Int32;

typedef M_ <Matrix> M_Matrix;


extern M_String nodeLookUpStr;
extern M_String containerFieldLookUpStr;


/*
class S_Crd  {
public:
    int i;
    Art* p;
};
typedef M_ <S_Crd, MFTRIANGLE> M_Crd;
class S_Coord : public Vec3f {
public:
    Vec3f	operator =(const  Vec3f& v){
        x=v.x;
        y=v.y;
        z=v.z;
        return *this;
    }
    M_Crd ref;
};
typedef M_ <S_Coord, MFTRIANGLE> M_Coord;
*/
class S_VertexIndex {
public:
    Art* p;
    int i;
    int t;
    S_VertexIndex():
        p(0),
        i(0),
        t(0)
    {
    }
    S_VertexIndex(Art*& p, const int& i):
        p(p),
        i(i),
        t(0)
    {
    }
    S_VertexIndex(Art*& p, const int& i, const int& t):
        p(p),
        i(i),
        t(t)
    {
    }
};


class LocalVertexIndex{
public:
    int i;
    int t;
    LocalVertexIndex():
        i(0),
        t(0)
    {
    }
    LocalVertexIndex(const int& i):
        i(i),
        t(0)
    {
    }
    LocalVertexIndex(const int& i, const int& t):
        i(i),
        t(t)
    {
    }
};

/*
class ControlPoint{
public:
    Art* p;
    int i;
    ControlPoint():
        p(0),
        i(0)
    {
    }
    ControlPoint(Art*& p, const int& i):
        p(p),
        i(i)
    {
    }
};
*/

template <class T>class Ptr {
public:
    Ptr(){
        p=nullPtr;
    }
Ptr( T* par){
    p=par;
    if(par){
        par->incRefCount();
    }
}
Ptr(const Ptr& par){
    p=par.p;
    if(p!=NULL){
        p->incRefCount();
    }
}
Ptr& operator = (const Ptr& par){
    par.p->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par.p;
    return *this;
}
Ptr& operator = (int par){
    if(par!=0){
        throw this;
    }
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=0;
    return *this;
}
Ptr& operator = ( T* par){
    par->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par;
    return *this;
}
~Ptr(){
    if(p!=NULL&&p->decRefCount()==0){
        delete p;
        p=NULL;
    }
}
    bool	operator ==(const Node*& par)const{
        return p==par;
    }
    bool	operator !=(const Node*& par)const{
        return p!=par;
    }

    operator T*()const{
        return p;
    }

    T& operator *(){
        return *p;
    }
    T* operator ->()const{
        return p;
    }
    T* getRawPtr()const{
        return p;
    }
//protected:
    T* p;
};
typedef Ptr <Mesh>		MeshPtr;
typedef Ptr <Scene> ScenePtr;
typedef Ptr <Shape>	ShapePtr;
typedef Ptr <Art>		ArtPtr;
typedef Ptr <Build>		BuildPtr;
typedef Ptr <Appearance>	AppearancePtr;
typedef Ptr <IndexedFaceSet>	IndexedFaceSetPtr;
typedef Ptr <ImageTexture>	ImageTexturePtr;
typedef Ptr <Material>		MaterialPtr;
typedef Ptr <NavigationInfo>	NavigationInfoPtr;
typedef Ptr <Group>			GroupPtr;
typedef Ptr <NoOutputGroup>	NoOutputGroupPtr;
typedef Ptr <CCLathe>			CCLathePtr;
typedef Ptr <CCBend>			CCBendPtr;
typedef Ptr <SurfaceGenerator>	SurfaceGeneratorPtr;
typedef Ptr <Stem>				StemPtr;
typedef Ptr <NurbsPatch>		NurbsPatchPtr;
typedef Ptr <ColorPatch>		ColorPatchPtr;
typedef Ptr <NurbsLathe>		NurbsLathePtr;
typedef Ptr <NurbsLine>		NurbsLinePtr;
typedef Ptr <BranchJoin>		BranchJoinPtr;
typedef Ptr <GeomRender>		GeomRenderPtr;
typedef Ptr <SurfaceTransform>	SurfaceTransformPtr;
typedef Ptr <Place>			PlacePtr;
typedef Ptr <DirectionalLight>	DirectionalLightPtr;
typedef Ptr <CCScale>			CCScalePtr;
typedef Ptr <CCTranslate>		CCTranslatePtr;
typedef Ptr <CCRingScale>		CCRingScalePtr;
typedef Ptr <BezierLathe>		BezierLathePtr;
typedef Ptr <BezierTape>		BezierTapePtr;
typedef Ptr <ECLathe>			ECLathePtr;
typedef Ptr <ECBend>		ECBendPtr;
typedef Ptr <ECSyncBend>		ECSyncBendPtr;
typedef Ptr <Background>		BackgroundPtr;
typedef Ptr <Seamless>			SeamlessPtr;
typedef Ptr <JoinCloseVertex>	JoinCloseVertexPtr;
typedef Ptr <CenterJoint>		CenterJointPtr;
typedef Ptr <DelPoly>			DelPolyPtr;
typedef Ptr <JoinVertex>		JoinVertexPtr;
typedef Ptr <CopyPart>			CopyPartPtr;
typedef Ptr <Viewpoint>		ViewpointPtr;
typedef Ptr <TransferBCyl>		TransferBCylPtr;
typedef Ptr <TransferOwnership>	TransferOwnershipPtr;
typedef Ptr <TransferVertex>	TransferVertexPtr;
typedef Ptr <Coordinate>		CoordinatePtr;
typedef Ptr <Normal>			NormalPtr;
typedef Ptr <Color>			ColorPtr;
typedef Ptr <TextureCoordinate>TextureCoordinatePtr;
typedef Ptr <SineTug>	SineTugPtr;
typedef Ptr <GenPoly>	GenPolyPtr;
typedef Ptr <Script>	ScriptPtr;
typedef Ptr <BranchJoinNegX>	BranchJoinNegXPtr;
typedef Ptr <BranchJoinPosX>	BranchJoinPosXPtr;
typedef Ptr <BranchJoinNegY>	BranchJoinNegYPtr;
typedef Ptr <BranchJoinPosY>	BranchJoinPosYPtr;
typedef Ptr <BranchJoinNegZ>	BranchJoinNegZPtr;
typedef Ptr <BranchJoinPosZ>	BranchJoinPosZPtr;
typedef Ptr <ProjectOntoTri>	ProjectOntoTriPtr;
typedef Ptr <ColorSweep>		ColorSweepPtr;
typedef Ptr <ColorMapper>		ColorMapperPtr;
typedef Ptr <EvolveCoord>		EvolveCoordPtr;
typedef Ptr <MorphPart>		MorphPartPtr;
typedef Ptr <Morpher>		MorpherPtr;
typedef Ptr <TextureTransform>	TextureTransformPtr;
typedef Ptr <ProtoDeclare>			ProtoDeclarePtr;
typedef Ptr <ProtoInterface>			ProtoInterfacePtr;
typedef Ptr <ProtoBody>			ProtoBodyPtr;
typedef Ptr <ProtoInstance>			ProtoInstancePtr;
typedef Ptr <Anim>		AnimPtr;
typedef Ptr <OrientationInterpolator>OrientationInterpolatorPtr;
typedef Ptr <CameraSelector>		CameraSelectorPtr;
typedef Ptr <PositionInterpolator>	PositionInterpolatorPtr;
typedef Ptr <ScalarInterpolator>	ScalarInterpolatorPtr;
typedef Ptr <CPMorpher>	CPMorpherPtr;
typedef Ptr <PosCompensator>	PosCompensatorPtr;
typedef Ptr <NikInterpolator>	NikInterpolatorPtr;
typedef Ptr <Collision>	CollisionPtr;
typedef Ptr <Transform>	TransformPtr;

typedef Ptr <WorldInfo>	WorldInfoPtr;
typedef Ptr <TouchSensor>	TouchSensorPtr;
typedef Ptr <CCPath>		CCPathPtr;
typedef Ptr <BspTree>		BspTreePtr;
typedef Ptr <Inline>		InlinePtr;
typedef Ptr <Fog>			FogPtr;
typedef Ptr <Fog2>			Fog2Ptr;
typedef Ptr <Anchor>		AnchorPtr;
typedef Ptr <AudioClip>	AudioClipPtr;
typedef Ptr <Billboard>	BillboardPtr;
typedef Ptr <Box>			BoxPtr;
typedef Ptr <Cone>			ConePtr;
typedef Ptr <Cylinder>		CylinderPtr;
typedef Ptr <CylinderSensor>	CylinderSensorPtr;
typedef Ptr <ElevationGrid>	ElevationGridPtr;
typedef Ptr <Extrusion>	ExtrusionPtr;
typedef Ptr <FontStyle>	FontStylePtr;
typedef Ptr <IndexedLineSet>	IndexedLineSetPtr;
typedef Ptr <LOD>			LODPtr;
typedef Ptr <MovieTexture>	MovieTexturePtr;
typedef Ptr <PixelTexture>	PixelTexturePtr;
typedef Ptr <PlaneSensor>	PlaneSensorPtr;
typedef Ptr <PointLight>	PointLightPtr;
typedef Ptr <PointSet>		PointSetPtr;
typedef Ptr <ProximitySensor>	ProximitySensorPtr;
typedef Ptr <Sound>		SoundPtr;
typedef Ptr <Sphere>		SpherePtr;
typedef Ptr <SphereSensor>	SphereSensorPtr;
typedef Ptr <SpotLight>	SpotLightPtr;
typedef Ptr <Switch>		SwitchPtr;
typedef Ptr <Text>			TextPtr;
typedef Ptr <VisibilitySensor>	VisibilitySensorPtr;
typedef Ptr <TextureMapper>	TextureMapperPtr;
typedef Ptr <SwapTriDivide>	SwapTriDividePtr;
typedef Ptr <PlaceTexture>	PlaceTexturePtr;
typedef Ptr <HAnimHumanoid>	HAnimHumanoidPtr;
typedef Ptr <HAnimJoint>	HAnimJointPtr;
typedef Ptr <FloatAccelerator>	FloatAcceleratorPtr;
typedef Ptr <PosAccelerator>	PosAcceleratorPtr;
typedef Ptr <CopyToMorph>	CopyToMorphPtr;
typedef Ptr <Morpher>	MorpherPtr;
typedef Ptr <DelPlane>	DelPlanePtr;
typedef Ptr <DelSphere>	DelSpherePtr;
typedef Ptr <RadialTug>	RadialTugPtr;
typedef Ptr <Anim>	AnimPtr;
typedef Ptr <RotAccelerator>	RotAcceleratorPtr;
typedef Ptr <AnimByAnkle> AnimByAnklePtr;
typedef Ptr <AnimByWrist> AnimByWristPtr;

typedef Ptr <TextureEffect>	TextureEffectPtr;
typedef Ptr <ColorEffect>	ColorEffectPtr;
typedef Ptr <MonoColorEffect>	MonoColorEffectPtr;
typedef Ptr <TapeMeasure>		TapeMeasurePtr;
typedef Ptr <CodeLine>		CodeLinePtr;
typedef Ptr <AnimByWrist>		AnimByWristPtr;
typedef Ptr <RotFilter>	RotFilterPtr;
typedef Ptr <RotMult>		RotMultPtr;
typedef Ptr <VertexGeometry>		VertexGeometryPtr;
typedef Ptr <IndexedTriangleSet>		IndexedTriangleSetPtr;
typedef Ptr <Polyline2D>		Polyline2DPtr;
typedef Ptr <Brush>		BrushPtr;
typedef Ptr <Fill>		FillPtr;
typedef Ptr <Settings>		SettingsPtr;
typedef Ptr <RectangleBuild> RectangleBuildPtr;
typedef Ptr <StretchCurve> StretchCurvePtr;
typedef Ptr <ConvertToPatch> ConvertToPatchPtr;
typedef Ptr <OpenNetFile>		OpenNetFilePtr;
typedef Ptr <HAnimDisplacer> HAnimDisplacerPtr;
typedef Ptr <Geometry> GeometryPtr;
typedef Ptr <AGroup> AGroupPtr;
typedef Ptr <G_Variable>	G_VariablePtr;

typedef M_ <ProtoDeclarePtr> M_ProtoDeclare;
typedef M_ <ProtoInterfacePtr> M_ProtoInterface;
extern M_ProtoInterface g_lookupProtoInterface;

typedef Ptr <Strand>		StrandPtr;
typedef Ptr <Lock>		LockPtr;

class ControlPoint{
public:
    NurbsPatch* p;
    int i;
    ControlPoint():
        p(0),
        i(0)
    {
    }
    ControlPoint(NurbsPatch* p, const int i):
        p(p),
        i(i)
    {
    }
    bool compGT(const ControlPoint& cp) const;
    bool compEQ(const ControlPoint& cp) const;
};

class MirrorPatch{
public:
    NurbsPatch* patch;
    bool modifiedWest;

    MirrorPatch():
        patch(0),
        modifiedWest(true)
    {}

    MirrorPatch(NurbsPatch* patch, bool modifiedWest):
        patch(patch),
        modifiedWest(modifiedWest)
    {}
};


class Insert{
public:

    NurbsPatch* p;
    int uOrVIdx;
    bool doU;//if true u specifies column, otherwise v specifies row
    float ratio;
    bool invertRatio;
    bool diversion;
    bool leaveNorthOrEast;
    bool inLoop;
    Vec2i cutLine;

    int beginPoly;
    int entryPolyEdge;
    int exitPolyEdge;
    int markPolyEdge;
    int alternativeCentreIdx;

    NurbsPatch* otherPatch;
    Vec2i otherUV;
	bool first;
    //int otherIdx;

    M_Extra m_extra;

    Insert():
        p(0),
		first(false),
        uOrVIdx(0),
        doU(false),
        ratio(0),
        invertRatio(false),
        diversion(false),
        leaveNorthOrEast(false),
        inLoop(true),

        beginPoly(-1),
        entryPolyEdge(0),
        exitPolyEdge(0),
        markPolyEdge(0),
        alternativeCentreIdx(0),

        otherPatch(0)

    {
    }
    Insert(NurbsPatch* p, const int uOrVIdx, bool doU, float ratio, bool invertRatio):
        p(p),
		first(false),
        uOrVIdx(uOrVIdx),
        doU(doU),
        ratio(ratio),
        invertRatio(invertRatio),
        diversion(false),
        leaveNorthOrEast(false),
        inLoop(true),

        beginPoly(-1),
        entryPolyEdge(0),
        exitPolyEdge(0),
        markPolyEdge(0),
        alternativeCentreIdx(0),

        otherPatch(0)
    {
    }
    Insert(NurbsPatch* p, const int uOrVIdx, bool doU, float ratio, bool invertRatio, bool leaveNorthOrEast, Vec2i cutLine):
        p(p),
		first(false),
        uOrVIdx(uOrVIdx),
        doU(doU),
        ratio(ratio),
        invertRatio(invertRatio),
        diversion(false),
        leaveNorthOrEast(leaveNorthOrEast),
        inLoop(true),
        cutLine(cutLine),


        beginPoly(-1),
        entryPolyEdge(0),
        exitPolyEdge(0),
        markPolyEdge(0),
        alternativeCentreIdx(0),

        otherPatch(0)
    {
    }
    Insert(NurbsPatch* p, const int uOrVIdx, bool doU, NurbsPatch* otherPatch, Vec2i otherUV, float ratio, bool invertRatio, bool leaveNorthOrEast, Vec2i cutLine)://used for right angle
        p(p),
		first(false),
        uOrVIdx(uOrVIdx),
        doU(doU),


        ratio(ratio),
        invertRatio(invertRatio),
        diversion(true),
        leaveNorthOrEast(leaveNorthOrEast),
        inLoop(true),
        cutLine(cutLine),

        beginPoly(-1),
        entryPolyEdge(0),
        exitPolyEdge(0),
        markPolyEdge(0),
        alternativeCentreIdx(0),

        otherPatch(otherPatch),
        otherUV(otherUV)

    {
    }
    Insert(NurbsPatch* p, const int uOrVIdx, bool doU, float ratio, bool invertRatio, bool leaveNorthOrEast, Vec2i cutLine, int beginPoly, int entryPolyEdge, int exitPolyEdge, int markPolyEdge, int alternativeCentreIdx, NurbsPatch* otherPatch, Vec2i otherUV):
        p(p),
		first(false),
        uOrVIdx(uOrVIdx),
        doU(doU),
        ratio(ratio),
        invertRatio(invertRatio),
        diversion(true),
        leaveNorthOrEast(leaveNorthOrEast),
        inLoop(true),
        cutLine(cutLine),

        beginPoly(beginPoly),
        entryPolyEdge(entryPolyEdge),
        exitPolyEdge(exitPolyEdge),
        markPolyEdge(markPolyEdge),
        alternativeCentreIdx(alternativeCentreIdx),

        otherPatch(otherPatch),
        otherUV(otherUV)
    {
    }
    void operator = (const Insert& par){
        p = par.p;
		first = par.first,
        uOrVIdx = par.uOrVIdx;
        doU = par.doU;
        ratio = par.ratio;
        invertRatio = par.invertRatio;
        diversion = par.diversion;
        leaveNorthOrEast = par.leaveNorthOrEast;
        inLoop = par.inLoop;

        cutLine = par.cutLine;

        beginPoly = par.beginPoly;
        entryPolyEdge = par.entryPolyEdge;
        exitPolyEdge = par.exitPolyEdge;
        markPolyEdge = par.markPolyEdge;
        alternativeCentreIdx = par.alternativeCentreIdx;
        otherPatch = par.otherPatch;
        otherUV = par.otherUV;
        m_extra = par.m_extra;
    }
};


class DelCtrlLn{
public:
    NurbsPatch* p;
    int uOrVIdx;
    bool doU;
    int hidenLineOffset;
    DelCtrlLn():
        p(0),
        uOrVIdx(0),
        doU(false),
        hidenLineOffset(0)
    {
    }
    DelCtrlLn(NurbsPatch* p, const int uOrVIdx, bool doU):
        p(p),
        uOrVIdx(uOrVIdx),
        doU(doU),
        hidenLineOffset(0)
    {
    }
    void operator = (const DelCtrlLn& par){
        p = par.p;
        uOrVIdx = par.uOrVIdx;
        doU = par.doU;
        hidenLineOffset = par.hidenLineOffset;
    }
};

class CtrlPolyPoint{
public:
    NurbsPatch* p;
    unsigned short int i;
    short int len;
    CtrlPolyPoint():
        p(0),
        i(0),
        len(0)
    {
    }
    CtrlPolyPoint(NurbsPatch*& p, const unsigned short int& i, const unsigned short int& len):
        p(p),
        i(i),
        len(len)
    {
    }
    void operator=(const CtrlPolyPoint& par){
        p = par.p;
        i = par.i;
        len = par.len;
    }
};



class S_Triangle  {
public:
    Art* pa;
    int ia;
    int ta;
    Art* pb;
    int	ib;
    int tb;
    Art* pc;
    int ic;
    int tc;
    S_Triangle():
        pa(0),
        ia(0),
        ta(0),
        pb(0),
        ib(0),
        tb(0),
        pc(0),
        ic(0),
        tc(0)
    {
    }
    S_Triangle(Art* pa, int ia, Art* pb, int ib, Art* pc, int ic ):
        pa(pa),
        ia(ia),
        ta(0),
        pb(pb),
        ib(ib),
        tb(0),
        pc(pc),
        ic(ic),
        tc(0)
    {
    }
    S_Triangle(Art* pa, int ia, int ta, Art* pb, int ib, int tb, Art* pc, int ic, int tc):
        pa(pa),
        ia(ia),
        ta(ta),
        pb(pb),
        ib(ib),
        tb(tb),
        pc(pc),
        ic(ic),
        tc(tc)
    {
    }
/* why would we want this?
    S_Triangle& operator = (const S_Triangle& par){
        pa=par.pa;
        pb=par.pb;
        pc=par.pc;
        ia=par.ia;
        ib=par.ib;
        ic=par.ic;
        ta=par.ta;
        tb=par.tb;
        tc=par.tc;
        return *this;
    }
*/
    Art*& refPart(int corner)const{
        if((unsigned)corner > 2){
			throw this;
		}
        return ((S_VertexIndex*)this)[corner].p;
    }
    int& refIndex(int corner)const{
        if((unsigned)corner>2){
			throw this;
		}
        return ((S_VertexIndex*)this)[corner].i;
    }
    int& refTexIndex(int corner)const{
        if((unsigned)corner > 2){
			throw this;
		}
        return ((S_VertexIndex*)this)[corner].t;
    }
    S_VertexIndex& refCorner(int corner)const{
        return (((S_VertexIndex*)this)[corner]);
    }
    inline Art* getOwner()const;//get lowest bider
    Art* getOwner(int& idx)const;//get lowest bider

//	Vec3f calculateNormal();
    void toString(String& str)const;
};


/*
class LocalTriangle{
public:
    int ia;
    int ta;
    int	ib;
    int tb;
    int ic;
    int tc;
    LocalTriangle():
        ia(0),
        ib(0),
        ic(0),
        ta(0),
        tb(0),
        tc(0)
    {
    }
    LocalTriangle(int ia, int ib, int ic):
        ia(ia),
        ib(ib),
        ic(ic),
        ta(0),
        tb(0),
        tc(0)
    {
    }
    LocalTriangle(int ia, int ta, int ib, int tb, int ic, int tc):
        ia(ia),
        ib(ib),
        ic(ic),
        ta(ta),
        tb(tb),
        tc(tc)
    {
    }
    int& refIndex(int corner)const{
        if((unsigned)corner>2)throw this;
        return ((LocalVertexIndex*)this)[corner].i;
    }
    int& refTexIndex(int corner)const{
        if((unsigned)corner>2)throw this;
        return ((LocalVertexIndex*)this)[corner].t;
    }
    LocalVertexIndex& refCorner(int corner)const{
        return (((LocalVertexIndex*)this)[corner]);
    }
    void toString(String& str)const;
};

typedef M_<LocalTriangle> M_LocalTriangle;
*/

//typedef M_ <ControlPoint> M_ControlPoint;
typedef M_ <MirrorPatch> M_MirrorPatch;
typedef M_ <Insert> M_Insert;
typedef M_ <DelCtrlLn> M_DelCtrlLn;
typedef M_ <ControlPoint> M_ControlPoint;
typedef M_ <CtrlPolyPoint> M_CtrlPolyPoint;
typedef M_ <S_Triangle> M_Triangle;
typedef M_ <Art*> M_Art;
typedef M_ <NurbsPatch*> M_NurbsPatch;
typedef M_ <HAnimJoint*> M_HAnimJoint;

template<> inline Art*& M_Triangle::refPart(int i)const{
    if((unsigned) i >= length.rawLen * 3){
		throw this;
	}
    return ((S_VertexIndex*)length.pointer)[i].p;
}
template<> inline int& M_Triangle::refIndex(int i)const{
    if((unsigned) i >= length.rawLen * 3){
		throw this;
	}
    return ((S_VertexIndex*)length.pointer)[i].i;
}
template<> inline int& M_Triangle::refTexIndex(int i)const{
    if((unsigned)i>=length.rawLen * 3){
		throw this;
	}
    return ((S_VertexIndex*)length.pointer)[i].t;
}



#define defaultBufLen 0x200


void removeZeros(char* str);

inline char upperCaseChar(const char& c){
    if(c >= 'a' && c <= 'z'){
        return c - 0x20;
    }
    return c;
}
inline char lowerCaseChar(const char& c){
    if(c >= 'A' && c <= 'Z'){
        return c + 0x20;
    }
    return c;
}

bool equals(const char* pa, const char* pb, const int lenA, const int lenB);
bool notEquals(const char* pa, const char* pb, const int lenA, const int lenB);
bool lessThan(const char* pa, const char* pb, const int lenA, const int lenB);
bool lessEqual(const char* pa, const char* pb, const int lenA, const int lenB);
bool greaterThan(const char* pa, const char* pb, const int lenA, const int lenB);
bool greaterEqual(const char* pa, const char* pb, const int lenA, const int lenB);



template<class T> inline unsigned amountToRequest();
template<> inline unsigned amountToRequest<char>()         {return 4;}
template<> inline unsigned amountToRequest<Color3f>()		{return 60;}
template<> inline unsigned amountToRequest<int>()			{return 20;}
template<> inline unsigned amountToRequest<unsigned int>()	{return 20;}
template<> inline unsigned amountToRequest<float>()		{return 20;}
template<> inline unsigned amountToRequest<double>()		{return 20;}
template<> inline unsigned amountToRequest<Vec2i>()		{return 38;}
template<> inline unsigned amountToRequest<Vec2f>()		{return 40;}
template<> inline unsigned amountToRequest<Vec3f>()		{return 60;}
template<> inline unsigned amountToRequest<Vec3d>()		{return 70;}
template<> inline unsigned amountToRequest<Rotation>()		{return 80;}

class String{
public:
    class Data {
    public:
        unsigned refCount;
        unsigned bufLen;
        unsigned rawLen;
        char* rawChar;
    };
    friend Data;
class Length{
private:
    Data*& pointer;
public:
    Length(	Data*& i_pointer):
      pointer(i_pointer)
    {
    }
    operator int ()const{
        return pointer->rawLen;
    }
};

public:
    Data* createData(int i_bufLength=defaultBufLen, int length=0)const{
        Data* data=(Data*)new char[i_bufLength+sizeof(Data)+1];
        data->rawChar=(char*)data+sizeof(Data);
        data->refCount=1;	//must always be created with a pointer to it
        data->bufLen=i_bufLength;
        data->rawLen=length;
        return data;
    }
    String( Data* data):
        length(pointer)
    {
        pointer=data;
    }

    String(int bufSize, int length):
        length(pointer)
    {
        if(bufSize < length){
			throw this;
		}
        pointer = createData(bufSize, length);
    }

    String(const String& par1):
        length(pointer)
    {
        pointer=par1.pointer;
        pointer->refCount++;
    }
    String(const char* par1=""):
        length(pointer)
    {
        unsigned lEn=strlen(par1);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=par1[i];
    }
    String(char par1):
        length(pointer)
    {
        pointer=createData(1+defaultBufLen);
        pointer->rawLen=1;
        pointer->rawChar[0]=par1;
        pointer->rawChar[1]=0;
    }
    String(int par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,"%i",par1);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }
    String(unsigned int par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,"%u",par1);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }
    String(float par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,floatFormat,par1);
        removeZeros(tmp);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }
    String(double par1):
        length(pointer)
    {
        char tmp[50];
        sprintf(tmp,floatFormat,par1);
        removeZeros(tmp);
        unsigned lEn=strlen(tmp);
        pointer=createData(lEn+defaultBufLen);
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=tmp[i];
    }

    ~String(){
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer=NULL;
    }

    //operator const char*()const{
    //	return c_str();
    //}

    String& operator = (const String& par1){
        par1.pointer->refCount++;
        if(--pointer->refCount==0){
            delete pointer;
        }
        pointer = par1.pointer;
        return *this;
    }
    String& operator = (const char* par1){
        unsigned lEn=strlen(par1);

        if(pointer->refCount>1||lEn>pointer->bufLen){
            if(--pointer->refCount==0){
                delete pointer;
            }
            pointer=createData(lEn+defaultBufLen);
        }
        pointer->rawLen=lEn;
        for(unsigned i=0;i<pointer->rawLen+1;i++)pointer->rawChar[i]=par1[i];
        return *this;
    }
    friend String	operator + (const String& par1, const String& par2){
        return String(par1)+=par2;
    }
    String& operator = (const char& par);
    String& operator = (float par1);
    String& operator = (int par1);

    const char *showReason() const {
        return "Exception in String class.";
    }
    const char& operator ()(const int par1)const{
        if((unsigned)par1>=(unsigned)pointer->rawLen){
            throw this;
        }
        return pointer->rawChar[par1];
    }
    /*
    operator const String ()const{
        return *this;
    }
    */
    const char* c_str()const{
        return pointer->rawChar;
    }
    int getBufLength()const{
        return pointer->bufLen;
    }




////////////////////////////
friend bool	operator ==(const String &parA, const String &parB){
    return equals(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator ==(const char *parA, const String &parB){
    return equals(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator ==(const String& parA, const char* parB){
    return equals(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator !=(const String &parA, const String &parB){
    return notEquals(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator !=(const char *parA, const String &parB){
    return notEquals(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator !=(const String& parA, const char* parB){
    return notEquals(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator <(const String &parA, const String &parB){
    return lessThan(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator <(const char *parA, const String &parB){
    return lessThan(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator <(const String& parA, const char* parB){
    return lessThan(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator <=(const String &parA, const String &parB){
    return lessEqual(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator <=(const char *parA, const String &parB){
    return lessEqual(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator <=(const String& parA, const char* parB){
    return lessEqual(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator >(const String &parA, const String &parB){
    return greaterThan(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator >(const char *parA, const String &parB){
    return greaterThan(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator >(const String& parA, const char* parB){
    return greaterThan(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

friend bool	operator >=(const String &parA, const String &parB){
    return greaterEqual(parA.pointer->rawChar, parB.pointer->rawChar, parA.length, parB.length);
}
friend bool	operator >=(const char *parA, const String &parB){
    return greaterEqual(parA, parB.pointer->rawChar, strlen(parA), parB.length);
}
friend bool	operator >=(const String& parA, const char* parB){
    return greaterEqual(parA.pointer->rawChar, parB, parA.length, strlen(parB));
}

    bool greaterThanCaseInsensitive(const String& par)const;

    void requestMoreIfNeeded(unsigned more){
        unsigned i;
        if(pointer->refCount>1||pointer->rawLen+more>pointer->bufLen){
            Data* newPointer=createData(pointer->rawLen+more+defaultBufLen, pointer->rawLen);
            for(i=0;i<pointer->rawLen+1;i++){
                newPointer->rawChar[i]=pointer->rawChar[i];
            }
            if(--pointer->refCount==0){
                delete pointer;
            }
            pointer=newPointer;
        }
    }
    void catHex(unsigned int& par){
        requestMoreIfNeeded(20);
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,"%X",par);
        pointer->rawLen+=strlen(p);
    }
/*
    void cat(int& par){
        requestMoreIfNeeded(20);
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,"%i",par);
        pointer->rawLen+=strlen(p);
    }
    void cat(unsigned int& par){
        requestMoreIfNeeded(20);
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,"%u",par);
        pointer->rawLen+=strlen(p);
    }
    void cat(bool& par){
        if(par){
            *this+="TRUE";
        }
        else{
            *this+="FALSE";
        }
    }
    void cat(float& par){
        requestMoreIfNeeded(20);
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par);
        removeZeros(p);
        pointer->rawLen+=strlen(p);
    }
    void cat(double& par){
        requestMoreIfNeeded(20);
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par);
        removeZeros(p);
        pointer->rawLen+=strlen(p);
    }
    */
    template<typename T>void toValue(T& par){
        int i=0;
        par.getField(*this, i);
    }
    void toValue(bool& par){
        int i=0;
        par=getBool(*this, i);
    }
    void toValue(float& par){
        int i=0;
        par=getFloat(*this, i);
    }
    void toValue(double& par){
        int i=0;
        par=getFloat(*this, i);
    }
    void toValue(int& par){
        int i=0;
        par=getFloat(*this, i);
    }

    int hourMinSecFormatToValue(); //return seconds

    void cat(const String& par1);
    void cat(const bool& par){
        if(par){
            *this+="TRUE";
        }
        else{
            *this+="FALSE";
        }
    }
    void cat(const char& par);
    void cat(const unsigned char& par);
    void cat(const int& par){
        requestMoreIfNeeded(amountToRequest<int>());
        char* p = pointer->rawChar+pointer->rawLen;
        sprintf(p,"%i",par);
        pointer->rawLen += strlen(p);
    }
    void cat(const unsigned int& par){
        requestMoreIfNeeded(amountToRequest<unsigned int>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,"%u",par);
        pointer->rawLen+=strlen(p);
    }
    void cat(const float& par){
        requestMoreIfNeeded(amountToRequest<float>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par);
        removeZeros(p);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Image&){
        throw;
    }
    void cat(const double& par){
        requestMoreIfNeeded(amountToRequest<double>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par);
        removeZeros(p);
        pointer->rawLen+=strlen(p);
    }
    void cat(const char* par1){
        int length = strlen(par1);
        requestMoreIfNeeded(length);
        for(int i=0;i<length+1;i++){
            pointer->rawChar[pointer->rawLen+i]=par1[i];
        }
        pointer->rawLen+=length;
    }
    void cat(const Vec2i& par){
        requestMoreIfNeeded(amountToRequest<Vec2i>());

        char* p = pointer->rawChar+pointer->rawLen;
        sprintf(p,"%i %i", par.x, par.y);
        pointer->rawLen += strlen(p);

    }

    void cat(const Vec3f& par){
        requestMoreIfNeeded(amountToRequest<Vec3f>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.z);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Vec3d& par){
        requestMoreIfNeeded(amountToRequest<Vec3d>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.z);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Vec2f& par){
        requestMoreIfNeeded(amountToRequest<Vec2f>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Rotation& par){
        requestMoreIfNeeded(amountToRequest<Rotation>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.x);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.y);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.z);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.angle);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }
    void cat(const Color3f& par){
        requestMoreIfNeeded(amountToRequest<Color3f>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.r);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.g);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.b);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }

    void catColor4f(const Color3f& par){
        requestMoreIfNeeded(amountToRequest<Rotation>());
        char* p=pointer->rawChar+pointer->rawLen;
        sprintf(p,floatFormat,par.r);
        removeZeros(p);
        int length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.g);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,par.b);
        removeZeros(p+1);
        length=strlen(p);
        p+=length;
        pointer->rawLen+=length;
        sprintf(p,floatFormat2,1.0);
        removeZeros(p+1);
        pointer->rawLen+=strlen(p);
    }

    void cat(const S_Triangle& par);

    String& operator+=(const char* par){
        cat(par);
        return *this;
    }
    template<class T> String& operator+=(const T& par){
        cat(par);
        return *this;
    }
    template<class T> void cat(const M_<T>& par);
    template<class T> String& operator+=(const M_<T>& par){
        cat(par);
        return *this;
    }
    String& addEqTriCoordIndex(const M_Int& par){
        requestMoreIfNeeded(par.length/3*(8*3+5)+5);
        char* p=pointer->rawChar+pointer->rawLen;
        *p='[';
        p++;
        for(int i=0;i<par.length;i+=3){
            sprintf(p,"%i %i %i -1\n",par[i],par[i+1],par[i+2]);
            p+=strlen(p);
        }
        *p=']';
        p++;
        *p=0;
        pointer->rawLen=p-pointer->rawChar;
        return *this;
    }
    /*
    String& operator +=(const M_Float& par){
        requestMoreIfNeeded(8*par.length+5);
        char* p=pointer->rawChar+pointer->rawLen;
        *p='[';
        p++;

        for(int i=0;i<par.length;i++){
            sprintf(p,floatFormat2,par[i]);
            removeZeros(++p);
            p+=strlen(p);
        }
        *p=']';
        p++;
        *p=0;
        pointer->rawLen=p-pointer->rawChar;
        return *this;
    }
    String& operator +=(const M_Vec3f& par);
    String& operator +=(const M_Vec2f& par){
        requestMoreIfNeeded(10*2*par.length+5);
        char* p=pointer->rawChar+pointer->rawLen;
        *p='[';
        p++;

        for(int i=0;i<par.length;i++){
            sprintf(p,floatFormat2,par[i].x);
            removeZeros(++p);
            p+=strlen(p);
            sprintf(p,floatFormat2,par[i].y);
            removeZeros(++p);
            p+=strlen(p);
        }
        *p=']';
        p++;
        *p=0;
        pointer->rawLen=p-pointer->rawChar;
        return *this;
    }
    String& operator +=(const M_Color3f& par){
        requestMoreIfNeeded(10*3*par.length+5);
        char* p=pointer->rawChar+pointer->rawLen;
        *p='[';
        p++;

        for(int i=0;i<par.length;i++){
            sprintf(p,floatFormat2,par[i].r);
            removeZeros(++p);
            p+=strlen(p);
            sprintf(p,floatFormat2,par[i].g);
            removeZeros(++p);
            p+=strlen(p);
            sprintf(p,floatFormat2,par[i].b);
            removeZeros(++p);
            p+=strlen(p);
        }
        *p=']';
        p++;
        *p=0;
        pointer->rawLen=p-pointer->rawChar;
        return *this;
    }
    */


/*
    String ( M_Int& par){
        *this=par.toString();
    }
    String ( Vec3f& par){
        *this=par.toString();
    }
    String ( Rotation& par){
        *this=par.toString();
    }
    String ( Color3f& par){
        *this=par.toString();
    }
    String( M_Vec2f& par){
        *this=par.toString();
    }
    String( M_Vec3f& par){
        *this=par.toString();
    }
    String( M_Color3f& par){
        *this=par.toString();
    }
*/

    void insert(const char& ch, const int& index){
        (*this)+=' ';
        unsigned int i=pointer->rawLen;
        for(; --i>(unsigned)index;){
            (*this)[i]=(*this)[i-1];
        }
        (*this)[i]=ch;
    }
    void deleteChar(const int& index){
        *this=replace(index, 1, "");
    }
    const String substr(const int& index, const int desiredLen=0x7fffffff) const {
        unsigned i;
        unsigned realLen=length-(const unsigned)index; //potential length stage
        if((int)realLen < 0){
			throw this;
		}
        if(realLen>(const unsigned)desiredLen){
            realLen=desiredLen;
        }
        Data* newPointer=createData(realLen+defaultBufLen);
        newPointer->rawLen=realLen;
        for(i=0;i<realLen;i++){
            newPointer->rawChar[i]=pointer->rawChar[index+i];
        }
        newPointer->rawChar[i]=0;
        return newPointer;
    }
    const String substring(const int& startChar, const int endChar=-1) const {
        unsigned strLength,i;
        if(endChar==-1){
            strLength=pointer->rawLen-startChar;
        }
        else{
            strLength=endChar-startChar+1;
        }
        if((unsigned)startChar+strLength>pointer->rawLen){
			throw this;
		}

        Data* newPointer=createData(strLength+defaultBufLen);
        newPointer->rawLen=strLength;
        for(i=0;i<strLength;i++){
            newPointer->rawChar[i]=pointer->rawChar[startChar+i];
        }
        newPointer->rawChar[i]=0;
        return newPointer;
    }
    String replace(const int& index, const int& desiredLen, const String strPar) const{
        String str=substr(0,index);
        str+=strPar;
        unsigned newIndex=index+desiredLen;
        if(newIndex<(unsigned)length){
            str+=substr(newIndex);
        }
        return str;
    }
    char& operator [](const int& par){
        if((unsigned)par>=(unsigned)pointer->rawLen){
            throw this;
        }
        if(pointer->refCount > 1){
            Data* newPointer = createData(pointer->bufLen, pointer->rawLen);
            unsigned i = 0;
            do{
                newPointer->rawChar[i] = pointer->rawChar[i];
            }while(i++ < pointer->rawLen);

            pointer = newPointer;
        }
        return pointer->rawChar[par];
    }
    const char& operator [](const int& par)const{
        if((unsigned)par>=(unsigned)pointer->rawLen){
            throw this;
        }
        return pointer->rawChar[par];
    }
/*
    int fromText(char*& p,int i=0);
    int readWord(char* p){
        for(length=0;p[length]>' ';++length){
        }
        pointer=new SFChar[length+1];
        for(int i=0;i<length;i++,p++){
            pointer[i]=*p;
        }
        pointer[i]=0;
        return length;
    }
    */
    int indexOf (String searchString, int position=0)const;

    void getField(const String& str, int& i);

    String toUpperCase()const{
        String str;
        str.pointer = createData(pointer->bufLen, pointer->rawLen);
        const char* s = pointer->rawChar;
        char* d = str.pointer->rawChar;
        for(int i = pointer->rawLen + 1; --i >= 0;){
            d[i] = upperCaseChar(s[i]);
        }
        return str;
    }

    String toLowerCase()const{
        String str;
        str.pointer = createData(pointer->bufLen, pointer->rawLen);
        const char* s = pointer->rawChar;
        char* d = str.pointer->rawChar;
        for(int i = pointer->rawLen + 1; --i >= 0;){
            d[i] = lowerCaseChar(s[i]);
        }
        return str;
    }
void toStrCpp(String& str)const;
void toString(String& str);
void endOfName(int& index)const;
void endOfCppLabel(int& index)const;
String extractName(int& i)const;
String extractCppLabel(int& i)const;
//String extractCppOperator(int& i)const;

void stb(String& dest, int& position)const;
void stw(String& dest, int& position)const;

void stb(int& position)const; //searchTillBlack treats coma as white
void stbCpp(int& position)const; //searchTillBlack includes coma as black
void stbnlCpp(int& position)const;//searchTillBlack or new line includes coma as black
void stbc(int& position)const;//searchTillBlackComment
void stw(int& position)const; //searchTillWhite
void stwo(int& position)const; //searchTillWhiteOrOperator //try not to use this
void stwo2(int& position)const; //searchTillWhiteOrOperator //use this
void stw_stb(int& position)const; //searchTillWhite then searchTillBlack
void stwo_stb(int& position)const; //searchTillWhite or Operator then search till black
void stnl(int& position)const;//searchTill new line
//void stNonCppOperator(int& position)const; //searchTillNonOperator (C++)
int getLengthOfCppOperator(int position)const;
void endOfNum(int& position)const;


bool cmpKWord(const char* str,int& index)const;
bool cmpKWord(const String& str,int& index)const;
bool cmpCppWord(const char* str,int& index)const;
bool cmpCppWord(const String& str,int& index)const;
bool findWord( const String str, int& i)const;
String fromQuotes( int& position)const;
String getWord(int& position)const;
String getWord2(int& position)const;


    void write(FILE* file)const{
        fwrite(pointer->rawChar,1,pointer->rawLen,file);
    }
    void write(FILE* stream, char* name)const;
    bool write(String fileName)const;
    //bool read( String fileName);
//private:

    Length length;
private:
    Data* pointer;
};



String readFile(const String filename);

class Word {
public:
    virtual void toString(String& str)=0;
    virtual unsigned getColor()=0;
};
class W_Global;

class Matrix3{
    float m[3][3];
public:
    Matrix3(){
        m[0][0]=m[1][1]=m[2][2]= 1.0f;
        m[0][1]=m[0][2]= 0.0f;
        m[1][0]=m[1][2]= 0.0f;
        m[2][0]=m[2][1]= 0.0f;
    }
    Matrix3(float m00, float m01, float m02, float m10, float m11, float m12, float m20, float m21, float m22){
        m[0][0]=m00;
        m[0][1]=m01;
        m[0][2]=m02;

        m[1][0]=m10;
        m[1][1]=m11;
        m[1][2]=m12;

        m[2][0]=m20;
        m[2][1]=m21;
        m[2][2]=m22;
    }
    Vec2f operator * (Vec2f vec)const{
        Vec2f tmp;
        tmp.x = vec.x*m[0][0]+vec.y*m[1][0]+m[2][0];
        tmp.y = vec.x*m[0][1]+vec.y*m[1][1]+m[2][1];
        return tmp;
    }

    Matrix3 operator *(Matrix3 mat)const{
        Matrix3 tmp;
        for (int i=0; i<3; i++){
            for (int j=0; j<3; j++){
                tmp.m[i][j]=0;
                for (int k=0; k<3; k++){
                    tmp.m[i][j]+=this->m[i][k] * mat.m[k][j];
                }
            }
        }
        return tmp;
    }
    Matrix3 operator *=(Matrix3 mat){
        *this=*this*mat;
        return *this;
    }
};
inline Matrix3 scaleMatrix(Vec2f par){
    return Matrix3(
        par.x, 0, 0,
        0, par.y, 0,
        0, 0, 1
        );
}
inline Matrix3 translationMatrix(Vec2f par){
    return Matrix3(
        1, 0, 0,
        0, 1, 0,
        par.x, par.y, 1
        );
}
inline Matrix3 rotationMatrix(float angle){

    float cosine = cos(angle);
    float sine = sin(angle);

    return Matrix3(
        cosine, sine, 0,
        -sine, cosine, 0,
        0, 0, 1
        );
}



class Matrix{
public:
    union {
        struct {
            float        f11, f12, f13, f14;
            float        f21, f22, f23, f24;
            float        f31, f32, f33, f34;
            float        f41, f42, f43, f44;
        };
        float m[4][4];
    };
    Matrix(){
        f11 = f22 = f33 = f44 = 1.0f;
        f12 = f13 = f14 = f41 = 0.0f;
        f21 = f23 = f24 = f42 = 0.0f;
        f31 = f32 = f34 = f43 = 0.0f;
    }
    Matrix(float _f11, float _f12, float _f13, float _f14, float _f21, float _f22, float _f23, float _f24, float _f31, float _f32, float _f33, float _f34, float _f41, float _f42, float _f43, float _f44){
        f11=_f11;	f12=_f12;	f13=_f13;	f14=_f14;
        f21=_f21;	f22=_f22;	f23=_f23;	f24=_f24;
        f31=_f31;	f32=_f32;	f33=_f33;	f34=_f34;
        f41=_f41;	f42=_f42;	f43=_f43;	f44=_f44;
    }
    inline const float& operator()(int row, int column) const{
        return m[row][column];
    }
    inline float& operator()(int row, int column){
        return m[row][column];
    }


    Vec3f multVecMatrix(Vec3f vec)const;
    Vec3f operator * (Vec3f vec)const;

    //Matrix& operator(Matrix mat)const;
    void scale(Vec3f& sc);
    Matrix operator *(Matrix mat)const;
    Matrix operator *=(Matrix mat);
    //Matrix multLeft(Matrix mat);
    Matrix multRight(Matrix mat);
    void setTransform(const Vec3f translation, const Rotation rotation=Rotation(1,0,0,0), const Vec3f scaleFactor=Vec3f(1, 1, 1));
    //inline void setTransform(const SF<Vec3f,	crap>& translation, const SF<Rotation,	crap>& rotation, const SF<Vec3f,	crap>& scaleFactor);
    Rotation getRotation();
    void toString(String& str);
};


Matrix rotationMatrix(Rotation rot);
Matrix translationMatrix(Vec3f tran);
Matrix scaleMatrix(Vec3f sc);


class Anim;
class MField;

enum AccessType {
    inputOnly		=	0x01,
    outputOnly		=	0x02,
    inputOutput		=	0x03,
    initializeOnly	=	0x04
};
class SceneTree;
typedef Ptr <Node> NodePtr;
template<class T, class S> T dynamicCast(Ptr<S>& par){
    return dynamic_cast<T>(par.p);
}

class Field;
class S_Route {
public:
    S_Route():
        toNode(0),
        toFieldIdx(0),
        toFieldPtr(0)
    {};
    inline S_Route( Node* i_toNode, Field* i_toFieldPtr);
    S_Route(const String &str,int &i);
    S_Route& operator=(const S_Route& par){
        toNode=par.toNode;
        toFieldIdx=par.toFieldIdx;
        toFieldPtr=par.toFieldPtr;
        return *this;
    }

    S_Route( Node* fromNode, const int fromFieldIdx, Node* i_toNode, const int i_toFieldIdx);

    Node* toNode;

    int toFieldIdx;

    Field* toFieldPtr;
    void toString(String& str);
};

class Field {
public:
    String getTypeStr()const;
    virtual FieldType getType()const=0;
    virtual bool& isRef()=0;
    virtual void read(const String& str, int& i)=0;
    virtual void operator+=(S_Route& par)=0;
    virtual void toStrCpp(String& str)const=0;
    virtual inline void resize(int newLen)=0;
    virtual inline int size()=0;
    virtual void*& refRawPtr()=0;
    virtual void toString(String& str)=0;
    virtual void toString(String& str, Field *protoField, bool newList=true)=0;
    virtual void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList=true)=0;

    virtual void connect(Node* node, Node* protoInterface, Field *protoField)=0;
    virtual bool testRouteTo(const Field* destField)const=0;


    virtual const M_Route& getRoute()const=0;
    virtual M_Route& refRoute()=0;
    virtual int countItems(const String& str, int i)=0;
    virtual void set(const Field* par)=0;
    virtual void decodeAssign(Function& f, const String& str, int& i)=0;
    virtual void decodeAddAssign(Function& f, const String& str, int& i)=0;
    virtual int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr)=0;

    //virtual void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr)=0;
#ifdef SEAMLESS3DGUI
//	virtual void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index,const Properties& properties)=0;
    virtual FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win)=0;
    virtual void addValFromStr(const String& str)=0;
    virtual void addValSameAsLast()=0;
#endif
};
/*
template<typename T>
class SF : public Field {
public:
    T raw;
    M_Route route;
    friend MField;
    bool is;
    inline void addValFromStr(const String& str);
    inline void addValSameAsLast();
    inline int countItems(const String& str, int i);

    SF(const T& par):
        raw(par),
        is(false)
    {};
    FieldType getType();
    T& refValue(){
        return raw;
    }
    const T& refValue()const{
        return raw;
    }
    const T& getValue()const{
        return raw;
    }
    virtual void set(Field* par){
        *this=dynamic_cast<SF*>(par)->refValue();
    }
    virtual void operator=(const SF& par){
        raw=par.raw;
    }
    virtual void operator=(const T& par){
        raw=par;
    }
    operator const T& () const{
        return raw;
    }
//	operator T& () const {// causes ROUTES to be qued
//		return *raw;
//	}
//	operator SV ();
    bool& isRef(){return is;}

    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField){
        S_Route( protoInterface, protoInterface->getFieldIndex(protoField), node, node->getFieldIndex(this));
        is=true;
    }
    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    String toStrCpp()const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    String toCPPString(const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const T& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SF*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)raw;
    }

bool operator !(){
    return refValue()==false;
}
//bool operator >=( T par){
//	return refValue()>=par;
//}
//bool operator >( T par){
//	return refValue()>par;
//}
//bool operator <=( T par){
//	return refValue()<=par;
//}
//bool operator <( T par){
//	return refValue()<par;
//}
//T operator -( SV par){
//	return refValue()-par;
//}
T operator -( T par){
    return refValue()-par;
}
//T operator +( SV par){
//	return refValue()+par;
//}
//T operator +( T par){
//	return refValue()+par;
//}
//T operator /( SV par){
//	return refValue()/par;
//}
T operator /( T par){
    return refValue()/par;
}
//T operator *( const SV par){
//	return refValue()*par;
//}
//T operator *( SV& par){
//	return refValue()*par;
//}
T operator *( T& par){
    return refValue()*par;
}


//inline	const SV& operator [](const int& i)const;
//inline	T& operator +=(SF<T, SV, type>& par);

inline	T& operator +=(T& par){
    raw+=par;
    return raw;
}
//inline	T& operator +=(SV par){
//	raw+=par;
//	return raw;
//}
inline	T& operator -=(T& par){
    raw-=par;
    return raw;
}
//inline	T& operator -=(SV& par){
//	raw-=par;
//	return raw;
//}
inline	const T operator -()const{
    return -refValue();
}
//inline	T& operator +=(const NodePtr par);
String toXMLString(bool newList, String containerField);

#ifdef SEAMLESS3DGUI
    Button* setUpControl(Node& node, Field& field, const Properties& properties, const int &index, int yPos);
    void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};
*/
//typedef SF<bool>		SFBool;
//typedef SF<Color3f>	SFColor;
//typedef SF<float>	SFFloat;
//typedef SF<Image>	SFImage;
//typedef SF<int>	SFInt32;
//typedef SF<NodePtr,	void*>		SFNode;
//typedef SF<Rotation>	SFRotation;
//typedef SF<String>	SFString;
//typedef SF<double>	SFTime;
//typedef SF<Vec2f,	crap>	SFVec2f;
//typedef SF<Vec3f,	crap>	SFVec3f;




class Render;
struct COLORVERTEX_1WEIGHT;
struct TEXTUREVERTEX_1WEIGHT;
struct COLORVERTEX_0WEIGHT;

typedef FD<S_Bool,Single>SFBool;
typedef FD<Color3f,Single>SFColor;
typedef FD<S_Float,Single>SFFloat;
typedef FD<Image,Single>SFImage;
typedef FD<S_Int32,Single>SFInt32;
typedef FD<NodePtr,Single>SFNode;
typedef FD<Rotation,Single>SFRotation;
typedef FD<String,Single>SFString;
typedef FD<double,Single>SFTime;
typedef FD<Vec2f,Single>SFVec2f;
typedef FD<Vec3f,Single>SFVec3f;

typedef SFTime SFDouble;


template<class T> class FD<T,Single> : public Field {
//template <class T, class S> class FD : public Field {
public:
    T raw;
    M_Route route;
    friend MField;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();

    int countItems(const String& str, int i);

    FD<T,Single>(const T& par):
        raw(par),
        is(false)
    {};
    FieldType getType()const;
    T& refValue(){
        return raw;
    }
    const T& refValue()const{
        return raw;
    }
    const T& getValue()const{
        return raw;
    }
    virtual void set(const Field* par){
        *this = dynamic_cast<const FD<T,Single>*>(par)->refValue();
    }
    virtual void operator=(const FD<T,Single>& par){
        raw=par.raw;
    }
    virtual void operator=(const T& par){
        raw=par;
    }
    operator const T& () const{
        return raw;
    }
    bool& isRef(){return is;}

    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);

    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const T& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<FD<T,Single>*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)raw;
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};

template<> class FD<String,Single> : public Field {
public:
    String raw;
    M_Route route;
    friend MField;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const String& par):
        raw(par),
        is(false)
    {};
    FieldType getType()const;
    String& refValue(){
        return raw;
    }
    const String& refValue()const{
        return raw;
    }
    const String& getValue()const{
        return raw;
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFString*>(par)->refValue();
    }
    virtual void operator=(const SFString& par){
        raw=par.raw;
    }
    virtual void operator=(const String& par){
        raw=par;
    }
    operator const String& () const{
        return raw;
    }
    bool& isRef(){return is;}

    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);
    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const String& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SFString*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)raw;
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};
/*
template<> class FD<double,Single> : public Field {
public:
    double raw;
    M_Route route;
    friend MField;
    bool is;
    inline void addValFromStr(const String& str);
    inline void addValSameAsLast();
    inline int countItems(const String& str, int i);

    SFTime(const double& par):
        raw(par),
        is(false)
    {};
    FieldType getType();
    double& refValue(){
        return raw;
    }
    const double& refValue()const{
        return raw;
    }
    const double& getValue()const{
        return raw;
    }
    virtual void set(Field* par){
        *this=dynamic_cast<SFTime*>(par)->refValue();
    }
    virtual void operator=(const SFTime& par){
        raw=par.raw;
    }
    virtual void operator=(const double& par){
        raw=par;
    }
    operator const double& () const{
        return raw;
    }
    bool& isRef(){return is;}

    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    inline void connect(Node* node, Node* protoInterface, Field *protoField);
    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    String toStrCpp()const;
    inline virtual String toString();
    int size();
    virtual void resize(int newLen);
    String toString(Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const double& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SFTime*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)raw;
    }

void toXMLString(String& str, bool newList, String containerField);
void decodeAssign(Function& f, const String& str, int& i);

#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int &index, int yPos);
    void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};
*/

template<> class FD<Vec2f,Single> : public Field {
public:
    const float x;
    const float y;

    Vec2f	operator /(const float& v)const{
        return (Vec2f(x/v, y/v));
    }
    Vec2f	operator +(const Vec2f& v)const{
        return (Vec2f(x+v.x, y+v.y));
    }
    Vec2f	operator -(const Vec2f& v)const{
        return (Vec2f(x-v.x, y-v.y));
    }
    Vec2f	operator -()const{
        return (Vec2f(-x, -y));
    }

    Vec2f	operator *(const Vec2f& par)const{
        return (Vec2f(x*par.x, y*par.y));
    }
    Vec2f	operator *(float par)const{
        return (Vec2f(x*par, y*par));
    }
    friend Vec2f operator *(const float& par0, const Vec2f& par1){
        return (Vec2f(par0*par1.x, par0*par1.y));
    }

    Vec2f	operator /(const Vec2f& v)const{
        return (Vec2f(x/v.x, y/v.y));
    }

    friend bool	operator !=(const Vec2f& par1, const SFVec2f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Vec2f& par1, const SFVec2f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y
            )
            return true;
        else return false;
    }
    Vec2f normalize();
    float length()const;
    void getField(const String& str, int& i);
//////////////////////////////////////////////////
    M_Route route;
    friend MField;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Vec2f& par):
        x(par.x),
        y(par.y),
        is(false)
    {};
    FieldType getType()const;
    Vec2f& refValue(){
        return *(Vec2f*)&x;
    }
    const Vec2f& refValue()const{
        return *(Vec2f*)&x;
    }
    const Vec2f& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const FD*>(par)->refValue();
    }
    virtual void operator=(const FD& par){
        refValue()=par;
    }
    virtual void operator=(const Vec2f& par){
        refValue()=par;
    }
    operator const Vec2f& () const{
        return refValue();
    }
    bool& isRef(){return is;}

    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);
    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field* protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const Vec2f& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SFVec2f*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)refValue();
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};

template<> class FD<Color3f,Single> : public Field {

public:
    float r;
    float g;
    float b;

    Color3f	operator /(const float& v)const{
        return (Color3f(r/v, g/v, b/v));
    }
    Color3f	operator +(const Color3f& v)const{
        return (Color3f(r+v.r, g+v.g, b+v.b));
    }
    Color3f	operator -(const Color3f& v)const{
        return (Color3f(r-v.r, g-v.g, b-v.b));
    }
    Color3f	operator -()const{
        return (Color3f(-r, -g, -b));
    }

    Color3f	operator *(const Color3f& par)const{
        return (Color3f(r*par.r, g*par.g, b*par.b));
    }
    Color3f	operator *(float par)const{
        return (Color3f(r*par, g*par, b*par));
    }
    friend Color3f operator *(const float& par0, const Color3f& par1){
        return (Color3f(par0*par1.r, par0*par1.g, par0*par1.b ));
    }

    Color3f	operator /(const Color3f& v)const{
        return (Color3f(r/v.r, g/v.g, b/v.b));
    }

    friend bool	operator !=(const Color3f& par1, const SFColor& par2){
        if(
            par1.r==par2.r&&
            par1.g==par2.g&&
            par1.b==par2.b
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Color3f& par1, const SFColor& par2){
        if(
            par1.r==par2.r&&
            par1.g==par2.g&&
            par1.b==par2.b
            )
            return true;
        else return false;
    }
    float dot(const Color3f& v) const{
        return (r * v.r +	g * v.g +  b * v.b);
    }
    Color3f cross ( const Color3f& v)const{
       return Color3f(g * v.b - b * v.g,
                    b * v.r - r * v.b,
                    r * v.g - g * v.r);
    }

    Color3f operator *=(const Matrix matrix);
    Color3f normalize();
    float length()const;
    void getField(const String& str, int& i);
//////////////////////////////////////////////////
    M_Route route;
    friend MField;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Color3f& par):
        r(par.r),
        g(par.g),
        b(par.b),
        is(false)
    {};
    FieldType getType()const;
    Color3f& refValue(){
        return *(Color3f*)&r;
    }
    const Color3f& refValue()const{
        return *(Color3f*)&r;
    }
    const Color3f& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFColor*>(par)->refValue();
    }
    virtual void operator=(const SFColor& par){
        refValue()=par;
    }
    virtual void operator=(const Color3f& par){
        refValue()=par;
    }
    operator Color3f& (){
        return refValue();
    }
    operator const Color3f& () const{
        return refValue();
    }
    bool& isRef(){return is;}

    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);
    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    void toPOVRayString(String& str);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const Color3f& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SFColor*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)refValue();
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};

template<> class FD<Vec3f,Single> : public Field {

public:
    float x;
    float y;
    float z;

    Vec3f	operator /(const float& v)const{
        return (Vec3f(x/v, y/v, z/v));
    }
    Vec3f	operator +(const Vec3f& v)const{
        return (Vec3f(x+v.x, y+v.y, z+v.z));
    }
    Vec3f	operator -(const Vec3f& v)const{
        return (Vec3f(x-v.x, y-v.y, z-v.z));
    }
    Vec3f	operator -()const{
        return (Vec3f(-x, -y, -z));
    }

    Vec3f	operator *(const Vec3f& par)const{
        return (Vec3f(x*par.x, y*par.y, z*par.z));
    }
    Vec3f	operator *(float par)const{
        return (Vec3f(x*par, y*par, z*par));
    }
    friend Vec3f operator *(const float& par0, const Vec3f& par1){
        return (Vec3f(par0*par1.x, par0*par1.y, par0*par1.z ));
    }

    Vec3f	operator /(const Vec3f& v)const{
        return (Vec3f(x/v.x, y/v.y, z/v.z));
    }

    friend bool	operator !=(const Vec3f& par1, const SFVec3f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return false;
        else return true;
    }
    friend bool	operator ==(const Vec3f& par1, const SFVec3f& par2){
        if(
            par1.x==par2.x&&
            par1.y==par2.y&&
            par1.z==par2.z
            )
            return true;
        else return false;
    }
    float dot(const Vec3f& v) const{
        return (x * v.x +	y * v.y +  z * v.z);
    }
    Vec3f cross ( const Vec3f& v)const{
       return Vec3f(y * v.z - z * v.y,
                    z * v.x - x * v.z,
                    x * v.y - y * v.x);
    }

    Vec3f operator *=(const Matrix matrix);
    Vec3f normalize();
    float length()const;
    void getField(const String& str, int& i);
//////////////////////////////////////////////////
    M_Route route;
    friend MField;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Vec3f& par):
        x(par.x),
        y(par.y),
        z(par.z),
        is(false)
    {};
    FieldType getType()const;
    Vec3f& refValue(){
        return *(Vec3f*)&x;
    }
    const Vec3f& refValue()const{
        return *(Vec3f*)&x;
    }
    const Vec3f& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFVec3f*>(par)->refValue();
    }
    virtual void operator=(const SFVec3f& par){
        refValue()=par;
    }
    virtual void operator=(const Vec3f& par){
        refValue()=par;
    }
    operator Vec3f& (){
        return refValue();
    }
    operator const Vec3f& () const{
        return refValue();
    }
    bool& isRef(){return is;}

    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);
    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    void toPOVRayString(String& str);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const Vec3f& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SFVec3f*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)refValue();
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};

template<> class FD<Rotation,Single> : public Field {

public:
    float x;
    float y;
    float z;
    float angle;
Rotation operator -()const{
    return -refValue();
}
//////////////////////////////////////////////////
    M_Route route;
    friend MField;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const Rotation& par):
        x(par.x),
        y(par.y),
        z(par.z),
        angle(par.angle),
        is(false)
    {};
    FieldType getType()const;
    Rotation& refValue(){
        return *(Rotation*)&x;
    }
    const Rotation& refValue()const{
        return *(Rotation*)&x;
    }
    const Rotation& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFRotation*>(par)->refValue();
    }
    virtual void operator=(const SFRotation& par){
        refValue()=par;
        for(int i=route.length; --i>=0;){*dynamic_cast<SFRotation*>(route[i].toFieldPtr)=par;}
    }
    virtual void operator=(const Rotation& par){
        refValue()=par;
        for(int i=route.length; --i>=0;){*dynamic_cast<SFRotation*>(route[i].toFieldPtr)=par;}
    }
    Rotation	operator *(const Rotation& par1)const{
        return refValue() * par1;
    }
    operator Rotation& () {
        return refValue();
    }
    operator const Rotation& () const{
        return refValue();
    }
    bool& isRef(){return is;}
    void read(const String& str, int& i);


    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);
    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const Rotation& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SFRotation*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)refValue();
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};


template<class T> class FD<T,Multiple> : public Field {
public:
class Length{
    friend FD;
protected:
    unsigned bufLen;
    unsigned rawLen;
    T* pointer;
    M_Route route;
    friend MField;
    bool is;
public:
    Length():
        rawLen(0), bufLen(0), pointer(0)
    {
    }
    Length(const M_<T>& par):
        rawLen(par.length),
        bufLen(par.length*2),
        pointer(new T[bufLen]),
        is(false)
    {
        unsigned i;
        for(i=0;i<rawLen;i++)pointer[i]=par[i];
    }
    ~Length(){
        delete[] pointer;
    }
    operator int ()const{
        return rawLen;
    }
};


public:
    FD operator +(const T& par)const{
        FD m=*this;
        m+=par;
        return m;
    }
    FD operator +(const M_<T>& par)const{
        FD m=*this;
        m+=par;
        return m;
    }
    const char *showReason() const {
        return "Exception in FD class.";
    }
    const T& operator [](const int& index)const{
        if((unsigned)index>=length.rawLen){
            throw this;
        }
        return length.pointer[index];
    }
    const T* getArray()const{
        return length.pointer;
    }
    T* getArray(){
        return length.pointer;
    }
    bool operator ==(const M_<T>& par)const{
        if(length != par.length){
            return false;
        }
        for(int i=length; --i>=0;){
            if(length.pointer[i]!=par[i]){
                return false;
            }
        }
        return true;
    }
    bool operator !=(const M_<T>& par)const{
        if(length != par.length){
            return true;
        }
        for(int i=length; --i>=0;){
            if(length.pointer[i]!=par[i]){
                return true;
            }
        }
        return false;
    }
    void toString(String& str)const;
    void getField(const String& str,int& i);

    void toMNodeString(String& str, bool newList=true);
    void removeNode(T par1);
    void removeNode(FD& par1);
    Art*& refPart(int i)const;
    int& refIndex(int i)const;
    int& refTexIndex(int i)const;

    void rotate(Rotation rot,int offset=0);
    void rotate( float x, float y, float z, float angle, int offset=0);
    void translate(T tran,int offset=0);
    void translate( float x, float y, float z, int offset=0);
    void scale(float xsc,float ysc,float zsc);
    void translateToZero(int offset,int numOf, Rotation& center);

    void multMatrixLeft(Matrix& matrix);
    //void paint( int& yPos);
    int getIndex(String str);
    Length length;
//////////////////



    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

    FD(const M_<T>& par):
        length(par)
    {};
    FieldType getType()const;
    M_<T>& refValue(){
        return *(M_<T>*)&length.bufLen;
    }
    const M_<T>& refValue()const{
        return *(M_<T>*)&length.bufLen;
    }
    const M_<T>& getValue()const{
        return *(M_<T>*)&length.bufLen;
    }
    virtual void set(const Field* par){
        refValue()=dynamic_cast<const FD*>(par)->refValue();
    }
    virtual void operator=(const FD& par){
        refValue()=par.refValue();
    }
    virtual void operator=(const M_<T>& par){
        refValue()=par;
    }
    virtual void operator+=(const T& par){
        refValue()+=par;
    }
    operator const M_<T>& () const{
        return refValue();
    }
    operator M_<T>& () {
        return refValue();
    }
    bool& isRef(){return length.is;}

    void read(const String& str, int& i);

    void operator+=(S_Route& par){
        length.route+=par;
    }
    void connect(Node* node, Node* protoInterface, Field *protoField);

    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    int size();
    virtual void resize(int newLen);
    virtual void toString(String& str);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return length.route;
    }
    M_Route& refRoute(){
        return length.route;
    }
    void sendOutRoute(const M_<T>& par){
        *this=par;

        for(int i=0; i<length.route.size(); i++){
            *dynamic_cast<FD*>(length.route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)refValue();
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};

typedef FD<S_Bool, Multiple>		MFBool;
typedef FD<Color3f, Multiple>	MFColor;
typedef FD<S_Float, Multiple>		MFFloat;
typedef FD<Image, Multiple>	MFImage;
typedef FD<S_Int32, Multiple>		MFInt32;
typedef FD<NodePtr, Multiple>		MFNode;
typedef FD<Rotation, Multiple>	MFRotation;
typedef FD<String, Multiple>	MFString;
typedef FD<double, Multiple>		MFTime;
typedef FD<Vec2f, Multiple>	MFVec2f;
typedef FD<Vec3f, Multiple>	MFVec3f;

template<> class FD<NodePtr,Single> : public Field {
public:
    Node* p;
    M_Route route;
    friend MField;
    bool is;
    void addValFromStr(const String& str);
    void addValSameAsLast();
    int countItems(const String& str, int i);

inline ~FD();
FD(const NodePtr& par);
inline virtual Node* operator = ( const int par1);
inline virtual Node* operator = ( Node* par1);
inline Node* operator = ( FD& par);

    bool	operator ==(const NodePtr& par)const{
        return p==par.p;
    }
    bool	operator !=(const NodePtr& par)const{
        return p!=par.p;
    }
    bool	operator ==(const Node* par)const{
        return p==par;
    }
    bool	operator !=(const Node* par)const{
        return p!=par;
    }


    operator Node*()const{
        return p;
    }

    Node& operator *(){
        return *p;
    }
    Node* operator ->()const{
        return p;
    }
    Node* getRawPtr()const{
        return p;
    }

    FieldType getType()const;
    NodePtr& refValue(){
        return *(NodePtr*)&p;
    }
    const NodePtr& refValue()const{
        return *(NodePtr*)&p;
    }
    const NodePtr& getValue()const{
        return refValue();
    }
    virtual void set(const Field* par){
        *this=dynamic_cast<const SFNode*>(par)->refValue();
    }
    operator const NodePtr& () const{
        return refValue();
    }
    operator void* ();
    bool& isRef(){return is;}

    void read(const String& str, int& i);

    void operator+=(S_Route& par){
        route+=par;
    }

    void connect(Node* node, Node* protoInterface, Field *protoField);

    bool testRouteTo(const Field* destField)const{
        const M_Route& mroute=getRoute();
        for(int i=mroute.length; --i>=0;){
            if(mroute[i].toFieldPtr==destField){
                return true;
            }
        }
        return false;
    }

    void toStrCpp(String& str)const;
    virtual void toString(String& str);
    int size();
    virtual void resize(int newLen);
    void toString(String& str, Field *protoField, bool newList);
    void toCPPString(String& str, const String& nodeName, Field *protoField, bool newList);
    const M_Route& getRoute()const{
        return route;
    }
    M_Route& refRoute(){
        return route;
    }
    void sendOutRoute(const NodePtr& par){
        *this=par;

        for(int i=0; i<route.size(); i++){
            *dynamic_cast<SFNode*>(route[i].toFieldPtr)=par;
        }
    }
    void*& refRawPtr(){
        return (void*&)refValue();
    }

void toXMLString(String& str, bool newList, const char* containerField);
void decodeAssign(Function& f, const String& str, int& i);
void decodeAddAssign(Function& f, const String& str, int& i);
int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
//void boolCopyFieldToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
    FieldButton* setUpControl(Node& node, Field& field, const Properties& properties, const int index, int xPos, int yPos, Window& win);
//	void spinComtrol(const int& item, const int& mod, const int& upDown, const int &index, const Properties& properties);
#endif
};


extern const char* mFieldErrorMsg;

class DynamicMField {
public:
    DynamicMField():
        rawLen(0),
        pointer(0)
    {}
    DynamicMField(const DynamicMField& par):
        rawLen(par.rawLen),
        pointer(par.pointer)
    {
        for(int i=0;i<rawLen;i++)pointer[i]=par.pointer[i];
    }
    DynamicMField& resize(int newLength){
        if(newLength>rawLen){
            int i;
            Field** newPointer=new Field*[newLength];
            for(i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;

        }
        rawLen=newLength;
        return *this;
    }
    DynamicMField& operator +=(DynamicMField& par){
        int i;
        int newLength=par.rawLen+rawLen;
        Field** newPointer=new Field*[newLength];
        for(i=0;i<rawLen;i++){
            newPointer[i]=pointer[i];
        }
        for(i=0;i<par.size();i++){
            newPointer[i+rawLen]=par[i];
        }
        delete pointer;

        rawLen=newLength;
        pointer=newPointer;
        return *this;
    }
    void operator +=(Field* par){
        int i;
        int newLength=rawLen+1;
        Field** newPointer=new Field*[newLength];
        for(i=0;i<rawLen;i++){
            newPointer[i]=pointer[i];
        }
        newPointer[rawLen]=par;
        delete[] pointer;

        rawLen=newLength;
        pointer=newPointer;
    }
    void subLength(int i){
        rawLen-=i;
    }
    DynamicMField& addLength(int par){
        int newLength=par+rawLen;
        if(newLength>rawLen){
            int i;
            Field** newPointer=new Field*[newLength];
            for(i=0;i<rawLen;i++){
                newPointer[i]=pointer[i];
            }
            delete[] pointer;
            pointer=newPointer;

        }
        rawLen=newLength;
        return *this;
    }
    void read(M_Properties &properties, const String& str,int& i);
    void read2(Field *&field, Properties& sprop, bool& wasMF, const String& str,int& i);
    void read(M_Properties &properties, const BIProto& binode, String& str);
    void read2(Field *&field, Properties& sprop, bool& wasMF, const BIField& bi);

    void readForScript(Node* node, M_Properties &properties, Script* script,const String& str,int& i);
    int size()const{
        return rawLen;
    }
    Field* operator [](const int par)const{
        if((unsigned)par>=(unsigned)rawLen){
            throw this;
        }
        return pointer[par];
    }
    int rawLen;
    Field** pointer;
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField0 {
public:
    int size()const{return 0;}
    Field* operator [](const int)const{
		throw this;
	}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField1 {
public:
    MField1( Field* f0)
    {
        fieldArray[0]=f0;
    }
    Field* fieldArray[1];
    int size()const{return 1;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)1){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField2 {
public:
    MField2( Field* f0, Field* f1)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
    }
    Field* fieldArray[2];
    int size()const{return 2;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)2){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField3 {
public:
    MField3( Field* f0, Field* f1, Field* f2)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
    }
    Field* fieldArray[3];
    int size()const{return 3;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)3){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField4 {
public:
    MField4(Field* f0, Field* f1, Field* f2, Field* f3)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
    }
    Field* fieldArray[4];
    int size()const{return 4;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)4){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField5 {
public:
    MField5(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
    }
    Field* fieldArray[5];
    int size()const{return 5;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)5){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField6 {
public:
    MField6(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
    }
    Field* fieldArray[6];
    int size()const{return 6;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)6){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField7 {
public:
    MField7(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
    }
    Field* fieldArray[7];
    int size()const{return 7;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)7){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField8 {
public:
    MField8(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
    }
    Field* fieldArray[8];
    int size()const{return 8;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)8){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField9 {
public:
    MField9(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
    }
    Field* fieldArray[9];
    int size()const{return 9;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)9){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField10 {
public:
    MField10(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
    }
    ~MField10()
    {
    };
    Field* fieldArray[10];
    int size()const{return 10;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)10){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField11 {
public:
    MField11(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
    }
    Field* fieldArray[11];
    int size()const{return 11;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)11){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField12 {
public:
    MField12(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
    }
    Field* fieldArray[12];
    int size()const{return 12;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)12){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField13 {
public:
    MField13(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
    }
    Field* fieldArray[13];
    int size()const{return 13;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)13){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField14 {
public:
    MField14(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
    }
    Field* fieldArray[14];
    int size()const{return 14;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)14){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField15 {
public:
    MField15(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
    }
    Field* fieldArray[15];
    int size()const{return 15;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)15){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField16 {
public:
    MField16(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
    }
    Field* fieldArray[16];
    int size()const{return 16;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)16){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField17 {
public:
    MField17(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
    }
    Field* fieldArray[17];
    int size()const{return 17;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)17){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField18 {
public:
    MField18(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
    }
    Field* fieldArray[18];
    int size()const{return 18;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)18){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField19 {
public:
    MField19(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
    }
    Field* fieldArray[19];
    int size()const{return 19;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)19){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField20{
public:
    MField20(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
    }
    Field* fieldArray[20];
    int size()const{return 20;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)20){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField21{
public:
    MField21(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
    }
    Field* fieldArray[21];
    int size()const{return 21;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)21){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField22 {
public:
    MField22(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
    }
    Field* fieldArray[22];
    int size()const{return 22;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)22){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField23 {
public:
    MField23(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
    }
    Field* fieldArray[23];
    int size()const{return 23;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)23){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField24 {
public:
    MField24(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
    }
    Field* fieldArray[24];
    int size()const{return 24;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)24){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField25 {
public:
    MField25(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
    }
    Field* fieldArray[25];
    int size()const{return 25;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)25){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField26 {
public:
    MField26(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
    }
    Field* fieldArray[26];
    int size()const{return 26;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)26){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField27{
public:
    MField27(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
    }
    Field* fieldArray[27];
    int size()const{return 27;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)29){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField28{
public:
    MField28(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
    }
    Field* fieldArray[28];
    int size()const{return 28;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)28){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField29{
public:
    MField29(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
    }
    Field* fieldArray[29];
    int size()const{return 29;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)29){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField30 {
public:
    MField30(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
    }
    Field* fieldArray[30];
    int size()const{return 30;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)30){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField31 {
public:
    MField31(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
    }
    Field* fieldArray[31];
    int size()const{return 31;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)31){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField32 {
public:
    MField32(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
    }
    Field* fieldArray[32];
    int size()const{return 32;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)32){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};
class MField33 {
public:
    MField33(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
    }
    Field* fieldArray[33];
    int size()const{return 33;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)33){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField34 {
public:
    MField34(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32, Field* f33)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
        fieldArray[33]=f33;
    }
    Field* fieldArray[34];
    int size()const{return 34;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)34){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField35 {
public:
    MField35(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32, Field* f33, Field* f34)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
        fieldArray[33]=f33;
        fieldArray[34]=f34;
    }
    Field* fieldArray[35];
    int size()const{return 35;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)35){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};

class MField36 {
public:
    MField36(Field* f0, Field* f1, Field* f2, Field* f3, Field* f4, Field* f5, Field* f6, Field* f7, Field* f8, Field* f9, Field* f10, Field* f11, Field* f12, Field* f13, Field* f14, Field* f15, Field* f16, Field* f17, Field* f18, Field* f19, Field* f20, Field* f21, Field* f22, Field* f23, Field* f24, Field* f25, Field* f26, Field* f27, Field* f28, Field* f29, Field* f30, Field* f31, Field* f32, Field* f33, Field* f34, Field* f35)
    {
        fieldArray[0]=f0;
        fieldArray[1]=f1;
        fieldArray[2]=f2;
        fieldArray[3]=f3;
        fieldArray[4]=f4;
        fieldArray[5]=f5;
        fieldArray[6]=f6;
        fieldArray[7]=f7;
        fieldArray[8]=f8;
        fieldArray[9]=f9;
        fieldArray[10]=f10;
        fieldArray[11]=f11;
        fieldArray[12]=f12;
        fieldArray[13]=f13;
        fieldArray[14]=f14;
        fieldArray[15]=f15;
        fieldArray[16]=f16;
        fieldArray[17]=f17;
        fieldArray[18]=f18;
        fieldArray[19]=f19;
        fieldArray[20]=f20;
        fieldArray[21]=f21;
        fieldArray[22]=f22;
        fieldArray[23]=f23;
        fieldArray[24]=f24;
        fieldArray[25]=f25;
        fieldArray[26]=f26;
        fieldArray[27]=f27;
        fieldArray[28]=f28;
        fieldArray[29]=f29;
        fieldArray[30]=f30;
        fieldArray[31]=f31;
        fieldArray[32]=f32;
        fieldArray[33]=f33;
        fieldArray[34]=f34;
        fieldArray[35]=f35;
    }
    Field* fieldArray[36];
    int size()const{return 36;}
    Field* operator [](const int par)const{if((unsigned)par>=(unsigned)36){	throw this;	}return fieldArray[par];}
    const char* showReason() const{return mFieldErrorMsg;}
};


//Node"
//written by Graham John Perrett thyme@seamless3d.com www.seamless3d.com



//M_Node
//written by Graham John Perrett thyme@seamless3d.com www.seamless3d.com
class Length;

class PovRay{
public:
    virtual void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix)=0;
    virtual void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix)=0;
    virtual void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix)=0;
};

void processFieldsPlus(Node& node);

class Node {
public:
    const char *showReason() const {return g_exceptioninNode; }
    String name;
    union{
        struct {
            bool collapsed;
            bool renderProcessed;
        };
        unsigned short nodeFlags;
    };

    friend NodePtr;
    Node():
        nodeFlags(0),
        refCount(0)
    {}

    unsigned refCount;

    virtual ~Node(){};

    void getSceneDisplayString(String& str);


    virtual const M_Properties& getProperties()const;
    void getNode(const String& str,int& i);
    ProtoInterface* getProtoInterface()const;
    virtual void renderNode(Matrix parentMatrix, Vec3f parentColScale){};

    virtual void toString(String& str, bool newList=true);
    virtual void toXMLString(String& str, bool newList, const char* containerField);
    virtual void toCPPString(String& str, bool newList=true);
    void outputXMLFields(int startField,int fieldLen, const M_Properties& properties, ProtoInterface& protoInterface, bool newList, String& str, int& i);

    bool setDEFgetUSE( String &str, bool& newList);
    void getField(const int& fieldIdx,const String &str,int &i);

    S_Route& getRoute(const int index);
    void deleteRoute(const int index);
    void replaceNoderefForRouteInSceneTree(NodePtr from, Node* to);

    virtual int getFieldIndex(const Field* field) const;
    virtual void processFields(){};
    virtual void getFields(const String& str,int& i);
    virtual void decodeVoid(const String&, int&, int&){
		throw;
	};

#ifdef SEAMLESS3DGUI
//	virtual void pointDeclarationToBuiltInVaribles(){};
    void genNodeListRecursion(const int parent, int indent, Color3f marker, M_<SceneTreeListItem>& sceneTreeList, M_<Anim*>& animBarList);
    void paintSceneTree(int indent, int& line, int collapsedLine, bool visible, bool animMode, Function* lastFunction);
    void paintIcon( int indent, int line, int& paintLine, int collapsedLine, bool visible, bool animMode, Function* lastFunction);
    int winProc(void*,unsigned , unsigned , unsigned ){return 0;};
    virtual void paste(){};
    virtual int getFieldGroupIndex(const int&)const{return 0;}
    virtual void setFieldGroupIndex(const int&,const int&){}
    virtual M_Int* getIndexPtr()=0;
    virtual void updateBuild(){}
    virtual void senseTug(Matrix matrix, Matrix iMatrix, TouchSensor* sensor);
    virtual void catInverseRotation(Rotation& r);
    virtual void resetRotations();
    virtual void processForOutput(M_Node& parentChild, int& n, Script* gs);
//	int getFlatFieldEndLine(Field* target, int listPos);
//	int getFlatFieldEndLine2(Field* target, int& listPos);
//	int getNodeFlatLen(int& listPos);
    bool searchForRefNode();
    bool searchForRefNodeCollapsed(Node* check);
    void findLastColorInCollapsed(unsigned& color);
    void convertRoutesToScript(Function* f);
#endif


    virtual Field* getField(const int i)const=0;
    const Properties& getProp(const int index)const;

    //virtual Field* getField2(const int i){
//		return getField(i);
//	}
    virtual int getFLen()const=0; //get field rawLen
//	virtual int getFLen2(){
//		return getFLen();
//	};


    void incRefCount(){
        if(this!=NULL){ //is this line really needed?
           ++refCount;
        }
    }
    unsigned decRefCount(){
        --refCount;
        return refCount;
    }
    virtual int getType()const=0;

    String getTypeStr()const{
        return nodeLookUpStr[getType()];
    }
    const Properties& getProperties(Field& field)const;

    int getFieldIndex(const String& name);

    void copyFields(Node* node);//deprecated
	void copyFields2(Node* node);
    void deepCopyFields(Node* node, int& line);
    void undoIS(int fieldIdx);

    template<typename T, typename S>class TSF : public FD<T, Single> {
        public:
        Node& home;
        TSF(const T& value, Node& home):
            FD<T,Single>(value),
            home(home)
        {};
        void operator=(const T& par){
            FD<T, Single>::refValue() = par;
            processFieldsPlus(home);
        }
    };
    template<typename T, typename S>class TMF : public FD<T, Multiple> {//trigger multiple field
        public:
        Node& home;
        TMF(const M_<T>& value, Node& home):
            FD<T,Multiple>(value),
            home(home)
        {};
        void operator=(const T& par){
            FD<T, Multiple>::refValue() = par;
            processFieldsPlus(home);
        }
        void operator=(const M_<T>& par){
            FD<T, Multiple>::refValue() = par;
            processFieldsPlus(home);
        }
        void resize(int newLen){
            FD<T, Multiple>::refValue().length=newLen;
            processFieldsPlus(home);
        }
    };
    typedef TSF<bool, Single> TSFBool;
    typedef TSF<Color3f, Single> TSFColor3f;
    typedef TSF<float, Single> TSFFloat;
    typedef TSF<int, Single> TSFInt;
    typedef TSF<Rotation, Single> TSFRotation;
    typedef TSF<Vec2f, Single> TSFVec2f;
    typedef TSF<Vec3f, Single> TSFVec3f;
    typedef TSF<NodePtr, Single> TSFNode; //do not use because of operator = 0

    typedef TMF<bool, Multiple> TMFBool;
    typedef TMF<double, Multiple> TMFDouble;
    typedef TMF<float, Multiple> TMFFloat;
    typedef TMF<int, Multiple> TMFInt;
    typedef TMF<Vec2f, Multiple> TMFVec2f;
    typedef TMF<Vec3f, Multiple> TMFVec3f;
    typedef TMF<NodePtr, Multiple> TMFNode;


/*
    class HSFBool : public SFBool {
    public:
        Node& home;
        HSFBool(const bool& value, Node& home):
            SFBool(value),
            home(home)
        {};
        void operator=(const bool& par){
            refValue()=par;
            home.processFields();
        }
    };
    class HMFVec2f : public MFVec2f{
    public:
        Node& home;
        HMFVec2f(const M_Vec2f& value, Node& home):
            MFVec2f(value),
            home(home)
        {};
        void operator=(const M_Vec2f& par){
            refValue()=par;
            home.processFields();
        }
    };
*/
//protected:
//	Node* parent;
};


enum{
    DO_NOT_OUTPUT	= 0x0001,
    DO_NOT_SAVE		= 0x0002,
    BLOCK_USE		= 0x0004,
    BLOCK_NEW		= 0x0008,
    SAME_AS_LAST	= 0x0010,
    DISABLE			= 0x0020,
    DISABLE_LEN		= 0x0040,
    UNREADABLE		= 0x0080,
    UNWRITEABLE		= 0x0100,
    DO_NOT_DISPLY_NAME	= 0x0200,
    DO_NOT_DISPLY_VALUE	= 0x0400,
    DO_NOT_RESIZE	= 0x0800,
    BLUE_DOT		= 0x1000,
    RED_DOT			= 0x2000,
    AXIS3F			= 0x4000,
    PUSH			= 0x8000,
    LINK			= 0x10000,
};

class Properties {
public:
    unsigned int flags;

    void setBit(unsigned char bit, bool state);
template <int T> void set_(const bool& state){
    if(state){
        flags |= T;
    }
    else{
        flags &= ~T;
    }
}
    unsigned int doNotOutput()const{
        return flags & DO_NOT_OUTPUT;
    }
    unsigned int doNotSave()const{
        return flags & DO_NOT_SAVE;
    }
    unsigned int blockUse()const{
        return flags & BLOCK_USE;
    }
    unsigned int blockNew()const{
        return flags & BLOCK_NEW;
    }
    unsigned int sameAsLast()const{
        return flags & SAME_AS_LAST;
    }
    unsigned int doNotDisplayName()const{
        return flags & DO_NOT_DISPLY_NAME;
    }
    unsigned int doNotDisplayValue()const{
        return flags & DO_NOT_DISPLY_VALUE;
    }
    unsigned int doNotResize()const{
        return flags & DO_NOT_RESIZE;
    }
    unsigned int disable()const{
        return flags & DISABLE;
    }
    unsigned int disableLen()const{
        return flags & DISABLE_LEN;
    }
    unsigned int unreadable()const{
        return flags & UNREADABLE;
    }
    unsigned int unwritable()const{
        return flags & UNWRITEABLE;
    }
    unsigned int blueDot()const{
        return flags & BLUE_DOT;
    }
    unsigned int retDot()const{
        return flags & RED_DOT;
    }
    unsigned axis()const{
        return flags & AXIS3F;
    }
    unsigned push()const{
        return flags & PUSH;
    }
    unsigned link()const{
        return flags & LINK;
    }


//void set_inputOutput(const bool& state){ //does nothing
    //set_<0xffffffff ^ (UNREADABLE | UNWRITEABLE)>(state);
//}

void set_doNotOutput(const bool& state){
    set_<DO_NOT_OUTPUT>(state);
}
void set_doNotSave(const bool& state){
    set_<DO_NOT_SAVE>(state);
}
void set_blockUse(const bool& state){
    set_<BLOCK_USE>(state);
}
void set_blockNew(const bool& state){
    set_<BLOCK_NEW>(state);
}
void set_sameAsLast(const bool& state){
    set_<SAME_AS_LAST>(state);
}
void set_disable(const bool& state){
    set_<DISABLE>(state);
}
void set_disableLen(const bool& state){
    set_<DISABLE_LEN>(state);
}
void set_unreadable(const bool& state){
    set_<UNREADABLE>(state);
}
void set_unwritable(const bool& state){
    set_<UNWRITEABLE>(state);
}
void set_blueDot(const bool& state){
    set_<BLUE_DOT>(state);
}
void set_axis(const bool& state){
    set_<AXIS3F>(state);
}
void set_push(const bool& state){
    set_<PUSH>(state);
}
void set_link(const bool& state){
    set_<LINK>(state);
}

    M_Int promptNode;
    int groupIndex;
    int startAtIndex;
    int	initElLen;		//initial element length
    int	incElLen;	//increment element length
    double rangeFrom;
    double rangeTo;
    const char* tip;
    String name;
    String initElVal;
    String incElVal;
    Properties():
        groupIndex(-1),	//-1 = does not belong to a group
        startAtIndex(0),
        flags(0),
        initElLen(1),
        incElLen(1),
        rangeFrom(-2147483647),
        rangeTo(2147483647),
        tip(0)
    {
    }
    Properties(String i_name):
        name(i_name),
        groupIndex(-1),	//-1 = does not belong to a group
        startAtIndex(0),
        flags(0),
        initElLen(1),
        incElLen(1),
        rangeFrom(-2147483647),
        rangeTo(2147483647),
        tip(0)
    {
    }
    Properties(int)://promptNode ?
        groupIndex(-1),	//-1 = does not belong to a group
        startAtIndex(0),
        flags(0),
        initElLen(1),
        incElLen(1),
        rangeFrom(-2147483647),
        rangeTo(2147483647),
        tip(0)
    {
        //promptNode.resize(5);
    }
    Properties(const String& i_name, const int& groupIndex, const int& startAtIndex, const int& flags, const int& initElLen, const int& incElLen, const double& rangeFrom, const double& rangeTo, const char* tip=0):
        name(i_name),
        groupIndex(groupIndex),	//-1 = does not belong to a group
        startAtIndex(startAtIndex),
        flags(flags),
        initElLen(initElLen),
        incElLen(incElLen),
        rangeFrom(rangeFrom),
        rangeTo(rangeTo),
        tip(tip)
    {
    }
    ~Properties(){
    }
    void operator = (const Properties& par){
        name=par.name;
        groupIndex=par.groupIndex;
        startAtIndex=par.startAtIndex;
        flags=par.flags;
        initElLen=par.initElLen;
        incElLen=par.incElLen;
        initElVal=par.initElVal;
        incElVal=par.incElVal;
        rangeFrom=par.rangeFrom;
        rangeTo=par.rangeTo;
        promptNode=par.promptNode;
        tip=par.tip;
    }
    void read( Field& field, const String& str,int& i, bool& beginGroup, bool& endGroup);

    bool isReadable()const{
        return !(flags & UNREADABLE);
    };
    bool isWritable()const{
        return !(flags & UNWRITEABLE);
    };
    bool inputOutput()const{
        return !((flags & UNREADABLE) || (flags & UNWRITEABLE));
    }
    bool initializeOnly()const{
        return ((flags & UNREADABLE) && (flags & UNWRITEABLE));
    }
    bool inputOnly()const{
        return ((flags & UNREADABLE) && !(flags & UNWRITEABLE));
    }
    bool outputOnly()const{
        return (!(flags & UNREADABLE) && (flags & UNWRITEABLE));
    }
};

class MultiSelect{
public:
    static bool selectRectMode;
    static Vec3f topLeft;
    static Vec3f bottomRight;
    static Vec2f topLeft2d;
    static Vec2f bottomRight2d;
    static bool isInBox(Vec3f point);
    static bool isInBox(Vec2f point);
    static M_Vec3f beginDragPos;
    static Vec3f autoPivotPoint;
    static void backUpCoords(Art* part);
    static int clickedIndex;
    static bool withinSights;
    static bool withinSightsIsSelected; //point is already selected
    static int withinSightsIndex;
    static NodePtr withinSightsNode;
    static void calcPivotPoint();
    static void rotateSelection(Art* part, Vec3f vec);
    static bool accumulate;
    static float distance;

    static int selectedPoint;
    static M_<S_VertexIndex> triList;
    static M_<S_VertexIndex> coordList;
    static M_ControlPoint pointList;
    static M_Node objectList;
    static M_Int ctrlPolySelList;
    static M_ControlPoint boxPolySelList;
    static void selectTriangles(Art* part);
    static void processRectangle();

    static void selectFrontAndBackFaceVertices(Seamless* smls);
    static void selectFrontFaceVertices(Seamless* smls);

    static void selectFrontAndBackFaceTransformedVertices(Seamless* smls);
    static void selectFrontFaceRenderedVertices(Seamless* smls);
    static int uRingDim;
    static int vRingDim;
    static void resetDistance();
    static void checkToReset(Art* part);
    static void reset();
};

inline	Matrix scaleMatrix(SFVec3f& par){
    return scaleMatrix(par.refValue());
}
inline	Matrix rotationMatrix(SFRotation& par){
    return rotationMatrix(par.refValue());
}
inline	Matrix translationMatrix(SFVec3f& par){
    return translationMatrix(par.refValue());
}

class Download{
public:
    virtual bool download()=0;
    virtual void getUrlPath(String& url)=0;
};
extern M_<Download*> g_downloadQue;

class Progenitor{
public:
    MFNode& children;
    Progenitor(MFNode& i_children):
        children(i_children)
    {}
    virtual ~Progenitor()
    {
    }
};

class AGroup : public Node, public Progenitor {
public:
    MFNode addChildren;
    MFNode removeChildren;
    MFNode& children;
    SFVec3f bboxCenter;
    SFVec3f bboxSize;

    AGroup(MFNode& i_children):
        Progenitor(i_children),
        addChildren(M_Node()),
        removeChildren(M_Node()),
        children(i_children),
        bboxCenter(Vec3f()),
        bboxSize(Vec3f(-1,-1,-1)),
        ////////
        sensor(0)
    {
    }
    virtual ~AGroup()
    {
    }
    TouchSensor* sensor;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
#ifdef SEAMLESS3DGUI
    virtual void senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor);
    virtual void catInverseRotation(Rotation& r);

    void resetRotations();
    virtual void updateBuild();
#endif
    virtual void processFieldsLoop( HAnimHumanoid*){};
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
};
class VisualAid {
public:
#ifdef RENDER
    virtual void houseKeep()=0;
    virtual void drag(Vec3f pt){};
    virtual void centerViewToControlPoint();
#endif
};

class Link{
public:
    NurbsPatch* patch;
    int point;
    int poly;
    int corner;
    float between;
    Link():
        patch(0),
        point(0),
        poly(0),
        corner(0),
        between(0)
    {}
    Link(int point, int poly, int corner, float between):
        patch(0),
        point(point),
        poly(poly),
        corner(corner),
        between(between)
    {}
    Link(NurbsPatch* patch, int point, int poly, int corner, float between):
        patch(patch),
        point(point),
        poly(poly),
        corner(corner),
        between(between)
    {}
    void operator=(const Link& par){
        patch = par.patch;
        point = par.point;
        poly = par.poly;
        corner = par.corner;
        between = par.between;
    }
};

typedef M_ <MorpherPtr> M_Morpher;
typedef M_ <ArtPtr> M_SArt;
typedef M_ <BuildPtr> M_Build;
typedef M_ <StrandPtr> M_Strand;
typedef M_<Link> M_Link;

class MFBuild : public MFNode {
public:
    MFBuild(const M_Node& value):
        MFNode(value)
    {}
    M_Build& refValue(){
        return *reinterpret_cast<M_Build*>(&MFNode::refValue());
    }
    void operator+=(Build* par){
        refValue()+=par;
    }
    void operator+=(Ptr<Build> par){
        refValue()+=par;
    }
};

class StickMan{
public:
    StickMan();
    ~StickMan(){}

    ShapePtr shape;
    IndexedLineSet* ils;
    Coordinate* coord;
    M_Art partList;

    void renderStickMan(Seamless* smls, Matrix& matrix, Vec3f parentColScale);
    void addLineForBone(M_SArt& m_part, int parentLineIndex, M_Vec3f& point, M_Int& coordIndex, bool& beginNewLine);
    void process(Seamless* smls);
};
class CPMorpherPart{
public:
    CPMorpher* morpher;
    Art* part;

    CPMorpherPart():
        morpher(0),
        part(0)
    {}
    CPMorpherPart(CPMorpher* morpher, Art* part):
        morpher(morpher),
        part(part)
    {}
};
class HitElement{
public:
    Color3f rawColor;
    Color3f shadedColor;
    float hitZ;
    float transparency;

    HitElement():
        hitZ(0),
        transparency(0)
    {}
    void operator=(const HitElement& par){
        rawColor = par.rawColor;
        shadedColor = par.shadedColor;
        hitZ = par.hitZ;
        transparency = par.transparency;
    }

    void getColorFromPointInTriangle(const int fromCoord, const S_Triangle& triangle, const Vec3f hitPoint, const ColorEffect* colorEffect, M_Float& shade, Matrix& matrix);
};
class RayView{
public:
    Vec3f pointer;
    Vec3f clickPos;
    float hitZ;
    Art* part;
    int hitTri;
    bool perspective;
    int testTriIdx;
    Color3f hitColor;

    M_<HitElement> m_hit;

    Vec3f scale;
    Vec3f translation;
    Rotation rotation;

    Matrix matrix;


    RayView():
        hitZ(0),
        part(0),
        hitTri(0),
        perspective(false),
        testTriIdx(0),
        scale(Vec3f(1,1,1))
    {}
    bool rayTraceView(Seamless* smls, const Vec3f pointer);
    bool rayTraceLight(Seamless* smls, const Vec3f pointer, Art* blockPart, int blockTri);
    bool rayTraceLightNormalShading(Seamless* smls, const Vec3f pointer, Art* blockPart, int blockCoord);

};
class SFNodeRender : public SFNode {
public:
    bool& renderProcessed;
    SFNodeRender( const NodePtr& value, bool& renderProcessed):
        SFNode(value),
        renderProcessed(renderProcessed)
    {};
    Node* operator=(SFNodeRender& par){
        refValue() = par.p;
        return par.p;
    }

    Node* operator=(Node* par){
        refValue() = par;
        renderProcessed=false;
        return par;
    }
    Node* operator=(const int par){
        refValue() = par;
        renderProcessed=false;
        return 0;
    }
};
class Geometry : public Node{
public:
#ifdef RENDER
    virtual void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance)=0;
#endif
    virtual void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const =0;
};
class Setup : public SFNode {
public:
    Build& home;
    Setup( const NodePtr& par, Build& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par1);
};
class Build : public Node {
public:
    virtual void doBuild()=0;
    virtual NurbsPatch* getPatch(){return 0;}
    virtual Build* getMirrorBuild(){return 0;}
    virtual int numOfPatches(){return 0;}
    virtual bool isCaged(){return true;}
    virtual void setCage(bool){}
    virtual void setMirror(bool){}
    virtual void cpNodeListAdd(Seamless*){}
    virtual void remTransform(){}
    virtual void doCPTransform(){}
	virtual void joinStitchedEdges(){}
	virtual void createCpFromPatchPoints(){}
    void doSetup();

    Setup& setup;

    Build(Setup& i_setup):
        setup(i_setup)
    {};
    virtual bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const=0;
    void decodeVoid(const String& str, int& i, int& bracketCntr);
#ifdef SEAMLESS3DGUI
//	void cut();
//	void copy();
    //SeamlessPtr smls;
#endif
};


class Generator{
public:
    virtual void generate(M_Vec3f& coord, const int h)=0;

#ifdef SEAMLESS3DGUI
    void cut();
#endif
    int getType()const{return GENERATOR;}
    Generator(){
    }
};
class ALathe : public Generator, public Node {
protected:
public:
    virtual float getDistance(int index=0x7fffffff)const=0;
    virtual void getBandAndYDist(M_Int& bands, M_Float& bandDist, M_Float& yDistance)const=0;
    virtual void calcClose()=0;
    virtual void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY)=0;
};

class Mesh : public Node, public MField6 {
    public:
    int getType()const{return MESH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFVec3f coord;
    MFVec3f normal;
    MFColor color;
    MFVec2f texCoord;
    MFInt32 coordIndex;
    MFInt32 rc;

    Mesh():
        MField6(
            &coord,
            &normal,
            &color,
            &texCoord,
            &coordIndex,
            &rc
        ),

        coord(M_Vec3f()),
        normal(M_Vec3f()),
        color(M_Color3f()),
        texCoord(M_Vec2f()),
        coordIndex(M_Int()),
        rc(M_Int()),

    ///////
        render(0)
    {
#ifdef SEAMLESS3DGUI
        index.length=6;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    Render* render;

    inline void convertFromTexCoordIndex( M_Int& coordIndex, M_Int& ifsTexCoordIndex, const M_Vec2f& ifsTexCoord);
    inline void convertFromColorIndex( M_Int& coordIndex, M_Int& colorIndex, const M_Color3f& ifsColor);
    void genColorList(Color3f c);
    void genTransformedNormals(M_Art& partList);

    //void loadFromIndexedFaceSet(const IndexedFaceSet* ifs);
    void genNormals();
    int redirectCoord(const int& index, int& coordLen){	//coordLen = coord.length
        if(index<coordLen){
            return index;
        }
        else return rc[index-coordLen];
    }
#ifdef RENDER
    void convertToRenderFormat(Render*& render, const Appearance* appearance);
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Render*& render, const Appearance* appearance);
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void transformRenderTexture(Render*& render, TextureTransform* textureTransform);
#endif
    int getIndex(const int& ci){	//get redirected index
        if(ci<coord.length){
            return ci;
        }
        else{
            return rc[ci-coord.length];
        }
    }
    void reset(){
        coord.refValue().length=0;
        normal.refValue().length=0;
        color.refValue().length=0;
        texCoord.refValue().length=0;
        coordIndex.refValue().length=0;
        rc.refValue().length=0;
    }
    void ensureIntegrity(){
        for(int i=coordIndex.length; --i>=0;){
            int ci=getIndex(coordIndex[i]);
            if(ci>=coord.length){
                coordIndex.refValue().length=0;
                return;
            }
        }
    }
};
class Scene : public AGroup, public Download, public PovRay, public MField16{
    public:

////////////////
    int getType()const{return SCENE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{
		throw this;
	}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFFloat version;

//	Perspective perspective;
//	Zoom zoom;
//	Orientation orientation;
//	Position position;
    SFColor backgroundColor;
    SFColor scene3DFontColor;
    SFInt32 mode;
    //SFBool	xRayEdit;
    //SFBool	wireframe;
    SFInt32 selLine;
    SFInt32 scroll;
    MFInt32 collapse;
    MFInt32 wOpen;
    MFVec2f wPos;
    MFVec2f wSize;

    SFNode grid;
    SFNode exportMovie;
    SFNode exportVrmlX3d;
    SFNode brush;
    SFNode fill;
    MFNode children;

    Scene():
        MField16(
            &version,

//			&perspective,
//			&zoom,
//			&orientation,
//			&position,
            &backgroundColor,
            &scene3DFontColor,
            &mode,
            //&xRayEdit,
            //&wireframe,

            &selLine,
            &scroll,
            &collapse,
            &wOpen,
            &wPos,
            &wSize,

            &grid,
            &exportMovie,
            &exportVrmlX3d,
            &brush,
            &fill,
            &children
        ),

        version(0),
//		perspective(true,*this),
//		zoom(1,*this),
//		orientation(Rotation(0,0,1,0),*this),
//		position(Vec3f(0,0,3),*this),

        backgroundColor(Color3f(0, 0, 0)),
        scene3DFontColor(Color3f(1, 1, 0)),
        mode(HOME_MODE | BACKFACE_IGNORE_MODE | BRUSH_MODE | CREATE_MIRROR_OBJECT),
        //xRayEdit(false),
        //wireframe(true),

        selLine(0),
        scroll(0),
        collapse(M_Int()),

        wOpen(M_Int()),
        wPos(M_Vec2f()),
        wSize(M_Vec2f()),

        grid(0),
        exportMovie(0),
        exportVrmlX3d(0),
        brush(0),
        fill(0),
        children(M_Node()),

        /////////////////////
        AGroup(children),
        fieldOfView(.785398),
        scale(1,1,1),
        hInternetOpenUrl(0),
        netSave(false)
    {}
    virtual void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    inline Matrix dxWindMatrix(Matrix& parentMatrix);
#ifdef SEAMLESS3DGUI
    void newChildrenNode(Node* node);
#endif
    void removeChildren(M_Node& par1);
    bool download();
    bool openUrl(String str);
    void toString(String& str, bool newList=true);//only reason we need this is to insert protoStr for output
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
    void postLoadProcessCollapse();
    void preSaveProcessCollapse();
    int hInternetOpenUrl;
    M_<char> internetBuf;
    void outputNodeBind();

    void extraToString(String& str);

/*
void setSceneView(const	bool& i_perspective, const float& i_zoom, const Vec3f& i_position, const Rotation& i_orientation){
    perspective.refValue()=i_perspective;
    zoom.refValue()=i_zoom;
    position.refValue()=i_position;
    orientation=i_orientation;
}
*/
Matrix getIViewMatrix();
Matrix getViewMatrix();
Matrix getScaleTranRotMatrix(){
    return scaleMatrix(scale) * translationMatrix(translation) * rotationMatrix(rotation);
}


void rotateScene();

void setSceneFromViewpoint(const Viewpoint& viewpoint);
void setViewpointFromScene(Viewpoint& viewpoint);


float getZoom()const{
    return scale.x;
}
void setZoom(const float& par);

void storeModeZoomRotTran(int& storeMode, float& storeZoom, Vec3f& storeTranslation, Rotation& storeRotation){
    storeMode = mode;
    storeZoom = getZoom();
    storeTranslation = translation;
    storeRotation=rotation;
}
void restoreModeZoomRotTran(int& storeMode, float& storeZoom, Vec3f& storeTranslation, Rotation& storeRotation){
    mode = storeMode;
    scale = Vec3f(storeZoom, storeZoom, storeZoom);
    translation = storeTranslation;
    rotation = storeRotation;
}
void genNodeList2();
void genNodeList3();
//protected:
    float fieldOfView;
    Vec3f scale;
    Rotation rotation;
    Vec3f translation;
    void getUrlPath(String& url);
    String worldUrl;
    bool netSave;
};
class Coordinate : public Node, public MField1 {
    public:
class Point : public MFVec3f {
public:
    Coordinate* home;
    bool& renderProcessed;

    Point(const M_Vec3f& par, bool& renderProcessed):
        MFVec3f(par),
        renderProcessed(renderProcessed)
    {};
    void set(const Field* par){//Point
        refValue() = dynamic_cast<const FD*>(par)->refValue();
        renderProcessed = false;
    }
    void operator=(const Point& par){
        refValue() = par.refValue();
        renderProcessed = false;
    }
    void operator=(const MFVec3f& par){
        refValue() = par.refValue();
        renderProcessed = false;
    }
    void operator = (const M_Vec3f& par){
        refValue() = par;
        renderProcessed = false;
    }
    virtual void operator+=(const Vec3f& par){
        refValue()+=par;
        renderProcessed = false;
    }
    void resize(int newLen){
        refValue().length=newLen;
        renderProcessed = false;
    }
};
///////////////////////////

    int getType()const{return COORDINATE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}

#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif
    Point point;

    Coordinate():
        MField1(
            &point
        ),
        point(M_Vec3f(),renderProcessed)
    {
        point.home=this;

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class Normal : public Node, public MField1 {
    public:
class Vector : public MFVec3f {
public:
    bool& renderProcessed;

    Vector(const M_Vec3f& par, bool& renderProcessed):
        MFVec3f(par),
        renderProcessed(renderProcessed)
    {};
    void set(const Field* par){//Vector
        refValue() = dynamic_cast<const FD*>(par)->refValue();
        renderProcessed = false;
    }
    void operator=(const Vector& par){
        refValue() = par.refValue();
        renderProcessed = false;
    }
    void operator=(const MFVec3f& par){
        refValue() = par.refValue();
        renderProcessed = false;
    }
    void operator=(const M_Vec3f& par){
        refValue() = par;
        renderProcessed = false;
    }
    virtual void operator+=(const Vec3f& par){
        refValue()+=par;
        renderProcessed = false;
    }
    void resize(int newLen){
        refValue().length = newLen;
        renderProcessed = false;
    }
};
///////////////////////////
    int getType()const{return NORMAL;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Vector vector;

    Normal():
        MField1(
            &vector
        ),
        vector(M_Vec3f(), renderProcessed)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class Color : public Node, public MField1{
    public:
class ColorField : public MFColor {
public:
    bool& renderProcessed;

    ColorField(const M_Color3f& par, bool& renderProcessed):
        MFColor(par),
        renderProcessed(renderProcessed)
    {};
    void set(const Field* par){//Color
        refValue()=dynamic_cast<const FD*>(par)->refValue();
        renderProcessed=false;
    }
    void operator=(const ColorField& par){
        refValue()=par.refValue();
        renderProcessed=false;
    }
    void operator=(const MFColor& par){
        refValue()=par.refValue();
        renderProcessed=false;
    }
    void operator=(const M_Color3f& par){
        refValue()=par;
        renderProcessed=false;
    }
    virtual void operator+=(const Color3f& par){
        refValue()+=par;
        renderProcessed=false;
    }
    void resize(int newLen){
        refValue().length=newLen;
        renderProcessed=false;
    }
};
///////////////////////////
    int getType()const{return COLOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    ColorField color;

    Color():
        MField1(
            &color
        ),

        color(M_Color3f(), renderProcessed)
        ///////////
        {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class TextureCoordinate : public Node, public MField1 {
public:
    class Point : public MFVec2f {
    public:
        bool& renderProcessed;

        Point( M_Vec2f& par, bool& renderProcessed):
            MFVec2f(par),
            renderProcessed(renderProcessed)
        {};
        void set(const Field* par){//TexPoint
            refValue()=dynamic_cast<const FD*>(par)->refValue();
            renderProcessed=false;
        }
        void operator=(const MFVec2f& par){
            refValue()=par.refValue();
            renderProcessed=false;
        }
        void operator=(const M_Vec2f& par){
            refValue()=par;
            renderProcessed=false;
        }
        virtual void operator+=(const Vec2f& par){
            refValue()+=par;
            renderProcessed=false;
        }
        void resize(int newLen){
            refValue().length=newLen;
            renderProcessed=false;
        }
    };

    int getType()const{return TEXTURECOORDINATE;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    MFVec2f point;

    TextureCoordinate():
        MField1(
            &point
        ),

        point(M_Vec2f())

    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class VertexGeometry : public Geometry {
public:
    class Ccw : public SFBool {
    public:
        bool& renderProcessed;
        Ccw( const bool& value, bool& renderProcessed):
            SFBool(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const bool& par){
            refValue()=par;
            renderProcessed=false;
        }
    };
    class Solid : public SFBool {
    public:
        bool& renderProcessed;
        Solid( const bool& value, bool& renderProcessed):
            SFBool(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const bool& par){
            refValue()=par;
            renderProcessed=false;
        }
    };

    SFNodeRender	coord;
    SFNodeRender	normal;
    SFNodeRender	color;
    SFNodeRender	texCoord;
    Ccw	ccw;
    SFBool 	normalPerVertex;
    SFBool	colorPerVertex;
    Solid	solid;

    VertexGeometry():
        coord(0, renderProcessed),
        normal(0, renderProcessed),
        color(0, renderProcessed),
        texCoord(0, renderProcessed),
        ccw(true, renderProcessed),
        normalPerVertex(true),
        colorPerVertex(true),
        solid(true, renderProcessed)
        ///////
    {
    }


    M_Color3f& refColor(){
        if(Color* tmp = dynamic_cast<Color*>(color.refValue().p)){
            return tmp->color.refValue();
        }
        throw this;
    }
    M_Vec3f& refVector(){
        if(Normal* tmp = dynamic_cast<Normal*>(normal.p)){
            return tmp->vector.refValue();
        }
        throw this;
    }
    M_Vec3f& refPoint(){
        if(Coordinate* tmp = dynamic_cast<Coordinate*>(coord.p)){
            return tmp->point.refValue();
        }
        throw this;
    }
    M_Vec2f& refTexPoint(){
        if(TextureCoordinate* tmp=dynamic_cast<TextureCoordinate*>(texCoord.p)){
            return tmp->point.refValue();
        }
        throw this;
    }

    virtual void genNormals()=0;
    virtual void genTransformedNormals(M_Art& partList)=0;
    virtual void addExtraCoords(Seamless* smls)=0;
    virtual void groupTrianglesByEffect(Art* art, M_Node& m_shape, int& triPos, Effect* lastEffect)=0;
    virtual void deleteUnwantedTriFromSource()=0;

    void genColorList(Color3f c);
    void genColorList(float r=1, float g=1, float b=1);
};
class IndexedFaceSet : public VertexGeometry, public MField18 {
public:
    class CoordIndex : public MFInt32 {
    public:
        bool& renderProcessed;
        CoordIndex( const M_Int& value, bool& renderProcessed):
            MFInt32(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const M_Int& par){
            refValue()=par;
            renderProcessed=false;
        }
        void resize(int newLen){
            refValue().length=newLen;
            renderProcessed=false;
        }
    };
    class NormalIndex : public MFInt32 {
    public:
        bool& renderProcessed;
        NormalIndex( const M_Int& value, bool& renderProcessed):
            MFInt32(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const M_Int& par){
            refValue()=par;
            renderProcessed=false;
        }
        void resize(int newLen){
            refValue().length=newLen;
            renderProcessed=false;
        }
    };
    class ColorIndex : public MFInt32 {
    public:
        bool& renderProcessed;
        ColorIndex( const M_Int& value, bool& renderProcessed):
            MFInt32(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const M_Int& par){
            refValue()=par;
            renderProcessed=false;
        }
        void resize(int newLen){
            refValue().length=newLen;
            renderProcessed=false;
        }
    };
    class TexCoordIndex : public MFInt32 {
    public:
        bool& renderProcessed;
        TexCoordIndex( const M_Int& value, bool& renderProcessed):
            MFInt32(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const M_Int& par){
            refValue()=par;
            renderProcessed=false;
        }
        void resize(int newLen){
            refValue().length=newLen;
            renderProcessed=false;
        }
    };

    int getType()const{return INDEXEDFACESET;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    MFInt32 set_coordIndex;
    MFInt32 set_normalIndex;
    MFInt32 set_colorIndex;
    MFInt32 set_texCoordIndex;
    CoordIndex coordIndex;
    NormalIndex normalIndex;
    ColorIndex colorIndex;
    TexCoordIndex texCoordIndex;
    SFBool convex;
    SFFloat creaseAngle;

    IndexedFaceSet():
        MField18(
            &set_coordIndex,
            &set_normalIndex,
            &set_colorIndex,
            &set_texCoordIndex,
            &coord,
            &normal,
            &color,
            &texCoord,
            &ccw,
            &coordIndex,
            &normalIndex,
            &colorIndex,
            &texCoordIndex,
            &normalPerVertex,
            &colorPerVertex,
            &convex,
            &creaseAngle,
            &solid
        ),

        set_coordIndex(M_Int()),
        set_normalIndex(M_Int()),
        set_colorIndex(M_Int()),
        set_texCoordIndex(M_Int()),
        coordIndex(M_Int(), renderProcessed),
        normalIndex(M_Int(), renderProcessed),
        colorIndex(M_Int(), renderProcessed),
        texCoordIndex(M_Int(), renderProcessed),
        convex(true),
        creaseAngle(0)
    {
#ifdef SEAMLESS3DGUI
        index.length=4;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
#endif
    M_Int& refCoordIndex(){
        return coordIndex.refValue();
    }
    M_Int& refColorIndex(){
        return colorIndex.refValue();
    }
    M_Int& refTexCoordIndex(){
        return texCoordIndex.refValue();
    }


//	void insideOut();
    void triangle(
        float ax, float ay, float az,
        float bx, float by, float bz,
        float cx, float cy, float cz
    );

    void genRadialBand(int dwNumStaves, int dwNumRings, float outerRad, float innerRad, int i_bottomIsPoint, int i_topIsPoint);

//	void tug(Vec3f centre,float rad);
//	void tug2(Vec3f centre,float rad, Rotation rot=Rotation(0,0,1,0), float sc=1,float cutOut=-1000);
//	void invSphereTug(float x, float z, float height,float rad, float innerRad,float sc,float cutOut=-1000);
//	void tugMouthY(IndexedFaceSetPtr mouthNonFlat);
//	void tugMouthZ(IndexedFaceSetPtr mouthNonFlat);
//	void tugToShapeY(IndexedFaceSet& mouthNonFlat, float height);
//	void tugToShapeZ(IndexedFaceSet& mouthNonFlat, float height);

//	void rectToSphere();
//	void circleToSphere();
//	void negCircleToSphere();
//	void flattenY();
//	void flattenZ();



    void ellipticalStretch(float rad,float sc,float cutOut);
    void sectionTug(float tugX,float up,	float dn,float cutOut);
    void sectionScale(float start,float height);


    void genNormals();
//	void GenNormals(int bottomIsPoint,int topIsPoint);
//	void GenNormals(M_Vec3f& coordBottomRing, M_Vec3f& coordTopRing, int bottomIsPoint,int topIsPoint);
//	void genFlatTextureMapZ( float left,float right, float up, float down,float xOffset, float yOffset, float xScale, float yScale);

    void joinParts(IndexedFaceSetPtr Sec);
//	void blendNormals(IndexedFaceSetPtr sec,int count, M_Int& joinIndices, M_Int& joinSec);
//	void joinParts(IndexedFaceSetPtr sec,int count, M_Int& joinIndices, M_Int& joinSec, int joinOffsetDst=0, int joinOffsetSrc=0);
//	void joinPartsReplaceChildVertices(IndexedFaceSetPtr sec,int count, M_Int& joinIndices, M_Int& joinSec);
//	void replaceVertices(int count, M_Int& joinIndicesDst,M_Int&  joinIndicesSrc, int joinOffsetDst=0, int joinOffsetSrc=0);
//	void rightSnapToLeft(IndexedFaceSetPtr right, int count, M_Int& joinIndices, M_Int& joinSec, int joinOffsetDst=0, int joinOffsetSrc=0);
//	void leftSnapToRightJoin(IndexedFaceSetPtr right,int count, M_Int& joinIndices, M_Int& joinSec, int joinOffsetDst=0, int joinOffsetSrc=0);

//	void copyNotClose(IndexedFaceSetPtr sec);
    //IndexedFaceSetPtr copy();
    void translate(float tx,float ty,float tz);
    void translate(Vec3f vec);
    void rotate(Rotation);
    void rotate(float x, float y, float z, float angle);
    void scale(float tx,float ty,float tz);
    void scale(Vec3f sc);
    void scale(float sc);
    void scale(float yPlane,float sc);
    void convertToTriangles();
    //void convertToRenderFormat(Render*& render);

//	void GenFacetNormals();
//	void Init();
//	void GenIndices();
//	void genIndices();
//	void GenIndices(int ring,int ringsToDo,int oldNumStaves,int newNumStaves,int& returnIndices);
//	void GenFlat(int XPointCnt,int ZPointCnt,float SquareLength, int i_BottomIsPoint, int i_TopIsPoint, float uOffset, float vOffset, float uScale, float vScale);

//	void genGridZ(float xDistance=1, float yDistance=1, int xPointCnt=2,int yPointCnt=2, int flags =0);

//	void deleteNonTriangles();
//	void deleteVertex(int d);
//	void deleteUnusedVertices();
//	void deleteSamePlaceVertices(float dist=.0001);
//	void fastDeleteTriangle(int delTri);
//	bool deleteTriangle(int delTri);
//	bool deleteTriangles(int* delTri, int sum);
//	void splitTri(int ActiveTri, int ActiveCorner);
//	void mirrorTri(int ActTri, int ActTriCorner);
//	void setTriIndex(int ActiveTri, int ActiveCorner, int ToTri, int ToCorner);
//	void setTriCornerToIndex(int tri, int corner, int toCoord);
    //void DeleteDeadVertices();///untested
    //void cutXZ(float xpos1,float xpos2,float zpos1,float zpos2);
    //void cutXY(float xpos1,float xpos2,float ypos1,float ypos2);
//	Vec3f getTriPoint(int ActiveTri, int ActiveCorner);
//	int getTriPointIndex(int ActiveTri, int ActiveCorner);
//	void setTriPoint(int ActiveTri, int ActiveCorner,Vec3f coordNewPoint);
//	void setTriPoint(int ActiveTri, int ActiveCorner,Vec3f coordNewPoint,Vec3f normalNewPoint,Vec2f texNewPoint);
//	void joinVertex(int from, int to);
//	int generateNewVertex(int ActTri, int ActTriCorner);
//	int swapVertex(int oldIndex, int newIndex);

    //void halveStaves(int ring,int ringsToDo);
//	void insideOut();
//	void mirrorX();
//	void mirrorY();
//	void mirrorZ();
//	void vRotX(float angle);
//	void vRotY(float angle);
//	void vRotZ(float angle);
//	void vTran(float x,float y,float z);
//	void vScale(float x,float y,float z);
//	void vRectToEllipsoid(float eccentricity);
//	void vTransform();


//	void flatZToCylinder(float rad=1);
//	void ellipsoid(int xPointCnt=17,int yPointCnt=17,float startZAngle=-PI*.5,float endZAngle=PI*.5,float rad=1, float eccentricity=1,float startAt=0);
//	void flatZToEllipsoid(float rad, float eccentricity, float xzOffset);
    //int swapJoiningVertices(int triangle1,int triangle2);

//	void genEndRingNormalsY();
//	void genRingNormals( int count, int offset, int endMeetStart=0, Rotation rot=Rotation(), float startAngle=PI*.5, float endAngle=PI*2.5);

    void write(String filename);
    //bool loadGZippedPart(String filename);
    void swapTriangles(int poly, int search);
    void addExtraTexCoord();
    void addExtraColor();
    void addExtraCoordAndNormalAndTexCoord();
    void genTransformedNormals(M_Art& partList);
    void genTransformedNormalsOnlyBlueParts(M_Art& partList);
    void addExtraCoords(Seamless* smls);
    void groupTrianglesByEffect(Art* art, M_Node& m_shape, int& triPos, Effect* lastEffect);
    void deleteUnwantedTriFromSource();
    void convertFromGeomSource(const Mesh& source);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;

    void toPOVRayString(String& str);
    void toObjString(String& str);

};
class Seamless : public Node, public PovRay, public MField24, public VisualAid{
    public:

class Scale : public SFVec3f {
public:
    Seamless* home;

    Scale(const Vec3f& i_value):
        SFVec3f(i_value)
    {};
    void operator=(const Vec3f& par){
        refValue()=par;
        home->catMatrix();
        home->catIMatrix();
    }
    operator Vec3f& (){
        return refValue();
    }
    operator const Vec3f& () const{
        return refValue();
    }
};

class Rotation2 : public SFRotation {
public:
    Seamless* home;
        Rotation2(const Rotation& i_value):
            SFRotation(i_value)
        {}
        void operator=(const Rotation& par){
            refValue()=par;
            home->catMatrix();
            home->catIMatrix();
        }
};

class Translation : public SFVec3f {
public:
    Seamless* home;
        Translation(const Vec3f& i_value):
            SFVec3f(i_value)
        {}
        void operator=(const Vec3f& par){
            refValue()=par;
            home->catMatrix();
            home->catIMatrix();
        }
};

class Bnspe : public SFBool {
public:
    Seamless& home;

    Bnspe(const bool& value, Seamless& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class XRay : public SFBool {
public:
    Seamless& home;

    XRay(const bool& value, Seamless& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class AutoIn : public SFBool {
public:
    Seamless& home;

    AutoIn(const bool& value, Seamless& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class AnimCP : public SFBool {
public:
    Seamless& home;

    AnimCP(const bool& value, Seamless& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class BuildNormals : public SFBool {
public:
    Seamless& home;

    BuildNormals(const bool& value, Seamless& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class SurfaceNormals : public SFBool {
public:
    Seamless& home;

    SurfaceNormals(const bool& value, Seamless& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class Skeleton : public MFNode{
public:
    Seamless& home;
    Skeleton(const M_Node&, Seamless& home):
        MFNode(M_Node()),
        home(home)
    {}
    M_SArt& refValue(){
        return *(M_SArt*)&length;
    }
    operator const M_SArt&()const{
        return *(M_SArt*)&length;
    }
    void operator=(const M_Node& par);
    void operator+=(const NodePtr& par);
    void operator+=(Node* par);
};

class EffectField : public SFNode {
public:
    Seamless* home;

    EffectField( const NodePtr& par):
        SFNode(par)
    {};
    Node* operator = (Node* par1);

    operator Effect*()const {
        return (Effect*)p;
    }
    const Effect* operator ->()const{
        return (const Effect*)p;
    }
};


    int getType()const{return SEAMLESS;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return &index;}

    SFFloat version;
	MFVec3f cp;
	MFFloat cw;
    Scale scale;
    Rotation2 rotation;
    Translation translation;
    SFBool track;
    SFInt32 tesselate;
	SFBool autoJoin;
    SFBool autoStitch;
    Bnspe bnspe;
    XRay xRay;
    AutoIn autoIn;
    SFBool masterIn;
    AnimCP animCP;
    BuildNormals buildNormals;
    SurfaceNormals surfaceNormals;
    SFBool ignoreWireFrame;
    SFBool shadow;
    MFInt32 allot;
    MFInt32 stitchIndex;
    EffectField effect;
    Skeleton skeleton;
    MFBuild build;
    //SFNode mirrorGroup;

    Seamless():
        MField24(
            &version,
			&cp,
			&cw,
            &scale,
            &rotation,
            &translation,
            &track,
            &tesselate,
			&autoJoin,
            &autoStitch,
            &bnspe,
            &xRay,
            &autoIn,
            &masterIn,
            &animCP,
            &buildNormals,
            &surfaceNormals,
            &ignoreWireFrame,
            &shadow,
            &allot,
            &stitchIndex,
            &effect,
            &skeleton,
            &build
            //&mirrorGroup
        ),

        version(0),
		cp(M_Vec3f()),
		cw(M_Float()),
        scale(Vec3f(1,1,1)),
        rotation(Rotation()),
        translation(Vec3f()),
        track(true),
        tesselate(4),
		autoJoin(true),
        autoStitch(true),
        bnspe(false, *this),
        xRay(false, *this),
        autoIn(false, *this),
        masterIn(false),
        animCP(false, *this),
        buildNormals(false, *this),
        surfaceNormals(false, *this),
        ignoreWireFrame(false),
        shadow(true),
        allot(M_Int()),
        stitchIndex(M_Int()),
        effect(0),
        skeleton(M_Node(), *this),
        build(M_Node()),
        //mirrorGroup(0),

/////////////////////////////
        render(0),
//		selCtrlPoly(0),
        ActCtrlPnt(0),
        remActCtrlPnt(0),
        ActTri(0),
        actTriPart(0),
        remActTriPart(0),
        remActTri(0),
        ActTriCorner(0),
        remActTriCorner(0),
        morphFlag(false),
        animCPBuildFlag(false),
        ikProcessFlag(false),
        ctrf(false),

        outSegmented(false),
        atThisStage(this),
        stickMan(0),
        pointTotal(0),
        workAnimCP(false),
        cpMorpherPartLastLen(0),
		renderIndexLen(0)

    {
#ifdef SEAMLESS3DGUI
		index.length = 2;
#endif
        scale.home = this;
        rotation.home = this;
        translation.home = this;
        effect.home = this;
    }
    bool workAnimCP;
    int pointTotal;
    M_Int coordIndex;
    M_Int patchOwner;
    bool outSegmented;
    StickMan* stickMan;
    ShapePtr nurbsLineShape;
    M_<NurbsLine*> nurbsLine;

    Matrix matrix;
    Matrix imatrix;

	bool insertControlStitchPoly(const bool& perspective);
    void readLink(const String& str, int& i);
    void resetRelevantParts();
    bool verifyAllRelevantPartsExist();//does not check if any are irelevant
    void verifyIntegrity();//Fact: All inferior parts have no relivance (ilogical for them to own triangles that reference up)
    void removeBuildNodes(M_Node& par1);
    void convertToRc(int buildLength);
    void encodePatchOwner();
    void encodeAllot();
    void extraSeamlessToString(String& str);
    void setUpInternalEffectPtrs();
    void setUpInternalEffectPtrs2(); //this (2) wont be needed when setUpInternalEffectPtrs is fixed
    void processProduct(Art* part, VertexGeometryPtr& geometry, M_Int& partTree, M_Vec3f& scalePart, M_Rotation& o, M_Vec3f& translatePart, M_Vec3f& pivotPoint, Vec3f accumScale, Matrix accumMat, Matrix accumRotMat, int matrixStack=-1);
    void catMatrix(Matrix matrix=Matrix());
    void catTranslationMatrix();
    void catScaleTranslationMatrix();
    void catIMatrix(Matrix matrix=Matrix());
    void catMatrix(M_Rotation4d& m_accumRot);
    void catIMatrix(M_Rotation4d& m_accumRot);
    void buildPartListNoShow();
    void buildPartList();
    void processLoop(Art* part, int &coordLenTally, int &_plidx, Matrix parentMatrix );
    void process();
    void fillLegalCoordGroups();
    bool checkUsingPivotPoints();
    void convertPivotPointsToTranslationsNoChangePivotPoint();
    void resetFromCoordAndWorkoutVerticesForGroupCoords();
    void convertToColorIndex(IndexedFaceSet* ifs);
    void outputToString(String& str, bool allowRef=true);
    void orderPolygonsFromVertices();
    void defragEffect();
    void createContigLookUpList(M_<NurbsPatch*>& contigLookUpList);
    void cpNodeListAddForStem(M_Node& m_stem);
    void cpNodeListAdd();


//	IndexedFaceSetPtr renderIfs;
	void addCageControlPolys();
    void addCageControlPolysColorCode();

    void addStraightCageControlPolys();
    void addStraightCageControlPolysColorCode(const M_Bool& m_pointIsVisible);
    void colorLinkPoints();
    void getNextCtrlPoly();
    void getPrevCtrlPoly();
    void deleteControlRowColum();
    void insertControlRowColum();
    bool addDiversionMark();
    bool addDiversionMark2(M_Insert& m_insert, int& beginInsertIdx);
    void findClickedPolyEdge();
    void deleteControlLine();
	void mergePatchesStandby();
    void splitPatch();

    void processFields();
    void processBuild();

    IndexedFaceSetPtr geometry;
    void createControlPolygon(M_ControlPoint& pointList);
    void selectControlPoints();
    void redirectStitchPolyIndices(NurbsPatch* patch, const M_Int& lookup);
    void redirectStitchPolyIndices(NurbsPatch* patch, const M_ControlPoint& lookup);
    void calcAllPolyMidPoints();
    void calcAllLinks();
    void deleteSyncPolies();
    void fastDeletePoliesOwnedByPatch(NurbsPatch* patch);
    void deletePoliesOwnedByInvalidPatchs();
    void deletePoliesOwnedByPatch(NurbsPatch* patch);
    void deleteAwkwardPoliesLeftBehind(NurbsPatch* patch, DelPoly& delPoly);
    void transferBorderVertices(NurbsPatch* patch);
    void joinBorderVertices(NurbsPatch* patch);
    bool isCloseCoord(const NurbsPatch* patch, const Vec2f coord, Art* dstPart, int dstIdx);
    void transferCoordOverPatchBorder(Art* part, int crdIdx);
    bool getTransferedCoordOverPatchBorder(const Art* part, int crdIdx, Vec2f& returnCoord, NurbsPatch*& returnPatch)const;
    bool getTransferedCoordOverPatchBorderWest(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& returnPatch)const;
    bool getTransferedCoordOverPatchBorderSouth(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& returnPatch)const;
    bool getTransferedCoordOverPatchBorderEast(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& returnPatch)const;
    bool getTransferedCoordOverPatchBorderNorth(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& returnPatch)const;

    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    String genProduct();
    void adjustDescendents(int plidx, int coordDiff, int polyDiff);
    void convertTo028();
    int getIndexForLastBuildNode(M_Node*& r_build);


    Art* decToFindPart (int partIndex, int targetVertex);
    Art* incToFindPart (int partIndex, int targetVertex);
    Art* getPartFromVertex(int targetVertex, Art* part);

    Art* decToFindPartFromPoly (int partIndex, int targetPoly);
    Art* incToFindPartFromPoly (int partIndex, int targetPoly);
    Art* getPartFromPoly(int targetPoly, Art* part);
//	void processRenderIfs();
    void convertToRenderFormat();
    void convertToSimpleRenderFormat();
    void convertToOwnershipAidRenderFormat();
    bool loadGZippedPart(Art* part, String filename);
//	void convertToCoordLen();
//	void setFromToCoord();
    void doBuild();
    void doBorderlessBuild();

    void transformSkinMesh();

    //void replaceColor(Color3f oldColor, Color3f newColor);
    void updateAllVertexInBuffers();
    void addNewHAnimNode();
    void addNewSegmentedNode();
    void addExtraCoordsRender(M_Int& coordIndex);
    void addExtraCoordsRenderForGeomSource(M_Int& coordIndex);
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);

    void senseMirrorVertex(Art* part, int idx, Vec3f& newPt);
    Art* getMirrorVertex(Art* part, const int vertexIndex, int& returnVertex);
    Art* getMirrorTriangle(Art* part, int& returnCoordIdxIdx);

    void stitchControlPolygons();
    void stitchFlower();
    void commonEndSitchPolys();
    void mergeTo5CtrlPoly();
    void mergeTo6CtrlPoly();
    bool findThreeClosePoints(M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList);

    Matrix getSmlsViewMatrix();
    Matrix getSmlsIViewMatrix();

    void mergeControlPolygon(int polyA, int a0, int a1, int polyB, int b0, int b1, M_Int& deleteCtrlPolySelList);
    bool findCtrlPolyWith2SelectedClockwise(M_Int& ctrlPolySelList, NurbsPatch* patchA0, int polyA, int a0, int a1, const Vec3f vec0, const Vec3f vec1, M_Int& deleteCtrlPolySelList);
    void findCtrlPolyWith2SelectedAntiClockwise(M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList);
    bool findCloseCornerLoop(int polyA, int corner, M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList, M_Int& m_corner, int checkToPoly);
    void findCloseCorner(M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList, M_Int& m_corner);
    int getMidOffset(int poly, int corner);
    void create5SidedPoly(int polyA, int a0, int a1, int polyB, int b0, int b1, int polyC, int c0, int c1);




    void createSelContigList(M_<NurbsPatch*>& contigList);
    int getCtrlPolyLengthInTri(int& coordLen);
    void genTransparentCtrlPolys(Art* part, const M_Vec3f& point);
    void genSelectedBoxPolys(Art* part, const M_Vec3f& point);
    int findClosestMidPoint(const M_Int& ctrlPolySelList, const Vec3f selCorner, int& corner, int& blockIdx);
    int getNurbsCorner(int& c0, M_Int& ctrlPolySelList);
    void mergeTo5CtrlPoly2(int polyC, int c0, int polyA, int a0, int a1, int polyB, int b0, int b1);
    void selectCtrlPolys();
    void unselectCtrlPolys();
    void selectBoxModelingPatchPolys();

    bool isCtrlPolyTaken(int a, int b, NurbsPatch* patch);
    bool isCtrlPolyCornerTaken(int a, int b, NurbsPatch* patch);
//	void addSeamlessRefToSubdiv(M_Build& M_Build);

//	void updateRenderNormals();
    void decodePatchOwner();
    void decodeStitchIndex(M_<NurbsPatch*>& contigLookUpList);
    int isCornerStitchPoly(int poly);
    int get2Edges(int poly, int& edge0, int& edge1);
    int isBlockStitchPoly(int poly);
    int isWedgeStitchPoly(int poly);
    void sortStitchPolys(M_Int& sortedPoly, M_Int& sortedPoint, M_Int& ctrlPolySelList, M_Int& polyType);
    bool pointIsNotInNewPoly(int mark, NurbsPatch* patch, int idx);
    void shareWeightInStitchPoly(NurbsPatch* patchA, int& idxA);

    void newBuildNode(Build* node);
    M_Int index;
    void updateBuild();
    void senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor);
    void catInverseRotation(Rotation& r);
    void resetRotations();
    void resetRotationsAndTranslations();
	void genCage();
	void genStraightCage(const M_Bool& m_pointIsVisible);
    Render* render;

    void calcAbsolutePivotPoint();

    M_Art partList;
    M_<Effect*> effectList;
    M_<NurbsPatch*> cpNodeList;
    M_CtrlPolyPoint ctrlPolyPoint;
    M_Link link;
    Ptr<PosCompensator> posCompensator;

    //M_ControlPoint handlePoint;
    //handleData;
    //M_<StitchInstruction> stitchIdxIdx;

    //number of points in polygon
    //indices to points
    //number of mid points for first edge
    //indices to mid points
    //number of center points
    //indices to center points


    //M_Int ctrlPolyPointIndex;
/*
    int numOfStitchesInPoly(int sii){ //sii indexes stitchIdxIdx
        int i = stitchIdxIdx[sii];
        if(sii == stitchIdxIdx.length - 1){
            return workStitchIndex.length - i;
        }
        else{
            return stitchIdxIdx[sii + 1] - i;
        }
    }
*/
    void linkPointToEdge(int poly);
    void addMidPoint(M_ControlPoint& pointList);
    int getNextCtrlPoly(int ctrlPoly);
    void delAllCtrlPolyRefs(NurbsPatch* patch);
    void dragCtrlPoly(NurbsPatch* patch, int idx);
	void joinBetweenPoints(NurbsPatch* patch1, const int idx1, NurbsPatch* patch2, const int idx2);
    void moveNurbsCP(bool join = false);
    void populateMirrorNodes();
    void genVisibleMask();
    void removeAllPatchesFromPartsNotInList(M_Art& m_part);
    void deleteCtrlPoly(int ctrlPoly);
    void deleteCtrlPoly(M_Int& deletePolyList);
    void deleteLinksToPolys(M_Int& deletePolyList);
    void deleteLinksToNonExistentPatches();
    bool pointsExist(M_Int& deletePolyList, int poly, int& m, int len);
    void deletePolysWithNonExistingPoints();
    void genDynamicMatricesForSelectedPolyCoords();
    int setCoordFroms();
    void populate_g_m_coord();
    void genDynamicMatricesAndPopulate();
    void senseAllPartsVertex();
    void calcIK();
    void calcIKLoop(Art* pelvis, Art* hip, Art* knee, Art* ankle);
    void removeMidPointsOwnership(const NurbsPatch* patch, const int idx);
    void removeOwnershipPatch(const NurbsPatch* patch);
    void removeMidPointsOwnership(const int i, const M_CtrlPolyPoint& ctrlPolyPoint);
    void removeAllPolyMidPointsOwnership();
    void applyRectOwnership(const Art* part, M_<CPContigElement>& contigLookUpList, M_<NurbsPatch*>& cpNodeList, const int& selGroupIdx, const int& selGroupIdxMirror);
    void setModellingPatchMode();
    void clearObjects();
//private:
    //int selCtrlPoly;

    int ActTriCorner;
    int ActTri;
    int ActCtrlPnt;
    Art* actTriPart;
    int remActTri;
    int remActCtrlPnt;
    int remActTriCorner;
    Art* remActTriPart;
    NodePtr outputNode;
    bool addScaleField;
    bool ctrf;	//convert to render format
    bool morphFlag;
    bool animCPBuildFlag;
    bool ikProcessFlag;
    Node* atThisStage;
    static Vec3f selCoordPos;
    M_<CPMorpherPart> cpMorpherPart;
    int cpMorpherPartLastLen;
    bool inPolyCenter(NurbsPatch* patch, const int idx0, const int idx1, const int idx2);
    void whereYouSeeAddThis(const NurbsPatch* checkPatch, const int checkIndex, NurbsPatch* addPatch, int addIndex);
    void findIllegalStitchPoly(M_Int& deleteCtrlPolyList);
    void findIllegalStitchPoly(M_Int& deleteCtrlPolyList, int poly);
    void findStitchPolyAndAdd(const NurbsPatch* checkPatch, const int checkIdx, NurbsPatch* addPatchA, int addIdxA, int centerGuideIdxA, int nextCornerA, NurbsPatch* addPatchB, int addIdxB, int centerGuideIdxB, int nextCornerB);
    void findStitchPolyForExtrude(M_Int& deleteCtrlPolyList, NurbsPatch* checkPatch, int checkIdx);

    bool insertLineFindClickedPoly(NurbsPatch* checkPatch, const int originIdx, int checkIdx, int checkIdx2, bool doU, bool leaveNorthOrEast, const int firstStitchPoly, bool& foundFirstPatch, const bool leftSide, const float ratio, const bool invertRatio, M_Insert& m_insert, int insertIdx, int& beginInsertIdx, bool& forbid);
    void insertLineFindMatchingEdge(NurbsPatch* checkPatch, int checkIdx, int checkIdx2, bool doU, bool leaveNorthOrEast, const int firstStitchPoly, bool& foundFirstStitchPoly, const bool leftSide, const float ratio, const bool invertRatio, M_Insert& insert);

    void deleteLineFindMatchingEdge(NurbsPatch* checkPatch, int checkIdx, int checkIdx2, const NurbsPatch* firstPatch, const int firstIdx2, bool& foundFirst, M_DelCtrlLn& delCtrlLn);
    bool isPointInCenterOfNon4SidedStitchPoly(NurbsPatch* checkPatch, int checkIdx);
    void vEastFindStitchPolyAndSplit(M_Int& deleteCtrlPolyList, NurbsPatch* checkPatch, const int checkIdx, const int checkIdx2, const NurbsPatch* firstPatch, bool& foundFirstPatch, const float ratio);
    void uFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch);
    void uFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch, int uIdx);
    void vFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch);
    void vFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch, int vIdx);
    int uSlideEdgeStitchPoly(int poly, NurbsPatch* checkPatch, int uIdx);//returns next poly idx
    int vSlideEdgeStitchPoly(int poly, NurbsPatch* checkPatch, int vIdx);//returns next poly idx
    void copyStitchPoly(int poly);
    void uCopyStitchPolyAndSlide(int poly, int uOffset);
    void vCopyStitchPolyAndSlide(int poly, int vOffset);

    void joinAllSelectedStitchPolys();
    void initMultiCageTug(const bool& perspective);
    bool initSkeletonTug(const bool& perspective);
    void clickPointWithinSights(NurbsPatch* patch);
    void selectPatchGroupWithinSights();

    bool initTugStitchPoly(const bool& perspective);
    void generateRawStitchPolys();
    void synchronizeAllStemCells();
    void stitchAllPolys();
    void stitchSinglePatchPolys(NurbsPatch* patch, int numOfPatches = 1);
	void calcAllPolyMidPointsAndpatchToCentralCPs(NurbsPatch* patch, int numOfPatches = 1);
    void genRegionPolygons(Art* part, const M_Vec3f& point, int brdrOffstt, int numOfCorners, int s, const M_Int& region, M_Color3f& color, int regionSelectedIdx);
    bool getPatchJoinedByStitchPoly(const NurbsPatch* checkPatch, int checkIdx, NurbsPatch*& otherPatch, int& otherIdx)const;
    bool getPatchEdgeJoinedByStitchPoly(const NurbsPatch* checkPatch, int checkIdx, NurbsPatch*& otherPatch, int& otherIdx)const;
    bool transferVertexOverBorderWest(NurbsPatch* checkPatch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord);
    bool transferVertexOverBorderEast(NurbsPatch* checkPatch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord);
    bool transferVertexOverBorderSouth(NurbsPatch* checkPatch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord);
    bool transferVertexOverBorderNorth(NurbsPatch* checkPatch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord);

    bool joinBorderVerticesWest(const NurbsPatch* checkPatch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx);
    bool joinBorderVerticesEast(const NurbsPatch* checkPatch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx);
    bool joinBorderVerticesSouth(const NurbsPatch* checkPatch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx);
    bool joinBorderVerticesNorth(const NurbsPatch* checkPatch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx);

    bool triangleIsTransferable(const S_Triangle& triangle, NurbsPatch* delPatch);
    bool transferableVertexOverBorderWest(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch);
    bool transferableVertexOverBorderEast(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch);
    bool transferableVertexOverBorderSouth(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch);
    bool transferableVertexOverBorderNorth(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch);
    bool isPatchJoinedByStitchPoly(const NurbsPatch* checkPatch, const int checkIdx, const NurbsPatch* otherPatch);
    bool isPatchJoinedByStitchPolyCorner(const NurbsPatch* checkPatch, const int checkIdx, const NurbsPatch* otherPatch);
    int getStitchPoly(NurbsPatch* patch, int checkIdx, int checkIdx2)const;
    void joinBorderVertices();
    void joinAllParts();
    void deleteAllParts();
	void deleteAllSyncParts();
	void deleteAllPartsNonBnspe();
    void generateAllPatchesBnspe();
	void generateAllSyncPatchesBnspe();
    void importTriangles(Seamless* srcSmls);
    bool senseAllPartsTriangleRayTrace(RayView& rayView);
    void convertNormalsToShade(M_Float& shade);
    void rasterTrianglesToVoxels();
    bool colourWithBrush(Art* part);
	bool colourPatchesWithBrush();
	void updateColorFromColorMappers(Art* part);
    void getMostNSEW(float& north, float& south, float& east, float& west, float& in, float& out);
    void remObjTransform();
    void doObjTransform();
    void scaleSkeleton(Vec3f pivotPoint, Vec3f scale);

    void houseKeep();
    void drag(Vec3f pt);

    void mirrorMorphedCtrlPoints(Art* targetPart, Anim* anim, bool left);
    void linkToString(String& str);
    void resetCoordLen();
    void setPointFrom();
    void flagCtrlPolyPointRefs();
    void decodeLink(M_<NurbsPatch*>& contigLookUpList);
    void decodeAllot();
    void updateCage();
	bool isCrossOver(int midIdx, const NurbsPatch* patchA, const int actIdxA, const NurbsPatch* patchB, int& midActIdxA, int& midActIdxB)const;
	void convertToCpFormat();
	void centralCPsToPatches();
	void patchesToCentralCPs(); //for debugging only
	void createSharedPointsFromStitchPolys();
	void createCpFromPatchPoints();
	void cleanCps();
	bool stitchPolysAreCorrupt();
	void addCps(Build* build, int mark);
	void shareMidPointsTooInStitchPolys(NurbsPatch* patchA, const int idxA);
	void unShareMidPointsTooInStitchPolys(NurbsPatch* patchA, const int idxA);
	bool seekCtrlPointInStitchPolys(const NurbsPatch* patch, const int idx, int& poly, int& corner);
	bool seekStitchPolyContainingSharedPoint(const int cpIdx, int& poly, int& corner, int blockPoly);
	bool seekCornerContainingSharedPoint(const int cpIdx, int poly, int& corner);
	bool shareMidPointsTooInStitchPolys2(NurbsPatch* patchA0, const int idxA0, int& poly0);
	bool unShareMidPointsTooInStitchPolys2(NurbsPatch* patchA0, const int idxA0, int& poly0);
	void redirectSingleCpIndex(int toCpIdx, int fromCpIdx);
	int addCpsForStrip(NurbsPatch* shaft, int newUdim, int newVdim);
	void genDynamicTransforms();
	//void redirectSingleCpIndexExtra(NurbsPatch* patch1, const int idx1, NurbsPatch* patch2, const int idx2);
	void selectWholePatches();
	void doLastPhaseForSelectRing();
	void houseKeepNurbsCages();
	void generatePoliesForSelPatches();
	void generatePoliesForPatches();
	void replacePatchRef(NurbsPatch* from, NurbsPatch* to);
	

	int renderIndexLen;



    M_Vec3f& refPoint(){
        IndexedFaceSet* geom=geometry;
        if(geom==0)return *(M_Vec3f*)geom;
        Coordinate* tmp=dynamic_cast<Coordinate*>(geom->coord.refValue().getRawPtr());
        if(tmp==0)return *(M_Vec3f*)tmp;
        return tmp->point.refValue();
    }
    M_Vec3f& refVector(){
        IndexedFaceSet* geom=geometry;
        if(geom==0)return *(M_Vec3f*)geom;
        Normal* tmp=dynamic_cast<Normal*>(geom->normal.refValue().getRawPtr());
        if(tmp==0)return *(M_Vec3f*)tmp;
        return tmp->vector.refValue();
    }
    M_Color3f& refColor(){
        IndexedFaceSet* geom=geometry;
        if(geom==0)return *(M_Color3f*)geom;
        Color* tmp=dynamic_cast<Color*>(geom->color.refValue().getRawPtr());
        if(tmp==0)return *(M_Color3f*)tmp;
        return tmp->color.refValue();
    }
    M_Vec2f& refTexPoint(){
        IndexedFaceSet* geom=geometry;
        if(geom==0)return *(M_Vec2f*)geom;
        TextureCoordinate* tmp=dynamic_cast<TextureCoordinate*>(geom->texCoord.refValue().getRawPtr());
        if(tmp==0)return *(M_Vec2f*)tmp;
        return tmp->point.refValue();
    }
    M_Int& refCoordIndex(){
        IndexedFaceSet* geom=geometry;
        if(geom==0)return *(M_Int*)geom;
        return geom->coordIndex.refValue();
    }

    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;
};
class Effect : public Node {
public:
    Effect():
      triLen(0)
    {}
    virtual int getEffectLength()=0;
    int triLen;
    M_Int lookup;
    Shape* outputShape;
    M_Art partList;
    inline virtual void convertPartToIFS( Art* thisPart, IndexedFaceSet* ifs, M_Vec3f& ifsCoord, M_Int& ifsCoordIndex, const M_Vec3f& coord, const M_Triangle& triangle)=0;
    virtual void convertGeomSourceToPart(Art* part, const Mesh& source)=0;
    virtual void convertGeomSourceToPart(Art* part, const Mesh& source, const M_Vec2f& gsUvCoord, const M_<NurbsPatch*>& gsPatchOwner)=0;
    virtual void stemGenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s)=0;
    virtual void stem2GenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s)=0;
    virtual void stemGenerateTri(Stem& stem, const ALathe& lathe, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s)=0;
    virtual void stemDupMirrorTri(Art* part, M_Vec3f& coord, M_Triangle& triangle, const int h, const int c, const int s, const int coordLen, const int colorLen, const int triLen)=0;
    virtual int stemCopyLocalTri(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s)=0;
    virtual int stemCopyLocalTriNonMirror(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s)=0;
    virtual void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec2f& gsUvCoord, M_<NurbsPatch*>& gsPatchOwner, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec2f& uvCoord, const M_<NurbsPatch*>& patchOwner, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const=0;
    virtual void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const=0;
    virtual void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec3f& gsNormal, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec3f& normal, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const=0;
    inline virtual void optimize(M_Art&)=0;
    virtual void addColorNode(VertexGeometry* outputGeometry, VertexGeometry* sourceGeometry)const =0;
    virtual void copyFieldsData(Shape* shape)=0;
    virtual void addTextureMapper(Stem&){}
    virtual void textureMapDisk(Art*, float, float){}
    virtual void updateNewEffect(M_Art&)const{}
    virtual void copyColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle)=0;
    virtual void copyAddColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle,int& markTriangle)=0;
    virtual void copyColorData(Effect* effect)=0;
    virtual void setColorIndex(int index)=0;
    virtual void setUpVertexShaders(Matrix& parentMatrix, Seamless& smls)=0;
    virtual void renderMaterial(int triFrom, Matrix& parentMatrix, Seamless& smls, Matrix& originalMatrix, Vec3f& originalColScale)=0;
    virtual void outPovRay(const M_Art& partList, String& str, int vertexCount, M_Int& coordIndex, const int triLenTotal)const=0;
    virtual void outObj(const M_Art& partList, String& str, String& mtl, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const=0;
    virtual void outWebGL(const M_Art& partList, String& str, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const=0;
    virtual void outPovValues(String& str)const=0;
    virtual void smlsProcEffects(M_Art& partList)=0;
    virtual void clear()=0;

#ifdef SEAMLESS3DGUI
    virtual void breakColorJoin(Seamless& smls)=0;
#endif
};
class PartJoint{
    public:
//	virtual SFRotation& refSFRotation()=0;
//	virtual SFVec3f& refSFTranslation()=0;
};
class Art : public Node, public MField22, public PartJoint, public VisualAid{
public:

class Scale : public SFVec3f{
public:
    Art& home;
    Scale(const Vec3f& par, Art& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Scale& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
    void operator=(const Vec3f& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
};

class ArtRotation : public SFRotation{
public:
    Art& home;
    ArtRotation(const Rotation& par, Art& home):
        SFRotation(par),
        home(home)
    {};
    void operator=(const ArtRotation& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
    void operator=(const Rotation& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
};

class Translation : public SFVec3f{
public:
    Art& home;
    Translation(const Vec3f& par, Art& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Translation& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
    void operator=(const Vec3f& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
};

class IKRot : public SFRotation{
public:
    Art& home;
    IKRot(const Rotation& par, Art& home):
        SFRotation(par),
        home(home)
    {};
    void operator=(const Rotation& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
};

class IKPos : public SFVec3f{
public:
    Art& home;
    IKPos(const Vec3f& par, Art& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par){
        refValue() = par;
        home.smls->animCPBuildFlag = true;
        home.smls->ikProcessFlag = true;
    }
};

class PartChild : public MFNode{
public:
    Art& home;
    PartChild(const M_Node& value, Art& home):
        MFNode(value),
        home(home)
    {}
    M_SArt& refValue(){
        return *(M_SArt*)&length;
    }
    M_SArt& refValue()const{
        return *(M_SArt*)&length;
    }
    operator const M_SArt&()const{
        return *(M_SArt*)&getValue();
    }
    void operator=(const M_SArt& par);
    void operator=(const M_Node& par);
    void operator+=(const NodePtr& par);
    void operator+=(Art* par);
};

class MFMorpher : public MFNode{
public:
    Art& home;
    MFMorpher(const M_Node& value, Art& home):
        MFNode(value),
        home(home)
    {}
    M_Morpher& refValue(){
        return *(M_Morpher*)&length;
    }
    const M_Morpher& refValue()const{
        return *(M_Morpher*)&length;
    }
    operator const M_Morpher&()const{
        return *(M_Morpher*)&getValue();
    }
    void operator=(const M_Node& par);
    void operator+=(const NodePtr& par);
    inline const Morpher* operator [](const int& index)const;
};
class EffectField : public SFNode {
public:
    Art* home;
    EffectField( const NodePtr& par):
        SFNode(par)
    {};
    Node* operator = (Node* par1);

    operator Effect*()const {
        return (Effect*)p;
    }
};
class Coord : public MFVec3f {
public:
    Art& home;
    Coord( const M_Vec3f& par, Art& home):
        MFVec3f(par),
        home(home)
    {};
    void operator=(const M_Vec3f& par){
        refValue()=par;
        home.smls->convertToRenderFormat();
    }
};
class Normal : public MFVec3f {
public:
    Art& home;
    Normal( const M_Vec3f& par, Art& home):
        MFVec3f(par),
        home(home)
    {};
    void operator=(const M_Vec3f& par){
        refValue()=par;
        home.smls->convertToRenderFormat();
    }
};

class CoordGroup : public MFInt32{
public:
    Art& home;
    CoordGroup( const M_Int& par, Art& home):
        MFInt32(par),
        home(home)
    {};
    void operator=(const M_Int& par);
};
class Weight : public MFFloat {
public:
    Art& home;
    Weight( const M_Float& par, Art& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};

class Mirror : public SFBool{
public:
    Art& home;
    Mirror(const bool& par, Art& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class NikPush : public SFBool{
public:
    Art& home;
    NikPush(const bool& par, Art& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class PivotPoint : public SFVec3f{
public:
    Art& home;
    PivotPoint(const Vec3f& par, Art& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par);
    void operator=(const PivotPoint& par){
        *this = par.refValue();
    }
};

    virtual int getType()const{return ART;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}

#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Scale scale;
    ArtRotation rotation;
    Translation translation;
    SFBool track;
    Mirror mirror;
    SFBool ik;
    SFBool pin;
	SFBool rh;
	NikPush nikPush;
    IKRot ikRot;
    IKPos ikPos;
    SFFloat fka;
    SFFloat fkb;
    Coord coord;
    Normal normal;
    CoordGroup coordGroup;
    Weight weight;

    PivotPoint pivotPoint;
    MFVec2f uvCoord;

    EffectField effect;
    MFMorpher morpher;
    PartChild part;

    Art():
        MField22(
            &scale,
            &rotation,
            &translation,
            &track,
            &mirror,
            &ik,
            &pin,
			&rh,
			&nikPush,
            &ikRot,
            &ikPos,
            &fka,
            &fkb,

            &coord,
            &normal,
            &coordGroup,
            &weight,
            &pivotPoint,

            &uvCoord,

            &effect,
            &morpher,
            &part
        ),

        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        track(true),
        mirror(true, *this),
        ik(false),
        pin(false),
		rh(false),
		nikPush(false, *this),
        ikRot(Rotation(), *this),
        ikPos(Vec3f(), *this),
        fka(0),
        fkb(0),
        normal(M_Vec3f(), *this),
        coordGroup(M_Int(), *this),
        weight(M_Float(), *this),

        pivotPoint(Vec3f(), *this),
        uvCoord(M_Vec2f()),
        coord(M_Vec3f(), *this),

        effect(0),
        morpher(M_Node(), *this),
        part(M_Node(), *this),

//////////////////
        show(true),
        mirrorPart(0),
        coordLen(0),
        workEffect(0),
        fromCoord(0),
        toCoord(0),
        fromEffectCoord(0),
        toEffectCoord(0),
        fromPoly(0),
        toPoly(0),
        smls(0),
		niki(0)
        {
#ifdef SEAMLESS3DGUI
        index.length = 3;
#endif
        effect.home = this;

//		joinIndexSum=0;
        geometry = new IndexedFaceSet;
//		catPart=new IndexedFaceSet;
//		mirrorX=false;
//		parentSnapToThis=false;
//		replaceAtJoin=false;

//		cacheEntry=0;
//		highestEntry=0;
        parent = 0;

    }
    friend Seamless;
    Vec3f absolutePivotPoint;
    void calcAbsolutePivotPoint(Vec3f par);
    M_Color3f color;
    M_Vec2f texCoord;
    M_Int rc; //M_RedirectCoord
    M_<NurbsPatch*> patchOwner;

    Art* mirrorPart;
    M_ControlPoint controlPoint;

    Effect* workEffect;
	NikInterpolator* niki;

    M_Art relevantPart; //list of parts that use vertices from this part. Note all parts in list will be indexed less or equal
    M_Triangle triangle;
    static bool tugRadialTugSize;
    bool show;
    int coordLen;
//	M_Rotation temp;

    virtual void doClearCycle() = 0;
    virtual void doClearCycleAnimCP() = 0;

    bool sharedVertices();
    void cutSphere(const Vec3f location, const float radius);
    void cutLeft(float pos);
    void cutRight(float pos);
    void cutDown(float pos);
    void cutUp(float pos);
    void cutForward(float pos);
    void cutBackward(float pos);

    void cutLeftFull(float pos);
    void cutRightFull(float pos);
    void cutDownFull(float pos);
    void cutUpFull(float pos);
    void cutForwardFull(float pos);
    void cutBackwardFull(float pos);

    void mirrorTriangle(int triIdx, int corner);
    void splitTriangle(int triIdx, int corner);
    void uvSplitTriangle(int triIdx, int corner);
    void split2Triangles(int triIdx, int corner, Art* otherPart, int otherTriIdx, int otherCorner);
    void uvSplit2Triangles(int triIdx, int corner, Art* otherPart, int otherTriIdx, int otherCorner);
    void tripleSplitTriangle(int triIdx);
    void deleteTriangle(int triIndex);
    void deleteVertex(int vertexIndex);
    void fastDeleteVertex(int vertex);	//does not leave triangles with integrity
    void fastDeleteTriangle(int poly);	//does not leave triangles with integrity
    IndexedFaceSetPtr extractGeometry();
    void saveToWrl(String filename);
    bool createPartFromString(const String str);
    int swapVertex(int oldIndex, int newIndex);
    int uvSwapVertex(int oldIndex, int newIndex);

    void mirrorFuncX();
    void mirrorFuncY();
    void mirrorFuncZ();

    void mirrorFuncX(int triangleOffest, int coordOffest);
    void mirrorFuncY(int triangleOffest, int coordOffest);
    void mirrorFuncZ(int triangleOffest, int coordOffest);
    void insideOut();
    void insideOut(int offset);
    void staticTransform(int coordOffest, Matrix& matrix);

    void localPermenantTransform(Matrix catMatrix, Rotation rot);
    void translateToZero();
    void joinAllToGether(Art* upTo);
//	void findJoiningVertices(IndexedFaceSetPtr catPart, float dist);
//	void genUndoFilename();
//	void genCacheUrl(String& str ,int cacheEntry);
//	void backupToCache();
    void convertAllPartsToDX();
//	void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void convertGeomSourceToPart(const Mesh& source){workEffect->convertGeomSourceToPart(this,source);}
    void processSegmentedProduct(M_Node& children);
    void catMatrix(Matrix catMatrix);
    void catIMatrix(Matrix catMatrix);
    void catMatrix(Matrix catMatrix, Rotation4d catRot, M_Rotation4d& m_accumRot);
    void catIMatrix(Matrix catMatrix, Rotation4d catRot, M_Rotation4d& m_accumRot);

    Rotation getIRotation();
//	Matrix windCursorMatrix(Art* pj);
//	Matrix windMatrix(Art* pj, bool viewMat=true);
//	Matrix unWindMatrix(Art* pj, bool viewMat=true);
//	Matrix unWindMatrixNoTran(Art* pj, bool viewMat=true);
//	Rotation unWindRot(Art* pj, bool viewMat=true);
//	void resizePart(int newCoordLen, int newPolyLen);
    void colorize(Color3f color);
    void deleteGeometry();
    void deleteGeometryInBranch();
    void joinCloseVerticesFrom(Art* part, float joinRange, bool srcColor=false, bool dstColor=true);//this=destination
    void transferCloseVertices(Art* part, float joinRange);
    void getFields(const String& str, int& i);
    void genNormals();
    void genNormals(int h, int beginTriOffset);
    void findSetVertexRange(TextureMapper& textureMapper);
    void setDefaultVrmlTextureMap();
    void genDynamicTransforms(Matrix parentMatrix, int& smi);
    void genDynamicTransforms(M_Matrix& m_matrix, M_Int& m_smi, Matrix parentMatrix, int& smi);
    void genDynamicTransformsCP(Matrix parentMatrix, Matrix parentMatrixCP, int& smi);
    void catInverseRotation(Rotation& r);

    void extractGeometryGS(Mesh& geomSource, M_Int& maskIndex);
    void extractGeometryGS(Mesh& geomSource, M_Vec2f& gsUvCoord, M_<NurbsPatch*>& gsPatchOwner, M_Int& maskIndex);
    void addIndexUp(int amount);
    void outPovRayCoord(String& str, Matrix rotMatrix)const;
    void outPovRayNormal(String& str, Matrix matrix)const;
    void drag(Vec3f pt);
    bool checkPointInTriangle(const bool perspective, const int& testTriIndex);
    bool checkPointInTriangleRayTrace(RayView& rayView);
    bool checkPointInTriangle(const bool perspective, const int& testTriIndex, TouchSensor* sensor);
    bool checkPointInTriangleForProjectVertex(Vec3f& clickPos, const bool perspective, const int& triIdx);
    bool checkPointInTriangleForProjectOntoTri(const Vec3f& projectVertex, const Matrix& matrix, const int& triIdx)const;
    //bool checkPointInTriangleForProjectVertex(const bool perspective, const int& triIdx, TouchSensor* sensor);
    bool checkPointInVertex(const bool perspective, int i, Matrix& partMatrix);
    inline void initTugCP(const bool& perspective);
    Art* getTwinMirrorPart()const;
    void scaleSkeleton(Vec3f scale);
    void setUpInternalEffectPtrs2(Effect* lastEffect); //this (2) wont be needed when setUpInternalEffectPtrs is fixed
    void setUpInternalEffectPtrsTmpForRenderingTransparancy(Effect* lastEffect);

    inline bool isRightMirror()const{
        if(mirror && name.length > 1){
            if(name[1] < 'a' ){ //if uppercase or underbar
                if(name[0] == 'r'){
                    return true;
                }
            }
        }
        return false;
    }

    int numOfGroups()const;

#ifdef SEAMLESS3DGUI
    M_Int index;
    void cut();
    void copy();
#endif
    void genUndoDir(String fullPath);
    void Drag(Vec3f pt);
    bool polyBelongs(int poly){
        M_Int& coordIndex=geometry->coordIndex.refValue();
        int i=poly*4;
        if(coordIndex[i]>=fromCoord && coordIndex[i+1]>=fromCoord && coordIndex[i+2]>=fromCoord){
            if(coordIndex[i]<toCoord||coordIndex[i+1]<toCoord||coordIndex[i+2]<toCoord){
                return true;
            }
        }
        return false;
    }
    inline bool polyBelongs(S_Triangle& tri){
        if( tri.pa->plidx>=plidx && tri.pb->plidx>=plidx && tri.pc->plidx>=plidx ){
            if( tri.pa==this || tri.pb==this || tri.pc==this ){
                return true;
            }
        }
        return false;

/*
        if( tri.pa->plidx==plidx && tri.pb->plidx<=plidx && tri.pc->plidx<=plidx ){
            return true;
        }
        if( tri.pa->plidx<=plidx && tri.pb->plidx==plidx && tri.pc->plidx<=plidx ){
            return true;
        }
        if( tri.pa->plidx<=plidx && tri.pb->plidx<=plidx && tri.pc->plidx==plidx ){
            return true;
        }
        */
    }
    bool findNextPolyThatBelongs(int &poly);

//protected:
//	bool mirrorX;
//	bool replaceAtJoin;
//	bool parentSnapToThis;
    int fromEffectCoord;
    int toEffectCoord;
    int fromCoord;
    int toCoord;
    int fromPoly;
    int toPoly;
    int plidx;
    int renderFromCoord;

    IndexedFaceSetPtr geometry;

//	String undoName;
//	int cacheEntry;
//	int highestEntry;

//	Vec3f colorScale;

//	IndexedFaceSetPtr catPart;
//	String inPartFile;//legacy
//	M_Int joinIndex;//legacy
//	M_Int parentJoinIndex;//legacy
//	int joinIndexSum;//legacy
    M_Int legalCoordGroup;
    M_Int vCoordGroup;
    M_Int scratchC;
    M_Int partTree;
    M_Vec3f translatePart;
    Matrix matrix;
    Matrix imatrix;
    Matrix pivotPointMatrix;
    Seamless* smls;
    void removePart(M_SArt& par1);
    void addPart(Art* par1);

    M_Int matrixIndex;

/*
    M_Vec3f& refPoint(){
        IndexedFaceSet* geom=dynamic_cast<IndexedFaceSet*>(geometry.getRawPtr());
        if(geom==0)return *(M_Vec3f*)geom;
        Coordinate* tmp=dynamic_cast<Coordinate*>(geom->coord.refValue().getRawPtr());
        if(tmp==0)return *(M_Vec3f*)tmp;
        return tmp->point.refValue();
    }
    M_Vec3f& refVector(){
        IndexedFaceSet* geom=dynamic_cast<IndexedFaceSet*>(geometry.getRawPtr());
        if(geom==0)return *(M_Vec3f*)geom;
        Normal* tmp=dynamic_cast<Normal*>(geom->normal.refValue().getRawPtr());
        if(tmp==0)return *(M_Vec3f*)tmp;
        return tmp->vector.refValue();
    }
    M_Color3f& refColor(){
        IndexedFaceSet* geom=dynamic_cast<IndexedFaceSet*>(geometry.getRawPtr());
        if(geom==0)return *(M_Color3f*)geom;
        Color* tmp=dynamic_cast<Color*>(geom->color.refValue().getRawPtr());
        if(tmp==0)return *(M_Color3f*)tmp;
        return tmp->color.refValue();
    }
    M_Vec2f& refTexPoint(){
        IndexedFaceSet* geom=dynamic_cast<IndexedFaceSet*>(geometry.getRawPtr());
        if(geom==0)return *(M_Vec2f*)geom;
        TextureCoordinate* tmp=dynamic_cast<TextureCoordinate*>(geom->texCoord.refValue().getRawPtr());
        if(tmp==0)return *(M_Vec2f*)tmp;
        return tmp->point.refValue();
    }
    M_Int& refCoordIndex(){
        IndexedFaceSet* geom=dynamic_cast<IndexedFaceSet*>(geometry.getRawPtr());
        if(geom==0)return *(M_Int*)geom;
        return geom->coordIndex.refValue();
    }
    SFRotation& refSFRotation(){
        return rotation;
    }
    SFVec3f& refSFTranslation(){
        return translation;
    }
*/

    void houseKeep();
    void centerViewToControlPoint();
    int _matrixStack;// legacy (used for importing avs still)
    Art* parent;// legacy (only used for importing avs)
    void transformPoints(Matrix matrix);
    void transformPoints(Matrix matrix, NurbsPatch* targetPatch)const;


    void compileScaleLoop(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_Rotation& m_rot);
    void compileScaleLoopAnimCP(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_Rotation& m_rot);
    void compileScaleRotBackLoop(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_Rotation& m_rot);
    void compileScaleRotBackLoopAnimCP(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_Rotation& m_rot);
    void compileScaleRotationLoop(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_<SceneTreeListItem>& sceneTreeList);
    void compileScaleRotationLoopAnimCP(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_<SceneTreeListItem>& sceneTreeList);
    void compileTransformLoop(Matrix matrix, Rotation accumRot);
    void compileTransformLoopAnimCP(Matrix matrix, Rotation accumRot);
    void addPartToTreeForExportBVH(String& str, Function* function, M_Bool& pos, M_Art& m_interpolatorPart);

    NurbsLathe* latheFromTwoPoints(const Vec3f& from, const Vec3f& to);
    SphereBuild* sphereFromTwoPoints(const Vec3f& from, const Vec3f& to);

    void lathesFromPivotPoints(M_Build& m_build, const Vec3f accumOffset);
    void doBuild(NurbsPatch* patch);
    void doBuild(M_<NurbsPatchPtr>& m_patch);
    int getControlPointIndexFromPointListItem(const NurbsPatch* patch, const int idx)const;
    bool isLeftRightName();
    void randomizeHarmonicFreq();
    void mirrorMorphedCtrlPoints(bool left);
    void setUpInternalEffectPtrs(Effect* lastEffect);
    void buildSceneTranslationMatrices(Matrix parentMatrix);
    void buildSceneScaleTranslationMatrices(Matrix parentMatrix);
    void buildSceneITranslationMatrices(Matrix parentMatrix);
    void buildPartListNoShowLoop(int& _plidx);
    void buildPartListLoop(int& _plidx);
    void checkUsingPivotPoints(bool& state);
    void convertPivotPointsToTranslationsNoChangePivotPoint(Vec3f accumTran);
    void workoutVerticesForGroupCoords();

protected:
};
class SFPart : public SFNode {
public:

    SFPart(const NodePtr& value):
        SFNode(value)
    {};
    ArtPtr& operator = ( Art* par){
        par->incRefCount();
        if(p!=0){
            if(p->decRefCount()==0){
                delete p;
            }
        }
        p=par;
        return *((ArtPtr*)&SFNode::refValue());
    }
/*
    Art* operator =(ArtPtr par){
        par.pointer->incRefCount();
        if(pointer!=0){
            if(pointer->decRefCount()==0){
                delete pointer;
            }
        }
        pointer=par.pointer;
        return (Art*)pointer;
    }
*/
    const Art* operator ->()const{
        return (const Art*)p;
    }

    operator Art*()const {
        return (Art*)p;
    }
    operator ArtPtr()const {
        return refValue();
    }
    operator const Art*()const {
        return (const Art*)p;
    }
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    const ArtPtr& refValue()const{
        return *((const ArtPtr*)&SFNode::refValue());
    }
};
class ColorEffect : public Effect, public MField22{
    public:
class Diffuse : public SFColor{
public:
    ColorEffect& home;

    Diffuse(const Color3f& par, ColorEffect& home):
        SFColor(par),
        home(home)
    {};
    void operator=(const Color3f& par);
};
    int getType()const{return COLOREFFECT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    SFColor ambient;
    Diffuse diffuse;
    SFFloat brilliance;
    SFColor emissive;
    SFFloat shininess;
    SFColor specular;
    SFBool backfaceRender;
    SFFloat phong;
    SFFloat phong_size;
    SFFloat roughness;
    SFFloat crand;
    SFBool conserve_energy;
    SFColor reflection;
    SFBool fresnel;
    SFFloat falloff;
    SFFloat exponent;
    SFFloat metallic;
    SFFloat iridescence;
    SFFloat thickness;
    SFFloat turbulence;
    SFFloat transparency;
    MFColor color;

    ColorEffect():
        MField22(
            &ambient,
            &diffuse,
            &brilliance,
            &emissive,
            &shininess,
            &specular,
            &backfaceRender,
            &phong,
            &phong_size,
            &roughness,
            &crand,
            &conserve_energy,
            &reflection,
            &fresnel,
            &falloff,
            &exponent,
            &metallic,
            &iridescence,
            &thickness,
            &turbulence,
            &transparency,
            &color
        ),

        ambient(Color3f(.2, .2, .2)),
        diffuse(Color3f(.8,.8,.8), *this),
        brilliance(1),
        emissive(Color3f(0,0,0)),
        shininess(.2),
        specular(Color3f(0,0,0)),
        backfaceRender(false),
        phong(0),
        phong_size(40),
        roughness(.05),
        crand(0),
        conserve_energy(false),
        reflection(Color3f(0,0,0)),
        fresnel(false),
        falloff(0),
        exponent(1),
        metallic(1),
        iridescence(0),
        thickness(0),
        turbulence(0),
        transparency(0),
        color(M_Color3f())

    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    int getEffectLength(){
        return color.refValue().length;
    }
    inline void convertPartToIFS( Art* thisPart, IndexedFaceSet* ifs, M_Vec3f& ifsCoord, M_Int& ifsCoordIndex, const M_Vec3f& coord, const M_Triangle& triangle);
    void convertGeomSourceToPart(Art* part, const Mesh& source);
    void convertGeomSourceToPart(Art* part, const Mesh& source, const M_Vec2f& gsUvCoord, const M_<NurbsPatch*>& gsPatchOwner);
    void stemGenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stem2GenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stemGenerateTri(Stem& stem, const ALathe& lathe, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stemDupMirrorTri(Art* part, M_Vec3f& coord, M_Triangle& triangle, const int h, const int c, const int s, const int coordLen, const int colorLen, const int triLen);
    int polymapCopyLocalTri(Polymap* polymap, Art* part, M_Vec3f& dstCoord, int h, int s);
    int stemCopyLocalTri(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s);
    int stemCopyLocalTriNonMirror(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s);
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec2f& gsUvCoord, M_<NurbsPatch*>& gsPatchOwner, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec2f& uvCoord, const M_<NurbsPatch*>& patchOwner, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec3f& gsNormal, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec3f& normal, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    inline void optimize(M_Art&);
    void addColorNode(VertexGeometry* outputGeometry, VertexGeometry* sourceGeometry)const{outputGeometry->color=sourceGeometry->color;}
    void copyFieldsData(Shape* shape);
    void updateNewEffect(M_Art& partList)const{
        for(int k=0; k<partList.length; k++){
            Art* part=partList[k];
            M_Color3f& color=dynamic_cast<ColorEffect*>(part->workEffect)->color.refValue();
            color.length+=part->coord.length;
            M_Triangle& triangle=part->triangle;
            for(int i=triangle.length*3; --i>=0;){
                triangle.refTexIndex(i)=triangle.refIndex(i)+triangle.refPart(i)->fromCoord;
            }
        }
    }
    void copyColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle);
    void copyAddColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle,int& markTriangle);
    void copyColorData(Effect* effect);
    void setColorIndex(int i){index[0]=i;};
    void renderMaterial(int triFrom, Matrix& parentMatrix, Seamless& smls, Matrix& originalMatrix, Vec3f& originalColScale);
    void setUpVertexShaders(Matrix& parentMatrix, Seamless& smls);
    void outPovRay(const M_Art& partList, String& str, int vertexCount, M_Int& coordIndex, const int triLenTotal)const;
    void outObj(const M_Art& partList, String& str, String& mtl, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const;
    void outWebGL(const M_Art& partList, String& str, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const;
    void outPovValues(String& str)const;
    void smlsProcEffects(M_Art& partList);
    void clear();


#ifdef SEAMLESS3DGUI
    void breakColorJoin(Seamless& smls);
#endif
};
class TextureEffect : public Effect, public Download, public MField23{
    public:
class Url : public SFString {
public:
    TextureEffect& home;
    bool update;

    Url(const String& value, TextureEffect& home):
        SFString(value),
        home(home),
        update(true)
    {};
    void operator=(const String& par){
        refValue() = par;
        update = true;
    }
};
    int getType()const{return TEXTUREEFFECT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    SFColor ambient;
    SFColor diffuse;
    SFFloat brilliance;
    SFColor emissive;
    SFFloat shininess;
    SFColor specular;
    SFBool backfaceRender;
    SFFloat phong;
    SFFloat phong_size;
    SFFloat roughness;
    SFFloat crand;
    SFBool	conserve_energy;
    SFColor reflection;
    SFBool fresnel;
    SFFloat falloff;
    SFFloat exponent;
    SFFloat metallic;
    SFFloat iridescence;
    SFFloat thickness;
    SFFloat turbulence;
    SFFloat transparency;
    Url url;
    MFVec2f texCoord;

    TextureEffect():
        MField23(
            &ambient,
            &diffuse,
            &brilliance,
            &emissive,
            &shininess,
            &specular,
            &backfaceRender,
            &phong,
            &phong_size,
            &roughness,
            &crand,
            &conserve_energy,
            &reflection,
            &fresnel,
            &falloff,
            &exponent,
            &metallic,
            &iridescence,
            &thickness,
            &turbulence,
            &transparency,
            &url,
            &texCoord
        ),

        ambient(Color3f(.2, .2, .2)),
        diffuse(Color3f(.8,.8,.8)),
        brilliance(1),
        emissive(Color3f(0,0,0)),
        shininess(.2),
        specular(Color3f(0,0,0)),
        backfaceRender(false),
        phong(0),
        phong_size(40),
        roughness(.05),
        crand(0),
        conserve_energy(false),
        reflection(Color3f(0,0,0)),
        fresnel(false),
        falloff(0),
        exponent(1),
        metallic(1),
        iridescence(0),
        thickness(0),
        turbulence(0),
        transparency(0),
        url(String(), *this),
        texCoord(M_Vec2f()),
    ///////////////////////////////
#ifdef DIRECT3D
        dxTexture(0),
#endif
        hInternetOpenUrl(0),
        alphaTexture(false)
    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }

#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
#ifdef DIRECT3D
    IDirect3DTexture*  dxTexture;
#endif
    int getEffectLength(){
        return texCoord.refValue().length;
    }
    inline void convertPartToIFS( Art* thisPart, IndexedFaceSet* ifs, M_Vec3f& ifsCoord, M_Int& ifsCoordIndex, const M_Vec3f& coord, const M_Triangle& triangle);
    void convertGeomSourceToPart(Art* part, const Mesh& source);
    void convertGeomSourceToPart(Art* part, const Mesh& source, const M_Vec2f& gsUvCoord, const M_<NurbsPatch*>& gsPatchOwner);
    void stemGenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stem2GenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stemGenerateTri(Stem& stem, const ALathe& lathe, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stemDupMirrorTri(Art* part, M_Vec3f& coord, M_Triangle& triangle, const int h, const int c, const int s, const int coordLen, const int colorLen, const int triLen);
    int stemCopyLocalTri(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s);
    int stemCopyLocalTriNonMirror(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s);
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec2f& gsUvCoord, M_<NurbsPatch*>& gsPatchOwner, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec2f& uvCoord, const M_<NurbsPatch*>& patchOwner, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec3f& gsNormal, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec3f& normal, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    inline void optimize(M_Art&);
    void addColorNode(VertexGeometry* outputGeometry, VertexGeometry* sourceGeometry)const{outputGeometry->texCoord=sourceGeometry->texCoord;}
    void copyFieldsData(Shape* shape);
    void addTextureMapper(Stem& stem);

    void getColorWithEyeDropRadius();
    void writePixelsToTexture(const Color3f color);
    unsigned int readPixelFromTexture();
    void copyPixelsToStore();
    void colourWithFill(unsigned paletteColor);

    virtual void textureMapDisk(Art* part, float radius, float angle);
    void updateNewEffect(M_Art& partList)const{
        for(int k=0; k<partList.length; k++){
            Art* part=partList[k];
            M_Vec2f& texCoord=dynamic_cast<TextureEffect*>(part->workEffect)->texCoord.refValue();
            texCoord.length+=part->coord.length;
            M_Triangle& triangle=part->triangle;
            for(int i=triangle.length*3; --i>=0;){
                triangle.refTexIndex(i)=triangle.refIndex(i)+triangle.refPart(i)->fromCoord;
            }
        }
    }
    void copyColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle);
    void copyAddColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle,int& markTriangle);
    void copyColorData(Effect* effect);
    void setColorIndex(int i){index[1]=i;};
    void renderMaterial(int triFrom, Matrix& parentMatrix, Seamless& smls, Matrix& originalMatrix, Vec3f& originalColScale);
    void setUpVertexShaders(Matrix& parentMatrix, Seamless& smls);
    void outPovRay(const M_Art& partList, String& str, int vertexCount, M_Int& coordIndex, const int triLenTotal)const;
    void outObj(const M_Art& partList, String& str, String& mtl, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const;
    void outWebGL(const M_Art& partList, String& str, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const;
    void outPovValues(String& str)const;
    void smlsProcEffects(M_Art& partList);
    void clear();


#ifdef SEAMLESS3DGUI
    void breakColorJoin(Seamless& smls);
#endif

    bool download();
    bool openUrl();
    int hInternetOpenUrl;
    M_<char> internetBuf;
    bool update;
    void getUrlPath(String&){};
    NodePtr urlPathNode;
    bool alphaTexture;
};
class MonoColorEffect : public Effect, public MField22{
    public:
    int getType()const{return MONOCOLOREFFECT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFColor ambient;
    SFColor diffuse;
    SFFloat brilliance;
    SFColor emissive;
    SFFloat shininess;
    SFColor specular;
    SFBool backfaceRender;
    SFFloat phong;
    SFFloat phong_size;
    SFFloat roughness;
    SFFloat crand;
    SFBool conserve_energy;
    SFColor reflection;
    SFBool fresnel;
    SFFloat falloff;
    SFFloat exponent;
    SFFloat metallic;
    SFFloat iridescence;
    SFFloat thickness;
    SFFloat turbulence;
    SFFloat transparency;
    MFColor color;

    MonoColorEffect():
        MField22(
            &ambient,
            &diffuse,
            &brilliance,
            &emissive,
            &shininess,
            &specular,
            &backfaceRender,
            &phong,
            &phong_size,
            &roughness,
            &crand,
            &conserve_energy,
            &reflection,
            &fresnel,
            &falloff,
            &exponent,
            &metallic,
            &iridescence,
            &thickness,
            &turbulence,
            &transparency,
            &color
        ),

        ambient(Color3f(.2, .2, .2)),
        diffuse(Color3f(.8,.8,.8)),
        brilliance(1),
        emissive(Color3f(0,0,0)),
        shininess(.2),
        specular(Color3f(0,0,0)),
        backfaceRender(false),
        phong(0),
        phong_size(40),
        roughness(.05),
        crand(0),
        conserve_energy(false),
        reflection(Color3f(0,0,0)),
        fresnel(false),
        falloff(0),
        exponent(1),
        metallic(1),
        iridescence(0),
        thickness(0),
        turbulence(0),
        transparency(0),
        color(M_Color3f())
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    int getEffectLength(){
        return color.refValue().length;
    }
    inline void convertPartToIFS( Art* thisPart, IndexedFaceSet* ifs, M_Vec3f& ifsCoord, M_Int& ifsCoordIndex, const M_Vec3f& coord, const M_Triangle& triangle);
    void convertGeomSourceToPart(Art* part, const Mesh& source);
    void convertGeomSourceToPart(Art* part, const Mesh& source, const M_Vec2f& gsUvCoord, const M_<NurbsPatch*>& gsPatchOwner);
    void stemGenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stem2GenerateTri(NurbsPatch* patch, const Stem& stem, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stemGenerateTri(Stem& stem, const ALathe& lathe, const int& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper, const int h, const int s);
    void stemDupMirrorTri(Art* part, M_Vec3f& coord, M_Triangle& triangle, const int h, const int c, const int s, const int coordLen, const int colorLen, const int triLen);
    int stemCopyLocalTri(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s);
    int stemCopyLocalTriNonMirror(Stem* stem, Art* part, M_Vec3f& dstCoord, int h, int s);
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec2f& gsUvCoord, M_<NurbsPatch*>& gsPatchOwner, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec2f& uvCoord, const M_<NurbsPatch*>& patchOwner, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    void sortOutIndex(Mesh& gs, M_Vec3f& gsCoord, M_Vec3f& gsNormal, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Vec3f& normal, const M_Triangle& triangle, Matrix& imatrix, M_Int& maskIndex) const;
    inline void optimize(M_Art&);
    void addColorNode(VertexGeometry* outputGeometry, VertexGeometry* sourceGeometry)const{outputGeometry->color=sourceGeometry->color;}
    void copyFieldsData(Shape* shape);
    void updateNewEffect(M_Art& partList)const{
        for(int k=0; k<partList.length; k++){
            Art* part=partList[k];
            M_Color3f& color=dynamic_cast<MonoColorEffect*>(part->workEffect)->color.refValue();
            color.length+=part->coord.length;
            M_Triangle& triangle=part->triangle;
            for(int i=triangle.length*3; --i>=0;){
                triangle.refTexIndex(i)=triangle.refIndex(i)+triangle.refPart(i)->fromCoord;
            }
        }
    }
    void copyColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle);
    void copyAddColor(Art* src, Art* dst, const M_Triangle& srcTriangle, M_Triangle& dstTriangle,int& markTriangle);
    void copyColorData(Effect* effect);
    void setColorIndex(int i){index[0]=i;};
    void renderMaterial(int triFrom, Matrix& parentMatrix, Seamless& smls, Matrix& originalMatrix, Vec3f& originalColScale);
    void setUpVertexShaders(Matrix& parentMatrix, Seamless& smls);
    void outPovRay(const M_Art& partList, String& str, int vertexCount, M_Int& coordIndex, const int triLenTotal)const;
    void outObj(const M_Art& partList, String& str, String& mtl, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const;
    void outWebGL(const M_Art& partList, String& str, int& vidx, int& vtidx, int vertexCount, M_Int& coordIndex)const;
    void outPovValues(String& str)const;
    void smlsProcEffects(M_Art& partList);
    void clear();


#ifdef SEAMLESS3DGUI
    void breakColorJoin(Seamless& smls);
#endif
};
class Part : public Art {
    int getType()const{return PART;}
    void doClearCycle(){
        if(smls->bnspe == false){
            coord.refValue().length=0;
            normal.refValue().length=0;
            color.length=0;
            texCoord.length=0;
            triangle.length=0;
        }
        scale.refValue()=Vec3f(1,1,1);
        translation.refValue()=Vec3f();
        rotation.refValue()=Rotation();
    }
    void doClearCycleAnimCP(){
        coord.refValue().length=0;
        normal.refValue().length=0;
        color.length=0;
        texCoord.length=0;
        triangle.length=0;
    }
};
class ImmediatePart : public Art {
public:
    int getType()const{return IMMEDIATEPART;}
    void doClearCycle(){}
    void doClearCycleAnimCP(){}
};
class PartChild : public MFNode{
public:
    PartChild(const M_Node& value):
        MFNode(value)
    {}
    M_SArt& refValue(){
        return *(M_SArt*)&length;
    }
    const M_SArt& refValue()const{
        return *(M_SArt*)&length;
    }
    void operator=(const PartChild& par){
        refValue()=*(M_SArt*)&par.getValue();
    }
    void operator=(const M_SArt& par){
        refValue()=*(M_SArt*)&par;
    }
    void operator+=(Art* par){
        refValue()+=par;
    }
    operator const M_SArt&()const{
        return *(M_SArt*)&getValue();
    }
    const ArtPtr operator [](const int& index)const{
        return refValue()[index];
    }
};
class GroupBuild : public Build, public MField4, public VisualAid{
    public:
class Marker : public SFColor{
public:
    GroupBuild& home;

    Marker(const Color3f& par, GroupBuild& home):
        SFColor(par),
        home(home)
    {};
    void operator=(const Color3f& par);
	void houseKeep();
};

class HSFPart : public SFNode{
public:
    GroupBuild& home;
    HSFPart( const NodePtr& par, GroupBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    const ArtPtr& refValue()const{
        return *((const ArtPtr*)&SFNode::refValue());
    }

    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};

    int getType()const{return GROUPBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Marker marker;
    Setup setup;
    HSFPart part;
    MFBuild build;

    GroupBuild():
        Build(setup),
        MField4(
            &marker,
            &setup,
            &part,
            &build
        ),

        marker(Color3f(), *this),
        setup(0, *this),
        part(0, *this),
        build(M_Node())
    {
    }
    int getIndexForLastBuildNode(M_Node*& r_build);
    void doBuild();
    void cpNodeListAdd(Seamless* smls);

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
	void drag(Vec3f pt);
    void houseKeep();
};
class Stem : public Build, public MField32{
    public:
class Cells : public SFVec2f {
    public:
    Stem& home;
    Cells(const Vec2f& value, Stem& home):
        SFVec2f(value),
        home(home)
    {};
    void operator=(const Vec2f& par);
};

class Mirror : public SFBool{
public:
    Stem& home;
    Mirror(const bool& par, Stem& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class NorthPole : public SFBool {
public:
    Stem& home;

    NorthPole( const bool& value, Stem& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class SouthPole : public SFBool {
public:
    Stem& home;

    SouthPole( const bool& value, Stem& i_home):
        SFBool(value),
        home(i_home)
    {};
    void operator=(const bool& par);
};

class YDistance : public MFFloat {
public:
    Stem& home;
    YDistance( const M_Float& value, Stem& i_curve):
        MFFloat(value),
        home(i_curve)
    {};
    void operator=(const M_Float& par);
    void resize(int newLen);
};

    int getType()const{return STEM;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Cells cells;
    SFBool add;
    Mirror mirror;
    SFBool store;
    SFVec2f slide;
    SFRotation orientation;
    MFInt32 staves;
    MFFloat xDistance;
    TMFFloat height;
    YDistance yDistance;
    MFInt32 bands;
    //MFInt32	ownership;
    SFBool sync;
    SFBool mould;
    NorthPole northPole;
    SouthPole southPole;
    SFBool insideOut;
    SFBool xCentred;
    SFBool uWrap;
    SFBool vWrap;
    SFBool endSP;
    SFBool beginSP;

    MFVec2f coord;
    MFColor color;
    MFVec2f texCoord;
    MFInt32 coordIndex;
    MFInt32 rc;

    PartChild source;
    PartChild part;
    Setup setup;
    MFNode textureMapper;
    SFNode surface;
    MFNode fuse;

    Stem():
        Build(setup),
        MField32(
            &cells,
            &add,
            &mirror,
            &store,
            &slide,
            &orientation,
            &staves,
            &xDistance,
            &height,
            &yDistance,
            &bands,
            //&ownership,
            &sync,
            &mould,
            &northPole,
            &southPole,
            &insideOut,
            &xCentred,
            &uWrap,
            &vWrap,
            &endSP,
            &beginSP,
            &coord,
            &color,
            &texCoord,
            &coordIndex,
            &rc,
            &source,
            &part,
            &setup,
            &textureMapper,
            &surface,
            &fuse
        ),

        cells(Vec2f(), *this),
        add(true),
        mirror(false, *this),
        store(false),

        slide(Vec2f()),
        orientation(Rotation()),
        staves(16),
        xDistance(6.283185),
        height(M_Float(), *this),
        yDistance(M_Float(), *this),
        bands(M_Int()),
        //ownership(M_Int()),
        sync(false),
        mould(true),
        northPole(false, *this),
        southPole(false, *this),
        insideOut(false),
        xCentred(true),
        uWrap(false),
        vWrap(false),
        endSP(false),
        beginSP(false),
        coord(M_Vec2f()),
        color(M_Color3f()),
        texCoord(M_Vec2f()),
        coordIndex(M_Int()),
        rc(M_Int()),
        source(M_Node()),
        part(M_Node()),
        setup(0, *this),
        textureMapper(M_Node()),
        surface(0),
        fuse(M_Node()),

        ///
        mirrorStem(0),
        beginCoordOffset(0),
        beginTriOffset(0)
    {
        index.length = 5;
    }
    M_Int index;
    Stem* mirrorStem;
    static M_Float bandDist;
    int beginCoordOffset;
    int beginTriOffset;
    //M_LocalTriangle triangle;

    void doBuild();
    void doBuild(FuseSurface* parentFuse, NurbsPatch* patch);
    float getYDistance()const;
    void processFields();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;

    void insertBands(int& newLen);
    void uninsertBands(int& newLen);
    void convertYDistanceToHeight();
    void duplicateMirror(Art* part)const;//this function should be replaced by how mirror dup is done in void Stem::doBuild(FuseSurface* parentFuse, NurbsPatch* patch);
    //void postLoadProcess();
    void convertCoordIndexToTriangle(Art* part, int h, int c, int s);
    void copyPolyMapToPart(Art* part);
    void copyPartToPolyMap(Art* part);
    int rcCoord(int i){
        i = coordIndex[i];
        int g = i - coord.length;
        if(g >= 0){
            i = rc[g];
        }
        return i;
    }
    int rcCoord(int i, int& c){
        i = c = coordIndex[i];
        int g = i - coord.length;
        if(g >= 0){
            i = rc[g];
        }
        return i;
    }


private:
//	inline void buildMultiOwnedRings(Generator* surface);
};



class TextureMapper : public Node, public MField10 {
    public:
class Size2d : public SFVec2f {
public:
    TextureMapper& home;

    Size2d(const Vec2f& par, TextureMapper& home):
        SFVec2f(par),
        home(home)
    {};
    void operator=(const Vec2f& par);
};
class Position2d : public SFVec2f {
public:
    TextureMapper& home;

    Position2d(const Vec2f& par, TextureMapper& home):
        SFVec2f(par),
        home(home)
    {};
    void operator=(const Vec2f& par);
};
class Clear : public SFBool {
public:
    TextureMapper& home;

    Clear(const bool& par, TextureMapper& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
////////////////

    int getType()const{return TEXTUREMAPPER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFRotation orientation;
    SFBool autoSize3d;
    SFVec2f size3d;
    SFVec3f position3d;
    Size2d size2d;
    Position2d position2d;
    SFBool flat;
    SFBool spherical;
    SFBool cylindrical;
    Clear clear;

    TextureMapper():
        MField10(
            &orientation,
            &autoSize3d,
            &size3d,
            &position3d,
            &size2d,
            &position2d,
            &flat,
            &spherical,
            &cylindrical,
            &clear
        ),

        orientation(Rotation()),
        autoSize3d(false),
        size3d(Vec2f(1,1)),
        position3d(Vec3f()),
        size2d(Vec2f(1,1),*this),
        position2d(Vec2f(),*this),
        flat(true),
        spherical(false),
        cylindrical(false),
        clear(false, *this),
        ////////////
        pos(180, 181)
    {
    }
    const Vec2i pos;
    inline void drawTextureGrid(int& dstWidth, int& dstHeight, int* pixBuf);
    inline bool clickGrid(const Vec2i ptCursor);
    enum{
        tile=16,
    };
};

class ColorSweep : public Build, public MField11, public VisualAid{
    public:
class Orientation : public SFRotation {
public:
    ColorSweep& home;
    Orientation( const Rotation& value, ColorSweep& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par){
        home.orientation.refValue()=par;
        Vec3f location=home.location.refValue();
        home.matrix=rotationMatrix(par)*translationMatrix(location);
        home.imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    ColorSweep& home;
    Location( const Vec3f& value, ColorSweep& home):
        SFVec3f(value),
        home(home)
    {};

    void operator=(const Vec3f& par){
        home.location.refValue()=par;
        Rotation orientation=home.orientation.refValue();
        home.matrix=rotationMatrix(orientation)*translationMatrix(par);
        home.imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
class Spherical : public SFBool {
public:
    ColorSweep& home;

    Spherical(const bool& par, ColorSweep& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Straight : public SFBool {
public:
    ColorSweep& home;

    Straight(const bool& par, ColorSweep& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return COLORSWEEP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Orientation orientation;
    Location location;
    MFColor color;
    MFFloat distance;
    MFFloat opacity;
    Spherical spherical;
    SFBool mirror;
    SFBool cylindrical;
    Straight straight;
    PartChild part;
    Setup setup;

    ColorSweep():
        Build(setup),
        MField11(
            &orientation,
            &location,
            &color,
            &distance,
            &opacity,
            &spherical,
            &mirror,
            &cylindrical,
            &straight,
            &part,
            &setup
        ),

        orientation(Rotation(), *this),
        location(Vec3f(), *this),
        color(M_Color3f()),
        distance(M_Float()),
        opacity(M_Float()),
        spherical(false, *this),
        mirror(false),
        cylindrical(false),
        straight(false, *this),
        part(M_Node()),
        setup(0, *this)
    {
#ifdef SEAMLESS3DGUI
        index.length = 1;
#endif
    }
    Matrix matrix;
    Matrix imatrix;

	void doColorForStem(Art* prt, M_Vec3f& coord, const int h, const int texCoordBase);
    void doColorForStem(M_Vec3f& coord, M_Triangle& m_triangle, const int h, const int c, const int s);
    void doColorForStem(M_Triangle& m_triangle, M_Vec3f& coord, M_Color3f& eColor, const int h, const int c, const int s);
    void doBuild();
    void houseKeep();
    void drag(Vec3f pt);
    void processFields();
	
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    bool illegalImmediateRef(const Art*, M_Art&)const{
        return false;
    }
};

class ColorMapper : public Node, public MField1{
public:
    int getType()const{return COLORMAPPER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFColor color;

    ColorMapper():
        MField1(
            &color
        ),

        color(M_Color3f())
{

#ifdef SEAMLESS3DGUI
        index.length = 1;
#endif
}

#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};

class SurfaceGenerator : public Generator, public Node, public MField1 {
public:
    int getType()const{return SURFACEGENERATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

public:

    MFNode generator;

    SurfaceGenerator():
        MField1(
            &generator
        ),

        generator(M_Node())
    {}
    void generate(M_Vec3f& coord, const int h);

    void removeGenerator(M_Node& par1);

#ifdef SEAMLESS3DGUI
    void cut();
    void newGeneratorNode(Node* node);
#endif
};
class CCLathe : public ALathe, public MField4, public VisualAid{
    public:
class Curve : public MFFloat {
public:
    CCLathe& home;
    Curve( const M_Float& value, CCLathe& i_curve):
        MFFloat(value),
        home(i_curve)
    {};
    void operator=(const M_Float& par);
    void resize(int newLen);
};
///////////////////////////
    int getType()const{return CCLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Curve curve;
    MFFloat distance;
    SFFloat baseRad;
    SFBool autoClose;

    CCLathe():
        MField4(
            &curve,
            &distance,
            &baseRad,
            &autoClose
        ),

        curve(M_Float(),*this),
        distance(M_Float()),
        baseRad(0),
        autoClose(false),

///////////////
        startPosY(0)
    {

#ifdef SEAMLESS3DGUI
        index.length=2;
      angleDrag = false;
#endif
    }
    float startPosY;

#ifdef SEAMLESS3DGUI

   struct CCSegment {
      Vec2f C;          // centre
      Vec2f Ro;         // radius origin
      Vec2f Pf;         // end point
      float a;          // angle of arc

      CCSegment() : a(0) {}
      CCSegment( Vec2f C, Vec2f Ro, Vec2f Pf, float a ) : C(C), Ro(Ro), Pf(Pf), a(a) {}
   };

   typedef M_<CCSegment> M_CCSegment;
   M_CCSegment seg;

    M_Int index;
   int angleDrag;
   Vec2f anglePt;
//	StemPtr masterPtr;
    M_Int numStaves;
    M_Int numRings;
    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);
    void addControlPoint(Vec3f pt);
    void deleteControlPoint();
    void calcSegments();
    void setPoint(int index, Vec2f point);
    static const float controlLen;

#endif
    void generate(M_Vec3f& coord, const int h);
//	void genSource(IndexedFaceSetPtr& source);
//	void genRadialSource(IndexedFaceSetPtr& retPart);
//	void genTopRadialSource(IndexedFaceSetPtr& retPart);
    void genCurveFromRadSrc(M_Vec3f& coord);
    void calcClose();
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    float getCurve(int index=0x7fffffff)const;
    float getDistance(int index=0x7fffffff)const;
    float getPosX(int index)const;
    float getPosY(int index)const;
    void getBandAndYDist(M_Int& bands, M_Float& bandDist, M_Float& yDistance)const;
};

class NurbsPatch : public Build, public Generator, public MField24, public VisualAid{
public:
enum CurveType{
	MID_0,
	MID_1,
	MID_2,
	BEZIER,
};

enum Edge{
	SOUTH = 1,
	EAST = 2,
	NORTH = 3,
	WEST = 4,
};

class Dim : public SFVec2f {
	public:
	NurbsPatch& home;
	Dim(const Vec2f& value, NurbsPatch& home):
		SFVec2f(value),
		home(home)
	{};
	void operator=(const Vec2f& par);
};
class DPoint : public MFVec3f {
	public:
	NurbsPatch& home;
	DPoint(const M_Vec3f& value, NurbsPatch& home):
		MFVec3f(value),
		home(home)
	{};
	void operator=(const M_Vec3f& par);
};
class Weight : public MFFloat {
public:
	NurbsPatch& home;
	Weight( const M_Float& par, NurbsPatch& home):
		MFFloat(par),
		home(home)
	{};
	void operator=(const M_Float& par);
};

class Mirror : public SFBool{
public:
	NurbsPatch& home;
	Mirror(const bool& par, NurbsPatch& home):
		SFBool(par),
		home(home)
	{};
	void operator=(const bool& par);
};


	int getType()const{return NURBSPATCH;}
	int getFLen()const{return size();};
	Field* getField(const int i)const{return (*this)[i];}
	int getFieldGroupIndex(const int &i)const{return index[i];}
	void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
	M_Int* getIndexPtr(){return &index;}

	//int uGetCurveType()const; untested but probaly works
	//int vGetCurveType()const;

	TMFVec3f point;
	DPoint dpoint;
	Weight weight;
	MFInt32 cpIndex; //control point index
	//M_Bool sharpJoin; //sharp join. if length == 0 ignore, otherwise each point is a sharp join if true. Saved to a different format to reduce file sace
	//MFInt32 sharp //save format
	MFFloat uKnot;
	MFFloat vKnot;
	SFVec2f order;

	SFBool object;
	SFBool cage;
	TSFBool	showAll;
	Mirror mirror;
	Dim dim;


	TSFBool	endSP;
	TSFBool	uJoinEnds;
	TSFBool	beginSP;

	TSFBool	sync;
	TSFBool	insideOut;

	TSFVec3f scale;
	TSFRotation rotation;
	TSFVec3f translation;

	SFNode fuseSurface;
	MFNode stem;
	SFNode mapper;
	Setup setup;

	NurbsPatch():
		Build(setup),
		MField24(
			&point,
			&dpoint,
			&weight,
			&cpIndex,
			&uKnot,
			&vKnot,
			&order,

			&object,
			&cage,
			&showAll,
			&mirror,
			&dim,

			&endSP,
			&uJoinEnds,
			&beginSP,

			&sync,
			&insideOut,

			&scale,
			&rotation,
			&translation,
			&fuseSurface,
			&stem,
			&mapper,
			&setup
		),
//		point(M_Vec3f(FLOAT_LIST(initNurbsPointData)), *this),
//		dpoint(M_Vec3f(FLOAT_LIST(initNurbsDPointData)), *this),
//		weight(FLOAT_LIST(initNurbsWeightData), *this),
//		uKnot(M_Float(0, 0, 0, .25, .25, .5, .5, .75, .75, 1, 1, 1)),
//		vKnot(M_Float(0, 0, 0, 1, 1, 1)),

		point(M_Vec3f(), *this),
		dpoint(M_Vec3f(), *this),
		weight(M_Float(), *this),
		cpIndex(M_Int()),
		uKnot(M_Float()),
		vKnot(M_Float()),
		order(Vec2f(3,3)),

		object(false),
		cage(true),
		showAll(false, *this),
		mirror(false, *this),
		//dim(Vec2f(9,3), *this),
		dim(Vec2f(), *this),

		endSP(false, *this),
		uJoinEnds(true, *this),
		beginSP(false, *this),

		sync(false, *this),
		insideOut(false, *this),

		scale(Vec3f(1,1,1), *this),
		rotation(Rotation(), *this),
		translation(Vec3f(), *this),
		fuseSurface(0),
		stem(M_Node()),
		mapper(0),
		setup(0, *this),

/////////////
		uDegree(2),
		vDegree(2),
		fromPoint(0),
		smls(0),
		mirrorPatch(0),
		isCtrlPolyRef(0),
		plidx(0),
		container(this)
	{
		index.length = 4;
	}
	Seamless* smls;
	NurbsPatch* mirrorPatch;
	Build* container;
	M_Int index;
	void processFields();
	void houseKeep();
	void genCage(M_Color3f& color);
	void genStraightCage(M_Color3f& color, const M_Bool& m_pointIsVisible);


	void addCage(IndexedLineSet* polyLine);
	void addCage(IndexedLineSet* polyLine, const M_Bool& m_pointIsVisible);

	void addStraightCage(IndexedLineSet* polyLine);
	void addStraightCage(IndexedLineSet* polyLine, const M_Bool& m_pointIsVisible);
	void addStraightCageForShowBorder(IndexedLineSet* polyLine);
	void addStraightCageForShowBorder(IndexedLineSet* polyLine, const M_Bool& m_pointIsVisible);

	void addCageShowMid(IndexedLineSet* polyLine);
	void addNurbsEdgeCage(IndexedLineSet* polyLine);
	void calcScreenPoints(M_Vec2f& m_screenPoint, const Matrix& viewMatrix);
	void setBackfaceMaskForInnerPoints(M_Bool& m_pointIsVisible, const M_Vec2f& m_screenPoint);
	void setVisibleMaskForStitchEditMode(M_Bool& m_pointIsVisible);
	void centerViewToControlPoint();
	void drag(Vec3f pt);
	int uDegree;
	int vDegree;
	bool isCtrlPolyRef;

	M_Vec3f r_point;
	M_Float r_weight;
	M_Vec3f tpoint;
	int fromPoint;
	int plidx;

	float minU()const{
		return uKnot.refValue()[uDegree];
	}
	float minV()const{
		return vKnot.refValue()[vDegree];
	}
	float maxU()const{
		return uKnot.refValue()[uKnot.length-uDegree];
	}
	float maxV()const{
		return vKnot.refValue()[vKnot.length-vDegree];
	}
	void doBuild();
	NurbsPatch* getPatch(){return this;}
	Build* getMirrorBuild(){return mirrorPatch;}
	int numOfPatches(){return 1;}
	bool isCaged(){return cage;}
	void setCage(bool state){cage = state;}
	void setMirror(bool state){mirror = state;}
	void cpNodeListAdd(Seamless* smls);

	Matrix matrix;
	Matrix imatrix;


	inline bool checkPointCurveCage(const bool& perspective, const Vec3f& sr, const int& i);
	bool checkPointIgnoreBackface(const bool& perspective, const Vec3f& sr, const int& u, const int& v, const int& uDim, const int& vDim, const int& i, const Matrix& matrix, const M_Vec3f& dpoint);
	inline void initTug(const bool& perspective);
	inline bool initBoxModellingTugCP(const bool& perspective);
	inline bool initTugCP(const bool& perspective);
	inline bool initStraightCageTugCP(const bool& perspective);
	inline void applyRectOwnership(const Art* part, M_<CPContigElement>& contigLookUpList, const int& selGroupIdx, const int& selGroupIdxMirror);
	inline void applyRectOwnership(const Art* part, M_<CPContigElement>& contigLookUpList, int i, const int& selGroupIdx, const int& selGroupIdxMirror);
	inline void mirrorSingleCPOwnership(M_<CPContigElement>& contigLookUpList, int idx, const int& selGroupIdx);
	static int convertVisualToActual(int idx, const int uDim, const int vDim);
	static int convertActualToVisual(int idx, const int uDim, const int vDim);

	Vec3f calcRingNormal(int v, Vec3f pivotPoint)const;
	Vec3f calcRingPivotPoint(int v)const;
	int getSelRing()const;
	static bool tugRingPivotPoint;

	void dpointsToPoints();
	void pointsToDpoints();
	void pointsToCps();
	void pointsToCpws();
	void cpwsToPoints();
	void createCpFromPatchPoints();
	void generate(M_Vec3f& coord, const int h);
	void generate2(M_Vec3f& coord, const int h);
	void generate3(M_Vec3f& dst, const int dstIdx, Vec2f t);
	Vec3f generate(Vec2f t);
	Vec3f generateNormal(Vec2f t)const;
	void generateWithNormals(M_Vec3f& coord, M_Vec3f& normal, const int h);
	void saveToWrl(String filename);
	bool loadDataFromString(const String str);
	bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
	inline bool checkRingPivotPoint(const bool& perspective, const Matrix& matrix);
	inline bool doSameViewLocation(const Vec3f& pt, const Vec3f& svec, const int& c2, const int& r, const int& xDim, const int& yDim, M_Vec3f& point, const M_Vec3f& dpoint, const M_Float& weight, const Matrix& imatrix, const Matrix& mtrx);
	void singleRadialTug(M_Vec3f& point, const M_Float& weight, const Vec3f location, const Vec3f displacement, const float radius, const float slope);
	void singleRadialTugMirrorRight(M_Vec3f& point, const M_Float& weight, const Vec3f location, const Vec3f displacement, const float radius, const float slope);
	void singleRadialTugMirrorLeft(M_Vec3f& point, const M_Float& weight, const Vec3f location, const Vec3f displacement, const float radius, const float slope);
	void radialTug(Vec3f location, Vec3f pt);
	bool insertControlRowColum();
	bool deleteControlRowColum();
	bool deleteControlLine();
	bool clickedRowColum(const int idxA, const int idxB, Matrix matrix);

	NurbsPatch* uSplitPatch(int u);
	NurbsPatch* vSplitPatch(int v);

	NurbsPatchPtr uSplitPatch(int v, M_Node& m_node, int itemIdx, bool insert);

	void removeSouthEdge(NurbsPatch* patchA);
	void removeNorthEdge(NurbsPatch* patchA);
	void removeEastEdge(NurbsPatch* patchA);
	void removeWestEdge(NurbsPatch* patchA);

	void rotateIndexOrder90();
	void rotateIndexOrder180();
	void rotateIndexOrder270();
	void rotateIndexOrder360();

	bool isWestSide(int idx){
		int uDim = dim.x;
		int u = idx % uDim;
		return u < (uDim / 2);
	}


	static bool isSouthWestCorner(int idx){
		return idx == 0;
	}
	static bool isSouthEastCorner(int idx, int uDim){
		return idx == uDim - 1;
	}
	static bool isNorthWestCorner(int idx, int uDim, int vDim){
		return idx == uDim * (vDim - 1);
	}
	static bool isNorthEastCorner(int idx, int uDim, int vDim){
		return idx == uDim * vDim - 1;
	}

	static bool isSouthEdge(int idx, int uDim){
		return idx < uDim;
	}
	static bool isNorthEdge(int idx, int uDim, int vDim){
		return idx >= uDim * (vDim - 1);
	}
	static bool isWestEdge(int idx, int uDim){
		return idx % uDim == 0;
	}
	static bool isEastEdge(int idx, int uDim){
		return idx % uDim == uDim - 1;
	}
	bool isCellCorner(int idx);
	bool isPatchCorner(int idx);

	void mirrorTwinPatch(NurbsPatch* mirrorPatch);
	void mirrorTwinPatch2(NurbsPatch* mirrorPatch);
	void mirrorTwinlessPatch(int idx);
	bool splitPatch();
	void doDim(const Vec2f par, const float ratio, M_Vec3f& dpoint, M_Float& weight, Vec2f& order, Vec2f& dim, int& idx, const int uDim, const int vDim, M_Int& lookup);

	void uInsertLine2(float ratio, M_Vec3f& dpoint, int uDim, int vDim, int u, int v, bool leaveNorth, bool leftSide);
	void vInsertLine2(float ratio, M_Vec3f& dpoint, int uDim, int vDim, int u, int v, bool leaveEast, bool leftSide);

	bool uInsertLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, int uDim, int vDim, Vec2f& dim);
	bool vInsertLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, int uDim, int vDim, Vec2f& dim);
	bool deleteControlRowColum(const Vec2f par, const int idxA, const int idxB, Matrix matrix, M_Vec3f& dpoint, M_Vec3f& point, M_Float& weight, M_Float& uKnot, int uDim, int vDim, int& idx, Vec2f& dim, Vec2f& order, M_Int& lookup);

	void uInsertLineDivert(const int firstStitchPoly, bool& foundFirstStitchPoly, const bool leftSide, const int idx, const int idx2, const float ratio, const bool invertRatio, M_Insert& m_insert, int insertIdx, Vec2i cutLine, int poly, int entryPolyEdge, int otherPolyEdge, int altPolyEdge, const bool aSide, int alternativeCentreIdx, NurbsPatch* otherPatch, Vec2i otherUV, NurbsPatch* prevPatch, int& beginInsertIdx);
	void vInsertLineDivert(const int firstStitchPoly, bool& foundFirstStitchPoly, const bool leftSide, const int idx, const int idx2, const float ratio, const bool invertRatio, M_Insert& m_insert, int insertIdx, Vec2i cutLine, int poly, int entryPolyEdge, int otherPolyEdge, int altPolyEdge, const bool aSide, int alternativeCentreIdx, NurbsPatch* otherPatch, Vec2i otherUV, NurbsPatch* prevPatch, int& beginInsertIdx);

	void uInsertLineDivertInnerPatch(const int firstStitchPoly, bool& foundFirstStitchPoly, const bool leftSide, const int idx, const int idx2, const float ratio, const bool invertRatio, M_Insert& m_insert, int insertIdx, Vec2i cutLine, int& beginInsertIdx, Vec2i otherUV);
	void vInsertLineDivertInnerPatch(const int firstStitchPoly, bool& foundFirstStitchPoly, const bool leftSide, const int idx, const int idx2, const float ratio, const bool invertRatio, M_Insert& m_insert, int insertIdx, Vec2i cutLine, int& beginInsertIdx, Vec2i otherUV);

	void uInsertLine(const int firstStitchPoly, bool& foundFirstStitchPoly, const bool leftSide, const int idx, const int idx2, const float ratio, const bool invertRatio, M_Insert& m_insert);
	void vInsertLine(const int firstStitchPoly, bool& foundFirstStitchPoly, const bool leftSide, const int idx, const int idx2, float ratio, bool invertRatio, M_Insert& m_insert);
	void uInsertLine0(bool& foundFirstStitchPoly, const bool leftSide, const bool leaveNorthOrEast, const int u, const int v, const float ratio, const bool invertRatio, M_Insert& m_insert);
	void vInsertLine0(bool& foundFirstStitchPoly, const bool leftSide, const bool leaveNorthOrEast, const int u, const int v, const float ratio, const bool invertRatio, M_Insert& m_insert);
	bool uDeleteLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, M_Float& weight, M_Float& uKnot, int uDim, int vDim, int& idx, Vec2f& dim, Vec2f& order);
	bool vDeleteLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, M_Float& weight, M_Float& uKnot, int uDim, int vDim, int& idx, Vec2f& dim, Vec2f& order);
	void uDeleteLine(const int idx, const int idx2, const NurbsPatch* firstPatch, const int firstIdx2, bool& foundFirst, M_DelCtrlLn& delCtrlLn);
	void vDeleteLine(const int idx, const int idx2, const NurbsPatch* firstPatch, const int firstIdx2, bool& foundFirst, M_DelCtrlLn& delCtrlLn);


	static void updateKnot(int dim, M_Float& knot, float& order);
	static void updateKnot(int oldDim, int newDim, M_Float& knot, float& order);
	void selectControlPointsBackfaceIgnore(M_<NurbsPatch*>& contigList, const M_Bool& m_pointIsVisible);
	void selectControlPoints(M_<NurbsPatch*>& contigList, const bool perspective);
	void selectControlPointsStraightCage(M_<NurbsPatch*>& contigList, const bool perspective);
	bool objectIsInBox() const;
	void selectControlPointsNoBoxTest(M_<NurbsPatch*>& contigList);
	void fillEdgesWithCtrlPolys();
	void deleteStitchPolys();
	//void getRatioPhaseForInsert();

	void convert_to_1_v_mid_points();
	void convert_to_1_u_mid_points();

	void genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen);

	void mirrorModifyWest();
	void mirrorModifyEast();
	int getMirrorStateViaPart(); //0 == no mirror, 1 == centerMirror, 2 == has twin
	bool isInsideOut()const;
	void setInsideOut();
	void setInsideOut2();
	void setInsideOut3();
	void setInsideOut4();

	void addOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addNorthWestCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addSouthWestCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addNorthEastCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addSouthEastCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addTwoByOneCtrlPolyPntSouthMid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addTwoByOneCtrlPolyPntNorthMid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addUJoinCtrlPolyPntSouth(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addUJoinCtrlPolyPntNorth(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addOneByTwoCtrlPolyPntEast0Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addOneByTwoCtrlPolyPntWest0Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addOneByTwoCtrlPolyPntEast1Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);
	void addOneByTwoCtrlPolyPntWest1Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly);

	//void makeCtrlPolyInsideOut(const int poly, M_CtrlPolyPoint& ctrlPolyPoint);
	void selectRing(int v, int uDim, int vDim, M_<NurbsPatch*>& contigList);
	void selectCol(int u, int uDim, int vDim, M_<NurbsPatch*>& contigList);
	void selectRings(int v, int uDim, int vDim);
	void transformPoints(Vec3f scale, Rotation rotation, Vec3f translation);
	int getMirrorIdx(int idx) const;
	int getMirrorTwinlessIdx(int idx) const;
	int getMirrorTwinlessLineFormat1(int uOrVIdx, int uDim, bool doU);
	void mirrorSingleCP(int idx);
	static bool selectSelectedPoint(const bool& perspective);
	void updateVCoord(Oscillator& osc);
	void updateVCoordChords(int& limit, int staves, const M_Float& phase, const M_Float& m_multiple, float randomize, M_Vec3f& coord, M_Vec2f& vCoord, Oscillator* osc, bool doPatch, M_Vec3f& oscCoord, int oscStaveSteps);
	void hairOnSurface(int& coordIdx, M_Vec2f& coord, NurbsPatch* hairPatch);
	bool isEdge(int idx);
	void synchronizeAllStemCells();
	void synchronizeCellsAndBuild();
	void synchronizeCellsAndBuildSync();
	void setAllWeightsToOne();
	bool isInStitchPoly(int poly, const M_CtrlPolyPoint& ctrlPolyPoint) const;
	bool isInStitchPoly(int idx, int poly, const M_CtrlPolyPoint& ctrlPolyPoint) const;

	void insertCol2(const int uDim, const int vDim, M_Vec3f& dpoint, M_Float& weight, M_Int& cpIndex, M_Vec3f& oldPoints, M_Float& oldWeights, M_Int& oldCpIndex, const int col, const float ratio);
	void insertCol(M_Vec3f& oldPoints, M_Float& oldWeights, M_Int& oldCpIndex, const int col, const float ratio, int& uDim, const bool centerMirrorLine);
	void deleteCol(const int uDim, const int vDim, M_Int& cpIndex, M_Vec3f& dpoint, M_Float& weight, M_Int& oldCpIndex, M_Vec3f& oldPoints, M_Float& oldWeights, const int col);
	void insertRow2(const int uDim, const int vDim, M_Vec3f& dpoint, M_Float& weight, M_Int& cpIndex, M_Vec3f& oldPoints, M_Float& oldWeights, M_Int& oldCpIndex, const int row, const float ratio);
	void insertRow(M_Vec3f& oldPoints, M_Float& oldWeights, M_Int& oldCpIndex, const int row, const float ratio, int& vDim);
	void deleteRow(const int uDim, const int vDim, M_Int& cpIndex, M_Vec3f& dpoint, M_Float& weight, M_Int& oldCpIndex, M_Vec3f& oldPoints, M_Float& oldWeights, const int row);

	void uUpdateKnotMid1();
	void vUpdateKnotMid1();

	void setToCenterLine0(Vec2i& cutLine);
	void setToCenterLine(Vec2i& cutLine);

	void flipPolarity();
	NurbsPatch* createCopyOf();
	NurbsPatch* createCopyOf(M_Int& redirect);
	void copyData(const NurbsPatch* from);
	bool postTug();
	void uAddControlPolyLine(int uc, int vc, IndexedLineSet* polyLine)const;
	void vAddControlPolyLine(int uc, int vc, IndexedLineSet* polyLine)const;
	void patchToCentralCPs(); //for debugging only
	void setPointAndCp(int i, Vec3f vec);

	void redirectRow(int row);
	void redirectCol(int col);

	void redirectSouthToNorth();
	void redirectEastToWest();
	void joinEastToWest();
	void joinEastToEast(NurbsPatch* patchB);
	void joinWestToWest(NurbsPatch* patchB);
	void joinEastToWest(NurbsPatch* patchB);
	void joinWestToEast(NurbsPatch* patchB);
	void joinNorthToSouth(NurbsPatch* patchB);
	void joinEastToSouth(NurbsPatch* patchB);
	void joinEastToSouthReverse(NurbsPatch* patchB);
	void joinWestToSouth(NurbsPatch* patchB);
	void joinSouthToSouthReverse(NurbsPatch* patchB);
	void joinNorthToNorthReverse(NurbsPatch* patchB);
	void joinEastToNorth(NurbsPatch* patchB);
	void joinWestToNorth(NurbsPatch* patchB);
	void joinWestToNorthReverse(NurbsPatch* patchB);
	void joinSouthToNorth(NurbsPatch* patchB);
	void scrunchNorth();
	void scrunchSouth();
	void joinExtraCenterShaftPoints(NurbsPatch* extra);
	int closeEndPointsForSpiderWeb(Vec3f center);
	void breakControlPointJoin(int selIdx);
	void joinBetweenPoints(const int idA, NurbsPatch* patchB, const int idxB);//for stitching mode
	void collectSharedPoints(int cIdx, M_NurbsPatch& m_patch, M_Int& m_idx);
	int isMouseOverEdge(const Matrix& matrix);//0 == no edge deteced
	void colorEdge(int edge, const Color3f edgeColor);
	NurbsPatch* getNearestMatchingEdge(const int edgeA, int& nearestEdge)const;//returns nearest patch
	void selectAllPoints(M_<NurbsPatch*>& contigList);
	bool centerBackface();
	void makeExistingNameUniqueForMirror(NurbsPatch* twinPatch);
	void unstitchEdge(Insert& insert, M_Insert m_insert, const int isrtIdx, NurbsPatch*& prevPatch, int& prevIndex, const int offset, const int incAmount, const bool doLoop, const NurbsPatch* loopPatch, const int loopIndex);
	bool pointIsVisible(int idx);

	void bnspeBuildPolies();
	void bnspeBuildSyncPolies();

	virtual void doColorForStem(Art* part, M_Vec3f& coord, const int h, const int c){};
};


class ColorPatch : public NurbsPatch, public MField25{
public:
enum CurveType{
	MID_0,
	MID_1,
	MID_2,
	BEZIER,
};

enum Edge{
	SOUTH = 1,
	EAST = 2,
	NORTH = 3,
	WEST = 4,
};



	int getType()const{return COLORPATCH;}
    int getFLen()const{return MField25::size();};
    Field* getField(const int i)const{return MField25::fieldArray[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

	//int uGetCurveType()const; untested but probaly works
	//int vGetCurveType()const;

	MFColor color;

	ColorPatch():
		MField25(
			&point,
			&dpoint,
			&weight,
			&color,
			&cpIndex,
			&uKnot,
			&vKnot,
			&order,

			&object,
			&cage,
			&showAll,
			&mirror,
			&dim,

			&endSP,
			&uJoinEnds,
			&beginSP,

			&sync,
			&insideOut,

			&scale,
			&rotation,
			&translation,
			&fuseSurface,
			&stem,
			&mapper,
			&setup
		),

		color(M_Color3f()),


/////////////
		index(NurbsPatch::index)
	{}
	M_Int& index;
	void processFields();
	void doColorForStem(Art* part, M_Vec3f& coord, const int h, const int c);
	Color3f getColor(Vec2f crd);
};

class Shape : public Node, public MField2 {
public:
class AppearanceField : public SFNode {
public:
    Shape& home;
    AppearanceField( const NodePtr& par, Shape& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par);

    operator Appearance*()const {
        return (Appearance*)p;
    }
};
class GeometryField : public SFNode {
public:
    Shape& home;
    GeometryField( const NodePtr& par, Shape& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par);
};

    int getType()const{return SHAPE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    AppearanceField appearance;
    GeometryField geometry;

    Shape():
        MField2(
            &appearance,
            &geometry
        ),

        appearance(0, *this),
        geometry(0, *this),
        ////////////
        render(0)
    {
    }
    Mesh geomSource;
    Render* render;

    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void toPOVRayString(String& str);
    void toObjString(String& str);
};
class Appearance : public Node, public MField3 {
    public:
    int getType()const{return APPEARANCE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFNode material;
    SFNode texture;
    SFNode textureTransform;

    Appearance():
        MField3(
            &material,
            &texture,
            &textureTransform
        ),
        material(0),
        texture(0),
        textureTransform(0)
    {
    }
};
class Material : public Node, public MField6 {
public:
class Transparency : public SFFloat {
public:
    bool& renderProcessed;

    Transparency(float par, bool& renderProcessed):
        SFFloat(par),
        renderProcessed(renderProcessed)
    {};
    void operator=(const SFFloat& par){
        refValue() = par.refValue();
        renderProcessed = false;
    }
    void operator=(const float& par){
        refValue() = par;
        renderProcessed = false;
    }
};


    int getType()const{return MATERIAL;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}


    SFFloat ambientIntensity;
    SFColor diffuseColor;
    SFColor emissiveColor;
    SFFloat shininess;
    SFColor specularColor;
    Transparency transparency;

    Material():
        MField6(
            &ambientIntensity,
            &diffuseColor,
            &emissiveColor,
            &shininess,
            &specularColor,
            &transparency
            ),
        ambientIntensity(.2),
        diffuseColor(Color3f(.8,.8,.8)),
        emissiveColor(Color3f(0,0,0)),
        shininess(.2),
        specularColor(Color3f(0,0,0)),
        transparency(0.0f, renderProcessed)
    {}
};
class IndexedLineSet : public Geometry, public MField7 {
    public:
    int getType()const{return INDEXEDLINESET;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFInt32 set_colorIndex;
    MFInt32 set_coordIndex;
    SFNode coord;
    SFNode color;
    MFInt32 colorIndex;
    SFBool colorPerVertex;
    MFInt32 coordIndex;

    IndexedLineSet():
        MField7(
            &set_colorIndex,
            &set_coordIndex,
            &coord,
            &color,
            &colorIndex,
            &colorPerVertex,
            &coordIndex
        ),

        set_colorIndex(M_Int()),
        set_coordIndex(M_Int()),
        coord(0),
        color(0),
        colorIndex(M_Int()),
        colorPerVertex(true),
        coordIndex(M_Int()),

        ///
        indexCount(0),
        noZbuf(false)
    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToRenderFormat(Render*& render);
#endif
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
    void updateAllVertexInBuffers(Render*& render);
    bool noZbuf;
private:
    int indexCount;
};


class ImageTexture : public Node, public Download, public MField3{
    public:
    class Url : public MFString {
    public:
        ImageTexture& home;

        Url( const M_String& par, ImageTexture& home):
            MFString(par),
            home(home)
        {};
        void operator=(const ImageTexture& par){
            refValue()=par.url.refValue();
            home.update=true;
        }
        void operator=(const M_String& par){
            refValue()=par;
            home.update=true;
        }
    };

    int getType()const{return IMAGETEXTURE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}

#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Url url;
    SFBool repeatS;
    SFBool repeatT;

    ImageTexture():
        MField3(
            &url,
            &repeatS,
            &repeatT
        ),

        url(M_String(), *this),
        repeatS(true),
        repeatT(true),
/////////////////////
        update(true),
        alphaTexture(false),
#ifdef DIRECT3D
        dxTexture(0),
#endif
        hInternetOpenUrl(0)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
#ifdef DIRECT3D
    IDirect3DTexture*  dxTexture;
#endif
    bool download();
    bool openUrl();
    int hInternetOpenUrl;
    M_<char> internetBuf;
    bool update;
    bool alphaTexture;

    void getUrlPath(String&){};
    NodePtr urlPathNode;
};



class TextureTransform : public Node, public MField4 {
public:
    class Center : public SFVec2f {
    public:
        bool& renderProcessed;
        Center( const Vec2f& value, bool& renderProcessed):
            SFVec2f( value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const Vec2f& par){
            refValue()=par;
            renderProcessed=false;
        }
    };
    class Scale : public SFVec2f {
    public:
        bool& renderProcessed;
        Scale( const Vec2f& value, bool& renderProcessed):
            SFVec2f( value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const Vec2f& par){
            refValue()=par;
            renderProcessed=false;
        }
    };
    class Rotation : public SFFloat {
    public:
        bool& renderProcessed;
        Rotation( const float& value, bool& renderProcessed):
            SFFloat( value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const float& par){
            refValue()=par;
            renderProcessed=false;
        }
    };
    class Translation : public SFVec2f {
    public:
        bool& renderProcessed;
        Translation( const Vec2f& value, bool& renderProcessed):
            SFVec2f( value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const Vec2f& par){
            refValue()=par;
            renderProcessed=false;
        }
    };

    int getType()const{return TEXTURETRANSFORM;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Center center;
    Scale scale;
    Rotation rotation;
    Translation translation;

    TextureTransform():
        MField4(
            &center,
            &scale,
            &rotation,
            &translation
        ),
        center(Vec2f(), renderProcessed),
        scale(Vec2f(1,1), renderProcessed),
        rotation(0, renderProcessed),
        translation(Vec2f(), renderProcessed)
        /////////////
    {
    }
};





inline Vec2f calcNormal(const Vec2f& a, const Vec2f& b){
    return Vec2f((b.y - a.y) * -1, b.x - a.x).normalize();
}

inline void calcNormal(const Vec3f& a, const Vec3f& b, const Vec3f& c, Vec3f& normal){
    Vec3f q = b - a;
    Vec3f s = c - a;

    normal.x = s.y * q.z - s.z * q.y;
    normal.y = s.z * q.x - s.x * q.z;
    normal.z = s.x * q.y - s.y * q.x;

    normal = normal.normalize();
}

inline Vec3f calcNormal(const Vec3f& a, const Vec3f& b, const Vec3f& c){
    Vec3f q = b - a;
    Vec3f s = c - a;

    return Vec3f(
        s.y * q.z - s.z * q.y,
        s.z * q.x - s.x * q.z,
        s.x * q.y - s.y * q.x
    ).normalize();
}

class AbstractTransform {
};
class Group : public AGroup, public PovRay, public MField5 {
    public:
    int getType()const{return _GROUP;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}



    MFNode children;

    Group():
        MField5(
            &addChildren,
            &removeChildren,
            &children,
            &bboxCenter,
            &bboxSize
        ),
        children(M_Node()),
        AGroup(children)
    {
    }
    void processFieldsLoop(HAnimHumanoid* humanoid);
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
};
/*
#ifdef SEAMLESS3DGUI
    void genNodeListRecursion( int indent);
    void paintSceneTree(int indent, int& line);
    int winProc(void* hDlg,unsigned iMessage, unsigned wParam, unsigned lParam);
    void Group::newChildrenNode(NodePtr node);
#endif
};
*/
class NoOutputGroup : public AGroup, public MField5 {
    public:
    int getType()const{return NOOUTPUTGROUP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    MFNode children;
    Field* fieldArray[5];

    NoOutputGroup():
        MField5(
            &addChildren,
            &removeChildren,
            &children,
            &bboxCenter,
            &bboxSize
        ),
        children(M_Node()),
        AGroup(children)
    {
    }
//	void senseTug(Matrix matrix, Matrix iMatrix);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};
class GestureGroup : public AGroup, public MField8{
    public:
    int getType()const{return GESTUREGROUP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFRotation rotation;
    SFVec3f translation;
    MFNode children;
    MFNode mobility;

    GestureGroup():
        MField8(
            &addChildren,
            &removeChildren,
            &rotation,
            &translation,
            &bboxCenter,
            &bboxSize,
            &children,
            &mobility
        ),
        rotation(Rotation(0,0,1,0)),
        translation(Vec3f()),
        children(M_Node()),
        mobility(M_Node()),

        AGroup(children)
    {
    }
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
    static int numOfGestures;
    static bool addProto;
};
class Mobility : public Node, public MField3{
    public:
class OnRot : public SFNode {
public:
    Mobility& home;
    OnRot(const NodePtr& par, Mobility& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par1);
};
class OnTran : public SFNode {
public:
    Mobility& home;
    OnTran(const NodePtr& par, Mobility& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par1);
};
    int getType()const{return MOBILE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    MFNode children;
    OnRot onRot;
    OnTran onTran;

    Mobility():
        MField3(
            &children,
            &onRot,
            &onTran
        ),
        children(M_Node()),
        onRot(0, *this),
        onTran(0, *this)
    {
    }
    void decodeVoid(const String& str, int& i, int& bracketCntr);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
};

class Transform : public AGroup, public PovRay, public MField10 {
    public:
    int getType()const{return TRANSFORM;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFVec3f center;
    SFRotation scaleOrientation;
    SFVec3f scale;
    SFRotation rotation;
    SFVec3f translation;
    MFNode children;

    Transform():
        MField10(
            &addChildren,
            &removeChildren,
            &center,
            &scaleOrientation,
            &scale,
            &rotation,
            &translation,
            &bboxCenter,
            &bboxSize,
            &children
        ),
        center(Vec3f()),
        scaleOrientation(Rotation(0,0,1,0)),
        scale(Vec3f(1,1,1)),
        rotation(Rotation(0,0,1,0)),
        translation(Vec3f()),
        children(M_Node()),

        AGroup(children)
    {
    }
    virtual ~Transform()
    {
    };
inline Matrix dxWindMatrix(Matrix& parentMatrix);
void renderNode(Matrix parentMatrix, Vec3f parentColScale);
void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
#ifdef SEAMLESS3DGUI
    void senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor);
    void catInverseRotation(Rotation& r);
#endif
};







#define CalcoordLen numRings * numStaves + bottomIsPoint + topIsPoint


#define PLANE_X 0x1
#define PLANE_Y 0x2
#define PLANE_Z 0x4


#define MAX_URL 0x200

void searchTillNextWord(char*& p);
void searchTillText(char*& p);


void rotateVectorX(Vec3f& v,float angle);
void rotateVectorY(Vec3f& v,float angle);
void rotateVectorZ(Vec3f& v,float angle);
void tranMVec3f(Vec3f* vp,int count,float tx,float ty,float tz);
void rotXMVec3f(Vec3f* vp,int AngleCnt,float angle);
void rotYMVec3f(Vec3f* vp,int AngleCnt,float angle);
void rotZMVec3f(Vec3f* vp,int AngleCnt,float angle);
void scaleMVec3f(Vec3f* vp,int count,float sx,float sy,float sz);



/*
    public:
    int getType()const{throw this;}
    int getFLen()const{return 0;};
    Field* getField(const int i)const{throw this;}
    int getFieldGroupIndex(const int &i){throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}

    void copyFieldsData(Shape* shape){};
    int getEffectLength(){return 0;};
    M_Int lookup;
    Shape* outputShape;
    inline void convertPartToIFS( Art* thisPart, IndexedFaceSet* ifs, M_Vec3f& ifsCoord, M_Int& ifsCoordIndex, const M_Vec3f& coord, const M_Triangle& triangle){};
    void convertGeomSourceToPart(Art* part, const Mesh& source){};
    void stemGenerateTri(const Stem& stem, const& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper);
    void stemGenerateTri(const Stem& stem, const ALathe& lathe, const& k, Art*& part, M_Vec3f& coord, const M_Node& textureMapper);
    inline void sortOutIndex( Mesh& gs, M_Vec3f& gsCoord, M_Int& gsCoordIndex, const Art* thisPart, const M_Vec3f& coord, const M_Triangle& triangle, Matrix& imatrix) const;
    inline void optimize(M_Art&){};
    void addColorNode(VertexGeometry* outputGeometry, VertexGeometry* sourceGeometry)const{};

    Color3f diffuseColor;
#ifdef SEAMLESS3DGUI
    void breakColorJoin(Seamless& smls){};
#endif
};
*/
class FieldGroupExtra{
public:
    virtual bool processGroup(int groupIndex, int command, int& newLen)=0;
};

class Polyline2D : public Geometry, public MField1 {
    public:
class LineSegments : public MFVec2f {
public:
    bool& renderProcessed;

    LineSegments(const M_Vec2f& par, bool& renderProcessed):
        MFVec2f(par),
        renderProcessed(renderProcessed)
    {};
    void operator=(const MFVec2f& par){
        refValue()=par.refValue();
        renderProcessed=false;
    }
    void operator=(const M_Vec2f& par){
        refValue()=par;
        renderProcessed=false;
    }
    void operator+=(const Vec2f& par){
        refValue()+=par;
        renderProcessed=false;
    }
    void resize(int newLen){
        refValue().length=newLen;
        renderProcessed=false;
    }
};

    int getType()const{return POLYLINE2D;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    LineSegments lineSegments;

    Field* fieldArray[1];

    Polyline2D():
        MField1(
            &lineSegments
        ),

        lineSegments(M_Vec2f(), renderProcessed)

    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToRenderFormat(Render*& render);
#endif
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;

};



class NavigationInfo : public Node, public MField7 {
    public:
    int getType()const{return NAVIGATIONINFO;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFBool set_bind;
    MFFloat avatarSize;
    SFBool headlight;
    SFFloat speed;
    MFString type;
    SFFloat visibilityLimit;
    SFBool isBound;

    NavigationInfo():
        MField7(
            &set_bind,
            &avatarSize,
            &headlight,
            &speed,
            &type,
            &visibilityLimit,
            &isBound
        ),
        set_bind(false),
        avatarSize(M_Float(0.25, 1.6, 0.75)),
        headlight(true),
        speed(1),
        type(M_String()),
        visibilityLimit(0),
        isBound(false)

    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    M_Int index;
#endif
};
class DirectionalLight : public Node, public MField6 {
    public:
    int getType()const{return DIRECTIONALLIGHT;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFFloat ambientIntensity;
    SFColor color;
    SFVec3f direction;
    SFBool global;
    SFFloat intensity;
    SFBool on;

    DirectionalLight():
        MField6(
            &ambientIntensity,
            &color,
            &direction,
            &global,
            &intensity,
            &on
        ),

        ambientIntensity(0),
        color(Color3f(1,1,1)),
        direction(Vec3f(0,0,-1)),
        global(false),
        intensity(1),
        on(true)
    {
    }
#ifdef SEAMLESS3DGUI
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void setUpDirectionalLightRender();
#endif
};

class Background : public Node, public MField12 {
    public:
    int getType()const{return BACKGROUND;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFBool set_bind;
    MFFloat groundAngle;
    MFColor groundColor;
    MFFloat skyAngle;
    MFColor skyColor;
    MFString backUrl;
    MFString bottomUrl;
    MFString frontUrl;
    MFString leftUrl;
    MFString rightUrl;
    MFString topUrl;
    SFBool isBound;

    Background():
        MField12(
            &set_bind,
            &groundAngle,
            &groundColor,
            &skyAngle,
            &skyColor,
            &backUrl,
            &bottomUrl,
            &frontUrl,
            &leftUrl,
            &rightUrl,
            &topUrl,
            &isBound
        ),
        set_bind(false),
        groundAngle(M_Float()),
        groundColor(M_Color3f()),
        skyAngle(M_Float()),
        skyColor(M_Color3f()),
        backUrl(M_String()),
        bottomUrl(M_String()),
        frontUrl(M_String()),
        leftUrl(M_String()),
        rightUrl(M_String()),
        topUrl(M_String()),
        isBound(false)
    {
#ifdef SEAMLESS3DGUI
        index.length=10;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
    virtual void renderNode(Matrix parentMatrix, Vec3f parentColScale);
#endif
};
/*
class ViewModel : public Node, public MField {
    public:
class Perspective : public SFBool {
public:
    ViewModel& home;

    Perspective(const bool& par, ViewModel& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Capture : public SFBool {
public:
    ViewModel& home;

    Capture(const bool& par, ViewModel& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Zoom : public SFFloat {
public:
    ViewModel& home;

    Zoom(const bool& par, ViewModel& home):
        SFFloat(par),
        home(home)
    {};
    void operator=(const float& par);
};

class Orientation : public SFRotation {
public:
    ViewModel& home;

    Orientation(const Rotation& par, ViewModel& home):
        SFRotation(par),
        home(home)
    {};
    void operator=(const Rotation& par);
};
class Location : public SFVec3f {
public:
    ViewModel& home;

    Location(const Vec3f& par, ViewModel& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par);
};
    int getType()const{return VIEWMODEL;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i){throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}
#endif

    Perspective perspective;
    Zoom zoom;
    Orientation orientation;
    Location location;
    Capture capture;

    Field* fieldArray[5];

    ViewModel():
        MField(
            fieldArray,
            &perspective,
            &zoom,
            &orientation,
            &location,
            &capture
        ),

        perspective(false, *this),
        zoom(1, *this),
        orientation(Rotation(), *this),
        location(Vec3f(), *this),
        capture(false, *this)

    {
    }
};
*/
class Viewpoint : public Node, public MField18, public VisualAid{
    public:
class Perspective : public SFBool {
public:
    Viewpoint& home;

    Perspective(const bool& par, Viewpoint& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Capture : public SFBool {
public:
    Viewpoint& home;

    Capture(const bool& par, Viewpoint& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class AnimCapture : public SFBool {
public:
    Viewpoint& home;

    AnimCapture(const bool& par, Viewpoint& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class AnimDelete : public SFBool {
public:
    Viewpoint& home;

    AnimDelete(const bool& par, Viewpoint& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Zoom : public SFFloat {
public:
    Viewpoint& home;

    Zoom(const SFFloat& par, Viewpoint& home):
        SFFloat(par),
        home(home)
    {};
    void operator=(const float& par);
};

class FieldOfView : public SFFloat {
public:
    Viewpoint& home;

    FieldOfView(const SFFloat& par, Viewpoint& home):
        SFFloat(par),
        home(home)
    {};
    void operator=(const float& par);
};

class Orientation : public SFRotation {
public:
    Viewpoint& home;

    Orientation(const Rotation& par, Viewpoint& home):
        SFRotation(par),
        home(home)
    {};
    void operator=(const Rotation& par);
};
class LookAt : public SFVec3f {
public:
    Viewpoint& home;

    LookAt(const Vec3f& par, Viewpoint& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par);
};
class Up : public SFVec3f {
public:
    Viewpoint& home;

    Up(const Vec3f& par, Viewpoint& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par);
};
class Position : public SFVec3f {
public:
    Viewpoint& home;

    Position(const Vec3f& par, Viewpoint& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par);
};
class Live : public SFBool {
public:
    Viewpoint& home;

    Live(const bool& par, Viewpoint& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Aim : public SFBool {
public:
    Viewpoint& home;

    Aim(const bool& par, Viewpoint& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return VIEWPOINT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{return 0;}
    void setFieldGroupIndex(const int &, const int &){}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool set_bind;
	Live live;
	Aim aim;
    Perspective perspective;
    Zoom zoom;
    FieldOfView fieldOfView;
    Orientation orientation;
    LookAt lookAt;
	Up up;
    Position position;
    SFBool track;
    SFBool jump;
    SFString description;
    Capture capture;
    AnimCapture animCapture;
    AnimDelete animDelete;
    SFTime bindTime;
    SFBool isBound;

    Viewpoint():
        MField18(
            &set_bind,
			&live,
			&aim,
            &perspective,
            &zoom,
            &fieldOfView,
            &orientation,
            &lookAt,
			&up,
            &position,
            &track,
            &jump,
            &description,
            &capture,
            &animCapture,
            &animDelete,
            &bindTime,
            &isBound
        ),

        set_bind(false),
		live(false, *this),
		aim(false, *this),
        perspective(true, *this),
        zoom(1, *this),
        fieldOfView(.785398, *this),
        orientation(Rotation(0,0,1,0), *this),
        lookAt(Vec3f(0,0,0), *this),
		up(Vec3f(0,1,10), *this),
        position(Vec3f(0,0,10), *this),
        track(true),
        jump(true),
        description(String()),
        capture(false, *this),
        animCapture(false, *this),
        animDelete(false, *this),
        bindTime(0),
        isBound(false)
    {
    }
	static Viewpoint* withinSitesViewpoint;
	void calcOrientationFromLookAtUp();
	void calcLookAtFromOrientation();



	void houseKeep();
    void drag(Vec3f pt);
	bool isOverPoint();

	static int dragMode;
	static Vec3f remLookAt;
	static Vec3f remPosition;
	static float remLookAtDistance;
	static float remUpDistance;
	static Rotation remOrientation;

	enum{
		UP_LEVER = 1,
		POSITION = 2,
		LOOK_AT = 3,
	};
void senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor);

static void gotoFirstViewpoint();
static void gotoPrevViewpoint();
static void gotoNextViewpoint();
static bool neutral;
friend Perspective;
friend Zoom;
friend Orientation;
friend Position;

void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};

class Script : public Node, public DynamicMField {
    public:
    int getType()const{return SCRIPT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}


    MFString url;
    SFBool directOutput;
    SFBool mustEvaluate;


    Script():

        url(M_String()),
        directOutput(false),
        mustEvaluate(false)

    {
        *this+=&url;
        *this+=&directOutput;
        *this+=&mustEvaluate;

        Properties sprop;

        sprop.name=String("url");
        sprop.groupIndex=0;
        properties+=sprop;
        sprop.groupIndex=-1;
        sprop.name=String("directOutput");
        properties+=sprop;
        sprop.name=String("mustEvaluate");
        properties+=sprop;

#ifdef SEAMLESS3DGUI
        index.length=50;
#endif
    }
    M_Int index;

    M_Properties properties;

    M_Function function;

    void getFields(const String& str, int& i);
    const M_Properties& getProperties()const{
        return properties;
    }
//	M_Properties& getProperties2(){
//		return properties;
//	}
    void toString(String& str, bool newList=true);
    void toXMLString(String& str, bool newList, const char* containerField);
    void processFields();
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void useSFNode(Node* node);
};




void GetLineIntersection(float x1, float y1,	float x2, float y2,	float x3, float y3,	float x4, float y4,	float& xi, float& yi);


#define BRANCH_SIDE_JOIN 1
#define BRANCH_TOP_JOIN 2
#define LEAF 3


struct LocRotPointer{
    Rotation* rotlr;
    Vec3f* loclr;
    int cntlr;
    LocRotPointer(int sum){
        rotlr = new Rotation[sum];
        loclr = new Vec3f[sum];
        cntlr = 0;
    }
    void writeOutData(char* filename);
};




void copyTextFromFile(FILE* stream, char* fileName2);
void writeRotation(FILE* stream, char* header, Rotation& rot);


class StitchInstruction{
public:
    unsigned char instruct;
    unsigned char numOf;
    unsigned short int index;
    StitchInstruction():
        instruct(0),
        numOf(0),
        index(0)
    {}
    StitchInstruction(unsigned char instruct, unsigned char numOf, unsigned short int index):
        instruct(instruct),
        numOf(numOf),
        index(index)
    {}
};




/*
typedef HF <MFNode, M_Node, M_Build, Build> MFBuild;
template <typename F,typename V,typename M,typename S>
class HF : public F{
public:
    HF(const V& value):
        F(value)
    {}
    M& refValue(){
        return *(M*)&length;
    }
    void operator+=(S* par){
        refValue()+=par;
    }
    void operator+=(Ptr <S> par){
        refValue()+=par;
    }
};
typedef HF <MFNode, M_Node, M_Build, Build> MFBuild;
*/









void senseMirrorVertexModeX(Art* pj);
void modifyMirrorVertexX(Art* pj);


#define PROJECT_FORWARD 0
#define PROJECT_DOWN	1
#define PROJECT_UP		2

extern int g_projectDirection;

class SFSurface : public SFNode {
public:

    SFSurface(const NodePtr& value):
        SFNode(value)
    {};
    SurfaceGenerator* operator =(SurfaceGenerator* par){
        refValue()=par;
        return par;
    }
    SurfaceGenerator* operator ->()const{
        return dynamic_cast<SurfaceGenerator*>(refValue().getRawPtr());
    }
    operator SurfaceGenerator*(){
        return dynamic_cast<SurfaceGenerator*>(refValue().getRawPtr());
    }
};
/*
class AbstractStem{
public:
    virtual void doBuild(FuseSurface* parentFuse, NurbsPatch* patch) = 0;
    int beginTriOffset;
    AbstractStem():
        beginTriOffset(0)
    {}
    virtual M_Node& refFuse() = 0;
};
*/

/*
class Stem2 : public AbstractStem, public Build, public MField12{
    public:
class Cells : public SFVec2f {
    public:
    Stem2& home;
    Cells(const Vec2f& value, Stem2& home):
        SFVec2f(value),
        home(home)
    {};
    void operator=(const Vec2f& par);
};


    int getType()const{return STEM2;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    SFVec2f slide;
    MFFloat xDistance;
    MFFloat yDistance;
    MFInt32 staves;
    MFInt32 bands;
    SFBool sync;
    SFBool mould;
    Cells cells;

    PartChild part;
    Setup setup;
    MFNode textureMapper;
    MFNode fuse;

    Stem2():
        Build(setup),
        MField12(
            &slide,
            &xDistance,
            &yDistance,
            &staves,
            &bands,
            &sync,
            &mould,
            &cells,

            &part,
            &setup,
            &textureMapper,
            &fuse
        ),

            slide(Vec2f()),
            xDistance(M_Float()),
            yDistance(M_Float()),
            staves(M_Int()),
            bands(M_Int()),
            sync(false),
            mould(true),
            cells(Vec2f(), *this),

            part(M_Node()),
            setup(0, *this),
            textureMapper(M_Node()),
            fuse(M_Node())
        ///
    {
        index.length = 4;
    }
    M_Int index;

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild(FuseSurface* parentFuse, NurbsPatch* patch);
    void doBuild();
    M_Node& refFuse(){
        return fuse.refValue();
    }


private:
};
*/
class Polymap : public Build, public MField10{
    public:


    int getType()const{return POLYMAP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    SFBool add;
    MFVec2f coord;
    MFColor color;
    MFVec2f texCoord;
    MFInt32 coordIndex;
    MFInt32 rc;
    MFInt32 patchOwner;
    MFNode surface;
    SFPart part;
    Setup setup;

    Polymap():
        Build(setup),
        MField10(
            &add,
            &coord,
            &color,
            &texCoord,
            &coordIndex,
            &rc,
            &patchOwner,
            &surface,
            &part,
            &setup
        ),

        add(true),
        coord(M_Vec2f()),
        color(M_Color3f()),
        texCoord(M_Vec2f()),
        coordIndex(M_Int()),
        rc(M_Int()),
        patchOwner(M_Int()),
        surface(M_Node()),
        part(0),
        setup(0, *this),

        ///
        beginCoordOffset(0),
        beginTriOffset(0)
    {
        index.length = 2;
    }
    M_Int index;
    int beginCoordOffset;
    int beginTriOffset;

    int rcCoord(int i){
        i = coordIndex[i];
        int g = i - coord.length;
        if(g >= 0){
            i = rc[g];
        }
        return i;
    }
    int rcCoord(int i, int& c){
        i = c = coordIndex[i];
        int g = i - coord.length;
        if(g >= 0){
            i = rc[g];
        }
        return i;
    }
    void doBuild();
    void cpNodeListAdd(Seamless* smls);
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertCoordIndexToTriangle(Art* part, int h, int c, int s);
    void processFields();

private:
//	inline void buildMultiOwnedRings(Generator* surface);
};

/*
class Stem : public Build {
public:

    M_Node	source;
    SurfaceGeneratorPtr	surface;
    M_Node	part;
    M_Node	curveJoins;


    Rotation orientation;
    Vec2f slide;
    float xDistance;
    M_Float yDistance;
    M_Int xPointCnt;
    M_Int yPointCnt;
    bool xNegSide;
    bool yNegSide;
    bool xCentred;
    bool yCentred;
    bool beginSP;
    bool endSP;

    int getType()const{return STEM;}
    Stem(){
        xDistance=PI*2;
        xNegSide=false;
        yNegSide=false;
        xCentred=false;
        yCentred=false;
        beginSP=false;
        endSP=false;
#ifdef SEAMLESS3DGUI
        index=0;
#endif
    }
    Stem(SurfaceGeneratorPtr, IndexedFaceSetPtr, M_Node):
        surface(),
        source(),
        curveJoins()
    {
    }
    Stem(SurfaceGeneratorPtr,IndexedFaceSetPtr=IndexedFaceSetPtr(),IndexedFaceSetPtr=IndexedFaceSetPtr());
    void Stem::getFields( String& str,int& i);
#ifdef SEAMLESS3DGUI
    void genNodeListRecursion( int indent);
    void paintSceneTree(int indent, int& line);
    int winProc(void* hDlg,unsigned iMessage, unsigned wParam, unsigned lParam);
    int index;
#endif
    String toString(bool newList=true);

//private:
    IndexedFaceSetPtr	workIfs;
    void doBuild();
};
*/

class GeomRender: public Mesh{
public:
    AppearancePtr appearance;
    Render* render;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    GeomRender():
        render(0)
    {}
};

inline float _getPosX(int index, const float& baseRad, const M_Float& distance, const M_Float& curve){
    float accumHeight=0;
    float finishPosAx=baseRad;
    float finishPosAy=0;//startPosY;
    float accumAng=curve[0];
    float nrad=distance[0]/curve[1];

    float AxisBx=-nrad*cos(accumAng)+finishPosAx;
    float AxisBy=-nrad*sin(accumAng)+finishPosAy;


    float angDiff;

////////////////////////////////////////////////////////
    int s=1;
//	float r=0;


    for(;;){
        angDiff=curve[s];

        finishPosAx=nrad*cos(accumAng+angDiff);
        finishPosAy=nrad*sin(accumAng+angDiff);

        accumHeight+=distance[s-1];
        accumAng+=curve[s];
        if(++s>index)break;

        nrad=distance[s-1]/curve[s];

        AxisBx+=-nrad*cos(accumAng)+finishPosAx;
        AxisBy+=-nrad*sin(accumAng)+finishPosAy;
    }
    return finishPosAx+AxisBx;
}
inline float _getPosY(int index, float &baseRad, M_Float& distance, M_Float& curve){
    float accumHeight=0;
    float finishPosAx=baseRad;
    float finishPosAy=0;//startPosY;
    float accumAng=curve[0];
    float nrad=distance[0]/curve[1];

    float AxisBx=-nrad*cos(accumAng)+finishPosAx;
    float AxisBy=-nrad*sin(accumAng)+finishPosAy;


    float angDiff;

////////////////////////////////////////////////////////
    int s=1;
//	float r=0;


    while(true){
        angDiff=curve[s];

        finishPosAx=nrad*cos(accumAng+angDiff);
        finishPosAy=nrad*sin(accumAng+angDiff);

        accumHeight+=distance[s-1];
        accumAng+=curve[s];
        if(++s>index)break;

        nrad=distance[s-1]/curve[s];

        AxisBx+=-nrad*cos(accumAng)+finishPosAx;
        AxisBy+=-nrad*sin(accumAng)+finishPosAy;
    }
    return finishPosAy+AxisBy;
}
class CCBend : public Generator, public Node, public MField5, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    CCBend* home;
    Orientation( const Rotation& value):
        SFRotation(value)
    {};

    void operator=(const Rotation& par){
        home->orientation.refValue()=par;
        Vec3f location=home->location.refValue();
        home->matrix=rotationMatrix(par)*translationMatrix(location);
        home->imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    CCBend* home;
    Location( const Vec3f& value):
        SFVec3f(value)
    {};
    void operator=(const Vec3f& par){
        home->location.refValue()=par;
        Rotation orientation=home->orientation.refValue();
        home->matrix=rotationMatrix(orientation)*translationMatrix(par);
        home->imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
    friend Location;
    friend Orientation;

    int getType()const{return CCBEND;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Location location;
    Orientation orientation;
    MFFloat curve;
    MFFloat distance;
    MFFloat yAngle;

    CCBend():
        MField5(
            &location,
            &orientation,
            &curve,
            &distance,
            &yAngle
        ),
        location(Vec3f()),
        orientation(Rotation()),
        curve(M_Float()),
        distance(M_Float()),
        yAngle(M_Float()),

///////////////////////
        baseRad(0),
        flatRad(1),
        flatStartHeight(0),
        autoClose(false)
    {
        location.home=this;
        orientation.home=this;
#ifdef SEAMLESS3DGUI
        index.length=3;
#endif
    }
    void set_orientation(Rotation par){
        orientation=par;
        const Vec3f& location=this->location.refValue();
        matrix=rotationMatrix(par)*translationMatrix(location);
        imatrix=translationMatrix(-location)*rotationMatrix(-par);
    }
    void set_location(Vec3f par){
        location=par;
        const Rotation& orientation=this->orientation.refValue();
        matrix=rotationMatrix(orientation)*translationMatrix(par);
        imatrix=translationMatrix(-par)*rotationMatrix(-orientation);
    }
    float baseRad;
    float flatRad;
    float flatStartHeight;
    bool	autoClose;

    void generate(M_Vec3f& coord, const int h);
    float getDistance(int i)const;
    float getYangleSum(int index);
    void curveAllParts(NodePtr nd, Matrix parentMatrix, Matrix iParentMatrix);
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    void processFields();
    void houseKeep();
private:
    Matrix matrix;
    Matrix imatrix;
};

class CCScale : public Generator, public Node, public MField13, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    CCScale* home;
    Orientation( const Rotation& value):
        SFRotation(value)
    {};
    void operator=(const Rotation& par){
        home->orientation.refValue()=par;
        Vec3f location=home->location.refValue();
        home->matrix=rotationMatrix(par)*translationMatrix(location);
        home->imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    CCScale* home;
    Location( const Vec3f& value):
        SFVec3f(value)
    {};

    void operator=(const Vec3f& par){
        home->location.refValue()=par;
        Rotation orientation=home->orientation.refValue();
        home->matrix=rotationMatrix(orientation)*translationMatrix(par);
        home->imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
    friend Location;
    friend Orientation;

    int getType()const{return CCSCALE;}
    int getFLen()const{return size();}
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Location location;
    Orientation orientation;
    SFFloat baseRad;
    MFFloat curve;
    MFFloat distance;
    SFBool bNegX;
    SFBool bPosX;
    SFBool bNegZ;
    SFBool bPosZ;
    SFFloat negX;
    SFFloat posX;
    SFFloat negZ;
    SFFloat posZ;

    CCScale():
        MField13(
            &location,
            &orientation,
            &baseRad,
            &curve,
            &distance,
            &bNegX,
            &bPosX,
            &bNegZ,
            &bPosZ,
            &negX,
            &posX,
            &negZ,
            &posZ
        ),

        location(Vec3f()),
        orientation(Rotation()),
        baseRad(1),
        curve(M_Float()),
        distance(M_Float()),
        bNegX(false),
        bPosX(false),
        bNegZ(false),
        bPosZ(false),
        negX(1),
        posX(1),
        negZ(1),
        posZ(1)
    {
    location.home=this;
    orientation.home=this;
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    void generate(M_Vec3f& coord, const int h);
    void doPosX(M_Vec3f& coord);
    void doNegX(M_Vec3f& coord);
    void doPosZ(M_Vec3f& coord);
    void doNegZ(M_Vec3f& coord);

    void set_orientation(Rotation par){
        orientation=par;
        const Vec3f& location=this->location.refValue();
        matrix=rotationMatrix(par)*translationMatrix(location);
        imatrix=translationMatrix(-location)*rotationMatrix(-par);
    }
    void set_location(Vec3f par){
        location=par;
        const Rotation& orientation=this->orientation.refValue();
        matrix=rotationMatrix(orientation)*translationMatrix(par);
        imatrix=translationMatrix(-par)*rotationMatrix(-orientation);
    }
    float getDistance(int index=0x7fffffff)const;

    float getPosX(int index){
        return _getPosX( index, baseRad.refValue(), distance.refValue(), curve.refValue());
    }
    float getPosY(int index){
        return _getPosY( index, baseRad.refValue(), distance.refValue(), curve.refValue());
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    void processFields();
    void houseKeep();
private:
    void calcClose();
    void calcAbsolute(int i);

    Matrix matrix;
    Matrix imatrix;
};

class CCRingScale : public Generator, public Node, public MField11 {
    public:
class Orientation : public SFRotation {
public:
    CCRingScale* home;
    Orientation( const Rotation& value):
        SFRotation(value)
    {};
    void operator=(const Rotation& par){
        home->orientation.refValue()=par;
        Vec3f location=home->location.refValue();
        home->matrix=rotationMatrix(par)*translationMatrix(location);
        home->imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    CCRingScale* home;
    Location( const Vec3f& value):
        SFVec3f(value)
    {};
    void operator=(const Vec3f& par){
        home->location.refValue()=par;
        Rotation orientation=home->orientation.refValue();
        home->matrix=rotationMatrix(orientation)*translationMatrix(par);
        home->imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
    friend Location;
    friend Orientation;

    int getType()const{return CCRINGSCALE;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Location location;
    Orientation orientation;
    SFFloat centerOffset;
    SFFloat baseRad;
    MFFloat curve;
    MFFloat distance;
    SFFloat autoEnd;
    SFFloat negX;
    SFFloat posX;
    SFFloat negZ;
    SFFloat posZ;

    CCRingScale():
        MField11(
            &location,
            &orientation,
            &centerOffset,
            &baseRad,
            &curve,
            &distance,
            &autoEnd,
            &negX,
            &posX,
            &negZ,
            &posZ
        ),
        location(Vec3f()),
        orientation(Rotation()),
        centerOffset(0),
        baseRad(1),
        curve(M_Float()),
        distance(M_Float()),
        autoEnd(1),
        negX(1),
        posX(1),
        negZ(1),
        posZ(1)
    {
        location.home=this;
        orientation.home=this;
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
    void set_orientation(Rotation par){
        orientation=par;
        const Vec3f& location=this->location.refValue();
        matrix=rotationMatrix(par)*translationMatrix(location);
        imatrix=translationMatrix(-location)*rotationMatrix(-par);
    }
    void set_location(Vec3f par){
        location=par;
        const Rotation& orientation=this->orientation.refValue();
        matrix=rotationMatrix(orientation)*translationMatrix(par);
        imatrix=translationMatrix(-par)*rotationMatrix(-orientation);
    }
    void generate(M_Vec3f& coord, const int h);

    float getPosX(int index){
        return _getPosX( index, baseRad.refValue(), distance.refValue(), curve.refValue());
    }
    float getPosY(int index){
        return _getPosY( index, baseRad.refValue(), distance.refValue(), curve.refValue());
    }
    float getScaleVal(float scaleVal);
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    void processFields();
private:
    Matrix matrix;
    Matrix imatrix;
};


class CCTranslate : public Generator, public Node, public MField5, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    CCTranslate* home;
    Orientation( const Rotation& value):
        SFRotation(value)
    {};

    void operator=(const Rotation& par){
        home->orientation.refValue()=par;
        Vec3f location=home->location.refValue();
        home->matrix=rotationMatrix(par)*translationMatrix(location);
        home->imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    CCTranslate* home;
    Location( const Vec3f& value):
        SFVec3f(value)
    {};

    void operator=(const Vec3f& par){
        home->location.refValue()=par;
        Rotation orientation=home->orientation.refValue();
        home->matrix=rotationMatrix(orientation)*translationMatrix(par);
        home->imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
    friend Location;
    friend Orientation;

    int getType()const{return CCTRANSLATE;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Location location;
    Orientation orientation;
    SFFloat baseRad;
    MFFloat curve;
    MFFloat distance;

    CCTranslate():
        MField5(
            &location,
            &orientation,
            &baseRad,
            &curve,
            &distance
        ),
        location(Vec3f()),
        orientation(Rotation()),
        baseRad(0),
        curve(M_Float()),
        distance(M_Float()),

///////////////////////
        autoClose(false)
    {
        location.home=this;
        orientation.home=this;
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
    bool autoClose;
    void set_orientation(Rotation par){
        orientation=par;
        const Vec3f& location=this->location.refValue();
        matrix=rotationMatrix(par)*translationMatrix(location);
        imatrix=translationMatrix(-location)*rotationMatrix(-par);
    }
    void set_location(Vec3f par){
        location=par;
        const Rotation& orientation=this->orientation.refValue();
        matrix=rotationMatrix(orientation)*translationMatrix(par);
        imatrix=translationMatrix(-par)*rotationMatrix(-orientation);
    }
    void generate(M_Vec3f& coord, const int h);
    float getDistance(int i)const;
    float getYangleSum(int index);
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    void processFields();
    void houseKeep();
protected:
    Matrix matrix;
    Matrix imatrix;
};

class S_ECurve{
public:
    Vec2f centerPos;
    long double orientation;
    Matrix3 matrixA;
    Matrix3 matrixB;
};
typedef M_ <S_ECurve> M_ECurve;

extern short ControlWord;
extern short ModifiedControlWord;

#ifdef WIN32
void inline setMathHiPresision(){
_asm{
    fstcw ControlWord
    mov ax,ControlWord
    or ax,0300h
//	and ax,0fffeh//cause exception if invalid result
    mov ModifiedControlWord,ax
    fldcw ModifiedControlWord
}
}
void inline restoreMathPresision(){
_asm{
    fldcw ControlWord
}
}
#else
void inline setMathHiPresision(){}
void inline restoreMathPresision(){}
#endif

void setUpMatrices(M_ECurve& m_ecurve, const M_Vec2f& point, const M_Float& tangent, M_LongDouble& _a, M_LongDouble& _b);
//void setUpMatrices(M_ECurve& m_ecurve, const M_Vec3f& point, const M_Vec2f& tangent, M_LongDouble& _a, M_LongDouble& _b);

void setAB(M_ECurve& m_ecurve, const M_Vec2f& point, const M_Float& tangent, M_LongDouble& _a, M_LongDouble& _b);
//void setAB(M_ECurve& m_ecurve, const M_Vec3f& point, const M_Vec2f& tangent, M_LongDouble& _a, M_LongDouble& _b);
class Bezier{
    void getHypes(int i, float& xDiffA, float& yDiffA, float& hypA, float& xDiffB, float& yDiffB, float& hypB);
protected:
    static M_Vec2f pnt;
    void snapBorderTangentsForFirstCP(int i);
    void snapBorderTangentsForLastCP(int i);
    bool common_onActionForFix(Stem& stem, bool state);
    bool common_onActionForFix(bool state);
    void common_removeControlPoint(Stem& stem, int& newLen);
    void common_removeControlPoint(int& newLen);
    void calcMarkField(M_Float& mark, M_Float& t)const;
    inline float getCircumScaleVal(const float& scaleVal, const float& bezArcLen)const;


    //static M_Vec2f normA;
    //static M_Vec2f normB;
    //static M_Vec2f normC;
    //static M_Vec2f normD;
public:
inline Vec2f genCurve3(const int& numOfSteps, const int& p, const int& s, const float& t, M_Vec2f& pnt)const;
virtual bool onActionForFix(bool state)=0;

    enum{
        numOfSteps=1000
    };

    M_Vec2f& point;
    M_Int& index;
    M_Bool& fix;
    M_Int& nopfs; //nopfs number of points for segments

    Bezier(M_Vec2f& i_point, M_Int& i_index, M_Bool& i_border, M_Int& i_nopsf):
        point(i_point),
        index(i_index),
        fix(i_border),
        nopfs(i_nopsf)
    {}
    M_Vec2f line;
    M_Float flatHeight;
    M_Int heightIndex;
    M_Float segDistance;

    virtual void addControlPoint(int& newLen);
    virtual void removeControlPoint(int& newLen)=0;

    float getFirstTanCP();
    float getLastTanCP();

    void convertBorderToGroup();
    void convertGroupToFix();

    void matchBorderTangents(const Vec2f vec);
    void genSegDistance();
    void genCurve();
    void genFlatHeight();
    void genCircumHeight();
    void genHeightForScaleOrShift();
    void genHeightIndex();

    void bezHouseKeep();
    void bezProcessFields();
    int getGroupIndices(int& fixSegBeginIndex);
    virtual void drag(Vec3f pt);

    Matrix matrix;
    Matrix imatrix;
};
class Bezier3f{
    void getHypes(int i, float& xDiffA, float& yDiffA, float& zDiffA, float& hypA, float& xDiffB, float& yDiffB, float& zDiffB, float& hypB);
protected:
    static M_Vec3f pnt;
    void snapBorderTangentsForFirstCP(int i);
    void snapBorderTangentsForLastCP(int i);
    bool common_onActionForFix(Stem& stem, bool state);
    bool common_onActionForFix(bool state);
    void common_removeControlPoint(Stem& stem, int& newLen);
    void common_removeControlPoint(int& newLen);
    inline Vec3f genCurve3(const int& numOfSteps, const int& p, const int& s, const float& t, M_Vec3f& pnt)const;


    //static M_Vec2f normA;
    //static M_Vec2f normB;
    //static M_Vec2f normC;
    //static M_Vec2f normD;
public:
virtual bool onActionForFix(bool state)=0;

    enum{
        numOfSteps=1000
    };

    M_Vec3f& point;
    M_Int& index;
    M_Bool& fix;
    M_Int& nopfs; //nopfs number of points for segments

    Bezier3f(M_Vec3f& i_point, M_Int& i_index, M_Bool& i_border, M_Int& i_nopsf):
        point(i_point),
        index(i_index),
        fix(i_border),
        nopfs(i_nopsf)
    {}
    M_Vec3f line;
    M_Float flatHeight;
    M_Int heightIndex;
    M_Float segDistance;

    virtual void addControlPoint(int& newLen);
    virtual void removeControlPoint(int& newLen)=0;

    float getFirstTanCP();
    float getLastTanCP();

    void convertBorderToGroup();
    void convertGroupToFix();

    void matchBorderTangents(const Vec3f vec);
    void genSegDistance();
    void genCurve();
    void genFlatHeight();
    void genHeightForScaleOrShift();
    void genHeightIndex();

    void bezHouseKeep();
    void bezProcessFields();
    int getGroupIndices(int& fixSegBeginIndex);
    virtual void drag(Vec3f pt);

    Matrix matrix;
    Matrix imatrix;
};

class AnimCurve{
public:
    int numOfSteps;
    float selectedPeriod;
    M_Vec3f line;
    M_Float flatHeight;
    M_Int heightIndex;

    Vec3f firstPoint;
    bool processed;

    AnimCurve():
        numOfSteps(1000),
        selectedPeriod(0),
        processed(false)
        //positionInterpolator(0)
    {}

    void genFlatHeight();
    void genHeightIndex();
    void generate(float t, int& idx, float& ratio);
    void generate(Vec3f& coord, float t);
    void generate(Rotation& rotation, float t);


    void processFields(NurbsTrack* nurbsTrack, float selectedPeriod);
    //PositionInterpolatorPtr positionInterpolator;
};

class AnimCopy{
public:
    int stepsPerSecond;
    double selectFrom;
    double selectTo;
    double period;
    double pasteSelectedPeriod;
    double selectedPeriod;
    M_Node node;
    M_Int field;
    M_Int numOf;
    M_Float key;
    M_Rotation rot;
    M_Vec3f pos;
    M_Float scalar;
	M_Node m_node;
    Vec3f tranDiff;
    Rotation rotDiff;
    AnimCopy():
        stepsPerSecond(0),
        selectFrom(0),
        selectTo(0),
        period(0),
        pasteSelectedPeriod(0),
        selectedPeriod(0)
    {}
    void initCopySection(Anim* anim);
};

class EC2{
public:
    M_Vec2f& point;
    M_Float& tangent;

    Rotation& orientation;
    Vec3f& location;

    M_Int& index;
    M_LongDouble& a;
    M_LongDouble& b;

    EC2(M_Vec2f& i_point, M_Float& i_tangent, Rotation& i_orientation, Vec3f& i_location, M_Int& i_index, M_LongDouble& i_a, M_LongDouble& i_b):
        point(i_point),
        tangent(i_tangent),
        orientation(i_orientation),
        location(i_location),
        index(i_index),
        a(i_a),
        b(i_b)
    {}
    void ecHouseKeep();
    void genCurve(M_Vec3f& line);

    virtual void processFields()=0;
    void drag(Vec3f pt);

protected:
    Matrix matrix;
    Matrix imatrix;
};

class EC3{
public:
    Rotation& orientation;
    Vec3f& location;

    M_Vec2f& point;
    M_Float& tangent;


    M_Int& index;
    M_LongDouble& a;
    M_LongDouble& b;
    M_Float& relTangent;
    M_Float& yAngle;

    EC3(Rotation& i_orientation, Vec3f& i_location, M_Vec2f& i_point, M_Float& i_tangent, M_Float& i_relTangent, M_Float& i_yAngle, M_LongDouble& i_a, M_LongDouble& i_b, M_Int& i_index):
        orientation(i_orientation),
        location(i_location),
        point(i_point),
        tangent(i_tangent),


        relTangent(i_relTangent),
        yAngle(i_yAngle),

        a(i_a),
        b(i_b),
        index(i_index)
    {}
    void ecHouseKeep();
    virtual void genCurve(M_Vec3f& line)=0;

    virtual void processFields()=0;
    void drag(Vec3f pt);
};
class BezierLathe : public ALathe, public Bezier, public MField3, public VisualAid{
    public:

    int getType()const{return BEZIERLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}


    TMFVec2f point;
    TMFBool	fix;
    MFInt32 nopfs;

    BezierLathe():
        MField3(
            &point,
            &fix,
            &nopfs
        ),
        point(M_Vec2f(), *this),
        fix(M_Bool(), *this),
        nopfs(M_Int()),

        /////
        Bezier(point, index, fix, nopfs)
    {
        index.length=1;
    }
    M_Int index;
    //M_Float tangentDiff;

    float getDistance(int index=0x7fffffff)const;
    void getBandAndYDist(M_Int& bands, M_Float& bandDist, M_Float& yDistance)const;
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void processFields();
    void generate(M_Vec3f& coord, const int h);
    void calcClose(){};
    //void genTangentDiff();
    bool onActionForFix(bool state);
    void removeControlPoint(int& newLen);

    void houseKeep();
    void drag(Vec3f pt){Bezier::drag(pt);}
    void centerViewToControlPoint();
    using Bezier::drag;

    Vec2f calcSegmentXY(float t, M_Vec2f& pnt, int segDim, int bp);
    M_Int m_bp;
};
class ECLathe : public ALathe, public EC2, public MField2, public VisualAid{
    public:
class Point : public MFVec2f {
public:
    ECLathe& home;
    Point( const M_Vec2f& value, ECLathe& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par);
    void resize(int newLen);
};
class Tangent : public MFFloat {
public:
    ECLathe& home;
    Tangent( const M_Float& value, ECLathe& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        home.tangent.refValue()=par;
        home.processFields();
    }
};
    friend Point;
    friend Tangent;

    int getType()const{return ECLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Point point;
    Tangent tangent;

    ECLathe():
        MField2(
            &point,
            &tangent
        ),

        point(M_Vec2f(), *this),
        tangent(M_Float(), *this),
        /////
        EC2(point,tangent,orientation,location,index,a,b)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    M_Float distance;
    M_Float scaleDis;
    M_Float height;
    M_LongDouble a;
    M_LongDouble b;

int findSegment(float h){
    for (int i=distance.length; --i>=0;){
        if(h>=height[i] && h<=height[i+1]){
            return i;
        }
    }
    return -1;
}
    Vec3f location;
    Rotation orientation;
    float getDistance(int index=0x7fffffff)const;
    void getBandAndYDist(M_Int& bands, M_Float& bandDist, M_Float& yDistance)const;
    void calcClose(){};
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void drag(Vec3f pt){EC2::drag(pt);}

void processFields();
protected:
    Matrix matrix;
    Matrix imatrix;
};
class ECSyncBend : public ALathe, public EC2, public MField2, public VisualAid {
    public:
class Point : public MFVec2f {
public:
    ECSyncBend& home;
    Point( const M_Vec2f& value, ECSyncBend& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par);
    void resize(int newLen);
};
class Tangent : public MFFloat {
public:
    ECSyncBend& home;
    Tangent( const M_Float& value, ECSyncBend& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        home.tangent.refValue()=par;
        home.processFields();
    }
};
    friend Point;
    friend Tangent;

    int getType()const{return ECSYNCBEND;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Point point;
    Tangent tangent;

    ECSyncBend():
        MField2(
            &point,
            &tangent
        ),

        point(M_Vec2f(),*this),
        tangent(M_Float(),*this),
        //////

        orientation(Rotation(0,1,0,-PI*.5)),

        EC2(point,tangent,orientation,location,index,a,b)
    {

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    Rotation orientation;
    Vec3f location;

    M_Float distance;
    M_Float scaleDis;
    M_Float height;
    M_LongDouble a;
    M_LongDouble b;

int findSegment(float h){
    for (int i=distance.length; --i>=0;){
        if(h>=height[i] && h<=height[i+1]){
            return i;
        }
    }
    return -1;
}
    float getDistance(int index=0x7fffffff)const;
    void getBandAndYDist(M_Int& bands, M_Float& bandDist, M_Float& yDistance)const;
    void calcClose(){};
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);

void processFields();
void drag(Vec3f pt){EC2::drag(pt);}

protected:
    Matrix matrix;
    Matrix imatrix;
};


class ECScale : public Generator, public Node, public EC2, public MField12, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    ECScale& home;
    Orientation( const Rotation& value, ECScale& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par){
        home.orientation.refValue()=par;
        Vec3f location=home.location.refValue();
        home.matrix=rotationMatrix(par)*translationMatrix(location);
        home.imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    ECScale& home;
    Location( const Vec3f& value, ECScale& home):
        SFVec3f(value),
        home(home)
    {};

    void operator=(const Vec3f& par){
        home.location.refValue()=par;
        Rotation orientation=home.orientation.refValue();
        home.matrix=rotationMatrix(orientation)*translationMatrix(par);
        home.imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
class Point : public MFVec2f {
public:
    ECScale& home;
    Point( const M_Vec2f& value, ECScale& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par){
        M_Vec2f& point=home.point.refValue();
        point=par;
        setUpMatrices(home.m_ecurve, point, home.tangent, home.a, home.b);
    }
};
class Tangent : public MFFloat {
public:
    ECScale& home;
    Tangent( const M_Float& value, ECScale& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        M_Float& tangent=home.tangent.refValue();
        tangent=par;
        setUpMatrices(home.m_ecurve, home.point, tangent, home.a, home.b);
    }
};
    friend Location;
    friend Orientation;
    friend Point;
    friend Tangent;

    int getType()const{return ECSCALE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Location location;
    Orientation orientation;
    Point point;
    Tangent tangent;
    SFBool bNegX;
    SFBool bPosX;
    SFBool bNegZ;
    SFBool bPosZ;
    SFFloat negX;
    SFFloat posX;
    SFFloat negZ;
    SFFloat posZ;

    ECScale():
        MField12(
            &location,
            &orientation,
            &point,
            &tangent,
            &bNegX,
            &bPosX,
            &bNegZ,
            &bPosZ,
            &negX,
            &posX,
            &negZ,
            &posZ
        ),

        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec2f(),*this),
        tangent(M_Float(),*this),
        bNegX(false),
        bPosX(false),
        bNegZ(false),
        bPosZ(false),
        negX(1),
        posX(1),
        negZ(1),
        posZ(1),

        /////
        EC2(point,tangent,orientation.refValue(),location,index,a,b)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
void drag(Vec3f pt){EC2::drag(pt);}
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
M_ECurve m_ecurve;
    M_LongDouble a;
    M_LongDouble b;

int findSegment(float height){
    for (int i=point.length-1; --i>=0;){
        if(height>=point[i].y && height<=point[i+1].y){
            return i;
        }
    }
    return -1;
}
void processFields();
protected:
    Matrix matrix;
    Matrix imatrix;
};
class ECShift : public Generator, public Node, public EC2, public MField6, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    ECShift& home;
    Orientation( const Rotation& value, ECShift& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par){
        home.orientation.refValue()=par;
        Vec3f location=home.location.refValue();
        home.matrix=rotationMatrix(par)*translationMatrix(location);
        home.imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    ECShift& home;
    Location( const Vec3f& value, ECShift& home):
        SFVec3f(value),
        home(home)
    {};

    void operator=(const Vec3f& par){
        home.location.refValue()=par;
        Rotation orientation=home.orientation.refValue();
        home.matrix=rotationMatrix(orientation)*translationMatrix(par);
        home.imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
class Point : public MFVec2f {
public:
    ECShift& home;
    Point( const M_Vec2f& value, ECShift& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par){
        M_Vec2f& point=home.point.refValue();
        point=par;
        setUpMatrices(home.m_ecurve, point, home.tangent, home.a, home.b);
    }
};
class Tangent : public MFFloat {
public:
    ECShift& home;
    Tangent( const M_Float& value, ECShift& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        M_Float& tangent=home.tangent.refValue();
        tangent=par;
        setUpMatrices(home.m_ecurve, home.point, tangent, home.a, home.b);
    }
};
    friend Location;
    friend Orientation;
    friend Point;
    friend Tangent;
    int getType()const{return ECSHIFT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Location location;
    Orientation orientation;
    Point point;
    Tangent tangent;
    SFFloat negDecay;
    SFFloat posDecay;

    ECShift():
        MField6(
            &location,
            &orientation,
            &point,
            &tangent,
            &negDecay,
            &posDecay
        ),

        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec2f(),*this),
        tangent(M_Float(),*this),
        negDecay(1),
        posDecay(1),

        /////
        EC2(point,tangent,orientation.refValue(),location,index,a,b)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
M_ECurve m_ecurve;
    M_LongDouble a;
    M_LongDouble b;

int findSegment(float height){
    for (int i=point.length-1; --i>=0;){
        if(height>=point[i].y && height<=point[i+1].y){
            return i;
        }
    }
    return -1;
}
void processFields();
void drag(Vec3f pt){EC2::drag(pt);}
protected:
    Matrix matrix;
    Matrix imatrix;
};
class ECBend : public Generator, public Node, public EC3, public MField5, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    ECBend& home;
    Orientation( const Rotation& value, ECBend& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par){
        home.orientation.refValue()=par;
        Vec3f location=home.location.refValue();
        home.matrix=rotationMatrix(par)*translationMatrix(location);
        home.imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    ECBend& home;
    Location( const Vec3f& value, ECBend& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par){
        home.location.refValue()=par;
        Rotation orientation=home.orientation.refValue();
        home.matrix=rotationMatrix(orientation)*translationMatrix(par);
        home.imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
class Point : public MFVec2f {
public:
    ECBend& home;
    Point( const M_Vec2f& value, ECBend& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par){
        M_Vec2f& point=home.point.refValue();
        point=par;

        home.copyPointTangentToXZ();

        //setUpMatrices(home.m_ecurve, home.pointX, home.immediateTangent, home.a, home.b);
    }
};
class Tangent : public MFFloat {
public:
    ECBend& home;
    Tangent( const M_Float& value, ECBend& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        M_Float& tangent=home.tangent.refValue();
        tangent=par;

        home.copyPointTangentToXZ();

        //setUpMatrices(home.m_ecurve, home.pointX, home.immediateTangent, home.a, home.b);
    }
};
class YAngle : public MFFloat {
public:
    ECBend& home;
    YAngle( const M_Float& value, ECBend& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        M_Float& yAngle=home.yAngle.refValue();
        yAngle=par;

        home.copyPointTangentToXZ();

        //setUpMatrices(home.m_ecurve, home.pointX, home.immediateTangent, home.a, home.b);
    }
};

    friend Location;
    friend Orientation;
    friend Point;
    friend Tangent;
    int getType()const{return ECBEND;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Location location;
    Orientation orientation;
    Point point;
    Tangent tangent;
    YAngle yAngle;

    ECBend():
        MField5(
            &location,
            &orientation,
            &point,
            &tangent,
            &yAngle
        ),

        location(Vec3f(), *this),
        orientation(Rotation(), *this),
        point(M_Vec2f(), *this),
        tangent(M_Float(), *this),
        yAngle(M_Float(), *this),

        /////
        EC3( orientation.refValue(), location, point, tangent, relTangent, yAngle, a, b, index)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    M_Float relTangent;
    M_Float distance;
    M_Float scaleDis;
    M_Float height;

    M_ECurve m_ecurve;
    M_Matrix m_matrix;
    M_Matrix m_yMatrix;

    M_Vec3f relPoint;

    M_LongDouble a;
    M_LongDouble b;

virtual void genCurve(M_Vec3f& line);
void drag(Vec3f pt){EC3::drag(pt);}

int findSegment(float h){
    for (int i=distance.length; --i>=0;){
        if(h>=height[i] && h<=height[i+1]){
            return i;
        }
    }
    return -1;
}

void copyPointTangentToXZ();

float getYangleSum(int index){
    const M_Float& yAngle=this->yAngle;
    float yAngleSum=0;
    int i;
    for(i=0;i<=index;i++){
        yAngleSum+=yAngle[i];
    }
    return yAngleSum;
}
void processFields();
protected:
    Matrix matrix;
    Matrix imatrix;
};
class ECTwist : public Generator, public Node, public EC2, public MField4, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    ECTwist& home;
    Orientation( const Rotation& value, ECTwist& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par){
        home.orientation.refValue()=par;
        Vec3f location=home.location.refValue();
        home.matrix=rotationMatrix(par)*translationMatrix(location);
        home.imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    ECTwist& home;
    Location( const Vec3f& value, ECTwist& home):
        SFVec3f(value),
        home(home)
    {};

    void operator=(const Vec3f& par){
        home.location.refValue()=par;
        Rotation orientation=home.orientation.refValue();
        home.matrix=rotationMatrix(orientation)*translationMatrix(par);
        home.imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
class Point : public MFVec2f {
public:
    ECTwist& home;
    Point( const M_Vec2f& value, ECTwist& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par){
        M_Vec2f& point=home.point.refValue();
        point=par;
        setUpMatrices(home.m_ecurve, point, home.tangent, home.a, home.b);
    }
};
class Tangent : public MFFloat {
public:
    ECTwist& home;
    Tangent( const M_Float& value, ECTwist& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        M_Float& tangent=home.tangent.refValue();
        tangent=par;
        setUpMatrices(home.m_ecurve, home.point, tangent, home.a, home.b);
    }
};
    friend Location;
    friend Orientation;
    friend Point;
    friend Tangent;
    int getType()const{return ECTWIST;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Location location;
    Orientation orientation;
    Point point;
    Tangent tangent;

    ECTwist():
        MField4(
            &location,
            &orientation,
            &point,
            &tangent
        ),

        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec2f(),*this),
        tangent(M_Float(),*this),

        /////
        EC2(point,tangent,orientation.refValue(),location,index,a,b)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
M_ECurve m_ecurve;
    M_LongDouble a;
    M_LongDouble b;

int findSegment(float height){
    for (int i=point.length-1; --i>=0;){
        if(height>=point[i].y && height<=point[i+1].y){
            return i;
        }
    }
    return -1;
}
void processFields();
void drag(Vec3f pt){EC2::drag(pt);}
protected:
    Matrix matrix;
    Matrix imatrix;
};

class BranchJoin : public Build, public MField10{
    public:
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}



    SFRotation orientation;
    SFVec3f location;
    SFBool descendants;
    SFFloat decay;
    SFBool sawn;
    SFBool backward;
    SFInt32 numOf;
    PartChild part;
    Setup setup;
    SFSurface surface;

    BranchJoin():
        Build(setup),
        MField10(
            &orientation,
            &location,
            &descendants,
            &decay,
            &sawn,
            &backward,
            &numOf,
            &part,
            &setup,
            &surface
        ),
        orientation(Rotation()),
        location(Vec3f()),
        descendants(false),
        decay(0),
        sawn(false),
        backward(false),
        numOf(-1),
        part(M_Node()),
        setup(0, *this),
        surface(0)
    {
    }
/*
    M_Node& refGenerator(){
        SurfaceGenerator* tmp=dynamic_cast<SurfaceGenerator*>(surface.refValue().getRawPtr());
        if(tmp==0)return *(M_Node*)tmp;
        return tmp->generator.refValue();
    }
    */
    BranchJoin( SurfaceGeneratorPtr, NodePtr i_source, NodePtr i_part, Vec3f i_loc, Rotation i_brot, bool i_sawn=false, M_Node optTrnsfrm=M_Node(), int i_baseIndex=-1, int i_topIndex=-1);
    inline void genSurface(int &generatorLength, M_Vec3f& coord){
        SurfaceGenerator* surface=this->surface;
        M_Node& generator=surface->generator.refValue();
        for(int j=0; j<generatorLength;j++){
            dynamic_cast<Generator*>(generator[j].p)->generate(coord, 0);
        }
    }
    void processFields();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;

#ifdef SEAMLESS3DGUI
#endif
protected:
    int fromCoord;
    int toCoord;
    float peak;
    float tempTranslate;
};

class BranchJoinNegX : public BranchJoin {
public:
    friend BranchJoin;

    int getType()const{return BRANCHJOINNEGX;}
    BranchJoinNegX(){
    }
    BranchJoinNegX( SurfaceGeneratorPtr, NodePtr i_source, NodePtr i_part, Vec3f i_loc, Rotation i_brot, bool i_sawn=false, M_Node optTrnsfrm=M_Node(), int i_baseIndex=-1, int i_topIndex=-1);

    void genJoinCurve();
    void doBuild();
};
class BranchJoinPosX : public BranchJoin {
public:
    friend BranchJoin;
    int getType()const{return BRANCHJOINPOSX;}
    BranchJoinPosX(){
    }
    BranchJoinPosX( SurfaceGeneratorPtr, NodePtr i_source, NodePtr i_part, Vec3f i_loc, Rotation i_brot, bool i_sawn=false, M_Node optTrnsfrm=M_Node(), int i_baseIndex=-1, int i_topIndex=-1);

    void genJoinCurve();
    void doBuild();
};
class BranchJoinNegY : public BranchJoin {
public:
    friend BranchJoin;

    int getType()const{return BRANCHJOINNEGY;}
    BranchJoinNegY(){
    }
    BranchJoinNegY( SurfaceGeneratorPtr, NodePtr i_source, NodePtr i_part, Vec3f i_loc, Rotation i_brot, bool i_sawn=false, M_Node optTrnsfrm=M_Node(), int i_baseIndex=-1, int i_topIndex=-1);

    void genJoinCurve();
    void doBuild();
};
class BranchJoinPosY : public BranchJoin {
public:
    friend BranchJoin;

    int getType()const{return BRANCHJOINPOSY;}
    BranchJoinPosY(){
    }
    BranchJoinPosY(SurfaceGeneratorPtr, IndexedFaceSetPtr i_source, IndexedFaceSetPtr i_part, Vec3f i_loc, Rotation i_brot, bool i_sawn=false,M_Node optTrnsfrm=M_Node(), int i_baseIndex=-1, int i_topIndex=-1);
    void genJoinCurve();
    void doBuild();
};
class BranchJoinNegZ : public BranchJoin {
public:
    friend BranchJoin;

    int getType()const{return BRANCHJOINNEGZ;}
    BranchJoinNegZ(){
    }
    BranchJoinNegZ( SurfaceGeneratorPtr, NodePtr i_source, NodePtr i_part, Vec3f i_loc, Rotation i_brot, bool i_sawn=false, M_Node optTrnsfrm=M_Node(), int i_baseIndex=-1, int i_topIndex=-1);

    void genJoinCurve();
    void doBuild();
};
class BranchJoinPosZ : public BranchJoin {
public:
    friend BranchJoin;

    int getType()const{return BRANCHJOINPOSZ;}
    BranchJoinPosZ(){
    }
    BranchJoinPosZ( SurfaceGeneratorPtr, NodePtr i_source, NodePtr i_part, Vec3f i_loc, Rotation i_brot, bool i_sawn=false, M_Node optTrnsfrm=M_Node(), int i_baseIndex=-1, int i_topIndex=-1);

    void genJoinCurve();
    void doBuild();
};
class ProjectOntoTri : public Build, public MField11{
    public:
    int getType()const{return PROJECTONTOTRI;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFRotation orientation;
    SFVec3f location;
    SFBool descendants;
    SFFloat decay;
    SFBool sawn;
    SFBool backward;
    SFInt32 numOf;
    PartChild part;
    SFPart surface;
    SFNode feedback;
    Setup setup;

    ProjectOntoTri():
        Build(setup),
        MField11(
            &orientation,
            &location,
            &descendants,
            &decay,
            &sawn,
            &backward,
            &numOf,
            &part,
            &surface,
            &feedback,
            &setup
        ),
        orientation(Rotation()),
        location(Vec3f()),
        descendants(false),
        decay(0),
        sawn(false),
        backward(false),
        numOf(-1),
        part(M_Node()),
        surface(0),
        feedback(0),
        setup(0, *this)
    {
    }
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};

class SurfaceTransform : public Generator, public Node, public MField3 {
    public:
class Scale : public SFVec3f {
public:
    SurfaceTransform& home;

    Scale( const Vec3f& value, SurfaceTransform& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par){
        home.scale.refValue()=par;
        home.matrix.setTransform(home.translation, home.rotation, par);
    }
};
class Rotation2 : public SFRotation {
public:
    SurfaceTransform& home;

    Rotation2( const Rotation& value, SurfaceTransform& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par){
        home.rotation.refValue()=par;
        home.matrix.setTransform(home.translation, par, home.scale);
    }
};
class Translation : public SFVec3f {
public:
    SurfaceTransform& home;
    Translation( const Vec3f& value, SurfaceTransform& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par){
        home.translation.refValue()=par;
        home.matrix.setTransform(par, home.rotation, home.scale);
    }
};
    friend Scale;
    friend Rotation;
    friend Translation;
///////////////////////
    int getType()const{return SURFACETRANSFORM;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Scale scale;
    Rotation2 rotation;
    Translation translation;

    SurfaceTransform():
        MField3(
            &scale,
            &rotation,
            &translation
        ),

        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this)
    {}
    SurfaceTransform(Vec3f i_translation, Rotation i_rotation, Vec3f i_scale):
        MField3(
            &translation,
            &rotation,
            &scale
        ),

        scale(i_scale, *this),
        rotation(i_rotation, *this),
        translation(i_translation, *this)

    {
        matrix=scaleMatrix(i_scale);
    }
/*
    void set_scale(Vec3f par1){
        scale=par1;
        matrix.setTransform(translation.refValue(),rotation.refValue(),scale.refValue());
    }
    void set_rotation(Rotation par1){
        rotation=par1;
        matrix.setTransform(translation.refValue(),rotation.refValue(),scale.refValue());
    }
    void set_translation(Vec3f par1){
        translation=par1;
        matrix.setTransform(translation.refValue(),rotation.refValue(),scale.refValue());
    }
    Vec3f get_scale(){
        return scale.refValue();
    }
    Rotation get_rotation(){
        return rotation.refValue();
    }
    Vec3f get_translation(){
        return translation.refValue();
    }
*/
    void generate(M_Vec3f& coord, const int h);
    void processFields();
    private:
    Matrix matrix;
};
class Place : public Build, public MField12{
    public:
    int getType()const{return PLACE;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFVec3f scale;
    SFRotation rotation;
    SFInt32 tranVertex;
    SFVec3f multiply;
    SFBool alignX;
    SFBool alignY;
    SFBool alignZ;
    SFVec3f translation;
    SFBool descendants;
    SFBool staticTransform;
    PartChild part;
    Setup setup;

    Place():
        Build(setup),
        MField12(
            &scale,
            &rotation,
            &tranVertex,
            &multiply,
            &alignX,
            &alignY,
            &alignZ,
            &translation,
            &descendants,
            &staticTransform,
            &part,
            &setup
        ),
        scale(Vec3f(1,1,1)),
        rotation(Rotation()),
        tranVertex(0),
        multiply(Vec3f()),
        alignX(false),
        alignY(false),
        alignZ(false),
        translation(Vec3f()),
        descendants(true),
        staticTransform(true),
        part(M_Node()),
        setup(0, *this)
    {}
    void doBuild();
    void processFields();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
};

/*
class Place : public Build {
public:

    int getType()const{return PLACE;}
    Place(){
        scale=Vec3f(1,1,1);
        tranVertex=0;
        tranVertexB=0;
        mirrorX=false;
        mirrorY=false;
        mirrorZ=false;
        staticTransform=true;
    }
    Place(M_Node i_part, int indexA, Vec3f mulTranA);
    void doBuild();

    void getFields( String& str,int& i);
    String toString(bool newList=true);

    M_Node source;

    M_Node part;
    int tranVertex;
    Vec3f multiply;

    int tranVertexB;
    Vec3f multiplyB;

    bool mirrorX;
    bool mirrorY;
    bool mirrorZ;
    bool staticTransform;

    Vec3f translation;
    Rotation rotation;
    Vec3f scale;

#ifdef SEAMLESS3DGUI
    void genNodeListRecursion( int indent);
    void paintSceneTree(int indent, int& line);
    int winProc(void* hDlg,unsigned iMessage, unsigned wParam, unsigned lParam);
#endif
};
*/
class Browser{
public :
    static Scene* getWorld();
    static bool createVrmlFromFile(const String filename, M_Node& m_node);
    static M_Node createVrmlFromFile(const String filename, M_Color3f& backgroundColorList);
    static bool createSceneFromFile(const String filename);
//	static void createSceneFromString();

    static void print(char * str);
    static void print(int i);
    static void print(const String str);
};
void createSmlsFromString(M_Node& nodes, const String& str);
ScenePtr createSmlsFromString(const String& str);

class EvolveCoord : public Build, public EC2, public MField7, public VisualAid {
    public:
class Orientation : public SFRotation {
public:
    EvolveCoord& home;
    Orientation( const Rotation& value, EvolveCoord& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par){
        home.orientation.refValue()=par;
        Vec3f location=home.location.refValue();
        home.matrix=rotationMatrix(par)*translationMatrix(location);
        home.imatrix=translationMatrix(-location)*rotationMatrix(-Rotation(par));
    }
};
class Location : public SFVec3f {
public:
    EvolveCoord& home;
    Location( const Vec3f& value, EvolveCoord& home):
        SFVec3f(value),
        home(home)
    {};

    void operator=(const Vec3f& par){
        home.location.refValue()=par;
        Rotation orientation=home.orientation.refValue();
        home.matrix=rotationMatrix(orientation)*translationMatrix(par);
        home.imatrix=translationMatrix(-Vec3f(par))*rotationMatrix(-orientation);
    }
};
class Point : public MFVec2f {
public:
    EvolveCoord& home;
    Point( const M_Vec2f& value, EvolveCoord& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par){
        M_Vec2f& point=home.point.refValue();
        point=par;
        setUpMatrices(home.m_ecurve, point, home.tangent, home.a, home.b);
    }
};
class Tangent : public MFFloat {
public:
    EvolveCoord& home;
    Tangent( const M_Float& value, EvolveCoord& home):
        MFFloat(value),
        home(home)
    {};
    void operator=(const M_Float& par){
        M_Float& tangent=home.tangent.refValue();
        tangent=par;
        setUpMatrices(home.m_ecurve, home.point, tangent, home.a, home.b);
    }
};
    friend Location;
    friend Orientation;
    friend Point;
    friend Tangent;

    int getType()const{return EVOLVECOORD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Location location;
    Orientation orientation;
    Point point;
    Tangent tangent;
    PartChild source;
    PartChild part;
    Setup setup;

    EvolveCoord():
        Build(setup),
        MField7(
            &location,
            &orientation,
            &point,
            &tangent,
            &source,
            &part,
            &setup
        ),

        location(Vec3f(), *this),
        orientation(Rotation(0,0,1,0), *this),
        point(M_Vec2f(), *this),
        tangent(M_Float(), *this),
        source(M_Node()),
        part(M_Node()),
        setup(0, *this),
        /////
        EC2(point,tangent,orientation.refValue(),location,index,a,b)
    {

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
    void houseKeep();
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

M_ECurve m_ecurve;
    M_LongDouble a;
    M_LongDouble b;

int findSegment(float height){
    for (int i=point.length-1; --i>=0;){
        if(height>=point[i].y && height<=point[i+1].y){
            return i;
        }
    }
    return -1;
}

void processFields();

protected:
    Matrix matrix;
    Matrix imatrix;
};

class CenterJoint : public Build, public MField5, public VisualAid{
    public:
    int getType()const{return CENTERJOINT;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFInt32 fromVertex;
    SFInt32 numOf;
    SFVec3f translation;
    SFPart part;
    Setup setup;

    CenterJoint():
        Build(setup),
        MField5(
            &fromVertex,
            &numOf,
            &translation,
            &part,
            &setup
        ),

        fromVertex(0),
        numOf(0),
        translation(Vec3f()),
        part(0),
        setup(0, *this)
    {
    }
    void doBuild();
    void houseKeep();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
};

class CopyPart : public Build, public MField14{
    public:
    int getType()const{return COPYPART;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool mirrorX;
    SFBool mirrorY;
    SFBool mirrorZ;
    SFBool add;
    SFBool insideOut;
    SFBool staticTransform;
    SFBool copyTransform;
    SFBool resetTransform;
    SFVec3f scale;
    SFRotation rotation;
    SFVec3f translation;
    PartChild source;
    PartChild dest;
    Setup setup;

    CopyPart():
        Build(setup),
        MField14(
            &mirrorX,
            &mirrorY,
            &mirrorZ,
            &add,
            &insideOut,
            &staticTransform,
            &copyTransform,
            &resetTransform,
            &scale,
            &rotation,
            &translation,
            &source,
            &dest,
            &setup
        ),
        mirrorX(false),
        mirrorY(false),
        mirrorZ(false),
        add(false),
        insideOut(false),
        staticTransform(false),
        copyTransform(false),
        resetTransform(false),
        scale(Vec3f(1, 1, 1)),
        rotation(Rotation(0, 0, 1, 0)),
        translation(Vec3f(0, 0, 0)),
        source(M_Node()),
        dest(M_Node()),
        setup(0, *this),

        stemNurbsMirror(false)
    {
    }
    bool stemNurbsMirror;
    void doBuild();
    void addPart(Art* src,Art* dst, bool copyTriangles);
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
#ifdef SEAMLESS3DGUI
#endif
};

class DelPoly : public Build, public MField3{
    public:
    int getType()const{return DELPOLY;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}


    MFInt32 poly;
    SFPart part;
    Setup setup;

    DelPoly():
        Build(setup),
        MField3(
            &poly,
            &part,
            &setup
        ),

        poly(M_Int()),
        part(0),
        setup(0, *this)
    {
        index.length=1;
    }
    M_Int index;
    void doBuild();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
};

class JoinVertex : public Build , public MField10 {
    public:
    int getType()const{return JOINVERTEX;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
    M_Int index;
#endif


    SFBool move;
    SFBool srcColor;
    SFBool dstColor;
    MFInt32 srcCoord;
    MFInt32 dstCoord;
    SFBool in;
    SFFloat range;
    SFPart source;
    SFPart part;
    Setup setup;

    JoinVertex():
        Build(setup),
        MField10(
            &move,
            &srcColor,
            &dstColor,
            &srcCoord,
            &dstCoord,
            &in,
            &range,
            &source,
            &part,
            &setup
        ),

        move(false),
        srcColor(true),
        dstColor(true),
        srcCoord(M_Int()),
        dstCoord(M_Int()),
        in(false),
        range(.001),
        source(0),
        part(0),
        setup(0, *this)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
    void localIn(Art* part, int h, M_Int& fromVertex, M_Int& toVertex);
};


class JoinCloseVertex : public Build, public MField6{
    public:
    int getType()const{return JOINCLOSEVERTEX;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFBool srcColor;
    SFBool dstColor;
    SFFloat joinRange;
    PartChild source;
    PartChild part;
    Setup setup;

    JoinCloseVertex():
        Build(setup),
        MField6(
            &srcColor,
            &dstColor,
            &joinRange,
            &source,
            &part,
            &setup
        ),

        srcColor(true),
        dstColor(true),
        joinRange(.001),
        source(M_Node()),
        part(M_Node()),
        setup(0, *this)
    {
    }
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
//	void build2();
#ifdef SEAMLESS3DGUI
#endif
};
class TransferBCyl : public Build, public MField14, public VisualAid {
    public:
class Location : public SFVec3f {
public:
    TransferBCyl& home;
    Location( const Vec3f& par, TransferBCyl& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par){
        refValue()=par;
        home.in=true;
    }
};
class Orientation : public SFRotation{
public:
    TransferBCyl& home;
    Orientation( const Rotation& par, TransferBCyl& home):
        SFRotation(par),
        home(home)
    {};
    void operator=(const Rotation& par){
        refValue()=par;
        home.in=true;
    }
};
class Height : public SFFloat {
public:
    TransferBCyl& home;
    Height(const float& value, TransferBCyl& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par){
        refValue()=par;
        home.in=true;
    }
};
class Radius : public SFFloat {
public:
    TransferBCyl& home;
    Radius(const float& value, TransferBCyl& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par){
        refValue()=par;
        home.in=true;
    }
};
class IndexPosition : public SFBool{
public:
    TransferBCyl& home;
    IndexPosition(const bool& value, TransferBCyl& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par){
        refValue()=par;
        home.in=true;
    }
};

class Insert : public SFBool{
public:
    TransferBCyl& home;
    Insert(const bool& value, TransferBCyl& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par){
        refValue()=par;
        home.in=true;
    }
};

class GroupingHeight : public SFFloat {
public:
    TransferBCyl& home;
    GroupingHeight(const float& value, TransferBCyl& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par){
        refValue()=par;
        home.in=true;
    }
};
class NumOfGroups : public SFInt32{
public:
    TransferBCyl& home;
    NumOfGroups(const float& value, TransferBCyl& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const float& par){
        refValue()=par;
        home.in=true;
    }
};
class Group : public SFInt32{
public:
    TransferBCyl& home;
    Group(const int& value, TransferBCyl& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par){
        refValue()=par;
        home.in=true;
    }
};
class Vertex : public MFInt32 {
public:
    TransferBCyl& home;
    Vertex( const M_Int& par, TransferBCyl& home):
        MFInt32(par),
        home(home)
    {};
    void operator=(const M_Int& par){
        refValue()=par;
        home.in=true;
    }
};
class DestSource : public SFPart {
public:
    TransferBCyl& home;
    DestSource(const NodePtr& par, TransferBCyl& home):
        SFPart(par),
        home(home)
    {};
    Node* operator=(Node* par){
        refValue()=dynamic_cast<Art*>(par);
        home.in=true;
        return par;
    }
};

    int getType()const{return TRANSFERBCYL;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
    M_Int index;

    Location location;
    Orientation orientation;
    Height height;
    Radius radius;
    IndexPosition indexPosition;
    Insert insert;
    GroupingHeight groupingHeight;
    NumOfGroups numOfGroups;
    Group	group;
    Vertex	vertex;
    SFBool	in;
    DestSource source;
    DestSource dest;
    Setup setup;

    TransferBCyl():
        Build(setup),
        MField14(
            &location,
            &orientation,
            &height,
            &radius,
            &indexPosition,
            &insert,
            &groupingHeight,
            &numOfGroups,
            &group,
            &vertex,
            &in,
            &source,
            &dest,
            &setup
        ),

        location(Vec3f(), *this),
        orientation(Rotation(0,0,1,0), *this),
        height(.5, *this),
        radius(.25, *this),
        indexPosition(true, *this),
        insert(false, *this),
        groupingHeight(0, *this),
        numOfGroups(3, *this),
        group(-1, *this),
        vertex(M_Int(), *this),
        in(true),
        source(0, *this),
        dest(0, *this),
        setup(0, *this),

        index(0)
    {}
    void doBuild();
    void houseKeep();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;

private:
    static M_Int reOrderSrcLookup;
    static M_Int reOrderSrcLookup2;
    inline void transferVertexInsert(Art* source, Art* dest, M_Int& indices);
    inline void transferVertexAppend(Art* source, Art* dest, M_Int& indices);
    inline void commonTransferVertexByIndex(Art* source, Art* dest, int& oldSrcCoordLen, M_Vec3f& srcCoord, int& indicesLen);
    inline bool bubbleSortIndexByPosition(M_Int& vertex);
};
class MarkPart{
public:
    virtual int getPartOffset()=0;
    virtual void checkToPaintSceneTree()=0;
    virtual void resetCo(Art* part, int& co)=0;
};
class TransferOwnership : public Build, public MarkPart, public FieldGroupExtra, public MField6, public VisualAid {
    public:
    int getType()const{return TRANSFEROWNERSHIP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int index;
    M_Int* getIndexPtr(){return &index;}

    MFInt32 numOf;
    MFInt32 qty;
    SFBool beginSP;
    SFNode source;
    PartChild dest;
    Setup setup;

    TransferOwnership():
        Build(setup),
        MField6(
            &numOf,
            &qty,
            &beginSP,
            &source,
            &dest,
            &setup
        ),

        numOf(M_Int()),
        qty(M_Int()),
        beginSP(false),
        source(0),
        dest(M_Node()),
        setup(0, *this),

        ///////////
        index(0)
    {
    }
    void doBuild();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void incNumOfGroup(int& newLen);
    void decNumOfGroup(int& newLen);
    bool processGroup(int groupIndex, int command, int& newLen);
    int getPartOffset();
    void checkToPaintSceneTree();
    void resetCo(Art* part, int& co);
    Art* getSelectedPart(){
        if(dest.length){
            int i = index[0];
            if(i < dest.length){
                return dest[i];
            }
        }
        return 0;
    }
    void houseKeep();
};
class TransferVertex : public Build, public MField7{
    public:
    int getType()const{return TRANSFERVERTEX;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int index;
    M_Int* getIndexPtr(){return &index;}
#endif


    SFInt32 fromVertex;
    SFInt32 numOf;
    SFInt32 qty;
    SFBool beginSP;
    SFPart source;
    SFPart dest;
    Setup setup;

    TransferVertex():
        Build(setup),
        MField7(
            &fromVertex,
            &numOf,
            &qty,
            &beginSP,
            &source,
            &dest,
            &setup
        ),

        fromVertex(0),
        numOf(-1),
        qty(1),
        beginSP(false),
        source(0),
        dest(0),
        setup(0, *this),

        ///////////
        fromPoly(0),
        toPoly(0),
        index(0)

    {
    }
    int fromPoly;
    int toPoly;
    void doBuild();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
};
class SineTug : public Generator, public Node, public MField6, public VisualAid{
    public:
    int getType()const{return SINETUG;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFRotation orientation;
    MFVec3f location;
    MFFloat radius;
    MFFloat distance;
    MFFloat decay;
    MFFloat xScale;

    SineTug():
        MField6(
            &orientation,
            &location,
            &radius,
            &distance,
            &decay,
            &xScale
        ),

        orientation(M_Rotation()),
        location(M_Vec3f()),
        radius(M_Float()),
        distance(M_Float()),
        decay(M_Float()),
        xScale(M_Float())
    {
        index.length=1;
    }
    M_Int index;
    void generate(M_Vec3f& coord, const int h);
    void processFields();
    void drag(Vec3f pt);
    void houseKeep();
};
class SimulSineTug : public Generator, public Node, public MField4, public VisualAid{
    public:
    int getType()const{return SIMULSINETUG;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    SFVec3f location;
    MFRotation orientation;
    MFFloat distance;
    SFFloat radius;

    SimulSineTug():
        MField4(
            &location,
            &orientation,
            &distance,
            &radius
        ),

        location(Vec3f()),
        orientation(M_Rotation()),
        distance(M_Float()),
        radius(0)
    {
        index.length=1;
    }
    M_Int index;
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
};

/*
class SineTug : public Generator {
public:
    M_Vec3f location;
    M_Float radius;
    M_Float distance;
    M_Float xScale;
    M_Rotation orientation;


    int getType()const{return SINETUG;}
    SineTug(){
#ifdef SEAMLESS3DGUI
        index=0;
#endif
    }
    SineTug(Vec3f _location, float _radius, float _distance, Rotation _orientation=Rotation()){
        location+=_location;
        radius+=_radius;
        distance+=_distance;
        orientation+=_orientation;
    }

    String toString(bool newList=true);
    void getFields(String& str, int& i);
    void generate(IndexedFaceSetPtr& ifs);
#ifdef SEAMLESS3DGUI
    void genNodeListRecursion( int indent);
    int winProc(void* hDlg,unsigned iMessage, unsigned wParam, unsigned lParam);
    int index;
#endif
};
*/
class GenPoly : public Build, public MField3 {
    public:
    int getType()const{return GENPOLY;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFInt32 vertex;
    SFPart part;
    Setup setup;

    GenPoly():
        Build(setup),
        MField3(
            &vertex,
            &part,
            &setup
        ),
        vertex(M_Int()),
        part(0),
        setup(0, *this),
        independentColor(false)
    {
        index.length=1;
    }
    bool independentColor; //could be a field if a use if found for it as a build node

    M_Int index;
    M_Triangle triangle;
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};

class MorphPart : public Node, public MField6 {
public:
///////////////////////////
    int getType()const{return MORPHPART;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFFloat set_fraction;
    MFVec3f value_changed;
    MFFloat key;
    MFFloat weight;
    SFPart part;
    MFNode bank;

    MorphPart():
        MField6(
            &set_fraction,
            &value_changed,
            &key,
            &weight,
            &part,
            &bank
        ),

        set_fraction(0),
        value_changed(M_Vec3f()),
        key(M_Float()),
        weight(M_Float()),
        part(0),
        bank(M_Node())
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};

Node* getNode2(const String& str, int& i);
void copyIfsToPart(Art* &part, IndexedFaceSetPtr &ifs);


class Morpher : public Node, public MField2 {
public:
class Weight : public SFFloat {
public:
    Morpher& home;

    Weight(const float& value, Morpher& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};
///////////////////////////
    int getType()const{return MORPHER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Weight weight;
    MFVec3f displacement;

    Morpher():
        MField2(
            &weight,
            &displacement
        ),

        weight(0, *this),
        displacement(M_Vec3f()),

        smls(0),
        parent(0)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    Seamless* smls;
    Art* parent;
};
class Movie : public Node, public MField1{
public:
    int getType()const{return MOVIE;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}

    MFNode clip;
    Movie():
        MField1(
            &clip
        ),
        clip(M_Node())
    {}
};

class Video : public Node, public MField3{
public:
class Open : public SFBool {
public:
    Video& home;
    Open(const bool& value, Video& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return VIDEO;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}

    Open open;
    SFString file;
    MFInt32 frameRef;
    Video():
        MField3(
            &open,
            &file,
            &frameRef
        ),
        open(false, *this),
        file(String()),
        frameRef(M_Int()),

////////////
        frame(0),
		numOfDigits(0)
    {}
    int frame;
    String pcmFilename;
    static bool audioTimeMode;
	String path;
	String filenameOnly;
	int numOfDigits;

    void getFilename(String& filename, int frame, String& path, int numOfDigits);

    void repeatFrame(int frame, int numOf);
    void deleteFrame(int frameIndex, int numOf);
    void reduceFrames(int frameIndex, int toFrame, int keepCaptureForEvery);
    void outputFrames(int contigFrame, const int numOfFrames);
    void mixLayers();
    void slowFrames(int frameIndex, int numOf);
	void setPath();
	void processFields();
};

class Audio : public Node, public MField8{
public:
class Open : public SFBool {
public:
    Audio& home;
    Open(const bool& value, Audio& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return AUDIO;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}

#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Open open;
    SFString file;

	SFBool mute;
	SFFloat delay;
	SFBool videoSync;
	MFVec2f	lPoint;
	MFVec2f	rPoint;

    MFInt32 frameRef;

    Audio():
        MField8(
            &open,
            &file,
			&mute,
			&delay,
			&videoSync,
			&lPoint,
			&rPoint,
            &frameRef
        ),
        open(false, *this),
        file(String()),
		mute(false),
		delay(0),
		videoSync(true),
		lPoint(M_Vec2f()),
		rPoint(M_Vec2f()),
        frameRef(M_Int()),

////////////
        frame(0),
        audioTimeMode(false)
	{
#ifdef SEAMLESS3DGUI
        index.length=4;
#endif
    }
    M_Int index;
    int frame;
    String pcmFilename;
    bool audioTimeMode;
	String path;

	void openWaveFile(const char* filename);
};

class Anim : public Node, public MField18{
public:
class Slider : public SFFloat {
public:
    Anim& home;

    Slider(const float& value, Anim& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};

class Play : public SFBool {
public:
    Anim& home;
    Play(const bool& value, Anim& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class Pause : public SFBool {
public:
    Anim& home;
    Pause(const bool& value, Anim& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
/*
class CapturePose : public SFBool {
public:
    Anim& home;
    CapturePose(const bool& value, Anim& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
*/
class StepsPerSecond : public SFInt32{
public:
    Anim& home;

    StepsPerSecond(const int& value, Anim& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};
class CopyPose : public SFBool {
public:
    Anim& home;
    CopyPose(const bool& value, Anim& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class PastePose : public SFBool {
public:
    Anim& home;
    PastePose(const bool& value, Anim& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class DeletePose : public SFBool {
public:
    Anim& home;
    DeletePose(const bool& value, Anim& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class OnFrame : public SFNode {
public:
    Anim& home;
    OnFrame( const NodePtr& par, Anim& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par1);
};


    int getType()const{return ANIM;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}

    Slider slider;
    SFInt32 timeLine;
    SFInt32 animSelect;
    Play play;
    Pause pause;
    SFBool loop;
    SFTime zoom;
    SFTime period;
    StepsPerSecond stepsPerSecond;
    SFFloat speed;
//	CapturePose capture;
    SFFloat playFrom;
    CopyPose copy;
    PastePose paste;
    DeletePose remove;
    SFNode video;
    MFNode soundTrack;
    MFNode children;
    OnFrame onFrame;

    Anim():
        MField18(
            &slider,
            &timeLine,
            &animSelect,
            &play,
            &pause,
            &loop,
            &zoom,
            &period,
            &stepsPerSecond,
            &speed,
            //&capture,
            &playFrom,
            &copy,
            &paste,
            &remove,
            &video,
            &soundTrack,
            &children,
            &onFrame
        ),

        slider(0, *this),
        timeLine(0),
        animSelect(0),
        play(false, *this),
        pause(true, *this),
        loop(false),
        zoom(4.0),
        period(1),
        stepsPerSecond(25, *this),
        speed(1),
        //capture(false, *this),
        playFrom(0),
        copy(false, *this),
        paste(false, *this),
        remove(false, *this),
        video(0),
        soundTrack(M_Node()),
        children(M_Node()),
        onFrame(0, *this),
////////////////
        startTime(0),
        update(false),
        copySlider(0),
        selectFrom(0),
        selectTo(0),
        displayFrom(0),
		animZoom(1)
    {
    }
	double animZoom;

    bool update;
    static bool autoCapture;
    float copySlider;
    double startTime;
    double selectFrom;
    double selectTo;
    double displayFrom;
    void renderNodeFirst();
    void decodeVoid(const String& str, int& i, int& bracketCntr);
    bool connectNode(Node* node, Field* field, String& name, SamplePose* samplePose, const int commonNameCode);
    bool connectPartToCPMorpher(Art* part, String& name, SamplePose* samplePose);
    CPMorpher* connectPartToCPMorpher(Art* part);
    OrientationInterpolator* connectNodeToOrientationInterpolator(Node* node, Field* field, const int commonNameCode);
    PositionInterpolator* connectNodeToPositionInterpolator(Node* node, Field* field, const int commonNameCode);
    ScalarInterpolator* connectNodeToScalarInterpolator(Node* node, Field* field, const int commonNameCode);
	CameraSelector* connectNodeToCameraSelector(Node* node, Field* field, const int commonNameCode);
    PosCompensator* connectPartToPosCompensator(Art* part);
	NikInterpolator* connectPartToNikInterpolator(Art* part);
    void quantizeSlider();

    //void Anim::toString(String& str, bool newList);
    //String Anim::toXMLString(String& str, bool newList, String containerField);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
    void setMorphersForTransformDrag(Seamless* smls);
    void captureMorph(Seamless* smls);
	void captureMorph2(const int k, Art* part, Matrix& imatrix, int& channel, CPMorpher*& cpMorpher, M_<CPMorpher*>& morpherList, const M_ControlPoint& m_controlPoint);
    void captureMorphLoop(Art* part, Matrix iCatMatrix, M_<CPMorpher*>& morpherList);
	//void captureNiki(Seamless* smls);
    void captureMultiplePartPose(Art* selNode);
    void captureSinglePartPose(Art* part, Field* field);
	void captureCameraLive(Viewpoint* view); //much like captureSinglePartPose
    void removeCompensation(Art* part);
    void captureViewpoint(Viewpoint* viewpoint);
    void resetMorphers(M_Art& m_part);
    void plotRecordNotch(AnimTimeLine* tl, Node* selNode);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, Art* part, const float clickPos);
    void saveBvh(const char* filename);


    void capturePolyMorph(Seamless* smls);
    void capturePolyMorph2(const int k, Art* part, Matrix& imatrix, int& channel, CPMorpher*& cpMorpher, M_<CPMorpher*>& morpherList, M_Vec3f& coord);
    void capturePolyMorphLoop(Art* part, Matrix iCatMatrix, M_<CPMorpher*>& morpherList);
    void deleteMorpherChannel();
    void compileMorph(Seamless* smls);
    void transformInterpolators(const Art* part);
    void prePasteProcessTrack();
    void pasteSection();
    void copyStraightForCurveFirst();
    void copySection();
    void deleteSection();
    void scaleTime(double add);
    OrientationInterpolator* getOrientationInterpolator(const Node* selNode, const Field* field);
    PositionInterpolator* getPositionInterpolator(const Node* selNode, const Field* field);
    ScalarInterpolator* getScalarInterpolator(const Node* selNode, const Field* field);
	CameraSelector* getCameraSelector(const Node* selNode, const Field* field);
    PosCompensator* getPosCompensator(const Node* selNode);
	NikInterpolator* getNikInterpolator(const Node* selNode);
    CPMorpher* getCPMorpher(const Node* selNode);
    void pastePositionInterpolator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode);
    void pasteOrientationInterpolator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode);
    void pasteScalarInterpolator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode);
	void pasteCameraSelector(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode);
    void pastePosCompensator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Art* part);
    void pasteCPMorpher(int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, int& numOfIdx, Art* part, Seamless* selsmls);
	void pasteNikInterpolator(int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, int& numOfIdx, Art* part, Seamless* selsmls);
    //void insertTimeForMissingMirrorNode(Node* node, Art* part, int fieldIdx, bool beginEdge);
    void updateDisplayFrom();
    void setUpSoundPlay();
    void redirectCPMorpherIndicesToContigList(Seamless* smls);
    void deleteNonExistingMorpherPoints(Seamless* smls);
    void redirectContigListBackToCPMorpherIndices(Seamless* smls, M_<CPContigElement>& contigLookUpList);
    void morphAllParts();
};

class TimeSensor : public Node, public MField11{
public:
class Loop : public SFBool {
public:
    Loop(const bool& value):
        SFBool(value)
    {};
    void operator=(const bool& par);
};
class OnFractionChanged : public SFNode {
public:
    TimeSensor& home;
    OnFractionChanged( const NodePtr& par, TimeSensor& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par1);
};


    int getType()const{return TIMESENSOR;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}

    SFTime cycleInterval;
    SFBool enabled;
    Loop loop;
    SFTime startTime;
    SFTime stopTime;
    SFTime cycleTime;
    SFFloat fraction_changed;
    SFBool isActive;
    SFTime time;
    MFNode children;
    OnFractionChanged on_fraction_changed;

    TimeSensor():
        MField11(
            &cycleInterval,
            &enabled,
            &loop,
            &startTime,
            &stopTime,
            &cycleTime,
            &fraction_changed,
            &isActive,
            &time,
            &children,
            &on_fraction_changed
        ),
        cycleInterval(1),
        enabled(true),
        loop(false),
        startTime(0),
        stopTime(0),
        cycleTime(0),
        fraction_changed(0),
        isActive(false),
        time(0),
        children(M_Node()),
        on_fraction_changed(0, *this)

    {
    }
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void decodeVoid(const String& str, int& i, int& bracketCntr);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
};
class SamplePose {
public:
    virtual void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx)=0;
    virtual void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime)=0;
    virtual void insertTime(double selectFrom, double selectTo, bool beginEdge)=0;
    virtual void scaleTime(double add, double period)=0;
    virtual void take(float sliderconnectNode, Node* node, Field* field)=0;
    virtual void deleteSample(float slider)=0; //only used for viewpoint?
    virtual void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field)=0;
    virtual bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos)=0;
    virtual void setKeyDragPos(int& idx, const float newTime)=0;
    virtual void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function)=0;
};

class PositionInterpolator : public Node, public SamplePose, public MField4 {
    public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    PositionInterpolator* home;

    Set_fraction(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return POSITIONINTERPOLATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    MFFloat key;
    MFVec3f keyValue;
    SFVec3f value_changed;

    PositionInterpolator():
        MField4(
            &set_fraction,
            &key,
            &keyValue,
            &value_changed
        ),

        set_fraction(0),
        key(M_Float()),
        keyValue(M_Vec3f()),
        value_changed(Vec3f())
    {
///////////////////////////
        set_fraction.home=this;
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    M_Int index;
    void processFields();
    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field);
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);
};

class NurbsTrack : public Node, public SamplePose, public MField5, public VisualAid{
    public:
    void do_set_fraction();
class Frame : public SFFloat {
public:
    NurbsTrack& home;

    Frame(const float& value, NurbsTrack& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par){
        raw=par;
        home.do_set_fraction();
    }
};

class KeyValue : public MFVec3f {
public:
    NurbsTrack& home;
    KeyValue( const M_Vec3f& par, NurbsTrack& home):
        MFVec3f(par),
        home(home)
    {};
    void operator=(const M_Vec3f& par);
};

///////////////////////////
    int getType()const{return NURBSTRACK;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Frame frame;
    MFFloat key;
    KeyValue keyValue;
    MFInt32 dim;
    SFVec3f pos;

    NurbsTrack():
        MField5(
            &frame,
            &key,
            &keyValue,
            &dim,
            &pos
        ),

        frame(0, *this),
        key(M_Float()),
        keyValue(M_Vec3f(), *this),
        dim(M_Int()),
        pos(Vec3f()),
////////////
        order(3),
        degree(2)
    {
///////////////////////////
#ifdef SEAMLESS3DGUI
        index.length = 3;
#endif
    }
    M_Int index;
    M_Float weight;
    M_Float knot;
    M_Int keyValueRedirect;
    M_Int knotRedirect;
    int order;
    int degree;
    void processFields();
    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Art* part, int keyIndex, int keyValueIndex, int numOf);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field);
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);

    void insertControlPoint();
    void insertControlPoint(int atIndex, float ratio);
    void removeControlPoint(int atIndex);

    void houseKeep();
    void centerViewToControlPoint();
	bool isOverPoint();
    void drag(Vec3f pt);
    void genCurve(M_Vec3f& line, const int steps);

    float minT(const int offset){
        return knot[offset + degree];
    }
    float maxT(const int endOfKnotSeg){
        return knot[endOfKnotSeg - degree];
    }
};

class OrientationInterpolator : public Node, public SamplePose, public MField4 {
    public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    OrientationInterpolator* home;

    Set_fraction(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return ORIENTATIONINTERPOLATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];};
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;};
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    MFFloat key;
    MFRotation keyValue;
    SFRotation value_changed;

    OrientationInterpolator():
        MField4(
            &set_fraction,
            &key,
            &keyValue,
            &value_changed
        ),

        set_fraction(0),
        key(M_Float()),
        keyValue(M_Rotation()),
        value_changed(Rotation())
    {
///////////////////////////
        set_fraction.home=this;

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
    void processFields();
    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field);
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);
#endif
};

class ScalarInterpolator : public Node, public SamplePose, public MField4 {
    public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    ScalarInterpolator* home;

    Set_fraction(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return SCALARINTERPOLATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    MFFloat key;
    MFFloat keyValue;
    SFFloat value_changed;

    ScalarInterpolator():
        MField4(
            &set_fraction,
            &key,
            &keyValue,
            &value_changed
        ),

        set_fraction(0),
        key(M_Float()),
        keyValue(M_Float()),
        value_changed(0)
    {
///////////////////////////
        set_fraction.home=this;

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    M_Int index;
    void processFields();
    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field);
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);
};

class PosCompensator : public Node, public SamplePose, public MField2{
    public:
    void do_set_fraction();
class Frac : public SFFloat {
public:
    PosCompensator& home;

    Frac(const float& value, PosCompensator& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};
///////////////////////////
    int getType()const{return POSCOMPENSATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFVec2f moment;
    MFVec3f relPos;

    PosCompensator():
        MField2(
            &moment,
            &relPos
        ),

        moment(M_Vec2f()),
        relPos(M_Vec3f()),
        momentIdx(0),

        pelvis(0),
        ankle(0),
        frac(0)
    {
///////////////////////////
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    M_Int index;
    int momentIdx;
    float frac;
    Art* pelvis;
    Art* ankle;
    void comp(float v, Art& ankle);
    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Art* part, int keyIndex, int keyValueIndex, int numOf);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field);
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);
    static bool rightSideDrag;
};

class NikInterpolator : public Node, public SamplePose, public MField7, public VisualAid{
    public:
class SelectShort : public SFBool{
public:
    NikInterpolator& home;
    SelectShort(const bool& par, NikInterpolator& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class CalcLongOnly : public SFBool{
public:
    NikInterpolator& home;
    CalcLongOnly(const bool& par, NikInterpolator& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class SnapTwin : public SFBool{
public:
    NikInterpolator& home;
    SnapTwin(const bool& par, NikInterpolator& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

///////////////////////////
    int getType()const{return NIKINTERPOLATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

	SelectShort selectShort;
	CalcLongOnly calcLongOnly;
	SnapTwin snapTwin;
    MFFloat key;
    MFVec3f keyValue;
	MFFloat weight;
	MFFloat knot;

	NikInterpolator():
		MField7(
			&selectShort,
			&calcLongOnly,
			&snapTwin,
			&key,
			&keyValue,
			&weight,
			&knot
		),

		selectShort(false, *this),
		calcLongOnly(false, *this),
		snapTwin(false, *this),
        key(M_Float()),
		keyValue(M_Vec3f()),
		weight(M_Float()),
		knot(M_Float()),

		/////////
		focusOffset(0),
		degree(2),
		selPointIdx(0),
		smls(0)
    {
        index.length = 4;
    }
    M_Int index;
	int focusOffset; //0 == long 1 = short
	int degree;

	float uMin(){//min keyword allready used
        return knot.refValue()[degree];
    }
	float uMax()const{
		return knot.refValue()[knot.length - degree];
	}

	void calc(const float v, Art& part);
	void calcIK(Seamless* smls, Art* selHip);

    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field);
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);

	void take(float slider, const Vec3f& diff, const int channel, const int numOfChannels, const int focusOffset);
	void setIndex(float slider, const int channel, const int numOfChannels);
	void do_set_fraction(const float frac);
	bool isOverPoint();
	void drag(Vec3f pt);

	void houseKeep();
	void processFields();

	bool calkArmPoints(double& amount, M_Art& m_art, float& prevDistDiff2, double aInc, Vec3f fromVector);

	int getSampleLength() const;
	int getNurbLineLength() const;

	M_Vec3f interpolated;

	M_Vec3f longPoint;
	M_Vec3f shortPoint;
	M_Vec3f resultPoint;

	M_Art partSequence;

	int selPointIdx;

	Seamless* smls;

};

class ColorInterpolator : public Node, public MField4 {
    public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    ColorInterpolator* home;

    Set_fraction(const float&value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return COLORINTERPOLATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    MFFloat key;
    MFColor keyValue;
    SFColor value_changed;

    ColorInterpolator():
        MField4(
            &set_fraction,
            &key,
            &keyValue,
            &value_changed
        ),

        set_fraction(0),
        key(M_Float()),
        keyValue(M_Color3f()),
        value_changed(Color3f(1,1,1))
    {
///////////////////////////
        set_fraction.home=this;

        index.length=1;
    }
    M_Int index;
    void processFields();
};

class CoordinateInterpolator : public Node, public MField4 {
    public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    CoordinateInterpolator* home;

    Set_fraction(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return COORDINATEINTERPOLATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    MFFloat key;
    MFVec3f keyValue;
    MFVec3f value_changed;

    CoordinateInterpolator():
        MField4(
            &set_fraction,
            &key,
            &keyValue,
            &value_changed
        ),

        set_fraction(0),
        key(M_Float()),
        keyValue(M_Vec3f()),
        value_changed(M_Vec3f())
    {
///////////////////////////
        set_fraction.home=this;

#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
    M_Vec3f result;
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class NormalInterpolator : public Node, public MField4 {
    public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    NormalInterpolator* home;

    Set_fraction(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return NORMALINTERPOLATOR;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    MFFloat key;
    MFVec3f keyValue;
    MFVec3f value_changed;

    NormalInterpolator():
        MField4(
            &set_fraction,
            &key,
            &keyValue,
            &value_changed
        ),

        set_fraction(0),
        key(M_Float()),
        keyValue(M_Vec3f()),
        value_changed(M_Vec3f())
    {
///////////////////////////
        set_fraction.home=this;

#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
    M_Vec3f result;
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class ModelSequence : public Node, public MField3{
    public:
    void do_frame();
class Frame : public SFFloat {
public:
    ModelSequence& home;

    Frame(const float& value, ModelSequence& home):
        SFFloat(value),
            home(home)
    {};
    void operator=(const float& par){
        raw=par;
        home.do_frame();
    }
};
class MFModel : public MFNode{
public:
    ModelSequence& home;
    MFModel(const M_Node& value, ModelSequence& home):
        MFNode(value),
        home(home)
    {};
    void operator=(const NodePtr& par);
    void operator=(const M_Node& par);
    void resize(int newLen);
};

///////////////////////////
    int getType()const{return MODELSEQUENCE;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Frame frame;
    MFFloat key;
    MFModel model;

    ModelSequence():
        MField3(
            &frame,
            &key,
            &model
        ),

        frame(0, *this),
        key(M_Float()),
        model(M_Node(), *this),

        parent(0)
    {
///////////////////////////

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    ModelMorpher* parent;
};
class ModelMorpher : public Node, public PovRay, public MField1{
    public:
///////////////////////////
    int getType()const{return MODELMORPHER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    TMFNode sequence;

    ModelMorpher():
        MField1(
            &sequence
        ),
        sequence(M_Node(), *this),

        smls(new Seamless),

        justLoaded(true)
    {
///////////////////////////

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
    M_Vec3f result;
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
void updateBuild();
void processForOutput(M_Node& parentChild, int& n, Script* gs);
void renderNode(Matrix parentMatrix, Vec3f parentColScale);
void processFields();
void senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor);
void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);

    bool justLoaded;
    SeamlessPtr smls;
};
class WorldInfo : public Node, public MField5{
    public:
class NetSave : public SFBool {
public:
    WorldInfo& home;

    NetSave(const bool& par, WorldInfo& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return WORLDINFO;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFString title;
    SFString author;
    SFString website;
    MFString info;
    NetSave netSave;

    WorldInfo():
        MField5(
            &title,
            &author,
            &website,
            &info,
            &netSave
        ),

        title(String()),
        author(String()),
        website(String()),
        info(M_String()),
        netSave(false, *this)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class TouchSensor : public Node, public MField9 {
    public:
class OnTouchTime : public SFNode {
public:
    TouchSensor& home;
    OnTouchTime( const NodePtr& par, TouchSensor& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par1);
};

    int getType()const{return TOUCHSENSOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFString description;
    SFBool enabled;
    SFVec3f hitNormal_changed;
    SFVec3f hitPoint_changed;
    SFVec2f hitTexCoord_changed;
    SFBool isActive;
    SFBool isOver;
    SFTime touchTime;
    OnTouchTime onTouchTime;

    TouchSensor():
        MField9(
            &description,
            &enabled,
            &hitNormal_changed,
            &hitPoint_changed,
            &hitTexCoord_changed,
            &isActive,
            &isOver,
            &touchTime,
            &onTouchTime
        ),

        description(String()),
        enabled(true),
        hitNormal_changed(Vec3f()),
        hitPoint_changed(Vec3f()),
        hitTexCoord_changed(Vec2f()),
        isActive(false),
        isOver(false),
        touchTime(0),
        onTouchTime(0, *this),
/////
        hit(false)
    {
    }
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void decodeVoid(const String& str, int& i, int& bracketCntr);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
    bool hit;
    void sendClickToServer();
};

class Collision : public AGroup, public PovRay, public MField8 {
public:
    int getType()const{return COLLISION;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{return 0;}
    void setFieldGroupIndex(const int &, const int &){}
    M_Int* getIndexPtr(){return nullPtr;}


    MFNode children;
    SFBool collide;
    SFNode proxy;
    SFTime collideTime;

    Collision():
        MField8(
            &addChildren,
            &removeChildren,
            &children,
            &collide,
            &bboxCenter,
            &bboxSize,
            &proxy,
            &collideTime
        ),

        collide(true),
        proxy(0),
        collideTime(0),
        children(M_Node()),

        AGroup(children)
    {
    }
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
};
class CCPath : public Node, public MField9 {
    public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    CCPath* home;

    Set_fraction(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return CCPATH;}
    int getFLen()const{return size();};
        Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    SFFloat fraction_changed;
    SFVec3f location;
    SFRotation orientation;
    MFFloat curve;
    MFFloat distance;
    MFFloat zAngle;
    SFRotation rot_changed;
    SFVec3f pos_changed;

    CCPath():
        MField9(
            &set_fraction,
            &fraction_changed,
            &location,
            &orientation,
            &curve,
            &distance,
            &zAngle,
            &rot_changed,
            &pos_changed
        ),

        set_fraction(0),
        fraction_changed(0),
        location(Vec3f()),
        orientation(Rotation()),
        curve(M_Float()),
        distance(M_Float()),
        zAngle(M_Float()),
        rot_changed(Rotation()),
        pos_changed(Vec3f())
    {
        set_fraction.home=this;
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
float getDistance(int index)const;
float getZangleSum(int index);
void bend(Vec3f& position, bool& firstCalcDone, Rotation& accumCurveAngle, float beginHeight,float endHeight,float angle,float axis);
};
class BspTree : public Node, public MField4 {
    public:
    int getType()const{return BSPTREE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFRotation plane;
    SFNode front;
    SFNode overlap;
    SFNode back;

    BspTree():
        MField4(
            &plane,
            &front,
            &overlap,
            &back
        ),

        plane(Rotation()),
        front(0),
        overlap(0),
        back(0)
    {
    }
};
class Inline : public Node, public MField3 {
    public:
    int getType()const{return INLINE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFString url;
    SFVec3f bboxCenter;
    SFVec3f bboxSize;

    Inline():
        MField3(
            &url,
            &bboxCenter,
            &bboxSize
        ),
        url(M_String()),
        bboxCenter(Vec3f()),
        bboxSize(Vec3f(-1,-1,-1))
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    M_Node children;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};
class Fog : public Node, public MField5 {
    public:
    class Color : public SFColor{
    public:
        bool& renderProcessed;

        Color(const Color3f& value, bool& renderProcessed):
            SFColor(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const Color3f& par){
            this->refValue()=par;
            renderProcessed=false;
        }
    };
    class VisibilityRange : public SFFloat {
    public:
        bool& renderProcessed;

        VisibilityRange(const float& value, bool& renderProcessed):
            SFFloat(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const float& par){
            this->refValue()=par;
            renderProcessed=false;
        }
    };
///////////////////////////

    int getType()const{return FOG;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Color color;
    SFString fogType;
    VisibilityRange visibilityRange;
    SFBool set_bind;
    SFBool isBound;

    Fog():
        MField5(
            &color,
            &fogType,
            &visibilityRange,
            &set_bind,
            &isBound
        ),

        color(Color3f(1,1,1), renderProcessed),
        fogType(String("LINEAR")),
        visibilityRange(0, renderProcessed),
        set_bind(false),
        isBound(false)
    {
    }
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};
class Fog2 : public Node, public MField7 {
    public:
    class Color : public SFColor{
    public:
        bool& renderProcessed;

        Color(const Color3f& value, bool& renderProcessed):
            SFColor(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const Color3f& par){
            this->refValue()=par;
            renderProcessed=false;
        }
    };
    class VisibilityRange : public SFFloat {
    public:
        bool& renderProcessed;

        VisibilityRange(const float& value, bool& renderProcessed):
            SFFloat(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const float& par){
            this->refValue()=par;
            renderProcessed=false;
        }
    };
///////////////////////////
    int getType()const{return FOG2;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return &index;}


    Color color;
    SFString fogType;
    VisibilityRange visibilityRange;
    VisibilityRange visibilityStart;
    VisibilityRange density;
    SFBool set_bind;
    SFBool isBound;

    Fog2():
        MField7(
            &color,
            &fogType,
            &visibilityRange,
            &visibilityStart,
            &density,
            &set_bind,
            &isBound
        ),

        color(Color3f(1,1,1), renderProcessed),
        fogType(String("LINEAR")),
        visibilityRange(0, renderProcessed),
        visibilityStart(0, renderProcessed),
        density(0, renderProcessed),
        set_bind(false),
        isBound(false)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};
class Anchor : public AGroup, public PovRay, public MField8 {
    public:
    int getType()const{return ANCHOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFNode children;
    SFString description;
    MFString parameter;
    MFString url;

    Anchor():
        MField8(
            &addChildren,
            &removeChildren,
            &children,
            &description,
            &parameter,
            &url,
            &bboxCenter,
            &bboxSize
        ),

        description(String()),
        parameter(M_String()),
        url(M_String()),
        children(M_Node()),

        AGroup(children)
    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
};
class AudioClip : public Node, public MField8 {
    public:
    int getType()const{return AUDIOCLIP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFString description;
    SFBool loop;
    SFFloat pitch;
    SFTime startTime;
    SFTime stopTime;
    MFString url;
    SFTime duration_changed;
    SFBool isActive;

    AudioClip():
        MField8(
            &description,
            &loop,
            &pitch,
            &startTime,
            &stopTime,
            &url,
            &duration_changed,
            &isActive
        ),

        description(String()),
        loop(false),
        pitch(1),
        startTime(0),
        stopTime(0),
        url(M_String()),
        duration_changed(0),
        isActive(false)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class Billboard : public AGroup, public PovRay, public MField6 {
    public:
    int getType()const{return BILLBOARD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return &index;}


    SFVec3f axisOfRotation;
    MFNode children;

    Billboard():
        MField6(
            &addChildren,
            &removeChildren,
            &axisOfRotation,
            &children,
            &bboxCenter,
            &bboxSize
        ),

        axisOfRotation(Vec3f(0,1,0)),
        children(M_Node()),

        AGroup(children)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
};
class Box : public Geometry, public MField2 {
public:
    class Size : public SFVec3f {
    public:
        bool& renderProcessed;

        Size(const Vec3f& value, bool& renderProcessed):
            SFVec3f(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const Vec3f& par){
            this->refValue()=par;
            renderProcessed=false;
        }
    };
    class Solid : public SFBool {
    public:
        bool& renderProcessed;

        Solid(const bool& value, bool& renderProcessed):
            SFBool(value),
            renderProcessed(renderProcessed)
        {};
        void operator=(const bool& par){
            this->refValue()=par;
            renderProcessed=false;
        }
    };
///////////////////////////

    int getType()const{return BOX;}
    int getFLen()const{return MField2::size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Size size;
    Solid solid;

    Box():
        MField2(
            &size,
            &solid
        ),

        size(Vec3f(2,2,2), renderProcessed),
        solid(true, renderProcessed)

        ///////
    {
    }
#ifdef RENDER
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
#endif
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
};
class Cone : public Geometry, public MField5 {
public:
class BottomRadius : public SFFloat {
public:
    bool& renderProcessed;

    BottomRadius(const float& value, bool& renderProcessed):
        SFFloat(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const float& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Height : public SFFloat {
public:
    bool& renderProcessed;

    Height(const float& value, bool& renderProcessed):
        SFFloat(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const float& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Side : public SFBool {
public:
    bool& renderProcessed;

    Side(const bool& value, bool& renderProcessed):
        SFBool(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const bool& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Bottom : public SFBool {
public:
    bool& renderProcessed;

    Bottom(const bool& value, bool& renderProcessed):
        SFBool(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const bool& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Solid : public SFBool {
public:
    bool& renderProcessed;

    Solid(const bool& value, bool& renderProcessed):
        SFBool(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const bool& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
///////////////////////////

    int getType()const{return CONE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Height height;
    BottomRadius bottomRadius;
    Side side;
    Bottom bottom;
    Solid solid;

    Cone():
        MField5(
            &height,
            &bottomRadius,
            &side,
            &bottom,
            &solid
        ),

        height(2, renderProcessed),
        bottomRadius(1, renderProcessed),
        side(true, renderProcessed),
        bottom(true, renderProcessed),
        solid(true, renderProcessed)
        ///////
    {
    }
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
};
class Cylinder : public Geometry, public MField6 {
    public:
class Height : public SFFloat {
public:
    bool& renderProcessed;

    Height(const float& value, bool& renderProcessed):
        SFFloat(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const float& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Radius : public SFFloat {
public:
    bool& renderProcessed;

    Radius(const float& value, bool& renderProcessed):
        SFFloat(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const float& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Side : public SFBool {
public:
    bool& renderProcessed;

    Side(const bool& value, bool& renderProcessed):
        SFBool(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const bool& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Top : public SFBool {
public:
    bool& renderProcessed;

    Top(const bool& value, bool& renderProcessed):
        SFBool(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const bool& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Bottom : public SFBool {
public:
    bool& renderProcessed;

    Bottom(const bool& value, bool& renderProcessed):
        SFBool(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const bool& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
class Solid : public SFBool {
public:
    bool& renderProcessed;

    Solid(const bool& value, bool& renderProcessed):
        SFBool(value),
        renderProcessed(renderProcessed)
    {};
    void operator=(const bool& par){
        this->refValue()=par;
        renderProcessed=false;
    }
};
///////////////////////////

    public:
    int getType()const{return CYLINDER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Height height;
    Radius radius;
    Side side;
    Top top;
    Bottom bottom;
    Solid solid;

    Cylinder():
        MField6(
            &height,
            &radius,
            &side,
            &top,
            &bottom,
            &solid
        ),

        height(2, renderProcessed),
        radius(1, renderProcessed),
        side(true, renderProcessed),
        top(true, renderProcessed),
        bottom(true, renderProcessed),
        solid(true, renderProcessed)
////////////
    {
    }
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
    //IndexedFaceSetPtr its;
    //IndexedTriangleSetPtr its;
};
class CylinderSensor : public Node, public MField9 {
    public:
    int getType()const{return CYLINDERSENSOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFBool autoOffset;
    SFFloat diskAngle;
    SFBool enabled;
    SFFloat maxAngle;
    SFFloat minAngle;
    SFFloat offset;
    SFBool isActive;
    SFRotation rotation_changed;
    SFVec3f trackPoint_changed;

    CylinderSensor():
        MField9(
            &autoOffset,
            &diskAngle,
            &enabled,
            &maxAngle,
            &minAngle,
            &offset,
            &isActive,
            &rotation_changed,
            &trackPoint_changed
        ),

        autoOffset(true),
        diskAngle(.262),
        enabled(true),
        maxAngle(-1),
        minAngle(0),
        offset(0),
        isActive(false),
        rotation_changed(Rotation()),
        trackPoint_changed(Vec3f())
    {
    }
};
class ElevationGrid : public Geometry, public MField14 {
    public:
    int getType()const{return ELEVATIONGRID;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFFloat set_height;
    SFNode color;
    SFNode normal;
    SFNode texCoord;
    MFFloat height;
    SFBool ccw;
    SFBool colorPerVertex;
    SFFloat creaseAngle;
    SFBool normalPerVertex;
    SFBool solid;
    SFInt32 xDimension;
    SFFloat xSpacing;
    SFInt32 zDimension;
    SFFloat zSpacing;

    ElevationGrid():
        MField14(
            &set_height,
            &color,
            &normal,
            &texCoord,
            &height,
            &ccw,
            &colorPerVertex,
            &creaseAngle,
            &normalPerVertex,
            &solid,
            &xDimension,
            &xSpacing,
            &zDimension,
            &zSpacing
        ),

        set_height(M_Float()),
        color(0),
        normal(0),
        texCoord(0),
        height(M_Float()),
        ccw(true),
        colorPerVertex(true),
        creaseAngle(0),
        normalPerVertex(true),
        solid(true),
        xDimension(0),
        xSpacing(1),
        zDimension(0),
        zSpacing(1)
    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
};
class Extrusion : public Node, public MField14 {
    public:
    int getType()const{return EXTRUSION;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFVec2f set_crossSection;
    MFRotation set_orientation;
    MFVec2f set_scale;
    MFVec3f set_spine;
    SFBool beginCap;
    SFBool ccw;
    SFBool convex;
    SFFloat creaseAngle;
    MFVec2f crossSection;
    SFBool endCap;
    MFRotation orientation;
    MFVec2f scale;
    SFBool solid;
    MFVec3f spine;

    Extrusion():
        MField14(
            &set_crossSection,
            &set_orientation,
            &set_scale,
            &set_spine,
            &beginCap,
            &ccw,
            &convex,
            &creaseAngle,
            &crossSection,
            &endCap,
            &orientation,
            &scale,
            &solid,
            &spine
        ),

        set_crossSection(M_Vec2f()),
        set_orientation(M_Rotation()),
        set_scale(M_Vec2f()),
        set_spine(M_Vec3f()),
        beginCap(true),
        ccw(true),
        convex(true),
        creaseAngle(0),
        crossSection(M_Vec2f()),
        endCap(true),
        orientation(M_Rotation()),
        scale(M_Vec2f()),
        solid(true),
        spine(M_Vec3f())
    {
#ifdef SEAMLESS3DGUI
        index.length=4;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class FontStyle : public Node, public MField9 {
    public:
    int getType()const{return FONTSTYLE;}
    int getFLen()const{return MField9::size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}


    MFString family;
    SFBool horizontal;
    MFString justify;
    SFString language;
    SFBool leftToRight;
    SFFloat size;
    SFFloat spacing;
    SFString style;
    SFBool topToBottom;

    FontStyle():
        MField9(
            &family,
            &horizontal,
            &justify,
            &language,
            &leftToRight,
            &size,
            &spacing,
            &style,
            &topToBottom
        ),

        family(M_String("SERIF")),
        horizontal(true),
        justify(M_String("BEGIN")),
        language(String()),
        leftToRight(true),
        size(1),
        spacing(1),
        style(String("PLAIN")),
        topToBottom(true)
    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};

class NurbsSurface : public Geometry, public MField13 {
    public:
    int getType()const{return NURBSSURFACE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}

    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFVec3f	controlPoint;
    MFFloat	weight;
    MFFloat	uKnot;
    MFFloat	vKnot;
    SFNode	texCoord;
    SFFloat	uDimension;
    SFFloat	vDimension;
    SFFloat	uOrder;
    SFFloat	vOrder;
    SFFloat	uTessellation;
    SFFloat	vTessellation;
    SFBool	ccw;
    SFBool	solid;

    NurbsSurface():
        MField13(
            &controlPoint,
            &weight,
            &uKnot,
            &vKnot,
            &texCoord,
            &uDimension,
            &vDimension,
            &uOrder,
            &vOrder,
            &uTessellation,
            &vTessellation,
            &ccw,
            &solid
        ),
        controlPoint(M_Vec3f()),
        weight(M_Float()),
        uKnot(M_Float()),
        vKnot(M_Float()),
        texCoord(0),
        uDimension(0),
        vDimension(0),
        uOrder(3),
        vOrder(3),
        uTessellation(0),
        vTessellation(0),
        ccw(true),
        solid(true),
        ///
        index(0,0,0)
    {
    }
    M_Int index;
    void renderNode(Matrix, Vec3f, Mesh&, Render*&, const Appearance*){};
    void convertToGeomSource(Mesh&, const Appearance*&, M_Int&)const{};
};
class LOD : public AGroup, public PovRay, public MField7 { //children=level
    public:
    int getType()const{return LOD_TYPE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFVec3f center;
    MFFloat range;
    MFNode children;

    LOD():
        MField7(
            &addChildren,
            &removeChildren,
            &bboxCenter,
            &bboxSize,
            &center,
            &range,
            &children
        ),

        center(Vec3f()),
        range(M_Float()),
        children(M_Node()),

        AGroup(children)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
};
class MovieTexture : public Node, public MField9 {
    public:
    int getType()const{return MOVIETEXTURE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFBool loop;
    SFFloat speed;
    SFTime startTime;
    SFTime stopTime;
    MFString url;
    SFBool repeatS;
    SFBool repeatT;
    SFTime duration_changed;
    SFBool isActive;

    MovieTexture():
        MField9(
            &loop,
            &speed,
            &startTime,
            &stopTime,
            &url,
            &repeatS,
            &repeatT,
            &duration_changed,
            &isActive
        ),

        loop(false),
        speed(1),
        startTime(0),
        stopTime(0),
        url(M_String()),
        repeatS(true),
        repeatT(true),
        duration_changed(0),
        isActive(false)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class PixelTexture : public Node, public MField2 {
    public:
    int getType()const{return PIXELTEXTURE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFBool repeatS;
    SFBool repeatT;

    PixelTexture():
        MField2(
            &repeatS,
            &repeatT
        ),

        repeatS(true),
        repeatT(true)
    {
    }
};
class PlaneSensor : public Node, public MField8 {
    public:
    int getType()const{return PLANESENSOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFBool autoOffset;
    SFBool enabled;
    SFVec2f maxPosition;
    SFVec2f minPosition;
    SFVec3f offset;
    SFBool isActive;
    SFVec3f trackPoint_changed;
    SFVec3f translation_changed;

    PlaneSensor():
        MField8(
            &autoOffset,
            &enabled,
            &maxPosition,
            &minPosition,
            &offset,
            &isActive,
            &trackPoint_changed,
            &translation_changed
        ),

        autoOffset(true),
        enabled(true),
        maxPosition(Vec2f(-1,-1)),
        minPosition(Vec2f()),
        offset(Vec3f()),
        isActive(false),
        trackPoint_changed(Vec3f()),
        translation_changed(Vec3f())
    {
    }
};
class PointLight : public Node, public MField8 {
    public:
    int getType()const{return POINTLIGHT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFFloat ambientIntensity;
    SFVec3f attenuation;
    SFColor color;
    SFBool global;
    SFFloat intensity;
    SFVec3f location;
    SFBool on;
    SFFloat radius;

    PointLight():
        MField8(
            &ambientIntensity,
            &attenuation,
            &color,
            &global,
            &intensity,
            &location,
            &on,
            &radius
        ),

        ambientIntensity(0),
        attenuation(Vec3f(1,0,0)),
        color(Color3f(1,1,1)),
        global(true),
        intensity(1),
        location(Vec3f()),
        on(true),
        radius(100)
    {
    }
};
class PointSet : public Geometry, public MField2 {
    public:
    int getType()const{return POINTSET;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFNode color;
    SFNode coord;

    PointSet():
        MField2(
            &color,
            &coord
        ),

        color(0),
        coord(0)
    {
    }
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
};
class ProximitySensor : public Node, public MField8 {
    public:
    int getType()const{return PROXIMITYSENSOR;}
    int getFLen()const{return MField8::size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFVec3f center;
    SFVec3f size;
    SFBool enabled;
    SFBool isActive;
    SFVec3f position_changed;
    SFRotation orientation_changed;
    SFTime enterTime;
    SFTime exitTime;

    ProximitySensor():
        MField8(
            &center,
            &size,
            &enabled,
            &isActive,
            &position_changed,
            &orientation_changed,
            &enterTime,
            &exitTime
        ),

        center(Vec3f()),
        size(Vec3f()),
        enabled(true),
        isActive(false),
        position_changed(Vec3f()),
        orientation_changed(Rotation()),
        enterTime(0),
        exitTime(0)
    {
    }
};
class Sound : public Node, public MField10 {
    public:
    int getType()const{return SOUND;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFVec3f direction;
    SFFloat intensity;
    SFVec3f location;
    SFFloat maxBack;
    SFFloat maxFront;
    SFFloat minBack;
    SFFloat minFront;
    SFFloat priority;
    SFNode source;
    SFBool spatialize;

    Sound():
        MField10(
            &direction,
            &intensity,
            &location,
            &maxBack,
            &maxFront,
            &minBack,
            &minFront,
            &priority,
            &source,
            &spatialize
        ),

        direction(Vec3f(0,0,1)),
        intensity(1),
        location(Vec3f()),
        maxBack(10),
        maxFront(10),
        minBack(1),
        minFront(1),
        priority(0),
        source(0),
        spatialize(true)
    {
    }
};
class Sphere : public Geometry, public MField1 {
    public:
class Radius : public SFFloat {
public:
    Sphere& home;

    Radius(const float& value, Sphere& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par){
        this->refValue()=par;
        home.renderProcessed=false;
    }
};
///////////////////////////

    int getType()const{return SPHERE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Radius radius;

    Sphere():
        MField1(
            &radius
        ),
        radius(1, *this)
    {
    }
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
};
class SphereSensor : public Node, public MField6 {
    public:
    int getType()const{return SPHERESENSOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFBool autoOffset;
    SFBool enabled;
    SFRotation offset;
    SFBool isActive;
    SFRotation rotation_changed;
    SFVec3f trackPoint_changed;

    SphereSensor():
        MField6(
            &autoOffset,
            &enabled,
            &offset,
            &isActive,
            &rotation_changed,
            &trackPoint_changed
        ),

        autoOffset(true),
        enabled(true),
        offset(Rotation()),
        isActive(false),
        rotation_changed(Rotation()),
        trackPoint_changed(Vec3f())
    {
    }
};
class SpotLight : public Node, public MField11 {
    public:
    int getType()const{return SPOTLIGHT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFFloat ambientIntensity;
    SFVec3f attenuation;
    SFFloat beamWidth;
    SFColor color;
    SFFloat cutOffAngle;
    SFVec3f direction;
    SFBool global;
    SFFloat intensity;
    SFVec3f location;
    SFBool on;
    SFFloat radius;

    SpotLight():
        MField11(
            &ambientIntensity,
            &attenuation,
            &beamWidth,
            &color,
            &cutOffAngle,
            &direction,
            &global,
            &intensity,
            &location,
            &on,
            &radius
        ),

        ambientIntensity(0),
        attenuation(Vec3f(1,0,0)),
        beamWidth(1.570796),
        color(Color3f(1,1,1)),
        cutOffAngle(.785398),
        direction(Vec3f(0,0,-1)),
        global(true),
        intensity(1),
        location(Vec3f()),
        on(true),
        radius(100)
    {
    }
};
class Switch : public AGroup, public PovRay, public MField6 {
    public:
    int getType()const{return SWITCH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFInt32 whichChoice;
    MFNode children;

    Switch():
        MField6(
            &addChildren,
            &removeChildren,
            &whichChoice,
            &bboxCenter,
            &bboxSize,
            &children
        ),
        whichChoice(-1),
        children(M_Node()),

        AGroup(children)
    {
    }
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
};
class Text : public Geometry, public MField4 {
    public:
    int getType()const{return TEXT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFString string;
    MFFloat length;
    SFFloat maxExtent;
    SFNode fontStyle;

    Text():
        MField4(
            &string,
            &length,
            &maxExtent,
            &fontStyle
        ),

        string(M_String()),
        length(M_Float()),
        maxExtent(0),
        fontStyle(0)
    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
};

class VisibilitySensor : public Node, public MField6 {
    public:
    int getType()const{return VISIBILITYSENSOR;}
    int getFLen()const{return MField6::size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFVec3f center;
    SFBool enabled;
    SFVec3f size;
    SFTime enterTime;
    SFTime exitTime;
    SFBool isActive;

    VisibilitySensor():
        MField6(
            &center,
            &enabled,
            &size,
            &enterTime,
            &exitTime,
            &isActive
        ),

        center(Vec3f()),
        enabled(true),
        size(Vec3f()),
        enterTime(0),
        exitTime(0),
        isActive(false)
    {
    }
};


class ExitBuild : public Build, public MField2{
    public:
    int getType()const{return EXITBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool enabled;
    Setup setup;

    ExitBuild():
        Build(setup),
        MField2(
            &enabled,
            &setup
        ),

        enabled(true),
        setup(0, *this)

    {
    }
    bool illegalImmediateRef(const Art*, M_Art&)const{return false;}
    void doBuild();
};
class NullNode : public Node, public MField0 {
    public:
    int getType()const{return NULLNODE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}



    NullNode()
    {
    }
};
class SplitTri : public Build, public MField3{
    public:
    int getType()const{return SPLITTRI;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFInt32 corner;
    SFPart part;
    Setup setup;

    SplitTri():
        Build(setup),
        MField3(
            &corner,
            &part,
            &setup
        ),

        corner(M_Int()),
        part(0),
        setup(0, *this)

    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};
class SwapTriDivide : public Build, public MField3{
    public:
    int getType()const{return SWAPTRIDIVIDE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFInt32 corner;
    SFPart part;
    Setup setup;

    SwapTriDivide():
        Build(setup),
        MField3(
            &corner,
            &part,
            &setup
        ),

        corner(M_Int()),
        part(0),
        setup(0, *this)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};
class PlaceTexture : public Build, public MField5{
    public:
    int getType()const{return PLACETEXTURE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFVec2f scale;
    SFFloat rotation;
    SFVec2f translation;
    PartChild part;
    Setup setup;

    PlaceTexture():
        Build(setup),
        MField5(
            &scale,
            &rotation,
            &translation,
            &part,
            &setup
        ),

        scale(Vec2f(1,1)),
        rotation(0),
        translation(Vec2f()),
        part(M_Node()),
        setup(0, *this)

    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};

//class HAnimHumanoid : public Node, public PovRay, public MField18 {
class HAnimHumanoid : public Node, public MField18 {
    public:
class Skeleton : public MFNode{
public:
    HAnimHumanoid& home;
    Skeleton(const M_Node& value, HAnimHumanoid& home):
        MFNode(value),
        home(home)
    {}
    void operator=(const M_Node& par);
};

    int getType()const{return HANIMHUMANOID;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFString version;
    SFString name;
    MFString info;
    SFVec3f center;
    SFRotation scaleOrientation;
    SFVec3f scale;
    SFRotation rotation;
    SFVec3f translation;
    SFVec3f bboxCenter;
    SFVec3f bboxSize;
    MFNode skin;
    SFNode skinCoord;
    SFNode skinNormal;
    Skeleton skeleton;
    MFNode joints;
    MFNode segments;
    MFNode sites;
    MFNode viewpoints;

    HAnimHumanoid():
        MField18(
            &version,
            &name,
            &info,
            &center,
            &scaleOrientation,
            &scale,
            &rotation,
            &translation,
            &bboxCenter,
            &bboxSize,
            &skin,
            &skinCoord,
            &skinNormal,
            &skeleton,
            &joints,
            &segments,
            &sites,
            &viewpoints
        ),

        version(String()),
        name(String()),
        info(M_String()),
        center(Vec3f()),
        scaleOrientation(Rotation(0,0,1,0)),
        scale(Vec3f(1,1,1)),
        rotation(Rotation(0,0,1,0)),
        translation(Vec3f()),
        bboxCenter(Vec3f()),
        bboxSize(Vec3f(-1,-1,-1)),
        skin(M_Node()),
        skinCoord(0),
        skinNormal(0),
        skeleton(M_Node(), *this),
        joints(M_Node()),
        segments(M_Node()),
        sites(M_Node()),
        viewpoints(M_Node()),

        morphStarted(false),
        render(0)
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
    Render* render;
#endif
//	void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
//	void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);

    friend Skeleton;
    friend HAnimDisplacer;
    bool morphStarted;
    void renderNode(Matrix matrix, Vec3f parentColScale);
    void convertToRenderFormat(Render*& render, const Appearance* appearance);



    M_HAnimJoint jointList;
    void processFields();
    M_Int coordMask;
    M_Int maskIndex;
};
class HAnimJoint : public AGroup, public MField19, public PartJoint{
class Progenitor : public MFNode{
public:
    HAnimJoint& home;
    Progenitor(const M_Node& value, HAnimJoint& home):
        MFNode(value),
        home(home)
    {}
    virtual ~Progenitor()
    {
    }
    void operator=(const M_Node& par);
};
class Displacers : public MFNode{
public:
    HAnimJoint& home;
    Displacers(const M_Node& value, HAnimJoint& home):
        MFNode(value),
        home(home)
    {}
    void operator=(const M_Node& par);
};

    public:
    int getType()const{return HANIMJOINT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFNode addChildren;
    MFNode removeChildren;
    SFString name;
    SFVec3f center;
    SFRotation scaleOrientation;
    SFVec3f scale;
    SFRotation rotation;
    SFVec3f translation;
    MFVec3f skinCoordLocal;
    MFInt32 skinCoordIndex;
    MFFloat skinCoordWeight;
    SFRotation limitOrientation;
    MFFloat llimit;
    MFFloat stiffness;
    MFFloat ulimit;
    Progenitor children;
    Displacers displacers;

    HAnimJoint():
        MField19(
            &addChildren,
            &removeChildren,
            &name,
            &center,
            &scaleOrientation,
            &scale,
            &rotation,
            &translation,
            &bboxCenter,
            &bboxSize,
            &skinCoordLocal,
            &skinCoordIndex,
            &skinCoordWeight,
            &limitOrientation,
            &llimit,
            &stiffness,
            &ulimit,
            &children,
            &displacers
        ),

        addChildren(M_Node()),
        removeChildren(M_Node()),
        name(String()),
        center(Vec3f()),
        scaleOrientation(Rotation(0,0,1,0)),
        scale(Vec3f(1,1,1)),
        rotation(Rotation(0,0,1,0)),
        translation(Vec3f()),
        skinCoordLocal(M_Vec3f()),
        skinCoordIndex(M_Int()),
        skinCoordWeight(M_Float()),
        limitOrientation(Rotation(0,0,1,0)),
        llimit(M_Float()),
        stiffness(M_Float()),
        ulimit(M_Float()),
        children(M_Node(), *this),
        displacers(M_Node(), *this),

        AGroup(children)
    {
#ifdef SEAMLESS3DGUI
        index.length=5;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
friend Displacers;
friend Progenitor;
friend HAnimHumanoid;

    SFRotation& refSFRotation(){
        return rotation;
    }
    SFVec3f& refSFTranslation(){
        return translation;
    }
    void genDynamicTransforms(Matrix parentMatrix, int& smi);
    void setJointIndices(int& smi, int midx, COLORVERTEX_1WEIGHT* pVertices, M_Int& maskIndex);
    void setJointIndices(int& smi, int midx, TEXTUREVERTEX_1WEIGHT* pVertices, M_Int& maskIndex);
    void processFieldsLoop( HAnimHumanoid* humanoid);
protected:
    HAnimHumanoid* humanoid;
};
class HAnimDisplacer : public Node, public MField4 {
    public:
class Weight : public SFFloat {
public:
    HAnimDisplacer& home;

    Weight(const float& value, HAnimDisplacer& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};
friend Weight;
friend HAnimJoint;
///////////////////////////
    int getType()const{return HANIMDISPLACER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFInt32 coordIndex;
    MFVec3f displacements;
    SFString name;
    Weight weight;

    HAnimDisplacer():
        MField4(
            &coordIndex,
            &displacements,
            &name,
            &weight
        ),

        coordIndex(M_Int()),
        displacements(M_Vec3f()),
        name(String()),
        weight(0,*this)

    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
protected:
    HAnimHumanoid* humanoid;
};

class HAnimSegment : public AGroup, public MField11 {
    public:

    int getType()const{return HANIMSEGMENT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    SFVec3f		centerOfMass;
    MFNode		children;
    MFVec3f		coord;
    MFNode		displacers;
    SFFloat		mass;
    MFFloat		momentsOfInertia;
    SFString	name;

    HAnimSegment():
        MField11(
            &addChildren,
            &removeChildren,
            &centerOfMass,
            &children,
            &coord,
            &displacers,
            &mass,
            &momentsOfInertia,
            &name,
            &bboxCenter,
            &bboxSize
        ),
        centerOfMass(Vec3f()),
        children(M_Node()),
        coord(M_Vec3f()),
        displacers(M_Node()),
        mass(0),
        momentsOfInertia(M_Float()),
        name(String()),

        AGroup(children)
    {
        index.length=2;
    }
    M_Int index;

    void processFieldsLoop(HAnimHumanoid* humanoid);
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
};
class HAnimSite : public Node, public MField11 {
    public:
    int getType()const{return HANIMSITE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    MFNode		addChildren;
    MFNode		removeChildren;
    MFNode		children;
    SFString	name;
    SFVec3f		center;
    SFRotation	scaleOrientation;
    SFVec3f		scale;
    SFRotation	rotation;
    SFVec3f		translation;
    SFVec3f		bboxCenter;
    SFVec3f		bboxSize;

    HAnimSite():
        MField11(
            &addChildren,
            &removeChildren,
            &children,
            &name,
            &center,
            &scaleOrientation,
            &scale,
            &rotation,
            &translation,
            &bboxCenter,
            &bboxSize
        ),

        addChildren(M_Node()),
        removeChildren(M_Node()),
        children(M_Node()),
        name(String()),
        center(Vec3f()),
        scaleOrientation(Rotation(0,0,1,0)),
        scale(Vec3f(1,1,1)),
        rotation(Rotation(0,0,1,0)),
        translation(Vec3f()),
        bboxCenter(Vec3f()),
        bboxSize(Vec3f(-1,-1,-1))
    {
    }
};
class BooleanFilter : public Node, public MField4 {
    public:
    int getType()const{return BOOLEANFILTER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFBool set_boolean;
    SFBool inputFalse;
    SFBool inputNegate;
    SFBool inputTrue;

    BooleanFilter():
        MField4(
            &set_boolean,
            &inputFalse,
            &inputNegate,
            &inputTrue
        ),

        set_boolean(false),
        inputFalse(false),
        inputNegate(false),
        inputTrue(false)

    {
    }
};
class CameraSelector : public Node, public SamplePose, public MField3 {
    public:
	void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    CameraSelector& home;

    Set_fraction(const float& value, CameraSelector& home):
        SFFloat(value),
		home(home)
    {};
    void operator=(const float& par){
        raw = par;
        home.do_set_fraction();
    }
};

    int getType()const{return CAMERASELECTOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


	Set_fraction set_fraction;
	MFFloat key;
	MFNode keyValue;

    CameraSelector():
        MField3(
            &set_fraction,
            &key,
            &keyValue
        ),
        set_fraction(0, *this),
        key(M_Float()),
        keyValue(M_Node())
    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    void processFields();
    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Node* node, int keyIndex, int keyValueIndex, int numOf);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field);
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);
};
class BooleanToggle : public Node, public MField2 {
    public:
    int getType()const{return BOOLEANTOGGLE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFBool set_boolean;
    SFBool toggle;

    BooleanToggle():
        MField2(
            &set_boolean,
            &toggle
        ),

        set_boolean(false),
        toggle(false)

    {
    }
};
class BooleanTrigger : public Node, public MField2 {
    public:
    int getType()const{return BOOLEANTRIGGER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFTime set_triggerTime;
    SFBool triggerTrue;

    BooleanTrigger():
        MField2(
            &set_triggerTime,
            &triggerTrue
        ),

        set_triggerTime(0),
        triggerTrue(false)

    {
    }
};
class IntegerSequencer : public Node, public MField6 {
    public:
    int getType()const{return INTEGERSEQUENCER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    SFBool next;
    SFBool previous;
    SFFloat set_fraction;
    MFFloat key;
    MFInt32 keyValue;
    SFInt32 value_changed;

    IntegerSequencer():
        MField6(
            &next,
            &previous,
            &set_fraction,
            &key,
            &keyValue,
            &value_changed
        ),

        next(false),
        previous(false),
        set_fraction(0),
        key(M_Float()),
        keyValue(M_Int()),
        value_changed(0)

    {
#ifdef SEAMLESS3DGUI
        index.length=2;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
class IntegerTrigger : public Node, public MField3 {
    public:
    int getType()const{return INTEGERTRIGGER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool set_boolean;
    SFInt32 integerKey;
    SFInt32 triggerValue;

    IntegerTrigger():
        MField3(
            &set_boolean,
            &integerKey,
            &triggerValue
        ),

        set_boolean(false),
        integerKey(-1),
        triggerValue(0)
    {
    }
};
class TimeTrigger : public Node, public MField2 {
    public:
    int getType()const{return TIMETRIGGER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool set_boolean;
    SFTime triggerTime;

    TimeTrigger():
        MField2(
            &set_boolean,
            &triggerTime
        ),

        set_boolean(false),
        triggerTime(0)

    {
    }
};
class StaticGroup : public Node, public MField3 {
    public:
    int getType()const{return STATICGROUP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    MFNode children;
    SFVec3f bboxCenter;
    SFVec3f bboxSize;

    StaticGroup():
        MField3(
            &children,
            &bboxCenter,
            &bboxSize
        ),

        children(M_Node()),
        bboxCenter(Vec3f()),
        bboxSize(Vec3f(-1,-1,-1))

    {
    }
};
class FloatAccelerator : public Node, public MField10 {
    public:
class Set_fraction : public SFFloat {
public:
    FloatAccelerator& home;

    Set_fraction(const float& value, FloatAccelerator& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};
    int getType()const{return FLOATACCELERATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif

    Set_fraction set_fraction;
    SFFloat fraction_changed;
    SFFloat value_changed;
    SFFloat fracMult;
    SFFloat init;
    MFFloat delay;
    MFFloat accelerate;
    MFFloat attack;
    MFFloat sustain;
    MFFloat decay;

    FloatAccelerator():
        MField10(
            &set_fraction,
            &fraction_changed,
            &value_changed,
            &fracMult,
            &init,
            &delay,
            &accelerate,
            &attack,
            &sustain,
            &decay
        ),

        set_fraction(0, *this),
        fraction_changed(0),
        value_changed(0),
        fracMult(1),
        init(0),
        delay(M_Float()),
        accelerate(M_Float()),
        attack(M_Float()),
        sustain(M_Float()),
        decay(M_Float())
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
//	S_Float accumFloat(int targetIndex, float time);
};

class RotAccelerator : public Node, public MField13 {
public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    RotAccelerator& home;

    Set_fraction(const float& value, RotAccelerator& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par){
        raw=par;
        home.do_set_fraction();
    }
};
class CalcForReturn : public SFBool {
public:
    RotAccelerator& home;

    CalcForReturn(const bool& par, RotAccelerator& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class CalcForPose : public SFBool {
public:
    RotAccelerator& home;

    CalcForPose(const bool& par, RotAccelerator& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

///////////////////////////
    int getType()const{return ROTACCELERATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    SFFloat fraction_changed;
    SFRotation value_changed;
    SFFloat fracMult;
    SFRotation init;
    MFVec3f axis;
    MFFloat delay;
    MFFloat accelerate;
    MFFloat attack;
    MFFloat sustain;
    MFFloat decay;
    CalcForReturn calcForReturn;
    CalcForPose calcForPose;

    RotAccelerator():
        MField13(
            &set_fraction,
            &fraction_changed,
            &value_changed,
            &fracMult,
            &init,
            &axis,
            &delay,
            &accelerate,
            &attack,
            &sustain,
            &decay,
            &calcForReturn,
            &calcForPose
        ),

        set_fraction(0,*this),
        fraction_changed(0),
        value_changed(Rotation()),
        fracMult(1),
        init(Rotation(0,0,1,0)),
        axis(M_Vec3f()),
        delay(M_Float()),
        accelerate(M_Float()),
        attack(M_Float()),
        sustain(M_Float()),
        decay(M_Float()),
        calcForReturn(false,*this),
        calcForPose(false,*this)

    {

#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
    void calcPose(Rotation rotation);
#endif
    Rotation accumRotation(int targetIndex, float time);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
};
class CopyToMorph : public Build, public MField3{
    public:
    int getType()const{return COPYTOMORPH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFPart part;
    SFNode morpher;
    Setup setup;

    CopyToMorph():
        Build(setup),
        MField3(
            &part,
            &morpher,
            &setup
        ),

        part(0),
        morpher(0),
        setup(0, *this)

    {
    }
    bool illegalImmediateRef(const Art*, M_Art&)const{return false;}
    void doBuild();
};
class RelateMorpher : public Build, public MField2{
    public:
    int getType()const{return RELATEMORPHER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFPart part;
    Setup setup;

    RelateMorpher():
        Build(setup),
        MField2(
            &part,
            &setup
        ),

        part(0),
        setup(0, *this)

    {
    }
    bool illegalImmediateRef(const Art*, M_Art&)const{return false;}
    void doBuild();
};
class DelPlane : public Build, public MField12{
    public:
    int getType()const{return DELPLANE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFRotation orientation;
    SFVec3f location;
    SFBool negX;
    SFBool posX;
    SFBool negY;
    SFBool posY;
    SFBool negZ;
    SFBool posZ;
    SFBool full;
    SFBool align;
    PartChild part;
    Setup setup;

    DelPlane():
        Build(setup),
        MField12(
            &orientation,
            &location,
            &negX,
            &posX,
            &negY,
            &posY,
            &negZ,
            &posZ,
            &full,
            &align,
            &part,
            &setup
        ),

        orientation(Rotation()),
        location(Vec3f()),
        negX(false),
        posX(false),
        negY(false),
        posY(false),
        negZ(false),
        posZ(false),
        full(true),
        align(false),
        part(M_Node()),
        setup(0, *this)

    {
    }
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};
class DelSphere : public Build, public MField4{
    public:
    int getType()const{return DELSPHERE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFVec3f location;
    SFFloat radius;
    PartChild part;
    Setup setup;

    DelSphere():
        Build(setup),
        MField4(
            &location,
            &radius,
            &part,
            &setup
        ),

        location(Vec3f()),
        radius(.25),
        part(M_Node()),
        setup(0, *this)

    {
    }
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};
class RadialTug : public Generator, public Node, public MField4, public VisualAid{
    public:
    int getType()const{return RADIALTUG;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    MFVec3f location;
    MFVec3f displacement;
    MFFloat slope;
    MFFloat radius;

    RadialTug():
        MField4(
            &location,
            &displacement,
            &slope,
            &radius
        ),

        location(M_Vec3f()),
        displacement(M_Vec3f()),
        slope(M_Float()),
        radius(M_Float())
    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    void processFields();
//	bool tugVertex0X;
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
};
class TapeMeasure : public Build, public MField5 {
    public:
class Measurement : public SFFloat {
public:
    TapeMeasure* home;

    Measurement(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par);
};
////////////////////
    int getType()const{return TAPEMEASURE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Measurement measurement;
    SFInt32 baseVertex;
    SFInt32 staves;
    SFPart part;
    Setup setup;

    TapeMeasure():
        Build(setup),
        MField5(
            &measurement,
            &baseVertex,
            &staves,
            &part,
            &setup
        ),

        measurement(0),
        baseVertex(0),
        staves(0),
        part(0),
        setup(0, *this)
    {
        measurement.home=this;
    }
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void doBuild();
};


class SCGroupBuild : public GroupBuild{//Script Controlled GroupBuild
    public:
    int getType()const{return SCGROUPBUILD;}
    void doBuild(){
    };
};

/*
class OperationNode{
    public:
    Operation* operation;
    void OperationNode::decodeString(const String& str, int& i);
    String& nameRef;
    OperationNode(String& name);
};
*/
typedef M_<Word*> M_Word;
class CodeLine : public Node, public MField1 {
    bool outputVec3f_2(String& str, M_Word& m_word, int& w, int& bracketCounter);
    void outputVec3f(String& str, M_Word& m_word, int& w, int& bracketCounter);
    bool outputM_Vec3f(String& str, M_Word& m_word, int& w, int& bracketCounter);
    void outputRotation(String& str, M_Word& m_word, int& w, int& bracketCounter);
    void outputM_Rotation(String& str, M_Word& m_word, int& w, int& bracketCounter);
    void doTillClosingBracket(String& str, M_<Word*>& m_word, int& w, int& bracketCounter);
    void doTillClosingSquareBracket(String& str, M_<Word*>& m_word, int& w, int& bracketCounter);
    void outputToString(String& str, M_<Word*>& m_word, int& w, int& bracketCounter);
public:
    int getType()const{return CODELINE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    MFNode codeLine;

    CodeLine():
        MField1(
            &codeLine
        ),
        codeLine(M_Node()),
////
        opFrom(0),
        opTo(0),
        wordFrom(0),
        wordTo(0)
    {
    }

    int opFrom;
    int opTo;
    int wordFrom;
    int wordTo;


    void toString(String& str, Function* lastFunction);
    bool isIfElseForEtc(Function* f);
    Function* getParentFunction();
};
/*
class If : public Node, public OperationNode, public MField {
    public:
    int getType()const{return IF;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i){throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    MFNode codeLine;

    Field* fieldArray[1];

    If();
    void If::getInstructions(const String& str,int& i);
};
*/


class PosAccelerator : public Node, public MField11 {
public:
    void do_set_fraction();
class Set_fraction : public SFFloat {
public:
    PosAccelerator* home;

    Set_fraction(const float& value):
        SFFloat(value)
    {};
    void operator=(const float& par){
        raw=par;
        home->do_set_fraction();
    }
};
///////////////////////////
    int getType()const{return POSACCELERATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    Set_fraction set_fraction;
    SFFloat fraction_changed;
    SFVec3f value_changed;
    SFFloat fracMult;
    SFVec3f init;
    MFVec3f direction;
    MFFloat delay;
    MFFloat accelerate;
    MFFloat attack;
    MFFloat sustain;
    MFFloat decay;

    PosAccelerator():
        MField11(
            &set_fraction,
            &fraction_changed,
            &value_changed,
            &fracMult,
            &init,
            &direction,
            &delay,
            &accelerate,
            &attack,
            &sustain,
            &decay
        ),

        set_fraction(0),
        fraction_changed(0),
        value_changed(Vec3f()),
        fracMult(1),
        init(Vec3f()),
        direction(M_Vec3f()),
        delay(M_Float()),
        accelerate(M_Float()),
        attack(M_Float()),
        sustain(M_Float()),
        decay(M_Float())
    {
        set_fraction.home=this;
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
};
/*
class AnimByWrist : public Node, public MField {
    public:
    void do_set_wrist();
class Wrist : public SFVec3f {
public:
    SFRotation& elbow;
    SFRotation& sholder;
    SFVec3f& location;
    SFRotation& swivel;
    SFFloat& upperarmLen;
    SFFloat& forearmLen;

    Wrist( Vec3f& par, SFRotation& i_elbow, SFRotation& i_sholder, SFVec3f& i_location, SFRotation& i_swivel, SFFloat& i_upperarmLen, SFFloat& i_forearmLen):
        SFVec3f(par),
        elbow(i_elbow),
        sholder(i_sholder),
        location(i_location),
        swivel(i_swivel),
        upperarmLen(i_upperarmLen),
        forearmLen(i_forearmLen)
    {};
    void operator=(const Vec3f& par);
};
///////////////////////////
    int getType()const{return ANIMBYWRIST;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i){throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}

private:
    Rotation elbowMem;
    Rotation sholderMem;
    Vec3f locationMem;
    Vec3f wristMem;
    Rotation swivelMem;
    float upperarmLenMem;
    float forearmLenMem;

public:
    SFRotation elbow;
    SFRotation sholder;
    SFVec3f location;
    Wrist wrist;
    SFRotation swivel;
    SFFloat upperarmLen;
    SFFloat forearmLen;

    Field* fieldArray[7];

    AnimByWrist():
        MField(
            fieldArray,
            &elbow,
            &sholder,
            &location,
            &wrist,
            &swivel,
            &upperarmLen,
            &forearmLen
        ),

        elbow(elbowMem, outputOnly),
        sholder(sholderMem, outputOnly),
        location(locationMem),
        wrist(wristMem, elbow, sholder, location, swivel, upperarmLen, forearmLen),
        swivel(swivelMem),
        upperarmLen(upperarmLenMem),
        forearmLen(forearmLenMem),

        locationMem(0,0,0),
        wristMem(0,0,0),
        swivelMem(0,0,1,0),
        upperarmLenMem(1),
        forearmLenMem(1)
    {
    }
};
*/
class AnimByAnkle : public Node, public MField8 {
    public:
class Position : public SFVec3f {
public:
    SFRotation& catAnkle;
    SFBool& outAnkle;
    SFFloat& swivel;
    SFPart& pelvis;
    SFPart& hip;
    SFPart& knee;
    SFPart& ankle;

    Position(const Vec3f& value, SFRotation& i_catAnkle, SFBool& i_outAnkle, SFFloat& i_swivel, SFPart& i_pelvis, SFPart& i_hip, SFPart& i_knee, SFPart& i_ankle):
        SFVec3f(value),
        catAnkle(i_catAnkle),
        outAnkle(i_outAnkle),
        swivel(i_swivel),
        pelvis(i_pelvis),
        hip(i_hip),
        knee(i_knee),
        ankle(i_ankle)
    {};
    void operator=(const Vec3f& par);
};
///////////////////////////
    int getType()const{return ANIMBYANKLE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Position position;
    SFRotation catAnkle;
    SFBool outAnkle;
    SFFloat swivel;
    SFPart pelvis;
    SFPart hip;
    SFPart knee;
    SFPart ankle;

    AnimByAnkle():
        MField8(
            &position,
            &catAnkle,
            &outAnkle,
            &swivel,
            &pelvis,
            &hip,
            &knee,
            &ankle
        ),

        position(Vec3f(),catAnkle, outAnkle, swivel, pelvis, hip, knee, ankle),
        catAnkle(Rotation()),
        outAnkle(true),
        swivel(0),
        pelvis(0),
        hip(0),
        knee(0),
        ankle(0)
    {
    }
    void outputToString(String& str);
};
class AnimByWrist : public Node, public MField7 {
    public:
class Position : public SFVec3f {
public:
    SFRotation& catWrist;
    SFBool& outWrist;
    SFFloat& swivel;
    SFPart& sholder;
    SFPart& elbow;
    SFPart& wrist;

    Position(const Vec3f& value, SFRotation& i_catWrist, SFBool& i_outWrist, SFFloat& i_swivel, SFPart& i_sholder, SFPart& i_elbow, SFPart& i_wrist):
        SFVec3f(value),
        catWrist(i_catWrist),
        outWrist(i_outWrist),
        swivel(i_swivel),
        sholder(i_sholder),
        elbow(i_elbow),
        wrist(i_wrist)
    {};
    void operator=(const Vec3f& par);
};
///////////////////
    int getType()const{return ANIMBYWRIST;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Position position;
    SFRotation catWrist;
    SFBool outWrist;
    SFFloat swivel;
    SFPart sholder;
    SFPart elbow;
    SFPart wrist;

    AnimByWrist():
        MField7(
            &position,
            &catWrist,
            &outWrist,
            &swivel,
            &sholder,
            &elbow,
            &wrist
        ),

        position(Vec3f(), catWrist, outWrist, swivel, sholder, elbow, wrist),
        catWrist(Rotation()),
        outWrist(true),
        swivel(0),
        sholder(0),
        elbow(0),
        wrist(0)
    {
    }
};

class RotFilter : public Node, public MField4 {
    public:
class InRot : public SFRotation {
    RotFilter& rf;
public:
    InRot(const Rotation& value, RotFilter& i_rf):
        SFRotation(value),
        rf(i_rf)
    {};
    void operator=(const Rotation& par);
};
///////////////////////////
    int getType()const{return ROTFILTER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    InRot inRot;
    SFVec3f axis;
    SFFloat angle;
    SFRotation outRot;

    RotFilter():
        MField4(
            &inRot,
            &axis,
            &angle,
            &outRot
        ),

        inRot(Rotation(),*this),
        axis(Vec3f(1,1,1)),
        angle(1),
        outRot(Rotation())
    {
    }
};
class RotMult : public Node, public MField3 {
    public:
class InA : public SFRotation {
private:
    RotMult& home;
public:
    InA(const Rotation& value, RotMult& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};
class InB : public SFRotation {
private:
    RotMult& home;
public:
    InB(const Rotation value, RotMult& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};
///////////////////////////
    int getType()const{return ROTMULT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    InA inA;
    InB inB;
    SFRotation catenation;

    RotMult():
        MField3(
            &inA,
            &inB,
            &catenation
        ),
        inA(Rotation(),*this),
        inB(Rotation(),*this),
        catenation(Rotation())
    {
    }
};
class MultiUser : public Node, public MField1 {
    public:
    int getType()const{return MULTIUSER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return nullPtr;}
#endif

    MFString ip;

    MultiUser():
        MField1(
            &ip
        ),

        ip(M_String())

    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
void processFields();
};
class IndexedTriangleSet : public VertexGeometry, public MField10 {
    public:
    int getType()const{return INDEXEDTRIANGLESET;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return _index[i];}
    void setFieldGroupIndex(const int &i, const int &val){_index[i]=val;}
    M_Int* getIndexPtr(){return nullPtr;}
#endif


    MFInt32 set_index;
    MFInt32 index;

    IndexedTriangleSet():
        MField10(
            &set_index,
            &coord,
            &normal,
            &color,
            &texCoord,
            &ccw,
            &normalPerVertex,
            &colorPerVertex,
            &solid,
            &index
        ),

        set_index(M_Int()),
        index(M_Int())
    {
#ifdef SEAMLESS3DGUI
        _index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int _index;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale, Mesh& geomSource, Render*& render, const Appearance* appearance);
#endif
    void genNormals();
    void genTransformedNormals(M_Art& partList);

    void addExtraCoords(Seamless* smls);
    void groupTrianglesByEffect(Art* art, M_Node& m_shape, int& triPos, Effect* lastEffect);
    void deleteUnwantedTriFromSource();
    void convertToTriangles(const IndexedFaceSet* ifs);
    void convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const;
};
class ProtoInterface : public Node, public DynamicMField {
    public:
    int getType()const{return PROTOINTERFACE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
#endif


    ProtoInterface()
    {
#ifdef SEAMLESS3DGUI
        index.length=20;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    M_Properties properties;

    void getFields(const String& str, int& i);

    const M_Properties& getProperties()const{
        return properties;
    }
    void toCPPString(String& str, bool newList=true);
};
class ProtoBody : public Node, public Progenitor, public MField1 {
    public:
    int getType()const{return PROTOBODY;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    MFNode children;

    ProtoBody():
        Progenitor(children),
        MField1(
            &children
        ),
        children(M_Node())
    {
    }
};

/*
class Proto : public Node, public MField {
    public:
    int getType()const{return PROTO;}
    int getFLen()const{return size();};
    int getFLen2(){return 1;};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i){return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}


    M_Node childrenMem;
    MFNode children;


    Proto():
        children(childrenMem)
    {
#ifdef SEAMLESS3DGUI
        index.length=20;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif

    M_Properties properties;
    //String containerField;

    String toString(bool newList=true);
    void getFields(const String& str, int& i);


    M_Properties& getProperties(){
        return properties;
    }
    Field* getField2(const int i){
        return &children;
    }


    void Proto::renderNode(Matrix parentMatrix, Vec3f parentColScale);
#ifdef SEAMLESS3DGUI
    void updateBuild();
    void senseTug(Matrix matrix, Matrix iMatrix, Vec3f& pt, bool& tugFlag);
#endif
};
*/
class ProtoDeclare : public Node, public PovRay, public MField2 {
    public:
    class SFProtoInterface : public SFNode {
    public:

        SFProtoInterface(const NodePtr& value):
            SFNode(value)
        {};
        ProtoInterface*& operator =(ProtoInterface*& par){
            refValue()=par;
            return par;
        }
        ProtoInterface* operator ->()const{
            return dynamic_cast<ProtoInterface*>(refValue().getRawPtr());
        }
        operator ProtoInterface*(){
            return dynamic_cast<ProtoInterface*>(refValue().getRawPtr());
        }
        operator ProtoInterfacePtr(){
            return dynamic_cast<ProtoInterface*>(refValue().p);
        }
    };
    class SFProtoBody : public SFNode {
    public:

        SFProtoBody(const NodePtr& value):
            SFNode(value)
        {};
        ProtoBody*& operator =(ProtoBody*& par){
            refValue()=par;
            return par;
        }
        ProtoBody* operator ->()const{
            return dynamic_cast<ProtoBody*>(refValue().getRawPtr());
        }
    };

//////////////////
    int getType()const{return PROTODECLARE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    SFProtoInterface protoInterface;
    SFProtoBody protoBody;

    ProtoDeclare():
        MField2(
            &protoInterface,
            &protoBody
        ),

        protoInterface(0),
        protoBody(0),

        lookupDeclarationsIndex(0)
    {
    }
    int lookupDeclarationsIndex;
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void toString(String& str, bool newList=true);
    void getFields(const String& str, int& i);
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);

#ifdef SEAMLESS3DGUI
    void updateBuild();
    void senseTug(Matrix matrix, Matrix iMatrix, TouchSensor* sensor);
    void catInverseRotation(Rotation& r);
    void resetRotations();
#endif
};


class ProtoInstance : public Node, public DynamicMField {
    public:
    int userNodeType;
    int getType()const{return userNodeType;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
#ifdef SEAMLESS3DGUI
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}
    M_Int index;
#endif

    ProtoInstance(const int userNodeType);
    void toString(String& str, bool newList=true);
    void toXMLString(String& str, bool newList, const char* containerField);
};

class Variable {
public:
    //virtual void boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr)=0;
    virtual int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr)=0;
    virtual void preDec_insert2(Function& f)=0;
    virtual void preInc_insert2(Function& f)=0;
    virtual void decode(Function& f, const String& str, int& i)=0;
    virtual int copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr)=0;
    virtual void copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr)=0;
    virtual void copyAddressToScratchPad(Function& f, const String& str, int& i)=0;
    virtual int getValueType()const=0;
};
class G_Variable: public Node, public Variable {
protected:
    void commonDecodeTextCell(Word* wt, W_Global* wg, const String& str, int& i, int& bracketCntr);
public:
    virtual void addToOutputScript()const=0;
    virtual void decodeTextCell(const String& str, int& i, int& bracketCntr)=0;
    M_<Word*> m_word;
    void toString(String& str, bool newList);
    void toTextCell(String& str);
};
template <class T> class V_<T,Single> : public G_Variable, public MField1 {
    FD<T,Single>* getInitValue()const;
public:
    int getType()const;
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    FD<T,Single> value;

    V_():
        MField1(
            &value
        ),

        value(T())

    {
    }
    //void boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void preDec_insert2(Function& f);
    void preInc_insert2(Function& f);
    void decode(Function& f, const String& str, int& i);
    int copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyAddressToScratchPad(Function& f, const String& str, int& i);
    int getValueType()const;
    void addToOutputScript()const;
    void getGlobalValue(const String& str, int& i, int& bracketCntr);
    void decodeTextCell(const String& str, int& i, int& bracketCntr);
};
template <class T> class V_<T,Multiple> : public G_Variable, public MField1 {
    FD<T,Multiple>* getInitValue()const;
public:
    int getType()const;
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    FD<T,Multiple> value;

    V_():
        MField1(
            &value
        ),

        value(M_<T>())

    {
#ifdef SEAMLESS3DGUI
        index.length=1;
#endif
    }
#ifdef SEAMLESS3DGUI
    M_Int index;
#endif
    //void boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void preDec_insert2(Function& f);
    void preInc_insert2(Function& f);
    void decode(Function& f, const String& str, int& i);
    int copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyAddressToScratchPad(Function& f, const String& str, int& i);
    int getValueType()const;
    void addToOutputScript()const;
    void getGlobalValue(const String& str, int& i, int& bracketCntr);
    void decodeTextCell(const String& str, int& i, int& bracketCntr);
};
template <class T, class S> Node* decodeInitializedGlobal(const String& str, int& i, int& bracketCntr);

class L_Variable : public Variable{
public:
    const String name;
    L_Variable(const String& i_name, int i_varIndex):
        name(i_name),
        varIndex(i_varIndex)
    {}
    virtual int getType()const=0;
    int varIndex;
};

template <class T> class V_<T,Single,Local> : public L_Variable{
    public:
    int getType()const;

    V_(const String& i_name, int i_varIndex):
        L_Variable(
            i_name,
            i_varIndex
        )
    {
    }
    //void boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void preDec_insert2(Function& f);
    void preInc_insert2(Function& f);
    void decode(Function& f, const String& str, int& i);
    int copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyAddressToScratchPad(Function& f, const String& str, int& i);
    int getValueType()const;
};
template <class T> class V_<T,Multiple,Local> : public L_Variable{
    public:
    int getType()const;

    V_(const String& i_name, int i_varIndex):
        L_Variable(
            i_name,
            i_varIndex
        )
    {
    }
    //void boolCopyVarToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    int copyToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void preDec_insert2(Function& f);
    void preInc_insert2(Function& f);
    void decode(Function& f, const String& str, int& i);
    int copyComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyFloatElementComponentToScratchPad(Function& f, const String& str, int& i, int& bracketCntr);
    void copyAddressToScratchPad(Function& f, const String& str, int& i);
    int getValueType()const;
};

class RadicalPatch : public Node, public MField0{
public:
    int getType()const{return RADICAL_PATCH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    RadicalPatch():
        MField0(
        ),
		//////
		active(false)
	{
	}

	M_Float weight;
	bool active;

	M_Vec3f diff;
	M_<NurbsPatch*> srcP;
	M_Int srcI;

	M_Vec2f mapVec;
	M_<NurbsPatch*> mapP;

	void collectPointsProject();
	void mapToPatchProject(NurbsPatch* p, int i, NurbsPatch* selPatch, const Matrix& iMatrix, const Rotation rot);

	void collectPoints();
	void mapToPatch(NurbsPatch* p, int i, NurbsPatch* selPatch);
};


class Radial_Tug : public Node, public MField1, public VisualAid{
public:
class Radius : public SFFloat{
public:
    Radial_Tug& home;
    Radius(const float& value, Radial_Tug& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};


    int getType()const{return RADIAL_TUG;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Radius radius;

    Radial_Tug():
        MField1(
            &radius
        ),

        radius(.2, *this),

		//////
		active(false)
    {
    }
    void houseKeep();
	bool active;
};

class Palette : public Node, public MField13{
    public:

class Color : public SFColor{
public:
    Palette& home;
    Color(const Color3f& value, Palette& home):
        SFColor(value),
        home(home)
    {};
    void operator=(const Color3f& par);
};

class Coords : public SFInt32{
public:
    Palette& home;
    Coords(const int& value, Palette& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};

class Weight : public SFFloat{
public:
    Palette& home;
    Weight(const float& value, Palette& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};

class Insert : public SFBool{
public:
    Palette& home;
    Insert(const bool& value, Palette& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Delete : public SFBool{
public:
    Palette& home;
    Delete(const bool& value, Palette& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

////////////////
    int getType()const{return PALETTE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    Insert insert;
    Delete _delete;
    Coords coords;
    Weight weight;
    SFBool autoWeightDivide;
    Color color;
    SFBool define;
    SFBool dropper;
    SFBool rectangle;
    SFBool fill;
    SFBool vertex;
    SFBool triangle;
    MFColor	cell;

    Palette():
        MField13(
            &insert,
            &_delete,
            &coords,
            &weight,
            &autoWeightDivide,
            &color,
            &define,
            &dropper,
            &rectangle,
            &fill,
            &vertex,
            &triangle,
            &cell
        ),
        insert(false, *this),
        _delete(false, *this),
        coords(0, *this),
        weight(0, *this),
        autoWeightDivide(true),
        color(Color3f(), *this),
        define(false),
        dropper(false),
        rectangle(false),
        fill(false),
        vertex(false),
        triangle(false),
        cell(M_Color3f(FLOAT_LIST(initCustomColorData)))
    {
        index.length = 1;
    }
    M_Int index;

    static const float initCustomColorData[3 * 15 * 6];
};

class CommonSpec : public Node, public MField3{
public:
    int getType()const{return COMMON_SPEC;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    SFBool mirror;
    MFColor	color;
    MFString alternative;

    CommonSpec():
        MField3(
            &mirror,
            &color,
            &alternative
        ),

        mirror(false),
        color(M_Color3f()),
        alternative(M_String())
    {
        index.length = 2;
    }
    M_Int index;
};

class Brush : public Node, public MField4, public VisualAid{
public:
class Radius : public SFFloat{
public:
    Brush& home;
    Radius(const float& value, Brush& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};


    int getType()const{return BRUSH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFFloat fade;
    SFFloat opacity;
    Radius radius;
    SFBool smudge;

    Brush():
        MField4(
            &fade,
            &opacity,
            &radius,
            &smudge
        ),

        fade(.5),
        opacity(1),
        radius(.2, *this),
        smudge(false),
//////
        pitch(0)
    {
    }
    M_Color3f storeColor;
    M_Pix24 storePixel;
    M_Float storeOpacity;
    int pitch;
    void houseKeep();
};

class Fill : public Node, public MField4{
public:
    int getType()const{return FILL;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool model;
    SFBool part;
    SFBool patch;
    SFFloat tolerance;

    Fill():
        MField4(
            &model,
            &part,
            &patch,
            &tolerance
        ),
            model(true),
            part(false),
            patch(false),
            tolerance(0)
    {
    }
    M_Bool flagTriangle;
    M_Bool flagCoord;
};

class ColorPicker : public Node, public MField1{
public:
    int getType()const{return COLORPICKER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool blend;

    ColorPicker():
        MField1(
            &blend
        ),

        blend(true)
    {
    }
};

class Settings : public Node, public MField31{
public:
class ShowToolBar : public SFBool {
public:
    ShowToolBar(const bool& value):
        SFBool(value)
    {};
    void operator=(const bool& par);
};
class ExtendNodeBar : public SFBool {
public:
    ExtendNodeBar(const bool& value):
        SFBool(value)
    {};
    void operator=(const bool& par);
};
//class AutoOpen : public SFBool{
//public:
//	AutoOpen(const bool& value):
//		SFBool(value)
//	{};
//	void operator=(const bool& par);
//};
class SaveGZipped : public SFBool{
public:
    SaveGZipped(const bool& value):
        SFBool(value)
    {};
    void operator=(const bool& par);
};
class FreeCPU : public SFInt32{
public:
    FreeCPU(const int& value):
        SFInt32(value)
    {};
    void operator=(const int& par);
};

class LoadColorCodePresets : public SFBool{
public:
    LoadColorCodePresets(const bool& value):
        SFBool(value)
    {};
    void operator=(const bool& par);
};
///////////////////////////
    int getType()const{return SETTINGS;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    //AutoOpen autoOpen;
    SFBool screenshotWindows;
    SFBool screenshotMouse;
	SFBool multiPointLever;
    SaveGZipped saveGZipped;
    SFBool invertWindowColor;
    SFBool showFPS;
    ShowToolBar showToolBar;
    ExtendNodeBar extendNodeBar;
    LoadColorCodePresets loadColorCodePresets;
    MFString searchEngine;
    SFBool backfaceIgnore;
    //SFBool perspective;
    SFFloat incDecRes;
    SFFloat incDecResCtrl;
    SFFloat backupDelay;
    FreeCPU freeCPU;
    SFBool record;
    //SFBool shiftDragTranslate;
    //SFFloat tugSlope;
    //SFFloat tugRadius;
    SFBool multiPartTug;
    SFString htmlBrowser;
    SFFloat version;
    MFString file;
    SFBool sortByDate;
    SFVec2f win3dPos;
    SFVec2f win3dSize;
    SFVec2f sceneTreePos;
    SFVec2f sceneTreeSize;
    SFVec2f nodeWinPos;
    SFVec2f toolBarPos;
    SFFloat roundCoordsSave;
    SFBool wacomPackets;
    SFNode palette;
    MFNode commonSpec;

    Settings():
        MField31(
            //&autoOpen,
            &screenshotWindows,
            &screenshotMouse,
			&multiPointLever,
            &saveGZipped,
            &invertWindowColor,
            &showFPS,
            &showToolBar,
            &extendNodeBar,
            &loadColorCodePresets,
            &searchEngine,
            &backfaceIgnore,
            //&perspective,
            &incDecRes,
            &incDecResCtrl,
            &backupDelay,
            &freeCPU,
            &record,
            //&shiftDragTranslate,
            //&tugSlope,
            //&tugRadius,
            &multiPartTug,
            &htmlBrowser,
            &version,
            &file,
            &sortByDate,
            &win3dPos,
            &win3dSize,
            &sceneTreePos,
            &sceneTreeSize,
            &nodeWinPos,
            &toolBarPos,
            &roundCoordsSave,
            &wacomPackets,
            &palette,
            &commonSpec
        ),
        //autoOpen(false),
        screenshotWindows(false),
        screenshotMouse(false),
		multiPointLever(true),
        saveGZipped(true),
        invertWindowColor(false),
        showFPS(false),
        showToolBar(true),
        extendNodeBar(false),
        loadColorCodePresets(false),
        searchEngine(
            M_String(
                String("google|http://www.google.com/search?q="),
                String("wikipedia|http://en.wikipedia.org/wiki/index.php?title=Special%3ASearch&search="),
                String("youtube|http://www.youtube.com/results?search_type=&search_query=")
            )
        ),
        backfaceIgnore(true),
        //perspective(false),
        incDecRes(.01),
        incDecResCtrl(.001),
        backupDelay(.4),
        freeCPU(0),
        record(false),
        //shiftDragTranslate(false),
        //tugSlope(1),
        //tugRadius(.2),
        multiPartTug(true),
        htmlBrowser(String()),
        version(0),
        file(M_String()),
        sortByDate(false),
        win3dPos(Vec2f()),
        win3dSize(Vec2f(1280,720)),
        sceneTreePos(Vec2f(500,25)),
        sceneTreeSize(Vec2f(300,480)),
        nodeWinPos(Vec2f(400,400)),
        toolBarPos(Vec2f(0, 670)),
        roundCoordsSave(6),
        wacomPackets(true),
        palette(0),
        commonSpec(M_Node())
    {
        index.length = 1;
    }
    M_Int index;
    void writeINIFile();
    static void loadINI();
};
class StretchCurve : public Node, public MField3 {
    public:
class DoIt : public SFBool {
public:
    StretchCurve& home;

    DoIt(const bool& par, StretchCurve& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
////////////////
    int getType()const{return STRETCHCURVE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    DoIt doIt;
    SFVec2f at;
    SFVec2f add;

    StretchCurve():
        MField3(
            &doIt,
            &at,
            &add
        ),

        doIt(false, *this),
        at(Vec2f(1,1)),
        add(Vec2f(0,.01))

    {
    }
};
class OpenNetFile : public Node, public MField12{
    public:
class WorldList : public SFBool {
public:
    OpenNetFile& home;

    WorldList(const bool& par, OpenNetFile& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Go : public SFBool {
public:

    Go(const bool& par):
        SFBool(par)
    {};
    void operator=(const bool& par);
};
    int getType()const{return OPENNETFILE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    WorldList worldList;
    SFString worldUrl;
    SFString serverUrl;
    SFString nickname;
    SFString avatarUrl;
    SFFloat landOffset;
    SFBool male;
    SFString voice;
    SFString alarmFile;
    Go go;
    SFVec2f chatPos;
    SFVec2f chatSize;

    OpenNetFile():
        MField12(
            &worldList,
            &worldUrl,
            &serverUrl,
            &nickname,
            &avatarUrl,
            &landOffset,
            &male,
            &voice,
            &alarmFile,
            &go,
            &chatPos,
            &chatSize
        ),

        worldList(false, *this),
        worldUrl(init_worldUrl),
        serverUrl(init_serverUrl),
        nickname(String()),
        avatarUrl(init_avatarUrl),
        landOffset(0),
        male(false),
        voice(""),
        alarmFile(""),
        go(false),
        chatPos(Vec2f(175, 83)),
        chatSize(Vec2f(400, 132)),
////
        worldListRefreshed(false),
        worldListRefreshing(false)
    {
    }
    static const char* init_worldUrl;
    static const char* init_serverUrl;
    static const char* init_avatarUrl;

    static M_String worldListUrls;
    static M_String worldListTitles;
    static M_String worldListOccupants;

    int clickUrl(const Vec2i ptCursor);
    void printWorldsOpen(const int dstWidth, const int dstHeight, int* dstPix);
    bool worldListRefreshed;
    bool worldListRefreshing;
    void hoverUrls(const Vec2i ptCursor);
    int getWorldList(String& str);
};

class CPContigElement{
public:
    Node* cpNode;
    int group;
    CPContigElement():
        cpNode(0),
        group(-1)
    {
    }
    CPContigElement(Node* cpNode):
        cpNode(cpNode),
        group(-1)
    {
    }
    CPContigElement(Node* cpNode, int group):
        cpNode(cpNode),
        group(group)
    {
    }
};


NurbsPatch::CurveType checkNurbsType(int dim, const M_Float& knot);

template<typename T>class STSFNode : public SFNode {
    public:
    T& home;
    STSFNode(const NodePtr& par, T& home):
        SFNode(par),
        home(home)
    {};
    Node* operator = (Node* par){
        refValue()=par;
        home.processFields();
        return par;
    }
    T* operator=(const int){
        refValue() = 0;
        home.processFields();
        return 0;
    }
    T* operator ->(){
        return reinterpret_cast<T*>(p);
    }
};
/*
class SphereBuild : public Build, public MField12, public VisualAid{
public:
class HSFPart : public SFNode{
public:
    SphereBuild& home;
    HSFPart( const NodePtr& par, SphereBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    const ArtPtr& refValue()const{
        return *((const ArtPtr*)&SFNode::refValue());
    }

    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};

class Scale : public SFVec3f{
    public:
    SphereBuild& home;
    Scale(const Vec3f& value, SphereBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    SphereBuild& home;
    HRotation(const Rotation& value, SphereBuild& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    SphereBuild& home;
    Translation(const Vec3f& value, SphereBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

    int getType()const{return SPHEREBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    TSFFloat radius;
    TSFInt staves;
    TSFInt bands;
    TSFColor3f color;
    TSFBool yCentred;
    Scale scale;
    HRotation rotation;
    Translation translation;
    TSFBool descendants;
    HSFPart part;
    STSFNode<SphereBuild> textureMapper;
    Setup setup;

    SphereBuild():
        Build(setup),
        MField12(
            &radius,
            &staves,
            &bands,
            &color,
            &yCentred,
            &scale,
            &rotation,
            &translation,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        radius(.25, *this),
        staves(16, *this),
        bands(8, *this),
        color(Color3f(1,1,1), *this),
        yCentred(true, *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(0, *this),
        setup(0, *this),

        patch(new NurbsPatch),
        ctrlStaves(4),
        ctrlBands(3)
    {
        patch->beginSP.refValue() = true;
        patch->endSP.refValue() = true;
    }

    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();

    void processFields();
    void houseKeep();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();
    SphereBuild* getMirrorSphere()const;
    int ctrlStaves;
    int ctrlBands;
    void setStemValues(Stem* stem);

protected:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    NurbsPatchPtr patch;
};
*/

class SphereBuild : public Build, public MField14, public VisualAid{
public:
class Mirror : public SFBool{
    public:
    SphereBuild& home;
    Mirror(const bool& value, SphereBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class HSFPart : public SFNode{
public:
    SphereBuild& home;
    HSFPart( const NodePtr& par, SphereBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    const ArtPtr& refValue()const{
        return *((const ArtPtr*)&SFNode::refValue());
    }

    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};

class Radius : public SFFloat{
    public:
    SphereBuild& home;
    Radius(const float& value, SphereBuild& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};

class Staves : public SFInt32{
    public:
    SphereBuild& home;
    Staves(const int& value, SphereBuild& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};

class Bands : public SFInt32{
    public:
    SphereBuild& home;
    Bands(const int& value, SphereBuild& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};

class YCentred : public SFBool{
    public:
    SphereBuild& home;
    YCentred(const bool& value, SphereBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Scale : public SFVec3f{
    public:
    SphereBuild& home;
    Scale(const Vec3f& value, SphereBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    SphereBuild& home;
    HRotation(const Rotation& value, SphereBuild& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    SphereBuild& home;
    Translation(const Vec3f& value, SphereBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

    int getType()const{return SPHEREBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Radius radius;
    Staves staves;
    Bands bands;
    TSFColor3f color;
    YCentred yCentred;
    Scale scale;
    HRotation rotation;
    Translation translation;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    STSFNode<SphereBuild> textureMapper;
    Setup setup;

    SphereBuild():
        Build(setup),
        MField14(
            &radius,
            &staves,
            &bands,
            &color,
            &yCentred,
            &scale,
            &rotation,
            &translation,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        radius(.25, *this),
        mirror(false, *this),
        insideOut(false, *this),
        staves(16, *this),
        bands(8, *this),
        color(Color3f(1,1,1), *this),
        yCentred(true, *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(0, *this),
        setup(0, *this),

        stem(new Stem),
        patch(new NurbsPatch),
        colorSweep(new ColorSweep),
        ctrlStaves(4),
        ctrlBands(3),
        mirrorContainer(0)
    {
        patch->container = this;
        stem->xDistance.refValue()=1;
        stem->xCentred.refValue()=false;
        stem->yDistance.refValue()=1;
        patch->beginSP.refValue()=true;
        patch->endSP.refValue()=true;
        patch->stem.refValue()+=stem.p;
    }
    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void processFields();
    void houseKeep();

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();
	void joinStitchedEdges();
    SphereBuild* getMirrorSphere()const;
    int ctrlStaves;
    int ctrlBands;

    SphereBuild* mirrorContainer;
protected:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    StemPtr stem;
    NurbsPatchPtr patch;
    ColorSweepPtr colorSweep;
};


class CylinderBuild : public Build, public MField21, public VisualAid{
public:
class TopHeight : public SFFloat {
public:
    CylinderBuild& home;
    TopHeight(const float& value, CylinderBuild& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};
class BaseHeight : public SFFloat {
public:
    CylinderBuild& home;
    BaseHeight(const float& value, CylinderBuild& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};


class Scale : public SFVec3f{
    public:
    CylinderBuild& home;
    Scale(const Vec3f& value, CylinderBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    CylinderBuild& home;
    HRotation(const Rotation& value, CylinderBuild& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    CylinderBuild& home;
    Translation(const Vec3f& value, CylinderBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class Mirror : public SFBool{
    public:
    CylinderBuild& home;
    Mirror(const bool& value, CylinderBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class HSFPart : public SFNode{
public:
    CylinderBuild& home;
    HSFPart( const NodePtr& par, CylinderBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
    int getType()const{return CYLINDERBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    TSFFloat radius;
    TSFFloat height;
    TSFInt staves;
    TSFInt bands;
    TSFBool topCap;
    TopHeight topHeight;
    TSFInt topBands;
    TSFBool baseCap;
    BaseHeight baseHeight;
    TSFInt baseBands;
    TSFColor3f color;
    TSFBool	yCentred;
    Scale scale;
    HRotation rotation;
    Translation translation;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    CylinderBuild():
        Build(setup),
        MField21(
            &height,
            &radius,
            &staves,
            &bands,
            &topCap,
            &baseCap,
            &topHeight,
            &baseHeight,
            &topBands,
            &baseBands,
            &color,
            &yCentred,
            &scale,
            &rotation,
            &translation,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),

        radius(.25, *this),
        height(.5, *this),
        staves(16, *this),
        bands(4, *this),
        topCap(true, *this),
        baseCap(true, *this),
        topHeight(0, *this),
        baseHeight(0, *this),
        topBands(4, *this),
        baseBands(4, *this),
        color(Color3f(1,1,1), *this),
        yCentred(false, *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        mirror(false, *this),
        insideOut(false, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

        basePatch(new NurbsPatch),
        patch(new NurbsPatch),
        topPatch(new NurbsPatch),
        colorSweep(new ColorSweep),
        ctrlStaves(4),
        ctrlBands(1),
        mirrorContainer(0)
    {
        basePatch->container = this;
        patch->container = this;
        topPatch->container = this;

        Stem* stem = new Stem;
        Stem* topStem = new Stem;
        Stem* baseStem = new Stem;


        baseStem->xDistance.refValue() = 1;
        baseStem->xCentred.refValue() = false;
        baseStem->bands.refValue().length = 4;
        baseStem->beginSP.refValue() = true;

        stem->xDistance.refValue() = 1;
        stem->xCentred.refValue() = false;
        stem->bands.refValue().length = 4;

        topStem->xDistance.refValue() = 1;
        topStem->xCentred.refValue() = false;
        topStem->bands.refValue().length = 4;
        topStem->endSP.refValue() = true;

        basePatch->stem.refValue() += baseStem;
        patch->stem.refValue() += stem;
        topPatch->stem.refValue() += topStem;

        basePatch->uJoinEnds.refValue() = false;
        patch->uJoinEnds.refValue() = false;
        topPatch->uJoinEnds.refValue() = false;

        m_patch += basePatch;
        m_patch += patch;
        m_patch += topPatch;
    }
    void doBuild();
    NurbsPatch* getPatch(){return m_patch[0].p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return m_patch.length;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void processFields();
    void houseKeep();

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();

    int ctrlStaves;
    int ctrlBands;
    void processBaseCap(NurbsPatch* patch, int uDim, Matrix& matrix);
    void processShaft(NurbsPatch* patch, int uDim, Matrix& matrix);
    void processTopCap(NurbsPatch* patch, int uDim, Matrix& matrix);

    void convertToPatch(int nodeLine, M_Node& stemStore, M_Node& mirrorStemStore);
    void convertToPatch(ConvertToPatch* convertToPatch, M_Node& stemStore);
	void joinStitchedEdges();

    CylinderBuild* mirrorContainer;
    M_<NurbsPatchPtr> m_patch;
private:

    NurbsPatchPtr basePatch;
    NurbsPatchPtr patch;
    NurbsPatchPtr topPatch;
    ColorSweepPtr colorSweep;

    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;
};


class ConeBuild : public Build, public MField20, public VisualAid{
public:

class TopRad : public SFFloat {
public:
    ConeBuild& home;
    TopRad(const float& value, ConeBuild& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};

class TopCap : public SFBool{
    public:
    ConeBuild& home;
    TopCap(const bool& value, ConeBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class BaseCap : public SFBool{
    public:
    ConeBuild& home;
    BaseCap(const bool& value, ConeBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class BaseRad : public SFFloat {
public:
    ConeBuild& home;
    BaseRad(const float& value, ConeBuild& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};

class Scale : public SFVec3f{
    public:
    ConeBuild& home;
    Scale(const Vec3f& value, ConeBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    ConeBuild& home;
    HRotation(const Rotation& value, ConeBuild& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    ConeBuild& home;
    Translation(const Vec3f& value, ConeBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class Mirror : public SFBool{
    public:
    ConeBuild& home;
    Mirror(const bool& value, ConeBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class HSFPart : public SFNode{
public:
    ConeBuild& home;
    HSFPart( const NodePtr& par, ConeBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
    int getType()const{return CONEBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    TSFFloat height;
    TopRad topRad;
    TopCap topCap;
    TSFInt staves;
    TSFInt bands;
    TSFInt topBands;
    BaseRad baseRad;
    BaseCap baseCap;
    TSFInt baseBands;
    TSFColor3f color;
    TSFBool	yCentred;
    Scale scale;
    HRotation rotation;
    Translation translation;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    ConeBuild():
        Build(setup),
        MField20(
            &height,
            &topRad,
            &baseRad,
            &staves,
            &bands,
            &topCap,
            &baseCap,
            &topBands,
            &baseBands,
            &color,
            &yCentred,
            &scale,
            &rotation,
            &translation,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),

        height(.5, *this),
        topRad(0, *this),
        baseRad(.25, *this),
        staves(16, *this),
        bands(8, *this),
        topCap(false, *this),
        baseCap(true, *this),
        topBands(4, *this),
        baseBands(4, *this),
        color(Color3f(1,1,1), *this),
        yCentred(false, *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        mirror(false, *this),
        insideOut(false, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

        basePatch(new NurbsPatch),
        patch(new NurbsPatch),
        topPatch(new NurbsPatch),
        colorSweep(new ColorSweep),
        ctrlStaves(4),
        ctrlBands(2),
        mirrorContainer(0)
    {
        basePatch->container = this;
        patch->container = this;
        topPatch->container = this;

        Stem* stem = new Stem;
        Stem* topStem = new Stem;
        Stem* baseStem = new Stem;


        baseStem->xDistance.refValue() = 1;
        baseStem->xCentred.refValue() = false;
        baseStem->bands.refValue().length = 4;
        baseStem->beginSP.refValue() = true;

        stem->xDistance.refValue() = 1;
        stem->xCentred.refValue() = false;
        stem->bands.refValue().length = 4;

        topStem->xDistance.refValue() = 1;
        topStem->xCentred.refValue() = false;
        topStem->bands.refValue().length = 4;
        topStem->endSP.refValue() = false;

        basePatch->stem.refValue() += baseStem;
        patch->stem.refValue() += stem;
        topPatch->stem.refValue() += topStem;

        basePatch->uJoinEnds.refValue() = false;
        patch->uJoinEnds.refValue() = false;
        topPatch->uJoinEnds.refValue() = false;

        m_patch += basePatch;
        m_patch += patch;
        m_patch += topPatch;
    }
    void doBuild();
    NurbsPatch* getPatch(){return m_patch[0].p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return m_patch.length;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void processFields();
    void houseKeep();

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();

    void processBaseCap(NurbsPatch* patch, int uDim, Matrix& matrix);
    void processShaft(NurbsPatch* patch, int uDim, Matrix& matrix);
    void processTopCap(NurbsPatch* patch, int uDim, Matrix& matrix);

    void convertToPatch(int nodeLine, M_Node& stemStore, M_Node& mirrorStemStore);
    void convertToPatch(ConvertToPatch* convertToPatch, M_Node& stemStore);
	void joinStitchedEdges();

    int ctrlStaves;
    int ctrlBands;

    ConeBuild* mirrorContainer;
    M_<NurbsPatchPtr> m_patch;
private:
    NurbsPatchPtr basePatch;
    NurbsPatchPtr patch;
    NurbsPatchPtr topPatch;
    ColorSweepPtr colorSweep;

    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    static BezierLathePtr lathe;
};

class TorusBuild : public Build, public MField15, public VisualAid{
public:
class HoleRad : public SFFloat{
    public:
    TorusBuild& home;
    HoleRad(const float& value, TorusBuild& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};
class TubeRad : public SFFloat{
    public:
    TorusBuild& home;
    TubeRad(const float& value, TorusBuild& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};
class Staves : public SFInt32{
    public:
    TorusBuild& home;
    Staves(const int& value, TorusBuild& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};
class Bands : public SFInt32{
    public:
    TorusBuild& home;
    Bands(const int& value, TorusBuild& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};
class YCentred : public SFBool{
    public:
    TorusBuild& home;
    YCentred(const bool& value, TorusBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Scale : public SFVec3f{
    public:
    TorusBuild& home;
    Scale(const Vec3f& value, TorusBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    TorusBuild& home;
    HRotation(const Rotation& value, TorusBuild& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    TorusBuild& home;
    Translation(const Vec3f& value, TorusBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};


class Mirror : public SFBool{
    public:
    TorusBuild& home;
    Mirror(const bool& value, TorusBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class HSFPart : public SFNode{
public:
    TorusBuild& home;
    HSFPart( const NodePtr& par, TorusBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
    int getType()const{return TORUSBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    HoleRad holeRad;
    TubeRad tubeRad;
    Staves staves;
    Bands bands;
    TSFColor3f color;
    YCentred yCentred;
    Scale scale;
    HRotation rotation;
    Translation translation;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    TorusBuild():
        Build(setup),
        MField15(
            &holeRad,
            &tubeRad,
            &staves,
            &bands,
            &color,
            &yCentred,
            &scale,
            &rotation,
            &translation,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),

        holeRad(.1, *this),
        tubeRad(.1, *this),
        staves(32, *this),
        bands(16, *this),
        color(Color3f(1,1,1), *this),
        yCentred(false, *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        mirror(false, *this),
        insideOut(false, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

        patch(new NurbsPatch),

        ctrlStaves(6),
        ctrlBands(4),
        mirrorContainer(0)
    {
        patch->uJoinEnds.refValue() = false;
        patch->container = this;
    }
    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();

    void processFields();
    void houseKeep();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();
    void setStemValues(Stem* stem);
	void joinStitchedEdges();

    int ctrlStaves;
    int ctrlBands;

    TorusBuild* mirrorContainer;
    StemPtr stem;
protected:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    NurbsPatchPtr patch;
};
/*
class BoxBuild : public Build, public MField9, public VisualAid{
public:
class HSFPart : public SFNode{
public:
    BoxBuild& home;
    HSFPart( const NodePtr& par, BoxBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
    int getType()const{return BOXBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    TSFColor3f color;
    TSFBool yCentred;
    TSFVec3f scale;
    TSFRotation rotation;
    TSFVec3f translation;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    BoxBuild():
        Build(setup),
        MField9(
            &color,
            &yCentred,
            &scale,
            &rotation,
            &translation,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),

        color(Color3f(1,1,1), *this),
        yCentred(false, *this),
        scale(Vec3f(.25,.25,.25), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

        ctrlStaves(1),
        ctrlBands(1),
        ctrlZStaves(1)
    {
        for(int k = 6; --k>=0;){
            NurbsPatch* patch = new NurbsPatch;
            patch->uJoinEnds.refValue()=false;
            m_patch += patch;
        }
    }
    void doBuild();
    NurbsPatch* getPatch(){return m_patch[0].p;}
    int numOfPatches(){return 6;}
    bool isCaged(){return m_patch[0]->cage;}
    void setCage(bool state);
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();

    void processFields();
    void houseKeep();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();

    int ctrlStaves;
    int ctrlBands;
    int ctrlZStaves;

    M_<NurbsPatchPtr> m_patch;
protected:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

};
*/


class BoxBuild : public Build, public MField13, public VisualAid{
public:

class XCentred : public SFBool{
    public:
    BoxBuild& home;
    XCentred(const bool& value, BoxBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class YCentred : public SFBool{
    public:
    BoxBuild& home;
    YCentred(const bool& value, BoxBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class ZCentred : public SFBool{
    public:
    BoxBuild& home;
    ZCentred(const bool& value, BoxBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Scale : public SFVec3f{
    public:
    BoxBuild& home;
    Scale(const Vec3f& value, BoxBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    BoxBuild& home;
    HRotation(const Rotation& value, BoxBuild& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    BoxBuild& home;
    Translation(const Vec3f& value, BoxBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class Mirror : public SFBool{
    public:
    BoxBuild& home;
    Mirror(const bool& value, BoxBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class HSFPart : public SFNode{
public:
    BoxBuild& home;
    HSFPart( const NodePtr& par, BoxBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
    int getType()const{return BOXBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    TSFColor3f color;
    XCentred xCentred;
    YCentred yCentred;
    ZCentred zCentred;
    Scale scale;
    HRotation rotation;
    Translation translation;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    BoxBuild():
        Build(setup),
        MField13(
            &color,
            &xCentred,
            &yCentred,
            &zCentred,
            &scale,
            &rotation,
            &translation,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        color(Color3f(1,1,1), *this),
        xCentred(true, *this),
        yCentred(false, *this),
        zCentred(true, *this),
        scale(Vec3f(.25,.25,.25), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        mirror(false, *this),
        insideOut(false, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

        colorSweep(new ColorSweep),

        ctrlStaves(1),
        ctrlBands(1),
        ctrlZStaves(1),
        mirrorContainer(0)
    {
        for(int k = 6; --k>=0;){
            Stem* stem = new Stem;
            stem->uWrap.refValue() = false;
            stem->xDistance.refValue() = 1;
            stem->yDistance.refValue() += 1;
            stem->xCentred.refValue() = false;
            stem->bands.refValue() += 16;
            stem->staves.refValue() = 16;
            NurbsPatch* patch = new NurbsPatch;
            patch->container = this;
            patch->uJoinEnds.refValue() = false;
            patch->stem.refValue()+=stem;
            m_patch += patch;
        }
    }
    void doBuild();

    NurbsPatch* getPatch(){return m_patch[0].p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return 6;}
    bool isCaged(){return m_patch[0]->cage;}
    void setCage(bool state);
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();

    void processFields();
    void houseKeep();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();

    void setMirrorObjects(BoxBuild* mirrorContainer);
    void convertToPatch(ConvertToPatch* convertToPatch, M_Node& stemStore);
    void convertToPatch(int nodeLine, M_Node& stemStore, M_Node& mirrorStemStore);
	void joinStitchedEdges();

    int ctrlStaves;
    int ctrlBands;
    int ctrlZStaves;
    BoxBuild* mirrorContainer;
    M_<NurbsPatchPtr> m_patch;
protected:
    ColorSweepPtr colorSweep;
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;
};


class RectangleBuild : public Build, public MField20, public VisualAid{
    public:

class Scale : public SFVec3f{
    public:
    RectangleBuild& home;
    Scale(const Vec3f& value, RectangleBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    RectangleBuild& home;
    HRotation(const Rotation& value, RectangleBuild& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    RectangleBuild& home;
    Translation(const Vec3f& value, RectangleBuild& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};


class Mirror : public SFBool{
    public:
    RectangleBuild& home;
    Mirror(const bool& value, RectangleBuild& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class HSFPart : public SFNode{
public:
    RectangleBuild& home;
    HSFPart( const NodePtr& par, RectangleBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
    int getType()const{return RECTANGLEBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    TSFFloat xDistance;
    TSFFloat yDistance;
    TSFInt staves;
    TSFInt bands;
    TSFBool endSP;
    TSFBool beginSP;
    TSFColor3f color;
    TSFBool xCentred;
    TSFBool yCentred;
    TSFBool xNegSide;
    TSFBool yNegSide;
    Scale scale;
    HRotation rotation;
    Translation translation;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    STSFNode<RectangleBuild> textureMapper;
    Setup setup;

    RectangleBuild():
        Build(setup),
        MField20(
            &xDistance,
            &yDistance,
            &staves,
            &bands,
            &endSP,
            &beginSP,
            &color,
            &xCentred,
            &yCentred,
            &xNegSide,
            &yNegSide,
            &scale,
            &rotation,
            &translation,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),

        xDistance(1, *this),
        yDistance(1, *this),
        staves(4, *this),
        bands(4, *this),
        endSP(false, *this),
        beginSP(false, *this),
        color(Color3f(1,1,1), *this),
        xCentred(true, *this),
        yCentred(false, *this),
        xNegSide(false, *this),
        yNegSide(false, *this),

        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        mirror(false, *this),
        insideOut(false, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(0, *this),
        setup(0, *this),

        stem(new Stem),
        patch(new NurbsPatch),
        colorSweep(new ColorSweep),

        ctrlStaves(1),
        ctrlBands(1),
        mirrorContainer(0)
    {
        patch->container = this;
        patch->uJoinEnds.refValue() = false;

        patch->stem.refValue() += stem.p;
        stem->xDistance.refValue() = 1;
        stem->yDistance.refValue() += 1;
        stem->xCentred.refValue() = false;
        stem->bands.refValue().length = 1;
    }
    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void processFields();
    void houseKeep();

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();
	void joinStitchedEdges();

    int ctrlStaves;
    int ctrlBands;
    RectangleBuild * mirrorContainer;
protected:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    StemPtr stem;
    NurbsPatchPtr patch;
    ColorSweepPtr colorSweep;
};

class BezierLatheBuild : public Build, public Bezier, public MField14, public VisualAid{
public:
class Point : public MFVec2f {
public:
    BezierLatheBuild& home;
    Point( const M_Vec2f& value, BezierLatheBuild& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par);
    void resize(int newLen);
};
class DBands : public MFInt32 {
public:
    BezierLatheBuild& home;
    DBands( const M_Int& value, BezierLatheBuild& home):
        MFInt32(value),
        home(home)
    {};
    void operator=(const M_Int& par);
};
class HSFPart : public SFNode{
public:
    BezierLatheBuild& home;
    HSFPart( const NodePtr& par, BezierLatheBuild& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
    int getType()const{return BEZIERLATHEBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return nullPtr;}

    Point point;
    TMFBool	fix;
    DBands dbands;
    MFInt32 nopfs;
    TMFInt bands;
    TSFInt staves;
    TSFColor3f color;
    TSFVec3f scale;
    TSFRotation rotation;
    TSFVec3f translation;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    BezierLatheBuild():
        Build(setup),
        MField14(
            &point,
            &fix,
            &dbands,
            &nopfs,
            &bands,
            &staves,
            &color,
            &scale,
            &rotation,
            &translation,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        point(M_Vec2f(), *this),
        fix(M_Bool(), *this),
        dbands(16, *this),
        nopfs(M_Int()),
        bands(16, *this),

        staves(16, *this),
        color(Color3f(1,1,1), *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

        Bezier(point, index, fix, nopfs),

        colorSweep(new ColorSweep),

        ctrlStaves(4),
        ctrlBands(3)
    {
        index.length = 3;
    }
    M_Int index;
    void doBuild();
    NurbsPatch* getPatch(){return m_patch[0].p;}
    int numOfPatches(){return m_patch.length;}
    bool isCaged(){return m_patch[0]->cage;}
    void setCage(bool state);
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();

    void processFields();
    void houseKeep();
    void centerViewToControlPoint();

    void addControlPoint(int& newLen);

    bool onActionForFix(bool state);
    void removeControlPoint(int& newLen);
    void drag(Vec3f pt){Bezier::drag(pt);}
    using Bezier::drag;
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void convertToPatch();
    bool common_onActionForFix(bool state);
    void common_removeControlPoint(int& newLen);

    int ctrlStaves;
    int ctrlBands;

protected:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    M_<NurbsPatchPtr> m_patch;
    ColorSweepPtr colorSweep;
    JoinVertexPtr joinVertex;
};

class MirrorEdit : public Node, public MField1{
public:
    int getType()const{return MIRROREDIT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    PartChild part;

    MirrorEdit():
        MField1(
            &part
        ),
        part(M_Node())
    {}
};
class MirrorGroup : public Node, public MField1{
public:
    int getType()const{return MIRRORGROUP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    MFNode mirrorEdit;

    MirrorGroup():
        MField1(
            &mirrorEdit
        ),
        mirrorEdit(M_Node())
    {}
};
class PolyToNurbs : public Build, public MField16, public VisualAid{
public:

class Cycle : public SFBool {
public:
    PolyToNurbs& home;

    Cycle(const bool& par, PolyToNurbs& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};


class Spread : public SFBool {
public:
    PolyToNurbs& home;

    Spread(const bool& par, PolyToNurbs& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};


class MimicOriginalIndex : public SFBool{
public:
    PolyToNurbs& home;

    MimicOriginalIndex(const bool& par, PolyToNurbs& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return POLYTONURBS;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    static const int initPolyToNurbs_random[127];

	Cycle	cycle;
    SFFloat sumDiff;
    SFFloat averageDiff;
    SFBool	bond;
    SFFloat coordChange;
    SFBool	evolve;
    SFFloat pointChange;
    Spread spread;
    SFInt32 spreadRes;
    SFInt32 offset;

    SFInt32 selOriginalVertex;
    SFInt32 selMimicVertex;
    MimicOriginalIndex mimicOriginalIndex;

    SFPart original;
    SFNode mimic;
    Setup setup;

    PolyToNurbs():
        Build(setup),
        MField16(
			&cycle,
            &sumDiff,
            &averageDiff,
            &bond,
            &coordChange,
            &evolve,
            &pointChange,
            &spread,
            &spreadRes,
            &offset,

            &selOriginalVertex,
            &selMimicVertex,
            &mimicOriginalIndex,

            &original,
            &mimic,
            &setup
        ),
		cycle(false, *this),
        sumDiff(0),
        averageDiff(0),
        bond(false),
        coordChange(.001),
        evolve(false),
        pointChange(.001),
        spread(false, *this),
        spreadRes(100),
        offset(0),

        selOriginalVertex(0),
        selMimicVertex(0),
        mimicOriginalIndex(0, *this),

        original(0),
        mimic(0),
        setup(0, *this)
    {
        index.length=1;
    }
    M_Int index;

    void doBuild();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void houseKeep();
};
class RingBuild : public Build, public MarkPart, public MField15, public VisualAid{
public:
    int getType()const{return RINGBUILD;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    SFInt32 numOfRingsToInsert;
    MFVec2f	scaleX;
    MFVec2f	scaleZ;
    MFRotation orientation;
    MFVec3f	location;

    MFInt32 ownership;

    SFInt32 staves;
    SFBool dragLocationY;
    SFBool dragLocationXZ;
    SFBool dragOrientation;
    //SFBool insideOut;
    SFVec3f	scale;
    SFRotation rotation;
    SFVec3f	translation;
    PartChild part;
    Setup setup;

    RingBuild():
        Build(setup),
        MField15(
            &numOfRingsToInsert,
            &scaleX,
            &scaleZ,
            &orientation,
            &location,

            &ownership,

            &staves,
            &dragLocationY,
            &dragLocationXZ,
            &dragOrientation,
            //&insideOut,
            &scale,
            &rotation,
            &translation,
            &part,
            &setup
        ),
        numOfRingsToInsert(1),
        scaleX(M_Vec2f()),
        scaleZ(M_Vec2f()),
        orientation(M_Rotation()),
        location(M_Vec3f()),

        ownership(M_Int()),
        staves(16),
        dragLocationY(true),
        dragLocationXZ(true),
        dragOrientation(true),
        //insideOut(false),
        scale(Vec3f(1,1,1)),
        rotation(Rotation(0,0,1,0)),
        translation(Vec3f()),
        part(M_Node()),
        setup(0, *this)
    {
        index.length=2;
    }
    void doBuild();
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);
    M_Int index;
    void drag(Vec3f pt);
    void houseKeep();
    void centerViewToControlPoint();
    inline bool checkPoint(const bool& perspective, int& idx, const int& i, const Matrix& baseMatrix);
    static int quadrant;

    Art* getSelPart();
    void senseMirrorRing();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    int getPartOffset();
    void checkToPaintSceneTree();
    void resetCo(Art* part, int& co);
private:
    bool getGoodQuadrant(Matrix& matrix);
};
class BezierScale : public Generator, public Node, public Bezier, public MField13, public VisualAid{
public:
    int getType()const{return BEZIERSCALE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TSFVec3f location;
    TSFRotation orientation;
    TMFVec2f point;
    TMFBool	fix;
    MFInt32 nopfs;
    SFBool bNegX;
    SFBool bPosX;
    SFBool bNegZ;
    SFBool bPosZ;
    SFFloat negX;
    SFFloat posX;
    SFFloat negZ;
    SFFloat posZ;

    BezierScale():
        MField13(
            &location,
            &orientation,
            &point,
            &fix,
            &nopfs,
            &bNegX,
            &bPosX,
            &bNegZ,
            &bPosZ,
            &negX,
            &posX,
            &negZ,
            &posZ
        ),
        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec2f(), *this),
        fix(M_Bool(), *this),
        nopfs(M_Int()),
        bNegX(false),
        bPosX(false),
        bNegZ(false),
        bPosZ(false),
        negX(1),
        posX(1),
        negZ(1),
        posZ(1),

        /////
        Bezier(point, index, fix, nopfs)
    {
        index.length=1;
    }
    M_Int index;

    float getDistance(int index=0x7fffffff)const;
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void processFields();
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
    void calcClose(){};
    //void genTangentDiff();
    bool onActionForFix(bool state);
    void removeControlPoint(int& newLen);

    void drag(Vec3f pt){Bezier::drag(pt);}
    using Bezier::drag;

private:
    Matrix gen_matrix;
    Matrix gen_imatrix;
};
class BezierCircumScale : public Generator, public Node, public Bezier, public MField7, public VisualAid{
public:
    int getType()const{return BEZIERCIRCUMSCALE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TSFVec3f location;
    TSFRotation orientation;
    TMFVec2f point;
    TMFBool	fix;
    MFInt32 nopfs;
    SFFloat centerOffset;
    SFFloat level;

    BezierCircumScale():
        MField7(
            &location,
            &orientation,
            &point,
            &fix,
            &nopfs,
            &centerOffset,
            &level
        ),
        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec2f(), *this),
        fix(M_Bool(), *this),
        nopfs(M_Int()),
        centerOffset(0),
        level(1),

        /////
        Bezier(point, index, fix, nopfs)
    {
        index.length=1;
    }
    M_Int index;

    float getDistance(int index=0x7fffffff)const;
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void processFields();
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
    void calcClose(){};
    //void genTangentDiff();
    bool onActionForFix(bool state);
    void removeControlPoint(int& newLen);

    void drag(Vec3f pt){Bezier::drag(pt);}
    using Bezier::drag;

private:
    Matrix gen_matrix;
    Matrix gen_imatrix;
};
class BezierPatch : public Generator, public Node, public MField5, public VisualAid{
public:
    class Dim : public MFVec2f {
        public:
        BezierPatch& home;
        Dim(const M_Vec2f& value, BezierPatch& home):
            MFVec2f(value),
            home(home)
        {};
        void operator=(const M_Vec2f& par);
    };

    int getType()const{return BEZIERPATCH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TMFVec3f point;
    TMFFloat weight;
//	MFInt32 fixIdx;
    MFInt32 link;
    Dim dim;
    TMFVec2f zone;
    //TMFInt staves;
    //TMFInt bands;
//	MFInt32 fixIdxIdx;
    //TMFNode source;
    //MFNode task;

    BezierPatch():
        MField5(
            &point,
            &weight,
//			&fixIdx,
            &link,
            &dim,
            &zone
//			&staves,
//			&bands,
//			&fixIdxIdx
            //&source,
            //&task,
        ),
        point(M_Vec3f(Vec3f(-.5, 0, 0), Vec3f(.5, 0, 0), Vec3f(-.5, 1, 0), Vec3f(.5, 1, 0)), *this),
        weight(M_Float(1,1,1,1), *this),
        //fixIdx(M_Int()),
        link(M_Int()),
        dim(M_Vec2f(Vec2f(2, 2)), *this),
        zone(M_Vec2f(Vec2f()), *this)
//		staves(M_Int(10), *this),
//		bands(M_Int(10), *this),
//		fixIdxIdx(M_Int(0))
        //source(M_Node(), *this),
        //task(M_Node()),
    {
        index.length=5;
    }
    M_Int index;
    void generate(M_Vec3f& coord, const int h);
    void processFields();
    void houseKeep();
    void drag(Vec3f pt);

    M_Int fixEdge;
    enum{
        pointGroup,
        fixIdxGroup,
        linkGroup,
        numOfUGroup
    };

    void mirrorPatch();
    int getBasePoint(int par)const;
    int getPatchIndex(int point)const;
    void setNumOfGroupFromSelPoint();
    bool doCornerLink(M_Vec3f& point, const int idx, const Vec3f mousePos);

    void deletePatch();

    void joinEdge();

private:
    M_<int> zoneLookUp;
    Vec2i zoneDim;
    M_Int m_bp;
    M_Int linkToPatchIdx;

    Vec3f calculateU(float t,int row, int numOfU, float& rw, int bp);
    Vec3f calculateAltV(float t,int row, int numOfU, int numOfV, int bp);
    Vec3f calculateV(float t, M_Vec3f& pnt, M_Float& w, int bp);
    Vec3f calculateUEdge02(float t, int row, int idx, int numOfU, int bp);
    void simGenerate(M_Vec3f& coord);
    void simGenerateExtra(M_Vec3f& coord, int numOfU, int numOfV, int bp);
    bool findMatchingPoint(int exclude);
    enum{
        swingA,
        hingeA,
        hingeB,
        swingB
    };
    static void doLink(M_Vec3f& point, const M_Int& link, int idx, Vec3f diff, int exclude=-1);
    static void refreshLink(M_Int& link);
};
class PatchTask{
public:
    virtual void doTask(M_Vec3f& point, int uPoints, int vPoints)=0;
};
class Feedback : public Node, public MField3{
public:
    int getType()const{return FEEDBACK;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFVec2f	multiply;
    SFInt32	from;
    SFNode	source;

    Feedback():
        MField3(
            &multiply,
            &from,
            &source
        ),
        multiply(Vec2f(.5, .5)),
        from(0),
        source(0)
    {
    }
};
class PovLight : public Node, public PovRay, public MField11{
public:
class Pointlight : public SFBool{
public:
    PovLight& home;

    Pointlight(const bool& par, PovLight& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Spotlight : public SFBool{
public:
    PovLight& home;

    Spotlight(const bool& par, PovLight& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Shadowless : public SFBool{
public:
    PovLight& home;

    Shadowless(const bool& par, PovLight& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return POVLIGHT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFVec3f	location;
    SFColor	color;
    Pointlight pointlight;
    Spotlight	spotlight;
    Shadowless	shadowless;
    SFFloat	radius;
    SFFloat	falloff;
    SFFloat	tightness;
    SFVec3f	point_at;
    SFFloat	fade_distance;
    SFFloat	fade_power;

    PovLight():
        MField11(
            &location,
            &color,
            &pointlight,
            &spotlight,
            &shadowless,
            &radius,
            &falloff,
            &tightness,
            &point_at,
            &fade_distance,
            &fade_power
        ),
        location(Vec3f(0,0,100)),
        color(Color3f()),
        pointlight(true, *this),
        spotlight(false, *this),
        shadowless(false, *this),
        radius(0.523598776),
        falloff(0.785398163),
        tightness(0),
        point_at(Vec3f()),
        fade_distance(0),
        fade_power(0)
    {
    }
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
};
class PovFog : public Node, public PovRay, public MField9{
public:

    int getType()const{return POVFOG;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFFloat	lambda;
    SFInt32	fog_type;
    SFFloat	fog_offset;
    SFFloat	fog_alt;
    SFInt32 octaves;
    SFFloat	omega;
    SFVec3f turbulence;
    SFFloat	turb_depth;
    SFVec3f	up;

    PovFog():
        MField9(
            &lambda,
            &fog_type,
            &fog_offset,
            &fog_alt,
            &octaves,
            &omega,
            &turbulence,
            &turb_depth,
            &up
        ),
        lambda(2.0),
        fog_type(1),
        fog_offset(0.0),
        fog_alt(0.0),
        octaves(6),
        omega(0.5),
        turbulence(Vec3f()),
        turb_depth(0.5),
        up(Vec3f(0,1,0))
    {
    }
    void outPovRay(String& str, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outObj(String& str, String& mtl, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void outWebGL(String& str, int& vidx, int& vtidx, M_<Effect*>& effectList, Matrix matrix, Matrix rotMatrix);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
};

class BezierShift : public Generator, public Node, public Bezier, public MField7, public VisualAid{
    public:

    int getType()const{return BEZIERSHIFT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TSFVec3f location;
    TSFRotation orientation;
    TMFVec2f point;
    TMFBool	fix;
    MFInt32 nopfs;
    SFFloat negDecay;
    SFFloat posDecay;

    BezierShift():
        MField7(
            &location,
            &orientation,
            &point,
            &fix,
            &nopfs,
            &negDecay,
            &posDecay
        ),
        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec2f(), *this),
        fix(M_Bool(), *this),
        nopfs(M_Int()),
        negDecay(1),
        posDecay(1),

        /////
        Bezier(point, index, fix, nopfs)
    {
        index.length=1;
    }
    M_Int index;

    float getDistance(int index=0x7fffffff)const;
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void processFields();
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
    void calcClose(){};
    //void genTangentDiff();
    bool onActionForFix(bool state);
    void removeControlPoint(int& newLen);

    void drag(Vec3f pt){Bezier::drag(pt);}
    using Bezier::drag;

private:
    Matrix gen_matrix;
    Matrix gen_imatrix;
};
class BezierTwist : public Generator, public Node, public Bezier, public MField5, public VisualAid{
    public:
    int getType()const{return BEZIERTWIST;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}


    TSFVec3f location;
    TSFRotation orientation;
    TMFVec2f point;
    TMFBool	fix;
    MFInt32 nopfs;

    BezierTwist():
        MField5(
            &location,
            &orientation,
            &point,
            &fix,
            &nopfs
        ),
        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec2f(), *this),
        fix(M_Bool(), *this),
        nopfs(M_Int()),

        /////
        Bezier(point, index, fix, nopfs)
    {
        index.length=1;
    }
    M_Int index;

    float getDistance(int index=0x7fffffff)const;
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void processFields();
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
    void calcClose(){};
    //void genTangentDiff();
    bool onActionForFix(bool state);
    void removeControlPoint(int& newLen);

    void drag(Vec3f pt){Bezier::drag(pt);}
    using Bezier::drag;
private:
    Matrix gen_matrix;
    Matrix gen_imatrix;
};


class BezierTape : public ALathe, public Bezier, public MField3, public VisualAid{
    public:

    int getType()const{return BEZIERTAPE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TMFVec2f point;
    TMFBool	fix;
    MFInt32 nopfs;

    BezierTape():
        MField3(
            &point,
            &fix,
            &nopfs
        ),
        point(M_Vec2f(), *this),
        fix(M_Bool(), *this),
        nopfs(M_Int()),

        /////
        Bezier(point, index, fix, nopfs)
    {
        index.length=1;
    }
    M_Int index;

    float getDistance(int index=0x7fffffff)const;
    void getBandAndYDist(M_Int& bands, M_Float& bandDist, M_Float& yDistance)const;
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void processFields();
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
    void calcClose(){};
    //void genTangentDiff();
    bool onActionForFix(bool state);
    void removeControlPoint(int& newLen);

    void drag(Vec3f pt);
};
class BezierBend : public Generator, public Node, public Bezier3f, public MField5, public VisualAid{
    public:

    int getType()const{return BEZIERBEND;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TSFVec3f location;
    TSFRotation orientation;
    TMFVec3f point;
    TMFBool	fix;
    MFInt32 nopfs;

    BezierBend():
        MField5(
            &location,
            &orientation,
            &point,
            &fix,
            &nopfs
        ),
        location(Vec3f(),*this),
        orientation(Rotation(),*this),
        point(M_Vec3f(), *this),
        fix(M_Bool(), *this),
        nopfs(M_Int()),
        /////

        Bezier3f(point, index, fix, nopfs)
    {
        index.length=1;
    }
    M_Int index;

    float getDistance(int index=0x7fffffff)const;
    void inverseSurface(M_Node& generator, BranchJoinNegY& branchJoinNegY);
    void processFields();
    void generate(M_Vec3f& coord, const int h);
    void houseKeep();
    void calcClose(){};
    //void genTangentDiff();
    bool onActionForFix(bool state);
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);

    void drag(Vec3f pt){Bezier3f::drag(pt);}
    using Bezier3f::drag;

private:
    Matrix gen_matrix;
    Matrix gen_imatrix;
};
/*
class NurbsLathe : public Build, public MField17, public VisualAid{
public:
class DPoint : public MFVec2f{
    public:
    NurbsLathe& home;
    DPoint(const M_Vec2f& value, NurbsLathe& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par);


};

class Weight : public MFFloat{
public:
    NurbsLathe& home;
    Weight( const M_Float& par, NurbsLathe& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};

class Scale : public SFVec3f{
    public:
    NurbsLathe& home;
    Scale(const Vec3f& value, NurbsLathe& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    NurbsLathe& home;
    HRotation(const Rotation& value, NurbsLathe& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    NurbsLathe& home;
    Translation(const Vec3f& value, NurbsLathe& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HSFPart : public SFNode{
public:
    NurbsLathe& home;
    HSFPart( const NodePtr& par, NurbsLathe& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    const ArtPtr& refValue()const{
        return *((const ArtPtr*)&SFNode::refValue());
    }

    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};

    int getType()const{return NURBSLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    static const float initPointData[7*2];

    TMFVec2f point;
    DPoint dpoint;
    Weight weight;
    TMFFloat knot;
    TSFInt order;
    TSFInt staves;
    TSFInt bands;
    //TSFInt ctrlStaves;
    TSFColor3f color;
    Scale scale;
    HRotation rotation;
    Translation translation;
    TSFBool endSP;
    TSFBool beginSP;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    NurbsLathe():
        Build(setup),
        MField17(
            &point,
            &dpoint,
            &weight,
            &knot,
            &order,
            &staves,
            &bands,
            //&ctrlStaves,
            &color,
            &scale,
            &rotation,
            &translation,
            &endSP,
            &beginSP,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        point(M_Vec2f(FLOAT_LIST(initPointData)), *this),
        dpoint(M_Vec2f(FLOAT_LIST(initPointData)), *this),
        weight(M_Float(1, 1, 1, 1, 1, 1, 1), *this),
        knot(M_Float(0, 0, 0, 1, 2, 3, 4, 5, 5, 5), *this),
        order(3, *this),
        staves(16, *this),
        bands(16, *this),
        color(Color3f(), *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        endSP(true, *this),
        beginSP(true, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

/////////////
        degree(2),

        patch(new NurbsPatch),
        ctrlStaves(4)

    {
        index.length = 2;
    }
    M_Int index;
    void processFields();
    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);
    int degree;
    int ctrlStaves;

    float minT(){
        return knot.refValue()[degree];
    }
    float maxT(){
        return knot.refValue()[knot.length-degree];
    }

    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void convertToPatch();
    void convertToBendy();
    void mirrorPoints();
    NurbsLathe* getMirrorLathe()const;

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);
    void insertControlPoint();

    Matrix matrix;
    Matrix imatrix;

    void dpointsToPoints();

    void setStemValues(Stem* stem);
private:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    NurbsPatchPtr patch;
};
*/

class NurbsLine : public NurbsPatch, public MField9{
public:


    int getType()const{return NURBSLINE;}
    int getFLen()const{return MField9::size();};
    Field* getField(const int i)const{return MField9::fieldArray[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFColor color;
    SFInt32 steps;
    NurbsLine():
        MField9(
            &point,
			&dpoint,
            &weight,
            &color,
			&uKnot,
            &mirror,
            &uJoinEnds,
            &steps,
            &cage
        ),
        color(M_Color3f()),
        steps(64),
		index(NurbsPatch::index)


/////////////

	{
        index.length = 1;
        uJoinEnds.refValue() = false;

    }
    M_Int& index;
    void processFields();

    void doBuild();

    void deleteControlPoint();
    bool insertControlPoint();
    void cpNodeListAdd(Seamless* smls);

	bool convertToPatch(NurbsLine* lineB);
	bool convertToThickLine(NurbsPatch* patch);
	NurbsPatch* convertToPatch2(NurbsLine* lineB, bool insideOut);
	void convertToThickLine(NurbsLine* lineB, NurbsPatch* patch, bool insideOut);
	void convert_to_1_u_mid_points();
	void insertPoint(Vec3f newPoint, int k, float ratio);
	void cut();
	void mirrorPoints();
	void completeLoop(bool insert);
	SFRotation getOrientation(float t);

};

class NurbsLathe : public Build, public MField19, public VisualAid{
public:
class DPoint : public MFVec2f{
    public:
    NurbsLathe& home;
    DPoint(const M_Vec2f& value, NurbsLathe& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par);


};

class Weight : public MFFloat{
public:
    NurbsLathe& home;
    Weight( const M_Float& par, NurbsLathe& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};

class Scale : public SFVec3f{
    public:
    NurbsLathe& home;
    Scale(const Vec3f& value, NurbsLathe& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    NurbsLathe& home;
    HRotation(const Rotation& value, NurbsLathe& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    NurbsLathe& home;
    Translation(const Vec3f& value, NurbsLathe& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class BeginSP : public SFBool{
    public:
    NurbsLathe& home;
    BeginSP(const bool& value, NurbsLathe& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class EndSP : public SFBool{
    public:
    NurbsLathe& home;
    EndSP(const bool& value, NurbsLathe& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Mirror : public SFBool{
    public:
    NurbsLathe& home;
    Mirror(const bool& value, NurbsLathe& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class HSFPart : public SFNode{
public:
    NurbsLathe& home;
    HSFPart( const NodePtr& par, NurbsLathe& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    const ArtPtr& refValue()const{
        return *((const ArtPtr*)&SFNode::refValue());
    }

    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};

    int getType()const{return NURBSLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    static const float initPointData[7 * 2];

    TMFVec2f point;
    DPoint dpoint;
    Weight weight;
    TMFFloat knot;
    TSFInt order;
    TSFInt staves;
    TSFInt bands;
    //TSFInt ctrlStaves;
    TSFColor3f color;
    Scale scale;
    HRotation rotation;
    Translation translation;
    EndSP endSP;
    BeginSP beginSP;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    STSFNode<NurbsLathe> textureMapper;
    Setup setup;

    NurbsLathe():
        Build(setup),
        MField19(
            &point,
            &dpoint,
            &weight,
            &knot,
            &order,
            &staves,
            &bands,
            //&ctrlStaves,
            &color,
            &scale,
            &rotation,
            &translation,
            &endSP,
            &beginSP,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        point(M_Vec2f(FLOAT_LIST(initPointData)), *this),
        dpoint(M_Vec2f(FLOAT_LIST(initPointData)), *this),
        weight(M_Float(1, 1, 1, 1, 1, 1, 1), *this),
        knot(M_Float(0, 0, 0, 1, 2, 3, 4, 5, 5, 5), *this),
        order(3, *this),
        staves(16, *this),
        bands(16, *this),
        color(Color3f(), *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        endSP(true, *this),
        beginSP(true, *this),
        mirror(false, *this),
        insideOut(false, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(0, *this),
        setup(0, *this),

/////////////
        degree(2),

        stem(new Stem),
        patch(new NurbsPatch),
        colorSweep(new ColorSweep),
        ctrlStaves(4),
        mirrorContainer(0)

    {
        patch->container = this;
        index.length = 2;
        patch->uJoinEnds.refValue() = true;
        patch->stem.refValue() += stem.p;
        stem->xDistance.refValue() = 1;
        stem->yDistance.refValue() += 1;
        stem->xCentred.refValue() = false;
        stem->bands.refValue().length = 1;
    }
    M_Int index;
    void centerViewToControlPoint();
    void drag(Vec3f pt);
    int degree;
    int ctrlStaves;

    float minT(){
        return knot.refValue()[degree];
    }
    float maxT(){
        return knot.refValue()[knot.length-degree];
    }

    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void processFields();
    void houseKeep();

    void convertToPatch();
    void convertToBendy();
    void mirrorPoints();
    NurbsLathe* getMirrorLathe()const;

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void removeControlPoint(int idx);
    void insertControlPoint();
    void deleteControlPoint();
    void postTug();
    bool checkOverPoint(const bool& perspective);
	void joinStitchedEdges();

    Matrix matrix;
    Matrix imatrix;

    void dpointsToPoints();

    NurbsLathe* mirrorContainer;

private:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    StemPtr stem;
    NurbsPatchPtr patch;
    ColorSweepPtr colorSweep;

};





/*
class BendyLathe : public Build, public MField18, public VisualAid{
public:
class DPoint : public MFVec3f{
    public:
    BendyLathe& home;
    DPoint(const M_Vec3f& value, BendyLathe& home):
        MFVec3f(value),
        home(home)
    {};
    void operator=(const M_Vec3f& par);
};
class Weight : public MFFloat{
public:
    BendyLathe& home;
    Weight( const M_Float& par, BendyLathe& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};
class HSFPart : public SFNode{
public:
    BendyLathe& home;
    HSFPart(const NodePtr& par, BendyLathe& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};

    int getType()const{return BENDYLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    static const float initPointData[7*3];

    SFBool showRingControlPoints;
    TMFVec3f point;
    DPoint dpoint;
    Weight weight;
    TMFFloat knot;
    TSFInt order;
    TMFFloat radius;
    TMFFloat at;
    TSFInt staves;
    TSFInt bands;
    //TSFInt ctrlStaves;
    TSFColor3f color;
    TSFVec3f scale;
    TSFRotation rotation;
    TSFVec3f translation;
    TSFBool descendants;
    HSFPart part;
    TMFNode textureMapper;
    Setup setup;

    BendyLathe():
        Build(setup),
        MField18(
            &showRingControlPoints,
            &point,
            &dpoint,
            &weight,
            &knot,
            &order,
            &radius,
            &at,
            &staves,
            &bands,
            //&ctrlStaves,
            &color,
            &scale,
            &rotation,
            &translation,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        showRingControlPoints(false),
        point(M_Vec3f(FLOAT_LIST(initPointData)), *this),
        dpoint(M_Vec3f(FLOAT_LIST(initPointData)), *this),
        weight(M_Float(1, 1, 1, 1, 1, 1, 1), *this),
        knot(M_Float(0, 0, 0, 1, 2, 3, 4, 5, 5, 5), *this),
        order(3, *this),
        radius(M_Float(.1, .1, .1, .1, .1, .1, .1, .1, .1, .1, .1, .1), *this),
        at(M_Float(0, .05, .15, .25, .35, .45, .55, .65, .75, .85, .95, 1), *this),
        staves(16, *this),
        bands(25, *this),
        color(Color3f(), *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(M_Node(), *this),
        setup(0, *this),

/////////////
        degree(2),

        patch(new NurbsPatch),
        ratio(0),
        ctrlStaves(4)
    {
        index.length = 3;
    }
    M_Int index;
    void processFields();
    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);
    int degree;
    int ctrlStaves;

    float minT(){
        return knot.refValue()[degree];
    }
    float maxT(){
        return knot.refValue()[knot.length-degree];
    }
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);
    void addRadius(int& newLen);
    void removeRadius(int& newLen);

    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void setStemValues(Stem* stem);

    void convertToPatch();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    inline bool checkPoint(const bool& perspective, int& idx, const Matrix& baseMatrix);
    static int radIdx;

    float zeroinOnVerticalPoint(M_Vec3f& point, const M_Float& weight, const M_Float& knot, float minT, float maxT, float targetHeight);


    Matrix matrix;
    Matrix imatrix;

    NurbsPatchPtr patch;

private:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    M_Vec3f ringBase;
    M_Rotation ringRot;
    double ratio;

    void dpointsToPoints();
    void dragRingBase(Vec3f pt, int idx, M_Vec3f& point, const M_Float& weight, const M_Float& knot, float minT, float maxT, Vec3f& atVec);
    void dragRadius(Vec3f pt, int idx);
    void houseKeep3dNurbs(const M_Vec3f& dpoint, const M_Vec3f& point, const M_Float& weight, const M_Float& knot, const int idx, float minT, float maxT, Matrix matrix, NurbsPatch* patch, int selRadius);
};
*/
class BendyLathe : public Build, public MField20, public VisualAid{
public:
/*
class DragRings : public SFBool{
public:
    BendyLathe& home;
    DragRings(const bool& par, BendyLathe& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
*/
class DPoint : public MFVec3f{
    public:
    BendyLathe& home;
    DPoint(const M_Vec3f& value, BendyLathe& home):
        MFVec3f(value),
        home(home)
    {};
    void operator=(const M_Vec3f& par);
};
class Weight : public MFFloat{
public:
    BendyLathe& home;
    Weight( const M_Float& par, BendyLathe& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};

class Scale : public SFVec3f{
    public:
    BendyLathe& home;
    Scale(const Vec3f& value, BendyLathe& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class HRotation : public SFRotation{
    public:
    BendyLathe& home;
    HRotation(const Rotation& value, BendyLathe& home):
        SFRotation(value),
        home(home)
    {};
    void operator=(const Rotation& par);
};

class Translation : public SFVec3f{
    public:
    BendyLathe& home;
    Translation(const Vec3f& value, BendyLathe& home):
        SFVec3f(value),
        home(home)
    {};
    void operator=(const Vec3f& par);
};

class Mirror : public SFBool{
    public:
    BendyLathe& home;
    Mirror(const bool& value, BendyLathe& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class HSFPart : public SFNode{
public:
    BendyLathe& home;
    HSFPart(const NodePtr& par, BendyLathe& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};

    int getType()const{return BENDYLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    static const float initPointData[7*3];

    SFBool showRingControlPoints;
    TMFVec3f point;
    DPoint dpoint;
    Weight weight;
    TMFFloat knot;
    TSFInt order;
    TMFFloat radius;
    TMFFloat at;
    TSFInt staves;
    TSFInt bands;
    //TSFInt ctrlStaves;
    TSFColor3f color;
    Scale scale;
    HRotation rotation;
    Translation translation;
    Mirror mirror;
    TSFBool insideOut;
    TSFBool descendants;
    HSFPart part;
    STSFNode<BendyLathe> textureMapper;
    Setup setup;

    BendyLathe():
        Build(setup),
        MField20(
            &showRingControlPoints,
            &point,
            &dpoint,
            &weight,
            &knot,
            &order,
            &radius,
            &at,
            &staves,
            &bands,
            //&ctrlStaves,
            &color,
            &scale,
            &rotation,
            &translation,
            &mirror,
            &insideOut,
            &descendants,
            &part,
            &textureMapper,
            &setup
        ),
        showRingControlPoints(false),
        point(M_Vec3f(FLOAT_LIST(initPointData)), *this),
        dpoint(M_Vec3f(FLOAT_LIST(initPointData)), *this),
        weight(M_Float(1, 1, 1, 1, 1, 1, 1), *this),
        knot(M_Float(0, 0, 0, 1, 2, 3, 4, 5, 5, 5), *this),
        order(3, *this),
        radius(M_Float(.1, .1, .1, .1, .1, .1, .1, .1, .1, .1, .1, .1), *this),
        at(M_Float(0, .05, .15, .25, .35, .45, .55, .65, .75, .85, .95, 1), *this),
        staves(16, *this),
        bands(25, *this),
        color(Color3f(), *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
        mirror(false, *this),
        insideOut(false, *this),
        descendants(false, *this),
        part(0, *this),
        textureMapper(0, *this),
        setup(0, *this),

/////////////
        degree(2),

        stem(new Stem),
        patch(new NurbsPatch),
        colorSweep(new ColorSweep),
        ratio(0),
        ctrlStaves(4),
        mirrorContainer(0)

    {
        index.length=3;
        patch->container = this;
        stem->xDistance.refValue() = 1;
        stem->yDistance.refValue() += 1;
        stem->xCentred.refValue() = false;
        stem->bands.refValue().length=1;
        patch->stem.refValue()+=stem.p;
    }
    M_Int index;
    void processFields();
    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);
    int degree;
    int ctrlStaves;

    float minT(){
        return knot.refValue()[degree];
    }
    float maxT(){
        return knot.refValue()[knot.length-degree];
    }
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);
    void addRadius(int& newLen);
    void removeRadius(int& newLen);

    void doBuild();
    NurbsPatch* getPatch(){return patch.p;}
    Build* getMirrorBuild(){return mirrorContainer;}
    int numOfPatches(){return 1;}
    bool isCaged(){return patch->cage;}
    void setCage(bool state){patch->cage = state;}
    void setMirror(bool state){mirror = state;}
    void cpNodeListAdd(Seamless* smls);
    void remTransform();
    void doCPTransform();
    void convertToPatch();
    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    inline bool checkPoint(const bool& perspective, int& idx, const Matrix& baseMatrix);
    static int radIdx;

    float zeroinOnVerticalPoint(M_Vec3f& point, const M_Float& weight, const M_Float& knot, float minT, float maxT, float targetHeight);


    Matrix matrix;
    Matrix imatrix;

    StemPtr stem;
    NurbsPatchPtr patch;
    ColorSweepPtr colorSweep;

    BendyLathe* mirrorContainer;
private:
    Vec3f remScale;
    Rotation remRotation;
    Vec3f remTranslation;

    M_Vec3f ringBase;
    M_Rotation ringRot;
    double ratio;

    void dpointsToPoints();
    void dragRingBase(Vec3f pt, int idx, M_Vec3f& point, const M_Float& weight, const M_Float& knot, float minT, float maxT, Vec3f& atVec);
    void dragRadius(Vec3f pt, int idx);
    void houseKeep3dNurbs(const M_Vec3f& dpoint, const M_Vec3f& point, const M_Float& weight, const M_Float& knot, const int idx, float minT, float maxT, Matrix matrix, NurbsPatch* patch, int selRadius);
};

class FuseSurface : public Node, public FieldGroupExtra, public MField22, public VisualAid{
public:
class NumOf : public SFInt32 {
public:
    FuseSurface& home;

    NumOf(const float& value, FuseSurface& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};
class Qty : public SFInt32 {
public:
    FuseSurface& home;

    Qty(const float& value, FuseSurface& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};
class Spread : public SFBool {
public:
    FuseSurface& home;

    Spread(const bool& par, FuseSurface& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class PolyMapAid : public SFBool {
public:
    FuseSurface& home;

    PolyMapAid(const bool& par, FuseSurface& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class PolyMapAidMirror : public SFBool {
public:
    FuseSurface& home;

    PolyMapAidMirror(const bool& par, FuseSurface& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return FUSESURFACE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFVec2f baseCoord;
    SFVec2f diff;
    SFFloat	sumDiff;
    SFFloat	averageDiff;
    SFFloat vSpan;
    NumOf numOf;
    Qty qty;
    SFRotation orientation;
    SFBool bond;
    SFFloat	coordChange;
    Spread spread;
    SFInt32 spreadRes;

    PolyMapAid polyMapAid;
    PolyMapAidMirror polyMapAidMirror;

    MFInt32	srcMatch;
    MFInt32	dstMatch;
    SFBool	in;
    SFFloat	range;
    SFBool	snap;
    SFBool	evolve;
    SFFloat	pointChange;

    SFNode patch;

    FuseSurface():
        MField22(
            &baseCoord,
            &diff,
            &sumDiff,
            &averageDiff,
            &vSpan,
            &numOf,
            &qty,
            &orientation,
            &bond,
            &coordChange,
            &spread,
            &spreadRes,

            &polyMapAid,
            &polyMapAidMirror,

            &srcMatch,
            &dstMatch,
            &in,
            &range,
            &snap,
            &evolve,
            &pointChange,

            &patch
        ),
        baseCoord(M_Vec2f()),
        diff(Vec2f()),
        sumDiff(0),
        averageDiff(0),
        vSpan(.5),
        numOf(1, *this),
        qty(0, *this),
        orientation(Rotation()),
        bond(true),
        coordChange(.001),
        spread(false, *this),
        spreadRes(100),

        polyMapAid(false, *this),
        polyMapAidMirror(false, *this),

        srcMatch(M_Int()),
        dstMatch(M_Int()),
        in(false),
        range(.001),
        snap(false),
        pointChange(.001),
        evolve(false),

        patch(0),
        parentPatch(0),
        parentStem(0)


/////////////

    {
        index.length=2;
    }
    M_Int index;
    M_Vec3f curvedEdgePoint;
    M_Float vPan;
    static Vec3f baseVisPoint;
    static Vec3f thisVisPoint;
    M_Vec3f parentFlatCoord;

    NurbsPatch* parentPatch;
    Stem* parentStem;

    void storeV(M_Vec3f& coord);
    void doFuse(M_Vec3f& coord, NurbsPatch& nurbsPatch);
    void doFuse(NurbsPatch& patch, Stem& stem);
    void doSpread();

    void houseKeep();
    void incBaseCoordGroup(int& newLen);
    void decBaseCoordGroup(int& newLen);
    bool processGroup(int groupIndex, int command, int& newLen);
private:
    inline void doIn(Stem* stem, M_Vec2f& baseCoord, M_Int& srcMatch, M_Int& dstMatch);
    //inline void doIn(Stem* stem, Art* source, M_Vec2f& baseCoord, M_Int& srcMatch, M_Int& dstMatch, const float maxU, const float maxV, const float minU, const float minV, float* stemUKnotPtr, float* stemVKnotPtr, Vec3f* stemPointPtr, float* stemWeightPtr, int stemUOrder, int stemVOrder, int stemUDim, int stemVDim);
    //inline void guessPoints(NurbsPatch* patch, Vec3f amount, const M_Vec3f& coord, const M_Vec3f& baseCurvedPoint, float& guessSumDiff, float& currentSumDiff, const float maxU, const float maxV, const float minU, const float minV, float* stemUKnotPtr, float* stemVKnotPtr, Vec3f* stemPointPtr, float* stemWeightPtr, int stemUOrder, int stemVOrder, int stemUDim, int stemVDim);
    inline void guessPoints(NurbsPatch* patch, Vec3f amount, const M_Vec3f& parentFlatCoord, const M_Vec3f& baseCurvedPoint, float& guessSumDiff, float& currentSumDiff, const float maxU, const float maxV, const float minU, const float minV, float* uKnotPtr, float* vKnotPtr, Vec3f* pointPtr, float* weightPtr, Vec3f* topPointPtr, float* topWeightPtr, Vec3f* r_pointPtr, float* r_weightPtr, int uOrder, int vOrder, int uDegree, int vDegree, int uDim, int vDim);
};
class PolyIndex{
public:
    PolyIndex():
        idx(0),
        len(0)
    {}
    PolyIndex(int idx, int len):
        idx(idx),
        len(len)
    {}
    int idx;
    int len;
};
class Subdivision : public Build, public MField5, public VisualAid{
public:
    int getType()const{return SUBDIVISION;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFVec3f point;
    SFInt32 cycle;
    MFInt32	coordIndex;
    SFPart part;
    Setup setup;

    Subdivision():
        Build(setup),
        MField5(
            &point,
            &cycle,
            &coordIndex,
            &part,
            &setup
        ),
        point(
            M_Vec3f(
                Vec3f(-.25, -.25, .25),Vec3f(.25, -.25, .25),
                Vec3f(-.25, .25, .25),Vec3f(.25, .25, .25),

                Vec3f(-.25, -.25, -.25),Vec3f(.25, -.25, -.25),
                Vec3f(-.25, .25, -.25),Vec3f(.25, .25, -.25)
            )
        ),
        cycle(2),
        coordIndex(
            INT_LIST(initCoordIndexData)
        ),
        part(0),
        setup(0, *this),
        ////
        actPoly(0),
        actCorner(0)
    {
        index.length=2;
    }
    M_<PolyIndex> polyIndex;//indexed coord index

    int	actCorner;
    int actPoly;

    static int initCoordIndexData[24];

    M_Int index;

//	Seamless* smls;
    void houseKeep();
    void drag(Vec3f pt);
    void doBuild();
    void makePointerEqualIndexLocation();
    void extrudePoly();
    void selNextPoly();
    void selPrevPoly();
    void delPoly();
    void incToNextPolyThatSharesActivePoint();
    void decToNextPolyThatSharesActivePoint();
    void deleteNonIndexedPoints();
    bool pointIsInPoly();
    void mirrorPoly();
    void joinCloseVertices();
    void cutLeft();
    void mirrorControlPoints();
    bool getClosestVertex(int& dstIndex, int& dstTriangle, int& dstCorner);
    void joinVertex();

    bool illegalImmediateRef(const Art* p, M_Art& accumOthers)const;
    void processFields();
};

class SoundBuild : public Node{
public:
    virtual void buildSound(float masterVolume, float masterDelay, const int voices,  const float detune, float* fbuf, int bufLen)=0;
};

class Oscillator : public SoundBuild, public MField14, public VisualAid{ //Synth better name perhaps?
public:

class DPoint : public MFVec2f {
    public:
    Oscillator& home;
    DPoint(const M_Vec2f& value, Oscillator& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par);
};
class Weight : public MFFloat {
public:
    Oscillator& home;
    Weight( const M_Float& par, Oscillator& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};

    int getType()const{return OSCILLATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TMFVec2f note;
    DPoint	fdpoint;
    Weight	fWeight;
    MFFloat	fKnot;
    SFInt32	fOrder;
    SFFloat	volume;
    SFFloat	delay;

    SFInt32 staves;
    MFInt32 limit;
    MFVec2f vCoord;
    MFVec3f coord;

    SFNode larynx;
	SFNode melody;
    MFNode filter;

    Oscillator():
        MField14(
            &note,
            &fdpoint,
            &fWeight,
            &fKnot,
            &fOrder,
            &volume,
            &delay,
            &staves,
            &limit,
            &vCoord,
            &coord,
            &larynx,
			&melody,
            &filter
        ),
        note(M_Vec2f(Vec2f(), Vec2f(1,0)), *this),
        fdpoint(M_Vec2f(Vec2f(), Vec2f(1,0)), *this),
        fWeight(M_Float(), *this),
        fKnot(M_Float()),
        fOrder(3),
        volume(1),
        delay(0),

        staves(32),
        limit(M_Int()),
        vCoord(M_Vec2f()),
        coord(M_Vec3f()),

        larynx(0),
		melody(0),
        filter(M_Node()),

        fDegree(2)
    {
        index.length = 4;
    }

    M_Int index;


    int fDegree;
    M_Int chordsBufLen;
    M_<float*> chordsBuf;

    float minT()const{
        if(fKnot.length){
            return fKnot.refValue()[fDegree];
        }
        return 0;
    }
    float maxT()const{
        if(fKnot.length){
            return fKnot.refValue()[fKnot.length-fDegree];
        }
        return 0;
    }

    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);

    void processFields();
    void buildSound(float masterVolume, float masterDelay, const int voices,  const float detune, float* fbuf, int bufLen);
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);
    void alignWithPatch();

	void copyToMelody(NurbsLine* mel);
	void copyFromMelody(NurbsLine* mel);
};


class Envelope : public Node, public MField2, public VisualAid{
public:
    int getType()const{return ENVELOPE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFVec2f point;
    MFNode	chords;

    Envelope():
        MField2(
            &point,
            &chords
        ),
        point(M_Vec2f(Vec2f(0, 1), Vec2f(1,1))),
        chords(M_Node()),

//////
        order(3),
        degree(2),
        resolution(200)
    {
        index.length = 1;
    }

    M_Int index;

    M_Float	weight;
    M_Float	knot;
    int	order;
    int degree;

    M_Vec2f	coord;

    float minT()const{
        if(knot.length){
            return knot[degree];
        }
        return 0;
    }
    float maxT()const{
        if(knot.length){
            return knot[knot.length-degree];
        }
        return 0;
    }

    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);

    void processFields();
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);

    int resolution;
    void updateCoord();
    void doEnvelope(float* chordsBuf, int chordsBufLen);

};


class Modulator {
public:
    virtual void genCarrierSignal(const float volume, const float soundPeriod, const Oscillator& osc)=0;
    virtual void modulate(float* dbuf, int dbufLen)=0;
};

class AmplitudeModulator : public Node, public Modulator, public MField2, public VisualAid{
public:
    int getType()const{return AMPLITUDEMODULATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFVec2f wet;
    SFNode oscillator;

    AmplitudeModulator():
        MField2(
            &wet,
            &oscillator
        ),
        wet(M_Vec2f(Vec2f(0, 1), Vec2f(1,1))),
        oscillator(0),

//////
        order(3),
        degree(2),
        fbuf(0),
        bufLen(0),
        resolution(200)
    {
        index.length = 1;
    }

    M_Int index;

    M_Float	weight;
    M_Float	knot;
    int	order;
    int degree;

    M_Vec2f	coord;

    float minT()const{
        if(knot.length){
            return knot[degree];
        }
        return 0;
    }
    float maxT()const{
        if(knot.length){
            return knot[knot.length-degree];
        }
        return 0;
    }

    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);

    void processFields();
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);

    int resolution;
    void updateWetCoord();
    float* fbuf;
    int bufLen;

    void genCarrierSignal(const float volume, const float soundPeriod, const Oscillator& osc);
    void modulate(float* dbuf, int dbufLen);
};

class FrequencyModulator : public Node, public Modulator, public MField2, public VisualAid{
public:
    int getType()const{return FREQUENCYMODULATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFVec2f wet;
    SFNode oscillator;

    FrequencyModulator():
        MField2(
            &wet,
            &oscillator
        ),
        wet(M_Vec2f(Vec2f(0, 1), Vec2f(1,1))),
        oscillator(0),

//////
        order(3),
        degree(2),
        fbuf(0),
        bufLen(0),
        resolution(200)

    {
        index.length = 1;
    }

    M_Int index;

    M_Float	weight;
    M_Float	knot;
    int	order;
    int degree;

    M_Vec2f	coord;

    float minT()const{
        if(knot.length){
            return knot[degree];
        }
        return 0;
    }
    float maxT()const{
        if(knot.length){
            return knot[knot.length-degree];
        }
        return 0;
    }

    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);

    void processFields();
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);

    int resolution;
    void updateWetCoord();
    float* fbuf;
    int bufLen;

    void genCarrierSignal(const float volume, const float soundPeriod, const Oscillator& osc);
    void modulate(float* dbuf, int dbufLen);
};


class Chords : public Node, public MField16, public VisualAid{ //OSC better name perhaps?
public:
    int getType()const{return CHORDS;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}


    MFVec2f spectrum;
    MFFloat multiple;
    MFFloat amplitude;
    MFFloat phase;
    SFBool	sawTooth;
    SFBool	square;
    SFFloat randomize;
    SFInt32	harmonics;
    SFFloat	increment;
    SFBool	enable;
    SFFloat	interval;
	SFFloat	baseMultiple;
    SFFloat	level;
	SFNode patials;
    //SFNode envelope;
    SFNode modulator;
    SFNode phaseRef;

    Chords():
        MField16(
            &spectrum,
            &multiple,
            &amplitude,
            &phase,
            &sawTooth,
            &square,
            &randomize,
            &harmonics,
            &increment,
            &enable,
            &interval,
			&baseMultiple,
            &level,
			&patials,
            //&envelope,
            &modulator,
            &phaseRef
        ),
        spectrum(M_Vec2f(Vec2f(.022, 1), Vec2f(2,1))),
        multiple(M_Float()),
        amplitude(M_Float()),
        phase(M_Float()),
        sawTooth(true),
        square(false),
        randomize(0),
        harmonics(92),
        increment(1),
        enable(1),
        interval(0),
		baseMultiple(1),
        level(1),
		patials(0),
        //envelope(0),
        modulator(0),
        phaseRef(0),

        order(3),
        degree(2)
    {
        index.length = 2;
    }

    M_Int index;

    M_Float	weight;
    M_Float	knot;
    int	order;
    int degree;

    M_Vec3f	coord;
	M_Vec2f	uvCoord;

    float minT()const{
        if(knot.length){
            return knot[degree];
        }
        return 0;
    }
    float maxT()const{
        if(knot.length){
            return knot[knot.length-degree];
        }
        return 0;
    }

    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);

    void processFields();
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);

    void updateSpectrum();
	void updateSpectrumPatch();
	void addCoordsPatch();


};

class Larynx : public Node, public MField1{
public:
    int getType()const{return LARYNX;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    MFNode	chords;

    Larynx():
        MField1(
            &chords
        ),
        chords(M_Node())
    {
    }

};


class Phrase : public SoundBuild, public MField3{
public:
    int getType()const{return PHRASE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFFloat volume;
    SFFloat delay;
    MFNode sound;

    Phrase():
        MField3(
            &volume,
            &delay,
            &sound
        ),
        volume(1),
        delay(0),
        sound(M_Node())
    {}
    void buildSound(float masterVolume, float masterDelay, const int voices,  const float detune, float* fbuf, int bufLen);
};

class Timbre : public Node{
public:
    virtual void genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen)=0;
};

class Harmonic : public Timbre, public MField7, public VisualAid{
public:

class DPoint : public MFVec2f {
    public:
    Harmonic& home;
    DPoint(const M_Vec2f& value, Harmonic& home):
        MFVec2f(value),
        home(home)
    {};
    void operator=(const M_Vec2f& par);
};
class Weight : public MFFloat {
public:
    Harmonic& home;
    Weight( const M_Float& par, Harmonic& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};

    int getType()const{return HARMONIC;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    TMFVec2f point;
    DPoint	dpoint;
    Weight	weight;
    MFFloat	knot;
    SFInt32	order;
    //MFInt32	numOf;
    MFFloat	multiply;
    MFFloat	phase;

    Harmonic():
        MField7(
            &point,
            &dpoint,
            &weight,
            &knot,
            &order,
            //&numOf,
            &multiply,
            &phase
        ),
        point(M_Vec2f(Vec2f(), Vec2f(1,0)), *this),
        dpoint(M_Vec2f(Vec2f(), Vec2f(1,0)), *this),
        weight(M_Float(), *this),
        knot(M_Float()),
        order(3),
        //numOf(0),
        multiply(0),
        phase(0),

        degree(2)
    {
        index.length = 3;
    }

    M_Int index;


    int degree;

    float minT()const{
        if(knot.length){
            return knot.refValue()[degree];
        }
        return 0;
    }
    float maxT()const{
        if(knot.length){
            return knot.refValue()[knot.length-degree];
        }
        return 0;
    }

    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);

    void processFields();
    void addControlPoint(int& newLen);
    void removeControlPoint(int& newLen);
    void genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen);
    void toMesh(const Oscillator& osc, M_Vec3f& coord, int staves);

};

class HarmonicPatch : public Timbre, public MField11, public VisualAid{
public:
class Dim : public SFVec2f {
    public:
    HarmonicPatch& home;
    Dim(const Vec2f& value, HarmonicPatch& home):
        SFVec2f(value),
        home(home)
    {};
    void operator=(const Vec2f& par);
};
class DPoint : public MFVec3f {
    public:
    HarmonicPatch& home;
    DPoint(const M_Vec3f& value, HarmonicPatch& home):
        MFVec3f(value),
        home(home)
    {};
    void operator=(const M_Vec3f& par);
};
class Weight : public MFFloat {
public:
    HarmonicPatch& home;
    Weight( const M_Float& par, HarmonicPatch& home):
        MFFloat(par),
        home(home)
    {};
    void operator=(const M_Float& par);
};

    int getType()const{return HARMONICPATCH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    static const float initHarmonicPatch_point[48];
    static const float initHarmonicPatch_vCoord[41];
    static const float initHarmonicPatch_multiply[41];

    TMFVec3f point;
    DPoint dpoint;
    Weight weight;
    MFFloat uKnot;
    MFFloat vKnot;
    Dim dim;
    SFVec2f order;
    SFInt32 staves;
    MFFloat vCoord;
    MFVec3f coord;
    SFInt32	harmonics;

    HarmonicPatch():
        MField11(
            &point,
            &dpoint,
            &weight,
            &uKnot,
            &vKnot,
            &dim,
            &order,
            &staves,
            &vCoord,
            &coord,
            &harmonics
        ),
        point(M_Vec3f(FLOAT_LIST(initHarmonicPatch_point)), *this),
        dpoint(M_Vec3f(FLOAT_LIST(initHarmonicPatch_point)), *this),
        weight(M_Float(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1), *this),
        uKnot(M_Float(0,0,0,1,2,2,2)),
        vKnot(M_Float(0,0,0,1,2,2,2)),
        dim(Vec2f(4,4), *this),
        order(Vec2f(3,3)),
        staves(100),
        vCoord(M_Float()),
        coord(M_Vec3f()),
        harmonics(30),

/////////////
        uDegree(2),
        vDegree(2)

    {
        index.length=4;
    }
    M_Int index;
    M_Vec3f r_point;
    M_Float r_weight;

    void processFields();
    void houseKeep();
    void centerViewToControlPoint();
    void drag(Vec3f pt);
    int uDegree;
    int vDegree;

    float minU() {
        return uKnot.refValue()[uDegree];
    }
    float minV() {
        return vKnot.refValue()[vDegree];
    }
    float maxU() {
        return uKnot.refValue()[uKnot.length-uDegree];
    }
    float maxV() {
        return vKnot.refValue()[vKnot.length-vDegree];
    }

    void genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen);
    void updateVCoord(const Oscillator& osc);

private:
    void dpointsToPoints();
};

class Sample : public Timbre, public MField1{
public:
    int getType()const{return SAMPLE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    SFString url;

    Sample():
        MField1(
            &url
        ),
        url(String()),

        /////
        srcBuf(0),
        srcBufLen(0)
    {
        index.length=1;
    }
    M_Int index;

    void genSound(const float volume, const float delay, const int voices,  const float detune, Oscillator& osc, float* fbuf, int bufLen);
    short int* srcBuf;
    int srcBufLen;
    void processFields();
};

class WaveToPoly : public Node, public MField17{
public:
class Open : public SFBool {
public:
    WaveToPoly& home;
    Open(const bool& value, WaveToPoly& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Convert : public SFBool {
public:
    WaveToPoly& home;
    Convert(const bool& value, WaveToPoly& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class AddMark : public SFBool {
public:
    WaveToPoly& home;
    AddMark(const bool& value, WaveToPoly& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class DeleteMark : public SFBool {
public:
    WaveToPoly& home;
    DeleteMark(const bool& value, WaveToPoly& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return WAVETOPOLY;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}

	Open open;
	SFString file;
	SFInt32 viewFrom;
	SFInt32 zoom;
	Convert convert;
	SFInt32 cMark;
	SFInt32 nextCycle;
	AddMark addMark;
	DeleteMark deleteMark;
	MFInt32 cycle;
	SFBool sawTooth;
	SFInt32 windowSize;
	SFBool overlap;
	SFInt32 partials;
	SFFloat pitchLimit;
	SFPart part;
	SFNode osc;

    WaveToPoly():
        MField17(
            &open,
            &file,
            &viewFrom,
            &zoom,
            &convert,
            &cMark,
            &nextCycle,
            &addMark,
            &deleteMark,
            &cycle,
            &sawTooth,
			&windowSize,
			&overlap,
			&partials,
			&pitchLimit,
            &part,
            &osc
        ),
        open(false, *this),
        file(String()),
        viewFrom(0),
        zoom(1),
        convert(false, *this),
        cMark(0),
        nextCycle(0),
        addMark(false, *this),
        deleteMark(false, *this),
        cycle(M_Int()),
        sawTooth(true),
		windowSize(8192),
		overlap(false),
		partials(1024),
		pitchLimit(440),
        part(0),
        osc(0),
//////////
        srcBuf(0),
        srcBufLen(0),
        sampleRate(48000)
    {}
    float* srcBuf;
    int srcBufLen;
    int sampleRate;
    void processFields();
    int getNextCMark();
    int getPrevCMark();
    void harmonicsToMesh(M_<M_<double> >& mm_amp, int numOfHarmonics);
    void findSignal();
	void keepCyclesInRangeOfBuffer();
	void addWindowMarkers();
	void divideMarkers();
	void fadeInOutAtMarkers();
};

class MidiFormant : public Node, public MField11{
public:
    int getType()const{return MIDIFORMANT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

	SFString chord;
	MFInt32	pitch;
	MFInt32	range;
	MFInt32	cent;
	MFInt32	vLine;
	MFInt32	before;
	MFInt32	after;
	MFInt32	bw;
	MFInt32	trough;
	MFInt32	peak;
	SFNode	patch;

    MidiFormant():
        MField11(
			&chord,
			&pitch,
			&range,
			&cent,
			&vLine,
			&before,
			&after,
			&bw,
			&trough,
			&peak,
			&patch
        ),
		chord(String()),
		pitch(M_Int()),
		range(M_Int()),
		cent(M_Int()),
		vLine(M_Int()),
		before(M_Int()),
		after(M_Int()),
		bw(M_Int()),
		trough(M_Int()),
		peak(M_Int()),
		patch(0)
    {
        index.length = 1;
    }
    M_Int index;

	M_Bool update;
	void genChordString(String& str);
	int getFormantFromPitchRange(int key);
	double getFrequency(int formantIdx);
};

class Track : public Node, public MField7{
public:
    int getType()const{return TRACK;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}
/*
class Output : public SFBool {
public:
    Track& home;

    Output(const bool& par, Track& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
*/
    //Output output;
    SFInt32	voices;
    SFFloat	detune;
    SFInt32 echos;
    SFFloat echoDelay;
    SFFloat decay;
    SFFloat volume;
    MFNode sound;

    Track():
        MField7(
            //&output,
            &voices,
            &detune,
            &echos,
            &echoDelay,
            &decay,
            &volume,
            &sound
        ),
        //output(false, *this),
        voices(1),
        detune(0),
        echos(0),
        echoDelay(0),
        decay(0),
        volume(1),
        sound(M_Node())
    {}
    void genSound(float masterVolume, float delay, float* fbuf, int bufLen);
};

class Composition : public Node, public MField5{
public:
    int getType()const{return COMPOSITION;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

class Output : public SFBool {
public:
    Composition& home;

    Output(const bool& par, Composition& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    Output output;
    SFFloat volume;
    SFFloat delay;
    SFFloat duration;
    MFNode track;

    Composition():
        MField5(
            &output,
            &volume,
            &delay,
            &duration,
            &track
        ),
        output(false, *this),
        volume(1),
        delay(0),
        duration(10),
        track(M_Node()),
//
        bufLen(0),
        fbuf(0),
        ibuf(0)
    {}
    void evolveSound();

    int bufLen;
    float* fbuf;
    short int* ibuf;
    void updateAndPlay(Anim* anim, NurbsPatch* patch);
    void updateAndPlay(Anim* anim, Oscillator* osc);
    void updateAndPlay(Anim* anim, Chords* chords);
    void plotRecordNotch(AnimControlPanel* animCP, Anim* anim);
};

class CPMorpher : public Node, public SamplePose, public MField3{
public:

///////////////////////////
    int getType()const{return CPMORPHER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    MFFloat key;
    MFVec3f keyValue;
    MFInt32 cpIndex;

    CPMorpher():
        MField3(
            &key,
            &keyValue,
            &cpIndex
        ),

        key(M_Float()),
        keyValue(M_Vec3f()),
        cpIndex(M_Int()),

        frame(0)
    {
        index.length = 3;
    }
    M_Int index;
    float frame;
    //M_Int cpIndexRunTime;
    void morph(float v, Art& part);
    void do_frame(Art* part);
    void do_polyFrame(const Art* part);
    void do_displayPolyFrame(const Art* part);
    void insertChannel(int& channel, const int k, M_Int& cpIndex, const float slider, const Vec3f& diff, M_Vec3f& keyValue, int& numOfChannels, const int numOfSections);
    void insertChannel(int& channel, const int k, M_Int& cpIndex, M_Vec3f& keyValue, int& numOfChannels, const int numOfSections);
    void deleteChannel(const int& channel, M_Int& cpIndex, M_Float& key, M_Vec3f& keyValue, const int numOfChannels);
    void deleteChannelCoords(const Art* part);
    void deleteChannelPoints(const Art* part);
    void take(float slider, const Vec3f& diff, const int channel, const int numOfChannels);
    void processForOutput(M_Node& parentChild, int& n, Script* gs);
    void copySection(double selectFrom, double selectTo, Node* node, int fieldIdx);
    void pasteSection(double slider, double period, Art* part, int keyIndex, int keyValueIndex, int numOf, int srcNumOfChannels, M_Int& cpIndexSrc, int cpIndexSrcBase);
    void deleteSection(double selectFrom, double selectTo, bool beginEdge, bool endEdge, bool deleteTime);
    void insertTime(double selectFrom, double selectTo, bool beginEdge);
    void scaleTime(double add, double period);
    void take(float sliderconnectNode, Node* node, Field* field); //not used
    void deleteSample(float slider);
    void plotRecordNotch(AnimTimeLine* tl, Anim* anim, Node* node, Field* field);
    bool searchKeyFrameInTimeLine(AnimControlPanel& animCP, const float clickPos);
    void setKeyDragPos(int& idx, const float newTime);
    void connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function);
    void matchChannelLen(M_Int& srcCPIndex, int srcCPIndexOffset, int srcSampleLen);
};

class Strand : public NurbsPatch, public MField2{
    public:

    int getType()const{return STRAND;}
    int getFLen()const{return MField2::size();};
    Field* getField(const int i)const{return MField2::fieldArray[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    Strand():
        MField2(
            &point,
            &cage
        ),

        ////
        geometry(new NurbsPatch),
        hairFrom(0),
        //generate(true),
        slidx(0)
    {
        index.length = 1;
        uJoinEnds.refValue() = false;
    }
    M_Int index;

    int numOfPatches(){return 2;}
    NurbsPatch* geometry;
    int hairFrom;
    //M_Float at;
    //M_Vec3f ringBase;
    //M_Rotation ringRot;
    //bool generate;

    int slidx; //strand list index
    void setStemValues(Stem* stem, int staves, int bands);
    void generateGeometry(Art* part, float radius, int staves, int bands);
    void processFields();
    void houseKeep();
    void drag(Vec3f pt);

    void deleteControlPoint();
    void insertControlPoint();
};

class Lock : public NurbsPatch, public MField21{
    public:

    int getType()const{return LOCK;}
    int getFLen()const{return MField21::size();};
    Field* getField(const int i)const{return MField21::fieldArray[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}


    MFVec2f	hole;
    MFInt32 scpi; //strand control point index

    MFVec2f border;
    MFInt32 region;
    //SFBool dummy;

    MFNode lock;
    MFNode unify;
    MFNode textureMapper;

    Lock():
        MField21(
            &point,
            &dpoint,
            &weight,
            &uKnot,
            &vKnot,
            &order,

            &object,
            &cage,
            &showAll,
            &mirror,
            &dim,

            &endSP,
            &uJoinEnds,
            &beginSP,

            &hole,
            &scpi, //strand control point index

            &border,
            &region,
            //&dummy,

            &lock,
            &unify,
            &textureMapper
        ),

        hole(M_Vec2f()),
        scpi(M_Int()), //strand control point index

        border(M_Vec2f()),
        region(M_Int()),
        //dummy(false),

        lock(M_Node()),
        unify(M_Node()),
        textureMapper(M_Node()),

/////////////
        llidx(0)
    {
        index.length=5;
    }

    int llidx; //lock list index

    M_Int index;
    M_<Strand*> scpiLathe;

    static int selBorder;
    static LockPtr selBorderLock;
    static M_<Lock*> relevantLockList;
    static M_Int borderOffset;
    static M_Bool regionSelected;
    static M_Int copyBorderList;
    static bool splitRegionMode;
    static bool breakBorderMode;
    static M_Vec2f splitRegionPoints;

    void deleteSeperatorMap();
    void copyMapRepete(Lock* src);
    void autoLengthStrands(Lock* src);
    void copyMapRepete(Lock* src, const M_Bool& flag);
    void copyMap(Lock* src);
    void setFlag(M_Bool& flag, bool state);
    void selectFreeEnds(M_Bool& flag);
    void selectRings(M_Bool& flag);
    void scaleMap();
    void showHideBoundStrands(bool state);
    void cutLock();
    //void decrementMap();
    void splitLockMap(Lock* lockB, float ratio, bool doU);
    void houseKeep();
    void houseKeepRegions(M_Vec3f& coord, M_Node& children, const float inverseSize, int& numOfCorners, M_Int& coordIndex, int& s, M_Int& region, M_Color3f& color, int regionSelectedIdx);
    void initTugRegion(const bool perspective);
    void drag(Vec3f pt);
    void insertBorderPoint();
    int numOfPolys();
    bool addedAllready(int& brdrOffstt, M_Vec3f& coord);
    void selectRegions();
    void deleteSelectedRegions();
    void deleteStrandsInSelRegions();
    void seedStrandsInSelRegions();
    void flagStrandsInLock(M_Bool& flag, int& numOfCorners, int& s, const M_Int& region, int polyCntr);
    void flagStrandsInLock(M_Bool& flag);
    void selectLocks(M_Bool& flag);
    void deleteStrandsInSelRegions(M_Bool& flag);
    bool isInsideRegion(int& numOfCorners, int& s, const M_Int& region, const Vec2f hl);
    void connectLockToRegion(M_Int& region, int s, const M_Vec2f& border, int numOfCorners, Lock* copyLock, M_Int& redirect);
    void splitLock2(M_Int& region, int sA, int sB, const M_Vec2f& border, int numOfCorners, Lock* src, M_Int& redirect);
    void connectLockToSelRegions(Lock* src, Lock* copyLock, M_Int& redirect);
    void scaleBorder(int borderOffset);
    Lock* extrudeRegion(M_Int& redirect);
    Lock* splitLock(Lock* src, M_Int& redirect);
    Lock* createLockFromRegion(int s, int numOfCorners, int borderOffset);
    void splitRegion();
    bool splitLineIntersectsRegion(int s, Lock* src, int sSrc, int& numOfCorners, int brdrOffstt, int brdrOffsttSrc);
    bool splitLineIntersectsRegion(const int s, int numOfCorners, int brdrOffstt, const M_Vec3f& coord, Vec2f point, float& ratio, int& roffset0, int& roffset1);
    void selectLocksFromRegions(Lock* src);
    void freePointFromSeperator(int scpiIdx);
    int findMatchingChildBase(M_Int& compareRegion, int compareS, int numOfCorners);
    void redirectChildBase(int sDst, M_Int& fromRegion, int numOfCorners);
    void joinBorderPoints(Lock* src, int closestBorderIdx);
    int getClosestBorderPoint(int compareBorderIdx);
    int getBaseRegionOffset(Lock* src, int blockRegion, Lock*& returnSrc, int& numOfCorners);
    void mergeRegions(Lock* src);
    bool isPointInRegion(int regionIdx, int pointIdx, int numOfCorners, int& h);
    void fixBorderIndexForDelete(Lock* src);
    void breakBorder();
    void centerViewToControlPoint();
    void deleteCorner(Lock* src);
    void rotateSelectedRegions();
};

class Region : public Node, public MField3{
    public:
class BorderPoint : public SFVec2f {
    public:
    Region& home;
    BorderPoint(const Vec2f& value, Region& home):
        SFVec2f(value),
        home(home)
    {};
    void operator=(const Vec2f& par);
};

    int getType()const{return REGION;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    BorderPoint borderPoint;
    SFInt32 borderPoints;
    SFInt32 referencedBorderPoints;

    Region():
        MField3(
            &borderPoint,
            &borderPoints,
            &referencedBorderPoints
        ),

        borderPoint(Vec2f(0, 0), *this),
        borderPoints(0),
        referencedBorderPoints(0)

/////////////
    {
    }
};

class Seed : public Node, public MField2{
    public:
class Ok : public SFBool {
public:
    Seed& home;
    Ok(const bool& value, Seed& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return SEED;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFInt32 number_of_strands;
    Ok ok;

    Seed():
        MField2(
            &number_of_strands,
            &ok
        ),

        number_of_strands(10),
        ok(false, *this)

/////////////
    {
    }
};

class Thinner : public Node, public MField2{
    public:
class Ok : public SFBool {
public:
    Thinner& home;
    Ok(const bool& value, Thinner& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return THINNER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFInt32 number_of_strands;
    Ok ok;

    Thinner():
        MField2(
            &number_of_strands,
            &ok
        ),

        number_of_strands(10),
        ok(false, *this)

/////////////
    {
    }
};

class InsertLock : public Node, public MField2{
    public:
class Ok : public SFBool {
public:
    InsertLock& home;
    Ok(const bool& value, InsertLock& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return INSERTLOCK;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFInt32 insertions;
    Ok ok;

    InsertLock():
        MField2(
            &insertions,
            &ok
        ),

        insertions(1),
        ok(false, *this)

/////////////
    {
    }
};

class Extrude : public Node, public MField6{
    public:

class Distance : public SFFloat{
public:
    Extrude& home;
    Distance(const float& value, Extrude& home):
        SFFloat(value),
        home(home)
    {};
    void operator=(const float& par);
};

class Insertions : public SFInt32{
public:
    Extrude& home;
    Insertions(const int& value, Extrude& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};
class Cap : public SFBool{
public:
    Extrude& home;
    Cap(const bool& value, Extrude& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class Hole : public SFBool{
public:
    Extrude& home;
    Hole(const bool& value, Extrude& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class SpiderWebCenter : public SFBool{
public:
    Extrude& home;
    SpiderWebCenter(const bool& value, Extrude& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class Done : public SFBool {
public:
    Extrude& home;
    Done(const bool& value, Extrude& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return EXTRUDE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Distance distance;
    Insertions insertions;
    Cap _cap;
    Hole hole;
    SpiderWebCenter spiderWebCenter;
    Done done;

    Extrude():
        MField6(
            &distance,
            &insertions,
            &_cap,
            &hole,
            &spiderWebCenter,
            &done
        ),

        distance(0, *this),
        insertions(1, *this),
        _cap(false, *this),
        hole(false, *this),
        spiderWebCenter(false, *this),
        done(false, *this),

/////////////
        centerMirror(false),
        extraCenterMirrorPatch(0),
        smls(0),
        r_m_node(0),
        r_itemIdx(0),
        l_m_node(0),
        l_itemIdx(0),
        leftBeforeRight(false),
		markCp(0)
    {
    }
    Seamless* smls;
    Vec3f normal;
    bool centerMirror;
    void processFields();
    M_Vec3f storePointList;
	M_Int r_storeCpIndex;
	M_Int l_storeCpIndex;
    NurbsPatchPtr r_cap;
    NurbsPatchPtr l_cap;
    NurbsPatchPtr r_shaft;
    NurbsPatchPtr extraCenterMirrorPatch;
    M_Node* r_m_node;
    int r_itemIdx;
    M_Node* l_m_node;
    int l_itemIdx;
    bool leftBeforeRight;
	int markCp;

    void doDistanceA();
    void doDistance(int fromRow = 0);
    void resetTopShaftPoints();
    void closeEndPointsForSpiderWeb();
    void refreshCap();

    void insertCaps();
    void deleteCaps();
    void tidyUpCells();
    void adjustRightItemIdx();
};

class InsertComplete : public Node, public MField1{
    public:
class Done : public SFBool {
public:
    InsertComplete& home;
    Done(const bool& value, InsertComplete& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return INSERTCOMPLETE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Done done;
    //SFBool loop;

    InsertComplete():
        MField1(
            &done
            //&loop
        ),

        done(false, *this)
        //loop(false)
    {
    }
};


class CutComplete : public Node, public MField3{
    public:
class Done : public SFBool {
public:
    CutComplete& home;
    Done(const bool& value, CutComplete& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Continue : public SFBool {
public:
    CutComplete& home;
    Continue(const bool& value, CutComplete& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return CUTCOMPLETE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Done done;
    SFBool unstitch;
	Continue _continue;

    CutComplete():
        MField3(
            &done,
            &unstitch,
			&_continue
        ),

        done(false, *this),
        unstitch(false),
		_continue(false, *this)
    {
    }
	void unstitchPatches(M_Insert& m_insert);
};


class AddLathe : public Node, public MField1, public VisualAid{
    public:
class Done : public SFBool {
public:
    AddLathe& home;
    Done(const bool& value, AddLathe& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return ADDLATHE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Done done;

    AddLathe():
        MField1(
            &done
        ),

        done(false, *this),

////
        mousePhase(0),
        orientationPointArray(Vec3f(-.5, 0, 0), Vec3f(.5, 0, 0), Vec3f(0, 0, -.5), Vec3f(0, 0, .5), Vec3f(0, -.5, 0), Vec3f(0, .5, 0)),
        orientationArray(Rotation(0, 0, 1, PI * .5), Rotation(0, 0, 1, -PI * .5), Rotation(1, 0, 0, -PI * .5), Rotation(1, 0, 0, PI * .5), Rotation(0, 0, 1, PI), Rotation(0, 0, 1, 0))
    {
    }
    enum{
        NEUTRAL = 0,
        CREATE_SPINDLE = 1,
        SET_SPINDLE_ORIENTATION = 2,
        INITIAL_POINTS = 3
    };

    NurbsLathePtr lathe;
    int mousePhase;
    M_Vec3f orientationPointArray;
    M_Rotation orientationArray;

    void addPoint();
    void houseKeep();

	void checkOverInitPoint(const bool perspective);
	void drag(Vec3f pt);

    bool checkOrientationPoint(const bool perspective);
};

class AddBox{
public:
    AddBox():
        mousePhase(0)
    {}
    enum{
        NEUTRAL = 0,
        READY_FOR_A = 1,
        READY_FOR_B = 2,
        READY_FOR_C = 3
    };
    Ptr<BoxBuild> box;
    Vec3f clickPosA;
    Vec3f clickPosB;
    Vec3f clickPosC;
    int mousePhase;

    void addPoint();
    void houseKeep();
    void updateDimensions();
};

class AddSphere{
public:
    AddSphere():
        mousePhase(0)
    {}
    enum{
        NEUTRAL = 0,
        READY_FOR_LOCATION = 1,
        READY_FOR_RADIUS = 2,
    };
    Ptr<SphereBuild> sphere;
    void addPoint();
    void houseKeep();
    int mousePhase;
    void updateRadius();
};

class AddNurbsLine : public Node, public MField2, public VisualAid{
    public:

class Palette : public SFBool {
public:
    AddNurbsLine& home;
    Palette(const bool& value, AddNurbsLine& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Done : public SFBool {
public:
    AddNurbsLine& home;
    Done(const bool& value, AddNurbsLine& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return ADDNURBSLINE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

	Palette palette;
    Done done;

    AddNurbsLine():
        MField2(
			&palette,
            &done
        ),

		palette(false, *this),
        done(false, *this),

////
        mousePhase(0),
        line(0),
        smls(0)
    {
    }
    enum {
        NEUTRAL = 0,
        INITIAL_POINTS = 1
    };

    Seamless* smls;
    NurbsLinePtr line;
    int mousePhase;	//why dont we use a bool instead?
    void addPoint();
    void houseKeep();
	void drag(Vec3f pt);
	void checkOverInitPoint(const bool perspective);
	void newLine();
};

class DrawNurbsLine : public Node, public MField11, public VisualAid{
    public:
class Palette : public SFBool {
public:
    DrawNurbsLine& home;
    Palette(const bool& value, DrawNurbsLine& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class Done : public SFBool {
public:
    DrawNurbsLine& home;
    Done(const bool& value, DrawNurbsLine& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class CtrlPoints : public SFInt32 {
public:
    DrawNurbsLine& home;

    CtrlPoints(const int& par, DrawNurbsLine& home):
        SFInt32(par),
        home(home)
    {};
    void operator=(const int& par);
};
class Loop : public SFBool {
public:
    DrawNurbsLine& home;
    Loop(const bool& value, DrawNurbsLine& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class Evolve : public SFBool {
public:
    DrawNurbsLine& home;
    Evolve(const bool& par, DrawNurbsLine& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return DRAW_NURBSLINE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

	Palette palette;
    Done done;
	CtrlPoints ctrlPoints;
	SFBool thickLine;
	SFBool colorPressure;
	SFFloat colorMult;
	SFBool thickPressure;
	SFFloat thickness;
	SFFloat thickMult;
	Loop loop;
	Evolve evolve;

    DrawNurbsLine():
        MField11(
			&palette,
            &done,
			&ctrlPoints,
			&thickLine,
			&colorPressure,
			&colorMult,
			&thickPressure,
			&thickness,
			&thickMult,
			&loop,
			&evolve
        ),

		palette(false, *this),
        done(false, *this),
		ctrlPoints(6, *this),
		thickLine(false),
		colorPressure(false),
		colorMult(1),
		thickPressure(false),
		thickness(.05),
		thickMult(1),
		loop(false, *this),
		evolve(false, *this),

////
        mousePhase(0),
        line(0),
		mousePressure(0),
        smls(0)
    {
    }
    enum {
        NEUTRAL = 0,
        DRAW = 1,
        READY_TO_DRAG = 2,
    };

    Seamless* smls;
    NurbsLinePtr line;
	NurbsPatchPtr patch;
    int mousePhase;
    void addPoint();
    void houseKeep();
	void drag(Vec3f pt);
	void checkOverInitPoint(const bool perspective);

	int mousePressure;
	M_Vec3f pixelSketch;
	M_Int m_pressure;
	M_Float tSketch;
	M_Float m_processedPressure;
	void displayDrawnLine();
	void convertDrawnLineToNurbsLine();
	void mapDrawnPointToNurbs(float& closestT, const Vec3f targetPoint, const Vec3f* pointPtr, const float* weightPtr, const float* knotPtr, const int dim, float minT, float maxT, const int degree);
	float addUpError();

	void mapDrawnPointsToNurbs();
	void evolveNurbsPointsToDrawnPoints();
	static void getPointOnCurve(const M_Vec3f& point, const M_Float& weight, const M_Float& knot, float minT, float maxT, const int degree, Vec3f& prevCrd, double& t, double tInc, float targetDist, Vec3f& fromVector);
	void processPressure(int idx, int processedPressureIdx);
	Color3f convertPressureToCol(int idx);
	void newLine();
};

class AddBone : public Node, public MField1{
    public:
class Done : public SFBool {
public:
    AddBone& home;
    Done(const bool& value, AddBone& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return ADDBONE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Done done;

    AddBone():
        MField1(
            &done
        ),

        done(false, *this),
        //////
        mousePhase(NEUTRAL),
        ratio(0),
        partA(0),
        k(0),
        rightKeyDown(false)
    {
    }
    enum{
        NEUTRAL = 0,
        ACTIVE = 1,
        INSERT = 2, //over line
    };
    float ratio;
    Art* partA;
    ArtPtr partB;
    int k;
    bool rightKeyDown;

    int mousePhase;
    void addBone();
    void initMouseOverLine(Seamless* smls);
    void insertBone();

    M_Vec3f beginDragPivotPoint; //not actually used for AddBone but is used for g_dragWithDescendants
};


class Grid : public Node, public MField8{
    public:

class Scale : public SFFloat {
public:
    Grid& home;

    Scale(const float& par, Grid& home):
        SFFloat(par),
        home(home)
    {};
    void operator=(const float& par);
};

class Squares : public SFInt32 {
public:
    Grid& home;

    Squares(const int& par, Grid& home):
        SFInt32(par),
        home(home)
    {};
    void operator=(const int& par);
};

class SubSquares : public SFInt32 {
public:
    Grid& home;

    SubSquares(const int& par, Grid& home):
        SFInt32(par),
        home(home)
    {};
    void operator=(const int& par);
};

class XzPlane : public SFBool {
public:
    Grid& home;

    XzPlane(const bool& par, Grid& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class XyPlane : public SFBool {
public:
    Grid& home;

    XyPlane(const bool& par, Grid& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class YzPlane : public SFBool {
public:
    Grid& home;

    YzPlane(const bool& par, Grid& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return GRID;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Scale scale;
    Squares	squares;
    SubSquares subSquares;
    XzPlane	xzPlane;
    XyPlane	xyPlane;
    YzPlane	yzPlane;
    SFBool	snap;
    SFBool	show;

    Grid():
        MField8(
            &scale,
            &squares,
            &subSquares,
            &xzPlane,
            &xyPlane,
            &yzPlane,
            &snap,
            &show
        ),

        scale(1, *this),
        squares(16, *this),
        subSquares(10, *this),
        xzPlane(false, *this),
        xyPlane(false, *this),
        yzPlane(false, *this),
        snap(false),
        show(true)

////
    {
    }
    ~Grid()
    {
    };
    void houseKeep();
    void snapIt(Vec2f& par);
    void snapIt(Vec3f& par);
};

class Duplicate : public Node, public MField5{

public:
class Run : public SFBool {
public:
    Duplicate& home;

    Run(const bool& par, Duplicate& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return DUPLICATE_PATCHES;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Run run;
    SFInt32	iterations;
	SFVec3f scale;
    SFRotation rotation;
	SFVec3f translation;

    Duplicate():
        MField5(
            &run,
            &iterations,
			&scale,
            &rotation,
			&translation
        ),

        run(false, *this),
        iterations(1),
		scale(Vec3f(1,1,1)),
        rotation(Rotation()),
		translation(Vec3f())
////
    {
    }
    ~Duplicate()
    {
    };
};


class InsideOutPart : public Node, public MField2{

public:
class Ok : public SFBool {
public:
    InsideOutPart& home;

    Ok(const bool& par, InsideOutPart& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return INSIDEOUT_PART;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool add;
    Ok ok;

    InsideOutPart():
        MField2(
            &add,
            &ok
        ),

        add(true),
        ok(false, *this),

        //
        active(false),
        source(0)
    {
    }
    ~InsideOutPart()
    {
    };

    bool active;

    Art* source;
    void doInsideOutBnspe(CopyPart& cp, Art* dest);
};

class ExportMovie : public Node, public MField7{
public:
class Output : public SFBool {
public:
    ExportMovie& home;

    Output(const bool& par, ExportMovie& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class OpenOutputFolder : public SFBool {
public:
    ExportMovie& home;

    OpenOutputFolder(const bool& par, ExportMovie& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class PovRay : public SFBool {
public:
    ExportMovie& home;

    PovRay(const bool& par, ExportMovie& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class MovieMPG : public SFBool {
public:
    ExportMovie& home;

    MovieMPG(const bool& par, ExportMovie& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class MovieMP4 : public SFBool {
public:
    ExportMovie& home;

    MovieMP4(const bool& par, ExportMovie& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class MovieAVI : public SFBool {
public:
    ExportMovie& home;

    MovieAVI(const bool& par, ExportMovie& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class MovieFLV : public SFBool {
public:
    ExportMovie& home;

    MovieFLV(const bool& par, ExportMovie& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

    int getType()const{return EXPORT_MOVIE;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Output exportFile;
    OpenOutputFolder openExportFolder;
    PovRay povRay;
    MovieAVI avi;
    MovieMPG mpg;
    MovieMP4 mp4;
    MovieFLV flv;

    ExportMovie():
        MField7(
            &exportFile,
            &openExportFolder,

            &povRay,

            &avi,
            &mpg,
            &mp4,
            &flv
        ),
        exportFile(false, *this),
        openExportFolder(false, *this),

        povRay(false, *this),

        avi(false, *this),
        mpg(false, *this),
        mp4(false, *this),
        flv(false, *this)
    {}
    void processFields();
    void exportScene(Anim* anim = 0);
};

class ExportVrmlX3d : public Node, public MField14{
    public:
class Output : public SFBool {
public:
    ExportVrmlX3d& home;

    Output(const bool& par, ExportVrmlX3d& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class OpenOutputFolder : public SFBool {
public:
    ExportVrmlX3d& home;

    OpenOutputFolder(const bool& par, ExportVrmlX3d& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};
class Vrml : public SFBool {
public:
    ExportVrmlX3d& home;

    Vrml(const bool& par, ExportVrmlX3d& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class X3dVrmlClassic : public SFBool {
public:
    ExportVrmlX3d& home;

    X3dVrmlClassic(const bool& par, ExportVrmlX3d& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};

class X3dXml : public SFBool {
public:
    ExportVrmlX3d& home;

    X3dXml(const bool& par, ExportVrmlX3d& home):
        SFBool(par),
        home(home)
    {};
    void operator=(const bool& par);
};


    int getType()const{return EXPORT_VRMLX3D;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    Output exportFile;
    OpenOutputFolder openExportFolder;

    SFBool gZip;
    Vrml vrml;
    X3dVrmlClassic x3dVrmlClassic;
    X3dXml x3dXml;


    SFBool genHAnimNodes;
    SFBool genIdxTriSetNodes;
    SFBool genContactSBVT;
    SFBool genNormals;
    SFBool genColorIndex;
    SFBool segmentedParts;
    SFFloat roundCoords;
    SFBool scriptAsRoutes;

    ExportVrmlX3d():
        MField14(
            &exportFile,
            &openExportFolder,

            &gZip,
            &vrml,
            &x3dVrmlClassic,
            &x3dXml,

            &genHAnimNodes,
            &genIdxTriSetNodes,
            &genContactSBVT,
            &genNormals,
            &genColorIndex,
            &segmentedParts,
            &roundCoords,
            &scriptAsRoutes
        ),
        exportFile(false, *this),
        openExportFolder(false, *this),
        gZip(true),
        vrml(true, *this),
        x3dVrmlClassic(false, *this),
        x3dXml(false, *this),

        genHAnimNodes(false),
        genIdxTriSetNodes(false),
        genContactSBVT(true),
        genNormals(true),
        genColorIndex(false),
        segmentedParts(false),
        roundCoords(3),
        scriptAsRoutes(false)

////
    {
    }
    void exportScene();
};

class RingletGenerator : public Node, public MField6{
    public:

    int getType()const{return RINGLETGENERATOR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    TSFVec2f _size;
    TSFFloat radius;
    TSFBool clockwise;
    TSFFloat distance;
    TSFInt revolutions;
    TSFInt insertions;

    RingletGenerator():
        MField6(
            &_size,
            &radius,
            &clockwise,
            &distance,
            &revolutions,
            &insertions
        ),

        //distance(0, *this)
        _size(Vec2f(.01, .03), *this),
        radius(.1, *this),
        clockwise(false, *this),
        distance(.5, *this),
        revolutions(2, *this),
        insertions(4, *this)
/////////////
    {
    }
    Vec3f normal;

    void processFields();
};

class RobotSave : public Node, public MField6{
    public:
enum {
    ACCUMULATE_SELECT = 0x0001,
    HIDE_POLYS_IN_HIDDEN_PATCHES = 0x0002,
};
    int getType()const{return ROBOTSAVE;}
	int getFLen()const{return size();};
	Field* getField(const int i)const{return (*this)[i];}
	int getFieldGroupIndex(const int &i)const{return index[i];}
	void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
	M_Int* getIndexPtr(){return &index;}

    SFInt32 mode;
	SFInt32 selSmls;
    MFInt32 pointList;
	MFVec3f beginDragPos;
	SFNode transformCP;
	SFNode animLever;

    RobotSave():
        MField6(
            &mode,
			&selSmls,
            &pointList,
			&beginDragPos,
			&transformCP,
			&animLever
        ),

        //distance(0, *this)
        mode(0),
		selSmls(0),
        pointList(M_Int()),
		beginDragPos(M_Vec3f()),
		transformCP(0),
		animLever(0)
    {
		index.length = 1;
	}
	M_Int index;

    void processFields();
	void copyData();
};

class Hair : public Build, public MField13, public VisualAid{
public:

class MFStrand : public MFNode{
public:
    MFStrand(const M_Node& value):
        MFNode(value)
    {}
    M_Strand& refValue(){
        return *(M_Strand*)&length;
    }
    operator const M_Strand&()const{
        return *(M_Strand*)&getValue();
    }
    void operator=(const M_Node& par){
        refValue()=*(M_Strand*)&par;
    }
    void operator+=(const NodePtr& par){
        refValue()+=dynamic_cast<Strand*>(par.p);
    }
    void operator+=(Strand* par){
        refValue()+=par;
    }

};

class HSFPart : public SFNode{
public:
    Hair& home;
    HSFPart( const NodePtr& par, Hair& home):
        SFNode(par),
        home(home)
    {};
    Node* operator=(Node* par);
    ArtPtr& refValue(){
        return *((ArtPtr*)&SFNode::refValue());
    }
    const ArtPtr& refValue()const{
        return *((const ArtPtr*)&SFNode::refValue());
    }

    Art* operator ->(){
        return reinterpret_cast<Art*>(p);
    }
};
///////////////////////////
    int getType()const{return HAIR;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &i)const{return index[i];}
    void setFieldGroupIndex(const int &i, const int &val){index[i]=val;}
    M_Int* getIndexPtr(){return &index;}

    SFFloat	radius;
    SFInt32 staves;
    SFInt32 bands;
    SFInt32 strands;
    MFVec2f	hole;
    MFInt32 scpi; //strand control point index
    MFInt32 beginIndex;
    MFInt32 numOfHoles;
    HSFPart part;
    MFStrand strand;
    MFNode lock;
    TMFNode textureMapper;
    Setup setup;

    Hair():
        Build(setup),
        MField13(
            &radius,
            &staves,
            &bands,
            &strands,
            &hole,
            &scpi,
            &beginIndex,
            &numOfHoles,
            &part,
            &strand,
            &lock,
            &textureMapper,
            &setup
        ),
        radius(.0005),
        staves(8),
        bands(4),
        strands(0),
        hole(M_Vec2f()),
        scpi(M_Int()),
        beginIndex(M_Int()),
        numOfHoles(M_Int()),
        part(0, *this),
        strand(M_Node()),
        lock(M_Node()),
        textureMapper(M_Node(), *this),
        setup(0, *this),
////
        //postLoadProcess(true),
        postLoadProcess2(true)

    {
        index.length = 2;
    }
    M_<Lock*> lockList;
    M_<Strand*> scpiLathe;
    M_Int index;
    //bool postLoadProcess;
    bool postLoadProcess2;
    static LockPtr selLock;
    static int selHoleIndex;

    bool illegalImmediateRef(const Art*, M_Art&)const{return false;};
    void doBuild();
    void doLocksUpdateStrands();
    void cpNodeListAdd(Seamless* smls);

    void houseKeep();
    void drag(Vec3f pt);
    void centerViewToControlPoint();

    void initTug();
    void processFields();
    void processToFileFormat();
    void processToWorkFormat();
    void bindToSeperator(Strand* sd);
    void locateBendyLatheNode();
    Strand* clone(Strand* oldStrand);
    void cloneSeperator(Strand* oldSd, Strand* newSd);

    bool isBoundToLock(Strand* ctrlLine);
    bool isBoundToLock(Strand* ctrlLine, int idx, Lock*& lock, int& scpiIdx);

    void deleteStrandPoint(Strand* strand, int idx);
    void insertStrandPoint(Strand* strand, int idx);
    void insertStrandControlPoints(Lock* lock, const M_Bool& flag);
    void deleteBoundControlPoints(Lock* lock);
    void insertStrandControlPoints(Lock* lock);

    void refreshHairOffset();
    void generateMultiples();
    void bindToLock(Lock* lock, Strand* sd, const int pointIdx, Vec2f hl);

    void copySeperatorMapForRepete(int fromIdx, int toIdx);
    void copySeperatorMap(int fromIdx, int toIdx);

    void selectRings(Lock* lock);
    //void insertStrandControlPoints(Seamless* smls, Lock* lock, float ratio);
    void convertOldScpiToNew();
    void buildLockListLoop(Lock* lock, int &_llidx);
    void buildLockList();
    void showHideBoundStrands(bool state);
    void selectLocks(M_Bool& flag);
    void flagStrandsInLock(Lock* lock, M_Bool& flag);
    void deleteStrandsInSelRegions(M_Bool& flag);
    void seedStrandsInSelRegions(Lock* lock, int numOfSeeds);
    void addNewBoundStrand(Lock* lock, const Vec2f hl);
	void convertToPatches();
};

class PartAnimClue{
public:
    virtual void dragPart(Vec3f pt)=0;
};
class Demo : public Node, public MField5{
public:
class Operation : public SFInt32 {
public:
    Demo& home;

    Operation(const float& value, Demo& home):
        SFInt32(value),
        home(home)
    {};
    void operator=(const int& par);
};
class Play : public SFBool {
public:
    Demo& home;
    Play(const bool& value, Demo& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class Stop : public SFBool {
public:
    Demo& home;
    Stop(const bool& value, Demo& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
    int getType()const{return DEMO;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}

    Operation operation;
    Play play;
    Stop stop;
    SFBool express;
    SFString status;

    Demo():
        MField5(
            &operation,
            &play,
            &stop,
            &express,
            &status
        ),
        operation(0, *this),
        play(false, *this),
        stop(false, *this),
        express(false),
        status(String()),

//////////////
        fileName(0),
        firstOperation(false)
    {
    }
    char* fileName;
    void backupToCache();
    void (*function)();
    bool firstOperation;
};
class ConvertToPatch : public Node, public MField7{
    public:
class Done : public SFBool {
public:
    ConvertToPatch& home;
    Done(const bool& value, ConvertToPatch& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

////////////////
    int getType()const{return CONVERTTOPATCH;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    TSFInt ctrlStaves;
    TSFInt ctrlBands;
    TSFInt ctrlZStaves;
    TSFBool stitched;
    TSFBool stem;
    TSFBool set_all_weights_to_1;
    Done done;

    ConvertToPatch():
        MField7(
            &ctrlStaves,
            &ctrlBands,
            &ctrlZStaves,
            &stitched,
            &stem,
            &set_all_weights_to_1,
            &done
        ),

        ctrlStaves(4, *this),
        ctrlBands(4, *this),
        ctrlZStaves(4, *this),
        stitched(true, *this),
        stem(false, *this),
        set_all_weights_to_1(true, *this),
        done(false, *this),


		markCp(0)
    {
    }
	int markCp;
    BuildPtr convertBuild;
    BuildPtr mirrorConvertBuild; //needed to hold memory
    M_Node stemStore;
    M_Node mirrorStemStore;

    void convertToPatch();
    void commonStemless(NurbsPatch* patch, M_Node& stemStore);
    void commonStemless(M_<NurbsPatchPtr>& m_patch, M_Node& stemStore);
    void processFields();
    void processFields(Build* convertBuild, M_Node& stemStore, const int mark);
};
class ReduceCapturedPoses : public Node, public MField2 {
    public:
////////////////
    int getType()const{return REDUCECAPTUREDPOSES;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFInt32 keepCaptureForEvery;
    TSFBool reduce;

    ReduceCapturedPoses():
        MField2(
            &keepCaptureForEvery,
            &reduce
        ),

        keepCaptureForEvery(2),
        reduce(false, *this)
    {
    }
    void processFields();
};

class TransformObj : public Node, public MField5{
public:
enum {
    NOTHING,//delete this?
    NEUTRAL,
    ROTATE,
    TRANSLATE,
	SELECT
};
    int getType()const{return TRANSFORMOBJ;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFVec3f pivotPoint;
    SFRotation scaleOrientation;
    SFVec3f scale;
    SFRotation rotation;
    SFVec3f translation;

    TransformObj():
        MField5(
            &pivotPoint,
            &scaleOrientation,
            &scale,
            &rotation,
            &translation
        ),
        pivotPoint(Vec3f()),
        scaleOrientation(Rotation()),
        scale(Vec3f(1,1,1)),
        rotation(Rotation()),
        translation(Vec3f()),
//
        stretchHappened(false),

        leverAxis(M_Vec3f(Vec3f(1, 0, 0), Vec3f(0, 1, 0))),
        defaultRotCtrlPnt(M_Vec3f(Vec3f(.5, 0, 0), Vec3f(0, .5, 0))),
        leverDistance(M_Float(.5, .5)),
        leverIdx(0),
        mode(NOTHING),
        leverAngle(0)
    {
        rotCtrlPnt.length = 2;
    }
    int mode;
    bool stretchHappened;
    M_Vec3f rotCtrlPnt;
    Vec3f remRotCtrlPnt;
    const M_Vec3f leverAxis;
    const M_Vec3f defaultRotCtrlPnt;
    M_Float leverDistance;
    float leverAngle;
    int leverIdx;
    Rotation remRotation;
    Vec3f remTranslation;

    void getPivotPointMatrix(Matrix& pivotPointMatrix, Node* node) const;
    bool checkCtrlPoint(const bool perspective);
    void selectNodeAsObject();
    void calcPivotPoint();
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void resetRotationLever();
    void houseKeep();
    void calcRotCtrlPnts();
    bool isOverLever();
    void refreshRemPoints();
    void setMode(int newMode);
    int getKeyMode() const;
    void dragRot(Seamless* smls, Vec3f pt);
    void calcNonSelectedRotCtrlPnts();
    void processFields();
};

class TransformCP : public Node, public MField23, public VisualAid{
public:
enum {
    NOTHING,//delete this?
    NEUTRAL,
    ROTATE,
    SCALE1D,
    SCALE2D,
    SCALE3D,
    ROTATE_AND_SCALE1D,
    TRANSLATE
};

class XTug : public SFBool {
public:
    TransformCP& home;
    XTug(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class YTug : public SFBool {
public:
    TransformCP& home;
    YTug(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class ZTug : public SFBool {
public:
    TransformCP& home;
    ZTug(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};


class LeverNeutral : public SFBool {
public:
    TransformCP& home;
    LeverNeutral(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class LeverRotate : public SFBool {
public:
    TransformCP& home;
    LeverRotate(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class LeverStretch : public SFBool {
public:
    TransformCP& home;
    LeverStretch(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class LeverRotateAndStretch : public SFBool {
public:
    TransformCP& home;
    LeverRotateAndStretch(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class LeverScale : public SFBool {
public:
    TransformCP& home;
    LeverScale(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class LeverTranslate : public SFBool {
public:
    TransformCP& home;
    LeverTranslate(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class SelectObject : public SFBool {
public:
    TransformCP& home;
    SelectObject(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class SameScreenPosition : public SFBool {
public:
    TransformCP& home;
    SameScreenPosition(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class RotAxisX : public SFBool {
public:
    TransformCP& home;
    RotAxisX(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class RotAxisY : public SFBool {
public:
    TransformCP& home;
    RotAxisY(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class RotAxisZ : public SFBool {
public:
    TransformCP& home;
    RotAxisZ(const bool& value, TransformCP& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class PivotPoint : public SFVec3f{
public:
    TransformCP& home;
    PivotPoint(const Vec3f& par, TransformCP& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par);
    void operator=(const PivotPoint& par){
        *this = par.refValue();
    }
};

class Translation : public SFVec3f{
public:
    TransformCP& home;
    Translation(const Vec3f& par, TransformCP& home):
        SFVec3f(par),
        home(home)
    {};
    void operator=(const Vec3f& par);
    void operator=(const Translation& par){
        *this = par.refValue();
    }
};

    int getType()const{return TRANSFORMCP;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    XTug	xTug;
    YTug	yTug;
    ZTug	zTug;

	SFInt32 leverSelection;

	LeverNeutral leverNeutral;
    LeverRotate leverRotate;
    LeverStretch leverStretch;
	LeverRotateAndStretch leverRotateAndStretch;
	LeverScale leverScale;
	LeverTranslate leverTranslate;

    SelectObject selectObject;
    SameScreenPosition sameScreenPosition;
    RotAxisX rotAxisX;
    RotAxisY rotAxisY;
    RotAxisZ rotAxisZ;
    PivotPoint pivotPoint;
    TSFRotation scaleOrientation;
    TSFVec3f scale;
    TSFRotation rotation;
    Translation translation;
	SFInt32 leverIdx;
    SFFloat leverAngle;
	MFFloat leverDistance;

    TransformCP():
        MField23(
            &xTug,
            &yTug,
            &zTug,
			&leverSelection,
			&leverNeutral,
			&leverRotate,
			&leverStretch,
			&leverRotateAndStretch,
			&leverScale,
			&leverTranslate,

            &selectObject,
            &sameScreenPosition,
            &rotAxisX,
            &rotAxisY,
            &rotAxisZ,
            &pivotPoint,
            &scaleOrientation,
            &scale,
            &rotation,
            &translation,
			&leverIdx,
			&leverAngle,
			&leverDistance
        ),

        xTug(true, *this),
        yTug(true, *this),
        zTug(true, *this),
		leverSelection(3),

		leverNeutral(false, *this),
		leverRotate(true, *this),
		leverStretch(false, *this),
		leverRotateAndStretch(false, *this),
		leverScale(false, *this),
		leverTranslate(false, *this),

        selectObject(false, *this),
        sameScreenPosition(false, *this),
        rotAxisX(false, *this),
        rotAxisY(false, *this),
        rotAxisZ(true, *this),
        pivotPoint(Vec3f(), *this),
        scaleOrientation(Rotation(), *this),
        scale(Vec3f(1,1,1), *this),
        rotation(Rotation(), *this),
        translation(Vec3f(), *this),
		leverIdx(0),
        leverAngle(0),
		leverDistance(M_Float(.5, .5)),

////
		showDimFilterVisual(false),
		dimFilterClickEnable(false),
		selectRing(false),
		selectRings(false),
        dragRotPointMode(false),
        dragPivotPointMode(false),
        stretchHappened(false),
//		leverAxis(M_Vec3f(Vec3f(1, 0, 0), Vec3f(0, 1, 0), Vec3f(-1, 0, 0), Vec3f(0, -1, 0))),
//		defaultRotCtrlPnt(M_Vec3f(Vec3f(.5, 0, 0), Vec3f(0, .5, 0), Vec3f(-.5, 0, 0), Vec3f(0, -.5, 0))),
//		leverDistance(M_Float(.5, .5, .5, .5)),

        leverAxis(M_Vec3f(Vec3f(1, 0, 0), Vec3f(0, 1, 0))),
        defaultRotCtrlPnt(M_Vec3f(Vec3f(.5, 0, 0), Vec3f(0, .5, 0))),
		defaultRotCtrlPnt2Offset(M_Vec3f(Vec3f(0, .07, 0), Vec3f(-.07, 0, 0))),//neutral
		defaultRotCtrlPnt3Offset(M_Vec3f(Vec3f(.07, 0, 0), Vec3f(0, .07, 0))),//squash/stretch
		defaultRotCtrlPnt4Offset(M_Vec3f(Vec3f(-.07, 0, 0), Vec3f(0, -.07, 0))),//translate
		defaultRotCtrlPnt5Offset(M_Vec3f(Vec3f(-.07, .07, 0), Vec3f(-.07, -.07, 0))),//scale3d
        mode(NOTHING),
		overWhatPoint(NOTHING),
		isOverDimPoint(false),
        isOverPivotPoint(false),
		selectWholePatches(false)
    {
        rotCtrlPnt.length = 2;
		rotCtrlPnt2.length = 2;
		rotCtrlPnt3.length = 2;
		rotCtrlPnt4.length = 2;
		rotCtrlPnt5.length = 2;
    }
    int mode;
	int overWhatPoint;
	Vec3f mouseOffset;
	Vec2f robotOffset;

	bool showDimFilterVisual;
	bool dimFilterClickEnable;
	bool selectRing;
	bool selectRings;
    bool stretchHappened;
    bool dragRotPointMode;
    bool dragPivotPointMode;
    bool isOverPivotPoint;
	bool isOverDimPoint;
	bool selectWholePatches;
    M_Vec3f rotCtrlPnt;//rotate
	M_Vec3f rotCtrlPnt2;//neutral
	M_Vec3f rotCtrlPnt3;//squash
	M_Vec3f rotCtrlPnt4;//translate
	M_Vec3f rotCtrlPnt5;//scale3d
    Vec3f remRotCtrlPnt;
	Vec3f remCtrlPnt; //so add newLine can be set to previous clicked z layer
	Vec3f dimRadialAdjust;
	Vec3f clickPoint;
    const M_Vec3f leverAxis;
    const M_Vec3f defaultRotCtrlPnt;//rotate
	const M_Vec3f defaultRotCtrlPnt2Offset;//newutral
	const M_Vec3f defaultRotCtrlPnt3Offset;//squash
	const M_Vec3f defaultRotCtrlPnt4Offset;//translate
	const M_Vec3f defaultRotCtrlPnt5Offset;//scale3d
	Rotation leverRot;
    Rotation remRotation;
    Vec3f remTranslation;
    Vec3f lockedAxis;

    void houseKeep();

    bool checkCtrlPoint(const bool perspective);
	bool checkDimPoints(const bool perspective);
    void dragRot(Seamless* smls, Vec3f pt);
	void refreshRemPointsMin();
    void refreshRemPoints();
    void processFields();
    void processFieldsForAnim(Art* part);
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    void getPivotPointMatrix(Matrix& pivotPointMatrix, Seamless* smls) const;
    void resetRotationLever();
	void calcScaleOrientation();
	void calcLeverAngle();
	void calcPivotPoint();
    void quantizeRotationLever();
    void selectObjectPoints(M_<NurbsPatch*>& contigList);
    int getKeyMode() const;
    void setMode(int newMode);
	void setRemRotCtrlPnt();
	void restoreCursorPoint();
    void calcNonSelectedRotCtrlPnts();
    void calcRotCtrlPnts();
    bool isOverLever();
	void drag(Vec3f pt);
	void scalePoints(bool lock, bool x, bool z);
	Vec2f getMultiPoint(const int leverIdx, const Vec2f& rotPoint, const M_Vec3f& defaultRotCtrlPntOffset) const;

	void doNurbsRadicalChange(const	M_Vec3f& diff, const M_<NurbsPatch*>& srcP, const M_Int& srcI, const M_Vec2f& mapVec, const M_<NurbsPatch*>& mapP, Seamless* smls);

    void setFilteredDimension(Vec3f& point, const Vec3f& vec){
        if(xTug){
            point.x = vec.x;
        }
        if(yTug){
            point.y = vec.y;
        }
        if(zTug){
            point.z = vec.z;
        }
    }
    void setFilteredDimension(M_Vec3f& point, M_Vec3f& cp, M_Int& cpIndex, const int& idx, const Vec3f& vec){
#ifdef CPINDEX
        if(xTug){
			cp[cpIndex[idx]].x = vec.x;
            point[idx].x = vec.x;
        }
        if(yTug){
			cp[cpIndex[idx]].y = vec.y;
            point[idx].y = vec.y;
        }
        if(zTug){
			cp[cpIndex[idx]].z = vec.z;
            point[idx].z = vec.z;
        }
#else
        if(xTug){
            point[idx].x = vec.x;
        }
        if(yTug){
            point[idx].y = vec.y;
        }
        if(zTug){
            point[idx].z = vec.z;
        }
#endif
    }

};


class TimelineFilter : public Node, public MField19{
public:
enum {
    POSCOMPENSATION = -1,
    MORPH = -2,
	NIK = -3,
};

class Ok : public SFBool {
public:
    TimelineFilter& home;
    Ok(const bool& value, TimelineFilter& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class UnTickAllFields : public SFBool {
public:
    TimelineFilter& home;
    UnTickAllFields(const bool& value, TimelineFilter& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class TickPartFieldsOnly : public SFBool {
public:
    TimelineFilter& home;
    TickPartFieldsOnly(const bool& value, TimelineFilter& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class TickViewpointFieldsOnly : public SFBool {
public:
    TimelineFilter& home;
    TickViewpointFieldsOnly(const bool& value, TimelineFilter& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class SelectAllFields : public SFBool {
public:
    TimelineFilter& home;
    SelectAllFields(const bool& value, TimelineFilter& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};


int getType()const{return TIMELINEFILTER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFBool morph;
    SFBool rotation;
    SFBool translation;
	SFBool nik;
    SFBool posCompensation;
    SFBool fka;
    SFBool fkb;
	SFBool live;
	SFBool zoom;
    SFBool fieldOfView;
    SFBool orientation;
	SFBool lookAt;
	SFBool up;
    SFBool position;
    UnTickAllFields unTickAllFields;
    TickPartFieldsOnly tickPartFieldsOnly;
    TickViewpointFieldsOnly tickViewpointFieldsOnly;
    SelectAllFields selectAllFields;
    Ok ok;

    TimelineFilter():
        MField19(
            &morph,
            &rotation,
            &translation,
			&nik,
            &posCompensation,
            &fka,
            &fkb,
			&live,
            &zoom,
            &fieldOfView,
            &orientation,
			&lookAt,
			&up,
            &position,
            &unTickAllFields,
            &tickPartFieldsOnly,
            &tickViewpointFieldsOnly,
            &selectAllFields,
            &ok
        ),
        morph(true),
        rotation(true),
        translation(true),
		nik(true),
        posCompensation(true),
        fka(true),
        fkb(true),
		live(true),
        zoom(true),
        fieldOfView(true),
        orientation(true),
		lookAt(true),
		up(true),
        position(true),
        unTickAllFields(false, *this),
        tickPartFieldsOnly(false, *this),
        tickViewpointFieldsOnly(false, *this),
        selectAllFields(false, *this),
        ok(false, *this),
////////

        okMode(0)

    {
    }
enum OkModeCommands{
    back_to_selected_node,
    back_to_pasteOptions,
    back_to_deleteOptions
};

    int okMode;
};


class PasteOptions : public Node, public MField16{
public:

class Filter : public SFBool {
public:
    PasteOptions& home;
    Filter(const bool& value, PasteOptions& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class CalcRot : public SFBool {
public:
    PasteOptions& home;
    CalcRot(const bool& value, PasteOptions& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Curve : public SFBool {
public:
    PasteOptions& home;
    Curve(const bool& value, PasteOptions& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Ok : public SFBool {
public:
    PasteOptions& home;
    Ok(const bool& value, PasteOptions& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

int getType()const{return PASTEOPTIONS;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Filter filter;
    SFBool insert;
    SFBool beginEdge;
    SFBool endEdge;
    SFInt32 steps;
    SFBool quantize;
    SFBool transform;
    CalcRot calcRot;
    SFVec3f pivotPoint;
    SFVec3f scale;
    SFRotation rotate;
    SFVec3f translate;
    SFBool mirror;
    Curve curve;
    SFFloat curveDistance;
    Ok ok;

    PasteOptions():
        MField16(
            &filter,
            &insert,
            &beginEdge,
            &endEdge,
            &steps,
            &quantize,
            &transform,
            &calcRot,
            &pivotPoint,
            &scale,
            &rotate,
            &translate,
            &mirror,
            &curve,
            &curveDistance,
            &ok
        ),
        filter(false, *this),
        insert(false),
        beginEdge(true),
        endEdge(true),
        steps(0),
        quantize(true),
        transform(false),
        calcRot(false, *this),
        pivotPoint(Vec3f()),
        scale(Vec3f(1,1,1)),
        rotate(Rotation()),
        translate(Vec3f()),
        mirror(false),
        curve(false, *this),
        curveDistance(0),
        ok(false, *this),

        insert2(false),
        stepFrac(0),
        selectedPeriod(0),
        selectedFrac(0),
        selectedFrac2(0),
        frameDiffRatio(0),
        periodWarp(0),
        newPeriod(0)
    {
    }
    bool insert2;
    double stepFrac;
    double selectedPeriod;
    double selectedFrac;
    double selectedFrac2;
    double frameDiffRatio;
    double periodWarp;
    double newPeriod;

    void process(Anim* anim);
    void pasteSection(SamplePose* samplePose, double slider, double period, int& firstSample, int& lastSample, int numOf, double& add, double& scaleTime);
};

class DeleteOptions : public Node, public MField5{
public:

class Filter : public SFBool {
public:
    DeleteOptions& home;
    Filter(const bool& value, DeleteOptions& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

class Ok : public SFBool {
public:
    DeleteOptions& home;
    Ok(const bool& value, DeleteOptions& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};

int getType()const{return DELETEOPTIONS;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Filter filter;
    SFBool time;
    SFBool beginEdge;
    SFBool endEdge;
    Ok ok;

    DeleteOptions():
        MField5(
            &filter,
            &time,
            &beginEdge,
            &endEdge,
            &ok
        ),
        filter(false, *this),
        time(false),
        beginEdge(true),
        endEdge(true),
        ok(false, *this)
    {
    }
};


class FloorHeight : public Node, public MField2{
    public:
class ZeroHeight : public SFBool{
public:
    FloorHeight& home;

    ZeroHeight(const bool& par, FloorHeight& home):
        SFBool(par),
        home(home)
    {};
    void operator = (const bool& par);
};
class BlaxxunHeight : public SFBool{
public:
    FloorHeight& home;

    BlaxxunHeight(const bool& par, FloorHeight& home):
        SFBool(par),
        home(home)
    {};
    void operator = (const bool& par);
};

////////////////
    int getType()const{return FLOORHEIGHT;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    ZeroHeight zeroHeight;
    BlaxxunHeight blaxxunHeight;

    FloorHeight():
        MField2(
            &zeroHeight,
            &blaxxunHeight
        ),
        zeroHeight(false, *this),
        blaxxunHeight(false, *this)
    {
    }
};


class Message1Button : public Node, public MField2{
    public:

class Ok : public SFBool{
public:
    Message1Button& home;

    Ok(const bool& par, Message1Button& home):
        SFBool(par),
        home(home)
    {};
    void operator = (const bool& par);
};

////////////////
    int getType()const{return MESSAGE_1_BUTTON;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFString msg;
    Ok ok;

    Message1Button():
        MField2(
            &msg,
            &ok
        ),
        msg(""),
        ok(false, *this)

////
    {
    }
    static void setupMessageWindow(const String& msg);
};


class QuestionMessage3Button : public Node, public MField4{
    public:

class Yes : public SFBool{
public:
    QuestionMessage3Button& home;

    Yes(const bool& par, QuestionMessage3Button& home):
        SFBool(par),
        home(home)
    {};
    void operator = (const bool& par);
};
class No : public SFBool{
public:
    QuestionMessage3Button& home;

    No(const bool& par, QuestionMessage3Button& home):
        SFBool(par),
        home(home)
    {};
    void operator = (const bool& par);
};
class Cancel : public SFBool{
public:
    QuestionMessage3Button& home;

    Cancel(const bool& par, QuestionMessage3Button& home):
        SFBool(par),
        home(home)
    {};
    void operator = (const bool& par);
};

////////////////
    int getType()const{return QUESTION_MESSAGE_3_BUTTON;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFString msg;
    Yes yes;
    No no;
    Cancel cancel;

    QuestionMessage3Button():
        MField4(
            &msg,
            &yes,
            &no,
            &cancel
        ),
        msg(""),
        yes(false, *this),
        no(false, *this),
        cancel(false, *this)

////
    {
    }
    static void (*yesFunc)();
    static void (*noFunc)();
    static void (*cancelFunc)();
    static void setupMessageWindow(const String& msg, void (*i_yesFunc)(), void (*i_noFunc)(), void (*i_cancelFunc)());
};
class RecentFiles : public Node, public MField6{
    public:

class Select : public SFBool{
public:
    RecentFiles& home;

    Select(RecentFiles& home):
        SFBool(false),
        home(home)
    {};
    void operator = (const bool& par);
};

////////////////
    int getType()const{return RECENT_FILES;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    Select sel0;
    Select sel1;
    Select sel2;
    Select sel3;
    Select sel4;
    Select sel5;

    RecentFiles():
        MField6(
            &sel0,
            &sel1,
            &sel2,
            &sel3,
            &sel4,
            &sel5
        ),
        sel0(*this),
        sel1(*this),
        sel2(*this),
        sel3(*this),
        sel4(*this),
        sel5(*this)

////
    {
    }
};

class AnimLever : public Node, public PartAnimClue, public MField2{
public:
    int getType()const{return ANIMLEVER;}
    int getFLen()const{return size();};
    Field* getField(const int i)const{return (*this)[i];}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    SFFloat leverAngle;
	MFFloat leverDistance;

    AnimLever():
        MField2(
			&leverAngle,
			&leverDistance
        ),

        leverAngle(0),
		leverDistance(M_Float(.5, .5)),

		//////

        leverAxis(M_Vec3f(Vec3f(1, 0, 0), Vec3f(0, 1, 0))),
        defaultRotCtrlPnt(M_Vec3f(Vec3f(.5, 0, 0), Vec3f(0, .5, 0))),
        leverIdx(0),
		showLever(true)

    {
        rotCtrlPnt.length = 2;

    }

	bool showLever;
	Vec3f remRotCtrlPnt;
    const M_Vec3f leverAxis;
    const M_Vec3f defaultRotCtrlPnt;
    int leverIdx;

    M_Vec3f rotCtrlPnt;
    Vec3f remTranValue;
    void dragPart(Vec3f pt);
    void dragPartForMorph(Vec3f pt);
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
    AnimPtr anim;
    bool initLeverTug(Art* selPart, const bool& perspective);

    void resetRotationLever();
    void calcNonSelectedRotCtrlPnts();
    void calcRotCtrlPnts();

    void houseKeep(Art* part, Anim* anim);
    void houseKeep();
};

class PartClue : public Node, public PartAnimClue, public MField0{
public:
    int getType()const{return 0;}
    int getFLen()const{return size();};
    Field* getField(const int)const{return 0;}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    PartClue():
        MField0()
        //polySel(false),
        //vertexSel(false)
    {}
    SeamlessPtr smls;
    SeamlessPtr transpSmls;
    Ptr<FuseSurface> fuseSurface;
    //bool polySel;
    //bool vertexSel;
    Vec3f position;
    void dragPart(Vec3f pt);
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};

class PartCageClue : public Node, public PartAnimClue, public MField0{
public:
    int getType()const{return 0;}
    int getFLen()const{return size();};
    Field* getField(const int)const{return 0;}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}

    PartCageClue():
        MField0(),
        mode(0)
    {
    }
    SeamlessPtr smls;
	int mode;
	enum{
		BONE_ROTATION_LEVER = 1,
		BONE_PIVOT_POINT = 2,
		MORPH_ROTATION_LEVER = 3,
		MORPH_PIVOT_POINT = 4,
		MORPH_CP = 5,
	};
    void genCage();
    void dragPart(Vec3f pt);
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};

class CenterJointClue : public Node, public MField0{
public:
    int getType()const{return 0;}
    int getFLen()const{return size();};
    Field* getField(const int)const{return 0;}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    CenterJointClue():
        MField0()
    {}
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};
class TransferOwnershipClue : public Node, public MField0{
public:
    int getType()const{return 0;}
    int getFLen()const{return size();};
    Field* getField(const int)const{return 0;}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    TransferOwnershipClue():
        MField0()
    {}
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};
class TransferBCylClue : public Node, public MField0{
public:
    int getType()const{return 0;}
    int getFLen()const{return size();};
    Field* getField(const int)const{return 0;}
    int getFieldGroupIndex(const int &)const{throw this;}
    void setFieldGroupIndex(const int &, const int &){throw this;}
    M_Int* getIndexPtr(){return nullPtr;}


    TransferBCylClue():
        MField0()
    {}
    void renderNode(Matrix parentMatrix, Vec3f parentColScale);
};

/*
class MatchingNameQuery : public Node, public MField5{
public:
class RenameImport : public SFBool {
public:
    MatchingNameQuery& home;
    RenameImport(const bool& value, MatchingNameQuery& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class RenameExisting : public SFBool {
public:
    MatchingNameQuery& home;
    RenameExisting(const bool& value, MatchingNameQuery& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class ReplaceImport : public SFBool {
public:
    MatchingNameQuery& home;
    ReplaceImport(const bool& value, MatchingNameQuery& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
class ReplaceExisting : public SFBool {
public:
    MatchingNameQuery& home;
    ReplaceExisting(const bool& value, MatchingNameQuery& home):
        SFBool(value),
        home(home)
    {};
    void operator=(const bool& par);
};
    int getType()const{return MATCHINGNAMEQUERY;}
    virtual int getFLen()const{return size();}; //get field length
    Field* getField(const int i)const{return (*this)[i];}
    M_Int* getIndexPtr(){return nullPtr;}

    SFString match;
    RenameImport renameImport;
    RenameExisting	renameExisting;
    ReplaceImport	replaceImport;
    ReplaceExisting	replaceExisting;

    MatchingNameQuery():
        MField5(
            &match,
            &renameImport,
            &renameExisting,
            &replaceImport,
            &replaceExisting
        ),
        match(""),
        renameImport(false, *this),
        renameExisting(false, *this),
        replaceImport(false, *this),
        replaceExisting(false, *this)

//////////////
    {
    }
};
*/
//extern M_Art g_changePart;
//extern M_Node g_changeNode;
//extern ScriptPtr g_glue;
extern float g_version;
extern M_Node g_defLookUp;

extern bool g_outputMode;
extern bool g_writeX3DVRMLClassic;
extern bool g_writeX3DXML;

extern bool g_genHAnimNodes;
extern bool g_genIdxTriSetNodes;
extern bool g_genContactSBVT;
extern bool g_genNormals;
extern bool g_genColorIndex;

extern int g_formatCoords;
extern const int g_formatColor;
extern int g_userProtoBegin;
extern M_Int g_tmpIndex;
extern String g_loadStr;
extern M_<ScriptPtr> g_outputScriptStack;
extern M_Bool g_outputScriptStackExist;


void initLookup();
void findMatchingIs(Field* field, ProtoDeclare *&protoDeclare, int& i);
void findMatchingIs(Field* field, ProtoInterface *&protoInterface, int& i);
int swapJoiningVertices( Art* partA, int triangleIndexA, Art* partB, int triangleIndexB);
void transferOneVertex(Art* source, Art* dest, int vertex);
int getIndexRelativeToPart(Art* part, Art* relitivePart, int index);

#define VERYSMALLDIFF .000001
inline bool notCloseEnough(const bool& a, const bool& b){
    if(a!=b){
        return true;
    }
    return false;
}
inline bool notCloseEnough(const Rotation& a, const Rotation& b){
    if(fabs(a.x-b.x)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.y-b.y)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.z-b.z)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.angle-b.angle)>VERYSMALLDIFF){
        return true;
    }
    return false;
}
inline bool notCloseEnough(const Color3f& a, const Color3f& b){
    if(fabs(a.r-b.r)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.g-b.g)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.b-b.b)>VERYSMALLDIFF){
        return true;
    }
    return false;
}
inline bool notCloseEnough(const Vec3f& a, const Vec3f& b){
    if(fabs(a.x-b.x)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.y-b.y)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.z-b.z)>VERYSMALLDIFF){
        return true;
    }
    return false;
}
inline bool notCloseEnough(const Vec2f& a, const Vec2f& b){
    if(fabs(a.x-b.x)>VERYSMALLDIFF){
        return true;
    }
    if(fabs(a.y-b.y)>VERYSMALLDIFF){
        return true;
    }
    return false;
}
inline bool notCloseEnough(const float& a, const float& b){
    if(fabs(a-b)>VERYSMALLDIFF){
        return true;
    }
    return false;
}
inline bool notCloseEnough(const double& a, const double& b){
    if(fabs(a-b)>VERYSMALLDIFF){
        return true;
    }
    return false;
}
inline bool notCloseEnough(const String& a, const String& b){
    if(a!=b){
        return true;
    }
    return false;
}
template<>
inline FieldType SFBool::getType()const{
    return SFBOOL;
}
//template<>
inline FieldType SFColor::getType()const{
    return SFCOLOR;
}
template<>
inline FieldType SFFloat::getType()const{
    return SFFLOAT;
}
template<>
inline FieldType SFImage::getType()const{
    return SFIMAGE;
}
template<>
inline FieldType SFInt32::getType()const{
    return SFINT32;
}
//template<>
inline FieldType SFNode::getType()const{
    return SFNODE;
}
//template<>
inline FieldType SFRotation::getType()const{
    return SFROTATION;
}
//template<>
inline FieldType SFString::getType()const{
    return SFSTRING;
}
template<>
inline FieldType SFTime::getType()const{
    return SFTIME;
}
//template<>
inline FieldType SFVec2f::getType()const{
    return SFVEC2F;
}
//template<>
inline FieldType SFVec3f::getType()const{
    return SFVEC3F;
}
template<>
inline FieldType MFBool::getType()const{
    return MFBOOL;
}
template<>
inline FieldType MFColor::getType()const{
    return MFCOLOR;
}
template<>
inline FieldType MFFloat::getType()const{
    return MFFLOAT;
}
template<>
inline FieldType MFImage::getType()const{
    return MFIMAGE;
}
template<>
inline FieldType MFInt32::getType()const{
    return MFINT32;
}
template<>
inline FieldType MFNode::getType()const{
    return MFNODE;
}
template<>
inline FieldType MFRotation::getType()const{
    return MFROTATION;
}
template<>
inline FieldType MFString::getType()const{
    return MFSTRING;
}
template<>
inline FieldType MFTime::getType()const{
    return MFDOUBLE;
}
template<>
inline FieldType MFVec2f::getType()const{
    return MFVEC2F;
}
template<>
inline FieldType MFVec3f::getType()const{
    return MFVEC3F;
}


/*
inline void setTransform(const SFVec3f& translation, const SFRotation& rotation, const SFVec3f& scaleFactor){
    setTransform(translation,rotation,scaleFactor);
}
*/
inline	SFNode::operator void* (){
    return p;
}
extern double g_time;
//g_vrmlTime is legacy from freeze time days I think I remember so is now obsolete
#define g_vrmlTime g_time
/*
inline NodePtr::NodePtr( Node* par1){
    par1->incRefCount();
    p=par1;
}
inline NodePtr::NodePtr(const NodePtr& par1){
    p=par1.p;
    if(p!=NULL){
        p->incRefCount();
    }
}
*/
/*
inline NodePtr& NodePtr::operator = ( const int par1){
 if(par1!=NULL)throw this;
    if(pointer!=NULL){
        if(pointer->decRefCount()==0){
            delete pointer;
        }
    }
    pointer=NULL;
    return *this;
}
*/
/*
inline NodePtr& NodePtr::operator = (const NodePtr& par1){
    par1.p->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par1.p;
    return *this;
}
inline NodePtr& NodePtr::operator = ( Node* par1){
    par1->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par1;
    return *this;
}
*/
/*
inline NodePtr& NodePtr::operator = ( SF<NodePtr,	void*>& par){
    Node* parVal=&*(par.refValue());
    parVal->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=parVal;
    return *this;
}
inline NodePtr::~NodePtr(){
    if(p!=NULL&&p->decRefCount()==0){
        delete p;
        p=NULL;
    }
}
*/

inline SFNode::~SFNode(){
    if(p!=0&&p->decRefCount()==0){
        delete p;
    }
}
inline FD<NodePtr,Single>::FD(const NodePtr& par):
    p(par.getRawPtr()),
    is(false)
{
    if(p!=0){
        p->incRefCount();
    }
};
inline Node* SFNode::operator = ( const int par1){
 if(par1 != 0)throw this;
    if(p != 0){
        if(p->decRefCount() == 0){
            delete p;
        }
    }
    p=NULL;
    return p;
}
/*
inline Node* SFNode::operator = (const NodePtr& par1){
    par1.pointer->incRefCount();
    if(pointer!=NULL){
        if(pointer->decRefCount()==0){
            delete pointer;
        }
    }
    pointer=par1.pointer;
    return pointer;
}
*/
inline Node* SFNode::operator = ( Node* par1){
    par1->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=par1;
    return p;
}
inline Node* SFNode::operator = ( SFNode& par){
    Node* parVal=&*(par.refValue());
    parVal->incRefCount();
    if(p!=NULL){
        if(p->decRefCount()==0){
            delete p;
        }
    }
    p=parVal;
    return p;
}


inline Art* S_Triangle::getOwner()const{//get lowest bider
    Art* part=pa;
    if(pb->plidx<part->plidx){
        part=pb;
    }
    if(pc->plidx<part->plidx){
        part=pc;
    }
    return part;
}

template<class T> void FD<T,Single>::connect(Node* node, Node* protoInterface, Field *protoField){
    S_Route( protoInterface, protoInterface->getFieldIndex(protoField), node, node->getFieldIndex(this));
    is = true;
}

inline const Morpher* Art::MFMorpher::operator [](const int& index)const{
    if((unsigned)index>=(unsigned)length){
        throw this;
    }
    return refValue()[index];
}
inline void rotateZ(long double& x, long double& y, long double angle){
    long double newX=x*cosl(-angle+PI*.5)-y*sinl(-angle+PI*.5);
    y=x*sinl(-angle+PI*.5)+y*cosl(-angle+PI*.5);
    x=newX;
}
inline void rotateZ( float& x, float& y, const float& angle){
    long double newX=x*cosl(angle)-y*sinl(angle);
    y=x*sinl(angle)+y*cosl(angle);
    x=newX;
}
inline void rotateZ(Vec2f& vec, float angle){
    rotateZ(vec.x, vec.y, angle);
}
inline S_Route::S_Route( Node* i_toNode, Field* i_toFieldPtr):
    toNode(i_toNode),
    toFieldIdx(i_toNode->getFieldIndex(i_toFieldPtr)),
    toFieldPtr(i_toFieldPtr)
{};
//template <class T>
//void M_<T>::deleteElement(int targetIndex);
int getNonJoiningVertices( Art* partA, int indexA, Art* partB, int indexB, int& notConectedA, int& notConectedB);
Effect* workOutEffectType(const Appearance*& appearance);
void addGeometry(Art* part, const Shape* shape);
void optimize(AGroup* groupingNode);


inline void changeLength(M_Int& mint){
    mint.length=5;
}
template <class T> static void copyElements(int targetFrom, int len, M_<T>& src, M_<T>& dst){
    dst.length = len;
    for(int i = 0; i < len; i++){
        dst[i] = src[targetFrom + i];
    }
}

template <class T> static void copyElements(int srcTargetFrom, int dstTargetFrom, int len, M_<T>& src, M_<T>& dst){ //untested
    for(int i = 0; i < len; i++){
        dst[dstTargetFrom + i] = src[srcTargetFrom + i];
    }
}

template <class T> static void swapElements(int srcTargetFrom, int dstTargetFrom, int len, M_<T>& src, M_<T>& dst){
    for(int i = 0; i < len; i++){
        T swapB = dst[dstTargetFrom + i];
        dst[dstTargetFrom + i] = src[srcTargetFrom + i];
        src[srcTargetFrom + i] = swapB;
    }
}

template <class T> static void insertElements(int targetIndex, int deleteLen, M_<T>& dst, const M_<T>& src){
    int srcLen=src.length;

    int diff=deleteLen-srcLen;
    if(diff<0){ //make gap bigger
        diff=-diff;
        dst.length+=diff;

        int lastMove=targetIndex+deleteLen;
        for(int i=dst.length-diff; --i>=lastMove; ){
            dst[i+diff]=dst[i];
        }
    }
    else if(diff>0){	//make gap smaller
        for(int i=targetIndex+srcLen+diff; i<dst.length-diff; i++){
            dst[i]=dst[diff+i];
        }
        dst.length-=diff;
    }
    for(int i=srcLen; --i>=0; ){
        dst[targetIndex+i]=src[i];
    }
}
inline double& getTime(){
    return g_time;
}
enum TypeColor{
    NODE_COLOR=0x005000,
    GLOBAL_COLOR=0xc00000,
    PROPERTY_COLOR=0x400080,
    TYPE_COLOR=0x0000ff,
    CONSTANT_COLOR=0x808000,
};
Vec2f lineIntersect(Vec2f p1, Vec2f p2, Vec2f p3, Vec2f p4);

float quantize(float v, float res);
void createUniqueName(Node* node);
void makeExistingNameUnique(Node* node);

template <class A, class B>void bubbleSort(M_<A>& a, M_<B>& b){
    bool flag;
    do{
        flag=false;
        for(int i=0;i<a.length-1;i++){

            if (a[i] > a[i+1]){

                A swapA = a[i+1];
                a[i+1]=a[i];
                a[i]=swapA;

                B swapB = b[i+1];
                b[i+1]=b[i];
                b[i]=swapB;

                flag=true;
            }
        }
    }while(flag);
}

template <class A>void bubbleSort(M_<A>& a){
    bool flag;
    do{
        flag=false;
        for(int i=0;i<a.length-1;i++){

            if (a[i] > a[i+1]){

                A swap = a[i+1];
                a[i+1]=a[i];
                a[i]=swap;

                flag=true;
            }
        }
    }while(flag);
}

class MidiNoteOnItem{
public:
    unsigned char noteNumber;
    unsigned char velocity;
    unsigned delay;
    MidiNoteOnItem():
        delay(0),
        noteNumber(0),
        velocity(0)
    {}
    MidiNoteOnItem(unsigned delay, unsigned char noteNumber, unsigned char velocity):
        delay(delay),
        noteNumber(noteNumber),
        velocity(velocity)
    {}
};
inline float degrees(float radians){
    return radians/PI*180;
}
inline double degrees(double radians){
    return radians/PI*180;
}
inline float radians(float degrees){
    return degrees/180*PI;
}
inline double radians(double degrees){
    return degrees/180*PI;
}
class SceneTreeListItem{
public:
    bool ref;
    NodePtr node;
    int parent;
    int indent;
    Color3f marker;
    SceneTreeListItem():
        ref(0),
        node(0),
        parent(0),
        indent(0)
    {}
    SceneTreeListItem(bool i_ref, Node* i_node, int i_parent, int i_indent, const Color3f& i_marker):
        ref(i_ref),
        node(i_node),
        parent(i_parent),
        indent(i_indent),
        marker(i_marker)
    {}
    ~SceneTreeListItem()
    {};

};

class CornerEdge{
public:
    NurbsPatch* patch;
    int edgeIdx;
    int oppositeEdgeIdx;
    int innerPointIdx;
    int cornerIdx;
    CornerEdge():
        patch(0),
        edgeIdx(0),
        oppositeEdgeIdx(0),
        innerPointIdx(0),
        cornerIdx(0)
    {}
        //operator =
};

inline bool roughEQ(double a, double b){
    return abs(a - b) < .000001;
}

inline bool roughNE(double a, double b){
    return !(abs(a - b) < .000001);
}

inline bool roughGT(double a, double b){
    return (a - b) > .000001;
}

inline bool roughLT(double a, double b){
    return (b - a) > .000001;
}

inline bool roughGE(double a, double b){
    return (a - b) > -.000001;
}

inline bool roughLE(double a, double b){
    return (b - a) > -.000001;
}

extern bool g_joinAndTransferActive;

class MergePatches{
public:
    enum {
        NEUTRAL = 0,
        STANDBY = 1,
    };
    int mousePhase;
	MergePatches():
		mousePhase(NEUTRAL),
		smls(0),
		patchA(0),
		edgeA(0),
		patchB(0),
		edgeB(0)
	{}
	void addVisualClue(const Color3f edgeColor);
	void doLastPhase();

	NurbsPatch* patchA;
	int edgeA;
	NurbsPatch* patchB;
	int edgeB;
	Seamless* smls;
};

class InsertPatch{
public:
    enum {
        NEUTRAL = 0,
        STANDBY = 1,
        LINE_MODE = 2,
        DIVERT_STANDBY = 3,
        MONITOR_DIRECTION = 4,
    };
    int imousePhase;
    M_Insert m_insert;
    M_Insert m_insertOD; //Opposite Direction

    M_Insert m_insert_tempDivertBackup;
    M_Insert m_insertOD_tempDivertBackup; //Opposite Direction

    M_Insert m_insert_loopBackup;
    M_Insert m_insertOD_loopBackup;

    int beginCoordVisualClue;
    int beginCoordIndexVisualClue;
    NurbsPatch* mouseOverPatch;
    bool centerMirrorLine;
    bool remLeftSide;
    bool remLeftSide_tempDivertBackup;
    float remRatio;
    bool remInvertRatio;
    int beginInsertIdx;
    int beginInsertIdxOD;

    int beginInsertIdx_tempDivertBackup;
    int beginInsertIdxOD_tempDivertBackup;
    bool diversionHover;


	int init_element;
    NurbsPatch* init_patch;
    bool init_doU;
    float init_ratio;
    int init_idx;
    bool init_leaveNorthOrEast;
    //NurbsPatch* remPatch;
    //int remRowCol;
    //bool remDo_uInsert;
    M_Vec2i clickedPolyEdge; //x: poly y: edge
    bool loop;
	bool foundFirstStitchPoly;
	bool cutMode;

    InsertPatch():
        imousePhase(NEUTRAL),
        beginCoordVisualClue(0),
        beginCoordIndexVisualClue(0),
        mouseOverPatch(0),
        centerMirrorLine(false),
        remLeftSide(false),
        remLeftSide_tempDivertBackup(false),
        remRatio(0),
        remInvertRatio(false),
        beginInsertIdx(0),
        beginInsertIdxOD(0),
        beginInsertIdx_tempDivertBackup(0),
        beginInsertIdxOD_tempDivertBackup(0),
        //remPatch(0),
        //remRowCol(0),
        //remDo_uInsert(true)
		init_element(0),
        init_patch(0),
        init_doU(false),
        init_ratio(0),
        init_idx(0),
        init_leaveNorthOrEast(false),
        loop(false),
		foundFirstStitchPoly(false),
        diversionHover(false),
		cutMode(false)
    {}
    bool monitorDirection();
    void addInsertVisualClue();
    void doLastPhase();
    void showDirection(M_Int& iFScoordIndex, M_Vec3f& iFSpoint, M_Color3f& iFScolor)const;
    void sortDirectionsOut(int leaveNorthOrEast, int uOrV, int uOrVDim);
    void reverseDirection(M_Insert& m_insert);
    int numberOfPointsInLine(const M_Insert& m_insert)const;
	void changeInitElement();
	void verifyFirst();
};

class DeleteControlLine{
public:
    enum {
        NEUTRAL = 0,
        STANDBY = 1,
    };
    bool selectRing;
    int mousePhase;
    M_DelCtrlLn m_delCtrlLn;
    M_DelCtrlLn mirror_delCtrlLn;
    int beginCoordVisualClue;
    int beginCoordIndexVisualClue;
    NurbsPatch* mouseOverPatch;
    Seamless* smls;
    bool centerMirrorLine;

    DeleteControlLine():
        mousePhase(0),
        beginCoordVisualClue(0),
        beginCoordIndexVisualClue(0),
        mouseOverPatch(0),
        smls(0),
        centerMirrorLine(false),
        selectRing(false)
    {}
	void cleanUp();
	void addMirrorEntries();
	bool crossesCenter() const;
    void addVisualClue();
    void doLastPhase();
};

//void connect(Node* node, Node* protoInterface, Field *protoField){
//    S_Route( protoInterface, protoInterface->getFieldIndex(protoField), node, node->getFieldIndex(this));
//    length.is=true;
//}

class SelNode {
public:
    Node* operator ->()const;
    Node* getRaw()const;
};
class CopyNode {
public:
    Node* operator ->()const;
    Node* getRaw()const;
};
extern SelNode g_selNode;
extern CopyNode g_copyNode;

class BIProto{//built in proto
public:
    const char* ntype;
    const char* containerField;
    const Vec2i iconSize;
    const Vec2i iconSrcPos;
    const BIField* bifieldRef;
    const int numOf;
    BIProto(const char* ntype, const char* containerField, const Vec2i& iconSize, const Vec2i& iconSrcPos, const int numOf, const BIField* bifieldRef):
        ntype(ntype),
        containerField(containerField),
        iconSize(iconSize),
        iconSrcPos(iconSrcPos),
        numOf(numOf),
        bifieldRef(bifieldRef)
    {}
};


struct Rect {
    int    left;
    int    top;
    int    right;
    int    bottom;
};

struct Point
{
    int  x;
    int  y;
};

inline unsigned int Rgb(unsigned char r, unsigned char g, unsigned char b){
    unsigned int rgb = r;
    rgb <<= 8;
    rgb |= g;
    rgb <<= 8;
    rgb |= b;
    return rgb;
}

enum {
	_MORPHER_,
	_POS_COMPENSATOR_,
	_NIK_INTERPOLATOR_,
	_ORIENTATION_,
	_ROTATION_,
	_POSITION_,
	_TRANSLATION_,
	_LOOK_AT_,
	_UP_,
	_FOV_,
	_LIVE_,
	_ZOOM_,
	_FA_,
	_FB_,
};

void addCommonName(String& str, const int commonNameCode);

inline Vec2f toVec2f(const Vec3f& par){
	return Vec2f(par.x, par.y);
}
