/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#define _WIN32_WINNT 0x0400
#define WINVER 0x0500

#ifdef WIN32
#include <direct.h>
#else
#include <unistd.h>
#endif

#include "seamless3d.h"
#include "script.h"
#include "techuelife_triangle_editor.h"
#include "gui.h"
#include "scene_tree.h"
#include "input.h"

extern int g_sampleRate;

extern Ptr<VideoControlPanel> g_vcp;
extern int g_fps;
extern const BIProto* builtInProtoList[];
extern bool g_editVideoFrames;

extern const char* msg_unable_to_decode_script;
extern const char* msg_wont_use;
extern const char* msg_because_of_node_type_for_field;
extern const char* msg_Wont_DEFine_node_as;
extern const char* msg_because_node_name_exists;
extern const char* msg_Wont_set;
extern const char* msg_Wont_set_Node;
extern const char* msg_blank_name_not_allowed;
extern const char* msg_blank_name_not_allowed_because_of_ROUTE;
extern const char* msg_blank_name_not_allowed_because_of_ROUTE2;
extern const char* msg_Can_not_create_output_file_name;
extern const char* msg_cpp_file_already_exists;
extern const char* msg_Identical_node_name_encountered;

extern M_<Ptr<Function> > g_functionList;
extern Ptr<ExportMovie> g_exportMovie;
extern Ptr<ExportVrmlX3d> g_exportVrmlX3d;


void closeMessageWindow();
extern Ptr<DeleteOptions> g_deleteOptions;

void changeFloatFormat(int numOfDigits);
void listRoutes(String&);
void openWithPovRay(String url);
void renderScene();
bool nodeExist(Node* node);
void deleteNode(int nodeLine, int selCount = 1);
extern Ptr<AnimControlPanel> g_acp;
void createFunctionListFromSceneTreeList();
void makeExistingNameUniqueUsingDefList(Node* node);
void makeExistingSmlsNamesUniqueUsingDefList(Seamless* smls);
void exportObj();
void exportSculpties(M_Node&);
void deleteAllRoutesToNonExistant();
void compileScaleRotation(Seamless* smls, M_<SceneTreeListItem>& sceneTreeList);
void openWithHtmlBrowser(String url, const String htmlBrowser);
bool getMorphFunction(M_Word& m_word, W_Node*& w_morpher, W_Node*& w_part, M_Node& m_codeLine, int i);
bool getCalcFunction(M_Word& m_word, W_Node*& w_morpher, W_Node*& w_part, M_Node& m_codeLine, int i);
char checkBvhFileExtension(const char* s);
bool importBVHGesture(const String& filename, ScenePtr& importScene, M_<SceneTreeListItem>& importSceneTreeList, M_Node& importDefList);
void setWorkAnimCPsInScene();
extern bool g_browseMode;

void importControlPointOwnership(Seamless* srcSmls, Seamless* dstSmls);

extern AnimCurve g_animCurve;
extern AnimCopy g_animCopy;
extern Ptr<TimelineFilter> g_timelineFilter;
extern Ptr<PasteOptions> g_pasteOptions;
extern Ptr<DeleteOptions> g_deleteOptions;

extern int g_animBarListIndex;
extern int g_viewIndex;
extern M_<Ptr<Viewpoint> > g_viewpoints;


bool getMorphFunction(M_Word& m_word, W_Node*& w_morpher, W_Node*& w_part, M_Node& m_codeLine, int i);
bool getCompFunction(M_Word& m_word, W_Node*& w_compensator, W_Node*& w_part, M_Node& m_codeLine, int i);


Function* CodeLine::getParentFunction(){
	int selLine=getNodeLine(this);
	int k=selLine;

	if(k==0){
		return 0;
	}

	while(dynamic_cast<CodeLine*>(g_sceneTreeList[--k].node.p));

	if(g_sceneTreeList[selLine].indent<=g_sceneTreeList[k].indent){ //check we are not a global line under a function
		return 0;
	}

	return dynamic_cast<Function*>(g_sceneTreeList[k].node.p);
}
void resetDefList(){
	g_defLookUp.length=0;
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(g_sceneTreeList[k].ref)continue;
		Node* node=g_sceneTreeList[k].node.p;
		if(node->name!=""){
			g_defLookUp+=node;
		}
	}
}
void createDefList(M_Node& defLookUp){
	defLookUp.length=0;
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(g_sceneTreeList[k].ref)continue;
		Node* node=g_sceneTreeList[k].node.p;
		if(node->name!=""){
			defLookUp+=node;
		}
	}
}
bool checkOkToUSE(Field* field, const Properties &sprop, int copyNodeType){
	if(
		sprop.blockUse()||
		field->getType()!=SFNODE&&
		field->getType()!=MFNODE
		)
		{
		return false;
	}

	for(int j=0; j<sprop.promptNode.length; j++){
		if(sprop.promptNode[j]==copyNodeType){
			return true;
		}
	}
	return false;
}
void Function::headerToString(String& str){
	if(g_outputMode){
		str+="function ";
	}
	else{
//		if(!dynamic_cast<MemberFunction*>(this)){
		str+=typeNameLookUp[returnType];
		str+=' ';
//		}
	}
	str+=functionName;

	str+='(';

	if(paramLen)for(int k=0;;){
		if(g_outputMode==false){
			str+=String(typeNameLookUp[m_local[k]->getType()]);
			switch(m_local[k]->getType()){
				case M_BOOL:
				case M_COLOR:
				case M_FLOAT:
				case M_IMAGE:
				case M_INT32:
				case M_NODE:
				case M_ROTATION:
				case M_STRING:
				case M_TIME:
				case M_VEC2F:
				case M_VEC3F:{
					str+='&';
				}
			}
			str+=' ';
		}

		str+=m_local[k]->name;

		if(++k>=paramLen)break;
		str+=", ";
	}
	str+=')';
}
void Function::headerToColorFormat(String& str, M_ColorFormat& format, int _x){

	str=typeNameLookUp[returnType];
	str+=' ';
	format+=ColorFormat(TYPE_COLOR, str.length, _x);
	int x=_x+str.length*charWidth;

	str+=functionName;
	format+=ColorFormat(FUNCTION_COLOR, str.length, x);
	x=_x+str.length*charWidth;

	str+='(';
	format+=ColorFormat(OPERATOR_COLOR, str.length, x);
	x+=charWidth;

	if(paramLen)for(int k=0;;){
		str+=String(typeNameLookUp[m_local[k]->getType()]);
		format+=ColorFormat(TYPE_COLOR, str.length, x);
		x=_x+str.length*charWidth;
		switch(m_local[k]->getType()){
			case M_BOOL:
			case M_COLOR:
			case M_FLOAT:
			case M_IMAGE:
			case M_INT32:
			case M_NODE:
			case M_ROTATION:
			case M_STRING:
			case M_TIME:
			case M_VEC2F:
			case M_VEC3F:{
				str+='&';
			}
		}

		str+=' ';
		format+=ColorFormat(OPERATOR_COLOR, str.length, x);
		x=_x+str.length*charWidth;

		str+=m_local[k]->name;
		format+=ColorFormat(LOCAL_COLOR, str.length, x);
		x=_x+str.length*charWidth;

		if(++k>=paramLen)break;
		str+=", ";
		format+=ColorFormat(OPERATOR_COLOR, str.length, x);
		x=_x+str.length*charWidth;
	}
	str+=')';
	format+=ColorFormat(OPERATOR_COLOR, str.length, x);
}
/*
void SceneTree::closeNodeNameEditBox(){
	int i,k;
	bool setNameFlag=true;
	String& str=textCell.cellStr;
//		getDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, str);
	//dLog("testA "+str);
	while(str.length&&str(0)==' '){
		str=str.substr(1);
	}
	
	if(CodeLine* codeLine=dynamic_cast<CodeLine*>(g_selNode.p)){
		Function& f=*getFunctionNode();
		String& str=g_loadStr="";
		Function::selectedLine=codeLine;
		onlyFunctionsToString(str);

		initCompileForBody();

		try{
			if(str==""){
			}
			else{
				int i=0;

				resetDefList();
				decodeFunctions(str);
				Function::selectedLine=0;
				genNodeList();
				textCell.cellStr="";
				for(int w=codeLine->wordFrom; w<codeLine->wordTo; w++){
					Word* wrd=f.m_word[w];
					wrd->toString(textCell.cellStr);
				}

				//function->decodeCppLine(codeLine->codeLine, *codeLine, str, i);
			}
		}
		catch(...){
			Function::selectedLine=0;
			str=g_selNode->name;
			msG(String("unable to decode script"), OK_BUTTON | ICONEXCLAMATION);
			setNameFlag=false;
		}
	}
	else{
		if(textCell.cellStr==selNode->name){
			textCell.cellActive=false;
			return;
		}
		if(str.length>0&&( //fix first char if it begins with a number
			str(0)>='0'&&str(0)<='9'||
			str(0)=='-'||
			str(0)=='+')){
			str=str.replace(0,0,"_");
		}
		for(i=0; i<str.length; i++){
			if(	isLegalNameChar(str(i))==false){
				str[i]='_';
			}
		}
		if(str!=""){
			for(k=0; k<g_sceneTreeList.length; k++){
				Node* node=g_sceneTreeList[k].node;
				if(node->name==str && node!=g_selNode){
					if(g_sceneTreeList[selLine].ref==true ){

						str=g_selNode->name;
						setNameFlag=false;
						

						if(g_selNode->getType()==SCENE)break;
						int selFieldIdx=0;
						int selItemIdx=0;
						int copyFieldIdx=0;
						int copyItemIdx=0;
						
						int parentSelLine=getParent(selLine);
						int parentCopyLine=getParent(k);
						
						NodePtr parentSelNode=g_sceneTreeList[parentSelLine].node;
						NodePtr parentCopyNode=g_sceneTreeList[parentCopyLine].node;
						
						getFNodeElementForParent(parentSelLine, selLine, g_sceneTreeList, selFieldIdx, selItemIdx );
						getFNodeElementForParent(parentCopyLine, k, g_sceneTreeList, copyFieldIdx, copyItemIdx );

						if(checkOkToUSE(parentSelNode->getField(selFieldIdx), parentSelNode->getProperties()[selFieldIdx], node->getType())){
							NodePtr srcPtr;
							if(parentCopyNode->getField(copyFieldIdx)->getType()==MFNODE){
								srcPtr=dynamic_cast<MFNode*>(parentCopyNode->getField(copyFieldIdx))->refValue()[copyItemIdx];
							}
							else {
								srcPtr=dynamic_cast<SFNode*>(parentCopyNode->getField(copyFieldIdx))->refValue();
							}
							if(parentSelNode->getField(selFieldIdx)->getType()==MFNODE){
								MFNode& mfnode=*dynamic_cast<MFNode*>(parentSelNode->getField(selFieldIdx));
								M_Node& mnode=mfnode.refValue();
								mnode[selItemIdx]=srcPtr;
								mfnode=mnode;
							}
							else { //dest SFNode type
								SFNode& fdst=*dynamic_cast<SFNode*>(parentSelNode->getField(selFieldIdx));
								fdst=srcPtr;
							}
							textCell.cellStr=node->name;
							commonPostInsertNodeStuff();
						}
						else{
							msG(String("Wont USE "), str, String( " because this type of node can not be used for this field!"),OK_BUTTON|ICONEXCLAMATION);
						}
						textCell.cellActive=false;
						return;
					}
					else{
	//						setDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, g_activeCellNode->name);
						str=g_selNode->name;
						msG(String("Wont DEFine node to "), str, String( " because a node DEFined with this name already exists!\nAll DEFined names must be different"),OK_BUTTON|ICONEXCLAMATION);
						setNameFlag=false;
					}
					break;
				}
			}
		}
		else {
			for(k=0; k<g_sceneTreeList.length; k++){
				Node* node=g_sceneTreeList[k].node;
				if(node->getType()==PROTOINTERFACE)continue;
				if(g_sceneTreeList[k].node==g_selNode && k!=selLine){
					//setDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, g_activeCellNode->name);
					str=g_selNode->name;
					msG(String("Wont set "), g_selNode->name, String( " to blank name (unDEFine) because this node is USEd! (referenced by some other node)"),OK_BUTTON|ICONEXCLAMATION);
					setNameFlag=false;
					break;
				}
				if(nodeHasRouteTo(node,g_selNode)){
					//setDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, g_activeCellNode->name);
					str=g_selNode->name;
					msG(String("Wont set node "), g_selNode->name, String( " to blank name (unDEFine) because this node has a field ROUTEd to it by " + node->name),OK_BUTTON|ICONEXCLAMATION);
					setNameFlag=false;
					break;
				}
				if(nodeHasRouteTo(g_selNode,node)){
					str=g_selNode->name;
					//setDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, g_activeCellNode->name);
					msG(String("Wont set "), g_selNode->name, String( " to blank name (unDEFine) because this node has a field ROUTEd to " + node->name),OK_BUTTON|ICONEXCLAMATION);
					setNameFlag=false;
					break;
				}
			}
		}
		if(setNameFlag){
			BackUp::monostable.beginCycle();
			g_selNode->name=str;
			ProtoDeclare* protoDeclare=dynamic_cast<ProtoDeclare*>(g_selNode.p);
			if(protoDeclare){
				nodeLookUpStr[protoDeclare->lookupDeclarationsIndex]=str;
			}
		}
	}
	//DestroyWindow( hWndActiveCell);
	//hWndActiveCell=NULL;
//		ShowWindow( hWndActiveCell, SW_HIDE );
//		UpdateWindow( hWndActiveCell );
//	g_activeCellNode=0;
	textCell.cellActive=false;
	paint();
	setTitleName();
}
*/
void G_Variable::toTextCell(String& str){
	str="";
	for(int w=0; w<m_word.length; w++){
		m_word[w]->toString(str);
	}
}
void SceneTree::STTextCell::replaceVariableType(G_Variable* gv, int i){
	int parentLine=getParent(win.selLine);
	Node* parent=g_sceneTreeList[parentLine].node.p;

	int fieldIdx=0;
	int element=0;

	getFNodeElementForParent(parentLine, win.selLine, g_sceneTreeList, fieldIdx, element);

	const Properties &sprop=parent->getProp(fieldIdx);

	Field* field=parent->getField(fieldIdx);

	MFNode *mfnode=dynamic_cast<MFNode*>(field);

	mfnode->refValue()[element]=gv;

	int bracketCntr=0;

	gv->decodeTextCell(cellStr, i, bracketCntr);
	gv->toTextCell(cellStr);
	win.genNodeList();
	win.cp->setUpControlPanel(gv, gv->getProperties());
}
void SceneTree::STTextCell::decodeGlobal(Node* node){
	int i=0;
	int bracketCntr=0;
	if(cellStr.cmpCppWord("bool",i)){
		if(G_S_Bool* gv=dynamic_cast<G_S_Bool*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Bool, i);
		}
	}
	else if(cellStr.cmpCppWord("Color3f",i)){
		if(G_S_Color* gv=dynamic_cast<G_S_Color*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Color, i);
		}
	}
	else if(cellStr.cmpCppWord("float",i)){
		if(G_S_Float* gv=dynamic_cast<G_S_Float*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Float, i);
		}
	}
	else if(cellStr.cmpCppWord("Image",i)){
		dynamic_cast<V_<Image, Single>*>(node)->decodeTextCell(cellStr, i, bracketCntr);
	}
	else if(cellStr.cmpCppWord("int",i)){
		if(G_S_Int32* gv=dynamic_cast<G_S_Int32*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Int32, i);
		}
	}
	else if(cellStr.cmpCppWord("Rotation",i)){
		if(G_S_Rotation* gv=dynamic_cast<G_S_Rotation*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Rotation, i);
		}
	}
	else if(cellStr.cmpCppWord("String",i)){
		if(G_S_String* gv=dynamic_cast<G_S_String*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_String, i);
		}
	}
	else if(cellStr.cmpCppWord("double",i)){
		if(G_S_Time* gv=dynamic_cast<G_S_Time*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Time, i);
		}
	}
	else if(cellStr.cmpCppWord("Vec2f",i)){
		if(G_S_Vec2f* gv=dynamic_cast<G_S_Vec2f*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Vec2f, i);
		}
	}
	else if(cellStr.cmpCppWord("Vec3f",i)){
		if(G_S_Vec3f* gv=dynamic_cast<G_S_Vec3f*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_S_Vec3f, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Bool",i)){
		if(G_M_Bool* gv=dynamic_cast<G_M_Bool*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Bool, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Color3f",i)){
		if(G_M_Color* gv=dynamic_cast<G_M_Color*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Color, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Float",i)){
		if(G_M_Float* gv=dynamic_cast<G_M_Float*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Float, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Image",i)){
		dynamic_cast<V_<M_Image, Single>*>(node)->decodeTextCell(cellStr, i, bracketCntr);
	}
	else if(cellStr.cmpCppWord("M_Int",i)){
		if(G_M_Int* gv=dynamic_cast<G_M_Int*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Int, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Rotation",i)){
		if(G_M_Rotation* gv=dynamic_cast<G_M_Rotation*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Rotation, i);
		}
	}
	else if(cellStr.cmpCppWord("M_String",i)){
		if(G_M_String* gv=dynamic_cast<G_M_String*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_String, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Double",i)){
		if(G_M_Time* gv=dynamic_cast<G_M_Time*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Time, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Vec2f",i)){
		if(G_M_Vec2f* gv=dynamic_cast<G_M_Vec2f*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Vec2f, i);
		}
	}
	else if(cellStr.cmpCppWord("M_Vec3f",i)){
		if(G_M_Vec3f* gv=dynamic_cast<G_M_Vec3f*>(node)){
			gv->decodeTextCell(cellStr, i, bracketCntr);
		}
		else{
			replaceVariableType(new G_M_Vec3f, i);
		}
	}
}

void updateBuildAllSmlsNodes(){
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Seamless* smls=dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->doBuild();
		}
	}
}

Art* getMirrorPartNoName(Art* part);
PaletteCP* paletteCPAllreadyOpen();


void setCellActiveFalseEnd(){
	createFunctionListFromSceneTreeList();
	Function::selectedLine=g_scene;
	String& str=g_loadStr="";
	onlyFunctionsToString(str);

	initCompileForBody();
	try{
		if(str==""){
		}
		else{
			resetDefList();
			decodeFunctions(str);
			Function::selectedLine=0;
			g_scene->genNodeList2();

	//						f->getSceneDisplayString(cellStr="");
		}
	}
	catch(...){
		Function::selectedLine=0;
		str=g_selNode->name;
		msg(String("unable to decode script"), OK_BUTTON | ICONEXCLAMATION);
	}
}

void SceneTree::STTextCell::setCellActiveFalse(){

	removeFromList<CallBack>(g_callBackList, this);
//	if(cellActive == false){
//		return;
//	}
	firstVisChar = 0;
	caretPos = 0;
	highlightPos = 0;


	bool setNameFlag = true;
	while(cellStr.length&&cellStr(0)==' '){
		cellStr=cellStr.substr(1);
	}
	
	if(CodeLine* codeLine=dynamic_cast<CodeLine*>(g_selNode.getRaw())){
		if(codeLine->getParentFunction()){

			String& str=g_loadStr="";
			Function::selectedLine=codeLine;
			onlyFunctionsToString(str);

			initCompileForBody();

			try{
				if(str==""){
				}
				else{
					int i=0;

					resetDefList();
					decodeFunctions(str);
					Function::selectedLine=0;
					g_scene->genNodeList2();

					codeLine->getSceneDisplayString(cellStr="");
					updateBuildAllSmlsNodes();

					BackUp::monostable.beginCycle();

					//function->decodeCppLine(codeLine->codeLine, *codeLine, str, i);
				}
			}
			catch(...){
				Function::selectedLine=0;
				str=g_selNode->name;
				msg(String("unable to decode script"), OK_BUTTON | ICONEXCLAMATION);
				setNameFlag = false;
			}
		}
		else{
			decodeGlobal(codeLine);
			setCellActiveFalseEnd();
		}
	}
	else if(Function* f=dynamic_cast<Function*>(g_selNode.getRaw())){
		String& str=g_loadStr="";
		Function::selectedLine=f;
		onlyFunctionsToString(str);

		initCompileForBody();

			initCompileForBody();
		try{
			if(str==""){
			}
			else{
				int i=0;

				resetDefList();
				decodeFunctions(str);
				Function::selectedLine=0;
				g_scene->genNodeList2();

				f->getSceneDisplayString(cellStr="");
				updateBuildAllSmlsNodes();

				BackUp::monostable.beginCycle();

				//function->decodeCppLine(codeLine->codeLine, *codeLine, str, i);
			}
		}
		catch(...){
			Function::selectedLine=0;
			str=g_selNode->name;
			msg(msg_unable_to_decode_script, OK_BUTTON | ICONEXCLAMATION);
			setNameFlag = false;
		}
	}
	else if(G_Variable* gv=dynamic_cast<G_Variable*>(g_selNode.getRaw())){
		static M_Word storeMWord;
		storeMWord=gv->m_word;
		try{
			if(cellStr==""){
			}
			else{
				decodeGlobal(gv);
				{
					G_Variable* gv=dynamic_cast<G_Variable*>(g_selNode.getRaw());
					gv->getSceneDisplayString(cellStr="");
					setCellActiveFalseEnd();
				}
			}
		}
		catch(...){
			Function::selectedLine=0;
			gv->m_word=storeMWord;
			msg(String("unable to decode initialization of variable"), OK_BUTTON | ICONEXCLAMATION);
			setNameFlag = false;
		}
	}
	else{
		if(cellStr==g_selNode->name){
			win.paint();
			return;
		}
		if(cellStr.length>0&&( //fix first char if it begins with a number
			cellStr(0)>='0'&&cellStr(0)<='9'||
			cellStr(0)=='-'||
			cellStr(0)=='+')){
			cellStr=cellStr.replace(0,0,"_");
		}
		for(int i=0; i<cellStr.length; i++){
			if(	isLegalNameChar(cellStr(i))==false){
				cellStr[i]='_';
			}
		}
		if(cellStr!=""){
			for(int k=0; k<g_sceneTreeList.length; k++){
				Node* node=g_sceneTreeList[k].node;
				if(node->name==cellStr && node!=g_selNode.getRaw()){
					if(g_sceneTreeList[win.selLine].ref==true ){

						setNameFlag = false;
						

						if(g_selNode->getType()==SCENE)break;
						int selFieldIdx=0;
						int selItemIdx=0;
						int copyFieldIdx=0;
						int copyItemIdx=0;
						
						int parentSelLine=getParent(win.selLine);
						int parentCopyLine=getParent(k);
						
						NodePtr parentSelNode=g_sceneTreeList[parentSelLine].node;
						NodePtr parentCopyNode=g_sceneTreeList[parentCopyLine].node;
						
						getFNodeElementForParent(parentSelLine, win.selLine, g_sceneTreeList, selFieldIdx, selItemIdx );
						getFNodeElementForParent(parentCopyLine, k, g_sceneTreeList, copyFieldIdx, copyItemIdx );

						if(checkOkToUSE(parentSelNode->getField(selFieldIdx), parentSelNode->getProperties()[selFieldIdx], node->getType())){
							NodePtr srcPtr;
							if(parentCopyNode->getField(copyFieldIdx)->getType()==MFNODE){
								srcPtr=dynamic_cast<MFNode*>(parentCopyNode->getField(copyFieldIdx))->refValue()[copyItemIdx];
							}
							else {
								srcPtr=dynamic_cast<SFNode*>(parentCopyNode->getField(copyFieldIdx))->refValue();
							}
							if(parentSelNode->getField(selFieldIdx)->getType()==MFNODE){
								MFNode& mfnode=*dynamic_cast<MFNode*>(parentSelNode->getField(selFieldIdx));
								M_Node& mnode=mfnode.refValue();
								mnode[selItemIdx]=srcPtr;
								mfnode=mnode;
							}
							else { //dest SFNode type
								SFNode& fdst=*dynamic_cast<SFNode*>(parentSelNode->getField(selFieldIdx));
								fdst=srcPtr;
							}
							cellStr=node->name;
							g_stw->commonPostInsertNodeStuff();
						}
						else{
							msg(String(msg_wont_use), cellStr, String( msg_because_of_node_type_for_field),OK_BUTTON|ICONEXCLAMATION);
							cellStr=g_selNode->name;
						}
						win.paint();
						return;
					}
					else{
						msg(msg_Wont_DEFine_node_as, cellStr, msg_because_node_name_exists,OK_BUTTON|ICONEXCLAMATION);
						cellStr=g_selNode->name;
						setNameFlag = false;
					}
					break;
				}
			}
		}
		else {
			for(int k=0; k<g_sceneTreeList.length; k++){
				Node* node=g_sceneTreeList[k].node;
				if(node->getType()==PROTOINTERFACE)continue;
				if(g_sceneTreeList[k].node==g_selNode.getRaw() && k!=win.selLine){
					//setDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, g_activeCellNode->name);
					cellStr=g_selNode->name;
					msg(String(msg_Wont_set), g_selNode->name, String( msg_blank_name_not_allowed),OK_BUTTON|ICONEXCLAMATION);
					setNameFlag = false;
					break;
				}
				if(nodeHasRouteTo(node,g_selNode.getRaw())){
					//setDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, g_activeCellNode->name);
					cellStr=g_selNode->name;
					msg(String(msg_Wont_set_Node), g_selNode->name, msg_blank_name_not_allowed_because_of_ROUTE2 + node->name,OK_BUTTON|ICONEXCLAMATION);
					setNameFlag = false;
					break;
				}
				if(nodeHasRouteTo(g_selNode.getRaw(),node)){
					cellStr=g_selNode->name;
					//setDlgItem(hSceneTreeWnd, IDC_SCENE_TREE_EDIT, g_activeCellNode->name);
					msg(msg_Wont_set, g_selNode->name, msg_blank_name_not_allowed_because_of_ROUTE + node->name,OK_BUTTON|ICONEXCLAMATION);
					setNameFlag = false;
					break;
				}
			}
		}
		if(setNameFlag){
			BackUp::monostable.beginCycle();
			if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
				String mirrorName;
				if(cellStr.length > 1){
					if(cellStr[1] < 'a' ){ //if uppercase or underbar
						if(Art* mirrorPart = part->mirrorPart){
							if(cellStr[0] == 'l'){
								mirrorName = cellStr;
								mirrorName[0] = 'r';
								mirrorPart->name = mirrorName;
							}
							else if(cellStr[0] == 'r'){
								mirrorName = cellStr;
								mirrorName[0] = 'l';
								mirrorPart->name = mirrorName;
							}
						}
					}
				}
				g_selNode->name = cellStr;
				part->smls->populateMirrorNodes();
			}
			else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
				String mirrorName;
				if(cellStr.length > 1){
					if(cellStr[1] < 'a' ){ //if uppercase or underbar
						if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
							if(cellStr[0] == 'l'){
								mirrorName = cellStr;
								mirrorName[0] = 'r';
								mirrorPatch->name = mirrorName;
							}
							else if(cellStr[0] == 'r'){
								mirrorName = cellStr;
								mirrorName[0] = 'l';
								mirrorPatch->name = mirrorName;
							}
						}
					}
				}
				g_selNode->name = cellStr;
				patch->smls->populateMirrorNodes();
			}
			else{
				g_selNode->name = cellStr;
			}
#ifdef PROG_ROB
			dLog("new TypeString(\"" + cellStr + "\"),");
#endif
			ProtoDeclare* protoDeclare=dynamic_cast<ProtoDeclare*>(g_selNode.getRaw());
			if(protoDeclare){
				nodeLookUpStr[protoDeclare->lookupDeclarationsIndex]=cellStr;
			}
			setCellActiveFalseEnd();
		}
	}
	if(g_colorOwnership){
		paletteCPAllreadyOpen()->paint();
	}
	win.cp->paint();
	win.paint();
	setTitleName();
}
bool writeISScriptField(String& str, Field* field){
	if(field->isRef()){
		str+=" IS ";
		ProtoDeclare* protoDeclare=0;
		int j=0;
		findMatchingIs(field, protoDeclare, j);
		str+=protoDeclare->protoInterface->properties[j].name;
		return true;
	}
	return false;
}
void Script::toString(String& str, bool newList){
	bool listRoutesFlag=newList;
	if(setDEFgetUSE( str, newList)){
		return;
	}
	str+="Script{";

	M_String& url = this->url.refValue();
	String urlStr;
	
	if(function.length){
		if(g_writeX3DVRMLClassic){
			urlStr="\nurl \"ecmascript:\n";
		}
		else{
			urlStr="\nurl \"javascript:\n";
		}
		for(int k=0; k<function.length; k++){
			function[k]->toString(urlStr);
		}
		urlStr+='\"';
	}

	for(int i=1; i<3; i++){
		Field* field=getField(i);
		const Properties& sprop=getProp(i);

		if(	(sprop.isReadable()^sprop.isWritable())==false){

			Field* protoField=g_lookupProtoInterface[getType()]->getField(i);

			String tmpStr;
			field->toString(tmpStr, protoField, newList);
			if(tmpStr!=""){
				str+='\n';
				str+=properties[i].name;
				str+=' ';
				str+=tmpStr;
			}
		}
	}

	{
		for(int i = 3; i < getFLen(); i++){
			Field* field=getField(i);
			const Properties& sprop=getProp(i);
			str+='\n';
			if(	sprop.isWritable()){
				if(	sprop.isReadable()){
					str+=g_exposedFieldPtr;
					str+=' ';
					str+=field->getTypeStr();
					str+=' ';
					str+=sprop.name;
					if(writeISScriptField(str,field)==false){
						str+=' ';
						field->toString(str);
					}
				}
				else{
					str+=g_eventInPtr;
					str+=' ';
					str+=field->getTypeStr();
					str+=' ';
					str+=sprop.name;

					writeISScriptField(str,field);
				}
			}
			else if( sprop.isReadable()){
				str+=g_eventOutPtr;
				str+=' ';
				str+=field->getTypeStr();
				str+=' ';
				str+=sprop.name;

				writeISScriptField(str,field);
			}
			else {
				str+=g_fieldPtr;
				str+=' ';
				str+=field->getTypeStr();
				str+=' ';
				str+=sprop.name;
				if(writeISScriptField(str,field)==false){
					str+=' ';
					if(field->getType()==SFNODE){
						NodePtr node=*dynamic_cast<SFNode*>(field);
						if(node){
							node->toString(str,newList);
						}
						else{
							str+="NULL";
						}
					}
					else if(field->getType()==MFNODE){
						dynamic_cast<MFNode*>(field)->refValue().toMNodeString(str, newList);
					}
					else{
						field->toString(str);
					}
				}
			}
		}
	}
	if(function.length){
		str+=urlStr;
	}
	else if(url.length>0){
		str+="\nurl ";
		int i = str.length+1;
		this->url.toString(str);
		if(g_writeX3DVRMLClassic && str.length>i+11 && str.substr(i, 11).toUpperCase()=="JAVASCRIPT:"){
			str = str.replace(i, 4, "ecma");
		}
	}
	str+="\n}";
}
void Script::toXMLString(String& str, bool newList, const char* containerField){
	static String strIS;
	strIS="\n<IS>";
	bool listRoutesFlag=newList;
	int i=0;
	const M_Properties &properties=getProperties();
	str+="<Script";
	if(containerField && (containerFieldLookUpStr[getType()]!=containerField) ){
		str+=" containerField='";
		str+=containerField;
		str+="'";
	}
	if(setDEFgetUSE( str, newList)){
		return;
	}

	M_String& url = this->url.refValue();
	String urlStr;

	if(function.length){
		urlStr+="\n<![CDATA[";
		urlStr+="ecmascript:\n";
		for(int k=0; k<function.length; k++){
			function[k]->toString(urlStr);
		}
		urlStr+="]]>";
	}
	if(*dynamic_cast<SFBool*>(getField(1))){
		str+=' ';
		str+=properties[1].name;
		str+="='true'";
	}
	if(*dynamic_cast<SFBool*>(getField(2))){
		str+=' ';
		str+=properties[2].name;
		str+="='true'";
	}
	str+='>';
	{
		for(int i=3; i<getFLen(); i++){
			Field* field = getField(i);
			const Properties& sprop=properties[i];
			if(field->isRef()){
				strIS+="\n<connect nodeField='";
				strIS+=properties[i].name;
				strIS+="' protoField='";
				ProtoDeclare* protoDeclare=0;
				int j=0;
				findMatchingIs(field, protoDeclare, j);
				strIS+=protoDeclare->protoInterface->properties[j].name;
				strIS+="'/>";
			}
			str+="\n<field name='";
			str+=sprop.name;
			str+="' type='";
			str+=field->getTypeStr();
			str+="' accessType='";
			if((sprop.isReadable()^sprop.isWritable())==false){
				if(sprop.isReadable()){
					str+="inputOutput'";
				}
				else{
					str+="initializeOnly'";
				}
				if(SFNode* sfnode=dynamic_cast<SFNode*>(field)){
					str+=">\n";
					if(Node* node=sfnode->p){
						node->toXMLString(str, newList, 0);
					}
					str+="\n</field>";
					continue;
				}
				else if(MFNode* mfnode=dynamic_cast<MFNode*>(field)){
					str+=">\n";
					if(mfnode->refValue().length){
						mfnode->toXMLString(str, newList, 0);
					}
					str+="\n</field>";
					continue;
				}
				else{
					str+=" value='";
					field->toString(str);
					str+='\'';
				}
			}
			else if(sprop.isWritable()){
				str+="inputOnly'";
			}
			else if(sprop.isReadable()){
				str+="outputOnly'";
			}
			str+="/>";
		}
	}
	if(strIS.length!=5){
		str+=strIS;
		str+="\n</IS>";
	}
	if(function.length){
		str+=urlStr;
	}
	else if(url.length>0){
		str+="\n<![CDATA[";
		int i = str.length;
		if(url.length){
			str+=url[0];
		}
		if(str.length>i+11 && str.substr(i, 11).toUpperCase()=="JAVASCRIPT:"){
			str = str.replace(i, 4, "ecma");
		}
		str+="\n]]>";
	}
	str+="\n</Script>";
}

void Node::getNode(const String& str,int& i){
	checkBackgroundColor(str, i);
//	str.stb(i);
	if(str[i]!='{')throw this;
	i++;
	str.stb(i);
	Function* function=dynamic_cast<Function*>(this);
	if(function && str[i]!='}'){
		int bracketCntr=1;
		function->decodeHeader(str, i, bracketCntr);
	}
	else{
		while(str[i]!='}'){
			getFields(str, i);
			str.stb(i);
		}
	}
	i++;
	checkBackgroundColor(str, i);
}

Node* newFunction(){
	return new Function;
}
#pragma warning( disable : 4731)

void paintTrack(int x, int y, int dstWidth, int dstHeight, int* dstPix){
	//drawHorizontalLine2(0, y-4, x-12, x, dstWidth, dstHeight, dstPix);
	//drawHorizontalLine2(0, y-3, x-12, x, dstWidth, dstHeight, dstPix);

	for(int i = 0; i < 2; i++){
		drawVerticalLine2(0, x - 24 - i * 4, y - 7, y + 1, dstWidth, dstHeight, dstPix);
	}
}

void Node::paintIcon( int indent, int line, int& paintLine, int collapsedLine, bool visible, bool animMode, Function* lastFunction){
	paintLine=line-g_stw->getScrollPosCollapsed();

	int dstWidth=g_stw->size.x;
	int dstHeight=g_stw->size.y;
	int* dstPix=g_stw->pixBuf.getArray();

	if(line>=g_stw->getScrollPosCollapsed()-1){
		int x = tLnOffst+indent*pixSymbolCX+16+3;
		int y = SceneTree::beginYOffset+paintLine*pixSymbolCY+12;
		static String str;
		str="";
		static M_ColorFormat format;
		format.length=0;
		/*
		if(visible && line==collapsedLine){

			paintColorSweepForCurvedBar(&g_tb->iconPixels[(176-1-25-24)*480+470], &g_tb->iconPixels[(176-1-25-24)*480+471], 9, y-16, Vec2i(dstWidth - 9, 24), dstWidth, dstHeight, dstPix);
			pasteSprite2( 0, y-16, 461, 25+24, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 10, 24, dstPix, g_tb->iconPixels);
			pasteSprite2( dstWidth-9, y-16, 471, 25+24, dstWidth, dstHeight, iconSrcBufCX, iconSrcBufCY, 9, 24, dstPix, g_tb->iconPixels);

			//paintBlock2( 0xffff80, x, y-10, g_stw->size.x-8, y+4, g_stw->size.x, g_stw->size.y, g_stw->pixBuf.getArray());
		}
*/
		unsigned horizontalColor=treeLineColor;
		if(!g_sceneTreeCollapsed[line].ref && g_sceneTreeCollapsed[line].node->collapsed){
			horizontalColor=collapsedLineColor;
			drawHorizontalLine2(horizontalColor, SceneTree::beginYOffset+8+paintLine*pixSymbolCY-1, indent*pixSymbolCX-1, tLnOffst+indent*pixSymbolCX, dstWidth, dstHeight, dstPix);
			drawHorizontalLine2(horizontalColor, SceneTree::beginYOffset+8+paintLine*pixSymbolCY+1, indent*pixSymbolCX-1, tLnOffst+indent*pixSymbolCX, dstWidth, dstHeight, dstPix);
		}
		drawHorizontalLine2(horizontalColor, SceneTree::beginYOffset+8+paintLine*pixSymbolCY, indent*pixSymbolCX-1, tLnOffst+indent*pixSymbolCX, dstWidth, dstHeight, dstPix);

		Art* part = dynamic_cast<Art*>(this);
		Build* bld = dynamic_cast<Build*>(this);
		if((part && part->show == false) || (bld && bld->isCaged() == false)){
			pasteSprite2(tLnOffst+indent*pixSymbolCX, SceneTree::beginYOffset+paintLine*pixSymbolCY+5, hiddenPart_x, hiddenPart_y, g_stw->size.x, g_stw->size.y, iconSrcBufCX, iconSrcBufCY, hiddenPart_w, hiddenPart_h, g_stw->pixBuf.getArray());
		}
		else{
			int type=getType();
			if(type>=g_userProtoBegin){
				type=PROTOINSTANCE;
			}
			pasteSprite2(tLnOffst+indent*pixSymbolCX, SceneTree::beginYOffset+paintLine*pixSymbolCY, builtInProtoList[type]->iconSrcPos.x, builtInProtoList[type]->iconSrcPos.y, g_stw->size.x, g_stw->size.y, iconSrcBufCX, iconSrcBufCY, pixSymbolCX, pixSymbolCY, g_stw->pixBuf.getArray());
		}
	
		if(CodeLine* codeLine=dynamic_cast<CodeLine*>(this)){
			int _x=x;
			for(int w=codeLine->wordFrom; w<codeLine->wordTo; w++){
				Word* wrd=lastFunction->m_word[w];
				wrd->toString(str);
				format+=ColorFormat( wrd->getColor(), str.length, _x);
				_x=x+str.length*charWidth;
			}
		}
		if(G_Variable* g_Variable=dynamic_cast<G_Variable*>(this)){
			int _x=x;
			M_Word& m_word = g_Variable->m_word;
			for(int w=0; w<m_word.length; w++){
				Word* wrd=m_word[w];
				wrd->toString(str);
				format+=ColorFormat( wrd->getColor(), str.length, _x);
				_x=x+str.length*charWidth;
			}
		}
		else if(this==lastFunction){
			lastFunction->headerToColorFormat(str, format, x);
		}
		else{
			unsigned color=NODE_COLOR;
			if(dynamic_cast<G_Variable*>(this)){
				color=GLOBAL_COLOR;
			}
//			drawText(name, color, _x, y, g_st.bitMap.bmWidth, g_st.bitMap.bmHeight, g_st.pixBuf.getArray());
			str=name;
			int _x = x;
			format+=ColorFormat(color, str.length, _x);
		}
		if(visible && line == collapsedLine && g_stw->textCell.cellActive){
			g_stw->textCell.paintCellStr();
		}
		else{
			for(int charBegin=0, i=0; i<format.length; i++){
				drawText2(str, charBegin, format[i].end ,format[i].color, Vec2i(format[i].x, y), dstWidth, dstHeight, dstPix);
				charBegin=format[i].end;
			}
		}
		if(g_sceneTreeCollapsed[line].ref){
			pasteSprite2(tLnOffst+indent*pixSymbolCX-3, SceneTree::beginYOffset+paintLine*pixSymbolCY+9, 192, 96, g_stw->size.x, g_stw->size.y, iconSrcBufCX, iconSrcBufCY, 7, 7, g_stw->pixBuf.getArray());
		}
		if(animMode){
			if(part){
				if(part->track && part->smls->track){
					paintTrack(x, y, dstWidth, dstHeight, dstPix);
				}
			}
			else if(Seamless* smls = dynamic_cast<Seamless*>(this)){
				if(smls->track){
					paintTrack(x, y, dstWidth, dstHeight, dstPix);
				}
			}
			else if(Viewpoint* view = dynamic_cast<Viewpoint*>(this)){
				if(view->track){
					paintTrack(x, y, dstWidth, dstHeight, dstPix);
				}
			}
		}

/*
		else{		
			TextOut(
				g_st.hdc,           // handle to device context
				nameOffset+pixSymbolCX+indent*pixSymbolCX,       // x-coordinate of starting position
				tLnOffst+paintLine*pixSymbolCY,       // y-coordinate of starting position
				name.c_str(), name.length
			);
		}
	*/
	}
}


void Node::paintSceneTree(int indent, int& line, int collapsedLine, bool visible, bool animMode, Function* lastFunction){
	if(Function* f=dynamic_cast<Function*>(this)){
		lastFunction=f;
	}
	int paintLine;
	/*
	Art* part=dynamic_cast<Art*>(this);
	if(part && part->show==false){
		paintIcon( indent, line, 403, 13, paintLine, collapsedLine, visible, lastFunction);
	}
	else{
		int type=getType();
		if(type>=g_userProtoBegin){
			type=PROTOINSTANCE;
		}
		//paintIcon( indent, line, nodeIconLookUp[type].srcPos.x, nodeIconLookUp[type].srcPos.y, paintLine, collapsedLine, visible, lastFunction);
	}
*/
	paintIcon(indent, line, paintLine, collapsedLine, visible, animMode, lastFunction);

	if(dynamic_cast<Viewpoint*>(this)){
		if(g_viewIndex < g_viewpoints.length && Viewpoint::neutral == false){
			if(this == g_viewpoints[g_viewIndex]){
				pasteSprite2(indent*16-8, SceneTree::beginYOffset+paintLine*pixSymbolCY, 240, 176, g_stw->size.x, g_stw->size.y, iconSrcBufCX, iconSrcBufCY, 16, 16, g_stw->pixBuf.getArray());
			}
		}
	}
	else if(dynamic_cast<Anim*>(this)){
		if(g_animBarListIndex < g_animBarList.length){
			if(this == g_animBarList[g_animBarListIndex]){
				pasteSprite2(indent*16-8, SceneTree::beginYOffset+paintLine*pixSymbolCY, 240, 176, g_stw->size.x, g_stw->size.y, iconSrcBufCX, iconSrcBufCY, 16, 16, g_stw->pixBuf.getArray());
			}
		}
	}

	if(g_sceneTreeCollapsed[line].ref==0 && g_sceneTreeCollapsed[line].node->collapsed==false){
		line++;
		int remLine=paintLine+1;
		int remIndent=indent;
		indent++;
		const M_Properties &mprop = getProperties();

		for(int i = 0; i < getFLen(); i++){
			Field* field = getField(i);
			const Properties& sprop = mprop[i];
			if(sprop.disable()){
				continue;
			}
			if(field->getType() == MFNODE){
				M_Node &mnode=dynamic_cast<MFNode*>(field)->refValue();
				int length = mnode.length;
				for( int k=0; k<length; k++){
					if(k == length - 1){
						drawVerticalLine2(treeLineColor, 14+remIndent*pixSymbolCX, SceneTree::beginYOffset+remLine*pixSymbolCY, SceneTree::beginYOffset+8+(line-g_stw->getScrollPosCollapsed())*pixSymbolCY, g_stw->size.x, g_stw->size.y, g_stw->pixBuf.getArray());
					}
					int remLine = line;
					mnode[k]->paintSceneTree( indent, line, collapsedLine, visible, animMode, lastFunction);
					if(sprop.blueDot()){
						g_stw->paintBlueDot( indent, remLine);
					}
				}
			}
			else if(field->getType() == SFNODE){
				NodePtr &snode = dynamic_cast<SFNode*>(field)->refValue();
				if(snode != NULL && sprop.disable() == false){
					drawVerticalLine2(treeLineColor, 14+remIndent*pixSymbolCX, SceneTree::beginYOffset+remLine*pixSymbolCY, SceneTree::beginYOffset+8+(line-g_stw->getScrollPosCollapsed())*pixSymbolCY, g_stw->size.x, g_stw->size.y, g_stw->pixBuf.getArray());
					int remLine = line;
					snode->paintSceneTree( indent, line, collapsedLine, visible, animMode, lastFunction);
					if(sprop.blueDot()){
						g_stw->paintBlueDot( indent, remLine);
					}
					/*
					if(!g_sceneTreeCollapsed.ref[remLine] && g_sceneTreeCollapsed.node[remLine]->collapsed){
						paintCollapsedIcon( indent, remLine);;
					}
					*/
				}
			}
		}
	}
	else{
		line++;
	}
}
/*
void Anim::toString(String& str, bool newList){
	int fieldLen=getFLen();
	int startField=0;
	if(g_outputMode){
		fieldLen-=2; //avoid outputting the last 2 fields children and onFrame
	}
	bool listRoutesFlag=newList;
	if(setDEFgetUSE( str, newList)){
		return;
	}
	M_Properties &properties=getProperties();
	str+=getTypeStr();
	str+=" {\n";

	for(int i=startField; i<fieldLen; i++){
		if(g_outputMode && properties[i].doNotOutput()){
			continue;
		}
		Field* field=getField(i);
		if(field->isRef()){
			str+=properties[i].name;
			str+=" IS ";
			ProtoDeclare* protoDeclare=0;
			int j=0;
			findMatchingIs(field, protoDeclare, j);
			str+=protoDeclare->protoInterface->properties[j].name;
			str+='\n';
		}
		else{
			if(	(field->isReadable()^field->isWritable())==false){

				Field* protoField=g_lookupProtoInterface[getType()]->getField(i);

				String tmpStr=field->toString(protoField, newList);

				if(onFrame.p && properties[i].name=="onFrame"){
					//str+="void onFrame ";
					str+=tmpStr;
					str+='\n';
				}
				else if(tmpStr!=""){
					str+=properties[i].name;
					str+=' ';
					str+=tmpStr;
					str+='\n';
				}
			}
		}
	}

	str+="\n}\n";

	if(g_outputMode==true){
		if(Function* f=dynamic_cast<Function*>(onFrame.p)){
			Script* script=getOutputScriptPtr();
			script->function+=f;
			script->name='s';
			makeNodeNameFromAddr(script->name,script);
			script->properties+=Properties(f->functionName);
			Field*  fieldIn = new SFFloat(0);
			*script+=fieldIn;
			slider.route+=S_Route(script, fieldIn);
		}
		for(int k=0; k<children.length; k++){
			children[k]->toString(str, newList);
		}
	}
}
String Anim::toXMLString(bool newList, String containerField){
	int fieldLen=getFLen();
	int startField=0;
	if(g_outputMode==true){
		startField=2; //avoid outputting the first 2 fields freezeControlled and samplePoseEnabled
		fieldLen-=2; //avoid outputting the last 2 fields children and onFrame
	}

	bool listRoutesFlag=newList;
	String str;
	int i;
	M_Properties& properties=getProperties();
	str+='<';
	str+=getTypeStr();
	str+=" ";
	if(containerFieldLookUpStr[getType()]!=containerField){
		if(containerField.length!=0&&containerField!="ProtoBody"){
			str+="containerField='";
			str+=containerField;
			str+="' ";
		}
	}
	if(setDEFgetUSE( str, newList)){
		return str;
	}

	outputXMLFields(startField, fieldLen, properties, *(g_lookupProtoInterface[getType()]), newList, str,i);

	if(Function* f=dynamic_cast<Function*>(onFrame.p)){
		Script* script=getOutputScriptPtr();
		script->function+=f;
		script->name='s';
		makeNodeNameFromAddr(script->name,script);
		script->properties+=Properties(f->functionName);
		Field*  fieldIn = new SFFloat(0);
		*script+=fieldIn;
		slider.route+=S_Route(script, fieldIn);
	}
	for(int k=0; k<children.length; k++){
		str+=children[k]->toXMLString(newList,containerField);
	}
	return str;
}
*/
/*
void TimeSensor::toString(String& str, bool newList){
	int fieldLen=getFLen();
	int startField=0;
	if(g_outputMode==true){
		startField=2; //avoid outputting the first 2 fields freezeControlled and samplePoseEnabled
		fieldLen-=2; //avoid outputting the last 2 fields children and on_fraction_changed
	}
	bool listRoutesFlag=newList;
	if(g_outputMode==true && on_fraction_changed.p && g_scene->outputPovRay == false){
		dynamic_cast<MemberFunction*>(on_fraction_changed.p)->functionName="on_fraction_changed";
		Scene::Output::tempNamedNodeList+=on_fraction_changed;
		if(name == "" ){
			name="t";
			makeNodeNameFromAddr(name,this);
			Scene::Output::tempNamedNodeList+=this;
		}
	}
	if(setDEFgetUSE( str, newList)){
		return;
	}
	M_Properties &properties=getProperties();
	str+=getTypeStr();
	str+=" {\n";

	for(int i=startField; i<fieldLen; i++){
		Field* field=getField(i);
		if(field->isRef()){
			str+=properties[i].name;
			str+=" IS ";
			ProtoDeclare* protoDeclare=0;
			int j=0;
			findMatchingIs(field, protoDeclare, j);
			str+=protoDeclare->protoInterface->properties[j].name;
			str+='\n';
		}
		else{
			if(	(field->isReadable()^field->isWritable())==false){

				Field* protoField=g_lookupProtoInterface[getType()]->getField(i);

				String tmpStr=field->toString(protoField, newList);
				if(on_fraction_changed.p && properties[i].name=="on_fraction_changed"){
					str+="void on_fraction_changed ";
					str+=tmpStr;
					str+='\n';
				}
				else if(tmpStr!=""){
					str+=properties[i].name;
					str+=' ';
					str+=tmpStr;
					str+='\n';
				}
			}
		}
	}

	str+="\n}\n";

	if(g_outputMode==true){
		if(Function* f=dynamic_cast<Function*>(on_fraction_changed.p)){
			Script* script=getOutputScriptPtr();
			script->function+=f;
			script->name='s';
			makeNodeNameFromAddr(script->name,script);
			script->properties+=Properties(f->functionName);
			Field*  fieldIn = new SFFloat(0);
			*script+=fieldIn;
			fraction_changed.route+=S_Route(script, fieldIn);
		}
		for(int k=0; k<children.length; k++){
			children[k]->toString(str, newList);
		}
	}
}
String TimeSensor::toXMLString(bool newList, String containerField){
	int fieldLen=getFLen();
	int startField=0;
	if(g_outputMode==true){
		startField=2; //avoid outputting the first 2 fields freezeControlled and samplePoseEnabled
		fieldLen-=2; //avoid outputting the last 2 fields children and on_fraction_changed
	}

	bool listRoutesFlag=newList;
	String str;
	int i;
	M_Properties& properties=getProperties();
	str+='<';
	str+=getTypeStr();
	str+=" ";
	if(containerFieldLookUpStr[getType()]!=containerField){
		if(containerField.length!=0&&containerField!="ProtoBody"){
			str+="containerField='";
			str+=containerField;
			str+="' ";
		}
	}
	if(setDEFgetUSE( str, newList)){
		return str;
	}

	outputXMLFields(startField, fieldLen, properties, *(g_lookupProtoInterface[getType()]), newList, str,i);

	if(Function* f=dynamic_cast<Function*>(on_fraction_changed.p)){
		Script* script=getOutputScriptPtr();
		script->function+=f;
		script->name='s';
		makeNodeNameFromAddr(script->name,script);
		script->properties+=Properties(f->functionName);
		Field*  fieldIn = new SFFloat(0);
		*script+=fieldIn;
		fraction_changed.route+=S_Route(script, fieldIn);
	}
	for(int k=0; k<children.length; k++){
		str+=children[k]->toXMLString(newList,containerField);
	}
	return str;
}
*/
bool modifyVrmlX3dFilename(String& filename, bool writeX3DXML, bool writeX3DVRMLClassic){
	if(filename=="")return false;
	if(filename.length>5){
		String sbstr6=filename.substr(filename.length-6);
		String sbstr5=filename.substr(filename.length-5);
		String sbstr4=filename.substr(filename.length-4);
		if(sbstr4==".wrl"||sbstr4==".wrz"||sbstr4==".x3d"||sbstr5==".x3dz"||sbstr5==".x3dv"||sbstr6==".x3dvz"){
			if(writeX3DXML){
				filename+=".x3d";
			}
			else if(writeX3DVRMLClassic){
				filename+=".x3dv";
			}
			else{
				filename+=".wrl";
			}
			return true;
		}
	}
	for(int i=filename.length; --i>=1;){
		if(filename[i]=='.'){
			if(writeX3DXML){
				filename=filename.replace(i, 0x7fffffff, ".x3d");
			}
			else if(writeX3DVRMLClassic){
				filename=filename.replace(i, 0x7fffffff, ".x3dv");
			}
			else{
				filename=filename.replace(i, 0x7fffffff, ".wrl");
			}
			return true;
		}
		if(i==1||filename[i]=='\\'||filename[i]=='/'){
			if(writeX3DXML){
				filename+=".x3d";
			}
			else if(writeX3DVRMLClassic){
				filename+=".x3dv";
			}
			else{
				filename+=".wrl";
			}
			return true;
		}
	}
	return false;
}
bool modifyFilenameForBind(String& filename){
	if(filename=="")return false;
	if(filename.length>5){
		String sbstr4=filename.substr(filename.length-4);
		if(sbstr4==".cpp"){
			filename+=".cpp";
			return true;
		}
	}
	for(int i=filename.length; --i>=1;){
		if(filename[i]=='.'){
			filename=filename.replace(i, 0x7fffffff, ".cpp");
			return true;
		}
		if(i==1||filename[i]=='\\'||filename[i]=='/'){
			filename+=".cpp";
			return true;
		}
	}
	return false;
}
Seamless* debugSmls=0;
void convertSeamlessNodesForOutput(){
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Seamless* smls=dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p)){
			smls->buildPartListNoShow();
			smls->outputNode=0;
			convertRoutesToScript(smls);
			if(smls->outputNode==0 && g_genHAnimNodes){ //do we need to ask about smls->outputNode == 0 ?
				smls->addNewHAnimNode();
			}
			debugSmls=smls;
		}
	}
}
void reBuildPartList(){
	for(int k=0; k<g_sceneTreeList.length; k++){
		Seamless* smls=dynamic_cast<Seamless*>(g_sceneTreeList[k].node.p);
		if(smls){
			smls->process();
		}
	}
}
const char* strProtoDeclare_CCPath97="\
PROTO CCPath [\n\
eventIn	SFFloat set_fraction\n\
eventOut	SFFloat fraction_changed\n\
field SFVec3f	location	0 0 0\n\
exposedField SFRotation	orientation	0 1 0 0\n\
field MFFloat  curve      []\n\
field MFFloat  distance   []\n\
field MFFloat  zAngle     []\n\
eventOut     SFRotation rot_changed\n\
eventOut     SFVec3f pos_changed\n\
]\n\
{\n\
Script {\n\
eventIn	SFFloat set_fraction IS set_fraction\n\
eventOut	SFFloat fraction_changed IS fraction_changed\n\
field SFVec3f	location	IS location\n\
exposedField SFRotation	orientation	IS orientation\n\
field MFFloat  curve      IS curve\n\
field MFFloat  distance   IS distance\n\
field MFFloat  zAngle     IS zAngle\n\
eventOut     SFRotation rot_changed IS rot_changed\n\
eventOut     SFVec3f pos_changed IS pos_changed\n\
field SFVec3f	position 0 0 0\n\
field SFVec3f	reletiveDis 0 0 0\n\
field SFVec3f	v 0 0 0\n\
field SFBool	firstCalcDone FALSE\n\
field SFRotation	accumCurveAngle 0 1 0 0\n\
field SFFloat beginHeight 0\n\
field SFFloat endHeight 0\n\
field SFFloat angle 0\n\
field SFFloat axis 0\n\
url	\"javascript:\n\
function  bend(){\n\
axisRot=new SFRotation (0,1,0, axis);\n\
position = axisRot.multVec(position);\n\
diff=endHeight-beginHeight;\n\
if(diff==0){\n\
position.y-=beginHeight;\n\
position = new SFRotation (0,0,1, angle).multVec(position);\n\
position.y+=beginHeight;\n\
position = new SFRotation (0,-1,0, axis).multVec(position);\n\
return;\n\
}\n\
rad=diff/angle;\n\
v=position;\n\
if(!firstCalcDone&&v.y>=beginHeight){\n\
firstCalcDone=true;\n\
reletiveDis.y=v.y-beginHeight;\n\
stepAngle=reletiveDis.y/diff*angle;\n\
accumCurveAngle.angle=stepAngle;\n\
v.y=0;\n\
v.x+=rad;\n\
newX=v.x*Math.cos(stepAngle)-v.y*Math.sin(stepAngle);\n\
v.y=v.x*Math.sin(stepAngle)+v.y*Math.cos(stepAngle);\n\
v.x=newX;\n\
v.y+=beginHeight;\n\
v.x-=rad;\n\
position=v;\n\
}\n\
else if(firstCalcDone){\n\
reletiveDis.y=v.y-beginHeight;\n\
v.y=(rad+v.x)*Math.sin(angle)+beginHeight+	(position.y-endHeight)*(Math.cos(angle));\n\
v.x=(rad+v.x)*Math.cos(angle)-rad	+	(position.y-endHeight)*-(Math.sin(angle));\n\
position=v;\n\
accumCurveAngle=accumCurveAngle.multiply(new SFRotation(0,-1,0,angle));\n\
}\n\
axisRot.y=-1;\n\
position = axisRot.multVec(position);\n\
}\n\
function getDistance(index){\n\
distanceSum=0;\n\
for(i=0;i<=index;i++){\n\
distanceSum+=distance[i];\n\
}\n\
return distanceSum;\n\
}\n\
function getZangleSum(index){\n\
zAngleSum=0;\n\
for(i=0;i<=index;i++){\n\
zAngleSum+=zAngle[i];\n\
}\n\
return zAngleSum;\n\
}\n\
function set_fraction(){\n\
accumCurveAngle=new SFRotation(0,-1,0,0);\n\
position.x=0;\n\
position.y=getDistance(distance.length-1)*set_fraction;\n\
position.z=0;\n\
i=0;\n\
firstCalcDone=false;\n\
for(i=curve.length;--i>1;){\n\
beginHeight=getDistance(i-2);\n\
endHeight=getDistance(i-1);\n\
angle=curve[i];\n\
axis=getZangleSum(i-1);\n\
bend();\n\
}\n\
if(i==1){\n\
beginHeight=0;\n\
endHeight=distance[0];\n\
angle=curve[1];\n\
axis=zAngle[0];\n\
bend();\n\
}\n\
position=new SFRotation(1,0,0,Math.PI*.5).multVec(position);\n\
position=orientation.multVec(position);\n\
position=position.add(location);\n\
accumCurveAngle=accumCurveAngle.multiply(orientation);;\n\
rot_changed=accumCurveAngle;\n\
pos_changed=position;\n\
fraction_changed=set_fraction;\n\
}\n\"\n}\n}";

const char* strProtoDeclare_RotFilter97="\
PROTO RotFilter [\n\
eventIn		SFRotation	inRot\n\
eventOut	SFRotation	outRot\n\
field	SFVec3f	axis 1 1 1\n\
exposedField	SFFloat angle 1\n\
]\n\
{\n\
Script {\n\
eventIn		SFRotation	inRot	IS	inRot\n\
eventOut	SFRotation	outRot	IS	outRot\n\
field	SFVec3f	axis IS	axis\n\
exposedField	SFFloat angle	IS	angle\n\
url	\"javascript:\n\
function inRot(v){\n\
outRot=new SFRotation((new SFVec3f(axis.x*v.x,axis.y*v.y,axis.z*v.z)).normalize(),angle*v.angle);\n\
}\n\"\n}\n}\n";

const char* strProtoDeclare_RotMult97="\
PROTO RotMult [\n\
eventIn	SFRotation	inA\n\
eventIn	SFRotation	inB\n\
eventOut	SFRotation	catenation\n\
]\n\
{\n\
Script {\n\
eventIn	SFRotation	inA IS	inA\n\
eventIn	SFRotation	inB	IS	inB\n\
eventOut	SFRotation	catenation IS	catenation\n\
url	\"javascript:\n\
function inA(par){\n\
catenation=par.multiply(inB);\n\
}\n\
function inB(par){\n\
catenation=inA.multiply(par);\n\
}\n\"\n}\n}";

const char* strProtoDeclare_AnimByAnkle97Hanim="\
PROTO AnimByAnkle [\n\
field SFNode pelvis NULL\n\
field SFNode hip NULL\n\
field SFNode knee NULL\n\
field SFNode ankle NULL\n\
eventIn SFVec3f position\n\
field SFBool outAnkle TRUE\n\
eventIn SFFloat swivel\n\
]{\n\
Script{\n\
directOutput TRUE\n\
field SFNode pelvis IS pelvis\n\
field SFNode hip IS	hip\n\
field SFNode knee IS knee\n\
field SFNode ankle IS ankle\n\
eventIn SFVec3f position IS	position\n\
field SFBool outAnkle IS outAnkle\n\
eventIn SFFloat swivel IS	swivel\n\
field SFVec3f hipTran 0 0 0\n\
field SFFloat kneeTran 0\n\
field SFFloat ankleTran 0\n\
url	\"javascript:\n\
function initialize(){\n\
hipTran=hip.center.subtract(pelvis.center);\n\
kneeTran=hip.center.y-knee.center.y;\n\
ankleTran=knee.center.y-ankle.center.y;\n\
}\n\
function position(val){\n\
location=hipTran;\n\
matrixA = new VrmlMatrix();\n\
matrixB = new VrmlMatrix();\n\
matrixA.setTransform(pelvis.translation.negate());\n\
matrixB.setTransform(new SFVec3f(), pelvis.rotation.inverse());\n\
matrixA=matrixB.multLeft(matrixA);\n\
val=matrixA.multMatrixVec(val);\n\
upperarmLen=Math.abs(ankleTran);\n\
xDist=location.x-val.x;\n\
yDist=Math.abs(location.y-val.y);\n\
zDist=location.z-val.z;\n\
distance=Math.sqrt(xDist*xDist+yDist*yDist+zDist*zDist);\n\
halfKneeAng=Math.acos(distance*.5/upperarmLen);\n\
xyDist=Math.sqrt(xDist*xDist+yDist*yDist);\n\
zHipAng=-Math.asin(xDist/xyDist);\n\
xAng=Math.acos(zDist/Math.sqrt(xyDist*xyDist+zDist*zDist));\n\
xHipAng=-halfKneeAng-xAng+Math.PI*.5;\n\
xAnkleAng=-halfKneeAng+xAng-Math.PI*.5;\n\
hip.rotation=new SFRotation(1,0,0,xHipAng).multiply(new SFRotation(0,0,1,zHipAng));\n\
knee.rotation=new SFRotation(1,0,0,halfKneeAng*2);\n\
if(outAnkle){\n\
ankle.rotation=pelvis.rotation.inverse().multiply(new SFRotation(0,0,-1,zHipAng)).multiply(new SFRotation(1,0,0,xAnkleAng));\n\
}\n}\n\"\n}\n}";

const char* strProtoDeclare_AnimByAnkle97="\
PROTO AnimByAnkle [\n\
field SFNode script NULL\n\
field SFInt32 pelvisIdx 0\n\
field SFInt32 hipIdx 0\n\
field SFInt32 kneeIdx 0\n\
field SFInt32 ankleIdx 0\n\
eventIn		SFVec3f position\n\
field SFBool outAnkle TRUE\n\
eventIn SFFloat swivel\n\
field SFVec3f hipTran .077557 0 -.005738\n\
field SFVec3f kneeTran -.001977 -.402827 .004605\n\
field SFVec3f ankleTran -.001142 -.436121 -.007118\n\
eventOut SFRotation	hipOut\n\
eventOut SFRotation	kneeOut\n\
eventOut SFRotation	ankleOut\n\
]\n\
{\n\
Script {\n\
field SFNode script IS script\n\
field SFInt32 pelvisIdx IS pelvisIdx\n\
field SFInt32 hipIdx IS	hipIdx\n\
field SFInt32 kneeIdx IS kneeIdx\n\
field SFInt32 ankleIdx IS ankleIdx\n\
eventIn		SFVec3f position IS	position\n\
field SFBool outAnkle IS	outAnkle\n\
eventIn SFFloat swivel IS	swivel\n\
field SFVec3f hipTran IS	hipTran\n\
field SFVec3f kneeTran IS	kneeTran\n\
field SFVec3f ankleTran IS	ankleTran\n\
field SFVec3f location 0 0 0\n\
eventOut SFRotation	hipOut IS hipOut\n\
eventOut SFRotation	kneeOut	IS kneeOut\n\
eventOut SFRotation	ankleOut IS	ankleOut\n\
url	\"javascript:\n\
function position(val){\n\
o=script.o;\n\
translatePart=script.translatePart;\n\
location=hipTran;\n\
matrixA = new VrmlMatrix();\n\
matrixB = new VrmlMatrix();\n\
matrixA.setTransform(translatePart[pelvisIdx].negate());\n\
matrixB.setTransform(new SFVec3f(), o[pelvisIdx].inverse());\n\
matrixA=matrixB.multLeft(matrixA);\n\
val=matrixA.multMatrixVec(val);\n\
upperarmLen=Math.abs(ankleTran.y);\n\
xDist=location.x-val.x;\n\
yDist=Math.abs(location.y-val.y);\n\
zDist=location.z-val.z;\n\
distance=Math.sqrt(xDist*xDist+yDist*yDist+zDist*zDist);\n\
halfKneeAng=Math.acos(distance*.5/upperarmLen);\n\
xyDist=Math.sqrt(xDist*xDist+yDist*yDist);\n\
zHipAng=-Math.asin(xDist/xyDist);\n\
xAng=Math.acos(zDist/Math.sqrt(xyDist*xyDist+zDist*zDist));\n\
xHipAng=-halfKneeAng-xAng+Math.PI*.5;\n\
xAnkleAng=-halfKneeAng+xAng-Math.PI*.5;\n\
hipOut = new SFRotation(1,0,0,xHipAng).multiply(new SFRotation(0,0,1,zHipAng));\n\
kneeOut=new SFRotation(1,0,0,halfKneeAng*2);\n\
if(outAnkle){\n\
ankleOut=o[pelvisIdx].inverse().multiply(new SFRotation(0,0,-1,zHipAng)).multiply(new SFRotation(1,0,0,xAnkleAng));\n\
}\n}\n\"\n}\n}";

const char* strProtoDeclare_FloatAccelerator97="\
PROTO FloatAccelerator[\n\
eventIn	SFFloat   set_fraction\n\
eventOut	SFFloat fraction_changed\n\
eventOut	SFFloat value_changed\n\
eventOut	SFFloat negValue_changed\n\
field SFFloat fracMult 1\n\
field SFFloat	init 0\n\
field MFFloat delay[0]\n\
field MFFloat accelerate[2]\n\
field MFFloat attack[.5]\n\
field MFFloat sustain[0]\n\
field MFFloat decay [3]\n\
field SFFloat addToNegValue 0\n\
]\n\
{\n\
Script{\n\
directOutput TRUE\n\
eventIn	SFFloat    set_fraction IS	set_fraction\n\
eventOut	SFFloat fraction_changed IS fraction_changed\n\
eventOut	SFFloat value_changed IS value_changed\n\
eventOut	SFFloat negValue_changed IS negValue_changed\n\
field SFFloat fracMult IS fracMult\n\
field SFFloat init IS init\n\
field MFFloat delay IS delay\n\
field MFFloat accelerate IS	accelerate\n\
field MFFloat attack IS	attack\n\
field MFFloat sustain IS sustain\n\
field MFFloat decay IS decay\n\
field SFFloat addToNegValue IS addToNegValue\n\
field SFFloat angle 0\n\
field SFFloat time 0\n\
field SFFloat accum 0\n\
url	\"javascript:\n\
function set_fraction(v){\n\
accum=init;\n\
remTime=set_fraction*fracMult;\n\
for(i=0; i<delay.length; i++){\n\
time=remTime-delay[i];\n\
if(time<=0){\n\
continue;\n\
}\n\
_accelerate=accelerate[i];\n\
_attack=attack[i];\n\
_sustain=sustain[i];\n\
_decay=decay[i];\n\
if(time<_attack){\n\
angle=_accelerate*.5*(time*time);\n\
accum+=angle;\n\
continue;\n\
}\n\
time-=_attack;\n\
if(time<_sustain){\n\
attackDist=_accelerate*.5*(_attack*_attack);\n\
attackVelocity=_accelerate*_attack;\n\
angle=attackDist+attackVelocity*time;\n\
accum+=angle;\n\
continue;\n\
}\n\
time-=_sustain;\n\
if(time>_decay){\n\
time=0;\n\
}\n\
else {\n\
time=_decay-time;\n\
}\n\
attackDist=_accelerate*.5*(_attack*_attack);\n\
attackVelocity=_accelerate*_attack;\n\
decayAccel=attackVelocity/_decay;\n\
sustainDist=attackDist+attackVelocity*_sustain;\n\
decayDist=decayAccel*0.5*(_decay*_decay);\n\
decayOffset=sustainDist+decayDist;\n\
angle=decayOffset-decayAccel*0.5*(time*time);\n\
accum+=angle;\n\
}\n\
value_changed=accum;\n\
negValue_changed=-accum+addToNegValue;\n\
fraction_changed=set_fraction;\n\
}\n\"\n}\n}";

const char* strProtoDeclare_PosAccelerator97="\n\
PROTO PosAccelerator [\n\
eventIn	SFFloat   set_fraction\n\
eventOut	SFFloat fraction_changed\n\
eventOut	SFVec3f value_changed\n\
field SFFloat fracMult 1\n\
field SFVec3f	init 0 0 0\n\
field MFVec3f direction[1 0 0]\n\
field MFFloat delay[0]\n\
field MFFloat accelerate[2]\n\
field MFFloat attack[.5]\n\
field MFFloat sustain[0]\n\
field MFFloat decay [3]\n\
]\n\
{\n\
Script {\n\
eventIn	SFFloat    set_fraction IS	set_fraction\n\
eventOut	SFFloat fraction_changed IS fraction_changed\n\
eventOut	SFVec3f value_changed IS value_changed\n\
field SFFloat fracMult IS fracMult\n\
field SFVec3f init IS init\n\
field MFVec3f direction IS direction\n\
field MFFloat delay IS delay\n\
field MFFloat accelerate IS	accelerate\n\
field MFFloat attack IS	attack\n\
field MFFloat sustain IS sustain\n\
field MFFloat decay IS decay\n\
field SFFloat angle 0\n\
field SFFloat time 0\n\
field SFVec3f accum 0 0 0\n\
url	\"javascript:\n\
function set_fraction(v){\n\
accum=init;\n\
remTime=set_fraction*fracMult;\n\
for(i=0; i<delay.length; i++){\n\
time=remTime-delay[i];\n\
if(time<=0){\n\
continue;\n\
}\n\
_accelerate=accelerate[i];\n\
_attack=attack[i];\n\
_sustain=sustain[i];\n\
_decay=decay[i];\n\
if(time<_attack){\n\
angle=_accelerate*.5*(time*time);\n\
accum=accum.add(direction[i].multiply(angle));\n\
continue;\n\
}\n\
time-=_attack;\n\
if(time<_sustain){\n\
attackDist=_accelerate*.5*(_attack*_attack);\n\
attackVelocity=_accelerate*_attack;\n\
angle=attackDist+attackVelocity*time;\n\
accum=accum.add(direction[i].multiply(angle));\n\
continue;\n\
}\n\
time-=_sustain;\n\
if(time>_decay){\n\
time=0;\n\
}\n\
else {\n\
time=_decay-time;\n\
}\n\
attackDist=_accelerate*.5*(_attack*_attack);\n\
attackVelocity=_accelerate*_attack;\n\
decayAccel=attackVelocity/_decay;\n\
sustainDist=attackDist+attackVelocity*_sustain;\n\
decayDist=decayAccel*0.5*(_decay*_decay);\n\
decayOffset=sustainDist+decayDist;\n\
angle=decayOffset-decayAccel*0.5*(time*time);\n\
accum=accum.add(direction[i].multiply(angle));\n\
}\n\
value_changed=accum;\n\
fraction_changed=set_fraction;\n\
}\n\"\n}\n}";

const char* strProto_RotAccelerator97="\
PROTO PRotAccelerator[\n\
eventIn	SFFloat   set_fraction\n\
eventOut	SFFloat fraction_changed\n\
eventOut	SFRotation value_changed\n\
field SFFloat fracMult 1\n\
field SFRotation	init 0 0 1 0\n\
field MFVec3f axis[0 0 1]\n\
field MFFloat delay[0]\n\
field MFFloat accelerate[2]\n\
field MFFloat attack[.5]\n\
field MFFloat sustain[0]\n\
field MFFloat decay [3]\n\
]\n\
{\n\
Script{\n\
directOutput TRUE\n\
eventIn	SFFloat	set_fraction IS	set_fraction\n\
eventOut	SFFloat fraction_changed IS fraction_changed\n\
eventOut	SFRotation value_changed IS value_changed\n\
field SFFloat fracMult IS fracMult\n\
field SFRotation init IS init\n\
field MFVec3f axis IS axis\n\
field MFFloat delay IS delay\n\
field MFFloat accelerate IS	accelerate\n\
field MFFloat attack IS	attack\n\
field MFFloat sustain IS sustain\n\
field MFFloat decay IS decay\n\
field SFFloat angle 0\n\
field SFFloat time 0\n\
field SFRotation accum 0 0 1 0\n\
url	\"javascript:\n\
function set_fraction(v){\n\
accum=init;\n\
time=v*fracMult;\n\
remTime=time;\n\
for(i=0; i<delay.length; i++){\n\
time=remTime;\n\
time-=delay[i];\n\
if(time<=0){\n\
continue;\n\
}\n\
_accelerate=accelerate[i];\n\
_attack=attack[i];\n\
_sustain=sustain[i];\n\
_decay=decay[i];\n\
if(time<_attack){\n\
angle=_accelerate*.5*(time*time);\n\
accum=accum.multiply(new SFRotation(axis[i], angle));\n\
continue;\n\
}\n\
time-=_attack;\n\
if(time<_sustain){\n\
attackDist=_accelerate*.5*(_attack*_attack);\n\
attackVelocity=_accelerate*_attack;\n\
angle=attackDist+attackVelocity*time;\n\
accum=accum.multiply(new SFRotation (axis[i], angle));\n\
continue;\n\
}\n\
time-=_sustain;\n\
if(time>_decay){\n\
time=0;\n\
}\n\
else {\n\
time=_decay-time;\n\
}\n\
attackDist=_accelerate*.5*(_attack*_attack);\n\
attackVelocity=_accelerate*_attack;	//exactly same as attackDist?\n\
decayAccel=attackVelocity/_decay;\n\
sustainDist=attackDist+attackVelocity*_sustain;\n\
decayDist=decayAccel*0.5*(_decay*_decay);\n\
decayOffset=sustainDist+decayDist;\n\
angle=decayOffset-decayAccel*0.5*(time*time);\n\
accum=accum.multiply(new SFRotation (axis[i], angle));\n\
}\n\
value_changed=accum;\n\
fraction_changed=set_fraction;\n\
}\n\"\n}\n}";

const char* strProto_Anim97="\
PROTO PAnim[\n\
exposedField SFBool play FALSE\n\
exposedField SFBool pause TRUE\n\
exposedField SFTime period 1\n\
eventOut SFFloat slider\n\
field SFBool loop FALSE\n\
exposedField MFNode	children[]\n\
exposedField SFTime	startTime 0\n\
]{\n\
DEF timeSensor TimeSensor{\n\
cycleInterval IS period\n\
fraction_changed IS	slider\n\
loop IS	loop\n\
startTime IS startTime\n\
}\n\
Script{\n\
directOutput TRUE\n\
exposedField SFBool play IS play\n\
exposedField SFBool pause IS pause\n\
exposedField SFTime period IS period\n\
field SFNode timeSensor USE timeSensor\n\
field SFBool block FALSE\n\
url	\"javascript:\n\
function initialize(){\n\
if(play==false){\n\
timeSensor.stopTime=1;\n\
}\n\
pause=!play;\n\
}\n\
function play(v, t){\n\
if(block){\n\
return;\n\
}\n\
block=true;\n\
if(v){\n\
timeSensor.startTime=t;\n\
}\n\
else{\n\
timeSensor.stopTime=t;\n\
}\n\
if(v==false){\n\
pause=true;\n\
}\n\
else{\n\
pause=false;\n\
}\n\
block=false;\n\
}\n\
function pause(v, t){\n\
if(block){\n\
return;\n\
}\n\
block=true;\n\
if(!v){\n\
t2=t-timeSensor.fraction_changed*period;\n\
timeSensor.stopTime=t2-1;\n\
timeSensor.startTime=t2;\n\
}\n\
else{\n\
timeSensor.stopTime=t;\n\
}\n\
if(v==false){\n\
play=true;\n\
}\n\
else{\n\
play=false;\n\
}\n\
block=false;\n\
}\"\n\
}\n\
}\n";

inline void endProto(String& str){
	if(g_exportVrmlX3d->x3dXml){
		str+="]]>\n</Script>\n</ProtoBody>\n</ProtoDeclare>";
	}
	else{
		str+="\"\n}\n}\n";
	}
}

void extractFileName(String& filename, const String& path){
	static String drive;
	static String dir;
	static String fname;
	static String ext;
	splitPath(path, drive, dir, fname, ext);
	filename+=fname;
	filename+=ext;
}

void ExportVrmlX3d::Vrml::operator=(const bool& par){
	home.vrml.refValue() = true;
	home.x3dVrmlClassic.refValue() = false;
	home.x3dXml.refValue() = false;
	Window::activeWindow->paint();
}
void ExportVrmlX3d::X3dVrmlClassic::operator=(const bool& par){
	home.vrml.refValue() = false;
	home.x3dVrmlClassic.refValue() = true;
	home.x3dXml.refValue() = false;
	Window::activeWindow->paint();
}
void ExportVrmlX3d::X3dXml::operator=(const bool& par){
	home.vrml.refValue() = false;
	home.x3dVrmlClassic.refValue() = false;
	home.x3dXml.refValue() = true;
	Window::activeWindow->paint();
}
bool noPovLight(){
	for(int k = 0; k < g_sceneTreeList.length; k++){
		if(dynamic_cast<PovLight*>(g_sceneTreeList[k].node.p)){
			return false;
		}
	}
	return true;
}
void ExportMovie::PovRay::operator=(const bool& par){
	home.povRay.refValue() = par;
	home.processFields();
	if(par && noPovLight()){
		g_scene->children.refValue().insertElement(0, new PovLight);
		g_scene->genNodeList2();
		g_stw->paint();
	}
	Window::activeWindow->paint();
}

void ExportMovie::MovieMPG::operator=(const bool& par){
	home.mpg.refValue() = true;
	home.mp4.refValue() = false;
	home.avi.refValue() = false;
	home.flv.refValue() = false;
	Window::activeWindow->paint();
}
void ExportMovie::MovieMP4::operator=(const bool& par){
	home.mpg.refValue() = false;
	home.mp4.refValue() = true;
	home.avi.refValue() = false;
	home.flv.refValue() = false;
	Window::activeWindow->paint();
}
void ExportMovie::MovieAVI::operator=(const bool& par){
	home.mpg.refValue() = false;
	home.mp4.refValue() = false;
	home.avi.refValue() = true;
	home.flv.refValue() = false;
	Window::activeWindow->paint();
}
void ExportMovie::MovieFLV::operator=(const bool& par){
	home.mpg.refValue() = false;
	home.mp4.refValue() = false;
	home.avi.refValue() = false;
	home.flv.refValue() = true;
	Window::activeWindow->paint();
}

void PovLight::Spotlight::operator=(const bool& par){
	home.spotlight.refValue()=true;
	home.pointlight.refValue()=false;
	home.shadowless.refValue()=false;
	Window::activeWindow->paint();
}
void PovLight::Pointlight::operator=(const bool& par){
	home.spotlight.refValue()=false;
	home.pointlight.refValue()=true;
	home.shadowless.refValue()=false;
	Window::activeWindow->paint();
}
void PovLight::Shadowless::operator=(const bool& par){
	home.spotlight.refValue()=false;
	home.pointlight.refValue()=false;
	home.shadowless.refValue()=true;
	Window::activeWindow->paint();
}

void addNewProtoDeclareToList(ProtoDeclare* protoDeclare);

extern M_ProtoDeclare g_protoList;

static ProtoDeclarePtr g_gestureProtoDeclare;
static ProtoInterfacePtr g_gestureProtoInterface;
static ProtoBodyPtr g_gestureProtoBody;
static ProtoInstancePtr g_gestureProtoInstance;

void Node::processForOutput(M_Node& parentChild, int& n, Script* gs){
	n++;
}
void Function::processForOutput(M_Node& parentChild, int& n, Script* gs){
	gs->function+=this;
	parentChild.deleteElement(n);
}
void NoOutputGroup::processForOutput(M_Node& parentChild, int& n, Script* gs){
	parentChild.deleteElement(n);
}
void CPMorpher::processForOutput(M_Node& parentChild, int& n, Script* gs){
	parentChild.deleteElement(n);
}
void PovLight::processForOutput(M_Node& parentChild, int& n, Script* gs){
	parentChild.deleteElement(n);
}
void PovFog::processForOutput(M_Node& parentChild, int& n, Script* gs){
	parentChild.deleteElement(n);
}
void TimeSensor::processForOutput(M_Node& parentChild, int& n, Script* gs){
	M_Node temp;
	if(MemberFunction* on_fraction_changed=dynamic_cast<MemberFunction*>(this->on_fraction_changed.p)){
		name="t";
		makeNodeNameFromAddr(name,this);

		on_fraction_changed->name="f";
		makeNodeNameFromAddr(on_fraction_changed->name, on_fraction_changed);
		on_fraction_changed->functionName=on_fraction_changed->name;

		gs->function+=on_fraction_changed;
		this->on_fraction_changed=0;
		gs->name='s';
		makeNodeNameFromAddr(gs->name, gs);
		gs->properties+=Properties(on_fraction_changed->functionName);
		Field*  fieldIn = new SFFloat(0);
		*gs+=fieldIn;
		//ts->fraction_changed.route+=S_Route(gs, fieldIn);
		S_Route(this, getFieldIndex(&fraction_changed), gs, gs->size()-1);

	}
	temp+=children.refValue();
	children.refValue().length=0;
	insertElements<NodePtr>(++n, 0, parentChild, temp);
}
NodePtr g_fun;
void TouchSensor::processForOutput(M_Node& parentChild, int& n, Script* gs){
	if(MemberFunction* onTouchTime=dynamic_cast<MemberFunction*>(this->onTouchTime.p)){
		name="t";
		makeNodeNameFromAddr(name,this);

		onTouchTime->name="f";
		makeNodeNameFromAddr(onTouchTime->name, onTouchTime);
		onTouchTime->functionName=onTouchTime->name;

		g_fun=this;

		gs->function+=onTouchTime;
//		this->onTouchTime=0;
		gs->name='s';
		makeNodeNameFromAddr(gs->name, gs);
		gs->properties+=Properties(onTouchTime->functionName);
		Field*  fieldIn = new SFDouble(0);
		*gs+=fieldIn;
//		ts->fraction_changed.route+=S_Route(gs, fieldIn);
		S_Route(this, getFieldIndex(&touchTime), gs, gs->size()-1);
	}
	n++;
}
void set(ProtoInstance* protoInstance, const int destIdx, const Field& srcField){
	const_cast<M_Route&>(protoInstance->getField(destIdx)->getRoute())=srcField.getRoute();
	protoInstance->getField(destIdx)->set(&srcField);
}
M_Node g_holdNodes;
M_Node g_tsMorpherList;

bool notAllZero(M_Vec3f& keyValue, int idx, int length){
	for(int i = length; --i>=0;){
		if(keyValue[idx * length + i] != Vec3f()){
			return true;
		}
	}
	return false;
}

void addDisplacer(CPMorpher* morpher, ScalarInterpolator* scalarInterpolator, int i, int fromCoord, HAnimJoint* joint, TimeSensor* ts){
	HAnimDisplacer* displacer = new HAnimDisplacer;
	scalarInterpolator->name += morpher->name;
	scalarInterpolator->name += i;
	displacer->Node::name = morpher->name;
	displacer->Node::name += i;
	displacer->name = displacer->Node::name;
	const M_Int& cpIndex = morpher->cpIndex.refValue();
	const M_Vec3f& keyValue = morpher->keyValue.refValue();
	M_Int& coordIndex = displacer->coordIndex.refValue();
	M_Vec3f& displacements = displacer->displacements.refValue();
	for(int j = 0; j < cpIndex.length; j++){
		displacements += keyValue[i * cpIndex.length + j];
		coordIndex += fromCoord + cpIndex[j];
	}
	joint->displacers += displacer;
	S_Route(ts, ts->getFieldIndex(&ts->fraction_changed), scalarInterpolator, scalarInterpolator->getFieldIndex(&scalarInterpolator->set_fraction));
	S_Route(scalarInterpolator, scalarInterpolator->getFieldIndex(&scalarInterpolator->value_changed), displacer, displacer->getFieldIndex(&displacer->weight));
}

void addHanimDisplacers(M_Node& children){
	for(int k = 0; k < g_tsMorpherList.length;){
		if(TimeSensor* ts = dynamic_cast<TimeSensor*>(g_tsMorpherList[k++].p)){
			if(Art* part = dynamic_cast<Art*>(g_tsMorpherList[k++].p)){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(g_tsMorpherList[k++].p)){
					Seamless* smls = part->smls;
					smls->setCoordFroms();
					int fromCoord = part->fromCoord;
					if(HAnimHumanoid* hanim = dynamic_cast<HAnimHumanoid*>(smls->outputNode.p)){
						HAnimJoint* joint = dynamic_cast<HAnimJoint*>(hanim->joints[part->plidx].p);
						
						ScalarInterpolator* scalarInterpolator = 0;

						//int i = 1;

						//if(notAllZero(morpher->keyValue.refValue(), 0, morpher->cpIndex.length)){
						//	scalarInterpolator = new ScalarInterpolator;
						//	children+=scalarInterpolator;
						//	scalarInterpolator->name = "si_";

						//	scalarInterpolator->key.refValue() += morpher->key[0];
						//	scalarInterpolator->key.refValue() += morpher->key[1];
						//	scalarInterpolator->keyValue.refValue() += 1;
						//	scalarInterpolator->keyValue.refValue() += 0;

						//	if(morpher->key.length > 2){
						//		scalarInterpolator->key.refValue() += 1;
						//		scalarInterpolator->keyValue.refValue() += 0;
						//	}
						//	addDisplacer(morpher, scalarInterpolator, 0, fromCoord, joint, ts);
						//}

						//if(notAllZero(morpher->keyValue.refValue(), 1, morpher->cpIndex.length)){
						//	i = 0;
						//}
						for(int i = 0; i < morpher->key.length; i++){
							if(notAllZero(morpher->keyValue.refValue(), i, morpher->cpIndex.length)){
								scalarInterpolator = new ScalarInterpolator;
								children+=scalarInterpolator;
								scalarInterpolator->name = "si_";
								if(i > 0){
									scalarInterpolator->key.refValue() += 0;
									scalarInterpolator->keyValue.refValue() += 0;
								}
								if(i > 1){
									scalarInterpolator->key.refValue() += morpher->key[i - 1];
									scalarInterpolator->keyValue.refValue() += 0;
								}
								scalarInterpolator->key.refValue() += morpher->key[i];
								scalarInterpolator->keyValue.refValue() += 1;

								if(morpher->key.length > i + 1){
									scalarInterpolator->key.refValue() += morpher->key[i + 1];
									scalarInterpolator->keyValue.refValue() += 0;
								}

								if(morpher->key.length > i + 2){
									scalarInterpolator->key.refValue() += 1;
									scalarInterpolator->keyValue.refValue() += 0;
								}

								addDisplacer(morpher, scalarInterpolator, i, fromCoord, joint, ts);
							}
						}
					}
				}
			}
		}
	}
}

void convertPartToHAnimJoint(Art* parentPart, HAnimJoint* parentJoint, M_Node& joints){
	M_SArt& partField=parentPart->part.refValue();
	for(int k=0; k<partField.length;k++){
		Art* part=partField[k];
		if(part->show==false&&part->relevantPart.length==1)continue;
		HAnimJoint* joint= new HAnimJoint;
		joint->name=joint->Node::name=part->name;
		joint->rotation=part->rotation;
		parentJoint->children.refValue()+=joint;
		joints+=joint;
		convertPartToHAnimJoint(part, joint, joints);
	}
}
void Seamless::addNewHAnimNode(){
	HAnimHumanoid* humanoid=new HAnimHumanoid;
	outputNode=humanoid;
	M_Node& joints=humanoid->joints.refValue();
	M_Node& hskeleton=humanoid->skeleton.refValue();
	M_SArt& skeleton=this->skeleton.refValue();
	for(int k=0; k<skeleton.length;k++){
		Art* part=skeleton[k];
		if(part->show==false&&part->relevantPart.length==1)continue;
		HAnimJoint* joint= new HAnimJoint;
		joint->name=joint->Node::name=part->name;
		
		joint->rotation=part->rotation;
		hskeleton+=joint;;
		joints+=joint;
		convertPartToHAnimJoint(part, joint, joints);
	}
}

void Function::deleteControlPointFromAllMorphers(Art* part, int idx){
	W_Node* w_morpher = 0;
	W_Node* w_part = 0;

	M_Node& m_codeLine = this->codeLine;

	for(int k = m_codeLine.length; --k>=0;){
		if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, k)){
			if(w_part->node == part){
				if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
					M_Int& cpIndex = morpher->cpIndex.refValue();
					M_Float& key = morpher->key.refValue();
					M_Vec3f& keyValue = morpher->keyValue.refValue();

					for(int channel = cpIndex.length; --channel>=0;){
						if(cpIndex[channel] == idx){
							morpher->deleteChannel(channel, cpIndex, key, keyValue, cpIndex.length);
						}
					}
				}
			}
		}
	}
}

void deleteControlPointFromAllMorphers(Art* part, int i){
	for(int k=0; k<g_functionList.length; k++){
		g_functionList[k]->deleteControlPointFromAllMorphers(part, i);
	}
}

void Anim::processForOutput(M_Node& parentChild, int& n, Script* gs){
	M_Node temp;
	if(g_exportVrmlX3d->scriptAsRoutes){
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		if(MemberFunction* onFrame=dynamic_cast<MemberFunction*>(this->onFrame.p)){
			M_Word& m_word=onFrame->m_word;

			M_Node& m_codeLine = onFrame->codeLine;

			TimeSensor* ts = new TimeSensor;
			if(name!=""){
				ts->name=name;
			}
			else{
				ts->name="t";
				makeNodeNameFromAddr(ts->name,ts);
			}
			ts->loop=loop;
			ts->cycleInterval=period;

			temp+=ts;

			for(int k = 0; k<children.length; k++){
				Node* node = children[k];
				if(dynamic_cast<CPMorpher*>(node)){
					continue;
				}
				temp+=node;
			}


			for(int k = m_codeLine.length; --k>=0;){
				if(getMorphFunction(m_word, w_morpher, w_part, m_codeLine, k)){
					if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_morpher->node)){
						g_tsMorpherList += ts;
						g_tsMorpherList += w_part->node;
						g_tsMorpherList += morpher;
					}
				}
				else{
					CodeLine* codeLine = dynamic_cast<CodeLine*>(m_codeLine[k].p);
					for(int w=codeLine->wordFrom; w<codeLine->wordTo; w++){
						if(dynamic_cast<W_NoOutputNode*>(m_word[w])){
							if(dynamic_cast<W_NoOutputDot*>(m_word[++w])){
								if(W_Field* toField = dynamic_cast<W_Field*>(m_word[++w])){
									Node* toNode = toField->node;
									if(dynamic_cast<W_Assignment*>(m_word[++w])){
										if(dynamic_cast<W_NoOutputNode*>(m_word[++w])){
											if(dynamic_cast<W_NoOutputDot*>(m_word[++w])){
												if(W_Field* fromField = dynamic_cast<W_Field*>(m_word[++w])){
													Node* fromNode = fromField->node;
													S_Route(fromNode, fromField->fieldIdx, toNode, toField->fieldIdx);
												}
											}
										}
										else if(dynamic_cast<W_Local*>(m_word[w])){
											S_Route(ts, ts->getFieldIndex(&ts->fraction_changed), toNode, toField->fieldIdx);
										}
									}
								}
							}
						}
					}
				}
			}
		}
		insertElements<NodePtr>(n, 1, parentChild, temp);
		n+=temp.length;
	}
	else{
		g_holdNodes+=this;
		ProtoInstance* protoInstance=0;
		ProtoInterface* protoInterface=0;

		for(int k=g_userProtoBegin; k<nodeLookUpStr.length; k++){
			if(nodeLookUpStr[k]=="PAnim"){;
				protoInstance=new ProtoInstance(k);
				protoInterface=g_lookupProtoInterface[k];
				break;
			}
		}

		temp+=protoInstance;

		if(name==""){
			protoInstance->name="t";
			makeNodeNameFromAddr(protoInstance->name, protoInstance);
		}
		else{
			protoInstance->name=name;
		}
		if(MemberFunction* onFrame=dynamic_cast<MemberFunction*>(this->onFrame.p)){
			onFrame->name="f";
			makeNodeNameFromAddr(onFrame->name, onFrame);
			onFrame->functionName=onFrame->name;
			slider = slider * speed;

			gs->function+=onFrame;
			gs->name='s';
			makeNodeNameFromAddr(gs->name, gs);
			Properties onFrameProp = Properties(onFrame->functionName);
			onFrameProp.unreadable();
			gs->properties+=Properties(onFrameProp);
			Field*  fieldIn = new SFFloat(0);
			*gs+=fieldIn;
			S_Route(this, this->getFieldIndex(&slider), gs, gs->size()-1);
		}
		for(int k=0; k<children.length; ){
			children[k]->processForOutput(children, k, gs);
		}
		set(protoInstance, 0, play);
		set(protoInstance, 1, pause);
		set(protoInstance, 2, period);
		set(protoInstance, 3, slider);
		set(protoInstance, 4, loop);
		set(protoInstance, 5, children);
		insertElements<NodePtr>(n, 1, parentChild, temp);
		n++;
	}
}
void Mobility::processForOutput(M_Node& parentChild, int& n, Script* gs){
	g_holdNodes+=this;
	M_Node temp;

	M_Properties& m_prop = g_gestureProtoInterface->properties;
	Transform* walkTransform = dynamic_cast<Transform*>(g_gestureProtoBody->children.refValue()[0].p);

	if(MemberFunction* onRot=dynamic_cast<MemberFunction*>(this->onRot.p)){
		onRot->name="f";
		makeNodeNameFromAddr(onRot->name, onRot);
		onRot->functionName=onRot->name;

		gs->function+=onRot;
		gs->name='s';
		makeNodeNameFromAddr(gs->name, gs);
		Properties onRotProp = Properties(onRot->functionName);
		onRotProp.set_unreadable(true);
		int onRotIdx = gs->properties.length;
		gs->properties+=Properties(onRotProp);
		Field*  fieldIn = new SFRotation(Rotation());
		fieldIn->isRef()=true;
		*gs+=fieldIn;
		for(int k=m_prop.length; --k>=0;){
			if(m_prop[k].name=="rotation"){
				Field* field = g_gestureProtoInterface->getField(k);
				S_Route(g_gestureProtoInterface, k, gs, onRotIdx);
				break;
			}
		}
	}
	if(MemberFunction* onTran=dynamic_cast<MemberFunction*>(this->onTran.p)){
		onTran->name="f";
		makeNodeNameFromAddr(onTran->name, onTran);
		onTran->functionName=onTran->name;

		gs->function+=onTran;
		gs->name='s';
		makeNodeNameFromAddr(gs->name, gs);
		Properties onTranProp = Properties(onTran->functionName);
		onTranProp.set_unreadable(true);
		int onTranIdx = gs->properties.length;
		gs->properties+=Properties(onTranProp);
		Field*  fieldIn = new SFVec3f(Vec3f());
		fieldIn->isRef()=true;
		*gs+=fieldIn;
		for(int k=m_prop.length; --k>=0;){
			if(m_prop[k].name=="set_translation"){
				Field* field = g_gestureProtoInterface->getField(k);
				S_Route(g_gestureProtoInterface, k, gs, onTranIdx);
				break;
			}
		}
	}
	//for(int k=0; k<children.length; ){
	//	Node* node = children[k];
	//	temp+=node;
	//	node->processForOutput(children, k, gs);
	//}
	/*
	for(int i=0; i<protoInterface->properties.length; i++){
		if(protoInterface->properties[i].name=="children"){
			Field* fieldPtr=&children;
			protoInstance->getField(i)->set(fieldPtr);
			//M_Node& instanceChildren=dynamic_cast<MFNode*>(protoInstance->getField(i))->refValue();
			//instanceChildren=children;
		}
	}
	*/
	insertElements<NodePtr>(parentChild.length, 0, parentChild, children);
	n++;
//	parentChild.deleteElement(n);
}
void replaceNodeRefInNode(Node* parent, NodePtr from, Node* to){
	for(int i=parent->getFLen(); --i>=0;){
		Field* field=parent->getField(i);
		if(SFNode* sfnode=dynamic_cast<SFNode*>(field)){
			if(sfnode->p==from){
				*sfnode = NodePtr(to);
			}
		}
		if(MFNode* mfnode=dynamic_cast<MFNode*>(field)){
			M_Node& m_node=*mfnode;
			for(int k = m_node.length; --k>=0;){
				if(m_node[k]==from){
					m_node[k]=to;
				}
			}
			*mfnode = *mfnode;
		}
	}
}
int matchFieldIndex(int fieldIdx, Node* from, Node* to){
	const String& name=from->getProperties()[fieldIdx].name;
	const M_Properties& m_prop = to->getProperties();
	for(int i = m_prop.length; --i>=0;){
		if(m_prop[i].name==name){
			return i;
		}
	}
	dLog("matchFieldIndex not found");
	throw;
}
void Function::replaceNodeRef(NodePtr from, Node* to){
	for(int w = m_word.length; --w>=0;){
		Word* wrd = m_word[w];
		if(W_Node* w_node=dynamic_cast<W_Node*>(wrd)){
			if(w_node->node==from){
				w_node->node=to;
			}
		}
		else if(W_Field* w_field=dynamic_cast<W_Field*>(wrd)){
			if(w_field->node==from){
				w_field->fieldIdx = matchFieldIndex(w_field->fieldIdx, from, to);
				w_field->node = to;
			}
		}
	}
}
void Function::replaceNodeFieldRef(NodePtr from, Node* to, Field* fieldTo){
	for(int w = m_word.length; --w>=0;){
		Word* wrd = m_word[w];
		if(W_Field* w_field=dynamic_cast<W_Field*>(wrd)){
			if(w_field->node==from){
				w_field->node=to;
				w_field->fieldIdx=to->getFieldIndex(fieldTo);
			}
		}
	}
}
void replaceNodeFieldRef(NodePtr from, Node* to, Field* fieldTo){
	for(int k=0; k<g_functionList.length; k++){
		g_functionList[k]->replaceNodeFieldRef(from, to, fieldTo);
	}
}

void Node::replaceNoderefForRouteInSceneTree(NodePtr from, Node* to){
	for(int k = getFLen(); --k >= 0;){
		M_Route& m_route=getField(k)->refRoute();
		for(int i=m_route.length; --i>=0;){
			S_Route& s_route=m_route[i];
			if(s_route.toNode==from){
				s_route.toNode=to;
				s_route.toFieldPtr=to->getField(s_route.toFieldIdx);
			}
		}
	}
}
void replaceNodeRefInScripts(NodePtr from, Node* to){
	for(int k=g_functionList.length; --k>=0;){
		g_functionList[k]->replaceNodeRef(from, to);
	}
}
void replaceNodeRefInSceneTree(NodePtr from, Node* to){
	for(int k=g_sceneTreeList.length; --k>=0;){
		NodePtr node=g_sceneTreeList[k].node;
		node->replaceNoderefForRouteInSceneTree(from, to);
		if(node==from){
			replaceNodeRefInNode(g_sceneTreeList[g_sceneTreeList[k].parent].node, from, to);
		}
	}
	replaceNodeRefInScripts(from, to);
}
void RotAccelerator::processForOutput(M_Node& parentChild, int& n, Script* gs){
	NodePtr hold=this;
	M_Node temp;
	ProtoInstance* protoInstance=0;
	ProtoInterface* protoInterface=0;

	for(int k=g_userProtoBegin; k<nodeLookUpStr.length; k++){
		if(nodeLookUpStr[k]=="PRotAccelerator"){;
			protoInstance=new ProtoInstance(k);
			protoInterface=g_lookupProtoInterface[k];
			break;
		}
	}

	temp+=protoInstance;

	if(name==""){
		protoInstance->name="t";
		makeNodeNameFromAddr(protoInstance->name, protoInstance);
	}
	else{
		protoInstance->name=name;
	}
	for(int k = protoInstance->getFLen(); --k>=0;){
		set(protoInstance, k, *((*this)[k]));
	}
	replaceNodeRefInSceneTree(this, protoInstance);
	insertElements<NodePtr>(n, 1, parentChild, temp);
	n++;
}
/*
void Anim::processForOutput(M_Node& parentChild, int& n, Script* gs){
//	M_Node temp;
	if(MemberFunction* onFrame=dynamic_cast<MemberFunction*>(this->onFrame.p)){
		if(name==""){
			name="t";
			makeNodeNameFromAddr(name,this);
		}
		onFrame->name="f";
		makeNodeNameFromAddr(onFrame->name, onFrame);
		onFrame->functionName=onFrame->name;
		slider=slider*speed;

		gs->function+=onFrame;
		gs->name='s';
		makeNodeNameFromAddr(gs->name, gs);
		gs->properties+=Properties(onFrame->functionName);
		Field*  fieldIn = new SFFloat(0);
		*gs+=fieldIn;
		S_Route(this, this->getFieldIndex(&slider), gs, gs->size()-1);

	//	temp+=children.refValue();

	//	TimeSensor* ts = new TimeSensor;
	//	if(name!=""){
	//		ts->name=name;
	//	}
	//	else{
	//		ts->name="t";
	//		makeNodeNameFromAddr(ts->name,ts);
	//	}
	//	ts->loop=loop;
	//	ts->cycleInterval=period;

	//	temp+=ts;

	//	onFrame->name="f";
	//	makeNodeNameFromAddr(onFrame->name, onFrame);
	//	onFrame->functionName=onFrame->name;

	//	gs->function+=onFrame;
	//	gs->name='s';
	//	makeNodeNameFromAddr(gs->name, gs);
	//	gs->properties+=Properties(onFrame->functionName);
	//	Field*  fieldIn = new SFFloat(0);
	//	*gs+=fieldIn;
	//	S_Route(ts, ts->getFieldIndex(&ts->fraction_changed), gs, gs->size()-1);

	//	temp+=children.refValue();
	}
	//insertElements<NodePtr>(n, 1, parentChild, temp);
	n++;
}
*/
void outputVrmlMorph2(Seamless* smls, Anim* anim);


void outputVrmlMorph(){
	Anim* anim = Window::animMode();

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(anim->onFrame.p)){

		M_Node& m_codeLine = function->codeLine;
		M_Word& m_word=function->m_word;
		for(int k = 0; k < m_codeLine.length; k++){
			CodeLine* codeLine = dynamic_cast<CodeLine*>(m_codeLine[k].p);
			for(int w = codeLine->wordFrom; w < codeLine->wordTo; w++){
				Word* wrd = m_word[w];
				if(W_Node* w_node = dynamic_cast<W_Node*>(wrd)){
					if(Art* part = dynamic_cast<Art*>(w_node->node)){
						outputVrmlMorph2(part->smls, anim);
						return;
					}
				}
			}
		}
	}
}

void ModelMorpher::processForOutput(M_Node& parentChild, int& n, Script* gs){
	M_Node temp;
	M_Node& sequence = this->sequence;
	Coordinate* firstCoord = 0;
	for(int s = 0; s < sequence.length; s++){
		Ptr<ModelSequence> modelSequence = dynamic_cast<ModelSequence*>(sequence[s].p);
		M_Node& model = modelSequence->model;
		Ptr<CoordinateInterpolator> interpolator = new CoordinateInterpolator;
		interpolator->name = modelSequence->name;
		interpolator->key.refValue()=modelSequence->key;
		bool remGenNormals = g_genNormals;
		g_genNormals=false;
		for(int k = 0; k < model.length; k++){
			Seamless* smls = dynamicCast<Seamless*>(model[k]);
			String str;
			smls->outputToString(str, false);
			createSmlsFromString(temp, str);
			ShapePtr shape = dynamic_cast<Shape*>(temp[temp.length-1].p);
			IndexedFaceSet* ifs = dynamic_cast<IndexedFaceSet*>(shape->geometry.p);
			Coordinate* coord = dynamic_cast<Coordinate*>(ifs->coord.p);
			interpolator->keyValue.refValue()+=coord->point;
			if(k == 0){
				if(s==0){
					temp+=interpolator.p;
					firstCoord = coord;
				}
				else{
					temp[temp.length-1]=interpolator;
				}
				S_Route(interpolator, interpolator->getFieldIndex(&interpolator->value_changed), firstCoord, firstCoord->getFieldIndex(&firstCoord->point));
				replaceNodeFieldRef(modelSequence.p, interpolator, &interpolator->set_fraction);
			}
			else{
				temp.length = s + 2;
			}
		}
		g_genNormals = remGenNormals;
	}
	insertElements<NodePtr>(n, 1, parentChild, temp);
	n++;
}

String convertUnderbarToSpace(String str){
	for(int i = str.length; --i>=0;){
		if(str[i] == '_'){
			str[i] = ' ';
		}
	}
	return str;
}
String convertSpaceToUnderbar(String str){
	for(int i = str.length; --i>=0;){
		if(str[i] == ' '){
			str[i] = '_';
		}
	}
	return str;
}

void GestureGroup::processForOutput(M_Node& parentChild, int& n, Script* gs){
	GestureGroup::addProto=true;
	NodePtr hold=this;
	g_gestureProtoDeclare->name="Avatar";
	g_gestureProtoDeclare->protoInterface=g_gestureProtoInterface.p;

	addNewProtoDeclareToList(g_gestureProtoDeclare);
	g_gestureProtoInstance=new ProtoInstance(g_lookupProtoInterface.length-1);


	g_gestureProtoDeclare->protoInterface=g_gestureProtoInterface.p;
	g_gestureProtoDeclare->protoBody=g_gestureProtoBody.p;
	M_Node temp;
	M_String gestureNames;

	M_Node& children =this->children;
	bool something = false;
	if(children.length){
		something=true;
		String fieldName;
		for(int k=0; k<children.length; ){
			int remK = k;
			Anim* anim = dynamic_cast<Anim*>(children[k].p);
			gestureNames += convertUnderbarToSpace(anim->name);
			anim->processForOutput(children, k, gs);
			Field* field=0;
			Node* timer=0; 
			timer = children[remK].p;
			const int startTimeIndex = g_exportVrmlX3d->scriptAsRoutes ? 3 : 6; //add a field to the proto
			field=timer->getField(startTimeIndex);
			SFDouble* protoField = new SFDouble(0);
			(*g_gestureProtoInterface)+=protoField;

			String fieldName="gesture";	//and the properties for the field
			fieldName+=GestureGroup::numOfGestures++ + 1;
			g_gestureProtoInterface->properties+=Properties(fieldName);

			field->connect(timer, g_gestureProtoInterface, protoField);
			S_Route(g_gestureProtoInterface, g_gestureProtoInterface->getFieldIndex(protoField), timer, startTimeIndex);

	
			/*
			if(MemberFunction* onFrame=dynamic_cast<MemberFunction*>(anim->onFrame.p)){
				TimeSensor* ts = new TimeSensor;
				ts->name="t";
				makeNodeNameFromAddr(ts->name,ts);
				ts->loop=anim->loop;
				ts->cycleInterval=anim->period;

				temp+=ts;
				SFDouble* field = new SFDouble(0);
				(*g_gestureProtoInterface)+=field;
				S_Route( g_gestureProtoInterface, g_gestureProtoInterface->getFieldIndex(field), ts, ts->getFieldIndex(&ts->startTime));
				ts->startTime.is=true;

				onFrame->name="f";
				makeNodeNameFromAddr(onFrame->name, onFrame);
				onFrame->functionName=onFrame->name;

				fieldName="gesture";
				fieldName+=GestureGroup::numOfGestures++ + 1;
				g_gestureProtoInterface->properties+=Properties(fieldName);

				gs->function+=onFrame;
				gs->name='s';
				makeNodeNameFromAddr(gs->name, gs);
				gs->properties+=Properties(onFrame->functionName);
				Field*  fieldIn = new SFFloat(0);
				*gs+=fieldIn;
				S_Route(ts, ts->getFieldIndex(&ts->fraction_changed), gs, gs->size()-1);

				temp+=anim->children.refValue();
			}
			*/
		}
		temp+=children;
	}
	if(mobility.length){
		something=true;
		Transform* walkTransform = new Transform;
		walkTransform->name=name;
		g_gestureProtoBody->children.refValue()+=walkTransform;

		SFRotation* protoRotField = new SFRotation(Rotation());
		(*g_gestureProtoInterface)+=protoRotField;
		Properties rotProp = Properties("rotation");
		rotProp.set_unreadable(true);
		g_gestureProtoInterface->properties+=rotProp;

		SFVec3f* protoTranField = new SFVec3f(Vec3f());
		(*g_gestureProtoInterface)+=protoTranField;
		Properties tranProp = Properties("set_translation");
		tranProp.set_unreadable(true);
		g_gestureProtoInterface->properties+=tranProp;

		String fieldName;
		for(int k=0; k<mobility.length; ){
			Mobility* mobility = dynamic_cast<Mobility*>(this->mobility[k].p);
			mobility->processForOutput(temp, k, gs);
			//ProtoInstance* protoInstance = dynamic_cast<ProtoInstance*>(children[k-1].p);
		}
	}
	if(something){
		(*g_gestureProtoInterface)+=new SFBool(true);
		g_gestureProtoInterface->properties+=Properties("isAvatar");

		(*g_gestureProtoInterface)+=new MFString(gestureNames);
		g_gestureProtoInterface->properties+=Properties("gestureNames");

		insertElements<NodePtr>(n, 1, parentChild, temp);
		n++;
	}
	else{
		parentChild.deleteElement(n);
	}
}
void AGroup::processForOutput(M_Node& parentChild, int& n, Script* gs){
	for(int k=0; k<children.length; ){
		children[k]->processForOutput(children, k, gs);
	}
	n++;
}
void ProtoDeclare::processForOutput(M_Node& parentChild, int& n, Script* gs){
	M_Node& children=protoBody->children;
	for(int k=0; k<children.length; ){
		children[k]->processForOutput(children, k, gs);
	}
	n++;
}
void Scene::processForOutput(M_Node& parentChild, int& n, Script* gs){
	g_gestureProtoInterface->resize(0);
	for(int k=0; k<children.length; ){
		children[k]->processForOutput(children, k, gs);
	}
}
Script* g_outputScript;
AnimPtr g_selAnim=new Anim;
static String g_protoStr;

Properties& getS_Prop(M_Properties& m_properties, const char* fieldName){
	for(int k=m_properties.length; --k>=0;){
		if(m_properties[k].name==fieldName){
			return m_properties[k];
		}
	}
	throw;
}
void createBatchFile(String& batchFileName);

extern M_<Ptr<Viewpoint> > g_viewpoints;
void toStringRoundDown(float number, int numOfDigits, String& str);

void writeOutputHeader(String& str,  const Properties* switchProp, const Properties* lodProp){
	if(g_exportVrmlX3d->x3dXml){
		str="<?xml version='1.0' encoding='UTF-8'?>\n<X3D version='3.1' profile='Immersive'>\n<head>\n";
		if(g_exportVrmlX3d->genHAnimNodes){
			str+="<component level='1' name='H-Anim'/>\n";
		}
		str+="<meta content='Seamless3d ";
		str+=g_version;
		str+=", http://www.seamless3d.com' name='generator'/>\n";
		str+="</head>\n<Scene>";
	}
	else if(g_exportVrmlX3d->x3dVrmlClassic){
		str="#X3D V3.0 utf8\nPROFILE Immersive\n";
		if(g_exportVrmlX3d->genHAnimNodes){
			str+="COMPONENT H-Anim:1\n";
		}
		str+="META \"generator\" \"Seamless3d";
		str+=g_version;
		str+=", http://www.seamless3d.com\"\n";
	}
	else{
		str="#VRML V2.0 utf8\n";
		str+="#generated by Seamless3d ";
		str+=g_version;
		str+=" www.seamless3d.com\n";
		if(switchProp){
			const_cast<Properties*>(switchProp)->name="choice";
			const_cast<Properties*>(lodProp)->name="level";
		}
	}
}

void ExportVrmlX3d::exportScene(){
	if(g_browseMode){
		return;
	}
	g_callBackList.length = 0; //this stops crashing at getHoverButton but why do we exactly need it?

//	g_gestureProtoInterface->resize(0);
//	g_gestureProtoInterface->properties.length=0;
	g_tsMorpherList.length = 0;
	g_defLookUp.length=0;
	static String str;
	static String openFile;
	str="";
	openFile="";
	g_protoStr="";
	g_vrmlSquareOpenBracket[0]='[';
	g_vrmlSquareCloseBracket[0]=']';
	g_vrmlQuote[0]='\"';

	//refValue()=par;
	String filename = g_currentFilename;
	if(false == modifyVrmlX3dFilename(filename, x3dXml, x3dVrmlClassic)){
		msg(msg_Can_not_create_output_file_name,OK_BUTTON | ICONEXCLAMATION);
		return;
	};

	if(gZip && (x3dXml || x3dVrmlClassic)){
		filename+='z';
	}
	g_gestureProtoDeclare=new ProtoDeclare;
	g_gestureProtoInterface=new ProtoInterface;
	g_gestureProtoBody=new ProtoBody;

	const Properties& switchProp=getS_Prop(g_lookupProtoInterface[SWITCH]->properties, "children");
	const Properties& lodProp=getS_Prop(g_lookupProtoInterface[LOD_TYPE]->properties, "children");
	const Properties& colorEffect_emissiveProp=getS_Prop(g_lookupProtoInterface[COLOREFFECT]->properties, "emissive");
	const Properties& colorEffect_diffuseProp=getS_Prop(g_lookupProtoInterface[COLOREFFECT]->properties, "diffuse");
	const Properties& colorEffect_specularProp=getS_Prop(g_lookupProtoInterface[COLOREFFECT]->properties, "specular");
	const Properties& colorEffect_ambientProp=getS_Prop(g_lookupProtoInterface[COLOREFFECT]->properties, "ambient");

	const Properties& textureEffect_emissiveProp=getS_Prop(g_lookupProtoInterface[TEXTUREEFFECT]->properties, "emissive");
	const Properties& textureEffect_diffuseProp=getS_Prop(g_lookupProtoInterface[TEXTUREEFFECT]->properties, "diffuse");
	const Properties& textureEffect_specularProp=getS_Prop(g_lookupProtoInterface[TEXTUREEFFECT]->properties, "specular");
	const Properties& textureEffect_ambientProp=getS_Prop(g_lookupProtoInterface[TEXTUREEFFECT]->properties, "ambient");

	writeOutputHeader(str,  &switchProp, &lodProp);

	const_cast<Properties&>(colorEffect_emissiveProp).name="emissiveColor";
	const_cast<Properties&>(colorEffect_diffuseProp).name="diffuseColor";
	const_cast<Properties&>(colorEffect_specularProp).name="specularColor";
	const_cast<Properties&>(colorEffect_ambientProp).name="ambientIntensity";

	const_cast<Properties&>(textureEffect_emissiveProp).name="emissiveColor";
	const_cast<Properties&>(textureEffect_diffuseProp).name="diffuseColor";
	const_cast<Properties&>(textureEffect_specularProp).name="specularColor";
	const_cast<Properties&>(textureEffect_ambientProp).name="ambientIntensity";

	bool newList=true;


	chdir(g_seamlessDir.c_str());

	M_Int lookUpExist;

	lookUpExist.length=0;
	lookUpExist.length=g_lookupProtoInterface.length;
	lookUpExist[CCPATH]=1;
	lookUpExist[ROTACCELERATOR]=1;
	lookUpExist[FLOATACCELERATOR]=1;
	lookUpExist[POSACCELERATOR]=1;
	lookUpExist[ROTFILTER]=1;
	lookUpExist[ROTMULT]=1;
	lookUpExist[ANIMBYWRIST]=1;
	lookUpExist[ANIMBYANKLE]=1;
	lookUpExist[ANIM]=1;

	for(int k=0; k<g_sceneTreeList.length; k++){
		if(lookUpExist[g_sceneTreeList[k].node->getType()]==1){
			lookUpExist[g_sceneTreeList[k].node->getType()]=2;
		}
	}
	if(lookUpExist[CCPATH]==2){
		g_protoStr+=strProtoDeclare_CCPath97;
	}
	if(lookUpExist[FLOATACCELERATOR]==2){
		g_protoStr+=strProtoDeclare_FloatAccelerator97;
	}
	if(lookUpExist[POSACCELERATOR]==2){
		g_protoStr+=strProtoDeclare_PosAccelerator97;
	}
	if(lookUpExist[ROTACCELERATOR]==2){
		g_protoStr+=strProto_RotAccelerator97;
	}
	if(scriptAsRoutes == false && lookUpExist[ANIM]==2){
		g_protoStr+=strProto_Anim97;
	}
	if(lookUpExist[ANIMBYANKLE]==2){
		if(g_genHAnimNodes){
			g_protoStr+=strProtoDeclare_AnimByAnkle97Hanim;
		}
		else{
			g_protoStr+=strProtoDeclare_AnimByAnkle97;
		}
	}
	if(lookUpExist[ROTFILTER]==2){
		g_protoStr+=strProtoDeclare_RotFilter97;
	}
	if(lookUpExist[ROTMULT]==2){
		g_protoStr+=strProtoDeclare_RotMult97;
	}


	
	int rem_g_protoListLength=g_protoList.length;
	int rem_g_lookupProtoInterfaceLength=g_lookupProtoInterface.length;
	int rem_nodeLookUpStrLength=nodeLookUpStr.length;
	int rem_containerFieldLookUpStrLength=containerFieldLookUpStr.length;


	M_<Ptr<Function> > storeFunctions = g_functionList;

	g_defLookUp.length=0;
	g_protoList=M_ProtoDeclare();

	g_scene->toString(g_loadStr="", false);
	listRoutes(g_loadStr);
	ScenePtr newScene = new Scene;
	initCompile();
	createSmlsFromString(newScene->children, g_loadStr);
	decodeFunctions(g_loadStr);
	newScene->genNodeList2();
	postLoadProcessNodes(g_sceneTreeList);


	{
		M_Node parentChild;
		int n=0;
		GestureGroup::numOfGestures=0;
		GestureGroup::addProto=false;
		g_outputScript=new Script;
		g_outputScript->directOutput=true;
//		gs->name="q";
//		makeNodeNameFromAddr(gs->name, gs);
		newScene->processForOutput(parentChild, n, g_outputScript);
		if(g_outputScript->getFLen() > 3){
			newScene->children += g_outputScript;
		}
	}
	if(GestureGroup::addProto){
		bool touchSensorExists = false;
		Group* group = new Group;
		M_Node protos;
		M_Node& protoBodyChildren=g_gestureProtoBody->children.refValue();
		M_Node& newSceneChildren=newScene->children.refValue();
		M_Node& groupChildren=group->children.refValue();
		for(int k = 0; k < newSceneChildren.length; k++){
			Node* node = newSceneChildren[k].p;
			if(dynamic_cast<ProtoDeclare*>(node)){
				protos+=node;
				continue;
			}
			if(dynamic_cast<TouchSensor*>(node)){
				touchSensorExists=true;
			}
			groupChildren+=node;
		}
		newSceneChildren.length=0;
		if(protoBodyChildren.length){
			newSceneChildren+=protos;
			M_Node& walkChildren = dynamic_cast<Transform*>(protoBodyChildren[0].p)->children.refValue();
			walkChildren+=groupChildren;
		}
		else if(touchSensorExists){
			newSceneChildren+=protos;
			protoBodyChildren+=group;
		}
		else{
			newSceneChildren+=protos;
			protoBodyChildren+=groupChildren;
		}
		newSceneChildren+=g_gestureProtoDeclare.p;
		newSceneChildren+=g_gestureProtoInstance.p;

//		M_Node& children=newScene->children;
///		g_gestureProtoBody->children=children;
//		children.length=0;
//		children+=g_gestureProtoDeclare.p;
//		children+=g_gestureProtoInstance.p;
	}
	g_defLookUp.length=0;
	g_protoList=M_ProtoDeclare();

	Function::readyForOutput=false;

	g_outputScriptStack.length++;
	g_outputScriptStackExist.length++;

	g_formatCoords = roundCoords;
	g_outputMode = true;

	g_writeX3DXML = x3dXml;
	g_writeX3DVRMLClassic = x3dVrmlClassic;
	g_genHAnimNodes = genHAnimNodes;
	g_genIdxTriSetNodes = genIdxTriSetNodes;
	g_genContactSBVT = genContactSBVT;
	g_genNormals = genNormals;
	if(genIdxTriSetNodes == false){
		g_genColorIndex = genColorIndex;
	}
/*
	if(protoStr!="" && g_scene->outputX3DXML){//convert vrml97 PROTO strings to xml format
		ScenePtr protoScene=new Scene;
		M_Node& children=protoScene->children;
		createSmlsFromString(protoScene->children, protoStr);
		g_defLookUp.length=0;
		protoStr="";
		for(int k=0;k<children.length;k++){
			children[k]->toXMLString(protoStr, true, "");
		}
	}
*/

	newScene->genNodeList2();
	convertSeamlessNodesForOutput();
	addHanimDisplacers(newScene->children.refValue());


	if(g_writeX3DVRMLClassic){
		g_exposedFieldPtr=g_inputOutput;
		g_eventInPtr=g_inputOnly;
		g_eventOutPtr=g_outputOnly;
		g_fieldPtr=g_initializeOnly;
	}

	//newList=true;
	g_defLookUp.length=0;
	if(g_writeX3DXML){
		g_vrmlSquareOpenBracket[0]=0;
		g_vrmlSquareCloseBracket[0]=0;
		g_vrmlQuote[0]=0;
		str+='\n';
		newScene->children.toXMLString(str, false, "children");
		g_vrmlSquareOpenBracket[0]='[';
		g_vrmlSquareCloseBracket[0]=']';
		g_vrmlQuote[0]='\"';
	}
	else{
		M_Node& children = newScene->children;
		for(int i=0;i<children.length;i++){
			str+='\n';
			children[i]->toString(str, false);
		}
	}
	outputUserScripts(str, newList);

	changeFloatFormat(g_formatCoords=g_settings->roundCoordsSave.getValue());

	listRoutes(str);
	if(g_writeX3DXML){
		str+="\n</Scene>\n</X3D>";
	}
	g_writeGzipped = gZip;
	str.write(filename);
	g_writeGzipped=g_settings->saveGZipped;
	g_protoList.length=rem_g_protoListLength;
	g_lookupProtoInterface.length=rem_g_lookupProtoInterfaceLength;
	nodeLookUpStr.length=rem_nodeLookUpStrLength;
	containerFieldLookUpStr.length=rem_containerFieldLookUpStrLength;
	g_exposedFieldPtr=g_exposedField;
	g_eventInPtr=g_eventIn;
	g_eventOutPtr=g_eventOut;
	g_fieldPtr=g_field;


	reBuildPartList();

	g_outputMode=false;
	g_writeX3DXML=false;
	g_writeX3DVRMLClassic=false;

	g_genHAnimNodes=false;
	g_genIdxTriSetNodes=false;
	g_genContactSBVT=true;
	g_genNormals=true;
	g_genColorIndex=false;

	const_cast<Properties&>(switchProp).name="children";
	const_cast<Properties&>(lodProp).name="children";
	const_cast<Properties&>(colorEffect_emissiveProp).name="emissive";
	const_cast<Properties&>(colorEffect_diffuseProp).name="diffuse";
	const_cast<Properties&>(colorEffect_specularProp).name="specular";
	const_cast<Properties&>(colorEffect_ambientProp).name="ambient";

	const_cast<Properties&>(textureEffect_emissiveProp).name="emissive";
	const_cast<Properties&>(textureEffect_diffuseProp).name="diffuse";
	const_cast<Properties&>(textureEffect_specularProp).name="specular";
	const_cast<Properties&>(textureEffect_ambientProp).name="ambient";

	g_functionList=storeFunctions;
	g_scene->genNodeList2();
	Function::selectedLine=g_scene;
	g_loadStr="";
	onlyFunctionsToString(g_loadStr);
	initCompileForBody();
	resetDefList();
	decodeFunctions(g_loadStr);
	g_scene->genNodeList2();
	Function::selectedLine=0;//restore(probably not needed)
	Function::reset_m_word=true;
	resetLooseEnds();
	g_holdNodes.length=0;


	String outputMsg;
	//outputMsg="File:\n";
	extractFileName(openFile, filename);

	outputMsg+=openFile;
	outputMsg+=" has been created.\n\nOpen ";
	outputMsg+=openFile;
	outputMsg+=" in html web browser?";


	unsigned question=msG(outputMsg,YESNO_BUTTON|ICONQUESTION);
	if(question==SMLS_IDYES){
		openWithHtmlBrowser(filename, g_settings->htmlBrowser.refValue());
	}
	
}
float screenRatio(){
	return static_cast<float>(g_3d->size.x) / static_cast<float>(g_3d->size.y);
}

void ExportMovie::exportScene(Anim* anim){
	if(g_browseMode){
		return;
	}
	g_callBackList.length = 0; //this stops crashing at getHoverButton but why do we exactly need it?

//	g_gestureProtoInterface->resize(0);
//	g_gestureProtoInterface->properties.length=0;
	g_tsMorpherList.length = 0;
	g_defLookUp.length=0;
	static String str;
	static String openFile;
	str="";
	openFile="";
	g_protoStr="";
	g_vrmlSquareOpenBracket[0]='[';
	g_vrmlSquareCloseBracket[0]=']';
	g_vrmlQuote[0]='\"';

	//refValue()=par;
	String filename = g_currentFilename;
	static String drive;
	static String dir;
	static String fname;
	static String ext;
	static String iniFileName;
	splitPath(filename, drive, dir, fname, ext);
	filename = drive + dir + fname;


	float time = 0;	///g_freezeTime->time;//needs to be fixed
	float multiply = 1;
	float period = 1;
	float step = 0;

	if(anim){
		multiply = anim->speed;
		period = anim->period;
		step = 1.0 / (anim->stepsPerSecond * period);
		anim->slider = 0;
	}
		

	int numOfFrames = 1;

	String outputMsg;
	//outputMsg="Files:\n\n";

	g_writeGzipped=false;


	if(anim){
		numOfFrames+=period*anim->stepsPerSecond;
		str="Initial_Frame=0\nFinal_Frame=";
		str+=numOfFrames-1;
		str+="\nInput_File_Name=";
		str+=fname;
		str+=".pov";
		str+="\nOutput_File_Type=N";

		iniFileName = filename;
		iniFileName+=".ini";
		str.write(iniFileName);

		extractFileName(outputMsg, iniFileName);
		outputMsg+="\n\n";

	}

	//////////// start inc files
	static M_<Effect*> effectList;
	effectList.length=0;
	float remTime=0; //freezeTime->time;
	if(anim){
		for(int i = 0; i < numOfFrames; i++){
			anim->slider = step * i * multiply + time;
			//SendMessage(hWndMain, WM_PAINT, 0, 0);
			renderScene();

			str="";
			effectList.length = 0;
			g_scene->outPovRay(str, effectList, Matrix(), Matrix());
			ext=filename;
			if(anim){
				ext+=i;
			}
			ext+=".inc";
			str.write(ext);

			if(i==0 && anim){
				extractFileName(outputMsg, ext);
				outputMsg+=" ...to ";
			}
		}
	}
	else{
		str="";
		g_scene->outPovRay(str, effectList, Matrix(), Matrix());
		ext = filename;
		ext += ".inc";
		str.write(ext);
	}
	extractFileName(outputMsg, ext);
	outputMsg+="\n\n";

	//g_freezeTime->time=remTime;
	///////////// end of inc files

		
	str="background { color ";
	g_scene->backgroundColor.refValue().toPOVRayString(str);
	str+="}\n";
	//str+="light_source {<0,0,-100> color rgb<1,1,1>}\n";

/*		for(int k=0; k<effectList.length; k++){
		Effect* effect=effectList[k];
		str+="#declare ";
		if(effect->name==""){
			str+='e';
			makeNodeNameFromAddr(str, effect);
		}
		else{
			str+=effect->name;
		}
		effect->outPovValues(str);
	}*/
	str+="union{\n\t#include ";
	if(anim){
		str+="concat(\"";
		str+=fname;
		str+="\", str(clock*";
		str+=numOfFrames-1;
		str+=",0,0), \".inc\")";
	}
	else{
		str+='\"';
		str+=fname;
		str+=".inc\"";
	}
	str+="\n\ttransform{";
	str+="\n\t\tscale<1,1,1>\n";
	str+="\n\t\trotate<0,0,0>\n";
	str+="\n\t\ttranslate<0,0,0>\n";
	str+="\n\t}\n}\n";

	str+="camera {";
	if(g_perspective){
		str+="perspective ";
	}
	else{
		str+="orthographic ";
	}

	//Viewpoint* viewpoint = g_viewpoints[0];
	str+=" location vlocation";

	//str+= -g_viewDist;
	//str+="> ";
	//g_settings->translation.toPOVRayString(str);
	str+=" look_at vlook_at sky vsky";
	if(g_perspective){
		str += " right<1,0,0> angle zangle";
		//str += " up<0,480/640,0> scale ";
		//str += " up<0,720/1280,0> scale ";
		str += " up<0,";
		str += static_cast<int>(g_3d->size.y);
		str += '/';
		str += static_cast<int>(g_3d->size.x);
		str += ",0> scale vscale";
			
	}
	else{
		str += " right<";
		str += static_cast<int>(g_3d->size.x);
		str += '/';
		str += static_cast<int>(g_3d->size.y);
			
		str += "*2,0,0>";
		//str += " up<0,480/640*4,0> scale ";
		//str += " up<0,720/1280*4,0> scale ";
		str += " up<0,2,0> scale vscale";
	}
	str += "}\n";
	str += "global_settings {max_trace_level 5}\n";


	ext=filename;
	ext+=".pov";
	str.write(ext);

	extractFileName(outputMsg, ext);

	if(anim){
		String batchFileName;
		createBatchFile(batchFileName);

		outputMsg+="\n\n";
		extractFileName(outputMsg, batchFileName);
	}


/////////////////
	outputMsg+="\n\nhave been created.\n\nOpen ";
	if(anim){
		//outputMsg+=iniFileName;
		extractFileName(outputMsg, iniFileName);
		outputMsg+=" in Pov-Ray?";
		unsigned question=msG(outputMsg,YESNO_BUTTON|ICONQUESTION);
		if(question==SMLS_IDYES){
			openWithPovRay(iniFileName);
		}
	}
	else{
		extractFileName(outputMsg, ext);
		outputMsg+=" in Pov-Ray?";
		unsigned question=msG(outputMsg,YESNO_BUTTON|ICONQUESTION);
		if(question==SMLS_IDYES){
			openWithPovRay(ext);
		}
	}
	g_writeGzipped = g_exportVrmlX3d->gZip;

}

void ExportVrmlX3d::Output::operator = (const bool& par){
	refValue()=false;
	home.exportScene();
}

void ExportMovie::Output::operator = (const bool& par){
	refValue()=false;
	home.exportScene();
}

void Scene::toString(String& str, bool newList){//only reason we need this is to insert protoStr for output
	if(g_browseMode && netSave == false){
		return;
	}
	g_defLookUp.length=0;
	newList=false;//tempory fix
	int i=0;
	const M_Properties &properties=getProperties();
	str+='\n';
	str+=getTypeStr();
	str+='{';

	extraToString(str);

	for(i=0; i < getFLen(); i++){
		const Properties& sprop = properties[i];
		if(sprop.doNotSave() || g_outputMode && sprop.doNotOutput()){
			continue;
		}
		Field* protoField=g_lookupProtoInterface[getType()]->getField(i);
		Field* field=getField(i);
		if(	(sprop.isReadable()^sprop.isWritable())==false){
			if(sprop.name=="children"){
				str+="\nchildren[\n";
				str+=g_protoStr;
				g_protoStr="";
				const M_Node& children=dynamic_cast<MFNode*>(field)->refValue();
				for(int k=0;k<children.length;k++){
					children[k]->toString(str,newList);
					str+="\n";
				}
				str+=']';
				continue;
			}

			String tmpStr;
			field->toString(tmpStr, protoField, newList);

			if(tmpStr!=""){
				str+='\n';
				str+=properties[i].name;
				str+=' ';
				str+=tmpStr;
			}
		}
	}
	listRoutes(str);
	str+="\n}";
}

Node* checkIsNamedNode(int k){
	Node* node = g_sceneTreeList[k].node;
	if(dynamic_cast<Function*>(node) == 0 && dynamic_cast<CodeLine*>(node) == 0 && dynamic_cast<Variable*>(node) == 0 && node->name!=""){
		return node;
	}
	return 0;
}

void Scene::outputNodeBind(){
	static String str;
	String filename = g_currentFilename;
	if(modifyFilenameForBind(filename) == false){
		msg(msg_Can_not_create_output_file_name,OK_BUTTON | ICONEXCLAMATION);
		return;
	};
	FILE* file = fopen(filename.c_str(), "rb");
	if(file){
		fclose(file);
		msg(filename + msg_cpp_file_already_exists,OK_BUTTON | ICONEXCLAMATION);
		return;
	}
	str="\
#include \"seamless3d.h\"\n\
//g_bindSmlsFile contains the URL to our .smls file we want to bind to\n\
String g_bindSmlsFile=\"\
";
	for(int i = 0; i < g_currentFilename.length; i++){
		char ch = g_currentFilename[i];
		str+=ch;
		if(ch == '\\'){
			str+='\\';
		}
	}
//"C:\\Documents and Settings\\thyme\\My Documents\\Content\\rotate_box_bind\\rotate_box_bind_no_script.smls";
	str+="\";\n\n//insert global variables here:\n\n\
class ScriptBind{//user definable class for binding nodes and containing script functions\n\
public:\n\
\t//define node references to match nodes that will be found in the .smls file\n";

	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Node* node = checkIsNamedNode(k)){
			str+='\t';
			str+=node->getTypeStr();
			str+="& ";
			str+=node->name;
			str+=";\n";
		}
	}
	str+="\n\tScriptBind(//bind the nodes from the smls file to the ScriptBind class\n";
	bool firstNode=true;
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Node* node = checkIsNamedNode(k)){
			if(firstNode==false){
				str+=",\n";
			}
			firstNode=false;
			str+="\t\t";
			str+=node->getTypeStr();
			str+="& i_";
			str+=node->name;
		}
	}
	str+="\n\t):\n";
	firstNode=true;
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Node* node = checkIsNamedNode(k)){
			if(firstNode==false){
				str+=",\n";
			}
			firstNode=false;
			str+="\t";
			str+=node->name;
			str+="(i_";
			str+=node->name;
			str+=")";
		}
	}
	str+="\n{\n\tinitialize();\n}\nvoid initialize(){//equivelent to initialize in vrml\n//insert initialize code here\n}\n\n\
void onFrame(float v){//simulate Anim's onFrame (can add break point here and single step to check a point in time for the animation)\n}\n\
//add user defined script functions here\n\
\n}; //end ScriptBind class\n\n";

str+="NodePtr getNodeFromName(const String str);\n\n\
ScriptBind* scriptBind=0;\n\
void initializeDeveloperScript(){\n\
\tscriptBind = new ScriptBind(//new instance of ScriptBind binds the nodes to our script\n";

	firstNode=true;
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Node* node = checkIsNamedNode(k)){
			if(firstNode==false){
				str+=",\n";
			}
			firstNode=false;
			str+="\t\t*dynamic_cast<";
			str+=node->getTypeStr();
			str+="*>(getNodeFromName(\"";
			str+=node->name;
			str+="\").p)";
		}
	}
	str+="\n\t);\n}\nvoid scriptBindAnimation(){\n\
\tfloat period = 1;\n\
\tscriptBind->onFrame(fmodf(g_time/period, period)); // create slider from period and pass as value\n}";


	bool storeState = g_writeGzipped;
	g_writeGzipped=false;
	str.write(filename);
	g_writeGzipped = storeState;
}

void togglePlaySound(bool state, short int* ibuf, int bufLen);
extern int g_counter;
void Anim::renderNodeFirst(){
	setMathHiPresision();

	if(play || update){
		if(update == false){
			Video* video = dynamic_cast<Video*>(this->video.p);
				
			if(video && video->audioTimeMode){
			}
			else{
				if(loop.getValue() == false && g_time >= startTime + period / speed){
					play.refValue() = false;
					slider.refValue() = 1;
					//if(Composition* composition = dynamic_cast<Composition*>(soundTrack.p)){
					//	togglePlaySound(false, composition->ibuf, composition->bufLen);
					//}
					updateDisplayFrom();
					houseKeepVisualClue();

				}//else otherwise same as loop
				else{
					slider.refValue() = fmod((g_time - startTime) * speed, period) / period;
				}
			}
		}
		Function* function=dynamic_cast<Function*>(onFrame.p);
		if(function){
			unsigned char* func=function->startCodePtr;
			float v=slider;
			unsigned localSize=function->localSize;
			
			__asm{
				push ebp
				push ebx
				mov esi,esp
				sub esp,0x400
				sub esi,0x300
				mov edi,func
				push v
				sub esp,localSize
				call edi
				add esp,0x400 + 4;
				add esp,localSize;
				pop ebx
				pop ebp
			}
		}
//		if(ControlPanel* controlPanel = dynamic_cast<ControlPanel*>(Window::activeWindow.getActiveWindow())){
//			if(dynamic_cast<Anim*>(controlPanel->node.p)){
////				controlPanel->paint();
//		}


		if(play == false){
			update = false;

			if(g_vcArtAnimBase->anim == this){
				g_vcArtAnimBase->resetRotationLever();
				g_vcArtAnimBase->showLever = true;
				g_stw->cp->paint();
			}
		}
	}
}
void TimeSensor::renderNode(Matrix parentMatrix, Vec3f parentColScale){
	setMathHiPresision();
	//||(loop.getValue()==false&&g_time>startTime+cycleInterval)
	if(enabled.getValue()==false)return;
	if(loop.getValue()==false && g_time>=startTime+cycleInterval){
		if(fraction_changed==1)return;
		fraction_changed.refValue()=1;
		fraction_changed.sendOutRoute(1);
	}//else otherwise same as loop
	else{
		fraction_changed.sendOutRoute(fmod(g_time-startTime,cycleInterval)/cycleInterval);
	}
	Function* function=dynamic_cast<Function*>(on_fraction_changed.p);
	if(function){
		unsigned char* func=function->startCodePtr;
		float v=fraction_changed;
		unsigned localSize=function->localSize;
		
		__asm{
			push ebp
			push ebx
			mov esi,esp
			sub esp,0x400
			sub esi,0x300
			mov edi,func
			push v
			push dword ptr [g_time + 4]
			push dword ptr [g_time]
			sub esp,localSize
			call edi
			add esp,0x400 + 4 + 8;
			add esp,localSize;
			pop ebx
			pop ebp
		}
	}
}
extern M_String g_shared_touchSensors;
inline bool checkSharedTouchSensor(const String& name){
	for(int i = g_shared_touchSensors.length; --i>=0;){
		if(g_shared_touchSensors[i] == name){
			return true;
		}
	}
	return false;
}
void TouchSensor::renderNode(Matrix parentMatrix, Vec3f parentColScale){
	setMathHiPresision();
	//||(loop.getValue()==false&&g_time>startTime+cycleInterval)
	if((mouse3dMode == false && hit == true) || checkSharedTouchSensor(name)){
		hit=false;
		Function* function=dynamic_cast<Function*>(onTouchTime.p);
		if(function){
			unsigned char* func=function->startCodePtr;
			unsigned localSize=function->localSize;
			
			__asm{
//				int 3
				push ebp
				push ebx
				mov esi,esp
				sub esp,0x400
				sub esi,0x300
				mov edi,func
				push dword ptr [g_time + 4]
				push dword ptr [g_time]
				push dword ptr [g_time + 4]
				push dword ptr [g_time]
				sub esp,localSize
				call edi
				add esp,0x400 + 8 + 8;
				add esp,localSize;
				pop ebx
				pop ebp
			}
		}
	}
}
void initialize();
//typedef void (Func)();
void initializeScript(){
	setMathHiPresision();
	int len=g_sceneTreeList.length;
	for(int k=0; k<len; k++){
		Function* function=dynamic_cast<Function*>(g_sceneTreeList[k].node.p);
		if(function && function->functionName=="initialize"){
			//function->doFunction();
			unsigned char* func=function->startCodePtr;
			__asm{
				push ebp
				push ebx
				mov esi,esp
				sub esp,0x400
				sub esi,0x300
				mov edi,func
				call edi
				add esp,0x400
				pop ebx
				pop ebp
			}
		}
	}
//	initialize();
}

void Build::doSetup(){
	if(Function* function=dynamic_cast<Function*>(setup.p)){
		unsigned char* func=function->startCodePtr;
		if(func==0){
			return;
		}
		__asm{
			push ebp
			push ebx
			mov esi,esp
			sub esp,0x400
			sub esi,0x300
			mov edi,func
			call edi
			add esp,0x400
			pop ebx
			pop ebp
		}
	}
}
void createFunctionListFromSceneTreeList(){
	g_functionList.length=0;
	for(int k = 0; k<g_sceneTreeList.length; k++){
		if(Function* f=dynamic_cast<Function*>(g_sceneTreeList[k].node.p)){
			g_functionList+=f;
		}
	}
}
unsigned char tempFunction = 0xc3; //return

Node* Anim::OnFrame::operator =(Node* par){
	if(Function* f=dynamic_cast<Function*>(par)){
		f->functionName="onFrame";
		f->returnType=_VOID;
		f->addNewLocal<S_Float,Single>(String("v"));
		f->paramLen=1;
		f->startCodePtr=&tempFunction;
		home.onFrame.refValue()=par;
		createFunctionListFromSceneTreeList();
	}
	else{
		refValue()=0;
	}
	return par;
}
Node* TimeSensor::OnFractionChanged::operator =(Node* par){
	if(Function* f=dynamic_cast<Function*>(par)){
		f->functionName="on_fraction_changed";
		f->returnType=_VOID;
		f->addNewLocal<S_Float,Single>(String("v"));
		f->addNewLocal<double,Single>(String("t"));
		f->paramLen=2;
		f->startCodePtr=&tempFunction;
		home.on_fraction_changed.refValue()=par;
		createFunctionListFromSceneTreeList();
	}
	else{
		refValue()=0;
	}
	return par;
}
Node* TouchSensor::OnTouchTime::operator =(Node* par){
	if(Function* f=dynamic_cast<Function*>(par)){
		f->functionName="onTouchTime";
		f->returnType=_VOID;
		//f->addNewLocal<double,Single>(String("v"));
		//f->addNewLocal<double,Single>(String("t"));
		//f->paramLen=2;
		f->startCodePtr=&tempFunction;
		home.onTouchTime.refValue()=par;
		createFunctionListFromSceneTreeList();
	}
	else{
		refValue()=0;
	}
	return par;
}
Node* Mobility::OnRot::operator =(Node* par){
	if(Function* f=dynamic_cast<Function*>(par)){
		if(home.onRot.refValue()==0){
			f->functionName="onRot";
			f->returnType=_VOID;
			f->addNewLocal<Rotation,Single>(String("v"));
			f->startCodePtr=&tempFunction;
			f->paramLen=1;
			home.onRot.refValue()=f;
			createFunctionListFromSceneTreeList();
		}
		if(home.onTran.refValue()==0){
			MemberFunction* f = new MemberFunction;
			f->functionName="onTran";
			f->returnType=_VOID;
			f->addNewLocal<Vec3f,Single>(String("v"));
			f->startCodePtr=&tempFunction;
			f->paramLen=1;
			home.onTran.refValue()=f;
			createFunctionListFromSceneTreeList();
		}
	}
	else{
		refValue()=0;
	}
	return par;
}
Node* Mobility::OnTran::operator =(Node* par){
	if(Function* f=dynamic_cast<Function*>(par)){
		f->functionName="onTran";
		f->returnType=_VOID;
		f->startCodePtr=&tempFunction;
		home.onTran.refValue()=par;
		createFunctionListFromSceneTreeList();
	}
	else{
		refValue()=0;
	}
	return par;
}
Node* Setup::operator =(Node* par){
	if(Function* f=dynamic_cast<Function*>(par)){
		f->functionName="setup";
		f->returnType=_VOID;
		f->startCodePtr=&tempFunction;
		home.setup.refValue()=par;
		createFunctionListFromSceneTreeList();
	}
	else{
		refValue()=0;
	}
	return par;
}
void Node::getSceneDisplayString(String& str){
	if(CodeLine* codeLine=dynamic_cast<CodeLine*>(this)){
		Function* f = codeLine->getParentFunction();
		if(f){
			M_Word& m_word = f->m_word;
			for(int w=codeLine->wordFrom; w < codeLine->wordTo; w++){
				Word* wrd = m_word[w];
				wrd->toString(str);
			}
		}
	}
	else if(Function* f=dynamic_cast<Function*>(this)){
		f->headerToString(str);
	}
	else if(G_Variable* gv=dynamic_cast<G_Variable*>(this)){
		M_Word& m_word = gv->m_word;
		for(int w=0; w < m_word.length; w++){
			Word* wrd = m_word[w];
			wrd->toString(str);
		}
	}
	else{
		str+=name;
	}
}
Node* searchForMatchingName(String& str);

void changeSrcPointersToExistingNames(NodePtr& ptr){
	if(ptr==0)return;

	if(ptr->name!=""){
		Node* s=searchForMatchingName(ptr->name);
		if(s){
			ptr=s;
			return;		//return because logically it must be a USE
		}
	}
	for(int i = 0; i < ptr->getFLen(); i++){
		Field* field=ptr->getField(i);
		
		field->isRef()=false;

		{
			M_Route& m_route=field->refRoute();
			for(int i=0; i<m_route.length; i++){
				S_Route& s_route=m_route[i];
				Node* s=searchForMatchingName(s_route.toNode->name);
				if(s){
					s_route.toFieldPtr=s->getField(s_route.toFieldIdx);
					s_route.toNode=s;
				}
			}
		}
		if(field->getType()==SFNODE){
			changeSrcPointersToExistingNames(dynamic_cast<SFNode*>(field)->refValue());
		}
		else if(field->getType()==MFNODE){
			M_Node& mnode=*dynamic_cast<MFNode*>(field);
			for(int k=mnode.length; --k>=0;){
				changeSrcPointersToExistingNames(mnode[k]);
			}
		}
	}
}

void importAnimationNodesLoop(M_Node& rawLoad, M_Node& animation){
	for(int i=0; i<rawLoad.length; i++){
		Node* node=rawLoad[i];
		if(dynamic_cast<Anim*>(node)){
			animation += rawLoad[i];
			rawLoad[i] = new NullNode;
		}
		if(ProtoDeclare* protoDeclare = dynamic_cast<ProtoDeclare*>(node)){
			node = protoDeclare->protoBody;
		}
		if(Progenitor* group=dynamic_cast<Progenitor*>(node)){
			importAnimationNodesLoop(group->children, animation);
		}
	}
}

bool findNonExistentNodeRefInLine(CodeLine* codeLine, M_Word& m_word){
	for(int w = codeLine->wordFrom; w < codeLine->wordTo; w++){
		if(W_Field* w_field = dynamic_cast<W_Field*>(m_word[w])){
			if(nodeExist(w_field->node)==false){
				return true;
			}
		}
		else if(W_Node* w_node = dynamic_cast<W_Node*>(m_word[w])){
			if(nodeExist(w_node->node)==false){
				return true;
			}
		}
	}
	return false;
}

void deleteAllCodeLinesThatRefNonExistentNodes(){
	for(int k=g_sceneTreeList.length; --k>=0;){
		if(CodeLine* codeLine=dynamic_cast<CodeLine*>(g_sceneTreeList[k].node.p)){
			Function* function = codeLine->getParentFunction();
			if(findNonExistentNodeRefInLine(codeLine, function->m_word)){
				codeLine->wordFrom=codeLine->wordTo;
			}
		}
	}
}

void resetCPNodeList();

void finishImportNodesFromFile(){
	String& str=g_loadStr;

	g_scene->genNodeList2();

	deleteAllRoutesToNonExistant();
	deleteAllCodeLinesThatRefNonExistentNodes();
	resetCPNodeList();

	g_functionList.length=0;
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(Function* function=dynamic_cast<Function*>(g_sceneTreeList[k].node.p)){
			g_functionList+=function;
		}
	}

/////////////////

	Function::selectedLine=g_scene;//to prevent new nodes being created, acts like a flag thats all
	Function::reset_m_word=false;

	str="";
	onlyFunctionsToString(str);
	initCompileForBody();

	resetDefList();
	decodeFunctions(str);
	Function::selectedLine=0;
	g_scene->genNodeList2();
	Function::selectedLine=0;//restore(probably not needed)
	Function::reset_m_word=true;

	resetLooseEnds();
	//Viewpoint::gotoFirstViewpoint();

	BackUp::monostable.beginCycle();
}
bool checkForMatchingName(String name, M_Node& list){
	for(int k=list.length; --k>=0; ){
		if(list[k]->name==name){
			return true;
		}
	}
	return false;
}
void makeNameUnique(Node* node, M_Node& existingDefList){
	String originalName = node->name;
	int num=2;
	do{
		node->name = originalName;
		node->name += num++;
	}while(checkForMatchingName(node->name, existingDefList));
}

void checkForNameClash(M_Node& importDefList, M_Node& existingDefList){
	for(int i=importDefList.length; --i>=0;){
		Node* importNode = importDefList[i];
		for(int j = existingDefList.length; --j>=0;){
			if(importNode->name==existingDefList[j]->name){
				makeNameUnique(importNode, existingDefList);
			}
		}
		existingDefList += importNode;
	}
}


void checkForNameClashExceptParts(M_Node& importDefList, M_Node& existingDefList){
	for(int i = importDefList.length; --i>=0;){
		Node* importNode = importDefList[i];
		if(dynamic_cast<Art*>(importNode) || dynamic_cast<Seamless*>(importNode) || dynamic_cast<Viewpoint*>(importNode) || dynamic_cast<PovLight*>(importNode) || importNode->name.length == 0){
			continue;
		}
		for(int j = existingDefList.length; --j>=0;){
			if(importNode->name == existingDefList[j]->name){
				makeNameUnique(importNode, existingDefList);
			}
		}
		existingDefList += importNode;
	}
}


void genDynamicTransforms(Art* part, Matrix parentMatrix){
	
	part->pivotPointMatrix = translationMatrix(part->translation + part->pivotPoint) * parentMatrix;

	part->pivotPointMatrix = scaleMatrix(part->scale) * rotationMatrix(part->rotation) * part->pivotPointMatrix;

	for(int k = part->part.length; --k >= 0;){
		genDynamicTransforms(part->part.refValue()[k], part->pivotPointMatrix);
	}
}

void genDynamicTransforms(Seamless* smls){
	
	Matrix parentMatrix = scaleMatrix(smls->scale) * rotationMatrix(smls->rotation) * translationMatrix(smls->translation);

	for(int k = smls->skeleton.length; --k >= 0;){
		genDynamicTransforms(dynamic_cast<Art*>(smls->skeleton[k].p), parentMatrix);
	}
}
static Seamless* recalcSmlsNode = 0;
void setRotationFromMatchingNames(M_<SceneTreeListItem>& importSceneTreeList, M_<SceneTreeListItem>& existingSceneTreeList){
	for(int i = 0; i < importSceneTreeList.length; i++){
		if(Art* importNode = dynamic_cast<Art*>(importSceneTreeList[i].node.p)){
			if(Art* importParent = dynamic_cast<Art*>(importSceneTreeList[importSceneTreeList[i].parent].node.p)){
				for(int j = 0; j < existingSceneTreeList.length; j++){
					if(Art* existingNode = dynamic_cast<Art*>(existingSceneTreeList[j].node.p)){
						if(Art* existingParent = dynamic_cast<Art*>(existingSceneTreeList[existingSceneTreeList[j].parent].node.p)){
							if(existingParent->show){
								if(importNode->name == existingNode->name){

									recalcSmlsNode = importNode->smls;

									genDynamicTransforms(recalcSmlsNode);
									genDynamicTransforms(existingNode->smls);

									Vec3f importPoint = importNode->pivotPointMatrix * Vec3f(0,0,0) - importParent->pivotPointMatrix * Vec3f(0,0,0);
									Vec3f existingPoint = existingNode->pivotPointMatrix * Vec3f(0,0,0) - existingParent->pivotPointMatrix * Vec3f(0,0,0);

									if(importPoint == Vec3f() || existingPoint == Vec3f()){
										continue;
									}

									importParent->rotation.refValue() = Rotation(importPoint, existingPoint);
								}
							}
						}
					}
				}
			}
		}
	}
}

void copyPivotPointsFromMatchingNames(Seamless* smls, M_<SceneTreeListItem>& importSceneTreeList){
	M_Art& partList = smls->partList;
	for(int j = partList.length; --j >= 0;){
		if(Art* existingNode = dynamic_cast<Art*>(partList[j])){
			if(existingNode->show){
				for(int i = 0; i < importSceneTreeList.length; i++){
					if(Art* importNode = dynamic_cast<Art*>(importSceneTreeList[i].node.p)){
						if(importNode->name == existingNode->name){
							existingNode->pivotPoint = importNode->pivotPoint;
						}
					}
				}
			}
		}
	}
}

void recalculatePivotPointDistancesFromMatchingNames(Seamless* smls, M_<SceneTreeListItem>& importSceneTreeList){
	M_Art& partList = smls->partList;
	for(int j = partList.length; --j >= 0;){
		if(Art* existingNode = dynamic_cast<Art*>(partList[j])){
			if(existingNode->show){
				for(int i = 0; i < importSceneTreeList.length; i++){
					if(Art* importNode = dynamic_cast<Art*>(importSceneTreeList[i].node.p)){
						if(importNode->name == existingNode->name){
							float importDistance = importNode->pivotPoint.refValue().length();
							if(fabs(importDistance) > .0001){
								float existingDistance = existingNode->pivotPoint.refValue().length();
								if(fabs(existingDistance) > .0001){
									float ratio = importDistance / existingDistance;
									existingNode->pivotPoint = existingNode->pivotPoint.refValue() * ratio;
								}
							}
						}
					}
				}
			}
		}
	}
}

void addGroupBuild(Seamless* smls, Scene* importScene, M_<SceneTreeListItem>& importSceneTreeList){
	M_Build& m_build = smls->build.refValue();
	for(int k = m_build.length; --k>=0;){
		if(dynamic_cast<GroupBuild*>(m_build[k].p) == 0){
			if(smls->skeleton.length && smls->skeleton[0]->name != ""){
				GroupBuild* group = new GroupBuild;
				group->part.refValue() = smls->skeleton.refValue()[0].p;
				group->build.refValue() = smls->build.refValue();
				m_build.length = 0;
				m_build += group;

				importScene->genNodeList3();
				importSceneTreeList = g_sceneTreeList;
			}
			break;
		}
	}
}

Seamless* findFirstSeamlessNode(M_<SceneTreeListItem>& sceneTreeList){
	for(int k = 0; k < sceneTreeList.length; k++){
		if(Seamless* smls = dynamic_cast<Seamless*>(sceneTreeList[k].node.p)){
			return smls;
		}
	}
	return 0;
}

bool importNodesSetup(const String& filename, ScenePtr& importScene, M_<SceneTreeListItem>& importSceneTreeList, M_Node& importDefList, Seamless*& importSmls, int task){
	if(checkBvhFileExtension(filename.c_str())){
		importBVHGesture(filename, importScene, importSceneTreeList, importDefList);
		return true;
	}

	setMathHiPresision();
	Function::reset_m_word = false;
	String& str=g_loadStr;
	str = readFile(filename);
	if(str=="")return false;
	importScene = new Scene;
	M_Node& rawLoad = importScene->children;
	Function::selectedLine=0;//create new code lines
	initCompile();
	createSmlsFromString(rawLoad, str);

	decodeFunctions(str);


	importScene->genNodeList3();
	importSceneTreeList = g_sceneTreeList;

	if(task == ONLY_BUILD){
		if(importSmls = findFirstSeamlessNode(importSceneTreeList)){
			addGroupBuild(importSmls, importScene, importSceneTreeList);
		}
	}


	createDefList(importDefList);
	postLoadProcessNodes(g_sceneTreeList);

	importScene->postLoadProcessCollapse();
	g_scene->genNodeList2();
	return true;
}


void reconnectMatchingNames(M_<SceneTreeListItem>& sceneTreeList, M_Node& defList){
	for(int d = defList.length; --d >= 0;){
		Node* defNode = defList[d];
		for(int k = sceneTreeList.length; --k >= 0;){
			Node* node = sceneTreeList[k].node;
			if(CodeLine* codeLine = dynamic_cast<CodeLine*>(node)){
				if(Function* f = dynamic_cast<Function*>(sceneTreeList[sceneTreeList[k].parent].node.p)){
					M_Word& m_word = f->m_word;
					for(int w = codeLine->wordFrom; w < codeLine->wordTo; w++){
						Word* wrd = m_word[w];
						if(W_Node* w_node = dynamic_cast<W_Node*>(wrd)){
							Node* node = w_node->node;
							if(node->name == defNode->name && node != defNode){
								w_node->node = defNode;
							}
						}
						else if(W_NoOutputNode* w_node = dynamic_cast<W_NoOutputNode*>(wrd)){
							Node* node = w_node->node;
							if(node->name == defNode->name && node != defNode){
								w_node->node = defNode;
							}
						}
						else if(W_Field* w_node = dynamic_cast<W_Field*>(wrd)){
							Node* node = w_node->node;
							if(node->name == defNode->name && node != defNode){
								w_node->node = defNode;
							}
						}
					}
				}
			}
			else if(sceneTreeList[k].ref && node->name == defNode->name && node != defNode){
				int fieldIdx = 0;
				int element = 0;
				getFNodeElementForParent(sceneTreeList[k].parent, k, sceneTreeList, fieldIdx, element);

				Node* parent = sceneTreeList[sceneTreeList[k].parent].node;

				if(MFNode* mfnode = dynamic_cast<MFNode*>(parent->getField(fieldIdx))){
					mfnode->refValue()[element] = defNode;
					*mfnode = *mfnode;
				}
				else if(SFNode* sfnode = dynamic_cast<SFNode*>(parent->getField(fieldIdx))){
					*sfnode = defNode;
				}
			}
		}
	}
}

Art* getMatchingPart(const String& name, const M_Art& partList){
	for(int k = partList.length; --k>=0;){
		Art* part = partList[k];
		if(part->name == name){
			return part;
		}
	}
	return 0;
}

void Seamless::importTriangles(Seamless* srcSmls){
	M_Vec2i offset; //x is coord. y is texture
	offset.length = partList.length;
	for(int j = partList.length; --j>=0;){
		Art* part = partList[j];
		offset[j] = Vec2i(part->coord.length, part->workEffect->getEffectLength());
	}

	M_Art& srcPartList = srcSmls->partList;
	for(int k = partList.length; --k>=0;){
		Art* part = partList[k];
		if(Art* srcPart = getMatchingPart(part->name, srcPartList)){
			if(part->name == srcPart->name){
				part->coord.refValue() += srcPart->coord;
				part->uvCoord.refValue() += srcPart->uvCoord;
				part->patchOwner += srcPart->patchOwner;
				if(ColorEffect* dstColorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
					if(ColorEffect* srcColorEffect = dynamic_cast<ColorEffect*>(srcPart->workEffect)){
						dstColorEffect->color.refValue() += srcColorEffect->color;
					}
				}
				else if(TextureEffect* dstTextureEffect = dynamic_cast<TextureEffect*>(part->workEffect)){
					if(TextureEffect* srcTextureEffect = dynamic_cast<TextureEffect*>(srcPart->workEffect)){
						dstTextureEffect->texCoord.refValue() += srcTextureEffect->texCoord;
					}
				}

				int triangleOffset = part->triangle.getTriangleLength();
				M_Triangle& triangle = part->triangle;
				triangle += srcPart->triangle;

				for(int i = triangleOffset; i < triangle.getTriangleLength(); i++){
					Art* tp = triangle.refPart(i);
					if(tp->name == part->name){
						triangle.refPart(i) = part;
						triangle.refIndex(i) += offset[k].x;
						triangle.refTexIndex(i) += offset[k].y;
					}
					else if(Art* partB = getMatchingPart(tp->name, srcPartList)){
						triangle.refPart(i) = partB;
						triangle.refIndex(i) += offset[partB->plidx].x;
						triangle.refTexIndex(i) += offset[partB->plidx].y;
					}
				}
			}
		}
	}
}

void addPartIfNotReferenced(Node* node, M_Node& defList, M_SArt& skeleton){
	for(int k = defList.length; --k>=0;){
		if(defList[k]->name == node->name){
			return;
		}
	}

	Art* newPart = new Part;
	newPart->name = node->name;
	skeleton += newPart;
	defList += newPart;
}

void addPartFromImport(Seamless* importSmls, M_Build& m_build, Seamless* destSmls, M_Node& existingDefList){
	M_SArt& skeleton = destSmls->skeleton.refValue();


	for(int k = 0; k < m_build.length; k++){
		
		Build* bld = m_build[k].p;
	
		if(SphereBuild* sphere = dynamic_cast<SphereBuild*>(bld)){
			if(Art* part = sphere->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(CylinderBuild* cylinder = dynamic_cast<CylinderBuild*>(bld)){
			if(Art* part = cylinder->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(ConeBuild* cone = dynamic_cast<ConeBuild*>(bld)){
			if(Art* part = cone->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(TorusBuild* torus = dynamic_cast<TorusBuild*>(bld)){
			if(Art* part = torus->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(BoxBuild* box = dynamic_cast<BoxBuild*>(bld)){
			if(Art* part = box->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(RectangleBuild* rectangle = dynamic_cast<RectangleBuild*>(bld)){
			if(Art* part = rectangle->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(NurbsLathe* lathe = dynamic_cast<NurbsLathe*>(bld)){
			if(Art* part = lathe->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(BendyLathe* bendy = dynamic_cast<BendyLathe*>(bld)){
			if(Art* part = bendy->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(Hair* hair = dynamic_cast<Hair*>(bld)){
			if(Art* part = hair->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
		}
		else if(GroupBuild* group = dynamic_cast<GroupBuild*>(bld)){
			if(Art* part = group->part.refValue()){
				addPartIfNotReferenced(part, existingDefList, skeleton);
			}
			addPartFromImport(importSmls, group->build.refValue(), destSmls, existingDefList);
		}
		else if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(bld)){
			M_Node& m_stem = patch->stem.refValue();
			for(int j = m_stem.length; --j>=0;){
				if(Stem* stem = dynamic_cast<Stem*>(m_stem[j].p)){
					M_SArt& m_part = stem->part.refValue();
					for(int g = m_part.length; --g>=0;){
						if(Art* part = dynamic_cast<Art*>(m_part[g].p)){
							addPartIfNotReferenced(part, existingDefList, skeleton);
						}
					}
				}
			}
		}
	}
	destSmls->process();
}

void importNodeField(Node* parentSelNode, int selFieldIdx, int selElement, M_Node& src, unsigned type);

void importNodesFromFileAndInsert(M_Node& children, int importWhere){
	int selFieldIdx = 0;
	int selElement = 0;
	
	int parentSelLine = getParent(g_stw->selLine);
	
	Node* parentSelNode = g_sceneTreeList[parentSelLine].node;
	
	getFNodeElementForParent(parentSelLine, g_stw->selLine, g_sceneTreeList, selFieldIdx, selElement );
	
	importNodeField(parentSelNode, selFieldIdx, selElement, children, importWhere);
}

void importBuildNodes(Seamless* importSmls, Seamless* destSmls, M_Node& importDefList, M_<SceneTreeListItem>& importSceneTreeList, int importWhere){
	M_<SceneTreeListItem>& existingSceneTreeList = g_sceneTreeList;
	M_Node existingDefList;
	createDefList(existingDefList);
	if(importSmls){

		for(int k = importSmls->link.length; --k>=0;){
			importSmls->link[k].poly += destSmls->ctrlPolyPoint.length;
		}
		destSmls->link += importSmls->link;
		destSmls->ctrlPolyPoint += importSmls->ctrlPolyPoint;

		addPartFromImport(importSmls, importSmls->build.refValue(), destSmls, existingDefList);
		g_scene->genNodeList3();
		createDefList(existingDefList);
		destSmls->importTriangles(importSmls);
	
		checkForNameClashExceptParts(importDefList, existingDefList);
		reconnectMatchingNames(importSceneTreeList, existingDefList);
		if(importWhere == 0){
			destSmls->build.refValue() += importSmls->build.refValue();
		}
		else{
			importNodesFromFileAndInsert(*reinterpret_cast<M_Node*>(&importSmls->build.refValue()), importWhere);
		}

		importControlPointOwnership(importSmls, destSmls);
		destSmls->process();
		destSmls->resetRelevantParts();

		destSmls->createCpFromPatchPoints();
		destSmls->createSharedPointsFromStitchPolys();
		destSmls->cleanCps();

		destSmls->verifyIntegrity();
	}
}

void importNodesFromFileField(const String& filename, M_Node& children, int task, Seamless* destSmls){
	M_Node animation;
	M_Node importDefList;
	M_<SceneTreeListItem> importSceneTreeList;
	ScenePtr importScene;
	Seamless* importSmls;

	if(importNodesSetup(filename, importScene, importSceneTreeList, importDefList, importSmls, task) == false){
		return;
	}
	M_Node& rawLoad = importScene->children.refValue();

	M_<SceneTreeListItem>& existingSceneTreeList = g_sceneTreeList;

	if(task == IMPORT_PIVOT_POINTS){
		copyPivotPointsFromMatchingNames(dynamic_cast<Seamless*>(g_selNode.getRaw()), importSceneTreeList);
	}
	else if(task == IMPORT_PIVOT_POINTS_DISTANCES){
		recalculatePivotPointDistancesFromMatchingNames(dynamic_cast<Seamless*>(g_selNode.getRaw()), importSceneTreeList);
	}
	else if(task == RECALC){
		g_scene->resetRotations();
		importScene->resetRotations();
		setRotationFromMatchingNames(importSceneTreeList, existingSceneTreeList);
		compileScaleRotation(recalcSmlsNode, importSceneTreeList);
		importAnimationNodesLoop(rawLoad, animation);
		children += animation;
	}
	else if(task == ONLY_ANIMATION){
		M_<SceneTreeListItem>& existingSceneTreeList = g_sceneTreeList;
		M_Node existingDefList;
		createDefList(existingDefList);
		importAnimationNodesLoop(rawLoad, animation);

		checkForNameClashExceptParts(importDefList, existingDefList);
		reconnectMatchingNames(importSceneTreeList, existingDefList);
		children += animation;


//		importAnimationNodesLoop(rawLoad, animation);
	//	children += animation;
		//g_stw->genNodeList();
	}
	else if(task == ONLY_BUILD){
		importBuildNodes(importSmls, destSmls, importDefList, importSceneTreeList, 0);
	}
	else{
		M_Node existingDefList;
		createDefList(existingDefList);
		checkForNameClash(importDefList, existingDefList);
		children += rawLoad;
	}
	finishImportNodesFromFile();
	restoreMathPresision();
	fastFullBuildForAutoIn(true);
	g_scene->updateBuild();
	fastFullBuildForAutoIn(false);
	if(g_scene->mode & (ANIMATION_MODE | HOME_MODE)){
		setWorkAnimCPsInScene();
	}
	houseKeepVisualClue();
}

void importNodesFromFileAndInsert(const String& filename, int importWhere, int task, Seamless* destSmls){
	M_Node animation; //also used for build nodes
	M_Node importDefList;
	M_<SceneTreeListItem> importSceneTreeList;
	ScenePtr importScene;
	Seamless* importSmls = 0;

	if(importNodesSetup(filename, importScene, importSceneTreeList, importDefList, importSmls, task) == false){
		return;
	}
	M_Node& rawLoad = importScene->children.refValue();

	M_<SceneTreeListItem>& existingSceneTreeList = g_sceneTreeList;

	if(task == ONLY_ANIMATION){
		M_<SceneTreeListItem>& existingSceneTreeList=g_sceneTreeList;
		M_Node existingDefList;
		createDefList(existingDefList);
		importAnimationNodesLoop(rawLoad, animation);

		checkForNameClashExceptParts(importDefList, existingDefList);
		reconnectMatchingNames(importSceneTreeList, existingDefList);
		importNodesFromFileAndInsert(animation, importWhere);
	}
	else if(task == ONLY_BUILD){
		importBuildNodes(importSmls, destSmls, importDefList, importSceneTreeList, importWhere);
	}
	else{
		M_<SceneTreeListItem>& existingSceneTreeList=g_sceneTreeList;
		M_Node existingDefList;
		createDefList(existingDefList);
		checkForNameClash(importDefList, existingDefList);//i am now in doubt this is as good a way as the import build way because what about nodes that are about to be replaced?
		importNodesFromFileAndInsert(rawLoad, importWhere);
	}
	finishImportNodesFromFile();
	restoreMathPresision();
	fastFullBuildForAutoIn(true);
	g_scene->updateBuild();
	fastFullBuildForAutoIn(false);
	houseKeepVisualClue();
}


void addLocalParAssignment(Function* f, Node* node, int fieldIndex){
	M_Word& m_word = f->m_word;

	CodeLine* codeLine=new CodeLine;
	codeLine->wordFrom=m_word.length;
	m_word+=new W_NoOutputNode(node);
	m_word+=new W_NoOutputDot();
	m_word+=new W_Field(node, fieldIndex);
	m_word+=new W_Assignment();
	m_word+=new W_Local(f,0);
	codeLine->wordTo = m_word.length;
	for(int w = codeLine->wordFrom; w<codeLine->wordTo; w++){
		m_word[w]->toString(codeLine->name);
	}
	f->codeLine += codeLine;
}
void addFieldFieldAssignment(Function* f, Node* fromNode, int fromField, Node* toNode, int toField){
	M_Word& m_word=f->m_word;

	CodeLine* codeLine=new CodeLine;
	codeLine->wordFrom=m_word.length;
	m_word+=new W_NoOutputNode(toNode);
	m_word+=new W_NoOutputDot();
	m_word+=new W_Field(toNode, toField);
	m_word+=new W_Assignment();
	m_word+=new W_NoOutputNode(fromNode);
	m_word+=new W_NoOutputDot();
	m_word+=new W_Field(fromNode, fromField);

	codeLine->wordTo=m_word.length;

	for(int w=codeLine->wordFrom; w<codeLine->wordTo; w++){
		m_word[w]->toString(codeLine->name);
	}

	f->codeLine+=codeLine;
}

void addMorphFunction(Function* f, CPMorpher* mo, Art* part){
	M_Word& m_word = f->m_word;

	CodeLine* codeLine=new CodeLine;
	codeLine->wordFrom=m_word.length;

	m_word+=new W_Node(mo);
	m_word+=new W_Dot();
	m_word+=new W_Morph();
	m_word+=new W_OpenBracket();

	m_word+=new W_Local(f,0);

	m_word+=new W_Comma();
	m_word += new W_Node(part);
	m_word += new W_CloseBracket();

	codeLine->wordTo = m_word.length;
	for(int w = codeLine->wordFrom; w<codeLine->wordTo; w++){
		m_word[w]->toString(codeLine->name);
	}
	f->codeLine += codeLine;
}

void addCalcFunction(Function* f, NikInterpolator* niki, Art* part){
	M_Word& m_word = f->m_word;

	CodeLine* codeLine=new CodeLine;
	codeLine->wordFrom=m_word.length;

	m_word+=new W_Node(niki);
	m_word+=new W_Dot();
	m_word+=new W_Calc();
	m_word+=new W_OpenBracket();

	m_word+=new W_Local(f,0);

	m_word+=new W_Comma();
	m_word += new W_Node(part);
	m_word += new W_CloseBracket();

	codeLine->wordTo = m_word.length;
	for(int w = codeLine->wordFrom; w<codeLine->wordTo; w++){
		m_word[w]->toString(codeLine->name);
	}
	f->codeLine += codeLine;
}

void addMorphFunction(Function* f, PosCompensator* pc, Art* part){
	M_Word& m_word = f->m_word;

	CodeLine* codeLine=new CodeLine;
	codeLine->wordFrom=m_word.length;

	m_word+=new W_Node(pc);
	m_word+=new W_Dot();
	m_word+=new W_Comp();
	m_word+=new W_OpenBracket();

	m_word+=new W_Local(f,0);

	m_word+=new W_Comma();
	m_word += new W_Node(part);
	m_word += new W_CloseBracket();

	codeLine->wordTo = m_word.length;
	for(int w = codeLine->wordFrom; w<codeLine->wordTo; w++){
		m_word[w]->toString(codeLine->name);
	}
	f->codeLine += codeLine;
}

void addOIAssignment(Function* f, CameraSelector* bi, Node* node, Field* field){
	addLocalParAssignment(f, bi, bi->getFieldIndex(&bi->set_fraction));
}

void addOIAssignment(Function* f, ScalarInterpolator* si, Node* node, Field* field){
	addLocalParAssignment(f, si, si->getFieldIndex(&si->set_fraction));
	addFieldFieldAssignment(f, si, si->getFieldIndex(&si->value_changed), node, node->getFieldIndex(field));
}

void addOIAssignment(Function* f, OrientationInterpolator* oi, Art* part){
	addLocalParAssignment(f, oi, oi->getFieldIndex(&oi->set_fraction));
	if(part->ik){
		addFieldFieldAssignment(f, oi, oi->getFieldIndex(&oi->value_changed), part, part->getFieldIndex(&part->ikRot));
	}
	else{
		addFieldFieldAssignment(f, oi, oi->getFieldIndex(&oi->value_changed), part, part->getFieldIndex(&part->rotation));
	}
}
void addOIAssignment(Function* f, OrientationInterpolator* oi, Node* view, Field* field){
	addLocalParAssignment(f, oi, oi->getFieldIndex(&oi->set_fraction));
	addFieldFieldAssignment(f, oi, oi->getFieldIndex(&oi->value_changed), view, view->getFieldIndex(field));
}
void addOIAssignment(Function* f, PositionInterpolator* pi, Art* part){
	addLocalParAssignment(f, pi, pi->getFieldIndex(&pi->set_fraction));
	if(part->ik){
		addFieldFieldAssignment(f, pi, pi->getFieldIndex(&pi->value_changed), part, part->getFieldIndex(&part->ikPos));
		g_vcArtAnimBase->remTranValue = part->ikPos.refValue();
		//part->ikPos.refValue() = part->absolutePivotPoint;
	}
	else{
		addFieldFieldAssignment(f, pi, pi->getFieldIndex(&pi->value_changed), part, part->getFieldIndex(&part->translation));
	}
}
void addRaAssignment(Function* f, RotAccelerator* ra, Art* part){
	addLocalParAssignment(f, ra, ra->getFieldIndex(&ra->set_fraction));
	addFieldFieldAssignment(f, ra, ra->getFieldIndex(&ra->value_changed), part, part->getFieldIndex(&part->rotation));
}

void addOIAssignment(Function* f, PositionInterpolator* pi, Node* node, Field* field){
	addLocalParAssignment(f, pi, pi->getFieldIndex(&pi->set_fraction));
	addFieldFieldAssignment(f, pi, pi->getFieldIndex(&pi->value_changed), node, node->getFieldIndex(field));
}

GestureGroup* findGestureGroup(){
	for(int k=0; k<g_sceneTreeList.length; k++){
		if(GestureGroup* gestureGroup=dynamic_cast<GestureGroup*>(g_sceneTreeList[k].node.p)){
			return gestureGroup;
		}
	}
	return 0;
}

void CPMorpher::take(float slider, Node* node, Field* field){
}

void CPMorpher::connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function){
	if(Art* part = dynamic_cast<Art*>(node)){
		this->name = name;
		makeExistingNameUnique(this);

		addMorphFunction(function, this, part);
		anim->children.refValue() += this;
		//keyValue.refValue().length = key.refValue().length = 2;
		//key.refValue()[1] = 1;
	}
}

void OrientationInterpolator::connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function){
	this->name = name;
	makeExistingNameUnique(this);

	addOIAssignment(function, this, node, field);
	anim->children.refValue() += this;
}

void PosCompensator::connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function){
	if(Art* part = dynamic_cast<Art*>(node)){
		this->name = name;
		makeExistingNameUnique(this);

		addMorphFunction(function, this, part);
		anim->children.refValue() += this;
	}
}

void PositionInterpolator::connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function){
	this->name = name;
	makeExistingNameUnique(this);

	addOIAssignment(function, this, node, field);
	anim->children.refValue()+=this;
}

void ScalarInterpolator::connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function){
	this->name = name;
	makeExistingNameUnique(this);

	addOIAssignment(function, this, node, field);
	anim->children.refValue()+=this;
}

void CameraSelector::connectNode(Node* node, Field* field, String& name, Anim* anim, Function* function){
	this->name = name;
	makeExistingNameUnique(this);

	addOIAssignment(function, this, node, field);
	anim->children.refValue()+=this;
}

void addCommonName(String& str, const int commonNameCode){
	switch(commonNameCode){
		case _MORPHER_:
			str += "_M_";
			break;
		case _POS_COMPENSATOR_:
			str += "_PC_";
			break;
		case _NIK_INTERPOLATOR_:
			str += "_NIK_";
			break;
		case _POSITION_:
			str += "_P_";
			break;
		case _TRANSLATION_:
			str += "_T_";
			break;
		case _LOOK_AT_:
			str += "_A_";
			break;
		case _UP_:
			str += "_U_";
			break;
		case _ORIENTATION_:
			str += "_O_";
			break;
		case _ROTATION_:
			str += "_R_";
			break;
		case _FOV_:
			str += "_F_";
			break;
		case _LIVE_:
			str += "_L_";
			break;
		case _ZOOM_:
			str += "_Z_";
			break;
		case _FA_:
			str += "_FA_";
			break;
		case _FB_:
			str += "_FB_";
			break;
		default:
			throw;
	}
}


bool Anim::connectPartToCPMorpher(Art* part, String& name, SamplePose* samplePose){
	if(this->name==""){
		createUniqueName(this);
	}
	if(part->name==""){
		createUniqueName(part);
	}
	g_stw->textCell.updateText();
	name=part->name;
	addCommonName(name, _MORPHER_);
	name+=this->name;

	Function* function=dynamic_cast<MemberFunction*>(onFrame.p);
	if(function==0){
		function = new MemberFunction;
		onFrame=function;
		g_functionList+=function;
	}
	samplePose->connectNode(part, 0, name, this, function);

	return true;
}

bool Anim::connectNode(Node* node, Field* field, String& name, SamplePose* samplePose, const int commonNameCode){
	if(this->name==""){
		createUniqueName(this);
	}
	if(node->name==""){
		createUniqueName(node);
	}
	g_stw->textCell.updateText();
	name = node->name;
	addCommonName(name, commonNameCode);
	name += this->name;

	Function* function=dynamic_cast<MemberFunction*>(onFrame.p);
	if(function==0){
		function = new MemberFunction;
		onFrame = function;
		g_functionList+=function;
	}
	samplePose->connectNode(node, field, name, this, function);

	return true;
}

void commonEndForSetupAnim(){
	Function::selectedLine=g_scene;//to prevent new nodes being created, acts like a flag thats all

	g_scene->genNodeList2();
	String& str=g_loadStr;
	str="";
	onlyFunctionsToString(str);
	initCompileForBody();

	resetDefList();
	decodeFunctions(str);
	Function::selectedLine=0;
	g_scene->genNodeList2();
	Function::selectedLine=0;//restore(probably not needed)

//		resetLooseEnds();
	g_stw->paint();
	//BackUp::monostable.beginCycle();
}

CPMorpher* Anim::connectPartToCPMorpher(Art* part){
	if(part && part->show){
		String name;
		CPMorpher* cpMorpher = new CPMorpher;
		if(connectPartToCPMorpher(part, name, cpMorpher)){
			commonEndForSetupAnim();
			return cpMorpher;
		}
	}
	return 0;
}

OrientationInterpolator* Anim::connectNodeToOrientationInterpolator(Node* node, Field* field, const int commonNameCode){
	String name;
	OrientationInterpolator* orientationInterpolator = new OrientationInterpolator;
	if(connectNode(node, field, name, orientationInterpolator, commonNameCode)){
		commonEndForSetupAnim();
		return orientationInterpolator;
	}
	return 0;
}

PositionInterpolator* Anim::connectNodeToPositionInterpolator(Node* node, Field* field, const int commonNameCode){
	String name;
	PositionInterpolator* positionInterpolator = new PositionInterpolator;
	if(connectNode(node, field, name, positionInterpolator, commonNameCode)){
		commonEndForSetupAnim();
		return positionInterpolator;
	}
	return 0;
}

ScalarInterpolator* Anim::connectNodeToScalarInterpolator(Node* node, Field* field, const int commonNameCode){
	String name;
	ScalarInterpolator* scalarInterpolator = new ScalarInterpolator;
	if(connectNode(node, field, name, scalarInterpolator, commonNameCode)){
		commonEndForSetupAnim();
		return scalarInterpolator;
	}
	return 0;
}

CameraSelector* Anim::connectNodeToCameraSelector(Node* node, Field* field, const int commonNameCode){
	String name;
	CameraSelector* cameraSelector = new CameraSelector;
	if(connectNode(node, field, name, cameraSelector, commonNameCode)){
		commonEndForSetupAnim();
		return cameraSelector;
	}
	return 0;
}

PosCompensator* Anim::connectPartToPosCompensator(Art* part){
	String name;
	PosCompensatorPtr posCompensator = new PosCompensator;
	if(connectNode(part, 0, name, posCompensator, _POS_COMPENSATOR_)){
		commonEndForSetupAnim();
		return posCompensator;
	}
	return 0;
}

NikInterpolator* Anim::connectPartToNikInterpolator(Art* part){
	String name;
	NikInterpolatorPtr niki = new NikInterpolator;
	if(connectNode(part, 0, name, niki, _NIK_INTERPOLATOR_)){
		commonEndForSetupAnim();
		return niki;
	}
	return 0;
}
void Node::convertRoutesToScript(Function* f){
	for(int i = 0; i < getFLen(); i++){
		Field* field=getField(i);
		M_Route &m_route=field->refRoute();
		for(int r=0; r<m_route.length;r++){
			S_Route& route=m_route[r];
			Node* toNode = route.toNode;
			addFieldFieldAssignment(f, this, i, toNode, route.toFieldIdx);
			toNode->convertRoutesToScript(f);
		}
		m_route.length=0;
	}
}

void addNodeToList(Node* node, M_Node& children){
	for(int k = children.length; --k>=0;){
		if(children[k]==node){
			return;
		}
	}
	children+=node;
}

Anim* convertTimeSensorToAnim(TimeSensor* ts, bool addNodesToList=false){
	String name;

	Anim* anim=new Anim;
	anim->name=ts->name;
	Function* function = new MemberFunction;
	anim->onFrame=function;
	int oldFunctionLen=g_functionList.length;
	g_functionList+=function;
	anim->period.refValue()=ts->cycleInterval;
	anim->loop.refValue()=ts->loop;
	anim->play.refValue()=ts->enabled;
	anim->children.refValue()=ts->children;

	M_Route& m_route=ts->fraction_changed.route;

	for(int k=0; k<m_route.length; k++){
		S_Route& route=m_route[k];
		addLocalParAssignment(function, route.toNode, route.toFieldIdx);
		if(addNodesToList){
			addNodeToList(route.toNode, anim->children.refValue());
		}
		route.toNode->convertRoutesToScript(function);
	}
	return anim;
}
Art* getPart(M_Art& partList, String name){
	for(int k = partList.length; --k>=0;){
		Art* part = partList[k];
		if(part->name == name){
			return part;
		}
	}
	return 0;
}
void changeWelderToPart(M_Word& m_word, Seamless* smls, Script* welder){
	for(int k = m_word.length; --k>=0;){
		if(W_NoOutputNode* w_NoOutputNode = dynamic_cast<W_NoOutputNode*>(m_word[k])){
			if(w_NoOutputNode->node==welder){
				if(W_Field* w_Field = dynamic_cast<W_Field*>(m_word[k+2])){
					if(w_Field->node==welder){
						String welderName=welder->properties[w_Field->fieldIdx].name;
						Field* field = w_Field->node->getField(w_Field->fieldIdx);
						if(welderName=="P1"){
							Art* part = smls->partList[1];
							w_NoOutputNode->node = w_Field->node = part;
							w_Field->fieldIdx = part->getFieldIndex(&part->translation);
						}
						else{
							welderName=welderName.substring(4);
							Art* part = getPart(smls->partList, welderName);
							w_NoOutputNode->node = w_Field->node = part;
							if(dynamic_cast<SFRotation*>(field)){
								w_Field->fieldIdx = part->getFieldIndex(&part->rotation);
							}
							else if(dynamic_cast<SFVec3f*>(field)){
								w_Field->fieldIdx = part->getFieldIndex(&part->translation);
							}
						}
					}
				}
			}
		}
	}
}
void convertGestures(Seamless* smls, M_Node& m_ts, M_Node& m_anim, Script* welder, M_String& gestureNames){
	for(int k = 0; k < m_ts.length; k++){
		Anim* anim = convertTimeSensorToAnim(dynamic_cast<TimeSensor*>(m_ts[k].p), true);
		anim->name = convertSpaceToUnderbar(gestureNames[k]);

		anim->collapsed=true;
		Function* function=dynamic_cast<MemberFunction*>(anim->onFrame.p);
		M_Node& codeLine = function->codeLine.refValue();
		codeLine.length--;
		anim->children.refValue().length--;
		changeWelderToPart(function->m_word, smls, welder);

		m_anim+=anim;
	}
}

void convertTimeSensorToAnim(){
	NodePtr ts = g_selNode.getRaw();

	AnimPtr anim = convertTimeSensorToAnim(dynamic_cast<TimeSensor*>(ts.p), true);

	const M_Node& animChildren = anim->children;
	for(int k = animChildren.length; --k>=0;){
		deleteNode(getNodeLine(animChildren[k]));
		g_stw->genNodeList();
	}
	int selLine = getNodeLine(ts);

	int parentSelLine=getParent(selLine);
	Node* parentSelNode=g_sceneTreeList[parentSelLine].node;
	M_Node& children=dynamic_cast<Progenitor*>(parentSelNode)->children;
	int selFieldIdx=0;
	int selElement=0;
	getFNodeElementForParent(parentSelLine, selLine, g_sceneTreeList, selFieldIdx, selElement );
	children[selElement]=anim;

	Function::selectedLine=g_scene;//to prevent new nodes being created, acts like a flag thats all

	g_scene->genNodeList2();
	String& str=g_loadStr;
	str="";
	onlyFunctionsToString(str);
	initCompileForBody();

	resetDefList();
	decodeFunctions(str);
	Function::selectedLine=0;
	g_stw->genNodeList();
	Function::selectedLine=0;//restore(probably not needed)

	deleteAllRoutesToNonExistant();
	deleteAllCodeLinesThatRefNonExistentNodes();
	//resetLooseEnds();
	Window::closeWindow(g_stw->cp.p);
	g_stw->setSelNodeLine(anim);
	g_acp->setupNodeWindow(anim);
	Window::insertUnderActive(g_acp.p);

	g_stw->paint();
	BackUp::monostable.beginCycle();
}
void Function::compileScalePositionInterpolator(const Art* part, Matrix& matrix){

	W_Field* w_dstField=0;
	W_Field* w_srcField=0;

	for(int i=codeLine.length; --i>=0;){
		if(getEquiveRoute(m_word, w_dstField, w_srcField, codeLine, i)){
			if(part==w_dstField->node){
				if(PositionInterpolator* positionInterpolator=dynamic_cast<PositionInterpolator*>(w_srcField->node)){
					M_Vec3f& keyValue=positionInterpolator->keyValue.refValue();
					for(int i=keyValue.length; --i>=0;){
						keyValue[i]=matrix*keyValue[i];
					}
				}
			}
		}
	}
}
void compileScalePositionInterpolator(const Art* part, Matrix& matrix){
	for(int k=g_sceneTreeList.length; --k>=0;){
		if(Function* function = dynamic_cast<Function*>(g_sceneTreeList[k].node.p)){
			function->compileScalePositionInterpolator(part, matrix);
		}
	}
}
void Function::compileOrientationInterpolator(const Art* part, Rotation accumRot){

	W_Field* w_dstField=0;
	W_Field* w_srcField=0;

	Rotation accumRot2 = part->rotation.refValue() * accumRot;

	for(int i=codeLine.length; --i>=0;){
		if(getEquiveRoute(m_word, w_dstField, w_srcField, codeLine, i)){
			if(part==w_dstField->node){
				if(OrientationInterpolator* orientationInterpolator=dynamic_cast<OrientationInterpolator*>(w_srcField->node)){
					M_Rotation& keyValue=orientationInterpolator->keyValue.refValue();
					for(int i=keyValue.length; --i>=0;){
						if(part->rotation.angle==0){
							keyValue[i] = -accumRot2 * keyValue[i] * accumRot2;
							//keyValue[i] = keyValue[i] * accumRot;
						}
						else{
							keyValue[i] = -accumRot2 * keyValue[i] * accumRot;
						}
					}
				}
			}
		}
	}
}

void compileOrientationInterpolator(const Art* part, Rotation accumRot, M_<SceneTreeListItem>& sceneTreeList){
	for(int k=sceneTreeList.length; --k>=0;){
		if(Function* function = dynamic_cast<Function*>(sceneTreeList[k].node.p)){
			function->compileOrientationInterpolator(part, accumRot);
		}
	}
}
void buildModelMorpherForSelSmls(Seamless* smls){
	resetDefList();
	ModelMorpher* modelMorpher = new ModelMorpher;
	ModelSequence* modelSequence = new ModelSequence;
	modelSequence->key = M_Float(0, .5, 1);
	modelMorpher->sequence.refValue() += modelSequence;
	modelSequence->name="seq";
	makeExistingNameUniqueUsingDefList(modelSequence);


	modelSequence->model.refValue() += smls;

	String str;
	smls->toString(str="");
	M_Node m_smls;
	createSmlsFromString(m_smls, str);
	resetDefList();
	if(smls->name==""){
		smls->name="smls";
		makeExistingNameUniqueUsingDefList(smls);
	}
	Seamless* smlsCopy = dynamic_cast<Seamless*>(m_smls[0].p);
	makeExistingSmlsNamesUniqueUsingDefList(smlsCopy);
	smlsCopy->processFields();
	smlsCopy->processBuild();
	smlsCopy->doBuild();
	modelSequence->model.refValue() += smlsCopy;

	modelSequence->model.refValue() += smls;

	int selLine = g_stw->selLine;
	int parentLine=getParent(selLine);
	Node* parentNode= g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	AnimPtr anim = new Anim;
	Function* function = new MemberFunction;
	anim->onFrame=function;
	g_functionList+=function;
	addLocalParAssignment(function, modelSequence, modelSequence->getFieldIndex(&modelSequence->frame));

	if(field->getType()==MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element]=modelMorpher;
		mfnode->refValue().insertElement(++element, anim.p);
		*mfnode=mfnode->refValue();
	}
	modelMorpher->processFields();
	g_scene->genNodeList2();
	resetLooseEnds();
	commonEndForSetupAnim();
}
const int childrenDragBarTypes[]={
	SEAMLESS,
	TEXTUREEFFECT,
	COLOREFFECT,
	STEM,
	PART,
	IMMEDIATEPART,
	NURBSPATCH,
	FUSESURFACE,
	SUBDIVISION,
	SURFACETRANSFORM,
	TRANSFORM,
	FOG2,
	PLACE,
	COPYPART,
	SINETUG,
	//MODELSEQUENCE,
	MOVIE,
	GROUPBUILD,
	GESTUREGROUP,
	NOOUTPUTGROUP,
	PLACETEXTURE,
	JOINVERTEX,
	TRANSFERBCYL,
	TRANSFEROWNERSHIP,
	COLORSWEEP,
	TEXTUREMAPPER,
	TOUCHSENSOR,
	ANIM,
	VIEWPOINT,
	CODELINE,
	FUNCTION,
	MEMBERFUNCTION,
	EXITBUILD,


	RADIALTUG,
	CCLATHE,
	SURFACEGENERATOR,
	SHAPE,
	APPEARANCE,
	MOVIETEXTURE,
	PIXELTEXTURE,
	IMAGETEXTURE,
	MATERIAL,
	INDEXEDFACESET,
	INDEXEDTRIANGLESET,
	COORDINATE,
	NORMAL,
	COLOR,
	TEXTURECOORDINATE,
	SPHERE,
	CONE,
	CYLINDER,
	POLYLINE2D,
	BOX,
	TEXT,
	FONTSTYLE,
	INDEXEDLINESET,
	ELEVATIONGRID,
	EXTRUSION,


	WORLDINFO,
	NAVIGATIONINFO,
	_GROUP,
	TRANSFORM,
	ANCHOR,
	BILLBOARD,
	SWITCH,
	BSPTREE,
	INLINE,
	COLLISION,
	BACKGROUND,
	DIRECTIONALLIGHT,
	SPOTLIGHT,
	POINTLIGHT,
	TEXTURETRANSFORM,
	CYLINDERSENSOR,
	PLANESENSOR,
	PROXIMITYSENSOR,
	SPHERESENSOR,
	VISIBILITYSENSOR,
	ORIENTATIONINTERPOLATOR,
	POSITIONINTERPOLATOR,
	SCALARINTERPOLATOR,
	COLORINTERPOLATOR,
	COORDINATEINTERPOLATOR,
	NORMALINTERPOLATOR,
	FOG, 
	SOUND,
	AUDIOCLIP,
	COMPOSITION,
	TRACK,
	PHRASE,
	OSCILLATOR,
	HARMONIC,
	SAMPLE,
	ANIMBYANKLE,
	ANIMBYWRIST,
	FLOATACCELERATOR,
	POSACCELERATOR,
	ROTACCELERATOR,
	CCPATH,
	SCRIPT,
};
const int seamlessDragBarTypes[]={
	SEAMLESS,
	TEXTUREEFFECT,
	COLOREFFECT,
	STEM,
	PART,
	IMMEDIATEPART,
	NURBSPATCH,
	FUSESURFACE,
	SUBDIVISION,
	SURFACETRANSFORM,
	TRANSFORM,
	FOG2,
	PLACE,
	COPYPART,
	SINETUG,
	MOVIE,
	GROUPBUILD,
	GESTUREGROUP,
	NOOUTPUTGROUP,
	VIDEO,
	JOINVERTEX,
	TRANSFERBCYL,
	TRANSFEROWNERSHIP,
	COLORSWEEP,
	TEXTUREMAPPER,
	TOUCHSENSOR,
	ANIM,
	VIEWPOINT,
	CODELINE,
	FUNCTION,
	MEMBERFUNCTION,
	EXITBUILD,
};

void DragNode::initNewNodeBarList(Node* node){
	if(node->getType()==SEAMLESS){
		node=node;
	}
	for(int i=newNodeBarList.length; --i>=0;){
		newNodeBarList[i].length=0;
	}
	if(g_settings->extendNodeBar == false){
	//if(	dynamic_cast<Build*>(node)||
	//	dynamic_cast<Generator*>(node)||
	//	dynamic_cast<Effect*>(node)||
	//	dynamic_cast<SurfaceGenerator*>(node)||
	//	dynamic_cast<TextureMapper*>(node)||
	//	dynamic_cast<Seamless*>(node)||
	//	dynamic_cast<Art*>(node)||
	//	dynamic_cast<Morpher*>(node)||
	//	dynamic_cast<FuseSurface*>(node)||
	//	dynamic_cast<MirrorGroup*>(node)||
	//	dynamic_cast<MirrorEdit*>(node)||
	//	dynamic_cast<ModelMorpher*>(node)||
	//	dynamic_cast<ModelSequence*>(node)){

		fillNewNodeBarList(seamlessDragBarTypes, sizeof(seamlessDragBarTypes)/sizeof(int));
	}
	else{
		fillNewNodeBarList(childrenDragBarTypes, sizeof(childrenDragBarTypes)/sizeof(int));
	}

	size=Vec2i(pixSymbolCX*newNodeBarList.length, pixSymbolCY*newNodeBarList[0].length);
}

void Viewpoint::AnimDelete::operator=(const bool& par){
	Anim* anim = Window::animMode();
	if(anim == 0){
		return;
	}

	if(MemberFunction* function=dynamic_cast<MemberFunction*>(anim->onFrame.p)){

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField = 0;
		W_Field* w_srcField = 0;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;

		for(int i=m_codeLine.length; --i>=0;){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				if(Viewpoint* view=dynamic_cast<Viewpoint*>(w_dstField->node)){
					if(SamplePose* samplePose=dynamic_cast<SamplePose*>(w_srcField->node)){
						samplePose->deleteSample(anim->slider);
					}
				}
			}
		}
		anim->update=true;
	}
}

bool getNodeFieldEqualsSomething(M_Word& m_word, W_Field*& w_dstField, M_Node& m_codeLine, int i);

void convertToNurbsTrack(){
	Ptr<NurbsTrack> track = new NurbsTrack;

	PositionInterpolator* oldPI = dynamic_cast<PositionInterpolator*>(g_selNode.getRaw());

	track->name = oldPI->name;
	track->key = oldPI->key;
	for(int k = 0; k < oldPI->keyValue.length - 1; k++){
		track->keyValue.refValue() += oldPI->keyValue[k];
		track->keyValue.refValue() += (oldPI->keyValue[k] + oldPI->keyValue[k + 1]) * .5;
		track->dim.refValue()+=3;
	}
	track->keyValue.refValue() += oldPI->keyValue[oldPI->keyValue.length - 1];
	track->processFields();

	int selLine = g_stw->selLine;
	int parentLine=getParent(selLine);
	Node* parentNode= g_sceneTreeList[parentLine].node.p;
	int fieldIdx=0, element=0;

	getFNodeElementForParent(parentLine, selLine, g_sceneTreeList, fieldIdx, element);

	Field* field=parentNode->getField(fieldIdx);

	if(field->getType()==MFNODE){
		MFNode *mfnode=dynamic_cast<MFNode*>(field);
		mfnode->refValue()[element]=track;
		*mfnode=mfnode->refValue();
	}


	for(int k = g_functionList.length; --k>=0;){
		if(Function* function=dynamic_cast<Function*>(g_functionList[k].p)){

			M_Node& m_codeLine=function->codeLine;
			M_Word& m_word=function->m_word;
			W_Field* w_dstField = 0;
			W_Field* w_srcField = 0;

			for(int i=m_codeLine.length; --i>=0;){
				if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
					if(PositionInterpolator* positionInterpolator=dynamic_cast<PositionInterpolator*>(w_srcField->node)){
						if(oldPI == w_srcField->node && &oldPI->value_changed == w_srcField->node->getField(w_srcField->fieldIdx)){
							w_srcField->node = track;
							w_srcField->fieldIdx = track->getFieldIndex(&track->pos);
						}
					}
				}
				else if(getNodeFieldEqualsSomething(m_word, w_dstField, m_codeLine, i)){
					if(PositionInterpolator* positionInterpolator=dynamic_cast<PositionInterpolator*>(w_dstField->node)){
						if(oldPI == w_dstField->node && &oldPI->set_fraction == w_dstField->node->getField(w_dstField->fieldIdx)){
							w_dstField->node = track;
							w_dstField->fieldIdx = track->getFieldIndex(&track->frame);
						}
					}
				}
			}
		}
	}


	g_stw->commonPostInsertNodeStuff();
}


bool rotationPartPass(Art* part){
	if(part->track){
		if(part->smls->track){
			if(g_timelineFilter->rotation){
				return true;
			}
		}
	}
	return false;
}

bool rotationViewPass(Viewpoint* view){
	if(view->track){
		if(g_timelineFilter->orientation){
			return true;
		}
	}
	return false;
}

bool translationPartPass(Art* part){
	if(part->track){
		if(part->smls->track){
			if(g_timelineFilter->translation){
				return true;
			}
		}
	}
	return false;
}

bool lookAtViewPass(Viewpoint* view){
	if(view->track){
		if(g_timelineFilter->lookAt){
			return true;
		}
	}
	return false;
}

bool upViewPass(Viewpoint* view){
	if(view->track){
		if(g_timelineFilter->up){
			return true;
		}
	}
	return false;
}

bool positionViewPass(Viewpoint* view){
	if(view->track){
		if(g_timelineFilter->position){
			return true;
		}
	}
	return false;
}

bool fkaPartPass(Art* part){
	if(part->track){
		if(part->smls->track){
			if(g_timelineFilter->fka){
				return true;
			}
		}
	}
	return false;
}

bool fkbPartPass(Art* part){
	if(part->track){
		if(part->smls->track){
			if(g_timelineFilter->fkb){
				return true;
			}
		}
	}
	return false;
}

bool zoomViewPass(Viewpoint* view){
	if(view->track){
		if(g_timelineFilter->zoom){
			return true;
		}
	}
	return false;
}

bool fieldOfViewPass(Viewpoint* view){
	if(view->track){
		if(g_timelineFilter->fieldOfView){
			return true;
		}
	}
	return false;
}

bool morphPartPass(Art* part){
	if(part->track){
		if(part->smls->track){
			if(g_timelineFilter->morph){
				return true;
			}
		}
	}
	return false;
}

bool posCompensationPartPass(Art* part){	
	if(part->track){
		if(part->smls->track){
			if(g_timelineFilter->posCompensation){
				return true;
			}
		}
	}
	return false;
}

void Anim::redirectCPMorpherIndicesToContigList(Seamless* smls){
	if(MemberFunction* function = dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;
		W_Node* w_node = 0;

		for(int i = 0; i < m_codeLine.length; i++){
			if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(part->smls == smls){
						if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
							M_Int& cpIndex = morpher->cpIndex;
							for(int i = cpIndex.length; --i>=0;){
								ControlPoint& controlPoint = part->controlPoint[cpIndex[i]];
								cpIndex[i] = controlPoint.p->fromPoint + controlPoint.i;
							}
						}
					}
				}
			}
		}
	}
}

void Anim::deleteNonExistingMorpherPoints(Seamless* smls){
	if(MemberFunction* function = dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;
		W_Node* w_node = 0;

		for(int i = 0; i < m_codeLine.length; i++){
			if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(part->smls == smls){
						if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
							M_Int& cpIndex = morpher->cpIndex;
							M_ControlPoint& m_controlPoint = part->controlPoint;
							int d = 0;
							for(int s = 0; s < cpIndex.length; s++){
								int idx = cpIndex[s];
								if(idx < m_controlPoint.length){
									ControlPoint& controlPoint = m_controlPoint[idx];
									NurbsPatch* patch = controlPoint.p;
									int i = controlPoint.i;
									if(patch->smls == smls && i < patch->point.length){
										cpIndex[d++] = idx;
									}
								}
							}
							cpIndex.length = d;
						}
					}
				}
			}
		}
	}
}


void Anim::redirectContigListBackToCPMorpherIndices(Seamless* smls, M_<CPContigElement>& m_contigLookUpList){
	if(MemberFunction* function = dynamic_cast<MemberFunction*>(onFrame.p)){

		M_Node& m_codeLine = function->codeLine;

		M_Word& m_word = function->m_word;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;
		W_Node* w_node = 0;

		for(int i = 0; i < m_codeLine.length; i++){
			if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(part->smls == smls){
						if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
							M_Int& cpIndex = morpher->cpIndex;
							M_ControlPoint& controlPoint = part->controlPoint;
							int s =0, d = 0;
							for(; s < cpIndex.length; s++){
								CPContigElement& contigElement = m_contigLookUpList[cpIndex[s]];
								if(contigElement.cpNode == part){
									cpIndex[d++] = contigElement.group;
								}
							}
							cpIndex.length = d;
							bubbleSort(cpIndex);
						}
					}
				}
			}
		}
	}
}

void Anim::deleteSection(){
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){
		if(selectFrom == selectTo){
			selectFrom = selectTo = slider;
		}

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;
		W_Node* w_node = 0;

		for(int i = 0; i < m_codeLine.length; i++){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				Node* node = w_dstField->node;
				Field* field = node->getField(w_dstField->fieldIdx);
				if(Art* part = dynamic_cast<Art*>(node)){
					if(OrientationInterpolator* interp = dynamic_cast<OrientationInterpolator*>(w_srcField->node)){
						if(rotationPartPass(part)){
							interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
						}
						else if(g_deleteOptions->time){
							interp->scaleTime((selectFrom - selectTo) * period, period);
						}
					}
					else if(PositionInterpolator* interp = dynamic_cast<PositionInterpolator*>(w_srcField->node)){
						if(translationPartPass(part)){
							interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
						}
						else if(g_deleteOptions->time){
							interp->scaleTime((selectFrom - selectTo) * period, period);
						}
					}
					else if(ScalarInterpolator* interp = dynamic_cast<ScalarInterpolator*>(w_srcField->node)){
						if(&part->fkb == field){
							if(fkbPartPass(part)){
								interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
							}
							else if(g_deleteOptions->time){
								interp->scaleTime((selectFrom - selectTo) * period, period);
							}
						}
						if(&part->fka == field){
							if(fkaPartPass(part)){
								interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
							}
							else if(g_deleteOptions->time){
								interp->scaleTime((selectFrom - selectTo) * period, period);
							}
						}
					}
				}
				if(Viewpoint* view = dynamic_cast<Viewpoint*>(w_dstField->node)){
					if(OrientationInterpolator* interp = dynamic_cast<OrientationInterpolator*>(w_srcField->node)){
						if(rotationViewPass(view)){
							interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
						}
						else if(g_deleteOptions->time){
							interp->scaleTime((selectFrom - selectTo) * period, period);
						}
					}
					else if(PositionInterpolator* interp = dynamic_cast<PositionInterpolator*>(w_srcField->node)){
						if(&view->lookAt == field){
							if(lookAtViewPass(view)){
								interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
							}
							else if(g_deleteOptions->time){
								interp->scaleTime((selectFrom - selectTo) * period, period);
							}
						}
						if(&view->up == field){
							if(upViewPass(view)){
								interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
							}
							else if(g_deleteOptions->time){
								interp->scaleTime((selectFrom - selectTo) * period, period);
							}
						}
						if(&view->position == field){
							if(positionViewPass(view)){
								interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
							}
							else if(g_deleteOptions->time){
								interp->scaleTime((selectFrom - selectTo) * period, period);
							}
						}
					}
					else if(ScalarInterpolator* interp = dynamic_cast<ScalarInterpolator*>(w_srcField->node)){
						if(&view->zoom == field){
							if(zoomViewPass(view)){
								interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
							}
							else if(g_deleteOptions->time){
								interp->scaleTime((selectFrom - selectTo) * period, period);
							}
						}
						if(&view->fieldOfView == field){
							if(fieldOfViewPass(view)){
								interp->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
							}
							else if(g_deleteOptions->time){
								interp->scaleTime((selectFrom - selectTo) * period, period);
							}
						}
					}
				}
			}
			else if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
						if(morphPartPass(part)){
							morpher->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
						}
						else if(g_deleteOptions->time){
							morpher->scaleTime((selectFrom - selectTo) * period, period);
						}
					}
				}
			}
			else if(getCalcFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_node->node)){
						if(morphPartPass(part)){
							niki->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
						}
						else if(g_deleteOptions->time){
							niki->scaleTime((selectFrom - selectTo) * period, period);
						}
					}
				}
			}
			else if(getCompFunction(m_word, w_node, w_part, m_codeLine, i) && g_timelineFilter->posCompensation){
				if(Art* part=dynamic_cast<Art*>(w_part->node)){
					if(PosCompensator* compensator = dynamic_cast<PosCompensator*>(w_node->node)){
						if(posCompensationPartPass(part)){
							compensator->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
						}
						else if(g_deleteOptions->time){
							compensator->scaleTime((selectFrom - selectTo) * period, period);
						}
					}
				}
			}
			else if(getNodeFieldEqualsSomething(m_word, w_dstField, m_codeLine, i)){
				if(CameraSelector* cameraSelector = dynamic_cast<CameraSelector*>(w_dstField->node)){
					if(g_timelineFilter->live){
						cameraSelector->deleteSection(selectFrom, selectTo, g_deleteOptions->beginEdge, g_deleteOptions->endEdge, g_deleteOptions->time);
					}
					else if(g_deleteOptions->time){
						cameraSelector->scaleTime((selectFrom - selectTo) * period, period);
					}
				}
			}
		}
	}
	if(Video* video = dynamic_cast<Video*>(this->video.p)){
		if(g_editVideoFrames){
			int frame = video->frame;
			int numOf =  (selectTo - selectFrom) * period / (1.0 / stepsPerSecond) + .5;
			if(numOf == 0){
				numOf = 5;
			}
			else{
				frame = selectFrom * period / (1.0 / stepsPerSecond) + .5;
			}
			video->deleteFrame(frame, numOf);
		}
	}
	if(g_deleteOptions->time){
		double diff = selectTo - selectFrom;
		double s = 0;
		if((1.0 - diff) > .000001){
			s = 1.0 / (1.0 - diff);
		}

		double add = diff * s;

		if(slider > selectTo){
			slider = slider - diff;
		}
		slider = slider * s;
		period = period - diff * period;
		quantizeSlider();
	}
	selectFrom = 0;
	selectTo = 0;

	updateDisplayFrom();
	removeTypeFromList < Button>(g_callBackList);
	removeTypeFromList < DownButton>(g_callBackList);
	g_stw->commonPostInsertNodeStuff();

	update = true;
	BackUp::monostable.beginCycle();
}


void CPMorpher::matchChannelLen(M_Int& srcCPIndex, int srcCPIndexOffset, int srcNumOfChannels){
	M_Float& key = this->key.refValue();
	M_Vec3f& keyValue = this->keyValue.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	int numOfChannels = cpIndex.length;
	for(int channel = 0, s = 0; s < srcNumOfChannels; s++){
		int k = srcCPIndex[srcCPIndexOffset + s];
		for(; channel < srcNumOfChannels; channel++){
			if(channel >= numOfChannels){
				insertChannel(channel, k, cpIndex, keyValue, numOfChannels, key.length);
				break;
			}
			int cpIdx = cpIndex[channel];
			if(cpIdx == k){
				channel++;
				break;
			}
			else if(cpIdx > k){
				insertChannel(channel, k, cpIndex, keyValue, numOfChannels, key.length);
				break;
			}
		}
	}
}

void Anim::pasteOrientationInterpolator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode){
	OrientationInterpolator* interp = 0;
	if(pass){
		if(interp = getOrientationInterpolator(node, field)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
		else if(interp = connectNodeToOrientationInterpolator(node, field, commonNameCode)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
	}
	else if(g_pasteOptions->insert2){
		if(interp = getOrientationInterpolator(node, field)){
			interp->scaleTime(g_pasteOptions->newPeriod - period, period);
		}
	}
	keyValueIndex += numOf;
	keyIndex += numOf;
	partIndex++;
}


void Anim::pastePositionInterpolator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode){
	PositionInterpolator* interp = 0;
	if(pass){
		if(interp = getPositionInterpolator(node, field)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
		else if(interp = connectNodeToPositionInterpolator(node, field, commonNameCode)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
	}
	else if(g_pasteOptions->insert2){
		if(interp = getPositionInterpolator(node, field)){
			interp->scaleTime(g_pasteOptions->newPeriod - period, period);
		}
	}
	keyValueIndex += numOf;
	keyIndex += numOf;
	partIndex++;
}

void Anim::pasteCameraSelector(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode){
	CameraSelector* interp = 0;
	if(pass){
		if(interp = getCameraSelector(node, field)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
		else if(interp = connectNodeToCameraSelector(node, field, commonNameCode)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
	}
	else if(g_pasteOptions->insert2){
		if(interp = getCameraSelector(node, field)){
			interp->scaleTime(g_pasteOptions->newPeriod - period, period);
		}
	}
	keyValueIndex += numOf;
	keyIndex += numOf;
	partIndex++;
}

void Anim::pasteScalarInterpolator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, Field* field, const int commonNameCode){
	ScalarInterpolator* interp = 0;
	if(pass){
		if(interp = getScalarInterpolator(node, field)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
		else if(interp = connectNodeToScalarInterpolator(node, field, commonNameCode)){
			interp->pasteSection(slider, period, node, keyIndex, keyValueIndex, numOf);
		}
	}
	else if(g_pasteOptions->insert2){
		if(interp = getScalarInterpolator(node, field)){
			interp->scaleTime(g_pasteOptions->newPeriod - period, period);
		}
	}
	keyValueIndex += numOf;
	keyIndex += numOf;
	partIndex++;
}

void Anim::pastePosCompensator(const bool pass, int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Art* part){
	PosCompensator* interp = 0;
	if(pass){
		if(interp = getPosCompensator(part)){
			interp->pasteSection(slider, period, part, keyIndex, keyValueIndex, numOf);
		}
		else if(interp = connectPartToPosCompensator(part)){
			interp->pasteSection(slider, period, part, keyIndex, keyValueIndex, numOf);
		}
	}
	else if(g_pasteOptions->insert2){
		if(interp = getPosCompensator(part)){
			interp->scaleTime(g_pasteOptions->newPeriod - period, period);
		}
	}
	keyValueIndex += numOf;
	keyIndex += numOf * 2;
	partIndex++;
}

void Anim::pasteCPMorpher(int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, int& numOfIdx, Art* part, Seamless* selsmls){
	CPMorpher* interp = 0;
	int srcNumOfChannels = g_animCopy.numOf[numOfIdx++];
	if(morphPartPass(part)){
		if(interp = getCPMorpher(part)){
			interp->matchChannelLen(g_animCopy.numOf, numOfIdx, srcNumOfChannels);
			interp->pasteSection(slider, period, part, keyIndex, keyValueIndex, numOf, srcNumOfChannels, g_animCopy.numOf, numOfIdx);
		}
		else if(interp = connectPartToCPMorpher(part)){
			M_Int& cpIndex = interp->cpIndex.refValue();
			cpIndex.length = srcNumOfChannels;
			//cpMorpher->keyValue.refValue().length = sampleLen * 2;
			for(int i = 0; i < srcNumOfChannels; i++){
				cpIndex[i] = g_animCopy.numOf[numOfIdx + i];
			}
			interp->pasteSection(slider, period, part, keyIndex, keyValueIndex, numOf, srcNumOfChannels, g_animCopy.numOf, numOfIdx);
		}
	}
	else if(g_pasteOptions->insert2){
		if(interp = getCPMorpher(part)){
			interp->scaleTime(g_pasteOptions->newPeriod - period, period);
		}
	}
	numOfIdx += srcNumOfChannels;
	keyValueIndex += numOf * srcNumOfChannels;
	keyIndex += numOf;
	partIndex++;
}

void Anim::pasteNikInterpolator(int& keyIndex, int& keyValueIndex, int& partIndex, const int numOf, Node* node, int& numOfIdx, Art* part, Seamless* selsmls){
	NikInterpolator* interp = 0;
	if(morphPartPass(part)){
		if(interp = getNikInterpolator(part)){
			interp->pasteSection(slider, period, part, keyIndex, keyValueIndex, numOf);
		}
		else if(interp = connectPartToNikInterpolator(part)){
			interp->pasteSection(slider, period, part, keyIndex, keyValueIndex, numOf);
		}
	}
	else if(g_pasteOptions->insert2){
		if(interp = getNikInterpolator(part)){
			interp->scaleTime(g_pasteOptions->newPeriod - period, period);
		}
	}
	
	keyValueIndex += numOf;
	keyIndex += numOf * interp->getSampleLength();
	partIndex++;
}

void PasteOptions::process(Anim* anim){
	int numOfSteps = (g_animCopy.selectTo - g_animCopy.selectFrom) * g_animCopy.period * g_animCopy.stepsPerSecond + .5;
	g_animCopy.selectedPeriod = static_cast<double>(numOfSteps) / g_animCopy.stepsPerSecond;
	g_animCopy.pasteSelectedPeriod = static_cast<double>(steps) / anim->stepsPerSecond;

	newPeriod = anim->period;
	frameDiffRatio = 1;
	if(numOfSteps){
		frameDiffRatio = g_animCopy.pasteSelectedPeriod / g_animCopy.selectedPeriod;
	}
	selectedPeriod = (g_animCopy.selectTo - g_animCopy.selectFrom) * g_animCopy.period;
	if(insert){
		newPeriod = selectedPeriod * frameDiffRatio + anim->period;
	}
	else{
		newPeriod = selectedPeriod * frameDiffRatio;
		if(selectedPeriod < anim->period){
			newPeriod = anim->period - selectedPeriod + newPeriod;
		}
	}


	int frames = anim->stepsPerSecond * newPeriod + .5;
	stepFrac = 1.0 / frames;

	selectedFrac = selectedPeriod / anim->period;
	selectedFrac2 = selectedFrac * frameDiffRatio;
	periodWarp = g_animCopy.pasteSelectedPeriod - g_animCopy.selectedPeriod;
	insert2 = insert;
	if(roughEQ(g_animCopy.selectedPeriod, 0)){
		insert2 = false;
	}
}
/*
void copyStraightForCurveFirst2(int keyIndex, int keyValueIndex, int numOf){
	if(NurbsTrack* nurbsTrack = dynamic_cast<NurbsTrack*>(g_copyNode.getRaw())){
		g_animCurve.positionInterpolator = new PositionInterpolator;

		M_Float& key = g_animCurve.positionInterpolator->key.refValue();
		M_Vec3f& keyValue = g_animCurve.positionInterpolator->keyValue.refValue();
		key.length = numOf;
		keyValue.length = numOf;

		for(int k = 0; k < numOf; k++){
			key[k] = g_animCopy.key[keyIndex + k];
			keyValue[k] = g_animCopy.pos[keyValueIndex + k];			
		}
		g_animCurve.processFields(nurbsTrack, (g_animCopy.selectTo - g_animCopy.selectFrom) * g_animCopy.period);
	}
}
*/
/*
void Anim::copyStraightForCurveFirst(){
	int keyIndex = 0;
	int keyRotIndex = 0;
	int keyPosIndex = 0;
	int keyScalarIndex = 0;
	int partIndex = 0;
	int numOfIdx = 0;

	for(int k = 0; k < g_animCopy.node.length; k++){
		int numOf = g_animCopy.numOf[numOfIdx++];
		Node* node = g_animCopy.node[partIndex];
		int fieldIdx = g_animCopy.field[partIndex];
		if(Art* part = dynamic_cast<Art*>(node)){
			if(fieldIdx == TimelineFilter::MORPH){
				int srcNumOfChannels = g_animCopy.numOf[numOfIdx++];
				numOfIdx += srcNumOfChannels;
				keyPosIndex += numOf * srcNumOfChannels;
				keyIndex += numOf;
				partIndex++;
				continue;
			}
			if(fieldIdx == TimelineFilter::POSCOMPENSATION){
				keyPosIndex += numOf;
				keyIndex += numOf * 2;
				partIndex++;
				continue;
			}
			Field* field = node->getField(fieldIdx);
			if(&part->rotation == field || &part->ikRot == field){
				keyRotIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&part->translation == field || &part->ikPos == field){
				if(part == g_selNode.getRaw()){
					copyStraightForCurveFirst2(keyIndex, keyPosIndex, numOf);
				}
				keyPosIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&part->fkb == field || &part->fka == field){
				keyScalarIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
		}
		else if(Viewpoint* view = dynamic_cast<Viewpoint*>(node)){
			Field* field = node->getField(fieldIdx);
			if(&view->orientation == field){
				keyRotIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&view->position == field){
				keyPosIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&view->zoom == field || &view->fieldOfView == field){
				keyScalarIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
		}
	}
}
*/
/*
void Anim::insertTimeForMissingMirrorNode(Node* node, Art* part, int fieldIdx, bool beginEdge){
	if(fieldIdx == TimelineFilter::MORPH){
		if(morphPartPass(part)){
			if(SamplePose* samplePose = getCPMorpher(part)){
				samplePose->insertTime(slider, slider + g_pasteOptions->selectedFrac2, beginEdge);
			}
		}
		return;
	}
	if(fieldIdx == TimelineFilter::POSCOMPENSATION){
		if(posCompensationPartPass(part)){
			if(SamplePose* samplePose = getPosCompensator(part)){
				samplePose->insertTime(slider, slider + g_pasteOptions->selectedFrac2, beginEdge);
			}
		}
		return;
	}
	Field* field = node->getField(fieldIdx);
	if(&part->rotation == field || &part->ikRot == field){
		if(rotationPartPass(part)){
			if(SamplePose* samplePose = getOrientationInterpolator(part, field)){
				samplePose->insertTime(slider, slider + g_pasteOptions->selectedFrac2, beginEdge);
			}
		}
	}
	else if(&part->translation == field || &part->ikPos == field){
		if(translationPartPass(part)){
			if(SamplePose* samplePose = getPositionInterpolator(part, field)){
				samplePose->insertTime(slider, slider + g_pasteOptions->selectedFrac2, beginEdge);
			}
		}
	}
	else if(&part->fkb == field){
		if(fkbPartPass(part)){
			if(SamplePose* samplePose = getScalarInterpolator(part, field)){
				samplePose->insertTime(slider, slider + g_pasteOptions->selectedFrac2, beginEdge);
			}
		}
	}
	else if(&part->fka == field){
		if(fkaPartPass(part)){
			if(SamplePose* samplePose = getScalarInterpolator(part, field)){
				samplePose->insertTime(slider, slider + g_pasteOptions->selectedFrac2, beginEdge);
			}
		}
	}
}
*/
bool nodeFieldInList(Node* node, int field, M_Node& m_node, M_Int& m_field){
	for(int k = m_node.length; --k>=0;){
		if(node == m_node[k] && field == m_field[k]){
			return true;
		}
	}
	return false;
}

bool isNodeInAnimCopy(Node* compareNode){
	int keyIndex = 0;
	int keyRotIndex = 0;
	int keyPosIndex = 0;
	int keyScalarIndex = 0;
	int partIndex = 0;
	int numOfIdx = 0;

	for(int k = 0; k < g_animCopy.node.length; k++){
//		int numOf = g_animCopy.numOf[numOfIdx++];
		Node* node = g_animCopy.node[partIndex++];
		if(Art* part = dynamic_cast<Art*>(node)){
			if(g_pasteOptions->mirror && part->mirrorPart){
				if(compareNode == part->mirrorPart){
					return true;
				}
			}
			else if(node == compareNode){
				return true;
			}
		}
		else if(node == compareNode){
			return true;
		}
/*			
		int fieldIdx = g_animCopy.field[partIndex];
		if(Art* part = dynamic_cast<Art*>(node)){
			if(fieldIdx == TimelineFilter::MORPH){
				int srcNumOfChannels = g_animCopy.numOf[numOfIdx++];
				numOfIdx += srcNumOfChannels;
				keyPosIndex += numOf * srcNumOfChannels;
				keyIndex += numOf;
				partIndex++;
				continue;
			}
			if(fieldIdx == TimelineFilter::POSCOMPENSATION){
				keyPosIndex += numOf;
				keyIndex += numOf * 2;
				partIndex++;
				continue;
			}
			Field* field = node->getField(fieldIdx);
			if(&part->rotation == field || &part->ikRot == field){
				keyRotIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&part->translation == field || &part->ikPos == field){
				keyPosIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&part->fkb == field){
				keyScalarIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&part->fka == field){
				keyScalarIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
		}
		else if(Viewpoint* view = dynamic_cast<Viewpoint*>(node)){
			Field* field = node->getField(fieldIdx);
			if(&view->orientation == field){
				keyRotIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&view->position == field){
				keyPosIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&view->zoom == field){
				keyScalarIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
			else if(&view->fieldOfView == field){
				keyScalarIndex += numOf;
				keyIndex += numOf;
				partIndex++;
			}
		}
		*/
	}
	return false;
}

void Anim::prePasteProcessTrack(){
	if(MemberFunction* function=dynamic_cast<MemberFunction*>(onFrame.p)){
		if(selectFrom == selectTo){
			selectFrom = selectTo = slider;
		}

		M_Node& m_codeLine=function->codeLine;

		M_Word& m_word=function->m_word;
		W_Field* w_dstField=0;
		W_Field* w_srcField=0;
		W_Node* w_morpher = 0;
		W_Node* w_part = 0;
		W_Node* w_node = 0;

		for(int i = 0; i < m_codeLine.length; i++){
			if(getEquiveRoute(m_word, w_dstField, w_srcField, m_codeLine, i)){
				Node* node = w_dstField->node;
				if(isNodeInAnimCopy(node)){
					continue;
				}
				Field* field = node->getField(w_dstField->fieldIdx);
				if(Art* part = dynamic_cast<Art*>(node)){
					if(OrientationInterpolator* interp = dynamic_cast<OrientationInterpolator*>(w_srcField->node)){
						if(part->track && part->smls->track && g_pasteOptions->insert){
							interp->insertTime(slider, slider + g_pasteOptions->selectedFrac2, g_pasteOptions->beginEdge);
						}
						else{
							interp->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
					else if(PositionInterpolator* interp = dynamic_cast<PositionInterpolator*>(w_srcField->node)){
						if(part->track && part->smls->track && g_pasteOptions->insert){
							interp->insertTime(slider, slider + g_pasteOptions->selectedFrac2, g_pasteOptions->beginEdge);
						}
						else{
							interp->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
					else if(ScalarInterpolator* interp = dynamic_cast<ScalarInterpolator*>(w_srcField->node)){
						if(part->track && part->smls->track && g_pasteOptions->insert){
							interp->insertTime(slider, slider + g_pasteOptions->selectedFrac2, g_pasteOptions->beginEdge);
						}
						else{
							interp->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
				}
				if(Viewpoint* view = dynamic_cast<Viewpoint*>(w_dstField->node)){
					if(OrientationInterpolator* interp = dynamic_cast<OrientationInterpolator*>(w_srcField->node)){
						if(view->track && g_pasteOptions->insert){
							interp->insertTime(slider, slider + g_pasteOptions->selectedFrac2, g_pasteOptions->beginEdge);
						}
						else{
							interp->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
					else if(PositionInterpolator* interp = dynamic_cast<PositionInterpolator*>(w_srcField->node)){
						if(view->track && g_pasteOptions->insert){
							interp->insertTime(slider, slider + g_pasteOptions->selectedFrac2, g_pasteOptions->beginEdge);
						}
						else{
							interp->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
					else if(ScalarInterpolator* interp = dynamic_cast<ScalarInterpolator*>(w_srcField->node)){
						if(view->track && g_pasteOptions->insert){
							interp->insertTime(slider, slider + g_pasteOptions->selectedFrac2, g_pasteOptions->beginEdge);
						}
						else{
							interp->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
				}
			}
			else if(getMorphFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(CPMorpher* morpher = dynamic_cast<CPMorpher*>(w_node->node)){
						if(part->track && g_pasteOptions->insert){
							morpher->insertTime(selectFrom, selectTo, g_pasteOptions->beginEdge);
						}
						else{
							morpher->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
				}
			}
			else if(getCalcFunction(m_word, w_node, w_part, m_codeLine, i)){
				if(Art* part = dynamic_cast<Art*>(w_part->node)){
					if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(w_node->node)){
						if(part->track && g_pasteOptions->insert){
							niki->insertTime(selectFrom, selectTo, g_pasteOptions->beginEdge);
						}
						else{
							niki->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
				}
			}
			else if(getCompFunction(m_word, w_node, w_part, m_codeLine, i) && g_timelineFilter->posCompensation){
				if(Art* part=dynamic_cast<Art*>(w_part->node)){
					if(PosCompensator* compensator = dynamic_cast<PosCompensator*>(w_node->node)){
						if(part->track && g_pasteOptions->insert){
							compensator->insertTime(selectFrom, selectTo, g_pasteOptions->beginEdge);
						}
						else{
							compensator->scaleTime(g_pasteOptions->newPeriod - period, period);
						}
					}
				}
			}
		}
	}
}


void Anim::pasteSection(){
	Seamless* selsmls = 0;
	int keyIndex = 0;
	int keyRotIndex = 0;
	int keyPosIndex = 0;
	int keyScalarIndex = 0;
	int keyBooleanIndex = 0;
	int partIndex = 0;
	int numOfIdx = 0;

	g_pasteOptions->process(this);

	//if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
	//	part->smls->scaleSkeleton(g_pasteOptions->pivotPoint, g_pasteOptions->scale);
	//}

	prePasteProcessTrack();

	for(int k = 0; k < g_animCopy.node.length; k++){
		int numOf = g_animCopy.numOf[numOfIdx++];
		Node* node = g_animCopy.node[partIndex];
		int fieldIdx = g_animCopy.field[partIndex];
		if(Art* part = dynamic_cast<Art*>(node)){
			if(g_pasteOptions->mirror && part->mirrorPart){
//				if(g_pasteOptions->insert2 && nodeFieldInList(part->mirrorPart, fieldIdx, g_animCopy.node, g_animCopy.field) == false){
//					insertTimeForMissingMirrorNode(node, part, fieldIdx, g_pasteOptions->beginEdge);
//				}
				part = part->mirrorPart;
				node = part;
			}
			if(fieldIdx == TimelineFilter::MORPH){
				pasteCPMorpher(keyIndex, keyPosIndex, partIndex, numOf, node, numOfIdx, part, selsmls);
				continue;
			}
			if(fieldIdx == TimelineFilter::NIK){
				pasteNikInterpolator(keyIndex, keyPosIndex, partIndex, numOf, node, numOfIdx, part, selsmls);
				continue;
			}
			if(fieldIdx == TimelineFilter::POSCOMPENSATION){
				pastePosCompensator(posCompensationPartPass(part), keyIndex, keyPosIndex, partIndex, numOf, part);
				continue;
			}
			Field* field = node->getField(fieldIdx);
			if(&part->rotation == field || &part->ikRot == field){
				pasteOrientationInterpolator(rotationPartPass(part), keyIndex, keyRotIndex, partIndex, numOf, node, field, _ROTATION_);
			}
			else if(&part->translation == field || &part->ikPos == field){
				pastePositionInterpolator(translationPartPass(part), keyIndex, keyPosIndex, partIndex, numOf, node, field, _POSITION_);
			}
			else if(&part->fkb == field){
				pasteScalarInterpolator(fkbPartPass(part), keyIndex, keyScalarIndex, partIndex, numOf, node, field, _FB_);
			}
			else if(&part->fka == field){
				pasteScalarInterpolator(fkaPartPass(part), keyIndex, keyScalarIndex, partIndex, numOf, node, field, _FA_);
			}
		}
		else if(Viewpoint* view = dynamic_cast<Viewpoint*>(node)){
			Field* field = node->getField(fieldIdx);
			if(&view->zoom == field){
				pasteScalarInterpolator(zoomViewPass(view), keyIndex, keyScalarIndex, partIndex, numOf, node, field, _ZOOM_);
			}
			else if(&view->orientation == field){
				pasteOrientationInterpolator(rotationViewPass(view), keyIndex, keyRotIndex, partIndex, numOf, node, field, _ORIENTATION_);
			}
			else if(&view->lookAt == field){
				pastePositionInterpolator(lookAtViewPass(view), keyIndex, keyPosIndex, partIndex, numOf, node, field, _LOOK_AT_);
			}
			else if(&view->up == field){
				pastePositionInterpolator(upViewPass(view), keyIndex, keyPosIndex, partIndex, numOf, node, field, _UP_);
			}
			else if(&view->position == field){
				pastePositionInterpolator(positionViewPass(view), keyIndex, keyPosIndex, partIndex, numOf, node, field, _POSITION_);
			}
			else if(&view->fieldOfView == field){
				pasteScalarInterpolator(fieldOfViewPass(view), keyIndex, keyScalarIndex, partIndex, numOf, node, field, _FOV_);
			}
		}
		else if(CameraSelector* cameraSelector = dynamic_cast<CameraSelector*>(node)){
			Field* field = node->getField(fieldIdx);
			pasteCameraSelector(g_timelineFilter->live, keyIndex, keyBooleanIndex, partIndex, numOf, node, field, _LIVE_);
		}
	}
	if(Video* video = dynamic_cast<Video*>(this->video.p)){
		if(g_editVideoFrames){
			video->repeatFrame(video->frame, g_pasteOptions->steps);
		}
	}

	removeTypeFromList < Button>(g_callBackList);
	removeTypeFromList < DownButton>(g_callBackList);
	g_stw->commonPostInsertNodeStuff();
	

	slider = slider * (period / g_pasteOptions->newPeriod);
	period = g_pasteOptions->newPeriod;
	selectTo = 0;
	selectFrom = 0;
	updateDisplayFrom();

	update = true;
	BackUp::monostable.beginCycle();
}
