/* Seamless3d

Copyright (C) 2001-2013 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/

#include <windows.h>
#include <direct.h>

#include <sys/stat.h>
#include <share.h>
#include <fcntl.h>
#include <io.h>

#include <algorithm>    // std::max
using namespace std;

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "seamless3d_gui_render.h"
#include "gui.h"
#include "scene_tree.h"
#include "robot_aid.h"

bool clickLine(Vec2f click, Vec2f ptA, Vec2f ptB, Vec2f& crossPoint);
void processAllPatches(M_<NurbsPatch*>& cpNodeList);
void commonPostTug();
int getEdgeInnerPoint(NurbsPatch* patch, int idx);
void getOppositeEdgePoint(NurbsPatch* patch, int idx, CornerEdge& ce);
void calcMidPoints(const int i, const M_CtrlPolyPoint& ctrlPolyPoint);
static M_<CornerEdge> m_cornerEdge;

void makeStitchPointsPlanar(Seamless* smls, const M_CtrlPolyPoint& ctrlPolyPoint){
	
	for(int i = 0; i < ctrlPolyPoint.length; ){
		int h = i;
		const int len = ctrlPolyPoint[i].len;
		if(len < 5){
			calcMidPoints(i, ctrlPolyPoint);
			i += len;
			for(int k = len + 1; --k >= 0;){
				int midLen = ctrlPolyPoint[i].len;
				if(midLen == 0){
					midLen++;
				}
				i += midLen;
			}
			continue;
		}
		int centerIdx = i + len;

		for(int k = len; --k >= 0;){
			centerIdx += ctrlPolyPoint[centerIdx].len;
		}

		Vec3f a = ctrlPolyPoint[centerIdx].p->dpoint[ctrlPolyPoint[centerIdx].i];

		Vec3f centerNormal;

		for(int q = len; --q >= 0;){
			Vec3f b = ctrlPolyPoint[h].p->dpoint[ctrlPolyPoint[h].i];

			if(++h >= i + len){
				h = i;
			}
			Vec3f c = ctrlPolyPoint[h].p->dpoint[ctrlPolyPoint[h].i];

			centerNormal += calcNormal(a, b, c);
		}
/*
		int i = stitchIdxIdx[sii].index;
		int numOf = stitchIdxIdx[sii].numOf;
		int cornerLen = numOf - 1;
		m_cornerEdge.length = 0;

		for(int k = 1; k < numOf; k++){
			int w = i + k;
			int innerPointIdx = getEdgeInnerPoint(patch, ctrlPolyPoint[w].i);
			if(idx == innerPointIdx){
				Vec3f centerNormal;

				for(int j = 0; j < numOf; j++){
					CornerEdge ce;
					int w = i + j;
					getOppositeEdgePoint(ctrlPolyPoint[w].p, ctrlPolyPoint[w].i, ce);
					m_cornerEdge += ce;
				}

				Vec3f cornerPoint = m_cornerEdge[0].patch->dpoint.refValue()[m_cornerEdge[0].cornerIdx];

				for(int j = 0; j < cornerLen; j++){
					int w = i + j;
					Vec3f normal;
					Vec3f v1 = m_cornerEdge[j].patch->dpoint.refValue()[m_cornerEdge[j].edgeIdx];
					Vec3f v2 = m_cornerEdge[j].patch->dpoint.refValue()[m_cornerEdge[j].oppositeEdgeIdx];
					calcNormal(cornerPoint, v1, v2, normal);
					centerNormal += normal;
				}
*/

		centerNormal = centerNormal.normalize();
		Rotation rot = Rotation(centerNormal, Vec3f(0, 1, 0));
		a = rot * a;

		for(int q = len, h = i; --q >= 0; h++){
			NurbsPatch* p = ctrlPolyPoint[h].p;
			int idx = ctrlPolyPoint[h].i;

			const Vec3f v0 = p->dpoint[idx];
			Vec3f v1 = rot * v0;
			v1.y = a.y;
			v1 = -rot * v1;
			if(	abs(v1.x - v0.x) > .00001 ||
				abs(v1.y - v0.y) > .00001 ||
				abs(v1.z - v0.z) > .00001){

				for(int q = len, h = i; --q >= 0; h++){
					NurbsPatch* p = ctrlPolyPoint[h].p;
					int idx = ctrlPolyPoint[h].i;

					Vec3f v = rot * p->dpoint[idx];
					v.y = a.y;
					v = -rot * v;
					p->point.refValue()[idx] = v * p->weight[idx];
				}
					
				break;
			}
		}

		calcMidPoints(i, ctrlPolyPoint);
		i = centerIdx + ctrlPolyPoint[centerIdx].len;
	}
}

void makeStitchPointsPlanar(){
	if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		Seamless* smls = nurbsPatch->smls;
		makeStitchPointsPlanar(smls, smls->ctrlPolyPoint);
		processAllPatches(smls->cpNodeList);
//		nurbsPatch->houseKeep();
		smls->doBuild();
	}
	else if(Art* part=dynamic_cast<Art*>(g_selNode.getRaw())){

		if(MultiSelect::pointList.length){
			for(int i = MultiSelect::pointList.length; --i>=0;){
				int idx = MultiSelect::pointList[i].i;
				NurbsPatch* patch = MultiSelect::pointList[i].p;
				Seamless* smls = part->smls;
				makeStitchPointsPlanar(smls, smls->ctrlPolyPoint);
				processAllPatches(smls->cpNodeList);
				commonPostTug();
				smls->updateCage();
			}
		}
	}
	updateBuild3dWindow();
	BackUp::monostable.beginCycle();
}

/*
int getEdgeInnerPoint(NurbsPatch* patch, int idx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int u = idx % uDim;
	int v = idx / uDim;
	//which edge?
	if(u == 0){//left edge
		return idx + 1;
	}
	else if(u == uDim - 1){//right edge
		return idx - 1;
	}
	else if(v == 0){//south edge
		return idx + uDim;
	}
	else if(v == vDim - 1){//north edge
		return idx - uDim;
	}
	return 0;
}

void getOppositeEdgePoint(NurbsPatch* patch, int idx, CornerEdge& ce){
	ce.patch=patch;
	ce.edgeIdx=idx;

	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int u = idx % uDim;
	int v = idx / uDim;

	//which edge?
	if(u == 0){//left edge
		ce.innerPointIdx = idx + 1;
		if(v == 1){
			ce.oppositeEdgeIdx = ce.innerPointIdx - uDim;
			ce.cornerIdx = 0;
		}
		else{
			ce.oppositeEdgeIdx = ce.innerPointIdx + uDim;
			ce.cornerIdx = uDim * (vDim - 1);
		}
	}
	else if(u == uDim - 1){//right edge
		ce.innerPointIdx = idx - 1;
		if(v == 1){
			ce.oppositeEdgeIdx = ce.innerPointIdx - uDim;
			ce.cornerIdx = uDim - 1;
		}
		else{
			ce.oppositeEdgeIdx = ce.innerPointIdx + uDim;
			ce.cornerIdx = uDim * vDim - 1;
		}
	}
	else if(v == 0){//south edge
		ce.innerPointIdx = idx + uDim;
		if(u == 1){
			ce.oppositeEdgeIdx = ce.innerPointIdx - 1;
			ce.cornerIdx = 0;
		}
		else{
			ce.oppositeEdgeIdx = ce.innerPointIdx + 1;
			ce.cornerIdx = uDim - 1;
		}
	}
	else if(v == vDim - 1){//north edge
		ce.innerPointIdx = idx - uDim;
		if(u == 1){
			ce.oppositeEdgeIdx = ce.innerPointIdx - 1;
			ce.cornerIdx = uDim * (vDim - 1);
		}
		else{
			ce.oppositeEdgeIdx = ce.innerPointIdx + 1;
			ce.cornerIdx = uDim * vDim - 1;
		}
	}
}
*/

void deleteControlPointFromAllMorphers(Art* part, int i);

void Seamless::removeOwnershipPatch(const NurbsPatch* patch){
	for(int k = partList.length; --k>=0;){
		Art* part = partList[k];
		M_ControlPoint& controlPoint = part->controlPoint;
		for(int i = controlPoint.length; --i>=0;){
			if(controlPoint[i].p == patch){
				deleteControlPointFromAllMorphers(part, i);
				controlPoint.deleteElement(i);
			}
		}
	}
}
void Seamless::removeMidPointsOwnership(const NurbsPatch* patch, const int idx){
	for(int k = partList.length; --k>=0;){
		M_ControlPoint& controlPoint = partList[k]->controlPoint;
		for(int i = controlPoint.length; --i>=0;){
			if(controlPoint[i].p == patch && controlPoint[i].i == idx){
				controlPoint.deleteElement(i);
			}
		}
	}
}

void Seamless::removeMidPointsOwnership(const int i, const M_CtrlPolyPoint& ctrlPolyPoint){
	const int len = ctrlPolyPoint[i].len;
	const int midIdx = i + len;
	int g = midIdx;
	int h = i;
	for(int q = 0; q < len; q++){

		if(++h >= i + len){
			h = i;
		}
		int midLen = ctrlPolyPoint[g].len;
		if(midLen){
			for(int m = midLen; --m >= 0;){
				NurbsPatch* p = ctrlPolyPoint[g].p;
				int idx = ctrlPolyPoint[g++].i;
				removeMidPointsOwnership(p, idx);
			}
		}
		else{
			g++;
		}
	}

	int centerLen = ctrlPolyPoint[g].len;
	for(int q = centerLen; --q >= 0;){
		NurbsPatch* p = ctrlPolyPoint[g].p;
		int idx = ctrlPolyPoint[g++].i;
		removeMidPointsOwnership(p, idx);
	}
}

void Seamless::removeAllPolyMidPointsOwnership(){
	for(int i = 0; i < ctrlPolyPoint.length;){
		removeMidPointsOwnership(i, ctrlPolyPoint);
		i = getNextCtrlPoly(i);
	}
	processAllPatches(cpNodeList);
}


void calcMidPoints(const int i, const M_CtrlPolyPoint& ctrlPolyPoint){
	const int len = ctrlPolyPoint[i].len;
	const int midIdx = i + len;
	int g = midIdx;
	int h = i;
	Vec3f centerPoint;
	//mid points
	for(int q = 0; q < len; q++){
		//NurbsPatch* patch = ctrlPolyPoint[h].p;
		//int idx = ctrlPolyPoint[h].i;
		//Vec3f v = patch->dpoint.refValue()[idx] = patch->point[idx] / patch->weight[idx];

		Vec3f v = ctrlPolyPoint[h].p->dpoint[ctrlPolyPoint[h].i];
		centerPoint += v;

		if(++h >= i + len){
			h = i;
		}
		int midLen = ctrlPolyPoint[g].len;
		if(midLen){
			Vec3f midPoint = (v + ctrlPolyPoint[h].p->dpoint[ctrlPolyPoint[h].i]) / 2.0f;

			for(int m = midLen; --m >= 0;){
				NurbsPatch* p = ctrlPolyPoint[g].p;
				int idx = ctrlPolyPoint[g++].i;
#ifdef CPINDEX
				p->smls->cp.refValue()[p->cpIndex[idx]] = p->point.refValue()[idx] = midPoint * p->weight[idx];
#endif
				p->point.refValue()[idx] = midPoint * p->weight[idx];
			}
		}
		else{
			g++;
		}
	}
	centerPoint /= len;

	int centerLen = ctrlPolyPoint[g].len;
	for(int q = centerLen; --q >= 0;){
		NurbsPatch* p = ctrlPolyPoint[g].p;
		int idx = ctrlPolyPoint[g++].i;
#ifdef CPINDEX
		p->smls->cp.refValue()[p->cpIndex[idx]] = p->point.refValue()[idx] = centerPoint * p->weight[idx];
#endif
		p->point.refValue()[idx] = centerPoint * p->weight[idx];
	}
}


void Seamless::dragCtrlPoly(NurbsPatch* patch, int idx){

	for(int i = 0; i < ctrlPolyPoint.length;){
		int selCtrlPoly = i;
		int len = ctrlPolyPoint[selCtrlPoly].len;
		for(int k = len; --k >= 0;){
			if(patch == ctrlPolyPoint[i].p && idx == ctrlPolyPoint[i].i){
				calcMidPoints(selCtrlPoly, ctrlPolyPoint);
				break;
			}
			i++;
		}
		i = selCtrlPoly + len;
		for(int k = len + 1; --k >= 0;){
			int midLen = ctrlPolyPoint[i].len;
			if(midLen == 0){
				midLen++;
			}
			i += midLen;
		}
	}
	processAllPatches(cpNodeList);
}

void Seamless::decodeStitchIndex(M_<NurbsPatch*>& contigLookUpList){
	M_Int& stitchIndex = this->stitchIndex.refValue();
	for(int i = 0; i < stitchIndex.length;){

		CtrlPolyPoint cpp;
		cpp.len = stitchIndex[i++];

		for(int q = cpp.len; --q >= 0;){
			int b = stitchIndex[i++];
			cpp.p = contigLookUpList[b];
			cpp.i = b - cpp.p->fromPoint;
			ctrlPolyPoint += cpp;
		}
		//mid points
		for(int q = cpp.len; --q >= 0;){
			if(cpp.len = stitchIndex[i++]){
				for(int m = cpp.len; --m >= 0;){
					int b = stitchIndex[i++];
					cpp.p = contigLookUpList[b];
					cpp.i = b - cpp.p->fromPoint;
					ctrlPolyPoint += cpp;
				}
			}
			else{
				ctrlPolyPoint += cpp;
			}
		}
		//center points
		if(cpp.len = stitchIndex[i++]){
			for(int m = cpp.len; --m >= 0;){
				int b = stitchIndex[i++];
				cpp.p = contigLookUpList[b];
				cpp.i = b - cpp.p->fromPoint;
				ctrlPolyPoint += cpp;
			}
		}
		else{
			ctrlPolyPoint += cpp;
		}
	}
}

bool isFixed(NurbsPatch* patch, int idx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int u = idx % uDim;
	int v = idx / uDim;
	if(u % 3 == 0){
		return true;
	}
	if(v % 3 == 0){
		return true;
	}
	return false;
}

void Seamless::createControlPolygon(M_ControlPoint& pointList){
	MultiSelect::ctrlPolySelList.length = 0;
	MultiSelect::ctrlPolySelList += ctrlPolyPoint.length;

	M_Vec2f screenPoint;
	screenPoint.length = pointList.length;
	const bool perspective = g_perspective;

	for(int k = pointList.length; --k >=0;){
		screenPoint[k] = procPerspective(perspective, g_viewMatrix * pointList[k].p->dpoint[pointList[k].i]);
	}


	Vec2f center;

	for(int k = pointList.length; --k >=0;){
		center += screenPoint[k];
	}
	center /= pointList.length;

	M_Float angle;
	angle.length = pointList.length;
	for(int k = pointList.length; --k >=0;){
		angle[k] = (screenPoint[k] - center).getAngle3();
	}

	M_ControlPoint orderedPoint;

	while(pointList.length){
		float mostVal = -1;
		int mostIdx = 0;
		for(int k = pointList.length; --k >=0;){
			if(angle[k] > mostVal){
				mostVal = angle[k];
				mostIdx = k;
			}
		}
		orderedPoint += pointList[mostIdx];
		pointList.deleteElement(mostIdx);
		angle.deleteElement(mostIdx);
	}

	int len = orderedPoint.length;
	for(int k = 0; k < len; k++){
		ctrlPolyPoint += CtrlPolyPoint(orderedPoint[k].p, orderedPoint[k].i, len);
	}
	ctrlPolyPoint.length += len + 1;


	processAllPatches(cpNodeList);
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}

void createMidPointList(const int i, const M_CtrlPolyPoint& ctrlPolyPoint, M_Vec3f& midPoint){
	const int len = ctrlPolyPoint[i].len;
	midPoint.length = 0;
	int h = i;
	Vec3f centerPoint;
	//mid points
	for(int q = 0; q < len; q++){
		Vec3f v = ctrlPolyPoint[h].p->dpoint[ctrlPolyPoint[h].i];
		centerPoint += v;

		if(++h >= i + len){
			h = i;
		}
		midPoint += (v + ctrlPolyPoint[h].p->dpoint[ctrlPolyPoint[h].i]) / 2.0f;
	}
	midPoint += centerPoint / len;
}

void insertMidPoint(M_CtrlPolyPoint& ctrlPolyPoint, int ctrlPolyIdx, NurbsPatch* patch, int idx, int shortestK){
	int midPointIdx = ctrlPolyIdx + ctrlPolyPoint[ctrlPolyIdx].len;
	for(int m = 0; ; m++){
		if(m == shortestK){
			break;
		}
		int midLen = ctrlPolyPoint[midPointIdx].len;
		if(midLen == 0){
			midLen++;
		}
		midPointIdx += midLen;
	}

	int newLen = ctrlPolyPoint[midPointIdx].len + 1;

	if(newLen == 1){
		ctrlPolyPoint[midPointIdx] = CtrlPolyPoint(patch, idx, newLen);
	}
	else{
		ctrlPolyPoint.insertElement(midPointIdx, CtrlPolyPoint(patch, idx, newLen));
	}
}

void insertMidPoint(M_CtrlPolyPoint& ctrlPolyPoint, int midPointIdx, NurbsPatch* patch, int idx){
	int newLen = ctrlPolyPoint[midPointIdx].len + 1;

	if(newLen == 1){
		ctrlPolyPoint[midPointIdx] = CtrlPolyPoint(patch, idx, newLen);
	}
	else{
		ctrlPolyPoint.insertElement(midPointIdx, CtrlPolyPoint(patch, idx, newLen));
	}
}

void Seamless::addMidPoint(M_ControlPoint& pointList){
	M_Vec3f midPoint;
	int selCtrlPoly = MultiSelect::ctrlPolySelList[0];

	createMidPointList(selCtrlPoly, ctrlPolyPoint, midPoint);
	int midPointIdx = selCtrlPoly + ctrlPolyPoint[selCtrlPoly].len;

	for(int q = 0; q < pointList.length; q++){
		float shortest = 10000000;
		int shortestK = 0;
		int shortestQ = 0;
	
		for(int k = 0; k < midPoint.length; k++){
			NurbsPatch* patch = pointList[q].p;
			int idx = pointList[q].i;
			float distance = (midPoint[k] - patch->dpoint[idx]).length();
			if(distance < shortest){
				shortest = distance;
				shortestK = k;
				shortestQ = q;
			}
		}
		insertMidPoint(ctrlPolyPoint, selCtrlPoly, pointList[shortestQ].p, pointList[shortestQ].i, shortestK);
	}

	pointList.length = 0;
	calcMidPoints(selCtrlPoly, ctrlPolyPoint);
	//removeAllPolyMidPointsOwnership();
	processAllPatches(cpNodeList);
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}

void Seamless::stitchAllPolys(){
	if(autoStitch){
		MultiSelect::ctrlPolySelList .length = 0;
		for(int poly = 0; poly < ctrlPolyPoint.length; ){
			MultiSelect::ctrlPolySelList += poly;
			poly = getNextCtrlPoly(poly);
		}
		joinAllSelectedStitchPolys();
	}
	else{
		for(int k = this->cpNodeList.length; --k>=0;){
			NurbsPatch* patch = cpNodeList[k];
			patch->processFields();
		}
	}
}

bool NurbsPatch::isInStitchPoly(int poly, const M_CtrlPolyPoint& ctrlPolyPoint) const{
	int len = ctrlPolyPoint[poly].len;
	int midIndex = poly + len;

	for(int k = len; --k >= 0;){
		int idxA = poly + k;
		if(ctrlPolyPoint[idxA].p == this){
			return true;
		}
	}
	poly += len;
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		if(midLen == 0){
			midLen++;
		}
		else{
			for(int k = midLen; --k >= 0;){
				int idxA = poly + k;
				if(ctrlPolyPoint[idxA].p == this){
					return true;
				}
			}
		}
		poly += midLen;
	}
	return false;
}

bool NurbsPatch::isInStitchPoly(int idx, int poly, const M_CtrlPolyPoint& ctrlPolyPoint) const{
	int len = ctrlPolyPoint[poly].len;
	int midIndex = poly + len;

	for(int k = len; --k >= 0;){
		int idxA = poly + k;
		if(ctrlPolyPoint[idxA].p == this && ctrlPolyPoint[idxA].i == idx){
			return true;
		}
	}
	poly += len;
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		if(midLen == 0){
			midLen++;
		}
		else{
			for(int k = midLen; --k >= 0;){
				int idxA = poly + k;
				if(ctrlPolyPoint[idxA].p == this && ctrlPolyPoint[idxA].i == idx){
					return true;
				}
			}
		}
		poly += midLen;
	}
	return false;
}

void Seamless::calcAllPolyMidPointsAndpatchToCentralCPs(NurbsPatch* patch, int numOfPatches){
	int plidx = patch->plidx;
	calcAllPolyMidPoints();

	for(int k = 0; k < numOfPatches; k++){
		NurbsPatch* patch = cpNodeList[plidx + k];
		patch->patchToCentralCPs();
	}
}

void Seamless::stitchSinglePatchPolys(NurbsPatch* patch, int numOfPatches){
	MultiSelect::ctrlPolySelList.length = 0;
	int plidx = patch->plidx;
	for(int k = 0; k < numOfPatches; k++){
		NurbsPatch* p = cpNodeList[plidx + k];
		for(int poly = 0; poly < ctrlPolyPoint.length; ){
			if(p->isInStitchPoly(poly, ctrlPolyPoint)){
				MultiSelect::ctrlPolySelList += poly;
			}
			poly = getNextCtrlPoly(poly);
		}
	}
	joinAllSelectedStitchPolys();

	//calcAllPolyMidPointsAndpatchToCentralCPs(patch, numOfPatches);
}

void encodeStitchIndex(M_Int& stitchIndex, const M_CtrlPolyPoint& ctrlPolyPoint){
	stitchIndex.length = 0;
	for(int k = 0; k < ctrlPolyPoint.length; ){
		int len = ctrlPolyPoint[k].len;
		stitchIndex += len;
		if(len){
			for(int g = 0; g < len; g++){
				stitchIndex += ctrlPolyPoint[k].i + ctrlPolyPoint[k].p->fromPoint;
				k++;
			}
		}
		else{
			k++;
		}
	}
}


int getCorner(int corner, int offset, int len){
	return (corner + offset) % len;
}

int getCornerDifference(int cornerA, int cornerB, int len){
	if(cornerA  < cornerB){
		cornerB += len;
	}
	return cornerA - cornerB;
}

int getMidPointAbsoluteIdx(const int midBeginIndex, int edge, const M_CtrlPolyPoint& ctrlPolyPoint){
	int idx = midBeginIndex;
	for(int e = 0; e < edge; e++){
		int len = ctrlPolyPoint[idx].len;
		if(len == 0){
			len = 1;
		}
		idx += len;
	}
	return idx;
}

void Seamless::mergeControlPolygon(int polyA, int a0, int a1, int polyB, int b0, int b1, M_Int& deleteCtrlPolySelList){

	int midB = getMidOffset(polyB, b1);
	int midA = getMidOffset(polyA, a0);

//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b1].p, ctrlPolyPoint[polyB + b1].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b1].p, ctrlPolyPoint[polyB + b1].i);

	int lenA = ctrlPolyPoint[midA].len;
	int lenB = ctrlPolyPoint[midB].len;
	//return;

	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + 4 + a0].p, ctrlPolyPoint[polyA + 4 + a0].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a0].p, ctrlPolyPoint[polyA + a0].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i);

//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b0].p, ctrlPolyPoint[polyB + b0].i);
//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b1].p, ctrlPolyPoint[polyB + b1].i);

	CtrlPolyPoint cpp;

	int len = 4;
	cpp.len = len;
	int idx = getCorner(a0, 3, len);
	cpp.p = ctrlPolyPoint[polyA + idx].p;
	cpp.i = ctrlPolyPoint[polyA + idx].i;
	ctrlPolyPoint += cpp;
	//MultiSelect::pointList += ControlPoint(cpp.p, cpp.i);

	idx = getCorner(b0, 1, len);
	cpp.p = ctrlPolyPoint[polyB + idx].p;
	cpp.i = ctrlPolyPoint[polyB + idx].i;
	ctrlPolyPoint += cpp;
	//MultiSelect::pointList += ControlPoint(cpp.p, cpp.i);

	idx = getCorner(b0, 2, len);
	cpp.p = ctrlPolyPoint[polyB + idx].p;
	cpp.i = ctrlPolyPoint[polyB + idx].i;
	ctrlPolyPoint += cpp;
	//MultiSelect::pointList += ControlPoint(cpp.p, cpp.i);

	idx = getCorner(a0, 2, len);
	cpp.p = ctrlPolyPoint[polyA + idx].p;
	cpp.i = ctrlPolyPoint[polyA + idx].i;
	ctrlPolyPoint += cpp;
	//MultiSelect::pointList += ControlPoint(cpp.p, cpp.i);

	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyA + a0].p, ctrlPolyPoint[polyA + a0].i, 2);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyB + b0].p, ctrlPolyPoint[polyB + b0].i, 2);
	//------------
	ctrlPolyPoint.length++;
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i, 2);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyB + b1].p, ctrlPolyPoint[polyB + b1].i, 2);
	ctrlPolyPoint.length++;

	int centerLen = lenA + lenB;
	if(centerLen){
		for(int i = 0; i < lenA; i++){
			ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[midA + i].p, ctrlPolyPoint[midA + i].i, centerLen);
		}
		for(int i = 0; i < lenB; i++){
			ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[midB + i].p, ctrlPolyPoint[midB + i].i, centerLen);
		}
	}
	else{
		ctrlPolyPoint.length++;
	}
	deleteCtrlPolySelList += polyA;
	deleteCtrlPolySelList += polyB;
	ctrlPolyPoint[polyB].p = 0;
	ctrlPolyPoint[polyA].p = 0;
}

bool Seamless::findCtrlPolyWith2SelectedClockwise(M_Int& ctrlPolySelList, NurbsPatch* patchA0, int polyA, int a0, int a1, const Vec3f vec0, const Vec3f vec1, M_Int& deleteCtrlPolySelList){
	for(int q = 0; q < ctrlPolySelList.length; q++){
		int polyB = ctrlPolySelList[q];
		int len = ctrlPolyPoint[polyB].len;
		for(int b0 = 0; b0 < len; b0++){
			if(ctrlPolyPoint[polyB].p){
				NurbsPatch* patchB0 = ctrlPolyPoint[polyB + b0].p;
				int iB0 = ctrlPolyPoint[polyB + b0].i;
	////
				int b1 = getCorner(b0, len - 1, len);
				NurbsPatch* patchB1 = ctrlPolyPoint[polyB + b1].p;
				int iB1 = ctrlPolyPoint[polyB + b1].i;

				if((vec0 - patchB0->dpoint[iB0]).length() < .00001){
					if((vec1 - patchB1->dpoint[iB1]).length() < .00001){
						mergeControlPolygon(polyA, a0, a1, polyB, b0, b1, deleteCtrlPolySelList);
						return true;
					}
				}
			}
		}
	}
	return false;
}

void Seamless::findCtrlPolyWith2SelectedAntiClockwise(M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList){
	for(int q = 0; q < ctrlPolySelList.length; q++){
		int polyA = ctrlPolySelList[q];
		int len = ctrlPolyPoint[polyA].len;
		for(int a0 = 0; a0 < len; a0++){
			if(ctrlPolyPoint[polyA].p){
				NurbsPatch* patchA0 = ctrlPolyPoint[polyA + a0].p;
				int iA0 = ctrlPolyPoint[polyA + a0].i;

				int a1 = getCorner(a0, 1, len);
				NurbsPatch* patchA1 = ctrlPolyPoint[polyA + a1].p;
				//if(patchA0 == patchA1){
					int iA1 = ctrlPolyPoint[polyA + a1].i;

					Vec3f vecA0 = patchA0->dpoint[iA0];
					Vec3f vecA1 = patchA1->dpoint[iA1];

					if(findCtrlPolyWith2SelectedClockwise(MultiSelect::ctrlPolySelList, patchA0, polyA, a0, a1, vecA0, vecA1, deleteCtrlPolySelList)){
						//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a0].p, ctrlPolyPoint[polyA + a0].i);
						//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i);
						return;
					}
				//}
			}
		}
	}
}

inline bool polyInList(int poly, M_Int& deleteCtrlPolySelList, int checkToPoly){
	for(int i = deleteCtrlPolySelList.length; --i >= checkToPoly;){
		if(deleteCtrlPolySelList[i] == poly){
			return true;
		}
	}
	return false;
}

bool Seamless::findCloseCornerLoop(int polyA, int corner, M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList, M_Int& m_corner, int checkToPoly){
	int len = ctrlPolyPoint[polyA].len;
	int a = corner;
	for(int c = 1; c < len; c++){
		a = getCorner(a, 1, len);
		NurbsPatch* patchA = ctrlPolyPoint[polyA + a].p;
		int iA = ctrlPolyPoint[polyA + a].i;
		Vec3f vecA = patchA->dpoint[iA];
	
		for(int q = 0; q < ctrlPolySelList.length; q++){
			int polyB = ctrlPolySelList[q];
			int len = ctrlPolyPoint[polyA].len;
			if(false == polyInList(polyB, deleteCtrlPolySelList, checkToPoly)){

				for(int b = 0; b < len; b++){
					NurbsPatch* patchB = ctrlPolyPoint[polyB + b].p;
					if(patchA != patchB){
						int iB = ctrlPolyPoint[polyB + b].i;
						Vec3f vecB = patchB->dpoint[iB];
						if(	abs(vecA.x - vecB.x) < .00001 &&
							abs(vecA.y - vecB.y) < .00001 &&
							abs(vecA.z - vecB.z) < .00001){

							//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b].p, ctrlPolyPoint[polyB + b].i);
							if(m_corner.length == 3){
								checkToPoly++;
							}
							m_corner += a;
							m_corner += b;
							if(polyB == deleteCtrlPolySelList[checkToPoly-1]){
								m_corner[checkToPoly-1] = b;
								return true;
							}
							deleteCtrlPolySelList += polyB;
							return findCloseCornerLoop(polyB, b, ctrlPolySelList, deleteCtrlPolySelList, m_corner, checkToPoly);
						}
					}
				}
			}
		}
	}
	return false;
}

bool is5Poly(M_Int& m_corner, M_Int& m_ab, int& c){
	bool flag = false;
	for(int k = 0; k < (m_corner.length - 1) / 2; k++){
		if(m_corner[k * 2] - m_corner[k * 2 + 1] == 2){
			flag = true;
			c = k;
		}
		else if(m_corner[k * 2] - m_corner[k * 2 + 1] == -2){
			flag = true;
			c = k;
		}
		else{
			m_ab += k;
		}
	}
	return flag;
}

void Seamless::create5SidedPoly(int polyA, int a1, int a0, int polyB, int b1, int b0, int polyC, int c1, int c3){
//	int polyC = getNurbsCorner(c0, MultiSelect::ctrlPolySelList);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a0].p, ctrlPolyPoint[polyA + a0].i);
	int c0 = getCorner(c1, 3, 4);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyC + c0].p, ctrlPolyPoint[polyC + c0].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyC + c3].p, ctrlPolyPoint[polyC + c3].i);

//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a0].p, ctrlPolyPoint[polyA + a0].i);
//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b0].p, ctrlPolyPoint[polyB + b0].i);

	NurbsPatch* blockPatch = 0;
	int blockIdxA = -1;

	
	Vec3f selVec = ctrlPolyPoint[polyC + c0].p->dpoint[ctrlPolyPoint[polyC + c0].i];

//	int polyA = findClosestMidPoint(selVec, a0, blockIdxA);
//	int blockIdxB = blockIdxA;
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[blockIdxA].p, ctrlPolyPoint[blockIdxA].i);
//	int polyB = findClosestMidPoint(selVec, b0, blockIdxB);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[blockIdxB].p, ctrlPolyPoint[blockIdxB].i);

	//int a1 = getCorner(a0, 1, 4);
	//int b1 = getCorner(b0, 1, 4);

	Vec3f nearA = ctrlPolyPoint[polyA + a1].p->dpoint[ctrlPolyPoint[polyA + a1].i];
	Vec3f nearB = ctrlPolyPoint[polyB + b1].p->dpoint[ctrlPolyPoint[polyB + b1].i];
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b1].p, ctrlPolyPoint[polyB + b1].i);

	//int c3 = getCorner(c0, 3, 4);
	Vec3f selVec3 = ctrlPolyPoint[polyC + c3].p->dpoint[ctrlPolyPoint[polyC + c3].i];
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyC + c3].p, ctrlPolyPoint[polyC + c3].i);

	if((selVec3 - nearA).length() > (selVec3 - nearB).length()){
		mergeTo5CtrlPoly2(polyC, c0, polyA, a0, a1, polyB, b0, b1);
	}
	else{
		mergeTo5CtrlPoly2(polyC, c0, polyB, b0, b1, polyA, a0, a1);
	}
}

void Seamless::findCloseCorner(M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList, M_Int& m_corner){
	M_Int m_poly;
	for(int q = 0; q < ctrlPolySelList.length; q++){
		int polyA = ctrlPolySelList[q];
		if(false == polyInList(polyA, deleteCtrlPolySelList, 0)){
			int aLen = ctrlPolyPoint[polyA].len;
			for(int a = 0; a < aLen; a++){
				NurbsPatch* patchA = ctrlPolyPoint[polyA + a].p;
				int iA = ctrlPolyPoint[polyA + a].i;
				Vec3f vecA = patchA->dpoint[iA];
			
				for(int q = 0; q < ctrlPolySelList.length; q++){
					int polyB = ctrlPolySelList[q];
					int len = ctrlPolyPoint[polyA].len;
					if(polyA != polyB){

						for(int b = 0; b < len; b++){
							NurbsPatch* patchB = ctrlPolyPoint[polyB + b].p;
							if(patchA != patchB){
								int iB = ctrlPolyPoint[polyB + b].i;
								Vec3f vecB = patchB->dpoint[iB];
								if(	abs(vecA.x - vecB.x) < .00001 &&
									abs(vecA.y - vecB.y) < .00001 &&
									abs(vecA.z - vecB.z) < .00001){

									//deleteCtrlPolySelList += polyA;
									//deleteCtrlPolySelList += polyB;

									m_poly.length = 0;
									m_poly += polyA;
									m_poly += polyB;

									m_corner.length = 0;
									m_corner.length++;
									m_corner += a;
									m_corner += b;
									//int end = ctrlPolyPoint[polyA + a].len;
									//end = ctrlPolyPoint[polyB + b].len;
									//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a].p, ctrlPolyPoint[polyA + a].i);
									//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b].p, ctrlPolyPoint[polyB + b].i);
									//int checkToPoly = deleteCtrlPolySelList.length - 2;
									int checkToPoly = 0;
									if(findCloseCornerLoop(polyB, b, ctrlPolySelList, m_poly, m_corner, checkToPoly)){
										M_Int m_ab;
										int c = 0;
										if(is5Poly(m_corner, m_ab, c)){
											int polyA = m_poly[m_ab[0]];
											int a0 = m_corner[m_ab[0] * 2];
											int a1 = m_corner[m_ab[0] * 2 + 1];
											int polyB = m_poly[m_ab[1]];
											int b0 = m_corner[m_ab[1] * 2];
											int b1 = m_corner[m_ab[1] * 2 + 1];
											int polyC = m_poly[c];
											int c0 = m_corner[c * 2];
											int c1 = m_corner[c * 2 + 1];
											create5SidedPoly(polyA, a0, a1, polyB, b0, b1, polyC, c0, c1);
										}
										else{
											for(int k = checkToPoly; k < m_poly.length; k++){
												int poly = m_poly[k];
												int c1 = getCorner(m_corner[(k - checkToPoly) * 2 + 1], 3, 4);
												int c0 = getCorner(m_corner[(k - checkToPoly) * 2], 1, 4);
												//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly + c0].p, ctrlPolyPoint[poly + c0].i);
												//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly + c1].p, ctrlPolyPoint[poly + c1].i);
												ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[poly + c0].p, ctrlPolyPoint[poly + c0].i, 6);
												ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[poly + c1].p, ctrlPolyPoint[poly + c1].i, 6);
											}
											for(int k0 = checkToPoly; k0 < m_poly.length; k0++){
												int poly0 = m_poly[k0];
												int k1 = k0 + 1;
												if(k1 >= m_poly.length){
													k1 = checkToPoly;
												}
												int poly1 = m_poly[k1];
												int c0 = m_corner[(k0 - checkToPoly) * 2 + 1];
												int c1 = m_corner[(k1 - checkToPoly) * 2];
												ctrlPolyPoint.length++;
												ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[poly0 + c0].p, ctrlPolyPoint[poly0 + c0].i, 2);
												ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[poly1 + c1].p, ctrlPolyPoint[poly1 + c1].i, 2);
											}
											int end = 0;
											for(int k = checkToPoly; k < m_poly.length; k++){
												int poly = m_poly[k];
												int m = getMidOffset(poly, m_corner[(k - checkToPoly) * 2 + 1]);
												end += ctrlPolyPoint[m].len;
											}
											for(int k = checkToPoly; k < m_poly.length; k++){
												int poly = m_poly[k];
												int m = getMidOffset(poly, m_corner[(k - checkToPoly) * 2 + 1]);

												MultiSelect::pointList += ControlPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i);
												ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i, end);
											}
										}
										deleteCtrlPolySelList += m_poly;
										q = ctrlPolySelList.length;
										a = aLen;
										break;
									}
								}
							}
						}
					}
				}
			}
		}
	}
}
void Seamless::commonEndSitchPolys(){
	MultiSelect::ctrlPolySelList.length = 0;
	if(false == (g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE)){
		calcAllPolyMidPoints();
	}
	//removeAllPolyMidPointsOwnership();
	//updateBuild3dWindow();
	//doBuild();
	//houseKeepVisualClue();

	BackUp::monostable.beginCycle();
}

void Seamless::stitchControlPolygons(){

	//if(MultiSelect::ctrlPolySelList.length == 3){
	//	mergeTo5CtrlPoly();
	//}
	if(MultiSelect::ctrlPolySelList.length == 2){
		M_Int deleteCtrlPolySelList;

		findCtrlPolyWith2SelectedAntiClockwise(MultiSelect::ctrlPolySelList, deleteCtrlPolySelList);

		bubbleSort(deleteCtrlPolySelList);
		deleteCtrlPoly(deleteCtrlPolySelList);

		commonEndSitchPolys();
	}
}

void Seamless::shareWeightInStitchPoly(NurbsPatch* patchA, int& idxA){
	float weightA = patchA->weight[idxA];
	int k = 0;
	while(k < ctrlPolyPoint.length){
		int mainLen = ctrlPolyPoint[k].len;
		k += mainLen;
		mainLen++;
		for(int g = 0; g < mainLen; g++){
			int len = ctrlPolyPoint[k].len;
			for(int i = 0; i < len; i++){
				NurbsPatch* patchB = ctrlPolyPoint[k + i].p;
				int idxB = ctrlPolyPoint[k + i].i;
				if(patchA == patchB && idxA == idxB){
					for(int j = 0; j < len; j++){
						patchB = ctrlPolyPoint[k + j].p;
						idxB = ctrlPolyPoint[k + j].i;
						patchB->weight.refValue()[idxB] = weightA;
						patchB->point.refValue()[idxB] = patchB->dpoint[idxB] * patchB->weight[idxB];
					}
					break;
				}
			}
			k += len == 0 ? 1 : len;
		}
	}
}

enum PolyType{
	CORNER_STITCH,
	CORNER_WEDGE_STITCH,
	BLOCK_STITCH,
	WEDGE_STITCH
};

bool NurbsPatch::isEdge(int idx){
	int uDim = dim.x;
	int vDim = dim.y;
	int u = idx % uDim;
	int v = idx / uDim;
	//which edge?
	if(u == 0){//left edge
		return true;
	}
	else if(u == uDim - 1){//right edge
		return true;
	}
	else if(v == 0){//south edge
		return true;
	}
	else if(v == vDim - 1){//north edge
		return true;
	}
	return false;
}

int Seamless::isBlockStitchPoly(int poly){ //return first edge index or -1 if fails
	int len = ctrlPolyPoint[poly].len;
	NurbsPatch* patch = ctrlPolyPoint[poly].p;

	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	for(int i = 0; i < len; i++){
		NurbsPatch* patch = ctrlPolyPoint[poly + i].p;
		int idx = ctrlPolyPoint[poly + i].i;
		if(patch->isEdge(idx)){
			int idx2 = ctrlPolyPoint[poly + getCorner(i, 1, len)].i;
			if(patch->isEdge(idx2)){
				return i;
			}
		}
	}
	return -1;
}

int Seamless::isWedgeStitchPoly(int poly){ //return first edge index or -1 if fails
	int len = ctrlPolyPoint[poly].len;
	NurbsPatch* patch = ctrlPolyPoint[poly].p;

	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	for(int i = 0; i < len; i++){
		NurbsPatch* patchA = ctrlPolyPoint[poly + i].p;
		int idxA = ctrlPolyPoint[poly + i].i;
		Vec3f vecA = patchA->dpoint[idxA];
		int ib = getCorner(i, 1, len);
		NurbsPatch* patchB = ctrlPolyPoint[poly + ib].p;
		int idxB = ctrlPolyPoint[poly + ib].i;
		Vec3f vecB = patchB->dpoint[idxB];

		if((vecA - vecB).length() < .00001){
			return i;
		}
	}
	return -1;
}

int Seamless::isCornerStitchPoly(int poly){ //return corner index or -1 if fails
	int len = ctrlPolyPoint[poly].len;
	NurbsPatch* patch = ctrlPolyPoint[poly].p;
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int southEastCornerIdx = uDim - 1;
	int northEastCornerIdx = uDim * vDim - 1;
	int northWestCornerIdx = northEastCornerIdx - (uDim - 1);

	for(int i = len; --i>=0;){
		int idx = ctrlPolyPoint[poly + i].i;
		if(	idx == 0 ||
			idx == southEastCornerIdx ||
			idx == northEastCornerIdx ||
			idx == northWestCornerIdx
			
			){
			return i;
		}
	}
	return -1;
}

int Seamless::get2Edges(int poly, int& edge0, int& edge1){
	int len = ctrlPolyPoint[poly].len;
	//int corner = isWedgeStitchPoly(poly);
	//if(corner >= 0){
	//	edge0 = getCorner(corner, len - 1, len);
	//	edge1 = getCorner(corner, 2, len);
	//	return WEDGE_STITCH;
	//}
	int corner = isCornerStitchPoly(poly);
	if(corner >= 0){
		edge0 = getCorner(corner, len -  1, len);
		edge1 = getCorner(corner, + 1, len);
		return CORNER_STITCH;
	}
	edge0 = isBlockStitchPoly(poly);
	edge1 = getCorner(edge0, 1, len);
	return BLOCK_STITCH;
}

void Seamless::sortStitchPolys(M_Int& sortedPoly, M_Int& sortedCorner, M_Int& ctrlPolySelList, M_Int& polyType){
	

	int polyA = ctrlPolySelList[0];
	int aLen = ctrlPolyPoint[polyA].len;
	int edgeA0 = 0;
	int edgeA1 = 0;
	int typeA = get2Edges(polyA, edgeA0, edgeA1);

//	polyType += typeA;

//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + edgeA0].p, ctrlPolyPoint[polyA + edgeA0].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + edgeA1].p, ctrlPolyPoint[polyA + edgeA1].i);
	//return;


	int edgeAW0 = getCorner(edgeA0, aLen - 1, aLen);
	NurbsPatch* patchA3 = ctrlPolyPoint[polyA + edgeAW0].p;
	int idxAW3 = ctrlPolyPoint[polyA + edgeAW0].i;
	Vec3f vecA3 = patchA3->dpoint[idxAW3];

	for(int i = 0; i < ctrlPolySelList.length; i++){
		bool found = false;

		//if(typeA != CORNER_STITCH){
		{
			for(int k = 0; k < ctrlPolySelList.length; k++){
				//MultiSelect::pointList += ControlPoint(patchA3, idxAW3);
				int polyB = ctrlPolySelList[k];
				if(polyA == polyB){
					continue;
				}
				int bLen = ctrlPolyPoint[polyB].len;
				int edgeB0 = 0;
				int edgeB1 = 0;
				int typeB = get2Edges(polyB, edgeB0, edgeB1);
				int edgeBW1 = getCorner(edgeB1, 1, bLen);

				NurbsPatch* patchB1 = ctrlPolyPoint[polyB + edgeBW1].p;
				int idxB1 = ctrlPolyPoint[polyB + edgeBW1].i;
				Vec3f vecB1 = patchB1->dpoint[idxB1];

				float d = (vecA3 - vecB1).length();
				if((vecA3 - vecB1).length() < .00001){
					sortedPoly += polyA;
					sortedCorner += edgeA0;
					sortedCorner += edgeA1;
					if(typeA == CORNER_STITCH){
						polyType += CORNER_WEDGE_STITCH;
					}
					else{
						polyType += WEDGE_STITCH;
					}

					polyA = polyB;
					aLen = bLen;
					edgeA0 = edgeB0;
					edgeA1 = edgeB1;
					typeA = typeB;

					edgeAW0 = getCorner(edgeA0, aLen - 1, aLen);
					patchA3 = ctrlPolyPoint[polyA + edgeAW0].p;
					idxAW3 = ctrlPolyPoint[polyA + edgeAW0].i;
					vecA3 = patchA3->dpoint[idxAW3];
					found = true;
					break;
				}
			}
		}
		if(found){
			continue;
		}
		NurbsPatch* patchA0 = ctrlPolyPoint[polyA + edgeA0].p;
		int idxA0 = ctrlPolyPoint[polyA + edgeA0].i;
		Vec3f vecA0 = patchA3->dpoint[idxA0];
		for(int k = 0; k < ctrlPolySelList.length; k++){
			int polyB = ctrlPolySelList[k];
			if(polyA == polyB){
				continue;
			}
			int bLen = ctrlPolyPoint[polyB].len;
			int edgeB0 = 0;
			int edgeB1 = 0;
			int typeB = get2Edges(polyB, edgeB0, edgeB1);

			NurbsPatch* patchB1 = ctrlPolyPoint[polyB + edgeB1].p;
			int idxB1 = ctrlPolyPoint[polyB + edgeB1].i;
			Vec3f vecB1 = patchB1->dpoint[idxB1];

			if((vecA0 - vecB1).length() < .00001){
				sortedPoly += polyA;
				sortedCorner += edgeA0;
				sortedCorner += edgeA1;
				polyType += typeA;

				polyA = polyB;
				aLen = bLen;
				edgeA0 = edgeB0;
				edgeA1 = edgeB1;
				typeA = typeB;

				//NurbsPatch* patchA3 = ctrlPolyPoint[polyA + edgeA0].p;
				//idxA3 = ctrlPolyPoint[polyA + edgeA0].i;
				//vecA3 = patchA3->dpoint[idxA3];
				//MultiSelect::pointList += ControlPoint(patchA3, idxA3);

				edgeAW0 = getCorner(edgeA0, aLen - 1, aLen);
				patchA3 = ctrlPolyPoint[polyA + edgeAW0].p;
				idxAW3 = ctrlPolyPoint[polyA + edgeAW0].i;
				vecA3 = patchA3->dpoint[idxAW3];
				break;
			}
		}
	}
}

bool Seamless::pointIsNotInNewPoly(int mark, NurbsPatch* patch, int idx){
	int len = ctrlPolyPoint.length - mark;
	for(int i = len; --i>= 0;){
		if(ctrlPolyPoint[mark].i == idx && ctrlPolyPoint[mark].p == patch){
			return false;
		}
		mark++;
	}
	return true;
}


bool Seamless::findThreeClosePoints(M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList){
//			int corner = getCorner(cornerList[0], 3, aLen);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + corner].p, ctrlPolyPoint[polyA + corner].i);
	//return;
	const int newPolyBegin = ctrlPolyPoint.length;

	M_Int polyType;
	M_Int sortedPoly;
	M_Int sortedCorner;
	sortStitchPolys(sortedPoly, sortedCorner, ctrlPolySelList, polyType);
	if(sortedPoly.length < 2){
		return false;
	}

	deleteCtrlPolySelList = ctrlPolySelList;
	int newPolyLen = 0;

	int sortedPolyLen = sortedPoly.length;

	//for(int k = 0; k < sortedPolyLen; k++){
	//	switch(polyType[k]){
	//		case CORNER_STITCH:
	//			newPolyLen++;
	//			break;
	//		case BLOCK_STITCH:
	//			newPolyLen += 2;
	//			break;
	//		case WEDGE_STITCH:
	//			newPolyLen += 1;
	//			break;
	//	}
	//}
	//int poly = sortedPoly[0];
	//int edge0 = sortedCorner[0];
	//int edge1 = sortedCorner[1];
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly + edge0].p, ctrlPolyPoint[poly + edge0].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly + edge1].p, ctrlPolyPoint[poly + edge1].i);
	//return;


	for(int k = 0; k < sortedPolyLen; k++){
		if(polyType[k] == CORNER_WEDGE_STITCH){
			continue;
		}
		int poly = sortedPoly[k];
		int len = ctrlPolyPoint[poly].len;
		{
			int edge1 = sortedCorner[k * 2 + 1];
			int edgeW1 = getCorner(edge1, 1, len);
		
			int m = poly + edgeW1;
			NurbsPatch* patch = ctrlPolyPoint[m].p;
			int idx = ctrlPolyPoint[m].i;

			ctrlPolyPoint += CtrlPolyPoint(patch, idx, 0);
			newPolyLen++;
		}

		if(polyType[k] == BLOCK_STITCH){
			int poly = sortedPoly[k];
			int len = ctrlPolyPoint[poly].len;
			int edge0 = sortedCorner[k * 2];
			int edgeW0 = getCorner(edge0, len - 1, len);
		
			int m = poly + edgeW0;
			NurbsPatch* patch = ctrlPolyPoint[m].p;
			int idx = ctrlPolyPoint[m].i;

			ctrlPolyPoint += CtrlPolyPoint(patch, idx, 0);
			newPolyLen++;
		}
		

		//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i);

		//int a3 = getCorner(a2, 1, aLen);

//			MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a3].p, ctrlPolyPoint[polyA + a3].i);
	}
	int j = ctrlPolyPoint.length;
	for(int i = newPolyLen; --i>=0;){
		ctrlPolyPoint[--j].len = newPolyLen;
	}
//	for(int a = 0; a < 6; a++){
	for(int a = 0; a < sortedPolyLen; a++){
		int polyA = sortedPoly[a];
		if(polyType[a] == CORNER_WEDGE_STITCH){
			continue;
		}
		if(polyType[a] == WEDGE_STITCH){
			int polyA = sortedPoly[a];
			int len = ctrlPolyPoint[polyA].len;
			int edge0A = sortedCorner[a * 2];
			int corner = getCorner(edge0A, len - 1, len);
			int m = polyA + corner;
			NurbsPatch* patch = ctrlPolyPoint[m].p;
			int idx = ctrlPolyPoint[m].i;

			if(patch->isEdge(idx)){
				ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i, 2);
				//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i);

				int b = getCorner(a,  1, sortedPolyLen);

				int polyB = sortedPoly[b];
				int edge1B = sortedCorner[b * 2 + 1];

				m = polyB + edge1B;
				ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i, 2);


				//ctrlPolyPoint.length++;
				continue;
			}
			else{
				//{
				//	int edge1A = sortedCorner[a * 2 + 1];
				//	int m = polyA + edge1A;
				//	NurbsPatch* patch = ctrlPolyPoint[m].p;
				//	int idx = ctrlPolyPoint[m].i;
				//	if(patch->isEdge(idx)){
				//		continue;
				//	}
				//}
				
				ctrlPolyPoint.length++;
			}
			//ctrlPolyPoint.length++;
			//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i);
			continue;
		}


		int edge0A = sortedCorner[a * 2];

		int m = polyA + edge0A;
		if(polyType[a] == BLOCK_STITCH){
			ctrlPolyPoint.length++;
		}
		//ctrlPolyPoint.length++;
		//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i);
		ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i, 2);

		int b = getCorner(a,  1, sortedPolyLen);

		int polyB = sortedPoly[b];
		int edge1B = sortedCorner[b * 2 + 1];

		m = polyB + edge1B;
		ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i, 2);
	}
	
	int numOfCenterPoints = 0;
	for(int k = 0; k < sortedPolyLen; k++){
		int poly = sortedPoly[k];

		int len = ctrlPolyPoint[poly].len;
		for(int e = 0; e < len; e++){
			NurbsPatch* patch = ctrlPolyPoint[poly + e].p;
			int idx = ctrlPolyPoint[poly + e].i;
			if(patch->isEdge(idx)){
				if(pointIsNotInNewPoly(newPolyBegin, patch, idx)){
					numOfCenterPoints++;
					ctrlPolyPoint += CtrlPolyPoint(patch, idx, 0);
				}
			}
		}

		int midOffset = poly + len;


		for(int e = 0; e < len + 1; e++){
			int midLen = ctrlPolyPoint[midOffset].len;
			if(midLen == 0){
				midOffset++;
				continue;
			}
			for(int i = 0; i < midLen; i++){
				NurbsPatch* patch = ctrlPolyPoint[midOffset].p;
				int idx = ctrlPolyPoint[midOffset].i;
				midOffset++;
				if(patch->isEdge(idx)){
					if(pointIsNotInNewPoly(newPolyBegin, patch, idx)){
						numOfCenterPoints++;
						ctrlPolyPoint += CtrlPolyPoint(patch, idx, 0);
					}
				}
			}
		}
	}
//	ctrlPolyPoint.length++;
//	numOfCenterPoints++;
//	return;
	j = ctrlPolyPoint.length;
	for(int i = numOfCenterPoints; --i>=0;){
		ctrlPolyPoint[--j].len = numOfCenterPoints;
	}
	return true;
}

void Seamless::stitchFlower(){
	if(MultiSelect::ctrlPolySelList.length > 1){
		M_Int deleteCtrlPolySelList;

		if(findThreeClosePoints(MultiSelect::ctrlPolySelList, deleteCtrlPolySelList)){
			bubbleSort(deleteCtrlPolySelList);
			deleteCtrlPoly(deleteCtrlPolySelList);
			commonEndSitchPolys();
		}
	}
}

bool Seamless::isCtrlPolyTaken(int a, int b, NurbsPatch* patch){
	
	for(int i = 0; i < ctrlPolyPoint.length;){
		int len = ctrlPolyPoint[i].len;
		for(int k = len; --k >= 0;){
			if(ctrlPolyPoint[i + k].p == patch){
				if(ctrlPolyPoint[i + k].i == a){
					while(--k >= 0){
						if(ctrlPolyPoint[i + k].p == patch){
							if(ctrlPolyPoint[i + k].i == b){
								return true;
							}
						}
					}
					break;
				}
				if(ctrlPolyPoint[i + k].i == b){
					while(--k >= 0){
						if(ctrlPolyPoint[i + k].p == patch){
							if(ctrlPolyPoint[i + k].i == a){
								return true;
							}
						}
					}
					break;
				}
			}
		}
		i = getNextCtrlPoly(i);
	}
	return false;
}

bool checkCtrlPoly(M_CtrlPolyPoint& ctrlPolyPoint, int k, int a, int b, NurbsPatch* patch, int poly, int len){
	while(--k >= 0){
		if(ctrlPolyPoint[poly + k].p == patch && ctrlPolyPoint[poly + k].i == b){
			return true;
		}
	}
	poly += len;
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		if(midLen){
			for(int i = 0; i < midLen; i++){
				if(ctrlPolyPoint[poly + i].p == patch && ctrlPolyPoint[poly + i].i == b){
					return true;
				}
			}			
		}
		else{
			midLen++;
		}
		poly += midLen;
	}

	return false;
}

bool Seamless::isCtrlPolyCornerTaken(int a, int b, NurbsPatch* patch){
	
	for(int i = 0; i < ctrlPolyPoint.length;){
		int len = ctrlPolyPoint[i].len;
		for(int k = len; --k >= 0;){
			if(ctrlPolyPoint[i + k].p == patch){
				if(ctrlPolyPoint[i + k].i == a){
					if(checkCtrlPoly(ctrlPolyPoint, k, a, b, patch, i, len)){
						return true;
					}
					break;
				}
				if(ctrlPolyPoint[i + k].i == b){
					if(checkCtrlPoly(ctrlPolyPoint, k, b, a, patch, i, len)){
						return true;
					}
					break;
				}
			}
		}
		i = getNextCtrlPoly(i);
	}
	return false;
}

void makeCtrlPolyInsideOut(const int poly, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	insideOutCtrlPoly.length = 0;
	const int len = ctrlPolyPoint[poly].len;
	for(int c = 0; c < len; c++){
		insideOutCtrlPoly += ctrlPolyPoint[poly + c];
	}
	int midIdx = poly + len;
	for(int c = 0; c < len; c++){
		int ln = ctrlPolyPoint[midIdx].len;
		int rLen = ln;
		if(ln == 0){
			ln = 1;
		}
		for(int c = 0; c < ln; c++){
			ctrlPolyPoint[midIdx].len = rLen;
			insideOutCtrlPoly += ctrlPolyPoint[midIdx++];
		}
	}
	// copy back inside out poly
	int i = poly;
	for(int r = len; --r >= 0;){
		int corner = r + 1;
		if(corner > len - 1){
			corner = 0;
		}
		ctrlPolyPoint[i++] = insideOutCtrlPoly[corner];
	}
	for(int r = insideOutCtrlPoly.length; --r >= len;){
		ctrlPolyPoint[i++] = insideOutCtrlPoly[r];
	}

}

void NurbsPatch::addOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + 1;
	int c = u + 1 + uDim;
	int d = u + uDim;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	ctrlPolyPoint.length += 4 + 1;
	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addSouthEastCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + 1;
	int c = u + 1 + uDim;
	int d = u + uDim;
	if(smls->isCtrlPolyCornerTaken(d, b, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	ctrlPolyPoint.length += 4 + 1;
	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addNorthEastCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + 1;
	int c = u + 1 + uDim;
	int d = u + uDim;
	if(smls->isCtrlPolyCornerTaken(a, c, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	ctrlPolyPoint.length += 4 + 1;
	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addNorthWestCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + 1;
	int c = u + 1 + uDim;
	int d = u + uDim;
	if(smls->isCtrlPolyCornerTaken(b, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	ctrlPolyPoint.length += 4 + 1;
	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addSouthWestCornerOneByOneCtrlPolyPnt(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + 1;
	int c = u + 1 + uDim;
	int d = u + uDim;
	if(smls->isCtrlPolyCornerTaken(c, a, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	ctrlPolyPoint.length += 4 + 1;
	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}


void NurbsPatch::addTwoByOneCtrlPolyPntSouthMid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + 1;
	int c = u + 1 + uDim;
	int d = u + uDim;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	ctrlPolyPoint.length += 5;

	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addTwoByOneCtrlPolyPntNorthMid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u - 1;
	int c = u - 1 - uDim;
	int d = u - uDim;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;


	ctrlPolyPoint.length += 5;

	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addUJoinCtrlPolyPntSouth(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u + uDim - 2;
	int b = u + 1;
	int c = u + 1 + uDim;
	int d = u + uDim - 2 + uDim;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	//2 mid point1
	cpp.len = 2;
	cpp.i = u;
	ctrlPolyPoint += cpp;

	cpp.i = uDim - 1;
	ctrlPolyPoint += cpp;

	cpp.len = 4;

	ctrlPolyPoint.length += 4;
	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addUJoinCtrlPolyPntNorth(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u - uDim + 2;
	int b = u - 1;
	int c = u - 1 - uDim;
	int d = u - uDim + 2 - uDim;

	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	//2 mid point1
	cpp.len = 2;
	cpp.i = u;
	ctrlPolyPoint += cpp;

	cpp.i = u - uDim + 1;
	ctrlPolyPoint += cpp;

	cpp.len = 4;

	ctrlPolyPoint.length += 4;
	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addOneByTwoCtrlPolyPntEast1Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u - uDim;
	int c = u - uDim + 1;
	int d = u + 1;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	ctrlPolyPoint.length += 5;

	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addOneByTwoCtrlPolyPntEast0Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u - uDim;
	int c = u - uDim + 1;
	int d = u + 1;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	// mid point

	ctrlPolyPoint.length += 5;

	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addOneByTwoCtrlPolyPntWest1Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + uDim;
	int c = u + uDim - 1;
	int d = u - 1;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	// mid point

	ctrlPolyPoint.length += 5;

	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}

void NurbsPatch::addOneByTwoCtrlPolyPntWest0Mid(const int u, const int uDim, CtrlPolyPoint& cpp, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& insideOutCtrlPoly){
	int poly = ctrlPolyPoint.length;
	int a = u;
	int b = u + uDim;
	int c = u + uDim - 1;
	int d = u - 1;
	if(smls->isCtrlPolyTaken(c, d, cpp.p)){
		return;
	}

	cpp.i = a;
	ctrlPolyPoint += cpp;

	cpp.i = b;
	ctrlPolyPoint += cpp;

	cpp.i = c;
	ctrlPolyPoint += cpp;

	cpp.i = d;
	ctrlPolyPoint += cpp;

	// mid point

	ctrlPolyPoint.length += 5;

	if(isInsideOut()){
		makeCtrlPolyInsideOut(poly, ctrlPolyPoint, insideOutCtrlPoly);
	}
}
/*
void NurbsPatch::makeCtrlPolyInsideOut(const int poly, M_CtrlPolyPoint& ctrlPolyPoint){//does not work

	int len = ctrlPolyPoint[poly].len;
	int hLen = len >> 1;
	for(int c = 0; c < hLen; c++){
		CtrlPolyPoint swapCpp = ctrlPolyPoint[poly + c];
		ctrlPolyPoint[poly + c] = ctrlPolyPoint[poly + len - 1 - c];
		ctrlPolyPoint[poly + len - 1 - c] = swapCpp;
	}
	int midIdx = poly + len;
	int midLenSum = 0;
	for(int c = 0; c < len; c++){
		midLenSum += ctrlPolyPoint[midIdx + midLenSum].len;
	}
	int hMidLenSum = midLenSum >> 1;
	for(int c = 0; c < hMidLenSum; c++){
		CtrlPolyPoint swapCpp = ctrlPolyPoint[midIdx + c];
		ctrlPolyPoint[midIdx + c] = ctrlPolyPoint[midIdx + midLenSum - 1 - c];
		ctrlPolyPoint[midIdx + midLenSum - 1 - c] = swapCpp;
	}
}
*/


void invertSelCtrlPoly(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		M_CtrlPolyPoint insideOutCtrlPoly;
		Seamless* smls = patch->smls;
		M_CtrlPolyPoint& ctrlPolyPoint = smls->ctrlPolyPoint;
		M_Link& link = smls->link;

		for(int i = MultiSelect::ctrlPolySelList.length; --i >= 0;){
			int poly = MultiSelect::ctrlPolySelList[i];
			makeCtrlPolyInsideOut(poly, smls->ctrlPolyPoint, insideOutCtrlPoly);
			for(int j = link.length; --j >= 0;){
				if(poly == link[j].poly){
					int len = ctrlPolyPoint[poly].len;
					int corner = len - link[j].corner - 1;
					//if(corner < 0){
					//	corner = len - 1;
					//}
					link[j].corner = corner;
					link[j].between = 1.0 - link[j].between;
				}
			}
		}

		patch->houseKeep();
		BackUp::monostable.beginCycle();
	}
	else if(Oscillator* osc = dynamic_cast<Oscillator*>(g_selNode.getRaw())){
		osc->alignWithPatch();
	}
}


void NurbsPatch::fillEdgesWithCtrlPolys(){
	const int uDim = this->dim.x;
	const int vDim = this->dim.y;
	M_CtrlPolyPoint ctrlPolyPoint;
	M_CtrlPolyPoint insideOutCtrlPoly;

	CtrlPolyPoint cpp;
	cpp.len = 4;
	cpp.p = this;

	int u = 0;
	int q = uDim * (vDim - 2);

	int vMid = checkNurbsType(vDim, vKnot);

	if(uJoinEnds){
		if(beginSP == false){
			addUJoinCtrlPolyPntSouth(u, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
		if(endSP == false){
			addUJoinCtrlPolyPntNorth(uDim * vDim - 1, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
	}
	else{
		if(beginSP == false){
			addSouthWestCornerOneByOneCtrlPolyPnt(u, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
		if(endSP == false){
			addNorthWestCornerOneByOneCtrlPolyPnt(q + u, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
	}

	for(u = 1; u < uDim - 2; u++){
		if(beginSP == false){
			addTwoByOneCtrlPolyPntSouthMid(u, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
		if(endSP == false){
			addTwoByOneCtrlPolyPntNorthMid(q + uDim + u + 1, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
	}

	if(uJoinEnds){
	}
	else{
		if(beginSP == false){
			addSouthEastCornerOneByOneCtrlPolyPnt(u, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
		if(endSP == false){
			addNorthEastCornerOneByOneCtrlPolyPnt(q + u, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
		}
		if(NurbsPatch::MID_0 == vMid){
			for(int v = 1; v < vDim - 2; v++){
				addOneByTwoCtrlPolyPntEast0Mid((v + 1) * uDim, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
				addOneByTwoCtrlPolyPntWest0Mid(v * uDim + uDim - 1, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
			}
		}
		else{
			for(int v = 1; v < vDim - 2; v++){
				addOneByTwoCtrlPolyPntEast1Mid((v + 1) * uDim, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
				addOneByTwoCtrlPolyPntWest1Mid(v * uDim + uDim - 1, uDim, cpp, ctrlPolyPoint, insideOutCtrlPoly);
			}
		}
	}
	smls->ctrlPolyPoint += ctrlPolyPoint;
//	smls->calcAllPolyMidPoints();
//	houseKeepVisualClue();
//	BackUp::monostable.beginCycle();
}

/*
int Seamless::findClosestMidPoint(const M_Int& ctrlPolySelList, const Vec3f selCorner, int& corner, int& blockIdx){//returns poly
	int poly = -1;
	int midIdx = 0;
	float shortest = 1000000.0f;
	for(int i = 0; i < ctrlPolyPoint.length;){
		int len = ctrlPolyPoint[i].len;
		int m = i + len;
		for(int c = 0; c <len; c++){
			int midLen = ctrlPolyPoint[m].len;
			for(int q = 0; q < midLen; q++){
				NurbsPatch* patch = ctrlPolyPoint[m].p;
				int idx = ctrlPolyPoint[m].i;
				if(m != blockIdx){

					Vec3f vec = patch->dpoint[idx];
					
					float distance = (selCorner - vec).length();
					if(distance < shortest){
						//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i);
						shortest = distance;
						midIdx = m;
						poly = i;
						corner = c;
					}
					m++;
				}
			}
		}
		i = getNextCtrlPoly(i);
	}
	blockIdx = midIdx;
	return poly;
}
*/
int Seamless::findClosestMidPoint(const M_Int& ctrlPolySelList, const Vec3f selCorner, int& corner, int& blockIdx){//returns poly
	int poly = -1;
	int midIdx = 0;
	float shortest = 1000000.0f;

	for(int q = ctrlPolySelList.length; --q>=0;){
		int i = ctrlPolySelList[q];
		int len = ctrlPolyPoint[i].len;
		int m = i + len;
		for(int c = 0; c <len; c++){
			int midLen = ctrlPolyPoint[m].len;
			if(midLen){
				for(int q = 0; q < midLen; q++){
					NurbsPatch* patch = ctrlPolyPoint[m].p;
					int idx = ctrlPolyPoint[m].i;
					if(m != blockIdx){
						Vec3f vec = patch->dpoint[idx];
						float distance = (selCorner - vec).length();
						if(distance < shortest){
							//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[m].p, ctrlPolyPoint[m].i);
							shortest = distance;
							midIdx = m;
							poly = i;
							corner = c;
						}
					}
				}
			}
			else{
				midLen++;
			}
			m += midLen;
		}
	}
	blockIdx = midIdx;
	return poly;
}

int Seamless::getNurbsCorner(int& c0, M_Int& ctrlPolySelList){//return poly

	for(int q = ctrlPolySelList.length; --q>=0;){
		int i = ctrlPolySelList[q];
		int len = ctrlPolyPoint[i].len;

		for(int c = 0; c < len; c++){
			NurbsPatch* patch = ctrlPolyPoint[i + c].p;
			int uDim = patch->dim.x;
			int vDim = patch->dim.y;
			int idx = ctrlPolyPoint[i + c].i;
			int u = idx % uDim;
			int v = idx / uDim;
			if(u == 0 || u == uDim - 1){
				if(v == 0 || v == vDim - 1){
					c0 = c;
					return i;
				}
			}
			else if(v == 0 || v == vDim - 1){
				if(u == 0 || u == uDim - 1){
					c0 = c;
					return i;
				}
			}
		}
	}
	return -1;
}

void Seamless::mergeTo5CtrlPoly2(int polyC, int c0, int polyA, int a0, int a1, int polyB, int b0, int b1){
	int c2 = getCorner(c0, 2, 4);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyC + c2].p, ctrlPolyPoint[polyC + c2].i);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyC + c2].p, ctrlPolyPoint[polyC + c2].i, 5);
	int b2 = getCorner(b1, 1, 4);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b2].p, ctrlPolyPoint[polyB + b2].i);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyB + b2].p, ctrlPolyPoint[polyB + b2].i, 5);
	int b3 = getCorner(b2, 1, 4);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b3].p, ctrlPolyPoint[polyB + b3].i);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyB + b3].p, ctrlPolyPoint[polyB + b3].i, 5);
	int a2 = getCorner(a1, 1, 4);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a2].p, ctrlPolyPoint[polyA + a2].i);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyA + a2].p, ctrlPolyPoint[polyA + a2].i, 5);
	int a3 = getCorner(a2, 1, 4);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a3].p, ctrlPolyPoint[polyA + a3].i);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyA + a3].p, ctrlPolyPoint[polyA + a3].i, 5);

	int c3 = getCorner(c2, 1, 4);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyC + c3].p, ctrlPolyPoint[polyC + c3].i, 2);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyB + b1].p, ctrlPolyPoint[polyB + b1].i, 2);
	ctrlPolyPoint.length++;
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyB + b0].p, ctrlPolyPoint[polyB + b0].i, 2);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i, 2);
	ctrlPolyPoint.length++;
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyA + a0].p, ctrlPolyPoint[polyA + a0].i, 2);
	int c1 = getCorner(c0, 1, 4);
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyC + c1].p, ctrlPolyPoint[polyC + c1].i, 2);

	int midB = getMidOffset(polyB, b0);
	int midA = getMidOffset(polyA, a0);

	int centerLen = ctrlPolyPoint[midB].len + ctrlPolyPoint[midA].len + 1;
	ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[polyC + c0].p, ctrlPolyPoint[polyC + c0].i, centerLen);
	for(int i = 0; i < ctrlPolyPoint[midB].len; i++){
		ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[midB + i].p, ctrlPolyPoint[midB + i].i, centerLen);
	}
	for(int i = 0; i < ctrlPolyPoint[midA].len; i++){
		ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[midA + i].p, ctrlPolyPoint[midA + i].i, centerLen);
	}
}

int Seamless::getMidOffset(int poly, int corner){

	int m = poly + ctrlPolyPoint[poly].len;

	for(int c = 0; c < corner; c++){
		int midLen = ctrlPolyPoint[m].len;
		if(midLen == 0){
			m++;
			continue;
		}
		for(int k = 0; k < midLen; k++){
			m++;
		}
	}
	return m;
}

void Seamless::mergeTo6CtrlPoly(){
	M_Int deleteCtrlPolySelList;
	M_Int m_corner;

	findCloseCorner(MultiSelect::ctrlPolySelList, deleteCtrlPolySelList, m_corner);

	//ctrlPolyPoint.length += 1;

	bubbleSort(deleteCtrlPolySelList);
	deleteCtrlPoly(deleteCtrlPolySelList);

	commonEndSitchPolys();
}

void Seamless::mergeTo5CtrlPoly(){
	int c0 = 0;
	int polyC = getNurbsCorner(c0, MultiSelect::ctrlPolySelList);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyC + c0].p, ctrlPolyPoint[polyC + c0].i);

	int a0 = 0;
	int b0 = 0;

	NurbsPatch* blockPatch = 0;
	int blockIdxA = -1;

	
	Vec3f selVec = ctrlPolyPoint[polyC + c0].p->dpoint[ctrlPolyPoint[polyC + c0].i];

	int polyA = findClosestMidPoint(MultiSelect::ctrlPolySelList, selVec, a0, blockIdxA);
	int blockIdxB = blockIdxA;
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[blockIdxA].p, ctrlPolyPoint[blockIdxA].i);
	int polyB = findClosestMidPoint(MultiSelect::ctrlPolySelList, selVec, b0, blockIdxB);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[blockIdxB].p, ctrlPolyPoint[blockIdxB].i);

	int a1 = getCorner(a0, 1, 4);
	int b1 = getCorner(b0, 1, 4);

	Vec3f nearA = ctrlPolyPoint[polyA + a1].p->dpoint[ctrlPolyPoint[polyA + a1].i];
	Vec3f nearB = ctrlPolyPoint[polyB + b1].p->dpoint[ctrlPolyPoint[polyB + b1].i];

	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a0].p, ctrlPolyPoint[polyA + a0].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i);
//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b0].p, ctrlPolyPoint[polyB + b0].i);

	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyA + a1].p, ctrlPolyPoint[polyA + a1].i);
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyB + b1].p, ctrlPolyPoint[polyB + b1].i);

	int c3 = getCorner(c0, 3, 4);
	Vec3f selVec3 = ctrlPolyPoint[polyC + c3].p->dpoint[ctrlPolyPoint[polyC + c3].i];
	//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[polyC + c3].p, ctrlPolyPoint[polyC + c3].i);

	if((selVec3 - nearA).length() > (selVec3 - nearB).length()){
		mergeTo5CtrlPoly2(polyC, c0, polyA, a0, a1, polyB, b0, b1);
	}
	else{
		mergeTo5CtrlPoly2(polyC, c0, polyB, b0, b1, polyA, a0, a1);
	}
	M_Int m_poly = M_Int(polyA, polyB, polyC);

	bubbleSort(m_poly);
	deleteCtrlPoly(m_poly);

	commonEndSitchPolys();
}

Vec2f convertPointToScreen2d(Vec3f point);
bool triIntersectsRect(Vec2f a, Vec2f b, Vec2f c);
bool isRectCornerInTriangle(const Vec2f* tri);
bool backFace(const Vec2f& v0, const Vec2f& v1, const Vec2f& v2);

void Seamless::selectCtrlPolys(){
	g_viewMatrix = g_scene->getViewMatrix();
	for(int i = 0; i < ctrlPolyPoint.length;){
		Vec3f va3f = ctrlPolyPoint[i].p->dpoint[ctrlPolyPoint[i].i];
		int len = ctrlPolyPoint[i].len;
		for(int j = 1; j < len - 1; j++){
			Vec3f vb3f = ctrlPolyPoint[i + j].p->dpoint[ctrlPolyPoint[i + j].i];
			Vec3f vc3f = ctrlPolyPoint[i + j + 1].p->dpoint[ctrlPolyPoint[i + j + 1].i];
		
			Vec2f va = convertPointToScreen2d(va3f);
			Vec2f vb = convertPointToScreen2d(vb3f);
			Vec2f vc = convertPointToScreen2d(vc3f);

			if(g_scene->mode & BACKFACE_IGNORE_MODE){
				if(!backFace(va, vb, vc)){
					continue;
				}
			}

			Vec2f tri2d[3] = {va, vb, vc};

			if(MultiSelect::isInBox(va) || MultiSelect::isInBox(vb) || MultiSelect::isInBox(vc) || triIntersectsRect(va, vb, vc) || isRectCornerInTriangle(tri2d)){
				MultiSelect::ctrlPolySelList += i;
				break;
			}
		}
		i = getNextCtrlPoly(i);
	}

	//order list incremntaly and remove any double selections

	M_Bool sortList;
	sortList.length = ctrlPolyPoint.length;
	for(int k = MultiSelect::ctrlPolySelList.length; --k>=0;){
		sortList[MultiSelect::ctrlPolySelList[k]] = true;
	}
	MultiSelect::ctrlPolySelList.length = 0;
	for(int k = 0; k < sortList.length; k++){
		if(sortList[k]){
			MultiSelect::ctrlPolySelList+=k;
		}
	}
}


void selectBoxModelingPatchPolys2(int ia, int ib, int ic, int id, NurbsPatch* patch, const int uDim, const int vDim, const M_Vec3f& dpoint){
	Vec3f va3f = dpoint[ia];
	Vec3f vb3f = dpoint[ib];
	Vec3f vc3f = dpoint[ic];
	Vec3f vd3f = dpoint[id];

	Vec2f va = convertPointToScreen2d(va3f);
	Vec2f vb = convertPointToScreen2d(vb3f);
	Vec2f vc = convertPointToScreen2d(vc3f);
	Vec2f vd = convertPointToScreen2d(vd3f);

	if(g_scene->mode & BACKFACE_IGNORE_MODE){
		if(!backFace(va, vb, vc) || !backFace(vc, vd, va)){
			return;
		}
	}

	Vec2f tria2d[3] = {va, vb, vc};
	Vec2f trib2d[3] = {vc, vd, va};

	if(MultiSelect::isInBox(va) || MultiSelect::isInBox(vb) || MultiSelect::isInBox(vc) || MultiSelect::isInBox(vd) ||
		triIntersectsRect(va, vb, vc) || triIntersectsRect(vc, vd, va) ||
		isRectCornerInTriangle(tria2d) || isRectCornerInTriangle(trib2d)){
		MultiSelect::boxPolySelList += ControlPoint(patch, ia);
		MultiSelect::boxPolySelList += ControlPoint(patch, ib);
		MultiSelect::boxPolySelList += ControlPoint(patch, ic);
		MultiSelect::boxPolySelList += ControlPoint(patch, id);
	}
}

void bubbleSortBoxPolys(M_ControlPoint& boxPolySelList){
	bool flag = false;
	do{
		flag = false;
		for(int i = 0; i < boxPolySelList.length - 4; i += 4){

			if(boxPolySelList[i].compGT(boxPolySelList[i + 4])){

				ControlPoint swap = boxPolySelList[i + 4];
				boxPolySelList[i + 4] = boxPolySelList[i];
				boxPolySelList[i] = swap;

				swap = boxPolySelList[i + 5];
				boxPolySelList[i + 5] = boxPolySelList[i + 1];
				boxPolySelList[i + 1] = swap;

				swap = boxPolySelList[i + 6];
				boxPolySelList[i + 6] = boxPolySelList[i + 2];
				boxPolySelList[i + 2] = swap;

				swap = boxPolySelList[i + 7];
				boxPolySelList[i + 7] = boxPolySelList[i + 3];
				boxPolySelList[i + 3] = swap;

				flag = true;
			}
		}
	}while(flag);
}

void deleteDuplicateBoxPolys(M_ControlPoint& boxPolySelList){
	int d = 0;
	for(int s = 0; s <= boxPolySelList.length - 4; s += 4, d += 4){

		boxPolySelList[d + 0] = boxPolySelList[s + 0];
		boxPolySelList[d + 1] = boxPolySelList[s + 1];
		boxPolySelList[d + 2] = boxPolySelList[s + 2];
		boxPolySelList[d + 3] = boxPolySelList[s + 3];

		while(s < boxPolySelList.length - 4 && boxPolySelList[d].compEQ(boxPolySelList[s + 4])){
			s += 4;
		}
	}
	boxPolySelList.length = d;
}

void Seamless::selectBoxModelingPatchPolys(){
	g_viewMatrix = g_scene->getViewMatrix();

	if(MultiSelect::accumulate == false){
		MultiSelect::boxPolySelList.length = 0;
	}
	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];
		const int uDim = patch->dim.x;
		const int vDim = patch->dim.y;
		const M_Vec3f& dpoint = patch->dpoint;

		if(patch->isInsideOut()){
			for(int v = 0; v < vDim - 1; v+=2){
				for(int u = 0; u < uDim - 1; u+=2){
					int ia = (v + 2) * uDim + u;
					int ib = (v + 2) * uDim + u + 2;
					int ic = v * uDim + u + 2;
					int id = v * uDim + u;
					selectBoxModelingPatchPolys2(ia, ib, ic, id, patch, uDim, vDim, dpoint);
				}
			}
		}
		else{
			for(int v = 0; v < vDim - 1; v+=2){
				for(int u = 0; u < uDim - 1; u+=2){

					int ia = v * uDim + u;
					int ib = v * uDim + u + 2;
					int ic = (v + 2) * uDim + u + 2;
					int id = (v + 2) * uDim + u;
					selectBoxModelingPatchPolys2(ia, ib, ic, id, patch, uDim, vDim, dpoint);
				}
			}
		}
	}
	bubbleSortBoxPolys(MultiSelect::boxPolySelList);
	deleteDuplicateBoxPolys(MultiSelect::boxPolySelList);
}

void Seamless::calcAllLinks(){
	for(int k = link.length; --k>=0;){
		Link lnk = link[k];
		int idxA = lnk.poly + lnk.corner;
		Vec3f vecA = ctrlPolyPoint[idxA].p->dpoint[ctrlPolyPoint[idxA].i];
		int idxB = lnk.poly + getCorner(lnk.corner, 1, ctrlPolyPoint[lnk.poly].len);
		Vec3f vecB = ctrlPolyPoint[idxB].p->dpoint[ctrlPolyPoint[idxB].i];
		Vec3f linkVec = vecA + (vecB - vecA) * lnk.between;
		lnk.patch->point.refValue()[lnk.point] = linkVec * lnk.patch->weight[lnk.point];
	}
}

void Seamless::linkPointToEdge(int poly){
	NurbsPatch* patch = MultiSelect::pointList[0].p;
	int idx = MultiSelect::pointList[0].i;
	Vec3f selVec = patch->dpoint[idx];
	int len = ctrlPolyPoint[poly].len;
	float shortest = 10000000;
	Vec3f shortestEdgePoint;
	int shortestCorner = 0;
	float shortestRatio = 0;
	for(int a = 0; a < len; a++){
		Vec3f vecA = ctrlPolyPoint[poly + a].p->dpoint[ctrlPolyPoint[poly + a].i];
		int b = getCorner(a, 1, len);
		Vec3f vecB = ctrlPolyPoint[poly + b].p->dpoint[ctrlPolyPoint[poly + b].i];
		float edgeDistance = (vecB - vecA).length();
		float selDistance = (selVec - vecA).length();
		float ratio = selDistance / edgeDistance;

		Vec3f edgePoint = vecA + (vecB - vecA) * ratio;
		float diffDistance = (edgePoint - selVec).length();
		if(diffDistance < shortest){
			shortest = diffDistance;
			shortestEdgePoint = edgePoint;
			shortestCorner = a;
			shortestRatio = ratio;
		}
	}
	patch->point.refValue()[idx] = shortestEdgePoint * patch->weight[idx];
	link += Link(patch, idx, poly, shortestCorner, shortestRatio);

	MultiSelect::pointList.length = 0;
	BackUp::monostable.beginCycle();
	calcAllPolyMidPoints();
	houseKeepVisualClue();
}


void printMidPointIndices(int midIndex, const M_CtrlPolyPoint& ctrlPolyPoint, int len){
	for(int c = 0; c < len; c++){
		int midLen = ctrlPolyPoint[midIndex].len;
		dLog(String("midLen ") + midLen);
		for(int i = 0; i < midLen; i++){
			String str = ctrlPolyPoint[midIndex + i].p->name;
			str += ' ';
			str += ctrlPolyPoint[midIndex + i].i;
			dLog(str);
		}
		if(midLen == 0){
			midLen = 1;
		}
		midIndex += midLen;
	}
}

void copyMidPoint(M_CtrlPolyPoint& ctrlPolyPoint, int& poly, const int beginMid, int& corner, const int len, const NurbsPatch* block){
	int midLen = ctrlPolyPoint[poly].len;
	if(midLen == 0){
		ctrlPolyPoint += CtrlPolyPoint();
		midLen = 1;
	}
	else{
		int mark = ctrlPolyPoint.length;
		int actualLen = 0;
		for(int i = 0; i < midLen; i++){
			NurbsPatch* p = ctrlPolyPoint[poly + i].p;
			if(p != block){
				ctrlPolyPoint += CtrlPolyPoint(p, ctrlPolyPoint[poly + i].i, 0);
				actualLen ++;
			}
		}
		if(actualLen == 0){
			ctrlPolyPoint += CtrlPolyPoint();
		}
		else{
			ctrlPolyPoint[mark].len = actualLen;
		}
	}
	if(++corner == len){
		corner = 0;
		poly = beginMid;
	}
	else{
		poly += midLen;
	}
}

void copyMidPoint(M_CtrlPolyPoint& ctrlPolyPoint, int poly){
	int midLen = ctrlPolyPoint[poly].len;
	if(midLen == 0){
		ctrlPolyPoint += CtrlPolyPoint();
		midLen = 1;
	}
	else{
		int mark = ctrlPolyPoint.length;
		for(int i = 0; i < midLen; i++){
			NurbsPatch* p = ctrlPolyPoint[poly + i].p;
			ctrlPolyPoint += CtrlPolyPoint(p, ctrlPolyPoint[poly + i].i, midLen);
		}
	}
}

void copyMidPointExclusive(M_CtrlPolyPoint& ctrlPolyPoint, int& poly, const int beginMid, int& corner, const int len, const NurbsPatch* target, NurbsPatch* newPatch, int newIdx){
	int mark = ctrlPolyPoint.length;

	int midLen = ctrlPolyPoint[poly].len;
	if(midLen == 0){
		ctrlPolyPoint += CtrlPolyPoint();
		midLen = 1;
	}
	else{
		int mark = ctrlPolyPoint.length;
		int actualLen = 0;
		for(int i = 0; i < midLen; i++){
			NurbsPatch* p = ctrlPolyPoint[poly + i].p;
			if(p == target){
				ctrlPolyPoint += CtrlPolyPoint(p, ctrlPolyPoint[poly + i].i, 0);
				actualLen ++;
			}
		}
		if(actualLen == 0){
			ctrlPolyPoint += CtrlPolyPoint();
		}
		else{
			ctrlPolyPoint[mark].len = actualLen;
		}
	}
	if(++corner == len){
		corner = 0;
		poly = beginMid;
	}
	else{
		poly += midLen;
	}

	insertMidPoint(ctrlPolyPoint, mark, newPatch, newIdx);
}





void Seamless::findStitchPolyForExtrude(M_Int& deleteCtrlPolyList, NurbsPatch* checkPatch, const int checkIdx){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		int beginPoly = poly;
		poly += len;
		int remCtrlPoly = poly;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			for(int i = 0; i < midLen; i++){
				if(ctrlPolyPoint[poly + i].p == checkPatch && ctrlPolyPoint[poly + i].i == checkIdx){
					deleteCtrlPolyList += beginPoly;
					return;
				}
			}
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}
		
		int centerLen = ctrlPolyPoint[poly].len;

		for(int i = 0; i < centerLen; i++){
			if(ctrlPolyPoint[poly + i].p == checkPatch && ctrlPolyPoint[poly + i].i == checkIdx){
				deleteCtrlPolyList += beginPoly;
				return;
			}
		}

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}
/*
void Seamless::uSouthFindStitchPolyAndSplit(M_Int& deleteCtrlPolyList, NurbsPatch* checkPatch, int checkIdx, int checkIdx2, const float ratio){
	int uDim = checkPatch->dim.x;
	bool eastOfHidden = false;
	if(((checkIdx % uDim) & 1) == 0){
		eastOfHidden = true;
		checkIdx--;
		checkIdx2--;
	}
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		for(int k = len; --k >= 0;){
			if(ctrlPolyPoint[poly + k].p == checkPatch && ctrlPolyPoint[poly + k].i == checkIdx){
				int chk2 = getCorner(k, 3, len);
				if(ctrlPolyPoint[poly + chk2].p == checkPatch && ctrlPolyPoint[poly + chk2].i == checkIdx2){
					int uIdx2 = checkIdx2 % uDim;
					uSlideEdgeStitchPoly(poly, checkPatch, uIdx2);


					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + 2, len);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx - uDim + 2, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx2 - uDim, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx2, 0);

					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx2 - uDim - 1, 1);
					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint();

					ctrlPolyPoint += CtrlPolyPoint();
					return;
				}
			}
		}
		poly += len;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}
		
		int centerLen = ctrlPolyPoint[poly].len;


		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}
*/
bool isContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const int absoluteIdx, const NurbsPatch* checkPatch, const int checkIdx){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch && ctrlPolyPoint[absoluteIdx + i].i == checkIdx){
			return true;
		}
	}
	return false;
}

bool isPatchContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const int absoluteIdx, const NurbsPatch* checkPatch){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch){
			return true;
		}
	}
	return false;
}

int getIndexContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const int absoluteIdx, const NurbsPatch* checkPatch){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch){
			return ctrlPolyPoint[absoluteIdx + i].i;
		}
	}
	return -1;
}

int getIndexContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const int absoluteIdx, const NurbsPatch* checkPatch, const int entryIdx2, const bool doU){
	int uDim = checkPatch->dim.x;
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		NurbsPatch* p = ctrlPolyPoint[absoluteIdx + i].p;
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch){
			int entryIdx = ctrlPolyPoint[absoluteIdx + i].i;
			if(doU){
				if(abs(entryIdx2 - entryIdx) == 1){
					return entryIdx;
				}
			}
			else{
				if(abs(entryIdx2 - entryIdx) == uDim){
					return entryIdx;
				}
			}
			
		}
	}
	return -1;
}

bool getMatchingMidPoint2(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* checkPatch, int& otherIdx, const int idx2){
	otherIdx = -1;
	int diff = 0;
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch){
			int d = ctrlPolyPoint[absoluteIdx + i].i;
			if(otherIdx < 0){
				otherIdx = d;
				diff = abs(d - idx2);
			}
			else if(abs(d - idx2) < diff){
				otherIdx = d;
				diff = abs(d - idx2);
			}
		}
	}
	if(otherIdx >= 0){
		return true;
	}
	return false;
}

bool getMatchingMidPoint2B(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* checkPatch, const int checkIdx, const NurbsPatch* otherPatch, int& otherIdx, const int otherIdx2){
	int other_uDim = otherPatch->dim.x;
	int diff = 0;
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == otherPatch){
			otherIdx = ctrlPolyPoint[absoluteIdx + i].i;
			if(false == (checkPatch == otherPatch && otherIdx == checkIdx)){
				if(otherIdx == otherIdx2 - 1){
					return true;
				}
				if(otherIdx == otherIdx2 + 1){
					return true;
				}
				if(otherIdx == otherIdx2 - other_uDim){
					return true;
				}
				if(otherIdx == otherIdx2 + other_uDim){
					return true;
				}
			}
		}
	}
	return false;
}
/*
bool getMatchingMidPoint3(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* checkPatch, int& otherIdx, const int idx2, int blockIdx){
	otherIdx = -1;
	int diff = 0;
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch && ctrlPolyPoint[absoluteIdx + i].i != blockIdx){
			int d = ctrlPolyPoint[absoluteIdx + i].i;
			if(otherIdx < 0){
				otherIdx = d;
				diff = abs(d - idx2);
			}
			else if(abs(d - idx2) < diff){
				otherIdx = d;
				diff = abs(d - idx2);
			}
		}
	}
	if(otherIdx >= 0){
		return true;
	}
	return false;
}
*/
bool getMatchingMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* checkPatch, int& otherIdx){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch){
			otherIdx = ctrlPolyPoint[absoluteIdx + i].i;
			return true;
		}
	}
	return false;
}

void uGetEdgePoint(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* altPatch, int& altIdx, int& altIdx2, bool& altLeaveNorthOrEast){
	int altUDim = altPatch->dim.x;
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == altPatch){
			int d = ctrlPolyPoint[absoluteIdx + i].i;
			if(altIdx - d == altUDim){
				altLeaveNorthOrEast = true;
				altIdx = d;
				altIdx2 -= altUDim;
				break;
			}
			if(altIdx - d == -altUDim){
				altLeaveNorthOrEast = false;
				altIdx = d;
				altIdx2 += altUDim;
				break;
			}
		}
	}
}

void vGetEdgePoint(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* altPatch, int& altIdx, int& altIdx2, bool& altLeaveNorthOrEast){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == altPatch){
			int d = ctrlPolyPoint[absoluteIdx + i].i;
			if(altIdx - d == 1){
				altLeaveNorthOrEast = true;
				altIdx = d;
				altIdx2 -= 1;
				break;
			}
			if(altIdx - d == -1){
				altLeaveNorthOrEast = false;
				altIdx = d;
				altIdx2 += 1;
				break;
			}
		}
	}
}

//NurbsPatch* anotherPatchContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int midIdx, NurbsPatch* checkPatch, int& otherMidIdx){
//	int len = ctrlPolyPoint[midIdx].len;
//	for(int i = len; --i >= 0;){
//		NurbsPatch* otherPatch = ctrlPolyPoint[midIdx + i].p;
//		if(otherPatch != checkPatch){
//			otherMidIdx = ctrlPolyPoint[midIdx + i].i;
//			return otherPatch;
//		}
//	}
//	return 0;
//}

NurbsPatch* anotherPatchContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int midIdx, int centerIdx, NurbsPatch* checkPatch, int& otherMidIdx, int& otherCenterIdx){
	int len = ctrlPolyPoint[centerIdx].len;
	for(int i = len; --i >= 0;){
		NurbsPatch* otherPatch = ctrlPolyPoint[centerIdx + i].p;
		if(otherPatch != checkPatch){
			otherCenterIdx = ctrlPolyPoint[centerIdx + i].i;
			if(getMatchingMidPoint(ctrlPolyPoint, midIdx, otherPatch, otherMidIdx)){
				return otherPatch;
			}
		}
	}
	return 0;
}


NurbsPatch* anotherPatchContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const NurbsPatch* checkPatch, int centerPoly, int midPoly, int& otherIdx, int& otherIdx2){
	int len = ctrlPolyPoint[midPoly].len;
	for(int i = len; --i >= 0;){
		NurbsPatch* otherPatch = ctrlPolyPoint[midPoly + i].p;
		if(otherPatch != checkPatch){
			otherIdx2 = ctrlPolyPoint[midPoly + i].i;
			if(getMatchingMidPoint2(ctrlPolyPoint, centerPoly, otherPatch, otherIdx, otherIdx2)){
				return otherPatch;
			}
		}
	}
	return 0;
}

NurbsPatch* anotherPointContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const NurbsPatch* checkPatch, const int checkIdx, const int checkIdx2, int centerPoly, int midPoly, int& otherIdx, int& otherIdx2){
	int len = ctrlPolyPoint[midPoly].len;
	for(int i = len; --i >= 0;){
		NurbsPatch* otherPatch = ctrlPolyPoint[midPoly + i].p;
		otherIdx2 = ctrlPolyPoint[midPoly + i].i;
		if(otherPatch != checkPatch || checkIdx2 != otherIdx2){
			if(getMatchingMidPoint2B(ctrlPolyPoint, centerPoly, checkPatch, checkIdx, otherPatch, otherIdx, otherIdx2)){
				return otherPatch;
			}
		}
	}
	return 0;
}
/*
NurbsPatch* anotherPointSamePatchContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const NurbsPatch* checkPatch, int centerPoly, int midPoly, int& otherIdx, int& otherIdx2, const int checkIdx){
	int len = ctrlPolyPoint[midPoly].len;
	for(int i = len; --i >= 0;){
		NurbsPatch* otherPatch = ctrlPolyPoint[midPoly + i].p;
		if(otherPatch == checkPatch){
			otherIdx2 = ctrlPolyPoint[midPoly + i].i;
			if(getMatchingMidPoint3(ctrlPolyPoint, centerPoly, otherPatch, otherIdx, otherIdx2, checkIdx)){
				return otherPatch;
			}
		}
	}
	return 0;
}
*/

NurbsPatch* anotherPatchContainedInMidPointForDeleteControlLine(const M_CtrlPolyPoint& ctrlPolyPoint, int poly, int midIdx, NurbsPatch* checkPatch, int& otherMidIdx, int& otherMidIdx2){
	int len = ctrlPolyPoint[midIdx].len;
	for(int i = len; --i >= 0;){
		NurbsPatch* otherPatch = ctrlPolyPoint[midIdx + i].p;
		if(otherPatch != checkPatch){
			otherMidIdx = ctrlPolyPoint[midIdx + i].i;
			otherMidIdx2 = ctrlPolyPoint[poly].i;
			if(ctrlPolyPoint[poly].p == otherPatch){
				return otherPatch;
			}
		}
	}
	return 0;
}
/*
void Seamless::uNorthFindStitchPolyAndSplit(M_Int& deleteCtrlPolyList, NurbsPatch* checkPatch, int checkIdx, int checkIdx2, M_Int& lookup, const NurbsPatch* firstPatch, bool& foundFirstPatch, const float ratio, M_Insert& insert){
	int uDim = checkPatch->dim.x;
	bool eastOfHidden = false;
	if(((checkIdx % uDim) & 1) == 0){
		eastOfHidden = true;
		checkIdx--;
		checkIdx2--;
	}

	int vIdx = checkIdx / uDim;
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		for(int k = len; --k >= 0;){
			if(ctrlPolyPoint[poly + k].p == checkPatch && ctrlPolyPoint[poly + k].i == checkIdx){
				int chk2 = getCorner(k, len - 1, len);
				int midIndex = poly + len;
				int absoluteIdx = getMidPointAbsoluteIdx(midIndex, chk2, ctrlPolyPoint);
				
				if(ctrlPolyPoint[poly + chk2].p == checkPatch && ctrlPolyPoint[poly + chk2].i == checkIdx2){
					absoluteIdx = getMidPointAbsoluteIdx(midIndex, k, ctrlPolyPoint);
					if(isContainedInMidPoint(ctrlPolyPoint, absoluteIdx, checkPatch, checkIdx + uDim)){
						int kb = getCorner(k, 1, len);
						NurbsPatch* patchB = ctrlPolyPoint[poly + kb].p;
						int uDimB = patchB->dim.x;
						int idxB = ctrlPolyPoint[poly + kb].i;
						int uB = idxB % uDimB;
						int vB = idxB / uDimB;

						if(patchB != checkPatch){
							if(patchB == firstPatch){
								foundFirstPatch = true;
								//uFindStitchPolyAndSlideInnerEdge(checkPatch);
								//return;
							}
							else{
								int col = idxB % uDimB - 2;
								if(eastOfHidden){
									col++;
								}
								patchB->uInsertLine(lookup, firstPatch, foundFirstPatch, deleteCtrlPolyList, , ratio, insert);
								uDimB += 2;
								uB += 2;
							}
						}
						else{
							//int uIdx = checkIdx % uDim;
							//uFindStitchPolyAndSlideInnerEdge(checkPatch, uIdx);
							uFindStitchPolyAndSlideInnerEdge(checkPatch);
							return;
						}
						int nextLine = 2;
						int nextLineB = 2;
						idxB = vB * uDimB + uB;

						ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx, 4);
						ctrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);
						ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - nextLineB, 0);
						ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx - nextLine, 0);

						ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim, 2);
						ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - uDimB, 0);
						ctrlPolyPoint += CtrlPolyPoint();
						ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim - nextLine, 2);
						ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - uDimB - nextLine, 0);
						ctrlPolyPoint += CtrlPolyPoint();

						ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim - 1, 2);
						ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - uDimB - 1, 0);
						int uIdx = checkIdx % uDim;
						uSlideEdgeStitchPoly(poly, checkPatch, uIdx);
						uSlideEdgeStitchPoly(poly, patchB, uB);
						//uFindStitchPolyAndSlideInnerEdge(checkPatch, uIdx);
						return;
					}
					int uIdx = checkIdx % uDim;
					int vIdx = checkIdx / uDim;
					int nextLine = uDim * 2;
					uSlideEdgeStitchPoly(poly, checkPatch, uIdx);

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx, len);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx2 + uDim + 2, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx2 + 2, 0);

					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim - 1, 1);
					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint();

					ctrlPolyPoint += CtrlPolyPoint();

					//uFindStitchPolyAndSlideInnerEdge(checkPatch, uIdx);

//					uCopyStitchPolyAndSlide(poly, 2);

						//deleteCtrlPolyList += poly;

					return;
				}
				else if(isContainedInMidPoint(ctrlPolyPoint, absoluteIdx, checkPatch, checkIdx2)){
					int nextLine = 2;

					int kb = getCorner(k, 1, len);
					NurbsPatch* patchB = ctrlPolyPoint[poly + kb].p;
					int uDimB = patchB->dim.x;
					int idxB = ctrlPolyPoint[poly + kb].i;
					int uB = idxB % uDimB;
					int vB = idxB / uDimB;

					if(patchB != checkPatch){
						if(patchB == firstPatch){
							//foundFirstPatch = true;
							//uFindStitchPolyAndSlideInnerEdge(checkPatch);
							//return;
						}
						else{
							int col = idxB % uDimB - 2;
							patchB->uInsertLine(lookup, firstPatch, foundFirstPatch, deleteCtrlPolyList, col, ratio, insert);
							uB += 2;
						}
					}
					else{
						//int uIdx = checkIdx % uDim;
						//uFindStitchPolyAndSlideInnerEdge(checkPatch, uIdx);
						uFindStitchPolyAndSlideInnerEdge(checkPatch);
						return;
					}
//				if(vIdx == 3){
					//int uIdx = checkIdx % uDim;
					uFindStitchPolyAndSlideInnerEdge(checkPatch);
					
					uDimB = patchB->dim.x;
					int nextLineB = 2;
					idxB = vB * uDimB + uB;

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx, 4);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - nextLineB, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx - nextLine, 0);

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim, 2);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - uDimB, 0);
					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim - nextLine, 2);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - uDimB - nextLine, 0);
					ctrlPolyPoint += CtrlPolyPoint();

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim - 1, 2);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - uDimB - 1, 0);
					return;
					//deleteCtrlPolyList += poly;

					k = getCorner(k, 1, len);
					patchB = ctrlPolyPoint[poly + k].p;
					idxB = ctrlPolyPoint[poly + k].i;

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx, len);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - 2, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx - 2, 0);

					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB - 1, 1);
					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint();

					ctrlPolyPoint += CtrlPolyPoint();
					////////////
					
					k = getCorner(k, 1, len);
					NurbsPatch* patch2B = ctrlPolyPoint[poly + k].p;
					int idx2B = ctrlPolyPoint[poly + k].i;

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx - 2, len);
					ctrlPolyPoint += CtrlPolyPoint(patch2B, idx2B + 2, 0);
					ctrlPolyPoint += CtrlPolyPoint(patch2B, idx2B, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx - 4, 0);

					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint(patch2B, idx2B + 1, 1);
					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint();

					ctrlPolyPoint += CtrlPolyPoint();
					return;
				}
			}
		}
		poly += len;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}
		
		int centerLen = ctrlPolyPoint[poly].len;


		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}
*/

int Seamless::getStitchPoly(NurbsPatch* patch, int checkIdx, int checkIdx2)const{
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int beginPoly = poly;
		int len = ctrlPolyPoint[poly].len;
		int midIndex = poly + len;

		poly += len;
		for(int k = len + 1; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			else{
				for(int k = midLen; --k >= 0;){
					int idxA = poly + k;
					if(ctrlPolyPoint[idxA].p == patch && ctrlPolyPoint[idxA].i == checkIdx){
						if(patch->isInStitchPoly(checkIdx2, beginPoly, ctrlPolyPoint)){
							return beginPoly;
						}
					}
				}
			}
			poly += midLen;
		}
	}
	return -1;
}
/*
bool getMarkEdgeIndex(const M_CtrlPolyPoint& ctrlPolyPoint, const int midIdx, const NurbsPatch* p, const int mIdxA, const int mIdxB, int& alternativeEdgeIdx){
	int len = ctrlPolyPoint[midIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[midIdx + i].p == p){
			int edgeIdx = ctrlPolyPoint[midIdx + i].i;
			
			if(edgeIdx == mIdxA){
				alternativeEdgeIdx = mIdxA;
				return true;
			}
			if(edgeIdx == mIdxB){
				alternativeEdgeIdx = mIdxB;
				return true;
			}
		}
	}
	return false;
}

NurbsPatch* Seamless::getAlternativeEdge(const int len, const int midBegin, int centerBegin, int& alternativeCentreIdx, int& alternativeEdgeIdx, M_DiversionMark& diversionMark, int& diversionMarkIdx, NurbsPatch* otherPatch, bool prevDoU){
	int centerLen = ctrlPolyPoint[centerBegin].len;
	for(int i = len; --i >= 0;){
		NurbsPatch* p = ctrlPolyPoint[centerBegin + i].p;
		for(int k = 0; k < g_insertPatch.m_diversionMark.length; k++){
			if(p == diversionMark[k].markPatch){//markPatch is alternatie patch
				int uDim = p->dim.x;
				if(diversionMark[k].doU){
					alternativeCentreIdx = diversionMark[k].markIdxA;
					alternativeCentreIdx = alternativeCentreIdx - uDim + 1;
					if(alternativeCentreIdx == uDim - 1 && prevDoU == false){
						alternativeCentreIdx = 0;
					}
					if(alternativeCentreIdx == ctrlPolyPoint[centerBegin + i].i){
						//now we have verified the center, verify the edge
						const int mIdxA = diversionMark[k].markIdxA - uDim;
						const int mIdxB = diversionMark[k].markIdxB - uDim;
						int poly = midBegin;
						for(int m = len; --m >= 0;){
							int midLen = ctrlPolyPoint[poly].len;
							if(midLen){
								if(isPatchContainedInMidPoint(ctrlPolyPoint, poly, otherPatch)){
									if(getMarkEdgeIndex(ctrlPolyPoint, poly, p, mIdxA, mIdxB, alternativeEdgeIdx)){
										return p;
									}
								}
							}
							else{
								midLen++;
							}
							poly += midLen;
						}
					}
				}
				else{
					int vDim = p->dim.y;
					alternativeCentreIdx = diversionMark[k].markIdxA;
					alternativeCentreIdx = alternativeCentreIdx - 1 + uDim;
					if(prevDoU && alternativeCentreIdx == (vDim - 1) * uDim){
						alternativeCentreIdx = 0;
					}
					if(alternativeCentreIdx == ctrlPolyPoint[centerBegin + i].i){
						//now we have verified the center, verify the edge
						const int mIdxA = diversionMark[k].markIdxA - 1;
						const int mIdxB = diversionMark[k].markIdxB - 1;
						int poly = midBegin;
						for(int m = len; --m >= 0;){
							int midLen = ctrlPolyPoint[poly].len;
							if(midLen){
								if(isPatchContainedInMidPoint(ctrlPolyPoint, poly, otherPatch)){
									if(getMarkEdgeIndex(ctrlPolyPoint, poly, p, mIdxA, mIdxB, alternativeEdgeIdx)){
										return p;
									}
								}
							}
							else{
								midLen++;
							}
							poly += midLen;
						}
					}
				}
			}
		}
	}
	return 0;
}
*/

int getMarkMidPoint(M_CtrlPolyPoint& ctrlPolyPoint, int midBegin, int edgeIdx, NurbsPatch* patch){
	int poly = midBegin;
	for(int k = 0; k <= edgeIdx; k++){
		int midLen = ctrlPolyPoint[poly].len;
		if(k == edgeIdx){
			for(int i = midLen; --i>=0;){
				if(ctrlPolyPoint[poly + i].p == patch){
					return ctrlPolyPoint[poly + i].i;
				}
			}
		}
		if(midLen == 0){
			midLen++;
		}
		poly += midLen;
	}
	return -1;
}

int getMarkCentrePoint(M_CtrlPolyPoint& ctrlPolyPoint, int centreBegin, NurbsPatch* patch, int alternativeEdgeIdx, bool markDoU, int mark_uDim){
	int len = ctrlPolyPoint[centreBegin].len;
	for(int i = len; --i>=0;){
		if(ctrlPolyPoint[centreBegin + i].p == patch){
			int centerIdx = ctrlPolyPoint[centreBegin + i].i;
			if(markDoU){
				if(abs(alternativeEdgeIdx - centerIdx) == 1){
					return centerIdx;
				}
			}
			else{
				if(abs(alternativeEdgeIdx - centerIdx) == mark_uDim){
					return centerIdx;
				}
			}
		}
	}
	return -1;
}

bool compareMirrorPoints(NurbsPatch* patchA, int idxA, NurbsPatch* patchB, int idxB){
	if(patchA->mirror && patchB->mirror){
		if(patchA->mirrorPatch){
			if(patchA->mirrorPatch == patchB){
				if(idxA == idxB){
					return true;
				}
			}
		}
		else if(patchA == patchB){
			if(patchA->getMirrorTwinlessIdx(idxA) == idxB){
				return true;
			}
		}
	}
	return false;
}

int returnMirrorIdx(NurbsPatch* patch, int idx){
	if(patch->mirror){
		if(patch->mirrorPatch){
			return idx;
		}
		else{
			return patch->getMirrorTwinlessIdx(idx);
		}
	}
	return 0;
}

bool compareMirrorEdge(const M_CtrlPolyPoint& ctrlPolyPoint, int beginPolyA, int polyEdgeA, int beginPolyB, int polyEdgeB, int len){
	if(beginPolyB < 0){
		return false;
	}
	int ai = beginPolyA + polyEdgeA;
	int bi = beginPolyB + getCorner(polyEdgeB, 1, len);
	if(compareMirrorPoints(ctrlPolyPoint[ai].p, ctrlPolyPoint[ai].i, ctrlPolyPoint[bi].p, ctrlPolyPoint[bi].i)){
		ai = beginPolyA + getCorner(polyEdgeA, 1, len);
		bi = beginPolyB + polyEdgeB;
		if(compareMirrorPoints(ctrlPolyPoint[ai].p, ctrlPolyPoint[ai].i, ctrlPolyPoint[bi].p, ctrlPolyPoint[bi].i)){
			return true;
		}
	}
	return false;
}

bool matchingDiversionEntryMirror(const M_CtrlPolyPoint& ctrlPolyPoint, int beginPoly, int entryPolyEdge, int exitPolyEdge, int alternativePolyEdge, int len, const M_Insert& m_insert, int& diversionMarkIdx){
	for(diversionMarkIdx = m_insert.length; --diversionMarkIdx>=0;){
		bool failed = false;
		const Insert& diversionMark = m_insert[diversionMarkIdx];
		for(int i = 0; i < len; i++){
			if(false == compareMirrorEdge(ctrlPolyPoint, beginPoly, getCorner(alternativePolyEdge, i, len), diversionMark.beginPoly, getCorner(diversionMark.entryPolyEdge, len - i, len), len)){
				failed = true;
				break;
			}
		}
		if(failed == false){
			return true;
		}
	}
	return false;
}

//bool matchingDiversionEntry(int beginPoly, int entryPolyEdge, int exitPolyEdge, M_Insert& m_insert, int& diversionMarkIdx){
//	for(int diversionMarkIdx = m_insert.length; --diversionMarkIdx>=0;){
//		const Insert& diversionMark = m_insert[diversionMarkIdx];
//		if(	diversionMark.beginPoly == beginPoly &&
//			diversionMark.entryPolyEdge == entryPolyEdge &&
//			diversionMark.exitPolyEdge == exitPolyEdge
//			){
//			return true;
//		}
//	}
//
//	return false;
//}


bool isEdgeClicked(const M_CtrlPolyPoint& ctrlPolyPoint, int const len, const int beginPoly, const int entryPolyEdge, int& alternativePolyEdge, NurbsPatch*& sharedMarkPatch, int& sharedMarkIdx, int& mark_uDim, int& mark_vDim, NurbsPatch* checkPatch){
	alternativePolyEdge = getCorner(entryPolyEdge, 1, len);
	NurbsPatch* markPatchA = ctrlPolyPoint[beginPoly + alternativePolyEdge].p;
	int markIdxA = ctrlPolyPoint[beginPoly + alternativePolyEdge].i;
	NurbsPatch* markPatchB = ctrlPolyPoint[beginPoly + getCorner(alternativePolyEdge, 1, len)].p;
	int markIdxB = ctrlPolyPoint[beginPoly + getCorner(alternativePolyEdge, 1, len)].i;

	//MultiSelect::pointList.length = 0;
	//MultiSelect::pointList += ControlPoint(markPatchA, markIdxA);
	//MultiSelect::pointList += ControlPoint(markPatchB, markIdxB);
	//houseKeepVisualClue();
	//return;
									
	sharedMarkPatch = markPatchB;
	sharedMarkIdx = markIdxB;
	mark_uDim = sharedMarkPatch->dim.x;
	mark_vDim = sharedMarkPatch->dim.y;

	Vec3f dpA = g_viewMatrix * markPatchA->dpoint[markIdxA];
	Vec3f dpB = g_viewMatrix * markPatchB->dpoint[markIdxB];

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(dpA.x, dpA.y);
	Vec2f ptB = Vec2f(dpB.x, dpB.y);
	Vec2f crossPoint;

									
	if(clickLine(click, ptA, ptB, crossPoint)){
		if(checkPatch == markPatchA){
			sharedMarkPatch = markPatchA;
			sharedMarkIdx = markIdxA;
		}
		return true;
	}
	alternativePolyEdge = getCorner(entryPolyEdge, 3, len);

	markPatchA = ctrlPolyPoint[beginPoly + alternativePolyEdge].p;
	markIdxA = ctrlPolyPoint[beginPoly + alternativePolyEdge].i;
	markPatchB = ctrlPolyPoint[beginPoly + getCorner(alternativePolyEdge, 1, len)].p;
	markIdxB = ctrlPolyPoint[beginPoly + getCorner(alternativePolyEdge, 1, len)].i;

	//MultiSelect::pointList.length = 0;
	//MultiSelect::pointList += ControlPoint(markPatchA, markIdxA);
	//MultiSelect::pointList += ControlPoint(markPatchB, markIdxB);
	//houseKeepVisualClue();
									
	sharedMarkPatch = markPatchA;
	sharedMarkIdx = markIdxA;
	mark_uDim = sharedMarkPatch->dim.x;


	dpA = g_viewMatrix * markPatchA->dpoint[markIdxA];
	dpB = g_viewMatrix * markPatchB->dpoint[markIdxB];

	click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	ptA = Vec2f(dpA.x, dpA.y);
	ptB = Vec2f(dpB.x, dpB.y);

	if(clickLine(click, ptA, ptB, crossPoint)){
		if(checkPatch == markPatchB){
			sharedMarkPatch = markPatchB;
			sharedMarkIdx = markIdxB;
		}

		return true;
	}
	return false;
}

bool isOnPatchEdge(int& checkIdx, int& checkIdx2, int uCheckPatchDim, int vCheckPatchDim, bool doU, bool leaveNorthOrEast){
	int opposite_uOrV = 0;
	if(doU){
		opposite_uOrV = checkIdx / uCheckPatchDim;
		if(leaveNorthOrEast){
			if(opposite_uOrV < vCheckPatchDim - 1){
				checkIdx += 2 * uCheckPatchDim;
				checkIdx2 += 2 * uCheckPatchDim;
				return true;
			}
		}
		else{
			if(opposite_uOrV > 0){
				checkIdx -= 2 * uCheckPatchDim;
				checkIdx2 -= 2 * uCheckPatchDim;
				return true;
			}
		}
	}
	else{
		opposite_uOrV = checkIdx % uCheckPatchDim;
		if(leaveNorthOrEast){
			if(opposite_uOrV < uCheckPatchDim - 1){
				checkIdx += 2;
				checkIdx2 += 2;
				return true;
			}
		}
		else{
			if(opposite_uOrV > 0){
				checkIdx -= 2;
				checkIdx2 -= 2;
				return true;
			}
		}
	}
	return false;
}



void Seamless::deleteLineFindMatchingEdge(NurbsPatch* checkPatch, int checkIdx, int checkIdx2, const NurbsPatch* firstPatch, const int firstIdx2, bool& foundFirst, M_DelCtrlLn& delCtrlLn){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int beginPoly = poly;
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		const int midBegin = poly;
		for(int k = 0; k < len; k++){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen){
				if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
					int d = 0;
					int e = beginPoly + k;
					int f = beginPoly + getCorner(k, 1, len);
					int g = e;
					if(ctrlPolyPoint[e].p == checkPatch && ctrlPolyPoint[e].i == checkIdx2){
						g = f;
					}
					else if(ctrlPolyPoint[f].p != checkPatch || ctrlPolyPoint[f].i != checkIdx2){
						return;
					}

					
					//if(checkPatch->name == "main" && checkIdx2 == 23){
					//	MultiSelect::pointList.length = 0;
					//	MultiSelect::pointList += ControlPoint(ctrlPolyPoint[beginPoly + getCorner(k, d, len)].p, ctrlPolyPoint[beginPoly + getCorner(k, d, len)].i);
					//	//MultiSelect::pointList += ControlPoint(checkPatch, checkIdx);
					//	//MultiSelect::pointList += ControlPoint(checkPatch, 230);
					//	//MultiSelect::pointList += ControlPoint(checkPatch, 207);
					//	//MultiSelect::pointList += ControlPoint(checkPatch, 1);
					//	//MultiSelect::pointList += ControlPoint(checkPatch, 2);
					//	MultiSelect::ctrlPolySelList.length = 0;
					//	MultiSelect::ctrlPolySelList+=beginPoly;
					//	houseKeepVisualClue();
					//	return;
					//}

					const int holdPoly = poly;
					poly = midBegin;
					int otherIdx = 0;
					int otherIdx2 = 0;
					if(NurbsPatch* otherPatch = anotherPatchContainedInMidPointForDeleteControlLine(ctrlPolyPoint, g, holdPoly, checkPatch, otherIdx, otherIdx2)){
						if(firstPatch == otherPatch && firstIdx2 == otherIdx2){
							foundFirst = true;
							return;
						}

						int uDim = otherPatch->dim.x;
						if(abs(otherIdx - otherIdx2) == 1){
							otherPatch->uDeleteLine(otherIdx, otherIdx2, firstPatch, firstIdx2, foundFirst, delCtrlLn);
						}
						else{
							otherPatch->vDeleteLine(otherIdx, otherIdx2, firstPatch, firstIdx2, foundFirst, delCtrlLn);
						}
					}
					return;
				}
			}
			else{
				midLen++;
			}
			poly += midLen;
		}

		int centerLen = ctrlPolyPoint[poly].len;

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}

bool getOtherPatchContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* checkPatch, NurbsPatch*& otherPatch, int& otherIdx){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		//if(ctrlPolyPoint[absoluteIdx + i].p != checkPatch || ctrlPolyPoint[absoluteIdx + i].i != checkIdx){
		if(ctrlPolyPoint[absoluteIdx + i].p != checkPatch){
			otherPatch = ctrlPolyPoint[absoluteIdx + i].p;
			otherIdx = ctrlPolyPoint[absoluteIdx + i].i;
			return true;
		}
	}
	return false;
}

bool getOtherPatchEdgeContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* checkPatch, int checkIdx, NurbsPatch*& otherPatch, int& otherIdx){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p != checkPatch || ctrlPolyPoint[absoluteIdx + i].i != checkIdx){
		//if(ctrlPolyPoint[absoluteIdx + i].p != checkPatch){
			otherPatch = ctrlPolyPoint[absoluteIdx + i].p;
			otherIdx = ctrlPolyPoint[absoluteIdx + i].i;
			return true;
		}
	}
	return false;
}

bool patchIsContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, const NurbsPatch* checkPatch){
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch){
			return true;
		}
	}
	return false;
}

bool Seamless::getPatchJoinedByStitchPoly(const NurbsPatch* checkPatch, int checkIdx, NurbsPatch*& otherPatch, int& otherIdx)const{
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int beginPoly = poly;
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		int midBegin = poly;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen){
				if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
					if(getOtherPatchContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, otherPatch, otherIdx)){
						return true;
					}
				}
			}
			else{
				midLen++;
			}
			poly += midLen;
		}
		int centerBegin = poly;
		if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
			if(getOtherPatchContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, otherPatch, otherIdx)){
				return true;
			}
		}

		int centerLen = ctrlPolyPoint[poly].len;

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
	return false;
}

bool Seamless::getPatchEdgeJoinedByStitchPoly(const NurbsPatch* checkPatch, int checkIdx, NurbsPatch*& otherPatch, int& otherIdx)const{//identical to getPatchJoinedByStitchPoly except function calls getOtherPatchEdgeContainedInMidPoint
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int beginPoly = poly;
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		int midBegin = poly;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen){
				if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
					if(getOtherPatchEdgeContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx, otherPatch, otherIdx)){
						return true;
					}
				}
			}
			else{
				midLen++;
			}
			poly += midLen;
		}
		int centerBegin = poly;
		//if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
		//	if(getOtherPatchEdgeContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx, otherPatch, otherIdx)){
		//		return true;
		//	}
		//}

		int centerLen = ctrlPolyPoint[poly].len;

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
	return false;
}

bool Seamless::isPatchJoinedByStitchPoly(const NurbsPatch* checkPatch, const int checkIdx, const NurbsPatch* otherPatch){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int beginPoly = poly;
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		int midBegin = poly;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen){
				if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
					if(patchIsContainedInMidPoint(ctrlPolyPoint, poly, otherPatch)){
						return true;
					}
				}
			}
			else{
				midLen++;
			}
			poly += midLen;
		}
		int centerBegin = poly;
		if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
			if(patchIsContainedInMidPoint(ctrlPolyPoint, poly, otherPatch)){
				return true;
			}
		}

		int centerLen = ctrlPolyPoint[poly].len;

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
	return false;
}

bool Seamless::isPatchJoinedByStitchPolyCorner(const NurbsPatch* checkPatch, const int checkIdx, const NurbsPatch* otherPatch){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int beginPoly = poly;
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		int midBegin = poly;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen){
			}
			else{
				midLen++;
			}
			poly += midLen;
		}
		int centerBegin = poly;
		if(isContainedInMidPoint(ctrlPolyPoint, poly, checkPatch, checkIdx)){
			if(patchIsContainedInMidPoint(ctrlPolyPoint, poly, otherPatch)){
				return true;
			}
		}

		int centerLen = ctrlPolyPoint[poly].len;

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
	return false;
}

int getRowEdgeIndex(const int uDim, const int vDim, const M_Vec2f& uvCoord, const int crdIdx, float& y){
	y = uvCoord[crdIdx].y;
	int row = (vDim - 1) >> 1;
	y = y * row;
	int i = y;
	if(i >= row){
		i--;
	}
	int j = i * 2 + 1;

	j = j * uDim;
	y -= i;
	return j;
}


int getColEdgeIndex(const int uDim, const int vDim, const M_Vec2f& uvCoord, const int crdIdx, float& x){
	x = uvCoord[crdIdx].x;
	int col = (uDim - 1) >> 1;
	x = x * col;
	int i = x;
	if(i >= col){
		i--;
	}
	int j = i * 2 + 1;

	x -= i;
	return j;
}

void modifyOtherRowEdge(const NurbsPatch* patch, float& y, int otherIdx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int row = (vDim - 1) >> 1; //number of bezier patches
	float f = 1.0 / row;

	int i = otherIdx / uDim;
	i = i >> 1;

	y = y / row;

	y += f * i;
}

void modifyOtherColEdge(const NurbsPatch* patch, float& x, int otherIdx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int col = (uDim - 1) >> 1;
	float f = 1.0 / col;

	int i = otherIdx % uDim;
	i = i >> 1;

	x = x / col;

	x += f * i;
}

bool Seamless::transferableVertexOverBorderWest(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);

	if(y < .0001 && isPatchJoinedByStitchPolyCorner(patch, pointIdx - uDim, delPatch)){//SW
		return true;
	}
	if(y > .9999 && isPatchJoinedByStitchPolyCorner(patch, pointIdx + uDim, delPatch)){//NW
		return true;
	}

	if(isPatchJoinedByStitchPoly(patch, pointIdx, delPatch)){
		return true;
	}
	return false;
}

bool Seamless::getTransferedCoordOverPatchBorderWest(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& otherPatch)const{
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int otherIdx = 0;
	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			y = 1.0 - y;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			y = 1.0 - y;
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(0, y);
			return true;
		}
		else if(u == uDim - 1){//East edge
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(1, y);
			return true;
		}
		else if(v == 0){//south edge
			modifyOtherColEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(y, 0);
			return true;
		}
		else if(v == vDim - 1){//north edge
			y = 1.0 - y;
			modifyOtherColEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(y, 1);
			return true;
		}
	}
	return false;
}

bool Seamless::transferVertexOverBorderWest(NurbsPatch* patch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			y = 1.0 - y;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			y = 1.0 - y;
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(0, y);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(u == uDim - 1){//East edge
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(1, y);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == 0){//south edge
			modifyOtherColEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(y, 0);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == vDim - 1){//north edge
			y = 1.0 - y;
			modifyOtherColEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(y, 1);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
	}
	return false;
}

bool Seamless::joinBorderVerticesWest(const NurbsPatch* patch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);

	if(getPatchEdgeJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			y = 1.0 - y;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			y = 1.0 - y;
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(0, y), dstPart, dstIdx)){
				return true;
			}
		}
		else if(u == uDim - 1){//East edge
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(1, y), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == 0){//south edge
			modifyOtherColEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(y, 0), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == vDim - 1){//north edge
			y = 1.0 - y;
			modifyOtherColEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(y, 1), dstPart, dstIdx)){
				return true;
			}
		}
	}
	return false;
}

bool Seamless::transferableVertexOverBorderEast(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);
	pointIdx += uDim - 1;

	if(y < .0001 && isPatchJoinedByStitchPolyCorner(patch, pointIdx - uDim, delPatch)){//SE
		return true;
	}
	if(y > .9999 && isPatchJoinedByStitchPolyCorner(patch, pointIdx + uDim, delPatch)){//NE
		return true;
	}

	if(isPatchJoinedByStitchPoly(patch, pointIdx, delPatch)){
		return true;
	}
	return false;
}

bool Seamless::getTransferedCoordOverPatchBorderEast(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& otherPatch)const{
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int otherIdx = 0;
	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);
	pointIdx += uDim - 1;

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			y = 1.0 - y;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(0, y);
			return true;
		}
		else if(u == uDim - 1){//East edge
			y = 1.0 - y;
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(1, y);
			return true;
		}
		else if(v == 0){//south edge
			y = 1.0 - y;
			modifyOtherColEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(y, 0);
			return true;
		}
		else if(v == vDim - 1){//north edge
			modifyOtherColEdge(otherPatch, y, otherIdx);
			returnCoord = Vec2f(y, 1);
			return true;
		}
	}
	return false;
}

bool Seamless::transferVertexOverBorderEast(NurbsPatch* patch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);
	pointIdx += uDim - 1;

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			y = 1.0 - y;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(0, y);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(u == uDim - 1){//East edge
			y = 1.0 - y;
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(1, y);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == 0){//south edge
			y = 1.0 - y;
			modifyOtherColEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(y, 0);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == vDim - 1){//north edge
			modifyOtherColEdge(otherPatch, y, otherIdx);
			uvCoord[crdIdx] = Vec2f(y, 1);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
	}
	return false;
}

bool Seamless::joinBorderVerticesEast(const NurbsPatch* patch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float y = 0;

	int pointIdx = getRowEdgeIndex(uDim, vDim, uvCoord, crdIdx, y);
	pointIdx += uDim - 1;

	if(getPatchEdgeJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			y = 1.0 - y;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(0, y), dstPart, dstIdx)){
				return true;
			}
		}
		else if(u == uDim - 1){//East edge
			y = 1.0 - y;
			modifyOtherRowEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(1, y), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == 0){//south edge
			y = 1.0 - y;
			modifyOtherColEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(y, 0), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == vDim - 1){//north edge
			modifyOtherColEdge(otherPatch, y, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(y, 1), dstPart, dstIdx)){
				return true;
			}
		}
	}
	return false;
}

bool Seamless::transferableVertexOverBorderSouth(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	if(x < .0001 && isPatchJoinedByStitchPolyCorner(patch, pointIdx - 1, delPatch)){//SW
		return true;
	}
	if(x > .9999 && isPatchJoinedByStitchPolyCorner(patch, pointIdx + 1, delPatch)){//SE
		return true;
	}

	if(isPatchJoinedByStitchPoly(patch, pointIdx, delPatch)){
		return true;
	}
	return false;
}

bool Seamless::getTransferedCoordOverPatchBorderSouth(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& otherPatch)const{
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int otherIdx = 0;
	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			x = 1.0 - x;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(0, x);
			return true;
		}
		else if(u == uDim - 1){//East edge
			x = 1.0 - x;
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(1, x);
			return true;
		}
		else if(v == 0){//south edge
			x = 1.0 - x;
			modifyOtherColEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(x, 0);
			return true;
		}
		else if(v == vDim - 1){//north edge
			modifyOtherColEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(x, 1);
			return true;
		}
	}
	return false;
}


bool Seamless::transferVertexOverBorderSouth(NurbsPatch* patch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			x = 1.0 - x;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(0, x);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(u == uDim - 1){//East edge
			x = 1.0 - x;
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(1, x);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == 0){//south edge
			x = 1.0 - x;
			modifyOtherColEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(x, 0);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == vDim - 1){//north edge
			modifyOtherColEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(x, 1);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
	}
	return false;
}

bool Seamless::joinBorderVerticesSouth(const NurbsPatch* patch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	if(getPatchEdgeJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			x = 1.0 - x;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(0, x), dstPart, dstIdx)){
				return true;
			}
		}
		else if(u == uDim - 1){//East edge
			x = 1.0 - x;
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(1, x), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == 0){//south edge
			x = 1.0 - x;
			modifyOtherColEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(x, 0), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == vDim - 1){//north edge
			modifyOtherColEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(x, 1), dstPart, dstIdx)){
				return true;
			}
		}
	}
	return false;
}

bool Seamless::transferableVertexOverBorderNorth(NurbsPatch* patch, int crdIdx, M_Vec2f& uvCoord, NurbsPatch* delPatch){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	pointIdx += uDim * (vDim - 1);

	if(x < .0001 && isPatchJoinedByStitchPolyCorner(patch, pointIdx - 1, delPatch)){//NW
		return true;
	}
	if(x > .9999 && isPatchJoinedByStitchPolyCorner(patch, pointIdx + 1, delPatch)){//NE
		return true;
	}

	if(isPatchJoinedByStitchPoly(patch, pointIdx, delPatch)){
		return true;
	}
	return false;
}
bool Seamless::getTransferedCoordOverPatchBorderNorth(const NurbsPatch* patch, int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Vec2f& returnCoord, NurbsPatch*& otherPatch)const{
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	int otherIdx = 0;
	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	pointIdx += uDim * (vDim - 1);

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			x = 1.0 - x;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			x = 1.0 - x;
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(0, x);
			return true;
		}
		else if(u == uDim - 1){//East edge
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(1, x);
			return true;
		}
		else if(v == 0){//south edge
			modifyOtherColEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(x, 0);
			return true;
		}
		else if(v == vDim - 1){//north edge
			x = 1 - x;
			modifyOtherColEdge(otherPatch, x, otherIdx);
			returnCoord = Vec2f(x, 1);
			return true;
		}
	}
	return false;
}


bool Seamless::transferVertexOverBorderNorth(NurbsPatch* patch, int crdIdx, M_NurbsPatch& partPatchOwner, M_Vec2f& uvCoord){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	pointIdx += uDim * (vDim - 1);

	if(getPatchJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			x = 1.0 - x;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;
		//which edge?
		if(u == 0){//West edge
			x = 1.0 - x;
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(0, x);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(u == uDim - 1){//East edge
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(1, x);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == 0){//south edge
			modifyOtherColEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(x, 0);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
		else if(v == vDim - 1){//north edge
			x = 1 - x;
			modifyOtherColEdge(otherPatch, x, otherIdx);
			uvCoord[crdIdx] = Vec2f(x, 1);
			partPatchOwner[crdIdx] = otherPatch;
			return true;
		}
	}
	return false;
}

bool Seamless::joinBorderVerticesNorth(const NurbsPatch* patch, const int crdIdx, const M_NurbsPatch& partPatchOwner, const M_Vec2f& uvCoord, Art* dstPart, int dstIdx){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;

	NurbsPatch* otherPatch = 0;
	int otherIdx = 0;
	float x = 0;

	int pointIdx = getColEdgeIndex(uDim, vDim, uvCoord, crdIdx, x);

	pointIdx += uDim * (vDim - 1);

	if(getPatchEdgeJoinedByStitchPoly(patch, pointIdx, otherPatch, otherIdx)){
		if(patch->isInsideOut() != otherPatch->isInsideOut()){
			x = 1.0 - x;
		}
		int uDim = otherPatch->dim.x;
		int vDim = otherPatch->dim.y;
		int u = otherIdx % uDim;
		int v = otherIdx / uDim;

		//which edge?
		if(u == 0){//West edge
			x = 1.0 - x;
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(0, x), dstPart, dstIdx)){
				return true;
			}
		}
		else if(u == uDim - 1){//East edge
			modifyOtherRowEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(1, x), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == 0){//south edge
			modifyOtherColEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(x, 0), dstPart, dstIdx)){
				return true;
			}
		}
		else if(v == vDim - 1){//north edge
			x = 1 - x;
			modifyOtherColEdge(otherPatch, x, otherIdx);
			if(isCloseCoord(otherPatch, Vec2f(x, 1), dstPart, dstIdx)){
				return true;
			}
		}
	}
	return false;
}

void Seamless::vEastFindStitchPolyAndSplit(M_Int& deleteCtrlPolyList, NurbsPatch* checkPatch, const int checkIdx, const int checkIdx2, const NurbsPatch* firstPatch, bool& foundFirstPatch, const float ratio){
	int poly = 0;
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		for(int k = len; --k >= 0;){
			if(ctrlPolyPoint[poly + k].p == checkPatch && ctrlPolyPoint[poly + k].i == checkIdx){
				int chk2 = getCorner(k, 3, len);
				if(ctrlPolyPoint[poly + chk2].p == checkPatch && ctrlPolyPoint[poly + chk2].i == checkIdx2){
					deleteCtrlPolyList += poly;

					int uDim = checkPatch->dim.x;
					k = getCorner(k, 1, len);
					NurbsPatch* patchB = ctrlPolyPoint[poly + k].p;
					int idxB = ctrlPolyPoint[poly + k].i;

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx, len);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB + uDim * 2, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim * 2, 0);

					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint(patchB, idxB + uDim, 1);
					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint();

					ctrlPolyPoint += CtrlPolyPoint();
					////////////
					
					k = getCorner(k, 1, len);
					NurbsPatch* patch2B = ctrlPolyPoint[poly + k].p;
					int idx2B = ctrlPolyPoint[poly + k].i;

					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim * 2, len);
					ctrlPolyPoint += CtrlPolyPoint(patch2B, idx2B - uDim * 2, 0);
					ctrlPolyPoint += CtrlPolyPoint(patch2B, idx2B, 0);
					ctrlPolyPoint += CtrlPolyPoint(checkPatch, checkIdx + uDim * 4, 0);

					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint(patch2B, idx2B - uDim, 1);
					ctrlPolyPoint += CtrlPolyPoint();
					ctrlPolyPoint += CtrlPolyPoint();

					ctrlPolyPoint += CtrlPolyPoint();
					return;
				}
			}
		}
		poly += len;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}
		
		int centerLen = ctrlPolyPoint[poly].len;


		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}


bool containsSouthEdgeMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int midIndex, NurbsPatch* checkPatch, int len){
	int uDim = checkPatch->dim.x;
	for(int k = 0; k < len; k++){
		int midLen = ctrlPolyPoint[midIndex].len;
		if(midLen == 0){
			midIndex++;
		}
		else{
			for(int i = midLen; --i >= 0;){
				if(ctrlPolyPoint[midIndex].p == checkPatch && ctrlPolyPoint[midIndex].i / uDim == 0){
					return true;
				}
				midIndex++;
			}
		}
	}
	return false;
}

bool containsWestEdgeMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int midIndex, NurbsPatch* checkPatch, int len){
	int uDim = checkPatch->dim.x;
	for(int k = 0; k < len; k++){
		int midLen = ctrlPolyPoint[midIndex].len;
		if(midLen == 0){
			midIndex++;
		}
		else{
			for(int i = midLen; --i >= 0;){
				if(ctrlPolyPoint[midIndex].p == checkPatch && ctrlPolyPoint[midIndex].i % uDim == 0){
					return true;
				}
				midIndex++;
			}
		}
	}
	return false;
}

bool containsSouthCorner(const M_CtrlPolyPoint& ctrlPolyPoint, int midIndex, NurbsPatch* checkPatch, int len){
	int uDim = checkPatch->dim.x;
	for(int k = 0; k < len; k++){
		int midLen = ctrlPolyPoint[midIndex].len;
		if(midLen == 0){
			midIndex++;
		}
		else{
			for(int i = midLen; --i >= 0;){
				if(ctrlPolyPoint[midIndex].p == checkPatch && (ctrlPolyPoint[midIndex].i == 0 || ctrlPolyPoint[midIndex].i == uDim - 1)){
					return true;
				}
				midIndex++;
			}
		}
	}
	return false;
}


bool isSouthEdgeMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int absoluteIdx, NurbsPatch* checkPatch){
	int uDim = checkPatch->dim.x;
	int len = ctrlPolyPoint[absoluteIdx].len;
	for(int i = len; --i >= 0;){
		if(ctrlPolyPoint[absoluteIdx + i].p == checkPatch && ctrlPolyPoint[absoluteIdx + i].i / uDim == 0){
			return true;
		}
	}
	return false;
}



void Seamless::uFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch){
	int poly = 0;
	int uDim = checkPatch->dim.x;

	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		int midIndex = poly + len;
		for(int k = len; --k >= 0;){
			int idxA = poly + k;
			if(ctrlPolyPoint[idxA].p == checkPatch &&  ctrlPolyPoint[idxA].i % uDim == 3){
				if(containsWestEdgeMidPoint(ctrlPolyPoint, midIndex, checkPatch, len)){
//					if(false == containsSouthCorner(ctrlPolyPoint, midIndex, checkPatch, len + 1)){
					ctrlPolyPoint[idxA].i -= 2;
					//}
				}
			}
		}
		poly += len;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			else{
				for(int k = midLen; --k >= 0;){
					int idxA = poly + k;
					if(ctrlPolyPoint[idxA].p == checkPatch &&  ctrlPolyPoint[idxA].i % uDim == 3){
						if(containsWestEdgeMidPoint(ctrlPolyPoint, midIndex, checkPatch, len)){
							ctrlPolyPoint[idxA].i -= 2;
						}
					}
				}
			}
			poly += midLen;
		}
		int centerLen = ctrlPolyPoint[poly].len;

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}

void Seamless::vFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch){
	int poly = 0;
	int uDim = checkPatch->dim.x;

	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		int midIndex = poly + len;
		for(int k = len; --k >= 0;){
			int idxA = poly + k;
			if(ctrlPolyPoint[idxA].p == checkPatch &&  ctrlPolyPoint[idxA].i / uDim == 3){
				if(containsSouthEdgeMidPoint(ctrlPolyPoint, midIndex, checkPatch, len)){
//					if(false == containsSouthCorner(ctrlPolyPoint, midIndex, checkPatch, len + 1)){
					ctrlPolyPoint[idxA].i -= uDim * 2;
					//}
				}
			}
		}
		poly += len;
		for(int k = len; --k >= 0;){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			else{
				for(int k = midLen; --k >= 0;){
					int idxA = poly + k;
					if(ctrlPolyPoint[idxA].p == checkPatch &&  ctrlPolyPoint[idxA].i / uDim == 3){
						if(containsSouthEdgeMidPoint(ctrlPolyPoint, midIndex, checkPatch, len)){
							ctrlPolyPoint[idxA].i -= uDim * 2;
						}
					}
				}
			}
			poly += midLen;
		}
		int centerLen = ctrlPolyPoint[poly].len;

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
}

bool uContainsEdge(const M_CtrlPolyPoint& ctrlPolyPoint, int poly, NurbsPatch* checkPatch, int uIdx){
	int uDim = checkPatch->dim.x;
	int len = ctrlPolyPoint[poly].len;
	for(int k = len; --k >= 0;){
		if(ctrlPolyPoint[poly].p == checkPatch && ctrlPolyPoint[poly].i % uDim == uIdx){
			return true;
		}
		poly++;
	}
	return false;
}

bool vContainsEdge(const M_CtrlPolyPoint& ctrlPolyPoint, int poly, NurbsPatch* checkPatch, int vIdx){
	int uDim = checkPatch->dim.x;
	int len = ctrlPolyPoint[poly].len;
	for(int k = len; --k >= 0;){
		if(ctrlPolyPoint[poly].p == checkPatch && ctrlPolyPoint[poly].i / uDim == vIdx){
			return true;
		}
		poly++;
	}
	return false;
}

bool containsEdgeMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, int poly, NurbsPatch* checkPatch, int vIdx){
	int uDim = checkPatch->dim.x;
	int len = ctrlPolyPoint[poly].len;
	int midIndex = poly + len;
	for(int k = 0; k < len; k++){
		int midLen = ctrlPolyPoint[midIndex].len;
		if(midLen == 0){
			midIndex++;
		}
		else{
			for(int i = midLen; --i >= 0;){
				if(ctrlPolyPoint[midIndex].p == checkPatch && ctrlPolyPoint[midIndex].i / uDim == vIdx){
					return true;
				}
				midIndex++;
			}
		}
	}
	return false;
}

void Seamless::vFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch, int vIdx){
	int poly = 0;
	int uDim = checkPatch->dim.x;
	int polyLen = ctrlPolyPoint.length;
	while(poly < polyLen){
		int len = ctrlPolyPoint[poly].len;
		int midIndex = poly + len;
		bool flag = false;
		for(int k = len; --k >= 0;){
			int idx = poly + k;
			if(ctrlPolyPoint[idx].p == checkPatch &&  ctrlPolyPoint[idx].i / uDim == vIdx){
				if(vContainsEdge(ctrlPolyPoint, poly, checkPatch, vIdx - 4)){
					ctrlPolyPoint[idx].i -= uDim * 2;
					flag = true;
				}
			}
		}
		for(int k = len + 1; --k >= 0;){
			int midLen = ctrlPolyPoint[midIndex].len;
			if(midLen == 0){
				midLen++;
			}
			else{
				for(int k = midLen; --k >= 0;){
					int idx = midIndex + k;
					if(flag && ctrlPolyPoint[idx].p == checkPatch &&  ctrlPolyPoint[idx].i / uDim == vIdx - 1){
						ctrlPolyPoint[idx].i -= uDim * 2;
					}
				}
			}
			midIndex += midLen;
		}
		if(flag){
			vCopyStitchPolyAndSlide(poly, 2);
		}

		poly = midIndex;
	}
}

int Seamless::uSlideEdgeStitchPoly(int poly, NurbsPatch* checkPatch, int uIdx){
	int uDim = checkPatch->dim.x;
	int len = ctrlPolyPoint[poly].len;
	int midIndex = poly + len;
	bool flag = false;
	for(int k = len; --k >= 0;){
		int idx = poly + k;
		if(ctrlPolyPoint[idx].p == checkPatch &&  ctrlPolyPoint[idx].i % uDim == uIdx){
			if(uContainsEdge(ctrlPolyPoint, poly, checkPatch, uIdx - 4)){
				ctrlPolyPoint[idx].i -= 2;
				flag = true;
			}
		}
	}
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[midIndex].len;
		if(midLen == 0){
			midLen++;
		}
		else if(flag){
			for(int k = midLen; --k >= 0;){
				int idx = midIndex + k;
				if(ctrlPolyPoint[idx].p == checkPatch){
					if((ctrlPolyPoint[idx].i % uDim == uIdx) || (ctrlPolyPoint[idx].i % uDim == uIdx - 1)){
						ctrlPolyPoint[idx].i -= 2;
					}
				}
			}
		}
		midIndex += midLen;
	}
	return midIndex;
}

int Seamless::vSlideEdgeStitchPoly(int poly, NurbsPatch* checkPatch, int vIdx){
	int uDim = checkPatch->dim.x;
	int len = ctrlPolyPoint[poly].len;
	int midIndex = poly + len;
	bool flag = false;
	for(int k = len; --k >= 0;){
		int idx = poly + k;
		if(ctrlPolyPoint[idx].p == checkPatch &&  ctrlPolyPoint[idx].i / uDim == vIdx){
			if(vContainsEdge(ctrlPolyPoint, poly, checkPatch, vIdx - 4)){
				ctrlPolyPoint[idx].i -= uDim * 2;
				flag = true;
			}
		}
	}
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[midIndex].len;
		if(midLen == 0){
			midLen++;
		}
		else if(flag){
			for(int k = midLen; --k >= 0;){
				int idx = midIndex + k;
				if(ctrlPolyPoint[idx].p == checkPatch){
					if((ctrlPolyPoint[idx].i % uDim == vIdx) || (ctrlPolyPoint[idx].i / uDim == vIdx - 1)){
						ctrlPolyPoint[idx].i -= uDim * 2;
					}
				}
			}
		}
		midIndex += midLen;
	}
	return midIndex;
}

void Seamless::uFindStitchPolyAndSlideInnerEdge(NurbsPatch* checkPatch, int uIdx){ //this did not work laste time we used it
	int poly = 0;
	int uDim = checkPatch->dim.x;
	int polyLen = ctrlPolyPoint.length;
	while(poly < polyLen){
		poly += uSlideEdgeStitchPoly(poly, checkPatch, uIdx);
	}
}

bool NurbsPatch::isCellCorner(int idx){//for box modelling
	int uDim = this->dim.x;
	int vDim = this->dim.y;

	int u = idx % uDim;
	int v = idx / uDim;

	if(u == 0 || u == uDim - 1){
		if(v == 0 || v == vDim - 1){
			return true;
		}
		else if(v % 2 == 0){
			return true;
		}
	}
	if(v == 0 || v == vDim - 1){
		if(u == 0 || u == vDim - 1){
			return true;
		}
		else if(u % 2 == 0){
			return true;
		}
	}
	return false;
}


bool NurbsPatch::isPatchCorner(int idx){//for box modelling
	int uDim = this->dim.x;
	int vDim = this->dim.y;


	if(idx == 0){
		return true;
	}
	if(idx == uDim - 1){
		return true;
	}

	int len = uDim * vDim;

	if(idx == len - 1){
		return true;
	}
	if(idx == len - uDim){
		return true;
	}
	return false;
}

bool findCloseCorner2(M_Int& ctrlPolySelList, M_Int& tempBlockList, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& midCtrlPolyPoint, M_CtrlPolyPoint& centerCtrlPolyPoint, NurbsPatch* patchA, const int idxA, int cpiA, const int cornerIdxA3, int cornerCpiA, int polyA, int lPolyIdxA, NurbsPatch* returnPatch, const int returnIdx){
	int cornerIdxB0 = 0;
	int cornerIdxB1 = 0;
	int cornerIdxB2 = 0;
	int cornerIdxB3 = 0;

	for(int lPolyIdxB = 0; lPolyIdxB < ctrlPolySelList.length; lPolyIdxB++){
		int polyB = ctrlPolySelList[lPolyIdxB];
		if(polyB < 0){
			continue;
		}
		int absoluteIdx = polyB;
		int len = ctrlPolyPoint[absoluteIdx].len;
		for(int k = 0; k < len; k++){
			int idxB = ctrlPolyPoint[absoluteIdx].i;
			NurbsPatch* patchB = ctrlPolyPoint[absoluteIdx].p;

			if(patchB->isCellCorner(idxB)){
				int cpiB = patchB->cpIndex[idxB];
				if(cpiA == cpiB){

					int k0 = getCorner(k, 1, len);
					int k1 = getCorner(k0, 1, len);
					cornerIdxB1 = ctrlPolyPoint[polyB + k1].i;
					int k2 = getCorner(k1, 1, len);
					cornerIdxB2 = ctrlPolyPoint[polyB + k2].i;
					cornerIdxB3 = ctrlPolyPoint[polyB + k].i;
					cornerIdxB0 = ctrlPolyPoint[polyB + k0].i;

					int cornerCpiB = patchB->cpIndex[cornerIdxB0];
					int returnCpiB2 = patchB->cpIndex[cornerIdxB2];
					int returnCpi = returnPatch->cpIndex[returnIdx];

					if(cornerCpiA == cornerCpiB){

						tempBlockList += lPolyIdxB;
						ctrlPolySelList[lPolyIdxB] = -polyB - 1;

						midCtrlPolyPoint += CtrlPolyPoint(patchA, cornerIdxA3, 2);
						midCtrlPolyPoint += CtrlPolyPoint(patchB, cornerIdxB0, 0);
						ctrlPolyPoint += CtrlPolyPoint(patchB,  cornerIdxB1, 0);
						centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);


						if(returnCpi == returnCpiB2){
										
							midCtrlPolyPoint += CtrlPolyPoint(patchB, cornerIdxB2, 2);
							midCtrlPolyPoint += CtrlPolyPoint(returnPatch, returnIdx, 0);
							return true;
						}
						return false;

					}
					else if(returnCpi == returnCpiB2){
										
						tempBlockList += lPolyIdxB;
						ctrlPolySelList[lPolyIdxB] = -polyB - 1;

						midCtrlPolyPoint.length = 0;
						midCtrlPolyPoint += CtrlPolyPoint();
						ctrlPolyPoint += CtrlPolyPoint(patchA, cornerIdxA3, 0);
						midCtrlPolyPoint += CtrlPolyPoint(patchA, idxA, 2);
						midCtrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);
						ctrlPolyPoint += CtrlPolyPoint(patchB, cornerIdxB0, 0);
						midCtrlPolyPoint += CtrlPolyPoint();
						ctrlPolyPoint += CtrlPolyPoint(patchB, cornerIdxB1, 0);
						midCtrlPolyPoint += CtrlPolyPoint(patchA, returnIdx, 2);
						midCtrlPolyPoint += CtrlPolyPoint(patchB, cornerIdxB2, 0);
						return true;
					}
				}
			}
			absoluteIdx++;
		}
	}
	return false;
}

bool findTJoin(M_Int& ctrlPolySelList, M_Int& tempBlockList, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& midCtrlPolyPoint, M_CtrlPolyPoint& centerCtrlPolyPoint, NurbsPatch* patchA, const int idxA, const int visIdxA, int lPolyIdxA, NurbsPatch* returnPatch, const int returnIdx);
//bool findTJoinForEdge(M_Int& ctrlPolySelList, M_Int& tempBlockList, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& midCtrlPolyPoint, M_CtrlPolyPoint& centerCtrlPolyPoint, NurbsPatch* patchA, const int idxA, const int visIdxA, int lPolyIdxA, NurbsPatch* returnPatch, const int returnIdx);

void addToList(M_CtrlPolyPoint& ctrlPolyPoint, NurbsPatch* p, int i){
	for(int k = ctrlPolyPoint.length; --k>=0;){
		if(ctrlPolyPoint[k].p == p && ctrlPolyPoint[k].i == i){
			return;
		}
	}
	ctrlPolyPoint += CtrlPolyPoint(p, i, 0);
}

void addToMidList(M_CtrlPolyPoint& ctrlPolyPoint, NurbsPatch* pA, int iA , NurbsPatch* pB, int iB){
	if(pA == pB && iA == iB){
		ctrlPolyPoint += CtrlPolyPoint(pA, iA, 1);
	}
	else{
		ctrlPolyPoint += CtrlPolyPoint(pA, iA, 2);
		ctrlPolyPoint += CtrlPolyPoint(pB, iB, 0);
	}
}

bool findCloseCorner(M_Int& ctrlPolySelList, M_Int& tempBlockList, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& midCtrlPolyPoint, M_CtrlPolyPoint& centerCtrlPolyPoint, NurbsPatch* patchA, const int idxA, const int idxA3, int lPolyIdxA, NurbsPatch* returnPatch, const int returnIdx){
	int cpiA = patchA->cpIndex[idxA];
	const int cornerCpiA = patchA->cpIndex[idxA3];

	for(int lPolyIdxB = 0; lPolyIdxB < ctrlPolySelList.length; lPolyIdxB++){
		int polyB = ctrlPolySelList[lPolyIdxB];
		if(polyB < 0){
			continue;
		}
		int absoluteIdx = polyB;
		int len = ctrlPolyPoint[absoluteIdx].len;
		for(int k = 0; k < len; k++){
			int idxB = ctrlPolyPoint[absoluteIdx].i;
			NurbsPatch* patchB = ctrlPolyPoint[absoluteIdx].p;
			if(patchB->isCellCorner(idxB)){
				int cpiB = patchB->cpIndex[idxB];
				if(cpiA == cpiB){
					int k0 = getCorner(k, 1, len);
					int idxB0 = ctrlPolyPoint[polyB + k0].i;
					int k1 = getCorner(k0, 1, len);
					int idxB1 = ctrlPolyPoint[polyB + k1].i;
					int k2 = getCorner(k1, 1, len);
					int idxB2 = ctrlPolyPoint[polyB + k2].i;
					int idxB3 = ctrlPolyPoint[polyB + k].i;

					int cornerCpiB = patchB->cpIndex[idxB0];
					//Vec3f cornerPointB = patchB->dpoint[idxB1];
					if(cornerCpiA == cornerCpiB){


						tempBlockList += lPolyIdxB;
						ctrlPolySelList[lPolyIdxB] = -polyB - 1;

						int returnCpiB2 = patchB->cpIndex[idxB2];
						int cornerCpiB1 = patchB->cpIndex[idxB1];


						if(cpiB == returnCpiB2){

							//centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);
							addToList(centerCtrlPolyPoint, patchB, idxB);

							if(returnPatch == patchB && returnIdx == idxB1){
								//centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB2, 0);
								addToList(centerCtrlPolyPoint, patchB, idxB2);

								midCtrlPolyPoint += CtrlPolyPoint();
								return true;
							}
							
							midCtrlPolyPoint += CtrlPolyPoint();
							ctrlPolyPoint += CtrlPolyPoint(patchB,  idxB1, 0);

							return findTJoin(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchB, idxB, idxB1, lPolyIdxA, returnPatch, returnIdx);

						}
						//else if(patchA == patchB && idxA3 == idxB0){
						//	addToList(centerCtrlPolyPoint, patchB, idxB);

						//	
						//	ctrlPolyPoint += CtrlPolyPoint(patchB,  idxB1, 0);
						//	return findTJoinForEdge(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchB, idxB, idxB2, lPolyIdxA, returnPatch, returnIdx);

						//}
						else{
							addToMidList(midCtrlPolyPoint, patchA, idxA3, patchB, idxB0);
							ctrlPolyPoint += CtrlPolyPoint(patchB,  idxB1, 0);
//							centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB, 0);
							addToList(centerCtrlPolyPoint, patchB, idxB);

							int returnCpi = returnPatch->cpIndex[returnIdx];

							if(returnCpi == returnCpiB2){
										
								addToMidList(midCtrlPolyPoint, patchB, idxB2, returnPatch, returnIdx);
								return true;
							}
				
							return findCloseCorner(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchB, idxB, idxB2, lPolyIdxB, returnPatch, returnIdx);
						}
					}
				}
			}
			absoluteIdx++;
		}
	}
	return false;
}
/*
bool findTJoin(M_Int& ctrlPolySelList, M_Int& tempBlockList, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& midCtrlPolyPoint, M_CtrlPolyPoint& centerCtrlPolyPoint, NurbsPatch* patchA, const int idxA, const int visIdxA, int lPolyIdxA, NurbsPatch* returnPatch, const int returnIdx){

	for(int lPolyIdxB = 0; lPolyIdxB < ctrlPolySelList.length; lPolyIdxB++){
		int polyB = ctrlPolySelList[lPolyIdxB];
		if(polyB < 0){
			continue;
		}
		int absoluteIdx = polyB;
		int len = ctrlPolyPoint[absoluteIdx].len;
		for(int k = 0; k < len; k++){
			int visIdxB = ctrlPolyPoint[absoluteIdx].i;
			NurbsPatch* patchB = ctrlPolyPoint[absoluteIdx].p;
			if(patchA == patchB && visIdxA == visIdxB){
				int k0 = getCorner(k, 1, len);
				int idxB0 = ctrlPolyPoint[polyB + k0].i;
				Vec3f cornerPointB = patchB->dpoint[idxB0];
				int k1 = getCorner(k0, 1, len);
				int idxB1 = ctrlPolyPoint[polyB + k1].i;
				int k2 = getCorner(k1, 1, len);
				int idxB2 = ctrlPolyPoint[polyB + k2].i;
				int idxB3 = ctrlPolyPoint[polyB + k].i;


				Vec3f pointB0 = patchB->dpoint[idxB0];
				Vec3f pointB2 = patchB->dpoint[idxB2];

				centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB1, 0);
				centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB2, 0);

				Vec3f returnPoint = returnPatch->dpoint[returnIdx];

				if(returnPatch == patchB){
					if(returnIdx == idxB0){
						tempBlockList += lPolyIdxB;
						ctrlPolySelList[lPolyIdxB] = -polyB;
						midCtrlPolyPoint += CtrlPolyPoint();
						return true;
					}
					else if(returnIdx == idxB3){
						midCtrlPolyPoint += CtrlPolyPoint();
						return true;
					}
				}

				tempBlockList += lPolyIdxB;
				ctrlPolySelList[lPolyIdxB] = -polyB;

				return findCloseCorner(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchB, idxB2, pointB2, idxB0, polyB, lPolyIdxB, returnPatch, returnIdx);
			}
			absoluteIdx++;
		}
	}
	return false;
}
*/

bool findTJoin(M_Int& ctrlPolySelList, M_Int& tempBlockList, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& midCtrlPolyPoint, M_CtrlPolyPoint& centerCtrlPolyPoint, NurbsPatch* patchA, const int idxA, const int visIdxA, int lPolyIdxA, NurbsPatch* returnPatch, const int returnIdx){
	Vec3f centerPointA = patchA->dpoint[idxA];

	for(int lPolyIdxB = 0; lPolyIdxB < ctrlPolySelList.length; lPolyIdxB++){
		int polyB = ctrlPolySelList[lPolyIdxB];
		if(polyB < 0){
			continue;
		}
		int absoluteIdx = polyB;
		int len = ctrlPolyPoint[absoluteIdx].len;
		if(patchA != ctrlPolyPoint[absoluteIdx].p){
			continue;
		}
		for(int k = 0; k < len; k++){
			NurbsPatch* patchB = ctrlPolyPoint[absoluteIdx].p;
			int centerIdxB = ctrlPolyPoint[absoluteIdx].i;
			Vec3f centerPointB = patchB->dpoint[centerIdxB];

			if(	abs(centerPointA.x - centerPointB.x) < .00001 &&
				abs(centerPointA.y - centerPointB.y) < .00001 &&
				abs(centerPointA.z - centerPointB.z) < .00001){

				int k0 = getCorner(k, 1, len);
				int k1 = getCorner(k0, 1, len);
				int idxB1 = ctrlPolyPoint[polyB + k1].i;
				int k2 = getCorner(k1, 1, len);
				int idxB2 = ctrlPolyPoint[polyB + k2].i;
				int idxB3 = ctrlPolyPoint[polyB + k].i;
				int idxB0 = ctrlPolyPoint[polyB + k0].i;

				if(visIdxA == idxB1){
					//centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB0, 0);
					//centerCtrlPolyPoint += CtrlPolyPoint(patchB, idxB3, 0);

					addToList(centerCtrlPolyPoint, patchB, idxB0);
					addToList(centerCtrlPolyPoint, patchB, idxB3);

					const Vec3f returnPoint = returnPatch->dpoint[returnIdx];
					const Vec3f cornerPointB = patchB->dpoint[idxB2];

					//if(returnPatch == patchB){
					//	if(	abs(returnPoint.x - cornerPointB.x) < .00001 &&
					//		abs(returnPoint.y - cornerPointB.y) < .00001 &&
					//		abs(returnPoint.z - cornerPointB.z) < .00001){

					//		tempBlockList += lPolyIdxB;
					//		ctrlPolySelList[lPolyIdxB] = -polyB - 1;
					//		midCtrlPolyPoint += CtrlPolyPoint();
					//		return true;
					//	}
					//	else if(returnIdx == idxB3){
					//		midCtrlPolyPoint += CtrlPolyPoint();
					//		return true;
					//	}
					//}
					if(	abs(returnPoint.x - cornerPointB.x) < .00001 &&
						abs(returnPoint.y - cornerPointB.y) < .00001 &&
						abs(returnPoint.z - cornerPointB.z) < .00001){

						tempBlockList += lPolyIdxB;
						ctrlPolySelList[lPolyIdxB] = -polyB - 1;
						midCtrlPolyPoint += CtrlPolyPoint();
						return true;
					}

					tempBlockList += lPolyIdxB;
					ctrlPolySelList[lPolyIdxB] = -polyB - 1;

					///Vec3f pointB0 = patchB->dpoint[idxB0];

					return findCloseCorner(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchB, idxB0, idxB2, lPolyIdxB, returnPatch, returnIdx);
				}
			}
			absoluteIdx++;
		}
	}
	return false;
}


/*
bool findTJoinForEdge(M_Int& ctrlPolySelList, M_Int& tempBlockList, M_CtrlPolyPoint& ctrlPolyPoint, M_CtrlPolyPoint& midCtrlPolyPoint, M_CtrlPolyPoint& centerCtrlPolyPoint, NurbsPatch* patchA, const int idxA, const int visIdxA, int lPolyIdxA, NurbsPatch* returnPatch, const int returnIdx){
	Vec3f centerPointA = patchA->dpoint[idxA];

	for(int lPolyIdxB = 0; lPolyIdxB < ctrlPolySelList.length; lPolyIdxB++){
		int polyB = ctrlPolySelList[lPolyIdxB];
		if(polyB < 0){
			continue;
		}
		int absoluteIdx = polyB;
		int len = ctrlPolyPoint[absoluteIdx].len;
		for(int k = 0; k < len; k++){
			NurbsPatch* patchB = ctrlPolyPoint[absoluteIdx].p;
			int centerIdxB = ctrlPolyPoint[absoluteIdx].i;
			Vec3f centerPointB = patchB->dpoint[centerIdxB];

			if(	abs(centerPointA.x - centerPointB.x) < .00001 &&
				abs(centerPointA.y - centerPointB.y) < .00001 &&
				abs(centerPointA.z - centerPointB.z) < .00001){

				int k0 = getCorner(k, 1, len);
				int k1 = getCorner(k0, 1, len);
				int idxB1 = ctrlPolyPoint[polyB + k1].i;
				int k2 = getCorner(k1, 1, len);
				int idxB2 = ctrlPolyPoint[polyB + k2].i;
				int idxB3 = ctrlPolyPoint[polyB + k].i;
				int idxB0 = ctrlPolyPoint[polyB + k0].i;

				Vec3f vec3fB0 = patchB->dpoint[idxB0];
				Vec3f vec3fVisA = patchA->dpoint[visIdxA];

				if(	abs(vec3fB0.x - vec3fVisA.x) < .00001 &&
					abs(vec3fB0.y - vec3fVisA.y) < .00001 &&
					abs(vec3fB0.z - vec3fVisA.z) < .00001){

					addToList(centerCtrlPolyPoint, patchB, idxB3);
					addToMidList(midCtrlPolyPoint, patchA, visIdxA, patchB, idxB0);
					ctrlPolyPoint += CtrlPolyPoint(patchB,  idxB1, 0);

					Vec3f returnPoint = returnPatch->dpoint[returnIdx];

					tempBlockList += lPolyIdxB;
					ctrlPolySelList[lPolyIdxB] = -polyB - 1;

					return findCloseCorner(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchB, idxB3, idxB2, lPolyIdxB, returnPatch, returnIdx);
				}
			}
			absoluteIdx++;
		}
	}
	return false;
}
*/

void unblockList(M_Int& tempBlockList, M_Int& ctrlPolySelList){
	for(int k = tempBlockList.length; --k >= 0;){
		int lPolyIdx = tempBlockList[k];
		ctrlPolySelList[lPolyIdx] = -ctrlPolySelList[lPolyIdx] - 1;
	}
	tempBlockList.length = 0;
}

void addBlockListToDeleteList(M_Int& tempBlockList, M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList){
	for(int k = tempBlockList.length; --k >= 0;){
		int lPolyIdx = tempBlockList[k];
		deleteCtrlPolySelList += -ctrlPolySelList[lPolyIdx] - 1;
	}
	tempBlockList.length = 0;
}

void findCornerToBeginWith(M_Int& ctrlPolySelList, M_Int& deleteCtrlPolySelList, M_CtrlPolyPoint& ctrlPolyPoint){
	M_CtrlPolyPoint midCtrlPolyPoint;
	M_CtrlPolyPoint centerCtrlPolyPoint;
	M_Int tempBlockList;


	for(int lPolyIdxA = 0; lPolyIdxA < ctrlPolySelList.length; lPolyIdxA++){
		midCtrlPolyPoint.length = 0;
		centerCtrlPolyPoint.length = 0;
		tempBlockList.length = 0;

		int polyA = ctrlPolySelList[lPolyIdxA];
		if(polyA < 0){
			continue;
		}
		int absoluteIdx = polyA;
		int len = ctrlPolyPoint[absoluteIdx].len;
		for(int k = 0; k < len; k++){
			int idxA = ctrlPolyPoint[absoluteIdx].i;
			NurbsPatch* patchA = ctrlPolyPoint[absoluteIdx].p;
			if(patchA->isCellCorner(idxA)){
				int cpiA = patchA->cpIndex[idxA];
				int idxA0 = ctrlPolyPoint[polyA + k].i;
				int k1 = getCorner(k, 1, len);
				int idxA1 = ctrlPolyPoint[polyA + k1].i;
				int k2 = getCorner(k1, 1, len);
				int idxA2 = ctrlPolyPoint[polyA + k2].i;
				int k3 = getCorner(k2, 1, len);
				int idxA3 = ctrlPolyPoint[polyA + k3].i;

				int cornerCpiA = patchA->cpIndex[idxA3];

				int mark = ctrlPolyPoint.length;

				if(idxA2 == 64 && idxA == 86){
					int g = 0;
				}

				ctrlPolyPoint += CtrlPolyPoint(patchA, idxA2, 0);
				centerCtrlPolyPoint += CtrlPolyPoint(patchA, idxA, 0);
				tempBlockList += lPolyIdxA;
				ctrlPolySelList[lPolyIdxA] = -polyA - 1;

				int cpiA0 = patchA->cpIndex[idxA0];
				int cpiA1 = patchA->cpIndex[idxA1];
				int cpiA3 = patchA->cpIndex[idxA3];

				if(cpiA0 == cpiA3){

					if(findTJoin(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchA, idxA, idxA2, lPolyIdxA, patchA, idxA1)){
					//if(findCloseCorner(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchA, idxA, idxA2, polyA, lPolyIdxA, patchA, idxA1)){

						addBlockListToDeleteList(tempBlockList, ctrlPolySelList, deleteCtrlPolySelList);

						ctrlPolyPoint[mark].len = ctrlPolyPoint.length - mark;

						ctrlPolyPoint += midCtrlPolyPoint;

						centerCtrlPolyPoint[0].len = centerCtrlPolyPoint.length;
						ctrlPolyPoint += centerCtrlPolyPoint;
						break;
					}
				}
				else if(cpiA0 == cpiA1){

					//if(findTJoin(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchA, idxA, idxA3, lPolyIdxA, patchA, idxA2)){
					if(findCloseCorner(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchA, idxA, idxA3, lPolyIdxA, patchA, idxA2)){

						addBlockListToDeleteList(tempBlockList, ctrlPolySelList, deleteCtrlPolySelList);

						ctrlPolyPoint[mark].len = ctrlPolyPoint.length - mark;

						ctrlPolyPoint += midCtrlPolyPoint;

						centerCtrlPolyPoint[0].len = centerCtrlPolyPoint.length;
						ctrlPolyPoint += centerCtrlPolyPoint;
						break;
					}
					else{
						ctrlPolyPoint.length = mark;
						unblockList(tempBlockList, ctrlPolySelList);
						midCtrlPolyPoint.length = 0;
						centerCtrlPolyPoint.length = 0;
					}

				}
				else if(findCloseCorner(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchA, idxA, idxA3, lPolyIdxA, patchA, idxA1)){

					addBlockListToDeleteList(tempBlockList, ctrlPolySelList, deleteCtrlPolySelList);

					ctrlPolyPoint[mark].len = ctrlPolyPoint.length - mark;

					ctrlPolyPoint += midCtrlPolyPoint;

					centerCtrlPolyPoint[0].len = centerCtrlPolyPoint.length;
					ctrlPolyPoint += centerCtrlPolyPoint;
					break;

				}
				else{
					ctrlPolyPoint.length = mark + 1;
					unblockList(tempBlockList, ctrlPolySelList);

					midCtrlPolyPoint.length = 0;

					if(findCloseCorner2(ctrlPolySelList, tempBlockList, ctrlPolyPoint, midCtrlPolyPoint, centerCtrlPolyPoint, patchA, idxA, cpiA, idxA3, cornerCpiA, polyA, lPolyIdxA, patchA, idxA1)){

						tempBlockList += lPolyIdxA;
						ctrlPolySelList[lPolyIdxA] = -polyA - 1;
						addBlockListToDeleteList(tempBlockList, ctrlPolySelList, deleteCtrlPolySelList);

						ctrlPolyPoint[mark].len = ctrlPolyPoint.length - mark;

						ctrlPolyPoint += midCtrlPolyPoint;

						ctrlPolyPoint += CtrlPolyPoint();
						break;
					}
					else{
						ctrlPolyPoint.length = mark;
						unblockList(tempBlockList, ctrlPolySelList);
						midCtrlPolyPoint.length = 0;
						centerCtrlPolyPoint.length = 0;
					}
				}
			}
			absoluteIdx++;
		}
	}
}

void Seamless::joinAllSelectedStitchPolys(){
	if(MultiSelect::ctrlPolySelList.length >= 2){
		M_Int deleteCtrlPolySelList;
		findCornerToBeginWith(MultiSelect::ctrlPolySelList, deleteCtrlPolySelList, ctrlPolyPoint);

		bubbleSort(deleteCtrlPolySelList);
		deleteCtrlPoly(deleteCtrlPolySelList);

		commonEndSitchPolys();
	}
}

void Seamless::copyStitchPoly(int poly){
	int len = ctrlPolyPoint[poly].len;
	for(int k = len; --k >= 0;){
		ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[poly].p, ctrlPolyPoint[poly].i, len);
		poly++;
	}
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		do{
			ctrlPolyPoint += CtrlPolyPoint(ctrlPolyPoint[poly].p, ctrlPolyPoint[poly].i, midLen);
			poly++;
		}while(--midLen > 0);
	}
}

void Seamless::uCopyStitchPolyAndSlide(int poly, int uOffset){
	int len = ctrlPolyPoint[poly].len;
	for(int k = len; --k >= 0;){
		if(NurbsPatch* patch = ctrlPolyPoint[poly].p){
			int i = ctrlPolyPoint[poly].i;
			int uDim = patch->dim.x;
			int u = i % uDim + uOffset;
			int v = i / uDim;
			int idx = v * uDim + u;
			ctrlPolyPoint += CtrlPolyPoint(patch, idx, len);
		}
		else{
			ctrlPolyPoint += CtrlPolyPoint();
		}
		poly++;
	}
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		do{
			if(NurbsPatch* patch = ctrlPolyPoint[poly].p){
				int i = ctrlPolyPoint[poly].i;
				int uDim = patch->dim.x;
				int u = i % uDim + uOffset;
				int v = i / uDim;
				int idx = v * uDim + u;
				ctrlPolyPoint += CtrlPolyPoint(patch, idx, midLen);
			}
			else{
				ctrlPolyPoint += CtrlPolyPoint();
			}
			poly++;
		}while(--midLen > 0);
	}
}

void Seamless::vCopyStitchPolyAndSlide(int poly, int vOffset){
	int len = ctrlPolyPoint[poly].len;
	for(int k = len; --k >= 0;){
		if(NurbsPatch* patch = ctrlPolyPoint[poly].p){
			int i = ctrlPolyPoint[poly].i;
			int uDim = patch->dim.x;
			int u = i % uDim;
			int v = i / uDim + vOffset;
			int idx = v * uDim + u;
			ctrlPolyPoint += CtrlPolyPoint(patch, idx, len);
		}
		else{
			ctrlPolyPoint += CtrlPolyPoint();
		}
		poly++;
	}
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		do{
			if(NurbsPatch* patch = ctrlPolyPoint[poly].p){
				int i = ctrlPolyPoint[poly].i;
				int uDim = patch->dim.x;
				int u = i % uDim;
				int v = i / uDim + vOffset;
				int idx = v * uDim + u;
				ctrlPolyPoint += CtrlPolyPoint(patch, idx, midLen);
			}
			else{
				ctrlPolyPoint += CtrlPolyPoint();
			}
			poly++;
		}while(--midLen > 0);
	}
}

void Seamless::generateRawStitchPolys(){
	ctrlPolyPoint.length = 0;

	for(int k = cpNodeList.length; --k >= 0;){
		Build* bld = cpNodeList[k]->container;
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(bld)){
			if(patch->isCtrlPolyRef){
				patch->fillEdgesWithCtrlPolys();
			}
		}
	}
}

bool deleteStitchPolys2(NurbsPatch* targetPatch, int poly, const M_CtrlPolyPoint& ctrlPolyPoint){
	int len = ctrlPolyPoint[poly].len;
	int midIndex = poly + len;

	for(int k = len; --k >= 0;){
		int idxA = poly + k;
		if(ctrlPolyPoint[idxA].p == targetPatch){
			return true;
		}
	}
	poly += len;
	for(int k = len + 1; --k >= 0;){
		int midLen = ctrlPolyPoint[poly].len;
		if(midLen == 0){
			midLen++;
		}
		else{
			for(int k = midLen; --k >= 0;){
				int idxA = poly + k;
				if(ctrlPolyPoint[idxA].p == targetPatch){
					return true;
				}
			}
		}
		poly += midLen;
	}
	return false;
}

void NurbsPatch::deleteStitchPolys(){
	M_CtrlPolyPoint& ctrlPolyPoint = smls->ctrlPolyPoint;
	M_Int deleteCtrlPoly;
	int poly = 0;

	while(poly < ctrlPolyPoint.length){
		if(deleteStitchPolys2(this, poly, ctrlPolyPoint)){
			deleteCtrlPoly += poly;
		}
		poly = smls->getNextCtrlPoly(poly);
	}
	smls->deleteCtrlPoly(deleteCtrlPoly);
}

bool Seamless::isPointInCenterOfNon4SidedStitchPoly(NurbsPatch* checkPatch, int checkIdx){
	int poly = 0;

	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		for(int k = 0; k < len; k++){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}

		int centerLen = ctrlPolyPoint[poly].len;

		if(len != 4){
			for(int i = 0; i < centerLen; i++){
				if(ctrlPolyPoint[poly + i].p == checkPatch && ctrlPolyPoint[poly + i].i == checkIdx){
					return true;
				}
			}
		}

		

		if(centerLen == 0){
			centerLen++;
		}
		poly += centerLen;
	}
	return false;
}


bool stitchPolysAreCorrupt2(M_CtrlPolyPoint& ctrlPolyPoint, int poly, int midLen){
	for(int k = midLen; --k >= 0;){
		NurbsPatch* patch = ctrlPolyPoint[poly + k].p;
		int idx = ctrlPolyPoint[poly + k].i;
		if(idx >= patch->cpIndex.length){
			return true;
		}
	}
	return false;
}


bool Seamless::stitchPolysAreCorrupt(){
	int cpIdx = 0;
	int poly = 0;

	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		for(int k = 0; k < len + 1; k++){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen){
				if(stitchPolysAreCorrupt2(ctrlPolyPoint, poly, midLen)){
					return true;
				}
			}
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}

	}
	return false;
}


int getLowestCpIndex(M_CtrlPolyPoint& ctrlPolyPoint, int poly, int len){
	NurbsPatch* patch = ctrlPolyPoint[poly].p;
	int idx = ctrlPolyPoint[poly].i;
	int lowest = patch->cpIndex[idx];
	for(int k = len; --k >= 1;){
		patch = ctrlPolyPoint[poly + k].p;
		idx = ctrlPolyPoint[poly + k].i;
		int cpIdx = patch->cpIndex[idx];
		if(cpIdx < lowest){
			lowest = cpIdx;
		}
	}
	return lowest;
}
void redirectCpIndex(int newCpIdx, M_CtrlPolyPoint& ctrlPolyPoint, int poly, int len){
	for(int k = len; --k >= 0;){
		NurbsPatch* patch = ctrlPolyPoint[poly + k].p;
		int idx = ctrlPolyPoint[poly + k].i;
		patch->cpIndex.refValue()[idx] = newCpIdx;
	}
}

void Seamless::createSharedPointsFromStitchPolys(){
	int cpIdx = 0;
	int poly = 0;

	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		poly += len;
		for(int k = 0; k < len + 1; k++){
			int midLen = ctrlPolyPoint[poly].len;
			if(midLen >= 2){
				cpIdx = getLowestCpIndex(ctrlPolyPoint, poly, midLen);
				redirectCpIndex(cpIdx, ctrlPolyPoint, poly, midLen);
			}
			if(midLen == 0){
				midLen++;
			}
			poly += midLen;
		}

	}
}

bool Seamless::seekCtrlPointInStitchPolys(const NurbsPatch* patch, const int idx, int& poly, int& corner){
	while(poly < ctrlPolyPoint.length){
		int len = ctrlPolyPoint[poly].len;
		for(corner = len; --corner>=0;){
			int plyIdx = poly + corner;
			if(patch == ctrlPolyPoint[plyIdx].p && idx == ctrlPolyPoint[plyIdx].i){
				return true;
			}
		}
		poly = getNextCtrlPoly(poly);
	}
	return false;
}


bool Seamless::seekStitchPolyContainingSharedPoint(const int cpIdx, int& poly, int& corner, int blockPoly){
	while(poly < ctrlPolyPoint.length){
		if(poly != blockPoly){
			for(corner = ctrlPolyPoint[poly].len; --corner >= 0;){
				NurbsPatch* patch = ctrlPolyPoint[poly + corner].p;
				int idx = ctrlPolyPoint[poly + corner].i;
				if(patch->cpIndex[idx] == cpIdx){
					return true;
				}
			}
		}
		poly = getNextCtrlPoly(poly);
	}
	return false;
}

bool Seamless::seekCornerContainingSharedPoint(const int cpIdx, int poly, int& corner){
	for(corner = ctrlPolyPoint[poly].len; --corner >= 0;){
		NurbsPatch* patch = ctrlPolyPoint[poly + corner].p;
		int idx = ctrlPolyPoint[poly + corner].i;
		if(patch->cpIndex[idx] == cpIdx){
			return true;
		}
	}
	return false;
}

bool seekCtrlPointInStitchPolyOnEdge(M_CtrlPolyPoint& ctrlPolyPoint, int poly, int& corner, const int blockCorner, NurbsPatch*& patch, int& idx){
	for(int i = ctrlPolyPoint[poly].len; --i>=0;){
		if(i == blockCorner){
			continue;
		}
		patch = ctrlPolyPoint[poly + i].p;
		idx = ctrlPolyPoint[poly + i].i;
		if(patch->isEdge(idx)){
			corner = i;
			return true;
		}
	}
	return false;
}

int preGetMidCorner(M_CtrlPolyPoint& ctrlPolyPoint, int poly, int cornerA, int cornerB, int& edge, const int len){
	if(cornerA + 1 == cornerB){
		edge = cornerA;
	}
	if(cornerA - 1 == cornerB){
		edge = cornerB;
	}
	if(cornerA + (len - 1) == cornerB){
		edge = cornerB;
	}
	if(cornerA - (len - 1) == cornerB){
		edge = cornerA;
	}

	return getMidPointAbsoluteIdx(poly + len, edge, ctrlPolyPoint);

}

static int cntr = 0;

void showControlPoly(M_CtrlPolyPoint& ctrlPolyPoint, int poly){
	int len = ctrlPolyPoint[poly].len;
	for(int k = 0; k < len; k++){
		MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly + k].p, ctrlPolyPoint[poly + k].i);
	}
}

bool Seamless::shareMidPointsTooInStitchPolys2(NurbsPatch* patchA0, const int idxA0, int& poly0){
	int poly1 = 0;
	int cornerA0 = 0;
	int cornerB0 = 0;

	int cornerA1 = 0;
	int cornerB1 = 0;

	NurbsPatch* patchB0 = 0;
	int idxB0 = 0;

	cntr++;

	if(seekCtrlPointInStitchPolys(patchA0, idxA0, poly0, cornerA0)){
		if(seekCtrlPointInStitchPolyOnEdge(ctrlPolyPoint, poly0, cornerB0, cornerA0, patchB0, idxB0)){
			while(poly1 < ctrlPolyPoint.length){
				if(poly0 == 101 && poly1 == 236){
					int a = 0;
				}
				if(seekStitchPolyContainingSharedPoint(patchA0->cpIndex[idxA0], poly1, cornerA1, poly0)){

					//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly1 + cornerA1].p, ctrlPolyPoint[poly1 + cornerA1].i);
					if(poly0 == 101 && poly1 == 236){
						//showControlPoly(ctrlPolyPoint, poly0);
						//showControlPoly(ctrlPolyPoint, 101);
						//showControlPoly(ctrlPolyPoint, 236);
						//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly1 + cornerA1].p, ctrlPolyPoint[poly1 + cornerA1].i);
						//MultiSelect::pointList += ControlPoint(patchA0, idxA0);
						//MultiSelect::pointList += ControlPoint(patchB0, idxB0);
					}

					if(seekCornerContainingSharedPoint(patchB0->cpIndex[idxB0], poly1, cornerB1)){
						int len0 = ctrlPolyPoint[poly0].len;
						int len1 = ctrlPolyPoint[poly1].len;

						if(poly0 == 101 && poly1 == 236){
							//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly1 + cornerA1].p, ctrlPolyPoint[poly1 + cornerA1].i);
							//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly1 + cornerB1].p, ctrlPolyPoint[poly1 + cornerB1].i);
							//int gg = ctrlPolyPoint[poly1].len;
							//showControlPoly(ctrlPolyPoint, poly1);
							//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly1 + cornerA1].p, ctrlPolyPoint[poly1 + cornerA1].i);
							//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly1 + cornerB1].p, ctrlPolyPoint[poly1 + cornerB1].i);
						}

						int mid0 = preGetMidCorner(ctrlPolyPoint, poly0, cornerA0, cornerB0, mid0, len0);
						int mid1 = preGetMidCorner(ctrlPolyPoint, poly1, cornerA1, cornerB1, mid1, len1);

	//					MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly0 + cornerA0].p, ctrlPolyPoint[poly0 + cornerA0].i);
	//					MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly0 + cornerB0].p, ctrlPolyPoint[poly0 + cornerB0].i);

						//MultiSelect::pointList += ControlPoint(ctrlPolyPoint[poly1 + cornerB1].p, ctrlPolyPoint[poly1 + cornerB1].i);


						if(ctrlPolyPoint[mid0].len > 0 && ctrlPolyPoint[mid1].len > 0){
							NurbsPatch* patch = ctrlPolyPoint[mid1].p;
							int idx = ctrlPolyPoint[mid1].i;
							int cpIdx = patch->cpIndex[idx];

							for(int i = ctrlPolyPoint[mid0].len; --i>=0;){
								NurbsPatch* patch = ctrlPolyPoint[mid0 + i].p;
								int idx = ctrlPolyPoint[mid0 + i].i;
								patch->cpIndex.refValue()[idx] = cpIdx;
							}
						}
					}
					poly1 = getNextCtrlPoly(poly1);
				}
			}
		}
	}
	return poly0 < ctrlPolyPoint.length;
}


bool Seamless::unShareMidPointsTooInStitchPolys2(NurbsPatch* patchA0, const int idxA0, int& poly0){
	M_Vec3f& cp = this->cp.refValue();
	M_Float& cw = this->cw.refValue();

	int poly1 = 0;
	int cornerA0 = 0;
	int cornerB0 = 0;

	int cornerA1 = 0;
	int cornerB1 = 0;

	NurbsPatch* patchB0 = 0;
	int idxB0 = 0;

	cntr++;

	if(seekCtrlPointInStitchPolys(patchA0, idxA0, poly0, cornerA0)){
		if(seekCtrlPointInStitchPolyOnEdge(ctrlPolyPoint, poly0, cornerB0, cornerA0, patchB0, idxB0)){
			while(poly1 < ctrlPolyPoint.length){
				if(seekStitchPolyContainingSharedPoint(patchA0->cpIndex[idxA0], poly1, cornerA1, poly0)){

					if(seekCornerContainingSharedPoint(patchB0->cpIndex[idxB0], poly1, cornerB1)){
						int len0 = ctrlPolyPoint[poly0].len;
						int len1 = ctrlPolyPoint[poly1].len;

						int mid0 = preGetMidCorner(ctrlPolyPoint, poly0, cornerA0, cornerB0, mid0, len0);
						int mid1 = preGetMidCorner(ctrlPolyPoint, poly1, cornerA1, cornerB1, mid1, len1);

						if(ctrlPolyPoint[mid0].len > 0 && ctrlPolyPoint[mid1].len > 0){
							NurbsPatch* patch = ctrlPolyPoint[mid1].p;
							int idx = ctrlPolyPoint[mid1].i;
							int cpIdx = patch->cpIndex[idx];

							for(int i = ctrlPolyPoint[mid0].len; --i>=0;){
								NurbsPatch* patch = ctrlPolyPoint[mid0 + i].p;
								int idx = ctrlPolyPoint[mid0 + i].i;

								M_Int& cpIndex = patch->cpIndex.refValue();

								cpIndex[idx] = cp.length;
								cp += patch->point[idx];
								cw += patch->weight[idx];

							}
						}
					}
					poly1 = getNextCtrlPoly(poly1);
				}
			}
		}
	}
	return poly0 < ctrlPolyPoint.length;
}


void Seamless::shareMidPointsTooInStitchPolys(NurbsPatch* patchA0, const int idxA0){
	int poly0 = 0;
	cntr = 0;

	while(shareMidPointsTooInStitchPolys2(patchA0, idxA0, poly0)){
		poly0 = getNextCtrlPoly(poly0);
	}

	centralCPsToPatches();


	//ctrf = true;
	//BackUp::monostable.beginCycle();
}

void Seamless::unShareMidPointsTooInStitchPolys(NurbsPatch* patchA0, const int idxA0){
	int poly0 = 0;
	//MultiSelect::pointList.length = 0;
	cntr = 0;

	while(unShareMidPointsTooInStitchPolys2(patchA0, idxA0, poly0)){
		poly0 = getNextCtrlPoly(poly0);
	}

	centralCPsToPatches();
	generateRawStitchPolys();
	stitchAllPolys();
	joinAllParts();


	houseKeepVisualClue();

}