/* Seamless3d

Copyright (C) 2001-2017 Graham John Perrett (AKA thyme) thyme@seamless3d.com www.seamless3d.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify,
merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*/


//#include <windows.h>
#include <algorithm>    // std::max
using namespace std;

#ifdef WIN32
#include <direct.h>
#else
#include <unistd.h>
#endif

#include "seamless3d.h"
#include "techuelife_triangle_editor.h"
#include "gui.h"
#include "scene_tree.h"
#include "robot_aid.h"

extern const char* title_save_file_as;

S_Triangle triVertexRem=S_Triangle(0,0,0,0,0,0);

#define halfArrowlen .1
#define g_spaghettiRad .025

Node* g_parent;
bool g_perspective = false;
bool g_panViewMode = false;

bool extern g_browseMode;

extern bool g_viewZRotDragMode;

M_Vec2f g_screenPoint;
M_Bool g_pointIsVisible;
M_Int g_pointLookup;

#ifdef PROG_ROB
StoreCrossPoint g_storeCrossPoint;
#endif

extern bool RotateMode;
extern bool MouseFlag;
extern bool tugFlag;
extern bool g_dragWithDescendants;
extern bool g_viewPosDragMode;
extern Vec3f g_mouseFrom;

extern bool g_tugRotFlag;
extern bool orientationTugFlag;
extern float g_version;
extern bool outPutIndices;
extern bool g_leftDoubleClicked;
extern String g_bindSmlsFile;

extern String g_seamlessDir;
extern bool	g_mirrorModeVertexFound;
extern int	g_mirrorVertex;

extern OpenNetFilePtr g_openNetFile;
extern Ptr<InsertComplete> g_insertComplete;
extern Ptr<CutComplete> g_cutComplete;
extern Vec3f g_viewDragMousePos;

Vec3f Seamless::selCoordPos;
void addNodesForNewFile();

bool g_paletteMode = false;
bool g_colorOwnership = false;
bool g_curvedCageMode = false;
extern bool g_dragMorphFlag;
bool g_showPatchBorderMode = false;

extern Ptr<PasteOptions> g_pasteOptions;
extern Ptr<DeleteOptions> g_deleteOptions;
extern Ptr<TimelineFilter> g_timelineFilter;
extern bool g_joinActive;
void mouse2dSync();

SettingsPtr g_settings;

GeomRenderPtr g_redPoint = new GeomRender;
GeomRenderPtr g_lastPoint = new GeomRender;
GeomRenderPtr g_nextPoint = new GeomRender;
GeomRenderPtr g_yellowPoint = new GeomRender;
GeomRenderPtr g_cyanPoint = new GeomRender;
GeomRenderPtr g_redConePoint = new GeomRender;
GeomRenderPtr g_greenConePoint = new GeomRender;
GeomRenderPtr g_blueConePoint = new GeomRender;

TransformPtr g_redConeBase = new Transform;
TransformPtr g_greenConeBase = new Transform;
TransformPtr g_blueConeBase = new Transform;

M_Node g_nailDown;

ArtPtr g_mirrorPart;
Node* g_visualClue = nullPtr;
SeamlessPtr g_workPolyMapSmlsColor = new Seamless;
SeamlessPtr g_workPolyMapSmlsTexture = new Seamless;
ImmediatePart* g_workPolyMapColor = new ImmediatePart;
ImmediatePart* g_workPolyMapTexture = new ImmediatePart;

SeamlessPtr g_vcSeamless = new Seamless;
SeamlessPtr g_vcOscSmls;
TransformPtr g_vcOscBase = new Transform;
TransformPtr g_vcRingBuildBase;
TransformPtr g_vcRingBuildPivotPoint;
TransformPtr g_vcRingBuildOrientationA;
TransformPtr g_vcRingBuildOrientationB;
TransformPtr g_vcRingBuildOrientationC;
TransformPtr g_vcRingBuildOrientationD;
SeamlessPtr g_vcTransferBCyl;
CylinderBuild* g_vcTransferBCylCylinderBuild = new CylinderBuild;
TransformPtr g_vcTransferBCylBase=new Transform;
TransformPtr g_vcECScaleBase=new Transform;
TransformPtr g_vcBezierBase = new Transform;
TransformPtr g_rawDrawNurbsLineClueBase = new Transform;
SeamlessPtr g_vcNurbsCtrlPoly = new Seamless;
ArtPtr g_vcNurbsCtrlPolyPart;
TransformPtr g_vcCCLatheBase;
Ptr<PartClue> g_vcArtTriBase;



IndexedLineSet* g_triClue = new IndexedLineSet;
Coordinate* g_triClueCoord = new Coordinate;
Color* g_triClueColor = new Color;
Shape* g_triClueShape = new Shape;

IndexedLineSet* g_cursorOverPointIls = new IndexedLineSet;
ShapePtr g_cursorOverPointIlsShape = new Shape;
TransformPtr g_cursorOverPointIlsBase = new Transform;

IndexedLineSet* g_nurbsIls = new IndexedLineSet;
Coordinate* g_nurbsIlsCoord = new Coordinate;
Color* g_nurbsIlsColor = new Color;
ShapePtr g_nurbsIlsShape = new Shape;
TransformPtr g_nurbsIlsBase = new Transform;
Ptr<PartCageClue> g_partCageClue = new PartCageClue;


ShapePtr g_gridShape = new Shape;
IndexedLineSet* g_gridIls = new IndexedLineSet;
Coordinate* g_gridCoord = new Coordinate;
Color* g_gridColor = new Color;


Ptr<CenterJointClue> g_vcCenterJointBase = new CenterJointClue;
Ptr<TransferBCylClue> g_transferBCylClue = new TransferBCylClue;
Ptr<TransferOwnershipClue> g_transferOwnershipClue = new TransferOwnershipClue;
ShapePtr g_vcSkeletonLinesShape;
Coordinate* g_vcSkeletonLinesCoordinate = 0;


TransformPtr g_insertArrow;
TransformPtr g_vcECScaleTangentArrow;
ShapePtr g_vcEllipticalPolyline2D = new Shape;
ShapePtr g_rawDrawNurbsLineClue = new Shape;
ShapePtr g_transformLeverClue = new Shape;
ShapePtr g_translateDimensionClue = new Shape;
ShapePtr g_vcFuseSurfaceFlatStageAidShape = new Shape;
Material* g_vcEllipticalMaterial = new Material;
Material* g_stickManMaterial = new Material;

ShapePtr g_vcSelectRectShape = new Shape;
ShapePtr g_vcTransformFallOffShape = g_vcSelectRectShape;
Material* g_vcSelectRectMaterial = new Material;
Group* g_vcSelectRectGroup = new Group;

AppearancePtr g_vcCommonAppearance = new Appearance;

int Viewpoint::dragMode = 0;
Vec3f Viewpoint::remLookAt;
Vec3f Viewpoint::remPosition;
float Viewpoint::remLookAtDistance = 0;
float Viewpoint::remUpDistance = 0;
Rotation Viewpoint::remOrientation;
ArtPtr g_vcPart;
StemPtr g_vcStem;
SurfaceGeneratorPtr g_vcSurface;
CCLathePtr g_vcCCLathe;
SurfaceTransformPtr g_vcSurfaceTransform;
CCBendPtr g_vcCCBend;
SurfaceTransformPtr g_vcBaseRad;
Matrix g_viewMatrix;

bool NurbsPatch::tugRingPivotPoint = false;
Node* SenseTugGlobals::node=0;
Art* SenseTugGlobals::actTriPart=0;
int SenseTugGlobals::ActTri=0;
float SenseTugGlobals::hitZ = 0;
Vec3f SenseTugGlobals::clickPos;
int SenseTugGlobals::corner=0;
int SenseTugGlobals::ActCtrlPnt=0;
//bool SenseTugGlobals::sameVertexHit=false;
Vec3f SenseTugGlobals::pointer;

LockPtr Hair::selLock = 0;
int Hair::selHoleIndex = 0;
InsertPatch g_insertPatch;
DeleteControlLine g_deleteControlLine;
MergePatches g_mergePatches;
Ptr<AddLathe> g_addLathe = new AddLathe;
AddBox g_addBox;
AddSphere g_addSphere;

Ptr<AddNurbsLine> g_addNurbsLine = new AddNurbsLine;
Ptr<DrawNurbsLine> g_drawNurbsLine = new DrawNurbsLine;
Ptr<AddBone> g_addBone = new AddBone;
bool g_rectangleSelectPointReady = false;
bool g_rectangleUnselectPointReady = false;
bool g_rectangleSelectPolyReady = false;

Stem* g_radialTugStem;
RadialTug* g_radialTug;
Ptr<Radial_Tug> g_radial_Tug = new Radial_Tug;
Ptr<RadicalPatch> g_radicalPatch = new RadicalPatch;

int BendyLathe::radIdx = 0;


int Lock::selBorder = 0;
LockPtr Lock::selBorderLock;
M_<Lock*> Lock::relevantLockList;
M_Int Lock::borderOffset;
M_Bool Lock::regionSelected;
M_Int Lock::copyBorderList;
bool Lock::splitRegionMode = false;
bool Lock::breakBorderMode = false;
float g_mouseSense = .01;
//float g_mouseSense2 = g_mouseSense * 5;
float g_mouseSense2 = g_mouseSense * 2.5;

M_Vec2f Lock::splitRegionPoints;


//bool g_spaghetti=true;

//extern ScenePtr g_scene;
Pointer g_pointer;
extern Rotation g_beginViewRot;
extern Vec2i g_beginMousePos;
int g_lockFile = 0;

extern int getFileSize(FILE* Stream);
extern bool g_createdINI;

int getCenterBegin(const M_CtrlPolyPoint& ctrlPolyPoint, int poly);
void search(char* match ,char*& from,char* BufEnd);
void searchTillText(char*& p);
void searchTillNextWord(char*& p);
void compileScalePositionInterpolator(const Art* part, Matrix& matrix);
void compileOrientationInterpolator(const Art* part, Rotation rotation, M_<SceneTreeListItem>& sceneTreeList);
Vec2f convertSouthPoleToMercator(const Vec2f& coord);
Vec2f convertNorthPoleToMercator(const Vec2f& coord);

void colorRowBlue(NurbsPatch* patch, M_Color3f& color, int row);
void colorColBlue(NurbsPatch* patch, M_Color3f& color, int col);

void fillPolyLineIndex(M_Vec3f& point, M_Int& coordIndex);
bool checkAllPivotPoints(bool perspective, StickMan* stickMan);

float g_viewDist = 3;
void setTitleName();
extern Ptr<Palette> g_palette;

unsigned colorToHex(Color3f color);
int BackUp::backedUpFileCntr = 0;
int BackUp::backedUpFileCntrEnd = 0;
bool SurfacePoint(
				  float u, float v, int uOrder, int vOrder, int uDimension, int vDimension,
				  const float *uKnot, const float * vKnot,
				  const Vec3f *P,const float *W,
				  Vec3f *dest // result
				  );



extern M_Int progIndices;
void changeFloatFormat(int numOfDigits);
bool decodeSmlsVrmlHeader(String& str, int& i);
extern bool forceSaveAs=false;
void resetCPNodeList();

M_Matrix m_matrix;
M_Int m_smi;
M_Vec3f g_m_coord;

Matrix g_catchMatrix;
Matrix g_catchIMatrix;
String BackUp::ourBackUpName="0_";
Ptr<_3DWin> g_3d = new _3DWin;

BrushPtr g_brush;
FillPtr g_fill;
Ptr<Grid> g_grid;
Ptr<ExportVrmlX3d> g_exportVrmlX3d;
Ptr<ExportMovie> g_exportMovie;



Ptr<ColorPicker> g_colorPicker;

int cameraCoordIndex[] = {0, 1, -1, 2, 3};

extern SeamlessPtr g_cameraObject;
extern bool g_showCamerasInAnimMode;
void createCameraObject();
PaletteCP* paletteCPAllreadyOpen();
bool g_hidePolysInHiddenPatches = false;

/*
enum WinSizePosIndices{
	THREE_D_POS_X=0,
	THREE_D_POS_Y=1,
	THREE_D_CX=2,
	THREE_D_CY=3,

	TOOLBAR_POS_X=4,
	TOOLBAR_POS_Y=5,
	TOOLBAR_CX=6,
	TOOLBAR_CY=7,

	SCENETREE_POS_X=8,
	SCENETREE_POS_Y=9,
	SCENETREE_CX=10,
	SCENETREE_CY=11,

	NODE_POS_X=12,
	NODE_POS_Y=13
};
*/

/*
void Scene::setViewFromRaw(){
	position.refValue()=-translation+(-rotation*Vec3f(0,0,g_viewDist));
	orientation.refValue()=-rotation;
	if(dynamic_cast<Scene*>(g_pw->node.p)){
		g_pw->paint();
	}
	//BackUp::monostable.beginCycle();
}
*/

Vec3f mouse2dSync(Vec2i pixelPos);

void Scene::rotateScene(){
	Vec2i relMousePos = g_newMousePos - g_beginMousePos;
	if(relMousePos.x == 0 && relMousePos.y == 0){
		return;
	}


//	trackball code
	if(g_viewZRotDragMode){
		Vec2f beginPoint = Vec2f(g_beginMousePos.x - (g_3d->size.x >> 1), g_beginMousePos.y - (g_3d->size.y >> 1));
		float begin_zAngle = -beginPoint.getAngle2();

		Vec2f point = Vec2f(g_newMousePos.x - (g_3d->size.x >> 1), g_newMousePos.y - (g_3d->size.y >> 1));
		float zAngle = -point.getAngle2();
		rotation = g_beginViewRot * Rotation(Vec3f(0, 0, 1), zAngle - begin_zAngle);
	}
	else{
		Vec2f diff = Vec2f(relMousePos.x, relMousePos.y);
		float a = diff.getAngle2();
		float distance = diff.length();
		Vec3f axis = Vec3f(0,1,0);
		axis = Rotation(0, 0, 1, -a) * axis;

		/*
		Vec3f pointer = mouse2dSync(g_newMousePos);
		//pointer = pointer * .0025;
		pointer.z = g_scene->translation.z;
			g_scene->translation = pointer;
			//g_viewDragMousePos = g_pointer.translation;
			return;
		*/

		rotation = g_beginViewRot * Rotation(axis, distance *.0025);
	}
	//rotation = g_beginViewRot * Rotation(Vec3f(0,0,1), distance*.0025);

//	rotation=rotation*Rotation(0,1,0, PI*.005*(float)dimslX)*Rotation(1,0,0, PI*.005*(float)dimslY);
//	BackUp::monostable.beginCycle();

	if(g_scene->mode & SELECT_OBJECT_MODE){
		g_transformObj->resetRotationLever();
	}
	else if(g_scene->mode & MODELLING_MODE){
		g_transformCP->resetRotationLever();
		g_transformCP->houseKeep();

	}
	else if(g_scene->mode & ANIMATION_MODE){
		g_vcArtAnimBase->resetRotationLever();
	}

	g_viewMatrix = g_scene->getViewMatrix();
	
	g_beginMousePos = g_newMousePos;
	g_beginViewRot = g_scene->rotation;

	if(g_scene->mode & BACKFACE_IGNORE_MODE && g_scene->mode & (MODELLING_MODE | ANIMATION_MODE)){
		houseKeepVisualClue();
	}
	else if(g_scene->mode & PAINT_MODE){
		if(g_scene->mode & BRUSH_MODE){
			g_brush->houseKeep();
		}
	}
}
void BackUp::commonUndoRedo(){
	//g_scene->grid = g_grid;

	decodeFunctions(g_loadStr);
	g_stw->genNodeList();
	postLoadProcessNodes(g_sceneTreeList);

	g_scene->postLoadProcessCollapse();
	resetLooseEnds();
	Window::postLoadProcessWin(*g_scene);
}

bool checkVertexBelongsToTriangle(Art* vertexPart, Art* triPart, int vertexIndex, int &corner, int triangleIndex){
	if(triPart==0)return 0;
	int i=triangleIndex*3+corner;
	M_Triangle& triangle=triPart->triangle;
	if((unsigned)i>=(unsigned)triangle.length*3)return false;
	if(vertexPart!=triangle.refPart(i))return false;
	if(vertexIndex!=triangle.refIndex(i))return false;
	return true;
}
inline Art* findFirstRelivantPartContainingTriangles(M_Art& relevantPart){
	for(int k=0; k<relevantPart.length;k++){
		if(relevantPart[k]->triangle.length){
			return relevantPart[k];
		}
	}
	throw;
}
inline int findRelivantPartIndex(M_Art& relevantPart, Art*& part){
	for(int k=0; k<relevantPart.length;k++){
		if(relevantPart[k]==part){
			return k;
		}
	}
	throw;
}
int findNextTriangleFromVertex(Art* vertexPart, Art*& triPart, const int vertexIndex, int &corner, int triangleIndex){
	if(vertexIndex>=vertexPart->coord.refValue().length)throw vertexPart;

	M_Art& relevantPart=vertexPart->relevantPart;

	if(checkVertexBelongsToTriangle(vertexPart, triPart, vertexIndex, corner, triangleIndex)==false){
		triPart=findFirstRelivantPartContainingTriangles( relevantPart);
		corner=0;
		triangleIndex=0;
	}
	M_Triangle* triangle=&triPart->triangle;
	int k=findRelivantPartIndex(relevantPart, triPart);

	while(1){
		if(++triangleIndex>=triangle->length){
			if(++k>=relevantPart.length){
				k=0;
			}
			triangle=&relevantPart[k]->triangle;
			triangleIndex=0;
		}
		if(triangle->length){
			S_Triangle tri=(*triangle)[triangleIndex];

			if(	vertexIndex==tri.ia && tri.pa == vertexPart){
				corner=0;
				triPart=relevantPart[k];
				return triangleIndex;
			}
			if(	vertexIndex==tri.ib && tri.pb == vertexPart){
				corner=1;
				triPart=relevantPart[k];
				return triangleIndex;
			}
			if(	vertexIndex==tri.ic && tri.pc == vertexPart){
				corner=2;
				triPart=relevantPart[k];
				return triangleIndex;
			}
		}
	}
}
int findPrevTriangleFromVertex(Art* vertexPart, Art*& triPart, const int vertexIndex, int &corner, int triangleIndex){
	if(vertexIndex>=vertexPart->coord.refValue().length)throw vertexPart;

	M_Art& relevantPart=vertexPart->relevantPart;

	if(checkVertexBelongsToTriangle(vertexPart, triPart, vertexIndex, corner, triangleIndex)==false){
		triPart=findFirstRelivantPartContainingTriangles( relevantPart);
		corner=0;
		triangleIndex=0;
	}
	M_Triangle* triangle=&triPart->triangle;
	int k=findRelivantPartIndex(relevantPart, triPart);

	while(1){
		if(--triangleIndex<0){
			if(--k<0){
				k=relevantPart.length-1;
			}
			triangle=&relevantPart[k]->triangle;
			triangleIndex=triangle->length-1;
		}
		S_Triangle tri=(*triangle)[triangleIndex];

		if(triangle->length){
			if(	vertexIndex==tri.ia && tri.pa == vertexPart){
				corner=0;
				triPart=relevantPart[k];
				return triangleIndex;
			}
			if(	vertexIndex==tri.ib && tri.pb == vertexPart){
				corner=1;
				triPart=relevantPart[k];
				return triangleIndex;
			}
			if(	vertexIndex==tri.ic && tri.pc == vertexPart){
				corner=2;
				triPart=relevantPart[k];
				return triangleIndex;
			}
		}
	}
}

inline bool findPolyThatDoesBelong(M_Triangle& triangle, Art*& target, int& triIndex, int vi){
	for(int i=0; i<triangle.length; i++ ){
		S_Triangle tri=triangle[i];
		if(tri.pa==target && tri.ia==vi){
			triIndex=i;
			return true;
		}
		if(tri.pb==target && tri.ib==vi){
			triIndex=i;
			return true;
		}
		if(tri.pc==target && tri.ic==vi){
			triIndex=i;
			return true;
		}
	}
	return false;
}
inline void refreshCorner(Art* part, const int& actCtrlPnt, S_Triangle& tri, int& corner){
	Vec3f* activeVertex = &part->coord.refValue()[actCtrlPnt];
	if(activeVertex==&tri.pa->coord.refValue()[tri.ia]){
		corner=0;
	}
	else if(activeVertex==&tri.pb->coord.refValue()[tri.ib]){
		corner=1;
	}
	else if(activeVertex==&tri.pc->coord.refValue()[tri.ic]){
		corner=2;
	}
	else{
		throw;
	}
}
inline void refreshActTriPart(Art*& part, Seamless*& smls){
	if(checkVertexBelongsToTriangle(part, smls->actTriPart, smls->ActCtrlPnt, smls->ActTriCorner, smls->ActTri)){
		return;
	}
/*
	if(	smls->actTriPart!=0 &&
		smls->actTriPart->triangle.length &&
		(unsigned)smls->ActTri<(unsigned)smls->actTriPart->triangle.length){

		S_Triangle& actTri=smls->actTriPart->triangle[smls->ActTri];
		if(
		   (actTri.ia==smls->ActCtrlPnt && actTri.pa==part ||
			actTri.ib==smls->ActCtrlPnt && actTri.pb==part ||
			actTri.ic==smls->ActCtrlPnt && actTri.pc==part )
		){
			return;
		}
	}
*/
	//first look for a poly owned by us that references us
	if(findPolyThatDoesBelong(part->triangle, part, smls->ActTri, smls->ActCtrlPnt)){
		smls->actTriPart=part;
		return;
	}
	//then look for a poly from a relevant part that references us
	M_Art& relevantPart=part->relevantPart;
	for(int k=relevantPart.length; --k>=0;){
		if(findPolyThatDoesBelong(relevantPart[k]->triangle, part, smls->ActTri, smls->ActCtrlPnt)){
			smls->actTriPart=relevantPart[k];
			return;
		}
	}
	smls->ActTri=0;
	smls->ActCtrlPnt=0;
	smls->actTriPart=0;
}
inline void setCoordIndexToPointerIndex(Art& part, Seamless& smls){
	if(part.index[0]!=smls.ActCtrlPnt||g_before!=g_selNode.getRaw()){
		part.index[0]=smls.ActCtrlPnt;
		if(part.workEffect!=0){
			int tri=smls.ActTri;
			int corner=smls.ActTriCorner;
			Art* triPart =smls.actTriPart;
			int ti=tri*3+corner;
			if(triPart!=0&&triPart->triangle.length>ti){
				part.workEffect->setColorIndex(triPart->triangle.refTexIndex(ti));
			}
		}
		if(Art* part=dynamic_cast<Art*>(g_stw->cp->node.p)){
			if(part->index[0]>=part->coord.refValue().length){
				part->index[0]=0;
			}
			g_stw->cp->paint();
		}
	}
}
void makePointerEqualIndexLocation(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	if(part==0)return;
	Seamless* smls=part->smls;
	const M_Vec3f& coord=part->coord;

	if(coord.length==0){
		smls->ActTriCorner=0;
		smls->ActTri=0;
		smls->ActCtrlPnt=0;
		smls->actTriPart=0;
		setCoordIndexToPointerIndex(*part,*smls);
		return;
	}
	if((unsigned)smls->ActCtrlPnt>=(unsigned)coord.length){
		smls->ActCtrlPnt=coord.length-1;
	}
	if(smls->ActCtrlPnt<0){
		smls->ActCtrlPnt=0;
		smls->ActTri=0;
		smls->ActTriCorner=0;
	}
	refreshActTriPart(part, smls);
	if(smls->actTriPart){
		refreshCorner(part, smls->ActCtrlPnt, smls->actTriPart->triangle[smls->ActTri], smls->ActTriCorner);
	}

	Matrix viewMatrix = g_scene->getViewMatrix();
	
	g_pointer.translation=part->matrix * viewMatrix * coord[smls->ActCtrlPnt];

	setCoordIndexToPointerIndex(*part,*smls);
	if(g_before!=g_selNode.getRaw()){
		setTitleName();
		g_stw->setSelNodeLine(part);
		g_stw->paint();
		g_stw->cp->paint();
	}
}

void Settings::writeINIFile(){

	String str="#VRML V2.0 utf8\n";
	str+="#generated by Seamless3d ";
	str+=g_version;
	str+='\n';

//	nodeWinPos=Vec2f(g_stw->cp->pos.x, g_stw->cp->pos.y);
//	sceneTreePos=Vec2f(g_stw->pos.x, g_stw->pos.y);
//	sceneTreeSize=Vec2f(g_stw->size.x, g_stw->size.y);
	toolBarPos=Vec2f(g_tb->pos.x, g_tb->pos.y);

	toString(str,true);
	g_openNetFile->toString(str, true);
	g_pasteOptions->toString(str, true);
	g_deleteOptions->toString(str, true);
	g_timelineFilter->toString(str, true);
	str.write(g_setupPath);
}

void buildSceneMatrices( ArtPtr part, Matrix catmat);

inline void genTransferBCylClue(){
	g_vcTransferBCyl = new Seamless;
	ColorEffect* colorEffect = new ColorEffect;
	colorEffect->transparency.refValue()=.5;
	g_vcTransferBCyl->effect=colorEffect;
	Part* part = new Part;
	g_vcTransferBCyl->skeleton.refValue()+=part;
	g_vcTransferBCylCylinderBuild->part.refValue()=part;
	g_vcTransferBCyl->build.refValue()+=g_vcTransferBCylCylinderBuild;
}
inline void genMultiSelClue(){
	Seamless* smls = new Seamless;
	ColorEffect* colorEffect = new ColorEffect;
	colorEffect->transparency.refValue()=.5;
	smls->effect=colorEffect;
	ImmediatePart* part = new ImmediatePart;
	smls->skeleton.refValue()+=part;
	smls->process();
	g_vcArtTriBase->transpSmls = smls;
}
inline void genOscClue(){
	g_vcOscSmls = new Seamless;
	ColorEffect* colorEffect = new ColorEffect;
	g_vcOscSmls->effect = colorEffect;
	Part* part = new Part;
	g_vcOscSmls->skeleton+=part;
	Stem* stem = new Stem;
	stem->part += part;
	//stem->slide=Vec2f(.5,0);
	stem->xCentred = false;
	stem->xDistance = 1;
	stem->sync = false;
	stem->staves = 64;
	stem->uWrap = false;
	stem->add = false;
	stem->store.refValue() = true;
	//stem->insideOut=true;
	NurbsPatch* nurbsPatch = new NurbsPatch;
	nurbsPatch->uJoinEnds = false;
	nurbsPatch->stem.refValue() += stem;
	g_vcOscSmls->build += nurbsPatch;
	g_vcOscBase->children += g_vcOscSmls.p;
	g_vcOscBase->children += g_vcBezierBase.p;

/////////
}
IndexedTriangleSet* convertToTriangleSet(Mesh& gs){
	gs.genNormals();
	IndexedTriangleSet* its = new IndexedTriangleSet;
	Coordinate* coord = new Coordinate;
	Color* color = new Color;
	Normal* normal = new Normal;
	its->coord = coord;
	its->color = color;
	its->normal = normal;
	coord->point = gs.coord;
	color->color = gs.color;
	normal->vector = gs.normal;
	its->index=gs.coordIndex;
	if(gs.texCoord.length > 0){
		TextureCoordinate* textureCoordinate = new TextureCoordinate;
		textureCoordinate->point = gs.texCoord;
		its->texCoord=textureCoordinate;
	}
	return its;
}
IndexedFaceSet* convertToFaceSet(MeshPtr gs){
	gs->genNormals();
	IndexedFaceSetPtr its=new IndexedFaceSet;
	Coordinate* coord=new Coordinate;
	Color* color=new Color;
	Normal* normal=new Normal;
	its->coord=coord;
	its->color=color;
	its->normal=normal;
	coord->point=gs->coord;
	color->color=gs->color;
	normal->vector=gs->normal;
	its->index=gs->coordIndex;
	if(gs->texCoord.length>0){
		TextureCoordinate* textureCoordinate=new TextureCoordinate;
		textureCoordinate->point=gs->texCoord;
		its->texCoord=textureCoordinate;
	}
	return its;
}

#define CURSOR_DIST_A .01
#define CURSOR_DIST_B .037

#define CURSOR_DIST_C 0.707106781 * CURSOR_DIST_A
#define CURSOR_DIST_D 0.707106781 * CURSOR_DIST_B

void createCursorOverPointIls(){
	Coordinate* cursorOverPointIlsCoord = new Coordinate;
	Color* cursorOverPointIlsColor = new Color;

	M_Vec3f& m_coord = cursorOverPointIlsCoord->point.refValue();
	M_Color3f& m_color = cursorOverPointIlsColor->color.refValue();

	M_Int& m_coordIndex = g_cursorOverPointIls->coordIndex.refValue();




	m_coord += Vec3f(CURSOR_DIST_A,0,0);
	m_coord += Vec3f(CURSOR_DIST_B,0,0);

	m_coord += Vec3f(0,CURSOR_DIST_A,0);
	m_coord += Vec3f(0,CURSOR_DIST_B,0);

	m_coord += Vec3f(-CURSOR_DIST_A,0,0);
	m_coord += Vec3f(-CURSOR_DIST_B,0,0);

	m_coord += Vec3f(0,-CURSOR_DIST_A,0);
	m_coord += Vec3f(0,-CURSOR_DIST_B,0);
	////////

	m_coord += Vec3f(CURSOR_DIST_C, CURSOR_DIST_C, 0);
	m_coord += Vec3f(CURSOR_DIST_D, CURSOR_DIST_D, 0);

	m_coord += Vec3f(-CURSOR_DIST_C, CURSOR_DIST_C, 0);
	m_coord += Vec3f(-CURSOR_DIST_D, CURSOR_DIST_D, 0);

	m_coord += Vec3f(-CURSOR_DIST_C, -CURSOR_DIST_C, 0);
	m_coord += Vec3f(-CURSOR_DIST_D, -CURSOR_DIST_D, 0);

	m_coord += Vec3f(CURSOR_DIST_C, -CURSOR_DIST_C, 0);
	m_coord += Vec3f(CURSOR_DIST_D, -CURSOR_DIST_D, 0);


	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);


	m_coordIndex += 0;
	m_coordIndex += 1;
	m_coordIndex += -1;

	m_coordIndex += 2;
	m_coordIndex += 3;
	m_coordIndex += -1;

	m_coordIndex += 4;
	m_coordIndex += 5;
	m_coordIndex += -1;

	m_coordIndex += 6;
	m_coordIndex += 7;
	m_coordIndex += -1;

	m_coordIndex += 8;
	m_coordIndex += 9;
	m_coordIndex += -1;

	m_coordIndex += 10;
	m_coordIndex += 11;
	m_coordIndex += -1;

	m_coordIndex += 12;
	m_coordIndex += 13;
	m_coordIndex += -1;

	m_coordIndex += 14;
	m_coordIndex += 15;
	m_coordIndex += -1;

	g_cursorOverPointIls->coord.refValue() = cursorOverPointIlsCoord;
	g_cursorOverPointIls->color.refValue() = cursorOverPointIlsColor;

	Appearance* appearance = new Appearance;
	appearance->material = new Material;

	g_cursorOverPointIlsShape->appearance.refValue() = appearance;
	g_cursorOverPointIlsShape->geometry.refValue() = g_cursorOverPointIls;
	g_cursorOverPointIlsBase->children.refValue() += g_cursorOverPointIlsShape.p;

	g_cursorOverPointIls->noZbuf = true;
}

/*

#define CURSOR_DIST_A .01
#define CURSOR_DIST_B .1
#define CURSOR_DIST_C .002
#define CURSOR_DIST_D CURSOR_DIST_A + CURSOR_DIST_C
#define CURSOR_DIST_E CURSOR_DIST_B + CURSOR_DIST_C

void createCursorOverPointIls(){

	Coordinate* cursorOverPointIlsCoord = new Coordinate;
	Color* cursorOverPointIlsColor = new Color;

	M_Vec3f& m_coord = cursorOverPointIlsCoord->point.refValue();
	M_Color3f& m_color = cursorOverPointIlsColor->color.refValue();

	M_Int& m_coordIndex = g_cursorOverPointIls->coordIndex.refValue();




	m_coord += Vec3f(CURSOR_DIST_A,0,0);
	m_coord += Vec3f(CURSOR_DIST_B,0,0);

	m_coord += Vec3f(0,CURSOR_DIST_A,0);
	m_coord += Vec3f(0,CURSOR_DIST_B,0);

	m_coord += Vec3f(-CURSOR_DIST_A,0,0);
	m_coord += Vec3f(-CURSOR_DIST_B,0,0);

	m_coord += Vec3f(0,-CURSOR_DIST_A,0);
	m_coord += Vec3f(0,-CURSOR_DIST_B,0);


	m_coord += Vec3f(CURSOR_DIST_A, CURSOR_DIST_A,0);
	m_coord += Vec3f(-CURSOR_DIST_A, CURSOR_DIST_A,0);
	m_coord += Vec3f(-CURSOR_DIST_A, -CURSOR_DIST_A,0);
	m_coord += Vec3f(CURSOR_DIST_A, -CURSOR_DIST_A,0);


	m_coord += Vec3f(CURSOR_DIST_A, -CURSOR_DIST_C, 0);
	m_coord += Vec3f(CURSOR_DIST_B, -CURSOR_DIST_C, 0);

	m_coord += Vec3f(CURSOR_DIST_A, CURSOR_DIST_C, 0);
	m_coord += Vec3f(CURSOR_DIST_B, CURSOR_DIST_C, 0);

	m_coord += Vec3f(-CURSOR_DIST_C, CURSOR_DIST_A, 0);
	m_coord += Vec3f(-CURSOR_DIST_C, CURSOR_DIST_B, 0);

	m_coord += Vec3f(CURSOR_DIST_C, CURSOR_DIST_A, 0);
	m_coord += Vec3f(CURSOR_DIST_C, CURSOR_DIST_B, 0);

	m_coord += Vec3f(-CURSOR_DIST_A, -CURSOR_DIST_C, 0);
	m_coord += Vec3f(-CURSOR_DIST_B, -CURSOR_DIST_C, 0);

	m_coord += Vec3f(-CURSOR_DIST_A, CURSOR_DIST_C, 0);
	m_coord += Vec3f(-CURSOR_DIST_B, CURSOR_DIST_C, 0);

	m_coord += Vec3f(-CURSOR_DIST_C, -CURSOR_DIST_A, 0);
	m_coord += Vec3f(-CURSOR_DIST_C, -CURSOR_DIST_B, 0);

	m_coord += Vec3f(CURSOR_DIST_C, -CURSOR_DIST_A, 0);
	m_coord += Vec3f(CURSOR_DIST_C, -CURSOR_DIST_B, 0);

	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);

	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);
	m_color += Color3f(1, 0, 0);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_color += Color3f(1, 1, 1);
	m_color += Color3f(1, 1, 1);

	m_coordIndex += 0;
	m_coordIndex += 1;
	m_coordIndex += -1;

	m_coordIndex += 2;
	m_coordIndex += 3;
	m_coordIndex += -1;

	m_coordIndex += 4;
	m_coordIndex += 5;
	m_coordIndex += -1;

	m_coordIndex += 6;
	m_coordIndex += 7;
	m_coordIndex += -1;

	m_coordIndex += 8;
	m_coordIndex += 9;
	m_coordIndex += 10;
	m_coordIndex += 11;
	m_coordIndex += 8;
	m_coordIndex += -1;

	m_coordIndex += 12;
	m_coordIndex += 13;
	m_coordIndex += -1;

	m_coordIndex += 14;
	m_coordIndex += 15;
	m_coordIndex += -1;

	m_coordIndex += 16;
	m_coordIndex += 17;
	m_coordIndex += -1;

	m_coordIndex += 18;
	m_coordIndex += 19;
	m_coordIndex += -1;

	m_coordIndex += 20;
	m_coordIndex += 21;
	m_coordIndex += -1;

	m_coordIndex += 22;
	m_coordIndex += 23;
	m_coordIndex += -1;

	m_coordIndex += 24;
	m_coordIndex += 25;
	m_coordIndex += -1;

	m_coordIndex += 26;
	m_coordIndex += 27;
	m_coordIndex += -1;

	g_cursorOverPointIls->coord.refValue() = cursorOverPointIlsCoord;
	g_cursorOverPointIls->color.refValue() = cursorOverPointIlsColor;

	Appearance* appearance = new Appearance;
	appearance->material = new Material;

	g_cursorOverPointIlsShape->appearance.refValue() = appearance;
	g_cursorOverPointIlsShape->geometry.refValue() = g_cursorOverPointIls;
	g_cursorOverPointIlsBase->children.refValue() += g_cursorOverPointIlsShape;

	g_cursorOverPointIls->noZbuf = true;
}
*/


void initVisualClue(){
//do pointer stuff
	g_vcArtTriBase = new PartClue;

	genSphere(g_mouseSense * 1.5, 8, 8, *g_redPoint, new ColorEffect);
	genSphere(g_mouseSense * 1.5, 8, 8, *g_nextPoint, new ColorEffect);
	genSphere(g_mouseSense * 1.5, 8, 8, *g_lastPoint, new ColorEffect);
	genSphere(g_mouseSense * 1.5, 8, 8, *g_yellowPoint, new ColorEffect);
	genSphere(g_mouseSense * 1.5, 8, 8, *g_cyanPoint, new ColorEffect);

	g_redPoint->genColorList(Color3f(1,0,0));
	g_lastPoint->genColorList(Color3f(0,0,1));
	g_nextPoint->genColorList(Color3f(0,1,0));
	g_yellowPoint->genColorList(Color3f(1,1,0));
	g_cyanPoint->genColorList(Color3f(0,1,1));

	genCone(g_mouseSense * 5 * .8, g_mouseSense * 2 * .8, true, true, 8, *g_redConePoint, new ColorEffect);
	genCone(g_mouseSense * 5 * .8, g_mouseSense * 2 * .8, true, true, 8, *g_greenConePoint, new ColorEffect);
	genCone(g_mouseSense * 5 * .8, g_mouseSense * 2 * .8, true, true, 8, *g_blueConePoint, new ColorEffect);

	g_vcCommonAppearance->material = new Material;

	g_redPoint->appearance = g_vcCommonAppearance;
	g_lastPoint->appearance = g_vcCommonAppearance;
	g_nextPoint->appearance = g_vcCommonAppearance;
	g_yellowPoint->appearance = g_vcCommonAppearance;
	g_cyanPoint->appearance = g_vcCommonAppearance;

	g_redConePoint->appearance = g_vcCommonAppearance;
	g_blueConePoint->appearance = g_vcCommonAppearance;
	g_greenConePoint->appearance = g_vcCommonAppearance;

	g_redConeBase->children.refValue() += g_redConePoint.p;
	g_greenConeBase->children.refValue() += g_greenConePoint.p;
	g_blueConeBase->children.refValue() += g_blueConePoint.p;

	g_redConeBase->rotation = Rotation(0, 0, 1, -PI * .5);
	g_blueConeBase->rotation = Rotation(1, 0, 0, PI * .5);
	
	genTransferBCylClue();
	genMultiSelClue();
	genOscClue();


	{
		g_triClue->coord.refValue() = g_triClueCoord;
		g_triClue->color.refValue() = g_triClueColor;
		Appearance* appearance = new Appearance;
		appearance->material = new Material;
		g_triClueShape->appearance.refValue() = appearance;
		g_triClueShape->geometry.refValue() = g_triClue;
		g_triClueCoord->point.refValue().length = 6;
		g_triClueCoord->point.refValue() = M_Vec3f(Vec3f(-1, 0, 0), Vec3f(0, 1, 0), Vec3f(0, 1, 0), Vec3f(1, 0, 0), Vec3f(1, 0, 0), Vec3f(-1, 0, 0));
		g_triClueColor->color.refValue() = M_Color3f(Color3f(1, 0, 0), Color3f(1, 0, 0), Color3f(0, 1, 0), Color3f(0, 1, 0), Color3f(0, 0, 1), Color3f(0, 0, 1));

		g_triClue->coordIndex.refValue() = M_Int(0, 1, -1, 2, 3, -1, 4, 5);
		g_triClue->noZbuf = true;
	}

	createCursorOverPointIls();

	//g_nurbsIls->noZbuf = (g_scene->mode.refValue() & ZBUFFER_CAGE_MODE);
	g_nurbsIls->coord.refValue() = g_nurbsIlsCoord;
	g_nurbsIls->color.refValue() = g_nurbsIlsColor;
	g_nurbsIlsShape->appearance.refValue() = new Appearance;
	g_nurbsIlsShape->geometry.refValue() = g_nurbsIls;
	g_nurbsIlsBase->children.refValue() += g_nurbsIlsShape.p;

	g_gridShape->geometry.refValue() = new IndexedLineSet;
	Appearance* appearence = new Appearance;
	Material* material = new Material;
	//material->diffuseColor = Color3f(.5,.5,.5);
	appearence->material.refValue() = appearence->material.refValue() = material;
	g_gridShape->appearance.refValue() = appearence;
	g_gridIls->coord.refValue() =  g_gridCoord;
	g_gridIls->color.refValue() =  g_gridColor;
	g_gridShape->geometry.refValue() = g_gridIls;

	ColorEffect* colorEffect = new ColorEffect;
	colorEffect->transparency = .5;
	g_vcNurbsCtrlPoly->effect = colorEffect;
	g_vcNurbsCtrlPolyPart = new Part;
	g_vcNurbsCtrlPoly->skeleton += g_vcNurbsCtrlPolyPart;
	g_vcNurbsCtrlPoly->ignoreWireFrame.refValue() = true;

	//g_visualClue=g_vcSeamless;

	g_workPolyMapSmlsColor->effect = new ColorEffect;;
	g_workPolyMapSmlsTexture->effect = new ColorEffect;
	g_workPolyMapColor->mirror.refValue() = false;
	g_workPolyMapTexture->mirror.refValue() = false;
	g_workPolyMapSmlsColor->skeleton += g_workPolyMapColor;
	g_workPolyMapSmlsTexture->skeleton += g_workPolyMapTexture;
	g_workPolyMapSmlsColor->process();
	g_workPolyMapSmlsTexture->process();
	g_workPolyMapSmlsColor->resetRelevantParts();
	g_workPolyMapSmlsTexture->resetRelevantParts();


	g_vcSeamless->effect = new ColorEffect;
	g_vcPart = new ImmediatePart;
	g_vcSeamless->skeleton += g_vcPart;
	g_vcStem = new Stem;
	g_vcStem->sync = true;
	g_vcStem->sync.refValue()=true;
	g_vcSurface = new SurfaceGenerator;
	g_vcCCLathe = new CCLathe;
	g_vcSurfaceTransform = new SurfaceTransform;
	g_vcCCBend = new CCBend;
	g_vcCCBend->location=Vec3f(0, -.0000001, 0);
	g_vcBaseRad = new SurfaceTransform;

	M_Build build;
	build += g_vcStem.p;
	g_vcSeamless->build.refValue() += g_vcStem.p;

	g_vcStem->part = g_vcPart;
	M_Node& generator = g_vcSurface->generator.refValue();
	generator += g_vcCCLathe.p;
	generator += g_vcSurfaceTransform.p;
	generator += g_vcCCBend.p;
	generator += g_vcBaseRad.p;


	g_vcStem->surface = g_vcSurface;
	g_vcStem->staves.refValue() += 6;
	g_vcStem->bands.refValue() += 16;

	g_vcCCLathe->distance.refValue() += 1;
	g_vcCCLathe->curve.refValue() += -.00001;
	g_vcCCLathe->curve.refValue() += .00001;
	g_vcCCLathe->baseRad.refValue() = g_spaghettiRad;

	g_vcSeamless->process();



	IndexedLineSet* ils = new IndexedLineSet;
	ils->coord = new Coordinate;
	g_vcEllipticalPolyline2D->geometry = ils;
	IndexedLineSet* ils2 = new IndexedLineSet;
	ils2->coord = new Coordinate;
	g_vcFuseSurfaceFlatStageAidShape->geometry = ils2;
	IndexedLineSet* ils3 = new IndexedLineSet;
	ils3->coord = new Coordinate;
	g_transformLeverClue->geometry = ils3;
	IndexedLineSet* ils4 = new IndexedLineSet;
	ils4->coord = new Coordinate;
	ils4->color = new Color;
	g_translateDimensionClue->geometry = ils4;


	IndexedLineSet* ils5 = new IndexedLineSet;
	ils5->coord = new Coordinate;
	ils5->color = new Color;
	g_rawDrawNurbsLineClue->geometry = ils5;


	g_vcCCLatheBase = new Transform;
	g_vcRingBuildBase = new Transform;
	g_vcRingBuildPivotPoint = new Transform;
	g_vcRingBuildOrientationA = new Transform;
	g_vcRingBuildOrientationB = new Transform;
	g_vcRingBuildOrientationC = new Transform;
	g_vcRingBuildOrientationD = new Transform;
	g_vcArtAnimBase = new AnimLever;

	
	g_vcRingBuildBase->children.refValue() += g_vcRingBuildPivotPoint.p;
	g_vcRingBuildPivotPoint->children.refValue() += g_yellowPoint.p;
	g_vcRingBuildBase->children.refValue() += g_vcRingBuildOrientationA.p;
	g_vcRingBuildBase->children.refValue() += g_vcRingBuildOrientationB.p;
	g_vcRingBuildBase->children.refValue() += g_vcRingBuildOrientationC.p;
	g_vcRingBuildBase->children.refValue() += g_vcRingBuildOrientationD.p;
	g_vcRingBuildOrientationA->children.refValue() += g_redPoint.p;
	g_vcRingBuildOrientationB->children.refValue() += g_redPoint.p;
	g_vcRingBuildOrientationC->children.refValue() += g_redPoint.p;
	g_vcRingBuildOrientationD->children.refValue() += g_redPoint.p;

	AppearancePtr g_vcCommonAppearance = new Appearance;
	g_vcCommonAppearance->material = new Material;

	g_vcSelectRectMaterial->emissiveColor = Color3f(1,1,0);
	g_vcSelectRectMaterial->diffuseColor = Color3f(1,1,0);
	Appearance* selectRectAppearance = new Appearance;
	selectRectAppearance->material = g_vcSelectRectMaterial;
	g_vcSelectRectShape->appearance = selectRectAppearance;
	g_transformLeverClue->appearance = selectRectAppearance;
	g_translateDimensionClue->appearance = selectRectAppearance;
	g_rawDrawNurbsLineClue->appearance = selectRectAppearance;

	{
		IndexedLineSet* ils = new IndexedLineSet;
		ils->noZbuf = true;
		ils->coord=new Coordinate;
		g_vcSelectRectShape->geometry=ils;
	}


	g_stickManMaterial->emissiveColor = Color3f(0,1,0);
	g_stickManMaterial->diffuseColor = Color3f(0,1,0);
	g_vcEllipticalMaterial->emissiveColor = Color3f(0,1,1);
	g_vcEllipticalMaterial->diffuseColor = Color3f(0,1,1);
	Appearance* appearance =new Appearance;
	appearance->material = g_vcEllipticalMaterial;
	g_vcEllipticalPolyline2D->appearance = appearance;
	g_vcFuseSurfaceFlatStageAidShape->appearance = appearance;
	g_rawDrawNurbsLineClueBase->children.refValue()+=g_rawDrawNurbsLineClue.p;
	g_vcECScaleBase->children.refValue()+=g_vcEllipticalPolyline2D.p;
	g_vcBezierBase->children.refValue()+=g_vcEllipticalPolyline2D.p;
	g_vcCCLatheBase->children.refValue()+=g_vcEllipticalPolyline2D.p;
	g_vcTransferBCylBase->children.refValue()+=g_vcEllipticalPolyline2D.p;
	g_vcTransferBCylBase->children.refValue()+=g_vcTransferBCyl.p;

	g_vcECScaleTangentArrow = new Transform;
	Shape* shape = new Shape;
	shape->appearance = appearance;
	IndexedLineSet* polyline2D = new IndexedLineSet;
	Coordinate* polyline2DCoord = new Coordinate;
	polyline2D->coord = polyline2DCoord;
	polyline2DCoord->point=M_Vec3f(Vec3f(-halfArrowlen,0,0),Vec3f(halfArrowlen,0,0));
	shape->geometry=polyline2D;
	g_vcECScaleTangentArrow->children.refValue()+=shape;

	Transform* transform=new Transform;
	Appearance* sharedAppearance=new Appearance;
	shape->appearance=sharedAppearance;
	transform->translation=Vec3f(-halfArrowlen,0,0);
	transform->children.refValue()+=g_lastPoint.p;
	g_vcECScaleTangentArrow->children.refValue() += transform;


	shape = new Shape;
	appearance = new Appearance;
	material = new Material;
	material->diffuseColor = Color3f(1,0,0);
	appearance->material = material;
	shape->appearance = appearance;
	Cone* cone = new Cone;
	shape->geometry = cone;
	cone->bottomRadius = g_mouseSense * 3;
	cone->height = g_mouseSense * 5;
	transform = new Transform;
	transform->translation = Vec3f(halfArrowlen, 0, 0);
	transform->rotation = Rotation(0, 0, 1, -PI*.5);
	transform->children.refValue() += shape;
	g_vcECScaleTangentArrow->children.refValue() += transform;

	g_vcECScaleBase->children.refValue() += g_vcECScaleTangentArrow.p;

	g_insertArrow = new Transform;
	g_insertArrow->children.refValue() += shape;


	g_nailDown += g_visualClue;
	g_nailDown += g_vcEllipticalMaterial;
	g_nailDown += g_stickManMaterial;
	g_nailDown += g_vcSelectRectMaterial;
	g_nailDown += g_vcSelectRectGroup;
}
void initRadialTug(){
	g_radialTugStem=new Stem;
	g_radialTugStem->sync=true;
	g_radialTugStem->part.resize(1);
	SurfaceGenerator* surface=new SurfaceGenerator;
	g_radialTugStem->surface=surface;

	g_radialTug=new RadialTug;
	g_radialTug->radius=0;
	g_radialTug->location.refValue().length=1;
	g_radialTug->displacement.refValue().length=1;
	g_radialTug->slope.refValue().length=1;
	g_radialTug->name="immediate";
	surface->generator.refValue()+=g_radialTug;
}
/*
void pointToPartByDefault(){
	g_selNode=g_scene;
	if(!g_scene->children.length)return;
	if(g_scene->children[0]->getType()!=SEAMLESS)return;
	SeamlessPtr smls=g_scene->children[0];
	if(!smls->skeleton.length)return;
	g_selNode=smls->skeleton[0];
}
*/
void genCylinder(float height, float radius, bool side, bool top, bool bottom, int staves, int bands, Mesh& geomSource , Effect* effect){
	Seamless smls;
	smls.effect = effect;
	Art* part = new Part;
	smls.skeleton+=part;
	M_Vec3f& coord = part->coord;
	if(side){
		CCLathe* lathe=new CCLathe;
		SurfaceGenerator* surface=new SurfaceGenerator;
		surface->generator.refValue()+=lathe;
		SurfaceTransform* transform = new SurfaceTransform;
		surface->generator.refValue()+=transform;
		transform->translation=Vec3f(0,-height*.5,0);

		Stem stem;
		stem.sync.refValue()=true;
		stem.surface=surface;
		stem.part.refValue()+=part;
		stem.staves.refValue()=staves;
		stem.bands=bands;
		effect->addTextureMapper(stem);

		lathe->curve.refValue()+=0;
		lathe->curve.refValue()+=-.0001;
		lathe->distance.refValue()+=height;
		lathe->baseRad=radius;

		stem.doBuild();
		JoinCloseVertex joinCloseVertex;
		joinCloseVertex.part+=part;
		joinCloseVertex.doBuild();
	}
	if(top){
		Art* topCap = new Part;
		smls.skeleton+=topCap;
		CCLathe* lathe=new CCLathe;
		SurfaceGenerator* surface=new SurfaceGenerator;
		surface->generator.refValue()+=lathe;
		SurfaceTransform* transform = new SurfaceTransform;
		surface->generator.refValue()+=transform;
		transform->translation=Vec3f(0,height*.5,0);

		Stem stem;
		stem.sync.refValue()=true;
		stem.surface=surface;
		stem.part.refValue()+=topCap;
		stem.staves.refValue()=staves;
		stem.bands=bands;
		stem.beginSP=true;
		stem.insideOut=true;

		lathe->curve.refValue()+=PI*.5;
		lathe->curve.refValue()+=-.0001;
		lathe->distance.refValue()+=radius;

		stem.doBuild();
		JoinCloseVertex joinCloseVertex;
		joinCloseVertex.part+=topCap;
		joinCloseVertex.doBuild();

		if(side){

			M_Vec3f& coordTopCap=topCap->coord;

			for(int i=staves; --i>=0;){
				coordTopCap[i+1]=coord[staves+i];
			}
		}
		effect->textureMapDisk(topCap, radius, -PI*.5);

		TransferVertex transferVertex;
		transferVertex.source=topCap;
		transferVertex.dest=part;
		transferVertex.doBuild();
	}
	if(bottom){
		Art* base = new Part;
		smls.skeleton+=base;
		CCLathe* lathe=new CCLathe;
		SurfaceGenerator* surface=new SurfaceGenerator;
		surface->generator.refValue()+=lathe;
		SurfaceTransform* transform = new SurfaceTransform;
		surface->generator.refValue()+=transform;
		transform->translation=Vec3f(0,-height*.5,0);

		Stem stem;
		stem.sync.refValue()=true;
		stem.surface=surface;
		stem.part.refValue()+=base;
		stem.staves.refValue()=staves;
		stem.bands=bands;
		stem.beginSP=true;
		effect->addTextureMapper(stem);

		lathe->curve.refValue()+=PI*.5;
		lathe->curve.refValue()+=-.0001;
		lathe->distance.refValue()+=radius;

		TransferVertex transferVertex;
		transferVertex.source=base;
		transferVertex.dest=part;

		stem.doBuild();
		JoinCloseVertex joinCloseVertex;
		joinCloseVertex.part+=base;
		joinCloseVertex.doBuild();

		if(side){

			M_Vec3f& coordBase=base->coord;

			for(int i=staves; --i>=0;){
				coordBase[i+1]=coord[i];
			}
		}
		effect->textureMapDisk(base, radius, PI*.5);

		transferVertex.doBuild();
	}
	part->extractGeometryGS(geomSource, g_tmpIndex);
}
void genSphere(float rad, int staves, int bands, Mesh& geomSource, Effect* effect){
	Seamless smls;
	smls.effect=effect;
	Art* part = new Part;
	smls.skeleton+=part;
	CCLathe* lathe=new CCLathe;
	SurfaceGenerator* surface=new SurfaceGenerator;
	surface->generator.refValue()+=lathe;
	SurfaceTransform* transform = new SurfaceTransform;
	surface->generator.refValue()+=transform;

	transform->translation=Vec3f(0,-rad,0);

	Stem stem;
	stem.sync=true;
	stem.surface=surface;
	stem.beginSP=true;
	stem.endSP=true;
	stem.uWrap = true;
	stem.xDistance.refValue() + 2 * PI;
	stem.part+=part;
	stem.staves=staves;
	stem.bands=bands;
	effect->addTextureMapper(stem);

	lathe->curve.refValue()+=-PI*.5;
	lathe->curve.refValue()+=PI;
	lathe->distance.refValue()+=rad*PI;

	stem.doBuild();
	//JoinCloseVertex joinCloseVertex;
	//joinCloseVertex.part+=part;
	//joinCloseVertex.doBuild();

	part->extractGeometryGS(geomSource, g_tmpIndex);
}
void genCone(float height, float radius, bool side, bool bottom, int staves, Mesh& geomSource, Effect* effect){
	Seamless smls;
	smls.effect = effect;
	Art* part = new Part;
	smls.skeleton+=part;
	M_Vec3f& coord=part->coord;
	if(side){
		CCLathe* lathe=new CCLathe;
		SurfaceGenerator* surface=new SurfaceGenerator;
		surface->generator.refValue()+=lathe;
		SurfaceTransform* transform = new SurfaceTransform;
		surface->generator.refValue()+=transform;
		transform->translation=Vec3f(0,-height*.5,0);

		Stem stem;
		stem.sync.refValue()=true;
		stem.surface=surface;
		stem.part.refValue()+=part;
		stem.staves.refValue()=staves;
		stem.bands=1;
		stem.uWrap=false;
		effect->addTextureMapper(stem);

		DelPoly delPoly;
		delPoly.part=part;
		M_Int& poly=delPoly.poly;
		poly.length=staves;
		for(int i=0; i<staves; i++){
			poly[i]=i*2+1;
		}

		float hyp=sqrt(height*height+radius*radius);
		float angle;
		if(height>radius){
			angle=PI*.5-asin(height/hyp);
		}
		else{
			angle=PI*.5-acos(radius/hyp);
		}
		lathe->curve.refValue()+=angle;
		lathe->curve.refValue()+=-.0001;
		lathe->distance.refValue()+=hyp;
		lathe->baseRad=radius;
		stem.doBuild();
		delPoly.doBuild();

		JoinVertex joinVertex;
		joinVertex.part=part;
		joinVertex.srcCoord.refValue()+=2;
		joinVertex.dstCoord.refValue()+=3*(staves-1);
		joinVertex.doBuild();

		for( int i=staves; --i>=0;){
			coord[staves+i]=Vec3f(0,height*.5,0);
		}
	}
	if(bottom){
		Art* base = new Part;
		smls.skeleton+=base;
		CCLathe* lathe=new CCLathe;
		SurfaceGenerator* surface=new SurfaceGenerator;
		surface->generator.refValue()+=lathe;
		SurfaceTransform* transform = new SurfaceTransform;
		surface->generator.refValue()+=transform;
		transform->translation=Vec3f(0,-height*.5,0);

		Stem stem;
		stem.sync.refValue()=true;
		stem.surface=surface;
		stem.part.refValue()+=base;
		stem.staves.refValue()=staves;
		stem.bands=1;
		stem.beginSP=true;
		effect->addTextureMapper(stem);

		lathe->curve.refValue()+=PI*.5;
		lathe->curve.refValue()+=-.0001;
		lathe->distance.refValue()+=radius;

		TransferVertex transferVertex;
		transferVertex.source=base;
		transferVertex.dest=part;

		stem.doBuild();
		JoinCloseVertex joinCloseVertex;
		joinCloseVertex.part+=base;
		joinCloseVertex.doBuild();

		if(side){

			M_Vec3f& coordBase=base->coord;

			for(int i=staves; --i>=0;){
				coordBase[i+1]=coord[i];
			}
		}
		effect->textureMapDisk(base, radius, PI*.5);

		transferVertex.doBuild();
	}
	part->extractGeometryGS(geomSource, g_tmpIndex);
}
void genBox(Vec3f size, int staves, int bands, Mesh& geomSource, Effect* effect){
	Seamless smls;
	smls.effect = effect;
	Art* part = new Part;
	Art* part2 = new Part;
	smls.skeleton+=part;
	smls.skeleton+=part2;
	smls.process();
	smls.doBuild();

	Stem stem;
	M_Vec3f& coord=part->coord;
	effect->addTextureMapper(stem);
	TextureEffect* e=dynamic_cast<TextureEffect*>(effect);
	M_Vec2f& texCoord=e->texCoord;
	stem.part.refValue()+=part2;
	stem.staves.refValue()=staves;
	stem.bands.refValue()=bands;
	stem.xDistance=1;
	stem.yDistance=1;
	stem.uWrap=false;
	stem.doBuild();
	Place place;
	place.part+=part2;
	place.translation=Vec3f(0,-.5,.5);
	place.doBuild();
	CopyPart copyPart;
	copyPart.dest+=part;
	copyPart.source+=part2;
	copyPart.doBuild();
	copyPart.mirrorX=true;
	copyPart.mirrorZ=true;
	copyPart.add=true;
	copyPart.doBuild();
	copyPart.mirrorX=false;
	copyPart.mirrorZ=false;
	place.translation=Vec3f(0,0,0);
	place.rotation=Rotation(0,1,0,PI*.5);
	place.doBuild();
	copyPart.doBuild();
	copyPart.mirrorX=true;
	copyPart.mirrorZ=true;
	copyPart.doBuild();
	place.rotation=Rotation(0,1,0,-PI*.5)*Rotation(1,0,0,-PI*.5);
	place.doBuild();
	copyPart.mirrorX=false;
	copyPart.mirrorZ=false;
	copyPart.doBuild();
	copyPart.mirrorY=true;
	copyPart.mirrorZ=true;
	copyPart.doBuild();
	place.part.refValue()[0]=part;
	place.scale=size;
	place.rotation=Rotation(0,0,1,0);
	place.doBuild();

	part->extractGeometryGS(geomSource, g_tmpIndex);
}
void Cylinder::convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const{
	geomSource.reset();
	genCylinder( height, radius, side, top, bottom, 12, 1, geomSource,workOutEffectType(appearance));
}
void Sphere::convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const{
	geomSource.reset();
	genSphere(radius, 12, 6, geomSource,workOutEffectType(appearance));
}
void Box::convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const{
	geomSource.reset();
	genBox( size, 1, 1, geomSource, workOutEffectType(appearance));
}
void Cone::convertToGeomSource(Mesh& geomSource, const Appearance*& appearance, M_Int& maskIndex)const{
	geomSource.reset();
	genCone(height, bottomRadius, side, bottom, 16, geomSource,workOutEffectType(appearance));
}

/*
void workOutActTriFromActPoint(){
	ArtPtr pj=g_selNode;
	SeamlessPtr smls=pj->smls;
	M_Int& coordIndex=smls->refCoordIndex();

	for(int t=0;t<coordIndex.length;t+=4){
		if(coordIndex[t]==smls->ActCtrlPnt){
			smls->ActTri=t/4;
			smls->ActTriCorner=t=0;
			break;
		}
		if(coordIndex[t+1]==smls->ActCtrlPnt){
			smls->ActTri=t/4;
			smls->ActTriCorner=t=1;
			break;
		}
		if(coordIndex[t+2]==smls->ActCtrlPnt){
			smls->ActTri=t/4;
			smls->ActTriCorner=t=2;
			break;
		}
	}
}
*/
bool backFace(const Vec3f& v0, const Vec3f& v1, const Vec3f& v2){
Vec3f fNormal;
	calcNormal(
		v0,
		v1,
		v2,
		fNormal
	);
	return fNormal.z < 0;
}
bool backFace(const Vec2f& v0, const Vec2f& v1, const Vec2f& v2){
Vec3f fNormal;
	calcNormal(
		Vec3f(v0.x, v0.y, 0),
		Vec3f(v1.x, v1.y, 0),
		Vec3f(v2.x, v2.y, 0),
		fNormal
	);
	return fNormal.z < 0;
}
bool backFace(Vec2f* v){
Vec3f fNormal;
	calcNormal(
		Vec3f(v[0].x, v[0].y, 0),
		Vec3f(v[1].x, v[1].y, 0),
		Vec3f(v[2].x, v[2].y, 0),
		fNormal
	);
	return fNormal.z < 0;
}

int backFace3(Vec3f *p){
	Vec3f v0=p[0];
	Vec3f v1=p[1];
	Vec3f v2=p[2];
	float x1=v0.x;
	float x2=v1.x;
	float x3=v2.x;

	float y1=v0.y;
	float y2=v1.y;
	float y3=v2.y;

	float z1=v0.z;
	float z2=v1.z;
	float z3=v2.z;

	float c=	(x3*((z1*y2)-(y1*z2)))+
				(y3*((x1*z2)-(z1*x2)))+
				(z3*((y1*x2)-(x1*y2)));

	return (c<0);
}

int backFace2(Vec3f& v0, Vec3f& v1, Vec3f& v2){
float c=(v2.x*((v0.z*v1.y)-(v0.y*v1.z)))+
		(v2.y*((v0.x*v1.z)-(v0.z*v1.x)))+
		(v2.z*((v0.y*v1.x)-(v0.x*v1.y)));
	return (c<0);
}


bool checkOrientationPoint(const bool perspective, const Vec2f point, const float tangent, Vec3f& pt, bool& tugFlag, Matrix &partMatrix){
	Vec3f sr;
//	if(point.length==0)return false;
	Vec2f spoint=Vec2f(-halfArrowlen*g_vcECScaleTangentArrow->scale.x,0);
	rotateZ(spoint, tangent);
	spoint+=point;



	sr= partMatrix * Vec3f(spoint.x, spoint.y, 0);

	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x=pt.x/(-pt.z + g_viewDist);
		mv.y=pt.y/(-pt.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=pt.x;
		mv.y=pt.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		pt.x=sr.x;
		pt.y=sr.y;
		pt.z=sr.z;

		if(false == controlKeyState()){
			tugFlag=true;
			showCursor(false);
		}
		orientationTugFlag=true;

		return true;
	}
	else return false;
}
bool RingBuild::getGoodQuadrant(Matrix& matrix){
	int i = index[0];
	matrix = rotationMatrix(orientation.refValue()[i]) * translationMatrix(location.refValue()[i]) * 
		scaleMatrix(scale.refValue()) * rotationMatrix(rotation.refValue()) * translationMatrix(translation.refValue());

	g_vcRingBuildPivotPoint->translation=matrix * Vec3f();
	Vec3f a = matrix * g_scene->getViewMatrix() * Vec3f(scaleX.refValue()[i].y,0,0);
	Vec3f b = matrix * g_scene->getViewMatrix() * Vec3f(0,0,scaleZ.refValue()[i].y);
	Vec3f screenPivotPoint = matrix * g_scene->getViewMatrix() * Vec3f();
	Vec2f aDiff = Vec2f(a.x - screenPivotPoint.x, a.y - screenPivotPoint.y);
	Vec2f bDiff = Vec2f(b.x - screenPivotPoint.x, b.y - screenPivotPoint.y);
	float distanceA = sqrt(aDiff.x*aDiff.x + aDiff.y*aDiff.y);
	float distanceB = sqrt(bDiff.x*bDiff.x + bDiff.y*bDiff.y);
	return distanceA > distanceB;
}
inline bool BendyLathe::checkPoint(const bool& perspective, int& idx, const Matrix& matrix){
	M_Vec3f& dpoint = patch->dpoint;
	int numOfU = patch->dim.x;
	int numOfV = patch->dim.y;
	Vec3f selPoint = ringBase[idx];

	Vec3f sr = matrix * selPoint;
	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

		g_pointer.translation=sr;

		if(false == controlKeyState()){
			tugFlag=true;
			showCursor(false);
		}
		radIdx = -1;

		return true;
	}
	double a = PI/ctrlStaves;
	for(int q = 0, c = 1; q < ctrlStaves; q++, c+=2){
		double b = a*c;
		sr = matrix * (ringRot[idx] * Vec3f(-sin(b)*radius[idx]*ratio, 0, -cos(b)*radius[idx]*ratio) + selPoint);
		
		Vec3f v;
		Vec3f mv;

		if(perspective){
			v.x = sr.x / (-sr.z + g_viewDist);
			v.y = sr.y / (-sr.z + g_viewDist);

			mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
			mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
		}
		else{
			v.x=sr.x;
			v.y=sr.y;

			mv.x=g_pointer.translation.x;
			mv.y=g_pointer.translation.y;
		}
		if(
			v.x < mv.x+g_mouseSense&&
			v.x > mv.x-g_mouseSense&&
			v.y < mv.y+g_mouseSense&&
			v.y > mv.y-g_mouseSense){

			g_pointer.translation=sr;
//			String str;
//			sr.toString(str);
//			dLog(str);

			if(false == controlKeyState()){
				tugFlag=true;
				showCursor(false);
			}
			radIdx=q;
			dLog(radIdx);

			return true;
		}
	}
	return false;
}
inline bool RingBuild::checkPoint(const bool& perspective, int& idx, const int& i, const Matrix& baseMatrix){
	Vec3f sr;
	if(scaleX.length==0)return false;

	Matrix matrix = rotationMatrix(orientation.refValue()[i]) * translationMatrix(location.refValue()[i]) * baseMatrix;

	for(int q = 5; --q>=0;){
		switch(q){
			case 0:
				sr= matrix * Vec3f(0, 0, -scaleZ[i].x);
				break;
			case 1:
				sr= matrix * Vec3f(-scaleX[i].x, 0, 0);
				break;
			case 2:
				sr= matrix * Vec3f(0, 0, scaleZ[i].y);
				break;
			case 3:
				sr= matrix * Vec3f(scaleX[i].y, 0, 0);
				break;
			case 4:
				sr= matrix * Vec3f(0, 0, 0);
				break;
		}

		Vec3f v;
		Vec3f mv;

		if(perspective){
			v.x = sr.x / (-sr.z + g_viewDist);
			v.y = sr.y / (-sr.z + g_viewDist);

			mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
			mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
		}
		else{
			v.x=sr.x;
			v.y=sr.y;

			mv.x=g_pointer.translation.x;
			mv.y=g_pointer.translation.y;
		}
		
		if(
			v.x < mv.x+g_mouseSense&&
			v.x > mv.x-g_mouseSense&&
			v.y < mv.y+g_mouseSense&&
			v.y > mv.y-g_mouseSense){


			{/*
				Matrix matrix;
				if(getGoodQuadrant(matrix)){
					if(q == 0 || q == 2){
						return false;
					}
				}
				else{
					if(q == 1 || q == 3){
						return false;
					}
				}*/
			}

			g_pointer.translation.x=sr.x;
			g_pointer.translation.y=sr.y;
			g_pointer.translation.z=sr.z;

			if(false == controlKeyState()){
				tugFlag=true;
				showCursor(false);
			}
			idx=i;
			quadrant=q;
			dynamic_cast<VisualAid*>(g_selNode.getRaw())->houseKeep();

			return true;
		}
	}
	return false;
}
inline bool checkPoint(const bool perspective, const Vec2f point, int& idx, int i, Matrix &partMatrix){//used by both bezier and EC2, and CCLathe
	Vec3f sr;

	sr= partMatrix * Vec3f(point.x, point.y, 0);

	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		g_pointer.translation.x=sr.x;
		g_pointer.translation.y=sr.y;
		g_pointer.translation.z=sr.z;

		if(false == controlKeyState()){
			tugFlag = true;
			showCursor(false);
		}
		idx=i;
		dynamic_cast<VisualAid*>(g_selNode.getRaw())->houseKeep();

		return true;
	}
	else return false;
}

Vec2f convertMouseToScreen2d(){
	Vec2f mv;
	if(g_perspective){

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	return mv;
}
inline bool checkPoint(const bool perspective, const Vec3f sr, int& idx, int i){//used by NurbsPatch, bezier3f and BezierPatch
	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		g_pointer.translation.x=sr.x;
		g_pointer.translation.y=sr.y;
		g_pointer.translation.z=sr.z;

		if(false == controlKeyState()){
			tugFlag=true;
			showCursor(false);
		}
		idx=i;
		dynamic_cast<VisualAid*>(g_selNode.getRaw())->houseKeep();

		return true;
	}
	else return false;
}
//inline bool valid_TL(const bool& perspective, const Vec2f& sel, const int& u, const int& v, const int& uDim, const int& vDim, const Matrix& matrix, const M_Vec3f& dpoint, bool insideOut){
//	if(u > 0 && v < vDim - 1){
//		int i0 = (v+1)*uDim + u;
//		Vec2f pnt0 = procPerspective(perspective, matrix*dpoint[i0]);
//		int i1 = v*uDim + (u-1);
//		Vec2f pnt1 = procPerspective(perspective, matrix*dpoint[i1]);
//		
//		if(insideOut){
//			if(backFace(pnt0, pnt1, sel)){
//				return false;
//			}
//			return true;
//		}
//		else{
//			if(backFace(pnt0, pnt1, sel)){
//				return true;
//			}
//			return false;
//		}
//	}
//	return false;
//}
//
//inline bool valid_BL(const bool& perspective, const Vec2f& sel, const int& u, const int& v, const int& uDim, const int& vDim, const Matrix& matrix, const M_Vec3f& dpoint, bool insideOut){
//	if(u > 0 && v > 0){
//		int i0 = v*uDim + (u-1);
//		Vec2f pnt0 = procPerspective(perspective, matrix*dpoint[i0]);
//		int i1 = (v-1)*uDim + u;
//		Vec2f pnt1 = procPerspective(perspective, matrix*dpoint[i1]);
//		
//		if(insideOut){
//			if(backFace(pnt0, pnt1, sel)){
//				return false;
//			}
//			return true;
//		}
//		else{
//			if(backFace(pnt0, pnt1, sel)){
//				return true;
//			}
//			return false;
//		}
//	}
//	return false;
//}
//
//inline bool valid_TR(const bool& perspective, const Vec2f& sel, const int& u, const int& v, const int& uDim, const int& vDim, const Matrix& matrix, const M_Vec3f& dpoint, bool insideOut){
//	if(u < uDim - 1 && v < vDim - 1){
//		int i0 = v*uDim + (u+1);
//		Vec2f pnt0 = procPerspective(perspective, matrix*dpoint[i0]);
//		int i1 = (v+1)*uDim + u;
//		Vec2f pnt1 = procPerspective(perspective, matrix*dpoint[i1]);
//		
//		if(insideOut){
//			if(backFace(pnt0, pnt1, sel)){
//				return false;
//			}
//			return true;
//		}
//		else{
//			if(backFace(pnt0, pnt1, sel)){
//				return true;
//			}
//			return false;
//		}
//	}
//	return false;
//}
//
//inline bool valid_BR(const bool& perspective, const Vec2f& sel, const int& u, const int& v, const int& uDim, const int& vDim, const Matrix& matrix, const M_Vec3f& dpoint, bool insideOut){
//	if(u < uDim - 1 && v > 0){
//		int i0 = (v-1)*uDim + u;
//		Vec2f pnt0 = procPerspective(perspective, matrix*dpoint[i0]);
//		int i1 = v*uDim + (u+1);
//		Vec2f pnt1 = procPerspective(perspective, matrix*dpoint[i1]);
//		
//		if(insideOut){
//			if(backFace(pnt0, pnt1, sel)){
//				return false;
//			}
//			return true;
//		}
//		else{
//			if(backFace(pnt0, pnt1, sel)){
//				return true;
//			}
//			return false;
//		}
//	}
//	return false;
//}
inline bool NurbsPatch::checkPointCurveCage(const bool& perspective, const Vec3f& sr, const int& i){
	Vec2f sel;
	Vec3f mv;

	if(perspective){
		sel.x = sr.x / (-sr.z + g_viewDist);
		sel.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		sel.x=sr.x;
		sel.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		sel.x < mv.x + g_mouseSense2 &&
		sel.x > mv.x - g_mouseSense2 &&
		sel.y < mv.y + g_mouseSense2 &&
		sel.y > mv.y - g_mouseSense2){

		//if(g_scene->mode & BACKFACE_IGNORE_MODE && g_pointIsVisible[fromPoint + i] == false){
		//	return false;
		//}
		Vec2f diff = Vec2f(sel.x - mv.x, sel.y - mv.y);
		float distance = fabs(diff.length());
		if(distance < MultiSelect::distance){
			MultiSelect::distance = distance;
			MultiSelect::withinSights = true;
			MultiSelect::withinSightsIndex = convertVisualToActual(i, dim.x, dim.y);
			MultiSelect::withinSightsNode = this;
			return true;
		}
	}
	return false;
}

bool isCloserIn3d(const Vec3f sr){
	//Vec3f position = -g_scene->translation + (-g_scene->rotation * Vec3f(0, 0, g_viewDist));
	Vec3f pointer = g_pointer.translation;
	pointer.z = 0;
	//pointer = position;

	Vec3f isr = sr;
	isr.z -= g_viewDist;


	Vec3f isrDiff = isr - pointer;
	float isrDist = isrDiff.length();

	if(MultiSelect::withinSights){
		if(dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p)){
			if(isrDist < MultiSelect::distance){
				MultiSelect::distance = isrDist;
				return true;
			}
			return false;
		}
	}
	MultiSelect::distance = isrDist;
	return true;
}

bool isCloserForBone(const Vec3f sr){
	//Vec3f position = -g_scene->translation + (-g_scene->rotation * Vec3f(0, 0, g_viewDist));
	Vec3f pointer = g_pointer.translation;
	pointer.z = 0;
	//pointer = position;

	Vec3f isr = sr;
	isr.z -= g_viewDist;


	Vec3f isrDiff = isr - pointer;
	float isrDist = isrDiff.length();

	if(MultiSelect::withinSights){
		if(dynamic_cast<Art*>(MultiSelect::withinSightsNode.p)){
			if(isrDist < MultiSelect::distance){
				MultiSelect::distance = isrDist;
				return true;
			}
			return false;
		}
	}
	MultiSelect::distance = isrDist;
	return true;
}

bool NurbsPatch::checkPointIgnoreBackface(const bool& perspective, const Vec3f& sr, const int& u, const int& v, const int& uDim, const int& vDim, const int& i, const Matrix& matrix, const M_Vec3f& dpoint){//used by NurbsPatch
	Vec2f sel;
	Vec3f mv;

	if(perspective){
		sel.x = sr.x / (-sr.z + g_viewDist);
		sel.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		sel.x = sr.x;
		sel.y = sr.y;

		mv.x = g_pointer.translation.x;
		mv.y = g_pointer.translation.y;
	}
	
	if(
		sel.x < mv.x + g_mouseSense2 &&
		sel.x > mv.x - g_mouseSense2 &&
		sel.y < mv.y + g_mouseSense2 &&
		sel.y > mv.y - g_mouseSense2){

		if(g_scene->mode & BACKFACE_IGNORE_MODE && g_pointIsVisible[fromPoint + i] == false){
			return false;
		}
		if(false == (MultiSelect::withinSightsNode == this && MultiSelect::withinSightsIndex == i)){
			if(isCloserIn3d(sr)){
				MultiSelect::withinSights = true;
				MultiSelect::withinSightsIndex = i;
				MultiSelect::withinSightsNode = this;

				g_cursorOverPointIlsBase->translation.refValue() = sr;
				return true;
			}
		}
	}
	return false;
}

bool checkOverPoint2(const bool& perspective, const Vec3f& sr){//used by NurbsPatch, NurbsLathe
	Vec2f sel;
	Vec3f mv;

	if(perspective){
		sel.x = sr.x / (-sr.z + g_viewDist);
		sel.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		sel.x = sr.x;
		sel.y = sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		sel.x < mv.x + g_mouseSense2 &&
		sel.x > mv.x - g_mouseSense2 &&
		sel.y < mv.y + g_mouseSense2 &&
		sel.y > mv.y - g_mouseSense2){

		if(isCloserIn3d(sr)){
			MultiSelect::withinSights = true;
			g_cursorOverPointIlsBase->translation.refValue() = sr;
			return true;
		}
	}
	return false;
}

bool checkPoint(const bool perspective, EC3& ec, int i, Matrix &partMatrix){
	M_Vec2f& point=ec.point;
	Vec3f sr;
	if(point.length==0)return false;


	sr= partMatrix * Vec3f(point[i].x, point[i].y, 0);

	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		g_pointer.translation=sr;

		if(false == controlKeyState()){
			tugFlag=true;
			showCursor(false);
		}
		ec.index[0]=i;
		dynamic_cast<VisualAid*>(g_selNode.getRaw())->houseKeep();

		return true;
	}
	else return false;
}

bool Art::checkPointInVertex(const bool perspective, int i, Matrix& partMatrix){
	Vec3f sr;
	const M_Vec3f& coord=this->coord;
	if(coord.length==0)return false;

	sr = partMatrix * coord[i];

	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		int corner;
		S_Triangle* triangle;
		S_Triangle* beginTri;
		int tri;

		Art* triPart=0;
		tri = findNextTriangleFromVertex( this, triPart, i, corner, smls->ActTri-1);
		triangle=&triPart->triangle[tri];
		beginTri=triangle;

		while(1){

			int vecIndexA=triangle->ia;
			int vecIndexB=triangle->ib;
			int vecIndexC=triangle->ic;

			Art* partA=triangle->pa;
			Art* partB=triangle->pb;
			Art* partC=triangle->pc;

			Vec3f vA = partA->matrix * g_viewMatrix * partA->coord.refValue()[vecIndexA];
			Vec3f vB = partB->matrix * g_viewMatrix * partB->coord.refValue()[vecIndexB];
			Vec3f vC = partC->matrix * g_viewMatrix * partC->coord.refValue()[vecIndexC];

			if(g_settings->backfaceIgnore){
				if(backFace(vA,vB,vC))	break; //if success exit loop 
			}
			else {
				break;
			}

			tri=findNextTriangleFromVertex(this, triPart, i, corner, tri);
			triangle=&triPart->triangle[tri];
			if(triangle!=beginTri) continue;
			else return false; //looped full circle
		}
		if(SenseTugGlobals::node == 0 || sr.z >= SenseTugGlobals::hitZ){

			SenseTugGlobals::pointer = sr;
			SenseTugGlobals::hitZ = sr.z;
			SenseTugGlobals::node = this;
			SenseTugGlobals::actTriPart = triPart;
			SenseTugGlobals::ActTri = tri;
			SenseTugGlobals::corner = corner;
			SenseTugGlobals::ActCtrlPnt = i;
			return true;
		}

	}
	return false;
}
//=IF(AND(((D$4-D6)*(C7-C6)-(C$4-C6)*(D7-D6))*((D$4-D7)*(C8-C7)-(C$4-C7)*(D8-D7))>-0.0001,((D$4-D7)*(C8-C7)-(C$4-C7)*(D8-D7))*((D$4-D8)*(C6-C8)-(C$4-C8)*(D6-D8))>-0.001),"Inside","Not Inside")
//=((D$4-D6)*(C7-C6)-(C$4-C6)*(D7-D6))*((D$4-D7)*(C8-C7)-(C$4-C7)*(D8-D7))
//=((D$4-D7)*(C8-C7)-(C$4-C7)*(D8-D7))*((D$4-D8)*(C6-C8)-(C$4-C8)*(D6-D8))
//4=p
//6=a
//7=b
//8=c
//C=x
//D=y
//(p.y-a.y)*(b.x-a.x)-(p.x-a.x)*(b.y-a.y))*((p.y-b.y)*(c.x-b.x)-(p.x-b.x)*(c.y-b.y) > -.00001 &&
//(p.y-a.y)*(b.x-a.x)-(p.x-a.x)*(b.y-a.y))*((p.y-b.y)*(c.x-b.x)-(p.x-b.x)*(c.y-b.y))

//(p.y-b.y)*(c.x-b.x)-(p.x-b.x)*(c.y-b.y))*((p.y-c.y)*(a.x-c.x)-(p.x-c.x)*(a.y-c.y) > -.00001){
//(p.y-b.y)*(c.x-b.x)-(p.x-b.x)*(c.y-b.y))*((p.y-c.y)*(a.x-c.x)-(p.x-c.x)*(a.y-c.y))


//thanks to:
//http://mcraefamily.com/MathHelp/GeometryPointAndTriangle.htm
//for showing how to find a point in a triangle

bool pointInTriangle(const Vec2f& p, const Vec2f& a, const Vec2f& b, const Vec2f& c){

	if(	((p.y-a.y)*(b.x-a.x)-(p.x-a.x)*(b.y-a.y))*((p.y-b.y)*(c.x-b.x)-(p.x-b.x)*(c.y-b.y)) > 0 &&
		((p.y-b.y)*(c.x-b.x)-(p.x-b.x)*(c.y-b.y))*((p.y-c.y)*(a.x-c.x)-(p.x-c.x)*(a.y-c.y)) > 0){
		return true;
	}
	return false;
}

float getDistanceFromVertex(const Vec2f& p0, const Vec2f& p1){
	float xDiff = fabs(p0.x - p1.x);
	float yDiff = fabs(p0.y - p1.y);
	return sqrt(xDiff * xDiff + yDiff * yDiff);
}
float getDistanceFromSide(Vec2f mv, const Vec2f* v, const int i){
	int nextCorner = (i + 1) % 3;

	Vec2f c = Vec2f(v[nextCorner].x - v[i].x, v[nextCorner].y - v[i].y);
	mv -= Vec2f(v[i].x, v[i].y);

	float a = -c.getAngle();
	c = c.rotate(a);
	mv = mv.rotate(a);
	return fabs(c.y - mv.y);
}
Art* g_testPart;
extern Vec2f g_storeClickPos;

inline bool findLineIntersectInTriangle2(int& c, const Vec2f& pt, const Vec3f* v, Vec3f& edgeHit){
	for(int i = 0; i < 2; i++, c++){
		c = c % 3;
		int n = (c + 1) % 3;
		if(pt.y >= v[n].y && pt.y <= v[c].y){
			Vec3f diff = v[c] - v[n];
			edgeHit.y = pt.y - v[n].y;
			float yRatio = edgeHit.y / diff.y;
			edgeHit.x = diff.x * yRatio;
			edgeHit.z = diff.z * yRatio;
			edgeHit += v[n];
			return true;
		}
	}
	return false;
}
inline bool findLineIntersectInTriangle2(int& c, const Vec2f& pt, const Vec2f* v, Vec2f& edgeHit){
	for(int i = 0; i < 2; i++, c++){
		c = c % 3;
		int n = (c + 1) % 3;
		if(pt.y >= v[n].y && pt.y <= v[c].y){
			Vec2f diff = v[c] - v[n];
			edgeHit.y = pt.y - v[n].y;
			float yRatio = edgeHit.y / diff.y;
			edgeHit.x = diff.x * yRatio;
			edgeHit += v[n];
			return true;
		}
	}
	return false;
}

inline bool findLineIntersectInTriangle(int& c, const Vec2f& pt, const Vec3f* v, Vec3f& edgeHit){
	for(int i = 0; i < 3; i++, c++){
		c = c % 3;
		int n = (c + 1) % 3;
		if(pt.y >= v[c].y && pt.y <= v[n].y){
			Vec3f diff = v[n] - v[c];
			edgeHit.y = pt.y - v[c].y;
			float yRatio = edgeHit.y / diff.y;
			edgeHit.x = diff.x * yRatio;
			edgeHit.z = diff.z * yRatio;
			edgeHit += v[c];
			return true;
		}
	}
	return false;
}

inline bool findLineIntersectInTriangle(int& c, const Vec2f& pt, const Vec2f* v, Vec2f& edgeHit){
	for(int i = 0; i < 3; i++, c++){
		c = c % 3;
		int n = (c + 1) % 3;
		if(pt.y >= v[c].y && pt.y <= v[n].y){
			Vec2f diff = v[n] - v[c];
			edgeHit.y = pt.y - v[c].y;
			float yRatio = edgeHit.y / diff.y;
			edgeHit.x = diff.x * yRatio;
			edgeHit += v[c];
			return true;
		}
	}
	return false;
}
bool getTriangleHitPos(Vec3f& clickPos, const Vec2f& pt, const Vec3f* v){
	Vec3f edgeHitA;
	Vec3f edgeHitB;
	int c = 0;
	if(	findLineIntersectInTriangle(c,	  pt, v, edgeHitA) &&
		findLineIntersectInTriangle2(++c, pt, v, edgeHitB)){
		//if findLineIntersectInTriangle2 fails we could ask if either end in the first line was a direct hit (== || ==) return;

		if(pt.x <= edgeHitA.x && pt.x >= edgeHitB.x){

			float xDiff = edgeHitA.x - edgeHitB.x;
			float zDiff = edgeHitA.z - edgeHitB.z;
			float xRatio = (pt.x - edgeHitB.x) / xDiff;

			clickPos = Vec3f(pt.x, pt.y, edgeHitB.z + zDiff * xRatio);

			return true;
		}
	}
	return false;
}
bool isTriangleHit(const Vec2f& pt, const Vec2f* v){
	Vec2f edgeHitA;
	Vec2f edgeHitB;
	int c = 0;
	if(	findLineIntersectInTriangle(c,	  pt, v, edgeHitA) &&
		findLineIntersectInTriangle2(++c, pt, v, edgeHitB)){

		if(pt.x <= edgeHitA.x && pt.x >= edgeHitB.x){
			return true;
		}
	}
	return false;
}
bool Art::checkPointInTriangle(const bool perspective, const int& triIdx){
	Art* part[3] = {
		triangle.refPart(triIdx),
		triangle.refPart(triIdx+1),
		triangle.refPart(triIdx+2)
	};

	int crdIdx[3] = {
		triangle.refIndex(triIdx),
		triangle.refIndex(triIdx+1),
		triangle.refIndex(triIdx+2)
	};

	Vec3f sr[3] = {
		part[0]->matrix * g_viewMatrix * part[0]->coord.refValue()[crdIdx[0]],
		part[1]->matrix * g_viewMatrix * part[1]->coord.refValue()[crdIdx[1]],
		part[2]->matrix * g_viewMatrix * part[2]->coord.refValue()[crdIdx[2]]
	};

	Vec2f v[3];
	Vec2f mv;

	if(perspective){
		v[0].x = sr[0].x / (-sr[0].z + g_viewDist);
		v[0].y = sr[0].y / (-sr[0].z + g_viewDist);

		v[1].x = sr[1].x / (-sr[1].z + g_viewDist);
		v[1].y = sr[1].y / (-sr[1].z + g_viewDist);

		v[2].x = sr[2].x / (-sr[2].z + g_viewDist);
		v[2].y = sr[2].y / (-sr[2].z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v[0].x=sr[0].x;
		v[0].y=sr[0].y;

		v[1].x=sr[1].x;
		v[1].y=sr[1].y;

		v[2].x=sr[2].x;
		v[2].y=sr[2].y;

		mv.x = g_pointer.translation.x;
		mv.y = g_pointer.translation.y;
	}
	
	//if(pointInTriangle(Vec2f(mv.x, mv.y), Vec2f(v[0].x, v[0].y), Vec2f(v[1].x, v[1].y), Vec2f(v[2].x, v[2].y))){

	if(isTriangleHit(Vec2f(mv.x, mv.y), v)){
		if(g_settings->backfaceIgnore){
			if(backFace(v) == false)	return false; //if success exit loop 
		}
		g_storeClickPos = mv;
		//dLog("pointer: " + String(mv.x)  + ", " + String(mv.y));

		Vec3f v3[3]={
			Vec3f(v[0].x, v[0].y, sr[0].z),
			Vec3f(v[1].x, v[1].y, sr[1].z),
			Vec3f(v[2].x, v[2].y, sr[2].z)
		};
		Vec3f clickPos;
		getTriangleHitPos(clickPos, Vec2f(g_pointer.translation.x, g_pointer.translation.y), v3);
		if(SenseTugGlobals::node == 0 || clickPos.z > SenseTugGlobals::hitZ){

			int corner = 0;
			if((altKeyState() || InputEvent::l_altDown)){
				corner = 0;
				float closest = getDistanceFromSide(Vec2f(mv.x, mv.y), v, 0);
				for(int i = 1; i < 3; i++){
					float distance = getDistanceFromSide(Vec2f(mv.x, mv.y), v, i);

					if(distance < closest){
						closest = distance;
						corner = i;
					}
				}
			}
			else{
				float closest = getDistanceFromVertex(mv, v[0]);
				for(int i = 1; i < 3; i++){
					float distance = getDistanceFromVertex(mv, v[i]);
					if(distance < closest){
						closest = distance;
						corner = i;
					}
				}	
			}
			g_testPart=part[corner];


			SenseTugGlobals::pointer = sr[corner];
			SenseTugGlobals::hitZ = clickPos.z;
			SenseTugGlobals::clickPos = clickPos;
			SenseTugGlobals::node = part[corner];
			SenseTugGlobals::actTriPart = this;
			SenseTugGlobals::ActTri = triIdx / 3;
			SenseTugGlobals::corner = corner;
			SenseTugGlobals::ActCtrlPnt = crdIdx[corner];

			return true;
		}
	}
	return false;
}
bool Art::checkPointInTriangle(const bool perspective, const int& triIdx, TouchSensor* sensor){
	Art* part[3] = {
		triangle.refPart(triIdx),
		triangle.refPart(triIdx+1),
		triangle.refPart(triIdx+2)
	};

	int crdIdx[3] = {
		triangle.refIndex(triIdx),
		triangle.refIndex(triIdx+1),
		triangle.refIndex(triIdx+2)
	};

	Vec3f sr[3] = {
		part[0]->matrix * g_viewMatrix * part[0]->coord.refValue()[crdIdx[0]],
		part[1]->matrix * g_viewMatrix * part[1]->coord.refValue()[crdIdx[1]],
		part[2]->matrix * g_viewMatrix * part[2]->coord.refValue()[crdIdx[2]]
	};

	Vec2f v[3];
	Vec2f mv;

	if(perspective){
		v[0].x = sr[0].x / (-sr[0].z + g_viewDist);
		v[0].y = sr[0].y / (-sr[0].z + g_viewDist);

		v[1].x = sr[1].x / (-sr[1].z + g_viewDist);
		v[1].y = sr[1].y / (-sr[1].z + g_viewDist);

		v[2].x = sr[2].x / (-sr[2].z + g_viewDist);
		v[2].y = sr[2].y / (-sr[2].z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v[0].x = sr[0].x;
		v[0].y = sr[0].y;

		v[1].x = sr[1].x;
		v[1].y = sr[1].y;

		v[2].x = sr[2].x;
		v[2].y = sr[2].y;

		mv.x = g_pointer.translation.x;
		mv.y = g_pointer.translation.y;
	}
	
	//if(pointInTriangle(Vec2f(mv.x, mv.y), Vec2f(v[0].x, v[0].y), Vec2f(v[1].x, v[1].y), Vec2f(v[2].x, v[2].y))){

	if(isTriangleHit(Vec2f(mv.x, mv.y), v)){
		if(g_settings->backfaceIgnore){
			if(backFace(v) == false)	return false; //if success exit loop 
		}
		g_storeClickPos = mv;
		//dLog("pointer: " + String(mv.x)  + ", " + String(mv.y));

		Vec3f v3[3]={
			Vec3f(v[0].x, v[0].y, sr[0].z),
			Vec3f(v[1].x, v[1].y, sr[1].z),
			Vec3f(v[2].x, v[2].y, sr[2].z)
		};
		Vec3f clickPos;
		getTriangleHitPos(clickPos, Vec2f(g_pointer.translation.x, g_pointer.translation.y), v3);
		if(SenseTugGlobals::node == 0 || clickPos.z > SenseTugGlobals::hitZ){

			if(sensor->name != ""){
				sensor->sendClickToServer();
			}
			sensor->hit = true;
			SenseTugGlobals::hitZ = clickPos.z;
			SenseTugGlobals::node = sensor;
			SenseTugGlobals::actTriPart = this;

			return true;
		}
	}
	return false;
}
/*
bool Art::checkPointInTriangleForProjectVertex(const bool perspective, const int& triIdx){
	Art* part[3] = {
		triangle.refPart(triIdx),
		triangle.refPart(triIdx+1),
		triangle.refPart(triIdx+2)
	};

	int crdIdx[3] = {
		triangle.refIndex(triIdx),
		triangle.refIndex(triIdx+1),
		triangle.refIndex(triIdx+2)
	};

	Vec3f sr[3] = {
		part[0]->matrix * g_viewMatrix * part[0]->coord.refValue()[crdIdx[0]],
		part[1]->matrix * g_viewMatrix * part[1]->coord.refValue()[crdIdx[1]],
		part[2]->matrix * g_viewMatrix * part[2]->coord.refValue()[crdIdx[2]]
	};

	Vec2f v[3];
	Vec2f mv;

	if(perspective){
		v[0].x = sr[0].x / (-sr[0].z + g_viewDist);
		v[0].y = sr[0].y / (-sr[0].z + g_viewDist);

		v[1].x = sr[1].x / (-sr[1].z + g_viewDist);
		v[1].y = sr[1].y / (-sr[1].z + g_viewDist);

		v[2].x = sr[2].x / (-sr[2].z + g_viewDist);
		v[2].y = sr[2].y / (-sr[2].z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v[0].x = sr[0].x;
		v[0].y = sr[0].y;

		v[1].x = sr[1].x;
		v[1].y = sr[1].y;

		v[2].x = sr[2].x;
		v[2].y = sr[2].y;

		mv.x = g_pointer.translation.x;
		mv.y = g_pointer.translation.y;
	}
	
	//if(pointInTriangle(Vec2f(mv.x, mv.y), Vec2f(v[0].x, v[0].y), Vec2f(v[1].x, v[1].y), Vec2f(v[2].x, v[2].y))){

	if(isTriangleHit(Vec2f(mv.x, mv.y), v)){
		if(g_settings->backfaceIgnore){
			if(backFace(v) == false)	return false; //if success exit loop 
		}
		g_storeClickPos = mv;
		//dLog("pointer: " + String(mv.x)  + ", " + String(mv.y));
		return true;
	}
	return false;
}
*/

bool Art::checkPointInTriangleForProjectVertex(Vec3f& clickPos, const bool perspective, const int& triIdx){
	g_viewMatrix = g_scene->getViewMatrix();
	Art* part[3] = {
		triangle.refPart(triIdx),
		triangle.refPart(triIdx+1),
		triangle.refPart(triIdx+2)
	};

	int crdIdx[3] = {
		triangle.refIndex(triIdx),
		triangle.refIndex(triIdx+1),
		triangle.refIndex(triIdx+2)
	};

	Vec3f sr[3] = {
		part[0]->matrix * g_viewMatrix * part[0]->coord.refValue()[crdIdx[0]],
		part[1]->matrix * g_viewMatrix * part[1]->coord.refValue()[crdIdx[1]],
		part[2]->matrix * g_viewMatrix * part[2]->coord.refValue()[crdIdx[2]]
	};

	Vec2f v2[3];
	Vec2f mv;

	if(perspective){
		v2[0] = Vec2f(sr[0].x, sr[0].y) / abs(-sr[0].z + g_viewDist) * g_viewDist;
		v2[1] = Vec2f(sr[1].x, sr[1].y) / abs(-sr[1].z + g_viewDist) * g_viewDist;
		v2[2] = Vec2f(sr[2].x, sr[2].y) / abs(-sr[2].z + g_viewDist) * g_viewDist;
		mv = Vec2f(g_pointer.translation.x, g_pointer.translation.y) / abs(-g_pointer.translation.z + g_viewDist) * g_viewDist;
	}
	else{
		v2[0] = Vec2f(sr[0].x, sr[0].y);
		v2[1] = Vec2f(sr[1].x, sr[1].y);
		v2[2] = Vec2f(sr[2].x, sr[2].y);
		mv.x = g_pointer.translation.x;
		mv.y = g_pointer.translation.y;
	}
	

	if(isTriangleHit(mv, v2)){
		//if(g_settings->backfaceIgnore){
		//	if(backFace(v) == false)	return false; //if success exit loop 
		//}

		if(perspective){
			Vec3f v3[3]={
				sr[0] / abs(-sr[0].z + g_viewDist) * g_viewDist,
				sr[1] / abs(-sr[1].z + g_viewDist) * g_viewDist,
				sr[2] / abs(-sr[2].z + g_viewDist) * g_viewDist
			};
			if(getTriangleHitPos(clickPos, mv, v3)){
				Rotation orientation = Rotation(Vec3f(0, 0, -g_viewDist), Vec3f(clickPos.x, clickPos.y, -g_viewDist));

				v3[0] = sr[0];
				v3[1] = sr[1];
				v3[2] = sr[2];
				
				v3[0].z -= g_viewDist;
				v3[1].z -= g_viewDist;
				v3[2].z -= g_viewDist;

				v3[0] = -orientation * v3[0];
				v3[1] = -orientation * v3[1];
				v3[2] = -orientation * v3[2];

				v3[0].z += g_viewDist;
				v3[1].z += g_viewDist;
				v3[2].z += g_viewDist;


				if(getTriangleHitPos(clickPos, Vec2f(), v3)){
					clickPos.z -= g_viewDist;
					clickPos = orientation * clickPos;
					clickPos.z += g_viewDist;
					return true;
				}
			}
		}
		else{
			getTriangleHitPos(clickPos, mv, sr);
			return true;
		}

	}
	return false;
}

bool isPointInTriangleHit(Vec3f& clickPos, Vec3f a, Vec3f b, Vec3f c, const bool perspective){
	g_viewMatrix = g_scene->getViewMatrix();

	Vec3f sr[3] = {
		g_viewMatrix * a,
		g_viewMatrix * b,
		g_viewMatrix * c
	};

	Vec2f v2[3];
	Vec2f mv;

	if(perspective){
		v2[0] = Vec2f(sr[0].x, sr[0].y) / abs(-sr[0].z + g_viewDist) * g_viewDist;
		v2[1] = Vec2f(sr[1].x, sr[1].y) / abs(-sr[1].z + g_viewDist) * g_viewDist;
		v2[2] = Vec2f(sr[2].x, sr[2].y) / abs(-sr[2].z + g_viewDist) * g_viewDist;
		mv = Vec2f(g_pointer.translation.x, g_pointer.translation.y) / abs(-g_pointer.translation.z + g_viewDist) * g_viewDist;
	}
	else{
		v2[0] = Vec2f(sr[0].x, sr[0].y);
		v2[1] = Vec2f(sr[1].x, sr[1].y);
		v2[2] = Vec2f(sr[2].x, sr[2].y);
		mv.x = g_pointer.translation.x;
		mv.y = g_pointer.translation.y;
	}
	

	if(isTriangleHit(mv, v2)){
		//if(g_settings->backfaceIgnore){
		//	if(backFace(v) == false)	return false; //if success exit loop 
		//}

		if(perspective){
			Vec3f v3[3]={
				sr[0] / abs(-sr[0].z + g_viewDist) * g_viewDist,
				sr[1] / abs(-sr[1].z + g_viewDist) * g_viewDist,
				sr[2] / abs(-sr[2].z + g_viewDist) * g_viewDist
			};

			if(getTriangleHitPos(clickPos, mv, v3)){
				Rotation orientation = Rotation(Vec3f(0, 0, -g_viewDist), Vec3f(clickPos.x, clickPos.y, -g_viewDist));

				v3[0] = sr[0];
				v3[1] = sr[1];
				v3[2] = sr[2];
				
				v3[0].z -= g_viewDist;
				v3[1].z -= g_viewDist;
				v3[2].z -= g_viewDist;

				v3[0] = -orientation * v3[0];
				v3[1] = -orientation * v3[1];
				v3[2] = -orientation * v3[2];

				v3[0].z += g_viewDist;
				v3[1].z += g_viewDist;
				v3[2].z += g_viewDist;


				if(getTriangleHitPos(clickPos, Vec2f(), v3)){
					clickPos.z -= g_viewDist;
					clickPos = orientation * clickPos;
					clickPos.z += g_viewDist;
					clickPos = g_scene->getIViewMatrix() * clickPos;
					return true;
				}
			}
		}
		else{
			if(getTriangleHitPos(clickPos, mv, sr)){
				clickPos = g_scene->getIViewMatrix() * clickPos;
				return true;
			}
		}
	}
	return false;
}

void SceneTree::setResultsForSelectingNode(Node* node){
	setSelNodeLine(node);
	lastSelNodeLine = selLine;
	cp->setupNodeWindow(node);
	cp->paint();
	setNodeWinTitleBar(node);
	textCell.cellStr = g_selNode->name;
	removeTypeFromList<CaretWindow>(g_callBackList);
	textCell.cellActive = false;
	paint();
	setTitleName();
}

void SceneTree::setResultsForSelectingVertex(Art* part){//how is this any different to setResultsForSelectingNode ?
	setSelNodeLine(part);
	lastSelNodeLine = selLine;
	if(g_addBone->mousePhase == AddBone::NEUTRAL){
		cp->setupNodeWindow(part);
		cp->paint();
		setNodeWinTitleBar(part);
		if(g_scene->mode & ANIMATION_MODE){
			g_vcArtAnimBase->resetRotationLever();
		}
	}
	if(PaletteCP* paletteCP = paletteCPAllreadyOpen()){
		paletteCP->paint();
	}
	textCell.cellStr = g_selNode->name;
	removeTypeFromList<CaretWindow>(g_callBackList);
	textCell.cellActive = false;
	paint();
	setTitleName();
}

void SceneTree::setResultsForSelectingPoint(Build* bld){//except for the last line how is this any different to setResultsForSelectingNode ?
	NurbsPatch* patch = bld->getPatch();
	setSelNodeLine(bld);
	lastSelNodeLine = selLine;
	if(g_drawNurbsLine->mousePhase != DrawNurbsLine::READY_TO_DRAG){
		cp->setupNodeWindow(bld);
		cp->paint();
		setNodeWinTitleBar(bld);
	}
	textCell.cellStr = g_selNode->name;
	removeTypeFromList<CaretWindow>(g_callBackList);
	textCell.cellActive = false;
	paint();
	setTitleName();
	patch->houseKeep();
}

//anim control point for part
bool checkAnimCtrlPoint(const bool perspective, Vec3f sr){
	
	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		g_pointer.translation.x = sr.x;
		g_pointer.translation.y = sr.y;
		g_pointer.translation.z = sr.z;
		SenseTugGlobals::node = g_visualClue;

//		if(false == Window::animMode()){
		tugFlag = true;
		showCursor(false);
//		}

		return true;
	}
	else return false;
}
bool checkWithInSights(const bool perspective, Vec3f sr){
	
	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x + g_mouseSense2&&
		v.x > mv.x - g_mouseSense2&&
		v.y < mv.y + g_mouseSense2&&
		v.y > mv.y - g_mouseSense2){

				
		//g_pointer.translation = sr;
		g_cursorOverPointIlsBase->translation.refValue() = sr;
		g_transformCP->mouseOffset = sr - g_pointer.translation;
		SenseTugGlobals::node = g_visualClue;

//		if(false == Window::animMode()){
//		}

		return true;
	}
	else return false;
}


bool checkWithInSightsForBone(const bool perspective, Vec3f sr){
	
	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x + g_mouseSense2&&
		v.x > mv.x - g_mouseSense2&&
		v.y < mv.y + g_mouseSense2&&
		v.y > mv.y - g_mouseSense2){

		if(isCloserForBone(sr)){
			MultiSelect::withinSights = true;
			g_cursorOverPointIlsBase->translation.refValue() = sr;
			g_transformCP->mouseOffset = sr - g_pointer.translation;

			return true;
		}
	}
	return false;
}

bool checkRadialTugSize(const bool perspective, const ImmediatePart* part){
	if(part->coord.length == 0){
		return false;
	}
	const Seamless* smls=part->smls;
	Vec3f sr = part->matrix * g_viewMatrix * part->coord[smls->ActCtrlPnt];
	sr.x += g_scene->getZoom() * g_radial_Tug->radius;

	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		g_pointer.translation.x=sr.x;
		g_pointer.translation.y=sr.y;
		g_pointer.translation.z=sr.z;
		SenseTugGlobals::node=g_visualClue;

		tugFlag=true;
		showCursor(false);
		Art::tugRadialTugSize=true;

		return true;
	}
	Art::tugRadialTugSize=false;
	return false;
}
bool checkRadialTugSizeNurbs(const bool perspective, NurbsPatch* patch){
	if(patch->point.length == 0){
		return false;
	}
	Vec3f sr = g_viewMatrix * patch->dpoint[patch->index[0]];
	sr.x += g_scene->getZoom() * g_radial_Tug->radius;

	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		g_pointer.translation.x=sr.x;
		g_pointer.translation.y=sr.y;
		g_pointer.translation.z=sr.z;
		SenseTugGlobals::node=g_visualClue;

		tugFlag=true;
		showCursor(false);
		Art::tugRadialTugSize=true;

		return true;
	}
	Art::tugRadialTugSize=false;
	return false;
}
void Node::senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor){
}
void ProtoDeclare::senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor){
	//if(SenseTugGlobals::sameVertexHit)return;
	if(SenseTugGlobals::node)return;
	M_Node& children=protoBody->children.refValue();
	for(int k=0; k<children.length;k++){
		children[k]->senseTug(matrix,imatrix, 0);
	}
}

int getSeamlessParent(int line){//diminished better to use Seamless* getSeamless(int& selLine){
	do{
		line = g_sceneTreeList[line].parent;
	}while(dynamic_cast<Seamless*>(g_sceneTreeList[line].node.p)==0);
	return line;
}
void BezierPatch::setNumOfGroupFromSelPoint(){
	const M_Vec2f& dim=this->dim;
	int pointIndex=index[pointGroup];
	int numOfUGroupIdx=0;

	for(int k=0; k<dim.length; k++){
		int noU=dim[k].x;
		int noV=dim[k].y;
		numOfUGroupIdx+=noU*noV;
		if(numOfUGroupIdx>pointIndex){
			index[numOfUGroup]=k;
			break;
		}
	}
}
M_Vec3f g_backUpCoord;
void backUpSmlsCoord(const M_Art& partList){
	g_backUpCoord.length=0;
	for(int c=0, k=0; k<partList.length; k++){
		Art* part=partList[k];
		M_Vec3f& coord=part->coord.refValue();
		int length=part->coord.length;
		g_backUpCoord.length+=length;
		{
			for(int i=0; i<length; c++, i++){
				g_backUpCoord[c]=coord[i];
			}
		}
	}
}

void backUpSmlsPoint(const Seamless* smls){
	const M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;

	g_backUpCoord.length = 0;
	for(int k = 0; k < cpNodeList.length; k++){
		g_backUpCoord += cpNodeList[k]->dpoint;
	}
}

void restoreCoord(const M_Art& partList){
	for(int c=0, k=0; k<partList.length; k++){
		Art* part=partList[k];
		M_Vec3f& coord=part->coord.refValue();
		int length=part->coord.length;
		{
			for(int i=0; i<length; c++, i++){
				coord[i]=g_backUpCoord[c];
			}
		}
	}
}
/*
inline void NurbsPatch::initTug(const bool& perspective){
	tugRingPivotPoint=false;
	int& idx = index[0];
	Matrix matrix = this->matrix * g_viewMatrix;
	int j=index[0];
	const int uDim = dim.x;
	const int vDim = dim.y;

	Vec2i idx2i = Vec2i(idx % uDim, idx / uDim);
	if(checkPoint(perspective, matrix*dpoint[idx], idx, idx)){
		g_selNode->processFields();
		g_stw->cp->paint();
		tugFlag=true;
		return;
	}
	if(showAll){
		for(int v=0; v < vDim; v++){
			for(int u=0; u < uDim; u++){
				int i = v*uDim + u;
				Vec3f pnt = matrix * dpoint[i];
				if(checkPointIgnoreBackface(perspective, pnt, idx, u, v, uDim, vDim, i, matrix, dpoint)){
					g_selNode->processFields();
					g_stw->cp->paint();
					tugFlag = true;
					return;
				}
			}
		}
	}
	else{
		if(uJoinEnds){
			for(int v=0; v < vDim; v++){
				for(int u=1; u < uDim; u+=2){
					int i = v*uDim + u;
					Vec3f pnt = matrix * dpoint[i];
					if(checkPointIgnoreBackface(perspective, pnt, idx, u, v, uDim, vDim, i, matrix, dpoint)){
						g_selNode->processFields();
						g_stw->cp->paint();
						tugFlag=true;
						return;
					}
				}
			}
		}
		else{
			if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
				for(int v=0; v < vDim; v += 2){
					for(int u=0; u<uDim; u += 2){
						int i = v * uDim + u;
						Vec3f pnt = matrix*dpoint[i];
						if(checkPointIgnoreBackface(perspective, pnt, idx, u, v, uDim, vDim, i, matrix, dpoint)){
							g_selNode->processFields();
							g_stw->cp->paint();
							tugFlag=true;
							return;
						}
						if(u == 0 || u == uDim - 2){
							u--;
						}
					}
					if(v == 0 || v == vDim - 2){
						v--;
					}
				}
			}
			else{
				for(int v = 0; v < vDim; v++){
					for(int u = 0; u < uDim; u += 2){
						int i = v * uDim + u;
						Vec3f pnt = matrix * dpoint[i];
						if(checkPointIgnoreBackface(perspective, pnt, idx, u, v, uDim, vDim, i, matrix, dpoint)){
							g_selNode->processFields();
							g_stw->cp->paint();
							tugFlag = true;
							return;
						}
						if(u == 0 || u == uDim - 2){
							u--;
						}
					}
				}
			}
		}
	}
	return;
}
*/
inline bool NurbsPatch::initBoxModellingTugCP(const bool& perspective){
	tugRingPivotPoint = false;
	Matrix matrix = smls->getSmlsViewMatrix();

	const int uDim = dim.x;
	const int vDim = dim.y;

	bool hit = false;

	for(int v = 0; v < vDim; v += 2){
		for(int u = 0; u < uDim; u += 2){
			int i = v * uDim + u;
			Vec3f pnt = matrix * dpoint[i];
			if(checkPointIgnoreBackface(perspective, pnt,  u, v, uDim, vDim, i, matrix, dpoint)){
				hit = true;
			}
		}
	}
	return hit;
}

inline bool NurbsPatch::initTugCP(const bool& perspective){
	bool hit = false;

	Matrix matrix = smls->getSmlsViewMatrix();

	const int wireResolution = 4;

	tugRingPivotPoint = false;

	//int j = index[0];
	const int uDim = dim.x;
	const int vDim = dim.y;

	int uCells = uDim / 2;
	int vCells = vDim / 2;

	M_Vec3f& point = dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;
	int lookUpIndex = g_pointLookup[plidx];

	for(int vc = vCells + 2; --vc >= 0;){
		for(int uc = uCells + 2; --uc >= 0;){
			int i = vc * (uCells + 2) + uc;
			Vec3f pnt = matrix * point[lookUpIndex + i];
			if(checkPointCurveCage(perspective, pnt, i)){
				hit = true;
			}
		}
	}
	return hit;
}
inline bool NurbsPatch::initStraightCageTugCP(const bool& perspective){
	tugRingPivotPoint = false;
	Matrix matrix = smls->getSmlsViewMatrix();

	//int j = index[0];
	const int uDim = dim.x;
	const int vDim = dim.y;

	bool hit = false;

	//Vec2i idx2i = Vec2i(idx % uDim, idx / uDim);
/*
	if(uJoinEnds){
		if(idx2i.x == uDim-1){
			idx2i.x = 0;
		}
		if(beginSP){
			if(idx2i.y == 0){
				idx2i.x = 0;
				if(showAll==false){
					idx2i.x = 1;
				}
			}
		}
		if(endSP){
			if(idx2i.y == vDim-1){
				idx2i.x = 0;
				if(showAll==false){
					idx2i.x = 1;
				}
			}
		}
	}
	else{
		if(beginSP){
			pointLen-=uDim-1;
			if(idx2i.y == 0){
				idx2i.x = 0;
			}
		}
		if(endSP){
			pointLen-=uDim-1;
			if(idx2i.y == vDim-1){
				idx2i.x = 0;
			}

		}
	}
	idx = idx2i.y * uDim + idx2i.x;
*/

	//if(idx < dpoint.length && checkPoint(perspective, matrix*dpoint[idx], idx, idx)){
	//	g_selNode->processFields();
	//	g_stw->cp->paint();
	//	tughit=true;
	//	return;
	//}
	if(showAll){
		for(int v=0; v < vDim; v++){
			for(int u=0; u < uDim; u++){
				int i = v*uDim + u;
				Vec3f pnt = matrix * dpoint[i];
				if(checkPointIgnoreBackface(perspective, pnt,  u, v, uDim, vDim, i, matrix, dpoint)){
					hit = true;
				}
			}
		}
	}
	else{
		if(uJoinEnds){
			if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
				if(vDim == 1){
					for(int u = 1; u < uDim; u++){
						Vec3f pnt = matrix * dpoint[u];
						if(checkPointIgnoreBackface(perspective, pnt, u, 0, uDim, vDim, u, matrix, dpoint)){
							hit = true;
						}
					}
				}
				else{
					for(int v = 0; v < vDim; v += 2){
						for(int u = 1; u < uDim; u += 2){
							int i = v * uDim + u;
							Vec3f pnt = matrix*dpoint[i];
							if(checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, i, matrix, dpoint)){
								hit = true;
							}
						}
						if(v == 0 || v == vDim - 2){
							v--;
						}
					}
				}
			}
			else{
				if(vDim == 1){
					for(int u = 1; u < uDim; u++){
						Vec3f pnt = matrix * dpoint[u];
						if(checkPointIgnoreBackface(perspective, pnt, u, 0, uDim, vDim, u, matrix, dpoint)){
							hit = true;
						}
					}
				}
				else{
					for(int v = 0; v < vDim; v++){
						for(int u = 1; u < uDim; u += 2){
							int i = v * uDim + u;
							Vec3f pnt = matrix * dpoint[i];
							if(checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, i, matrix, dpoint)){
								hit = true;
							}
						}
					}
				}
			}
		}
		else{
			if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
				if(isCtrlPolyRef){
					for(int v = 1; v < vDim; v += 2){
						for(int u = 1; u < uDim; u += 2){
							int i = v * uDim + u;
							Vec3f pnt = matrix*dpoint[i];
							if(checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, i, matrix, dpoint)){
								hit = true;
							}
						}
					}
				}
				else{
					if(vDim == 1){
						for(int u = 0; u < uDim; u++){
							Vec3f pnt = matrix*dpoint[u];
							if(checkPointIgnoreBackface(perspective, pnt, u, 0, uDim, vDim, u, matrix, dpoint)){
								hit = true;
							}
						}
					}
					else{
						for(int v = 0; v < vDim; v += 2){
							for(int u = 0; u < uDim; u += 2){
								int i = v * uDim + u;
								Vec3f pnt = matrix*dpoint[i];
								if(checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, i, matrix, dpoint)){
									hit = true;
								}

								if(u == 0 || u == uDim - 2){
									u--;
								}
							}
							if(v == 0 || v == vDim - 2){
								v--;
							}
						}
					}
				}
			}
			else{
				if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
					for(int v = 0; v < vDim; v++){
						for(int u = 0; u < uDim; u += 2){
							int i = v * uDim + u;
							Vec3f pnt = matrix * dpoint[i];
							if(checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, i, matrix, dpoint)){
								hit = true;
							}
							if(u == 0 || u == uDim - 2){
								u--;
							}
						}
					}
				}
				else{
					for(int v = 0; v < vDim; v++){
						for(int u = 0; u < uDim; u++){
							int i = v * uDim + u;
							Vec3f pnt = matrix * dpoint[i];
							if(checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, i, matrix, dpoint)){
								hit = true;
							}
						}
					}
				}
			}
		}
	}
	return hit;
}

Vec3f calcRingPivotPoint2(int selUdim, int from){
	if(selUdim == 0){
		return MultiSelect::beginDragPos[from];
	}
	Vec3f pivotPoint;
	for(int u = 0; u < selUdim; u++){
		pivotPoint += MultiSelect::beginDragPos[from + u];
	}

	return pivotPoint / selUdim;
}

Vec3f calcRingPivotPointForHair(NurbsPatch* patch){
	Vec3f pivotPoint;
	int len = MultiSelect::pointList.length;
	int total = 0;
	for(int i = 0; i < len; i++){
		if(MultiSelect::pointList[i].p == patch){
			pivotPoint += MultiSelect::beginDragPos[i];
			total++;
		}
	}

	return pivotPoint / total;
}

bool NurbsLathe::checkOverPoint(const bool& perspective){
	Matrix matrix = this->matrix * g_viewMatrix;
	M_Vec2f& dpoint = this->dpoint;


	int mid = checkNurbsType(dpoint.length, knot);
	if(NurbsPatch::MID_0 == mid){
		for(int i = dpoint.length; --i >= 0;){
			Vec2f pnt2f = dpoint[i];
			Vec3f pnt = matrix * Vec3f(pnt2f.x, pnt2f.y, 0);
			if(checkOverPoint2(perspective, pnt)){
				MultiSelect::withinSightsNode = this;
				MultiSelect::clickedIndex = i;
				return true;
			}
		}
	}
	else{
		for(int i = dpoint.length; --i >= 0;){
			Vec2f pnt2f = dpoint[i];
			Vec3f pnt = matrix * Vec3f(pnt2f.x, pnt2f.y, 0);
			if(checkOverPoint2(perspective, pnt)){
				MultiSelect::withinSightsNode = this;
				MultiSelect::clickedIndex = i;
				return true;
			}
			if(i != dpoint.length - 1 && i != 1){
				i--;
			}
		}
	}
	return false;
}

bool NurbsPatch::selectSelectedPoint(const bool& perspective){
	for(int i = MultiSelect::pointList.length; --i >= 0;){
		NurbsPatch* patch = MultiSelect::pointList[i].p;
		int idx = MultiSelect::pointList[i].i;
		Matrix matrix = patch->matrix * g_viewMatrix;
		M_Vec3f& dpoint = patch->dpoint;
		Vec3f pnt = matrix * dpoint[idx];

		const int uDim = patch->dim.x;
		const int vDim = patch->dim.y;
		int v = idx / uDim;
		int u = idx - v * uDim;

		if(checkOverPoint2(perspective, pnt)){
			MultiSelect::withinSightsIndex = idx;
			MultiSelect::withinSightsNode = patch;
			MultiSelect::clickedIndex = i;
		}
	}
	return MultiSelect::withinSights;
}


void findPartsCPReference(NurbsPatch* patch, Seamless* smls){
	int& idx = patch->index[0];
	M_Art& partList = smls->partList;
	for(int k = partList.length; --k >= 0;){
		Art* part = partList[k];
		M_ControlPoint& m_controlPoint = part->controlPoint;

		for(int i = m_controlPoint.length; --i >= 0;){
			ControlPoint& cp = m_controlPoint[i];
			if(cp.p == patch && cp.i == idx){
				//MultiSelect::selectedPoint = i;

				//MultiSelect::beginDragPos += patch->dpoint[idx];
				//MultiSelect::pointList += ControlPoint(patch, idx);

				if(part != g_selNode.getRaw()){
					g_stw->setResultsForSelectingVertex(part);
				}

				return;
			}
		}
	}
}

int NurbsPatch::convertVisualToActual(int idx, const int uDim, const int vDim){
	int uCells = uDim / 2;
	int vCells = vDim / 2;

	int vVis = idx / (uCells + 2);
	int uVis = idx - vVis * (uCells + 2);

	int u = uVis * 2;
	if(u == uDim + 1){
		u--;
	}
	if(u > 1){
		u--;
	}

	int v = vVis * 2;
	if(v == vDim + 1){
		v--;
	}
	if(v > 1){
		v--;
	}

	return v * uDim + u;
}

int NurbsPatch::convertActualToVisual(int idx, const int uDim, const int vDim){
	int uCells = uDim / 2;
	int vCells = vDim / 2;

	int vAct = idx / uDim;
	int uAct = idx - vAct * uDim;

	int u = 0;
	int v = 0;

	if(uAct <= 1){
		u = uAct;
	}
	else if(uAct == uDim - 1){
		u = uCells + 1;
	}
	else{
		u = uAct / 2 + 1;
	}

	if(vAct <= 1){
		v = vAct;
	}
	else if(vAct == vDim - 1){
		v = vCells + 1;
	}
	else{
		v = vAct / 2 + 1;
	}

	return v * (uCells + 2) + u;
}

void initMultiCageTug2(NurbsPatch* patch){
	tugFlag = true;
	int idx = MultiSelect::withinSightsIndex;

	//int idx = patch->convertVisualToActual(idxVis, patch->dim.x, patch->dim.y);

	patch->index[0] = idx;
	Vec3f sr = patch->dpoint[idx];
	g_transformCP->remCtrlPnt = sr;
	if(MultiSelect::withinSightsIsSelected == false){
		MultiSelect::beginDragPos = sr;
		MultiSelect::pointList = ControlPoint(patch, idx);
		MultiSelect::clickedIndex = 0;
	}
	for(int i = MultiSelect::pointList.length; --i>=0;){
		if(MultiSelect::pointList[i].p == MultiSelect::withinSightsNode && MultiSelect::pointList[i].i == MultiSelect::withinSightsIndex){
			MultiSelect::selectedPoint = i;
			break;
		}
	}
	if(MultiSelect::pointList.length && (g_scene->mode & SINGLE_ACTIVE_MODE)){
		g_transformCP->pivotPoint.refValue() = MultiSelect::pointList[MultiSelect::selectedPoint].p->dpoint[MultiSelect::pointList[MultiSelect::selectedPoint].i];
	}

	Seamless* smls = patch->smls;

	sr = smls->getSmlsViewMatrix() * sr;
//	sr =smls->getSmlsIViewMatrix() * sr;

	g_pointer.remTranslation = g_pointer.translation;
	g_transformCP->mouseOffset = sr - g_pointer.translation;
	//g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt;

	if(g_scene->mode & ANIMATION_MODE){
		findPartsCPReference(patch, patch->smls);
	}
	else{
		if(patch->container != g_selNode.getRaw()){
			g_stw->setResultsForSelectingPoint(patch->container);
		}
	}
	//recentChange
	showCursor(false);
}

void Seamless::selectPatchGroupWithinSights(){
	if(MultiSelect::accumulate == false){
		MultiSelect::reset();
		clearObjects();
	}
	M_<NurbsPatch*> contigList;  //make sure we dont add points twice by making contig list
	createSelContigList(contigList);

	for(int k = 0; k < cpNodeList.length; ){
		Build* bld = cpNodeList[k]->container;
		int remK = k;
		for(int i = 0; i < bld->numOfPatches(); i++){
			NurbsPatch* patch = cpNodeList[k++];
			if(patch == MultiSelect::withinSightsNode){
				k = remK;
				bld->remTransform();
				for(int i = 0; i < bld->numOfPatches(); i++){
					NurbsPatch* patch = cpNodeList[k++];
					patch->selectControlPointsNoBoxTest(contigList);
				}
				break;
			}
		}
	}

	MultiSelect::pointList.length = 0;
	for(int k = 0; k < pointTotal; k++){
		if(NurbsPatch* patch = contigList[k]){
			if(patch == MultiSelect::withinSightsNode && k - patch->fromPoint == MultiSelect::withinSightsIndex){
				MultiSelect::clickedIndex = MultiSelect::pointList.length;
			}
			MultiSelect::pointList += ControlPoint(patch, k - patch->fromPoint);
		}
	}

	g_transformCP->calcPivotPoint();
	g_transformCP->resetRotationLever();

	
	MultiSelect::withinSightsIsSelected = true;
}


void Seamless::clickPointWithinSights(NurbsPatch* patch){
	const bool perspective = g_perspective;
	Build* bld = patch;
	int k = patch->plidx;
	int& idx = patch->index[0];
	if(g_transformCP->selectObject){
		//initMultiCageTug2(patch);
		//return;					
		if(MultiSelect::withinSightsIsSelected == false){
			selectPatchGroupWithinSights();

			////k = remK;
			//bld->remTransform();
			//Seamless* smls = patch->smls;
			//if(MultiSelect::accumulate == false){
			//	MultiSelect::reset();
			//	smls->clearObjects();
			//}
			//M_<NurbsPatch*> contigList;  //make sure we dont add points twice by making contig list
			//patch->smls->createSelContigList(contigList);
			//			
			//for(int i = 0; i < bld->numOfPatches(); i++){
			//	NurbsPatch* patch = cpNodeList[k++];
			//	patch->selectControlPointsNoBoxTest(contigList);
			//}

			//MultiSelect::pointList.length = 0;
			//for(int k = 0; k < smls->pointTotal; k++){
			//	if(NurbsPatch* patch = contigList[k]){
			//		MultiSelect::pointList += ControlPoint(patch, k - patch->fromPoint);
			//	}
			//}

			//MultiSelect::beginDragPos.length = MultiSelect::pointList.length;
			//for(int i = MultiSelect::pointList.length; --i >= 0;){
			//	NurbsPatch* patch = MultiSelect::pointList[i].p;
			//	int idx = MultiSelect::pointList[i].i;
			//	Vec3f v = patch->dpoint[idx];
			//	MultiSelect::beginDragPos[i] = v;
			//}

			//if(bld != g_selNode.getRaw()){
			//	g_stw->setResultsForSelectingPoint(bld);
			//}
			//patch->selectSelectedPoint(perspective);
		}
		initMultiCageTug2(patch);
		return;					
	}
	else if(g_transformCP->selectRing || g_transformCP->selectRings){
		if(MultiSelect::accumulate == false){
			MultiSelect::beginDragPos.length = 0;
			MultiSelect::pointList.length = 0;
		}
		int idx = patch->index[0];
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		int v = idx / uDim;
		Vec3f pivotPoint;
						
		if(Hair* hair = getHair(getNodeLine(patch))){
			if(Strand* strand = dynamic_cast<Strand*>(patch)){
				Lock* lock = 0;
				int scpiIdx = 0;
				if(hair->isBoundToLock(strand, idx, lock, scpiIdx)){
					patch = lock;
					uDim = lock->dim.x;
					vDim = lock->dim.y;
					g_stw->setResultsForSelectingPoint(lock);
				}
				/*
				else{
					goto here;
				}
				*/
			}
			else if(Lock* lock = dynamic_cast<Lock*>(patch)){
				if(g_transformCP->selectRing){
					patch->selectRings(0, uDim, vDim);
				}
				else{
					hair->selectRings(lock);
				}
			}
			pivotPoint = calcRingPivotPointForHair(patch);
			g_stw->setResultsForSelectingPoint(patch);
		}
		/*
		else{
here:
			if(g_transformCP->selectRing){
				patch->selectRing(v, uDim, vDim);
			}
			else{
				patch->selectRings(v, uDim, vDim);
			}
			g_stw->setResultsForSelectingPoint(patch);
			pivotPoint = calcRingPivotPoint2(MultiSelect::uRingDim, 0);
		}
		*/
						

		g_transformCP->scale.refValue() = Vec3f(1, 1, 1);
		g_transformCP->scaleOrientation.refValue() = Rotation();
		g_transformCP->pivotPoint.refValue() = pivotPoint;
		g_transformCP->rotation.refValue() = Rotation();
		g_transformCP->translation.refValue() = Vec3f();

		for(int k = g_transformCP->rotCtrlPnt.length; --k>=0;){
			g_transformCP->rotCtrlPnt[k] = g_scene->getViewMatrix() * g_transformCP->pivotPoint + g_transformCP->defaultRotCtrlPnt[k];
		}

		g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];

		g_transformCP->houseKeep();

		//patch->selectSelectedPoint(perspective);
	}
	initMultiCageTug2(patch);
}

bool isNurbsPatch(){
	if(dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
		return true;
	}
	if(g_scene->mode & ANIMATION_MODE){
		if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
			if(part->smls->animCP){
				return true;
			}
		}
	}
	return false;
}

void Seamless::initMultiCageTug(const bool& perspective){
	if(	g_addLathe->mousePhase != AddLathe::NEUTRAL ||
		g_panViewMode ||
		g_insertPatch.imousePhase != InsertPatch::NEUTRAL ||
		g_deleteControlLine.mousePhase != DeleteControlLine::NEUTRAL ||
		g_rectangleSelectPointReady ||
		g_rectangleUnselectPointReady ||
		g_rectangleSelectPolyReady){

		return;
	}

	g_viewMatrix = g_scene->getViewMatrix();
	MultiSelect::resetDistance();
	MultiSelect::withinSightsIsSelected = false;

	NurbsLathe* lathe = dynamic_cast<NurbsLathe*>(g_selNode.getRaw());
	if(lathe && g_transformCP->selectObject == false){
		if(lathe->checkOverPoint(perspective)){
			if(MultiSelect::withinSights){
				changeModellingCursor(g_lockedOnPointCursor);
				return;
			}
		}
		else{
			changeModellingCursor(g_crossCursor);
		}
		return;
	}
	if( false == isNurbsPatch() && 0 == dynamic_cast<GroupBuild*>(g_selNode.getRaw())){
		return;
	}
	if(MultiSelect::accumulate || (g_transformCP->selectRing == false && g_transformCP->selectRings == false)){
		if(NurbsPatch::selectSelectedPoint(perspective)){
			if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p)){
				MultiSelect::withinSightsIsSelected = true;
			
				Vec3f sr = g_viewMatrix * patch->dpoint[MultiSelect::withinSightsIndex];
				g_cursorOverPointIlsBase->translation.refValue() = sr;
				changeModellingCursor(g_lockedOnPointCursor);
			}
			return;
		}
	}


	//check non selected points
	if(g_transformCP->selectRings == false){
		MultiSelect::clickedIndex = 0;
	}

	if(NurbsLine* selLine = dynamic_cast<NurbsLine*>(g_selNode.getRaw())){
		if(g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE){
			selLine->initBoxModellingTugCP(perspective);
		}
		else if(g_curvedCageMode){
			selLine->initTugCP(perspective);
		}
		else{
			selLine->initStraightCageTugCP(perspective);
		}
		if(MultiSelect::withinSights){
			changeModellingCursor(g_lockedOnPointCursor);
			return;
		}
		if(g_addNurbsLine->mousePhase != AddNurbsLine::NEUTRAL){
			changeModellingCursor(g_addPointCursor);
			return;
		}
	}

	for(int k = 0; k < cpNodeList.length;){
		Build* bld = cpNodeList[k];
		//if(g_transformCP->selectObject == false && dynamic_cast<NurbsPatch*>(bld) == 0){
		//	k += bld->numOfPatches();
		//	continue;
		//}
		int remK = k;
		
		for(int i = 0; i < bld->numOfPatches(); i++){
			NurbsPatch* patch = cpNodeList[k++];
			if(patch->cage){
				if(dynamic_cast<Strand*>(patch)){
					if((g_scene->mode & SHOW_STRAND_CTRL_LINE_MODE) == false){
						continue;
					}
				}
				if(g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE){
					patch->initBoxModellingTugCP(perspective);
				}
				else if(g_curvedCageMode){
					patch->initTugCP(perspective);
				}
				else{
					patch->initStraightCageTugCP(perspective);
				}
			}
		}
	}
	if(false == (g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE)){
		initTugStitchPoly(perspective);
	}
	if(MultiSelect::withinSights){
		changeModellingCursor(g_lockedOnPointCursor);
		return;
	}
	changeModellingCursor(g_crossCursor);
	
}


inline void Art::initTugCP(const bool& perspective){

	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
	
	for(int i = MultiSelect::pointList.length; --i >= 0;){
		NurbsPatch* patch = MultiSelect::pointList[i].p;
		int idx = MultiSelect::pointList[i].i;
		Matrix matrix = patch->matrix * g_viewMatrix;
		M_Vec3f& dpoint = patch->dpoint;
		Vec3f pnt = matrix*dpoint[idx];

		const int uDim = patch->dim.x;
		const int vDim = patch->dim.y;
		int v = idx / uDim;
		int u = idx - v * uDim;

		if(patch->checkPointIgnoreBackface(perspective, pnt, u, v, uDim, vDim, idx, matrix, dpoint)){
			MultiSelect::clickedIndex = i;
			MultiSelect::beginDragPos.length = MultiSelect::pointList.length;
			for(int i = MultiSelect::pointList.length; --i >= 0;){
				NurbsPatch* patch = MultiSelect::pointList[i].p;
				int idx = MultiSelect::pointList[i].i;
				MultiSelect::beginDragPos[i] = patch->dpoint[idx];
			}
			return;
		}
	}
	//check non selected points
	MultiSelect::clickedIndex = 0;
	MultiSelect::beginDragPos.length = MultiSelect::pointList.length = 0;
	NurbsPatch* patch = dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p);
	patch->index = MultiSelect::withinSightsIndex;

	tugFlag = true;
	findPartsCPReference(patch, smls);
}

void MultiSelect::calcPivotPoint(){
	autoPivotPoint = Vec3f();
	for(int i = beginDragPos.length; --i>=0;){
		autoPivotPoint += MultiSelect::beginDragPos[i];
	}
	autoPivotPoint /= beginDragPos.length;
}
int NurbsPatch::getSelRing()const{
	int dimU = this->dim.x;
	return index[0]/dimU;
}
Vec3f NurbsPatch::calcRingPivotPoint(int v)const{
	int dimU = dim.x;
	int from = v * dimU;

	Vec3f pivotPoint;
	for(int i = dimU; --i>=0;){
		pivotPoint += MultiSelect::beginDragPos[from + i];
	}
	return pivotPoint /= dimU;
}

bool twoLineSegmentsIntersect(Vec2f AS, Vec2f AE, Vec2f BS, Vec2f BE);

bool clickLine(Vec2f click, Vec2f ptA, Vec2f ptB, Vec2f& crossPoint){
	Vec2f normal = calcNormal(ptA, ptB);
	if(normal == Vec2f()){
		return false;
	}
	Vec2f clickA = click + normal * g_mouseSense;
	Vec2f clickB = click + normal * -g_mouseSense;

	if(twoLineSegmentsIntersect(clickA, clickB, ptA, ptB)){
		crossPoint = lineIntersect(clickA, clickB, ptA, ptB);
		return true;
	}
	return false;
}

float getMidPointRatio(Vec2f ptA, Vec2f ptB, Vec2f crossPoint){
	float xDiff = ptB.x - ptA.x;
	float yDiff = ptB.y - ptA.y;
	if(fabs(xDiff) > fabs(yDiff)){
		return (crossPoint.x - ptA.x) / xDiff;
	}
	else{
		return (crossPoint.y - ptA.y) / yDiff;
	}
}

void updateNurbsCurveKnotMid0(int dim, M_Float& knot);
void updateNurbsCurveKnotMid1(int dim, M_Float& knot);


void NurbsLine::insertPoint(Vec3f newPoint, int k, float ratio){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();
	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Float& weight = this->weight.refValue();
	M_Color3f& color = this->color.refValue();
	M_Float& knot = this->uKnot.refValue();


	Color3f clA = color[k];
	Color3f clB = color[k + 1];
	Color3f newColor = clA + (clB - clA) * ratio;

	cpIndex.insertElement(k + 1, cp.length);

	cp += newPoint;
	cw += 1;
	dpoint.insertElement(k + 1, newPoint);
	point.insertElement(k + 1, newPoint);
	weight.insertElement(k + 1, 1);
	color.insertElement(k + 1, newColor);
	updateNurbsCurveKnotMid0(point.length, knot);
	processFields();
}

bool NurbsLine::insertControlPoint(){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Float& weight = this->weight.refValue();
	M_Color3f& color = this->color.refValue();
	M_Float& knot = this->uKnot.refValue();

	for(int k = 0; k < dpoint.length - 1; k++){

		Vec3f dpA = dpoint[k];
		Vec3f dpB = dpoint[k + 1];

		Vec3f mdpA = g_viewMatrix * dpA;
		Vec3f mdpB = g_viewMatrix * dpB;

		Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		Vec2f ptA = Vec2f(mdpA.x, mdpA.y);
		Vec2f ptB = Vec2f(mdpB.x, mdpB.y);
		Vec2f crossPoint;

		if(clickLine(click, ptA, ptB, crossPoint)){
			float ratio = getMidPointRatio(ptA, ptB, crossPoint);
#ifdef PROG_ROB
			g_storeCrossPoint.set(fromPoint + k, fromPoint + k + 1, ratio);
#endif

			Vec3f newPoint = dpA + (dpB - dpA) * ratio;

			if(mirror){
				Vec3f newPointMirror = newPoint;
				newPointMirror.x = - newPointMirror.x;

				int uDim = dim.x;
				if(NurbsLine* mirrorLine = dynamic_cast<NurbsLine*>(mirrorPatch)){
					insertPoint(newPoint, k, ratio);
					mirrorLine->insertPoint(newPointMirror, k, ratio);
				}
				else{
					int km = this->getMirrorIdx(k) - 1;
					if(k <= (uDim >> 1)){
						insertPoint(newPointMirror, km, 1.0 - ratio);
						insertPoint(newPoint, k, ratio);
					}
					else{
						insertPoint(newPoint, k, ratio);
						insertPoint(newPointMirror, km, 1.0 - ratio);
					}
				}
			}
			else{
				insertPoint(newPoint, k, ratio);
			}


			int idx = k + 1;
			MultiSelect::withinSightsIndex = idx;
			MultiSelect::withinSightsNode = this;
			NurbsPatch::index[0] = idx;
			smls->populateMirrorNodes();
			smls->doBuild();
			g_insertControlPoint = true;
			/*
			index[0] = idx;
			NurbsPatch::index[0] = idx;
			//uKnot.refValue().length = 0;
			smls->populateMirrorNodes();

			smls->doBuild();
			g_insertControlPoint = true;

			MultiSelect::pointList = ControlPoint(this, idx);
			MultiSelect::beginDragPos = dpoint[idx];
			MultiSelect::clickedIndex = 0;
			MultiSelect::selectedPoint = 0;
			initMultiCageTug2(this);

			*/

			MultiSelect::selectedPoint = 0;
			tugFlag = true;
			showCursor(false);
			g_pointer.translation = g_viewMatrix * newPoint;
			g_cursorOverPointIlsBase->translation = g_pointer.translation;
			MultiSelect::withinSightsNode = this;
			index[0] = MultiSelect::withinSightsIndex;
			MultiSelect::beginDragPos = newPoint;
			MultiSelect::pointList = ControlPoint(this, MultiSelect::withinSightsIndex);
			MultiSelect::clickedIndex = 0;
			//initMultiCageTug2(this);


			g_transformCP->translation.refValue() = Vec3f();
			g_transformCP->remTranslation = Vec3f();
			g_transformCP->clickPoint = MultiSelect::beginDragPos[MultiSelect::clickedIndex];
			g_transformCP->calcPivotPoint();


			houseKeep();

			return true;
		}
	}
	return false;
}

void addNewPointsAndMakeRedirectTable(Seamless* smls, const M_<NurbsPatchPtr>& m_selPatchList, M_Int& redirect);


void cutNewLine(M_Vec3f& cp, M_Float& cw, M_Vec3f& dpoint, M_Vec3f& point, M_Int& cpIndex, M_Float& weight, M_Color3f& color, M_Float& knot, Vec3f newPoint, Color3f newColor, int k){
	point.deleteElement(0, k + 1);
	dpoint.deleteElement(0, k + 1);
	cpIndex.deleteElement(0, k + 1);
	weight.deleteElement(0, k + 1);
	color.deleteElement(0, k + 1);

	dpoint.insertElement(0, newPoint);
	point.insertElement(0, newPoint);
	cpIndex.insertElement(0, cp.length);
	cp += newPoint;
	cw += 1;
	weight.insertElement(0, 1);
	color.insertElement(0, newColor);
	updateNurbsCurveKnotMid0(point.length, knot);
}

void NurbsLine::cut(){
	g_insertPatch.cutMode = false;

	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Float& weight = this->weight.refValue();
	M_Color3f& color = this->color.refValue();
	M_Float& knot = this->uKnot.refValue();

	for(int k = 0; k < dpoint.length - 1; k++){

		Vec3f dpA = dpoint[k];
		Vec3f dpB = dpoint[k + 1];

		Vec3f mdpA = g_viewMatrix * dpA;
		Vec3f mdpB = g_viewMatrix * dpB;

		Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		Vec2f ptA = Vec2f(mdpA.x, mdpA.y);
		Vec2f ptB = Vec2f(mdpB.x, mdpB.y);
		Vec2f crossPoint;

		if(clickLine(click, ptA, ptB, crossPoint)){
			float ratio = getMidPointRatio(ptA, ptB, crossPoint);
#ifdef PROG_ROB
			g_storeCrossPoint.set(fromPoint + k, fromPoint + k + 1, ratio);
#endif
			Color3f clA = color[k];
			Color3f clB = color[k + 1];
			Color3f newColor = clA + (clB - clA) * ratio;

			Vec3f newPoint = dpA + (dpB - dpA) * ratio;
			//////
			NurbsLine* newLine = dynamic_cast<NurbsLine*>(this->createCopyOf());

			smls->build.refValue() += newLine;

			newLine->processFields();

			cutNewLine(cp, cw, newLine->dpoint.refValue(), newLine->point.refValue(), newLine->cpIndex.refValue(), newLine->weight.refValue(), newLine->color.refValue(), newLine->uKnot.refValue(), newPoint, newColor, k);

			newLine->processFields();
			//////


			point.length = k + 1;
			dpoint.length = k + 1;
			cpIndex.length = k + 1;
			weight.length = k + 1;
			color.length = k + 1;




			dpoint += newPoint;
			point += newPoint;
			cpIndex += cp.length;
			cp += newPoint;
			cw += 1;
			weight += 1;
			color += newColor;
			updateNurbsCurveKnotMid0(point.length, knot);
			processFields();



			/////


			if(mirror){
				Vec3f newPointMirror = newPoint;
				newPointMirror.x = - newPointMirror.x;

				int uDim = dim.x;
				if(NurbsLine* mirrorLine = dynamic_cast<NurbsLine*>(mirrorPatch)){
					insertPoint(newPoint, k, ratio);
					mirrorLine->insertPoint(newPointMirror, k, ratio);
				}
				else{
					int km = this->getMirrorIdx(k) - 1;
					if(k <= (uDim >> 1)){
						insertPoint(newPointMirror, km, 1.0 - ratio);
						insertPoint(newPoint, k, ratio);
					}
					else{
						insertPoint(newPoint, k, ratio);
						insertPoint(newPointMirror, km, 1.0 - ratio);
					}
				}
			}



			index[0] = k;
			//uKnot.refValue().length = 0;
			g_stw->genNodeList();
			smls->populateMirrorNodes();
			g_stw->commonPostInsertNodeStuff();

			smls->doBuild();
			houseKeepVisualClue();
			
			break;
		}
	}
}





void Strand::insertControlPoint(){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Int& cpIndex = this->cpIndex.refValue();
	M_Float& weight = this->weight.refValue();
	M_Float& knot = this->vKnot.refValue();

	for(int k = 0; k < dpoint.length - 1; k++){

		Vec3f dpA = dpoint[k];
		Vec3f dpB = dpoint[k + 1];

		Vec3f mdpA = g_viewMatrix * dpA;
		Vec3f mdpB = g_viewMatrix * dpB;

		Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		Vec2f ptA = Vec2f(mdpA.x, mdpA.y);
		Vec2f ptB = Vec2f(mdpB.x, mdpB.y);
		Vec2f crossPoint;

		if(clickLine(click, ptA, ptB, crossPoint)){
			float ratio = getMidPointRatio(ptA, ptB, crossPoint);
#ifdef PROG_ROB
			g_storeCrossPoint.set(fromPoint + k, fromPoint + k + 1, ratio);
#endif

			Vec3f newPoint = dpA + (dpB - dpA) * ratio;


			cpIndex.insertElement(k + 1, cp.length);

			cp += newPoint;
			cw += 1;
			dpoint.insertElement(k + 1, newPoint);
			point.insertElement(k + 1, newPoint);
			weight.insertElement(k + 1, 1);
			this->dim.refValue().y = point.length;
			updateNurbsCurveKnotMid0(point.length, knot);
			


			index[0] = k + 1;
			//uKnot.refValue().length = 0;
			processFields();
			int selLine = g_stw->selLine;
			Hair* hair = 0;
			Art* part = 0;
			if(hair = dynamic_cast<Hair*>(getParentNode(selLine))){
				hair->processFields();
				hair->refreshHairOffset();
				part = hair->part.refValue();
			}

			smls->populateMirrorNodes();
			//geometry->synchronizeCellsAndBuild();

			g_insertControlPoint = true;
			resetCPNodeList();


			part->deleteGeometry();
			part->smls->convertToRenderFormat();
			makePointerEqualIndexLocation();


			smls->ctrf = true;
			hair->doBuild();
			houseKeepVisualClue();



			break;
		}
	}
}

void NurbsLathe::insertControlPoint(){
	DPoint& dpoint = this->dpoint;
	M_Vec2f& point = this->point;
	M_Float& weight = this->weight;
	M_Float& knot = this->knot;

	for(int k = 0; k < dpoint.length - 1; k++){

		//Vec2f click3f = g_pointer.translation;
		Vec2f ptA2f = dpoint[k];
		Vec2f ptB2f = dpoint[k + 1];

		Matrix matrix = this->matrix * g_viewMatrix;

		Vec3f ptA3f = matrix * Vec3f(ptA2f.x, ptA2f.y, 0);
		Vec3f ptB3f = matrix * Vec3f(ptB2f.x, ptB2f.y, 0);

		Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		Vec2f ptA = Vec2f(ptA3f.x, ptA3f.y);
		Vec2f ptB = Vec2f(ptB3f.x, ptB3f.y);
		Vec2f crossPoint;
		if(clickLine(click, ptA, ptB, crossPoint)){
			
			float ratio = getMidPointRatio(ptA, ptB, crossPoint);
#ifdef PROG_ROB
			g_storeCrossPoint.set(k, k + 1, ratio);
#endif

			Vec2f newPoint = ptA2f + (ptB2f - ptA2f) * ratio;

			point.insertElement(k + 1, newPoint);
			weight.insertElement(k + 1, 1);

			point.insertElement(k + 1, newPoint);
			weight.insertElement(k + 1, 1);

			bands.refValue() = point.length / 2 * patch->smls->tesselate;
			updateNurbsCurveKnotMid1(point.length, knot);
			index[0] = k + 2;
			processFields();

			if(mirrorContainer){
				newPoint.x = -newPoint.x;
				DPoint& dpoint = mirrorContainer->dpoint;
				M_Vec2f& point = mirrorContainer->point;
				M_Float& weight = mirrorContainer->weight;
				M_Float& knot = mirrorContainer->knot;

				point.insertElement(k + 1, newPoint);
				weight.insertElement(k + 1, 1);

				point.insertElement(k + 1, newPoint);
				weight.insertElement(k + 1, 1);

				updateNurbsCurveKnotMid1(point.length, knot);
				mirrorContainer->index[0] = k + 2;
				mirrorContainer->bands.refValue() = bands;
				mirrorContainer->processFields();
				mirrorContainer->patch->synchronizeCellsAndBuild();
			}


			patch->synchronizeCellsAndBuild();
			houseKeepVisualClue();
			g_insertControlPoint = true;
			break;
		}
	}
/*
		g_stw->textCell.cellActive = false;
		Art* newPart = createPartOfType(part);
		//newPart->pivotPoint.refValue() = g_pointer.translation - part->absolutePivotPoint;;
		Vec3f vz = g_scene->getViewMatrix() * part->pivotPointMatrix * Vec3f(0,0,0);
		g_pointer.translation.z = vz.z;
		Vec3f newAbsolutePivotPoint = g_scene->getIViewMatrix() * g_catchIMatrix * part->imatrix * g_pointer.translation;
		Vec3f newPivotPoint = newAbsolutePivotPoint - part->absolutePivotPoint;
		part->part.refValue() += newPart;
		if(part->mirror){
			if(part->part.length == 1 && fabs(part->absolutePivotPoint.x) < .000001){
				if(fabs(newAbsolutePivotPoint.x) < 1 / g_scene->getZoom() * .05){
					newPivotPoint.x = 0;
				}
				else{
					addMirrorPart(part, newPivotPoint);
				}
			}
			else if(Art* mirrorPart = part->mirrorPart){
				addMirrorPart(mirrorPart, newPivotPoint);
			}
			else{
				addMirrorPart(part, newPivotPoint);
			}
		}
		newPart->pivotPoint.refValue() = newPivotPoint;
		
		part->smls->process();
		g_stw->genNodeList();
		g_stw->selLine = getNodeLine(newPart);
		g_stw->commonPostInsertNodeStuff();
		return;
	}
*/
}

void newNurbsTrackSegment(){
	NurbsTrack* nurbsTrack = dynamic_cast<NurbsTrack*>(g_selNode.getRaw());
	int atIndex = nurbsTrack->index[1];
	M_Float& key = nurbsTrack->key;
	M_Vec3f& keyValue = nurbsTrack->keyValue;
	M_Float& weight = nurbsTrack->weight;
	M_Float& knot = nurbsTrack->knot;
	M_Int& dim = nurbsTrack->dim;
	int d = -1;
	int i = 0;
	while(i + dim[++d] - 1 <= atIndex){
		i += dim[d] - 1;
	}
	int g = atIndex - i + 1;
	int s = dim[d];
	int h = s - g + 1;
	dim[d] = g;
	dim.insertElement(d + 1, h);

	float t = key[d] + (key[d + 1] - key[d]) * .5;
	key.insertElement(d + 1, t);

	updateNurbsCurveKnotMid0(keyValue.length, knot);
	nurbsTrack->processFields();
	updateBuild3dWindow();
	nurbsTrack->houseKeep();
}

void NurbsTrack::removeControlPoint(int atIndex){
	M_Float& key = this->key;
	M_Vec3f& keyValue = this->keyValue;
	M_Float& weight = this->weight;
	M_Float& knot = this->knot;
	M_Int& dim = this->dim;
	int d = -1;
	int i = 0;
	while(++d < dim.length && i + dim[d] - 1 <= atIndex){
		i += dim[d] - 1;
	}

	if(atIndex > 0 && i == atIndex){
		if(d == dim.length){
			d--;
			goto here;
		}
		d--;
		keyValue.deleteElement(atIndex);
		weight.deleteElement(atIndex);
		dim[d] += dim[d + 1] - 2;
		dim.deleteElement(d + 1);
		key.deleteElement(d * 2 + 1);
	}
	else{
here:
		if(dim[d] <= 3){
			return;
		}
		keyValue.deleteElement(atIndex);
		weight.deleteElement(atIndex);
		dim[d]--;
	}

	if(index[0] >= key.length){
		index[0]--;
	}
	if(atIndex >= keyValue.length){
		index[1]--;
	}
	if(index[2] >= dim.length){
		index[2]--;
	}

	updateNurbsCurveKnotMid0(keyValue.length, knot);
	processFields();
	updateBuild3dWindow();
	houseKeep();
}
void NurbsTrack::insertControlPoint(int atIndex, float ratio){
	M_Vec3f& keyValue = this->keyValue;
	M_Float& weight = this->weight;
	M_Float& knot = this->knot;

	Vec3f ptA3f = keyValue[atIndex];
	Vec3f ptB3f = keyValue[atIndex + 1];
	Vec3f newPoint = ptA3f + (ptB3f - ptA3f) * ratio;

	keyValue.insertElement(atIndex + 1, newPoint);
	weight.insertElement(atIndex + 1, 1);

	int d = -1;
	for(int i = 0; i <= atIndex + 1; i++){
		i += dim[++d] - 1;
	}
	dim.refValue()[d]++;

	index[1] = atIndex + 1;

	updateNurbsCurveKnotMid0(keyValue.length, knot);
	processFields();
	updateBuild3dWindow();
	houseKeep();
}
void NurbsTrack::insertControlPoint(){
	M_Vec3f& keyValue = this->keyValue;
	M_Float& weight = this->weight;
	M_Float& knot = this->knot;

	for(int k = 0; k < keyValue.length - 1; k++){
		Matrix matrix = g_viewMatrix;

		Vec3f ptA3f = matrix * keyValue[k];
		Vec3f ptB3f = matrix * keyValue[k + 1];

		Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		Vec2f ptA = Vec2f(ptA3f.x, ptA3f.y);
		Vec2f ptB = Vec2f(ptB3f.x, ptB3f.y);
		Vec2f crossPoint;
		if(clickLine(click, ptA, ptB, crossPoint)){
			
			float ratio = getMidPointRatio(ptA, ptB, crossPoint);

			insertControlPoint(k, ratio);

			g_insertControlPoint = true;
			break;
		}
	}
}

void updateNurbsCurveKnotMid1(int dim, M_Float& knot);
void updateNurbsCurveKnotMid2(int dim, M_Float& knot);

inline int evenEdge(int par);

bool allreadyInList(NurbsPatch* p, const int uOrVIdx, bool doU, float ratio, bool invertRatio, Vec2i cutLine, const M_Insert& m_insert){
	for(int k = m_insert.length; --k>=0;){
		const Insert& insert = m_insert[k];
		if(	insert.p == p &&
			insert.uOrVIdx == uOrVIdx &&
			insert.doU == doU &&
			insert.ratio == ratio &&
			insert.invertRatio == invertRatio &&
			insert.cutLine == cutLine
			){
			return true;
		}
	}
	return false;
}









void uGetOthersideIndices(const int u, int& checkIdx, int& checkIdx2, const int idx, const int idx2, const NurbsPatch* patch, const Vec2i& cutLine, const int uDim);
void vGetOthersideIndices(const int v, int& checkIdx, int& checkIdx2, const int idx, const int idx2, const NurbsPatch* patch, const Vec2i& cutLine, const int uDim);
bool beginsFromCenterLineFunc(int k, const Insert& insert);
bool endsOnCenterLineFunc(int k, int length, const Insert& insert);
bool lastEndsAtMirrorCenter(const M_Insert& m_insert);

void NurbsPatch::uInsertLine0(bool& foundFirstStitchPoly, const bool leftSide, const bool leaveNorthOrEast, const int u, const int v, float ratio, bool invertRatio, M_Insert& m_insert){
	int uDim = dim.x;
	int vDim = dim.y;

	int checkIdx = u;
	int checkIdx2 = checkIdx + 1;
	int originIdx = checkIdx + v * uDim;

	if(leaveNorthOrEast){
		checkIdx += (vDim - 1) * uDim;
		checkIdx2 += (vDim - 1) * uDim;
	}

	float rat = ratio;
	bool iRat = invertRatio;

	if(u & 1){
		swap(checkIdx, checkIdx2);
		originIdx++;
		//ratio = 1.0 - ratio;
		//invertRatio = !invertRatio;
		rat = 1.0 - ratio;
		iRat = !invertRatio;
	}
	if(v > 0 && v < vDim - 1){
		if(leaveNorthOrEast){
			originIdx -= uDim;
		}
		else{
			originIdx += uDim;
		}
	}
	//Vec2i cutLine = Vec2i(originIdx / uDim, checkIdx / uDim);
	Vec2i cutLine = Vec2i(v, 0);
	if(leaveNorthOrEast){
		cutLine = Vec2i(v, vDim - 2);
	}


	m_insert.length = 0;
	m_insert += Insert(this, u, true, rat, iRat, leaveNorthOrEast, cutLine);

	if(foundFirstStitchPoly){
		return;
	}

	Vec2i cutLine2 = Vec2i(v, 0);
	if(leaveNorthOrEast == false){
		cutLine2 = Vec2i(v, vDim - 2);
	}

	int oppositeIdx = 0;
	int oppositeIdx2 = 0;
	uGetOthersideIndices(u, oppositeIdx, oppositeIdx2, checkIdx, checkIdx2, this, cutLine2, uDim);

	int firstStitchPoly = smls->getStitchPoly(this, oppositeIdx, oppositeIdx2);

	smls->insertLineFindMatchingEdge(this, checkIdx, checkIdx2, false, leaveNorthOrEast, firstStitchPoly, foundFirstStitchPoly, leftSide, ratio, invertRatio, m_insert);
}


inline bool isMidControlLine(NurbsPatch* mirrorPatch, int u, int uDim){
	if(mirrorPatch == 0){
		int halfDim = uDim / 2;
		if(halfDim & 1){
			if(u == halfDim){
				return true;
			}
		}
	}
	return false;
}

void NurbsPatch::vInsertLine0(bool& foundFirstStitchPoly, const bool leftSide, const bool leaveNorthOrEast, const int u, const int v, float ratio, bool invertRatio, M_Insert& m_insert){
	int uDim = dim.x;
	int vDim = dim.y;

	int checkIdx = v * uDim;
	int checkIdx2 = checkIdx + uDim;
	int originIdx = checkIdx + u;

	if(leaveNorthOrEast){
		checkIdx += uDim - 1;
		checkIdx2 += uDim - 1;
	}

	float rat = ratio;
	bool iRat = invertRatio;

	if(v & 1){
		swap(checkIdx, checkIdx2);
		originIdx += uDim;
		rat = 1.0 - ratio;
		iRat = !invertRatio;
	}
	if(u > 0 && u < uDim - 1){
		if(false == isMidControlLine(mirrorPatch, u, uDim)){
			if(leaveNorthOrEast){
				originIdx--;
			}
			else{
				originIdx++;
			}
		}
	}
	//Vec2i cutLine = Vec2i(originIdx % uDim, checkIdx % uDim);
	Vec2i cutLine = Vec2i(u, 0);
	if(leaveNorthOrEast){
		cutLine = Vec2i(u, uDim - 2);
	}

	setToCenterLine0(cutLine);

	m_insert.length = 0;
	m_insert += Insert(this, v, false, rat, iRat, leaveNorthOrEast, cutLine);

	if(lastEndsAtMirrorCenter(m_insert)){
		return;
	}

	if(foundFirstStitchPoly){
		return;
	}



	Vec2i cutLine2 = Vec2i(u, 0);
	if(leaveNorthOrEast == false){
		cutLine2 = Vec2i(u, uDim - 2);
	}
	//crossesMiddleLine(cutLine2);

	int oppositeIdx = 0;
	int oppositeIdx2 = 0;
	vGetOthersideIndices(v, oppositeIdx, oppositeIdx2, checkIdx, checkIdx2, this, cutLine2, uDim);

	int firstStitchPoly = smls->getStitchPoly(this, oppositeIdx, oppositeIdx2);

	smls->insertLineFindMatchingEdge(this, checkIdx, checkIdx2, false, leaveNorthOrEast, firstStitchPoly, foundFirstStitchPoly, leftSide, ratio, invertRatio, m_insert);
}

void NurbsPatch::uDeleteLine(const int idx, const int idx2, const NurbsPatch* firstPatch, const int firstIdx2, bool& foundFirst, M_DelCtrlLn& delCtrlLn){
	int checkIdx = 0;
	int checkIdx2 = 0;
	int uDim = dim.x;
	int vDim = dim.y;
	int u = idx % uDim;
	int v = idx / uDim;
	delCtrlLn += DelCtrlLn(this, v, true);
	if(u == 0){
		checkIdx = idx + (uDim - 1);
		checkIdx2 = checkIdx - 1;
	}
	else{
		checkIdx = idx - (uDim - 1);
		checkIdx2 = checkIdx + 1;
	}
	smls->deleteLineFindMatchingEdge(this, checkIdx, checkIdx2, firstPatch, firstIdx2, foundFirst, delCtrlLn);
}

void NurbsPatch::vDeleteLine(const int idx, const int idx2, const NurbsPatch* firstPatch, const int firstIdx2, bool& foundFirst, M_DelCtrlLn& delCtrlLn){
	int checkIdx = 0;
	int checkIdx2 = 0;
	int uDim = dim.x;
	int vDim = dim.y;
	int u = idx % uDim;
	int v = idx / uDim;
	delCtrlLn += DelCtrlLn(this, u, false);
	if(foundFirst){
		return;
	}
	if(idx < uDim){
		checkIdx = idx + (vDim - 1) * uDim;
		checkIdx2 = checkIdx - uDim;
	}
	else{
		checkIdx = idx - (vDim - 1) * uDim;
		checkIdx2 = checkIdx + uDim;
	}

	smls->deleteLineFindMatchingEdge(this, checkIdx, checkIdx2, firstPatch, firstIdx2, foundFirst, delCtrlLn);
}
/*
void NurbsPatch::uInsertLine(const int idxA, M_Int& lookup, const NurbsPatch* firstPatch, int& foundFirstPatch, M_Int& deleteCtrlPolyList){
	doDim(par, ratio, dpoint, weight, order, dim, idx, uDim, vDim, lookup);

	uDim = dim.x;
	vDim = dim.y;

	if(NurbsPatch::MID_1 == uMid || NurbsPatch::MID_1 == vMid){
		Vec2f par2 = dim;
		if(NurbsPatch::MID_1 == uMid){
			if(uOldDim < uDim){
				par2.x = uDim + 1;
			}
		}
		if(NurbsPatch::MID_1 == vMid){
			if(vOldDim < vDim){
				par2.y = vDim + 1;
			}
		}
		doDim(par2, 0, dpoint, weight, order, dim, idx, uDim, vDim, lookup);
	}
	if(uOldDim < uDim){
		idx--;
		int uNewDim = dim.x;
		int uCorner = (idx + 2) % uNewDim;
		int checkIdx = (vDim - 2) * uNewDim + uCorner;
		smls->uWestFindStitchPolyAndSplit(deleteCtrlPolyList, this, checkIdx, checkIdx - 4);
		checkIdx = uCorner + uNewDim;
		smls->uEastFindStitchPolyAndSplit(deleteCtrlPolyList, this, checkIdx - 4, checkIdx);
	}
	else if(vOldDim < vDim){
		uDim = dim.x;
		vDim = dim.y;
		uIdx = idx % uDim;
		vIdx = idx / uDim;
		int checkIdx = (vIdx + 2) * uDim + 1;
		int checkIdx2 = checkIdx - uDim * 4;

		if(vIdx == 2){
			checkIdx -= uDim;
			checkIdx2;
		}

		smls->vWestFindStitchPolyAndSplit(deleteCtrlPolyList, this, checkIdx, checkIdx2);
		checkIdx += uDim - 3;
		//smls->vEastFindStitchPolyAndSplit(deleteCtrlPolyList, this, checkIdx - uDim * 4, checkIdx);
		smls->vFindStitchPolyAndSlideInnerEdge(this);
	}
	updateKnot(uOldDim, dim.x, uKnot, order.x);
	updateKnot(vOldDim, dim.y, vKnot, order.y);
		
	dpointsToPoints();
	synchronizeAllStemCells();

	processFields();
	synchronizeAllStemCells();
}
*/



void colorRowBlue(NurbsPatch* patch, M_Color3f& color, int row){
	int uDim = patch->dim.x;
	int offset = row * uDim + patch->fromPoint;
	for(int i = uDim; --i>=0;){
		color[offset++] = Color3f(0, 0, 1);
	}
}

void colorColBlue(NurbsPatch* patch, M_Color3f& color, int col){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int offset = col + patch->fromPoint;
	for(int i = vDim; --i>=0;){
		color[offset] = Color3f(0, 0, 1);
		offset+=uDim;
	}
}

void colorRowRed(NurbsPatch* patch, M_Color3f& color, int row){
	int uDim = patch->dim.x;
	int offset = row * uDim + patch->fromPoint;
	for(int i = uDim; --i>=0;){
		color[offset++] = Color3f(1, 0, 0);
	}
}

void colorColRed(NurbsPatch* patch, M_Color3f& color, int col){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	int offset = col + patch->fromPoint;
	for(int i = vDim; --i>=0;){
		color[offset] = Color3f(1, 0, 0);
		offset+=uDim;
	}
}

void addVisualClue2(M_DelCtrlLn& m_delCtrlLn, M_Color3f& color){
	for(int k = 0; k < m_delCtrlLn.length; k++){
		NurbsPatch* patch = m_delCtrlLn[k].p;
		int uOrVIdx = m_delCtrlLn[k].uOrVIdx;
		if(m_delCtrlLn[k].doU){
			colorRowBlue(patch, color, uOrVIdx);
		}
		else{
			colorColBlue(patch, color, uOrVIdx);
		}
	}
}

void addVisualClueForSelectRing(M_DelCtrlLn& m_delCtrlLn, M_Color3f& color){
	for(int k = 0; k < m_delCtrlLn.length; k++){
		NurbsPatch* patch = m_delCtrlLn[k].p;
		int uOrVIdx = m_delCtrlLn[k].uOrVIdx;
		if(m_delCtrlLn[k].doU){
			colorRowRed(patch, color, uOrVIdx);
		}
		else{
			colorColRed(patch, color, uOrVIdx);
		}
	}
}

bool isInLine(NurbsPatch* patch, int idx, M_DelCtrlLn& m_delCtrlLn){
	int uDim = patch->dim.x;
	int u = idx % uDim;
	int v = idx / uDim;

	for(int k = m_delCtrlLn.length; --k>=0;){
		if(m_delCtrlLn[k].p == patch){
			if(m_delCtrlLn[k].doU){
				if(m_delCtrlLn[k].uOrVIdx == v){
					return true;
				}
			}
			else{
				if(m_delCtrlLn[k].uOrVIdx == u){
					return true;
				}
			}
		}
	}
	return false;
}

void Seamless::doLastPhaseForSelectRing(){
	g_deleteControlLine.mousePhase = 0;

	if(MultiSelect::accumulate == false){
		MultiSelect::beginDragPos.length = 0;
		MultiSelect::pointList.length = 0;
	}
	M_<NurbsPatch*> contigList;  //make sure we dont add points twice by making contig list
	createSelContigList(contigList);

	M_DelCtrlLn& m_delCtrlLn = g_deleteControlLine.m_delCtrlLn;

	for(int k = 0; k < m_delCtrlLn.length; k++){
		NurbsPatch* patch = m_delCtrlLn[k].p;
		int uOrVIdx = m_delCtrlLn[k].uOrVIdx;
		if(m_delCtrlLn[k].doU){
			patch->selectRing(uOrVIdx, patch->dim.x, patch->dim.y, contigList);
		}
		else{
			patch->selectCol(uOrVIdx, patch->dim.x, patch->dim.y, contigList);
		}
	}


	for(int i = 0; i < ctrlPolyPoint.length;){
		int ctrlPoly = i;
		int len = ctrlPolyPoint[ctrlPoly].len;
		for(int k = len; --k >= 0;){
			NurbsPatch* patch = ctrlPolyPoint[i].p;
			int idx = ctrlPolyPoint[i].i;
			if(patch->cage && isInLine(patch, idx, m_delCtrlLn)){
				contigList[patch->fromPoint + idx] = patch;
			}
			i++;
		}
		i = getNextCtrlPoly(ctrlPoly);
	}
	MultiSelect::pointList.length = 0;
	for(int k = 0; k < pointTotal; k++){
		if(NurbsPatch* patch = contigList[k]){
			MultiSelect::pointList += ControlPoint(patch, k - patch->fromPoint);
		}
	}
	g_transformCP->calcPivotPoint();
	g_transformCP->resetRotationLever();

						

	g_transformCP->scale.refValue() = Vec3f(1, 1, 1);
	g_transformCP->scaleOrientation.refValue() = Rotation();
	//g_transformCP->pivotPoint.refValue() = pivotPoint;
	g_transformCP->rotation.refValue() = Rotation();
	g_transformCP->translation.refValue() = Vec3f();
	g_transformCP->resetRotationLever();

	g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx]; // do we need this?

	g_transformCP->houseKeep();

	houseKeepVisualClue();
}


void DeleteControlLine::addVisualClue(){
	M_Int& iFScoordIndex = g_nurbsIls->coordIndex.refValue();
	M_Color3f& color = g_nurbsIlsColor->color.refValue();

	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
	
	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];

		Color3f c = (g_transformCP->selectObject && patch->object) ? Color3f(1, 0, 0) : Color3f(0, 1, 1);
		int fromPoint = patch->fromPoint;
		for(int i = patch->dpoint.length; --i>=0;){
				color[fromPoint + i] = c;
		}
	}
	if(g_deleteControlLine.selectRing){
		addVisualClueForSelectRing(m_delCtrlLn, color);
	}
	else{
		addVisualClue2(m_delCtrlLn, color);
		addVisualClue2(mirror_delCtrlLn, color);
	}

//	addInsertVisualClue2(m_insert, iFScoordIndex, iFSpoint, iFScolor);
//	addInsertVisualClue2(mirror_insert, iFScoordIndex, iFSpoint, iFScolor);


	g_nurbsIls->convertToRenderFormat(g_nurbsIlsShape->render);
}

void NurbsPatch::colorEdge(int edge, const Color3f edgeColor){
	M_Int& iFScoordIndex = g_nurbsIls->coordIndex.refValue();
	M_Color3f& color = g_nurbsIlsColor->color.refValue();
	int uDim = dim.x;
	int vDim = dim.y;

	if(edge == SOUTH){
		for(int u = 0; u < uDim; u++){
				color[fromPoint + u] = edgeColor;
		}
	}
	if(edge == EAST){
		for(int v = 0; v < vDim; v++){
				color[fromPoint + v * uDim + uDim - 1] = edgeColor;
		}
	}
	if(edge == NORTH){
		int offset = (vDim - 1) * uDim;
		for(int u = 0; u < uDim; u++){
				color[fromPoint + u + offset] = edgeColor;
		}
	}
	if(edge == WEST){
		for(int v = 0; v < vDim; v++){
				color[fromPoint + v * uDim] = edgeColor;
		}
	}
}

void MergePatches::addVisualClue(const Color3f edgeColor){

	if(patchA){
		patchA->colorEdge(edgeA, edgeColor);
	}
	if(patchB){
		patchB->colorEdge(edgeB, edgeColor);
	}


	g_nurbsIls->convertToRenderFormat(g_nurbsIlsShape->render);
}






/*

void commonInsertLine(Seamless* smls, M_Insert& m_insert){

	//bubbleSortInsertData(insert);

	for(int k = 0; k < m_insert.length; k++){
		NurbsPatch* patch = m_insert[k].p;

		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		M_Vec3f& dpoint = patch->dpoint.refValue();
		M_Float& weight = patch->weight.refValue();

		M_Float& uKnot = patch->uKnot;
		M_Float& vKnot = patch->vKnot;
		M_Vec3f oldPoints;
		M_Float oldWeights;
		NurbsPatch* newPatch = 0;

		Insert& insert = m_insert[k];
		int uOrVIdx = insert.uOrVIdx;
		bool doU = insert.doU;
		adjustInsert(m_insert, patch, uOrVIdx, doU);
		if(doU){
			if(g_insertPatch.centerMirrorLine){
				if(uOrVIdx == (uDim >> 1) - 1 || uOrVIdx == (uDim >> 1)){
					patch->insertCol(oldPoints, oldWeights, (uDim >> 1) - 1, .5, uDim, g_insertPatch.centerMirrorLine);
					patch->insertCol(oldPoints, oldWeights, (uDim >> 1), .5, uDim, g_insertPatch.centerMirrorLine);
				}
				else{
					patch->insertCol(oldPoints, oldWeights, 0, 0, uDim, g_insertPatch.centerMirrorLine);
					patch->insertCol(oldPoints, oldWeights, 0, 0, uDim, g_insertPatch.centerMirrorLine);

					uSetMidPosition(patch, -1, newPatch);
				}
			}
			else{
				float ratio = insert.ratio;
				if(patch->mirror && g_insertPatch.centerMirrorLine == false && patch->mirrorPatch == 0){ //make sure we have west side
					if(uOrVIdx >= uDim >> 1){
						uOrVIdx = patch->getMirrorTwinlessIdx(uOrVIdx) - 1;
						ratio = 1.0 - ratio;
					}
				}
				patch->insertCol(oldPoints, oldWeights, uOrVIdx, ratio, uDim, g_insertPatch.centerMirrorLine);
				patch->insertCol(oldPoints, oldWeights, uOrVIdx + 1, 0, uDim, g_insertPatch.centerMirrorLine);
				uSetMidPosition(patch, uOrVIdx, newPatch);
			}
			patch->uUpdateKnotMid1();

			uOrVIdx = insert.uOrVIdx;

			if(insert.diversion){
				bool leaveNorthOrEast = insert.leaveNorthOrEast;
				int prevIdx2 = 0;

				int idx3 = uOrVIdx;
				int idx2 = uOrVIdx - 1;
				if(uOrVIdx & 1){
					idx3 = uOrVIdx - 1;
					idx2 = uOrVIdx;
				}
				if(leaveNorthOrEast == false){//north ?
					idx3 += (vDim - 1) * uDim;
					idx2 += (vDim - 1) * uDim;
				}

				int prev_uOrVIdx = m_insert[k - 1].uOrVIdx;

				NurbsPatch* prevPatch = m_insert[k - 1].p;
				int prev_uDim = prevPatch->dim.x;
				int prev_vDim = prevPatch->dim.y;
				bool prevDoU = m_insert[k - 1].doU;
				bool prevNorthOrEast = m_insert[k - 1].leaveNorthOrEast;
				int prevCenterIdx = m_insert[k - 1].uOrVIdx;
//				prevCenterIdx = insert.prevUorVCenterIdx;
//				prevCenterIdx -= 2;
				if(prevDoU){
					if(prevNorthOrEast){//north
						prevCenterIdx += (prev_vDim - 1) * prev_uDim;
					}
					prevIdx2 = prevCenterIdx - 1;
					//prevIdx3 = prevIdx2 + 1;

					if(prev_uOrVIdx & 1){
						prevIdx2 = prevCenterIdx + 1;
						//prevIdx3 = prevIdx2 - 1;
					}
				}
				else{
					prevCenterIdx = prevCenterIdx * prev_uDim;
					if(prevNorthOrEast){//east
						prevCenterIdx += prev_uDim - 1;
					}
					prevIdx2 = prevCenterIdx + prev_uDim;
					//prevIdx3 = prevIdx2 + prev_uDim;


					if(prev_uOrVIdx & 1){
						prevIdx2 = prevCenterIdx - prev_uDim;
						//prevIdx3 = prevIdx2 - prev_uDim;
					}
				}


				NurbsPatch* otherPatch = insert.otherPatch;
				//int otherCenterIdx = insert.otherIdx;
				int otherCenterIdx = convertToActualFormat(insert.otherUV, otherPatch->dim.x);
				Vec3f prevVec = prevPatch->dpoint[prevCenterIdx];
				Vec3f prevVec2 = prevPatch->dpoint[prevIdx2];
				otherPatch->dpoint.refValue()[otherCenterIdx] = prevVec;
				patch->dpoint.refValue()[idx3] = prevVec;
				patch->dpoint.refValue()[idx2] = prevVec2;
				otherPatch->dpointsToPoints();
				mirrorPatchForInsert(otherPatch, otherCenterIdx, true);
				
			}
		}
		else{
			patch->insertRow(oldPoints, oldWeights, uOrVIdx, insert.ratio, vDim);
			patch->insertRow(oldPoints, oldWeights, uOrVIdx + 1, 0, vDim);
			vSetMidPosition(patch, uOrVIdx);
			patch->vUpdateKnotMid1();

			uOrVIdx = insert.uOrVIdx;

			if(insert.diversion){
				

				//bool northOrEast = m_diversionMark[diversionMarkIdx].northOrEast;



				bool leaveNorthOrEast = insert.leaveNorthOrEast;


				int prevIdx2 = 0;


				int idx3 = uOrVIdx * uDim;
				int idx2 = (uOrVIdx - 1) * uDim;
				if(uOrVIdx & 1){
					idx3 = (uOrVIdx - 1) * uDim;
					idx2 = uOrVIdx * uDim;
				}
				if(leaveNorthOrEast == false){//north ?
					idx3 += uDim - 1;
					idx2 += uDim - 1;
				}


				//int idx3 = (uOrVIdx + 2) * uDim;
				//int idx2 = (uOrVIdx + 1) * uDim;
				//if(uOrVIdx & 1){
				//	idx3 = (uOrVIdx + 1) * uDim;
				//	idx2 = (uOrVIdx + 2) * uDim;
				//}
				//if(leaveNorthOrEast == false){//dealing with East edge
				//	idx3 += uDim - 1;
				//	idx2 += uDim - 1;
				//}
//				idx3 = 8;
//				idx2 = 5;

				int prev_uOrVIdx = m_insert[k - 1].uOrVIdx;



				NurbsPatch* prevPatch = m_insert[k - 1].p;
				int prev_uDim = prevPatch->dim.x;
				int prev_vDim = prevPatch->dim.y;
				bool prevDoU = m_insert[k - 1].doU;
				bool prevNorthOrEast = m_insert[k - 1].leaveNorthOrEast;
				int prevCenterIdx = m_insert[k - 1].uOrVIdx;
//				prevCenterIdx = insert.prevUorVCenterIdx;
				//prevCenterIdx-= 2;
				if(prevDoU){
					if(prevNorthOrEast){//north
						prevCenterIdx += (prev_vDim - 1) * prev_uDim;
					}
					prevIdx2 = prevCenterIdx - 1;
					//prevIdx3 = prevIdx2 + 1;

					if(prev_uOrVIdx & 1){
						prevIdx2 = prevCenterIdx + 1;
						//prevIdx3 = prevIdx2 - 1;
					}
				}
				else{
					prevCenterIdx = prevCenterIdx * prev_uDim;
					if(prevNorthOrEast){//east
						prevCenterIdx += prev_uDim - 1;
					}
					prevIdx2 = prevCenterIdx + prev_uDim;
					//prevIdx3 = prevIdx2 + prev_uDim;


					if(prev_uOrVIdx & 1){
						prevIdx2 = prevCenterIdx - prev_uDim;
						//prevIdx3 = prevIdx2 - prev_uDim;
					}
				}


				NurbsPatch* otherPatch = insert.otherPatch;
				//int otherCenterIdx = insert.otherIdx;
				int otherCenterIdx = convertToActualFormat(insert.otherUV, otherPatch->dim.x);

//				otherCenterIdx = 0;
//				idx3 = 8;
//				idx2 = 5;
//				prevCenterIdx = 32;
//				prevIdx2 = 31;

//				otherCenterIdx = 0;
//				idx3 = 5;
//				idx2 = 2;
//				prevCenterIdx = 31;
			//	prevIdx2 = 30;

				Vec3f prevVec = prevPatch->dpoint[prevCenterIdx];
				Vec3f prevVec2 = prevPatch->dpoint[prevIdx2];
				otherPatch->dpoint.refValue()[otherCenterIdx] = prevVec;
				patch->dpoint.refValue()[idx3] = prevVec;
				patch->dpoint.refValue()[idx2] = prevVec2;
				otherPatch->dpointsToPoints();
				mirrorPatchForInsert(otherPatch, otherCenterIdx, true);
				patch->dpointsToPoints();
				mirrorPatchForInsert(patch, uOrVIdx, true);
				
			}

		}
		
		
		patch->dpointsToPoints();
		if(patch->isCtrlPolyRef){
			smls->generateRawStitchPolys();
		}

		//patch->synchronizeAllStemCells();
		patch->processFields();
		//patch->synchronizeAllStemCells();
		patch->synchronizeCellsAndBuild();

		if(m_insert[k].doU){
			mirrorPatchForInsert(patch, 0, m_insert[k].doU);
		}
		else{
			mirrorPatchForInsert(patch, uOrVIdx, m_insert[k].doU);
		}
		if(newPatch){
			//newPatch->synchronizeAllStemCells();
			newPatch->dpointsToPoints();
			if(newPatch->isCtrlPolyRef){
				smls->generateRawStitchPolys();
			}
			//newPatch->synchronizeAllStemCells();
			newPatch->processFields();
			newPatch->synchronizeCellsAndBuild();
			//patch->synchronizeAllStemCells();
			//bnspeBuildPolies(newPatch);
		}

		if(Strand* strand = dynamic_cast<Strand*>(patch)){
			strand->index[0] = patch->index[0];
			if(Hair* hair = dynamic_cast<Hair*>(getParentNode(getNodeLine(patch)))){
				hair->insertStrandPoint(strand, strand->index[0]);
				hair->refreshHairOffset();
			}
		}
		//adjustInsert(m_insert, patch, uOrVIdx);
	}
	smls->populateMirrorNodes();
	MultiSelect::pointList.length = 0;


	fastFullBuildForAutoIn(true, smls);
	smls->doBuild();
	smls->stitchAllPolys();
	for(int k = m_insert.length; --k >= 0;){
		NurbsPatch* patch = m_insert[k].p;
		bnspeBuildPolies(patch);
	}

	smls->ctrf = true;
	houseKeepVisualClue();
	BackUp::monostable.beginCycle();
}
*/
bool isContained(NurbsPatch* patch, M_Insert& insert){
	for(int k = insert.length; --k>= 0;){
		if(insert[k].p == patch){
			return true;
		}
	}
	return false;
}

void NurbsPatch::uInsertLine2(float ratio, M_Vec3f& dpoint, int uDim, int vDim, int u, int v, bool leaveNorthOrEast, bool leftSide){
	g_insertPatch.clickedPolyEdge.length = 0;
	bool invertRatio = false;
	g_insertPatch.foundFirstStitchPoly = false;

		
	if(mirror){
		if(mirrorPatch){
			M_Vec3f& mirror_dpoint = mirrorPatch->dpoint.refValue();
			if(
				(u == uDim - 2 && mirror_dpoint[uDim - 1].x == 0 && dpoint[uDim - 1].x == 0)
				){
				if(ratio > .75){
					ratio = 1;
					g_insertPatch.centerMirrorLine = true;
				}
			}
			else if(
				(u == 0 && dpoint[0].x == 0 && mirror_dpoint[0].x == 0)
				){
				if(ratio < .25){
					ratio = 0;
					g_insertPatch.centerMirrorLine = true;
				}
			}
		}
		else{
			if(
				(u == (uDim >> 1) - 1 ) ||
				(u == uDim - 2 && dpoint[0].x == 0 && dpoint[uDim - 1].x == 0)
				){
				if(ratio > .75){
					ratio = 1;
					g_insertPatch.centerMirrorLine = true;
				}
			}
			else if(
				(u == (uDim >> 1)) ||
				(u == 0 && dpoint[0].x == 0 && dpoint[uDim - 1].x == 0)
				){
				if(ratio < .25){
					ratio = 0;
					g_insertPatch.centerMirrorLine = true;
				}
			}
		}
	}

	int checkIdx = u;
	int checkIdx2 = u + 1;
	if(leaveNorthOrEast == false){
		checkIdx += (vDim - 1) * uDim;
		checkIdx2 += (vDim - 1) * uDim;
	}

	if(u & 1){
		swap(checkIdx, checkIdx2);
		ratio = 1.0 - ratio;
		invertRatio = !invertRatio;
	}
	g_insertPatch.remRatio = ratio;
	g_insertPatch.remInvertRatio = invertRatio;
	int mirrorIdx = 0;
	int mirrorIdx2 = 0;
	if(mirror && mirrorPatch == 0){
		mirrorIdx = getMirrorTwinlessIdx(checkIdx);
		mirrorIdx2 = getMirrorTwinlessIdx(checkIdx2);
	}
	int firstStitchPoly = smls->getStitchPoly(this, checkIdx, checkIdx2);

	//leftSide = false;
	//if(dpoint[checkIdx].x == 0){
	//	if(dpoint[checkIdx2].x < 0){
	//		if(leftSide != true){
	//			leftSide = leftSide;
	//		}
	//		leftSide = true;
	//	}
	//}
	//else if(dpoint[checkIdx].x < 0){
	//		if(leftSide != true){
	//			leftSide = leftSide;
	//		}
	//	leftSide = true;
	//}
	//else if(leftSide != false){
	//	leftSide = leftSide;
	//}
	g_insertPatch.remLeftSide = leftSide;
	uInsertLine0(g_insertPatch.foundFirstStitchPoly, leftSide, leaveNorthOrEast, u, v, ratio, invertRatio, g_insertPatch.m_insert);

	if(g_insertPatch.foundFirstStitchPoly == false && v > 0 && v < vDim - 1){

		leaveNorthOrEast = !leaveNorthOrEast;

		uInsertLine0(g_insertPatch.foundFirstStitchPoly, leftSide, leaveNorthOrEast, u, v, ratio, invertRatio, g_insertPatch.m_insertOD);

		if(g_insertPatch.m_insertOD.length == 1){
			if(g_insertPatch.m_insertOD[0].p == g_insertPatch.m_insert[0].p){
				g_insertPatch.m_insertOD.length = 0;
				if(g_insertPatch.m_insert[0].leaveNorthOrEast){
					g_insertPatch.m_insert[0].cutLine.x = 0;
				}
				else{
					g_insertPatch.m_insert[0].cutLine.x = g_insertPatch.m_insert[0].p->dim.y - 1;
				}
			}
		}

		//sortDirectionsOut(leaveNorthOrEast, u, uDim);
	}
	if(g_insertPatch.m_insert.length){
		g_insertPatch.m_insert[0].first = true;
		g_insertPatch.init_idx = u + v * uDim;
		g_insertPatch.init_patch = this;
		g_insertPatch.init_doU = true;
		g_insertPatch.init_leaveNorthOrEast = g_insertPatch.m_insert[0].leaveNorthOrEast;
		g_insertPatch.init_ratio = ratio;
	}
	if(g_insertPatch.m_insertOD.length > 1){
		g_insertPatch.sortDirectionsOut(g_insertPatch.m_insert[0].leaveNorthOrEast, g_insertPatch.m_insert[0].uOrVIdx, g_insertPatch.m_insert[0].p->dim.y);
		g_insertPatch.m_insertOD.length = 0;
	}

	g_insertPatch.mouseOverPatch = this;
}

bool NurbsPatch::uInsertLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, int uDim, int vDim, Vec2f& dim){

	NurbsPatch* mirrorPatch = this->mirrorPatch;

	g_insertPatch.centerMirrorLine = false;

	if(this->cage == false){
		return false;
	}

	int u = idxA % uDim;

	Vec3f ldpA = dpoint[idxA];
	Vec3f ldpB = dpoint[idxB];
	Vec3f dpA = matrix * ldpA;
	Vec3f dpB = matrix * ldpB;

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(dpA.x, dpA.y);
	Vec2f ptB = Vec2f(dpB.x, dpB.y);
	Vec2f crossPoint;

	if(clickLine(click, ptA, ptB, crossPoint)){
		float ratio = getMidPointRatio(ptA, ptB, crossPoint);
#ifdef PROG_ROB
		g_storeCrossPoint.set(fromPoint + idxA, fromPoint + idxB, ratio);
#endif
		bool leftSide = false;
		if(ldpA.x < 0){
			leftSide = true;
		}
		//g_insertPatch.remPatch = this;
		//g_insertPatch.remRowCol = col;
		//g_insertPatch.remDo_uInsert = true;
		int v = idxA / uDim;

		bool leaveNorthOrEast = false;
		if(v < (vDim >> 1)){
			leaveNorthOrEast = true;
		}

		uInsertLine2(ratio, dpoint, uDim, vDim, u, v, leaveNorthOrEast, leftSide);
		g_insertPatch.imousePhase = InsertPatch::LINE_MODE;
		return true;
	}
	return false;
}


void NurbsPatch::vInsertLine2(float ratio, M_Vec3f& dpoint, int uDim, int vDim, int u, int v, bool leaveNorthOrEast, bool leftSide){
	g_insertPatch.clickedPolyEdge.length = 0;
	bool invertRatio = false;
//		Vec3f newPoint = (dpB - dpA) * ratio;
	g_insertPatch.foundFirstStitchPoly = false;

	int originIdx = v * uDim + u + 1;
	if(leaveNorthOrEast == false){
		originIdx -= 2;
	}


	int checkIdx = v * uDim;
	int checkIdx2 = (v + 1) * uDim;
	if(leaveNorthOrEast == false){
		checkIdx += uDim - 1;
		checkIdx2 += uDim - 1;
	}

	if(v & 1){
		swap(checkIdx, checkIdx2);
		ratio = 1.0 - ratio;
		invertRatio = !invertRatio;
	}
	g_insertPatch.remRatio = ratio;
	g_insertPatch.remInvertRatio = invertRatio;

	int mirrorIdx = 0;
	int mirrorIdx2 = 0;
	if(mirror && mirrorPatch == 0){
		mirrorIdx = this->getMirrorTwinlessIdx(checkIdx);
		mirrorIdx2 = this->getMirrorTwinlessIdx(checkIdx2);
	}


	//if(g_transformCP->selectRing || g_transformCP->selectRings){
	//	if(Hair* hair = dynamic_cast<Hair*>(getParentNode(getNodeLine(this)))){
	//		Lock* lock = 0;
	//		int scpiIdx = 0;
	//		if(Strand* strand = dynamic_cast<Strand*>(this)){
	//			if(hair->isBoundToLock(strand, checkIdx2, lock, scpiIdx)){
	//				hair->insertStrandControlPoints(smls, lock, 1.0f - ratio);
	//				return true;
	//			}
	//		}
	//	}
	//}
	//int firstStitchPoly = smls->getStitchPoly(this, checkIdx, checkIdx2);
	//leftSide = false;
	//if(dpoint[checkIdx].x < 0){
	//	if(leftSide != true){
	//		leftSide = leftSide;
	//	}
	//	leftSide = true;
	//}
	//else if(leftSide != false){
	//	leftSide = leftSide;
	//}
	
//	vInsertLine(firstStitchPoly, foundFirstPatch, leftSide, checkIdx, checkIdx2, ratio, invertRatio, g_insertPatch.m_insert, -1, markClicked);

	int idx = v * uDim + u;
	if(mirror){
		float x = dpoint[idx].x;
		if(x > -CROSS_OVER_AMOUNT && x < CROSS_OVER_AMOUNT){
			if(leaveNorthOrEast){
				if(u + 1 < uDim){
					if(dpoint[idx + 1].x > 0){
						leftSide = false;
					}
				}
			}
			else{
				if(u - 1 >= 0){
					if(dpoint[idx - 1].x < 0){
						leftSide = true;
					}
				}
			}
		}
	}
	g_insertPatch.remLeftSide = leftSide;

	vInsertLine0(g_insertPatch.foundFirstStitchPoly, leftSide, leaveNorthOrEast, u, v, ratio, invertRatio, g_insertPatch.m_insert);

	//if(foundFirstPatch == false && u > 0 && u < uDim - 1 && beginsFromCenterLineFunc(0, g_insertPatch.m_insert[0]) == false){
	if(u > 0 && u < uDim - 1 && beginsFromCenterLineFunc(0, g_insertPatch.m_insert[0]) == false){

		leaveNorthOrEast = !leaveNorthOrEast;

		vInsertLine0(g_insertPatch.foundFirstStitchPoly, leftSide, leaveNorthOrEast, u, v, ratio, invertRatio, g_insertPatch.m_insertOD);

		if(g_insertPatch.m_insertOD.length == 1){
			if(g_insertPatch.m_insertOD[0].p == g_insertPatch.m_insert[0].p){
				g_insertPatch.m_insertOD.length = 0;
				NurbsPatch* p = g_insertPatch.m_insert[0].p;
				int pUdim = p->dim.x;
				if(p->mirror && p->mirrorPatch == 0 && g_insertPatch.m_insert[0].cutLine.x == pUdim / 2){
				}
				else{
					if(g_insertPatch.m_insert[0].leaveNorthOrEast){
						g_insertPatch.m_insert[0].cutLine.x = 0;
					}
					else{
						g_insertPatch.m_insert[0].cutLine.x = pUdim - 1;
					}
				}
			}
		}
		//sortDirectionsOut(leaveNorthOrEast, u, uDim);
	}
	if(g_insertPatch.m_insert.length){
		g_insertPatch.m_insert[0].first = true;
		g_insertPatch.init_idx = v * uDim + u;
		g_insertPatch.init_patch = this;
		g_insertPatch.init_doU = false;
		g_insertPatch.init_leaveNorthOrEast = g_insertPatch.m_insert[0].leaveNorthOrEast;
		g_insertPatch.init_ratio = ratio;
	}
	if(g_insertPatch.m_insertOD.length > 1){
		g_insertPatch.sortDirectionsOut(g_insertPatch.m_insert[0].leaveNorthOrEast, g_insertPatch.m_insert[0].uOrVIdx, g_insertPatch.m_insert[0].p->dim.y);
		g_insertPatch.m_insertOD.length = 0;
	}


	g_insertPatch.mouseOverPatch = this;
}

bool NurbsPatch::vInsertLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, int uDim, int vDim, Vec2f& dim){
	NurbsPatch* mirrorPatch = this->mirrorPatch;
	g_insertPatch.centerMirrorLine = false;

	if(this->cage == false){
		return false;
	}

	int v = idxA / uDim;

	Vec3f ldpA = dpoint[idxA];
	Vec3f ldpB = dpoint[idxB];
	Vec3f dpA = matrix * ldpA;
	Vec3f dpB = matrix * ldpB;

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(dpA.x, dpA.y);
	Vec2f ptB = Vec2f(dpB.x, dpB.y);
	Vec2f crossPoint;

	if(clickLine(click, ptA, ptB, crossPoint)){
		float ratio = getMidPointRatio(ptA, ptB, crossPoint);
#ifdef PROG_ROB
		g_storeCrossPoint.set(fromPoint + idxA, fromPoint + idxB, ratio);
#endif

		bool leftSide = false;
		if(ldpA.x < 0){
			leftSide = true;
		}
		//if(mirror && mirrorPatch == 0){
		//	int u = idxA % uDim;
		//	if(u == uDim / 2 + 1){
		//		leftSide = false;
		//		if(u == 0){
		//			leftSide = true;
		//		}
		//	}
		//}

		//g_insertPatch.remPatch = this;
		g_insertPatch.remRatio = ratio;
		//g_insertPatch.remRowCol = row;
		//g_insertPatch.remDo_uInsert = false;
		int u = idxA % uDim;

		bool leaveNorthOrEast = false;
		if(u < (uDim >> 1)){
			leaveNorthOrEast = true;
		}

		vInsertLine2(ratio, dpoint, uDim, vDim, u, v, leaveNorthOrEast, leftSide);
		g_insertPatch.imousePhase = InsertPatch::LINE_MODE;
		return true;
	}
	return false;
}

bool foundInList(const NurbsPatch* patch, const int uOrVIdx, const M_DelCtrlLn& m_delCtrlLn){
	for(int k = m_delCtrlLn.length; --k>=0;){
		if(m_delCtrlLn[k].p == patch && m_delCtrlLn[k].uOrVIdx == uOrVIdx){
			return true;
		}
	}
	return false;
}

bool NurbsPatch::uDeleteLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, M_Float& weight, M_Float& uKnot, int uDim, int vDim, int& idx, Vec2f& dim, Vec2f& order){
	NurbsPatch* mirrorPatch = this->mirrorPatch;

	g_deleteControlLine.centerMirrorLine = false;

	if(this->cage == false){
		return false;
	}

	int row = idxA / uDim;

	Vec3f dpA = matrix * dpoint[idxA];
	Vec3f dpB = matrix * dpoint[idxB];

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(dpA.x, dpA.y);
	Vec2f ptB = Vec2f(dpB.x, dpB.y);
	Vec2f crossPoint;

	if(clickLine(click, ptA, ptB, crossPoint)){
#ifdef PROG_ROB
		g_storeCrossPoint.set(fromPoint + idxA, fromPoint + idxB, getMidPointRatio(ptA, ptB, crossPoint));
#endif

		idx = idxA;
		bool foundFirst = false;

		
		int checkIdx = row * uDim;
		int checkIdx2 = checkIdx + 1;

		//int mirrorIdx = 0;
		//int mirrorIdx2 = 0;
		//if(mirror && mirrorPatch == 0){
		//	mirrorIdx = getMirrorTwinlessIdx(checkIdx);
		//	mirrorIdx2 = getMirrorTwinlessIdx(checkIdx2);
		//}
		int firstStitchPoly = smls->getStitchPoly(this, checkIdx, checkIdx2);
		uDeleteLine(checkIdx, checkIdx2, this, checkIdx2, foundFirst, g_deleteControlLine.m_delCtrlLn);
		if(foundFirst == false){
			smls->deleteLineFindMatchingEdge(this, checkIdx, checkIdx2, this, checkIdx2, foundFirst, g_deleteControlLine.m_delCtrlLn);


			//if(g_deleteControlLine.m_delCtrlLn.length <= g_deleteControlLine.mirror_delCtrlLn.length){
			//	g_deleteControlLine.m_delCtrlLn += g_deleteControlLine.mirror_delCtrlLn;
			//}
			//else{
			//	g_deleteControlLine.mirror_delCtrlLn += g_deleteControlLine.m_delCtrlLn;
			//	g_deleteControlLine.m_delCtrlLn = g_deleteControlLine.mirror_delCtrlLn;
			//}

		}

		g_deleteControlLine.mirror_delCtrlLn.length = 0;

		//if(g_deleteControlLine.centerMirrorLine == false && mirror && false == (foundFirst && g_deleteControlLine.m_delCtrlLn.length == 1)){

		//	if(mirrorPatch){
		//		foundFirst = false;
		//		g_deleteControlLine.mirror_delCtrlLn.length = 0;
		//		mirrorPatch->uDeleteLine(checkIdx, checkIdx2, mirrorPatch, checkIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);
		//		if(foundFirst == false){
		//			smls->deleteLineFindMatchingEdge(mirrorPatch, checkIdx, checkIdx2, mirrorPatch, checkIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);
		//		}
		//	}
		//	else if(foundFirst == false){
		//		uDeleteLine(mirrorIdx, mirrorIdx2, this, mirrorIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);
		//		if(foundFirst == false){
		//			smls->deleteLineFindMatchingEdge(this, mirrorIdx, mirrorIdx2, this, mirrorIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);
		//		}
		//	}
		//}
		//if(g_deleteControlLine.m_delCtrlLn.length && foundInList(g_deleteControlLine.m_delCtrlLn[0].p, g_deleteControlLine.m_delCtrlLn[0].uOrVIdx, g_deleteControlLine.mirror_delCtrlLn)){
		//	g_deleteControlLine.mirror_delCtrlLn.length = 0;
		//}

		if(false == g_deleteControlLine.crossesCenter()){
			//g_deleteControlLine.cleanUp();
			g_deleteControlLine.addMirrorEntries();
		}

		g_deleteControlLine.mouseOverPatch = this;
		g_deleteControlLine.mousePhase = 2;
		return true;
	}
	return false;
}

bool checkForTwinlessMirrorIndex(const M_DelCtrlLn& m_delCtrlLn, const DelCtrlLn& delCtrlLn){
	for(int k = m_delCtrlLn.length; --k>=0;){
		if(m_delCtrlLn[k].doU == false){
			if(m_delCtrlLn[k].p == delCtrlLn.p){
				if(m_delCtrlLn[k].p->getMirrorTwinlessIdx(m_delCtrlLn[k].uOrVIdx) == delCtrlLn.uOrVIdx){
					return true;
				}
			}
		}
	}
	return false;
}

bool checkForMirrorPatch(const M_DelCtrlLn& m_delCtrlLn, const DelCtrlLn& delCtrlLn){
	for(int k = m_delCtrlLn.length; --k>=0;){
		if(const NurbsPatch* mirrorPatch = m_delCtrlLn[k].p->mirrorPatch){
			if(mirrorPatch == delCtrlLn.p){
				if(m_delCtrlLn[k].doU == delCtrlLn.doU){
					if(m_delCtrlLn[k].uOrVIdx == delCtrlLn.uOrVIdx){
						return true;
					}
				}
			}
		}
	}
	return false;
}


bool DeleteControlLine::crossesCenter() const{

	for(int k = m_delCtrlLn.length; --k>=0;){
		NurbsPatch* patch = m_delCtrlLn[k].p;

		if(patch->mirrorPatch == 0){
			if(m_delCtrlLn[k].doU){
				continue;
			}
			else if(checkForTwinlessMirrorIndex(m_delCtrlLn, m_delCtrlLn[k])){
				continue;
			}
		}
		else if(checkForMirrorPatch(m_delCtrlLn, m_delCtrlLn[k])){
			continue;
		}
		return false;
	}

	return true;
}

void deleteDuplicatesAndMirrorPatches(M_DelCtrlLn& m_delCtrlLn, int k2, const DelCtrlLn& delCtrlLn){
	for(int k = m_delCtrlLn.length; --k > k2;){
		const NurbsPatch* patch = m_delCtrlLn[k].p;

		if(patch == delCtrlLn.p || patch->mirrorPatch == delCtrlLn.p){
			if(m_delCtrlLn[k].doU == delCtrlLn.doU){
				if(m_delCtrlLn[k].uOrVIdx == delCtrlLn.uOrVIdx){
					m_delCtrlLn.deleteElement(k);
				}
				else if(patch == delCtrlLn.p){
					if(m_delCtrlLn[k].doU == false){
						if(m_delCtrlLn[k].p->getMirrorTwinlessIdx(m_delCtrlLn[k].uOrVIdx) == delCtrlLn.uOrVIdx){
							m_delCtrlLn.deleteElement(k);
						}
					}
				}
			}
		}
	}
}

void DeleteControlLine::cleanUp(){

	for(int k = 0; k < m_delCtrlLn.length; k++){
		deleteDuplicatesAndMirrorPatches(m_delCtrlLn, k, m_delCtrlLn[k]);
	}
}

void DeleteControlLine::addMirrorEntries(){
	for(int k = m_delCtrlLn.length; --k>=0;){
		const NurbsPatch* patch = m_delCtrlLn[k].p;
		if(patch->mirror){
			if(NurbsPatch* mirrorPatch = patch->mirrorPatch){
				DelCtrlLn delCtrlLn = m_delCtrlLn[k];
				delCtrlLn.p = mirrorPatch;
				mirror_delCtrlLn += delCtrlLn;
			}
			else if(m_delCtrlLn[k].doU == false){
				int uOrVIdx = m_delCtrlLn[k].p->getMirrorTwinlessIdx(m_delCtrlLn[k].uOrVIdx);
				if(uOrVIdx != m_delCtrlLn[k].uOrVIdx){
					DelCtrlLn delCtrlLn = m_delCtrlLn[k];
					delCtrlLn.uOrVIdx = uOrVIdx;
					mirror_delCtrlLn += delCtrlLn;
				}
			}
		}
	}
}

bool NurbsPatch::vDeleteLine(const int idxA, const int idxB, const Matrix& matrix, M_Vec3f& dpoint, M_Float& weight, M_Float& uKnot, int uDim, int vDim, int& idx, Vec2f& dim, Vec2f& order){
	NurbsPatch* mirrorPatch = this->mirrorPatch;
	g_deleteControlLine.centerMirrorLine = false;

	if(this->cage == false){
		return false;
	}

	int col = idxA % uDim;

	Vec3f dpA = matrix * dpoint[idxA];
	Vec3f dpB = matrix * dpoint[idxB];

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(dpA.x, dpA.y);
	Vec2f ptB = Vec2f(dpB.x, dpB.y);
	Vec2f crossPoint;

	if(clickLine(click, ptA, ptB, crossPoint)){
		
		idx = idxA;
		bool foundFirst = false;

		if(mirror && mirrorPatch == 0){
			if(col == (uDim >> 1)){
				g_deleteControlLine.centerMirrorLine = true;
			}
		}


		int checkIdx = col;
		int checkIdx2 = col + uDim;


		//int mirrorIdx = 0;
		//int mirrorIdx2 = 0;
		//if(mirror && mirrorPatch == 0){
		//	mirrorIdx = this->getMirrorTwinlessIdx(checkIdx);
		//	mirrorIdx2 = this->getMirrorTwinlessIdx(checkIdx2);
		//}

		//if(g_transformCP->selectRing || g_transformCP->selectRings){
		//	if(Hair* hair = dynamic_cast<Hair*>(getParentNode(getNodeLine(this)))){
		//		Lock* lock = 0;
		//		int scpiIdx = 0;
		//		if(Strand* strand = dynamic_cast<Strand*>(this)){
		//			if(hair->isBoundToLock(strand, checkIdx2, lock, scpiIdx)){
		//				hair->insertStrandControlPoints(smls, lock, 1.0f - ratio);
		//				return true;
		//			}
		//		}
		//	}
		//}
		//int firstStitchPoly = smls->getStitchPoly(this, checkIdx, checkIdx2);
		vDeleteLine(checkIdx, checkIdx2, this, checkIdx2, foundFirst, g_deleteControlLine.m_delCtrlLn);
		if(foundFirst == false){
			smls->deleteLineFindMatchingEdge(this, checkIdx, checkIdx2, this, checkIdx2, foundFirst, g_deleteControlLine.m_delCtrlLn);

			//if(g_deleteControlLine.m_delCtrlLn.length <= g_deleteControlLine.mirror_delCtrlLn.length){
			//	g_deleteControlLine.m_delCtrlLn += g_deleteControlLine.mirror_delCtrlLn;
			//}
			//else{
			//	g_deleteControlLine.mirror_delCtrlLn += g_deleteControlLine.m_delCtrlLn;
			//	g_deleteControlLine.m_delCtrlLn = g_deleteControlLine.mirror_delCtrlLn;
			//}
		}
		g_deleteControlLine.mirror_delCtrlLn.length = 0;

		//foundFirst = false;
		//if(mirror && g_deleteControlLine.centerMirrorLine == false){
		//	if(mirrorPatch){
		//		mirrorPatch->vDeleteLine(checkIdx, checkIdx2, mirrorPatch, checkIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);

		//		if(foundFirst == false){
		//			smls->deleteLineFindMatchingEdge(mirrorPatch, checkIdx, checkIdx2, mirrorPatch, checkIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);
		//		}

		//	}
		//	else{
		//		vDeleteLine(mirrorIdx, mirrorIdx2, this, mirrorIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);

		//		if(foundFirst == false){
		//			smls->deleteLineFindMatchingEdge(this, mirrorIdx, mirrorIdx2, this, mirrorIdx2, foundFirst, g_deleteControlLine.mirror_delCtrlLn);
		//		}
		//	}
		//}
		

		if(false == g_deleteControlLine.crossesCenter()){
			//g_deleteControlLine.cleanUp();
			g_deleteControlLine.addMirrorEntries();
		}


//////////////////
		//bubbleSortInsertData(g_deleteControlLine.m_delCtrlLn);

		g_deleteControlLine.mouseOverPatch = this;
		//getRatioPhaseForInsert();
		g_deleteControlLine.addVisualClue();
		g_deleteControlLine.mousePhase = 2;

		return true;
	}
	return false;
}

/*
void NurbsPatch::getRatioPhaseForInsert(){
	Matrix matrix = this->matrix * g_viewMatrix;

	g_viewMatrix = g_scene->getViewMatrix();

	int uDim = this->dim.x;
	int vDim = this->dim.y;
	int uOrVIdx = g_insertPatch.m_insert[0].uOrVIdx;
	//int idxA = uOrVIdx * uDim;
	Vec3f vA;
	Vec3f vB;

	if(g_insertPatch.m_insert[0].doU){
		int uIdx = uOrVIdx;
		if(uOrVIdx & 1 && uOrVIdx < uDim - 2){
			uIdx++;
		}
		int uIdxA = uIdx;
		int uIdxB = uIdx + 1;
		if(uOrVIdx > 0 && uOrVIdx < uDim - 2){
			uIdxA--;
		}
		vA = matrix * dpoint[uIdxA];
		vB = matrix * dpoint[uIdxB];
	}
	else{
		int vIdx = uOrVIdx * uDim;
		if(uOrVIdx & 1 && uOrVIdx < vDim - 2){
			vIdx += uDim;
		}
		int vIdxA = vIdx;
		int vIdxB = vIdx + uDim;
		if(uOrVIdx > 0 && uOrVIdx < vDim - 2){
			vIdxA -= uDim;
		}
		vA = matrix * dpoint[vIdxA];
		vB = matrix * dpoint[vIdxB];
	}

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(vA.x, vA.y);
	Vec2f ptB = Vec2f(vB.x, vB.y);

	Vec2f crossPoint;
	if(clickLine(click, ptA, ptB, crossPoint)){
		g_insertPatch.masterRatio = getMidPointRatio(ptA, ptB, crossPoint);
		g_insertPatch.mousePhase = 3;
	}
	else{
		g_insertPatch.mousePhase = 2;
	}
}
*/


/*
bool isSamePath(M_Insert& m_insert, const NurbsPatch* patch, const int uOrVIdx){
	for(int k = 0; k < m_insert.length; k++){
		if(m_insert[k].p == patch && m_insert[k].uOrVIdx == uOrVIdx){
			return true;
		}
	}
	return false;
}

bool isSamePath(M_Insert& m_insertA, M_Insert& m_insertB){
	int len = m_insertA.length;
	if(len != m_insertB.length){
		return false;
	}

	for(int k = 0; k < len; k++){
		if(false == isSamePath(m_insertB, m_insertA[k].p, m_insertA[k].uOrVIdx)){
			return false;
		}
	}
	return true;
}
*/
int convertToLineFormat1(int idx, int idx2, int uDim, bool doU){
	int uOrVIdx = 0;
	if(doU){
		if(idx > idx2){
			uOrVIdx = idx2 % uDim;
		}
		else{
			uOrVIdx = idx % uDim;
		}
	}
	else{
		if(idx > idx2){
			uOrVIdx = idx2 / uDim;
		}
		else{
			uOrVIdx = idx / uDim;
		}
	}
	return uOrVIdx;
}

void convertToLineFormat2(const int uOrVIdx, int& idx, int& idx2, int uDim, bool doU){
	if(doU){
		idx = uOrVIdx;
		idx2 = uOrVIdx + 1;
	}
	else{
		idx = uOrVIdx * uDim;
		idx2 = (uOrVIdx + 1) * uDim;
	}
	if(uOrVIdx & 1){
		swap(idx, idx2);
	}
}

int NurbsPatch::getMirrorTwinlessLineFormat1(int uOrVIdx, int uDim, bool doU){
	int idx = 0;
	int idx2 = 0;
	convertToLineFormat2(uOrVIdx, idx, idx2, uDim, doU);
	idx = getMirrorTwinlessIdx(idx);
	idx2 = getMirrorTwinlessIdx(idx2);
	return convertToLineFormat1(idx, idx2, uDim, doU);
}




void uRedirectDeleteLineData(const NurbsPatch* checkPatch, const int uOrVIdx, M_DelCtrlLn& m_delCtrlLn){
	for(int k = 0; k < m_delCtrlLn.length; k++){
		if(m_delCtrlLn[k].p == checkPatch){
			if(m_delCtrlLn[k].doU == false){
				if(m_delCtrlLn[k].uOrVIdx > uOrVIdx){
					m_delCtrlLn[k].uOrVIdx -= 2;
				}
			}
		}
	}
}

void vRedirectDeleteLineData(const NurbsPatch* checkPatch, const int uOrVIdx, M_DelCtrlLn& m_delCtrlLn){
	for(int k = 0; k < m_delCtrlLn.length; k++){
		if(m_delCtrlLn[k].p == checkPatch){
			if(m_delCtrlLn[k].doU){//thise line is the only differencer to uRedirectDeleteLineData
				if(m_delCtrlLn[k].uOrVIdx > uOrVIdx){
					m_delCtrlLn[k].uOrVIdx -= 2;
				}
			}
		}
	}
}

void setHidenLineOffsetValues(Seamless* smls, M_DelCtrlLn& m_delCtrlLn){
	for(int k = 0; k < m_delCtrlLn.length; k++){
		m_delCtrlLn[k].hidenLineOffset = 0;
		NurbsPatch* patch = m_delCtrlLn[k].p;
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		int checkIdx = m_delCtrlLn[k].uOrVIdx;
		if(m_delCtrlLn[k].doU == false){
			if(checkIdx >= uDim - 2){// if deleting the line the edge
				m_delCtrlLn[k].hidenLineOffset = -1;
			}
			else if(smls->isPointInCenterOfNon4SidedStitchPoly(patch, checkIdx + 1) || smls->isPointInCenterOfNon4SidedStitchPoly(patch, uDim * (vDim - 1) + checkIdx + 1)){
				m_delCtrlLn[k].hidenLineOffset = -1;
			}
		}
		else{
			if(checkIdx >= vDim - 1){// if deleting the line the edge
				m_delCtrlLn[k].hidenLineOffset = -1;
			}
			else{
				checkIdx = (checkIdx + 1) * uDim;
				if(smls->isPointInCenterOfNon4SidedStitchPoly(patch, checkIdx) || smls->isPointInCenterOfNon4SidedStitchPoly(patch, checkIdx + uDim - 1)){
					m_delCtrlLn[k].hidenLineOffset = -1;
				}
			}
		}
	}
}

void deleteNode(int nodeLine, int selCount = 1);
void updateAutoInForClick();

void deletePatch(NurbsPatch* patch){
	Seamless* smls = patch->smls;
	for(int k = g_sceneTreeList.length; --k>=0;){
		if(Build* build=dynamic_cast<Build*>(g_sceneTreeList[k].node.p)){
			if(build == patch){
				//smls->deletePoliesOwnedByPatch(patch);
				deleteNode(k);
			}
		}
	}
}


void doLastPhase2(M_DelCtrlLn& m_delCtrlLn, M_DelCtrlLn* mirror_delCtrlLn, M_<NurbsPatchPtr>& m_delPatchList){
	

	for(int k = 0; k < m_delCtrlLn.length; k++){

		NurbsPatch* patch = m_delCtrlLn[k].p;
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		M_Int& cpIndex = patch->cpIndex.refValue();
		M_Vec3f& dpoint = patch->dpoint.refValue();
		M_Float& weight = patch->weight.refValue();
		M_Float& uKnot = patch->uKnot.refValue();
		M_Float& vKnot = patch->vKnot.refValue();

		M_Int oldCpIndex = cpIndex;
		M_Vec3f oldPoints = dpoint;
		M_Float oldWeights = weight;

		int uOrVIdx = m_delCtrlLn[k].uOrVIdx;
		int hidenLineOffset = m_delCtrlLn[k].hidenLineOffset;

		if(m_delCtrlLn[k].doU){
			vRedirectDeleteLineData(patch, uOrVIdx + hidenLineOffset, m_delCtrlLn);
			if(mirror_delCtrlLn){
				vRedirectDeleteLineData(patch, uOrVIdx + hidenLineOffset, *mirror_delCtrlLn);
			}

			patch->redirectRow(uOrVIdx);

			if(vDim > 3){
				patch->deleteRow(uDim, vDim, cpIndex, dpoint, weight, oldCpIndex, oldPoints, oldWeights, uOrVIdx);
				vDim--;
				patch->deleteRow(uDim, vDim, cpIndex, dpoint, weight, oldCpIndex, oldPoints, oldWeights, uOrVIdx + hidenLineOffset);
				vDim--;
			}
			else{
				m_delPatchList += patch;
				continue;
			}
		}
		else{
			uRedirectDeleteLineData(patch, uOrVIdx + hidenLineOffset, m_delCtrlLn);
			if(mirror_delCtrlLn){
				uRedirectDeleteLineData(patch, uOrVIdx + hidenLineOffset, *mirror_delCtrlLn);
			}

			patch->redirectCol(uOrVIdx);

			if(uDim > 3){
				patch->deleteCol(uDim, vDim, cpIndex, dpoint, weight, oldCpIndex, oldPoints, oldWeights, uOrVIdx);
				uDim--;
				patch->deleteCol(uDim, vDim, cpIndex, dpoint, weight, oldCpIndex, oldPoints, oldWeights, uOrVIdx + hidenLineOffset);
				uDim--;
			}
			else{
				m_delPatchList += patch;
				continue;
			}
		}
		patch->dpointsToPoints();
		patch->dim.refValue() = Vec2f(uDim, vDim);
		if(patch->index[0] >= dpoint.length){
			patch->index[0] = 0;
		}

		updateNurbsCurveKnotMid1(uDim, uKnot);
		updateNurbsCurveKnotMid1(vDim, vKnot);
		patch->processFields();
	}
}

void bnspeBuildPolies(M_DelCtrlLn& m_delCtrlLn){
	for(int k = m_delCtrlLn.length; --k >= 0;){
		NurbsPatch* patch = m_delCtrlLn[k].p;
		patch->synchronizeAllStemCells();
		patch->bnspeBuildPolies();
	}
}

void bubbleSortPatchList(M_<NurbsPatchPtr>& a){
    bool flag;
    do{
        flag=false;
        for(int i=0; i < a.length - 1; i++){

            if (a[i]->plidx > a[i+1]->plidx){

                NurbsPatchPtr swap = a[i+1];
                a[i+1] = a[i];
                a[i] = swap;

                flag = true;
            }
        }
    }while(flag);
}

//void addPatchToList(M_<NurbsPatchPtr> m_nurbsPatch, NurbsPatch* patch){
//	for(int k = m_nurbsPatch.length; --k>=0;){
//		if(m_nurbsPatch[k] == patch){
//			return;
//		}
//	}
//	m_nurbsPatch += patch;
//}
//
//void addTwinPatches(M_<NurbsPatchPtr>& m_delPatchList){
//	for(int k = m_delPatchList.length; --k>=0;){
//		NurbsPatch* r_patch = m_delPatchList[k];
//		if(NurbsPatch* l_patch = r_patch->mirrorPatch){
//			addPatchToList(m_delPatchList, l_patch);
//		}
//	}
//}

void deletePatches(M_<NurbsPatchPtr>& m_delPatchList){
	bubbleSortPatchList(m_delPatchList);

	for(int j = m_delPatchList.length; --j>=0;){
		NurbsPatch* patch = m_delPatchList[j];
		deletePatch(patch);
	}
}

void DeleteControlLine::doLastPhase(){
	if(m_delCtrlLn.length){
		M_<NurbsPatchPtr> m_delPatchList;

		//commonDeleteLine(mouseOverPatch->smls, m_insert, lookup);
		setHidenLineOffsetValues(smls, m_delCtrlLn);
		setHidenLineOffsetValues(smls, mirror_delCtrlLn);
		doLastPhase2(m_delCtrlLn, &mirror_delCtrlLn, m_delPatchList);
		doLastPhase2(mirror_delCtrlLn, 0, m_delPatchList);

		deletePatches(m_delPatchList);
		//addTwinPatches(m_delPatchList);
		smls->generateRawStitchPolys();

//
		g_stw->genNodeList();
		M_Art m_part;
		smls->cpNodeListAdd();
		smls->deleteLinksToNonExistentPatches(); //not to be confused with non existing control polys
		smls->deletePolysWithNonExistingPoints();
		smls->removeAllPatchesFromPartsNotInList(m_part);
	
		smls->populateMirrorNodes();
		for(int k = g_animBarList.length; --k>=0;){
			g_animBarList[k]->deleteNonExistingMorpherPoints(smls);
		}
		smls->deletePoliesOwnedByInvalidPatchs();
		smls->generateRawStitchPolys();
		smls->stitchAllPolys();
		//smls->synchronizeAllStemCells();
		smls->generateAllPatchesBnspe();
		smls->doBuild();

		//g_stw->commonPostInsertNodeStuff(true);
		g_stw->lesserPostInsertNodeStuff();
		updateAutoInForClick();
//


		


		//patch->smls->redirectStitchPolyIndices(patch, lookup);
		//g_stw->commonPostInsertNodeStuff();
		smls->populateMirrorNodes();
		smls->generateRawStitchPolys();
		//smls->calcAllPolyMidPoints();
		smls->stitchAllPolys();
		smls->generateRawStitchPolys();
		smls->stitchAllPolys();//why do we need to stitch twice?

		//bnspeBuildPolies(m_delCtrlLn);
		//bnspeBuildPolies(mirror_delCtrlLn);
		smls->doBuild();

		m_delCtrlLn.length = 0;
		mirror_delCtrlLn.length = 0;

		smls->ctrf = true;
		g_deleteControlLine.mousePhase = DeleteControlLine::NEUTRAL;
		houseKeepVisualClue();
		BackUp::monostable.beginCycle();
		smls = 0;
	}
	//mousePhase = NEUTRAL;
}

bool NurbsPatch::clickedRowColum(const int idxA, const int idxB, Matrix matrix){

	Vec3f dpA = dpoint[idxA];
	Vec3f dpB = dpoint[idxB];

	Vec3f ptA3f = matrix * dpA;
	Vec3f ptB3f = matrix * dpB;

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(ptA3f.x, ptA3f.y);
	Vec2f ptB = Vec2f(ptB3f.x, ptB3f.y);
	Vec2f crossPoint;
	if(clickLine(click, ptA, ptB, crossPoint)){
		return true;
	}
	return false;
}


bool NurbsPatch::deleteControlRowColum(const Vec2f par, const int idxA, const int idxB, Matrix matrix, M_Vec3f& dpoint, M_Vec3f& point, M_Float& weight, M_Float& uKnot, int uDim, int vDim, int& idx, Vec2f& dim, Vec2f& order, M_Int& lookup){
	int uOldDim = uDim;
	int vOldDim = vDim;

	const int uMid = checkNurbsType(dim.x, uKnot);
	const int vMid = checkNurbsType(dim.y, vKnot);

	Vec3f dpA = dpoint[idxA];
	Vec3f dpB = dpoint[idxB];

	Vec3f ptA3f = matrix * dpA;
	Vec3f ptB3f = matrix * dpB;

	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	Vec2f ptA = Vec2f(ptA3f.x, ptA3f.y);
	Vec2f ptB = Vec2f(ptB3f.x, ptB3f.y);
	Vec2f crossPoint;
	if(clickLine(click, ptA, ptB, crossPoint)){
		
		idx = idxA;
		
		doDim(par, .5, dpoint, weight, order, dim, idx, uDim, vDim, lookup);


		uDim = dim.x;
		vDim = dim.y;

		if(NurbsPatch::MID_1 == uMid || NurbsPatch::MID_1 == vMid){
			Vec2f par2 = dim;
			if(NurbsPatch::MID_1 == uMid){
				if(uOldDim > uDim){
					par2.x = uDim - 2;
				}
			}
			if(NurbsPatch::MID_1 == vMid){
				if(vOldDim > vDim){
					par2.y = vDim - 2;
				}
			}
			doDim(par2, 0, dpoint, weight, order, dim, idx, uDim, vDim, lookup);
		}

		updateKnot(uOldDim, dim.x, uKnot, order.x);
		updateKnot(vOldDim, dim.y, vKnot, order.y);
		
		dpointsToPoints();
		synchronizeAllStemCells();

		if(idx >= dpoint.length){
			idx=dpoint.length-1;
		}

		processFields();
		MultiSelect::pointList.length = 0;
		houseKeep();
		return true;
	}
	return false;
}

int getMidPointAbsoluteIdx(const int midBeginIndex, int edge, const M_CtrlPolyPoint& ctrlPolyPoint);
int getIndexContainedInMidPoint(const M_CtrlPolyPoint& ctrlPolyPoint, const int absoluteIdx, const NurbsPatch* checkPatch);

bool Seamless::insertControlStitchPoly(const bool& perspective){
	Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
	for(int poly = 0; poly < ctrlPolyPoint.length;){
		int len = ctrlPolyPoint[poly].len;
		for(int e = 0; e < len; e++){
			int e2 = e + 1;
			if(e2 == len){
				e2 = 0;
			}
			NurbsPatch* patchA = ctrlPolyPoint[poly + e].p;
			if(patchA->cage){
				NurbsPatch* patchB = ctrlPolyPoint[poly + e2].p;
				int idxA = ctrlPolyPoint[poly + e].i;
				int idxB = ctrlPolyPoint[poly + e2].i;
				if(poly == 113){
					int a = 0;
				}
				Vec3f pntA = g_viewMatrix * patchA->dpoint[idxA];
				Vec3f pntB = g_viewMatrix * patchB->dpoint[idxB];

				Vec2f ptA = Vec2f(pntA.x, pntA.y);
				Vec2f ptB = Vec2f(pntB.x, pntB.y);

				Vec2f crossPoint;

				if(clickLine(click, ptA, ptB, crossPoint)){
					int absoluteIdx = getMidPointAbsoluteIdx(poly + len, e, ctrlPolyPoint);
					float ratio = getMidPointRatio(ptA, ptB, crossPoint);
					if(ratio > .5){
						swap(patchA, patchB);
						swap(idxA, idxB);
						swap(ptA, ptB); //could we not just: ptA = ptB ?
					}
					int idxb = getIndexContainedInMidPoint(ctrlPolyPoint, absoluteIdx, patchA);
					Matrix matrix = patchA->matrix * g_viewMatrix;
					int uDim = patchA->dim.x;
					int vDim = patchA->dim.y;

					if(idxb >= 0){
						Vec3f pntb = g_viewMatrix * patchA->dpoint[idxb];;
						Vec2f ptb = Vec2f(pntb.x, pntb.y);

						if(clickLine(click, ptA, ptb, crossPoint)){

							ratio = getMidPointRatio(ptA, ptb, crossPoint);

							int diff = abs(idxA - idxb);


							if(idxb < idxA){
								swap(idxb, idxA);
							}
							if(diff == 1){
								if(patchA->uInsertLine(idxA, idxb, matrix, patchA->dpoint, uDim, vDim, patchA->dim.refValue())){
									return true;
								}
							}
							else if(diff == uDim){
								if(patchA->vInsertLine(idxA, idxb, matrix, patchA->dpoint, uDim, vDim, patchA->dim.refValue())){
									return true;
								}
							}
						}
					}
					else{
						if(idxB < idxA){
							swap(idxB, idxA);
						}
						int diff = idxB - idxA;

						if(diff == 1){
							if(patchA->uInsertLine(idxA, idxB, matrix, patchA->dpoint, uDim, vDim, patchA->dim.refValue())){
								return true;
							}
						}
						else if(diff == uDim){
							if(patchA->vInsertLine(idxA, idxB, matrix, patchA->dpoint, uDim, vDim, patchA->dim.refValue())){
								return true;
							}
						}
					}
				}
			}
		}
		poly = getNextCtrlPoly(poly);
	}
	return false;
}


bool NurbsPatch::insertControlRowColum(){
	g_insertPatch.m_insert.length = 0;
	g_insertPatch.init_element = 0;
	g_insertPatch.m_insertOD.length = 0;
	g_insertPatch.beginInsertIdx = 0;


	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	Vec2f& dim = this->dim.refValue();
	int uDim = dim.x;
	int vDim = dim.y;
	Matrix matrix = this->matrix * g_viewMatrix;

	int uNurbsType = checkNurbsType(uDim, uKnot);
	int vNurbsType = checkNurbsType(vDim, vKnot);

	M_Bool& m_pointIsVisible = g_pointIsVisible;

	if(g_scene->mode & BACKFACE_IGNORE_MODE){
		for(int v = 0; v < vDim;){
			for(int u = 0; u < uDim - 1; u++){
				int idxA = v * uDim + u;
				int idxB = v * uDim + u + 1;
				if(m_pointIsVisible[fromPoint + idxA] || m_pointIsVisible[fromPoint + idxB]){
					if(uInsertLine(idxA, idxB, matrix, dpoint, uDim, vDim, dim)){
						return true;
					}
				}
			}
			if(v >= 1 && v < vDim - 2 && NurbsPatch::MID_1 == vNurbsType){
				v += 2;
			}
			else{
				v++;
			}
		}

		for(int u = 0; u < uDim;){
			for(int v = 0; v < vDim - 1; v++){
				int idxA = v * uDim + u;
				int idxB = (v + 1) * uDim + u;
				if(m_pointIsVisible[fromPoint + idxA] || m_pointIsVisible[fromPoint + idxB]){
					if(vInsertLine(idxA, idxB, matrix, dpoint, uDim, vDim, dim)){
						return true;
					}
				}
			}
			if(u >= 1 && u < uDim - 2 && NurbsPatch::MID_1 == uNurbsType){
				u += 2;
			}
			else{
				u++;
			}
		}
	}
	else{
		if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
			if(isCtrlPolyRef){
				for(int v = 3; v < vDim - 2;){
					for(int u = 1; u < uDim - 2; u++){
						int idxA = v * uDim + u;
						int idxB = v * uDim + u + 1;
						if(uInsertLine(idxA, idxB, matrix, dpoint, uDim, vDim, dim)){
							return true;
						}
					}
					v += 2;
				}

				for(int u = 3; u < uDim - 2;){
					for(int v = 1; v < vDim - 2; v++){
						int idxA = v * uDim + u;
						int idxB = (v + 1) * uDim + u;
						if(vInsertLine(idxA, idxB, matrix, dpoint, uDim, vDim, dim)){
							return true;
						}
					}
					u += 2;
				}
			}
			else{
				for(int v = 0; v < vDim;){
					for(int u = 0; u < uDim - 1; u++){
						int idxA = v * uDim + u;
						int idxB = v * uDim + u + 1;
						if(uInsertLine(idxA, idxB, matrix, dpoint, uDim, vDim, dim)){
							return true;
						}
					}
					if(v >= 1 && v < vDim - 2 && NurbsPatch::MID_1 == vNurbsType){
						v += 2;
					}
					else{
						v++;
					}
				}

				for(int u = 0; u < uDim;){
					for(int v = 0; v < vDim - 1; v++){
						int idxA = v * uDim + u;
						int idxB = (v + 1) * uDim + u;
						if(vInsertLine(idxA, idxB, matrix, dpoint, uDim, vDim, dim)){
							return true;
						}
					}
					if(u >= 1 && u < uDim - 2 && NurbsPatch::MID_1 == uNurbsType){
						u += 2;
					}
					else{
						u++;
					}
				}
			}
		}
		if(smls->insertControlStitchPoly(g_perspective)){
			int a = 0;
		}
	}
	return false;
}

bool NurbsPatch::deleteControlLine(){
	g_deleteControlLine.m_delCtrlLn.length = 0;
	g_deleteControlLine.mirror_delCtrlLn.length = 0;

	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	M_Float& uKnot = this->uKnot.refValue();
	Vec2f& dim = this->dim.refValue();
	Vec2f& order = this->order.refValue();
	int uDim = dim.x;
	int vDim = dim.y;
	int& idx = index[0];
	Matrix matrix = this->matrix * g_viewMatrix;

	int uNurbsType = checkNurbsType(uDim, uKnot);
	int vNurbsType = checkNurbsType(vDim, vKnot);
	
	M_Bool& m_pointIsVisible = g_pointIsVisible;

	if(g_scene->mode & BACKFACE_IGNORE_MODE){
		for(int v = 0; v < vDim;){
			for(int u = 0; u < uDim - 1; ){
				int idxA = v * uDim + u;
				int idxB = idxA + 1;
			
				if(m_pointIsVisible[fromPoint + idxA] || m_pointIsVisible[fromPoint + idxB]){
					if(uDeleteLine(idxA, idxB, matrix, dpoint, weight, uKnot, uDim, vDim, idx, dim, order)){
						return true;
					}
				}
				u++;
			}
			v++;
			if(v > 1 && v < vDim - 2 && NurbsPatch::MID_1 == vNurbsType){
				v++;
			}
		}
	
		for(int u = 0; u < uDim;){
			for(int v = 0; v < vDim - 1;){
				int idxA = v * uDim + u;
				int idxB = idxA + uDim;
				if(m_pointIsVisible[fromPoint + idxA] || m_pointIsVisible[fromPoint + idxB]){
					if(vDeleteLine(idxA, idxB, matrix, dpoint, weight, uKnot, uDim, vDim, idx, dim, order)){
						return true;
					}
				}
				v++;
			}
			u++;
			if(u > 1 && u < uDim - 2 && NurbsPatch::MID_1 == uNurbsType){
				u++;
			}
		}
	}
	else{
		for(int v = 0; v < vDim;){
			for(int u = 0; u < uDim - 1; ){
				int idxA = v * uDim + u;
				int idxB = idxA + 1;
			
				if(uDeleteLine(idxA, idxB, matrix, dpoint, weight, uKnot, uDim, vDim, idx, dim, order)){
					return true;
				}
				u++;
			}
			v++;
			if(v > 1 && v < vDim - 2 && NurbsPatch::MID_1 == vNurbsType){
				v++;
			}
		}
	
		for(int u = 0; u < uDim;){
			for(int v = 0; v < vDim - 1;){
				int idxA = v * uDim + u;
				int idxB = idxA + uDim;
				if(vDeleteLine(idxA, idxB, matrix, dpoint, weight, uKnot, uDim, vDim, idx, dim, order)){
					return true;
				}
				v++;
			}
			u++;
			if(u > 1 && u < uDim - 2 && NurbsPatch::MID_1 == uNurbsType){
				u++;
			}
		}
	}

	return false;
}

bool NurbsPatch::deleteControlRowColum(){
	M_Int lookup;
	if(cage == false){
		return false;
	}
	M_Vec3f& dpoint = this->dpoint.refValue();
	M_Vec3f& point = this->point.refValue();
	M_Float& weight = this->weight.refValue();
	M_Float& uKnot = this->uKnot.refValue();
	Vec2f& dim = this->dim.refValue();
	Vec2f& order = this->order.refValue();
	int uDim = dim.x;
	int vDim = dim.y;
	int& idx = index[0];
	Matrix matrix = this->matrix * g_viewMatrix;

	for(int v = 0; v < vDim; v++){
		for(int u = 0; u < uDim - 1; u++){
			int idxA = v * uDim + u;
			int idxB = v * uDim + u + 1;
			Vec2f par = Vec2f(uDim, vDim - 1);
			if(deleteControlRowColum(par, idxA, idxB, matrix, dpoint, point, weight, uKnot, uDim, vDim, idx, dim, order, lookup)){
				return true;
			}
		}
	}
	for(int u = 0; u < uDim; u++){
		for(int v = 0; v < vDim - 1; v++){
			int idxA = v * uDim + u;
			int idxB = (v + 1) * uDim + u;
			Vec2f par = Vec2f(uDim - 1, vDim);
			if(deleteControlRowColum(par, idxA, idxB, matrix, dpoint, point, weight, uKnot, uDim, vDim, idx, dim, order, lookup)){
				return true;
			}
		}
	}
	return false;
}

void Seamless::deleteControlRowColum(){
	for(int k = 0; k < cpNodeList.length; k++){
		if(cpNodeList[k]->deleteControlRowColum()){
			fastFullBuildForAutoIn(true, this);
			updateBuild3dWindow();
			return;
		}
	}
}

void Seamless::insertControlRowColum(){
	g_insertPatch.loop = false;
	g_insertPatch.init_element = 0;
	g_viewMatrix = g_scene->getViewMatrix();
	g_insertPatch.m_insert.length = 0;

	for(int k = 0; k < cpNodeList.length; k++){
		if(g_insertPatch.diversionHover = cpNodeList[k]->insertControlRowColum()){
			g_insertPatch.addInsertVisualClue();
			return;
		}
	}
	g_insertPatch.addInsertVisualClue();
}

bool Seamless::addDiversionMark2(M_Insert& m_insert, int& beginInsertIdx){
	bool foundFirstPatch = false;

	for(int k = beginInsertIdx; k < m_insert.length; k++){
		Insert& insert  = m_insert[k];

		NurbsPatch* patch = insert.p;
		int uDim = patch->dim.x;
		int vDim = patch->dim.y;
		int originIdx = 0;
		int checkIdx = 0;
		int checkIdx2 = 0;
		if(insert.doU){
		
			if(insert.uOrVIdx & 1){
				checkIdx2 = insert.uOrVIdx;
				checkIdx = checkIdx2 + 1;
			}
			else{
				checkIdx = insert.uOrVIdx;
				checkIdx2 = checkIdx + 1;
			}
			originIdx = checkIdx + insert.cutLine.x * uDim;
			if(insert.leaveNorthOrEast){
				checkIdx += (vDim - 1) * uDim;
				checkIdx2 += (vDim - 1) * uDim;
			}
		}
		else{
			if(insert.uOrVIdx & 1){
				checkIdx2 = insert.uOrVIdx * uDim;
				checkIdx = checkIdx2 + uDim;
			}
			else{
				checkIdx = insert.uOrVIdx * uDim;
				checkIdx2 = checkIdx + uDim;
			}
			originIdx = checkIdx + insert.cutLine.x;
			if(insert.leaveNorthOrEast){
				checkIdx += (uDim - 1);
				checkIdx2 += (uDim - 1);
			}
			//remPatch->uInsertLine2(g_insertPatch.remRatio, remPatch->dpoint.refValue(), remPatch->dim.x, remPatch->dim.y, g_insertPatch.remRowCol, true);
		}
		bool forbid = false;
		if(insertLineFindClickedPoly(patch, originIdx, checkIdx, checkIdx2, insert.doU, insert.leaveNorthOrEast, -1, foundFirstPatch, g_insertPatch.remLeftSide, g_insertPatch.remRatio, g_insertPatch.remInvertRatio, m_insert, k, beginInsertIdx, forbid)){
			return true;
		}
		if(forbid){
			return false;
		}
	}
	return false;
}

void Seamless::deleteControlLine(){
	g_viewMatrix = g_scene->getViewMatrix();
	g_deleteControlLine.m_delCtrlLn.length = 0;
	g_deleteControlLine.mirror_delCtrlLn.length = 0;

	for(int k = 0; k < cpNodeList.length; k++){
		if(cpNodeList[k]->deleteControlLine()){
			g_deleteControlLine.addVisualClue();
			return;
		}
	}
	g_deleteControlLine.addVisualClue();
}

int NurbsPatch::isMouseOverEdge(const Matrix& matrix){
	if(this->cage == false){
		return false;
	}

	int uDim = dim.x;
	int vDim = dim.y;

	int offset = (vDim - 1) * uDim;

	if(NurbsLine* line = dynamic_cast<NurbsLine*>(this)){
		Vec3f pnt = matrix * dpoint[0];
		if(checkOverPoint2(g_perspective, pnt)){
			return WEST;
		}
		pnt = matrix * dpoint[dpoint.length - 1];
		if(checkOverPoint2(g_perspective, pnt)){
			return EAST;
		}
	}

	for(int u = 0; u < uDim - 1; u++){

		Vec3f dpA = matrix * dpoint[u];
		Vec3f dpB = matrix * dpoint[u + 1];

		Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		Vec2f ptA = Vec2f(dpA.x, dpA.y);
		Vec2f ptB = Vec2f(dpB.x, dpB.y);
		Vec2f crossPoint;

		if(clickLine(click, ptA, ptB, crossPoint)){
			return SOUTH;
		}

		dpA = matrix * dpoint[u + offset];
		dpB = matrix * dpoint[u + offset + 1];

		click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		ptA = Vec2f(dpA.x, dpA.y);
		ptB = Vec2f(dpB.x, dpB.y);
		crossPoint;

		if(clickLine(click, ptA, ptB, crossPoint)){
			return NORTH;
		}
	}
	for(int v = 0;  v < vDim - 1; v++){

		Vec3f dpA = matrix * dpoint[v * uDim];
		Vec3f dpB = matrix * dpoint[v * uDim + uDim];

		Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		Vec2f ptA = Vec2f(dpA.x, dpA.y);
		Vec2f ptB = Vec2f(dpB.x, dpB.y);
		Vec2f crossPoint;

		if(clickLine(click, ptA, ptB, crossPoint)){
			return WEST;
		}

		dpA = matrix * dpoint[v * uDim + uDim - 1];
		dpB = matrix * dpoint[v * uDim + uDim * 2 - 1];

		click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
		ptA = Vec2f(dpA.x, dpA.y);
		ptB = Vec2f(dpB.x, dpB.y);
		crossPoint;

		if(clickLine(click, ptA, ptB, crossPoint)){
			return EAST;
		}
	}
	return 0;
}

void Seamless::mergePatchesStandby(){
	Matrix matrix = g_viewMatrix;
	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];
		if(int edge = patch->isMouseOverEdge(matrix)){
			if(patch == g_mergePatches.patchA && edge == g_mergePatches.edgeA){
				return;
			}
			int eb = 0;
			if(NurbsPatch* pb = patch->getNearestMatchingEdge(edge, eb)){
				g_mergePatches.addVisualClue(Color3f(0, 1, 0));
				g_mergePatches.patchA = patch;
				g_mergePatches.edgeA = edge;
				g_mergePatches.patchB = pb;
				g_mergePatches.edgeB = eb;
				g_mergePatches.addVisualClue(Color3f(1, 0, 0));
				return;
			}
		}
	}

	g_mergePatches.addVisualClue(Color3f(0, 1, 0));
	g_mergePatches.patchA = 0;
	g_mergePatches.patchB = 0;
}

void Seamless::splitPatch(){
	//if((g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE) == false){
	//	return;
	//}
	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];
		if(dynamic_cast<Strand*>(patch)){
			continue;
		}
		if(patch->cage){
			deleteAllParts();

			if(patch->splitPatch()){

				generateRawStitchPolys();
				stitchAllPolys();
				generateAllPatchesBnspe();
				joinAllParts();
				doBuild();

				return;
			}
		}
	}
}



void getColorWithEyeDrop(Art* selPart, Seamless* smls){
	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(selPart->workEffect)){
		M_Color3f& color = colorEffect->color;
		Art* actTriPart = SenseTugGlobals::actTriPart;
		S_Triangle& tri = actTriPart->triangle[SenseTugGlobals::ActTri];

		g_palette->color = color[tri.refCorner(SenseTugGlobals::corner).t];
		PaletteCP* paletteCP = paletteCPAllreadyOpen();
		paletteCP->paint();
	}
}
/*
int getMostNorthCoordInTriangle(const S_Triangle& triangle, const Vec3f hitPoint){
	float mostY = triangle.refPart(0)->coord[triangle.refIndex(0)].y;
	int idx = 0;
	for(int i = 1; i < 3; i++){
		float y = (g_viewMatrix * triangle.refPart(0)->coord[triangle.refIndex(0)]).y;
		if(y > mostY){
			idx = i;
			mostY = y;
		}
	}
	return idx;
}

int getMostSouthCoordInTriangle(const S_Triangle& triangle, const Vec3f hitPoint){
	float leastY = triangle.refPart(0)->coord[triangle.refIndex(0)].y;
	int idx = 0;
	for(int i = 1; i < 3; i++){
		float y = (g_viewMatrix * triangle.refPart(0)->coord[triangle.refIndex(0)]).y;
		if(y < leastY){
			idx = i;
			leastY = y;
		}
	}
	return idx;
}

int getMostEastCoordInTriangle(const S_Triangle& triangle, const Vec3f hitPoint){
	float leastY = triangle.refPart(0)->coord[triangle.refIndex(0)].y;
	int idx = 0;
	for(int i = 1; i < 3; i++){
		float y = triangle.refPart(0)->coord[triangle.refIndex(0)].y;
		if(y < leastY){
			idx = i;
			leastY = y;
		}
	}
	return idx;
}
*/

Color3f getColorFromPointInTriangle(const S_Triangle& triangle, const Vec3f hitPoint, const ColorEffect* colorEffect){
	Color3f returnColor;

	for(int i0A = 0; i0A < 3; i0A++){
		int i0B = i0A + 1;
		if(i0B == 3){
			i0B = 0;
		}
		Vec3f v0A = g_viewMatrix * triangle.refPart(i0A)->coord[triangle.refIndex(i0A)];
		Vec3f v0B = g_viewMatrix * triangle.refPart(i0B)->coord[triangle.refIndex(i0B)];

		if(hitPoint.y >= v0A.y && hitPoint.y <= v0B.y && v0A.y != v0B.y){
			int i1A = i0A;
			for(int k = 0; k < 2; k++){
				i1A++;
				if(i1A == 3){
					i1A = 0;
				}
				int i1B = i1A + 1;
				if(i1B == 3){
					i1B = 0;
				}
				Vec3f v1A = g_viewMatrix * triangle.refPart(i1A)->coord[triangle.refIndex(i1A)];
				Vec3f v1B = g_viewMatrix * triangle.refPart(i1B)->coord[triangle.refIndex(i1B)];

				if(hitPoint.y <= v1A.y && hitPoint.y >= v1B.y && v1A.y != v1B.y){

					float diff0 = v0B.y - v0A.y;
					float r0 = (hitPoint.y - v0A.y) / diff0;
					float diff1 = v1A.y - v1B.y;
					float r1 = (hitPoint.y - v1B.y) / diff1;

					Vec3f v2A = v0A * (1.0 - r0) + v0B * r0;
					Vec3f v2B = v1B * (1.0 - r1) + v1A * r1;

					Color3f c2A = colorEffect->color[triangle.refIndex(i0A)] * (1.0 - r0) + colorEffect->color[triangle.refIndex(i0B)] * r0;
					Color3f c2B = colorEffect->color[triangle.refIndex(i1B)] * (1.0 - r1) + colorEffect->color[triangle.refIndex(i1A)] * r1;
					float diff2 = v2B.x - v2A.x;
					float r2 = (hitPoint.x - v2A.x) / diff2;
					returnColor = c2A * (1.0 - r2) + c2B * r2;
					return returnColor;
				}
			}
		}
	}
	return returnColor;
}

void doColorPicker(Art* actTriPart, int ActTri, Seamless* smls){
	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(actTriPart->workEffect)){
		M_Color3f& color = colorEffect->color;
	}
}

Vec2f convertPointToScreen2d(Vec3f point){
	Vec3f sr = g_viewMatrix * point;

	Vec2f v;
	if(g_perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);
	}
	else{
		v.x = sr.x;
		v.y = sr.y;
	}
	return v;
}

Vec2f convertPointToScreen2dNoMatrix(Vec3f sr){

	Vec2f v;
	if(g_perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);
	}
	else{
		v.x = sr.x;
		v.y = sr.y;
	}
	return v;
}

void getColorWithEyeDropRadius(Art* selPart, Seamless* smls){
	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(selPart->workEffect)){
		M_Color3f& color = colorEffect->color;
		M_Color3f& storeColor = g_brush->storeColor;
		M_Float& storeOpacity = g_brush->storeOpacity;
		float rad = g_brush->radius;
		float fadeRad = rad * (1.0 - g_brush->fade);
		float opacity = g_brush->opacity;
		float iopacity = 1.0 - opacity;
		bool backfaceIgnore = (g_scene->mode & BACKFACE_IGNORE_MODE) != 0;


		M_Triangle& triangle = selPart->triangle;
		Color3f mixColor = Color3f(0,0,0);
		int cntr = 0;

		for(int t = triangle.length; --t >= 0;){
			S_Triangle& tri = triangle[t];

			if(backfaceIgnore){
				Art* partA = tri.pa;
				Art* partB = tri.pb;
				Art* partC = tri.pc;
				Vec3f coordA = m_matrix[m_smi[partA->fromCoord + tri.ia]] * g_viewMatrix * (g_m_coord[partA->fromCoord + tri.ia] - partA->absolutePivotPoint);
				Vec3f coordB = m_matrix[m_smi[partB->fromCoord + tri.ib]] * g_viewMatrix * (g_m_coord[partB->fromCoord + tri.ib] - partB->absolutePivotPoint);
				Vec3f coordC = m_matrix[m_smi[partC->fromCoord + tri.ic]] * g_viewMatrix * (g_m_coord[partC->fromCoord + tri.ic] - partC->absolutePivotPoint);

				if(!backFace(coordA, coordB, coordC)){
					continue;
				}
			}

			Art* owner = tri.getOwner();

			if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(owner->workEffect)){
				M_Color3f& m_color = colorEffect->color.refValue();


				for(int i = 3; --i >= 0; ){
					Art* part = tri.refPart(i);
					if(part->show == false){
						continue;
					}
					int idx = tri.refIndex(i);
					Vec2f v = convertPointToScreen2d(part->coord[idx]);
					v.x -= g_pointer.translation.x;
					v.y -= g_pointer.translation.y;


					float ADJ = abs(v.x);
					float OPP = abs(v.y);
					float HYP = sqrt(ADJ * ADJ + OPP * OPP);

					int ti = tri.refTexIndex(i);
					if(HYP <= fadeRad){
						mixColor = mixColor + storeColor[ti];
						cntr++;
					}
				}
			}
		}
		if(cntr > 0){
			mixColor = mixColor / cntr;
			g_palette->color = mixColor;
			if(PaletteCP* paletteCP = paletteCPAllreadyOpen()){
				paletteCP->paint();
			}
		}
	}
}

void TextureEffect::getColorWithEyeDropRadius(){
	Art* actTriPart = SenseTugGlobals::actTriPart;
	Art* part = dynamic_cast<Art*>(SenseTugGlobals::node);
	S_Triangle& triangle = actTriPart->triangle[SenseTugGlobals::ActTri];
	int tIdx = triangle.refTexIndex(SenseTugGlobals::corner);
	Vec2f texCrd = texCoord[tIdx];

	g_viewMatrix = g_scene->getViewMatrix();

	Vec3f sa = g_viewMatrix * triangle.pa->coord[triangle.ia];
	Vec3f sb = g_viewMatrix * triangle.pb->coord[triangle.ib];
	Vec3f sc = g_viewMatrix * triangle.pc->coord[triangle.ic];

	Vec2f ta = texCoord[triangle.ta];
	Vec2f tb = texCoord[triangle.tb];
	Vec2f tc = texCoord[triangle.tc];

	Vec2f hit = Vec2f(g_pointer.translation.x, g_pointer.translation.y);

	relateHitPointToMapTri(Vec2f(sa.x, sa.y), Vec2f(sb.x, sb.y), Vec2f(sc.x, sc.y), hit, ta, tb, tc);

	unsigned int len = g_brush->storePixel.length;
	int pitch = g_brush->pitch;
	int cxy = pitch / 4;
	float fcxy = cxy;
	float pixSize = 1.0 / fcxy;

	Vec2i texCrdi = Vec2i(hit.x * fcxy, (1.0 - hit.y) * fcxy);

//	pBits[cxy * texCrdi.y + texCrdi.x] = color.colorToHex();

	M_Pix24& storePixel = g_brush->storePixel;
	float rad = g_brush->radius * g_scene->getZoom();
	float fadeRad = rad * (1.0 - g_brush->fade);

	Vec2f v;
	float ADJ = 0;
	float OPP = 0;
	float HYP = 0;
	bool newCycle = true;
	unsigned int i = 0;
	Color3f mixColor = Color3f(0,0,0);
	int cntr = 0;

	while(true){
		while(true){
			ADJ = abs(v.x);
			OPP = abs(v.y);
			HYP = sqrt(ADJ * ADJ + OPP * OPP);

			texCrdi = Vec2i((hit.x + v.x) * fcxy, (1.0 - (hit.y + v.y)) * fcxy);
			i = cxy * texCrdi.y + texCrdi.x;
			if(i >= len){
				break;
			}

			if(HYP <= rad){
				mixColor = mixColor + pix24ToColor(storePixel[i]);
				cntr++;
			}
			else{
				break;
			}
			newCycle = false;
			v.x += pixSize;
		}
		if(newCycle){
			break;
		}
		newCycle = true;
		v.x = 0;
		v.y += pixSize;
	}
	v.x = -pixSize;
	v.y = 0;
	newCycle = true;
	while(true){
		while(true){
			ADJ = abs(v.x);
			OPP = abs(v.y);
			HYP = sqrt(ADJ * ADJ + OPP * OPP);

			texCrdi = Vec2i((hit.x + v.x) * fcxy, (1.0 - (hit.y + v.y)) * fcxy);
			i = cxy * texCrdi.y + texCrdi.x;
			if(i >= len){
				break;
			}

			if(HYP <= rad){
				mixColor = mixColor + pix24ToColor(storePixel[i]);
				cntr++;
			}
			else{
				break;
			}
			newCycle = false;
			v.x -= pixSize;
		}
		if(newCycle){
			break;
		}
		newCycle = true;
		v.x = 0;
		v.y += pixSize;
	}
	v.x = 0;
	v.y = -pixSize;
	newCycle = true;
	while(true){
		while(true){
			ADJ = abs(v.x);
			OPP = abs(v.y);
			HYP = sqrt(ADJ * ADJ + OPP * OPP);

			texCrdi = Vec2i((hit.x + v.x) * fcxy, (1.0 - (hit.y + v.y)) * fcxy);
			i = cxy * texCrdi.y + texCrdi.x;
			if(i >= len){
				break;
			}

			if(HYP <= rad){
				mixColor = mixColor + pix24ToColor(storePixel[i]);
				cntr++;
			}
			else{
				break;
			}
			newCycle = false;
			v.x += pixSize;
		}
		if(newCycle){
			break;
		}
		newCycle = true;
		v.x = 0;
		v.y -= pixSize;
	}
	v.x = -pixSize;;
	v.y = -pixSize;
	newCycle = true;
	while(true){
		while(true){
			ADJ = abs(v.x);
			OPP = abs(v.y);
			HYP = sqrt(ADJ * ADJ + OPP * OPP);

			texCrdi = Vec2i((hit.x + v.x) * fcxy, (1.0 - (hit.y + v.y)) * fcxy);
			i = cxy * texCrdi.y + texCrdi.x;
			if(i >= len){
				break;
			}

			if(HYP <= rad){
				mixColor = mixColor + pix24ToColor(storePixel[i]);
				cntr++;
			}
			else{
				break;
			}
			newCycle = false;
			v.x -= pixSize;
		}
		if(newCycle){
			break;
		}
		newCycle = true;
		v.x = 0;
		v.y -= pixSize;
	}
	if(cntr > 0){
		mixColor = mixColor / cntr;
		g_palette->color = mixColor;
		if(PaletteCP* paletteCP = paletteCPAllreadyOpen()){
			paletteCP->paint();
		}
	}
}

bool compareColorTolerance(Color3f color, Color3f targetColor, float tolerance){
	if(
		abs(color.r - targetColor.r) <= tolerance &&
		abs(color.g - targetColor.g) <= tolerance &&
		abs(color.b - targetColor.b) <= tolerance
		){
		return true;
	}
	return false;
}

void createPNG24(const char* filename, int width, int height, Pix24* pixels);



inline bool colourWithFill(M_Art& partList, const Color3f& targetColor, M_Color3f& color, M_Bool& flagTriangle, M_Bool& flagCoord, const float tolerance){
	bool complete = true;
	for(int k = 0; k < partList.length; k++){
		Art* part = partList[k];
		M_Triangle& triangle = part->triangle;
		for(int t = triangle.length; --t >= 0;){
			if(flagTriangle[t + part->fromPoly] == false){
				S_Triangle& tri = triangle[t];
				for(int i = 3; --i >= 0; ){
					if(flagCoord[tri.refIndex(i) + tri.refPart(i)->fromCoord] && compareColorTolerance(color[tri.refTexIndex(i)], targetColor, tolerance)){
						flagTriangle[t + part->fromPoly] = true;
						complete = false;
						for(int i = 3; --i >= 0; ){
							if(compareColorTolerance(color[tri.refTexIndex(i)], targetColor, tolerance)){
								flagCoord[tri.refIndex(i) + tri.refPart(i)->fromCoord] = true;
							}
						}
						break;
					}
				}
			}
		}
	}
	return complete;
}

inline void fillColor(M_Art& partList, const Color3f& targetColor, const Color3f& newColor, M_Color3f& color, M_Bool& flagTriangle, const float tolerance){
	for(int k = 0; k < partList.length; k++){
		Art* part = partList[k];
		M_Triangle& triangle = part->triangle;
		for(int t = triangle.length; --t >= 0;){
			if(flagTriangle[t + part->fromPoly]){
				S_Triangle& tri = triangle[t];
				for(int i = 3; --i >= 0; ){
					int colorIdx = tri.refTexIndex(i);
					if(compareColorTolerance(color[colorIdx], targetColor, tolerance)){
						color[colorIdx] = newColor;
					}
				}
			}
		}
	}
}

void colourWithFill(Art* selPart, Seamless* smls){
	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(selPart->workEffect)){
		M_Color3f& color = colorEffect->color;
		Art* actTriPart = SenseTugGlobals::actTriPart;
		S_Triangle& tri = actTriPart->triangle[SenseTugGlobals::ActTri];
		M_Bool& flagTriangle = g_fill->flagTriangle;
		M_Bool& flagCoord = g_fill->flagCoord;
		float tolerance = g_fill->tolerance;

		Color3f targetColor = color[tri.refCorner(SenseTugGlobals::corner).t];

		//color[tri.ta] = g_palette->color;
		//color[tri.tb] = g_palette->color;
		//color[tri.tc] = g_palette->color;
		M_Art& partList = smls->partList;

		int coordCount = 0;
		int polyCount = 0;

		for(int k = 0; k < partList.length; k++){
			Art* part = partList[k];
			part->fromCoord = coordCount;
			part->fromPoly = polyCount;
			coordCount += part->coord.length;
			polyCount += part->triangle.length;
		}
		flagTriangle.length = 0;
		flagCoord.length = 0;
		flagTriangle.length = polyCount;
		flagCoord.length = coordCount;

		flagTriangle[SenseTugGlobals::ActTri + actTriPart->fromPoly] = true;
		for(int i = 3; --i >= 0; ){
			int colorIdx = tri.refTexIndex(i);
			if(targetColor == color[colorIdx]){
				flagCoord[tri.refIndex(i) + tri.refPart(i)->fromCoord] = true;
			}
		}

		while(false == colourWithFill(partList, targetColor, color, flagTriangle, flagCoord, tolerance));

		fillColor(partList, targetColor, g_palette->color, color, flagTriangle, tolerance);


		smls->ctrf = true;
	}
}


bool Seamless::colourWithBrush(Art* part){
	//return colourPatchesWithBrush();
	bool hit = false;

	float rad = g_brush->radius * g_scene->getZoom();
	float fadeRad = rad * (1.0 - g_brush->fade);

	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
		M_Color3f& color = colorEffect->color;
		//int tri = SenseTugGlobals::ActTri;
		//String str;
		//g_pointer.translation.toString(str);
		//dLog(str);
		M_Color3f& storeColor = g_brush->storeColor;
		M_Float& storeOpacity = g_brush->storeOpacity;
		float opacity = g_brush->opacity;
		float iopacity = 1.0 - opacity;
		bool backfaceIgnore = (g_scene->mode & BACKFACE_IGNORE_MODE) != 0;


		M_Triangle& triangle = part->triangle;


		for(int t = triangle.length; --t >= 0;){
			S_Triangle& tri = triangle[t];

			if(backfaceIgnore){
				Art* partA = tri.pa;
				Art* partB = tri.pb;
				Art* partC = tri.pc;
				Vec3f coordA = m_matrix[m_smi[partA->fromCoord + tri.ia]] * g_viewMatrix * (g_m_coord[partA->fromCoord + tri.ia] - partA->absolutePivotPoint);
				Vec3f coordB = m_matrix[m_smi[partB->fromCoord + tri.ib]] * g_viewMatrix * (g_m_coord[partB->fromCoord + tri.ib] - partB->absolutePivotPoint);
				Vec3f coordC = m_matrix[m_smi[partC->fromCoord + tri.ic]] * g_viewMatrix * (g_m_coord[partC->fromCoord + tri.ic] - partC->absolutePivotPoint);

				if(!backFace(coordA, coordB, coordC)){
					continue;
				}
			}

			Art* owner = tri.getOwner();

			if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(owner->workEffect)){
				M_Color3f& m_color = colorEffect->color.refValue();


				for(int i = 3; --i >= 0; ){
					Art* part = tri.refPart(i);
					if(part->show == false){
						continue;
					}
					int idx = tri.refIndex(i);
					Vec2f v = convertPointToScreen2d(part->coord[idx]);
					v.x -= g_pointer.translation.x;
					v.y -= g_pointer.translation.y;


					float ADJ = abs(v.x);
					float OPP = abs(v.y);
					float HYP = sqrt(ADJ * ADJ + OPP * OPP);

					int ti = tri.refTexIndex(i);
					if(HYP <= fadeRad){
						hit = true;
						//m_color[ti] = opacity * g_palette->color + iopacity * storeColor[ti];
						if(opacity > storeOpacity[ti]){
							//m_color[ti] = opacity * mixColor + iopacity * storeColor[ti];
							m_color[ti] = opacity * g_palette->color + iopacity * storeColor[ti];
							storeOpacity[ti] = opacity;
						}
					}
					else if(HYP <= rad){
						hit = true;
						float iratio = (HYP - fadeRad) / (rad - fadeRad);
						float ratio = (1.0 - iratio)  * opacity;
						iratio = (1.0 - ratio);
						if(ratio > 1.0){
							int g = 0;
						}
						if(ratio > storeOpacity[ti]){
							//m_color[ti] = ratio * g_palette->color + iratio * storeColor[ti];
							//m_color[ti] = ratio * mixColor + iratio * storeColor[ti];
							m_color[ti] = ratio * g_palette->color + iratio * storeColor[ti];
							storeOpacity[ti] = ratio;
						}
					}
					//else{
						//storeColor[ti] = m_color[ti];
						//storeOpacity[ti] = 0;
					//}
				}
			}
		}
		ctrf = true;
		//smls->doBuild();
	}
	if(nurbsLine.length){
		for(int k = nurbsLine.length; --k>=0;){
			NurbsLine* line = nurbsLine[k];
			if(line->cage){
				const M_Vec3f& dpoint = line->dpoint;
				M_Color3f& m_color = line->color;
				for(int i = dpoint.length; --i>=0;){
					Vec2f v = convertPointToScreen2d(dpoint[i]);
					v.x -= g_pointer.translation.x;
					v.y -= g_pointer.translation.y;


					float ADJ = abs(v.x);
					float OPP = abs(v.y);
					float HYP = sqrt(ADJ * ADJ + OPP * OPP);

					if(HYP <= fadeRad){
						hit = true;
						m_color[i] = g_palette->color;
					}
					else if(HYP <= rad){
						hit = true;
						m_color[i] = g_palette->color;
					}
				}
			}
		}
		if(hit){
			doBuild();
		}
	}
	return hit;
}

bool Seamless::colourPatchesWithBrush(){
	bool hit = false;

	float rad = g_brush->radius * g_scene->getZoom();
	float fadeRad = rad * (1.0 - g_brush->fade);

	for(int i = cpNodeList.length; --i >= 0;){
		if(ColorPatch* patch = dynamic_cast<ColorPatch*>(cpNodeList[i])){
			if(patch->cage){
				int uDim = patch->dim.x;
				int vDim = patch->dim.y;
				M_Vec3f& dpoint = patch->dpoint.refValue();
				int len = dpoint.length;
				M_Color3f& m_color = patch->color.refValue();
				if(m_color.length != len){
					m_color.length = len;
				}
				for(int i = len; --i>=0;){
					Vec2f v = convertPointToScreen2d(dpoint[i]);
					v.x -= g_pointer.translation.x;
					v.y -= g_pointer.translation.y;


					float ADJ = abs(v.x);
					float OPP = abs(v.y);
					float HYP = sqrt(ADJ * ADJ + OPP * OPP);

					if(HYP <= fadeRad){
						hit = true;
						m_color[i] = g_palette->color;
					}
					else if(HYP <= rad){
						hit = true;
						m_color[i] = g_palette->color;
					}
				}
			}
		}
	}
	if(hit){
		ctrf = true;
		updateColorFromColorMappers(partList[0]);
	}
	return hit;
}

void Seamless::updateColorFromColorMappers(Art* part){

	if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
		M_Color3f& color = colorEffect->color;

		M_Triangle& triangle = part->triangle;


		for(int t = triangle.length; --t >= 0;){
			S_Triangle& tri = triangle[t];

			Art* owner = tri.getOwner();

			if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(owner->workEffect)){
				M_Color3f& m_color = colorEffect->color.refValue();


				for(int i = 3; --i >= 0; ){
					Art* part = tri.refPart(i);
					int idx = tri.refIndex(i);

					if(ColorPatch* colorPatch = dynamic_cast<ColorPatch*>(part->patchOwner[idx])){
						int ti = tri.refTexIndex(i);
						m_color[ti] = colorPatch->getColor(part->uvCoord[idx]);
					}
				}
			}
		}
		ctrf = true;
	}
}


bool AddLathe::checkOrientationPoint(const bool perspective){
	Matrix matrix = translationMatrix(lathe->translation) * g_scene->getViewMatrix();
	for(int i = 0; i < orientationPointArray.length; i++){
		//if(checkWithInSights(perspective, matrix * (Rotation(1, 0, 0, PI * .5) * orientationPointArray[i]))){
		Vec3f sr = matrix * orientationPointArray[i];
		if(checkWithInSights(perspective, sr)){
			MultiSelect::withinSights = true;
			MultiSelect::withinSightsIndex = i;
			MultiSelect::withinSightsNode = this;
			g_cursorOverPointIlsBase->translation.refValue() = sr;
			changeModellingCursor(g_lockedOnPointCursor);
			return true;
		}
	}
//	if(MultiSelect::withinSights){
	MultiSelect::resetDistance();
	MultiSelect::withinSightsNode = 0;
	changeModellingCursor(g_addPointCursor);
//	}
	return false;
}

void initBrush(){
	g_visualClue = 0;
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		Seamless* smls = part->smls;
		if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
			g_brush->storeColor = colorEffect->color;
			g_brush->storeOpacity.length = 0;
			g_brush->storeOpacity.length = colorEffect->color.length;
			smls->buildPartList();
			smls->genDynamicMatricesAndPopulate();

			if(g_brush->smudge){
				getColorWithEyeDropRadius(part, smls);
			}
			if(smls->colourWithBrush(part)){
				tugFlag = true;
				SenseTugGlobals::actTriPart = part;
				return;
			}
		}
		else if(TextureEffect* textureEffect = dynamic_cast<TextureEffect*>(part->workEffect)){
			g_scene->senseTug(Matrix(),Matrix(), 0);//why do we need this for textureEffect when colorEffect above does not?
			textureEffect->copyPixelsToStore();
			g_brush->storeOpacity.length = 0;
			g_brush->storeOpacity.length = g_brush->storePixel.length;

			if(g_brush->smudge){
				textureEffect->getColorWithEyeDropRadius();
			}
			textureEffect->writePixelsToTexture(g_palette->color);
			tugFlag = true;
			SenseTugGlobals::actTriPart = part;
		}
	}
}

void TransformObj::selectNodeAsObject(){
	if(Seamless* smls = dynamic_cast<Seamless*>(MultiSelect::withinSightsNode.p)){
		if(mode == SELECT){
			smls->remObjTransform();
			remTranslation = translation;
			MultiSelect::clickedIndex = 0;
			
			MultiSelect::objectList.length = 0;
			MultiSelect::objectList += smls;
			g_transformObj->calcPivotPoint();
			Vec3f sr = g_scene->getIViewMatrix() * SenseTugGlobals::clickPos;
			//Vec3f sr = smls->getSmlsViewMatrix() * SenseTugGlobals::clickPos;
			MultiSelect::beginDragPos = sr;
			g_pointer.remTranslation = g_pointer.translation = SenseTugGlobals::clickPos;
			
			g_stw->setResultsForSelectingNode(smls);
			SenseTugGlobals::node = smls;
			tugFlag = true;
			showCursor(false);
			smls->houseKeep();
			return;
		}
		if(mode == TRANSLATE){
			refreshRemPoints();
			remTranslation = translation;
			g_pointer.translation = g_scene->getViewMatrix() * pivotPoint;
		}
		else if(mode == ROTATE){
			leverIdx = MultiSelect::withinSightsIndex;
			remRotCtrlPnt = rotCtrlPnt[leverIdx];
			remRotation = rotation;
			//setMode(getKeyMode());
		}

		SenseTugGlobals::node = g_visualClue;
		tugFlag = true;
		showCursor(false);

		return;
	}
}


bool clickNurbsCage(Seamless* smls, const bool perspective){
	NurbsPatch* selPatch = dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p);
	if(g_transformCP->selectObject){
		if(Build* bld = dynamic_cast<Build*>(g_selNode.getRaw())){
			selPatch = bld->getPatch();
		}
	}

	if(Lock* lock = dynamic_cast<Lock*>(selPatch)){
		if((g_scene->mode & HAIR_REGION_MODE)){
			if(Lock::splitRegionMode){
				Lock::splitRegionPoints += convertPointToScreen2dNoMatrix(g_pointer.translation);
				if(Lock::splitRegionPoints.length == 2){
					lock->splitRegion();
					Lock::splitRegionMode = false;
				}
				return true;
			}

			if((false == (altKeyState()) && ((shiftKeyState()) || (r_shiftKeyState()))) ||
				(false == InputEvent::l_altDown && (InputEvent::l_shiftDown || InputEvent::r_shiftDown))
			
				){
				if(false == controlKeyState() && false == InputEvent::l_ctrlDown && false == (r_controlKeyState()) && false == InputEvent::r_ctrlDown){
					MultiSelect::reset();
				}
				showCursor(false);
				MultiSelect::selectRectMode = true;
				if(MultiSelect::accumulate == false){
					smls->clearObjects();
				}

				MultiSelect::bottomRight=MultiSelect::topLeft=g_pointer.translation;
				g_vcSelectRectGroup->children.refValue().length = 0;
				g_vcSelectRectGroup->children.refValue()+=g_vcSelectRectShape.p;
				g_visualClue = g_vcSelectRectGroup;
				houseKeepVisualClue();
			}
			else{
				lock->initTugRegion(perspective);
			}
			return true;
		}
	}
	if(deleteKeyState()){
		smls->deleteControlRowColum();
		return true;
	}
	else if(g_deleteControlLine.mousePhase == 2){
		changeModellingCursor(g_crossCursor);

		if(g_deleteControlLine.m_delCtrlLn.length == 0){
			g_deleteControlLine.mousePhase = DeleteControlLine::NEUTRAL;
			return true;
		}
		if(g_deleteControlLine.selectRing){
			g_deleteControlLine.smls->doLastPhaseForSelectRing();
		}
		else{
			g_deleteControlLine.doLastPhase();
		}
		return true;
	}
	else if(g_insertPatch.imousePhase == InsertPatch::DIVERT_STANDBY && g_insertPatch.diversionHover){
		g_insertPatch.m_insert_tempDivertBackup = g_insertPatch.m_insert_loopBackup;
		g_insertPatch.m_insertOD_tempDivertBackup = g_insertPatch.m_insertOD_loopBackup;
		g_insertPatch.remLeftSide_tempDivertBackup = g_insertPatch.remLeftSide;

		g_insertPatch.beginInsertIdx_tempDivertBackup = g_insertPatch.beginInsertIdx;
		g_insertPatch.beginInsertIdxOD_tempDivertBackup = g_insertPatch.beginInsertIdxOD;
		g_insertPatch.diversionHover = false;
		g_insertPatch.foundFirstStitchPoly = false; //impossible now to loop back to first stitch poly?
		g_insertPatch.addInsertVisualClue();
		return true;
	}
	else if(g_mergePatches.mousePhase == MergePatches::STANDBY && g_mergePatches.patchA){
		g_mergePatches.doLastPhase();
		return true;
	}
	else if(((enterKeyState()) && false == ((r_shiftKeyState() ||  InputEvent::r_shiftDown))) ||
		(g_insertPatch.m_insert.length > 0 && (g_insertPatch.imousePhase == InsertPatch::STANDBY || g_insertPatch.imousePhase == InsertPatch::LINE_MODE))){
		if(NurbsLine* nurbsLine = dynamic_cast<NurbsLine*>(g_selNode.getRaw())){
			if(g_insertPatch.cutMode){
				nurbsLine->cut();
			}
			else{
				for(int k = smls->cpNodeList.length; --k>=0;){
					if(NurbsLine* nurbsLine = dynamic_cast<NurbsLine*>(smls->cpNodeList[k])){
						if(nurbsLine->insertControlPoint()){
							g_stw->setResultsForSelectingPoint(nurbsLine);
							houseKeepVisualClue();
							break;
						}
					}
				}
			}
			g_insertPatch.imousePhase = InsertPatch::NEUTRAL;
			
			return true;
		}
		if(Strand* strand = dynamic_cast<Strand*>(g_selNode.getRaw())){
			g_insertPatch.imousePhase = InsertPatch::NEUTRAL;
			strand->insertControlPoint();
			return true;
		}
		g_insertPatch.imousePhase = InsertPatch::MONITOR_DIRECTION;
		if(g_insertPatch.cutMode){
			g_stw->cp->setupNodeWindow(g_cutComplete);
		}
		else{
			g_stw->cp->setupNodeWindow(g_insertComplete);
		}
		Window::activeWindow = g_stw->cp.p;
		g_insertPatch.diversionHover = false;
		g_insertPatch.addInsertVisualClue();
		//selPatch->houseKeep();
		//smls->insertControlRowColum();
		return true;
	}
//	else if(((r_controlKeyState()) || InputEvent::r_ctrlDown) && false == ((r_shiftKeyState() ||  InputEvent::r_shiftDown))){
		//smls->insertControlRowColum();
		//g_insertPatch.imousePhase = InsertPatch::NEUTRAL;
		//commonInsertLine(smls, g_insertPatch.m_insert, g_insertPatch.m_diversionMark);
		//return true;
	//}

	else if((false == altKeyState() && shiftKeyState()) ||
		(false == InputEvent::l_altDown && (InputEvent::l_shiftDown)) ||
		g_rectangleSelectPointReady || g_rectangleUnselectPointReady || g_rectangleSelectPolyReady
			
		){
		if(false == controlKeyState() && false == InputEvent::l_ctrlDown && false == (r_controlKeyState()) && false == InputEvent::r_ctrlDown && g_rectangleUnselectPointReady == false){
			MultiSelect::reset();
		}
		MultiSelect::withinSights = false;
		MultiSelect::withinSightsNode = 0;

		showCursor(false);
		MultiSelect::selectRectMode = true;
		if(MultiSelect::accumulate == false){
			smls->clearObjects();
		}

		MultiSelect::bottomRight = MultiSelect::topLeft = g_pointer.translation;
		g_vcSelectRectGroup->children.refValue().length = 0;
		g_vcSelectRectGroup->children.refValue() += g_vcSelectRectShape.p;
		g_visualClue = g_vcSelectRectGroup;
		houseKeepVisualClue();
		return true;
	}
	else if(g_transformCP->isOverDimPoint){
		if(MultiSelect::withinSightsIndex == 0){
			g_transformCP->yTug.refValue() = false;
			g_transformCP->zTug.refValue() = false;
		}
		if(MultiSelect::withinSightsIndex == 1){
			g_transformCP->xTug.refValue() = false;
			g_transformCP->zTug.refValue() = false;
		}
		if(MultiSelect::withinSightsIndex == 2){
			g_transformCP->xTug.refValue() = false;
			g_transformCP->yTug.refValue() = false;
		}

		if(MultiSelect::withinSightsIndex == 3){
			g_transformCP->zTug.refValue() = false;
		}
		if(MultiSelect::withinSightsIndex == 4){
			g_transformCP->xTug.refValue() = false;
		}
		if(MultiSelect::withinSightsIndex == 5){
			g_transformCP->yTug.refValue() = false;
		}

		g_transformCP->dimFilterClickEnable = false;
		MultiSelect::withinSights = false;
		MultiSelect::withinSightsNode = MultiSelect::pointList[0].p;
		g_transformCP->remTranslation = g_transformCP->translation;
		//g_pointer.translation = g_cursorOverPointIlsBase->translation;
		g_transformCP->dimRadialAdjust = g_scene->rotation * g_transformCP->dimRadialAdjust;
		//g_transformCP->calcPivotPoint();
		//g_transformCP->resetRotationLever();
		backUpSmlsPoint(smls);

		tugFlag = true;
		showCursor(false);


		//changeModellingCursor(g_crossCursor);
		g_transformCP->houseKeep();
		houseKeepVisualClue();
		enablePen();
		return true;
	}
	else if(MultiSelect::pointList.length >= g_transformCP->leverSelection && g_transformCP->sameScreenPosition == false && MultiSelect::withinSights && MultiSelect::withinSightsNode == g_transformCP){
		if(g_transformCP->xTug && g_transformCP->yTug && g_transformCP->zTug){
			g_transformCP->showDimFilterVisual = false;
			g_transformCP->dimFilterClickEnable = false;
		}


		if(g_transformCP->isOverPivotPoint){
			g_transformCP->dragPivotPointMode = true;
			g_transformCP->refreshRemPoints();
		}
		else{
/*
			g_transformCP->leverIdx = MultiSelect::withinSightsIndex;
			g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];
			g_pointer.translation = g_transformCP->remRotCtrlPnt;
			g_transformCP->clickPoint = g_transformCP->remRotCtrlPnt;
			g_transformCP->dragRotPointMode = true;
			g_transformCP->setMode(g_transformCP->getKeyMode());
			//

			g_transformCP->remTranslation = g_transformCP->translation;
			g_pointer.translation = g_cursorOverPointIlsBase->translation;
			g_transformCP->dimRadialAdjust = g_scene->rotation * g_transformCP->dimRadialAdjust;
			backUpSmlsPoint(smls);

			tugFlag = true;
			showCursor(false);


			g_transformCP->houseKeep();
			houseKeepVisualClue();
*/


			g_transformCP->leverIdx = MultiSelect::withinSightsIndex;
			g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];
			//g_pointer.translation = g_transformCP->remRotCtrlPnt;
			g_transformCP->dragRotPointMode = true;
			g_transformCP->setMode(g_transformCP->getKeyMode());
			g_transformCP->remTranslation = g_transformCP->translation;
			g_transformCP->clickPoint = g_scene->getIViewMatrix() * g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];
			g_transformCP->calcScaleOrientation();
			g_transformCP->calcLeverAngle();


			//g_transformCP->calcScaleOrientation();

			//Matrix matrix = translationMatrix(-g_transformCP->pivotPoint) * rotationMatrix(-g_transformCP->scaleOrientation) * scaleMatrix(g_transformCP->scale) * rotationMatrix(g_transformCP->scaleOrientation) * rotationMatrix(g_transformCP->rotation) * translationMatrix(g_transformCP->pivotPoint) * translationMatrix(g_transformCP->remTranslation);
			//float inverseSize = (1 / g_scene->getZoom());
			//Rotation rot = Rotation(0, 0, 1, g_transformCP->leverAngle);
			//g_transformCP->clickPoint = (g_transformCP->pivotPoint + g_transformCP->translation + -g_scene->rotation * rot * (g_transformCP->leverAxis[g_transformCP->leverIdx] * g_transformCP->leverDistance[g_transformCP->leverIdx] * inverseSize));
			
		}

		g_transformCP->isOverDimPoint = false;
		g_transformCP->dimRadialAdjust = Vec3f();
		g_transformCP->showDimFilterVisual = false;
		g_transformCP->dimFilterClickEnable = false;
		g_transformCP->xTug.refValue() = true;
		g_transformCP->yTug.refValue() = true;
		g_transformCP->zTug = true;


		SenseTugGlobals::node = g_visualClue;
		tugFlag = true;
		showCursor(false);
		enablePen();

		return true;
	}

	else if(g_transformCP->sameScreenPosition){
		MultiSelect::reset();
		//showCursor(false);
		//MultiSelect::selectRectMode = true;
		if(g_transformCP->xTug && g_transformCP->yTug && g_transformCP->zTug){
			g_transformCP->showDimFilterVisual = false;
			g_transformCP->dimFilterClickEnable = false;
		}

		MultiSelect::bottomRight = MultiSelect::topLeft = g_pointer.translation + Vec3f(-.01, .01, 0);
		MultiSelect::bottomRight = MultiSelect::bottomRight = g_pointer.translation + Vec3f(.01, -.01, 0);
		smls->selectControlPoints();

		//g_vcSelectRectGroup->children.refValue().length=0;
		//g_vcSelectRectGroup->children.refValue()+=g_vcSelectRectShape.p;
		//g_visualClue=g_vcSelectRectGroup;
		if(MultiSelect::pointList.length){
			MultiSelect::pointList[0].p->index[0] = MultiSelect::pointList[0].i;
			NurbsPatch* patch = MultiSelect::pointList[0].p;
			g_pointer.translation = g_scene->getViewMatrix() * patch->dpoint[MultiSelect::pointList[0].i];

			if(patch != selPatch){
				g_stw->setResultsForSelectingPoint(patch);
			}
			tugFlag = true;
			MultiSelect::clickedIndex = 0;
			showCursor(false);
			g_transformCP->refreshRemPoints();

			houseKeepVisualClue();
			fastFullBuildForAutoIn(false);
		}
		return true;
	}

	//smls->initMultiCageTug(selPatch, perspective);
	else if(controlKeyState() && altKeyState()){
		MultiSelect::withinSights = false;
		MultiSelect::withinSightsNode = 0;
	}
	else if(MultiSelect::withinSights && dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p)){
		//if(g_transformCP->xTug && g_transformCP->yTug && g_transformCP->zTug){
		if(g_transformCP->showDimFilterVisual && false == tabKeyState()){
			g_transformCP->isOverDimPoint = false;
			g_transformCP->dimRadialAdjust = Vec3f();
			g_transformCP->showDimFilterVisual = false;
			g_transformCP->dimFilterClickEnable = false;
			g_transformCP->xTug.refValue() = true;
			g_transformCP->yTug.refValue() = true;
			g_transformCP->zTug = true;
		}
		selPatch = dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p);
		smls->clickPointWithinSights(selPatch);

		if(MultiSelect::pointList.length && g_radial_Tug->active){
			selPatch = MultiSelect::pointList[MultiSelect::clickedIndex].p;
			int idx = MultiSelect::pointList[MultiSelect::clickedIndex].i;
			MultiSelect::pointList = ControlPoint(selPatch, idx);
			MultiSelect::beginDragPos = selPatch->dpoint[idx];
			MultiSelect::clickedIndex = 0;
			selPatch->index[0] = idx;
			if(checkRadialTugSizeNurbs(perspective, selPatch)){
				return true;
			}
			backUpSmlsPoint(smls);
		}

			
/*
	selPatch->initTug(perspective);
	*/
//		if(selPatch->ringTug){
//			MultiSelect::beginDragPos = selPatch->dpoint;
//		}
		

		if(tugFlag){
			houseKeepVisualClue();
			g_transformCP->remTranslation = g_transformCP->translation;
			int selLine = g_stw->selLine;
			if(Strand* strand = dynamic_cast<Strand*>(g_sceneTreeList[selLine].node.p)){
				strand->index[0] = strand->NurbsPatch::index[0];
				selLine = getParent(selLine);
				g_parent = g_sceneTreeList[selLine].node;
				if(Hair* hair = dynamic_cast<Hair*>(g_parent)){					
					hair->isBoundToLock(strand);
				}
			}
			else if(Lock* lock = dynamic_cast<Lock*>(g_sceneTreeList[selLine].node.p)){
				lock->index[0] = lock->NurbsPatch::index[0];
			}
			else if(NurbsLine* line = dynamic_cast<NurbsLine*>(g_sceneTreeList[selLine].node.p)){
				line->index[0] = line->NurbsPatch::index[0];
				g_stw->cp->paint();
			}
			if(MultiSelect::withinSightsIsSelected == false){
				g_transformCP->calcPivotPoint();
				g_transformCP->calcRotCtrlPnts();
			}

			Matrix matrix = translationMatrix(-g_transformCP->pivotPoint) * rotationMatrix(-g_transformCP->scaleOrientation) * scaleMatrix(g_transformCP->scale) * rotationMatrix(g_transformCP->scaleOrientation) * rotationMatrix(g_transformCP->rotation) * translationMatrix(g_transformCP->pivotPoint) * translationMatrix(g_transformCP->remTranslation);
			g_transformCP->clickPoint = matrix * MultiSelect::beginDragPos[MultiSelect::clickedIndex];

			//houseKeepVisualClue();
		}
		g_transformCP->houseKeep();
		fastFullBuildForAutoIn(false);
		enablePen();
		return true;
	}
	return false;
}

void SenseTugGlobals::initMidTug(){
	mouse2dSync();
							

	tugFlag = true;
	g_viewDragMousePos = g_pointer.translation;

	g_mouseFrom = g_pointer.translation;
	g_viewPosDragMode = true;
	MultiSelect::withinSights = false;
	MultiSelect::withinSightsNode = 0;
	changeModellingCursor(g_crossCursor);
	return;
}


extern bool g_winAPI3DMode;
extern bool g_viewRotDragMode;
extern bool g_zoomSpin;
extern bool g_leftButtonZoom;
extern bool g_radiusControl;
extern float g_beginRadius;


void initRadiusControl(){
	g_winAPI3DMode = true;
	g_radiusControl = true;
	
	g_beginMousePos = g_mousePos;
	if(g_scene->mode & MODELLING_MODE){
		g_beginRadius = g_radial_Tug->radius;
	}
	else if(g_scene->mode & (PAINT_MODE | BRUSH_MODE)){
		g_beginRadius = g_brush->radius;
	}
	mouse2dSync();
}

void initRotateView(){
	g_winAPI3DMode = true;
	g_viewRotDragMode = true;
		
	g_beginMousePos = g_mousePos;
	g_beginViewRot = g_scene->rotation;
	mouse2dSync();
}

void initZoomSpin(){
	g_winAPI3DMode = true;
	g_zoomSpin = true;
	g_beginMousePos = g_mousePos;
	g_beginViewRot = g_scene->rotation;
	mouse2dSync();
}

void SenseTugGlobals::initTug(){
#ifdef PROG_ROB
	dLog("new Move("+ String(g_mousePos.x) + ',' + String(g_mousePos.y) + "),");
	dLog("new LeftButtonDown(),");
#endif
	mouse2dSync();
	if((MultiSelect::withinSights && MultiSelect::withinSightsNode == g_transformCP) == false && altKeyState()){
		if(controlKeyState()){
			SenseTugGlobals::initMidTug();
			return;
		}
		if(shiftKeyState()){
			initRotateView();
			return;
		}
		initZoomSpin();
		return;
	}
	if(zKeyState()){
		if(shiftKeyState()){
			initRadiusControl();
			return;
		}
		else{
			initRotateView();
			return;
		}
	}

//	if(!(shiftKeyState())){
//			int g = 0;
//	}
	//if(g_scene->mode & BRUSH_MODE){
	//	SenseTugGlobals::actTriPart = 0;
	//}

	if(g_viewPosDragMode){
		return;
	}

	g_viewMatrix = g_scene->getViewMatrix();

	if(g_panViewMode){
		return;
	}
	//if(g_leftDoubleClicked && g_scene->mode & HOME_MODE){
	//	setModellingMode();
	//	houseKeepVisualClue();
	//	return;
	//}

	node = 0;


	const bool perspective = g_perspective;

	g_transformCP->dragPivotPointMode = false;
	g_transformCP->dragRotPointMode = false;
	g_transformCP->stretchHappened = false;

	//if(g_leftDoubleClicked){
	//	if(g_addNurbsLine->mousePhase == AddNurbsLine::NEUTRAL){
	//		//addNurbsLine();
	//		//g_addNurbsLine->addPoint();
	//	}
	//	else{
	//		g_addNurbsLine->done = true;
	//	}
	//	setHomeMode();
	//	return;
	//}
	if(g_addNurbsLine->mousePhase == AddNurbsLine::INITIAL_POINTS){
		g_addNurbsLine->addPoint();
		return;
	}
	if(g_drawNurbsLine->mousePhase == DrawNurbsLine::DRAW){
		g_drawNurbsLine->addPoint();
		return;
	}
	if(g_addLathe->mousePhase != AddLathe::NEUTRAL){
		g_addLathe->addPoint();
		return;
	}
	if(g_addBone->mousePhase != AddBone::NEUTRAL && MultiSelect::withinSights == false){
		g_addBone->addBone();
	}
	if(g_addSphere.mousePhase != AddSphere::NEUTRAL){
		g_addSphere.addPoint();
		return;
	}
	if(g_addBox.mousePhase != AddBox::NEUTRAL){
		g_addBox.addPoint();
		return;
	}
	if((g_scene->mode & (SELECT_OBJECT_MODE | ANIMATION_MODE)) && Viewpoint::withinSitesViewpoint){
		g_stw->setResultsForSelectingNode(Viewpoint::withinSitesViewpoint);
		return;
	}

	if(Viewpoint* selView = dynamic_cast<Viewpoint*>(MultiSelect::withinSightsNode.p)){
		if(MultiSelect::withinSights){
			SenseTugGlobals::node = selView;
			tugFlag = true;
			if(Viewpoint::dragMode == Viewpoint::LOOK_AT){
				Viewpoint::remOrientation = selView->orientation;
				Viewpoint::remLookAt = selView->lookAt;
				Viewpoint::remLookAtDistance = (Viewpoint::remLookAt - selView->position).length();
				Viewpoint::remUpDistance = selView->up.refValue().length();
				g_pointer.translation = g_scene->getViewMatrix() * selView->lookAt;
			}
			else if(Viewpoint::dragMode == Viewpoint::UP_LEVER){
				g_pointer.translation = g_scene->getViewMatrix() * (selView->up + selView->position);
			}
			else if(Viewpoint::dragMode == Viewpoint::POSITION){
				Viewpoint::remOrientation = selView->orientation;
				Viewpoint::remLookAt = selView->lookAt;
				Viewpoint::remPosition = selView->position;
				Viewpoint::remLookAtDistance = (Viewpoint::remLookAt - selView->position).length();
				Viewpoint::remUpDistance = selView->up.refValue().length();

				g_pointer.translation = g_scene->getViewMatrix() * selView->position;
			}
			showCursor(false);
			return;
		}
	}
	if(g_scene->mode & SELECT_OBJECT_MODE){
		if(MultiSelect::withinSights){
			g_transformObj->selectNodeAsObject();
		}
		return;
	}

	if(NurbsLathe* nurbsLathe = dynamic_cast<NurbsLathe*>(g_selNode.getRaw())){
		if(g_insertPatch.imousePhase == InsertPatch::STANDBY){
			g_insertPatch.imousePhase = InsertPatch::NEUTRAL;
			nurbsLathe->insertControlPoint();
			return;
		}

		if(enterKeyState()){
			nurbsLathe->insertControlPoint();
			return;
		}
		if(MultiSelect::withinSights){
			if(false == controlKeyState()){
				tugFlag = true;
				showCursor(false);
			}
			Matrix matrix = nurbsLathe->matrix * g_viewMatrix;
			Vec2f vec2f = nurbsLathe->dpoint[MultiSelect::clickedIndex];
			g_pointer.translation = matrix * Vec3f(vec2f.x, vec2f.y, 0);

			nurbsLathe->index[0] = MultiSelect::clickedIndex;
			nurbsLathe->houseKeep();
			nurbsLathe->processFields();
			g_stw->cp->paint();
			fastFullBuildForAutoIn(false);
			tugFlag = true;
			return;
		}
	}
	if(g_scene->mode & MODELLING_MODE && (MultiSelect::withinSightsNode.p == 0 || dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p) || dynamic_cast<TransformCP*>(MultiSelect::withinSightsNode.p))){
		if(Seamless* smls = getSeamless2()){
			if(clickNurbsCage(smls, perspective)){
				return;
			}
//			else if(g_drawNurbsLine->mousePhase == DrawNurbsLine::READY_TO_DRAG){
//				drawNurbsLine();
//				g_drawNurbsLine->addPoint();
//				return;
//			}
		}
	}
	if(Subdivision* subdivision=dynamic_cast<Subdivision*>(g_selNode.getRaw())){
		int& idx = subdivision->index[0];
		Matrix matrix = g_viewMatrix;
		int j=subdivision->index[0];
		M_Vec3f& point = subdivision->point;
		for(int i=point.length; --i>=0;){
			if (checkPoint(perspective, matrix * point[i], idx, i)){
				g_selNode->processFields();
				subdivision->makePointerEqualIndexLocation();
				subdivision->houseKeep();
				g_stw->cp->paint();
				fastFullBuildForAutoIn(false);
				return;
			}
		}
		return;
	}
	else if(EC2* ec2=dynamic_cast<EC2*>(g_selNode.getRaw())){
		Matrix matrix = rotationMatrix(ec2->orientation) * translationMatrix(ec2->location) * g_viewMatrix;
		int j=ec2->index[0];
		if(checkOrientationPoint(perspective, ec2->point[j], ec2->tangent[j], g_pointer.translation, tugFlag, matrix)){
			g_selNode->processFields();
			g_stw->cp->paint();
			if(g_controlPanelOpen==true){
//					g_stw->setSelNodeLine();
			}
			return;
		}
		for(int i=ec2->point.length; --i>=0;){
			if(checkPoint(perspective, ec2->point[i], ec2->index[0], i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
	}
	else if(EC3* ec3=dynamic_cast<EC3*>(g_selNode.getRaw())){
		Matrix matrix = rotationMatrix(ec3->orientation) * translationMatrix(ec3->location) * g_viewMatrix;
		int j=ec3->index[0];
		if (checkOrientationPoint(perspective, Vec2f(ec3->point[j].x, ec3->point[j].y), ec3->tangent[j], g_pointer.translation, tugFlag, matrix)){
			g_selNode->processFields();
			g_stw->cp->paint();
			if(g_controlPanelOpen==true){
//					g_stw->setSelNodeLine();
			}
			return;
		}
		for(int i=ec3->point.length; --i>=0;){
			if (checkPoint(perspective, *ec3, i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				if(g_controlPanelOpen==true){
//						g_stw->setSelNodeLine();
				}
				return;
			}
		}
	}
	else if(BezierTape* bezier=dynamic_cast<BezierTape*>(g_selNode.getRaw())){
		Matrix matrix = rotationMatrix(Rotation(0,-1,0,PI*.5)) * g_viewMatrix;
		int j=bezier->index[0];
		for(int i=bezier->point.length; --i>=0;){
			if (checkPoint(perspective, bezier->point.refValue()[i], bezier->index[0], i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(Bezier3f* bezier=dynamic_cast<Bezier3f*>(g_selNode.getRaw())){
		Matrix matrix = bezier->matrix * g_viewMatrix;
		int j=bezier->index[0];
		for(int i=bezier->point.length; --i>=0;){
			if (checkPoint(perspective, matrix * bezier->point[i], bezier->index[0], i)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(Bezier* bezier=dynamic_cast<Bezier*>(g_selNode.getRaw())){
		Matrix matrix = bezier->matrix * g_viewMatrix;
		int j=bezier->index[0];
		for(int i=bezier->point.length; --i>=0;){
			if (checkPoint(perspective, bezier->point[i], bezier->index[0], i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(BezierPatch* bezierPatch=dynamic_cast<BezierPatch*>(g_selNode.getRaw())){
		if(MultiSelect::selectRectMode){
			MultiSelect::bottomRight=MultiSelect::topLeft=g_pointer.translation;
			//Patch::bottomRight+=Vec3f(.2, -.2, .0);
			g_vcSelectRectGroup->children.refValue().length = 0;
			g_vcSelectRectGroup->children.refValue()+=g_vcSelectRectShape.p;
			g_vcSelectRectGroup->children.refValue()+=g_vcBezierBase.p;
			g_visualClue = g_vcSelectRectGroup;
			bezierPatch->houseKeep();
		}
		else{
			Matrix matrix = g_viewMatrix;
			int j=bezierPatch->index[0];
			int len=bezierPatch->point.length;
			const M_Vec3f& point = bezierPatch->point;
			for(int i=0; i<len; i++){
				if (checkPoint(perspective, matrix * point[i], bezierPatch->index[0], i)){
					bezierPatch->setNumOfGroupFromSelPoint();
					bezierPatch->houseKeep();
					g_selNode->processFields();
					g_stw->cp->paint();
					return;
				}
			}
		}
		return;
	}
	else if(RingBuild* ringBuild=dynamic_cast<RingBuild*>(g_selNode.getRaw())){
		Matrix matrix = scaleMatrix(ringBuild->scale.refValue()) * rotationMatrix(ringBuild->rotation.refValue()) * translationMatrix(ringBuild->translation.refValue()) * g_viewMatrix;
		int j=ringBuild->index[0];
		if (ringBuild->checkPoint(perspective, ringBuild->index[0], j, matrix)){
			g_selNode->processFields();
			g_stw->cp->paint();
			return;
		}
		for(int i=ringBuild->scaleX.length; --i>=0;){
			if (ringBuild->checkPoint(perspective, ringBuild->index[0], i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(SineTug* sineTug=dynamic_cast<SineTug*>(g_selNode.getRaw())){
		const M_Vec3f& location = sineTug->location;
		const M_Rotation& orientation = sineTug->orientation;
		const M_Float& distance = sineTug->distance;
		int j=sineTug->index[0];
		//if (sineTug->checkPoint(perspective, ringBuild->index[0], j, matrix)){
		//	g_selNode->processFields();
		//	g_stw->cp->paint();
		//	return;
		//}
		for(int i=sineTug->radius.length; --i>=0;){
			Matrix matrix = rotationMatrix(orientation[i]) * translationMatrix(location[i]) * g_viewMatrix;
			if(checkPoint(perspective, matrix * Vec3f(0, distance[i]*8, 0), sineTug->index[0], i)){
				sineTug->houseKeep();
				g_selNode->processFields();
				g_stw->cp->paint();
				fastFullBuildForAutoIn(false);
			}
		}
		return;
	}

	else if(BendyLathe* bendyLathe = dynamic_cast<BendyLathe*>(g_selNode.getRaw())){
		Matrix matrix = bendyLathe->matrix * g_viewMatrix;
		if(bendyLathe->showRingControlPoints){
			for(int i = bendyLathe->radius.length; --i>=0;){
				if(bendyLathe->checkPoint(perspective, i, matrix)){
					bendyLathe->index[2]=i;
					g_selNode->processFields();
					bendyLathe->houseKeep();
					g_stw->cp->paint();
					fastFullBuildForAutoIn(false);
					return;
				}
			}
		}
		else{
			int& idx = bendyLathe->index[0];
			M_Vec3f& dpoint = bendyLathe->dpoint;
			for(int i=dpoint.length; --i>=0;){
				if(checkPoint(perspective, matrix * dpoint[i], idx, i)){
					g_selNode->processFields();
					g_stw->cp->paint();
					fastFullBuildForAutoIn(false);
					return;
				}
			}
		}
		return;
	}
	else if(Hair* hair = dynamic_cast<Hair*>(g_selNode.getRaw())){
		hair->initTug();
		return;
	}
	else if(NikInterpolator* niki = dynamic_cast<NikInterpolator*>(MultiSelect::withinSightsNode.p)){
		if(MultiSelect::withinSights && niki->smls){
			M_Vec3f& focusPoint = niki->focusOffset ? niki->shortPoint : niki->longPoint;

			niki->selPointIdx = MultiSelect::clickedIndex;

			Matrix matrix = g_viewMatrix;
			Vec3f vec3f = focusPoint[MultiSelect::clickedIndex];
			g_pointer.translation = matrix * vec3f;

			if(Anim* anim = Window::getAnimBar()){
				niki->setIndex(anim->slider, niki->selPointIdx, focusPoint.length * 2);
			}




			niki->houseKeep();
			g_stw->cp->paint();
			fastFullBuildForAutoIn(false);
			if(false == controlKeyState()){
				tugFlag = true;
				showCursor(false);
			}
		}
		return;
	}
	else if(NurbsTrack* nurbsTrack = dynamic_cast<NurbsTrack*>(g_selNode.getRaw())){
		if((r_controlKeyState()) || InputEvent::r_ctrlDown){
			nurbsTrack->insertControlPoint();
			return;
		}
		if(MultiSelect::withinSights){
			int& idx = nurbsTrack->index[1];
			idx = MultiSelect::clickedIndex;
			M_Vec3f& keyValue = nurbsTrack->keyValue.refValue();


			Matrix matrix = g_viewMatrix;
			Vec3f vec3f = keyValue[MultiSelect::clickedIndex];
			g_pointer.translation = matrix * vec3f;

			nurbsTrack->houseKeep();
			nurbsTrack->processFields();
			g_stw->cp->paint();
			fastFullBuildForAutoIn(false);
			if(false == controlKeyState()){
				tugFlag = true;
				showCursor(false);
			}
		}
		return;
	}
	else if(Oscillator* oscillator=dynamic_cast<Oscillator*>(g_selNode.getRaw())){
		int& idx = oscillator->index[0];
		Matrix matrix = g_viewMatrix;
		int j=oscillator->index[0];
		M_Vec2f& fPoint = oscillator->note;
		M_Float& fWeight = oscillator->fWeight;
		for(int i=fPoint.length; --i>=0;){
			float w = 1;
			if(fWeight.length){
				w = fWeight[i];
			}
			if (checkPoint(perspective, Vec2f(fPoint[i].x/w, fPoint[i].y/w*.1), idx, i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(Envelope* envelope=dynamic_cast<Envelope*>(g_selNode.getRaw())){
		int& idx = envelope->index[0];
		Matrix matrix = g_viewMatrix;
		int j=envelope->index[0];
		M_Vec2f& point = envelope->point;
		M_Float& weight = envelope->weight;
		for(int i = point.length; --i>=0;){
			float w = 1;
			if(weight.length){
				w = weight[i];
			}
			if (checkPoint(perspective, Vec2f(point[i].x/w, point[i].y/w*.1), idx, i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(AmplitudeModulator* modulator=dynamic_cast<AmplitudeModulator*>(g_selNode.getRaw())){
		int& idx = modulator->index[0];
		Matrix matrix = g_viewMatrix;
		int j=modulator->index[0];
		M_Vec2f& point = modulator->wet;
		M_Float& weight = modulator->weight;
		for(int i = point.length; --i>=0;){
			float w = 1;
			if(weight.length){
				w = weight[i];
			}
			if (checkPoint(perspective, Vec2f(point[i].x/w, point[i].y/w*.1), idx, i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(FrequencyModulator* modulator=dynamic_cast<FrequencyModulator*>(g_selNode.getRaw())){
		int& idx = modulator->index[0];
		Matrix matrix = g_viewMatrix;
		int j=modulator->index[0];
		M_Vec2f& point = modulator->wet;
		M_Float& weight = modulator->weight;
		for(int i = point.length; --i>=0;){
			float w = 1;
			if(weight.length){
				w = weight[i];
			}
			if (checkPoint(perspective, Vec2f(point[i].x/w, point[i].y/w*.1), idx, i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(Chords* chords=dynamic_cast<Chords*>(g_selNode.getRaw())){
		int& idx = chords->index[0];
		Matrix matrix = g_viewMatrix;
		int j=chords->index[0];
		M_Vec2f& point = chords->spectrum;
		M_Float& weight = chords->weight;
		for(int i = point.length; --i>=0;){
			float w = 1;
			if(weight.length){
				w = weight[i];
			}
			if (checkPoint(perspective, Vec2f(point[i].x/w, point[i].y/w*.1), idx, i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(HarmonicPatch* harmonicPatch=dynamic_cast<HarmonicPatch*>(g_selNode.getRaw())){
		int& idx = harmonicPatch->index[0];
		Matrix matrix = g_viewMatrix;
		int j=harmonicPatch->index[0];
		M_Vec3f& point = harmonicPatch->point;
		M_Float& weight = harmonicPatch->weight;
		for(int i=point.length; --i>=0;){
			if (checkPoint(perspective, matrix * point[i]/weight[i], idx, i)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		return;
	}
	else if(ColorSweep* colorSweep=dynamic_cast<ColorSweep*>(g_selNode.getRaw())){
		int& idx = colorSweep->index[0];
		Matrix matrix = colorSweep->matrix * g_viewMatrix;
		int j=colorSweep->index[0];
		M_Float& distance = colorSweep->distance;
		float posY = 0;

		for(int i = 0; ;i++){
			if(checkPoint(perspective, Vec2f(0, posY), idx, i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
			if(i == distance.length){
				break;
			}
			posY+=distance[i];
		}
		fastFullBuildForAutoIn(false);
		return;
	}
	else if(CCLathe* ccLathe=dynamic_cast<CCLathe*>(g_selNode.getRaw())){
		fastFullBuildForAutoIn(false);
		Matrix matrix = g_viewMatrix;
		int idx = ccLathe->index[0];

		ccLathe->angleDrag = false;

		if ( idx < ccLathe->seg.length )
		{
			// check  current first
			if (checkPoint(perspective, ccLathe->seg[idx].Pf, ccLathe->index[0], idx, matrix))
			{
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}

			// check angle control point
			if (checkPoint(perspective, ccLathe->anglePt, ccLathe->angleDrag, 1, matrix))
			{
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}

		}
		for(int i=ccLathe->seg.length; --i>=0;){
			if (checkPoint(perspective, ccLathe->seg[i].Pf, ccLathe->index[0], i, matrix)){
				g_selNode->processFields();
				g_stw->cp->paint();
				return;
			}
		}
		// add point
		if(controlKeyState()){
			ccLathe->addControlPoint(g_pointer.translation);
		}

		return;
	}

	//if(((g_scene->mode & (HOME_MODE | ANIMATION_MODE | POLY_EDIT_MODE | XRAY_EDIT_MODE | OWNERSHIP_MODE | NURBS_OWNERSHIP_MODE | PAINT_MODE | SELECT_OBJECT_MODE)) || (g_scene->mode & MODELLING_MODE && MultiSelect::withinSightsNode == 0)) && g_insertControlPoint == false && g_deleteControlPoint == false && MultiSelect::selectRectMode == false){
	if(shiftKeyState() == false){
		if(((g_scene->mode & HOME_MODE) || (g_scene->mode & (MODELLING_MODE | ANIMATION_MODE | POLY_EDIT_MODE | XRAY_EDIT_MODE | OWNERSHIP_MODE | NURBS_OWNERSHIP_MODE) && MultiSelect::withinSightsNode == 0) || (g_scene->mode & PAINT_MODE && altKeyState())) && g_insertControlPoint == false && g_deleteControlPoint == false && MultiSelect::selectRectMode == false){
			SenseTugGlobals::initMidTug();
			return;
		}
	}

	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		//g_joinActive = false;
		Seamless* smls = part->smls;
		if((false == (altKeyState()) && ((shiftKeyState()) || (r_shiftKeyState()))) ||
			(false == InputEvent::l_altDown && (InputEvent::l_shiftDown || InputEvent::r_shiftDown))
		
			){
			MultiSelect::reset();
			showCursor(false);
			MultiSelect::selectRectMode = true;

			MultiSelect::bottomRight = MultiSelect::topLeft = g_pointer.translation;
			//Patch::bottomRight+=Vec3f(.2, -.2, .0);
			g_vcSelectRectGroup->children.refValue().length = 0;
			g_vcSelectRectGroup->children.refValue() += g_vcSelectRectShape.p;
			//g_vcSelectRectGroup->children.refValue() += g_vcBezierBase.p;
			g_visualClue = g_vcSelectRectGroup;
			part->houseKeep();
			return;
		}
		//else if(part->smls->xRay && MultiSelect::withinSights){
		else if(part->smls->xRay && MultiSelect::withinSights && g_partCageClue->mode == PartCageClue::BONE_PIVOT_POINT){
		//else if(smls->animCP && dynamic_cast<AnimClue*>(g_visualClue) && g_visualClue == g_partCageClue && g_partCageClue->mode == PartCageClue::BONE_PIVOT_POINT){
			if(MultiSelect::withinSights){
				if(Art* part = dynamic_cast<Art*>(MultiSelect::withinSightsNode.p)){
					if(MultiSelect::withinSightsNode != g_selNode.getRaw()){
						g_stw->setResultsForSelectingVertex(part);
					}
					smls->ActTri = 0;
					smls->ActTriCorner = 0;
					smls->actTriPart = 0;

					g_addBone->beginDragPivotPoint.length = 0;
					g_addBone->beginDragPivotPoint += part->pivotPoint;

					M_SArt& m_sart = part->part.refValue();
				
					for(int k = 0; k < m_sart.length; k++){
						g_addBone->beginDragPivotPoint += m_sart[k]->pivotPoint;
					}
					//g_transformCP->mouseOffset = part->pivotPointMatrix * Vec3f(0,0,0) - g_pointer.translation;
				}
			}

			//changeModellingCursor(g_lockedOnPointCursor);
			tugFlag = true;
			showCursor(false);
			part->houseKeep();


			if(g_scene->mode & ANIMATION_MODE){
				g_vcArtAnimBase->calcRotCtrlPnts();
				g_vcArtAnimBase->houseKeep();
			}

			SenseTugGlobals::node = g_visualClue;


			return;
		}

		if(smls->animCP && dynamic_cast<AnimLever*>(g_visualClue)){
		//else if(smls->animCP && dynamic_cast<AnimClue*>(g_visualClue) && g_partCageClue->mode == PartCageClue::MORPH_CP){
			if(MultiSelect::withinSights){
				if(MultiSelect::pointList.length >= g_transformCP->leverSelection && g_transformCP->sameScreenPosition == false && MultiSelect::withinSights && MultiSelect::withinSightsNode == g_transformCP){
					if(g_transformCP->isOverPivotPoint){
						g_transformCP->dragPivotPointMode = true;
						g_transformCP->refreshRemPoints();
					}
					else{
						g_transformCP->leverIdx = MultiSelect::withinSightsIndex;
						g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];
						g_pointer.translation = g_transformCP->remRotCtrlPnt;
						g_transformCP->dragRotPointMode = true;
						g_transformCP->setMode(g_transformCP->getKeyMode());
						g_transformCP->remTranslation = g_transformCP->translation;
						g_transformCP->clickPoint = g_scene->getIViewMatrix() * g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];

					}

					SenseTugGlobals::node = g_visualClue;
					tugFlag = true;
					showCursor(false);

					return;
				}

				if(g_deleteControlLine.mousePhase == 2){
					changeModellingCursor(g_crossCursor);

					if(g_deleteControlLine.m_delCtrlLn.length == 0){
						g_deleteControlLine.mousePhase = DeleteControlLine::NEUTRAL;
						return;
					}
					if(g_deleteControlLine.selectRing){
						g_deleteControlLine.smls->doLastPhaseForSelectRing();
					}
					return;
				}
				//g_partCageClue->cpTug = false;
				NurbsPatch* selPatch = dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p);
				if(selPatch && smls->workAnimCP){
					clickNurbsCage(smls, perspective);

				
					//smls->clickPointWithinSights(selPatch);

					//if(NurbsLine* line = dynamic_cast<NurbsLine*>(selPatch)){
					//	line->index[0] = line->NurbsPatch::index[0];
					//	g_stw->cp->paint();
					//}
					//g_transformCP->calcPivotPoint();
					//g_transformCP->resetRotationLever();

					//Matrix matrix = translationMatrix(-g_transformCP->pivotPoint) * rotationMatrix(-g_transformCP->scaleOrientation) * scaleMatrix(g_transformCP->scale) * rotationMatrix(g_transformCP->scaleOrientation) * rotationMatrix(g_transformCP->rotation) * translationMatrix(g_transformCP->pivotPoint) * translationMatrix(g_transformCP->remTranslation);
					//g_transformCP->clickPoint = matrix * MultiSelect::beginDragPos[MultiSelect::clickedIndex];


					//part->houseKeep();
					//fastFullBuildForAutoIn(false);
					//g_vcArtAnimBase->resetRotationLever();
					return;
				}
				if(g_partCageClue->mode == PartCageClue::BONE_PIVOT_POINT){
					part = dynamic_cast<Art*>(MultiSelect::withinSightsNode.p);
				
					node = part;
					g_stw->setResultsForSelectingVertex(part);
					SenseTugGlobals::node = g_vcArtAnimBase;
					g_vcArtAnimBase->remTranValue = part->translation;
					smls->ActTri = 0;
					smls->ActTriCorner = 0;
					smls->actTriPart = 0;
					g_vcArtAnimBase->resetRotationLever();

					tugFlag = true;
					showCursor(false);
					return;
				}
				SenseTugGlobals::node = g_visualClue;  //dag rotation lever
				g_vcArtAnimBase->leverIdx = MultiSelect::withinSightsIndex;
				tugFlag = true;
				showCursor(false);
				return;
			}
			return;
		}


		if(g_radial_Tug->active && Window::animMode() == 0){
			if(ImmediatePart* part=dynamic_cast<ImmediatePart*>(g_selNode.getRaw())){
				if(checkRadialTugSize(perspective, part)){
					return;
				}
			}
		}
	}
	//if(((g_scene->mode & (HOME_MODE | ANIMATION_MODE | POLY_EDIT_MODE | XRAY_EDIT_MODE | OWNERSHIP_MODE | NURBS_OWNERSHIP_MODE | PAINT_MODE | SELECT_OBJECT_MODE)) || (g_scene->mode & MODELLING_MODE && MultiSelect::withinSightsNode == 0)) && g_insertControlPoint == false && g_deleteControlPoint == false && MultiSelect::selectRectMode == false){
	if(((g_scene->mode & HOME_MODE) || (g_scene->mode & MODELLING_MODE && MultiSelect::withinSightsNode == 0) || (g_scene->mode & PAINT_MODE && altKeyState())) && g_insertControlPoint == false && g_deleteControlPoint == false && MultiSelect::selectRectMode == false){
		SenseTugGlobals::initMidTug();
		return;
	}
	if(g_scene->mode & MODELLING_MODE){
		return;
	}
	if((g_scene->mode & PAINT_MODE) && (g_scene->mode & BRUSH_MODE)){
		initBrush();
		return;
	}
	if(Window::animMode()){
		MultiSelect::clickedIndex = 0;
	}
	g_scene->senseTug(Matrix(),Matrix(), 0);

	if(g_scene->mode & PAINT_MODE){
		if(g_scene->mode & FILL_MODE){
			if(Art* part = dynamic_cast<Art*>(node)){
				if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
					colourWithFill(part, part->smls);
				}
				else if(TextureEffect* textureEffect = dynamic_cast<TextureEffect*>(part->workEffect)){
					textureEffect->colourWithFill(g_palette->color.refValue().colorToHex());
				}
				return;
			}
		}
		else if(g_scene->mode & COLOR_PICKER_MODE && dynamic_cast<Art*>(node)){
			tugFlag = true;
			g_visualClue = 0;
			if(Art* part = dynamic_cast<Art*>(node)){
				SenseTugGlobals::actTriPart = part;
				Seamless* smls = part->smls;
				if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(part->workEffect)){
					if(dynamic_cast<Art*>(node)){
						if(ColorEffect* colorEffect = dynamic_cast<ColorEffect*>(actTriPart->workEffect)){
							g_palette->color = getColorFromPointInTriangle(part->triangle[SenseTugGlobals::ActTri], g_pointer.translation, colorEffect);
							//doColorPicker(part, SenseTugGlobals::ActTri, smls);
						}
					}
				}
				else if(TextureEffect* textureEffect = dynamic_cast<TextureEffect*>(part->workEffect)){
					unsigned int pixel = textureEffect->readPixelFromTexture();
					g_palette->color = hexToColor(pixel);
				}
			}
			return;
		}
	}
	if(Art* part = dynamic_cast<Art*>(node)){
		if((g_scene->mode & HOME_MODE) && g_browseMode == false){
			return;
		}

		Seamless* smls = part->smls;

		if(g_dragMorphFlag == false && (Window::animMode() || (g_scene->mode & XRAY_EDIT_MODE)) && smls->xRay){
			if(part != g_selNode.getRaw()){
				g_stw->setResultsForSelectingVertex(part);
				smls->ActTri = 0;
				smls->ActTriCorner = 0;
				smls->actTriPart = 0;
			}
			return;
		}

		if(g_radial_Tug->active){
			backUpSmlsCoord(smls->partList);
		}
		smls->ActCtrlPnt=ActCtrlPnt;
		tugFlag=true;
		if(false == controlKeyState()){
			showCursor(false);
			//coord[i] = getIViewMatrix() * triPart->imatrix * pt;
		}
		g_pointer.translation = pointer;
		smls->ActTri=ActTri;
		smls->ActTriCorner=corner;
		smls->actTriPart=actTriPart;
		part->index[0]=ActCtrlPnt;
		part->workEffect->setColorIndex(actTriPart->triangle.refTexIndex(ActTri*3+corner));
		if(part != g_selNode.getRaw()){
			g_stw->setResultsForSelectingVertex(part);
		}
		else{
			g_vcArtTriBase->smls = part->smls;
			g_visualClue = g_vcArtTriBase;
		}
		g_stw->cp->paint();
		g_tb->updateToolBarContext();
		fastFullBuildForAutoIn(true);


		if(MultiSelect::coordList.length != MultiSelect::beginDragPos.length){
			MultiSelect::checkToReset(part);
			MultiSelect::backUpCoords(part);
			MultiSelect::calcPivotPoint();
		}

	//	g_vcArtTriBase->circle=false;
	}
}

void ModelMorpher::senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor){
	M_Node& sequence = this->sequence.refValue();
	for(int k = sequence.length; --k>=0;){
		ModelSequence* modelSequence = dynamic_cast<ModelSequence*>(sequence[k].p);
		M_Node& model = modelSequence->model.refValue();
		for(int k = model.length; --k>=0;){
			Seamless* smls = dynamic_cast<Seamless*>(model[k].p);
			smls->senseTug(matrix, imatrix, sensor);
		}
	}
}

bool checkAllPivotPoints(bool perspective, StickMan* stickMan){
	if(stickMan == 0){
		return false;
	}
	//if(MultiSelect::pointList.length >= g_transformCP->selNumberToShow){
	//	return false;
	//}
	M_Art& partList = stickMan->partList;
	SenseTugGlobals::node = 0;

	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		if(part->ik){
			if(checkWithInSights(perspective, g_scene->getViewMatrix() * part->ikPos)){
				SenseTugGlobals::node = part;
				return true;
			}
		}
		else{
			if(checkWithInSights(perspective, part->pivotPointMatrix * Vec3f(0,0,0))){
				SenseTugGlobals::node = part;
				return true;
			}
		}
	}

	bool hit = false;
	for(int k = partList.length; --k >= 0;){
		Art* part = partList[k];
		if(part->ik){
			if(checkWithInSightsForBone(perspective, g_scene->getViewMatrix() * part->ikPos)){
				MultiSelect::withinSightsNode = part;
				SenseTugGlobals::node = part;
				hit = true;
			}
		}
		else{
			if(checkWithInSightsForBone(perspective, part->pivotPointMatrix * Vec3f(0,0,0))){
				MultiSelect::withinSightsNode = part;
				SenseTugGlobals::node = part;
				hit = true;
			}
		}
	}
	return hit;
}
bool AnimLever::initLeverTug(Art* selPart, const bool& perspective){
	if(MultiSelect::pointList.length < g_transformCP->leverSelection){
		if(false == (selPart->ik && selPart->pin == false)){
			for(int i = 0; i < rotCtrlPnt.length; i++){
				Vec3f sr = g_vcArtAnimBase->rotCtrlPnt[i];
				if(checkWithInSights(perspective, sr)){
					g_transformCP->mouseOffset = sr - g_pointer.translation;
					SenseTugGlobals::node = selPart;
					MultiSelect::withinSightsNode = selPart;
					MultiSelect::withinSights = true;
					MultiSelect::withinSightsIndex = i;
					g_partCageClue->mode = PartCageClue::BONE_ROTATION_LEVER;
					changeModellingCursor(g_lockedOnPointCursor);
					return true;
				}
			}
		}
	}
	return false;
}

bool Seamless::initSkeletonTug(const bool& perspective){
	g_partCageClue->mode = 0;

	if(xRay && checkAllPivotPoints(perspective, stickMan)){
		MultiSelect::withinSightsNode = SenseTugGlobals::node;
		MultiSelect::withinSights = true;
		g_partCageClue->mode = PartCageClue::BONE_PIVOT_POINT;
		//changeModellingCursor(g_crossCursor);
		changeModellingCursor(g_lockedOnPointCursor);
		//g_partCageClue->mode = PartCageClue::BONE_PIVOT_POINT;
		return true;
	}
	else if(g_addBone->mousePhase == AddBone::NEUTRAL){
		changeModellingCursor(g_crossCursor);
	}
	return false;
}

bool isChildOf(Art* partA, Art* partB, int& k){

	M_SArt& m_part = partA->part.refValue();

	for(k = m_part.length; --k >=0;){
		Art* part = m_part[k].p;
		if(part == partB){
			return true;
		}
	}
	return false;
}

Art* createPartOfType(Art* part){
	if(dynamic_cast<Part*>(part)){
		return new Part;
	}
	return new ImmediatePart;
}

void mirrorInsertPart(ArtPtr mirrorPartA, ArtPtr mirrorPartB, int mirrorK, Vec3f pivotPoint){

	M_SArt& m_part = mirrorPartA->part.refValue();

	Art* mirrorNewPart = createPartOfType(mirrorPartA);
	m_part[mirrorK] = mirrorNewPart;
	mirrorNewPart->part.refValue() += mirrorPartB;

	Vec3f mirrorPivotPoint = pivotPoint;
	mirrorPivotPoint.x = -mirrorPivotPoint.x;

	mirrorNewPart->pivotPoint.refValue() = mirrorPivotPoint;
	mirrorPartB->pivotPoint.refValue() -= mirrorNewPart->pivotPoint;
}

void addMirrorPart(Art* part, Vec3f newPivotPoint){
	Art* newMirrorPart = createPartOfType(part);
	Vec3f newMirrorPivotPoint = newPivotPoint;
	newMirrorPivotPoint.x = -newMirrorPivotPoint.x;
	newMirrorPart->pivotPoint.refValue() = newMirrorPivotPoint;
	part->part.refValue() += newMirrorPart;
}

void AddBone::insertBone(){
	Seamless* smls = partA->smls;
	M_SArt& m_part = partA->part.refValue();

	g_stw->textCell.cellActive = false;
	Art* newPart = createPartOfType(partA);

	m_part[k] = newPart;
	newPart->part.refValue() += partB;
					
					
	Vec3f pivotPoint = (partB->absolutePivotPoint - partA->absolutePivotPoint) * ratio;

	newPart->pivotPoint.refValue() = pivotPoint;
	partB->pivotPoint.refValue() -= newPart->pivotPoint;


	if(g_scene->mode & CREATE_MIRROR_OBJECT){
		int mirrorK = 0;
		if(partA->mirrorPart && partB->mirrorPart && isChildOf(partA->mirrorPart, partB->mirrorPart, mirrorK)){
			mirrorInsertPart(partA->mirrorPart, partB->mirrorPart, mirrorK, pivotPoint);
		}
		else if(partB->mirrorPart && isChildOf(partA, partB->mirrorPart, mirrorK)){
			mirrorInsertPart(partA, partB->mirrorPart, mirrorK, pivotPoint);
		}
	}
	smls->process();
	g_stw->genNodeList();
	g_stw->selLine = getNodeLine(newPart);
	g_stw->commonPostInsertNodeStuff();

	smls->genDynamicTransforms();
	Vec3f newAbsolutePivotPoint = g_scene->getIViewMatrix() * g_catchIMatrix * newPart->imatrix * g_pointer.translation;

	MultiSelect::withinSights = true;
	MultiSelect::withinSightsNode = newPart;
	g_partCageClue->mode = PartCageClue::BONE_PIVOT_POINT;
	tugFlag = true;
	Vec3f sr = g_viewMatrix * newAbsolutePivotPoint;
	g_cursorOverPointIlsBase->translation.refValue() = sr;
	g_pointer.translation = sr;

	return;
}

void AddBone::initMouseOverLine(Seamless* smls){
	if(StickMan* stickMan = smls->stickMan){
		M_Art& partList = stickMan->partList;
		for(int m = partList.length; --m >= 0;){
			partA = partList[m];
			M_SArt& m_part = partA->part.refValue();
			for(k = m_part.length; --k >= 0; ){
				partB = m_part[k];

				//Vec2f click3f = g_pointer.translation;
				Vec3f ptA3f = partA->pivotPointMatrix * Vec3f(0,0,0);
				Vec3f ptB3f = partB->pivotPointMatrix * Vec3f(0,0,0);

				Vec2f click = Vec2f(g_pointer.translation.x, g_pointer.translation.y);
				Vec2f ptA = Vec2f(ptA3f.x, ptA3f.y);
				Vec2f ptB = Vec2f(ptB3f.x, ptB3f.y);
				Vec2f crossPoint;
				if(clickLine(click, ptA, ptB, crossPoint)){
					ratio = getMidPointRatio(ptA, ptB, crossPoint);
					mousePhase = INSERT;
					changeModellingCursor(g_insertBoneCursor);
					return;
				}
			}
		}
	}
	changeModellingCursor(g_addBoneCursor);
	mousePhase = ACTIVE;
}

bool getCollapsedLineNoGlobalChange(int nonCollapsedLine, int& collapsedLine);

void AddBone::addBone(){

	if(mousePhase == INSERT){
		insertBone();
		return;
	}
	if(Part* part = dynamic_cast<Part*>(g_selNode.getRaw())){
		if(StickMan* stickMan = part->smls->stickMan){
			g_stw->textCell.cellActive = false;
			Art* newPart = createPartOfType(part);
			MultiSelect::withinSights = true;
			MultiSelect::withinSightsNode = newPart;
			//newPart->pivotPoint.refValue() = g_pointer.translation - part->absolutePivotPoint;;
			//Vec3f vz = g_scene->getViewMatrix() * part->pivotPointMatrix * Vec3f(0,0,0); //this does not work if view position is not centre
			Vec3f vz = part->pivotPointMatrix * Vec3f(0,0,0);

			g_pointer.translation.z = vz.z;
			Vec3f newAbsolutePivotPoint = g_scene->getIViewMatrix() * g_catchIMatrix * part->imatrix * g_pointer.translation;

			Vec3f newPivotPoint = newAbsolutePivotPoint - part->absolutePivotPoint;
			part->part.refValue() += newPart;
			
			if(g_scene->mode & CREATE_MIRROR_OBJECT && (part->mirrorPart || part->absolutePivotPoint.x == 0)){
				if(part->part.length == 1 && fabs(part->absolutePivotPoint.x) < .000001){
					if(fabs(newAbsolutePivotPoint.x) < 1 / g_scene->getZoom() * .05){
						newPivotPoint.x = 0;
					}
					else{
						addMirrorPart(part, newPivotPoint);
					}
				}
				else if(Art* mirrorPart = part->mirrorPart){
					addMirrorPart(mirrorPart, newPivotPoint);
				}
				else{
					addMirrorPart(part, newPivotPoint);
				}
			}
			newPart->pivotPoint.refValue() = newPivotPoint;

			Vec3f sr = g_viewMatrix * newAbsolutePivotPoint;
			g_cursorOverPointIlsBase->translation.refValue() = sr;
			g_pointer.translation = sr;

		
			part->smls->process();
			g_stw->genNodeList();
			g_stw->selLine = getNodeLine(newPart);
			g_stw->commonPostInsertNodeStuff();


			MultiSelect::withinSightsNode = newPart;
			MultiSelect::withinSights = true;
			g_partCageClue->mode = PartCageClue::BONE_PIVOT_POINT;

			part->smls->genDynamicTransforms();


			if(g_stw->isVisible()){
				int collapsedNodeIdx = 0;
				int nodeIdx = getNodeLine(newPart);
				getCollapsedLineNoGlobalChange(nodeIdx, collapsedNodeIdx);
				//tLnOffst + g_sceneTreeList[nodeIdx].indent*pixSymbolCX + 10,	
				int tmpInt = SceneTree::beginYOffset + (collapsedNodeIdx - g_stw->getScrollPosCollapsed()) * pixSymbolCY + 8;
				//if(tmpInt < g_stw->size.y - 8){
				g_stw->textCell.cellPos.y = tmpInt;
				g_stw->selectNode(g_stw->selLine, newPart, 0, tmpInt);
				g_drag = 0; //as we did not click in the scene tree window
				Window::activeWindow = g_stw.p;
				//}
			}
			return;
		}
	}
}

Viewpoint* Viewpoint::withinSitesViewpoint = 0;

void Viewpoint::senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor){
	if(SenseTugGlobals::node || Viewpoint::withinSitesViewpoint) return;
	if(g_cameraObject == 0){
		createCameraObject();
	}

	const bool perspective=g_perspective;

	g_catchMatrix = matrix;
	g_catchIMatrix = imatrix;

	if(g_cameraObject){
		g_cameraObject->rotation.refValue() = orientation;
		g_cameraObject->translation.refValue() = position;
		g_cameraObject->catMatrix(matrix);
		g_cameraObject->catIMatrix(imatrix);

//	if(g_scene->mode & SELECT_OBJECT_MODE){
		g_stw->senseAllPartsTriangle(g_cameraObject);
		if(SenseTugGlobals::node){
			Viewpoint::withinSitesViewpoint = this;
			SenseTugGlobals::node = this;
		}
//		return;
//	}
	}
}

void Seamless::senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor){
	if(SenseTugGlobals::node) return;;

	const bool perspective=g_perspective;

	g_catchMatrix = matrix;
	g_catchIMatrix = imatrix;
	catMatrix(matrix);
	catIMatrix(imatrix);

//	if(dynamic_cast<Build*>(g_selNode.getRaw()) || dynamic_cast<Generator*>(g_selNode.getRaw())){
//		if(g_stw->selLine > 0 && this==g_sceneTreeList[getSeamlessParent(g_stw->selLine)].node){
//		}
//	}
	if(g_scene->mode & SELECT_OBJECT_MODE){
		g_stw->senseAllPartsTriangle(this);
		return;
	}

	if(sensor && sensor->enabled && (g_scene->mode & XRAY_EDIT_MODE) == false){
		g_stw->senseAllPartsTriangle(this, sensor);
	}
	if(SenseTugGlobals::node == 0){
		if(xRay){
			if(checkAllPivotPoints(perspective, stickMan)){
				Art* part = dynamic_cast<Art*>(SenseTugGlobals::node);
				g_stw->setResultsForSelectingVertex(part);
				SenseTugGlobals::node = g_vcArtAnimBase;
				g_partCageClue->mode = PartCageClue::BONE_PIVOT_POINT;
				g_vcArtAnimBase->remTranValue = part->translation;
				return;
			}
		}
		if((g_scene->mode & XRAY_EDIT_MODE) == false){
			if(altKeyState() || InputEvent::l_altDown || g_scene->mode & PAINT_MODE){
				g_stw->senseAllPartsTriangle(this);
			}
			else if(Window::animMode() == 0 || g_scene->mode & ENABLE_MORPH_CAPTURE){
				senseAllPartsVertex();
			}
		}
	}
}

void Transform::senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor){
	//if(SenseTugGlobals::sameVertexHit)return;
	if(SenseTugGlobals::node)return;
	Vec3f scale=this->scale;
	Rotation rotation=this->rotation;
	Vec3f translation=this->translation;

	matrix= scaleMatrix(scale)*rotationMatrix(rotation)*translationMatrix(translation)*matrix;
	imatrix=imatrix*translationMatrix(Vec3f(-translation.x, -translation.y, -translation.z)) * rotationMatrix( -rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z));

	M_Node& children=this->children.refValue();

	for(int k=0; k<children.length;k++){
		children[k]->senseTug(matrix, imatrix, this->sensor);
	}
}
void AGroup::senseTug(Matrix matrix, Matrix imatrix, TouchSensor* sensor){
	//if(SenseTugGlobals::sameVertexHit)return;
	if(SenseTugGlobals::node)return;
	M_Node& children = this->children.refValue();


	//if(0 == dynamic_cast<Viewpoint*>(g_scene->children[0].p)){
	//	int a = 0;
	//}

	for(int k=0; k<children.length;k++){
		children[k]->senseTug(matrix, imatrix, this->sensor);
	}
}
//void NoOutputGroup::senseTug(Matrix matrix, Matrix imatrix){
//}
void Node::resetRotations(){
}
void ProtoDeclare::resetRotations(){
	M_Node& children=protoBody->children.refValue();
	for(int k=0; k<children.length;k++){
		children[k]->resetRotations();
	}
}


void AGroup::resetRotations(){

	M_Node& children=this->children.refValue();

	for(int k=0; k<children.length;k++){
		children[k]->resetRotations();
	}
}


/*
Vec3f S_Triangle::calculateNormal(){

	Vec3f cornerA=pa->matrix * pa->coord[ia];
					
	Vec3f cornerB=pb->matrix * pb->coord[ib];

	Vec3f cornerC=pc->matrix * pc->coord[ic];

	float Qx, Qy, Qz, Px, Py, Pz;

	Qx = cornerC.x-cornerA.x;
	Qy = cornerC.y-cornerA.y;
	Qz = cornerC.z-cornerA.z;
	Px = cornerB.x-cornerA.x;
	Py = cornerB.y-cornerA.y;
	Pz = cornerB.z-cornerA.z;

	return Vec3f(
		Py*Qz - Pz*Qy,
		Pz*Qx - Px*Qz,
		Px*Qy - Py*Qx
		).normalize();

} 

inline Vec3f genSingleVertexNormal(Art* part, int index){
	M_Art& relevantPart=part->relevantPart;
	Vec3f norm;
	int hits=0;
	for(int k=relevantPart.length;  --k>-0; ){
		M_Triangle triangle=relevantPart[k]->triangle;
		for(int i=triangle.length*3; --i>=0; ){
			if(triangle.refIndex(i)==index && triangle.refPart(i)==part){
				S_Triangle tri=triangle[i/3];
				norm+=tri.calculateNormal();
				hits++;
			}
		}
	}
	return (norm/(float)hits).normalize();
}
*/
inline int convertToLinearVertexIndex(Art* target, M_Art& partList){
	int accumIndex=0;
	for(int k=0; k<partList.length; k++){
		Art* part=partList[k];
		if(part==target){
			return accumIndex;
		}
		accumIndex+=part->coord.refValue().length;
	}
	throw target;
}
int sensePartForRadialTug(M_Vec3f& coord, const Vec3f location, const Vec3f displacement, const float radius, const float slope){
	float pp=slope*PI;
	float ipp=PI-pp;
	float h=1/(cos(ipp)+1);
	for (int i=0;i<coord.length;i++){
		Vec3f vec=coord[i];
		float vecX=vec.x-location.x;
		float vecY=vec.y-location.y;
		float vecZ=vec.z-location.z;
		float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

		if(radius>HYP){
			return i;
		}
	}
	return -1;
}
void singleRadialTug(M_Vec3f& coord, const Vec3f location, const Vec3f displacement, const float radius, const float slope){
	float pp=slope*PI;
	float ipp=PI-pp;
	float h=1/(cos(ipp)+1);
	for (int i=0;i<coord.length;i++){
		Vec3f vec=coord[i];
		float vecX=vec.x-location.x;
		float vecY=vec.y-location.y;
		float vecZ=vec.z-location.z;
		float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

		if(radius>HYP){
			float scaleDecay=(cos((fabs(HYP)/radius)*pp+ipp)+1)*h;
			vec.x+=displacement.x*scaleDecay;
			vec.y+=displacement.y*scaleDecay;
			vec.z+=displacement.z*scaleDecay;
			coord[i]=vec;
		}
	}
}
void singleMirrorRadialTug(M_Vec3f& coord, const Vec3f location, const Vec3f mirrorLocation, const Vec3f displacement, const Vec3f mirrorDisplacement, const float radius, const float slope){
	float pp=slope*PI;
	float ipp=PI-pp;
	float h=1/(cos(ipp)+1);
	for(int i=0;i<coord.length;i++){
		Vec3f vec=coord[i];
		if(vec.x==0){
			float vecX=vec.x-location.x;
			float vecY=vec.y-location.y;
			float vecZ=vec.z-location.z;
			float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

			if(radius>HYP){
				float scaleDecay=(cos((fabs(HYP)/radius)*pp+ipp)+1)*h;
				vec.x=0;
				vec.y+=displacement.y*scaleDecay;
				vec.z+=displacement.z*scaleDecay;
				coord[i]=vec;
			}
		}
		else if(vec.x>0){
			float vecX=vec.x-location.x;
			float vecY=vec.y-location.y;
			float vecZ=vec.z-location.z;
			float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

			if(radius>HYP){
				float scaleDecay=(cos((fabs(HYP)/radius)*pp+ipp)+1)*h;
				vec.x+=displacement.x*scaleDecay;
				vec.y+=displacement.y*scaleDecay;
				vec.z+=displacement.z*scaleDecay;
				coord[i]=vec;
			}
		}
		else if(vec.x<0){
			float vecX=vec.x-mirrorLocation.x;
			float vecY=vec.y-mirrorLocation.y;
			float vecZ=vec.z-mirrorLocation.z;
			float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

			if(radius>HYP){
				float scaleDecay=(cos((fabs(HYP)/radius)*pp+ipp)+1)*h;
				vec.x+=mirrorDisplacement.x*scaleDecay;
				vec.y+=mirrorDisplacement.y*scaleDecay;
				vec.z+=mirrorDisplacement.z*scaleDecay;
				coord[i]=vec;
			}
		}
	}
}
/*
void singleSineTugNurbs(const M_Vec3f& dpoint, M_Vec3f& point, const M_Float& weight, const Vec3f location, const Rotation& orientation, const float height, const float rad){
	
	Matrix matrix=rotationMatrix(orientation)*translationMatrix(location);
	Matrix imatrix=translationMatrix(-location)*rotationMatrix(-orientation);
	const float sdecay=rad;

	float xsc=1;
	
	for(int i=point.length; --i>=0;){
		Vec3f ivec=imatrix*(point[i] / weight[i]);
		if(ivec.y<-sdecay)continue;
		float ADJ=(float)(fabs(ivec.x)*xsc);
		float OPP=(float)(fabs(ivec.z));
		float HYP=(float)(sqrt(ADJ*ADJ + OPP*OPP));

		if(rad>HYP && abs(ivec.y)<sdecay){
			float sineCurve=cos((fabs(ivec.y)/sdecay)*PI)*.5+.5;
			ivec.y+=((1+cos(HYP/rad*PI))/2.*height)*sineCurve;
			point[i]=matrix*(ivec * weight[i]);
		}
	}
}
*/


void NurbsPatch::mirrorSingleCP(int idx){
#ifdef CPINDEX
	M_Vec3f& cp = smls->cp.refValue();
#endif
	if(mirror){
		if(mirrorPatch){
			Vec3f vec = point[idx];
			vec.x = -vec.x;
			mirrorPatch->point.refValue()[idx] = vec;
#ifdef CPINDEX
			cp[mirrorPatch->cpIndex[idx]] = vec;
#endif
			vec = dpoint[idx];
			vec.x = -vec.x;
			mirrorPatch->dpoint.refValue()[idx] = vec;
		}
		else{
			int uDim = dim.x;
			int vDim = dim.y;
			int u = idx % uDim;
			int v = idx / uDim;
			int middleIdx = (uDim >> 1);

			if(u == middleIdx && (uDim & 1)){
				point.refValue()[idx].x = 0;
#ifdef CPINDEX
				cp[cpIndex[idx]].x = 0;
#endif
				dpoint.refValue()[idx].x = 0;
			}
			else{
				int midx = v * uDim + uDim - 1 - u;
				Vec3f vec = point[idx];
				vec.x = -vec.x;
				point.refValue()[midx] = vec;
#ifdef CPINDEX
				cp[cpIndex[midx]] = vec;
#endif
				vec = dpoint[idx];
				vec.x = -vec.x;
				dpoint.refValue()[midx] = vec;
			}
		}
	}
}

void NurbsPatch::singleRadialTugMirrorRight(M_Vec3f& point, const M_Float& weight, const Vec3f location, const Vec3f displacement, const float radius, const float slope){
	float pp=slope*PI;
	float ipp=PI-pp;
	float h=1/(cos(ipp)+1);
	for (int i = 0; i < point.length; i++){
		int idx = fromPoint + i;
		Vec3f vec = g_backUpCoord[idx];
		if(vec.x <= 0){
			float vecX=vec.x-location.x;
			float vecY=vec.y-location.y;
			float vecZ=vec.z-location.z;
			float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

			if(radius>HYP){
				float scaleDecay=(cos((fabs(HYP)/radius)*pp+ipp)+1)*h;
				vec+=displacement*scaleDecay;
				//point[i] = vec * weight[i];
				g_transformCP->setFilteredDimension(point, smls->cp.refValue(), cpIndex.refValue(), i, vec * weight[i]);
				mirrorSingleCP(i);
			}
		}
	}
}

void NurbsPatch::singleRadialTugMirrorLeft(M_Vec3f& point, const M_Float& weight, const Vec3f location, const Vec3f displacement, const float radius, const float slope){
	float pp=slope*PI;
	float ipp=PI-pp;
	float h=1/(cos(ipp)+1);
	for (int i = 0; i < point.length; i++){
		int idx = fromPoint + i;
		Vec3f vec = g_backUpCoord[idx];
		if(vec.x >= 0){
			float vecX=vec.x-location.x;
			float vecY=vec.y-location.y;
			float vecZ=vec.z-location.z;
			float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

			if(radius>HYP){
				float scaleDecay=(cos((fabs(HYP)/radius)*pp+ipp)+1)*h;
				vec+=displacement*scaleDecay;
				//point[i] = vec * weight[i];
				g_transformCP->setFilteredDimension(point, smls->cp.refValue(), cpIndex.refValue(), i, vec * weight[i]);
				mirrorSingleCP(i);
			}
		}
	}
}

void NurbsPatch::singleRadialTug(M_Vec3f& point, const M_Float& weight, const Vec3f location, const Vec3f displacement, const float radius, const float slope){
	float pp=slope*PI;
	float ipp=PI-pp;
	float h=1/(cos(ipp)+1);
	for (int i = 0; i < point.length; i++){
		int idx = fromPoint + i;
		Vec3f vec = g_backUpCoord[idx];
		float vecX=vec.x-location.x;
		float vecY=vec.y-location.y;
		float vecZ=vec.z-location.z;
		float HYP=(float)(sqrt(vecX*vecX + vecY*vecY + vecZ*vecZ));

		if(radius>HYP){
			float scaleDecay=(cos((fabs(HYP)/radius)*pp+ipp)+1)*h;
			vec+=displacement*scaleDecay;
			//point[i] = vec * weight[i];
			g_transformCP->setFilteredDimension(point, smls->cp.refValue(), cpIndex.refValue(), i, vec * weight[i]);
			mirrorSingleCP(i);
		}
	}
}

void NurbsPatch::radialTug(Vec3f location, Vec3f pt){

	float radius = g_radial_Tug->radius.refValue();


	const Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * pt;
	const Vec3f displacement = Vec3f(newPt.x - location.x, newPt.y - location.y, newPt.z - location.z);

	const float slope = 1;// do we need this any more?

	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;

	for(int k = 0; k < cpNodeList.length; k++){
		NurbsPatch* patch = cpNodeList[k];
		if(patch->cage){
			if(patch->mirror){
				if(location.x < 0){
					patch->singleRadialTugMirrorRight(patch->point.refValue(), patch->weight.refValue(), location, displacement, radius, slope);
				}
				else{
					patch->singleRadialTugMirrorLeft(patch->point.refValue(), patch->weight.refValue(), location, displacement, radius, slope);
				}
			}
			else{
				patch->singleRadialTug(patch->point.refValue(), patch->weight.refValue(), location, displacement, radius, slope);
			}
		}
	}
	g_transformCP->calcPivotPoint();
	g_transformCP->houseKeep();
}

int findMirrorVertex(const M_Vec3f& cPtr, Vec3f svec);

void radialTug3(const Vec3f coord, const Vec3f displacement, Vec3f disp, const Vec3f selCoord, const float radius, const float slope, Art* part, Art* prt){
	if(prt->show==false){
		return;
	}
	Vec3f location=part->matrix * (prt->imatrix * coord);

	int idx=sensePartForRadialTug(prt->coord, location, displacement, radius, slope);
	if(idx==-1){
		return;
	}
	Vec3f v = prt->matrix*prt->coord[idx];

	Art* mirrorPart = prt->mirrorPart;

	Vec3f mirrorDisp=displacement;

	Vec3f mirrorLocation=location;
	if(prt->mirror && mirrorPart == 0){
		if(location.x>0){
			mirrorLocation.x=-mirrorLocation.x;
			mirrorDisp.x=-mirrorDisp.x;
		}
		else{
			location.x=-location.x;
			disp.x=-displacement.x;
		}
		singleMirrorRadialTug(prt->coord, location, mirrorLocation, disp, mirrorDisp, radius, slope);
	}
	else if(mirrorPart){
		mirrorLocation.x=-mirrorLocation.x;
		mirrorDisp.x=-mirrorDisp.x;

		if(	(prt->plidx > mirrorPart->plidx && v.x<0 && selCoord.x<0)||
			(prt->plidx < mirrorPart->plidx && v.x>0 && selCoord.x>0)||
			(prt->plidx < mirrorPart->plidx && v.x<0 && selCoord.x<0)||
			(prt->plidx > mirrorPart->plidx && v.x>0 && selCoord.x>0)
		){

			singleRadialTug(prt->coord, location, disp, radius, slope);
			singleRadialTug(mirrorPart->coord, mirrorLocation, mirrorDisp, radius, slope);
		}
	}
	else{
		singleRadialTug(prt->coord, location, disp, radius, slope);
	}
}
void radialTug4(const Vec3f coord, const Vec3f displacement, Vec3f disp, const Vec3f selCoord, const float radius, const float slope, Art* part, Art* prt){
	if(prt->show==false){
		return;
	}
	Vec3f location=part->matrix * (prt->imatrix * coord);

	int idx=sensePartForRadialTug(prt->coord, location, displacement, radius, slope);
	if(idx==-1){
		return;
	}
	Vec3f v = prt->matrix*prt->coord[idx];

	Art* mirrorPart = prt->mirrorPart;


	Vec3f mirrorDisp=displacement;

	Vec3f mirrorLocation=location;
	if(prt->mirror && mirrorPart){
		if(location.x>0){
			mirrorLocation.x=-mirrorLocation.x;
			mirrorDisp.x=-mirrorDisp.x;
		}
		else{
			location.x=-location.x;
			disp.x=-displacement.x;
		}
		singleMirrorRadialTug(prt->coord, location, mirrorLocation, disp, mirrorDisp, radius, slope);
	}
	else if(mirrorPart){
		mirrorLocation.x=-mirrorLocation.x;
		mirrorDisp.x=-mirrorDisp.x;

		singleRadialTug(prt->coord, location, disp, radius, slope);
		singleRadialTug(mirrorPart->coord, mirrorLocation, mirrorDisp, radius, slope);
	}
	else{
		singleRadialTug(prt->coord, location, disp, radius, slope);
	}
}

void radialTug(Vec3f coord, Vec3f pt){

	float radius = g_radial_Tug->radius;
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	Seamless* smls=part->smls;
	M_Art& partList=smls->partList;

	const Vec3f newPt=g_scene->getIViewMatrix() * g_catchIMatrix * part->imatrix * pt;
	const Vec3f displacement=Vec3f(newPt.x-coord.x,newPt.y-coord.y,newPt.z-coord.z);
	Vec3f disp=displacement;

	const Vec3f selCoord = part->matrix*coord;

	const float slope = 1;//do we need this any more?
	if(g_settings->multiPartTug.refValue()){
		for(int k=partList.length; --k>=0;){
			radialTug3(coord, displacement, disp, selCoord, radius, slope, part, partList[k]);
		}
	}
	else{
		radialTug4(coord, displacement, disp, selCoord, radius, slope, part, part);
	}
}
void radialTug2(Vec3f coord, Vec3f pt){
	const float radius = g_radial_Tug->radius;
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	Seamless* smls=part->smls;
	M_Art& partList=smls->partList;

	Vec3f mtxPt=g_scene->getIViewMatrix() * part->imatrix * pt;
	Vec3f displacement=Vec3f(mtxPt.x-coord.x,mtxPt.y-coord.y,mtxPt.z-coord.z);

	const float slope = 1;//do we need this anymore?
	if(g_settings->multiPartTug.refValue()){
		for(int k=partList.length; --k>=0;){
			Art* prt=partList[k];
			g_radialTugStem->part.refValue()[0]=prt;
			Vec3f location=part->matrix * (prt->imatrix * coord);
			//g_radialTugStem->doBuild();
			singleRadialTug(prt->coord, location, displacement, radius, slope);
		}
	}
	else{
		g_radialTugStem->part.refValue()[0]=part;
		g_radialTug->location.refValue()[0]=coord;
		g_radialTugStem->doBuild();
	}
}
bool findAndSetMirrorVertex(M_Vec3f& cPtr, Vec3f svec, Vec3f newPt){
	for(int i = cPtr.length; --i >= 0;){
		if(svec == cPtr[i]){
			cPtr[i] = Vec3f(-newPt.x, newPt.y, newPt.z);
			return true;
		}
	}
	return false;
}
int findMirrorVertex(const M_Vec3f& cPtr, Vec3f svec){
	for(int i = cPtr.length; --i >= 0;){
		if(svec == cPtr[i]){
			return i;
		}
	}
	return -1;
}

Art* Seamless::getMirrorVertex(Art* part, const int vertexIndex, int& returnVertex){
	if(part->mirror == false){
		return 0;
	}
	M_Vec3f& coord=part->coord.refValue();
	Vec3f svec = coord[vertexIndex];
	svec.x=-svec.x;

	if(Art* mirrorPart = part->mirrorPart){
		if((returnVertex = findMirrorVertex(mirrorPart->coord.refValue(), svec))!=-1){
			return mirrorPart;
		}
	}
	else{
		if((returnVertex = findMirrorVertex(coord, svec))!=-1){
			return part;
		}
	}
	return 0;
}
Art* Seamless::getMirrorTriangle(Art* part, int& returnCoordIdxIdx){
	M_Triangle triangle=actTriPart->triangle;

	const int ia=triangle.refIndex(ActTri*3+0);
	const int ib=triangle.refIndex(ActTri*3+1);
	const int ic=triangle.refIndex(ActTri*3+2);

	Art* pa=triangle.refPart(ActTri*3+0);
	Art* pb=triangle.refPart(ActTri*3+1);
	Art* pc=triangle.refPart(ActTri*3+2);

	int mia=0;
	int mib=0;
	int mic=0;

	Art* mpa=getMirrorVertex(pa, ia, mia);
	Art* mpb=getMirrorVertex(pb, ib, mib);
	Art* mpc=getMirrorVertex(pc, ic, mic);

	if(mpa && mpb && mpc){
		for(int k=partList.length; --k>=0;){
			Art* part=partList[k];
			M_Triangle& triangle=part->triangle;
			for(int i=triangle.getTriangleLength(); (i-=3)>=0;){
				if(
					triangle.refPart(i+0)==mpa && triangle.refIndex(i+0)==mia&&
					triangle.refPart(i+2)==mpb && triangle.refIndex(i+2)==mib&&
					triangle.refPart(i+1)==mpc && triangle.refIndex(i+1)==mic
					){
					int mirrorCorner=0;
					if(ActTriCorner==1){
						mirrorCorner=2;
					}
					else if(ActTriCorner==2){
						mirrorCorner=1;
					}
					returnCoordIdxIdx=i+mirrorCorner;
					return part;
				}
			}
		}
	}
	return 0;
}

void Seamless::senseMirrorVertex(Art* part, int idx, Vec3f& newPt){
	if(part->mirror == false){
		return;
	}

	Seamless* smls = part->smls;
	M_Vec3f& coord = part->coord.refValue();

	Vec3f svec = coord[idx];
	svec.x=-svec.x;

	if(part->mirrorPart == 0){
		if(part->isLeftRightName() == false){
			if(fabs(svec.x) < .000001){
				newPt.x=0;
				return;
			}
			int i=0;
			if(-1 != (i = findMirrorVertex(coord, svec))){
				coord[i] = Vec3f(-newPt.x, newPt.y, newPt.z);
				return;
			}
		}
	}
	else{
		if(findAndSetMirrorVertex(part->mirrorPart->coord.refValue(), svec, newPt)){
			return;
		}
	}
}

bool getPolyMap(NurbsPatch*& patch, Stem*& stem, Polymap*& polymap, Art* part, int selTriIndex, ImmediatePart*& sourcePart){
	for(int k = g_sceneTreeList.length; --k>=0;){
		if(Build* bld = dynamic_cast<Build*>(g_sceneTreeList[k].node.p)){
			if(patch = bld->getPatch()){
				if(patch->stem.length){
					M_Node& m_stem = patch->stem.refValue();
					for(int j = m_stem.length; --j>=0;){
						if(stem = dynamic_cast<Stem*>(m_stem[j].p)){
							if(stem->store && selTriIndex >= stem->beginTriOffset){
								if(stem->part.length == 1 && stem->part.refValue()[0] == part){
									if(stem->source.length == 1){
										if(ImmediatePart* bluePart = dynamic_cast<ImmediatePart*>(stem->source.refValue()[0].p)){
											//if(polyMap->coord.length == part->coord.length && polyMap->triangle.length == part->triangle.length){
											sourcePart = bluePart;
											return true;
										}
										return false;
									}
									else{
										return true;
									}
								}
							}
						}
					}
				}
			}
			else if(polymap = dynamic_cast<Polymap*>(bld)){
				if(selTriIndex >= polymap->beginTriOffset){
					if(polymap->part.refValue() == part){
						return true;
					}
				}
			}
		}
	}
	return false;
}



Vec2f curvePoint2d(int dim, int degree, const float *knot, const Vec2f *pnt, const float *w, const float t);
void CurvePoint (int dim, int degree, const float *knot, const Vec3f *pnt, const float *w, const float t, Vec3f& outPoint);
inline float getDiff(Vec2f t, Vec3f& curvedEdgePoint, float maxU, float maxV, float minU, float minV, int uOrder, int vOrder, int uDegree, int vDegree, int uDim, int vDim, float* uKnotPtr, float* vKnotPtr, Vec3f* pointPtr, float* weightPtr, Vec3f* topPointPtr, float* topWeightPtr, Vec3f* r_pointPtr, float* r_weightPtr, Vec3f coord){
	//if(t.x < 0 || t.x > 1 || t.y < 0 || t.y > 1){
	//	return 0;
	//}

	float u = (maxU-minU)*t.x + minU;
	float v = (maxV-minV)*t.y + minV;

	if(u <= 0){
		u = 0;
	}
	if(v <= 0){
		v = 0;
	}

	if(t.x==1){
		CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, curvedEdgePoint);
	}
	else if(t.y==1){
		CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, curvedEdgePoint);
	}
	else{
		SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &curvedEdgePoint);
	}

	Vec3f crd = coord;
	Vec3f crd2 = curvedEdgePoint;

	Vec3f diff = crd - crd2;
	return fabs(sqrt(diff.x*diff.x + diff.y*diff.y + diff.z*diff.z));
}
inline Vec3f getSurfaceVertex(const Vec2f& t, float maxU, float maxV, float minU, float minV, int uOrder, int vOrder, int uDegree, int vDegree, int uDim, int vDim, float* uKnotPtr, float* vKnotPtr, Vec3f* pointPtr, float* weightPtr, Vec3f* topPointPtr, float* topWeightPtr, Vec3f* r_pointPtr, float* r_weightPtr){
	Vec3f curvedEdgePoint;
	float u = (maxU-minU)*t.x + minU;
	float v = (maxV-minV)*t.y + minV;

	if(t.x==1){
		CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, curvedEdgePoint);
	}
	else if(t.y==1){
		CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, curvedEdgePoint);
	}
	else{
		SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &curvedEdgePoint);
	}

	return curvedEdgePoint;
}
inline void guessPolyMapTug(NurbsPatch& patch, float& xCrd, float& yCrd, const Vec3f& pt, const bool& northPole, const bool& southPole){
	int uOrder = patch.order.x;
	int vOrder = patch.order.y;
	int uDegree = patch.uDegree;
	int vDegree = patch.vDegree;
	int uDim = patch.dim.x;
	int vDim = patch.dim.y;

	float* uKnotPtr = patch.uKnot.refValue().getArray();
	float* vKnotPtr = patch.vKnot.refValue().getArray();
	Vec3f* pointPtr = patch.point.refValue().getArray();
	float* weightPtr = patch.weight.refValue().getArray();

	Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
	float* topWeightPtr = weightPtr+uDim*vDim-uDim;

	Vec3f* r_pointPtr = patch.r_point.getArray();
	float* r_weightPtr = patch.r_weight.getArray();


	float coordChange = .001;

	Vec2f diff;

	Vec3f curvedEdgePoint0;

	float maxU = patch.maxU();
	float maxV = patch.maxV();
	float minU = patch.minU();
	float minV = patch.minV();

	float diffArray[4];
	Vec3f destPoint[4];
	Vec2f base0 = Vec2f(xCrd, yCrd);

	if(northPole){
		for(int i = 200; --i>=0;){
			Vec2f basePointOffset[4] = {
				Vec2f(-coordChange, 0),
				Vec2f( coordChange, 0),

				Vec2f(0,-coordChange),
				Vec2f(0, coordChange),
			};
			float error = getDiff(convertNorthPoleToMercator(base0), destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

			diffArray[0] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[0]), destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[1] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[1]), destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[2] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[2]), destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[3] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[3]), destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

			float leastDiff = diffArray[0];
			int k = 0;

			for(int j = 1; j < 4; j++){
				float curDiff = diffArray[j];
				if(curDiff < leastDiff){
					leastDiff = curDiff;
					k = j;
				}
			}
			if(diffArray[k] < error){
				base0+=basePointOffset[k];
				coordChange*=2;
			}
			else{
				coordChange*=.5;
				if(coordChange==0){
					break;
				}
			}
		}
	}
	else if(southPole){
		for(int i = 200; --i>=0;){
			Vec2f basePointOffset[4] = {
				Vec2f(-coordChange, 0),
				Vec2f( coordChange, 0),

				Vec2f(0,-coordChange),
				Vec2f(0, coordChange),
			};
			float error = getDiff(convertSouthPoleToMercator(base0), destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

			diffArray[0] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[0]), destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[1] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[1]), destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[2] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[2]), destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[3] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[3]), destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

			float leastDiff = diffArray[0];
			int k = 0;

			for(int j = 1; j < 4; j++){
				float curDiff = diffArray[j];
				if(curDiff < leastDiff){
					leastDiff = curDiff;
					k = j;
				}
			}
			if(diffArray[k] < error){
				base0+=basePointOffset[k];
				coordChange*=2;
			}
			else{
				coordChange*=.5;
				if(coordChange==0){
					break;
				}
			}
		}
	}
	else{
		for(int i = 200; --i>=0;){
			Vec2f basePointOffset[4] = {
				Vec2f(-coordChange, 0),
				Vec2f( coordChange, 0),

				Vec2f(0,-coordChange),
				Vec2f(0, coordChange),
			};
			float error = getDiff(base0, destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

			diffArray[0] = getDiff(base0 + basePointOffset[0], destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[1] = getDiff(base0 + basePointOffset[1], destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[2] = getDiff(base0 + basePointOffset[2], destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
			diffArray[3] = getDiff(base0 + basePointOffset[3], destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

			float leastDiff = diffArray[0];
			int k = 0;

			for(int j = 1; j < 4; j++){
				float curDiff = diffArray[j];
				if(curDiff < leastDiff){
					leastDiff = curDiff;
					k = j;
				}
			}
			if(diffArray[k] < error){
				base0+=basePointOffset[k];
				coordChange*=2;
			}
			else{
				coordChange*=.5;
				if(coordChange==0){
					break;
				}
			}
		}
		if(base0.x <= 0){
			base0.x = 0;
		}
		else if(base0.x >= 1){
			base0.x = 1;
		}
		if(base0.y <= 0){
			base0.y = 0;
		}
		else if(base0.y >= 1){
			base0.y = 1;
		}
	}
	xCrd = base0.x;
	yCrd = base0.y;
}

void addEdge0(NurbsPatch* patch, M_Vec3f& edgePoint, M_Float& edgeWeight){
	int uDim = patch->dim.x;
	edgePoint.length = uDim;
	edgeWeight.length = uDim;
	M_Vec3f& point = patch->point.refValue();
	M_Float& weight = patch->weight.refValue();

	for(int i = uDim; --i>=0;){
		edgePoint[i] = point[i];
		edgeWeight[i] = weight[i];
	}
}

void addEdge1(NurbsPatch* patch, M_Vec3f& edgePoint, M_Float& edgeWeight){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	edgePoint.length = vDim;
	edgeWeight.length = vDim;
	M_Vec3f& point = patch->point.refValue();
	M_Float& weight = patch->weight.refValue();

	for(int i = vDim; --i>=0;){
		int idx = i * uDim + uDim - 1;
		edgePoint[i] = point[idx];
		edgeWeight[i] = weight[idx];
	}
}

void addEdge2(NurbsPatch* patch, M_Vec3f& edgePoint, M_Float& edgeWeight){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	edgePoint.length = uDim;
	edgeWeight.length = uDim;
	M_Vec3f& point = patch->point.refValue();
	M_Float& weight = patch->weight.refValue();

	for(int i = uDim; --i>=0;){
		int idx = uDim * vDim - uDim + i;
		edgePoint[i] = point[idx];
		edgeWeight[i] = weight[idx];
	}
}

void addEdge3(NurbsPatch* patch, M_Vec3f& edgePoint, M_Float& edgeWeight){
	int uDim = patch->dim.x;
	int vDim = patch->dim.y;
	edgePoint.length = vDim;
	edgeWeight.length = vDim;
	M_Vec3f& point = patch->point.refValue();
	M_Float& weight = patch->weight.refValue();

	for(int i = vDim; --i>=0;){
		int idx = i * uDim;
		edgePoint[i] = point[idx];
		edgeWeight[i] = weight[idx];
	}
}

void guessEdge(const Vec3f& pt, float& closestDistance, NurbsPatch*& closestPatch, int& closestEdge, NurbsPatch* patch, int edge, NurbsPatch* blockPatch, int blockEdge, float& closestT, float& closestOther, float other, const Vec3f* edgePointPtr, const float* edgeWeightPtr, const float* edgeKnotPtr, const float order, const float degree, const int dim, const float minT, const float maxT){
	if(patch == blockPatch && edge == blockEdge){
		return;
	}

	for(int i = 0; i < 1000; i++){
		float t = 1.0f / 999 * i;
		t = (maxT - minT) * t + minT;
		Vec3f testCrd;
		CurvePoint(dim-1, degree, edgeKnotPtr, edgePointPtr, edgeWeightPtr, t, testCrd);
		Vec3f diff = pt - testCrd;
		float distance = fabs(sqrt(diff.x*diff.x + diff.y*diff.y + diff.z*diff.z));
		if(distance < .1 && distance < closestDistance){
			closestDistance = distance;
			closestPatch = patch;
			closestEdge = edge;
			closestT = t;
			closestOther = other;
		}
	}
}

bool guessEdge(const Vec3f& pt, Vec2f& base0, NurbsPatch*& patchOwner, const int blockEdge, M_<NurbsPatch*>& cpNodeList){
	static M_Vec3f edgePoint;
	static M_Float edgeWeight;
	NurbsPatch* closestPatch = 0;
	int closestEdge = -1;
	float closestU;
	float closestV;
	float closestDistance = 1000000000000;

	for(int k = cpNodeList.length; --k>=0;){
		NurbsPatch* patch = cpNodeList[k];
		const float* uKnotPtr = patch->uKnot.refValue().getArray();
		const float* vKnotPtr = patch->vKnot.refValue().getArray();
		const Vec3f* pointPtr = patch->point.refValue().getArray();
		const float* weightPtr = patch->weight.refValue().getArray();
		const int uOrder = patch->order.x;
		const int vOrder = patch->order.y;
		const int uDim = patch->dim.x;
		const int vDim = patch->dim.y;

		const float maxU = patch->maxU();
		const float maxV = patch->maxV();
		const float minU = patch->minU();
		const float minV = patch->minV();

		float uDegree = patch->uDegree;
		float vDegree = patch->vDegree;


		addEdge0(patch, edgePoint, edgeWeight);

		guessEdge(pt, closestDistance, closestPatch, closestEdge, patch, 0, patchOwner, blockEdge, closestU, closestV, 0, edgePoint.getArray(), edgeWeight.getArray(), uKnotPtr, uOrder, uDegree, uDim, minU, maxU);

		addEdge1(patch, edgePoint, edgeWeight);
		guessEdge(pt, closestDistance, closestPatch, closestEdge, patch, 1, patchOwner, blockEdge, closestV, closestU, 1, edgePoint.getArray(), edgeWeight.getArray(), vKnotPtr, vOrder, vDegree, vDim, minV, maxV);

		addEdge2(patch, edgePoint, edgeWeight);
		guessEdge(pt, closestDistance, closestPatch, closestEdge, patch, 2, patchOwner, blockEdge, closestU, closestV, 1, edgePoint.getArray(), edgeWeight.getArray(), uKnotPtr, uOrder, uDegree, uDim, minU, maxU);

		addEdge3(patch, edgePoint, edgeWeight);
		guessEdge(pt, closestDistance, closestPatch, closestEdge, patch, 3, patchOwner, blockEdge, closestV, closestU, 0, edgePoint.getArray(), edgeWeight.getArray(), vKnotPtr, vOrder, vDegree, vDim, minV, maxV);
	}
	if(closestPatch){
		base0.x = closestU;
		base0.y = closestV;
		patchOwner = closestPatch;
		return true;
	}
	return false;
}


inline void guessPolyMapTug(NurbsPatch& patch, Art* part, int crdIdx, float& xCrd, float& yCrd, NurbsPatch*& patchOwner, const Vec3f& pt){
	Seamless* smls = patch.smls;
	M_NurbsPatch& partPatchOwner = part->patchOwner;
	M_Vec2f& uvCoord = part->uvCoord.refValue();

	int uOrder = patch.order.x;
	int vOrder = patch.order.y;
	int uDegree = patch.uDegree;
	int vDegree = patch.vDegree;
	int uDim = patch.dim.x;
	int vDim = patch.dim.y;

	float* uKnotPtr = patch.uKnot.refValue().getArray();
	float* vKnotPtr = patch.vKnot.refValue().getArray();
	Vec3f* pointPtr = patch.point.refValue().getArray();
	float* weightPtr = patch.weight.refValue().getArray();

	Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
	float* topWeightPtr = weightPtr+uDim*vDim-uDim;

	Vec3f* r_pointPtr = patch.r_point.getArray();
	float* r_weightPtr = patch.r_weight.getArray();


	float coordChange = .001;

	Vec2f diff;

	Vec3f curvedEdgePoint0;

	float maxU = patch.maxU();
	float maxV = patch.maxV();
	float minU = patch.minU();
	float minV = patch.minV();

	float diffArray[4];
	Vec3f destPoint[4];
	Vec2f base0 = Vec2f(xCrd, yCrd);

	for(int i = 200; --i>=0;){
		Vec2f basePointOffset[4] = {
			Vec2f(-coordChange, 0),
			Vec2f( coordChange, 0),

			Vec2f(0,-coordChange),
			Vec2f(0, coordChange),
		};
		float error = getDiff(base0, destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

		diffArray[0] = getDiff(base0 + basePointOffset[0], destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
		diffArray[1] = getDiff(base0 + basePointOffset[1], destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
		diffArray[2] = getDiff(base0 + basePointOffset[2], destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);
		diffArray[3] = getDiff(base0 + basePointOffset[3], destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, pt);

		float leastDiff = diffArray[0];
		int k = 0;

		for(int j = 1; j < 4; j++){
			float curDiff = diffArray[j];
			if(curDiff < leastDiff){
				leastDiff = curDiff;
				k = j;
			}
		}
		if(diffArray[k] < error){
			base0+=basePointOffset[k];
			coordChange*=2;
		}
		else{
			coordChange*=.5;
			if(coordChange==0){
				break;
			}
		}
	}
	if(base0.x <= 0.0001f){
		if(smls->transferVertexOverBorderWest(patchOwner, crdIdx, partPatchOwner, uvCoord)){
			return;
		}
		else{
			base0.x = 0;
		}
	}
	else if(base0.x >= 0.9999f){
		if(smls->transferVertexOverBorderEast(patchOwner, crdIdx, partPatchOwner, uvCoord)){
			return;
		}
		else{
			base0.x = 1;
		}
	}
	if(base0.y <= 0.0001f){
		if(smls->transferVertexOverBorderSouth(patchOwner, crdIdx, partPatchOwner, uvCoord)){
			return;
		}
		else{
			base0.y = 0;
		}
	}
	else if(base0.y >= 0.9999f){
		if(smls->transferVertexOverBorderNorth(patchOwner, crdIdx, partPatchOwner, uvCoord)){
			return;
		}
		else{
			base0.y = 1;
		}
	}
	xCrd = base0.x;
	yCrd = base0.y;
}


void MultiSelect::backUpCoords(Art* part){
	Seamless* smls = part->smls;
	int len = coordList.length;
	beginDragPos.length = len;
	for(int i = 0; i<len; i++){
		Art* prt = coordList[i].p;
		int idx = coordList[i].i;
		if(part == prt && smls->ActCtrlPnt == idx){
			clickedIndex = i;
		}
		beginDragPos[i] = prt->coord[idx];
	}
}

inline void MultiSelect::rotateSelection(Art* part, Vec3f vec){

	Vec3f normal = -g_scene->rotation * Vec3f(0,0,1);

	Rotation normRot = Rotation(normal, Vec3f(0,0,1));

	Vec3f a = normRot * (vec-autoPivotPoint);
	Vec3f b = normRot * (beginDragPos[clickedIndex]-autoPivotPoint);
	//Rotation twistRot = Rotation(a, b);
	float angleA = Vec2f(a.y, a.x).getAngle();
	float angleB = Vec2f(b.y, b.x).getAngle();

	float rotateAngle = angleB - angleA;

	Rotation rot = Rotation(normal.x, normal.y, normal.z, rotateAngle);
	int len = beginDragPos.length;
	M_Vec3f& coord = part->coord.refValue();
	for(int i = triList.length; --i>=0;){
		coord[triList[i].i] = rot * (beginDragPos[i] - autoPivotPoint) + autoPivotPoint;
	}
}

int findNonRefPart(int seamlessLine, Art* part){
	int line=seamlessLine+part->plidx;
	while(g_sceneTreeList[line].node.p!= part){
		line++;
	};
	return line;
}

void processAllPatches(M_<NurbsPatch*>& cpNodeList);
void commonPostTug();

void dragPivotPoint(Seamless* smls, Vec3f pt){
	g_dragActuallyHappened = true;
	smls->catIMatrix();
	int line = g_scene->selLine;
	Art* part = dynamic_cast<Art*>(g_sceneTreeList[line].node.p);
	if(part == 0){
		return;
	}
	if(g_sceneTreeList[line].ref){
		line = findNonRefPart(getSeamlessParent(line), part);
	}
	//smls->catIMatrix();//is this needed again?

	int i = g_sceneTreeList[line].parent;

	Vec3f newPt;
	if(Art* parent=dynamic_cast<Art*>(g_sceneTreeList[i].node.p)){
		//Vec3f vz = part->pivotPointMatrix * Vec3f(0,0,0);
		//pt.z = vz.z;
		newPt = g_scene->getIViewMatrix() * g_catchIMatrix * parent->imatrix * pt - part->translation;
		newPt = newPt - parent->absolutePivotPoint;




		//newPt = g_scene->getIViewMatrix() * g_catchIMatrix * parent->imatrix * pt - part->translation;
		//newPt = newPt - parent->absolutePivotPoint;
	}
	else if(Seamless* parent=dynamic_cast<Seamless*>(g_sceneTreeList[i].node.p)){
		newPt = g_scene->getIViewMatrix() * g_catchIMatrix * parent->imatrix * (pt + g_transformCP->mouseOffset) - part->translation;
	}
	part->pivotPoint = newPt;

	g_cursorOverPointIlsBase->translation = g_pointer.translation + g_transformCP->mouseOffset;


	if(g_dragWithDescendants == false){
		M_SArt& m_sart = part->part.refValue();
		Vec3f diff = newPt - g_addBone->beginDragPivotPoint[0];
				
		for(int k = 0; k < m_sart.length; k++){
			Art* part = m_sart[k];
			Vec3f v = g_addBone->beginDragPivotPoint[k + 1] - diff;

			if(part->mirror && part->mirrorPart == 0){
				v.x = 0;
			}
			part->pivotPoint.refValue() = v;

			if(Art* mirrorPart = part->mirrorPart){
				v.x = - v.x;
				mirrorPart->pivotPoint.refValue() = v;
			}

		}
	}
}

void PartCageClue::dragPart(Vec3f pt){//do we need this function still?
	Art* art = dynamic_cast<Art*>(g_selNode.getRaw());
	Seamless* smls = art->smls;
	if((g_scene->mode & XRAY_EDIT_MODE) && smls->xRay){
		dragPivotPoint(smls, pt);
		return;
	}
	//if(MultiSelect::pointList.length){
	//	g_dragActuallyHappened = true;
	//	Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * pt;
	//	Vec3f diff = newPt - MultiSelect::beginDragPos[MultiSelect::clickedIndex];
	//	for(int i = MultiSelect::pointList.length; --i>=0;){
	//		int idx = MultiSelect::pointList[i].i;
	//		NurbsPatch* patch = MultiSelect::pointList[i].p;
	//		Vec3f svec = MultiSelect::beginDragPos[i] + diff;
	//		g_transformCP->setFilteredDimension(patch->point.refValue(), idx, svec * patch->weight[idx]);

	//		patch->smls->dragCtrlPoly(patch, idx);
	//	}
	//	processAllPatches(smls->cpNodeList);
	//	commonPostTug();
	//	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
	//		updateCage(part->smls);
	//	}
	//}
}

void PartClue::dragPart(Vec3f pt){
	if(altKeyState() || InputEvent::l_altDown){
		return;
	}
	Art* art = dynamic_cast<Art*>(g_selNode.getRaw());
	Seamless* smls = art->smls;
	if((g_scene->mode & XRAY_EDIT_MODE) && smls->xRay){
		dragPivotPoint(smls, pt);
		return;
	}
	g_dragActuallyHappened = true;

	if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(art)){
		smls->catIMatrix();

		if(g_radial_Tug->active){
			if(Art::tugRadialTugSize){
				float inverseSize = (1 / g_scene->getZoom());
				Vec3f sr = part->matrix * g_viewMatrix * part->coord[smls->ActCtrlPnt];
				g_pointer.translation.y = sr.y;
				g_radial_Tug->radius = inverseSize * abs(sr.x - pt.x);
				part->houseKeep();
			}
			else{
				restoreCoord(smls->partList);
				M_Vec3f& coord = part->coord.refValue();
				radialTug(coord[smls->ActCtrlPnt], pt);
			}
		}
		else if(MultiSelect::coordList.length){
			//if(g_tugRotFlag){
			//	Vec3f vec = g_scene->getIViewMatrix() * part->imatrix * pt;
			//	MultiSelect::rotateSelection(part, vec);
			//}
			//else
			{
				Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * part->imatrix * pt;
				Vec3f diff = newPt - MultiSelect::beginDragPos[MultiSelect::clickedIndex];
				for(int i = MultiSelect::coordList.length; --i >= 0;){
					if(ImmediatePart* part = dynamic_cast<ImmediatePart*>(MultiSelect::coordList[i].p)){
						int idx = MultiSelect::coordList[i].i;
						M_Vec3f& coord = part->coord.refValue();
						Vec3f newPt = MultiSelect::beginDragPos[i] + diff;
						smls->senseMirrorVertex(part, idx, newPt);
						coord[idx] = newPt;
					}
				}
			}
		}
		else{
			M_Vec3f& coord = part->coord.refValue();
			Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * part->imatrix * pt;
			smls->senseMirrorVertex(part, smls->ActCtrlPnt, newPt);
			coord[smls->ActCtrlPnt] = newPt;
		}
	}
	if(Part* whitePart = dynamic_cast<Part*>(art)){
		if(smls->bnspe){
			Seamless* smls = whitePart->smls;
			int ActTriCorner = smls->ActTriCorner;
			int crdIdx = smls->ActCtrlPnt;
			Vec2f& crd = whitePart->uvCoord.refValue()[crdIdx];
			NurbsPatch*& patchOwner = whitePart->patchOwner[crdIdx];
			Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * whitePart->imatrix * pt;
			guessPolyMapTug(*patchOwner, whitePart, crdIdx, crd.x, crd.y, patchOwner, newPt);
			//patchOwner->generate3(whitePart->coord.refValue(), crdIdx, crd);
			smls->doBuild();
			smls->ctrf = true;
			return;
		}
		smls->catIMatrix();
		M_Vec3f& coord = whitePart->coord.refValue();

		NurbsPatch* patch = 0;
		Stem* stem = 0;
		Polymap* polymap = 0;
		ImmediatePart* sourcePart = 0;
		if(getPolyMap(patch, stem, polymap, whitePart, smls->ActTri, sourcePart)){
			Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * whitePart->imatrix * pt;
			if(sourcePart){
				Vec3f& crd = sourcePart->coord.refValue()[smls->ActCtrlPnt-stem->beginCoordOffset];
				guessPolyMapTug(*patch, crd.x, crd.y, newPt, stem->northPole, stem->southPole);
			}
			else if(polymap){
				int ActTri = (smls->ActTri - polymap->beginTriOffset) * 3;
				int ActTriCorner = smls->ActTriCorner;
				int crdIdx = polymap->rcCoord(ActTri + ActTriCorner);
				Vec2f& crd = polymap->coord.refValue()[crdIdx];
				//NurbsPatch*& patchOwner = polymap->patchOwner.refValue()[crdIdx];
				//patch = dynamic_cast<NurbsPatch*>(polymap->surface.refValue()[patchOwner].p);
				//guessPolyMapTug(*patch, smls->cpNodeList, crd.x, crd.y, patchOwner, newPt);
			}
			else{
				int ActTri = (smls->ActTri - stem->beginTriOffset) * 3;
				int ActTriCorner = smls->ActTriCorner;
				if(stem->mould == false){
					if(stem->mirror && stem->mirrorStem == 0){
						if(ActTri >= stem->coordIndex.length){
							ActTri -= stem->coordIndex.length;
							ActTriCorner = abs(3 - ActTriCorner) % 3;
							newPt.x = -(newPt.x - .5) + .5;
						}
					}
					Vec2f& crd = stem->coord.refValue()[stem->rcCoord(ActTri + ActTriCorner)];
					if(false == (stem->mirror && stem->mirrorStem == 0 && crd.x == .5)){
						crd.x = newPt.x;
					}
					crd.y = newPt.y;
					if(crd.x <= 0){
						crd.x = 0;
					}
					else if(crd.x >= 1){
						crd.x = 1;
					}
					if(crd.y <= 0){
						crd.y = 0;
					}
					else if(crd.y >= 1){
						crd.y = 1;
					}
				}
				else{
					if(stem->mirror && stem->mirrorStem == 0){
						if(ActTri >= stem->coordIndex.length){
							ActTri -= stem->coordIndex.length;
							ActTriCorner = abs(3 - ActTriCorner) % 3;
							newPt.x = -newPt.x;
						}
						Vec2f& crd = stem->coord.refValue()[stem->rcCoord(ActTri + ActTriCorner)];
						if(crd.x == .5){
							guessPolyMapTug(*patch, crd.x, crd.y, newPt, stem->northPole, stem->southPole);
							crd.x = .5;
						}
						else{
							guessPolyMapTug(*patch, crd.x, crd.y, newPt, stem->northPole, stem->southPole);
						}
					}
					else{
						if(stem->mirrorStem && stem->coordIndex.length == 0){
							stem = stem->mirrorStem;
							ActTriCorner = abs(3 - ActTriCorner) % 3;
						}
						Vec2f& crd = stem->coord.refValue()[stem->rcCoord(ActTri + ActTriCorner)];
						guessPolyMapTug(*patch, crd.x, crd.y, newPt, stem->northPole, stem->southPole);
					}
				}
			}
			patch->smls->doBuild();
			g_pointer.translation = g_scene->getViewMatrix() * coord[whitePart->triangle.refIndex(smls->ActTri * 3 + smls->ActTriCorner)];
		}
	}
}

void Node::catInverseRotation(Rotation& r){
}
void ProtoDeclare::catInverseRotation(Rotation& r){
}
void Seamless::catInverseRotation(Rotation& r){
	r = -rotation.refValue() * r;
}
void Art::catInverseRotation(Rotation& r){
	r = -rotation.refValue() * r;
}
void Transform::catInverseRotation(Rotation& r){
	r = -rotation.refValue() * r;
}
void AGroup::catInverseRotation(Rotation& r){
}


void dragTranslate(Seamless* smls, const Vec3f pt){
	if(MultiSelect::pointList.length == 0){
		return;
	}


	//M_Vec3f& point = this->point;
	//M_Vec3f& dpoint = this->dpoint;

	//Matrix matrix = translationMatrix(-g_transformCP->pivotPoint) * rotationMatrix(-g_transformCP->scaleOrientation) * scaleMatrix(g_transformCP->scale) * rotationMatrix(g_transformCP->scaleOrientation) * rotationMatrix(g_transformCP->rotation) * translationMatrix(g_transformCP->pivotPoint) * translationMatrix(g_transformCP->remTranslation);
	//g_transformCP->clickPoint = matrix * MultiSelect::beginDragPos[MultiSelect::clickedIndex];

	Vec3f ipt = smls->getSmlsIViewMatrix() * (pt + g_transformCP->mouseOffset);

	if(g_scene->mode & MODELLING_MODE && g_transformCP->mode != TransformCP::TRANSLATE && false == g_transformCP->isOverDimPoint){
		NurbsPatch* hitPatch = MultiSelect::pointList[MultiSelect::selectedPoint].p;
		if(hitPatch->mirror && hitPatch->mirrorPatch == 0){
			int hitIdx = MultiSelect::pointList[MultiSelect::selectedPoint].i;
			int uDim = hitPatch->dim.x;
			int u = hitIdx % uDim;
			if(u == (uDim >> 1) && (uDim & 1)){
				ipt.x = 0;
			}
		}
	}

	if(g_scene->mode & SINGLE_ACTIVE_MODE){
		if(MultiSelect::selectedPoint >= 0){
			Vec3f vec = ipt - g_transformCP->clickPoint + g_transformCP->remTranslation;
			g_transformCP->setFilteredDimension(g_transformCP->translation.refValue(), vec);

			//smls->cp.refValue()[cpIndex[MultiSelect::withinSightsIndex]] = v;

			NurbsPatch* patch = MultiSelect::pointList[MultiSelect::selectedPoint].p;
			int idx = MultiSelect::pointList[MultiSelect::selectedPoint].i;
			Vec3f svec = MultiSelect::beginDragPos[MultiSelect::selectedPoint] + g_transformCP->translation;
			Vec3f pnt;
			float w = patch->weight[idx];
			g_transformCP->setFilteredDimension(patch->point.refValue(), smls->cp.refValue(), patch->cpIndex.refValue(), idx, svec * patch->weight[idx]);
			patch->dpoint.refValue()[idx] = patch->point[idx] / patch->weight[idx];
			patch->mirrorSingleCP(idx);

			if(false == (g_scene->mode.refValue() & BOX_PATCH_MODELLING_MODE)){
				smls->calcAllPolyMidPoints();
			}
			commonPostTug();
			smls->updateCage();
			patch->houseKeep();
		}
	}
	else{
		
		if(g_transformCP->mode == TransformCP::TRANSLATE){
			Vec3f vec = ipt - (g_transformCP->pivotPoint + g_transformCP->remTranslation);
			vec = g_scene->rotation * Rotation(0, 0, 1, -g_transformCP->leverAngle) * vec;
			vec = vec * g_transformCP->leverAxis[g_transformCP->leverIdx];
			vec = Rotation(0, 0, 1, g_transformCP->leverAngle) * -g_scene->rotation * vec;
			vec = vec + (g_transformCP->pivotPoint + g_transformCP->remTranslation);
			
			vec = vec - g_transformCP->clickPoint + g_transformCP->remTranslation;
			g_transformCP->translation.refValue() = vec;
		}
		else{
			Vec3f vec = ipt - g_transformCP->clickPoint + g_transformCP->remTranslation;
			g_transformCP->setFilteredDimension(g_transformCP->translation.refValue(), vec);
		}
		g_transformCP->calcRotCtrlPnts();
		g_transformCP->processFields();
		//g_pointer.translation = g_scene->getViewMatrix() * g_transformCP->translation.refValue();
	}
	g_transformCP->houseKeep();

}


void AnimLever::dragPart(Vec3f pt){
	if(g_dragMorphFlag){
		g_dragActuallyHappened = true;
		dragPartForMorph(pt);
		return;
	}
	


	int line = g_scene->selLine;
	Art* part = dynamic_cast<Art*>(g_sceneTreeList[line].node.p);
	if(part == 0){
		return;
	}
	Seamless* smls = part->smls;

	if(g_transformCP->dragRotPointMode){
		g_transformCP->mode = TransformCP::NOTHING;
		g_transformCP->setMode(g_transformCP->getKeyMode());

		switch(g_transformCP->mode){
		case TransformCP::SCALE3D:
			g_transformCP->scalePoints(true, true, true);
			break;
		case TransformCP::TRANSLATE:
			dragTranslate(smls, pt);
			//translatePoints(smls, pt);
			break;
		case TransformCP::ROTATE_AND_SCALE1D: 
			g_transformCP->scalePoints(false, false, false);
			g_transformCP->stretchHappened = true;
			break;
		case TransformCP::SCALE2D:
			g_transformCP->scalePoints(true, false, true);
			break;
		case TransformCP::SCALE1D:
			g_transformCP->scalePoints(true, false, false);
			break;
		case TransformCP::NEUTRAL:
			g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx] = g_pointer.translation + g_transformCP->mouseOffset;
			g_transformCP->calcNonSelectedRotCtrlPnts();
			g_transformCP->houseKeep();
			break;
		case TransformCP::ROTATE:
			if(smls->cpNodeList.length){
				g_transformCP->dragRot(smls, pt + g_transformCP->mouseOffset);
				//g_dragActuallyHappened = false;
			}
			break;
		}
		g_transformCP->houseKeep();
		return;

	}
	else if(g_transformCP->dragPivotPointMode){
		g_transformCP->pivotPoint.refValue() = g_scene->getIViewMatrix() * g_catchIMatrix * translationMatrix(-g_transformCP->translation) * (pt + g_transformCP->mouseOffset);
		g_transformCP->calcRotCtrlPnts();
		//g_transformCP->rotCtrlPnt[g_transformCP->leverIdx] = g_scene->getViewMatrix() * g_transformCP->pivotPoint + g_transformCP->leverAxis[g_transformCP->leverIdx] * .5;
		//g_transformCP->calcNonSelectedRotCtrlPnts();
		//g_cursorOverPointIlsBase->translation = (pt + g_transformCP->mouseOffset);
		g_transformCP->processFields();
		g_transformCP->houseKeep();
		return;
	}
	else if(g_partCageClue->mode == PartCageClue::MORPH_CP){
		if(g_scene->mode & ENABLE_MORPH_CAPTURE){
			//g_partCageClue->dragPart(pt);
			int idx = part->index[0];
			if(idx < part->controlPoint.length){
				dragTranslate(smls, pt);
				g_cursorOverPointIlsBase->translation.refValue() = pt;
			}
		}
		return;
	}

	if(g_sceneTreeList[line].ref){
		line = findNonRefPart(getSeamlessParent(line), part);
	}

	if(g_partCageClue->mode == PartCageClue::BONE_PIVOT_POINT){
		if(part->ik || part == part->smls->partList[0]){
			g_dragActuallyHappened = true;
			if(part->ik){
				Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * (pt + g_transformCP->mouseOffset);
				part->ikPos = newPt;
			}
			else{
				smls->catIMatrix();
				int i = g_sceneTreeList[line].parent;
				if(Art* parent=dynamic_cast<Art*>(g_sceneTreeList[i].node.p)){
					Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * parent->imatrix * (pt + g_transformCP->mouseOffset);
					part->translation = newPt - part->absolutePivotPoint;
				}
				else if(Seamless* parent=dynamic_cast<Seamless*>(g_sceneTreeList[i].node.p)){
					Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * parent->imatrix * (pt + g_transformCP->mouseOffset);
					part->translation = newPt - part->absolutePivotPoint;
					this->calcRotCtrlPnts();
					this->houseKeep(part, anim);
				}
			}
		}
	}
	else{
		g_dragActuallyHappened = true;
		Vec3f pivotPoint  = part->pivotPointMatrix * Vec3f(0,0,0);

		//Vec3f pivotPoint=Rotation(1, 0, 0, .698132) * g_scene->rotation * Vec3f(0, .5, 0);
		Vec2f toDiff = Vec2f(pt.x + g_transformCP->mouseOffset.x - pivotPoint.x, pt.y + g_transformCP->mouseOffset.y - pivotPoint.y);
		Vec2f fromDiff = Vec2f(rotCtrlPnt[leverIdx].x - pivotPoint.x, rotCtrlPnt[leverIdx].y - pivotPoint.y);
		float toAngle = toDiff.getAngle();
		float fromAngle = fromDiff.getAngle();

		leverAngle = toAngle - fromAngle;

		if(part->ik){
			Vec3f newPt = g_scene->getIViewMatrix() * pt;
			Vec2f toDiff = Vec2f(pt.x + g_transformCP->mouseOffset.x - pivotPoint.x, pt.y + g_transformCP->mouseOffset.y - pivotPoint.y);
			Vec3f axis;
			axis = (-g_scene->rotation * Vec3f(0, 0, 1)).normalize();
			part->ikRot = part->ikRot.refValue() * Rotation(axis, leverAngle);
		}
		else{
			Rotation unwind;
			int i = line;
			do{
				i = g_sceneTreeList[i].parent;
				g_sceneTreeList[i].node->catInverseRotation(unwind);
			}while(i>0);

			Vec3f axis;
			if(g_transformCP->rotAxisZ){
				axis = (-g_scene->rotation * unwind * Vec3f(0, 0, 1)).normalize();
			}
			else if(g_transformCP->rotAxisY){
				axis = (-g_scene->rotation * unwind * Vec3f(0, 1, 0)).normalize();
			}
			else if(g_transformCP->rotAxisX){
				axis = (-g_scene->rotation * unwind * Vec3f(-1, 0, 0)).normalize();
			}

			part->rotation = part->rotation.refValue() * Rotation(axis, leverAngle);
		}
		rotCtrlPnt[leverIdx] = pt + g_transformCP->mouseOffset;
		g_cursorOverPointIlsBase->translation.refValue() = pt + g_transformCP->mouseOffset;
		this->calcNonSelectedRotCtrlPnts();
		this->houseKeep(part, anim);
	}

	part->smls->catIMatrix();
	part->smls->catMatrix();

	anim->setMorphersForTransformDrag(smls);
}

inline Matrix dxWindMatrix(Matrix& parentMatrix, Vec3f& scale, Rotation& rotation, Vec3f& translation);

void TransformObj::getPivotPointMatrix(Matrix& pivotPointMatrix, Node* node)const{
	if(Seamless* smls = dynamic_cast<Seamless*>(node)){
		Matrix parentMatrix = g_scene->getViewMatrix();
		pivotPointMatrix = translationMatrix(translation + pivotPoint) * parentMatrix;
		pivotPointMatrix = rotationMatrix(scaleOrientation) * scaleMatrix(scale) * rotationMatrix(-scaleOrientation) * rotationMatrix(rotation) * pivotPointMatrix;
	}
}

void TransformCP::getPivotPointMatrix(Matrix& pivotPointMatrix, Seamless* smls)const{
	g_viewMatrix = g_scene->getViewMatrix();
	Matrix parentMatrix = dxWindMatrix(g_viewMatrix, smls->scale, smls->rotation, smls->translation);
	pivotPointMatrix = translationMatrix(translation + pivotPoint) * parentMatrix;
	pivotPointMatrix = rotationMatrix(scaleOrientation) * scaleMatrix(scale) * rotationMatrix(-scaleOrientation) * rotationMatrix(rotation) * pivotPointMatrix;
}


Vec3f calcRingNormal(int v, Vec3f pivotPoint, int selUdim, int from){
	Vec3f ringNormal;
	for(int u = 0; u < selUdim - 1; u++){
		Vec3f pa = pivotPoint;
		Vec3f pb = MultiSelect::beginDragPos[from + u + 1];
		Vec3f pc = MultiSelect::beginDragPos[from + u];
		Vec3f test = pb - pa;
		if(fabs(test.x) < .00001 && fabs(test.y) < .00001 && fabs(test.z) < .00001){
			continue;
		}
		test = pc - pa;
		if(fabs(test.x) < .00001 && fabs(test.y) < .00001 && fabs(test.z) < .00001){
			continue;
		}
		test = pb - pc;
		if(fabs(test.x) < .00001 && fabs(test.y) < .00001 && fabs(test.z) < .00001){
			continue;
		}

		ringNormal += calcNormal(pa, pb, pc);
	}
	return ringNormal.normalize();
}

inline void twistRing(const int& i, const int& selUdim, const int from, M_Vec3f& point, M_Vec3f& dpoint, M_Float& weight, float& twistAngle){

	Vec3f pivotPoint = calcRingPivotPoint2(selUdim, from);
	Vec3f normal = calcRingNormal(i, pivotPoint, selUdim, from);

	Rotation normRot = -Rotation(Vec3f(0, 1, 0), normal);

	Rotation rot = Rotation(normal.x, normal.y, normal.z, twistAngle);
	for(int u = 0; u < selUdim; u++){
		int srcIdx = from + u;
		int dstIdx = MultiSelect::pointList[srcIdx].i;
		dpoint[dstIdx] = rot * (MultiSelect::beginDragPos[srcIdx] - pivotPoint) + pivotPoint;
		point[dstIdx] = dpoint[dstIdx] * weight[dstIdx];
	}
}

void stretch(bool lock, bool x, bool z, M_Vec3f& dpoint){
	Vec3f pp = g_scene->getViewMatrix() * g_transformCP->pivotPoint;
	Vec3f d = g_scene->getViewMatrix() * MultiSelect::beginDragPos[MultiSelect::clickedIndex];
	pp.z = d.z;
	g_pointer.translation.z = pp.z;
	pp = g_scene->getIViewMatrix() * pp;
	//g_transformCP->pivotPoint.refValue() = pp;
	//pp = g_scene->getIViewMatrix() * pp;

	Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * g_pointer.translation;
	Vec3f ev = newPt - (g_transformCP->translation + pp);
	Vec3f bv = MultiSelect::beginDragPos[MultiSelect::clickedIndex] - pp;
	float bdis = bv.length();
	float edis = ev.length();
	Vec3f baxis = bv.normalize();
	Vec3f eaxis = ev.normalize();

	if(fabs(bdis) > .0001){
		float s = edis / bdis;
		if(x){
			g_transformCP->scale.refValue().x = s;
		}
		g_transformCP->scale.refValue().y = s;
		if(z){
			g_transformCP->scale.refValue().z = s;
		}
		if(lock == false){
			g_transformCP->rotation.refValue() = Rotation(baxis, eaxis);
		}
		g_transformCP->scaleOrientation = -Rotation(baxis, Vec3f(0, 1, 0));
		g_pointer.translation = g_scene->getViewMatrix() * dpoint[MultiSelect::pointList[MultiSelect::clickedIndex].i];
//		g_pointer.translation.z = 0;
	}
}

void TransformCP::calcScaleOrientation(){

	Vec3f pp = g_scene->getViewMatrix() * g_transformCP->pivotPoint;
	Vec3f d = rotCtrlPnt[leverIdx];
	pp.z = d.z;
	pp = g_scene->getIViewMatrix() * pp;

	Vec3f bv = g_scene->getIViewMatrix() * rotCtrlPnt[leverIdx] - pp;
	float bdis = bv.length();
	if(bdis > .00001){
		Vec3f baxis = bv.normalize();
		Rotation so = Rotation(baxis, Vec3f(0, 1, 0));
		scaleOrientation.refValue() = -so;
	}
}

void TransformCP::calcLeverAngle(){
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

	Vec3f v = rotCtrlPnt[leverIdx] - pp;
	if(v.length() > .00001){
		Vec3f la = leverAxis[leverIdx];
		leverAngle = atan2(v.y, v.x) - atan2(la.y, la.x);
		leverRot = Rotation(0, 0, 1, leverAngle);
	}
}

void TransformCP::scalePoints(bool lock, bool x, bool z){
	if(g_mousePos == Vec2i(663,426 + 15)){
		int a = 0;
	}

	Vec3f pp = g_scene->getViewMatrix() * pivotPoint;
	Vec3f d = remRotCtrlPnt;
	pp.z = d.z;
//	g_pointer.translation.z = pp.z;
	pp = g_scene->getIViewMatrix() * pp;

	Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * (g_pointer.translation + mouseOffset);
	Vec3f ev = newPt - (translation + pp);
	Vec3f bv = g_scene->getIViewMatrix() * remRotCtrlPnt - pp;
	float bdis = bv.length();
	float edis = ev.length();
	Vec3f baxis = bv.normalize();
	Vec3f eaxis = ev.normalize();
	Rotation so = Rotation(baxis, Vec3f(0, 1, 0));
	//Rotation so = -scaleOrientation;


	float s = 0;

	if(bdis >= .00001){
		s = edis / bdis;
	}

	Vec3f v = remRotCtrlPnt - g_scene->getViewMatrix() * (pivotPoint + translation);
	Vec3f la = leverAxis[1];
	float angle = atan2(v.y, v.x) - atan2(la.y, la.x);
	Rotation rot = Rotation(0, 0, 1, angle);


	Vec3f iPointer = -rot * ((g_pointer.translation + mouseOffset) - g_scene->getViewMatrix() * pivotPoint);

	if(iPointer.y <= 0){
		s = 0;
	}

	{
		if(x){
			scale.refValue().x = s;
		}
		scale.refValue().y = s;
		if(z){
			scale.refValue().z = s;
		}
		if(lock == false){
			rotation.refValue() = Rotation(baxis, eaxis);
			rotCtrlPnt[leverIdx] = g_pointer.translation + mouseOffset;
		}
		else{
			//Vec3f& rotPoint = (overWhatPoint == TransformCP::ROTATE) ? rotCtrlPnt[leverIdx] : rotCtrlPnt3[leverIdx];
			if(s == 0){
				rotCtrlPnt[leverIdx] = g_scene->getViewMatrix() * pivotPoint;
			}
			else{
				rotCtrlPnt[leverIdx] = g_scene->getViewMatrix() * (rotation * -so * Vec3f(0, edis, 0) + pp);
			}
		}
		calcNonSelectedRotCtrlPnts();
		//scaleOrientation = -so;
		//rotCtrlPnt = g_pointer.translation;
//		g_pointer.translation.z = 0;

		processFields();
	}
}


void TransformCP::quantizeRotationLever(){
	if(MultiSelect::pointList.length >= g_transformCP->leverSelection){
		//if(false == ((controlKeyState() || InputEvent::l_ctrlDown)) || stretchHappened){
		//	return;
		//}

		Vec3f rp = rotCtrlPnt[leverIdx];
		Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);
		rp.z = pp.z;
		rp = (rp - pp);
		Vec3f nv = rp.normalize();
		float closest = 100000;
		Vec3f snappedAxis;
		for(int i = 0; i < 8; i++){
			Vec3f v = Rotation(0, 0, 1, PI * 2 / 8 * i) * Vec3f(1, 0, 0);
			float diff = abs((nv - v).length());
			if(diff < closest){
				closest = diff;
				snappedAxis = v;
			}
		}
		//rotCtrlPnt[leverIdx] = snappedAxis * rp.length() + pp;
		rotCtrlPnt[leverIdx] = snappedAxis * .5 + pp;
		calcNonSelectedRotCtrlPnts();
	}
	houseKeep();
}

void Strand::drag(Vec3f pt){
	if(Hair* hair = getHair()){
		if(hair->selLock){
			hair->drag(pt);
		}
		else{
			NurbsPatch::drag(pt);
		}
	}
}

int TransformObj::getKeyMode()const{
	if((shiftKeyState() || InputEvent::l_shiftDown)){
		if((controlKeyState() || InputEvent::l_ctrlDown)){
		}
		else{
			return TRANSLATE;
		}
	}
	else if((controlKeyState() || InputEvent::l_ctrlDown)){
		return NEUTRAL;
	}
	else{
		return ROTATE;
	}
	return NOTHING;
}

int TransformCP::getKeyMode()const{
	if(overWhatPoint == ROTATE){
		if((shiftKeyState() || InputEvent::l_shiftDown)){
			if((controlKeyState() || InputEvent::l_ctrlDown)){
				if((altKeyState() || InputEvent::l_altDown)){
					return SCALE2D;
				}
				else{
					return TRANSLATE;
				}
			}
			else{
				return TRANSLATE;	// use to be ROTATE_AND_SCALE1D;
			}
		}
		else if((altKeyState() || InputEvent::l_altDown)){
			if((controlKeyState() || InputEvent::l_ctrlDown)){
				return SCALE3D;
			}
			else{
				return SCALE1D;
			}
		}
		else if((controlKeyState() || InputEvent::l_ctrlDown)){
			return NEUTRAL;
		}
		else{
			if(leverNeutral){
				return NEUTRAL;
			}
			else if(leverRotate){
				return ROTATE;
			}
			else if(leverStretch){
				return SCALE1D;
			}
			else if(leverRotateAndStretch){
				return ROTATE_AND_SCALE1D;
			}
			else if(leverScale){
				return SCALE3D;
			}
			else if(leverTranslate){
				return TRANSLATE;
			}
		}
	}
	else{
		if((controlKeyState() || InputEvent::l_ctrlDown)){
			return NEUTRAL;
		}
		return overWhatPoint;
	}
	return 0;
}

void TransformObj::setMode(int newMode){
	if(newMode != mode){
		if(mode == ROTATE){
			remRotCtrlPnt = rotCtrlPnt[leverIdx];
			remRotation = rotation;
		}
		else if(mode == TRANSLATE){
			remTranslation = translation;
		}
	}
}



void Seamless::drag(Vec3f pt){
	g_viewMatrix = g_scene->getViewMatrix();
	Matrix iViewMatrix = g_scene->getIViewMatrix();

	switch(g_transformObj->mode){
		case TransformObj::ROTATE:{
			g_transformObj->dragRot(this, pt);
			break;
		}
		case TransformObj::TRANSLATE:{
			Matrix matrix = translationMatrix(-g_transformObj->pivotPoint) * rotationMatrix(-g_transformObj->scaleOrientation) * scaleMatrix(g_transformObj->scale) * rotationMatrix(g_transformObj->scaleOrientation) * rotationMatrix(g_transformObj->rotation) * translationMatrix(g_transformObj->pivotPoint) * translationMatrix(g_transformObj->remTranslation);
			Vec3f originalPt = matrix * MultiSelect::beginDragPos[MultiSelect::clickedIndex];

			if(Seamless* smls = dynamic_cast<Seamless*>(MultiSelect::objectList[0].p)){ //should use MultiSelect::clickedIndex instead of 0
				Vec3f ipt = g_scene->getIViewMatrix() * pt;
				g_transformObj->translation.refValue() = ipt - originalPt + g_transformObj->remTranslation;
				g_transformObj->calcRotCtrlPnts();
				g_transformObj->processFields();
				g_cursorOverPointIlsBase->translation.refValue() = pt;
			}
			g_transformObj->houseKeep();
		}
	}
	g_dragActuallyHappened = true;
}



void dragNurbsCage(Seamless* smls, Vec3f pt){
	static String str;
	g_viewMatrix = g_scene->getViewMatrix();
	Matrix iViewMatrix = g_scene->getIViewMatrix();

	//Matrix smlsMatrix = scaleMatrix(smls->scale) * rotationMatrix(smls->rotation) * translationMatrix(smls->translation);
	////Matrix smlsIMatrix = scaleMatrix(Vec3f(1./smls->scale.x, 1./smls->scale.y, 1./smls->scale.z)) * rotationMatrix(-smls->rotation) * translationMatrix(-smls->translation);

	//pt = smls->getSmlsIViewMatrix() * pt;
	//pt = g_viewMatrix * pt;
	

	//if((r_controlKeyState() || InputEvent::r_ctrlDown) && (0x8000 & GetKeyState(VK_RSHIFT))){
	//	


	//		Matrix parentMatrix = g_scene->getViewMatrix();
	//		Matrix pivotPointMatrix = translationMatrix(g_transformCP->translation + g_transformCP->pivotPoint) * parentMatrix;
	//		pivotPointMatrix = rotationMatrix(g_transformCP->rotation) * pivotPointMatrix;
	//		Vec3f screenPivotPoint = pivotPointMatrix * Vec3f();

	//		Vec3f selCoord  = pivotPointMatrix * (MultiSelect::beginDragPos[MultiSelect::clickedIndex] - g_transformCP->pivotPoint);

	//		Vec3f bv = selCoord - screenPivotPoint;
	//		Vec3f ev = pt - screenPivotPoint;

	//		float bdis = bv.length();
	//		float edis = ev.length();

	//		Vec3f baxis = bv.normalize();
	//		Vec3f eaxis = ev.normalize();
	//		if(fabs(bdis) > .0001){
	//			float s = edis / bdis;
	//			Vec3f scale = Vec3f(s, s, s);
	//			g_transformCP->scale = scale;
	//			g_pointer.translation = g_scene->getViewMatrix() * dpoint[MultiSelect::pointList[MultiSelect::clickedIndex].i];
	//		}
	//	}
	if(g_transformCP->dragRotPointMode){
		g_transformCP->mode = TransformCP::NOTHING;
		g_transformCP->setMode(g_transformCP->getKeyMode());

		switch(g_transformCP->mode){
		case TransformCP::SCALE3D:
			g_transformCP->scalePoints(true, true, true);
			break;
		case TransformCP::TRANSLATE:
			dragTranslate(smls, pt);
			//translatePoints(smls, pt);
			break;
		case TransformCP::ROTATE_AND_SCALE1D: 
			g_transformCP->scalePoints(false, false, false);
			g_transformCP->stretchHappened = true;
			break;
		case TransformCP::SCALE2D:
			g_transformCP->scalePoints(true, false, true);
			break;
		case TransformCP::SCALE1D:
			g_transformCP->scalePoints(true, false, false);
			break;
		case TransformCP::NEUTRAL:
			g_pointer.translation = pt;
			g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx] = pt + g_transformCP->mouseOffset;
			g_transformCP->calcNonSelectedRotCtrlPnts();
			g_transformCP->calcScaleOrientation();
			g_transformCP->calcLeverAngle();
			g_transformCP->houseKeep();
			break;
			//break;
		case TransformCP::ROTATE:
			g_transformCP->dragRot(smls, pt + g_transformCP->mouseOffset);
			break;
		}

	}
	else if((shiftKeyState() || InputEvent::l_shiftDown)){
		if((controlKeyState() || InputEvent::l_ctrlDown)){
			if((altKeyState() || InputEvent::l_altDown)){
				g_transformCP->scaleOrientation.refValue() = Rotation();
				g_transformCP->remRotCtrlPnt = g_transformCP->rotCtrlPnt[g_transformCP->leverIdx];
				Vec3f newPt = g_scene->getIViewMatrix() * g_catchIMatrix * pt;
				Vec3f a = MultiSelect::beginDragPos[MultiSelect::clickedIndex] - g_transformCP->pivotPoint;
				Vec3f b = -g_transformCP->rotation.refValue() * (newPt - g_transformCP->pivotPoint - g_transformCP->translation);
				if(fabs(a.x) < .00001){
					a.x = 1;
					b.x = 1;
				}
				if(fabs(a.y) < .00001){
					a.y = 1;
					b.y = 1;
				}
				if(fabs(a.z) < .00001){
					a.z = 1;
					b.z = 1;
				}
				Vec3f scale = b / a;
				g_transformCP->scale = scale;
			}
			else{
			}
		}
		//else{
		//	stretch(true, false, true, dpoint);
		//}
	}
/*
	else if((altKeyState() || InputEvent::l_altDown)){
		if((controlKeyState() || InputEvent::l_ctrlDown)){
			stretch(true, false, false, dpoint);
		}
		else{
			stretch(true, true, true, dpoint);
			//stretch(false, false, false, dpoint);
		}
	}
*/
	else if(controlKeyState()){
	}
	else if(g_transformCP->dragPivotPointMode){
		g_transformCP->pivotPoint.refValue() = g_scene->getIViewMatrix() * g_catchIMatrix * translationMatrix(-g_transformCP->translation) * (pt + g_transformCP->mouseOffset);
		g_transformCP->calcRotCtrlPnts();
		//g_transformCP->rotCtrlPnt[g_transformCP->leverIdx] = g_scene->getViewMatrix() * g_transformCP->pivotPoint + g_transformCP->leverAxis[g_transformCP->leverIdx] * .5;
		//g_transformCP->calcNonSelectedRotCtrlPnts();
		g_transformCP->processFields();
	}
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(MultiSelect::withinSightsNode.p)){
		if(g_radial_Tug->active){
			M_Vec3f& point = patch->point;
			M_Vec3f& dpoint = patch->dpoint;
			int idx = patch->index[0];


			if(Art::tugRadialTugSize){

				float inverseSize = (1 / g_scene->getZoom());
				Vec3f sr = g_viewMatrix * patch->dpoint[idx];
				g_pointer.translation.y = sr.y;
				g_radial_Tug->radius = inverseSize * abs(sr.x - pt.x);
				houseKeepVisualClue();
			}
			else{
				//restorePatchPoint(this);
				Vec3f radialPt = pt + g_transformCP->mouseOffset;
				//pt.z += MultiSelect::beginDragPos[0].z 
				patch->radialTug(g_backUpCoord[patch->fromPoint + idx], radialPt - g_transformCP->dimRadialAdjust);
				smls->calcAllPolyMidPoints();
				commonPostTug();
				smls->updateCage();
				houseKeepVisualClue();
			}
		}
		else{
			dragTranslate(smls, pt);
		}
	}
	g_cursorOverPointIlsBase->translation = g_pointer.translation;//not wanted any more or is doing wrong thing?
	g_dragActuallyHappened = true;
}

void GroupBuild::drag(Vec3f pt){
	if(Seamless* smls = getSeamless2()){
		dragNurbsCage(smls, pt);
	}
}

void NurbsPatch::drag(Vec3f pt){
	if(Art* part = dynamic_cast<Art*>(g_selNode.getRaw())){
		part->drag(pt);
	}
	else{
		dragNurbsCage(smls, pt);
	}
}


void TransformCP::dragRot(Seamless* smls, Vec3f pt){

	Vec3f& pivotPoint = this->pivotPoint.refValue();
	//Rotation& rotation = this->rotation.refValue();
	Vec3f& translation = this->translation.refValue();

	Matrix pivotPointMatrix;
	getPivotPointMatrix(pivotPointMatrix, smls);

	int line = g_scene->selLine;

	Vec3f pivotPointVM  = pivotPointMatrix * Vec3f(0,0,0);

	//Vec3f pivotPoint=Rotation(1, 0, 0, .698132) * g_scene->rotation * Vec3f(0, .5, 0);
	Vec2f toDiff = Vec2f(pt.x - pivotPointVM.x, pt.y - pivotPointVM.y);
	Vec2f fromDiff = Vec2f(remRotCtrlPnt.x - pivotPointVM.x, remRotCtrlPnt.y - pivotPointVM.y);
	float toAngle = toDiff.getAngle();
	float fromAngle = fromDiff.getAngle();

	float angle = toAngle - fromAngle;

	if(selectRings && r_controlKeyState()){
		if(NurbsPatch* selPatch = dynamic_cast<NurbsPatch*>(g_selNode.getRaw())){
			M_Vec3f& point = selPatch->point.refValue();
			M_Vec3f& dpoint = selPatch->dpoint.refValue();
			M_Float& weight = selPatch->weight.refValue();

			for(int i = 0; i < MultiSelect::vRingDim; i++){
				int from = i * MultiSelect::uRingDim;
				twistRing(i, MultiSelect::uRingDim, from, point, dpoint, weight, angle);
			}
			rotCtrlPnt[leverIdx] = pt;
			rotation.refValue() = Rotation();
			selPatch->processFields();
			smls->doBuild();
			smls->ctrf = true;

			selPatch->houseKeep();
		}
		return;
	}

	Rotation unwind;
	int i = line;
	do{
		i = g_sceneTreeList[i].parent;
		g_sceneTreeList[i].node->catInverseRotation(unwind);
	}while(i>0);

	Vec3f axis;
	if(rotAxisZ){
		axis = (-g_scene->rotation * unwind * Vec3f(0, 0, 1)).normalize();
	}
	else if(rotAxisY){
		axis = (-g_scene->rotation * unwind * Vec3f(0, 1, 0)).normalize();
	}
	else if(rotAxisX){
		axis = (-g_scene->rotation * unwind * Vec3f(-1, 0, 0)).normalize();
	}
	
	rotCtrlPnt[leverIdx] = pt;
	g_cursorOverPointIlsBase->translation.refValue() = pt;

	rotation.refValue() = remRotation * Rotation(axis, angle);

	calcNonSelectedRotCtrlPnts();
	calcScaleOrientation();
	calcLeverAngle();
	processFields();

/*	
	Matrix matrix = translationMatrix(-pivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation) * translationMatrix(pivotPoint);


	if(MultiSelect::pointList.length){
		for(int i = MultiSelect::pointList.length; --i>=0;){
			int idx = MultiSelect::pointList[i].i;
			NurbsPatch* patch = MultiSelect::pointList[i].p;
			Vec3f svec = matrix * MultiSelect::beginDragPos[i];
			patch->setFilteredDimension(patch->point.refValue(), idx, svec * patch->weight[idx]);
		}
		smls->calcAllPolyMidPoints();
		processAllPatches(smls->cpNodeList);
		commonPostTug();
	}
	selPatch->houseKeep();
*/

}

void Viewpoint::drag(Vec3f pt){

	Vec3f newPt = g_scene->getIViewMatrix() * pt;


	if(dragMode == LOOK_AT){
//		calcOrientationFromLookAtUp();

		Vec3f upOffset = up;




		g_dragActuallyHappened = true;
		Vec3f pivotPoint = g_scene->getViewMatrix() * position;
		Vec3f rotCtrlPnt = g_scene->getViewMatrix() * remLookAt;

		Vec2f toDiff = Vec2f(pt.x - pivotPoint.x, pt.y - pivotPoint.y);
		Vec2f fromDiff = Vec2f(rotCtrlPnt.x - pivotPoint.x, rotCtrlPnt.y - pivotPoint.y);
		float toAngle = toDiff.getAngle();
		float fromAngle = fromDiff.getAngle();

		float leverAngle = toAngle - fromAngle;

		Vec3f axis = (-g_scene->rotation * Vec3f(0, 0, 1)).normalize();

		orientation.refValue() = remOrientation * Rotation(axis, leverAngle);

		float newLookAtDistance2d = toDiff.length();
		float oldLookAtDistance2d = fromDiff.length();
		float ratio = newLookAtDistance2d / oldLookAtDistance2d;

		float lookAtDistance = remLookAtDistance * ratio;

		lookAt.refValue() = orientation.refValue() * Vec3f(0, 0, -lookAtDistance) + position;

		//g_pointer.translation = g_scene->getViewMatrix() * lookAt;
		calcOrientationFromLookAtUp();
		//upOffset = orientation.refValue() * Vec3f(0, remUpDistance, 0);
		//up.refValue() = upOffset;

	}
	else if(dragMode == UP_LEVER){
		up.refValue() = newPt - position;
		calcOrientationFromLookAtUp();

		Rotation negRot = -orientation;

		Vec3f upOffset = negRot * up;

		float upDistance = upOffset.y;

		//up.refValue() = orientation.refValue() * Vec3f(0, upDistance, 0);
		g_pointer.translation = g_scene->getViewMatrix() * (up + position);

		g_dragActuallyHappened = true;

	}
	else if(dragMode == POSITION){

		g_dragActuallyHappened = true;
		Vec3f pivotPoint = g_scene->getViewMatrix() * remLookAt;
		Vec3f rotCtrlPnt = g_scene->getViewMatrix() * remPosition;

		Vec2f toDiff = Vec2f(pt.x - pivotPoint.x, pt.y - pivotPoint.y);
		Vec2f fromDiff = Vec2f(rotCtrlPnt.x - pivotPoint.x, rotCtrlPnt.y - pivotPoint.y);
		float toAngle = toDiff.getAngle();
		float fromAngle = fromDiff.getAngle();

		float leverAngle = toAngle - fromAngle;

		Vec3f axis = (-g_scene->rotation * Vec3f(0, 0, 1)).normalize();

		orientation.refValue() = remOrientation * Rotation(axis, leverAngle);


		float newLookAtDistance2d = toDiff.length();
		float oldLookAtDistance2d = fromDiff.length();
		float ratio = newLookAtDistance2d / oldLookAtDistance2d;

		float lookAtDistance = remLookAtDistance * ratio;

		position.refValue() = orientation.refValue() * Vec3f(0, 0, lookAtDistance) + remLookAt;

		//g_pointer.translation = g_scene->getViewMatrix() * position;


		//position.refValue() = newPt;
		calcOrientationFromLookAtUp();
		//Vec3f upOffset = orientation.refValue() * Vec3f(0, remUpDistance, 0);
		//up.refValue() = upOffset;

		//g_pointer.translation = g_scene->getViewMatrix() * position;

		//Vec3f upOffset = orientation.refValue() * Vec3f(0, remUpDistance, 0);
		//up.refValue() = upOffset + position;
		//calcOrientationFromLookAtUp();

	}
	
	houseKeep();
}

void TransformObj::dragRot(Seamless* smls, Vec3f pt){

	Vec3f& pivotPoint = this->pivotPoint.refValue();
	//Rotation& rotation = this->rotation.refValue();
	Vec3f& translation = this->translation.refValue();

	Matrix pivotPointMatrix;
	getPivotPointMatrix(pivotPointMatrix, smls);

	int line = g_scene->selLine;

	Vec3f pivotPointVM  = pivotPointMatrix * Vec3f(0,0,0);

	Vec2f toDiff = Vec2f(pt.x - pivotPointVM.x, pt.y - pivotPointVM.y);
	Vec2f fromDiff = Vec2f(remRotCtrlPnt.x - pivotPointVM.x, remRotCtrlPnt.y - pivotPointVM.y);
	float toAngle = toDiff.getAngle();
	float fromAngle = fromDiff.getAngle();

	float angle = toAngle - fromAngle;

	Rotation unwind;
	int i = line;
	do{
		i = g_sceneTreeList[i].parent;
		g_sceneTreeList[i].node->catInverseRotation(unwind);
	}while(i>0);

	Vec3f axis = (-g_scene->rotation * unwind * Vec3f(0, 0, 1)).normalize();
	
	rotCtrlPnt[leverIdx] = pt;
	g_cursorOverPointIlsBase->translation.refValue() = pt;
	rotation.refValue() = remRotation * Rotation(axis, angle);

	calcNonSelectedRotCtrlPnts();
	processFields();

}


const char* commonSpecStr ="\
DEF pelvis CommonSpec{\n\
color [1 .5 .5 .41 .41 1 0 1 1 .4 0 1 .5 .7 0 1 .5 0 1 0 1 1 1 0 0 0 1 0 1 0 1 0 0]\n\
}\n\
";

void Settings::loadINI(){
	#define maxLen 0x200
	//char tmpStr[maxLen+1];
	g_brush = new Brush;
	g_fill = new Fill;
	//g_grid = new Grid;
	//g_grid->xzPlane.refValue() = true;

	g_exportMovie = new ExportMovie;
	g_exportVrmlX3d = new ExportVrmlX3d;
	g_colorPicker = new ColorPicker;

	M_Node mnode;
	if(Browser::createVrmlFromFile(g_setupPath, mnode)){
		for(int k=0; k<mnode.length; k++){
			switch(mnode[k]->getType()){
				case SETTINGS:	 g_settings = dynamic_cast<Settings*>(mnode[k].p); break;
				case OPENNETFILE:	g_openNetFile = dynamic_cast<OpenNetFile*>(mnode[k].p); break;
				case PASTEOPTIONS:	g_pasteOptions = dynamic_cast<PasteOptions*>(mnode[k].p); break;
				case DELETEOPTIONS:	g_deleteOptions = dynamic_cast<DeleteOptions*>(mnode[k].p); break;
				case TIMELINEFILTER: g_timelineFilter = dynamic_cast<TimelineFilter*>(mnode[k].p); break;
			}
		}
	}
	else{
		g_settings = new Settings;
		//strcpy(tmpStr,"seamless3dSetup.wrl");
		//_fullpath( tmpStr, "seamless3dSetup.wrl", maxLen );
		g_settings->writeINIFile();
		g_createdINI = true;
	}
	if(g_settings->palette == 0){
		g_settings->palette = new Palette;
	}
	if(g_settings->win3dPos.x < 10){
		g_settings->win3dPos.refValue().x = 0;
	}
	if(g_settings->win3dPos.y < 10){
		g_settings->win3dPos.refValue().y = 0;
	}
	if(g_settings->win3dSize.x < 400){
		g_settings->win3dSize.refValue().x = 640;
	}
	if(g_settings->win3dSize.y < 300){
		g_settings->win3dSize.refValue().y = 480;
	}
	g_palette = dynamic_cast<Palette*>(g_settings->palette.p);
	if(g_settings->commonSpec.length == 0){
		createSmlsFromString(g_settings->commonSpec.refValue(), g_loadStr = commonSpecStr);
	}

	if(g_settings->file.length){
		String drive;
		String dir;
		String fname;
		String ext;
		splitPath(g_settings->file[0], drive, dir, fname, ext);
		if(drive.length){
			g_seamlessDir = drive;
			g_seamlessDir += dir;
		}
		else{
			g_settings->file.refValue().length = 0;
		}
		chdir(g_seamlessDir.c_str());
	}

	g_writeGzipped=g_settings->saveGZipped.refValue();

	g_settings->version.refValue()=g_version;
	g_settings->freeCPU = g_settings->freeCPU.refValue();


	

/*	const M_Int& win=g_settings->win;
	if(g_settings->win.length){

		g_3dwinSize.x=win[THREE_D_POS_X];
		g_3dwinSize.y=win[THREE_D_POS_Y];
		g_3dwinPos.x=win[THREE_D_CX];
		g_3dwinPos.y=win[THREE_D_CY];

		g_toolbarPos.x=win[TOOLBAR_POS_X];
		g_toolbarPos.y=win[TOOLBAR_POS_Y];
		//g_toolbarSize.x=win[TOOLBAR_CX];
		//g_toolbarSize.y=win[TOOLBAR_CY];

		g_sceneTreePos.x=win[SCENETREE_POS_X];
		g_sceneTreePos.y=win[SCENETREE_POS_Y];
		g_sceneTreeSize.x=win[SCENETREE_CX];
		g_sceneTreeSize.y=win[SCENETREE_CY];

		g_nodeWinPos.x=win[NODE_POS_X];
		g_nodeWinPos.y=win[NODE_POS_Y];

	}
*/
}

/*
void localPermanentTransform(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRawPtr());
	Seamless* smls=part->smls;
	BackUp::monostable.beginCycle();
	part->geometry->scale(part->scale.refValue());
	part->geometry->rotate(part->rotation.refValue());
	part->geometry->translate(part->translation.refValue());
	part->scale=Vec3f(1,1,1);
	part->rotation=Rotation(0,1,0,0);
	part->translation=Vec3f(0,0,0);
	smls->convertToRenderFormat();
}
void permanentTransform(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRawPtr());
	Seamless* smls=part->smls;
	BackUp::monostable.beginCycle();

	IndexedFaceSet* cpy=new IndexedFaceSet;
	Matrix matrix;
	matrix=scaleMatrix(g_scene->scale);
	matrix=matrix*translationMatrix(g_scene->translation);
	matrix=matrix*rotationMatrix(g_scene->rotation);
	CoordinatePtr(part->geometry->coord)->point.refValue().multMatrixLeft(matrix);
	NormalPtr(part->geometry->normal)->vector.refValue().multMatrixLeft(rotationMatrix(g_scene->rotation));

	g_scene->scale=Vec3f(1,1,1);
	g_scene->translation=Vec3f(0,0,0);
	g_scene->rotation=Rotation();
	smls->convertToRenderFormat();
}
/*
void Art::genUndoFilename(){
	char buf [0x200];
	sprintf(buf,"%s%i_",name.getArray(),undoUniqueNameCntr++);
	undoName=buf;
	backupToCache();
	saved=true;
}
void Art::genCacheUrl(String& str ,int cacheEntry){
	str=g_undoDir;
	str+=undoName;
	str+=cacheEntry;
	str+=".wrl";
}

void Art::backupToCache(){
	String backUpFileName;
	if(geometry==NULL)return; //never NULL the way it is so this line could be removed
	saved=false;
	setTitleName();
	highestEntry=cacheEntry;
	genCacheUrl(backUpFileName, cacheEntry++);
	writePart(backUpFileName);
}
void unDo(){
	ArtPtr pj=g_selNode;
	String backUpFileName;
	if(pj->cacheEntry<2)return;

	pj->genCacheUrl(backUpFileName ,--pj->cacheEntry-1);
	pj->loadGZippedPart(backUpFileName);

	if(pj->geometry->coord!=NULL){
		if(g_seamless->ActCtrlPnt>=pj->geometry->coord->point.length||g_seamless->ActTri>=pj->geometry->coordIndex.length/3){
			g_seamless->ActCtrlPnt=0;
			g_seamless->ActTri=0;
		}
	}
	g_seamless->convertToRenderFormat();
}
void reDo(){
	ArtPtr pj=g_selNode;
	String backUpFileName;
	if(pj->cacheEntry>pj->highestEntry)
		return;

	pj->genCacheUrl(backUpFileName, pj->cacheEntry++);
	pj->loadGZippedPart(backUpFileName);

	if(g_seamless->ActCtrlPnt>=pj->geometry->coord->point.length||g_seamless->ActTri>=pj->geometry->coordIndex.length/3){
		g_seamless->ActCtrlPnt=0;
		g_seamless->ActTri=0;
	}
	g_seamless->convertToRenderFormat();
}
*//*
void transformActiveVector(){
	Art* part=dynamic_cast<Art*>(g_selNode.getRawPtr());
	Matrix matrix = part->matrix;

	CoordinatePtr(pj->geometry->coord)->point.refValue().multMatrixLeft(matrix);
	NormalPtr(pj->geometry->normal)->vector.refValue().multMatrixLeft(rotationMatrix(g_scene->rotation));
}
void unTransformActiveVector(){
	ArtPtr pj=g_selNode;
	Matrix matrix;
	NormalPtr(pj->geometry->normal)->vector.refValue().multMatrixLeft(rotationMatrix(-g_scene->rotation));
	matrix=rotationMatrix(-g_scene->rotation);
	matrix=matrix*translationMatrix(-g_scene->translation);
	matrix=matrix*scaleMatrix(Vec3f(1,1,1)/g_scene->scale);
	CoordinatePtr(pj->geometry->coord)->point.refValue().multMatrixLeft(matrix);
}
*/

void modifyMirrorVertexX(ArtPtr selPart){
	/*
	if(!g_mirrorVertexModeX||!g_mirrorModeVertexFound)return;
	SeamlessPtr smls=selPart->smls;
	M_Vec3f& point=smls->refPoint();
	M_Vec3f& renderPoint=smls->renderIfs->refPoint();

	Vec3f svec = selPart->matrix*point[smls->ActCtrlPnt];
	svec.x=-svec.x;
	point[g_mirrorVertex]=svec-g_mirrorPart->matrix*Vec3f();

	renderPoint[g_mirrorVertex]=g_mirrorPart->matrix*point[g_mirrorVertex];
	*/
}
/*
void snapSinglePointToRem(){
	ArtPtr part=g_selNode;
	Seamless* smls=part->smls;
	M_Vec3f& coord=part->coord.refValue();

	Vec3f svec = remPJ->matrix*remPJ->coord.refValue()[smls->remActCtrlPnt];
	coord[smls->ActCtrlPnt]=svec-part->matrix*Vec3f();

	smls->convertToRenderFormat();
	makePointerEqualIndexLocation();
}
/*
void Seamless::joinVertex(){
	ArtPtr selPart=selectPartFromVertex(this, ActCtrlPnt);
	ArtPtr remPart=selectPartFromVertex(this, remActCtrlPnt);
	M_Int &coordIndex = refCoordIndex();
	
	int i,j;
	//check to see if a point is being joined to another point within the same triangle
	for( i=0; i<coordIndex.length; i++){
		if(coordIndex[i]==ActCtrlPnt){
			j=i/4*4;
			if( coordIndex[j+0]==remActCtrlPnt||
				coordIndex[j+1]==remActCtrlPnt||
				coordIndex[j+2]==remActCtrlPnt
				){
				msG(String("you have been prevented from joining 2 points together that are both from the same triangle"));
				return;
			}
		}
	}

	int from=ActCtrlPnt;
	int to=remActCtrlPnt;

	for(i=0;i<coordIndex.length;i+=4){
		if(coordIndex[i]==ActCtrlPnt){
			coordIndex[i]=remActCtrlPnt;
		}
		if(coordIndex[i+1]==ActCtrlPnt){
			coordIndex[i+1]=remActCtrlPnt;
		}
		if(coordIndex[i+2]==ActCtrlPnt){
			coordIndex[i+2]=remActCtrlPnt;
		}
	}
	deleteVertex(ActCtrlPnt);

	ActCtrlPnt=remActCtrlPnt;
	makePointerEqualIndexLocation(this);
	convertToRenderFormat();
}
*/
ArtPtr selectPartFromVertex(SeamlessPtr smls, int targetVertex){
	for(int i=0;i<smls->partList.length;i++){
		ArtPtr part=smls->partList[i];
		if( targetVertex>=part->fromCoord&&targetVertex<part->toCoord){
			return part;
		}
	}
	throw;
}
void Art::compileTransformLoop(Matrix matrix, Rotation accumRot){

	M_Vec3f& coord = this->coord.refValue();

	pivotPoint.refValue() += translation;

	if(accumRot.angle != 0 || rotation.refValue().angle != 0){
		compileOrientationInterpolator(this, accumRot, g_sceneTreeList);
	}
	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint+translation) * matrix;

	for(int i=coord.length; --i>=0; ){
		coord[i] *= matrix;
	}

	if(translation != Vec3f()){
		Matrix negTrandMatrix = translationMatrix(-translation);
		compileScalePositionInterpolator(this, negTrandMatrix);
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = Rotation();
	translation.refValue() = Vec3f();

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileTransformLoop(matrix, accumRot);
	}
}

void Art::compileTransformLoopAnimCP(Matrix matrix, Rotation accumRot){
	//pivotPoint.refValue() = accumRot * pivotPoint;//works great but does this break anything (if left uncommented) ?
	pivotPoint.refValue() += translation;

	if(accumRot.angle != 0 || rotation.refValue().angle != 0){
		compileOrientationInterpolator(this, accumRot, g_sceneTreeList);
	}
	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint+translation) * matrix;

	for(int i = controlPoint.length; --i>=0; ){
		NurbsPatch* patch = controlPoint[i].p;
		int idx = controlPoint[i].i;
		float w = patch->weight[idx];
		Vec3f vec = patch->tpoint[idx] / w;
		vec *= matrix;
		patch->tpoint[idx] = vec * w;
	}

	if(translation != Vec3f()){
		Matrix negTrandMatrix = translationMatrix(-translation);
		compileScalePositionInterpolator(this, negTrandMatrix);
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = Rotation();
	translation.refValue() = Vec3f();

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileTransformLoopAnimCP(matrix, accumRot);
	}
}

void compileTransform(){
	Seamless* smls=dynamic_cast<Seamless*>(g_selNode.getRaw());
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	if(smls){
		Matrix matrix= scaleMatrix(smls->scale.refValue())*rotationMatrix(smls->rotation.refValue())*translationMatrix(smls->translation.refValue());
		M_SArt& skeleton=smls->skeleton.refValue();

		if(smls->workAnimCP){
			for(int k=skeleton.length; --k>=0;){
				skeleton[k]->compileTransformLoopAnimCP(matrix, smls->rotation);
			}
		}
		else{
			for(int k=skeleton.length; --k>=0;){
				skeleton[k]->compileTransformLoop(matrix, smls->rotation);
			}
		}
		smls->scale=Vec3f(1,1,1);
		smls->rotation=Rotation();
		smls->translation=Vec3f();
	}
	else if(part){
		smls = part->smls;
		if(smls->workAnimCP){
			part->compileTransformLoopAnimCP(Matrix(), Rotation());
		}
		else{
			part->compileTransformLoop(Matrix(), Rotation());
		}
	}
	else return;

	smls->calcAbsolutePivotPoint();
	smls->catTranslationMatrix();
	smls->ctrf = true;
	smls->catMatrix();
	smls->catIMatrix();
	Window::paintAllControlPanels();
	BackUp::monostable.beginCycle();
}

void Art::compileScaleLoop(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_Rotation& m_rot){
	m_rot[plidx] = rotation;

	M_Vec3f& coord = this->coord.refValue();

	translation.refValue() = scaleRotMatrix * translation;
	pivotPoint.refValue() = scaleRotMatrix * pivotPoint;
	scaleRotMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * scaleRotMatrix;

	Rotation return_rot;
	if(accumRot.angle != 0 || rotation.refValue().angle != 0){

		if(rotation.angle != 0){
			Rotation accumRot2 = rotation * accumRot;
			return_rot = -accumRot2 * accumRot;
		}
	}
	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint) * matrix;

	for(int i=coord.length; --i>=0; ){
		coord[i] *= matrix;
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = return_rot;

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileScaleLoop(matrix, scaleRotMatrix, accumRot, m_rot);
	}
}

void Art::compileScaleLoopAnimCP(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_Rotation& m_rot){
	m_rot[plidx] = rotation;

	translation.refValue() = scaleRotMatrix * translation;
	pivotPoint.refValue() = scaleRotMatrix * pivotPoint;
	scaleRotMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * scaleRotMatrix;

	Rotation return_rot;
	if(accumRot.angle != 0 || rotation.refValue().angle != 0){

		if(rotation.angle != 0){
			Rotation accumRot2 = rotation * accumRot;
			return_rot = -accumRot2 * accumRot;
		}
	}
	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint) * matrix;

	for(int i = controlPoint.length; --i>=0; ){
		NurbsPatch* patch = controlPoint[i].p;
		int idx = controlPoint[i].i;
		float w = patch->weight[idx];
		Vec3f vec = patch->tpoint[idx] / w;
		vec *= matrix;
		patch->tpoint[idx] = vec * w;
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = return_rot;

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileScaleLoopAnimCP(matrix, scaleRotMatrix, accumRot, m_rot);
	}
}

void Art::compileScaleRotBackLoop(Matrix matrix, Matrix rotMatrix, Rotation accumRot, M_Rotation& m_rot){
	M_Vec3f& coord = this->coord.refValue();

	translation.refValue() = rotMatrix * translation;
	pivotPoint.refValue() = rotMatrix * pivotPoint;
	rotMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * rotMatrix;

	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint) * matrix;

	for(int i=coord.length; --i>=0; ){
		coord[i] *= matrix;
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = m_rot[plidx];

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileScaleRotBackLoop(matrix, rotMatrix, accumRot, m_rot);
	}
}

void Art::compileScaleRotBackLoopAnimCP(Matrix matrix, Matrix rotMatrix, Rotation accumRot, M_Rotation& m_rot){

	translation.refValue() = rotMatrix * translation;
	pivotPoint.refValue() = rotMatrix * pivotPoint;
	rotMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * rotMatrix;

	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint) * matrix;

	for(int i = controlPoint.length; --i>=0; ){
		NurbsPatch* patch = controlPoint[i].p;
		int idx = controlPoint[i].i;
		float w = patch->weight[idx];
		Vec3f vec = patch->tpoint[idx] / w;
		vec *= matrix;
		patch->tpoint[idx] = vec * w;
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = m_rot[plidx];

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileScaleRotBackLoopAnimCP(matrix, rotMatrix, accumRot, m_rot);
	}
}

/*
void Art::compileScaleLoop(Matrix matrix, Matrix scaleRotMatrix, Matrix unRotMatrix){
	Matrix tmpMtx=matrix*unRotMatrix;
	translation.refValue()*=tmpMtx;

	M_Vec3f& coord=this->coord.refValue();
	matrix= scaleMatrix(scale)*rotationMatrix(rotation)*matrix;
	unRotMatrix=unRotMatrix*rotationMatrix(-rotation);
	tmpMtx=matrix*unRotMatrix;

	for(int i=coord.length; --i>=0; ){
		coord[i]*=tmpMtx;
	}
	scale=Vec3f(1,1,1);
//	compileScalePositionInterpolator(this, tmpMtx);

	M_SArt& children=part.refValue();
	for(int k=0; k<children.length; k++){
		children[k]->compileScaleLoop(matrix, scaleRotMatrix, unRotMatrix);
	}
}
*/

void compileScale(){
	Seamless* smls=dynamic_cast<Seamless*>(g_selNode.getRaw());
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	M_Rotation m_rot;

	if(smls){
		m_rot.length = smls->partList.length;

		Vec3f sc=smls->scale;
		Matrix matrix= scaleMatrix(sc)*rotationMatrix(smls->rotation);
		Matrix unRotMatrix= rotationMatrix(-smls->rotation);

		smls->scale=Vec3f(1,1,1);

		M_SArt& skeleton=smls->skeleton.refValue();
		if(smls->workAnimCP){
			for(int k=skeleton.length; --k>=0;){
				skeleton[k]->compileScaleLoopAnimCP(matrix, matrix, smls->rotation, m_rot);
				smls->calcAbsolutePivotPoint();
				skeleton[k]->compileScaleRotBackLoopAnimCP(unRotMatrix, unRotMatrix, -smls->rotation, m_rot);
			}
		}
		else{
			for(int k=skeleton.length; --k>=0;){
				skeleton[k]->compileScaleLoop(matrix, matrix, smls->rotation, m_rot);
				smls->calcAbsolutePivotPoint();
				skeleton[k]->compileScaleRotBackLoop(unRotMatrix, unRotMatrix, -smls->rotation, m_rot);
			}
		}
	}
	else if(part){
		smls = part->smls;
		m_rot.length = smls->partList.length;
		if(smls->workAnimCP){
			part->compileScaleLoopAnimCP(Matrix(), Matrix(), Rotation(), m_rot);
			part->smls->calcAbsolutePivotPoint();
			part->compileScaleRotBackLoopAnimCP(Matrix(), Matrix(), Rotation(), m_rot);
		}
		else{
			part->compileScaleLoop(Matrix(), Matrix(), Rotation(), m_rot);
			part->smls->calcAbsolutePivotPoint();
			part->compileScaleRotBackLoop(Matrix(), Matrix(), Rotation(), m_rot);
		}
	}
	else return;
	smls->calcAbsolutePivotPoint();
	smls->catTranslationMatrix();
	smls->ctrf = true;
	smls->catMatrix();
	smls->catIMatrix();
	Window::paintAllControlPanels();
	BackUp::monostable.beginCycle();
}
/*
void compileScaleRotationLoop(Art* part, Matrix matrix, Rotation accumRot){
	part->translation.refValue()*=matrix;

	M_Vec3f &coord=part->coord.refValue();
	matrix= scaleMatrix(part->scale)*rotationMatrix(part->rotation)*matrix;
	accumRot=part->rotation.refValue()*accumRot;
	//matrix= scaleMatrix(part->scale)*rotationMatrix(accumRot);
	

	for(int i=coord.length; --i>=0; ){
		coord[i]*=matrix;
	}
//	Vec3f v = accumRot*Vec3f(1,0,0);
	//v = Rotation(0,0,1,PI);
//	Rotation r = Rotation (Vec3f(0,0,0), v);
	compileOrientationInterpolator(part, accumRot);

	part->scale=Vec3f(1,1,1);
	part->rotation=Rotation();

	M_SArt& children=part->part.refValue();
	for(int k=0; k<children.length; k++){
		compileScaleRotationLoop(children[k], matrix, accumRot);
	}
}
*/
void Art::compileScaleRotationLoop(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_<SceneTreeListItem>& sceneTreeList){

	M_Vec3f& coord = this->coord.refValue();

	translation.refValue() = scaleRotMatrix * translation;
	pivotPoint.refValue() = scaleRotMatrix * pivotPoint;
	scaleRotMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * scaleRotMatrix;

	if(accumRot.angle != 0 || rotation.refValue().angle != 0){
		compileOrientationInterpolator(this, accumRot, sceneTreeList);
	}
	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint) * matrix;

	for(int i=coord.length; --i>=0; ){
		coord[i] *= matrix;
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = Rotation();

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileScaleRotationLoop(matrix, scaleRotMatrix, accumRot, sceneTreeList);
	}
}

void Art::compileScaleRotationLoopAnimCP(Matrix matrix, Matrix scaleRotMatrix, Rotation accumRot, M_<SceneTreeListItem>& sceneTreeList){

	translation.refValue() = scaleRotMatrix * translation;
	pivotPoint.refValue() = scaleRotMatrix * pivotPoint;
	scaleRotMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * scaleRotMatrix;

	if(accumRot.angle != 0 || rotation.refValue().angle != 0){
		compileOrientationInterpolator(this, accumRot, sceneTreeList);
	}
	accumRot = rotation.refValue() * accumRot;
	
	matrix = translationMatrix(-absolutePivotPoint) * scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(absolutePivotPoint) * matrix;

	for(int i = controlPoint.length; --i>=0; ){
		NurbsPatch* patch = controlPoint[i].p;
		int idx = controlPoint[i].i;
		float w = patch->weight[idx];
		Vec3f vec = patch->tpoint[idx] / w;
		vec *= matrix;
		patch->tpoint[idx] = vec * w;
	}

	scale.refValue() = Vec3f(1,1,1);
	rotation.refValue() = Rotation();

	for(int k=0;k<part.length;k++){
		part.refValue()[k]->compileScaleRotationLoopAnimCP(matrix, scaleRotMatrix, accumRot, sceneTreeList);
	}
}

void compileScaleRotation(Seamless* smls, M_<SceneTreeListItem>& sceneTreeList){
	Vec3f sc=smls->scale;
	Matrix matrix= scaleMatrix(sc)*rotationMatrix(smls->rotation);

	smls->scale=Vec3f(1,1,1);
	smls->rotation=Rotation();

	M_SArt& skeleton=smls->skeleton.refValue();
	if(smls->workAnimCP){
		for(int k = skeleton.length; --k>=0;){
			skeleton[k]->compileScaleRotationLoopAnimCP(matrix, matrix, Rotation(), sceneTreeList);
		}
	}
	else{
		for(int k = skeleton.length; --k>=0;){
			skeleton[k]->compileScaleRotationLoop(matrix, matrix, Rotation(), sceneTreeList);
		}
	}
}


void compileScaleRotation(){
	Seamless* smls=dynamic_cast<Seamless*>(g_selNode.getRaw());
	Art* part=dynamic_cast<Art*>(g_selNode.getRaw());
	if(smls){
		compileScaleRotation(smls, g_sceneTreeList);
	}
	else if(part){
		smls = part->smls;
		if(smls->workAnimCP){
			part->compileScaleRotationLoopAnimCP(Matrix(), Matrix(), Rotation(), g_sceneTreeList);
		}
		else{
			part->compileScaleRotationLoop(Matrix(), Matrix(), Rotation(), g_sceneTreeList);
		}
	}
	else return;

	smls->calcAbsolutePivotPoint();
	smls->catTranslationMatrix();
	smls->ctrf = true;
	smls->catMatrix();
	smls->catIMatrix();
	Window::paintAllControlPanels();
	BackUp::monostable.beginCycle();
}


void listRoutes(String&);
bool writeSmlsFile( String filename){
	g_scene->preSaveProcessCollapse();
	Window::preSaveProcessWin(*g_scene);

	changeFloatFormat(g_formatCoords=g_settings->roundCoordsSave.getValue());
	g_scene->version=g_version;

	static String str;
	str="#VRML V2.0 utf8\n";
	g_scene->toString(str);

	listRoutes(str);


	if(str.write(filename)==false){
		msG("\
*ERROR*\nFailed to save file!!!!\n\
This can happen if the file has been opened from a computer on the network and the computer\n\
has been turned off or if there is a file write protected.\n\
Try saving the file with a new name in a folder that exists\n\
or try deleting all files in your Recover folder which is located in:\n\
My Documents\\Seamless3d\\Application Data For Seamless3d");
		return false;
	}
	return true;
}
void commonEndCCVisualClue(){
	g_vcSeamless->process();
	g_vcSeamless->doBuild();
	g_vcSeamless->convertToRenderFormat();
	g_visualClue=g_vcSeamless;
}

void fillPolyLineIndex(M_Vec3f& point, M_Int& coordIndex){
	coordIndex.length=point.length;
	for(int i=0; i<point.length; i++){
		coordIndex[i]=i;
	}
}
void fillPolyLineIndex(M_Vec3f& point, int markPoint, M_Int& coordIndex){
	//coordIndex+=-1;
	int mark = coordIndex.length;
	int pointsToAdd = point.length - markPoint;
	coordIndex.length+=pointsToAdd;
	for(int i = 0; i < pointsToAdd; i++){
		coordIndex[mark + i] = markPoint + i;
	}
}

// draw a circle starting at 'startAng' angle and rotate through 'deltaAng' radians 
void circle(M_Vec3f &coord, Vec2f centerPos, double radius, double startAng, double deltaAng)
{
   int numPoints = max(2.0, 100 * (fabs(deltaAng) / (2 * PI)));
   double delta = deltaAng / numPoints;
   double angle = startAng;
   for ( int i = 0; i <= numPoints; i++ )
   {     
	  coord += Vec3f(radius * cos(angle) + centerPos.x, radius * sin(angle) + centerPos.y, 0);
	  angle += delta;
   }
}

// length of angle control point stick
const float CCLathe::controlLen = 0.2;

void CCLathe::calcSegments()
{
   seg.length = 0;

   if ( curve.length == 0 )
   {
	  return;
   }

   Vec2f P = Vec2f(baseRad, 0);
   double a = curve[0];

   seg += CCSegment(Vec2f(0, 0), P, P, a);

   for ( int i = 1; i < curve.length; i++ ){
	  double r = (curve[i] == 0) ? distance[i - 1] : distance[i - 1] / curve[i];
	  Vec2f Ro = Vec2f(r, 0).rotate(a);
	  Vec2f C  = P - Ro;

	  a += curve[i];
	  Vec2f Rf = Vec2f(r, 0).rotate(a);        
	  P += Rf - Ro;

	  seg += CCSegment(C, Ro, P, curve[i]);
   }
}

void CCLathe::setPoint(int idx, Vec2f point)
{

   setMathHiPresision();

   if ( idx == 0 ) {
	  baseRad = max(0.0f, point.x);
   }
   else if ( idx < curve.length ) {
	  Vec2f Po = Vec2f(baseRad, 0);  // point at start of arc
	  double ao = curve[0];

	  for ( int i = 1; i < idx; i++ ){
		 double r = (curve[i] == 0) ? distance[i - 1] : distance[i - 1] / curve[i];

		 Vec2f Ro = Vec2f(r, 0).rotate(ao);
		 ao += curve[i];
		 Vec2f Rf = Vec2f(r, 0).rotate(ao);  
		 Po += Rf - Ro;
	  }

	  float zoom = g_scene->getZoom();
	  bool shift = shiftKeyState();
	  double ed = shift ? 0.03 / zoom : 1e-5;
	  double ea = shift ? 0.03 : 1e-5;

	  // normalize so P1 is at the origin and the start tangent is along the x-axis
	  Vec2f Pf = (point - Po).rotate(-(ao + PI / 2));

	  // calc angle and radius of arc
	  double a = curve[idx];
	  double d = distance[idx - 1];

	  if ( abs(Pf.y) < ed && Pf.x > 1e-2 && abs(a) < ea) {
		 a = 0.0001;
		 d = fabs(Pf.x); 
	  }
	  else if ( abs(Pf.y) > 1e-6 ) {
		 double r = (Pf.y + Pf.x * Pf.x / Pf.y) / 2;
		 Vec2f C  = Vec2f(0, r);
		 Vec2f T2 = (Pf - C).rotate(PI / 2);
		 a = T2.angle();

		 if ( C.y < 0 ){
			a -= PI;
		 }
		 else if ( Pf.x < 0 ){
			a += 2 * PI;
		 }

		 d = abs(a * r); 
	  }
	  else {
		 restoreMathPresision();
		 return;  // limit of precision - no work around
	  }

	  curve.refValue()[idx] = a;
	  distance.refValue()[idx - 1] = d; 
   }

   restoreMathPresision();
}

void CCLathe::houseKeep()
{

   setMathHiPresision();
   calcSegments();

   float inverseSize = (1 / g_scene->getZoom());
   int idx = index[0]; 
   IndexedLineSet* polyline = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
   M_Vec3f& coord=dynamic_cast<Coordinate*>(polyline->coord.p)->point;
   M_Node& children = g_vcCCLatheBase->children;
   children.length = 1; // IndexedLineSet
   coord.length = 0;

   // draw the cc control curve visual clue 
   float crossLen = controlLen * inverseSize * 0.2;
   float senseLen = g_mouseSense * inverseSize * 1.75;

   coord += Vec3f(0, 0, 0);
   coord += Vec3f(0, -crossLen, 0);
   coord += Vec3f(0, +crossLen, 0);
   coord += Vec3f(0, 0, 0);
   coord += Vec3f(-crossLen, 0, 0);
   coord += Vec3f(+crossLen, 0, 0);
   circle(coord, Vec2f(0, 0), senseLen, PI, 2 * PI);

   coord += Vec3f(baseRad, 0, 0);
   coord += Vec3f(baseRad, -crossLen / 2, 0);
   coord += Vec3f(baseRad, +crossLen / 2, 0);
   coord += Vec3f(baseRad, 0, 0);

   for ( int i = 0; i < seg.length; i++ )
   {
	  if ( i == 0 ) {
		 if ( i == idx ) {
			anglePt = Vec2f(controlLen * inverseSize, 0).rotate(seg[i].a );
			anglePt += seg[i].Pf;
			coord += Vec3f( seg[i].Pf.x, seg[i].Pf.y, 0);
			coord += Vec3f( anglePt.x, anglePt.y, 0); 
			coord += Vec3f( seg[i].Pf.x, seg[i].Pf.y, 0);
		 }
	  }
	  else {
		 circle(coord, seg[i].C, seg[i].Ro.length(), seg[i].Ro.angle(), seg[i].a);
		 if ( i == idx ){
			anglePt = seg[i].Pf - seg[i].C;
			double a = anglePt.angle() / PI * 4;
			anglePt *= (controlLen * inverseSize / max(1e-9, static_cast<double>(anglePt.length())));
			anglePt += seg[i].Pf;

			coord += Vec3f( seg[i].C.x, seg[i].C.y, 0);
			if ( fabs(a - floor(a + 0.5)) < 0.0001 ) {
			   Vec2f T = (seg[i].Pf - seg[i].C).rotate(PI/2);
			   T *= senseLen * 2 / max(1e-5, static_cast<double>(T.length()));
			   coord += Vec3f( seg[i].C.x + T.x, seg[i].C.y + T.y, 0);
			   coord += Vec3f( seg[i].C.x - T.x, seg[i].C.y - T.y, 0);
			   coord += Vec3f( seg[i].C.x, seg[i].C.y, 0);
			}
			circle(coord, seg[i].C, senseLen, seg[i].Ro.angle() + seg[i].a, 2 * PI);

			coord += Vec3f( anglePt.x, anglePt.y, 0);
			coord += Vec3f( seg[i].Pf.x, seg[i].Pf.y, 0);
		 }
	  }
   }
   fillPolyLineIndex(coord, polyline->coordIndex.refValue());

   // setup the control points
   if ( seg.length > 0 ){
	  int childDiff = (seg.length + 1) - children.length + 1;
	  if(childDiff<0){
		 children.length+=childDiff;
	  }
	  else{
		 for(int i=childDiff; --i>=0;){
			Transform* transform=new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		 }
	  }

	  for(int i=seg.length; --i>=0;){
		 Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
		 transform->translation = Vec3f(seg[i].Pf.x, seg[i].Pf.y, 0);

		 if(i==idx){
			transform->children.refValue()[0]=g_redPoint;
		 }
		 else{
			if( i == 0 || (i == seg.length - 1 && autoClose) ){
			   transform->children.refValue()[0]=g_cyanPoint;
			}
			else{
			   transform->children.refValue()[0]=g_nextPoint;
			}
		 }
		 transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	  }

	  // angle-drag control point
	  Transform* transform=dynamic_cast<Transform*>(children[seg.length + 1].p);
	  transform->translation = Vec3f(anglePt.x, anglePt.y, 0);
	  if ( angleDrag ) {
		 transform->children.refValue()[0]=g_redPoint;
	  }
	  else{
		 transform->children.refValue()[0]=g_nextPoint;
	  }
	  transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
   }

   g_visualClue = g_vcCCLatheBase;

   restoreMathPresision();
//   g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
   polyline->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void CCScale::houseKeep(){

	if(!distance.length){
		return;
	}
	int index=this->index[0];
	if(index==0){
		index=1;
	}

	g_vcCCLathe->distance.refValue()[0]=distance[index-1];
	if(index<=1){
		g_vcSurfaceTransform->translation=Vec3f();
	}
	else {
		g_vcSurfaceTransform->translation=Vec3f(0,getDistance(index-2),0);
	}

	g_vcCCBend->distance.resize(distance.length);
	g_vcCCBend->yAngle.resize(curve.length);
	g_vcCCBend->curve.resize(curve.length);
	g_vcCCBend->distance.refValue()=distance;
	g_vcCCBend->curve.refValue()=curve;

	g_vcCCLathe->baseRad.refValue()=g_spaghettiRad*(1/g_scene->getZoom());

	g_vcBaseRad->translation=Vec3f(baseRad+location.x,location.y,location.z);
	g_vcBaseRad->rotation=orientation;
	commonEndCCVisualClue();
}
void rotateZ(Vec2f *p,int numberOf,float angle);
void genSegment(	long double  pt1x,long double  pt1y,
		long double  ang1z,
		long double  pt2x,long double  pt2y,
		long double  ang2z,
		long double a, long double b,
		Vec3f* v,int step){

	long double x,y,xyt,t,j;

	x=pt2x-pt1x;
	y=pt2y-pt1y;

	rotateZ(x, y, ang1z);

	j=y/b;
	if(j>1)j=1;
	xyt=asinl(j);

	Matrix3 matrix=rotationMatrix(ang1z-PI*.5)*translationMatrix(Vec2f(pt1x,pt1y));
	for(int i=0;i<step;i++){
		t=xyt/step*i;
		Vec2f vec2f=matrix*Vec2f(a*cosl(t)-a, b*sinl(t));
		v[i]=Vec3f(vec2f.x,vec2f.y,0);
	}
}


void EC2::genCurve(M_Vec3f& line){

	int step=32;
	int pointLen=point.length;
	line.length=(pointLen-1)*step+1;
	int offset=0;
	int i=0;
	for(; i<pointLen-1; i++){
		genSegment(
			point[i].x,point[i].y,tangent[i],
			point[i+1].x,point[i+1].y,tangent[i+1],
			a[i], b[i],
			&line[offset],step
		);
		offset+=step;
	}
	line[offset]=Vec3f(point[i].x,point[i].y,0);
}

void ECBend::genCurve(M_Vec3f& line){

	int step=32;
	int pointLen=point.length;
	line.length=(pointLen-1)*step+1;
	int offset=0;
	int i=0;
	for(; i<pointLen-1; i++){
		genSegment(
			point[i].x,point[i].y,tangent[i],
			point[i+1].x,point[i+1].y,tangent[i+1],
			a[i], b[i],
			&line[offset],step
		);
		offset+=step;
	}
	line[offset]=Vec3f(point[i].x,point[i].y,0);
}
void BezierPatch::houseKeep(){
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();

	if(g_visualClue == g_vcSelectRectGroup){
		IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcSelectRectShape->geometry.p);
		M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

		coord.length=0;
		coord+=g_scene->getIViewMatrix() * Vec3f(MultiSelect::topLeft.x, MultiSelect::bottomRight.y, MultiSelect::topLeft.z);
		coord+=g_scene->getIViewMatrix() * MultiSelect::bottomRight;
		coord+=g_scene->getIViewMatrix() * Vec3f(MultiSelect::bottomRight.x, MultiSelect::topLeft.y, MultiSelect::topLeft.z);
		coord+=g_scene->getIViewMatrix() * MultiSelect::topLeft;
		fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
		polyLine->coordIndex.refValue()+=0;
		polyLine->convertToRenderFormat(g_vcSelectRectShape->render);
	}
	else{
		float inverseSize=(1 / g_scene->getZoom());
		int idx = index[0];

		IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

		M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

		coord = point;

		M_Int& coordIndex = polyLine->coordIndex.refValue();
		coordIndex.length=0;

		int numOfUGroupIdx=index[numOfUGroup];
		int bp=0;

		for(int k=0; k < dim.refValue().length; k++){
			int numOfU=this->dim.refValue()[k].x;
			int numOfV=this->dim.refValue()[k].y;

			for(int v=0; v<numOfV; v++){
				for(int u=0; u<numOfU; u++){
					coordIndex+=bp+v*numOfU+u;
				}
				coordIndex+=-1;
			}
			for(int u=0; u<numOfU; u++){
				for(int v=0; v<numOfV; v++){
					coordIndex+=bp+v*numOfU+u;
				}
				coordIndex+=-1;
			}
			bp+=numOfU*numOfV;
		}
	/////////////
		bp=getBasePoint(numOfUGroupIdx);
		{
			int numOfU=this->dim.refValue()[numOfUGroupIdx].x;
			int numOfV=this->dim.refValue()[numOfUGroupIdx].y;
			int pointLen=numOfU*numOfV;

			M_Node& children=g_vcBezierBase->children;
			children.length=1;
			int childDiff=pointLen-children.length+1;

			if(childDiff<0){
				children.length+=childDiff;
			}
			else{
				for(int i=childDiff; --i>=0;){
					Transform* transform=new Transform;
					transform->children.refValue()+=g_nextPoint.p; //could just add null?
					children+=transform;
				}
			}
			for(int i=pointLen; --i>=0;){
				Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
				Vec3f pt=point[bp+i];
				transform->translation=pt;

				if(bp+i==idx){
					transform->children.refValue()[0]=g_redPoint;
				}
				else{
					transform->children.refValue()[0]=g_nextPoint;
				}

				transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
			}
		}

	///////////
		g_vcBezierBase->rotation=Rotation();

		g_visualClue=g_vcBezierBase;
	//	g_scene->children.refValue()+=g_vcBezierBase.p;
	//	g_stw->genNodeList();


		//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
		polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
	}
	restoreMathPresision();
}


void houseKeep2dNurbs(const M_Vec2f& dpoint, const M_Vec2f& point, const M_Float& weight, const M_Float& knot, const int idx, float minT, float maxT, const int degree, float scale, Matrix matrix=Matrix()){
	static M_Vec2f line;
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	const float inverseSize = (1 / g_scene->getZoom());

	const int dim = point.length;

	const int steps = (dim == 2)? 0: 1000;

	line.length = steps;


	
	const Vec2f* p = point.getArray();
	const float* w=weight.getArray();


	for(int i=0; i<steps; i++){
		float t = static_cast<float>(i)/(steps+1);
		t = (maxT-minT)*t + minT;
		line[i]=curvePoint2d(dim-1, degree, knot.getArray(), p, w, t);
	}

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length = line.length + point.length;//normA.length*5;
	for(int i = 0; i < line.length; i++){
		coord[i] = matrix*Vec3f(line[i].x, line[i].y*scale, 0);
	}
	for(int j = line.length, i=point.length; --i>=0; j++){
		coord[j] = matrix * Vec3f(dpoint[i].x, dpoint[i].y*scale, 0);
	}
	fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
/////////////
	M_Node& children = g_vcBezierBase->children;
	children.length = 1;

	int numOfVisPoints = point.length;
	int nurbsType = checkNurbsType(numOfVisPoints, knot);
	if(nurbsType == NurbsPatch::MID_1){
		numOfVisPoints = numOfVisPoints / 2 + 2;
	}
	int childDiff = numOfVisPoints - children.length+1;

	if(childDiff < 0){
		children.length += childDiff;
	}
	else{
		for(int i = childDiff; --i>=0;){
			Transform* transform = new Transform;
			transform->children.refValue() += g_nextPoint.p; //could just add null?
			children += transform;
		}
	}


	for(int i = 0, i2 = 0; i < point.length; i++, i2++){
		Transform* transform = dynamic_cast<Transform*>(children[i2 + 1].p);
		Vec3f pt = matrix * Vec3f(dpoint[i].x, dpoint[i].y * scale, 0);
		transform->translation = pt;

		if(i == idx){
			transform->children.refValue()[0]=g_redPoint;
		}
		else{
			transform->children.refValue()[0]=g_nextPoint;
		}

		transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);

		if(nurbsType == NurbsPatch::MID_1 && i >= 1 && i <= point.length - 3){
			i++;
		}
	}

///////////
	g_vcBezierBase->rotation = Rotation();

	g_visualClue = g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void BendyLathe::dragRadius(Vec3f pt, int idx){
	Vec3f loc = ringBase[idx];
	Rotation orien = ringRot[idx];

	double a = PI/ctrlStaves;
	double b = a*(radIdx*2+1);
	Rotation polar = Rotation(0,1,0,-b);

	Matrix imatrix=translationMatrix(Vec3f(-translation.x, -translation.y, -translation.z)) * rotationMatrix( -rotation) * scaleMatrix(Vec3f(1./scale.x, 1./scale.y, 1./scale.z)) *
		translationMatrix(-loc) * rotationMatrix(-orien) * rotationMatrix(polar);

	Vec3f newPt = g_scene->getIViewMatrix() * imatrix * pt;

//	double distance = abs(sqrt(newPt.x*newPt.x + newPt.y*newPt.y + newPt.z*newPt.z));

//	radius.refValue()[idx] = distance/ratio;
	radius.refValue()[idx] = -newPt.z/ratio;


	Vec3f sr = matrix * g_scene->getViewMatrix() * (ringRot[idx] * Vec3f(-sin(b)*radius[idx]*ratio, 0, -cos(b)*radius[idx]*ratio) + loc);
	g_pointer.translation=sr;
}
void BendyLathe::dragRingBase(Vec3f pt, int idx, M_Vec3f& point, const M_Float& weight, const M_Float& knot, float minT, float maxT, Vec3f& atVec){
	M_Float& at = this->at.refValue();

	const int dim=point.length;
	Matrix windMatrix = matrix * g_viewMatrix;
	const Vec3f* p = point.getArray();
	const float* w=weight.getArray();
	const float* knotPtr = knot.getArray();

	float t = (maxT-minT)*at[idx] + minT;
	double tp = (maxT-minT)*(at[idx]+.0001) + minT;

	Vec3f vb;
	Vec3f vc;

	CurvePoint(dim-1, degree, knotPtr, p, w, t, vb);
	CurvePoint(dim-1, degree, knotPtr, p, w, tp, vc);
	vb*=windMatrix;
	vc*=windMatrix;
	double ang = Vec2f(vc.x-vb.x, vc.y-vb.y).getAngle();

	Vec2f dragDiff = Vec2f(pt.x - vb.x, pt.y - vb.y);

	Vec3f dragDiff3f = Rotation(0,0,1,-ang)*Vec3f(dragDiff.x, dragDiff.y, 0);

	at[idx] += dragDiff3f.x*.1;
	if(at[idx] < 0){
		at[idx] = 0;
	}
	if(at[idx] > 1){
		at[idx] = 1;
	}

	CurvePoint(dim-1, degree, knotPtr, p, w, t, atVec);

	g_pointer.translation = atVec = windMatrix * atVec;
}
void BendyLathe::houseKeep3dNurbs(const M_Vec3f& dpoint, const M_Vec3f& point, const M_Float& weight, const M_Float& knot, const int idx, float minT, float maxT, Matrix matrix, NurbsPatch* patch, int selRadius){
	float scale = 1;
	static M_Vec3f line;
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	const float inverseSize=(1/g_scene->getZoom());

	const int dim=point.length;

	const int steps = (dim == 2)? 0: 1000;

	line.length=steps;

	const Vec3f* p = point.getArray();
	const float* w=weight.getArray();
	Vec3f* linePtr = line.getArray();
	const float* knotPtr = knot.getArray();

	for(int i = 0; i < steps; i++){
		float t = static_cast<float>(i)/(steps+1);
		t = (maxT-minT)*t + minT;
		CurvePoint(dim-1, degree, knotPtr, p, w, t, *(linePtr+i));
	}

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length=line.length+point.length;//normA.length*5;
	for(int i=0; i<line.length; i++){
		coord[i]=matrix*Vec3f(line[i].x, line[i].y*scale, line[i].z);
	}
	for(int j=line.length, i=point.length; --i>=0; j++){
		coord[j]=matrix*Vec3f(dpoint[i].x, dpoint[i].y*scale, dpoint[i].z);
	}
	M_Int& coordIndex = polyLine->coordIndex.refValue();
	fillPolyLineIndex(coord, coordIndex);

/////////////
	M_Node& children = g_vcBezierBase->children;
	children.length = 1;
	if(showRingControlPoints == false){
		int childDiff = point.length - children.length + 1;
		if(childDiff<0){
			children.length += childDiff;
		}
		else{
			for(int i=childDiff; --i>=0;){
				Transform* transform=new Transform;
				transform->children.refValue() += g_nextPoint.p; //could just add null?
				children+=transform;
			}
		}
		for(int i=point.length; --i>=0;){
			Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
			Vec3f pt=matrix*Vec3f(dpoint[i].x, dpoint[i].y * scale, dpoint[i].z);
			transform->translation = pt;

			if(i==idx){
				transform->children.refValue()[0] = g_redPoint;
			}
			else{
				transform->children.refValue()[0] = g_nextPoint;
			}

			transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
		}
	}
	M_Vec3f& patchDpoint = patch->dpoint;
	int numOfU = patch->dim.x;
	int numOfV = patch->dim.y;
	int offset = coord.length;
	Vec3f selPoint = ringBase[selRadius];
	if(showRingControlPoints){
		int childDiff=((numOfU-1)>>1)-children.length+2;
		if(childDiff<0){
			children.length+=childDiff;
		}
		else{
			for(int i = radius.length; --i>=0;){
				Transform* transform=new Transform;
				transform->children.refValue() += g_nextPoint.p; //could just add null?
				children += transform;
			}
		}
		Transform* transform=dynamic_cast<Transform*>(children[1].p);
		transform->children.refValue()=M_Node(g_yellowPoint.p);
		Vec3f pt=matrix*Vec3f(selPoint.x, selPoint.y*scale, selPoint.z);
		transform->translation=pt;
		transform->scale=Vec3f(inverseSize, inverseSize, inverseSize);
	}

	coordIndex += -1;

	for(int r = 0, r2 = 0; r2 < numOfV; r++, r2++){
		for(int u = 0; u < numOfU; u++){
			Vec3f pt = patchDpoint[r2 * numOfU + u];
			if(r == selRadius && (u & 1) && showRingControlPoints){
				Transform* transform = dynamic_cast<Transform*>(children[2 + (u >> 1)].p);
				transform->children.refValue()=M_Node(g_redPoint.p);
				transform->translation = pt;
				transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
			}
			coord += pt;
			coordIndex += offset + r * numOfU + u;
		}
		coordIndex += -1;
		if(r2 > 0 && r2 < numOfV - 2){
			r2++;
		}
	}

///////////
	g_vcBezierBase->rotation = Rotation();

	g_visualClue = g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void Oscillator::houseKeep(){
	houseKeep2dNurbs(fdpoint, note, fWeight, fKnot, index[0], minT(), maxT(), fDegree, .1);
}

void Envelope::houseKeep(){
	houseKeep2dNurbs(point, point, weight, knot, index[0], minT(), maxT(), degree, .1);

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& ilsCoord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	int ci = ilsCoord.length;
	for(int i = 0; i < coord.length; i++){
		ilsCoord+=Vec3f(coord[i].x, 0, 0);
		ilsCoord+=Vec3f(coord[i].x, coord[i].y * .1, 0);
		coordIndex+=ci;
		ci++;
		coordIndex+=ci;
		ci++;
		coordIndex+=-1;
	}
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void AmplitudeModulator::houseKeep(){
	houseKeep2dNurbs(wet, wet, weight, knot, index[0], minT(), maxT(), degree, .1);

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& ilsCoord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	int ci = ilsCoord.length;
	for(int i = 0; i < coord.length; i++){
		ilsCoord+=Vec3f(coord[i].x, 0, 0);
		ilsCoord+=Vec3f(coord[i].x, coord[i].y * .1, 0);
		coordIndex+=ci;
		ci++;
		coordIndex+=ci;
		ci++;
		coordIndex+=-1;
	}
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void FrequencyModulator::houseKeep(){
	houseKeep2dNurbs(wet, wet, weight, knot, index[0], minT(), maxT(), degree, .1);
}

void Chords::houseKeep(){
	houseKeep2dNurbs(spectrum, spectrum, weight, knot, index[0], minT(), maxT(), degree, .1);

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& ilsCoord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	int ci = ilsCoord.length;
	for(int i = 0; i < coord.length; i++){
		//coord[i].y = amplitude[i] * (i + 1.0);
		ilsCoord+=Vec3f(coord[i].x, 0, 0);
		ilsCoord+=Vec3f(coord[i].x, coord[i].y * .1, 0);
		coordIndex+=ci;
		ci++;
		coordIndex+=ci;
		ci++;
		coordIndex+=-1;
	}
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void SphereBuild::houseKeep(){
	if(g_transformCP->selectObject){
		patch->houseKeep();
	}
	else{
		g_visualClue = nullPtr;
	}
}

void CylinderBuild::houseKeep(){
	if(g_transformCP->selectObject){
		patch->houseKeep();
	}
	else{
		g_visualClue = nullPtr;
	}
}

void ConeBuild::houseKeep(){
	if(g_transformCP->selectObject){
		patch->houseKeep();
	}
	else{
		g_visualClue = nullPtr;
	}
}

void TorusBuild::houseKeep(){
	if(g_transformCP->selectObject){
		patch->houseKeep();
	}
	else{
		g_visualClue = nullPtr;
	}
}

void BoxBuild::houseKeep(){
	if(g_transformCP->selectObject){
		m_patch[0]->houseKeep();
	}
	else{
		g_visualClue = nullPtr;
	}
}

void RectangleBuild::houseKeep(){
	if(g_transformCP->selectObject){
		patch->houseKeep();
	}
	else{
		g_visualClue = nullPtr;
	}
}

void AddNurbsLine::houseKeep(){
	setMathHiPresision();
	const float inverseSize = (1 / g_scene->getZoom());
	M_Vec3f& point = line->point.refValue();
	const int dim = point.length;

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	if(dim > 1){

		coord.length = dim;
		for(int i = 0; i < dim; i++){
			coord[i] = point[i];
		}
	}
	else{
		coord.length = 0;
	}


	fillPolyLineIndex(coord, coordIndex);

/////////////
	M_Node& children = g_vcBezierBase->children;
	children.length = 1;
	int childDiff = point.length - children.length + 1;

	int from = 0;
	int to = point.length;
	if(line->uJoinEnds){
		from = 1;
		to--;
	}

	int idx = line->index[0];
	for(int i = from; i < to; i++){
		Transform* transform = new Transform;
		Vec3f pt = point[i];
		transform->translation = pt;

		if(i == idx){
			transform->children.refValue() += g_redPoint.p;
		}
		else{
			transform->children.refValue() += g_nextPoint.p;
		}

		transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);

		children += transform;
	}


///////////

	g_visualClue = g_vcBezierBase;

	restoreMathPresision();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void DrawNurbsLine::houseKeep(){
	if(mousePhase != DrawNurbsLine::DRAW){
		return;
	}
	if(pixelSketch.length <= 1){
		return;
	}
	setMathHiPresision();
	const float inverseSize = (1 / g_scene->getZoom());
	M_Vec3f& point = line->point.refValue();
	const int dim = point.length;

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_rawDrawNurbsLineClue->geometry.p);

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	Color* colorVrml = dynamic_cast<Color*>(polyLine->color.p);
	M_Color3f& ilsColor = dynamic_cast<Color*>(polyLine->color.p)->color;
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	
	coord = pixelSketch;

	fillPolyLineIndex(coord, coordIndex);

/////////////
	M_Node& children = g_rawDrawNurbsLineClueBase->children;
	children.length = 1;
	int childDiff = point.length - children.length + 1;

	if(childDiff < 0){
		children.length += childDiff;
	}
	else{
		for(int i = childDiff; --i>=0;){
			Transform* transform = new Transform;
			transform->children.refValue() += g_nextPoint.p; //could just add null?
			children += transform;
		}
	}
	int idx = point.length - 1;
	for(int i = point.length; --i>=0;){
		Transform* transform = dynamic_cast < Transform*>(children[i + 1].p);
		Vec3f pt = point[i];
		transform->translation = pt;

		if(i == idx){
			transform->children.refValue()[0] = g_redPoint;
		}
		else{
			transform->children.refValue()[0] = g_nextPoint;
		}

		transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	}


///////////
	//Color3f color = g_palette->color;
	//g_vcEllipticalMaterial->emissiveColor = color;
	//g_vcEllipticalMaterial->diffuseColor = color;

	int len = coord.length;
	ilsColor.length = len;

	for(int c = 0; c < len; c++){
		if(colorPressure){
			ilsColor[c] = convertPressureToCol(c);
		}
		else{
			ilsColor[c] = g_palette->color;
		}
	}


	g_visualClue = g_rawDrawNurbsLineClueBase;

	restoreMathPresision();
	polyLine->convertToRenderFormat(g_rawDrawNurbsLineClue->render);
}

void AddLathe::houseKeep(){
//	if(!initPoints.length){
//		g_visualClue = 0;
//		return;
//	}
	//if(mousePhase == CREATE_SPINDLE){
	//	return;
	//}
	setMathHiPresision();
	Matrix matrix = rotationMatrix(lathe->rotation) * translationMatrix(lathe->translation);
	const float inverseSize = (1 / g_scene->getZoom());
	M_Vec2f& point = lathe->point.refValue();
	const int dim = point.length;

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();
	coord.length = dim;
	for(int i = 0; i < dim; i++){
		coord[i] = matrix * Vec3f(point[i].x, point[i].y, 0);
	}

	fillPolyLineIndex(coord, coordIndex);

	Vec3f upSpindlePoint = matrix * Vec3f(0, .5, 0);

	coord += matrix * Vec3f();
	coord += upSpindlePoint;

	if(coordIndex.length){
		coordIndex += -1;
	}
	coordIndex += dim;
	coordIndex += dim + 1;

	if(mousePhase == SET_SPINDLE_ORIENTATION){
		for(int j = 0; j < orientationPointArray.length; j++){
			coord += matrix * orientationPointArray[j];
			coordIndex += -1;
			coordIndex += dim;
			coordIndex += dim + 2 + j;
		}
	}

	if(mousePhase == INITIAL_POINTS){ // add positive symbol
		coordIndex += -1;
		coordIndex += coord.length;
		coord += matrix * Vec3f(.1, 0, 0);
		coordIndex += coord.length;
		coord += matrix * Vec3f(.2, 0, 0);
	
		coordIndex += -1;
		coordIndex += coord.length;
		coord += matrix * Vec3f(.15, -.05, 0);
		coordIndex += coord.length;
		coord += matrix * Vec3f(.15, .05, 0);
	}

/////////////
	M_Node& children = g_vcBezierBase->children;
	children.length = 1;
	int visPointLen = point.length;
	if(visPointLen > 3){
		visPointLen = visPointLen / 2 + 2;
	}
	int childDiff = visPointLen + 1 - children.length;

	if(childDiff < 0){
		children.length += childDiff;
	}
	else{
		for(int i = childDiff; --i>=0;){
			Transform* transform = new Transform;
			transform->children.refValue() += g_nextPoint.p; //could just add null?
			children += transform;
		}
	}
	int idx = point.length - 1;
	for(int i = 0, i2 = 0; i2 < visPointLen; i++, i2++){
		Transform* transform = dynamic_cast < Transform*>(children[i2 + 1].p);
		Vec3f pt = matrix * Vec3f(point[i].x, point[i].y, 0);
		transform->translation = pt;

		if(i == idx){
			transform->children.refValue()[0] = g_redPoint;
		}
		else{
			transform->children.refValue()[0] = g_nextPoint;
		}

		transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);

		if(i >= 1 && i <= point.length - 3){
			i++;
		}

	}

	Transform* transform = new Transform;
	transform->children.refValue() += g_nextPoint.p;
	transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	transform->translation = upSpindlePoint;
	children += transform;

	if(mousePhase == SET_SPINDLE_ORIENTATION){
		for(int j = 0; j < orientationPointArray.length; j++){
			Transform* transform = new Transform;
			transform->children.refValue() += g_nextPoint.p;
			transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
			transform->translation = matrix * orientationPointArray[j];
			children += transform;
		}
	}

///////////

	g_visualClue = g_vcBezierBase;

	restoreMathPresision();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void AddBox::houseKeep(){
}

void AddSphere::houseKeep(){
}

void NurbsLathe::houseKeep(){
	if(g_transformCP->selectObject){
		patch->houseKeep();
	}
	else{
		houseKeep2dNurbs(dpoint, point, weight, knot, index[0], minT(), maxT(), degree, 1, matrix);
	}

	if(g_addLathe->mousePhase != AddLathe::NEUTRAL){
		g_addLathe->houseKeep();
	}
}

void BendyLathe::houseKeep(){
	if(g_transformCP->selectObject){
		patch->houseKeep();
	}
	else{
		houseKeep3dNurbs(dpoint, point, weight, knot, index[0], minT(), maxT(), matrix, patch, index[2]);
	}
}



void NurbsTrack::houseKeep(){
	int idx = index[1];

	static M_Vec3f line;
	if(!keyValue.length){
		g_visualClue=0;
		return;
	}

	setMathHiPresision();
	line.length = 0;

	genCurve(line, 1000);

	const float inverseSize=(1 / g_scene->getZoom());
	
	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length = line.length + keyValue.length;
	for(int i = 0; i < line.length; i++){
		coord[i] = line[i];
	}
	for(int j = line.length, i = keyValue.length; --i >= 0; j++){
		coord[j] = keyValue[i];
	}
	M_Int& coordIndex = polyLine->coordIndex.refValue();
	fillPolyLineIndex(coord, coordIndex);

/////////////
	coordIndex += -1;
	M_Node& children = g_vcBezierBase->children;
	children.length = 1;
	int childDiff = keyValue.length - children.length + 1;

	if(childDiff < 0){
		children.length+=childDiff;
	}
	else{
		for(int i=childDiff; --i>=0;){
			Transform* transform = new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		}
	}
	for(int i = keyValue.length; --i>=0;){
		Transform* transform = dynamic_cast<Transform*>(children[i+1].p);
		Vec3f pt = keyValue[i];
		transform->translation = pt;

		if(i == idx){
			transform->children.refValue()[0] = g_redPoint;
		}
		else{
			transform->children.refValue()[0] = g_nextPoint;
		}

		transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	}

///////////
	g_vcBezierBase->rotation = Rotation();

	g_visualClue = g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}


void genNurbsLine(const M_Vec3f& dpoint, const M_Vec3f& point, const M_Float& weight, const M_Float& knot, const int idx, float minT, float maxT, const int degree, Shape* shape, const int steps){
	IndexedLineSet* ils = dynamic_cast<IndexedLineSet*>(shape->geometry.p);
	if(!point.length){
		return;
	}
#ifdef DIRECT3D
	setMathHiPresision();
#endif
	const float inverseSize = (1 / g_scene->getZoom());

	const int dim = point.length;

	M_Vec3f& coord = dynamic_cast<Coordinate*>(ils->coord.p)->point;

	int len = dim == 2 ? 2 : (dim - 1) * steps + 1;
	int markCoord = coord.length;

	coord.length += len;

	const Vec3f* p = point.getArray();
	const float* w = weight.getArray();

	int k = markCoord;

	if(dim == 2){
		coord[k] = dpoint[0];
		k++;
	}
	else{
		int i = len - 1;
		float d = i + 1;
		for(int i = 0; i < len - 1; i++){
			float t = static_cast<float>(i) / d;
			t = (maxT - minT) * t + minT;
			CurvePoint(dim - 1, degree, knot.getArray(), p, w, t, coord[k++]);
		}
	}
	coord[k] = dpoint[dpoint.length - 1];


	fillPolyLineIndex(coord, markCoord, ils->coordIndex.refValue());
	ils->coordIndex.refValue() += -1;

///////////
	ils->renderProcessed = false;
#ifdef DIRECT3D
	restoreMathPresision();
#endif
}



void NikInterpolator::houseKeep(){
	if(smls == 0){
		return;
	}
	M_Vec3f& focusPoint = focusOffset ? shortPoint : longPoint;

	static M_Vec3f line;
	if(!keyValue.length){
		g_visualClue=0;
		return;
	}

	setMathHiPresision();
	line.length = 0;

	//genCurve(line, 1000);

	const float inverseSize=(1 / g_scene->getZoom());
	
	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length = focusPoint.length * 1;

	for(int j = 0, i = 0; i < focusPoint.length; j++, i++){
		coord[j] = focusPoint[i];
	}
	M_Int& coordIndex = polyLine->coordIndex.refValue();
	fillPolyLineIndex(coord, coordIndex);

/////////////
	coordIndex += -1;
	M_Node& children = g_vcBezierBase->children;
	children.length = 1;
	int childDiff = focusPoint.length - children.length + 1;

	if(childDiff < 0){
		children.length+=childDiff;
	}
	else{
		for(int i=childDiff; --i>=0;){
			Transform* transform = new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		}
	}
	for(int i = focusPoint.length; --i>=0;){
		Transform* transform = dynamic_cast<Transform*>(children[i+1].p);
		Vec3f pt = focusPoint[i];
		transform->translation = pt;

		if(i == selPointIdx){
			transform->children.refValue()[0] = g_redPoint;
		}
		else{
			transform->children.refValue()[0] = g_nextPoint;
		}

		transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	}

///////////
	g_vcBezierBase->rotation = Rotation();

	g_visualClue = g_vcBezierBase;

	int steps = 100;

	if(calcLongOnly == false){
		genNurbsLine(longPoint, longPoint, weight, knot, selPointIdx, uMin(), uMax(), degree, g_vcEllipticalPolyline2D, steps);
	}
	if(calcLongOnly == false || focusOffset == 1){
		genNurbsLine(shortPoint, shortPoint, weight, knot, selPointIdx, uMin(), uMax(), degree, g_vcEllipticalPolyline2D, steps);
	}
	genNurbsLine(resultPoint, resultPoint, weight, knot, selPointIdx, uMin(), uMax(), degree, g_vcEllipticalPolyline2D, steps);


	g_visualClue = g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();

	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}


inline void createCurvedPolyMapVisualAid(NurbsPatch& patch, M_Vec3f& coord, M_Int& coordIndex, M_Vec2f& baseCoord, const int qty, Vec3f& average, const bool& northPole, const bool& southPole){
	int uOrder = patch.order.x;
	int vOrder = patch.order.y;
	int uDegree = patch.uDegree;
	int vDegree = patch.vDegree;
	int uDim = patch.dim.x;
	int vDim = patch.dim.y;

	float* uKnotPtr = patch.uKnot.refValue().getArray();
	float* vKnotPtr = patch.vKnot.refValue().getArray();
	Vec3f* pointPtr = patch.point.refValue().getArray();
	float* weightPtr = patch.weight.refValue().getArray();

	Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
	float* topWeightPtr = weightPtr+uDim*vDim-uDim;

	Vec3f* r_pointPtr = patch.r_point.getArray();
	float* r_weightPtr = patch.r_weight.getArray();


	float coordChange = .001;

	Vec2f diff;

	Vec3f curvedEdgePoint0;

	float maxU = patch.maxU();
	float maxV = patch.maxV();
	float minU = patch.minU();
	float minV = patch.minV();


	if(northPole){
		for(int i = 0; i < qty; i++){
			coordIndex+=coord.length;
			Vec3f v = getSurfaceVertex(convertNorthPoleToMercator(baseCoord[i]), maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr);
			average+=v;
			coord+=v;
		}
	}
	else if(southPole){
		for(int i = 0; i < qty; i++){
			coordIndex+=coord.length;
			Vec3f v = getSurfaceVertex(convertSouthPoleToMercator(baseCoord[i]), maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr);
			average+=v;
			coord+=v;
		}
	}
	else{
		for(int i = 0; i < qty; i++){
			coordIndex+=coord.length;
			Vec3f v = getSurfaceVertex(baseCoord[i], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr);
			average+=v;
			coord+=v;
		}
	}
}
void FuseSurface::houseKeep(){
	if(polyMapAid==false || parentPatch == 0 || parentStem == 0){
		g_visualClue=0;
		return;
	}

	setMathHiPresision();

	M_Node& children=g_vcBezierBase->children;
	children.length=1;

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcFuseSurfaceFlatStageAidShape->geometry.p);

	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length=2;


	M_Int& coordIndex = polyLine->coordIndex.refValue();
	coordIndex.length=0;
/*
	const Rotation orientation = this->orientation;
	Vec3f endPoint = Vec3f(0,1,0);
	if(baseCoord.length && curvedEdgePoint.length){

		M_Vec2f& baseCoord = this->baseCoord.refValue();
		const float vSpan=this->vSpan;
		NurbsPatch* stemPatch = parentPatch;
		if(stemPatch == 0){
			stemPatch = dynamic_cast<NurbsPatch*>(this->patch.p);
			if(stemPatch == 0){
				if(Stem* stem = dynamic_cast<Stem*>(this->patch.p)){
					stemPatch = dynamic_cast<NurbsPatch*>(stem->surface.p);
				}
			}
		}
		if(stemPatch){
			const int i=index[0];

			int uOrder = stemPatch->order.x;
			int vOrder = stemPatch->order.y;
			int uDim = stemPatch->dim.x;
			int vDim = stemPatch->dim.y;
			int uDegree = stemPatch->uDegree;
			int vDegree = stemPatch->vDegree;

			float* uKnotPtr = stemPatch->uKnot.refValue().getArray();
			float* vKnotPtr = stemPatch->vKnot.refValue().getArray();
			Vec3f* pointPtr = stemPatch->point.refValue().getArray();
			float* weightPtr = stemPatch->weight.refValue().getArray();
			Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
			float* topWeightPtr = weightPtr+uDim*vDim-uDim;
			Vec3f* r_pointPtr = stemPatch->r_point.getArray();
			float* r_weightPtr = stemPatch->r_weight.getArray();


			Vec2f t = baseCoord[i];
			if(parentStem->northPole){
				t = convertNorthPoleToMercator(t);
			}
			else if(parentStem->southPole){
				t = convertSouthPoleToMercator(t);
			}


			float u = (stemPatch->maxU()-stemPatch->minU())*t.x + stemPatch->minU();
			float v = (stemPatch->maxV()-stemPatch->minV())*t.y + stemPatch->minV();

			if(t.x==1){
				CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, curvedEdgePoint[i]);
			}
			else if(t.y==1){
				CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, curvedEdgePoint[i]);
			}
			else{
				SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &curvedEdgePoint[i]);
			}

			Vec3f ce = curvedEdgePoint[i];

			coord[0]=ce+orientation*endPoint;
			coord[1]=ce;
			fillPolyLineIndex(coord, coordIndex);
			coordIndex+=-1;
		}
	}
	else{
		coord[0]=orientation*endPoint;
		coord[1]=Vec3f();
		fillPolyLineIndex(coord, coordIndex);
		coord+=orientation*Vec3f(.03, .1, 0);
		coord+=orientation*Vec3f(-.03, .1, 0);
		coord+=orientation*Vec3f(0, .1, .03);
		coord+=orientation*Vec3f(0, .1, -.03);
		coordIndex+=-1;
		coordIndex+=2;
		coordIndex+=1;
		coordIndex+=4;
		coordIndex+=-1;
		coordIndex+=3;
		coordIndex+=1;
		coordIndex+=5;
		coordIndex+=-1;
	}
	*/
	if((polyMapAidMirror | polyMapAid) && baseCoord.length){
		Vec3f average;
		int remCoordLen=coord.length;
		if(dynamic_cast<ImmediatePart*>(g_selNode.getRaw())==0){
			createCurvedPolyMapVisualAid(*parentPatch, coord, coordIndex, baseCoord, qty, average, parentStem->northPole, parentStem->southPole);
		}
		else{
			if(polyMapAidMirror){
				for(int i = 0; i < qty; i++){
					coordIndex+=coord.length;
					Vec2f vec2f = baseCoord[i];
					vec2f.x-=.5;
					vec2f.x=-vec2f.x;
					vec2f.x+=.5;
					average.x+=vec2f.x;
					average.y+=vec2f.y;
					coord+=Vec3f(vec2f.x, vec2f.y, 0);
				}
			}
			else{
				for(int i = 0; i < qty; i++){
					coordIndex+=coord.length;
					Vec2f vec2f = baseCoord[i];
					average.x+=vec2f.x;
					average.y+=vec2f.y;
					coord+=Vec3f(vec2f.x, vec2f.y, 0);
				}
			}
		}
		coordIndex+=remCoordLen;
		coordIndex+=-1;
		average/=qty;

		for(int i = 0; i < baseCoord.length; i++){
			coordIndex+=remCoordLen+i;
			coordIndex+=coord.length;
			coordIndex+=-1;
			//coord+=(average+coord[remCoordLen+i]*7)*.125;
			coord+=average;
		}
	}

	g_vcBezierBase->rotation=Rotation();

	if(dynamic_cast<Art*>(g_selNode.getRaw())){
		g_vcArtTriBase->fuseSurface=this;
	}
	else{
		g_visualClue=g_vcFuseSurfaceFlatStageAidShape;
	}
	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	//polyLine->convertToRenderFormat(g_vcFuseSurfaceFlatStageAidShape->render);
	polyLine->renderProcessed=false;
}
void PolyToNurbs::houseKeep(){
	if(NurbsPatch* mimic = dynamic_cast<NurbsPatch*>(this->mimic.p)){
		if(mimic->stem.length){
			Stem* stem = dynamic_cast<Stem*>(mimic->stem[0].p);
			if(Art* originalPart = dynamic_cast<Art*>(this->original.p)){
				M_SArt& part = stem->part.refValue();
				if(part.length){
					M_SArt& source = stem->source.refValue();
					Art* guessPart = part[0];
					const M_Vec3f& mimicCoord = guessPart->coord;
					const M_Vec3f& originalCoord = originalPart->coord;

					if(source.length){
						Art* flatPart = source[0];

						setMathHiPresision();

						M_Node& children=g_vcBezierBase->children;
						children.length=1;

						IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcFuseSurfaceFlatStageAidShape->geometry.p);

						M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
						coord.length = 8;
						M_Int& coordIndex = polyLine->coordIndex.refValue();
						coordIndex.length=0;

						if(selMimicVertex >= mimicCoord.length){
							selMimicVertex.refValue() = mimicCoord.length-1;
						}
						Vec3f cb = mimicCoord[selMimicVertex];
						coord[0] = cb;
						Vec3f ce = cb;
						ce.x += 1;
						coord[1] = ce;

						coord[2] = cb + Vec3f(.1, .03, 0);
						coord[3] = cb + Vec3f(.1, -.03, 0);

						coordIndex+=0;
						coordIndex+=1;
						coordIndex+=-1;

						coordIndex+=2;
						coordIndex+=0;
						coordIndex+=3;
						coordIndex+=-1;

						int i = 0;
						float nearest = 10000000;
						int nearestIdx = 0;
						for(i = originalCoord.length; --i>=0;){
							Vec3f diff = originalCoord[i] - cb;
							float n = fabs(sqrt(diff.x*diff.x + diff.y*diff.y + diff.z*diff.z));
							if(n < nearest){
								nearest = n;
								nearestIdx=i;
							}
						}

						cb = originalCoord[nearestIdx];
						coord[4] = cb;
						ce = cb;
						ce.x -= 1;
						coord[5] = ce;

						coord[6] = cb + Vec3f(-.1, .03, 0);
						coord[7] = cb + Vec3f(-.1, -.03, 0);

						coordIndex+=4;
						coordIndex+=5;
						coordIndex+=-1;

						coordIndex+=6;
						coordIndex+=4;
						coordIndex+=7;

						selOriginalVertex.refValue() = nearestIdx;
						mimicOriginalIndex.refValue() = nearestIdx == selMimicVertex;

						g_visualClue=g_vcFuseSurfaceFlatStageAidShape;
						restoreMathPresision();
						polyLine->renderProcessed=false;
					}
				}
			}
		}
	}
}

void Harmonic::houseKeep(){
	houseKeep2dNurbs(dpoint, point, weight, knot, index[0], minT(), maxT(), degree, 1);
}

void HarmonicPatch::houseKeep(){
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	int numOfU=this->dim.x;
	int numOfV=this->dim.y;

	{
		float inverseSize=(1/g_scene->getZoom());
		int idx=index[0];

		IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

		M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

		coord.length=0;

		for(int i=point.length; --i>=0;){
			coord += point[i] / weight[i];
		}

		M_Int& coordIndex = polyLine->coordIndex.refValue();
		coordIndex.length=0;


		for(int v=0; v<numOfV; v++){
			for(int u=0; u<numOfU; u++){
				coordIndex+=v*numOfU+u;
			}
			coordIndex+=-1;
		}
		for(int u=0; u<numOfU; u++){
			for(int v=0; v<numOfV; v++){
				coordIndex+=v*numOfU+u;
			}
			coordIndex+=-1;
		}
	/////////////
		{
			int pointLen=numOfU*numOfV;

			M_Node& children=g_vcBezierBase->children;
			children.length=1;
			int childDiff=pointLen-children.length+1;

			if(childDiff<0){
				children.length+=childDiff;
			}
			else{
				for(int i=childDiff; --i>=0;){
					Transform* transform=new Transform;
					transform->children.refValue()+=g_nextPoint.p; //could just add null?
					children+=transform;
				}
			}
			for(int i=pointLen; --i>=0;){
				Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
				Vec3f pt=point[i] / weight[i];
				transform->translation=pt;

				if(i==idx){
					transform->children.refValue()[0] = g_redPoint;
				}
				else{
					transform->children.refValue()[0] = g_nextPoint;
				}

				transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
			}
		}

	///////////
		g_vcBezierBase->rotation=Rotation();

		//g_visualClue=g_vcBezierBase;
		NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(g_vcOscSmls->build[0].p);
		Stem* stem = dynamic_cast<Stem*>(nurbsPatch->stem[0].p);
		M_Int& stemCoordIndex = stem->coordIndex.refValue();

		stem->staves.refValue()[0] = staves;
		stem->coord.refValue().length = stem->color.refValue().length = vCoord.length;
		stemCoordIndex.length = (harmonics - 1) * staves * 3 * 2;
		int staves = this->staves;

		int tri = 0;
		for(int h = 0; h < harmonics - 1; h++){
			for(int s = 0; s < staves; s++){
				stemCoordIndex[tri++] = (staves + 1) * h + s + 1;
				stemCoordIndex[tri++] = (staves + 1) * (h + 1) + s;
				stemCoordIndex[tri++] = (staves + 1) * h + s;

				stemCoordIndex[tri++] = (staves + 1) * h + s + 1;
				stemCoordIndex[tri++] = (staves + 1) * (h + 1) + s + 1;
				stemCoordIndex[tri++] = (staves + 1) * (h + 1) + s;
			}
		}
		//String str;
		//for(int i = 0; i < stemCoordIndex.length; i++){
		//	str += stemCoordIndex[i];
		//	str += " ";
		//	if(stemCoordIndex[i] >= stem->coord.length){
		//		break;
		//	}
		//}
		//dLog(str);

		if(vCoord.length){
			float f = 1.0 / staves;
			for(int h = 0; h < harmonics; h++){
				for(int i = 0; i < staves + 1; i++){
					int i2 = h * (staves + 1) + i;
					//vCoord.refValue()[i2] = .01 * h;
					stem->coord.refValue()[i2] = Vec2f(f * i, vCoord[i2]);
				}
			}
		}
		nurbsPatch->point.refValue() = point;
		nurbsPatch->weight.refValue() = weight;
		nurbsPatch->uKnot.refValue() = uKnot;
		nurbsPatch->vKnot.refValue() = vKnot;
		nurbsPatch->dim.refValue() = dim;
		nurbsPatch->order.refValue() = order;
		nurbsPatch->showAll.refValue() = true;
		nurbsPatch->processFields();

		g_vcOscSmls->doBuild();
		Art* part = dynamic_cast<Art*>(stem->part[0].p);
		this->coord.refValue() = part->coord;
		g_visualClue = g_vcOscBase;

		g_vcOscSmls->convertToRenderFormat();

	//	g_scene->children.refValue()+=g_vcBezierBase.p;
	//	g_stw->genNodeList();


		//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
		polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
	}
	restoreMathPresision();
}
/*
void Subdivision::houseKeep(){
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	float inverseSize=(1/g_scene->getZoom());
	int idx=index[0];

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

	coord.length=0;

	coord = point;

	M_Int& polyCoordIndex = polyLine->coordIndex.refValue();
	polyCoordIndex.length=0;

	M_Int& coordIndex = this->coordIndex.refValue();

	for(int i=0; i<coordIndex.length; ){
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i-4];
		polyCoordIndex += -1;
	}


/////////////
	{
		int pointLen=point.length;

		M_Node& children=g_vcBezierBase->children;
		children.length=1;
		int childDiff=pointLen-children.length+1;

		if(childDiff<0){
			children.length+=childDiff;
		}
		else{
			for(int i=childDiff; --i>=0;){
				Transform* transform=new Transform;
				transform->children.refValue()+=g_nextPoint.p; //could just add null?
				children+=transform;
			}
		}
		for(int i=pointLen; --i>=0;){
			Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
			Vec3f pt=point[i];
			transform->translation=pt;

			if(i==idx){
				transform->children.refValue()[0]=g_redPoint;
			}
			else{
				transform->children.refValue()[0]=g_nextPoint;
			}

			transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
		}
	}

///////////
	g_vcBezierBase->rotation=Rotation();

	g_visualClue=g_vcBezierBase;

	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
	restoreMathPresision();
}
*/

void Subdivision::houseKeep(){
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	float inverseSize=(1/g_scene->getZoom());
	int idx=index[0];

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

	coord.length=0;

	coord = point;

	M_Int& polyCoordIndex = polyLine->coordIndex.refValue();
	polyCoordIndex.length=0;

	M_Int& coordIndex = this->coordIndex.refValue();

	for(int i=0; i<coordIndex.length; ){
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i++];
		polyCoordIndex += coordIndex[i-4];
		polyCoordIndex += -1;
	}


/////////////
	{
		int pointLen=point.length;

		M_Node& children=g_vcBezierBase->children;
		children.length=1;
		int childDiff=4-children.length+1;

		if(childDiff<0){
			children.length+=childDiff;
		}

		{
			for(int i=childDiff; --i>=0;){
				Transform* transform=new Transform;
				transform->children.refValue()+=g_nextPoint.p; //could just add null?
				children+=transform;
			}

			M_Int& coordIndex = this->coordIndex.refValue();

			for(int i=4; --i>=0;){
				int j = coordIndex[actPoly*4+i];
				Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
				Vec3f pt = point[j];
				transform->translation = pt;
				if(i==actCorner){
					transform->children.refValue()[0]=g_redPoint;
				}
				else if(i==(actCorner+1)%4){
					transform->children.refValue()[0]=g_nextPoint;
				}
				else{
					transform->children.refValue()[0]=g_lastPoint;
				}
				transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
			}
		}
	}

///////////
	g_vcBezierBase->rotation=Rotation();

	g_visualClue = g_vcBezierBase;

	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
	restoreMathPresision();
}
Vec3f g_immediateRadialTugResizeLocation;

void NurbsPatch::addCageShowMid(IndexedLineSet* polyLine){
	int uDim = this->dim.x;
	int vDim = this->dim.y;

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Color3f& color = dynamic_cast<Color*>(polyLine->color.p)->color;
	M_Int& coordIndex = polyLine->coordIndex.refValue();


	Color3f midColor = Color3f(1, 0, 1);
	Color3f visColor = Color3f(0, 1, 1);

	if(g_transformCP->selectObject && object){
		midColor = Color3f(1, 0, 0);
		visColor = Color3f(1, 0, 0);
	}

	for(int i = dpoint.length; --i>=0;){
		coord[fromPoint + i] = matrix * dpoint[i];
		color[fromPoint + i] = midColor;
	}
	if(cage == false){
		return;
	}

	for(int v = 0; v < vDim; v++){
		for(int u=0; u<uDim; u++){
			coordIndex += fromPoint + v * uDim+u;
		}
		coordIndex += -1;
	}
	for(int u = 0; u < uDim; u++){
		for(int v=0; v < vDim; v++){
			coordIndex += fromPoint + v * uDim + u;
		}
		coordIndex += -1;
	}

	if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
		if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
			for(int v = 1; v < vDim - 1; v += 2){
				for(int u = 1; u < uDim; u += 2){
					color[fromPoint + v * uDim + u] = visColor;
				}
				if(v == 0 || v == vDim - 2){
					v--;
				}
			}
		}
		else{
			for(int v = 1; v < vDim - 1; v += 2){
				for(int u = 1; u < uDim - 1; u += 1){
					color[fromPoint + v * uDim + u] = visColor;
				}
				if(v == 0 || v == vDim - 2){
					v--;
				}
			}
		}
	}
	else{
		if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
			for(int v = 1; v < vDim - 1; v++){
				for(int u = 1; u < uDim; u += 2){
					color[fromPoint + v * uDim + u] = visColor;
				}
			}
		}
		else{
			for(int v = 1; v < vDim - 1; v++){
				for(int u = 1; u < uDim -  1; u++){
					color[fromPoint + v * uDim + u] = visColor;
				}
			}
		}
	}
}

void NurbsPatch::addCage(IndexedLineSet* polyLine, const M_Bool& m_pointIsVisible){

	int uDim = this->dim.x;
	int vDim = this->dim.y;

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	for(int i = dpoint.length; --i>=0;){
		coord[fromPoint + i] = matrix * dpoint[i];
	}
	if(cage == false){
		return;
	}

	if(uJoinEnds){
		if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
			if(isCtrlPolyRef){
				if(vDim > 3){
					for(int v = 1; v < vDim - 1; v += 2){
						for(int u = 1; u < uDim - 1; u += 2){
							coordIndex += fromPoint + v * uDim+u;
						}
						coordIndex += -1;
					}
					for(int u = 1; u < uDim - 1; u += 2){
						for(int v = 1; v < vDim - 1; v++){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += -1;
					}
				}
			}
			else{
				if(vDim == 1){
					coordIndex += fromPoint;
					for(int u = 1; u < uDim; u++){
						coordIndex += fromPoint + u;
					}
					coordIndex += fromPoint + uDim - 1;
					coordIndex+=-1;
				}
				else{
					for(int v = 0; v < vDim; v += 2){
						coordIndex += fromPoint + v * uDim;
						for(int u = 1; u < uDim; u += 2){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += fromPoint + v * uDim + uDim - 1;
						coordIndex+=-1;

						if(v == 0 || v == vDim - 2){
							v--;
						}
					}
					for(int u=1; u<uDim; u += 2){
						for(int v=0; v<vDim; v++){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += -1;
					}
				}
			}
		}
		else{
			if(isCtrlPolyRef){
				if(vDim > 3){
					for(int v = 1; v < vDim - 1; v++){
						for(int u = 1; u < uDim - 1; u += 2){
							coordIndex += fromPoint + v * uDim+u;
						}
						coordIndex += -1;
					}
					for(int u = 1; u < uDim - 1; u += 2){
						for(int v = 1; v < vDim - 1; v++){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += -1;
					}
				}
			}
			else{
				for(int v=0; v < vDim; v++){
					coordIndex += fromPoint + v * uDim + uDim - 2;
					for(int u=1; u<uDim; u+=2){
						coordIndex += fromPoint + v * uDim + u;
					}
					coordIndex += -1;
				}
				for(int u=1; u<uDim; u+=2){
					for(int v=0; v<vDim; v++){
						coordIndex += fromPoint + v * uDim + u;
					}
					coordIndex += -1;
				}
			}
		}
	}
	else {
		if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
			if(isCtrlPolyRef){
				if(uDim > 3 && vDim > 3){
					for(int v = 1; v < vDim - 1; v += 2){
						for(int u = 1; u < uDim - 2; u += 2){
							int i = fromPoint + v * uDim + u;
							if(m_pointIsVisible[i]){
								int i2 = i + 2;
								if(u == uDim - 3 || m_pointIsVisible[i2] == false){
									continue;
								}
								coordIndex += i;
								coordIndex += i2;
								u += 4;
								for(; u < uDim - 1; u += 2){
									i = fromPoint + v * uDim + u;
									if(m_pointIsVisible[i] == false){
										break;
									}
									coordIndex += i;
								}
								coordIndex += -1;
							}
						}
					}
					for(int u = 1; u < uDim - 1; u += 2){
						for(int v = 1; v < vDim - 2; v += 2){
							int i = fromPoint + v * uDim + u;
							if(m_pointIsVisible[i]){
								int i2 = i + uDim * 2;
								if(v == vDim - 3 || m_pointIsVisible[i2] == false){
									continue;
								}
								coordIndex += i;
								coordIndex += i2;
								v += 4;
								for(; v < vDim - 1; v += 2){
									i = fromPoint + v * uDim + u;
									if(m_pointIsVisible[i] == false){
										break;
									}
									coordIndex += i;
								}
								coordIndex += -1;
							}
						}
					}
				}
			}
			else{
				if(vDim == 1){
					for(int u = 0; u < uDim; u++){
						coordIndex += fromPoint + u;
					}
					coordIndex += -1;
				}
				else{
					for(int v = 0; v < vDim; v += 2){
						coordIndex += fromPoint + v * uDim;
						for(int u = 1; u<uDim; u += 2){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += fromPoint + v * uDim + uDim - 1;
						coordIndex+=-1;

						if(v == 0 || v == vDim - 2){
							v--;
						}
					}
					for(int u=0; u<uDim; u += 2){
						coordIndex += fromPoint + u;
						for(int v=1; v<vDim; v+=2){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += fromPoint + uDim * vDim - uDim + u;
						coordIndex += -1;
						if(u == 0 || u == uDim - 2){
							u--;
						}
					}
				}
			}
		}
		else{
			if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
				if(isCtrlPolyRef){
					if(vDim > 3){
						for(int v = 1; v < vDim - 1; v++){
							for(int u = 1; u < uDim - 1; u += 2){
								coordIndex += fromPoint + v * uDim+u;
							}
							coordIndex += -1;
						}
						for(int u = 1; u < uDim - 1; u += 2){
							for(int v = 1; v < vDim - 1; v++){
								coordIndex += fromPoint + v * uDim + u;
							}
							coordIndex += -1;
						}
					}
				}
				else{
					for(int v = 0; v < vDim; v++){
						coordIndex += fromPoint + v * uDim;
						for(int u = 1; u < uDim; u +=2){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += fromPoint + v * uDim + uDim - 1;
						coordIndex += -1;
					}
					for(int u = 0; u < uDim; u += 2){
						coordIndex += fromPoint + u;
						for(int v = 0; v < vDim; v++){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += fromPoint + uDim * vDim - uDim + u;
						coordIndex += -1;
						if(u == 0 || u == uDim - 2){
							u--;
						}
					}
				}
			}
			else{
				if(isCtrlPolyRef){
					if(vDim > 3 && uDim > 3){
						for(int v = 1; v < vDim - 1; v++){
							for(int u = 1; u < uDim - 1; u++){
								coordIndex += fromPoint + v * uDim + u;
							}
							coordIndex += -1;
						}
						for(int u = 1; u < uDim - 1; u++){
							for(int v = 1; v < vDim - 1; v++){
								coordIndex += fromPoint + v * uDim + u;
							}
							coordIndex += -1;
						}
					}
				}
				else {
					for(int v = 0; v < vDim; v++){
						for(int u = 0; u < uDim; u++){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += -1;
					}
					for(int u = 0; u < uDim; u++){
						for(int v = 0; v < vDim; v++){
							coordIndex += fromPoint + v * uDim + u;
						}
						coordIndex += -1;
					}
				}
			}
		}
	}
}

// backface showing
void NurbsPatch::addCage(IndexedLineSet* polyLine){
	const int wireResolution = 4;

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();


	Vec3f pnt;
	int uOrder = order.x;
	int vOrder = order.y;

	int uDegree = this->uDegree;
	int vDegree = this->vDegree;

	int uDim = dim.x;
	int vDim = dim.y;

	float maxU = this->maxU();
	float maxV = this->maxV();
	float minU = this->minU();
	float minV = this->minV();

    const float* uKnotPtr = uKnot.refValue().getArray();
    const float* vKnotPtr = vKnot.refValue().getArray();
    const Vec3f* pointPtr = point.refValue().getArray();
    const float* weightPtr = weight.refValue().getArray();

	const Vec3f* r_pointPtr = r_point.getArray();
    const float* r_weightPtr = r_weight.getArray();
	const Vec3f* topPointPtr = pointPtr + uDim * vDim - uDim;
	const float* topWeightPtr = weightPtr + uDim * vDim - uDim;

	const int uCells = (uDim / 2);
	const int vCells = (vDim / 2);

	const int uSteps = uCells * wireResolution;
	const int vSteps = vCells * wireResolution;

	float u = 0;
	float v = 0;

	int cpBegin = coord.length;
	int idx = cpBegin;
	int cpidx = cpBegin + uCells + 2;


	for(int mvc = 0; mvc < vCells + 2; mvc++){
		v = 0;
		if(mvc == vCells + 1){
			v = 1;
		}
		else if(mvc > 0){
			v = static_cast<float>(mvc - 1) / vCells + 0.5 * (1.0 / vCells);
		}
		for(int muc = 0; muc < uCells + 2; muc++){
			u = 0;
			if(muc == uCells + 1){
				u = 1;
			}
			else if(muc > 0){
				u = static_cast<float>(muc - 1) / uCells + 0.5 * (1.0 / uCells);
			}

			if(u == 1){
				CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, pnt);
			}
			else if(v == 1){
				CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, pnt);
			}
			else{
				SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &pnt);
			}
			coord += pnt;
			idx++;
		}
	}

	if(cage == false){
		return;
	}

	for(int vc = 0; vc < vCells; vc++){
		v = static_cast<float>(vc) / vCells + 0.5 * (1.0 / vCells);
		int uResCounter = 1;
		for(int ui = 0; ui < uSteps + 1; ui++){
			if(--uResCounter == 0){
				if(ui == 0 || ui == uSteps - 2){
					uResCounter = wireResolution / 2;
				}
				else{
					uResCounter = wireResolution;
				}
				if(vc > 0 && vc < vCells - 1 && ui > 1 && ui < uSteps - 1 && uCells > 1){
					coordIndex += cpidx;
				}
				cpidx++;
			}
			else{
				u = static_cast<float>(ui) / uSteps;
				if(u == 1){
					CurvePoint(vDim-1, vDegree, vKnotPtr, r_pointPtr, r_weightPtr, v, pnt);
				}
				else{
					SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &pnt);
				}
				coord += pnt;
				if(vc > 0 && vc < vCells - 1 && ui > 1 && ui < uSteps - 1 && uCells > 1){
					coordIndex += idx;
				}
				idx++;
			}
		}
		if(vc > 0 && vc < vCells - 1 && uCells > 1){
			coordIndex += -1;
		}
	}

	cpidx = cpBegin + 1;

	for(int uc = 0; uc < uCells; uc++){
		u = static_cast<float>(uc) / uCells + 0.5 * (1.0 / uCells);
		int vResCounter = 1;
		for(int vi = 0; vi < vSteps + 1; vi++){
			if(--vResCounter == 0){
				if(vi == 0 || vi == vSteps - 2){
					vResCounter = wireResolution / 2;
				}
				else{
					vResCounter = wireResolution;
				}
				if(uc > 0 && uc < uCells - 1 && vi > 1 && vi < vSteps - 1 && vCells > 1){
					coordIndex += cpidx;
				}

				if(vi == vSteps){
					cpidx = cpBegin + uc + 2;
				}
				else{
					cpidx += uCells + 2;
				}
			}
			else{
				v = static_cast<float>(vi) / vSteps;

				if(v == 1){
					CurvePoint(uDim-1, uDegree, uKnotPtr, topPointPtr, topWeightPtr, u, pnt);
				}
				else{
					SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, &pnt);
				}
				coord += pnt;
				if(uc > 0 && uc < uCells - 1 && vi > 1 && vi < vSteps - 1 && vCells > 1){
					coordIndex += idx;
				}
				idx++;
			}
		}
		if(uc > 0 && uc < uCells - 1 && vCells > 1){
			coordIndex += -1;
		}
	}
}

void NurbsPatch::addNurbsEdgeCage(IndexedLineSet* polyLine){//for box modelling
	int uDim = this->dim.x;
	int vDim = this->dim.y;

	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Color3f& color = dynamic_cast<Color*>(polyLine->color.p)->color;
	M_Int& coordIndex = polyLine->coordIndex.refValue();


	//Color3f midColor = Color3f(1, 0, 1);
	Color3f visColor = Color3f(0, 1, 0);

	if(g_transformCP->selectObject && object){
		visColor = Color3f(1, 0, 0);
	}

	for(int i = dpoint.length; --i>=0;){
		coord[fromPoint + i] = matrix * dpoint[i];
		color[fromPoint + i] = visColor;
	}
	if(cage == false){
		return;
	}
	if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
		if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
			for(int v = 0; v < vDim; v += 2){
				for(int u = 0; u < uDim; u += 2){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
			for(int u = 0; u < uDim; u += 2){
				for(int v = 0; v < vDim; v += 2){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
		}
		else{
			for(int v = 0; v < vDim; v += 2){
				for(int u = 0; u < uDim; u++){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
			for(int u = 0; u < uDim; u++){
				for(int v = 0; v < vDim; v += 2){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
		}
	}
	else{
		if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
			for(int v = 0; v < vDim; v++){
				for(int u = 0; u < uDim; u += 2){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
			for(int u = 0; u < uDim; u += 2){
				for(int v = 0; v < vDim; v++){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
		}
		else{
			for(int v = 0; v < vDim; v++){
				for(int u = 0; u < uDim; u++){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
			for(int u = 0; u < uDim; u++){
				for(int v = 0; v < vDim; v++){
					coordIndex += fromPoint + v * uDim + u;
				}
				coordIndex += -1;
			}
		}
	}
}

void NurbsPatch::setVisibleMaskForStitchEditMode(M_Bool& m_pointIsVisible){//for box modelling
	int uDim = this->dim.x;
	int vDim = this->dim.y;


	if(cage == false){
		return;
	}
	if(NurbsPatch::MID_1 == checkNurbsType(vDim, vKnot)){
		if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
			for(int v = 0; v < vDim; v += 2){
				for(int u = 0; u < uDim; u += 2){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
			for(int u = 0; u < uDim; u += 2){
				for(int v = 0; v < vDim; v += 2){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
		}
		else{
			for(int v = 0; v < vDim; v += 2){
				for(int u = 0; u < uDim; u++){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
			for(int u = 0; u < uDim; u++){
				for(int v = 0; v < vDim; v += 2){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
		}
	}
	else{
		if(NurbsPatch::MID_1 == checkNurbsType(uDim, uKnot)){
			for(int v = 0; v < vDim; v++){
				for(int u = 0; u < uDim; u += 2){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
			for(int u = 0; u < uDim; u += 2){
				for(int v = 0; v < vDim; v++){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
		}
		else{
			for(int v = 0; v < vDim; v++){
				for(int u = 0; u < uDim; u++){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
			for(int u = 0; u < uDim; u++){
				for(int v = 0; v < vDim; v++){
					m_pointIsVisible[fromPoint + v * uDim + u] = true;
				}
			}
		}
	}
}

void setBackfaceMaskForInnerPoints2(M_Bool& m_pointIsVisible, const M_Vec2f& m_screenPoint, int& i0, int& i1, int& i2, int& i3, Vec2f& centerPoint){
	centerPoint = m_screenPoint[i0];
	centerPoint += m_screenPoint[i1];
	centerPoint += m_screenPoint[i2];
	centerPoint += m_screenPoint[i3];

	centerPoint = centerPoint / 4;

	Vec2f vec2fA = m_screenPoint[i0];
	Vec2f vec2fB = m_screenPoint[i1];

	if(backFace(vec2fA, vec2fB, centerPoint)){
		m_pointIsVisible[i0] = true;
		m_pointIsVisible[i1] = true;
	}

	vec2fA = m_screenPoint[i1];
	vec2fB = m_screenPoint[i2];

	if(backFace(vec2fA, vec2fB, centerPoint)){
		m_pointIsVisible[i1] = true;
		m_pointIsVisible[i2] = true;
	}

	vec2fA = m_screenPoint[i2];
	vec2fB = m_screenPoint[i3];

	if(backFace(vec2fA, vec2fB, centerPoint)){
		m_pointIsVisible[i2] = true;
		m_pointIsVisible[i3] = true;
	}

	vec2fA = m_screenPoint[i3];
	vec2fB = m_screenPoint[i0];

	if(backFace(vec2fA, vec2fB, centerPoint)){
		m_pointIsVisible[i3] = true;
		m_pointIsVisible[i0] = true;
	}
}

bool isSelected(NurbsPatch* patch, int idx){
	for(int k = MultiSelect::pointList.length; --k>= 0;){
		if(patch == MultiSelect::pointList[k].p && idx == MultiSelect::pointList[k].i){
			return true;
		}
	}
	return false;
}

void addRenderControlPoint(GeomRenderPtr geomRender, int k, const float inverseSize, M_Node& children){
	Transform* transform = new Transform;
	transform->children.refValue() += geomRender.p;
	NurbsPatch* patch = MultiSelect::pointList[k].p;
	int i = MultiSelect::pointList[k].i;
	transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	transform->translation.refValue() = patch->matrix * patch->dpoint[i];

	children += transform;
}


void Seamless::houseKeepNurbsCages(){
	setMathHiPresision();

	g_nurbsIlsBase->rotation.refValue() = Rotation();
	g_nurbsIlsBase->translation.refValue() = Vec3f();
	g_nurbsIlsBase->scale.refValue() = Vec3f(1,1,1);

	if(this == 0 || (g_scene->mode & HOME_MODE)){
		return;
	}

	if(workAnimCP == true){
		setModellingPatchMode();
		doBuild();
	}

	float inverseSize = (1 / g_scene->getZoom());

	if(MultiSelect::pointList.length){
		if(MultiSelect::pointList[0].p->smls != this){
			MultiSelect::pointList.length = 0;
			MultiSelect::beginDragPos.length = 0;
		}
	}

	if(g_visualClue == g_vcSelectRectGroup){
		IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcSelectRectShape->geometry.p);
		M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

		coord.length = 0;
		coord += g_scene->getIViewMatrix() * Vec3f(MultiSelect::topLeft.x, MultiSelect::bottomRight.y, MultiSelect::topLeft.z);
		coord += g_scene->getIViewMatrix() * MultiSelect::bottomRight;
		coord += g_scene->getIViewMatrix() * Vec3f(MultiSelect::bottomRight.x, MultiSelect::topLeft.y, MultiSelect::topLeft.z);
		coord += g_scene->getIViewMatrix() * MultiSelect::topLeft;
		fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
		polyLine->coordIndex.refValue() += 0;
		polyLine->convertToRenderFormat(g_vcSelectRectShape->render);
		g_vcSelectRectGroup->children.refValue().length = 0;
		//g_vcBezierBase->children.refValue() += g_vcSelectRectShape.p;
	}
	if(g_curvedCageMode){
		genCage();
	}
	else{
		genStraightCage(g_pointIsVisible);
	}

	M_Node& children = g_nurbsIlsBase->children;
	children.length = 1;

	if(g_transformCP->selectObject == false){
		if(g_scene->mode & SINGLE_ACTIVE_MODE){
			for(int k = 0; k < MultiSelect::pointList.length; k++){
				if(k != MultiSelect::selectedPoint){
					addRenderControlPoint(g_nextPoint, k, inverseSize, children);
				}
			}
			if(MultiSelect::selectedPoint >= 0 && MultiSelect::selectedPoint < MultiSelect::pointList.length){
				addRenderControlPoint(g_redPoint, MultiSelect::selectedPoint, inverseSize, children);
			}
		}
		else{
			M_Vec3f& point = dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;
			if(g_curvedCageMode){
				for(int k = 0; k < MultiSelect::pointList.length; k++){
					Transform* transform = new Transform;
					transform->children.refValue() += g_redPoint.p;
					NurbsPatch* patch = MultiSelect::pointList[k].p;
					int i = NurbsPatch::convertActualToVisual(MultiSelect::pointList[k].i, patch->dim.x, patch->dim.y);

					int lookUpIndex = g_pointLookup[patch->plidx];

					transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
					transform->translation.refValue() = point[lookUpIndex + i];

					children += transform;
				}
			}
			else{
				for(int k = 0; k < MultiSelect::pointList.length; k++){
					Transform* transform = new Transform;
					transform->children.refValue() += g_redPoint.p;
					NurbsPatch* patch = MultiSelect::pointList[k].p;
					int i = MultiSelect::pointList[k].i;
					transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
					transform->translation.refValue() = patch->matrix * patch->dpoint[i];

					children += transform;
				}
			}
		}
	//	if(g_scene->mode & MODELLING_MODE && g_insertPatch.imousePhase != InsertPatch::NEUTRAL && g_insertPatch.m_insert.length){
	//		int uDim = g_insertPatch.init_patch->dim.x;
	//		int u = g_insertPatch.init_idx % uDim;
	//		int v = g_insertPatch.init_idx / uDim;

	//		g_insertArrow->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	//		Vec3f basePoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx];

	//		Vec3f basePointB;
	//		Vec3f directionPoint;
	//		Vec3f directionPointB;
	//		
	//		if(g_insertPatch.init_doU){
	//			if(g_insertPatch.init_leaveNorthOrEast){
	//				directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + uDim];
	//				directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + uDim + 1];
	//			}
	//			else{
	//				directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - uDim];
	//				directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - uDim + 1];
	//			}
	//		}
	//		else{
	//			if(g_insertPatch.init_leaveNorthOrEast){
	//				directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + 1];
	//				directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + 1 + uDim];
	//			}
	//			else{
	//				directionPoint = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - 1];
	//				directionPointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx - 1 + uDim];
	//			}
	//		}



	//		if(g_insertPatch.init_doU){
	//			basePointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + 1];
	//		}
	//		else{
	//			basePointB = g_insertPatch.init_patch->dpoint[g_insertPatch.init_idx + uDim];
	//		}
	//		
	//		if((g_insertPatch.init_doU == true && u & 1) || (g_insertPatch.init_doU == false && v & 1)){
	//			basePoint = basePoint * g_insertPatch.init_ratio + basePointB * (1.0 - g_insertPatch.init_ratio);
	//			directionPoint = directionPoint * g_insertPatch.init_ratio + directionPointB * (1.0 - g_insertPatch.init_ratio);
	//		}
	//		else{
	//			basePoint = basePoint * (1.0 - g_insertPatch.init_ratio) + basePointB * g_insertPatch.init_ratio;
	//			directionPoint = directionPoint * (1.0 - g_insertPatch.init_ratio) + directionPointB * g_insertPatch.init_ratio;
	//		}


	//		directionPoint -= basePoint;
	//		directionPoint = directionPoint.normalize();
	//		
	//		Rotation directionOrientation = Rotation(Vec3f(0, 1, 0), directionPoint);
	//		
	//		g_insertArrow->rotation.refValue() = directionOrientation;

	//		g_insertArrow->translation.refValue() = basePoint;
	//		children += g_insertArrow.p;
	//	}
	}
	if(Build* build = dynamic_cast<Build*>(g_selNode.getRaw())){
		if(NurbsPatch* selPatch = build->getPatch()){ //selected patch may have changed from this
			int uDim = selPatch->dim.x;
			int vDim = selPatch->dim.y;
			int idx = selPatch->index[0];
			if(uDim == 1 && selPatch->cage && MultiSelect::pointList.length == 1){
				for(int q = vDim; --q >= 0;){
					if(q == idx){
						continue;
					}
					Transform* transform = new Transform;
					transform->children.refValue() += g_nextPoint.p;
					transform->scale=Vec3f(inverseSize, inverseSize, inverseSize);
					transform->translation.refValue() = selPatch->dpoint[q];
					children += transform;
				}
			}
			if(vDim == 1 && selPatch->cage && MultiSelect::pointList.length == 1){
				int from = 0;
				int to = uDim;
				if(selPatch->uJoinEnds){
					from = 1;
					to = uDim - 1;
				}
				for(int q = from; q < to; q++){
					if(q == idx){
						continue;
					}
					Transform* transform = new Transform;
					transform->children.refValue() += g_nextPoint.p;
					transform->scale=Vec3f(inverseSize, inverseSize, inverseSize);
					transform->translation.refValue() = selPatch->dpoint[q];
					children += transform;
				}
			}
		}
	}
	//if(MultiSelect::boxPolySelList.length){
	//if(MultiSelect::boxPolySelList.length && g_vcSelectRectGroup->children.length < 3){
	//	g_vcSelectRectGroup->children.refValue() += g_nurbsIlsBase.p;
	//	g_vcSelectRectGroup->children.refValue() += g_vcSelectRectShape.p;
	//	g_vcSelectRectGroup->children.refValue() += g_vcNurbsCtrlPoly.p;
	//	g_visualClue = g_vcSelectRectGroup;
	//}
	if(g_visualClue == g_vcSelectRectGroup){
		g_vcSelectRectGroup->children.refValue() += g_nurbsIlsBase.p;
		g_vcSelectRectGroup->children.refValue() += g_vcSelectRectShape.p;
		if(g_scene->mode & STITCH_POLY_MODE && ctrlPolyPoint.length){
			g_vcSelectRectGroup->children.refValue() += g_vcNurbsCtrlPoly.p;
		}
		if((MultiSelect::pointList.length >= g_transformCP->leverSelection || g_transformCP->showDimFilterVisual && MultiSelect::pointList.length) && g_transformCP->sameScreenPosition == false){
			children += g_transformCP.p;
		}
		if(MultiSelect::boxPolySelList.length){
			g_vcSelectRectGroup->children.refValue() += g_vcNurbsCtrlPoly.p;
		}
	}
	else{
		g_visualClue = g_nurbsIlsBase;
		if(MultiSelect::boxPolySelList.length){
			children += g_vcNurbsCtrlPoly.p;
		}
		if(g_scene->mode & STITCH_POLY_MODE && ctrlPolyPoint.length){
			children += g_vcNurbsCtrlPoly.p;
		}
		if((MultiSelect::pointList.length >= g_transformCP->leverSelection || g_transformCP->showDimFilterVisual && MultiSelect::pointList.length) && g_transformCP->sameScreenPosition == false){
			children += g_transformCP.p;
		}
	}
	if(g_scene->mode & MODELLING_MODE && g_insertPatch.imousePhase != InsertPatch::NEUTRAL){
		g_insertPatch.addInsertVisualClue();
	}

	return;
	/*
	}
	else{
		int uDim=this->dim.x;
		int vDim=this->dim.y;
		int idx = index[0];
		IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
		M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue();
		coord.length = 0;
		M_Int& coordIndex = polyLine->coordIndex.refValue();
		coordIndex.length = 0;
		addCage(polyLine);

		M_Node& children=g_vcBezierBase->children;
		children.length=1;
		Transform* transform=new Transform;
		transform->children.refValue() += g_redPoint.p;
		children+=transform;

		Vec2i idx2i = Vec2i(idx % uDim, idx / uDim);

		Vec3f pt = matrix * dpoint[idx2i.y*uDim+idx2i.x];
		transform->translation = pt;
		transform->scale=Vec3f(inverseSize, inverseSize, inverseSize);

		if(ringTug){
			if(MultiSelect::beginDragPos.length==0){
				MultiSelect::beginDragPos = dpoint;
			}
			transform=new Transform;
			transform->children.refValue() += g_yellowPoint.p;
			children+=transform;

			if(tugRingPivotPoint){
				transform->translation = g_scene->getIViewMatrix() * g_pointer.translation;
			}
			else{
				transform->translation = matrix * calcRingPivotPoint(getSelRing());
			}
			
			transform->scale=Vec3f(inverseSize, inverseSize, inverseSize);
		}

		
	///////////
		g_vcBezierBase->rotation=Rotation();

		g_visualClue=g_vcBezierBase;
	//	g_scene->children.refValue()+=g_vcBezierBase.p;
	//	g_stw->genNodeList();


		//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
		polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
	}
	*/
}

void NurbsPatch::houseKeep(){
	smls->houseKeepNurbsCages();
	float tugRadius = g_radial_Tug->radius;

	if(g_radial_Tug->active){
		IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_nurbsIlsShape->geometry.p);
		M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue();
		M_Color3f& color = dynamic_cast<Color*>(polyLine->color.p)->color.refValue();
		int mark = coord.length;
		int numOfPoints=129;
		coord.length+=numOfPoints;
		color.length+=numOfPoints;

		Vec3f location = dpoint[index[0]];
		Matrix matrix = rotationMatrix( -g_scene->rotation) * scaleMatrix(Vec3f(tugRadius,tugRadius,tugRadius)) * translationMatrix(location);

		for(int i=0; i<numOfPoints; i++){
			coord[mark+i]= matrix * Vec3f(cos(i*(PI*2/(numOfPoints-1))), sin(i*(PI*2/(numOfPoints-1))),0);
			color[mark+i]= Color3f(0,1,0);
		}
		fillPolyLineIndex(coord, mark, polyLine->coordIndex.refValue());

		//Transform* transform = new Transform;
		//transform->children.refValue() += g_redPoint.p;
		//children += transform;

		//transform->translation = matrix * Vec3f(1, 0, 0);
		//transform->scale=Vec3f(inverseSize, inverseSize, inverseSize);
		polyLine->convertToRenderFormat(g_nurbsIlsShape->render);
	}
	restoreMathPresision();
}

void GroupBuild::houseKeep(){
	if(Seamless* smls = getSeamless2()){
		smls->houseKeepNurbsCages();
		restoreMathPresision();
	}
}
//#include <crtdbg.h>
//static _CrtMemState s1, s2, s3;



void Bezier::bezHouseKeep(){
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	float inverseSize=(1/g_scene->getZoom());
	int idx=index[0];

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length=line.length+point.length;//normA.length*5;
	for(int i=0; i<line.length; i++){
		coord[i]=matrix*Vec3f(line[i].x, line[i].y, 0);
	}
	for(int j=line.length, i=point.length; --i>=0; j++){
		coord[j]=matrix*Vec3f(point[i].x, point[i].y, 0);
	}
	/*
	for(int j=point.length+line.length, i=normA.length; --i>=0; ){
		coord[j++]=Vec3f(normA[i].x, normA[i].y, 0);
		coord[j++]=Vec3f(normB[i].x, normB[i].y, 0);
		coord[j++]=Vec3f(normC[i].x, normC[i].y, 0);
		coord[j++]=Vec3f(normD[i].x, normD[i].y, 0);
		coord[j++]=Vec3f(normA[i].x, normA[i].y, 0);
	}
	*/
	fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
/////////////
//	_CrtMemCheckpoint( &s1 );

	M_Node& children=g_vcBezierBase->children;
	children.length=1;
	int childDiff=point.length-children.length+1;

	if(childDiff<0){
		children.length+=childDiff;
	}
	else{
		for(int i=childDiff; --i>=0;){
			Transform* transform=new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		}
	}
	for(int i=point.length; --i>=0;){
		Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
		Vec3f pt=matrix*Vec3f(point[i].x, point[i].y, 0);
		transform->translation=pt;

		if(i==idx){
			transform->children.refValue()[0]=g_redPoint;
		}
		else{
			if(fix[i]){
				transform->children.refValue()[0]=g_cyanPoint;
			}
			else{
				transform->children.refValue()[0]=g_nextPoint;
			}
		}

		transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
	}
//	_CrtMemCheckpoint( &s2 );
//	int flag = _CrtMemDifference(&s3, &s1, &s2);

///////////
	if(dynamic_cast<BezierTape*>(this)){
		g_vcBezierBase->rotation=Rotation(0,1,0,-PI*.5);
	}
	else{
		g_vcBezierBase->rotation=Rotation();
	}

	g_visualClue=g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);

//	dLog(flag);
}
void Bezier3f::bezHouseKeep(){
	if(!point.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	float inverseSize=(1/g_scene->getZoom());
	int idx=index[0];

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length=line.length+point.length;//normA.length*5;
	for(int i=0; i<line.length; i++){
		coord[i]=matrix*Vec3f(line[i].x, line[i].y, line[i].z);
	}
	for(int j=line.length, i=point.length; --i>=0; j++){
		coord[j]=matrix*Vec3f(point[i].x, point[i].y, point[i].z);
	}
	fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
/////////////
	M_Node& children=g_vcBezierBase->children;
	children.length=1;
	int childDiff=point.length-children.length+1;

	if(childDiff<0){
		children.length+=childDiff;
	}
	else{
		for(int i=childDiff; --i>=0;){
			Transform* transform=new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		}
	}
	for(int i=point.length; --i>=0;){
		Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
		Vec3f pt=matrix*point[i];
		transform->translation=pt;

		if(i==idx){
			transform->children.refValue()[0]=g_redPoint;
		}
		else{
			if(fix[i]){
				transform->children.refValue()[0]=g_cyanPoint;
			}
			else{
				transform->children.refValue()[0]=g_nextPoint;
			}
		}

		transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
	}

///////////
	g_vcBezierBase->rotation=Rotation();

	g_visualClue=g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}
void BezierLathe::houseKeep(){
	bezHouseKeep();
}

void BezierLatheBuild::houseKeep(){
	if(g_transformCP->selectObject){
		m_patch[0]->houseKeep();
	}
	else{
		bezHouseKeep();
	}
}

void BezierScale::houseKeep(){
	bezHouseKeep();
}
void BezierCircumScale::houseKeep(){
	bezHouseKeep();
}
void BezierShift::houseKeep(){
	bezHouseKeep();
}
void BezierTwist::houseKeep(){
	bezHouseKeep();
}
void BezierTape::houseKeep(){
	bezHouseKeep();
}
void BezierBend::houseKeep(){
	bezHouseKeep();
}
Seamless* getSeamless(int& selLine);
ModelMorpher* getModelMorpher(){
	int selLine = g_stw->selLine;
	if(Seamless* selSmls = getSeamless(selLine)){
		selLine = getParent(selLine);
		if(ModelSequence* modelSequence = dynamic_cast<ModelSequence*>(g_sceneTreeList[selLine].node.p)){
			return dynamic_cast<ModelMorpher*>(g_sceneTreeList[getParent(selLine)].node.p);
		}
	}
	return 0;
}


CommonSpec* getCommonSpec(const Art* part);

void PartCageClue::genCage(){
	if(smls == 0){
		return;
	}
	M_Vec3f& point = dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;

	point.length = 0;
	M_Int& coordIndex = g_nurbsIls->coordIndex.refValue();
	coordIndex.length = 0;

	M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;

	g_nurbsIlsCoord->point.refValue().length = smls->pointTotal;
	M_Color3f& m_color = g_nurbsIlsColor->color.refValue();
	m_color.length = smls->pointTotal;
	smls->genVisibleMask();

	if(g_scene->mode & BACKFACE_IGNORE_MODE){
		for(int k = 0; k < cpNodeList.length; k++){
			NurbsPatch* patch = cpNodeList[k];
			patch->addStraightCage(g_nurbsIls, g_pointIsVisible);
		}
	}
	else{
		for(int k = 0; k < cpNodeList.length; k++){
			NurbsPatch* patch = cpNodeList[k];
			patch->addStraightCage(g_nurbsIls);
		}
	}

	for(int i = m_color.length; --i >= 0;){
		m_color[i] = Color3f(.5, .5, .5);
	}
	M_Art& partList = smls->partList;

	for(int k = partList.length; --k >= 0;){

		Art* part = partList[k];
		const M_Int& coordGroup=part->coordGroup;
		const M_ControlPoint& controlPoint = part->controlPoint;

		if(g_selNode.getRaw() == part){
			if(const CommonSpec* commonSpec = getCommonSpec(part)){
				int k = 0;
				Color3f groupColor;
				if(commonSpec->color.length){

					for(int cg = 0; cg < coordGroup.length; cg++){
						int idx = cg + commonSpec->color.length - coordGroup.length - 1;
						if(idx < commonSpec->color.length){
							groupColor = commonSpec->color[idx];
						}
						for(int c = coordGroup[cg]; --c >= 0 && k < controlPoint.length; k++){
							const ControlPoint& cp = controlPoint[k];
							int idx = cp.i + cp.p->fromPoint;
							m_color[idx] = groupColor;
						}
					}
				}
				if(commonSpec->color.length){
					groupColor = commonSpec->color[commonSpec->color.length - 1];
				}
				for(; k < part->controlPoint.length; k++){
					const ControlPoint& cp = controlPoint[k];
					int idx = cp.i + cp.p->fromPoint;
					m_color[idx] = groupColor;
				}
			}
		}
		else{
			for(int k = 0; k < part->controlPoint.length; k++){
				const ControlPoint& cp = controlPoint[k];
				int idx = cp.i + cp.p->fromPoint;
				m_color[idx] = Color3f(1, .9, .9);
			}
		}
	}
	smls->addCageControlPolys();
	g_nurbsIls->convertToRenderFormat(g_nurbsIlsShape->render);
}


void setAllIns(M_Build& m_build, const bool inState);
void storePoints(M_<NurbsPatch*>& cpNodeList);

void updateFuseSurfaceClue(){
	if(dynamic_cast<Art*>(g_selNode.getRaw())){
		g_vcArtTriBase->fuseSurface = 0;
		for(int k = g_sceneTreeList.length; --k>=0;){
			if(FuseSurface* fuseSurface = dynamic_cast<FuseSurface*>(g_sceneTreeList[k].node.p)){
				if(fuseSurface->polyMapAid || fuseSurface->polyMapAidMirror){
					g_vcArtTriBase->fuseSurface=fuseSurface;
					fuseSurface->houseKeep();
					break;
				}
			}
		}
	}
}


void Art::houseKeep(){
	g_nurbsIlsBase->rotation.refValue() = Rotation();
	g_nurbsIlsBase->translation.refValue() = Vec3f();
	g_nurbsIlsBase->scale.refValue() = Vec3f(1,1,1);


	if(g_stw->cp->node.p == niki){
		 niki->houseKeep();
		 return;
	 }

	if(g_scene->mode & PAINT_MODE){
		if(g_scene->mode & BRUSH_MODE){
			g_brush->houseKeep();
		}
		return;
	}
	if(g_addLathe->mousePhase != AddLathe::NEUTRAL){
		g_addLathe->houseKeep();
		return;
	}
	updateFuseSurfaceClue();
	if(smls->animCP && smls->workAnimCP == false && (g_scene->mode & POLY_EDIT_MODE) == false && (g_scene->mode & MODELLING_MODE) == false){
		M_<NurbsPatch*>& cpNodeList = smls->cpNodeList;
		if(smls->workAnimCP == false){
			setAllIns(smls->build.refValue(), false);
			smls->workAnimCP = true;
			storePoints(cpNodeList);
			smls->doBuild();
			smls->ctrf = true;
		}
	}

	float inverseSize=(1/g_scene->getZoom());
	g_vcArtTriBase->smls = smls;
	Anim* anim=Window::animMode();
	g_nurbsIlsBase->children.refValue().length = 1;

	if(MultiSelect::selectRectMode){
		IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcSelectRectShape->geometry.p);
		M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

		coord.length = 0;
		Matrix iViewMatrix;
		if(anim == 0 || smls->animCP){
			iViewMatrix = g_scene->getIViewMatrix();
		}
		coord+=iViewMatrix * Vec3f(MultiSelect::topLeft.x, MultiSelect::bottomRight.y, MultiSelect::topLeft.z);
		coord+=iViewMatrix * MultiSelect::bottomRight;
		coord+=iViewMatrix * Vec3f(MultiSelect::bottomRight.x, MultiSelect::topLeft.y, MultiSelect::topLeft.z);
		coord+=iViewMatrix * MultiSelect::topLeft;
		fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
		polyLine->coordIndex.refValue() += 0;
		polyLine->convertToRenderFormat(g_vcSelectRectShape->render);
		if(smls->workAnimCP){
			g_nurbsIlsBase->children.refValue() += g_vcSelectRectShape.p;
		}
	}


	if(smls->workAnimCP && (g_scene->mode & (ANIMATION_MODE | NURBS_OWNERSHIP_MODE | XRAY_EDIT_MODE))){
		g_partCageClue->smls = smls;
		g_visualClue = g_partCageClue;
		g_partCageClue->genCage();
		if(anim == 0){
			g_visualClue = g_partCageClue;
			return;
		}
	}
	if(anim && getModelMorpher() == 0){
	//if(g_allowNSPE == false && anim && getModelMorpher() == 0){
		g_vcArtAnimBase->houseKeep(this, anim);
		g_visualClue = g_vcArtAnimBase;
	}
	else{
		if(MultiSelect::coordList.length){
			Seamless* smls = g_vcArtTriBase->transpSmls;
			Art* part = smls->skeleton.refValue()[0];
		}
		if(dynamic_cast<ImmediatePart*>(this) && g_radial_Tug->active){
			float tugRadius = g_radial_Tug->radius * g_scene->scale.x;
			IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
			M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
			int numOfPoints = 129;
			coord.length=numOfPoints;
			for(int i=0; i<numOfPoints; i++){
				coord[i]=Vec3f(tugRadius*cos(i*(PI*2/(numOfPoints-1))), tugRadius*sin(i*(PI*2/(numOfPoints-1))),0);
			}
			g_immediateRadialTugResizeLocation = Vec3f(tugRadius,0,0);

			fillPolyLineIndex(coord, polyLine->coordIndex.refValue());
			//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
			polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
			g_radial_Tug->active = true;
		}
		

		g_vcArtTriBase->smls = smls;
		g_visualClue = g_vcArtTriBase;
	}
}

void CenterJoint::houseKeep(){
	g_visualClue = g_vcCenterJointBase;
}
void TransferOwnership::houseKeep(){
	g_visualClue = g_transferOwnershipClue;
}

void RingBuild::houseKeep(){
	int i=index[0];
	if(i< scaleX.length){
		Matrix matrix;
		getGoodQuadrant(matrix);
		g_vcRingBuildOrientationA->translation=matrix * Vec3f(-scaleX.refValue()[i].x,0,0);
		g_vcRingBuildOrientationB->translation=matrix * Vec3f(scaleX.refValue()[i].y,0,0);
		g_vcRingBuildOrientationC->translation=matrix * Vec3f(0,0,-scaleZ.refValue()[i].x);
		g_vcRingBuildOrientationD->translation=matrix * Vec3f(0,0,scaleZ.refValue()[i].y);
//		}
//		else{
//			g_vcRingBuildOrientationA->translation=matrix * Vec3f(0,0,-scaleZ.refValue()[i].x);
//			g_vcRingBuildOrientationB->translation=matrix * Vec3f(0,0,scaleZ.refValue()[i].y);
//		}
		float inverseSize=(1/g_scene->getZoom());
		g_vcRingBuildPivotPoint->scale=Vec3f(inverseSize, inverseSize, inverseSize);
		g_vcRingBuildOrientationA->scale=Vec3f(inverseSize, inverseSize, inverseSize);
		g_vcRingBuildOrientationB->scale=Vec3f(inverseSize, inverseSize, inverseSize);
		g_vcRingBuildOrientationC->scale=Vec3f(inverseSize, inverseSize, inverseSize);
		g_vcRingBuildOrientationD->scale=Vec3f(inverseSize, inverseSize, inverseSize);
		g_visualClue = g_vcRingBuildBase;
	}
}
void ColorSweep::houseKeep(){
	if(!distance.length){
		g_visualClue=0;
		return;
	}
	setMathHiPresision();
	int idx=index[0];
	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	M_Int& coordIndex = polyLine->coordIndex.refValue();
	coordIndex.length=0;

	M_Float& distance = this->distance.refValue();

	coord.length=0;
	coord.length=2;

	M_Node& children=g_vcBezierBase->children;
	children.length=0;
	children+=g_vcEllipticalPolyline2D.p;
	int childDiff=color.length-children.length+1;

	if(childDiff<0){
		children.length+=childDiff;
	}
	else{
		for(int i=childDiff; --i>=0;){
			Transform* transform=new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		}
	}
	float inverseSize=(1/g_scene->getZoom());
	float sum=0;
	for(int i = 0; i<color.length; i++){
		Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
		Vec3f pt=matrix*Vec3f(0, sum, 0);
		transform->translation=pt;
		if(i==0 ){
			coord[0] = pt;
		}

		if(i==idx){
			transform->children.refValue()[0]=g_redPoint;
		}
		else{
			transform->children.refValue()[0]=g_nextPoint;
		}

		transform->scale=Vec3f(inverseSize, inverseSize, inverseSize);

		if(i==distance.length){
			coord[1] = pt;
		}
		else{
			sum+=distance[i];
		}
	}
	fillPolyLineIndex(coord, polyLine->coordIndex.refValue());

///////////
	g_vcBezierBase->rotation=Rotation();

	g_visualClue=g_vcBezierBase;

	restoreMathPresision();
	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}
void EC2::ecHouseKeep(){
	if(!point.length)return;
	setMathHiPresision();
	float inverseSize=(1/g_scene->getZoom());
	g_vcECScaleTangentArrow->scale=Vec3f( inverseSize, inverseSize, inverseSize);

	int idx=index[0];
	processFields();

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& line=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

	genCurve( line);
	fillPolyLineIndex(line, polyLine->coordIndex.refValue());

	M_Node& children = g_vcECScaleBase->children;

	int childDiff = point.length-children.length+2;

	if(childDiff<0){
		children.length += childDiff;
	}
	else{
		for(int i = childDiff; --i>=0;){
			Transform* transform = new Transform;
			transform->children.refValue() += g_nextPoint.p; //could just add null?
			children += transform;
		}
	}
	for(int i=point.length; --i>=0;){
		Transform* transform=dynamic_cast<Transform*>(children[i+2].p);
		Vec3f pt = Vec3f(point[i].x, point[i].y, 0);
		transform->translation = pt;

		if(i == idx){
			transform->children.refValue()[0] = g_redPoint;
			g_vcECScaleTangentArrow->translation = pt;
			g_vcECScaleTangentArrow->rotation = Rotation(0,0,1,tangent[i]);
		}
		else{
			transform->children.refValue()[0] = g_nextPoint;
		}
		
		transform->scale = Vec3f(inverseSize, inverseSize, inverseSize);
	}

	g_vcECScaleBase->translation=location;
	g_vcECScaleBase->rotation=orientation;

	g_visualClue = g_vcECScaleBase;

	restoreMathPresision();

	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}
void EC3::ecHouseKeep(){
	if(!point.length)return;
	setMathHiPresision();
	float inverseSize=(1/g_scene->getZoom());
	g_vcECScaleTangentArrow->scale=Vec3f( inverseSize, inverseSize, inverseSize);

	int idx=index[0];
	processFields();

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);

	M_Vec3f& line=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;

	genCurve( line);
	fillPolyLineIndex(line, polyLine->coordIndex.refValue());

	M_Node& children=g_vcECScaleBase->children;

	int childDiff=point.length-children.length+2;

	if(childDiff<0){
		children.length+=childDiff;
	}
	else{
		for(int i=childDiff; --i>=0;){
			Transform* transform=new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		}
	}
	for(int i=point.length; --i>=0;){
		Transform* transform=dynamic_cast<Transform*>(children[i+2].p);
		Vec3f pt=Vec3f(point[i].x, point[i].y, 0);
		transform->translation=pt;

		if(i==idx){
			transform->children.refValue()[0]=g_redPoint;
			g_vcECScaleTangentArrow->translation=pt;
			g_vcECScaleTangentArrow->rotation=Rotation(0,0,1,tangent[i]);
		}
		else{
			transform->children.refValue()[0]=g_nextPoint;
		}
		
		transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
	}


/*
	{
		Rotation tangRot=Rotation(0,0,1,PI*.5);
		for( int i=1; i<pointX.length; i++){
			tangRot=Rotation(0,1,0,-yAngle[i-1])*Rotation(0,0,1,relTangent[i])*tangRot;

			if(i==idx){
				g_vcECScaleTangentArrow->rotation=tangRot;
				break;
			}
		}
	}
*/
	g_vcECScaleBase->translation=location;
	g_vcECScaleBase->rotation=orientation;

	g_visualClue=g_vcECScaleBase;

	restoreMathPresision();

	//g_vcEllipticalMaterial->diffuseColor=g_settings->backgroundColor.refValue().invert();
	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);
}

void ECLathe::houseKeep(){
	ecHouseKeep();
}
void ECScale::houseKeep(){
	ecHouseKeep();
}
void ECShift::houseKeep(){
	ecHouseKeep();
}
void ECSyncBend::houseKeep(){
	ecHouseKeep();
}
void EvolveCoord::houseKeep(){
	ecHouseKeep();
}
/*
void Progressor::houseKeep(){
	ecHouseKeep();
}
*/
void ECBend::houseKeep(){
	ecHouseKeep();
}
void ECTwist::houseKeep(){
	ecHouseKeep();
}

void CCBend::houseKeep(){

	if(!distance.length){
		return;
	}
	int index=this->index[0];
	if(index==0){
		index=1;
	}

	g_vcCCLathe->distance.refValue()[0]=distance[index-1];
	if(index<=1){
		g_vcSurfaceTransform->translation=Vec3f();
	}
	else {
		g_vcSurfaceTransform->translation=Vec3f(0,getDistance(index-2),0);
	}
	g_vcCCBend->distance.resize(distance.length);
	g_vcCCBend->yAngle.resize(curve.length);
	g_vcCCBend->curve.resize(curve.length);

	g_vcCCBend->distance.refValue()=distance;
	g_vcCCBend->curve.refValue()=curve;
	g_vcCCBend->yAngle.refValue()=yAngle;

	g_vcCCLathe->baseRad.refValue()=g_spaghettiRad*(1/g_scene->getZoom());

	g_vcBaseRad->translation=location+Vec3f(baseRad,0,0);
	g_vcBaseRad->rotation=orientation;

	commonEndCCVisualClue();
}
void CCTranslate::houseKeep(){
	if(!distance.length){
		return;
	}
	int index=this->index[0];
	if(index==0){
		index=1;
	}

	g_vcCCLathe->distance.refValue()[0]=distance[index-1];
	if(index<=1){
		g_vcSurfaceTransform->translation=Vec3f();
	}
	else {
		g_vcSurfaceTransform->translation=Vec3f(0,getDistance(index-2),0);
	}

	g_vcCCBend->distance.resize(distance.length);
	g_vcCCBend->yAngle.resize(curve.length);
	g_vcCCBend->curve.resize(curve.length);
	g_vcCCBend->distance.refValue()=distance;
	g_vcCCBend->curve.refValue()=curve;

	g_vcCCLathe->baseRad.refValue()=g_spaghettiRad*(1/g_scene->getZoom());

	g_vcBaseRad->translation=Vec3f(baseRad+location.x,location.y,location.z);
	g_vcBaseRad->rotation=orientation;
	commonEndCCVisualClue();
}
float getDistance(int index, const M_Float& distance){
	float distanceSum=0;
	int i;
	for(i=0;i<=index;i++){
		distanceSum+=distance[i];
	}
	return distanceSum;
}
void SineTug::houseKeep(){
	if(!distance.refValue().length){
		return;
	}
	setMathHiPresision();

	int index=this->index[0];
	int numOf = this->radius.refValue().length;

	float radius=this->radius[index];
	const M_Float& distance=this->distance;
	float xScale=this->xScale[index];

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length=2002;
	Matrix matrix = scaleMatrix(Vec3f(radius*xScale,radius,radius)) * rotationMatrix(orientation.refValue()[index])*translationMatrix(location.refValue()[index]);
	for(int i=0; i<1000; i++){
		coord[i] = matrix * Vec3f(cos(i/999.0*(PI*2)), 0, sin(i/999.0*(PI*2)));
	}
	
	coord[1000]=matrix*Vec3f(0,0,0);
	
	matrix = scaleMatrix(Vec3f(radius*xScale,radius,radius)) * translationMatrix(Vec3f(0,distance[index]*8,0)) * rotationMatrix(orientation.refValue()[index])*translationMatrix(location.refValue()[index]);

	coord[1001]=matrix*Vec3f(0,0,0);

	M_Int& coordIndex = polyLine->coordIndex;
	coordIndex.length = 2004;

	for(int i = 0; i<1000; i++){
		coordIndex[i] = i;
	}
	coordIndex[1000]=-1;
	coordIndex[1001]=1000;
	coordIndex[1002]=1001;
	coordIndex[1003]=-1;

	matrix = scaleMatrix(Vec3f(radius*xScale,radius,radius)) * translationMatrix(Vec3f(0,-decay[index],0)) * rotationMatrix(orientation.refValue()[index])*translationMatrix(location.refValue()[index]);
	for(int i=0; i<1000; i++){
		coord[1002+i] = matrix * Vec3f(cos(i/999.0*(PI*2)), 0, sin(i/999.0*(PI*2)));
	}
	for(int i = 0; i<1000; i++){
		coordIndex[1004+i] = 1002+i;
	}
	
	float inverseSize = (1/g_scene->getZoom());
	M_Node& children=g_vcBezierBase->children;
	children.length=1;
	int childDiff=numOf-children.length+1;

	if(childDiff<0){
		children.length+=childDiff;
	}
	else{
		for(int i=childDiff; --i>=0;){
			Transform* transform=new Transform;
			transform->children.refValue()+=g_nextPoint.p; //could just add null?
			children+=transform;
		}
	}
	for(int i=numOf; --i>=0;){
		Transform* transform=dynamic_cast<Transform*>(children[i+1].p);
		Vec3f pt=orientation[i]*Vec3f(0,distance[i]*8,0)+location[i];
		transform->translation=pt;

		if(i==index){
			transform->children.refValue()[0]=g_redPoint;
		}
		else{
			transform->children.refValue()[0]=g_nextPoint;
		}

		transform->scale=Vec3f( inverseSize, inverseSize, inverseSize);
	}

	g_vcBezierBase->rotation=Rotation();
	g_visualClue=g_vcBezierBase;

	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);

	restoreMathPresision();
}
void SimulSineTug::houseKeep(){
}
void RadialTug::houseKeep(){
	if(!location.refValue().length){
		return;
	}
	setMathHiPresision();

	int index=this->index[0];

	float radius=this->radius[index];
	Vec3f displacement=this->displacement[index];

	IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
	coord.length=1002;
	
	Matrix matrix = rotationMatrix( -g_scene->rotation) * scaleMatrix(Vec3f(radius,radius,radius)) * translationMatrix(location.refValue()[index]);
	for(int i=0; i<1000; i++){
		coord[i] = matrix * Vec3f(cos(i/999.0*(PI*2)), sin(i/999.0*(PI*2)),0);
	}
	
	coord[1000]=matrix*Vec3f(0,0,0);
	
	matrix = translationMatrix(location.refValue()[index]+displacement);

	coord[1001]=matrix*Vec3f(0,0,0);

	M_Int& coordIndex = polyLine->coordIndex;
	coordIndex.length = 1003;

	for(int i = 0; i<1000; i++){
		coordIndex[i] = i;
	}
	coordIndex[1000]=-1;
	coordIndex[1001]=1000;
	coordIndex[1002]=1001;
	
	g_vcBezierBase->rotation=Rotation();
	g_visualClue=g_vcBezierBase;

	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);

	restoreMathPresision();
}
void TransferBCyl::houseKeep(){
	if(in){
		g_vcTransferBCylCylinderBuild->height.refValue()=height;
		g_vcTransferBCylCylinderBuild->radius.refValue()=radius;

		M_Node& children = g_vcTransferBCylBase->children.refValue();

		IndexedLineSet* polyLine=dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
		M_Vec3f& coord=dynamic_cast<Coordinate*>(polyLine->coord.p)->point;
		coord.length=0;
		M_Int& coordIndex = polyLine->coordIndex.refValue();
		coordIndex.length=0;

		if(groupingHeight>0){
			coord+=Vec3f(-radius, groupingHeight, 0);
			coord+=Vec3f(radius, groupingHeight, 0);
			coord+=Vec3f(0, groupingHeight, -radius);
			coord+=Vec3f(0, groupingHeight, radius);
			coordIndex+=0;
			coordIndex+=1;
			coordIndex+=-1;
			coordIndex+=2;
			coordIndex+=3;
			coordIndex+=-1;
			polyLine->renderProcessed=false;
		}
		g_vcTransferBCylBase->rotation=orientation;
		g_vcTransferBCylBase->translation=location;

		g_vcTransferBCylCylinderBuild->processFields();
		g_vcTransferBCyl->process();
		g_vcTransferBCyl->doBuild();
		g_vcTransferBCyl->convertToRenderFormat();

		g_visualClue=g_transferBCylClue;
	}
	else{
		g_visualClue = 0;
	}
}
void houseKeepVisualClue(){
	if(VisualAid* visualAid = dynamic_cast<VisualAid*>(g_selNode.getRaw())){

		visualAid->houseKeep();
		
		


/*
		if(g_visualClue){
			String str;
			g_visualClue->toString(str);
			dLog(str);
		}
*/
	}
	else{
		g_visualClue = nullPtr;
	}
}
void houseKeepVisualClueViewChange(){
	g_viewMatrix = g_scene->getViewMatrix();
	g_transformCP->mode = TransformCP::NOTHING;
	if(RingBuild* ringBuild=dynamic_cast<RingBuild*>(g_selNode.getRaw())){
		ringBuild->houseKeep();
	}
	else if(g_scene->mode & (MODELLING_MODE | ANIMATION_MODE)){
		g_transformCP->resetRotationLever();
		g_vcArtAnimBase->resetRotationLever();
		//if(g_radial_Tug->active || g_scene->mode & BACKFACE_IGNORE_MODE){
		houseKeepVisualClue();
		//}
	}
	else if(g_scene->mode & SELECT_OBJECT_MODE){
		g_transformObj->calcRotCtrlPnts();
		g_transformObj->houseKeep();
	}
	else if(g_scene->mode & PAINT_MODE){
		if(g_scene->mode & BRUSH_MODE){
			g_brush->houseKeep();
		}
	}
}

void ColorEffect::breakColorJoin(Seamless& smls){
	Art* triPart=smls.actTriPart;
	M_Color3f& color=this->color.refValue();
	color+=color[triPart->triangle.refTexIndex(smls.ActTri*3+smls.ActTriCorner)];
	triPart->triangle.refTexIndex(smls.ActTri*3+smls.ActTriCorner)=color.length-1;
	smls.convertToRenderFormat();
	BackUp::monostable.beginCycle();
}
void MonoColorEffect::breakColorJoin(Seamless& smls){
	Art* triPart=smls.actTriPart;
	M_Color3f& color=this->color.refValue();
	color+=color[triPart->triangle.refTexIndex(smls.ActTri*3+smls.ActTriCorner)];
	triPart->triangle.refTexIndex(smls.ActTri*3+smls.ActTriCorner)=color.length-1;
	smls.convertToRenderFormat();
	BackUp::monostable.beginCycle();
}
void TextureEffect::breakColorJoin(Seamless& smls){
	Art* triPart=smls.actTriPart;
	M_Vec2f& texCoord=this->texCoord.refValue();
	texCoord+=texCoord[triPart->triangle.refTexIndex(smls.ActTri*3+smls.ActTriCorner)];
	triPart->triangle.refTexIndex(smls.ActTri*3+smls.ActTriCorner)=texCoord.length-1;
	smls.convertToRenderFormat();
	BackUp::monostable.beginCycle();
}

void findThinTriangle(Seamless* smls){
	float triThinAmount=.0005;
	M_Art& partList=smls->partList;
	for(int k=partList.length; --k>=0;){
		ArtPtr part=partList[k];
		M_Triangle& triangle=part->triangle;
		for(int i=triangle.length*3; --i>=0;){
			Art* p;
			p=triangle.refPart(i);
			Vec3f a=p->matrix*p->coord.refValue()[triangle.refIndex(i)];

			p=triangle.refPart(--i);
			Vec3f b=p->matrix*p->coord.refValue()[triangle.refIndex(i)];

			p=triangle.refPart(--i);
			Vec3f c=p->matrix*p->coord.refValue()[triangle.refIndex(i)];

			if(
				fabs(a.x-b.x)<triThinAmount&&
				fabs(a.y-b.y)<triThinAmount&&
				fabs(a.z-b.z)<triThinAmount||

				fabs(a.x-c.x)<triThinAmount&&
				fabs(a.y-c.y)<triThinAmount&&
				fabs(a.z-c.z)<triThinAmount||

				fabs(b.x-c.x)<triThinAmount&&
				fabs(b.y-c.y)<triThinAmount&&
				fabs(b.z-c.z)<triThinAmount
				){
				smls->ActTri=i/3;
				smls->ActCtrlPnt=triangle.refIndex(smls->ActTri*3);// corner a
				g_stw->setSelNodeLine(triangle.refPart(smls->ActTri*3));// corner a
				smls->ActTriCorner=0;// corner a
				smls->actTriPart=part;
				checkVertexBelongsToTriangle(dynamic_cast<Art*>(g_selNode.getRaw()), smls->actTriPart, smls->ActCtrlPnt, smls->ActTriCorner, smls->ActTri);
				makePointerEqualIndexLocation();
				return;
			}
		}
	}
}

void searchDoBuild(M_Node& children){
	for(int i=0;i<children.length;i++){
		Node* node = children[i];
		if(node->getType()==SEAMLESS){
			Seamless* smls=dynamic_cast<Seamless*>(node);
			smls->doBuild();
			smls->convertToRenderFormat();
		}
		else if(dynamic_cast<AGroup*>(node)!=0){
			searchDoBuild(dynamic_cast<AGroup*>(node)->children.refValue());
		}
		else if(node->getType()==PROTODECLARE){
			searchDoBuild(dynamic_cast<ProtoDeclare*>(node)->protoBody->children.refValue());
		}
	}
}

void SceneTree::pasteFromClipboard(){
	g_stw->textCell.pasteFromClipboard();
}
void ChatWin::pasteFromClipboard(){
	g_cw->textCell.pasteFromClipboard();
}
void ControlPanel::pasteFromClipboard(){
	if(ControlPanel* controlPanel=dynamic_cast<ControlPanel*>(Window::activeWindow.getActiveWindow())){
		controlPanel->textCell.pasteFromClipboard();
	}
}
CaretWindow* activeTextCell(){
	CaretWindow* textCell=0;
	for(int i=g_callBackList.length; --i>=0;){
		if(textCell=dynamic_cast<CaretWindow*>(g_callBackList[i])){
			break;
		}
	}
	return textCell;
}

void incCaret(){
	if(CaretWindow* tc=activeTextCell()){
		tc->incCaretPosNH();
	}
	else if(Anim* anim = Window::animMode()){
		anim->slider.refValue() += 1.0 / (anim->stepsPerSecond * anim->period);
		if(anim->slider > 1){
			anim->slider.refValue() = 1;
		}
		anim->quantizeSlider();
		anim->updateDisplayFrom();
	}
}

void decCaret(){
	if(CaretWindow* tc=activeTextCell()){
		tc->decCaretPosNH();
	}
	else if(Anim* anim = Window::animMode()){
		anim->slider.refValue() -= 1.0 / (anim->stepsPerSecond * anim->period);
		if(anim->slider < 0){
			anim->slider.refValue() = 0;
		}
		anim->quantizeSlider();
		anim->updateDisplayFrom();
	}
}

void homeCaret(){
	if(CaretWindow* tc = activeTextCell()){
		tc->doHome();
	}
	else if(Anim* anim = Window::animMode()){
		anim->slider = 0;
		anim->updateDisplayFrom();
	}
}
void endCaret(){
	if(CaretWindow* tc=activeTextCell()){
		tc->doEnd();
	}
	else if(Anim* anim = Window::animMode()){
		anim->slider = 1;
		anim->updateDisplayFrom();
	}
}
void deleteChar(){
	if(CaretWindow* tc=activeTextCell()){
		tc->doDelete();
	}
}

void incCaretShift(){
	if(CaretWindow* tc=activeTextCell()){
		tc->incCaretPos();
	}
}
void decCaretShift(){
	if(CaretWindow* tc=activeTextCell()){
		tc->decCaretPos();
	}
}
void homeCaretShift(){
	if(CaretWindow* tc=activeTextCell()){
		tc->doHomeShift();
	}
}
void endCaretShift(){
	if(CaretWindow* tc=activeTextCell()){
		tc->doEndShift();
	}
}
void deleteCharShift(){
	if(CaretWindow* tc=activeTextCell()){
		tc->doDeleteShift();
	}
}
void BezierPatch::mirrorPatch(){
	BackUp::monostable.beginCycle();

	int idx=index[numOfUGroup];
	int pointIdx=index[pointGroup];

	M_Vec3f& point=this->point.refValue();
	M_Vec2f& dim=this->dim.refValue();
	M_Vec2f& zone=this->zone.refValue();
//	M_Int& fixIdx=this->fixIdx.refValue();
//	M_Int& fixIdxIdx=this->fixIdxIdx.refValue();
	M_Int& link=this->link.refValue();

	int noU=dim[idx].x;
	int noV=dim[idx].y;

	int bp=getBasePoint(idx);
	int nBp=point.length;

	int pointLen=noU*noV;

	if(pointIdx-bp<noU-1){//bottom edge
		for(int v=0; v<noV; v++){
			for(int u=0; u<noU; u++){
				int sw=bp+(noV-1-v)*noU+u;
				int h=bp+u;
				point+=point[h]+(point[h]-point[sw]);
			}
		}
/*
		for(int u=0; u<noU; u++){
			int hingeA=u+bp;
			int swingA=hingeA+noU;
			int hingeB=pointLen-noU+u+nBp;
			int swingB=hingeB-noU;

			link+=swingA;
			link+=hingeA;
			link+=hingeB;
			link+=swingB;
		}
		*/
	}
	if(pointIdx-bp>pointLen-noU){//top edge
		for(int v=0; v<noV; v++){
			for(int u=0; u<noU; u++){
				int sw=bp+(noV-1-v)*noU+u;
				int h=bp+pointLen-noU+u;
				point+=point[h]+(point[h]-point[sw]);
			}
		}
		/*
		for(int u=0; u<noU; u++){
			int hingeA=bp+pointLen-noU+u;
			int swingA=hingeA-noU;
			int hingeB=nBp+u;
			int swingB=hingeB+noU;

			link+=swingA;
			link+=hingeA;
			link+=hingeB;
			link+=swingB;
		}
		*/
	}
	else{
		if(pointIdx%noU>(noU>>1)){//right edge
			for(int v=0; v<noV; v++){
				for(int u=noU; --u>=0;){
					int h=bp+v*noU+noU-1;
					int sw=bp+v*noU+u;
					point+=point[h]+(point[h]-point[sw]);
				}
			}
			/*
			for(int v=0; v<noV; v++){
				int hingeA=bp+v*noU+noU-1;;
				int swingA=hingeA-1;
				int hingeB=nBp+v*noU;
				int swingB=hingeB+1;

				link+=swingA;
				link+=hingeA;
				link+=hingeB;
				link+=swingB;
			}
			*/
		}
		else{
			for(int v=0; v<noV; v++){
				for(int u=noU; --u>=0;){
					int h=bp+v*noU;
					int sw=bp+v*noU+u;
					point+=point[h]+(point[h]-point[sw]);
				}
			}
			/*
			for(int v=0; v<noV; v++){
				int hingeA=bp+v*noU;;
				int swingA=hingeA+1;
				int hingeB=nBp+v*noU+noU-1;
				int swingB=hingeB-1;

				link+=swingA;
				link+=hingeA;
				link+=hingeB;
				link+=swingB;
			}
			*/
		}
	}
	dim+=Vec2f(noU, noV);
	zone+=zone[idx];
//	fixIdxIdx+=fixIdx.length;

	processFields();

	updateBuild3dWindow();
}
bool BezierPatch::findMatchingPoint(int exclude){
	M_Vec3f& point=this->point.refValue();
	M_Vec2f& dim=this->dim.refValue();
	M_Vec2f& zone=this->zone.refValue();
//	M_Int& fixIdx=this->fixIdx.refValue();
//	M_Int& fixIdxIdx=this->fixIdxIdx.refValue();
	M_Int& link=this->link.refValue();

	for(int k=0, bp=0; k<dim.length; k++){
		int noU=dim[k].x;
		int noV=dim[k].y;
		int pointLen=noU*noV;

		for(int v=0; v<noV; v++){//check for left edge
			int i=v*noU;
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i+1])){
					if(bp+i!=exclude){
						int j=link.length;
						link.length+=4;
						link[j+swingB]=bp+i+1;
						link[j+hingeB]=bp+i;
						return true;
					}
				}
			}
			i+=noU-1;//check for right edge
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i-1])){
					if(bp+i!=exclude){
						int j=link.length;
						link.length+=4;
						link[j+swingB]=bp+i-1;
						link[j+hingeB]=bp+i;
						return true;
					}
				}
			}
		}
		for(int u=0; u<noU; u++){//check for bottom edge
			int i=u;
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i+noU])){
					if(bp+i!=exclude){
						int j=link.length;
						link.length+=4;
						link[j+swingB]=bp+i+noU;
						link[j+hingeB]=bp+i;
						return true;
					}
				}
			}
			i+=pointLen-noU;//check for top edge
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i-noU])){
					if(bp+i!=exclude){
						int j=link.length;
						link.length+=4;
						link[j+swingB]=bp+i-noU;
						link[j+hingeB]=bp+i;
						return true;
					}
				}
			}
		}
		bp+=pointLen;
	}
	return false;
}
void BezierPatch::joinEdge(){
	MultiSelect::topLeft2d=convertPointToScreen2d(MultiSelect::topLeft);
	MultiSelect::bottomRight2d=convertPointToScreen2d(MultiSelect::bottomRight);

	M_Vec3f& point=this->point.refValue();
	M_Vec2f& dim=this->dim.refValue();
	M_Vec2f& zone=this->zone.refValue();
//	M_Int& fixIdx=this->fixIdx.refValue();
//	M_Int& fixIdxIdx=this->fixIdxIdx.refValue();
	M_Int& link=this->link.refValue();


	for(int k=0, bp=0; k<dim.length; k++){
		int noU=dim[k].x;
		int noV=dim[k].y;
		int pointLen=noU*noV;

		for(int v=0; v<noV; v++){//check for left edge
			int i=v*noU;
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i+1])){
					if(findMatchingPoint(bp+i)){
						int j=link.length-4;
						link[j+hingeA]=bp+i;
						link[j+swingA]=bp+i+1;
						doLink(point, link, bp+i, Vec3f());
					}
				}
			}
			i+=noU-1;//check for right edge
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i-1])){
					if(findMatchingPoint(bp+i)){
						int j=link.length-4;
						link[j+hingeA]=bp+i;
						link[j+swingA]=bp+i-1;
						doLink(point, link, bp+i, Vec3f());
					}
				}
			}
		}
		for(int u=0; u<noU; u++){//check for bottom edge
			int i=u;
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i+noU])){
					if(findMatchingPoint(bp+i)){
						int j=link.length-4;
						link[j+hingeA]=bp+i;
						link[j+swingA]=bp+i+noU;
						doLink(point, link, bp+i, Vec3f());
					}
				}
			}
			i+=pointLen-noU;//check for top edge
			if(MultiSelect::isInBox(point[bp+i])){
				if(MultiSelect::isInBox(point[bp+i-noU])){
					if(findMatchingPoint(bp+i)){
						int j=link.length-4;
						link[j+hingeA]=bp+i;
						link[j+swingA]=bp+i-noU;
						doLink(point, link, bp+i, Vec3f());
					}
				}
			}
		}
		bp+=pointLen;
	}
	refreshLink(link);
	houseKeep();
}
void BezierPatch::deletePatch(){
	M_Vec3f& point=this->point.refValue();
	M_Vec2f& dim=this->dim.refValue();
	M_Vec2f& zone=this->zone.refValue();
//	M_Int& fixIdx=this->fixIdx.refValue();
//	M_Int& fixIdxIdx=this->fixIdxIdx.refValue();
	M_Int& link=this->link.refValue();

	int idx = index[numOfUGroup];
	int pointIdx = index[pointGroup];
	int pointLen = dim[idx].x*dim[idx].y;
	int bp = m_bp[idx];

	point.deleteElement(bp, pointLen);
	dim.deleteElement(idx);
	zone.deleteElement(idx);
//	fixIdxIdx.deleteElement(idx);

	if(idx>=dim.length){
		idx=dim.length-1;
	}
	if(idx<0){
		idx=0;
	}
	index[numOfUGroup]=idx;

	if(pointIdx>=point.length){
		pointIdx=point.length-1;
	}
	if(pointIdx<0){
		pointIdx=0;
	}
	index[pointGroup]=pointIdx;

	processFields();

	updateBuild3dWindow();
}
/*
inline void FuseSurface::doIn(Stem* stem, Art* source, M_Vec2f& baseCoord, M_Int& srcMatch, M_Int& dstMatch, const float maxU, const float maxV, const float minU, const float minV, float* stemUKnotPtr, float* stemVKnotPtr, Vec3f* stemPointPtr, float* stemWeightPtr, int stemUOrder, int stemVOrder, int stemUDim, int stemVDim){
	if(stem->part.length){
		if(Art* part = dynamic_cast<Art*>(stem->part.refValue()[0].p)){
			const M_Vec3f& coord = part->coord;

			srcMatch.length=0;
			dstMatch.length=0;
			for(int i=0;i<baseCoord.length;i++){

				const Vec2f base0 = baseCoord[i];

				float u = (maxU-minU)*base0.x + minU;
				float v = (maxV-minV)*base0.y + minV;
				Vec3f curvedEdgePoint;
				SurfacePoint(u, v, stemUOrder, stemVOrder, stemUDim, stemVDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, &curvedEdgePoint);

				for(int j=0;j<coord.length;j++){
					Vec3f crd=coord[j];
					if(
						fabs(crd.x-curvedEdgePoint.x)<range&&
						fabs(crd.y-curvedEdgePoint.y)<range&&
						fabs(crd.z-curvedEdgePoint.z)<range
						){
						srcMatch+=j;
						dstMatch+=i;
					}
				}
			}
		}
	}
}
*/
inline void FuseSurface::doIn(Stem* stem, M_Vec2f& baseCoord, M_Int& srcMatch, M_Int& dstMatch){
	if(stem->source.length){
		if(ImmediatePart* source = dynamic_cast<ImmediatePart*>(stem->source.refValue()[0].p)){
			const M_Vec3f& coord = source->coord;

			srcMatch.length=0;
			dstMatch.length=0;
			for(int i=0;i<baseCoord.length;i++){

				const Vec2f base0 = baseCoord[i];

				for(int j=0;j<coord.length;j++){
					Vec3f crd=coord[j];
					if(
						fabs(crd.x-base0.x)<range&&
						fabs(crd.y-base0.y)<range
						){
						srcMatch+=j;
						dstMatch+=i;
					}
				}
			}
		}
	}
	else if(stem->coord.length){
		M_Vec2f& coord = stem->coord.refValue();
		srcMatch.length=0;
		dstMatch.length=0;
		for(int i=0;i<baseCoord.length;i++){

			const Vec2f base0 = baseCoord[i];

			for(int j=0;j<coord.length;j++){
				Vec2f crd=coord[j];
				if(
					fabs(crd.x-base0.x)<range&&
					fabs(crd.y-base0.y)<range
					){
					srcMatch+=j;
					dstMatch+=i;
				}
			}
		}
	}
}

/*
inline void FuseSurface::guessPoints(NurbsPatch* patch, Vec3f amount, const M_Vec3f& parentFlatCoord, const M_Vec3f& baseCurvedPoint, float& guessSumDiff, float& currentSumDiff, const float maxU, const float maxV, const float minU, const float minV, float* uKnotPtr, float* vKnotPtr, Vec3f* pointPtr, float* weightPtr, Vec3f* topPointPtr, float* topWeightPtr, Vec3f* r_pointPtr, float* r_weightPtr, int uOrder, int vOrder, int uDegree, int vDegree, int uDim, int vDim){
	Vec3f dummy;
	for(int j = 1; j < uDim; j+=2){
		guessSumDiff = 0;
		Vec3f storePoint = pointPtr[j];
		pointPtr[j] += amount;
		patch->processFields();
		for(int i = srcMatch.length; --i>=0;){
			int d = dstMatch[i];
			guessSumDiff += getDiff(Vec2f(parentFlatCoord[d].x, parentFlatCoord[d].y), dummy, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, baseCurvedPoint[srcMatch[i]]);
		}
		if(guessSumDiff < currentSumDiff){
			currentSumDiff = guessSumDiff;
		}
		else{
			pointPtr[j] = storePoint;
		}
	}
}
*/
inline void FuseSurface::guessPoints(NurbsPatch* patch, Vec3f amount, const M_Vec3f& parentFlatCoord, const M_Vec3f& baseCurvedPoint, float& guessSumDiff, float& currentSumDiff, const float maxU, const float maxV, const float minU, const float minV, float* uKnotPtr, float* vKnotPtr, Vec3f* pointPtr, float* weightPtr, Vec3f* topPointPtr, float* topWeightPtr, Vec3f* r_pointPtr, float* r_weightPtr, int uOrder, int vOrder, int uDegree, int vDegree, int uDim, int vDim){
	Vec3f dummy;
	for(int j = 1; j < uDim; j+=2){
		guessSumDiff = 0;
		Vec3f storePoint = pointPtr[j];
		pointPtr[j] += amount;
		patch->processFields();
		for(int i = qty; --i>=0;){
			guessSumDiff += getDiff(Vec2f(parentFlatCoord[i].x, parentFlatCoord[i].y), dummy, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, baseCurvedPoint[i]);
		}
		if(guessSumDiff < currentSumDiff){
			currentSumDiff = guessSumDiff;
		}
		else{
			pointPtr[j] = storePoint;
		}
	}
}
void FuseSurface::doFuse(NurbsPatch& parentPatch, Stem& parentStem){
	this->parentPatch = &parentPatch;
	this->parentStem = &parentStem;
	NurbsPatch* childPatch = dynamic_cast<NurbsPatch*>(this->patch.p);
	M_Vec3f* childCoordPtr = 0;
	Seamless* smls = 0;
	const bool southPole = parentStem.southPole;
	const bool northPole = parentStem.northPole;
	if(childPatch){
		for(int s = 0; s < childPatch->stem.length; s++){
			if(Stem* childStem = dynamic_cast<Stem*>(childPatch->stem[s].p)){
				if(s == 0 && childStem->mould){
					childStem->doBuild(this, childPatch);
					if(childStem->part.length){
						if(Art* part = childStem->part[0].p){
							childCoordPtr = &part->coord.refValue();
							smls = part->smls;
						}
					}
				}
				else{
					childStem->doBuild(0, childPatch);
				}
			}
		}
	}
	else{
		return;
	}
	if(childCoordPtr == 0){
		return;
	}
	M_Vec3f& childCoord = *childCoordPtr;

	M_Vec2f& baseCoord = this->baseCoord.refValue();
	if(baseCoord.length == 0){
		return;
	}
	if(spread){
		spread.refValue()=false;
		doSpread();
	}

	const float vSpan=this->vSpan;

	int uOrder = parentPatch.order.x;
	int vOrder = parentPatch.order.y;
	int uDegree = parentPatch.uDegree;
	int vDegree = parentPatch.vDegree;
	int uDim = parentPatch.dim.x;
	int vDim = parentPatch.dim.y;

	float* uKnotPtr = parentPatch.uKnot.refValue().getArray();
	float* vKnotPtr = parentPatch.vKnot.refValue().getArray();
	Vec3f* pointPtr = parentPatch.point.refValue().getArray();
	float* weightPtr = parentPatch.weight.refValue().getArray();

	Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
	float* topWeightPtr = weightPtr+uDim*vDim-uDim;

	Vec3f* r_pointPtr = parentPatch.r_point.getArray();
	float* r_weightPtr = parentPatch.r_weight.getArray();


	const float coordChange = this->coordChange;

	int count = vPan.length;

	curvedEdgePoint.length = count;

	Vec2f diff;
	const int idx=index[0];

	Vec3f curvedEdgePoint0;

	float maxU = parentPatch.maxU();
	float maxV = parentPatch.maxV();
	float minU = parentPatch.minU();
	float minV = parentPatch.minV();

	if(bond){
		if(northPole){
			for(int i=0; i<count; i++){
				const Vec2f base0 = baseCoord[i];

				float diffArray[4];
				Vec3f destPoint[4];

				float error = getDiff(convertNorthPoleToMercator(base0), curvedEdgePoint0, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);

				Vec2f basePointOffset[4] = {
					Vec2f(-coordChange, 0),
					Vec2f( coordChange, 0),

					Vec2f(0,-coordChange),
					Vec2f(0, coordChange),
				};

				diffArray[0] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[0]), destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[1] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[1]), destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[2] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[2]), destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[3] = getDiff(convertNorthPoleToMercator(base0 + basePointOffset[3]), destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);


				float leastDiff = diffArray[0];
				int k = 0;

				for(int j = 1; j < 4; j++){
					float curDiff = diffArray[j];
					if(curDiff < leastDiff){
						leastDiff = curDiff;
						k = j;
					}
				}
				if(i == idx){
					this->diff.refValue().x = diffArray[k];
				}

				float a = 1.0 - vPan[i]/vSpan;
				//float pan = sqrt(1.0 - a * a);
				float pan = cos(a * PI) * .5 + .5;
				float ipan = 1.0 - pan;

				if(error >= diffArray[k]){
					baseCoord[i] += basePointOffset[k];
					if(bond){
						childCoord[i]=destPoint[k]*ipan+childCoord[i]*pan;
					}
				}
				else{
					if(bond){
						childCoord[i]=curvedEdgePoint0*ipan+childCoord[i]*pan;
					}
				}
			}
		}
		else if(southPole){
			for(int i=0; i<count; i++){
				const Vec2f base0 = baseCoord[i];

				float diffArray[4];
				Vec3f destPoint[4];

				float error = getDiff(convertSouthPoleToMercator(base0), curvedEdgePoint0, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);

				Vec2f basePointOffset[4] = {
					Vec2f(-coordChange, 0),
					Vec2f( coordChange, 0),

					Vec2f(0,-coordChange),
					Vec2f(0, coordChange),
				};

				diffArray[0] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[0]), destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[1] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[1]), destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[2] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[2]), destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[3] = getDiff(convertSouthPoleToMercator(base0 + basePointOffset[3]), destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);


				float leastDiff = diffArray[0];
				int k = 0;

				for(int j = 1; j < 4; j++){
					float curDiff = diffArray[j];
					if(curDiff < leastDiff){
						leastDiff = curDiff;
						k = j;
					}
				}
				if(i == idx){
					this->diff.refValue().x = diffArray[k];
				}

				float a = 1.0 - vPan[i]/vSpan;
				//float pan = sqrt(1.0 - a * a);
				float pan = cos(a * PI) * .5 + .5;
				float ipan = 1.0 - pan;

				if(error >= diffArray[k]){
					baseCoord[i] += basePointOffset[k];
					if(bond){
						childCoord[i]=destPoint[k]*ipan+childCoord[i]*pan;
					}
				}
				else{
					if(bond){
						childCoord[i]=curvedEdgePoint0*ipan+childCoord[i]*pan;
					}
				}
			}
		}
		else{
			for(int i=0; i<count; i++){
				const Vec2f base0 = baseCoord[i];

				float diffArray[4];
				Vec3f destPoint[4];

				float error = getDiff(base0, curvedEdgePoint0, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);

				Vec2f basePointOffset[4] = {
					Vec2f(-coordChange, 0),
					Vec2f( coordChange, 0),

					Vec2f(0,-coordChange),
					Vec2f(0, coordChange),
				};

				diffArray[0] = getDiff(base0 + basePointOffset[0], destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[1] = getDiff(base0 + basePointOffset[1], destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[2] = getDiff(base0 + basePointOffset[2], destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
				diffArray[3] = getDiff(base0 + basePointOffset[3], destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);


				float leastDiff = diffArray[0];
				int k = 0;

				for(int j = 1; j < 4; j++){
					float curDiff = diffArray[j];
					if(curDiff < leastDiff){
						leastDiff = curDiff;
						k = j;
					}
				}
				if(i == idx){
					this->diff.refValue().x = diffArray[k];
				}

				float a = 1.0 - vPan[i]/vSpan;
				//float pan = sqrt(1.0 - a * a);
				float pan = cos(a * PI) * .5 + .5;
				float ipan = 1.0 - pan;

				if(error >= diffArray[k]){
					baseCoord[i] += basePointOffset[k];
					if(bond){
						childCoord[i]=destPoint[k]*ipan+childCoord[i]*pan;
					}
				}
				else{
					if(bond){
						childCoord[i]=curvedEdgePoint0*ipan+childCoord[i]*pan;
					}
				}
			}
		}
	}
	if(parentStem.source.length){
		if(Art* source = dynamic_cast<Art*>(parentStem.source.refValue()[0].p)){
			M_Vec2f& baseCoord = this->baseCoord.refValue();
			M_Int& srcMatch = this->srcMatch.refValue();
			M_Int& dstMatch = this->dstMatch.refValue();
			if(in){
				//doIn(&parentStem, source, baseCoord, srcMatch, dstMatch, maxU, maxV, minU, minV, parentUKnotPtr, parentVKnotPtr, parentPointPtr, parentWeightPtr, parentUOrder, parentVOrder, parentUDim, parentVDim);
				doIn(&parentStem, baseCoord, srcMatch, dstMatch);
			}
			M_Vec3f& coord = source->coord.refValue();
			if(snap){
				if(in == false && smls->autoIn){
					in.refValue()=true;
					doIn(&parentStem, baseCoord, srcMatch, dstMatch);
				}
				/*
				if(in == false && g_potentialTrouble){
					int len = coord.length;
					for(int i = srcMatch.length; --i>=0;){
						int si = srcMatch[i];
						if(si<len){
							Vec3f srcCrd = coord[si];
							Vec2f dstCrd = baseCoord[dstMatch[i]];
							if(	fabs(srcCrd.x-dstCrd.x)<range&&
								fabs(srcCrd.y-dstCrd.y)<range){

								continue;
							}
						}
						in.refValue()=true;
						doIn(&parentStem, baseCoord, srcMatch, dstMatch);
						break;
					}
				}
				*/
				for(int i = srcMatch.length; --i>=0;){
					int d = dstMatch[i];
					coord[srcMatch[i]] = Vec3f(baseCoord[d].x, baseCoord[d].y, 0);
				}
			}
/*
			if(evolve && pointChange > 0){
				if(parentStem.part.length){
					if(Art* part = dynamic_cast<Art*>(parentStem.part.refValue()[0].p)){

						int uOrder = childPatch->order.x;
						int vOrder = childPatch->order.y;
						int uDegree = childPatch->uDegree;
						int vDegree = childPatch->vDegree;
						int uDim = childPatch->dim.x;
						int vDim = childPatch->dim.y;

						float* uKnotPtr = childPatch->uKnot.refValue().getArray();
						float* vKnotPtr = childPatch->vKnot.refValue().getArray();
						Vec3f* pointPtr = childPatch->point.refValue().getArray();
						float* weightPtr = childPatch->weight.refValue().getArray();

						Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
						float* topWeightPtr = weightPtr+uDim*vDim-uDim;

						Vec3f* r_pointPtr = childPatch->r_point.getArray();
						float* r_weightPtr = childPatch->r_weight.getArray();


						const M_Vec3f& baseCurvedPoint = part->coord;
						float currentSumDiff=0;
						float guessSumDiff=0;
						Vec3f dummy;
						for(int i = srcMatch.length; --i>=0;){
							int d = dstMatch[i];
							currentSumDiff += getDiff(Vec2f(parentFlatCoord[d].x, parentFlatCoord[d].y), dummy, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
						}
						guessPoints(childPatch, Vec3f(-pointChange, 0, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
						guessPoints(childPatch, Vec3f(pointChange, 0, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

						guessPoints(childPatch, Vec3f(0, -pointChange, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
						guessPoints(childPatch, Vec3f(0, pointChange, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

						guessPoints(childPatch, Vec3f(0, 0, -pointChange), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
						guessPoints(childPatch, Vec3f(0, 0, pointChange), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

						sumDiff.refValue()=currentSumDiff;
						averageDiff.refValue()=currentSumDiff/qty;
						childPatch->processFields();
					}
				}
			}
			*/
		}
	}
	else if(parentStem.coord.length){
		M_Vec2f& coord = parentStem.coord.refValue();
		M_Vec2f& baseCoord = this->baseCoord.refValue();
		M_Int& srcMatch = this->srcMatch.refValue();
		M_Int& dstMatch = this->dstMatch.refValue();
		if(in){
			doIn(&parentStem, baseCoord, srcMatch, dstMatch);
		}
		if(snap){
			if(in == false && smls->autoIn){
				in.refValue()=true;
				doIn(&parentStem, baseCoord, srcMatch, dstMatch);
			}
			for(int i = srcMatch.length; --i>=0;){
				int d = dstMatch[i];
				coord[srcMatch[i]] = Vec2f(baseCoord[d].x, baseCoord[d].y);
			}
		}
	}
	if(evolve && pointChange > 0 && childCoord.length >= qty){

		if(bond == false){
			if(northPole){
				for(int i=0; i<qty; i++){
					childCoord[i]=getSurfaceVertex(convertNorthPoleToMercator(baseCoord[i]), maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr);
				}
			}
			else if(southPole){
				for(int i=0; i<qty; i++){
					childCoord[i]=getSurfaceVertex(convertSouthPoleToMercator(baseCoord[i]), maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr);
				}
			}
			else{
				for(int i=0; i<qty; i++){
					childCoord[i]=getSurfaceVertex(baseCoord[i], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr);
				}
			}			
		}

		int uOrder = childPatch->order.x;
		int vOrder = childPatch->order.y;
		int uDegree = childPatch->uDegree;
		int vDegree = childPatch->vDegree;
		int uDim = childPatch->dim.x;
		int vDim = childPatch->dim.y;

		float* uKnotPtr = childPatch->uKnot.refValue().getArray();
		float* vKnotPtr = childPatch->vKnot.refValue().getArray();
		Vec3f* pointPtr = childPatch->point.refValue().getArray();
		float* weightPtr = childPatch->weight.refValue().getArray();

		Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
		float* topWeightPtr = weightPtr+uDim*vDim-uDim;

		Vec3f* r_pointPtr = childPatch->r_point.getArray();
		float* r_weightPtr = childPatch->r_weight.getArray();

		float currentSumDiff=0;
		float guessSumDiff=0;
		Vec3f dummy;
		for(int i = qty; --i>=0;){
			currentSumDiff += getDiff(Vec2f(parentFlatCoord[i].x, parentFlatCoord[i].y), dummy, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, childCoord[i]);
		}
		guessPoints(childPatch, Vec3f(-pointChange, 0, 0), parentFlatCoord, childCoord, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
		guessPoints(childPatch, Vec3f(pointChange, 0, 0), parentFlatCoord, childCoord, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

		guessPoints(childPatch, Vec3f(0, -pointChange, 0), parentFlatCoord, childCoord, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
		guessPoints(childPatch, Vec3f(0, pointChange, 0), parentFlatCoord, childCoord, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

		guessPoints(childPatch, Vec3f(0, 0, -pointChange), parentFlatCoord, childCoord, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
		guessPoints(childPatch, Vec3f(0, 0, pointChange), parentFlatCoord, childCoord, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

		sumDiff.refValue()=currentSumDiff;
		averageDiff.refValue()=currentSumDiff/qty;
		childPatch->processFields();
	}

	if((polyMapAid || polyMapAidMirror) && g_visualClue == g_vcArtTriBase && g_vcArtTriBase->fuseSurface==this){
		this->houseKeep();
	}
	else if(g_selNode.getRaw()==this){
		this->houseKeep();
	}
}
void FuseSurface::doFuse(M_Vec3f& coord, NurbsPatch& branchPatch){//old
	this->parentPatch = 0;
	M_Vec2f& baseCoord = this->baseCoord.refValue();
	//Matrix imatrix = rotationMatrix(-orientation);
	const float vSpan=this->vSpan;
	NurbsPatch* stemPatch = dynamic_cast<NurbsPatch*>(this->patch.p);
	if(stemPatch==0){
		if(Stem* stem = dynamic_cast<Stem*>(this->patch.p)){
			stemPatch = dynamic_cast<NurbsPatch*>(stem->surface.p);
		}
	}
	if(stemPatch){
		int uOrder = stemPatch->order.x;
		int vOrder = stemPatch->order.y;
		int uDegree = stemPatch->uDegree;
		int vDegree = stemPatch->vDegree;
		int uDim = stemPatch->dim.x;
		int vDim = stemPatch->dim.y;

		float* uKnotPtr = stemPatch->uKnot.refValue().getArray();
		float* vKnotPtr = stemPatch->vKnot.refValue().getArray();
		Vec3f* pointPtr = stemPatch->point.refValue().getArray();
		float* weightPtr = stemPatch->weight.refValue().getArray();

		Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
		float* topWeightPtr = weightPtr+uDim*vDim-uDim;

		Vec3f* r_pointPtr = stemPatch->r_point.getArray();
		float* r_weightPtr = stemPatch->r_weight.getArray();

		const float coordChange = this->coordChange;

		int count = vPan.length;

		curvedEdgePoint.length = count;

		Vec2f diff;
		const int idx=index[0];

		Vec3f curvedEdgePoint0;

		float maxU = stemPatch->maxU();
		float maxV = stemPatch->maxV();
		float minU = stemPatch->minU();
		float minV = stemPatch->minV();

		for(int i=0; i<count; i++){
			Vec2f base0 = baseCoord[i];


			float diffArray[4];
			Vec3f destPoint[4];

			float error = getDiff(baseCoord[i], curvedEdgePoint0, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, coord[i]);

			Vec2f basePointOffset[4] = {
				Vec2f(-coordChange, 0),
				Vec2f( coordChange, 0),

				Vec2f(0,-coordChange),
				Vec2f(0, coordChange),

				//Vec2f(-coordChange, -coordChange),
				//Vec2f(-coordChange,  coordChange),

				//Vec2f( coordChange, -coordChange),
				//Vec2f( coordChange,  coordChange),
			};

			diffArray[0] = getDiff(baseCoord[i] + basePointOffset[0], destPoint[0], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, coord[i]);
			diffArray[1] = getDiff(baseCoord[i] + basePointOffset[1], destPoint[1], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, coord[i]);
			diffArray[2] = getDiff(baseCoord[i] + basePointOffset[2], destPoint[2], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, coord[i]);
			diffArray[3] = getDiff(baseCoord[i] + basePointOffset[3], destPoint[3], maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, coord[i]);

//			diffArray[4] = getDiff(baseCoord[i] + basePointOffset[4], destPoint[4], maxU, maxV, minU, minV, stemUOrder, stemVOrder, stemUDim, stemVDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, coord[i]);
//			diffArray[5] = getDiff(baseCoord[i] + basePointOffset[5], destPoint[5], maxU, maxV, minU, minV, stemUOrder, stemVOrder, stemUDim, stemVDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, coord[i]);
//			diffArray[6] = getDiff(baseCoord[i] + basePointOffset[6], destPoint[6], maxU, maxV, minU, minV, stemUOrder, stemVOrder, stemUDim, stemVDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, coord[i]);
//			diffArray[7] = getDiff(baseCoord[i] + basePointOffset[7], destPoint[7], maxU, maxV, minU, minV, stemUOrder, stemVOrder, stemUDim, stemVDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, coord[i]);

			float leastDiff = diffArray[0];
			int k = 0;

			for(int j = 1; j < 4; j++){
				float curDiff = diffArray[j];
				if(curDiff < leastDiff){
					leastDiff = curDiff;
					k = j;
				}
			}
			if(i == idx){
				this->diff.refValue().x = diffArray[k];
			}

			float a = 1.0 - vPan[i]/vSpan;
			//float pan = sqrt(1.0 - a * a);
			float pan = cos(a * PI) * .5 + .5;
			float ipan = 1.0 - pan;

			if(error >= diffArray[k]){
				baseCoord[i] += basePointOffset[k];
				if(bond){
					coord[i]=destPoint[k]*ipan+coord[i]*pan;
				}
			}
			else{
				if(bond){
					coord[i]=curvedEdgePoint0*ipan+coord[i]*pan;
				}
			}
		}
		if(Stem* stem = dynamic_cast<Stem*>(this->patch.refValue().p)){
			if(stem->source.length){
				if(Art* source = dynamic_cast<Art*>(stem->source.refValue()[0].p)){
					M_Vec2f& baseCoord = this->baseCoord.refValue();
					M_Int& srcMatch = this->srcMatch.refValue();
					M_Int& dstMatch = this->dstMatch.refValue();
					if(in){
						//old way never had an in?
						//doIn(stem, source, baseCoord, srcMatch, dstMatch, maxU, maxV, minU, minV, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, stemUOrder, stemVOrder, stemUDim, stemVDim);
					}
					M_Vec3f& coord = source->coord.refValue();
					if(snap){
						for(int i = srcMatch.length; --i>=0;){
							int d = dstMatch[i];
							coord[srcMatch[i]] = Vec3f(baseCoord[d].x, baseCoord[d].y, 0);
						}
					}
					if(evolve && pointChange > 0){
						if(stem->part.length){
							if(Art* part = dynamic_cast<Art*>(stem->part.refValue()[0].p)){

								int uOrder = branchPatch.order.x;
								int vOrder = branchPatch.order.y;
								int uDegree = branchPatch.uDegree;
								int vDegree = branchPatch.vDegree;
								int uDim = branchPatch.dim.x;
								int vDim = branchPatch.dim.y;

								float* uKnotPtr = branchPatch.uKnot.refValue().getArray();
								float* vKnotPtr = branchPatch.vKnot.refValue().getArray();
								Vec3f* pointPtr = branchPatch.point.refValue().getArray();
								float* weightPtr = branchPatch.weight.refValue().getArray();

								Vec3f* topPointPtr = pointPtr+uDim*vDim-uDim;
								float* topWeightPtr = weightPtr+uDim*vDim-uDim;

								Vec3f* r_pointPtr = branchPatch.r_point.getArray();
								float* r_weightPtr = branchPatch.r_weight.getArray();

								float maxU = branchPatch.maxU();
								float maxV = branchPatch.maxV();
								float minU = branchPatch.minU();
								float minV = branchPatch.minV();


								const M_Vec3f& baseCurvedPoint = part->coord;
								float currentSumDiff=0;
								float guessSumDiff=0;
								Vec3f dummy;
								for(int i = srcMatch.length; --i>=0;){
									int d = dstMatch[i];
									currentSumDiff += getDiff(Vec2f(parentFlatCoord[d].x, parentFlatCoord[d].y), dummy, maxU, maxV, minU, minV, uOrder, vOrder, uDegree, vDegree, uDim, vDim, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, baseCurvedPoint[srcMatch[i]]);
								}
								guessPoints(&branchPatch, Vec3f(-pointChange, 0, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
								guessPoints(&branchPatch, Vec3f(pointChange, 0, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

								guessPoints(&branchPatch, Vec3f(0, -pointChange, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
								guessPoints(&branchPatch, Vec3f(0, pointChange, 0), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

								guessPoints(&branchPatch, Vec3f(0, 0, -pointChange), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);
								guessPoints(&branchPatch, Vec3f(0, 0, pointChange), parentFlatCoord, baseCurvedPoint, guessSumDiff, currentSumDiff, maxU, maxV, minU, minV, uKnotPtr, vKnotPtr, pointPtr, weightPtr, topPointPtr, topWeightPtr, r_pointPtr, r_weightPtr, uOrder, vOrder, uDegree, vDegree, uDim, vDim);

								sumDiff.refValue()=currentSumDiff;
								averageDiff.refValue()=currentSumDiff/qty;
								branchPatch.processFields();
							}
						}
					}
				}
			}
		}
		if((polyMapAid || polyMapAidMirror) && g_visualClue == g_vcArtTriBase && g_vcArtTriBase->fuseSurface==this){
			this->houseKeep();
		}
		else if(g_selNode.getRaw()==this){
			this->houseKeep();
		}
	}
}

void clearAllFuseSurfaceClues(FuseSurface* block){
	g_vcArtTriBase->fuseSurface = 0;
	for(int k = g_sceneTreeList.length; --k>=0;){
		if(FuseSurface* fuseSurface = dynamic_cast<FuseSurface*>(g_sceneTreeList[k].node.p)){
			if(block!=fuseSurface){
				fuseSurface->polyMapAid.refValue()=false;
				fuseSurface->polyMapAidMirror.refValue()=false;
			}
		}
	}
}
void FuseSurface::PolyMapAid::operator=(const bool& par){
	refValue()=par;
	home.polyMapAidMirror.refValue()=false;
	if(par){
		g_vcArtTriBase->fuseSurface=&home;
	}
	clearAllFuseSurfaceClues(&home);
}
void FuseSurface::PolyMapAidMirror::operator=(const bool& par){
	refValue()=par;
	home.polyMapAid.refValue()=false;
	if(par){
		g_vcArtTriBase->fuseSurface=&home;
	}
	clearAllFuseSurfaceClues(&home);
}
//#include <time.h>
template <class T> void inline updateFlatVertices(NurbsPatch& nurbsPatch, M_<T>& flatCoord, const M_Vec3f& guessCoord, const M_Vec3f& originalCoord, const int& bits, const float& coordChange){
	float* stemUKnotPtr = nurbsPatch.uKnot.refValue().getArray();
	float* stemVKnotPtr = nurbsPatch.vKnot.refValue().getArray();
	Vec3f* stemPointPtr = nurbsPatch.point.refValue().getArray();
	float* stemWeightPtr = nurbsPatch.weight.refValue().getArray();
	int uOrder = nurbsPatch.order.x;
	int vOrder = nurbsPatch.order.y;
	int uDim = nurbsPatch.dim.x;
	int vDim = nurbsPatch.dim.y;

	float maxU = nurbsPatch.maxU();
	float maxV = nurbsPatch.maxV();
	float minU = nurbsPatch.minU();
	float minV = nurbsPatch.minV();

	for(int i = flatCoord.length; --i>=0;){
		T pt = flatCoord[i];

		if(1 & bits){
			pt.x-=coordChange;
		}
		if(2 & bits){
			pt.x+=coordChange;
		}
		//if(pt.y > 0 && pt.y < 1){
			if(4 & bits){
				pt.y-=coordChange;
			}
			if(8 & bits){
				pt.y+=coordChange;
			}
		//}
		if(pt.x < 0){
			pt.x += 1;
		}
		else if(pt.x > 1){
			pt.x -= 1;
		}
		if(pt.y < 0){
			pt.y = 0;
		}
		else if(pt.y > 1){
			pt.y = 1;
		}

		float u = (maxU-minU)*pt.x + minU;
		float v = (maxV-minV)*pt.y + minV;

		Vec3f curvedEdgePoint;

		SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, &curvedEdgePoint);

		Vec3f diffA = guessCoord[i] - originalCoord[i];
		Vec3f diffB = curvedEdgePoint - originalCoord[i];
//		float fdA = fabs(sqrt(diffA.x*diffA.x + diffA.y*diffA.y + diffA.z*diffA.z));
//		float fdB = fabs(sqrt(diffB.x*diffB.x + diffB.y*diffB.y + diffB.z*diffB.z));
		float fdA = fabs(sqrt(diffA.x*diffA.x + diffA.z*diffA.z));
		float fdB = fabs(sqrt(diffB.x*diffB.x + diffB.z*diffB.z));
		if(fdB < fdA){
			flatCoord[i] = pt;
		}
	}
}



void inline updateFlatVertices(M_Node& surface, M_Int& patchOwner, M_Vec2f& flatCoord, const M_Vec3f& guessCoord, const M_Vec3f& originalCoord, const int& bits, const float& coordChange){

	for(int i = flatCoord.length; --i>=0;){
		Vec2f pt = flatCoord[i];

		if(1 & bits){
			pt.x-=coordChange;
		}
		if(2 & bits){
			pt.x+=coordChange;
		}
//		if(pt.y > 0 && pt.y < 1){
			if(4 & bits){
				pt.y-=coordChange;
			}
			if(8 & bits){
				pt.y+=coordChange;
			}
//		}
		if(pt.x < 0){
			pt.x += 1;
		}
		else if(pt.x > 1){
			pt.x -= 1;
		}
		if(pt.y < 0){
			pt.y = 0;
		}
		else if(pt.y > 1){
			pt.y = 1;
		}


		if(NurbsPatch* nurbsPatch = dynamic_cast<NurbsPatch*>(surface[patchOwner[i]].p)){
			float* stemUKnotPtr = nurbsPatch->uKnot.refValue().getArray();
			float* stemVKnotPtr = nurbsPatch->vKnot.refValue().getArray();
			Vec3f* stemPointPtr = nurbsPatch->point.refValue().getArray();
			float* stemWeightPtr = nurbsPatch->weight.refValue().getArray();
			int uOrder = nurbsPatch->order.x;
			int vOrder = nurbsPatch->order.y;
			int uDim = nurbsPatch->dim.x;
			int vDim = nurbsPatch->dim.y;

			float maxU = nurbsPatch->maxU();
			float maxV = nurbsPatch->maxV();
			float minU = nurbsPatch->minU();
			float minV = nurbsPatch->minV();

			Vec3f curvedEdgePoint;

			float u = (maxU-minU)*pt.x + minU;
			float v = (maxV-minV)*pt.y + minV;
			SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, &curvedEdgePoint);

			Vec3f diffA = guessCoord[i] - originalCoord[i];
			Vec3f diffB = curvedEdgePoint - originalCoord[i];
			float fdA = fabs(sqrt(diffA.x*diffA.x + diffA.y*diffA.y + diffA.z*diffA.z));
			float fdB = fabs(sqrt(diffB.x*diffB.x + diffB.y*diffB.y + diffB.z*diffB.z));
			if(fdB < fdA){
				flatCoord[i] = pt;
			}
		}
	}
}

void tugVertex(int i, Vec3f pt, M_Vec3f& dpoint, M_Vec3f& point, const M_Float& weight, const M_Vec3f& originalCoord, const M_Vec3f& guessCoord, NurbsPatch* patch, float& sumDiff, float& averageDiff, bool mirrorTug){
	Vec3f remPoint = point[i];
	Vec3f remMirrorPoint;
	int mirrorIdx = -1;

	if(mirrorTug){
		Vec3f svec = dpoint[i];
		if(svec.x == 0){
			pt.x = 0;
		}
		svec.x = -svec.x;
		mirrorIdx = findMirrorVertex(dpoint, svec);
		if(mirrorIdx >= 0){
			remMirrorPoint = point[mirrorIdx];
			svec = pt;
			svec.x = -svec.x;

			patch->setPointAndCp(mirrorIdx, svec * weight[mirrorIdx]);
		}
	}

	patch->setPointAndCp(i, pt * weight[i]);

	patch->processFields();
	
	patch->smls->doBorderlessBuild();

	float testSumDiff=0;
	for(int j = guessCoord.length; --j>=0;){
		Vec3f diff = guessCoord[j] - originalCoord[j];
		testSumDiff += fabs(sqrt(diff.x*diff.x + diff.y*diff.y + diff.z*diff.z));
	}
	//dLog(testSumDiff);
	if(testSumDiff < sumDiff){
	//	if(testSumDiff >= 12.362071){
//			int a = 0;
//		}
		sumDiff = testSumDiff;
		averageDiff = testSumDiff/guessCoord.length;
		//polyToNurbs->processFields();
	}
	else{//undo
		patch->setPointAndCp(i, remPoint);
		if(mirrorIdx >= 0){
			patch->setPointAndCp(mirrorIdx, remMirrorPoint);
		}
		patch->processFields();
	}
}

void tugVertex(int i, Vec3f pt, M_Vec3f& dpoint, M_Vec3f& point, const M_Float& weight, const M_Vec3f& originalCoord, const M_Vec3f& guessCoord, Polymap* polymap, NurbsPatch* patch, float& sumDiff, float& averageDiff){
	Seamless* smls = patch->smls;
	Vec3f remPoint = point[i];
	Vec3f remMirrorPoint;
	int mirrorIdx = -1;

	patch->setPointAndCp(i, pt * weight[i]);

	smls->calcAllPolyMidPoints();
	patch->processFields();
	
	polymap->doBuild();

	float testSumDiff=0;
	for(int j = guessCoord.length; --j>=0;){
		Vec3f diff = guessCoord[j] - originalCoord[j];
		testSumDiff += fabs(sqrt(diff.x*diff.x + diff.y*diff.y + diff.z*diff.z));
	}
	//dLog(testSumDiff);
	if(testSumDiff < sumDiff){
		sumDiff = testSumDiff;
		averageDiff = testSumDiff/guessCoord.length;
		//polyToNurbs->processFields();
	}
	else{//undo
		patch->setPointAndCp(i, remPoint);
		if(mirrorIdx>=0){
			patch->setPointAndCp(mirrorIdx, remMirrorPoint);
		}
		smls->calcAllPolyMidPoints();
		patch->processFields();
	}
}

void updateDiff(const M_Vec3f& originalCoord, const M_Vec3f& guessCoord, float& sumDiff, float& averageDiff){
	float testSumDiff=0;
	for(int i = guessCoord.length; --i>=0;){
		Vec3f diff = guessCoord[i] - originalCoord[i];
		testSumDiff += fabs(sqrt(diff.x*diff.x + diff.y*diff.y + diff.z*diff.z));
	}
	sumDiff = testSumDiff;
	averageDiff = testSumDiff/guessCoord.length;
}


void PolyToNurbs::Cycle::operator=(const bool& par){
	home.doBuild();
}


void PolyToNurbs::doBuild(){
	const int randomLen = 127;
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(this->mimic.p)){
		bool xTug = g_transformCP->xTug;
		bool yTug = g_transformCP->yTug;
		bool zTug = g_transformCP->zTug;
		const bool mirrorTug = patch->mirror;
		if(patch->stem.length){
			Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
			stem->add.refValue() = false;
			patch->smls->doBorderlessBuild();
			if(Art* originalPart = dynamic_cast<Art*>(this->original.p)){
				M_SArt& part = stem->part.refValue();
				if(part.length){
					Art* guessPart = part[0];
					const M_Vec3f& guessCoord = guessPart->coord;
					const M_Vec3f& originalCoord = originalPart->coord;
					const int uDim = patch->dim.x;
					const int vDim = patch->dim.y;


					M_Vec2f& flatCoord = guessPart->uvCoord.refValue();

					M_Vec3f& dpoint = patch->dpoint;
					M_Vec3f& point = patch->point;
					M_Float& weight = patch->weight;

					int r = offset;
					r++;
					if(r>=randomLen){
						r = 0;
					}
					offset.refValue() = r;

					int bits = initPolyToNurbs_random[r];

					updateDiff(originalCoord, guessCoord, sumDiff.refValue(), averageDiff.refValue());

					if(bond && coordChange != 0){
						updateFlatVertices(*patch, flatCoord, guessCoord, originalCoord, bits, coordChange);
					}
						//srand( (unsigned)time( NULL ) );
					if(evolve && pointChange != 0){
						int pointLen = point.length;
						for(int pk = 0; pk < pointLen; pk++){

							//if(pk + 1 == 4500){
							//	break;
							//}

							int r = offset;
							r++;
							if(r >= randomLen){
								r = 0;
							}
							offset.refValue() = r;

							int bits = initPolyToNurbs_random[r];


							Vec3f pt = dpoint[pk];
							//int bits = rand();
							//bits&=0x7;

							//dLog(bits);

							if(mirrorTug && pt.x < 0){
								continue;
							}
							/*
							if(xTug){
								if(1 & bits){
									pt.x-=pointChange;
								}
								if(2 & bits){
									pt.x+=pointChange;
								}
							}
							*/
							if(yTug){
								if(4 & bits){
									pt.y-=pointChange;
								}
								if(8 & bits){
									pt.y+=pointChange;
								}
								if(pt.y < 0){
									pt.y = 0;
								}
							}
							/*
							if(zTug){
								if(16 & bits){
									pt.z-=pointChange;
								}
								if(32 & bits){
									pt.z+=pointChange;
								}
							}
							*/
							tugVertex(pk, pt, dpoint, point, weight, originalCoord, guessCoord, patch, sumDiff.refValue(), averageDiff.refValue(), mirrorTug);
						}
					}
				}
			}
		}
	}
	if(Polymap* polymap = dynamic_cast<Polymap*>(this->mimic.p)){
		M_Node& surface = polymap->surface.refValue();
		M_Int& patchOwner = polymap->patchOwner.refValue();
		if(surface.length){
			polymap->doBuild();
			polymap->add.refValue() = false;
			if(Art* originalPart = dynamic_cast<Art*>(this->original.p)){
				Art* guessPart = polymap->part.refValue().p;
				const M_Vec3f& guessCoord = guessPart->coord;
				const M_Vec3f& originalCoord = originalPart->coord;

				M_Vec2f& flatCoord = polymap->coord.refValue();

				int r = offset;
				r++;
				if(r>=randomLen){
					r = 0;
				}
				offset.refValue()=r;

				int bits = initPolyToNurbs_random[r];

				updateDiff(originalCoord, guessCoord, sumDiff.refValue(), averageDiff.refValue());

				if(bond && coordChange != 0){
					updateFlatVertices(surface, patchOwner, flatCoord, guessCoord, originalCoord, bits, coordChange);
				}
					//srand( (unsigned)time( NULL ) );
				if(evolve && pointChange != 0){
					for(int k = surface.length; --k>=0;){
						NurbsPatch* patch = dynamic_cast<NurbsPatch*>(surface[k].p);
						const int uDim = patch->dim.x;
						const int vDim = patch->dim.y;
						M_Vec3f& dpoint = patch->dpoint;
						M_Vec3f& point = patch->point;
						M_Float& weight = patch->weight;

						for(int v = 0; v < vDim; v++){
							for(int u = 0; u < uDim; u+=1){
								int i = v * uDim + u;

								int r = offset;
								r++;
								if(r >= randomLen){
									r = 0;
								}
								offset.refValue() = r;

								int bits = initPolyToNurbs_random[r];


								Vec3f pt = dpoint[i];
								//int bits = rand();
								//bits&=0x7;

								//dLog(bits);

								//if(mirrorTug && pt.x < 0){
								//	continue;
								//}
								if(1 & bits){
									pt.x-=pointChange;
								}
								if(2 & bits){
									pt.x+=pointChange;
								}
								if(4 & bits){
									pt.y-=pointChange;
								}
								if(8 & bits){
									pt.y+=pointChange;
								}
								if(16 & bits){
									pt.z-=pointChange;
								}
								if(32 & bits){
									pt.z+=pointChange;
								}
								tugVertex(i, pt, dpoint, point, weight, originalCoord, guessCoord, polymap, patch, sumDiff.refValue(), averageDiff.refValue());
							}
						}
					}
				}
			}
		}
	}
}
/*
void inline updateFlatVertices(NurbsPatch& nurbsPatch, M_Vec3f& flatCoord, const M_Vec3f& guessCoord, const M_Vec3f& originalCoord, const M_Int& random, const int& bits, const float& coordChange){
	float* stemUKnotPtr = nurbsPatch.uKnot.refValue().getArray();
	float* stemVKnotPtr = nurbsPatch.vKnot.refValue().getArray();
	Vec3f* stemPointPtr = nurbsPatch.point.refValue().getArray();
	float* stemWeightPtr = nurbsPatch.weight.refValue().getArray();
	int uOrder = nurbsPatch.order.x;
	int vOrder = nurbsPatch.order.y;
	int uDim = nurbsPatch.dim.x;
	int vDim = nurbsPatch.dim.y;

	float maxU = nurbsPatch.maxU();
	float maxV = nurbsPatch.maxV();
	float minU = nurbsPatch.minU();
	float minV = nurbsPatch.minV();

	for(int i = flatCoord.length; --i>=0;){
		Vec3f pt = flatCoord[i];

		if(1 & bits){
			pt.x-=coordChange;
		}
		if(2 & bits){
			pt.x+=coordChange;
		}
		if(pt.y > 0 && pt.y < 1){
			if(4 & bits){
				pt.y-=coordChange;
			}
			if(8 & bits){
				pt.y+=coordChange;
			}
		}
		if(pt.x < 0){
			pt.x += 1;
		}
		else if(pt.x > 1){
			pt.x -= 1;
		}
		if(pt.y < 0){
			pt.y = 0;
		}
		else if(pt.y > 1){
			pt.y = 1;
		}

		float u = (maxU-minU)*pt.x + minU;
		float v = (maxV-minV)*pt.y + minV;

		Vec3f curvedEdgePoint;

		SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, &curvedEdgePoint);

		Vec3f diffA = guessCoord[i] - originalCoord[i];
		Vec3f diffB = curvedEdgePoint - originalCoord[i];
		float fdA = fabs(sqrt(diffA.x*diffA.x + diffA.y*diffA.y + diffA.z*diffA.z));
		float fdB = fabs(sqrt(diffB.x*diffB.x + diffB.y*diffB.y + diffB.z*diffB.z));
		if(fdB < fdA){
			flatCoord[i] = pt;
		}
	}
}
void PolyToNurbs::doBuild(){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(this->mimic.p)){
		const bool mirrorTug = patch->mirrorTug;
		if(patch->stem.length){
			Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
			patch->doBuild();
			if(Art* originalPart = dynamic_cast<Art*>(this->original.p)){
				M_SArt& part = stem->part.refValue();
				if(part.length){
					M_SArt& source = stem->source.refValue();
					Art* guessPart = part[0];
					const M_Vec3f& guessCoord = guessPart->coord;
					const M_Vec3f& originalCoord = originalPart->coord;
					const int uDim = patch->dim.x;
					const int vDim = patch->dim.y;


					if(source.length){
						Art* flatPart = source[0];
						M_Vec3f& flatCoord = flatPart->coord;

						M_Vec3f& dpoint = patch->dpoint;
						M_Vec3f& point = patch->point;
						M_Float& weight = patch->weight;

						if(update){

							int r = offset;
							r++;

							offset.refValue()=r;

							int random2[] = {1, 2, 4, 8};
							int bits=random2[r & 0x3];

							updateFlatVertices(*patch, flatCoord, guessCoord, originalCoord, random, bits, coordChange);
							//srand( (unsigned)time( NULL ) );
							if(pointChange){
								int random2[] = {1, 2, 4, 8, 16, 32};
								for(int k = cycle; --k>=0;){
									for(int v = 0; v < vDim; v++){
										for(int u = 1; u < uDim; u+=2){
											int i = v * uDim + u;
											if(i == 63){
												int a = 0;
											}

											if(r>=sizeof(random2)/sizeof(int)){
												r = 0;
											}
											//offset.refValue()=r;

											int bits=random2[r];
											//int i = 48;{
											//if(i!=46)continue;
			//								if(i==46){
			//									i=i;
			//								}


											Vec3f pt = dpoint[i];
											//int bits = rand();
											//bits&=0x7;

											//dLog(bits);


											if(mirrorTug && pt.x < 0){
												continue;
											}
											if(1 & bits){
												pt.x-=pointChange;
											}
											if(2 & bits){
												pt.x+=pointChange;
											}
											if(4 & bits){
												pt.y-=pointChange;
											}
											if(8 & bits){
												pt.y+=pointChange;
											}
											if(16 & bits){
												pt.z-=pointChange;
											}
											if(32 & bits){
												pt.z+=pointChange;
											}
											tugVertex(i, pt, dpoint, point, weight, originalCoord, guessCoord, patch, sumDiff.refValue(), averageDiff.refValue(), mirrorTug);
										}
									}
								}
								patch->doBuild();
								houseKeepVisualClue();
							}
							else{
								goto here;
							}
						}
						else{
here:
							float testSumDiff=0;
							for(int i = guessCoord.length; --i>=0;){
								Vec3f diff = guessPart->coord[i] - originalPart->coord[i];
								testSumDiff += fabs(sqrt(diff.x*diff.x + diff.y*diff.y + diff.z*diff.z));
							}
							sumDiff.refValue() = testSumDiff;
							averageDiff.refValue() = testSumDiff/guessCoord.length;
						}
					}
				}
			}
		}
	}
}
*/

template <class T> void inline spreadVertices(NurbsPatch& nurbsPatch, M_<T>& flatCoord, M_Vec3f& guessCoord, const M_Vec3f& originalCoord, const int& spreadRes, const float& spreadResFloat){
	float* stemUKnotPtr = nurbsPatch.uKnot.refValue().getArray();
	float* stemVKnotPtr = nurbsPatch.vKnot.refValue().getArray();
	Vec3f* stemPointPtr = nurbsPatch.point.refValue().getArray();
	float* stemWeightPtr = nurbsPatch.weight.refValue().getArray();
	int uOrder = nurbsPatch.order.x;
	int vOrder = nurbsPatch.order.y;
	int uDim = nurbsPatch.dim.x;
	int vDim = nurbsPatch.dim.y;

	float maxU = nurbsPatch.maxU();
	float maxV = nurbsPatch.maxV();
	float minU = nurbsPatch.minU();
	float minV = nurbsPatch.minV();

	for(int i = flatCoord.length; --i>=0;){
		for(int uPt = 0; uPt < spreadRes; uPt++){
			for(int vPt = 0; vPt < spreadRes; vPt++){
				T pt;
				pt.x = uPt / spreadResFloat;
				pt.y = vPt / spreadResFloat;

				float u = (maxU-minU)*pt.x + minU;
				float v = (maxV-minV)*pt.y + minV;

				Vec3f curvedEdgePoint;

				SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, &curvedEdgePoint);

				Vec3f diffA = guessCoord[i] - originalCoord[i];
				Vec3f diffB = curvedEdgePoint - originalCoord[i];
				float fdA = fabs(sqrt(diffA.x*diffA.x + diffA.y*diffA.y + diffA.z*diffA.z));
				float fdB = fabs(sqrt(diffB.x*diffB.x + diffB.y*diffB.y + diffB.z*diffB.z));
				if(fdB < fdA){
					flatCoord[i] = pt;
					guessCoord[i] = curvedEdgePoint;
				}
			}
		}
	}
}

void inline spreadVertices(NurbsPatch& nurbsPatch, M_Vec2f& flatCoord, M_Int& patchOwner, const int& patchOwnerIdx, M_Vec3f& guessCoord, const M_Vec3f& originalCoord, const int& spreadRes, const float& spreadResFloat){
	float* stemUKnotPtr = nurbsPatch.uKnot.refValue().getArray();
	float* stemVKnotPtr = nurbsPatch.vKnot.refValue().getArray();
	Vec3f* stemPointPtr = nurbsPatch.point.refValue().getArray();
	float* stemWeightPtr = nurbsPatch.weight.refValue().getArray();
	int uOrder = nurbsPatch.order.x;
	int vOrder = nurbsPatch.order.y;
	int uDim = nurbsPatch.dim.x;
	int vDim = nurbsPatch.dim.y;

	float maxU = nurbsPatch.maxU();
	float maxV = nurbsPatch.maxV();
	float minU = nurbsPatch.minU();
	float minV = nurbsPatch.minV();

	for(int i = flatCoord.length; --i>=0;){
		for(int uPt = 0; uPt < spreadRes; uPt++){
			for(int vPt = 0; vPt < spreadRes; vPt++){
				Vec2f pt = Vec2f(uPt / spreadResFloat, vPt / spreadResFloat);

				float u = (maxU-minU) * pt.x + minU;
				float v = (maxV-minV) * pt.y + minV;
				if(u >= 1.0f){
					u = .999;
				}
				if(v >= 1.0f){
					v = .999;
				}

				Vec3f curvedEdgePoint;

				SurfacePoint(u, v, uOrder, vOrder, uDim, vDim, stemUKnotPtr, stemVKnotPtr, stemPointPtr, stemWeightPtr, &curvedEdgePoint);

				Vec3f diffA = guessCoord[i] - originalCoord[i];
				Vec3f diffB = curvedEdgePoint - originalCoord[i];
				float fdA = fabs(sqrt(diffA.x*diffA.x + diffA.y*diffA.y + diffA.z*diffA.z));
				float fdB = fabs(sqrt(diffB.x*diffB.x + diffB.y*diffB.y + diffB.z*diffB.z));
				if(fdB < fdA){
					flatCoord[i] = pt;
					guessCoord[i] = curvedEdgePoint;
					patchOwner[i] = patchOwnerIdx;
				}
			}
		}
	}
}


void PolyToNurbs::Spread::operator=(const bool& par){
	if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(home.mimic.p)){
		const bool mirrorTug = patch->mirror;
		if(patch->stem.length){
			Stem* stem = dynamic_cast<Stem*>(patch->stem[0].p);
			patch->doBuild();
			if(Art* originalPart = dynamic_cast<Art*>(home.original.p)){
				M_SArt& part = stem->part.refValue();
				if(part.length){
					M_SArt& source = stem->source.refValue();
					Art* guessPart = part[0];
					M_Vec3f& guessCoord = guessPart->coord.refValue();
					const M_Vec3f& originalCoord = originalPart->coord;
					const int uDim = patch->dim.x;
					const int vDim = patch->dim.y;


					if(source.length){
						Art* flatPart = source[0];
						M_Vec3f& flatCoord = flatPart->coord;

						M_Vec3f& dpoint = patch->dpoint;
						M_Vec3f& point = patch->point;
						M_Float& weight = patch->weight;

						spreadVertices(*patch, flatCoord, guessCoord, originalCoord, home.spreadRes, home.spreadRes);
					}
					else{
						M_Vec2f& flatCoord = stem->coord.refValue();

						M_Vec3f& dpoint = patch->dpoint;
						M_Vec3f& point = patch->point;
						M_Float& weight = patch->weight;

						spreadVertices(*patch, flatCoord, guessCoord, originalCoord, home.spreadRes, home.spreadRes);
					}
				}
			}
		}
	}
	else if(Polymap* polymap = dynamic_cast<Polymap*>(home.mimic.p)){
		if(Art* originalPart = dynamic_cast<Art*>(home.original.p)){
			if(Art* guessPart = polymap->part.refValue()){
				M_Vec3f& guessCoord = guessPart->coord.refValue();
				M_Vec2f& flatCoord = polymap->coord.refValue();
				M_Int& patchOwner = polymap->patchOwner.refValue();
				patchOwner.length = flatCoord.length;
				const M_Vec3f& originalCoord = originalPart->coord;
				M_Node& surface = polymap->surface.refValue();

				for(int k = 0; k < surface.length; k++){
					NurbsPatch* patch = dynamic_cast<NurbsPatch*>(surface[k].p);

					spreadVertices(*patch, flatCoord, patchOwner, k, guessCoord, originalCoord, home.spreadRes, home.spreadRes);

				}
			}
		}
	}
}
void convertMercatorToNorthPole(M_Vec3f& coord);
void convertMercatorToSouthPole(M_Vec3f& coord);
void convertNorthPoleToMercator(M_Vec3f& coord);
void convertSouthPoleToMercator(M_Vec3f& coord);

void FuseSurface::doSpread(){
	M_Vec2f& baseCoord = this->baseCoord.refValue();
	if(NurbsPatch* parentPatch = this->parentPatch){
		if(NurbsPatch* patch = dynamic_cast<NurbsPatch*>(this->patch.p)){
			if(patch->stem.length){
				if(Stem* stem = dynamic_cast<Stem*>(patch->stem.refValue()[0].p)){
					M_SArt& part = stem->part.refValue();
					if(part.length){
						M_SArt& source = stem->source.refValue();
						Art* originalPart = part[0];
						const M_Vec3f& originalCoord = originalPart->coord;
						const int uDim = parentPatch->dim.x;
						const int vDim = parentPatch->dim.y;

						M_Vec3f& dpoint = parentPatch->dpoint;
						M_Vec3f& point = parentPatch->point;
						M_Float& weight = parentPatch->weight;

						M_Vec3f guessCoord;
						guessCoord.length = baseCoord.length;
						M_Vec3f baseCoord3f;
						baseCoord3f.length = baseCoord.length;

						spreadVertices(*parentPatch, baseCoord3f, guessCoord, originalCoord, spreadRes, spreadRes);

						if(parentStem->northPole){
							convertMercatorToNorthPole(baseCoord3f);
						}
						else if(parentStem->southPole){
							convertMercatorToSouthPole(baseCoord3f);
						}
						for(int i = baseCoord.length; --i>=0;){
							baseCoord[i] = Vec2f(baseCoord3f[i].x, baseCoord3f[i].y);
						}
					}
				}
			}
		}
	}
}
void FuseSurface::Spread::operator=(const bool& par){
	refValue()=true;
}
bool catchSelCoord(Seamless* smls){
	if(Part* part = dynamic_cast<Part*>(g_selNode.getRaw())){
		M_Vec3f& coord = part->coord.refValue();
		if(smls->ActCtrlPnt < coord.length){
			if(smls->selCoordPos == coord[smls->ActCtrlPnt]){
				return true;
			}
		}
	}
	return false;
}

void insertSection(M_Vec3f& keyValue, int channels, int sectionIndex){
	int from = keyValue.length;
	keyValue.length += channels;
	int to = sectionIndex * channels;
	for(int i = from; --i >= to;){
		keyValue[i + channels] = keyValue[i];
	}
}

void CPMorpher::insertChannel(int& channel, const int k, M_Int& cpIndex, M_Vec3f& keyValue, int& numOfChannels, const int numOfSections){
	int from = keyValue.length;
	keyValue.length += numOfSections;
	int dist = numOfSections;
	int cycle = numOfChannels - channel;

	for(int i = from; --i >= 0;){
		if(cycle == 0){
			keyValue[i + dist] = Vec3f();
			cycle = numOfChannels;
			dist--;
		}
		keyValue[i + dist] = keyValue[i];
		--cycle;
	}
	numOfChannels++;
	cpIndex.insertElement(channel, k);
	channel++;
}

void CPMorpher::insertChannel(int& channel, const int k, M_Int& cpIndex, const float slider, const Vec3f& diff, M_Vec3f& keyValue, int& numOfChannels, const int numOfSections){
	if(key.length == 0){
		keyValue.length = key.refValue().length = 2;
		key.refValue()[1] = 1;
	}

	int remChannel = channel;
	insertChannel(channel, k, cpIndex, keyValue, numOfChannels, numOfSections);
	take(slider, diff, remChannel, numOfChannels);
}

void CPMorpher::deleteChannel(const int& channel, M_Int& cpIndex, M_Float& key, M_Vec3f& keyValue, const int numOfChannels){
	index[0] = 0;
	index[1] = 0;
	cpIndex.deleteElement(channel);
	if(cpIndex.length == 0){
		key.length = 0;
		keyValue.length = 0;
		return;
	}
	int cycle = numOfChannels - 1;
	int dst = channel;
	int src = dst + 1;
	int less = 0;
	while(src < keyValue.length){
		keyValue[dst++] = keyValue[src++];

		if(--cycle == 0){
			src++;
			less++;
			cycle = numOfChannels - 1;
		}
	}
	keyValue.length -= key.length;
}

void CPMorpher::deleteChannelCoords(const Art* part){
	for(int k = MultiSelect::coordList.length; --k>=0;){
		if(part == MultiSelect::coordList[k].p){
			int idx = MultiSelect::coordList[k].i;
			for(int i = cpIndex.length; --i>=0;){
				if(cpIndex[i] == idx){
					deleteChannel(i, cpIndex.refValue(), key.refValue(), keyValue.refValue(), cpIndex.length);
					break;
				}
			}
		}
	}
}

int Art::getControlPointIndexFromPointListItem(const NurbsPatch* patch, const int idx)const{
	for(int i = controlPoint.length; --i>=0;){
		if(controlPoint[i].p == patch && controlPoint[i].i == idx){
			return i;
		}
	}
	return -1;
}

void CPMorpher::deleteChannelPoints(const Art* part){
	for(int k = MultiSelect::pointList.length; --k>=0;){
		if(int idx = part->getControlPointIndexFromPointListItem(MultiSelect::pointList[k].p, MultiSelect::pointList[k].i)){
			for(int i = cpIndex.length; --i>=0;){
				if(cpIndex[i] == idx){
					deleteChannel(i, cpIndex.refValue(), key.refValue(), keyValue.refValue(), keyValue.length / key.length);
					break;
				}
			}
		}
	}
}

void CPMorpher::take(float slider, const Vec3f& diff, const int channel, const int numOfChannels){
	M_Float& key = this->key;
	M_Vec3f& keyValue = this->keyValue;

	if(key.length == 0){
		key += slider;
		keyValue.length = key.length * numOfChannels;
		keyValue[channel] = diff;
	}
	else{
		
		for(int i = 0; i < key.length; i++){
			if(roughEQ(slider, key[i])){
				keyValue[numOfChannels * i + channel] = diff;
				index[0] = i;
				break;
			}
			else if(roughGT(key[i], slider)){
				key.insertElement(i, slider);
				insertSection(keyValue, numOfChannels, i);
				keyValue[numOfChannels * i + channel] = diff;
				index[0] = i;
				break;
			}
			else if(i == key.length - 1){
				i++;
				key.insertElement(i, slider);
				insertSection(keyValue, numOfChannels, i);
				keyValue[numOfChannels * i + channel] = diff;
				index[0] = i;
				break;
			}
		}
	}
}



void NurbsLathe::BeginSP::operator=(const bool& par){
	refValue() = par;
	home.processFields();
	if(NurbsLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->beginSP.refValue() = par;
		mirrorContainer->processFields();
	}
}

void NurbsLathe::EndSP::operator=(const bool& par){
	refValue() = par;
	home.processFields();
	if(NurbsLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->endSP.refValue() = par;
		mirrorContainer->processFields();
	}
}



void Art::genDynamicTransforms(M_Matrix& m_matrix, M_Int& m_smi, Matrix parentMatrix, int& smi){
	
	Matrix pivotPointMatrix = translationMatrix(translation + pivotPoint) * parentMatrix;
	int coordLen = coord.length;
	int groupLen = coordGroup.length;
	m_smi.length += coordLen;
	int k = 0;
	int coordsInGroups = 0;

	for(int cg = 0; cg < groupLen; cg++){
		int mark = k;
		for(int c = coordGroup[cg]; --c >= 0 && k < coordLen; k++){
			m_smi[fromCoord + k] = smi;
		}
		if(k > mark){
			m_matrix += scaleMatrix(scale) * rotationMatrix(Rotation(rotation.x, rotation.y, rotation.z, rotation.angle * weight[cg])) * pivotPointMatrix;
			smi++;
		}
	}

	pivotPointMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * pivotPointMatrix;

	int mark = k;
	for(int len = coord.length; k < len; k++){
		m_smi[fromCoord + k] = smi;
	}
	if(k > mark){
		m_matrix += pivotPointMatrix;
		smi++;
	}

	for(int k = 0; k < part.length; k++){
		part.refValue()[k]->genDynamicTransforms(m_matrix, m_smi, pivotPointMatrix, smi);
	}
}

inline bool checkPointInVertex(Art*& part, const bool& perspective, const int& i){
	if(part->coord.length==0)return false;

	Vec3f sr = m_matrix[m_smi[part->fromCoord + i]] * g_viewMatrix * (g_m_coord[part->fromCoord + i] - part->absolutePivotPoint);

	Vec3f v;
	Vec3f mv;

	if(perspective){
		v.x = sr.x / (-sr.z + g_viewDist);
		v.y = sr.y / (-sr.z + g_viewDist);

		mv.x = g_pointer.translation.x / (-g_pointer.translation.z + g_viewDist);
		mv.y = g_pointer.translation.y / (-g_pointer.translation.z + g_viewDist);
	}
	else{
		v.x=sr.x;
		v.y=sr.y;

		mv.x=g_pointer.translation.x;
		mv.y=g_pointer.translation.y;
	}
	
	if(
		v.x < mv.x+g_mouseSense&&
		v.x > mv.x-g_mouseSense&&
		v.y < mv.y+g_mouseSense&&
		v.y > mv.y-g_mouseSense){

				
		int corner;
		S_Triangle* triangle;
		S_Triangle* beginTri;
		int tri;

		Art* triPart=0;
		tri = findNextTriangleFromVertex(part, triPart, i, corner, part->smls->ActTri-1);
		triangle=&triPart->triangle[tri];
		beginTri=triangle;

		while(1){

			int vecIndexA=triangle->ia;
			int vecIndexB=triangle->ib;
			int vecIndexC=triangle->ic;

			Art* partA=triangle->pa;
			Art* partB=triangle->pb;
			Art* partC=triangle->pc;

			int idxA = partA->fromCoord + vecIndexA;
			int idxB = partB->fromCoord + vecIndexB;
			int idxC = partC->fromCoord + vecIndexC;
			Vec3f vA = m_matrix[m_smi[idxA]] * g_viewMatrix * (g_m_coord[idxA] - partA->absolutePivotPoint);
			Vec3f vB = m_matrix[m_smi[idxB]] * g_viewMatrix * (g_m_coord[idxB] - partB->absolutePivotPoint);
			Vec3f vC = m_matrix[m_smi[idxC]] * g_viewMatrix * (g_m_coord[idxC] - partC->absolutePivotPoint);


			if(g_settings->backfaceIgnore){
				if(backFace(vA,vB,vC))	break; //if success exit loop 
			}
			else {
				break;
			}

			tri=findNextTriangleFromVertex(part, triPart, i, corner, tri);
			triangle=&triPart->triangle[tri];
			if(triangle!=beginTri) continue;
			else return false; //looped full circle
		}
		if(SenseTugGlobals::node == 0 || sr.z >= SenseTugGlobals::hitZ){

			SenseTugGlobals::pointer = sr;
			SenseTugGlobals::hitZ = sr.z;
			SenseTugGlobals::node = part;
			SenseTugGlobals::actTriPart = triPart;
			SenseTugGlobals::ActTri = tri;
			SenseTugGlobals::corner = corner;
			SenseTugGlobals::ActCtrlPnt = i;
			return true;
		}

	}
	return false;
}

void MultiSelect::selectFrontAndBackFaceTransformedVertices(Seamless* smls){
	smls->genDynamicMatricesAndPopulate();

	M_Art& partList = smls->partList;

	Render* render = smls->render;
	if(render==0)return;


	for(int k = 0; k < partList.length; k++){
		Art* part = partList[k];
		if(part->show == false){
			continue;
		}
		int coordLen = part->coord.length;
		for(int i = 0; i <coordLen ; i++){
			if(isInBox(m_matrix[m_smi[part->fromCoord + i]] * (g_m_coord[part->fromCoord + i] - part->absolutePivotPoint))){
				coordList += S_VertexIndex(part, i);
			}
		}
	}
}

Vec2f convertPointToScreen2d(Vec3f point);

void selectFrontFaceRenderedVertices2(M_Art& partList, const bool& backfaceIgnore, M_Bool& selectCoord){
	for(int k = 0; k < partList.length; k++){
		Art* part = partList[k];
		if(part->show == false){
			continue;
		}
		M_Triangle& triangle = part->triangle;
		for(int i = triangle.length; --i>=0;){
			S_Triangle& tri = triangle[i];

			Art* partA = tri.pa;
			Art* partB = tri.pb;
			Art* partC = tri.pc;

			if(backfaceIgnore){
				Vec3f coordA = m_matrix[m_smi[partA->fromCoord + tri.ia]] * g_viewMatrix * (g_m_coord[partA->fromCoord + tri.ia] - partA->absolutePivotPoint);
				Vec3f coordB = m_matrix[m_smi[partB->fromCoord + tri.ib]] * g_viewMatrix * (g_m_coord[partB->fromCoord + tri.ib] - partB->absolutePivotPoint);
				Vec3f coordC = m_matrix[m_smi[partC->fromCoord + tri.ic]] * g_viewMatrix * (g_m_coord[partC->fromCoord + tri.ic] - partC->absolutePivotPoint);

				if(!backFace(coordA, coordB, coordC)){
					continue;
				}
			}
			Vec2f va = convertPointToScreen2d(m_matrix[m_smi[partA->fromCoord + tri.ia]] * (g_m_coord[partA->fromCoord + tri.ia] - partA->absolutePivotPoint));
			Vec2f vb = convertPointToScreen2d(m_matrix[m_smi[partB->fromCoord + tri.ib]] * (g_m_coord[partB->fromCoord + tri.ib] - partB->absolutePivotPoint));
			Vec2f vc = convertPointToScreen2d(m_matrix[m_smi[partC->fromCoord + tri.ic]] * (g_m_coord[partC->fromCoord + tri.ic] - partC->absolutePivotPoint));

			Vec2f tri2d[3] = {va, vb, vc};

			int contigIdx = partA->fromCoord + tri.ia;
			if(selectCoord[contigIdx] == false && MultiSelect::isInBox(va)){
				selectCoord[contigIdx] = true;
				MultiSelect::coordList += S_VertexIndex(partA, tri.ia);
			}

			contigIdx = partB->fromCoord + tri.ib;
			if(selectCoord[contigIdx] == false && MultiSelect::isInBox(vb)){
				selectCoord[contigIdx] = true;
				MultiSelect::coordList += S_VertexIndex(partB, tri.ib);
			}

			contigIdx = partC->fromCoord + tri.ic;
			if(selectCoord[contigIdx] == false && MultiSelect::isInBox(vc)){
				selectCoord[contigIdx] = true;
				MultiSelect::coordList += S_VertexIndex(partC, tri.ic);
			}
		}
	}
}


void MultiSelect::selectFrontFaceRenderedVertices(Seamless* smls){
	M_Art& partList = smls->partList;
	M_Bool selectCoord;
	int selectCoordLen = 0;

	for(int k = 0; k < partList.length; k++){
		Art* part = partList[k];
		selectCoordLen += part->coord.length;
	}
	selectCoord.length = selectCoordLen;

	for(int k = coordList.length; --k >= 0;){
		selectCoord[coordList[k].p->fromCoord + coordList[k].i] = true;
	}

	Render* render = smls->render;
	if(render==0)return;

	smls->genDynamicMatricesAndPopulate();

	selectFrontFaceRenderedVertices2(partList, (g_scene->mode & BACKFACE_IGNORE_MODE) != 0, selectCoord);
}


void CPMorpher::do_displayPolyFrame(const Art* part){
	M_Float& key = this->key.refValue();
	M_Vec3f& keyValue = this->keyValue.refValue();
	if(!key.length)return;

//	for(int k = controlPoint.length; --k >= 0;){
//	}
	int portionLen = cpIndex.length;

	float frac = frame;
	int i = 0;
	if(key.length < 2)goto here;
	for(; i<key.length-1; i++){
		if(key[i] > frac){
			i--;
			break;
		}
	}
	if(i < 0){
here:
		if(key.length > 0){
			//value_changed.sendOutRoute(keyValue[0]);
		}
		return;
	}
	if(i == key.length - 1){
		i = key.length - 2;
	}
	float diff = key[i + 1] - key[i];

	float interp = 1.0 / diff * (frac - key[i]);
	if(interp < 0){
		interp =- interp;
	}

	int coordLen = part->coord.length;
	int fromCoord = part->fromCoord;

	for(int j = 0; j < portionLen; j++){
		int h = i * portionLen + j;
		Vec3f result = keyValue[h] + (keyValue[(i + 1) * portionLen + j] - keyValue[h]) * interp;
		int idx = cpIndex[j];
		if(idx < coordLen){
			g_m_coord[fromCoord + idx] += result;
		}
	}
}
void Seamless::genDynamicMatricesAndPopulate(){
	g_m_coord.length = setCoordFroms();
	genDynamicMatricesForSelectedPolyCoords();
	populate_g_m_coord();
}

void Seamless::populate_g_m_coord(){
	int q = 0;
	for(int k = 0; k < partList.length; k++){
		const M_Vec3f& coord = partList[k]->coord;
		for(int i = 0, len = coord.length; i < len;){
			g_m_coord[q++] = coord[i++];
		}
	}
	for(int k = cpMorpherPart.length; --k >= 0;){
		CPMorpherPart& cpmp = cpMorpherPart[k];
		cpmp.morpher->do_displayPolyFrame(cpmp.part);
	}
}

void Seamless::genDynamicMatricesForSelectedPolyCoords(){
	m_matrix.length = 0;
	m_smi.length = 0;
	Matrix parentMatrix = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation);

	int smi = 0;
	M_SArt& skeleton = this->skeleton.refValue();
	for(int i = 0, smi = 0; i < skeleton.length; i++){
		skeleton[i]->genDynamicTransforms(m_matrix, m_smi, parentMatrix, smi);
	}
}
//bool checkInList(Art* part, int idx){
//	for(int i = MultiSelect::coordList.length; --i>=0;){
//		if(MultiSelect::coordList[i].p == part && MultiSelect::coordList[i].i == idx){
//			return true;
//		}
//	}
//	return false;
//}
void Seamless::senseAllPartsVertex(){
	genDynamicMatricesAndPopulate();

	const bool perspective=g_perspective;
	NodePtr before = g_selNode.getRaw();
	Anim* anim = Window::animMode();



	//check selected vertices first
	if(MultiSelect::coordList.length){
		if(MultiSelect::coordList[0].p->smls == this){
			for(int i = MultiSelect::coordList.length; --i >= 0;){
				Art* part = MultiSelect::coordList[i].p;
				int idx = MultiSelect::coordList[i].i;
				bool remBackfaceIgnore = g_settings->backfaceIgnore;
				g_settings->backfaceIgnore = false;
				if(checkPointInVertex(part, perspective, idx)){
					MultiSelect::clickedIndex = i;
					part->index[0] = idx;
					ActCtrlPnt = idx;
					g_settings->backfaceIgnore = remBackfaceIgnore;

					MultiSelect::beginDragPos.length = MultiSelect::coordList.length;
					for(int i = MultiSelect::coordList.length; --i >= 0;){
						Art* part = MultiSelect::coordList[i].p;
						int idx = MultiSelect::coordList[i].i;
						MultiSelect::beginDragPos[i] = g_m_coord[part->fromCoord + idx];
					}
					if(anim && (g_scene->mode & POLY_EDIT_MODE)){
						g_dragMorphFlag =  true;
					}
					return;
				}
				g_settings->backfaceIgnore = remBackfaceIgnore;
			}
		}
	}
	//before anything else sense the 3 vertices in selected triangle
	//bool clickedSelectedTriangleCoord = false;
	if(anim == 0){
		if(Art* part = dynamic_cast<Art*>(before.p)){
			if(part->smls == this && part->show && anim == 0){
				Matrix partMatrix = part->matrix * g_viewMatrix;
				if(part->show && part->coord.refValue().length){
					if(actTriPart){
						M_Triangle& triangle = actTriPart->triangle;
						for(int i=ActTri*3+ActTriCorner, j=3; --j>=0;){//check other 2 corners in tri first
							Art* prt = triangle.refPart(i);
							int idx = triangle.refIndex(i);
							if(checkPointInVertex(prt, perspective, idx)){
								SenseTugGlobals::ActTri=ActTri;
								SenseTugGlobals::actTriPart=actTriPart;
								SenseTugGlobals::corner=i%3;

								MultiSelect::coordList.length = 0;
								MultiSelect::beginDragPos.length = 0;
								MultiSelect::coordList += S_VertexIndex(prt, idx);
								MultiSelect::beginDragPos+=prt->coord[idx];
								ActCtrlPnt = idx;
								MultiSelect::clickedIndex = 0;
								prt->index[0] = idx;
								return;
							}
							i=ActTri*3+(i+1)%3;
						}
					}
				}
			}
		}
	}
	//if(clickedSelectedTriangleCoord){
	//	if(MultiSelect::coordList.length == 1){
	//		MultiSelect::coordList.length = 0;
	//		MultiSelect::beginDragPos.length = 0;
	//	}
	//	return;
	//}

	//check NON selected vertices next
	bool clear = true; //flag to indicate that we must clear all the selected vertices if we hit a vertex from here on


	for(int k = 0; k < partList.length; k++){

		Art* part=partList[k];
		M_Triangle& triangle = part->triangle;


		if(part->show){
			for(int i=triangle.getTriangleLength(); --i>=0; ){
				Art* part = triangle.refPart(i);
				if (checkPointInVertex(part, perspective, triangle.refIndex(i))){
					if(clear){
						MultiSelect::beginDragPos.length = MultiSelect::coordList.length = 0;
						clear = false;
					}
					part->index[0] = triangle.refIndex(i);
					MultiSelect::coordList += S_VertexIndex(part, part->index[0]);
					MultiSelect::clickedIndex = 0;
					ActCtrlPnt = part->index[0];
					MultiSelect::checkToReset(part);

					//if(anim == 0 && MultiSelect::coordList.length == 1){
					//	MultiSelect::coordList.length = 0;
					//	MultiSelect::beginDragPos.length = 0;
					//}

//						MultiSelect::backUpCoords(part);

					MultiSelect::beginDragPos.length = MultiSelect::coordList.length;
					for(int i = MultiSelect::coordList.length; --i >= 0;){
						Art* part = MultiSelect::coordList[i].p;
						int idx = MultiSelect::coordList[i].i;
						MultiSelect::beginDragPos[i] = g_m_coord[part->fromCoord + idx];
					}

					if(anim && (g_scene->mode & POLY_EDIT_MODE)){
						g_dragMorphFlag =  true;
					}

					MultiSelect::calcPivotPoint();
					return;

				}
			}
		}
	}
}

//void fillWithDraggedMorphedVertices(){//beginDragPos was use for the destination not the sourse
//	for(int i = MultiSelect::coordList.length; --i >= 0;){
//		Art* part = MultiSelect::coordList[i].p;
//		int idx = MultiSelect::coordList[i].i;
//		g_m_coord[part->fromCoord + idx] = MultiSelect::beginDragPos[i];
//	}
//}

void NurbsTrack::KeyValue::operator=(const M_Vec3f& par){
	refValue() = par;
	home.houseKeep();
	if(Anim* anim = Window::animMode()){
		anim->update = true;
		anim->renderNodeFirst();
		anim->update = false;
	}
}


void Grid::houseKeep(){
	setMathHiPresision();

	M_Vec3f& coord = g_gridCoord->point.refValue();
	M_Color3f& color = g_gridColor->color.refValue();
	M_Int& coordIndex = g_gridIls->coordIndex.refValue();
	coordIndex.length = 0;

	int columns = squares + 1;
	int rows = squares + 1;

	coord.length = columns * 2 + rows * 2;

	int halfOffset =  columns * 2;

	float width = scale * squares;
	float halfWidth = width / 2.0;


	if(xzPlane){
		for(int c = 0; c < columns; c++){
			coord[c * 2]	 = Vec3f(-halfWidth + c * scale, 0, -halfWidth);
			coord[c * 2 + 1] = Vec3f(-halfWidth + c * scale, 0,  halfWidth);
		}
		for(int r = 0; r < rows; r++){
			coord[halfOffset + r * 2]	  = Vec3f(-halfWidth, 0, -halfWidth + r * scale);
			coord[halfOffset + r * 2 + 1] = Vec3f( halfWidth, 0, -halfWidth + r * scale);
		}
	}
	else if(xyPlane){
		for(int c = 0; c < columns; c++){
			coord[c * 2]	 = Vec3f(-halfWidth + c * scale, -halfWidth, 0);
			coord[c * 2 + 1] = Vec3f(-halfWidth + c * scale,  halfWidth, 0);
		}
		for(int r = 0; r < rows; r++){
			coord[halfOffset + r * 2]	  = Vec3f(-halfWidth, -halfWidth + r * scale, 0);
			coord[halfOffset + r * 2 + 1] = Vec3f( halfWidth, -halfWidth + r * scale, 0);
		}
	}
	else if(yzPlane){
		for(int c = 0; c < columns; c++){
			coord[c * 2]	 = Vec3f(0, -halfWidth + c * scale, -halfWidth);
			coord[c * 2 + 1] = Vec3f(0, -halfWidth + c * scale,  halfWidth);
		}
		for(int r = 0; r < rows; r++){
			coord[halfOffset + r * 2]	  = Vec3f(0, -halfWidth, -halfWidth + r * scale);
			coord[halfOffset + r * 2 + 1] = Vec3f(0,  halfWidth, -halfWidth + r * scale);
		}
	}

	color.length = coord.length;
	for(int i = color.length; --i>=0;){
		color[i] = Color3f(.5, .5, .5);
	}
	

	if(columns & 1){
		color[columns - 1] = Color3f(.45, .45, .75);
		color[columns] = Color3f(.45, .45, .75);

		color[halfOffset + rows - 1] = Color3f(.45, .45, .75);
		color[halfOffset + rows] = Color3f(.45, .45, .75);
		
	//	int offset = columns * (columns / 2);
	//	for(int i = 0; i < columns; i++){
	//		color[offset + i] = Color3f(.45, .45, .75);
	//	}
	//	offset = columns / 2;
	//	for(int i = 0; i < columns; i++){
	//		color[offset] = Color3f(.45, .45, .75);
	//		offset += rows;
	//	}
	}


	for(int c = 0; c < columns; c++){
		coordIndex += c * 2;
		coordIndex += c * 2 + 1;
		coordIndex += -1;
	}

	for(int r = 0; r < rows; r++){
		coordIndex += halfOffset + r * 2;
		coordIndex += halfOffset + r * 2 + 1;
		coordIndex += -1;
	}

	restoreMathPresision();
	g_gridIls->convertToRenderFormat(g_gridShape->render);
}

void TransformObj::calcRotCtrlPnts(){
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);
	Rotation rot = Rotation(0, 0, 1, leverAngle);

	for(int k = rotCtrlPnt.length; --k>=0;){
		rotCtrlPnt[k] = rot * (leverAxis[k] * leverDistance[k]) + pp;
	}
}

void Seamless::genDynamicTransforms(){
	Matrix parentMatrix = g_scene->getViewMatrix();
	M_SArt& skeleton = this->skeleton.refValue();

	if(workAnimCP){
		for(int i = 0, smi = 0; i < skeleton.length; i++){
			skeleton[i]->genDynamicTransformsCP(parentMatrix, parentMatrix, smi);
		}
	}
	else{
		for(int i = 0, smi = 0; i < skeleton.length; i++){
			skeleton[i]->genDynamicTransforms(parentMatrix, smi);
		}
	}
}

void AnimLever::calcRotCtrlPnts(){
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	if(part == 0){
		return;
	}

	Seamless* smls = part->smls;

	smls->genDynamicTransforms();

	Vec3f pivotPoint  = part->pivotPointMatrix * Vec3f(0,0,0);

	//Vec3f pp = g_scene->getViewMatrix() * pivotPoint;
	Rotation rot = Rotation(0, 0, 1, leverAngle);

	for(int k = rotCtrlPnt.length; --k>=0;){
		rotCtrlPnt[k] = rot * (leverAxis[k] * leverDistance[k]) + pivotPoint;
	}
}

void TransformObj::calcNonSelectedRotCtrlPnts(){
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

	Vec3f v = rotCtrlPnt[leverIdx] - pp;
	Vec3f la = leverAxis[leverIdx];
	leverAngle = atan2(v.y, v.x) - atan2(la.y, la.x);
	Rotation rot = Rotation(0, 0, 1, leverAngle);

	leverDistance[leverIdx] = (rotCtrlPnt[leverIdx] - pp).length();

	for(int k = rotCtrlPnt.length; --k>=0;){
		if(k == leverIdx){
			continue;
		}
		rotCtrlPnt[k] = rot * (leverAxis[k] * leverDistance[k]) + pp;
	}
}

void TransformCP::calcRotCtrlPnts(){
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);
	Rotation rot = Rotation(0, 0, 1, leverAngle);

	for(int k = rotCtrlPnt.length; --k>=0;){
		rotCtrlPnt2[k] = rot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt2Offset[k]) + pp;
		rotCtrlPnt4[k] = rot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt4Offset[k]) + pp;
		rotCtrlPnt5[k] = rot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt5Offset[k]) + pp;
		rotCtrlPnt3[k] = rot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt3Offset[k]) + pp;

		rotCtrlPnt[k] = rot * (leverAxis[k] * leverDistance[k]) + pp;
	}
}

void TransformCP::calcNonSelectedRotCtrlPnts(){
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);




	leverDistance.refValue()[leverIdx] = (Vec2f(rotCtrlPnt[leverIdx].x, rotCtrlPnt[leverIdx].y) - Vec2f(pp.x, pp.y)).length();

	for(int k = rotCtrlPnt.length; --k>=0;){

		rotCtrlPnt2[k] = leverRot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt2Offset[k]) + pp;
		rotCtrlPnt4[k] = leverRot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt4Offset[k]) + pp;
		rotCtrlPnt5[k] = leverRot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt5Offset[k]) + pp;
		rotCtrlPnt3[k] = leverRot * (leverAxis[k] * leverDistance[k] + defaultRotCtrlPnt3Offset[k]) + pp;

		if(k == leverIdx){
			continue;
		}
		rotCtrlPnt[k] = leverRot * (leverAxis[k] * leverDistance[k]) + pp;

	}
}


Vec2f TransformCP::getMultiPoint(const int leverIdx, const Vec2f& rotPoint, const M_Vec3f& defaultRotCtrlPntOffset)const{
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

	Vec3f test = rotCtrlPnt[leverIdx] - pp;
	Vec3f v = Vec3f(rotPoint.x, rotPoint.y, 0) - pp;
	Vec3f la = leverAxis[leverIdx];
	float leverAngle = atan2(v.y, v.x) - atan2(la.y, la.x);
	Rotation rot = Rotation(0, 0, 1, leverAngle);


	float leverDistance = (rotPoint - Vec2f(pp.x, pp.y)).length();

	Vec3f reternPoint3f = rot * (leverAxis[leverIdx] * leverDistance + defaultRotCtrlPntOffset[leverIdx]) + pp;

	return Vec2f(reternPoint3f.x, reternPoint3f.y);
}

void AnimLever::calcNonSelectedRotCtrlPnts(){
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	if(part == 0){
		return;
	}
	Vec3f pivotPoint  = part->pivotPointMatrix * Vec3f(0,0,0);

	//Vec3f pp = g_scene->getViewMatrix() * pivotPoint;
	//Rotation rot = Rotation(defaultRotCtrlPnt[leverIdx], rotCtrlPnt[leverIdx] - pp);
	//leverAngle = rot.angle;
	Vec3f v = rotCtrlPnt[leverIdx] - pivotPoint;
	Vec3f la = leverAxis[leverIdx];
	leverAngle = atan2(v.y, v.x) - atan2(la.y, la.x);
	Rotation rot = Rotation(0, 0, 1, leverAngle);

	leverDistance.refValue()[leverIdx] = (rotCtrlPnt[leverIdx] - pivotPoint).length();

	for(int k = rotCtrlPnt.length; --k>=0;){
		if(k == leverIdx){
			continue;
		}
		rotCtrlPnt[k] = rot * (leverAxis[k] * leverDistance[k]) + pivotPoint;
	}
}

void AnimLever::resetRotationLever(){
	Art* part = dynamic_cast<Art*>(g_selNode.getRaw());
	if(part == 0){
		return;
	}

	Seamless* smls = part->smls;
	smls->genDynamicTransforms();


	Vec3f pivotPoint  = part->pivotPointMatrix * Vec3f(0,0,0);

	//Vec3f pp = g_scene->getViewMatrix() * pivotPoint;

	for(int k = rotCtrlPnt.length; --k>=0;){
		rotCtrlPnt[k] = pivotPoint + defaultRotCtrlPnt[k];
		leverDistance.refValue()[k] = .5;
	}
	leverAngle = 0;
	
	houseKeep(part, Window::getAnimBar());
}

void TransformCP::resetRotationLever(){
	if(MultiSelect::pointList.length >= leverSelection){
		Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

		for(int k = rotCtrlPnt.length; --k>=0;){
			rotCtrlPnt[k] = pp + defaultRotCtrlPnt[k];
			rotCtrlPnt2[k] = pp + defaultRotCtrlPnt[k] + defaultRotCtrlPnt2Offset[k];
			rotCtrlPnt3[k] = pp + defaultRotCtrlPnt[k] + defaultRotCtrlPnt3Offset[k];
			rotCtrlPnt4[k] = pp + defaultRotCtrlPnt[k] + defaultRotCtrlPnt4Offset[k];
			rotCtrlPnt5[k] = pp + defaultRotCtrlPnt[k] + defaultRotCtrlPnt5Offset[k];
			leverDistance.refValue()[k] = .5;
		}
		leverAngle = 0;
	}
	
	houseKeep();
}

void TransformObj::resetRotationLever(){
	Vec3f pp = g_scene->getViewMatrix() * (pivotPoint + translation);

	for(int k = rotCtrlPnt.length; --k>=0;){
		rotCtrlPnt[k] = pp + defaultRotCtrlPnt[k];
		leverDistance[k] = .5;
	}
	leverAngle = 0;
	
	houseKeep();
}

void NurbsLathe::postTug(){
	if(enterKeyState()){
	}
	else{
		int idx = index[0];
		g_grid->snapIt(point.refValue()[idx]);
		processFields();
		commonPostTug();
		houseKeep();
	}
}


void outPoint(NurbsPatch* patch, int i){
	Vec3f dpoint = patch->dpoint[i];
	Vec3f v = g_scene->getViewMatrix() * dpoint;
}

bool NurbsPatch::postTug(){
	if(MultiSelect::pointList.length == 0){
		return false;
	}
	if(g_transformCP->dragRotPointMode){
		g_transformCP->restoreCursorPoint();
	}

	if(g_grid->snap && g_transformCP->dragRotPointMode == false){
		g_pointer.translation = g_scene->getIViewMatrix() * g_pointer.translation;
		g_grid->snapIt(g_pointer.translation);
		g_pointer.translation = g_scene->getViewMatrix() * g_pointer.translation;
		drag(g_pointer.translation);
	}
	if(g_curvedCageMode){
		M_Vec3f& point = dynamic_cast<Coordinate*>(g_nurbsIlsCoord)->point;
		int lookUpIndex = g_pointLookup[plidx];

		g_pointer.translation = g_scene->getViewMatrix() * point[lookUpIndex + convertActualToVisual(MultiSelect::withinSightsIndex, dim.x, dim.y)];
	}
	return true;
}


void NurbsLathe::Mirror::operator=(const bool& par){
	refValue() = par;
	if(NurbsLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->mirror.refValue() = par;
		if(par == false){
			mirrorContainer->mirrorContainer = 0;
		}
		mirrorContainer->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}

void BendyLathe::Mirror::operator=(const bool& par){
	refValue() = par;
	if(BendyLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->mirror.refValue() = par;
		if(par == false){
			mirrorContainer->mirrorContainer = 0;
		}
		mirrorContainer->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}

void SphereBuild::Mirror::operator=(const bool& par){
	refValue() = par;
	if(SphereBuild* mirrorSphere = home.mirrorContainer){
		mirrorSphere->mirror.refValue() = par;
		if(par == false){
			mirrorSphere->mirrorContainer = 0;
		}
		mirrorSphere->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}

void CylinderBuild::Mirror::operator=(const bool& par){
	refValue() = par;
	if(CylinderBuild* mirrorCylinder = home.mirrorContainer){
		mirrorCylinder->mirror.refValue() = par;
		if(par == false){
			mirrorCylinder->mirrorContainer = 0;
		}
		mirrorCylinder->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}

void ConeBuild::Mirror::operator=(const bool& par){
	refValue() = par;
	if(ConeBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->mirror.refValue() = par;
		if(par == false){
			mirrorContainer->mirrorContainer = 0;
		}
		mirrorContainer->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}

void TorusBuild::Mirror::operator=(const bool& par){
	refValue() = par;
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->mirror.refValue() = par;
		if(par == false){
			mirrorContainer->mirrorContainer = 0;
		}
		mirrorContainer->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}

void BoxBuild::Mirror::operator=(const bool& par){
	refValue() = par;
	if(BoxBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->mirror.refValue() = par;
		if(par == false){
			mirrorContainer->mirrorContainer = 0;
		}
		mirrorContainer->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}
void RectangleBuild::Mirror::operator=(const bool& par){
	refValue() = par;
	if(RectangleBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->mirror.refValue() = par;
		if(par == false){
			mirrorContainer->mirrorContainer = 0;
		}
		mirrorContainer->processFields();
	}
	if(par == false){
		home.mirrorContainer = 0;
	}
	home.processFields();
}


void NurbsLathe::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(NurbsLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void BendyLathe::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(BendyLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void SphereBuild::Radius::operator=(const float& par){
	refValue()=par;
	home.processFields();
	if(SphereBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->radius.refValue() = par;
		mirrorContainer->processFields();
	}
}

void SphereBuild::Staves::operator=(const int& par){
	refValue()=par;
	home.processFields();
	processFieldsPlus(home);
	if(SphereBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->staves.refValue() = par;
		mirrorContainer->processFields();
		processFieldsPlus(*mirrorContainer);
	}
}

void SphereBuild::Bands::operator=(const int& par){
	refValue()=par;
	home.processFields();
	processFieldsPlus(home);
	if(SphereBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->bands.refValue() = par;
		mirrorContainer->processFields();
		processFieldsPlus(*mirrorContainer);
	}
}

void SphereBuild::YCentred::operator=(const bool& par){
	refValue()=par;
	home.processFields();
	if(SphereBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->yCentred.refValue() = par;
		mirrorContainer->processFields();
	}
}

void SphereBuild::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(SphereBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void CylinderBuild::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(CylinderBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void ConeBuild::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(ConeBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void TorusBuild::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void BoxBuild::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(BoxBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void RectangleBuild::Scale::operator=(const Vec3f& par){
	refValue()=par;
	home.processFields();
	if(RectangleBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->scale.refValue() = par;
		mirrorContainer->processFields();
	}
}

void NurbsLathe::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(NurbsLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void BendyLathe::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(BendyLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void SphereBuild::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(SphereBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void CylinderBuild::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(CylinderBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void ConeBuild::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(ConeBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void TorusBuild::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void BoxBuild::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(BoxBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void RectangleBuild::HRotation::operator=(const Rotation& par){
	refValue()=par;
	home.processFields();
	if(RectangleBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->rotation.refValue() = Rotation(par.x, -par.y, -par.z, par.angle);
		mirrorContainer->processFields();
	}
}

void NurbsLathe::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(NurbsLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}

void BendyLathe::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(BendyLathe* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}

void SphereBuild::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(SphereBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}

void CylinderBuild::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(CylinderBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}

void ConeBuild::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(ConeBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}

void TorusBuild::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}

void BoxBuild::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(BoxBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}

void RectangleBuild::Translation::operator=(const Vec3f& par){
	refValue() = par;
	home.processFields();
	if(RectangleBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->translation.refValue() = Vec3f(-par.x, par.y, par.z);
		mirrorContainer->processFields();
	}
}


void BoxBuild::XCentred::operator=(const bool& par){
	refValue()=par;
	home.processFields();
	if(BoxBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->xCentred.refValue() = par;
		mirrorContainer->processFields();
	}
}
void BoxBuild::YCentred::operator=(const bool& par){
	refValue()=par;
	home.processFields();
	if(BoxBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->yCentred.refValue() = par;
		mirrorContainer->processFields();
	}
}
void BoxBuild::ZCentred::operator=(const bool& par){
	refValue()=par;
	home.processFields();
	if(BoxBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->zCentred.refValue() = par;
		mirrorContainer->processFields();
	}
}


void ConvertToPatch::Done::operator=(bool const &){
#ifdef PROG_ROB
	dLog("new ToggleField(\"done\"),");
#endif
	Window::closeWindow(g_stw->cp.p);
}

void ConeBuild::TopCap::operator=(const bool& par){
	refValue() = par;
	Seamless* smls = home.m_patch[0].p->smls;
	if(par == false){
		NurbsPatch* topPatch = home.topPatch;
		smls->fastDeletePoliesOwnedByPatch(topPatch);
	}
	home.processFields();
	smls->populateMirrorNodes();
}

void ConeBuild::BaseCap::operator=(const bool& par){
	refValue() = par;
	Seamless* smls = home.m_patch[0].p->smls;
	if(par == false){
		NurbsPatch* basePatch = home.basePatch;
		smls->fastDeletePoliesOwnedByPatch(basePatch);
	}
	home.processFields();
	smls->populateMirrorNodes();
}

void ConeBuild::TopRad::operator=(const float& par){
	Seamless* smls = home.m_patch[0].p->smls;
	if(par > 0 && refValue() == 0){
		refValue() = par;
		home.processFields();
		smls->populateMirrorNodes();
		home.topPatch->bnspeBuildPolies();
		return;
	}
	else if(par == 0 && refValue() > 0){
		smls->fastDeletePoliesOwnedByPatch(home.topPatch);
	}
	refValue() = par;
	home.processFields();
	smls->populateMirrorNodes();
}

void ConeBuild::BaseRad::operator=(const float& par){
	Seamless* smls = home.m_patch[0].p->smls;
	if(par > 0 && refValue() == 0){
		refValue() = par;
		home.processFields();
		smls->populateMirrorNodes();
		home.basePatch->bnspeBuildPolies();
		return;
	}
	else if(par == 0 && refValue() > 0){
		smls->fastDeletePoliesOwnedByPatch(home.basePatch);
	}
	refValue() = par;
	home.processFields();
	smls->populateMirrorNodes();
}

extern bool g_viewPosDragMode;
extern Vec3f g_viewDragMousePos;

bool TransformObj::checkCtrlPoint(const bool perspective){
	for(int i = 0; i < rotCtrlPnt.length; i++){
		Vec3f sr = rotCtrlPnt[i];
		if(checkWithInSights(perspective, sr)){
			MultiSelect::withinSights = true;
			MultiSelect::withinSightsIndex = i;
			MultiSelect::withinSightsNode = MultiSelect::objectList[0];
			changeModellingCursor(g_lockedOnPointCursor);
			g_cursorOverPointIlsBase->translation.refValue() = sr;
			mode = ROTATE;
			return true;
		}
	}
	return false;
}

bool TransformCP::checkCtrlPoint(const bool perspective){
	mouseOffset = Vec3f();
	robotOffset = Vec2f();
	for(int i = 0; i < rotCtrlPnt.length; i++){
		Vec3f sr = rotCtrlPnt[i];
		if(checkWithInSights(perspective, sr)){
			overWhatPoint = ROTATE;
			MultiSelect::withinSights = true;
			MultiSelect::withinSightsIndex = i;
			MultiSelect::withinSightsNode = this;

			g_cursorOverPointIlsBase->translation.refValue() = sr;
			mouseOffset = rotCtrlPnt[i] - g_pointer.translation;

			changeModellingCursor(g_lockedOnPointCursor);
			return true;
		}
		if(g_settings->multiPointLever){
			sr = rotCtrlPnt2[i];
			if(checkWithInSights(perspective, sr)){
				overWhatPoint = NEUTRAL;
				MultiSelect::withinSights = true;
				MultiSelect::withinSightsIndex = i;
				MultiSelect::withinSightsNode = this;

				g_cursorOverPointIlsBase->translation.refValue() = sr;
				mouseOffset = rotCtrlPnt[i] - g_pointer.translation;
				robotOffset = toVec2f(rotCtrlPnt2[i]) - toVec2f(rotCtrlPnt[i]);
				//dLog(String("mouseOffset ") + mouseOffset.x);
				//dLog(String("g_pointer ") + g_pointer.translation.x);

				changeModellingCursor(g_lockedOnPointCursor);
				return true;
			}
			sr = rotCtrlPnt3[i];
			if(checkWithInSights(perspective, sr)){
				overWhatPoint = SCALE1D;
				MultiSelect::withinSights = true;
				MultiSelect::withinSightsIndex = i;
				MultiSelect::withinSightsNode = this;

				g_cursorOverPointIlsBase->translation.refValue() = sr;
				mouseOffset = rotCtrlPnt[i] - g_pointer.translation;
				robotOffset = toVec2f(rotCtrlPnt3[i]) - toVec2f(rotCtrlPnt[i]);

				changeModellingCursor(g_lockedOnPointCursor);
				return true;
			}
			sr = rotCtrlPnt4[i];
			if(checkWithInSights(perspective, sr)){
				overWhatPoint = TRANSLATE;
				MultiSelect::withinSights = true;
				MultiSelect::withinSightsIndex = i;
				MultiSelect::withinSightsNode = this;

				g_cursorOverPointIlsBase->translation.refValue() = sr;
				mouseOffset = rotCtrlPnt[i] - g_pointer.translation;
				robotOffset = toVec2f(rotCtrlPnt4[i]) - toVec2f(rotCtrlPnt[i]);
				//dLog(String("mouseOffset.x ") + mouseOffset.x);
				//dLog(String("mouseOffset.y ") + mouseOffset.y);
				//dLog(String("g_pointer.x ") + g_pointer.translation.x);
				//dLog(String("g_pointer.y ") + g_pointer.translation.y);
				changeModellingCursor(g_lockedOnPointCursor);
				return true;
			}
			sr = rotCtrlPnt5[i];
			if(checkWithInSights(perspective, sr)){
				overWhatPoint = SCALE3D;
				MultiSelect::withinSights = true;
				MultiSelect::withinSightsIndex = i;
				MultiSelect::withinSightsNode = this;

				g_cursorOverPointIlsBase->translation.refValue() = sr;
				mouseOffset = rotCtrlPnt[i] - g_pointer.translation;
				robotOffset = toVec2f(rotCtrlPnt5[i]) - toVec2f(rotCtrlPnt[i]);
				changeModellingCursor(g_lockedOnPointCursor);
				return true;
			}
		}
	}
	return false;
}

bool TransformCP::checkDimPoints(const bool perspective){
	Matrix pivotPointMatrix;
	getPivotPointMatrix(pivotPointMatrix, MultiSelect::pointList[0].p->smls);
	float inverseSize = (1 / g_scene->getZoom());
	float db = .2;
	//float db = .5;
	float d = inverseSize * db;
	float plb = .12;
	float pl = inverseSize * plb;

	dimRadialAdjust = Vec3f(db, 0, 0);
	Vec3f sr = pivotPointMatrix * (dimRadialAdjust * inverseSize);
	if(xTug && checkWithInSights(perspective, sr)){
		MultiSelect::withinSights = true;
		MultiSelect::withinSightsIndex = 0;
		MultiSelect::withinSightsNode = this;
		clickPoint = pivotPoint + translation + Vec3f(d, 0, 0);
		g_cursorOverPointIlsBase->translation.refValue() = sr;
		mouseOffset = sr - g_pointer.translation;

		changeModellingCursor(g_lockedOnPointCursor);
		return true;
	}

	dimRadialAdjust = Vec3f(0, db, 0);
	sr = pivotPointMatrix * (dimRadialAdjust * inverseSize);
	if(yTug && checkWithInSights(perspective, sr)){
		MultiSelect::withinSights = true;
		MultiSelect::withinSightsIndex = 1;
		MultiSelect::withinSightsNode = this;
		g_transformCP->clickPoint = pivotPoint + translation + Vec3f(0, d, 0);
		g_cursorOverPointIlsBase->translation.refValue() = sr;
		mouseOffset = sr - g_pointer.translation;

		changeModellingCursor(g_lockedOnPointCursor);
		return true;
	}

	dimRadialAdjust = Vec3f(0, 0, db);
	sr = pivotPointMatrix * (dimRadialAdjust * inverseSize);
	if(zTug && checkWithInSights(perspective, sr)){
		MultiSelect::withinSights = true;
		MultiSelect::withinSightsIndex = 2;
		MultiSelect::withinSightsNode = this;
		g_transformCP->clickPoint = pivotPoint + translation + Vec3f(0, 0, d);
		g_cursorOverPointIlsBase->translation.refValue() = sr;
		mouseOffset = sr - g_pointer.translation;

		changeModellingCursor(g_lockedOnPointCursor);
		return true;
	}


	dimRadialAdjust = Vec3f(plb, plb, 0);
	sr = pivotPointMatrix * (dimRadialAdjust * inverseSize);
	if(xTug && yTug && checkWithInSights(perspective, sr)){
		MultiSelect::withinSights = true;
		MultiSelect::withinSightsIndex = 3;
		MultiSelect::withinSightsNode = this;
		g_transformCP->clickPoint = pivotPoint + translation + Vec3f(pl, pl, 0);
		g_cursorOverPointIlsBase->translation.refValue() = sr;
		mouseOffset = sr - g_pointer.translation;

		changeModellingCursor(g_lockedOnPointCursor);
		return true;
	}

	dimRadialAdjust = Vec3f(0, plb, plb);
	sr = pivotPointMatrix * (dimRadialAdjust * inverseSize);
	if(yTug && zTug && checkWithInSights(perspective, sr)){
		MultiSelect::withinSights = true;
		MultiSelect::withinSightsIndex = 4;
		MultiSelect::withinSightsNode = this;
		g_transformCP->clickPoint = pivotPoint + translation + Vec3f(0, pl, pl);
		g_cursorOverPointIlsBase->translation.refValue() = sr;
		mouseOffset = sr - g_pointer.translation;

		changeModellingCursor(g_lockedOnPointCursor);
		return true;
	}

	dimRadialAdjust = Vec3f(plb, 0, plb);
	sr = pivotPointMatrix * (dimRadialAdjust * inverseSize);
	if(xTug && zTug && checkWithInSights(perspective, sr)){
		MultiSelect::withinSights = true;
		MultiSelect::withinSightsIndex = 5;
		MultiSelect::withinSightsNode = this;
		g_transformCP->clickPoint = pivotPoint + translation + Vec3f(pl, 0, pl);
		g_cursorOverPointIlsBase->translation.refValue() = sr;
		mouseOffset = sr - g_pointer.translation;

		changeModellingCursor(g_lockedOnPointCursor);
		return true;
	}
	return false;
}


bool Viewpoint::isOverPoint(){
	if(checkWithInSights(g_perspective, g_scene->getViewMatrix() * position)){
		dragMode = POSITION;
		return true;
	}
	else if(checkWithInSights(g_perspective, g_scene->getViewMatrix() * lookAt)){
		dragMode = LOOK_AT;
		return true;
	}
	else if(checkWithInSights(g_perspective, g_scene->getViewMatrix() * (up + position))){
		dragMode = UP_LEVER;
		return true;
	}
	return false;
}

bool TransformObj::isOverLever(){
	Matrix pivotPointMatrix;
	getPivotPointMatrix(pivotPointMatrix, MultiSelect::objectList[0].p);

	if(checkCtrlPoint(g_perspective)){
		return true;
	}
	else if(checkWithInSights(g_perspective, pivotPointMatrix * Vec3f())){
		mode = TRANSLATE;
		MultiSelect::withinSights = true;
		MultiSelect::withinSightsNode = MultiSelect::objectList[0];
		changeModellingCursor(g_lockedOnPointCursor);
		return true;
	}
	else{
		changeModellingCursor(g_crossCursor);
	}
	return false;
}

bool TransformCP::isOverLever(){
	//MultiSelect::withinSights = false;
	//MultiSelect::withinSightsNode = 0;
	isOverPivotPoint = false;
	isOverDimPoint = false;
	Matrix pivotPointMatrix;
	getPivotPointMatrix(pivotPointMatrix, MultiSelect::pointList[0].p->smls);

	if(dimFilterClickEnable && showDimFilterVisual && MultiSelect::pointList.length){
		if(checkDimPoints(g_perspective)){
			isOverDimPoint = true;
			return true;
		}
	}
	g_transformCP->dimRadialAdjust = Vec3f();

	if(MultiSelect::pointList.length >= g_transformCP->leverSelection){
		if((g_scene->mode & SINGLE_ACTIVE_MODE) == 0){
			if(checkCtrlPoint(g_perspective)){
				return true;
			}
			else if(checkWithInSights(g_perspective, pivotPointMatrix * Vec3f())){
				MultiSelect::withinSights = true;
				MultiSelect::withinSightsNode = this;
				isOverPivotPoint = true;

				Vec3f sr = pivotPointMatrix * Vec3f();
				g_cursorOverPointIlsBase->translation.refValue() = sr;
				g_transformCP->mouseOffset = sr - g_pointer.translation;

				changeModellingCursor(g_lockedOnPointCursor);
				return true;
			}
			else{
				changeModellingCursor(g_crossCursor);
			}
		}
	}
	return false;
}

void Brush::houseKeep(){
	mouse2dSync();
	g_visualClue = g_nurbsIlsBase;

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_nurbsIlsShape->geometry.p);
	polyLine->noZbuf = true;
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue();
	M_Color3f& color = dynamic_cast<Color*>(polyLine->color.p)->color.refValue();
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	coord.length = 0;
	color.length = 0;
	coordIndex.length = 0;

	int mark = coord.length;
	int numOfPoints = 129;
	coord.length += numOfPoints;
	color.length += numOfPoints;

	Vec3f location = g_scene->getIViewMatrix() * g_pointer.translation;
	Matrix matrix = rotationMatrix( -g_scene->rotation) * scaleMatrix(Vec3f(radius, radius, radius)) * translationMatrix(location);

	const Color3f selColor = g_palette->color;

	for(int i=0; i<numOfPoints; i++){
		coord[mark + i] = matrix * Vec3f(cos(i*(PI*2/(numOfPoints-1))), sin(i*(PI*2/(numOfPoints-1))),0);
		color[mark + i] = selColor;
	}
	fillPolyLineIndex(coord, mark, coordIndex);

	M_Node& children = g_nurbsIlsBase->children;
	children.length = 1;

	polyLine->convertToRenderFormat(g_nurbsIlsShape->render);

}

void Brush::Radius::operator=(const float& par){
	refValue() = par;
	home.houseKeep();
}

void Seamless::getMostNSEW(float& north, float& south, float& east, float& west, float& in, float& out){
	for(int k = partList.length; --k>=0;){
		Art* part = partList[k];
		M_Vec3f& coord = part->coord.refValue();

		for(int i = coord.length; --i>=0;){
			Vec3f crd = coord[i];

			if(crd.y > north){
				north = crd.y;
			}
			if(crd.y < south){
				south = crd.y;
			}
			if(crd.x > east){
				east = crd.x;
			}
			if(crd.x < west){
				west = crd.x;
			}
			if(crd.z > in){
				in = crd.z;
			}
			if(crd.z < out){
				out = crd.z;
			}
		}
	}
}
int boundBoxCoordIndex[] = {0, 1, -1, 0, 2, -1, 0, 3, -1, 4, 5, -1, 4, 6, -1, 4, 7, -1, 8, 9, -1, 8, 10, -1, 8, 11, -1, 12, 13, -1, 12, 14, -1, 12, 15, -1, 16, 17, -1, 16, 18, -1, 16, 19, -1, 20, 21, -1, 20, 22, -1, 20, 23,	-1, 24, 25, -1, 24, 26, -1, 24, 27, -1, 28, 29, -1, 28, 30, -1, 28, 31};

void Seamless::houseKeep(){
	if(g_scene->mode & SELECT_OBJECT_MODE){
		int selLine = 0;
		float north = -1000000;
		float south = 1000000;
		float east = -1000000;
		float west = 1000000;
		float in = -1000000;
		float out = 1000000;


		getMostNSEW(north, south, east, west, in, out);

		float xDiff = east - west;
		float yDiff = north - south;
		float zDiff = in - out;

		g_visualClue = g_nurbsIlsBase;

		IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_nurbsIlsShape->geometry.p);
		polyLine->noZbuf = true;
		M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue();
		M_Color3f& color = dynamic_cast<Color*>(polyLine->color.p)->color.refValue();
		M_Int& coordIndex = polyLine->coordIndex.refValue();

		coord.length = 32;
		color.length = 32;
		coordIndex.length = 72;

		coord[0] = Vec3f(west, south, out);
		coord[1] = Vec3f(west + xDiff * .25, south, out);
		coord[2] = Vec3f(west, south + yDiff * .25, out);
		coord[3] = Vec3f(west, south, out + zDiff * .25);

		coord[4] = Vec3f(west, south, in);
		coord[5] = Vec3f(west + xDiff * .25, south, in);
		coord[6] = Vec3f(west, south + yDiff * .25, in);
		coord[7] = Vec3f(west, south, in - zDiff * .25);

		coord[8] = Vec3f(east, south, out);
		coord[9] = Vec3f(east - xDiff * .25, south, out);
		coord[10] = Vec3f(east, south + yDiff * .25, out);
		coord[11] = Vec3f(east, south, out + zDiff * .25);

		coord[12] = Vec3f(east, south, in);
		coord[13] = Vec3f(east - xDiff * .25, south, in);
		coord[14] = Vec3f(east, south + yDiff * .25, in);
		coord[15] = Vec3f(east, south, in - zDiff * .25);

////
		coord[16] = Vec3f(west, north, out);
		coord[17] = Vec3f(west + xDiff * .25, north, out);
		coord[18] = Vec3f(west, north - yDiff * .25, out);
		coord[19] = Vec3f(west, north, out + zDiff * .25);

		coord[20] = Vec3f(west, north, in);
		coord[21] = Vec3f(west + xDiff * .25, north, in);
		coord[22] = Vec3f(west, north - yDiff * .25, in);
		coord[23] = Vec3f(west, north, in - zDiff * .25);

		coord[24] = Vec3f(east, north, out);
		coord[25] = Vec3f(east - xDiff * .25, north, out);
		coord[26] = Vec3f(east, north - yDiff * .25, out);
		coord[27] = Vec3f(east, north, out + zDiff * .25);

		coord[28] = Vec3f(east, north, in);
		coord[29] = Vec3f(east - xDiff * .25, north, in);
		coord[30] = Vec3f(east, north - yDiff * .25, in);
		coord[31] = Vec3f(east, north, in - zDiff * .25);

		//Matrix matrix = scaleMatrix(scale) * rotationMatrix(rotation) * translationMatrix(translation);

		for(int i = color.length; --i>=0;){
			//coord[i] = matrix * coord[i];
			color[i] = Color3f(1, .5, .5);
		}
		g_nurbsIlsBase->translation = translation;
		g_nurbsIlsBase->rotation = rotation;
		g_nurbsIlsBase->scale = scale;

		coordIndex = INT_LIST(boundBoxCoordIndex);


		M_Node& children = g_nurbsIlsBase->children;
		children.length = 1;

		g_transformObj->houseKeep();
		children += g_transformObj.p;

		polyLine->convertToRenderFormat(g_nurbsIlsShape->render);

	}
}


void Viewpoint::houseKeep(){
	if(0 == dynamic_cast<Viewpoint*>(g_selNode.getRaw())){
		return;
	}

	if(g_scene->mode & SELECT_OBJECT_MODE && g_cameraObject == 0){
		createCameraObject();
	}
	if(g_scene->mode & ANIMATION_MODE && g_showCamerasInAnimMode && g_cameraObject == 0){
		createCameraObject();
	}

	g_visualClue = this;

	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_nurbsIlsShape->geometry.p);
	polyLine->noZbuf = true;
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue();
	M_Color3f& color = dynamic_cast<Color*>(polyLine->color.p)->color.refValue();
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	coord.length = 4;
	color.length = 4;

	Rotation negRot = -orientation;

	Vec3f lookAtOffset = negRot * (lookAt - position);
	Vec3f upOffset = negRot * up;

	float lookAtDistance = lookAtOffset.z;
	float upDistance = upOffset.y;

	coord[0] = Vec3f();
	coord[1] = Vec3f(0, 0, lookAtDistance);
	coord[2] = Vec3f();
	coord[3] = Vec3f(0, upDistance, 0);



	for(int i = color.length; --i>=0;){
		color[i] = Color3f(1, .5, .5);
	}
	g_nurbsIlsBase->translation = position;
	g_nurbsIlsBase->rotation = orientation;
	g_nurbsIlsBase->scale = Vec3f(1, 1, 1);

	coordIndex = INT_LIST(cameraCoordIndex);


	M_Node& children = g_nurbsIlsBase->children;
	children.length = 1;

	//g_transformObj->houseKeep();
	//children += g_transformObj.p;

	polyLine->convertToRenderFormat(g_nurbsIlsShape->render);


	{//debugging
	IndexedLineSet* polyLine = dynamic_cast<IndexedLineSet*>(g_vcEllipticalPolyline2D->geometry.p);
	polyLine->noZbuf = true;
	M_Vec3f& coord = dynamic_cast<Coordinate*>(polyLine->coord.p)->point.refValue();
	M_Int& coordIndex = polyLine->coordIndex.refValue();

	coord.length = 4;


	coord[0] = position;
	coord[1] = lookAt;
	coord[2] = position;
	coord[3] = up;


		
	coordIndex = INT_LIST(cameraCoordIndex);

	polyLine->convertToRenderFormat(g_vcEllipticalPolyline2D->render);

	}//end of debugging

	//}
}

void TorusBuild::HoleRad::operator=(float const& par){
	refValue() = par;
	home.processFields();
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->holeRad.refValue() = par;
		mirrorContainer->processFields();
	}
}
void TorusBuild::TubeRad::operator=(float const& par){
	refValue() = par;
	home.processFields();
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->tubeRad.refValue() = par;
		mirrorContainer->processFields();
	}
}
void TorusBuild::Staves::operator=(int const& par){
	refValue() = par;
	processFieldsPlus(home);
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->staves.refValue() = par;
		processFieldsPlus(*mirrorContainer);
	}
}
void TorusBuild::Bands::operator=(int const& par){
	refValue() = par;
	processFieldsPlus(home);
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->bands.refValue() = par;
		processFieldsPlus(*mirrorContainer);
	}
	
}
void TorusBuild::YCentred::operator=(bool const& par){
	refValue() = par;
	home.processFields();
	if(TorusBuild* mirrorContainer = home.mirrorContainer){
		mirrorContainer->yCentred.refValue() = par;
		mirrorContainer->processFields();
	}
}
void paintCp(){
	g_stw->cp->paint();
}
void houseKeepAndPaintCp(){
	houseKeepVisualClue();
	g_stw->cp->paint();
}

void TransformObj::refreshRemPoints(){
	scaleOrientation.refValue() = Rotation();
	scale.refValue() = Vec3f(1, 1, 1);
	rotation.refValue() = Rotation();
	pivotPoint.refValue() += translation.refValue();
	translation.refValue() = Vec3f();
	remRotCtrlPnt = rotCtrlPnt[leverIdx];

	MultiSelect::beginDragPos.length = MultiSelect::objectList.length;
	if(MultiSelect::objectList.length){
		if(Seamless* smls = dynamic_cast<Seamless*>(MultiSelect::objectList[0].p)){
			smls->remObjTransform();

			for(int i = MultiSelect::objectList.length; --i >= 0;){
				if(Seamless* smls = dynamic_cast<Seamless*>(MultiSelect::objectList[i].p)){
					MultiSelect::beginDragPos[i] = smls->translation;
				}
			}
		}
	}
}

void TransformObj::calcPivotPoint(){
	Vec3f& pivotPoint = this->pivotPoint.refValue();

	pivotPoint = Vec3f(0, 0, 0);
	scale.refValue() = Vec3f(1, 1, 1);
	scaleOrientation.refValue() = Rotation();
	rotation.refValue() = Rotation();
	translation.refValue() = Vec3f();

	if(MultiSelect::objectList.length == 0){
		return;
	}

	pivotPoint = Vec3f();
	MultiSelect::beginDragPos.length = MultiSelect::objectList.length;
	for(int i = MultiSelect::objectList.length; --i >= 0;){
		Node* node = MultiSelect::objectList[i].p;
		if(Seamless* smls = dynamic_cast<Seamless*>(node)){
			Vec3f v = smls->translation;
			MultiSelect::beginDragPos[i] = v;
			pivotPoint += v;
		}
	}
	pivotPoint = pivotPoint / MultiSelect::beginDragPos.length;

	resetRotationLever();
}

void AddLathe::checkOverInitPoint(const bool perspective){
	MultiSelect::withinSightsNode = 0;
	if(lathe->checkOverPoint(perspective)){
		MultiSelect::withinSightsNode = this;
		changeModellingCursor(g_lockedOnPointCursor);
	}
	else{
		changeModellingCursor(g_addPointCursor);
	}
}

void AddNurbsLine::checkOverInitPoint(const bool perspective){
	MultiSelect::withinSightsNode = 0;

	Matrix matrix = g_viewMatrix;
	M_Vec3f& dpoint = line->dpoint.refValue();
	for(int i = dpoint.length; --i >= 0;){
		Vec3f pnt = matrix * dpoint[i];
		if(checkOverPoint2(perspective, pnt)){
			MultiSelect::withinSightsNode = this;
			MultiSelect::clickedIndex = i;
			changeModellingCursor(g_lockedOnPointCursor);
			return;
		}
	}

	changeModellingCursor(g_addPointCursor);
}

void AddLathe::drag(Vec3f pt){
	lathe->drag(pt);
}


void AddNurbsLine::drag(Vec3f pt){
	g_dragActuallyHappened = true;
	M_Vec3f& point = line->point.refValue();
	M_Vec3f& dpoint = line->dpoint.refValue();
	int idx = line->index[0];
	Vec3f vec= g_scene->getIViewMatrix() * pt;

	line->setPointAndCp(idx, vec);

	if(NurbsLine* mirrorContainer = dynamic_cast<NurbsLine*>(line->mirrorPatch)){
		Vec3f mirrorVec = vec;
		mirrorVec.x = -mirrorVec.x;

		mirrorContainer->setPointAndCp(idx, mirrorVec);
		
		mirrorContainer->processFields();
	}
	if(point.length > 1){
		processFields();
	}

	g_pointer.translation = g_scene->getViewMatrix() * vec;
	g_cursorOverPointIlsBase->translation = g_pointer.translation;


	commonPostTug();
	houseKeep();
}

void DrawNurbsLine::drag(Vec3f pt){
	g_dragActuallyHappened = true;
	M_Vec3f& point = line->point.refValue();
	M_Vec3f& dpoint = line->dpoint.refValue();
	int idx = line->index[0];
	Vec3f vec= g_scene->getIViewMatrix() * pt;

	if(mousePhase == DRAW){
		pixelSketch += vec;
		m_pressure += mousePressure;
	}

	line->setPointAndCp(idx, vec);

	if(NurbsLine* mirrorContainer = dynamic_cast<NurbsLine*>(line->mirrorPatch)){
		Vec3f mirrorVec = vec;
		mirrorVec.x = -mirrorVec.x;

		mirrorContainer->setPointAndCp(idx, mirrorVec);
		
		mirrorContainer->processFields();
	}
	if(point.length > 1){
		processFields();
	}

	g_pointer.translation = g_scene->getViewMatrix() * vec;
	g_cursorOverPointIlsBase->translation = g_pointer.translation;


	commonPostTug();
	houseKeep();
}

void TransformCP::drag(Vec3f pt){
	if(VisualAid* visualAid = dynamic_cast<VisualAid*>(g_selNode.getRaw())){
		if(g_transformCP->selectObject){
			if(Build* bld = dynamic_cast<Build*>(visualAid)){
				if(NurbsPatch* patch = bld->getPatch()){
					patch->drag(g_pointer.translation);
				}
			}
		}
		else if((!(controlKeyState())) || g_transformCP->dragRotPointMode){
			visualAid->drag(g_pointer.translation);
		}
	}
}

bool NurbsTrack::isOverPoint(){
	//if(checkWithInSights(g_perspective, g_scene->getViewMatrix() * keyValue[0])){
	//	return true;
	//}
	Matrix matrix = g_viewMatrix = g_scene->getViewMatrix();
	M_Vec3f& keyValue = this->keyValue;
	for(int i = keyValue.length; --i >= 0;){
		Vec3f pnt = matrix * keyValue[i];
		if(checkOverPoint2(g_perspective, pnt)){
			MultiSelect::withinSightsNode = this;
			MultiSelect::clickedIndex = i;
			return true;
		}
	}
	return false;
}

bool NikInterpolator::isOverPoint(){
	if(smls == 0){
		return false;
	}
	M_Vec3f& focusPoint = focusOffset ? shortPoint : longPoint;

	Matrix matrix = g_viewMatrix = g_scene->getViewMatrix();

	for(int i = focusPoint.length; --i >= 0;){
		Vec3f pnt = matrix * focusPoint[i];
		if(checkOverPoint2(g_perspective, pnt)){
			MultiSelect::withinSightsNode = this;
			MultiSelect::clickedIndex = i;
			return true;
		}
	}
	return false;
}

void TransformCP::restoreCursorPoint(){

	Vec3f v;

	switch(overWhatPoint){
		case TransformCP::ROTATE:
			v = rotCtrlPnt[leverIdx];
			break;
		case TransformCP::NEUTRAL:
			v = rotCtrlPnt2[leverIdx];
			break;
		case TransformCP::SCALE1D:
			v = rotCtrlPnt3[leverIdx];
			break;
		case TransformCP::TRANSLATE:
			v = rotCtrlPnt4[leverIdx];
			break;
		case TransformCP::SCALE3D:
			v = rotCtrlPnt5[leverIdx];
			break;
	}
	g_cursorOverPointIlsBase->translation = v;
}

bool NurbsPatch::pointIsVisible(int idx){
	if(g_scene->mode & BACKFACE_IGNORE_MODE){
		return g_pointIsVisible[idx];
	}
	return true;
}

void createNewBuildNode(Build* buildNode, Seamless* smls, Art* part);
/*
void DrawNurbsLine::convertDrawnLineToNurbsLine(){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& point = line->point.refValue();
	M_Vec3f& dpoint = line->dpoint.refValue();
	M_Float& weight = line->weight.refValue();
	M_Int& cpIndex = line->cpIndex.refValue();
	M_Color3f& color = line->color.refValue();
	M_Float& uKnot = line->uKnot.refValue();


	point.length = ctrlPoints;
	dpoint.length = ctrlPoints;
	weight.length = ctrlPoints;
	cpIndex.length = ctrlPoints;
	color.length = ctrlPoints;

	int div = pixelSketch.length / (ctrlPoints - 1);


	for(int k = 0; k < ctrlPoints; k++){
		int idx = k * div;
		if(k == ctrlPoints - 1){
			idx = pixelSketch.length - 1;
		}
		Vec3f newPoint = pixelSketch[idx];
		point[k] = newPoint;
		dpoint[k] = newPoint;
		cpIndex[k] = cp.length;
		cp += newPoint;
		cw += 1;
		color[k] = g_palette->color;
		weight[k] = 1;
	}

	

	line->processFields();
	updateNurbsCurveKnotMid0(point.length, uKnot);
	//done = true;


	mapDrawnPointsToNurbs();
	evolveNurbsPointsToDrawnPoints();
	line->pointsToCps();

	smls->populateMirrorNodes();
	smls->genVisibleMask();
	smls->doBuild();

	line->index[0] = point.length - 1;
	MultiSelect::pointList = ControlPoint(line, point.length - 1);


	line->houseKeep();
}
*/
void DrawNurbsLine::convertDrawnLineToNurbsLine(){
	M_Vec3f& cp = smls->cp.refValue();
	M_Float& cw = smls->cw.refValue();

	M_Vec3f& point = line->point.refValue();
	M_Vec3f& dpoint = line->dpoint.refValue();
	M_Float& weight = line->weight.refValue();
	M_Int& cpIndex = line->cpIndex.refValue();
	M_Color3f& color = line->color.refValue();
	M_Float& uKnot = line->uKnot.refValue();


	if(line->point.length != ctrlPoints){
		point.length = ctrlPoints;
		dpoint.length = ctrlPoints;
		weight.length = ctrlPoints;
		cpIndex.length = ctrlPoints;
		color.length = ctrlPoints;
		m_processedPressure.length = ctrlPoints;

		int div = pixelSketch.length / (ctrlPoints - 2);

		int idx = 0;


		for(int k = 0; k < ctrlPoints; k++){
			Vec3f newPoint = pixelSketch[idx];
			point[k] = newPoint;
			dpoint[k] = newPoint;
			cpIndex[k] = cp.length;
			cp += newPoint;
			cw += 1;
			color[k] = g_palette->color;
			weight[k] = 1;

			if(k == 0){
				idx += div >> 1;
			}
			else if(k == ctrlPoints - 2){
				idx = pixelSketch.length - 1;
			}
			else{
				idx += div;
			}
		}
	}

	

	line->processFields();
	updateNurbsCurveKnotMid0(point.length, uKnot);
	//done = true;


	mapDrawnPointsToNurbs();
	evolveNurbsPointsToDrawnPoints();

	if(thickLine){
		if(patch == 0){
			patch = new NurbsPatch;

			Stem* stem = new Stem;
			stem->sync.refValue() = true;
			stem->part.refValue() += smls->partList[0];
			ColorSweep* colorSweep = new ColorSweep;
			colorSweep->straight.refValue() = true;
			colorSweep->color.refValue() += g_palette->color;
			colorSweep->color.refValue() += g_palette->color;
			colorSweep->opacity.refValue() += 1;
			colorSweep->opacity.refValue() += 1;
			colorSweep->distance.refValue() += 1;
			stem->textureMapper.refValue() += colorSweep;
			patch->stem.refValue() += stem;
			patch->uJoinEnds.refValue() = false;

			createNewBuildNode(patch, smls, 0);

			patch->isCtrlPolyRef = true;
		}
	}
	else{
		if(getNodeLine(g_drawNurbsLine->line) == -1){
			createNewBuildNode(line, smls, 0);
		}
	}

	line->pointsToCps();

	smls->populateMirrorNodes();
	smls->genVisibleMask();
	smls->doBuild();

	line->index[0] = point.length - 1;
	MultiSelect::pointList = ControlPoint(line, point.length - 1);


	if(thickLine){
		line->convertToThickLine(patch);
	}
	else{
		line->houseKeep();
		smls->genVisibleMask();
	}
}


inline bool getPointOnCurve2(const Vec3f* p, const float* w, const M_Float& knot, const int dim, float minT, float maxT, const int degree, const Vec3f prevCrd, Vec3f& crd, double& t, double tInc, const float targetDist, float& prevDistDiff);


void DrawNurbsLine::getPointOnCurve(const M_Vec3f& point, const M_Float& weight, const M_Float& knot, float minT, float maxT, const int degree, Vec3f& prevCrd, double& t, double tInc, float targetDist, Vec3f& fromVector){

	const int dim = point.length;


	const Vec3f* p = point.getArray();
	const float* w = weight.getArray();

	Vec3f crd;
	float prevDistDiff = 10000;
	int limitCounter = 1000;


	while(prevDistDiff > .000001 && --limitCounter > 0){
		if(getPointOnCurve2(p, w, knot, dim, minT, maxT, degree, prevCrd, crd, t, tInc, targetDist, prevDistDiff)){
			continue;
		}
		else if(getPointOnCurve2(p, w, knot, dim, minT, maxT, degree, prevCrd, crd, t, -tInc, targetDist, prevDistDiff)){
			continue;
		}
		tInc = tInc * .5;
	};

	Vec3f toVector = crd - prevCrd;
	fromVector = toVector;

	prevCrd = crd;

}


inline bool mapToLineFineTune(float& u, float uInc, float& closestDist, const int degree, const int dim, float minT, float maxT, const Vec3f* pointPtr, const float* weightPtr, const float* knotPtr, const Vec3f& tartgetPoint, Vec3f& crd){
	bool negCrossOver = false;
	bool posCrossOver = false;
	float newU = u + uInc;
	if(newU < 0){
		negCrossOver = true;
		newU = 1 + newU;
	}
	else if(newU > 1){
		posCrossOver = true;
		newU = newU - 1;
	}

	float t = (maxT - minT) * newU + minT;

	CurvePoint(dim - 1, degree, knotPtr, pointPtr, weightPtr, t, crd);

	Vec3f diff = crd - tartgetPoint;

	float distance = abs(diff.length());
	if(distance < closestDist){
		closestDist = distance;
		if(negCrossOver || posCrossOver){
			u = newU;
			uInc = -uInc;
		}
		else{
			u += uInc;
		}
		return true;
	}

	return false;
}


void DrawNurbsLine::mapDrawnPointToNurbs(float& closestT, const Vec3f targetPoint, const Vec3f* pointPtr, const float* weightPtr, const float* knotPtr, const int dim, float minT, float maxT, const int degree){
	
	//Vec3f targetPoint = pixelSketch[drawnIdx];


	float closestDist = 1000000000;
	Vec3f crd;
	Vec3f closestCrd;

	int numOfTries = 100;

	float uInc = .01;

	while(--numOfTries >= 0){
		if(closestDist <= .0001){
			break;
		}
		if(false == mapToLineFineTune(closestT, uInc, closestDist, degree, dim, minT, maxT, pointPtr, weightPtr, knotPtr, targetPoint, closestCrd)){
			uInc = -uInc;
			if(false == mapToLineFineTune(closestT, uInc, closestDist, degree, dim, minT, maxT, pointPtr, weightPtr, knotPtr, targetPoint, closestCrd)){
				uInc = -uInc * .5;
			}
		}
	}
}

Color3f DrawNurbsLine::convertPressureToCol(int idx){
	int prsI = m_pressure[idx];
	float prs = prsI;
	prs = prs / 1023;
	prs *= 2;
	if(prs > 1){
		prs = 1;
	}

	float invPrs = 1.0 - prs;
	if(invPrs < 0){
		invPrs = 0;
	}
	return g_palette->color.refValue() * prs + Color3f(1,1,1) * invPrs;
}

void DrawNurbsLine::processPressure(int idx, int processedPressureIdx){
	int prsI = m_pressure[idx];
	float prs = prsI;
	prs = prs / 1023;
	//prs *= 2;
	if(prs > 1){
		prs = 1;
	}

	m_processedPressure[processedPressureIdx] = prs;

	if(colorPressure){

		float invPrs = 1.0 - prs;
		if(invPrs < 0){
			invPrs = 0;
		}
		line->color.refValue()[processedPressureIdx] = g_palette->color.refValue() * prs + Color3f(1,1,1) * invPrs;
	}
}


void DrawNurbsLine::mapDrawnPointsToNurbs(){//could be faster if we did only 2d points for line

	M_Color3f& m_color = line->color;
	const float* knotPtr = line->uKnot.refValue().getArray();
	const Vec3f* pointPtr = line->point.refValue().getArray();
	const float* weightPtr = line->weight.refValue().getArray();
	const int degree = 2;
	const int dim = line->point.length;
	const float maxT = line->maxU();
	const float minT = line->minU();

	float tColorSectionInc = dim - 2;
	tColorSectionInc = 1.0 / tColorSectionInc;
	float tColor = tColorSectionInc * .5;

	if(thickPressure || colorPressure){
		processPressure(1, 0);
	}

	int colorIndex = 1;

	tSketch.length = pixelSketch.length;
	tSketch[0] = 0;

	float closestT = 0;


	

	for(int i = 0; i < pixelSketch.length; i++){
		mapDrawnPointToNurbs(closestT, pixelSketch[i], pointPtr, weightPtr, knotPtr, dim, minT, maxT, degree);
		
		tSketch[i] = closestT;

		if(thickPressure || colorPressure){
			if(closestT >= tColor){
				processPressure(i, colorIndex++);

				tColor += tColorSectionInc;
			}
		}
	}

	if(thickPressure || colorPressure){
		processPressure(m_pressure.length - 2, m_color.length - 1);
	}
}


inline float getDistance(const float u, const Vec3f tartgetPoint, const Vec3f* pointPtr, const float* weightPtr, const float* knotPtr, const int dim, float minT, float maxT, const int degree){

	float t = (maxT - minT) * u + minT;

	Vec3f crd;

	CurvePoint(dim - 1, degree, knotPtr, pointPtr, weightPtr, t, crd);

	Vec3f diff = crd - tartgetPoint;

	float distance = abs(diff.length());

	return distance;
}

float DrawNurbsLine::addUpError(){//could be faster if we did only 2d points for line
	const float* knotPtr = line->uKnot.refValue().getArray();
	const Vec3f* pointPtr = line->point.refValue().getArray();
	const float* weightPtr = line->weight.refValue().getArray();
	const int degree = 2;
	const int dim = line->point.length;
	const float maxT = line->maxU();
	const float minT = line->minU();

	float sumDistance = 0;

	for(int i = 0; i < pixelSketch.length; i++){
		sumDistance += getDistance(tSketch[i], pixelSketch[i], pointPtr, weightPtr, knotPtr, dim, minT, maxT, degree);
	}

	return sumDistance;
}


void DrawNurbsLine::evolveNurbsPointsToDrawnPoints(){
	M_Vec3f& point = line->point.refValue();
	M_Vec3f& dpoint = line->dpoint.refValue();
	float closestSumDist = addUpError();
	int numOfTries = 100;
	float addInc = .2;
	


	while(--numOfTries >= 0){
		bool gotCloser = false;

		for(int k = 1; k < point.length - 1; k++){
			Vec3f remPoint = point[k];
			Vec3f newPoint = remPoint;

			newPoint.x += addInc;

			dpoint[k] = point[k] = newPoint;
			float sumDistance = addUpError();

			if(sumDistance < closestSumDist){
				remPoint = newPoint;
				gotCloser = true;
				closestSumDist = sumDistance;
			}
			else{
				dpoint[k] = point[k] = remPoint;
				newPoint = remPoint;
			}


			
			newPoint.x -= addInc;

			dpoint[k] = point[k] = newPoint;
			sumDistance = addUpError();

			if(sumDistance < closestSumDist){
				remPoint = newPoint;
				gotCloser = true;
				closestSumDist = sumDistance;
			}
			else{
				dpoint[k] = point[k] = remPoint;
				newPoint = remPoint;
			}


			newPoint.y += addInc;

			dpoint[k] = point[k] = newPoint;
			sumDistance = addUpError();

			if(sumDistance < closestSumDist){
				remPoint = newPoint;
				gotCloser = true;
				closestSumDist = sumDistance;
			}
			else{
				dpoint[k] = point[k] = remPoint;
				newPoint = remPoint;
			}



			newPoint.y -= addInc;

			dpoint[k] = point[k] = newPoint;
			sumDistance = addUpError();

			if(sumDistance < closestSumDist){
				remPoint = newPoint;
				gotCloser = true;
				closestSumDist = sumDistance;
			}
			else{
				dpoint[k] = point[k] = remPoint;
				newPoint = remPoint;
			}
		}
		if(gotCloser == false){
			addInc = addInc * .5;
		}
	}
}

void DrawNurbsLine::CtrlPoints::operator=(const int& par){
	refValue() = par;
	if(home.mousePhase == READY_TO_DRAG){
		home.convertDrawnLineToNurbsLine();
	}
}

void DrawNurbsLine::Evolve::operator=(const bool& par){
	if(home.mousePhase == READY_TO_DRAG){
		home.mapDrawnPointsToNurbs();
		home.evolveNurbsPointsToDrawnPoints();
		home.line->pointsToCps();
		home.smls->doBuild();

		home.line->houseKeep();
	}
}

void AddNurbsLine::Palette::operator=(const bool& par){
	openPalette();
}

void DrawNurbsLine::Palette::operator=(const bool& par){
	openPalette();
}

SFRotation NurbsLine::getOrientation(float at){//at = t

	float* knotPtr = this->uKnot.refValue().getArray();
	Vec3f* pointPtr = this->point.refValue().getArray();
	float* weightPtr = this->weight.getArray();
	int degree = order.x - 1;

	float mxu = maxU();
	float mnu = minU();

	double t = (mxu - mnu) * at + mnu;
	double tm = (mxu - mnu) * (at - .0001) + mnu;
	double tp = (mxu - mnu) * (at + .0001) + mnu;

	Rotation rot;
	Vec3f dest0;

	CurvePoint(point.length-1, degree, knotPtr, pointPtr, weightPtr, t, dest0);

	if(at < 1.0){
		Vec3f dest1;
		CurvePoint(point.length-1, degree, knotPtr, pointPtr, weightPtr, tp, dest1);
		rot = Rotation(Vec3f(0,1,0), Vec3f(dest1-dest0).normalize());
	}
	else{
		Vec3f dest1;
		CurvePoint(point.length-1, degree, knotPtr, pointPtr, weightPtr, tm, dest1);
		rot = Rotation(Vec3f(0,1,0), Vec3f(dest0-dest1).normalize());
	}


	return rot;
}
